/** vim: et:ts=4:sw=4:sts=4
 * @license RequireJS 2.1.9 Copyright (c) 2010-2012, The Dojo Foundation All Rights Reserved.
 * Available via the MIT or new BSD license.
 * see: http://github.com/jrburke/requirejs for details
 */
//Not using strict: uneven strict support in browsers, #392, and causes
//problems with requirejs.exec()/transpiler plugins that may not be strict.
/*jslint regexp: true, nomen: true, sloppy: true */
/*global window, navigator, document, importScripts, setTimeout, opera */

var requirejs, require, define;
(function (global) {
    var req, s, head, baseElement, dataMain, src,
        interactiveScript, currentlyAddingScript, mainScript, subPath,
        version = '2.1.9',
        commentRegExp = /(\/\*([\s\S]*?)\*\/|([^:]|^)\/\/(.*)$)/mg,
        cjsRequireRegExp = /[^.]\s*require\s*\(\s*["']([^'"\s]+)["']\s*\)/g,
        jsSuffixRegExp = /\.js$/,
        currDirRegExp = /^\.\//,
        op = Object.prototype,
        ostring = op.toString,
        hasOwn = op.hasOwnProperty,
        ap = Array.prototype,
        apsp = ap.splice,
        isBrowser = !!(typeof window !== 'undefined' && typeof navigator !== 'undefined' && window.document),
        isWebWorker = !isBrowser && typeof importScripts !== 'undefined',
        //PS3 indicates loaded and complete, but need to wait for complete
        //specifically. Sequence is 'loading', 'loaded', execution,
        // then 'complete'. The UA check is unfortunate, but not sure how
        //to feature test w/o causing perf issues.
        readyRegExp = isBrowser && navigator.platform === 'PLAYSTATION 3' ?
                      /^complete$/ : /^(complete|loaded)$/,
        defContextName = '_',
        //Oh the tragedy, detecting opera. See the usage of isOpera for reason.
        isOpera = typeof opera !== 'undefined' && opera.toString() === '[object Opera]',
        contexts = {},
        cfg = {},
        globalDefQueue = [],
        useInteractive = false;

    function isFunction(it) {
        return ostring.call(it) === '[object Function]';
    }

    function isArray(it) {
        return ostring.call(it) === '[object Array]';
    }

    /**
     * Helper function for iterating over an array. If the func returns
     * a true value, it will break out of the loop.
     */
    function each(ary, func) {
        if (ary) {
            var i;
            for (i = 0; i < ary.length; i += 1) {
                if (ary[i] && func(ary[i], i, ary)) {
                    break;
                }
            }
        }
    }

    /**
     * Helper function for iterating over an array backwards. If the func
     * returns a true value, it will break out of the loop.
     */
    function eachReverse(ary, func) {
        if (ary) {
            var i;
            for (i = ary.length - 1; i > -1; i -= 1) {
                if (ary[i] && func(ary[i], i, ary)) {
                    break;
                }
            }
        }
    }

    function hasProp(obj, prop) {
        return hasOwn.call(obj, prop);
    }

    function getOwn(obj, prop) {
        return hasProp(obj, prop) && obj[prop];
    }

    /**
     * Cycles over properties in an object and calls a function for each
     * property value. If the function returns a truthy value, then the
     * iteration is stopped.
     */
    function eachProp(obj, func) {
        var prop;
        for (prop in obj) {
            if (hasProp(obj, prop)) {
                if (func(obj[prop], prop)) {
                    break;
                }
            }
        }
    }

    /**
     * Simple function to mix in properties from source into target,
     * but only if target does not already have a property of the same name.
     */
    function mixin(target, source, force, deepStringMixin) {
        if (source) {
            eachProp(source, function (value, prop) {
                if (force || !hasProp(target, prop)) {
                    if (deepStringMixin && typeof value !== 'string') {
                        if (!target[prop]) {
                            target[prop] = {};
                        }
                        mixin(target[prop], value, force, deepStringMixin);
                    } else {
                        target[prop] = value;
                    }
                }
            });
        }
        return target;
    }

    //Similar to Function.prototype.bind, but the 'this' object is specified
    //first, since it is easier to read/figure out what 'this' will be.
    function bind(obj, fn) {
        return function () {
            return fn.apply(obj, arguments);
        };
    }

    function scripts() {
        return document.getElementsByTagName('script');
    }

    function defaultOnError(err) {
        throw err;
    }

    //Allow getting a global that expressed in
    //dot notation, like 'a.b.c'.
    function getGlobal(value) {
        if (!value) {
            return value;
        }
        var g = global;
        each(value.split('.'), function (part) {
            g = g[part];
        });
        return g;
    }

    /**
     * Constructs an error with a pointer to an URL with more information.
     * @param {String} id the error ID that maps to an ID on a web page.
     * @param {String} message human readable error.
     * @param {Error} [err] the original error, if there is one.
     *
     * @returns {Error}
     */
    function makeError(id, msg, err, requireModules) {
        var e = new Error(msg + '\nhttp://requirejs.org/docs/errors.html#' + id);
        e.requireType = id;
        e.requireModules = requireModules;
        if (err) {
            e.originalError = err;
        }
        return e;
    }

    if (typeof define !== 'undefined') {
        //If a define is already in play via another AMD loader,
        //do not overwrite.
        return;
    }

    if (typeof requirejs !== 'undefined') {
        if (isFunction(requirejs)) {
            //Do not overwrite and existing requirejs instance.
            return;
        }
        cfg = requirejs;
        requirejs = undefined;
    }

    //Allow for a require config object
    if (typeof require !== 'undefined' && !isFunction(require)) {
        //assume it is a config object.
        cfg = require;
        require = undefined;
    }

    function newContext(contextName) {
        var inCheckLoaded, Module, context, handlers,
            checkLoadedTimeoutId,
            config = {
                //Defaults. Do not set a default for map
                //config to speed up normalize(), which
                //will run faster if there is no default.
                waitSeconds: 7,
                baseUrl: './',
                paths: {},
                pkgs: {},
                shim: {},
                config: {}
            },
            registry = {},
            //registry of just enabled modules, to speed
            //cycle breaking code when lots of modules
            //are registered, but not activated.
            enabledRegistry = {},
            undefEvents = {},
            defQueue = [],
            defined = {},
            urlFetched = {},
            requireCounter = 1,
            unnormalizedCounter = 1;

        /**
         * Trims the . and .. from an array of path segments.
         * It will keep a leading path segment if a .. will become
         * the first path segment, to help with module name lookups,
         * which act like paths, but can be remapped. But the end result,
         * all paths that use this function should look normalized.
         * NOTE: this method MODIFIES the input array.
         * @param {Array} ary the array of path segments.
         */
        function trimDots(ary) {
            var i, part;
            for (i = 0; ary[i]; i += 1) {
                part = ary[i];
                if (part === '.') {
                    ary.splice(i, 1);
                    i -= 1;
                } else if (part === '..') {
                    if (i === 1 && (ary[2] === '..' || ary[0] === '..')) {
                        //End of the line. Keep at least one non-dot
                        //path segment at the front so it can be mapped
                        //correctly to disk. Otherwise, there is likely
                        //no path mapping for a path starting with '..'.
                        //This can still fail, but catches the most reasonable
                        //uses of ..
                        break;
                    } else if (i > 0) {
                        ary.splice(i - 1, 2);
                        i -= 2;
                    }
                }
            }
        }

        /**
         * Given a relative module name, like ./something, normalize it to
         * a real name that can be mapped to a path.
         * @param {String} name the relative name
         * @param {String} baseName a real name that the name arg is relative
         * to.
         * @param {Boolean} applyMap apply the map config to the value. Should
         * only be done if this normalization is for a dependency ID.
         * @returns {String} normalized name
         */
        function normalize(name, baseName, applyMap) {
            var pkgName, pkgConfig, mapValue, nameParts, i, j, nameSegment,
                foundMap, foundI, foundStarMap, starI,
                baseParts = baseName && baseName.split('/'),
                normalizedBaseParts = baseParts,
                map = config.map,
                starMap = map && map['*'];

            //Adjust any relative paths.
            if (name && name.charAt(0) === '.') {
                //If have a base name, try to normalize against it,
                //otherwise, assume it is a top-level require that will
                //be relative to baseUrl in the end.
                if (baseName) {
                    if (getOwn(config.pkgs, baseName)) {
                        //If the baseName is a package name, then just treat it as one
                        //name to concat the name with.
                        normalizedBaseParts = baseParts = [baseName];
                    } else {
                        //Convert baseName to array, and lop off the last part,
                        //so that . matches that 'directory' and not name of the baseName's
                        //module. For instance, baseName of 'one/two/three', maps to
                        //'one/two/three.js', but we want the directory, 'one/two' for
                        //this normalization.
                        normalizedBaseParts = baseParts.slice(0, baseParts.length - 1);
                    }

                    name = normalizedBaseParts.concat(name.split('/'));
                    trimDots(name);

                    //Some use of packages may use a . path to reference the
                    //'main' module name, so normalize for that.
                    pkgConfig = getOwn(config.pkgs, (pkgName = name[0]));
                    name = name.join('/');
                    if (pkgConfig && name === pkgName + '/' + pkgConfig.main) {
                        name = pkgName;
                    }
                } else if (name.indexOf('./') === 0) {
                    // No baseName, so this is ID is resolved relative
                    // to baseUrl, pull off the leading dot.
                    name = name.substring(2);
                }
            }

            //Apply map config if available.
            if (applyMap && map && (baseParts || starMap)) {
                nameParts = name.split('/');

                for (i = nameParts.length; i > 0; i -= 1) {
                    nameSegment = nameParts.slice(0, i).join('/');

                    if (baseParts) {
                        //Find the longest baseName segment match in the config.
                        //So, do joins on the biggest to smallest lengths of baseParts.
                        for (j = baseParts.length; j > 0; j -= 1) {
                            mapValue = getOwn(map, baseParts.slice(0, j).join('/'));

                            //baseName segment has config, find if it has one for
                            //this name.
                            if (mapValue) {
                                mapValue = getOwn(mapValue, nameSegment);
                                if (mapValue) {
                                    //Match, update name to the new value.
                                    foundMap = mapValue;
                                    foundI = i;
                                    break;
                                }
                            }
                        }
                    }

                    if (foundMap) {
                        break;
                    }

                    //Check for a star map match, but just hold on to it,
                    //if there is a shorter segment match later in a matching
                    //config, then favor over this star map.
                    if (!foundStarMap && starMap && getOwn(starMap, nameSegment)) {
                        foundStarMap = getOwn(starMap, nameSegment);
                        starI = i;
                    }
                }

                if (!foundMap && foundStarMap) {
                    foundMap = foundStarMap;
                    foundI = starI;
                }

                if (foundMap) {
                    nameParts.splice(0, foundI, foundMap);
                    name = nameParts.join('/');
                }
            }

            return name;
        }

        function removeScript(name) {
            if (isBrowser) {
                each(scripts(), function (scriptNode) {
                    if (scriptNode.getAttribute('data-requiremodule') === name &&
                            scriptNode.getAttribute('data-requirecontext') === context.contextName) {
                        scriptNode.parentNode.removeChild(scriptNode);
                        return true;
                    }
                });
            }
        }

        function hasPathFallback(id) {
            var pathConfig = getOwn(config.paths, id);
            if (pathConfig && isArray(pathConfig) && pathConfig.length > 1) {
                //Pop off the first array value, since it failed, and
                //retry
                pathConfig.shift();
                context.require.undef(id);
                context.require([id]);
                return true;
            }
        }

        //Turns a plugin!resource to [plugin, resource]
        //with the plugin being undefined if the name
        //did not have a plugin prefix.
        function splitPrefix(name) {
            var prefix,
                index = name ? name.indexOf('!') : -1;
            if (index > -1) {
                prefix = name.substring(0, index);
                name = name.substring(index + 1, name.length);
            }
            return [prefix, name];
        }

        /**
         * Creates a module mapping that includes plugin prefix, module
         * name, and path. If parentModuleMap is provided it will
         * also normalize the name via require.normalize()
         *
         * @param {String} name the module name
         * @param {String} [parentModuleMap] parent module map
         * for the module name, used to resolve relative names.
         * @param {Boolean} isNormalized: is the ID already normalized.
         * This is true if this call is done for a define() module ID.
         * @param {Boolean} applyMap: apply the map config to the ID.
         * Should only be true if this map is for a dependency.
         *
         * @returns {Object}
         */
        function makeModuleMap(name, parentModuleMap, isNormalized, applyMap) {
            var url, pluginModule, suffix, nameParts,
                prefix = null,
                parentName = parentModuleMap ? parentModuleMap.name : null,
                originalName = name,
                isDefine = true,
                normalizedName = '';

            //If no name, then it means it is a require call, generate an
            //internal name.
            if (!name) {
                isDefine = false;
                name = '_@r' + (requireCounter += 1);
            }

            nameParts = splitPrefix(name);
            prefix = nameParts[0];
            name = nameParts[1];

            if (prefix) {
                prefix = normalize(prefix, parentName, applyMap);
                pluginModule = getOwn(defined, prefix);
            }

            //Account for relative paths if there is a base name.
            if (name) {
                if (prefix) {
                    if (pluginModule && pluginModule.normalize) {
                        //Plugin is loaded, use its normalize method.
                        normalizedName = pluginModule.normalize(name, function (name) {
                            return normalize(name, parentName, applyMap);
                        });
                    } else {
                        normalizedName = normalize(name, parentName, applyMap);
                    }
                } else {
                    //A regular module.
                    normalizedName = normalize(name, parentName, applyMap);

                    //Normalized name may be a plugin ID due to map config
                    //application in normalize. The map config values must
                    //already be normalized, so do not need to redo that part.
                    nameParts = splitPrefix(normalizedName);
                    prefix = nameParts[0];
                    normalizedName = nameParts[1];
                    isNormalized = true;

                    url = context.nameToUrl(normalizedName);
                }
            }

            //If the id is a plugin id that cannot be determined if it needs
            //normalization, stamp it with a unique ID so two matching relative
            //ids that may conflict can be separate.
            suffix = prefix && !pluginModule && !isNormalized ?
                     '_unnormalized' + (unnormalizedCounter += 1) :
                     '';

            return {
                prefix: prefix,
                name: normalizedName,
                parentMap: parentModuleMap,
                unnormalized: !!suffix,
                url: url,
                originalName: originalName,
                isDefine: isDefine,
                id: (prefix ?
                        prefix + '!' + normalizedName :
                        normalizedName) + suffix
            };
        }

        function getModule(depMap) {
            var id = depMap.id,
                mod = getOwn(registry, id);

            if (!mod) {
                mod = registry[id] = new context.Module(depMap);
            }

            return mod;
        }

        function on(depMap, name, fn) {
            var id = depMap.id,
                mod = getOwn(registry, id);

            if (hasProp(defined, id) &&
                    (!mod || mod.defineEmitComplete)) {
                if (name === 'defined') {
                    fn(defined[id]);
                }
            } else {
                mod = getModule(depMap);
                if (mod.error && name === 'error') {
                    fn(mod.error);
                } else {
                    mod.on(name, fn);
                }
            }
        }

        function onError(err, errback) {
            var ids = err.requireModules,
                notified = false;

            if (errback) {
                errback(err);
            } else {
                each(ids, function (id) {
                    var mod = getOwn(registry, id);
                    if (mod) {
                        //Set error on module, so it skips timeout checks.
                        mod.error = err;
                        if (mod.events.error) {
                            notified = true;
                            mod.emit('error', err);
                        }
                    }
                });

                if (!notified) {
                    req.onError(err);
                }
            }
        }

        /**
         * Internal method to transfer globalQueue items to this context's
         * defQueue.
         */
        function takeGlobalQueue() {
            //Push all the globalDefQueue items into the context's defQueue
            if (globalDefQueue.length) {
                //Array splice in the values since the context code has a
                //local var ref to defQueue, so cannot just reassign the one
                //on context.
                apsp.apply(defQueue,
                           [defQueue.length - 1, 0].concat(globalDefQueue));
                globalDefQueue = [];
            }
        }

        handlers = {
            'require': function (mod) {
                if (mod.require) {
                    return mod.require;
                } else {
                    return (mod.require = context.makeRequire(mod.map));
                }
            },
            'exports': function (mod) {
                mod.usingExports = true;
                if (mod.map.isDefine) {
                    if (mod.exports) {
                        return mod.exports;
                    } else {
                        return (mod.exports = defined[mod.map.id] = {});
                    }
                }
            },
            'module': function (mod) {
                if (mod.module) {
                    return mod.module;
                } else {
                    return (mod.module = {
                        id: mod.map.id,
                        uri: mod.map.url,
                        config: function () {
                            var c,
                                pkg = getOwn(config.pkgs, mod.map.id);
                            // For packages, only support config targeted
                            // at the main module.
                            c = pkg ? getOwn(config.config, mod.map.id + '/' + pkg.main) :
                                      getOwn(config.config, mod.map.id);
                            return  c || {};
                        },
                        exports: defined[mod.map.id]
                    });
                }
            }
        };

        function cleanRegistry(id) {
            //Clean up machinery used for waiting modules.
            delete registry[id];
            delete enabledRegistry[id];
        }

        function breakCycle(mod, traced, processed) {
            var id = mod.map.id;

            if (mod.error) {
                mod.emit('error', mod.error);
            } else {
                traced[id] = true;
                each(mod.depMaps, function (depMap, i) {
                    var depId = depMap.id,
                        dep = getOwn(registry, depId);

                    //Only force things that have not completed
                    //being defined, so still in the registry,
                    //and only if it has not been matched up
                    //in the module already.
                    if (dep && !mod.depMatched[i] && !processed[depId]) {
                        if (getOwn(traced, depId)) {
                            mod.defineDep(i, defined[depId]);
                            mod.check(); //pass false?
                        } else {
                            breakCycle(dep, traced, processed);
                        }
                    }
                });
                processed[id] = true;
            }
        }

        function checkLoaded() {
            var map, modId, err, usingPathFallback,
                waitInterval = config.waitSeconds * 1000,
                //It is possible to disable the wait interval by using waitSeconds of 0.
                expired = waitInterval && (context.startTime + waitInterval) < new Date().getTime(),
                noLoads = [],
                reqCalls = [],
                stillLoading = false,
                needCycleCheck = true;

            //Do not bother if this call was a result of a cycle break.
            if (inCheckLoaded) {
                return;
            }

            inCheckLoaded = true;

            //Figure out the state of all the modules.
            eachProp(enabledRegistry, function (mod) {
                map = mod.map;
                modId = map.id;

                //Skip things that are not enabled or in error state.
                if (!mod.enabled) {
                    return;
                }

                if (!map.isDefine) {
                    reqCalls.push(mod);
                }

                if (!mod.error) {
                    //If the module should be executed, and it has not
                    //been inited and time is up, remember it.
                    if (!mod.inited && expired) {
                        if (hasPathFallback(modId)) {
                            usingPathFallback = true;
                            stillLoading = true;
                        } else {
                            noLoads.push(modId);
                            removeScript(modId);
                        }
                    } else if (!mod.inited && mod.fetched && map.isDefine) {
                        stillLoading = true;
                        if (!map.prefix) {
                            //No reason to keep looking for unfinished
                            //loading. If the only stillLoading is a
                            //plugin resource though, keep going,
                            //because it may be that a plugin resource
                            //is waiting on a non-plugin cycle.
                            return (needCycleCheck = false);
                        }
                    }
                }
            });

            if (expired && noLoads.length) {
                //If wait time expired, throw error of unloaded modules.
                err = makeError('timeout', 'Load timeout for modules: ' + noLoads, null, noLoads);
                err.contextName = context.contextName;
                return onError(err);
            }

            //Not expired, check for a cycle.
            if (needCycleCheck) {
                each(reqCalls, function (mod) {
                    breakCycle(mod, {}, {});
                });
            }

            //If still waiting on loads, and the waiting load is something
            //other than a plugin resource, or there are still outstanding
            //scripts, then just try back later.
            if ((!expired || usingPathFallback) && stillLoading) {
                //Something is still waiting to load. Wait for it, but only
                //if a timeout is not already in effect.
                if ((isBrowser || isWebWorker) && !checkLoadedTimeoutId) {
                    checkLoadedTimeoutId = setTimeout(function () {
                        checkLoadedTimeoutId = 0;
                        checkLoaded();
                    }, 50);
                }
            }

            inCheckLoaded = false;
        }

        Module = function (map) {
            this.events = getOwn(undefEvents, map.id) || {};
            this.map = map;
            this.shim = getOwn(config.shim, map.id);
            this.depExports = [];
            this.depMaps = [];
            this.depMatched = [];
            this.pluginMaps = {};
            this.depCount = 0;

            /* this.exports this.factory
               this.depMaps = [],
               this.enabled, this.fetched
            */
        };

        Module.prototype = {
            init: function (depMaps, factory, errback, options) {
                options = options || {};

                //Do not do more inits if already done. Can happen if there
                //are multiple define calls for the same module. That is not
                //a normal, common case, but it is also not unexpected.
                if (this.inited) {
                    return;
                }

                this.factory = factory;

                if (errback) {
                    //Register for errors on this module.
                    this.on('error', errback);
                } else if (this.events.error) {
                    //If no errback already, but there are error listeners
                    //on this module, set up an errback to pass to the deps.
                    errback = bind(this, function (err) {
                        this.emit('error', err);
                    });
                }

                //Do a copy of the dependency array, so that
                //source inputs are not modified. For example
                //"shim" deps are passed in here directly, and
                //doing a direct modification of the depMaps array
                //would affect that config.
                this.depMaps = depMaps && depMaps.slice(0);

                this.errback = errback;

                //Indicate this module has be initialized
                this.inited = true;

                this.ignore = options.ignore;

                //Could have option to init this module in enabled mode,
                //or could have been previously marked as enabled. However,
                //the dependencies are not known until init is called. So
                //if enabled previously, now trigger dependencies as enabled.
                if (options.enabled || this.enabled) {
                    //Enable this module and dependencies.
                    //Will call this.check()
                    this.enable();
                } else {
                    this.check();
                }
            },

            defineDep: function (i, depExports) {
                //Because of cycles, defined callback for a given
                //export can be called more than once.
                if (!this.depMatched[i]) {
                    this.depMatched[i] = true;
                    this.depCount -= 1;
                    this.depExports[i] = depExports;
                }
            },

            fetch: function () {
                if (this.fetched) {
                    return;
                }
                this.fetched = true;

                context.startTime = (new Date()).getTime();

                var map = this.map;

                //If the manager is for a plugin managed resource,
                //ask the plugin to load it now.
                if (this.shim) {
                    context.makeRequire(this.map, {
                        enableBuildCallback: true
                    })(this.shim.deps || [], bind(this, function () {
                        return map.prefix ? this.callPlugin() : this.load();
                    }));
                } else {
                    //Regular dependency.
                    return map.prefix ? this.callPlugin() : this.load();
                }
            },

            load: function () {
                var url = this.map.url;

                //Regular dependency.
                if (!urlFetched[url]) {
                    urlFetched[url] = true;
                    context.load(this.map.id, url);
                }
            },

            /**
             * Checks if the module is ready to define itself, and if so,
             * define it.
             */
            check: function () {
                if (!this.enabled || this.enabling) {
                    return;
                }

                var err, cjsModule,
                    id = this.map.id,
                    depExports = this.depExports,
                    exports = this.exports,
                    factory = this.factory;

                if (!this.inited) {
                    this.fetch();
                } else if (this.error) {
                    this.emit('error', this.error);
                } else if (!this.defining) {
                    //The factory could trigger another require call
                    //that would result in checking this module to
                    //define itself again. If already in the process
                    //of doing that, skip this work.
                    this.defining = true;

                    if (this.depCount < 1 && !this.defined) {
                        if (isFunction(factory)) {
                            //If there is an error listener, favor passing
                            //to that instead of throwing an error. However,
                            //only do it for define()'d  modules. require
                            //errbacks should not be called for failures in
                            //their callbacks (#699). However if a global
                            //onError is set, use that.
                            if ((this.events.error && this.map.isDefine) ||
                                req.onError !== defaultOnError) {
                                try {
                                    exports = context.execCb(id, factory, depExports, exports);
                                } catch (e) {
                                    err = e;
                                }
                            } else {
                                exports = context.execCb(id, factory, depExports, exports);
                            }

                            if (this.map.isDefine) {
                                //If setting exports via 'module' is in play,
                                //favor that over return value and exports. After that,
                                //favor a non-undefined return value over exports use.
                                cjsModule = this.module;
                                if (cjsModule &&
                                        cjsModule.exports !== undefined &&
                                        //Make sure it is not already the exports value
                                        cjsModule.exports !== this.exports) {
                                    exports = cjsModule.exports;
                                } else if (exports === undefined && this.usingExports) {
                                    //exports already set the defined value.
                                    exports = this.exports;
                                }
                            }

                            if (err) {
                                err.requireMap = this.map;
                                err.requireModules = this.map.isDefine ? [this.map.id] : null;
                                err.requireType = this.map.isDefine ? 'define' : 'require';
                                return onError((this.error = err));
                            }

                        } else {
                            //Just a literal value
                            exports = factory;
                        }

                        this.exports = exports;

                        if (this.map.isDefine && !this.ignore) {
                            defined[id] = exports;

                            if (req.onResourceLoad) {
                                req.onResourceLoad(context, this.map, this.depMaps);
                            }
                        }

                        //Clean up
                        cleanRegistry(id);

                        this.defined = true;
                    }

                    //Finished the define stage. Allow calling check again
                    //to allow define notifications below in the case of a
                    //cycle.
                    this.defining = false;

                    if (this.defined && !this.defineEmitted) {
                        this.defineEmitted = true;
                        this.emit('defined', this.exports);
                        this.defineEmitComplete = true;
                    }

                }
            },

            callPlugin: function () {
                var map = this.map,
                    id = map.id,
                    //Map already normalized the prefix.
                    pluginMap = makeModuleMap(map.prefix);

                //Mark this as a dependency for this plugin, so it
                //can be traced for cycles.
                this.depMaps.push(pluginMap);

                on(pluginMap, 'defined', bind(this, function (plugin) {
                    var load, normalizedMap, normalizedMod,
                        name = this.map.name,
                        parentName = this.map.parentMap ? this.map.parentMap.name : null,
                        localRequire = context.makeRequire(map.parentMap, {
                            enableBuildCallback: true
                        });

                    //If current map is not normalized, wait for that
                    //normalized name to load instead of continuing.
                    if (this.map.unnormalized) {
                        //Normalize the ID if the plugin allows it.
                        if (plugin.normalize) {
                            name = plugin.normalize(name, function (name) {
                                return normalize(name, parentName, true);
                            }) || '';
                        }

                        //prefix and name should already be normalized, no need
                        //for applying map config again either.
                        normalizedMap = makeModuleMap(map.prefix + '!' + name,
                                                      this.map.parentMap);
                        on(normalizedMap,
                            'defined', bind(this, function (value) {
                                this.init([], function () { return value; }, null, {
                                    enabled: true,
                                    ignore: true
                                });
                            }));

                        normalizedMod = getOwn(registry, normalizedMap.id);
                        if (normalizedMod) {
                            //Mark this as a dependency for this plugin, so it
                            //can be traced for cycles.
                            this.depMaps.push(normalizedMap);

                            if (this.events.error) {
                                normalizedMod.on('error', bind(this, function (err) {
                                    this.emit('error', err);
                                }));
                            }
                            normalizedMod.enable();
                        }

                        return;
                    }

                    load = bind(this, function (value) {
                        this.init([], function () { return value; }, null, {
                            enabled: true
                        });
                    });

                    load.error = bind(this, function (err) {
                        this.inited = true;
                        this.error = err;
                        err.requireModules = [id];

                        //Remove temp unnormalized modules for this module,
                        //since they will never be resolved otherwise now.
                        eachProp(registry, function (mod) {
                            if (mod.map.id.indexOf(id + '_unnormalized') === 0) {
                                cleanRegistry(mod.map.id);
                            }
                        });

                        onError(err);
                    });

                    //Allow plugins to load other code without having to know the
                    //context or how to 'complete' the load.
                    load.fromText = bind(this, function (text, textAlt) {
                        /*jslint evil: true */
                        var moduleName = map.name,
                            moduleMap = makeModuleMap(moduleName),
                            hasInteractive = useInteractive;

                        //As of 2.1.0, support just passing the text, to reinforce
                        //fromText only being called once per resource. Still
                        //support old style of passing moduleName but discard
                        //that moduleName in favor of the internal ref.
                        if (textAlt) {
                            text = textAlt;
                        }

                        //Turn off interactive script matching for IE for any define
                        //calls in the text, then turn it back on at the end.
                        if (hasInteractive) {
                            useInteractive = false;
                        }

                        //Prime the system by creating a module instance for
                        //it.
                        getModule(moduleMap);

                        //Transfer any config to this other module.
                        if (hasProp(config.config, id)) {
                            config.config[moduleName] = config.config[id];
                        }

                        try {
                            req.exec(text);
                        } catch (e) {
                            return onError(makeError('fromtexteval',
                                             'fromText eval for ' + id +
                                            ' failed: ' + e,
                                             e,
                                             [id]));
                        }

                        if (hasInteractive) {
                            useInteractive = true;
                        }

                        //Mark this as a dependency for the plugin
                        //resource
                        this.depMaps.push(moduleMap);

                        //Support anonymous modules.
                        context.completeLoad(moduleName);

                        //Bind the value of that module to the value for this
                        //resource ID.
                        localRequire([moduleName], load);
                    });

                    //Use parentName here since the plugin's name is not reliable,
                    //could be some weird string with no path that actually wants to
                    //reference the parentName's path.
                    plugin.load(map.name, localRequire, load, config);
                }));

                context.enable(pluginMap, this);
                this.pluginMaps[pluginMap.id] = pluginMap;
            },

            enable: function () {
                enabledRegistry[this.map.id] = this;
                this.enabled = true;

                //Set flag mentioning that the module is enabling,
                //so that immediate calls to the defined callbacks
                //for dependencies do not trigger inadvertent load
                //with the depCount still being zero.
                this.enabling = true;

                //Enable each dependency
                each(this.depMaps, bind(this, function (depMap, i) {
                    var id, mod, handler;

                    if (typeof depMap === 'string') {
                        //Dependency needs to be converted to a depMap
                        //and wired up to this module.
                        depMap = makeModuleMap(depMap,
                                               (this.map.isDefine ? this.map : this.map.parentMap),
                                               false,
                                               !this.skipMap);
                        this.depMaps[i] = depMap;

                        handler = getOwn(handlers, depMap.id);

                        if (handler) {
                            this.depExports[i] = handler(this);
                            return;
                        }

                        this.depCount += 1;

                        on(depMap, 'defined', bind(this, function (depExports) {
                            this.defineDep(i, depExports);
                            this.check();
                        }));

                        if (this.errback) {
                            on(depMap, 'error', bind(this, this.errback));
                        }
                    }

                    id = depMap.id;
                    mod = registry[id];

                    //Skip special modules like 'require', 'exports', 'module'
                    //Also, don't call enable if it is already enabled,
                    //important in circular dependency cases.
                    if (!hasProp(handlers, id) && mod && !mod.enabled) {
                        context.enable(depMap, this);
                    }
                }));

                //Enable each plugin that is used in
                //a dependency
                eachProp(this.pluginMaps, bind(this, function (pluginMap) {
                    var mod = getOwn(registry, pluginMap.id);
                    if (mod && !mod.enabled) {
                        context.enable(pluginMap, this);
                    }
                }));

                this.enabling = false;

                this.check();
            },

            on: function (name, cb) {
                var cbs = this.events[name];
                if (!cbs) {
                    cbs = this.events[name] = [];
                }
                cbs.push(cb);
            },

            emit: function (name, evt) {
                each(this.events[name], function (cb) {
                    cb(evt);
                });
                if (name === 'error') {
                    //Now that the error handler was triggered, remove
                    //the listeners, since this broken Module instance
                    //can stay around for a while in the registry.
                    delete this.events[name];
                }
            }
        };

        function callGetModule(args) {
            //Skip modules already defined.
            if (!hasProp(defined, args[0])) {
                getModule(makeModuleMap(args[0], null, true)).init(args[1], args[2]);
            }
        }

        function removeListener(node, func, name, ieName) {
            //Favor detachEvent because of IE9
            //issue, see attachEvent/addEventListener comment elsewhere
            //in this file.
            if (node.detachEvent && !isOpera) {
                //Probably IE. If not it will throw an error, which will be
                //useful to know.
                if (ieName) {
                    node.detachEvent(ieName, func);
                }
            } else {
                node.removeEventListener(name, func, false);
            }
        }

        /**
         * Given an event from a script node, get the requirejs info from it,
         * and then removes the event listeners on the node.
         * @param {Event} evt
         * @returns {Object}
         */
        function getScriptData(evt) {
            //Using currentTarget instead of target for Firefox 2.0's sake. Not
            //all old browsers will be supported, but this one was easy enough
            //to support and still makes sense.
            var node = evt.currentTarget || evt.srcElement;

            //Remove the listeners once here.
            removeListener(node, context.onScriptLoad, 'load', 'onreadystatechange');
            removeListener(node, context.onScriptError, 'error');

            return {
                node: node,
                id: node && node.getAttribute('data-requiremodule')
            };
        }

        function intakeDefines() {
            var args;

            //Any defined modules in the global queue, intake them now.
            takeGlobalQueue();

            //Make sure any remaining defQueue items get properly processed.
            while (defQueue.length) {
                args = defQueue.shift();
                if (args[0] === null) {
                    return onError(makeError('mismatch', 'Mismatched anonymous define() module: ' + args[args.length - 1]));
                } else {
                    //args are id, deps, factory. Should be normalized by the
                    //define() function.
                    callGetModule(args);
                }
            }
        }

        context = {
            config: config,
            contextName: contextName,
            registry: registry,
            defined: defined,
            urlFetched: urlFetched,
            defQueue: defQueue,
            Module: Module,
            makeModuleMap: makeModuleMap,
            nextTick: req.nextTick,
            onError: onError,

            /**
             * Set a configuration for the context.
             * @param {Object} cfg config object to integrate.
             */
            configure: function (cfg) {
                //Make sure the baseUrl ends in a slash.
                if (cfg.baseUrl) {
                    if (cfg.baseUrl.charAt(cfg.baseUrl.length - 1) !== '/') {
                        cfg.baseUrl += '/';
                    }
                }

                //Save off the paths and packages since they require special processing,
                //they are additive.
                var pkgs = config.pkgs,
                    shim = config.shim,
                    objs = {
                        paths: true,
                        config: true,
                        map: true
                    };

                eachProp(cfg, function (value, prop) {
                    if (objs[prop]) {
                        if (prop === 'map') {
                            if (!config.map) {
                                config.map = {};
                            }
                            mixin(config[prop], value, true, true);
                        } else {
                            mixin(config[prop], value, true);
                        }
                    } else {
                        config[prop] = value;
                    }
                });

                //Merge shim
                if (cfg.shim) {
                    eachProp(cfg.shim, function (value, id) {
                        //Normalize the structure
                        if (isArray(value)) {
                            value = {
                                deps: value
                            };
                        }
                        if ((value.exports || value.init) && !value.exportsFn) {
                            value.exportsFn = context.makeShimExports(value);
                        }
                        shim[id] = value;
                    });
                    config.shim = shim;
                }

                //Adjust packages if necessary.
                if (cfg.packages) {
                    each(cfg.packages, function (pkgObj) {
                        var location;

                        pkgObj = typeof pkgObj === 'string' ? { name: pkgObj } : pkgObj;
                        location = pkgObj.location;

                        //Create a brand new object on pkgs, since currentPackages can
                        //be passed in again, and config.pkgs is the internal transformed
                        //state for all package configs.
                        pkgs[pkgObj.name] = {
                            name: pkgObj.name,
                            location: location || pkgObj.name,
                            //Remove leading dot in main, so main paths are normalized,
                            //and remove any trailing .js, since different package
                            //envs have different conventions: some use a module name,
                            //some use a file name.
                            main: (pkgObj.main || 'main')
                                  .replace(currDirRegExp, '')
                                  .replace(jsSuffixRegExp, '')
                        };
                    });

                    //Done with modifications, assing packages back to context config
                    config.pkgs = pkgs;
                }

                //If there are any "waiting to execute" modules in the registry,
                //update the maps for them, since their info, like URLs to load,
                //may have changed.
                eachProp(registry, function (mod, id) {
                    //If module already has init called, since it is too
                    //late to modify them, and ignore unnormalized ones
                    //since they are transient.
                    if (!mod.inited && !mod.map.unnormalized) {
                        mod.map = makeModuleMap(id);
                    }
                });

                //If a deps array or a config callback is specified, then call
                //require with those args. This is useful when require is defined as a
                //config object before require.js is loaded.
                if (cfg.deps || cfg.callback) {
                    context.require(cfg.deps || [], cfg.callback);
                }
            },

            makeShimExports: function (value) {
                function fn() {
                    var ret;
                    if (value.init) {
                        ret = value.init.apply(global, arguments);
                    }
                    return ret || (value.exports && getGlobal(value.exports));
                }
                return fn;
            },

            makeRequire: function (relMap, options) {
                options = options || {};

                function localRequire(deps, callback, errback) {
                    var id, map, requireMod;

                    if (options.enableBuildCallback && callback && isFunction(callback)) {
                        callback.__requireJsBuild = true;
                    }

                    if (typeof deps === 'string') {
                        if (isFunction(callback)) {
                            //Invalid call
                            return onError(makeError('requireargs', 'Invalid require call'), errback);
                        }

                        //If require|exports|module are requested, get the
                        //value for them from the special handlers. Caveat:
                        //this only works while module is being defined.
                        if (relMap && hasProp(handlers, deps)) {
                            return handlers[deps](registry[relMap.id]);
                        }

                        //Synchronous access to one module. If require.get is
                        //available (as in the Node adapter), prefer that.
                        if (req.get) {
                            return req.get(context, deps, relMap, localRequire);
                        }

                        //Normalize module name, if it contains . or ..
                        map = makeModuleMap(deps, relMap, false, true);
                        id = map.id;

                        if (!hasProp(defined, id)) {
                            return onError(makeError('notloaded', 'Module name "' +
                                        id +
                                        '" has not been loaded yet for context: ' +
                                        contextName +
                                        (relMap ? '' : '. Use require([])')));
                        }
                        return defined[id];
                    }

                    //Grab defines waiting in the global queue.
                    intakeDefines();

                    //Mark all the dependencies as needing to be loaded.
                    context.nextTick(function () {
                        //Some defines could have been added since the
                        //require call, collect them.
                        intakeDefines();

                        requireMod = getModule(makeModuleMap(null, relMap));

                        //Store if map config should be applied to this require
                        //call for dependencies.
                        requireMod.skipMap = options.skipMap;

                        requireMod.init(deps, callback, errback, {
                            enabled: true
                        });

                        checkLoaded();
                    });

                    return localRequire;
                }

                mixin(localRequire, {
                    isBrowser: isBrowser,

                    /**
                     * Converts a module name + .extension into an URL path.
                     * *Requires* the use of a module name. It does not support using
                     * plain URLs like nameToUrl.
                     */
                    toUrl: function (moduleNamePlusExt) {
                        var ext,
                            index = moduleNamePlusExt.lastIndexOf('.'),
                            segment = moduleNamePlusExt.split('/')[0],
                            isRelative = segment === '.' || segment === '..';

                        //Have a file extension alias, and it is not the
                        //dots from a relative path.
                        if (index !== -1 && (!isRelative || index > 1)) {
                            ext = moduleNamePlusExt.substring(index, moduleNamePlusExt.length);
                            moduleNamePlusExt = moduleNamePlusExt.substring(0, index);
                        }

                        return context.nameToUrl(normalize(moduleNamePlusExt,
                                                relMap && relMap.id, true), ext,  true);
                    },

                    defined: function (id) {
                        return hasProp(defined, makeModuleMap(id, relMap, false, true).id);
                    },

                    specified: function (id) {
                        id = makeModuleMap(id, relMap, false, true).id;
                        return hasProp(defined, id) || hasProp(registry, id);
                    }
                });

                //Only allow undef on top level require calls
                if (!relMap) {
                    localRequire.undef = function (id) {
                        //Bind any waiting define() calls to this context,
                        //fix for #408
                        takeGlobalQueue();

                        var map = makeModuleMap(id, relMap, true),
                            mod = getOwn(registry, id);

                        removeScript(id);

                        delete defined[id];
                        delete urlFetched[map.url];
                        delete undefEvents[id];

                        if (mod) {
                            //Hold on to listeners in case the
                            //module will be attempted to be reloaded
                            //using a different config.
                            if (mod.events.defined) {
                                undefEvents[id] = mod.events;
                            }

                            cleanRegistry(id);
                        }
                    };
                }

                return localRequire;
            },

            /**
             * Called to enable a module if it is still in the registry
             * awaiting enablement. A second arg, parent, the parent module,
             * is passed in for context, when this method is overriden by
             * the optimizer. Not shown here to keep code compact.
             */
            enable: function (depMap) {
                var mod = getOwn(registry, depMap.id);
                if (mod) {
                    getModule(depMap).enable();
                }
            },

            /**
             * Internal method used by environment adapters to complete a load event.
             * A load event could be a script load or just a load pass from a synchronous
             * load call.
             * @param {String} moduleName the name of the module to potentially complete.
             */
            completeLoad: function (moduleName) {
                var found, args, mod,
                    shim = getOwn(config.shim, moduleName) || {},
                    shExports = shim.exports;

                takeGlobalQueue();

                while (defQueue.length) {
                    args = defQueue.shift();
                    if (args[0] === null) {
                        args[0] = moduleName;
                        //If already found an anonymous module and bound it
                        //to this name, then this is some other anon module
                        //waiting for its completeLoad to fire.
                        if (found) {
                            break;
                        }
                        found = true;
                    } else if (args[0] === moduleName) {
                        //Found matching define call for this script!
                        found = true;
                    }

                    callGetModule(args);
                }

                //Do this after the cycle of callGetModule in case the result
                //of those calls/init calls changes the registry.
                mod = getOwn(registry, moduleName);

                if (!found && !hasProp(defined, moduleName) && mod && !mod.inited) {
                    if (config.enforceDefine && (!shExports || !getGlobal(shExports))) {
                        if (hasPathFallback(moduleName)) {
                            return;
                        } else {
                            return onError(makeError('nodefine',
                                             'No define call for ' + moduleName,
                                             null,
                                             [moduleName]));
                        }
                    } else {
                        //A script that does not call define(), so just simulate
                        //the call for it.
                        callGetModule([moduleName, (shim.deps || []), shim.exportsFn]);
                    }
                }

                checkLoaded();
            },

            /**
             * Converts a module name to a file path. Supports cases where
             * moduleName may actually be just an URL.
             * Note that it **does not** call normalize on the moduleName,
             * it is assumed to have already been normalized. This is an
             * internal API, not a public one. Use toUrl for the public API.
             */
            nameToUrl: function (moduleName, ext, skipExt) {
                var paths, pkgs, pkg, pkgPath, syms, i, parentModule, url,
                    parentPath;

                //If a colon is in the URL, it indicates a protocol is used and it is just
                //an URL to a file, or if it starts with a slash, contains a query arg (i.e. ?)
                //or ends with .js, then assume the user meant to use an url and not a module id.
                //The slash is important for protocol-less URLs as well as full paths.
                if (req.jsExtRegExp.test(moduleName)) {
                    //Just a plain path, not module name lookup, so just return it.
                    //Add extension if it is included. This is a bit wonky, only non-.js things pass
                    //an extension, this method probably needs to be reworked.
                    url = moduleName + (ext || '');
                } else {
                    //A module that needs to be converted to a path.
                    paths = config.paths;
                    pkgs = config.pkgs;

                    syms = moduleName.split('/');
                    //For each module name segment, see if there is a path
                    //registered for it. Start with most specific name
                    //and work up from it.
                    for (i = syms.length; i > 0; i -= 1) {
                        parentModule = syms.slice(0, i).join('/');
                        pkg = getOwn(pkgs, parentModule);
                        parentPath = getOwn(paths, parentModule);
                        if (parentPath) {
                            //If an array, it means there are a few choices,
                            //Choose the one that is desired
                            if (isArray(parentPath)) {
                                parentPath = parentPath[0];
                            }
                            syms.splice(0, i, parentPath);
                            break;
                        } else if (pkg) {
                            //If module name is just the package name, then looking
                            //for the main module.
                            if (moduleName === pkg.name) {
                                pkgPath = pkg.location + '/' + pkg.main;
                            } else {
                                pkgPath = pkg.location;
                            }
                            syms.splice(0, i, pkgPath);
                            break;
                        }
                    }

                    //Join the path parts together, then figure out if baseUrl is needed.
                    url = syms.join('/');
                    url += (ext || (/^data\:|\?/.test(url) || skipExt ? '' : '.js'));
                    url = (url.charAt(0) === '/' || url.match(/^[\w\+\.\-]+:/) ? '' : config.baseUrl) + url;
                }

                return config.urlArgs ? url +
                                        ((url.indexOf('?') === -1 ? '?' : '&') +
                                         config.urlArgs) : url;
            },

            //Delegates to req.load. Broken out as a separate function to
            //allow overriding in the optimizer.
            load: function (id, url) {
                req.load(context, id, url);
            },

            /**
             * Executes a module callback function. Broken out as a separate function
             * solely to allow the build system to sequence the files in the built
             * layer in the right sequence.
             *
             * @private
             */
            execCb: function (name, callback, args, exports) {
                return callback.apply(exports, args);
            },

            /**
             * callback for script loads, used to check status of loading.
             *
             * @param {Event} evt the event from the browser for the script
             * that was loaded.
             */
            onScriptLoad: function (evt) {
                //Using currentTarget instead of target for Firefox 2.0's sake. Not
                //all old browsers will be supported, but this one was easy enough
                //to support and still makes sense.
                if (evt.type === 'load' ||
                        (readyRegExp.test((evt.currentTarget || evt.srcElement).readyState))) {
                    //Reset interactive script so a script node is not held onto for
                    //to long.
                    interactiveScript = null;

                    //Pull out the name of the module and the context.
                    var data = getScriptData(evt);
                    context.completeLoad(data.id);
                }
            },

            /**
             * Callback for script errors.
             */
            onScriptError: function (evt) {
                var data = getScriptData(evt);
                if (!hasPathFallback(data.id)) {
                    return onError(makeError('scripterror', 'Script error for: ' + data.id, evt, [data.id]));
                }
            }
        };

        context.require = context.makeRequire();
        return context;
    }

    /**
     * Main entry point.
     *
     * If the only argument to require is a string, then the module that
     * is represented by that string is fetched for the appropriate context.
     *
     * If the first argument is an array, then it will be treated as an array
     * of dependency string names to fetch. An optional function callback can
     * be specified to execute when all of those dependencies are available.
     *
     * Make a local req variable to help Caja compliance (it assumes things
     * on a require that are not standardized), and to give a short
     * name for minification/local scope use.
     */
    req = requirejs = function (deps, callback, errback, optional) {

        //Find the right context, use default
        var context, config,
            contextName = defContextName;

        // Determine if have config object in the call.
        if (!isArray(deps) && typeof deps !== 'string') {
            // deps is a config object
            config = deps;
            if (isArray(callback)) {
                // Adjust args if there are dependencies
                deps = callback;
                callback = errback;
                errback = optional;
            } else {
                deps = [];
            }
        }

        if (config && config.context) {
            contextName = config.context;
        }

        context = getOwn(contexts, contextName);
        if (!context) {
            context = contexts[contextName] = req.s.newContext(contextName);
        }

        if (config) {
            context.configure(config);
        }

        return context.require(deps, callback, errback);
    };

    /**
     * Support require.config() to make it easier to cooperate with other
     * AMD loaders on globally agreed names.
     */
    req.config = function (config) {
        return req(config);
    };

    /**
     * Execute something after the current tick
     * of the event loop. Override for other envs
     * that have a better solution than setTimeout.
     * @param  {Function} fn function to execute later.
     */
    req.nextTick = typeof setTimeout !== 'undefined' ? function (fn) {
        setTimeout(fn, 4);
    } : function (fn) { fn(); };

    /**
     * Export require as a global, but only if it does not already exist.
     */
    if (!require) {
        require = req;
    }

    req.version = version;

    //Used to filter out dependencies that are already paths.
    req.jsExtRegExp = /^\/|:|\?|\.js$/;
    req.isBrowser = isBrowser;
    s = req.s = {
        contexts: contexts,
        newContext: newContext
    };

    //Create default context.
    req({});

    //Exports some context-sensitive methods on global require.
    each([
        'toUrl',
        'undef',
        'defined',
        'specified'
    ], function (prop) {
        //Reference from contexts instead of early binding to default context,
        //so that during builds, the latest instance of the default context
        //with its config gets used.
        req[prop] = function () {
            var ctx = contexts[defContextName];
            return ctx.require[prop].apply(ctx, arguments);
        };
    });

    if (isBrowser) {
        head = s.head = document.getElementsByTagName('head')[0];
        //If BASE tag is in play, using appendChild is a problem for IE6.
        //When that browser dies, this can be removed. Details in this jQuery bug:
        //http://dev.jquery.com/ticket/2709
        baseElement = document.getElementsByTagName('base')[0];
        if (baseElement) {
            head = s.head = baseElement.parentNode;
        }
    }

    /**
     * Any errors that require explicitly generates will be passed to this
     * function. Intercept/override it if you want custom error handling.
     * @param {Error} err the error object.
     */
    req.onError = defaultOnError;

    /**
     * Creates the node for the load command. Only used in browser envs.
     */
    req.createNode = function (config, moduleName, url) {
        var node = config.xhtml ?
                document.createElementNS('http://www.w3.org/1999/xhtml', 'html:script') :
                document.createElement('script');
        node.type = config.scriptType || 'text/javascript';
        node.charset = 'utf-8';
        node.async = true;
        return node;
    };

    /**
     * Does the request to load a module for the browser case.
     * Make this a separate function to allow other environments
     * to override it.
     *
     * @param {Object} context the require context to find state.
     * @param {String} moduleName the name of the module.
     * @param {Object} url the URL to the module.
     */
    req.load = function (context, moduleName, url) {
        var config = (context && context.config) || {},
            node;
        if (isBrowser) {
            //In the browser so use a script tag
            node = req.createNode(config, moduleName, url);

            node.setAttribute('data-requirecontext', context.contextName);
            node.setAttribute('data-requiremodule', moduleName);

            //Set up load listener. Test attachEvent first because IE9 has
            //a subtle issue in its addEventListener and script onload firings
            //that do not match the behavior of all other browsers with
            //addEventListener support, which fire the onload event for a
            //script right after the script execution. See:
            //https://connect.microsoft.com/IE/feedback/details/648057/script-onload-event-is-not-fired-immediately-after-script-execution
            //UNFORTUNATELY Opera implements attachEvent but does not follow the script
            //script execution mode.
            if (node.attachEvent &&
                    //Check if node.attachEvent is artificially added by custom script or
                    //natively supported by browser
                    //read https://github.com/jrburke/requirejs/issues/187
                    //if we can NOT find [native code] then it must NOT natively supported.
                    //in IE8, node.attachEvent does not have toString()
                    //Note the test for "[native code" with no closing brace, see:
                    //https://github.com/jrburke/requirejs/issues/273
                    !(node.attachEvent.toString && node.attachEvent.toString().indexOf('[native code') < 0) &&
                    !isOpera) {
                //Probably IE. IE (at least 6-8) do not fire
                //script onload right after executing the script, so
                //we cannot tie the anonymous define call to a name.
                //However, IE reports the script as being in 'interactive'
                //readyState at the time of the define call.
                useInteractive = true;

                node.attachEvent('onreadystatechange', context.onScriptLoad);
                //It would be great to add an error handler here to catch
                //404s in IE9+. However, onreadystatechange will fire before
                //the error handler, so that does not help. If addEventListener
                //is used, then IE will fire error before load, but we cannot
                //use that pathway given the connect.microsoft.com issue
                //mentioned above about not doing the 'script execute,
                //then fire the script load event listener before execute
                //next script' that other browsers do.
                //Best hope: IE10 fixes the issues,
                //and then destroys all installs of IE 6-9.
                //node.attachEvent('onerror', context.onScriptError);
            } else {
                node.addEventListener('load', context.onScriptLoad, false);
                node.addEventListener('error', context.onScriptError, false);
            }
            node.src = url;

            //For some cache cases in IE 6-8, the script executes before the end
            //of the appendChild execution, so to tie an anonymous define
            //call to the module name (which is stored on the node), hold on
            //to a reference to this node, but clear after the DOM insertion.
            currentlyAddingScript = node;
            if (baseElement) {
                head.insertBefore(node, baseElement);
            } else {
                head.appendChild(node);
            }
            currentlyAddingScript = null;

            return node;
        } else if (isWebWorker) {
            try {
                //In a web worker, use importScripts. This is not a very
                //efficient use of importScripts, importScripts will block until
                //its script is downloaded and evaluated. However, if web workers
                //are in play, the expectation that a build has been done so that
                //only one script needs to be loaded anyway. This may need to be
                //reevaluated if other use cases become common.
                importScripts(url);

                //Account for anonymous modules
                context.completeLoad(moduleName);
            } catch (e) {
                context.onError(makeError('importscripts',
                                'importScripts failed for ' +
                                    moduleName + ' at ' + url,
                                e,
                                [moduleName]));
            }
        }
    };

    function getInteractiveScript() {
        if (interactiveScript && interactiveScript.readyState === 'interactive') {
            return interactiveScript;
        }

        eachReverse(scripts(), function (script) {
            if (script.readyState === 'interactive') {
                return (interactiveScript = script);
            }
        });
        return interactiveScript;
    }

    //Look for a data-main script attribute, which could also adjust the baseUrl.
    if (isBrowser && !cfg.skipDataMain) {
        //Figure out baseUrl. Get it from the script tag with require.js in it.
        eachReverse(scripts(), function (script) {
            //Set the 'head' where we can append children by
            //using the script's parent.
            if (!head) {
                head = script.parentNode;
            }

            //Look for a data-main attribute to set main script for the page
            //to load. If it is there, the path to data main becomes the
            //baseUrl, if it is not already set.
            dataMain = script.getAttribute('data-main');
            if (dataMain) {
                //Preserve dataMain in case it is a path (i.e. contains '?')
                mainScript = dataMain;

                //Set final baseUrl if there is not already an explicit one.
                if (!cfg.baseUrl) {
                    //Pull off the directory of data-main for use as the
                    //baseUrl.
                    src = mainScript.split('/');
                    mainScript = src.pop();
                    subPath = src.length ? src.join('/')  + '/' : './';

                    cfg.baseUrl = subPath;
                }

                //Strip off any trailing .js since mainScript is now
                //like a module name.
                mainScript = mainScript.replace(jsSuffixRegExp, '');

                 //If mainScript is still a path, fall back to dataMain
                if (req.jsExtRegExp.test(mainScript)) {
                    mainScript = dataMain;
                }

                //Put the data-main script in the files to load.
                cfg.deps = cfg.deps ? cfg.deps.concat(mainScript) : [mainScript];

                return true;
            }
        });
    }

    /**
     * The function that handles definitions of modules. Differs from
     * require() in that a string for the module should be the first argument,
     * and the function to execute after dependencies are loaded should
     * return a value to define the module corresponding to the first argument's
     * name.
     */
    define = function (name, deps, callback) {
        var node, context;

        //Allow for anonymous modules
        if (typeof name !== 'string') {
            //Adjust args appropriately
            callback = deps;
            deps = name;
            name = null;
        }

        //This module may not have dependencies
        if (!isArray(deps)) {
            callback = deps;
            deps = null;
        }

        //If no name, and callback is a function, then figure out if it a
        //CommonJS thing with dependencies.
        if (!deps && isFunction(callback)) {
            deps = [];
            //Remove comments from the callback string,
            //look for require calls, and pull them into the dependencies,
            //but only if there are function args.
            if (callback.length) {
                callback
                    .toString()
                    .replace(commentRegExp, '')
                    .replace(cjsRequireRegExp, function (match, dep) {
                        deps.push(dep);
                    });

                //May be a CommonJS thing even without require calls, but still
                //could use exports, and module. Avoid doing exports and module
                //work though if it just needs require.
                //REQUIRES the function to expect the CommonJS variables in the
                //order listed below.
                deps = (callback.length === 1 ? ['require'] : ['require', 'exports', 'module']).concat(deps);
            }
        }

        //If in IE 6-8 and hit an anonymous define() call, do the interactive
        //work.
        if (useInteractive) {
            node = currentlyAddingScript || getInteractiveScript();
            if (node) {
                if (!name) {
                    name = node.getAttribute('data-requiremodule');
                }
                context = contexts[node.getAttribute('data-requirecontext')];
            }
        }

        //Always save off evaluating the def call until the script onload handler.
        //This allows multiple modules to be in a file without prematurely
        //tracing dependencies, and allows for anonymous module support,
        //where the module name is not known until the script onload event
        //occurs. If no context, use the global queue, and get it processed
        //in the onscript load callback.
        (context ? context.defQueue : globalDefQueue).push([name, deps, callback]);
    };

    define.amd = {
        jQuery: true
    };


    /**
     * Executes the text. Normally just uses eval, but can be modified
     * to use a better, environment-specific call. Only used for transpiling
     * loader plugins, not for plain JS modules.
     * @param {String} text the text to execute/evaluate.
     */
    req.exec = function (text) {
        /*jslint evil: true */
        return eval(text);
    };

    //Set up with config info.
    req(cfg);
}(this));

define("requirejs", function(){});

/*!
 * jQuery JavaScript Library v1.10.2
 * http://jquery.com/
 *
 * Includes Sizzle.js
 * http://sizzlejs.com/
 *
 * Copyright 2005, 2013 jQuery Foundation, Inc. and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2013-07-03T13:48Z
 */
(function( window, undefined ) {

// Can't do this because several apps including ASP.NET trace
// the stack via arguments.caller.callee and Firefox dies if
// you try to trace through "use strict" call chains. (#13335)
// Support: Firefox 18+
//"use strict";
var
	// The deferred used on DOM ready
	readyList,

	// A central reference to the root jQuery(document)
	rootjQuery,

	// Support: IE<10
	// For `typeof xmlNode.method` instead of `xmlNode.method !== undefined`
	core_strundefined = typeof undefined,

	// Use the correct document accordingly with window argument (sandbox)
	location = window.location,
	document = window.document,
	docElem = document.documentElement,

	// Map over jQuery in case of overwrite
	_jQuery = window.jQuery,

	// Map over the $ in case of overwrite
	_$ = window.$,

	// [[Class]] -> type pairs
	class2type = {},

	// List of deleted data cache ids, so we can reuse them
	core_deletedIds = [],

	core_version = "1.10.2",

	// Save a reference to some core methods
	core_concat = core_deletedIds.concat,
	core_push = core_deletedIds.push,
	core_slice = core_deletedIds.slice,
	core_indexOf = core_deletedIds.indexOf,
	core_toString = class2type.toString,
	core_hasOwn = class2type.hasOwnProperty,
	core_trim = core_version.trim,

	// Define a local copy of jQuery
	jQuery = function( selector, context ) {
		// The jQuery object is actually just the init constructor 'enhanced'
		return new jQuery.fn.init( selector, context, rootjQuery );
	},

	// Used for matching numbers
	core_pnum = /[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,

	// Used for splitting on whitespace
	core_rnotwhite = /\S+/g,

	// Make sure we trim BOM and NBSP (here's looking at you, Safari 5.0 and IE)
	rtrim = /^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,

	// A simple way to check for HTML strings
	// Prioritize #id over <tag> to avoid XSS via location.hash (#9521)
	// Strict HTML recognition (#11290: must start with <)
	rquickExpr = /^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]*))$/,

	// Match a standalone tag
	rsingleTag = /^<(\w+)\s*\/?>(?:<\/\1>|)$/,

	// JSON RegExp
	rvalidchars = /^[\],:{}\s]*$/,
	rvalidbraces = /(?:^|:|,)(?:\s*\[)+/g,
	rvalidescape = /\\(?:["\\\/bfnrt]|u[\da-fA-F]{4})/g,
	rvalidtokens = /"[^"\\\r\n]*"|true|false|null|-?(?:\d+\.|)\d+(?:[eE][+-]?\d+|)/g,

	// Matches dashed string for camelizing
	rmsPrefix = /^-ms-/,
	rdashAlpha = /-([\da-z])/gi,

	// Used by jQuery.camelCase as callback to replace()
	fcamelCase = function( all, letter ) {
		return letter.toUpperCase();
	},

	// The ready event handler
	completed = function( event ) {

		// readyState === "complete" is good enough for us to call the dom ready in oldIE
		if ( document.addEventListener || event.type === "load" || document.readyState === "complete" ) {
			detach();
			jQuery.ready();
		}
	},
	// Clean-up method for dom ready events
	detach = function() {
		if ( document.addEventListener ) {
			document.removeEventListener( "DOMContentLoaded", completed, false );
			window.removeEventListener( "load", completed, false );

		} else {
			document.detachEvent( "onreadystatechange", completed );
			window.detachEvent( "onload", completed );
		}
	};

jQuery.fn = jQuery.prototype = {
	// The current version of jQuery being used
	jquery: core_version,

	constructor: jQuery,
	init: function( selector, context, rootjQuery ) {
		var match, elem;

		// HANDLE: $(""), $(null), $(undefined), $(false)
		if ( !selector ) {
			return this;
		}

		// Handle HTML strings
		if ( typeof selector === "string" ) {
			if ( selector.charAt(0) === "<" && selector.charAt( selector.length - 1 ) === ">" && selector.length >= 3 ) {
				// Assume that strings that start and end with <> are HTML and skip the regex check
				match = [ null, selector, null ];

			} else {
				match = rquickExpr.exec( selector );
			}

			// Match html or make sure no context is specified for #id
			if ( match && (match[1] || !context) ) {

				// HANDLE: $(html) -> $(array)
				if ( match[1] ) {
					context = context instanceof jQuery ? context[0] : context;

					// scripts is true for back-compat
					jQuery.merge( this, jQuery.parseHTML(
						match[1],
						context && context.nodeType ? context.ownerDocument || context : document,
						true
					) );

					// HANDLE: $(html, props)
					if ( rsingleTag.test( match[1] ) && jQuery.isPlainObject( context ) ) {
						for ( match in context ) {
							// Properties of context are called as methods if possible
							if ( jQuery.isFunction( this[ match ] ) ) {
								this[ match ]( context[ match ] );

							// ...and otherwise set as attributes
							} else {
								this.attr( match, context[ match ] );
							}
						}
					}

					return this;

				// HANDLE: $(#id)
				} else {
					elem = document.getElementById( match[2] );

					// Check parentNode to catch when Blackberry 4.6 returns
					// nodes that are no longer in the document #6963
					if ( elem && elem.parentNode ) {
						// Handle the case where IE and Opera return items
						// by name instead of ID
						if ( elem.id !== match[2] ) {
							return rootjQuery.find( selector );
						}

						// Otherwise, we inject the element directly into the jQuery object
						this.length = 1;
						this[0] = elem;
					}

					this.context = document;
					this.selector = selector;
					return this;
				}

			// HANDLE: $(expr, $(...))
			} else if ( !context || context.jquery ) {
				return ( context || rootjQuery ).find( selector );

			// HANDLE: $(expr, context)
			// (which is just equivalent to: $(context).find(expr)
			} else {
				return this.constructor( context ).find( selector );
			}

		// HANDLE: $(DOMElement)
		} else if ( selector.nodeType ) {
			this.context = this[0] = selector;
			this.length = 1;
			return this;

		// HANDLE: $(function)
		// Shortcut for document ready
		} else if ( jQuery.isFunction( selector ) ) {
			return rootjQuery.ready( selector );
		}

		if ( selector.selector !== undefined ) {
			this.selector = selector.selector;
			this.context = selector.context;
		}

		return jQuery.makeArray( selector, this );
	},

	// Start with an empty selector
	selector: "",

	// The default length of a jQuery object is 0
	length: 0,

	toArray: function() {
		return core_slice.call( this );
	},

	// Get the Nth element in the matched element set OR
	// Get the whole matched element set as a clean array
	get: function( num ) {
		return num == null ?

			// Return a 'clean' array
			this.toArray() :

			// Return just the object
			( num < 0 ? this[ this.length + num ] : this[ num ] );
	},

	// Take an array of elements and push it onto the stack
	// (returning the new matched element set)
	pushStack: function( elems ) {

		// Build a new jQuery matched element set
		var ret = jQuery.merge( this.constructor(), elems );

		// Add the old object onto the stack (as a reference)
		ret.prevObject = this;
		ret.context = this.context;

		// Return the newly-formed element set
		return ret;
	},

	// Execute a callback for every element in the matched set.
	// (You can seed the arguments with an array of args, but this is
	// only used internally.)
	each: function( callback, args ) {
		return jQuery.each( this, callback, args );
	},

	ready: function( fn ) {
		// Add the callback
		jQuery.ready.promise().done( fn );

		return this;
	},

	slice: function() {
		return this.pushStack( core_slice.apply( this, arguments ) );
	},

	first: function() {
		return this.eq( 0 );
	},

	last: function() {
		return this.eq( -1 );
	},

	eq: function( i ) {
		var len = this.length,
			j = +i + ( i < 0 ? len : 0 );
		return this.pushStack( j >= 0 && j < len ? [ this[j] ] : [] );
	},

	map: function( callback ) {
		return this.pushStack( jQuery.map(this, function( elem, i ) {
			return callback.call( elem, i, elem );
		}));
	},

	end: function() {
		return this.prevObject || this.constructor(null);
	},

	// For internal use only.
	// Behaves like an Array's method, not like a jQuery method.
	push: core_push,
	sort: [].sort,
	splice: [].splice
};

// Give the init function the jQuery prototype for later instantiation
jQuery.fn.init.prototype = jQuery.fn;

jQuery.extend = jQuery.fn.extend = function() {
	var src, copyIsArray, copy, name, options, clone,
		target = arguments[0] || {},
		i = 1,
		length = arguments.length,
		deep = false;

	// Handle a deep copy situation
	if ( typeof target === "boolean" ) {
		deep = target;
		target = arguments[1] || {};
		// skip the boolean and the target
		i = 2;
	}

	// Handle case when target is a string or something (possible in deep copy)
	if ( typeof target !== "object" && !jQuery.isFunction(target) ) {
		target = {};
	}

	// extend jQuery itself if only one argument is passed
	if ( length === i ) {
		target = this;
		--i;
	}

	for ( ; i < length; i++ ) {
		// Only deal with non-null/undefined values
		if ( (options = arguments[ i ]) != null ) {
			// Extend the base object
			for ( name in options ) {
				src = target[ name ];
				copy = options[ name ];

				// Prevent never-ending loop
				if ( target === copy ) {
					continue;
				}

				// Recurse if we're merging plain objects or arrays
				if ( deep && copy && ( jQuery.isPlainObject(copy) || (copyIsArray = jQuery.isArray(copy)) ) ) {
					if ( copyIsArray ) {
						copyIsArray = false;
						clone = src && jQuery.isArray(src) ? src : [];

					} else {
						clone = src && jQuery.isPlainObject(src) ? src : {};
					}

					// Never move original objects, clone them
					target[ name ] = jQuery.extend( deep, clone, copy );

				// Don't bring in undefined values
				} else if ( copy !== undefined ) {
					target[ name ] = copy;
				}
			}
		}
	}

	// Return the modified object
	return target;
};

jQuery.extend({
	// Unique for each copy of jQuery on the page
	// Non-digits removed to match rinlinejQuery
	expando: "jQuery" + ( core_version + Math.random() ).replace( /\D/g, "" ),

	noConflict: function( deep ) {
		if ( window.$ === jQuery ) {
			window.$ = _$;
		}

		if ( deep && window.jQuery === jQuery ) {
			window.jQuery = _jQuery;
		}

		return jQuery;
	},

	// Is the DOM ready to be used? Set to true once it occurs.
	isReady: false,

	// A counter to track how many items to wait for before
	// the ready event fires. See #6781
	readyWait: 1,

	// Hold (or release) the ready event
	holdReady: function( hold ) {
		if ( hold ) {
			jQuery.readyWait++;
		} else {
			jQuery.ready( true );
		}
	},

	// Handle when the DOM is ready
	ready: function( wait ) {

		// Abort if there are pending holds or we're already ready
		if ( wait === true ? --jQuery.readyWait : jQuery.isReady ) {
			return;
		}

		// Make sure body exists, at least, in case IE gets a little overzealous (ticket #5443).
		if ( !document.body ) {
			return setTimeout( jQuery.ready );
		}

		// Remember that the DOM is ready
		jQuery.isReady = true;

		// If a normal DOM Ready event fired, decrement, and wait if need be
		if ( wait !== true && --jQuery.readyWait > 0 ) {
			return;
		}

		// If there are functions bound, to execute
		readyList.resolveWith( document, [ jQuery ] );

		// Trigger any bound ready events
		if ( jQuery.fn.trigger ) {
			jQuery( document ).trigger("ready").off("ready");
		}
	},

	// See test/unit/core.js for details concerning isFunction.
	// Since version 1.3, DOM methods and functions like alert
	// aren't supported. They return false on IE (#2968).
	isFunction: function( obj ) {
		return jQuery.type(obj) === "function";
	},

	isArray: Array.isArray || function( obj ) {
		return jQuery.type(obj) === "array";
	},

	isWindow: function( obj ) {
		/* jshint eqeqeq: false */
		return obj != null && obj == obj.window;
	},

	isNumeric: function( obj ) {
		return !isNaN( parseFloat(obj) ) && isFinite( obj );
	},

	type: function( obj ) {
		if ( obj == null ) {
			return String( obj );
		}
		return typeof obj === "object" || typeof obj === "function" ?
			class2type[ core_toString.call(obj) ] || "object" :
			typeof obj;
	},

	isPlainObject: function( obj ) {
		var key;

		// Must be an Object.
		// Because of IE, we also have to check the presence of the constructor property.
		// Make sure that DOM nodes and window objects don't pass through, as well
		if ( !obj || jQuery.type(obj) !== "object" || obj.nodeType || jQuery.isWindow( obj ) ) {
			return false;
		}

		try {
			// Not own constructor property must be Object
			if ( obj.constructor &&
				!core_hasOwn.call(obj, "constructor") &&
				!core_hasOwn.call(obj.constructor.prototype, "isPrototypeOf") ) {
				return false;
			}
		} catch ( e ) {
			// IE8,9 Will throw exceptions on certain host objects #9897
			return false;
		}

		// Support: IE<9
		// Handle iteration over inherited properties before own properties.
		if ( jQuery.support.ownLast ) {
			for ( key in obj ) {
				return core_hasOwn.call( obj, key );
			}
		}

		// Own properties are enumerated firstly, so to speed up,
		// if last one is own, then all properties are own.
		for ( key in obj ) {}

		return key === undefined || core_hasOwn.call( obj, key );
	},

	isEmptyObject: function( obj ) {
		var name;
		for ( name in obj ) {
			return false;
		}
		return true;
	},

	error: function( msg ) {
		throw new Error( msg );
	},

	// data: string of html
	// context (optional): If specified, the fragment will be created in this context, defaults to document
	// keepScripts (optional): If true, will include scripts passed in the html string
	parseHTML: function( data, context, keepScripts ) {
		if ( !data || typeof data !== "string" ) {
			return null;
		}
		if ( typeof context === "boolean" ) {
			keepScripts = context;
			context = false;
		}
		context = context || document;

		var parsed = rsingleTag.exec( data ),
			scripts = !keepScripts && [];

		// Single tag
		if ( parsed ) {
			return [ context.createElement( parsed[1] ) ];
		}

		parsed = jQuery.buildFragment( [ data ], context, scripts );
		if ( scripts ) {
			jQuery( scripts ).remove();
		}
		return jQuery.merge( [], parsed.childNodes );
	},

	parseJSON: function( data ) {
		// Attempt to parse using the native JSON parser first
		if ( window.JSON && window.JSON.parse ) {
			return window.JSON.parse( data );
		}

		if ( data === null ) {
			return data;
		}

		if ( typeof data === "string" ) {

			// Make sure leading/trailing whitespace is removed (IE can't handle it)
			data = jQuery.trim( data );

			if ( data ) {
				// Make sure the incoming data is actual JSON
				// Logic borrowed from http://json.org/json2.js
				if ( rvalidchars.test( data.replace( rvalidescape, "@" )
					.replace( rvalidtokens, "]" )
					.replace( rvalidbraces, "")) ) {

					return ( new Function( "return " + data ) )();
				}
			}
		}

		jQuery.error( "Invalid JSON: " + data );
	},

	// Cross-browser xml parsing
	parseXML: function( data ) {
		var xml, tmp;
		if ( !data || typeof data !== "string" ) {
			return null;
		}
		try {
			if ( window.DOMParser ) { // Standard
				tmp = new DOMParser();
				xml = tmp.parseFromString( data , "text/xml" );
			} else { // IE
				xml = new ActiveXObject( "Microsoft.XMLDOM" );
				xml.async = "false";
				xml.loadXML( data );
			}
		} catch( e ) {
			xml = undefined;
		}
		if ( !xml || !xml.documentElement || xml.getElementsByTagName( "parsererror" ).length ) {
			jQuery.error( "Invalid XML: " + data );
		}
		return xml;
	},

	noop: function() {},

	// Evaluates a script in a global context
	// Workarounds based on findings by Jim Driscoll
	// http://weblogs.java.net/blog/driscoll/archive/2009/09/08/eval-javascript-global-context
	globalEval: function( data ) {
		if ( data && jQuery.trim( data ) ) {
			// We use execScript on Internet Explorer
			// We use an anonymous function so that context is window
			// rather than jQuery in Firefox
			( window.execScript || function( data ) {
				window[ "eval" ].call( window, data );
			} )( data );
		}
	},

	// Convert dashed to camelCase; used by the css and data modules
	// Microsoft forgot to hump their vendor prefix (#9572)
	camelCase: function( string ) {
		return string.replace( rmsPrefix, "ms-" ).replace( rdashAlpha, fcamelCase );
	},

	nodeName: function( elem, name ) {
		return elem.nodeName && elem.nodeName.toLowerCase() === name.toLowerCase();
	},

	// args is for internal usage only
	each: function( obj, callback, args ) {
		var value,
			i = 0,
			length = obj.length,
			isArray = isArraylike( obj );

		if ( args ) {
			if ( isArray ) {
				for ( ; i < length; i++ ) {
					value = callback.apply( obj[ i ], args );

					if ( value === false ) {
						break;
					}
				}
			} else {
				for ( i in obj ) {
					value = callback.apply( obj[ i ], args );

					if ( value === false ) {
						break;
					}
				}
			}

		// A special, fast, case for the most common use of each
		} else {
			if ( isArray ) {
				for ( ; i < length; i++ ) {
					value = callback.call( obj[ i ], i, obj[ i ] );

					if ( value === false ) {
						break;
					}
				}
			} else {
				for ( i in obj ) {
					value = callback.call( obj[ i ], i, obj[ i ] );

					if ( value === false ) {
						break;
					}
				}
			}
		}

		return obj;
	},

	// Use native String.trim function wherever possible
	trim: core_trim && !core_trim.call("\uFEFF\xA0") ?
		function( text ) {
			return text == null ?
				"" :
				core_trim.call( text );
		} :

		// Otherwise use our own trimming functionality
		function( text ) {
			return text == null ?
				"" :
				( text + "" ).replace( rtrim, "" );
		},

	// results is for internal usage only
	makeArray: function( arr, results ) {
		var ret = results || [];

		if ( arr != null ) {
			if ( isArraylike( Object(arr) ) ) {
				jQuery.merge( ret,
					typeof arr === "string" ?
					[ arr ] : arr
				);
			} else {
				core_push.call( ret, arr );
			}
		}

		return ret;
	},

	inArray: function( elem, arr, i ) {
		var len;

		if ( arr ) {
			if ( core_indexOf ) {
				return core_indexOf.call( arr, elem, i );
			}

			len = arr.length;
			i = i ? i < 0 ? Math.max( 0, len + i ) : i : 0;

			for ( ; i < len; i++ ) {
				// Skip accessing in sparse arrays
				if ( i in arr && arr[ i ] === elem ) {
					return i;
				}
			}
		}

		return -1;
	},

	merge: function( first, second ) {
		var l = second.length,
			i = first.length,
			j = 0;

		if ( typeof l === "number" ) {
			for ( ; j < l; j++ ) {
				first[ i++ ] = second[ j ];
			}
		} else {
			while ( second[j] !== undefined ) {
				first[ i++ ] = second[ j++ ];
			}
		}

		first.length = i;

		return first;
	},

	grep: function( elems, callback, inv ) {
		var retVal,
			ret = [],
			i = 0,
			length = elems.length;
		inv = !!inv;

		// Go through the array, only saving the items
		// that pass the validator function
		for ( ; i < length; i++ ) {
			retVal = !!callback( elems[ i ], i );
			if ( inv !== retVal ) {
				ret.push( elems[ i ] );
			}
		}

		return ret;
	},

	// arg is for internal usage only
	map: function( elems, callback, arg ) {
		var value,
			i = 0,
			length = elems.length,
			isArray = isArraylike( elems ),
			ret = [];

		// Go through the array, translating each of the items to their
		if ( isArray ) {
			for ( ; i < length; i++ ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret[ ret.length ] = value;
				}
			}

		// Go through every key on the object,
		} else {
			for ( i in elems ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret[ ret.length ] = value;
				}
			}
		}

		// Flatten any nested arrays
		return core_concat.apply( [], ret );
	},

	// A global GUID counter for objects
	guid: 1,

	// Bind a function to a context, optionally partially applying any
	// arguments.
	proxy: function( fn, context ) {
		var args, proxy, tmp;

		if ( typeof context === "string" ) {
			tmp = fn[ context ];
			context = fn;
			fn = tmp;
		}

		// Quick check to determine if target is callable, in the spec
		// this throws a TypeError, but we will just return undefined.
		if ( !jQuery.isFunction( fn ) ) {
			return undefined;
		}

		// Simulated bind
		args = core_slice.call( arguments, 2 );
		proxy = function() {
			return fn.apply( context || this, args.concat( core_slice.call( arguments ) ) );
		};

		// Set the guid of unique handler to the same of original handler, so it can be removed
		proxy.guid = fn.guid = fn.guid || jQuery.guid++;

		return proxy;
	},

	// Multifunctional method to get and set values of a collection
	// The value/s can optionally be executed if it's a function
	access: function( elems, fn, key, value, chainable, emptyGet, raw ) {
		var i = 0,
			length = elems.length,
			bulk = key == null;

		// Sets many values
		if ( jQuery.type( key ) === "object" ) {
			chainable = true;
			for ( i in key ) {
				jQuery.access( elems, fn, i, key[i], true, emptyGet, raw );
			}

		// Sets one value
		} else if ( value !== undefined ) {
			chainable = true;

			if ( !jQuery.isFunction( value ) ) {
				raw = true;
			}

			if ( bulk ) {
				// Bulk operations run against the entire set
				if ( raw ) {
					fn.call( elems, value );
					fn = null;

				// ...except when executing function values
				} else {
					bulk = fn;
					fn = function( elem, key, value ) {
						return bulk.call( jQuery( elem ), value );
					};
				}
			}

			if ( fn ) {
				for ( ; i < length; i++ ) {
					fn( elems[i], key, raw ? value : value.call( elems[i], i, fn( elems[i], key ) ) );
				}
			}
		}

		return chainable ?
			elems :

			// Gets
			bulk ?
				fn.call( elems ) :
				length ? fn( elems[0], key ) : emptyGet;
	},

	now: function() {
		return ( new Date() ).getTime();
	},

	// A method for quickly swapping in/out CSS properties to get correct calculations.
	// Note: this method belongs to the css module but it's needed here for the support module.
	// If support gets modularized, this method should be moved back to the css module.
	swap: function( elem, options, callback, args ) {
		var ret, name,
			old = {};

		// Remember the old values, and insert the new ones
		for ( name in options ) {
			old[ name ] = elem.style[ name ];
			elem.style[ name ] = options[ name ];
		}

		ret = callback.apply( elem, args || [] );

		// Revert the old values
		for ( name in options ) {
			elem.style[ name ] = old[ name ];
		}

		return ret;
	}
});

jQuery.ready.promise = function( obj ) {
	if ( !readyList ) {

		readyList = jQuery.Deferred();

		// Catch cases where $(document).ready() is called after the browser event has already occurred.
		// we once tried to use readyState "interactive" here, but it caused issues like the one
		// discovered by ChrisS here: http://bugs.jquery.com/ticket/12282#comment:15
		if ( document.readyState === "complete" ) {
			// Handle it asynchronously to allow scripts the opportunity to delay ready
			setTimeout( jQuery.ready );

		// Standards-based browsers support DOMContentLoaded
		} else if ( document.addEventListener ) {
			// Use the handy event callback
			document.addEventListener( "DOMContentLoaded", completed, false );

			// A fallback to window.onload, that will always work
			window.addEventListener( "load", completed, false );

		// If IE event model is used
		} else {
			// Ensure firing before onload, maybe late but safe also for iframes
			document.attachEvent( "onreadystatechange", completed );

			// A fallback to window.onload, that will always work
			window.attachEvent( "onload", completed );

			// If IE and not a frame
			// continually check to see if the document is ready
			var top = false;

			try {
				top = window.frameElement == null && document.documentElement;
			} catch(e) {}

			if ( top && top.doScroll ) {
				(function doScrollCheck() {
					if ( !jQuery.isReady ) {

						try {
							// Use the trick by Diego Perini
							// http://javascript.nwbox.com/IEContentLoaded/
							top.doScroll("left");
						} catch(e) {
							return setTimeout( doScrollCheck, 50 );
						}

						// detach all dom ready events
						detach();

						// and execute any waiting functions
						jQuery.ready();
					}
				})();
			}
		}
	}
	return readyList.promise( obj );
};

// Populate the class2type map
jQuery.each("Boolean Number String Function Array Date RegExp Object Error".split(" "), function(i, name) {
	class2type[ "[object " + name + "]" ] = name.toLowerCase();
});

function isArraylike( obj ) {
	var length = obj.length,
		type = jQuery.type( obj );

	if ( jQuery.isWindow( obj ) ) {
		return false;
	}

	if ( obj.nodeType === 1 && length ) {
		return true;
	}

	return type === "array" || type !== "function" &&
		( length === 0 ||
		typeof length === "number" && length > 0 && ( length - 1 ) in obj );
}

// All jQuery objects should point back to these
rootjQuery = jQuery(document);
/*!
 * Sizzle CSS Selector Engine v1.10.2
 * http://sizzlejs.com/
 *
 * Copyright 2013 jQuery Foundation, Inc. and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2013-07-03
 */
(function( window, undefined ) {

var i,
	support,
	cachedruns,
	Expr,
	getText,
	isXML,
	compile,
	outermostContext,
	sortInput,

	// Local document vars
	setDocument,
	document,
	docElem,
	documentIsHTML,
	rbuggyQSA,
	rbuggyMatches,
	matches,
	contains,

	// Instance-specific data
	expando = "sizzle" + -(new Date()),
	preferredDoc = window.document,
	dirruns = 0,
	done = 0,
	classCache = createCache(),
	tokenCache = createCache(),
	compilerCache = createCache(),
	hasDuplicate = false,
	sortOrder = function( a, b ) {
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}
		return 0;
	},

	// General-purpose constants
	strundefined = typeof undefined,
	MAX_NEGATIVE = 1 << 31,

	// Instance methods
	hasOwn = ({}).hasOwnProperty,
	arr = [],
	pop = arr.pop,
	push_native = arr.push,
	push = arr.push,
	slice = arr.slice,
	// Use a stripped-down indexOf if we can't use a native one
	indexOf = arr.indexOf || function( elem ) {
		var i = 0,
			len = this.length;
		for ( ; i < len; i++ ) {
			if ( this[i] === elem ) {
				return i;
			}
		}
		return -1;
	},

	booleans = "checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",

	// Regular expressions

	// Whitespace characters http://www.w3.org/TR/css3-selectors/#whitespace
	whitespace = "[\\x20\\t\\r\\n\\f]",
	// http://www.w3.org/TR/css3-syntax/#characters
	characterEncoding = "(?:\\\\.|[\\w-]|[^\\x00-\\xa0])+",

	// Loosely modeled on CSS identifier characters
	// An unquoted value should be a CSS identifier http://www.w3.org/TR/css3-selectors/#attribute-selectors
	// Proper syntax: http://www.w3.org/TR/CSS21/syndata.html#value-def-identifier
	identifier = characterEncoding.replace( "w", "w#" ),

	// Acceptable operators http://www.w3.org/TR/selectors/#attribute-selectors
	attributes = "\\[" + whitespace + "*(" + characterEncoding + ")" + whitespace +
		"*(?:([*^$|!~]?=)" + whitespace + "*(?:(['\"])((?:\\\\.|[^\\\\])*?)\\3|(" + identifier + ")|)|)" + whitespace + "*\\]",

	// Prefer arguments quoted,
	//   then not containing pseudos/brackets,
	//   then attribute selectors/non-parenthetical expressions,
	//   then anything else
	// These preferences are here to reduce the number of selectors
	//   needing tokenize in the PSEUDO preFilter
	pseudos = ":(" + characterEncoding + ")(?:\\(((['\"])((?:\\\\.|[^\\\\])*?)\\3|((?:\\\\.|[^\\\\()[\\]]|" + attributes.replace( 3, 8 ) + ")*)|.*)\\)|)",

	// Leading and non-escaped trailing whitespace, capturing some non-whitespace characters preceding the latter
	rtrim = new RegExp( "^" + whitespace + "+|((?:^|[^\\\\])(?:\\\\.)*)" + whitespace + "+$", "g" ),

	rcomma = new RegExp( "^" + whitespace + "*," + whitespace + "*" ),
	rcombinators = new RegExp( "^" + whitespace + "*([>+~]|" + whitespace + ")" + whitespace + "*" ),

	rsibling = new RegExp( whitespace + "*[+~]" ),
	rattributeQuotes = new RegExp( "=" + whitespace + "*([^\\]'\"]*)" + whitespace + "*\\]", "g" ),

	rpseudo = new RegExp( pseudos ),
	ridentifier = new RegExp( "^" + identifier + "$" ),

	matchExpr = {
		"ID": new RegExp( "^#(" + characterEncoding + ")" ),
		"CLASS": new RegExp( "^\\.(" + characterEncoding + ")" ),
		"TAG": new RegExp( "^(" + characterEncoding.replace( "w", "w*" ) + ")" ),
		"ATTR": new RegExp( "^" + attributes ),
		"PSEUDO": new RegExp( "^" + pseudos ),
		"CHILD": new RegExp( "^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\(" + whitespace +
			"*(even|odd|(([+-]|)(\\d*)n|)" + whitespace + "*(?:([+-]|)" + whitespace +
			"*(\\d+)|))" + whitespace + "*\\)|)", "i" ),
		"bool": new RegExp( "^(?:" + booleans + ")$", "i" ),
		// For use in libraries implementing .is()
		// We use this for POS matching in `select`
		"needsContext": new RegExp( "^" + whitespace + "*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\(" +
			whitespace + "*((?:-\\d)?\\d*)" + whitespace + "*\\)|)(?=[^-]|$)", "i" )
	},

	rnative = /^[^{]+\{\s*\[native \w/,

	// Easily-parseable/retrievable ID or TAG or CLASS selectors
	rquickExpr = /^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,

	rinputs = /^(?:input|select|textarea|button)$/i,
	rheader = /^h\d$/i,

	rescape = /'|\\/g,

	// CSS escapes http://www.w3.org/TR/CSS21/syndata.html#escaped-characters
	runescape = new RegExp( "\\\\([\\da-f]{1,6}" + whitespace + "?|(" + whitespace + ")|.)", "ig" ),
	funescape = function( _, escaped, escapedWhitespace ) {
		var high = "0x" + escaped - 0x10000;
		// NaN means non-codepoint
		// Support: Firefox
		// Workaround erroneous numeric interpretation of +"0x"
		return high !== high || escapedWhitespace ?
			escaped :
			// BMP codepoint
			high < 0 ?
				String.fromCharCode( high + 0x10000 ) :
				// Supplemental Plane codepoint (surrogate pair)
				String.fromCharCode( high >> 10 | 0xD800, high & 0x3FF | 0xDC00 );
	};

// Optimize for push.apply( _, NodeList )
try {
	push.apply(
		(arr = slice.call( preferredDoc.childNodes )),
		preferredDoc.childNodes
	);
	// Support: Android<4.0
	// Detect silently failing push.apply
	arr[ preferredDoc.childNodes.length ].nodeType;
} catch ( e ) {
	push = { apply: arr.length ?

		// Leverage slice if possible
		function( target, els ) {
			push_native.apply( target, slice.call(els) );
		} :

		// Support: IE<9
		// Otherwise append directly
		function( target, els ) {
			var j = target.length,
				i = 0;
			// Can't trust NodeList.length
			while ( (target[j++] = els[i++]) ) {}
			target.length = j - 1;
		}
	};
}

function Sizzle( selector, context, results, seed ) {
	var match, elem, m, nodeType,
		// QSA vars
		i, groups, old, nid, newContext, newSelector;

	if ( ( context ? context.ownerDocument || context : preferredDoc ) !== document ) {
		setDocument( context );
	}

	context = context || document;
	results = results || [];

	if ( !selector || typeof selector !== "string" ) {
		return results;
	}

	if ( (nodeType = context.nodeType) !== 1 && nodeType !== 9 ) {
		return [];
	}

	if ( documentIsHTML && !seed ) {

		// Shortcuts
		if ( (match = rquickExpr.exec( selector )) ) {
			// Speed-up: Sizzle("#ID")
			if ( (m = match[1]) ) {
				if ( nodeType === 9 ) {
					elem = context.getElementById( m );
					// Check parentNode to catch when Blackberry 4.6 returns
					// nodes that are no longer in the document #6963
					if ( elem && elem.parentNode ) {
						// Handle the case where IE, Opera, and Webkit return items
						// by name instead of ID
						if ( elem.id === m ) {
							results.push( elem );
							return results;
						}
					} else {
						return results;
					}
				} else {
					// Context is not a document
					if ( context.ownerDocument && (elem = context.ownerDocument.getElementById( m )) &&
						contains( context, elem ) && elem.id === m ) {
						results.push( elem );
						return results;
					}
				}

			// Speed-up: Sizzle("TAG")
			} else if ( match[2] ) {
				push.apply( results, context.getElementsByTagName( selector ) );
				return results;

			// Speed-up: Sizzle(".CLASS")
			} else if ( (m = match[3]) && support.getElementsByClassName && context.getElementsByClassName ) {
				push.apply( results, context.getElementsByClassName( m ) );
				return results;
			}
		}

		// QSA path
		if ( support.qsa && (!rbuggyQSA || !rbuggyQSA.test( selector )) ) {
			nid = old = expando;
			newContext = context;
			newSelector = nodeType === 9 && selector;

			// qSA works strangely on Element-rooted queries
			// We can work around this by specifying an extra ID on the root
			// and working up from there (Thanks to Andrew Dupont for the technique)
			// IE 8 doesn't work on object elements
			if ( nodeType === 1 && context.nodeName.toLowerCase() !== "object" ) {
				groups = tokenize( selector );

				if ( (old = context.getAttribute("id")) ) {
					nid = old.replace( rescape, "\\$&" );
				} else {
					context.setAttribute( "id", nid );
				}
				nid = "[id='" + nid + "'] ";

				i = groups.length;
				while ( i-- ) {
					groups[i] = nid + toSelector( groups[i] );
				}
				newContext = rsibling.test( selector ) && context.parentNode || context;
				newSelector = groups.join(",");
			}

			if ( newSelector ) {
				try {
					push.apply( results,
						newContext.querySelectorAll( newSelector )
					);
					return results;
				} catch(qsaError) {
				} finally {
					if ( !old ) {
						context.removeAttribute("id");
					}
				}
			}
		}
	}

	// All others
	return select( selector.replace( rtrim, "$1" ), context, results, seed );
}

/**
 * Create key-value caches of limited size
 * @returns {Function(string, Object)} Returns the Object data after storing it on itself with
 *	property name the (space-suffixed) string and (if the cache is larger than Expr.cacheLength)
 *	deleting the oldest entry
 */
function createCache() {
	var keys = [];

	function cache( key, value ) {
		// Use (key + " ") to avoid collision with native prototype properties (see Issue #157)
		if ( keys.push( key += " " ) > Expr.cacheLength ) {
			// Only keep the most recent entries
			delete cache[ keys.shift() ];
		}
		return (cache[ key ] = value);
	}
	return cache;
}

/**
 * Mark a function for special use by Sizzle
 * @param {Function} fn The function to mark
 */
function markFunction( fn ) {
	fn[ expando ] = true;
	return fn;
}

/**
 * Support testing using an element
 * @param {Function} fn Passed the created div and expects a boolean result
 */
function assert( fn ) {
	var div = document.createElement("div");

	try {
		return !!fn( div );
	} catch (e) {
		return false;
	} finally {
		// Remove from its parent by default
		if ( div.parentNode ) {
			div.parentNode.removeChild( div );
		}
		// release memory in IE
		div = null;
	}
}

/**
 * Adds the same handler for all of the specified attrs
 * @param {String} attrs Pipe-separated list of attributes
 * @param {Function} handler The method that will be applied
 */
function addHandle( attrs, handler ) {
	var arr = attrs.split("|"),
		i = attrs.length;

	while ( i-- ) {
		Expr.attrHandle[ arr[i] ] = handler;
	}
}

/**
 * Checks document order of two siblings
 * @param {Element} a
 * @param {Element} b
 * @returns {Number} Returns less than 0 if a precedes b, greater than 0 if a follows b
 */
function siblingCheck( a, b ) {
	var cur = b && a,
		diff = cur && a.nodeType === 1 && b.nodeType === 1 &&
			( ~b.sourceIndex || MAX_NEGATIVE ) -
			( ~a.sourceIndex || MAX_NEGATIVE );

	// Use IE sourceIndex if available on both nodes
	if ( diff ) {
		return diff;
	}

	// Check if b follows a
	if ( cur ) {
		while ( (cur = cur.nextSibling) ) {
			if ( cur === b ) {
				return -1;
			}
		}
	}

	return a ? 1 : -1;
}

/**
 * Returns a function to use in pseudos for input types
 * @param {String} type
 */
function createInputPseudo( type ) {
	return function( elem ) {
		var name = elem.nodeName.toLowerCase();
		return name === "input" && elem.type === type;
	};
}

/**
 * Returns a function to use in pseudos for buttons
 * @param {String} type
 */
function createButtonPseudo( type ) {
	return function( elem ) {
		var name = elem.nodeName.toLowerCase();
		return (name === "input" || name === "button") && elem.type === type;
	};
}

/**
 * Returns a function to use in pseudos for positionals
 * @param {Function} fn
 */
function createPositionalPseudo( fn ) {
	return markFunction(function( argument ) {
		argument = +argument;
		return markFunction(function( seed, matches ) {
			var j,
				matchIndexes = fn( [], seed.length, argument ),
				i = matchIndexes.length;

			// Match elements found at the specified indexes
			while ( i-- ) {
				if ( seed[ (j = matchIndexes[i]) ] ) {
					seed[j] = !(matches[j] = seed[j]);
				}
			}
		});
	});
}

/**
 * Detect xml
 * @param {Element|Object} elem An element or a document
 */
isXML = Sizzle.isXML = function( elem ) {
	// documentElement is verified for cases where it doesn't yet exist
	// (such as loading iframes in IE - #4833)
	var documentElement = elem && (elem.ownerDocument || elem).documentElement;
	return documentElement ? documentElement.nodeName !== "HTML" : false;
};

// Expose support vars for convenience
support = Sizzle.support = {};

/**
 * Sets document-related variables once based on the current document
 * @param {Element|Object} [doc] An element or document object to use to set the document
 * @returns {Object} Returns the current document
 */
setDocument = Sizzle.setDocument = function( node ) {
	var doc = node ? node.ownerDocument || node : preferredDoc,
		parent = doc.defaultView;

	// If no document and documentElement is available, return
	if ( doc === document || doc.nodeType !== 9 || !doc.documentElement ) {
		return document;
	}

	// Set our document
	document = doc;
	docElem = doc.documentElement;

	// Support tests
	documentIsHTML = !isXML( doc );

	// Support: IE>8
	// If iframe document is assigned to "document" variable and if iframe has been reloaded,
	// IE will throw "permission denied" error when accessing "document" variable, see jQuery #13936
	// IE6-8 do not support the defaultView property so parent will be undefined
	if ( parent && parent.attachEvent && parent !== parent.top ) {
		parent.attachEvent( "onbeforeunload", function() {
			setDocument();
		});
	}

	/* Attributes
	---------------------------------------------------------------------- */

	// Support: IE<8
	// Verify that getAttribute really returns attributes and not properties (excepting IE8 booleans)
	support.attributes = assert(function( div ) {
		div.className = "i";
		return !div.getAttribute("className");
	});

	/* getElement(s)By*
	---------------------------------------------------------------------- */

	// Check if getElementsByTagName("*") returns only elements
	support.getElementsByTagName = assert(function( div ) {
		div.appendChild( doc.createComment("") );
		return !div.getElementsByTagName("*").length;
	});

	// Check if getElementsByClassName can be trusted
	support.getElementsByClassName = assert(function( div ) {
		div.innerHTML = "<div class='a'></div><div class='a i'></div>";

		// Support: Safari<4
		// Catch class over-caching
		div.firstChild.className = "i";
		// Support: Opera<10
		// Catch gEBCN failure to find non-leading classes
		return div.getElementsByClassName("i").length === 2;
	});

	// Support: IE<10
	// Check if getElementById returns elements by name
	// The broken getElementById methods don't pick up programatically-set names,
	// so use a roundabout getElementsByName test
	support.getById = assert(function( div ) {
		docElem.appendChild( div ).id = expando;
		return !doc.getElementsByName || !doc.getElementsByName( expando ).length;
	});

	// ID find and filter
	if ( support.getById ) {
		Expr.find["ID"] = function( id, context ) {
			if ( typeof context.getElementById !== strundefined && documentIsHTML ) {
				var m = context.getElementById( id );
				// Check parentNode to catch when Blackberry 4.6 returns
				// nodes that are no longer in the document #6963
				return m && m.parentNode ? [m] : [];
			}
		};
		Expr.filter["ID"] = function( id ) {
			var attrId = id.replace( runescape, funescape );
			return function( elem ) {
				return elem.getAttribute("id") === attrId;
			};
		};
	} else {
		// Support: IE6/7
		// getElementById is not reliable as a find shortcut
		delete Expr.find["ID"];

		Expr.filter["ID"] =  function( id ) {
			var attrId = id.replace( runescape, funescape );
			return function( elem ) {
				var node = typeof elem.getAttributeNode !== strundefined && elem.getAttributeNode("id");
				return node && node.value === attrId;
			};
		};
	}

	// Tag
	Expr.find["TAG"] = support.getElementsByTagName ?
		function( tag, context ) {
			if ( typeof context.getElementsByTagName !== strundefined ) {
				return context.getElementsByTagName( tag );
			}
		} :
		function( tag, context ) {
			var elem,
				tmp = [],
				i = 0,
				results = context.getElementsByTagName( tag );

			// Filter out possible comments
			if ( tag === "*" ) {
				while ( (elem = results[i++]) ) {
					if ( elem.nodeType === 1 ) {
						tmp.push( elem );
					}
				}

				return tmp;
			}
			return results;
		};

	// Class
	Expr.find["CLASS"] = support.getElementsByClassName && function( className, context ) {
		if ( typeof context.getElementsByClassName !== strundefined && documentIsHTML ) {
			return context.getElementsByClassName( className );
		}
	};

	/* QSA/matchesSelector
	---------------------------------------------------------------------- */

	// QSA and matchesSelector support

	// matchesSelector(:active) reports false when true (IE9/Opera 11.5)
	rbuggyMatches = [];

	// qSa(:focus) reports false when true (Chrome 21)
	// We allow this because of a bug in IE8/9 that throws an error
	// whenever `document.activeElement` is accessed on an iframe
	// So, we allow :focus to pass through QSA all the time to avoid the IE error
	// See http://bugs.jquery.com/ticket/13378
	rbuggyQSA = [];

	if ( (support.qsa = rnative.test( doc.querySelectorAll )) ) {
		// Build QSA regex
		// Regex strategy adopted from Diego Perini
		assert(function( div ) {
			// Select is set to empty string on purpose
			// This is to test IE's treatment of not explicitly
			// setting a boolean content attribute,
			// since its presence should be enough
			// http://bugs.jquery.com/ticket/12359
			div.innerHTML = "<select><option selected=''></option></select>";

			// Support: IE8
			// Boolean attributes and "value" are not treated correctly
			if ( !div.querySelectorAll("[selected]").length ) {
				rbuggyQSA.push( "\\[" + whitespace + "*(?:value|" + booleans + ")" );
			}

			// Webkit/Opera - :checked should return selected option elements
			// http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
			// IE8 throws error here and will not see later tests
			if ( !div.querySelectorAll(":checked").length ) {
				rbuggyQSA.push(":checked");
			}
		});

		assert(function( div ) {

			// Support: Opera 10-12/IE8
			// ^= $= *= and empty values
			// Should not select anything
			// Support: Windows 8 Native Apps
			// The type attribute is restricted during .innerHTML assignment
			var input = doc.createElement("input");
			input.setAttribute( "type", "hidden" );
			div.appendChild( input ).setAttribute( "t", "" );

			if ( div.querySelectorAll("[t^='']").length ) {
				rbuggyQSA.push( "[*^$]=" + whitespace + "*(?:''|\"\")" );
			}

			// FF 3.5 - :enabled/:disabled and hidden elements (hidden elements are still enabled)
			// IE8 throws error here and will not see later tests
			if ( !div.querySelectorAll(":enabled").length ) {
				rbuggyQSA.push( ":enabled", ":disabled" );
			}

			// Opera 10-11 does not throw on post-comma invalid pseudos
			div.querySelectorAll("*,:x");
			rbuggyQSA.push(",.*:");
		});
	}

	if ( (support.matchesSelector = rnative.test( (matches = docElem.webkitMatchesSelector ||
		docElem.mozMatchesSelector ||
		docElem.oMatchesSelector ||
		docElem.msMatchesSelector) )) ) {

		assert(function( div ) {
			// Check to see if it's possible to do matchesSelector
			// on a disconnected node (IE 9)
			support.disconnectedMatch = matches.call( div, "div" );

			// This should fail with an exception
			// Gecko does not error, returns false instead
			matches.call( div, "[s!='']:x" );
			rbuggyMatches.push( "!=", pseudos );
		});
	}

	rbuggyQSA = rbuggyQSA.length && new RegExp( rbuggyQSA.join("|") );
	rbuggyMatches = rbuggyMatches.length && new RegExp( rbuggyMatches.join("|") );

	/* Contains
	---------------------------------------------------------------------- */

	// Element contains another
	// Purposefully does not implement inclusive descendent
	// As in, an element does not contain itself
	contains = rnative.test( docElem.contains ) || docElem.compareDocumentPosition ?
		function( a, b ) {
			var adown = a.nodeType === 9 ? a.documentElement : a,
				bup = b && b.parentNode;
			return a === bup || !!( bup && bup.nodeType === 1 && (
				adown.contains ?
					adown.contains( bup ) :
					a.compareDocumentPosition && a.compareDocumentPosition( bup ) & 16
			));
		} :
		function( a, b ) {
			if ( b ) {
				while ( (b = b.parentNode) ) {
					if ( b === a ) {
						return true;
					}
				}
			}
			return false;
		};

	/* Sorting
	---------------------------------------------------------------------- */

	// Document order sorting
	sortOrder = docElem.compareDocumentPosition ?
	function( a, b ) {

		// Flag for duplicate removal
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		var compare = b.compareDocumentPosition && a.compareDocumentPosition && a.compareDocumentPosition( b );

		if ( compare ) {
			// Disconnected nodes
			if ( compare & 1 ||
				(!support.sortDetached && b.compareDocumentPosition( a ) === compare) ) {

				// Choose the first element that is related to our preferred document
				if ( a === doc || contains(preferredDoc, a) ) {
					return -1;
				}
				if ( b === doc || contains(preferredDoc, b) ) {
					return 1;
				}

				// Maintain original order
				return sortInput ?
					( indexOf.call( sortInput, a ) - indexOf.call( sortInput, b ) ) :
					0;
			}

			return compare & 4 ? -1 : 1;
		}

		// Not directly comparable, sort on existence of method
		return a.compareDocumentPosition ? -1 : 1;
	} :
	function( a, b ) {
		var cur,
			i = 0,
			aup = a.parentNode,
			bup = b.parentNode,
			ap = [ a ],
			bp = [ b ];

		// Exit early if the nodes are identical
		if ( a === b ) {
			hasDuplicate = true;
			return 0;

		// Parentless nodes are either documents or disconnected
		} else if ( !aup || !bup ) {
			return a === doc ? -1 :
				b === doc ? 1 :
				aup ? -1 :
				bup ? 1 :
				sortInput ?
				( indexOf.call( sortInput, a ) - indexOf.call( sortInput, b ) ) :
				0;

		// If the nodes are siblings, we can do a quick check
		} else if ( aup === bup ) {
			return siblingCheck( a, b );
		}

		// Otherwise we need full lists of their ancestors for comparison
		cur = a;
		while ( (cur = cur.parentNode) ) {
			ap.unshift( cur );
		}
		cur = b;
		while ( (cur = cur.parentNode) ) {
			bp.unshift( cur );
		}

		// Walk down the tree looking for a discrepancy
		while ( ap[i] === bp[i] ) {
			i++;
		}

		return i ?
			// Do a sibling check if the nodes have a common ancestor
			siblingCheck( ap[i], bp[i] ) :

			// Otherwise nodes in our document sort first
			ap[i] === preferredDoc ? -1 :
			bp[i] === preferredDoc ? 1 :
			0;
	};

	return doc;
};

Sizzle.matches = function( expr, elements ) {
	return Sizzle( expr, null, null, elements );
};

Sizzle.matchesSelector = function( elem, expr ) {
	// Set document vars if needed
	if ( ( elem.ownerDocument || elem ) !== document ) {
		setDocument( elem );
	}

	// Make sure that attribute selectors are quoted
	expr = expr.replace( rattributeQuotes, "='$1']" );

	if ( support.matchesSelector && documentIsHTML &&
		( !rbuggyMatches || !rbuggyMatches.test( expr ) ) &&
		( !rbuggyQSA     || !rbuggyQSA.test( expr ) ) ) {

		try {
			var ret = matches.call( elem, expr );

			// IE 9's matchesSelector returns false on disconnected nodes
			if ( ret || support.disconnectedMatch ||
					// As well, disconnected nodes are said to be in a document
					// fragment in IE 9
					elem.document && elem.document.nodeType !== 11 ) {
				return ret;
			}
		} catch(e) {}
	}

	return Sizzle( expr, document, null, [elem] ).length > 0;
};

Sizzle.contains = function( context, elem ) {
	// Set document vars if needed
	if ( ( context.ownerDocument || context ) !== document ) {
		setDocument( context );
	}
	return contains( context, elem );
};

Sizzle.attr = function( elem, name ) {
	// Set document vars if needed
	if ( ( elem.ownerDocument || elem ) !== document ) {
		setDocument( elem );
	}

	var fn = Expr.attrHandle[ name.toLowerCase() ],
		// Don't get fooled by Object.prototype properties (jQuery #13807)
		val = fn && hasOwn.call( Expr.attrHandle, name.toLowerCase() ) ?
			fn( elem, name, !documentIsHTML ) :
			undefined;

	return val === undefined ?
		support.attributes || !documentIsHTML ?
			elem.getAttribute( name ) :
			(val = elem.getAttributeNode(name)) && val.specified ?
				val.value :
				null :
		val;
};

Sizzle.error = function( msg ) {
	throw new Error( "Syntax error, unrecognized expression: " + msg );
};

/**
 * Document sorting and removing duplicates
 * @param {ArrayLike} results
 */
Sizzle.uniqueSort = function( results ) {
	var elem,
		duplicates = [],
		j = 0,
		i = 0;

	// Unless we *know* we can detect duplicates, assume their presence
	hasDuplicate = !support.detectDuplicates;
	sortInput = !support.sortStable && results.slice( 0 );
	results.sort( sortOrder );

	if ( hasDuplicate ) {
		while ( (elem = results[i++]) ) {
			if ( elem === results[ i ] ) {
				j = duplicates.push( i );
			}
		}
		while ( j-- ) {
			results.splice( duplicates[ j ], 1 );
		}
	}

	return results;
};

/**
 * Utility function for retrieving the text value of an array of DOM nodes
 * @param {Array|Element} elem
 */
getText = Sizzle.getText = function( elem ) {
	var node,
		ret = "",
		i = 0,
		nodeType = elem.nodeType;

	if ( !nodeType ) {
		// If no nodeType, this is expected to be an array
		for ( ; (node = elem[i]); i++ ) {
			// Do not traverse comment nodes
			ret += getText( node );
		}
	} else if ( nodeType === 1 || nodeType === 9 || nodeType === 11 ) {
		// Use textContent for elements
		// innerText usage removed for consistency of new lines (see #11153)
		if ( typeof elem.textContent === "string" ) {
			return elem.textContent;
		} else {
			// Traverse its children
			for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
				ret += getText( elem );
			}
		}
	} else if ( nodeType === 3 || nodeType === 4 ) {
		return elem.nodeValue;
	}
	// Do not include comment or processing instruction nodes

	return ret;
};

Expr = Sizzle.selectors = {

	// Can be adjusted by the user
	cacheLength: 50,

	createPseudo: markFunction,

	match: matchExpr,

	attrHandle: {},

	find: {},

	relative: {
		">": { dir: "parentNode", first: true },
		" ": { dir: "parentNode" },
		"+": { dir: "previousSibling", first: true },
		"~": { dir: "previousSibling" }
	},

	preFilter: {
		"ATTR": function( match ) {
			match[1] = match[1].replace( runescape, funescape );

			// Move the given value to match[3] whether quoted or unquoted
			match[3] = ( match[4] || match[5] || "" ).replace( runescape, funescape );

			if ( match[2] === "~=" ) {
				match[3] = " " + match[3] + " ";
			}

			return match.slice( 0, 4 );
		},

		"CHILD": function( match ) {
			/* matches from matchExpr["CHILD"]
				1 type (only|nth|...)
				2 what (child|of-type)
				3 argument (even|odd|\d*|\d*n([+-]\d+)?|...)
				4 xn-component of xn+y argument ([+-]?\d*n|)
				5 sign of xn-component
				6 x of xn-component
				7 sign of y-component
				8 y of y-component
			*/
			match[1] = match[1].toLowerCase();

			if ( match[1].slice( 0, 3 ) === "nth" ) {
				// nth-* requires argument
				if ( !match[3] ) {
					Sizzle.error( match[0] );
				}

				// numeric x and y parameters for Expr.filter.CHILD
				// remember that false/true cast respectively to 0/1
				match[4] = +( match[4] ? match[5] + (match[6] || 1) : 2 * ( match[3] === "even" || match[3] === "odd" ) );
				match[5] = +( ( match[7] + match[8] ) || match[3] === "odd" );

			// other types prohibit arguments
			} else if ( match[3] ) {
				Sizzle.error( match[0] );
			}

			return match;
		},

		"PSEUDO": function( match ) {
			var excess,
				unquoted = !match[5] && match[2];

			if ( matchExpr["CHILD"].test( match[0] ) ) {
				return null;
			}

			// Accept quoted arguments as-is
			if ( match[3] && match[4] !== undefined ) {
				match[2] = match[4];

			// Strip excess characters from unquoted arguments
			} else if ( unquoted && rpseudo.test( unquoted ) &&
				// Get excess from tokenize (recursively)
				(excess = tokenize( unquoted, true )) &&
				// advance to the next closing parenthesis
				(excess = unquoted.indexOf( ")", unquoted.length - excess ) - unquoted.length) ) {

				// excess is a negative index
				match[0] = match[0].slice( 0, excess );
				match[2] = unquoted.slice( 0, excess );
			}

			// Return only captures needed by the pseudo filter method (type and argument)
			return match.slice( 0, 3 );
		}
	},

	filter: {

		"TAG": function( nodeNameSelector ) {
			var nodeName = nodeNameSelector.replace( runescape, funescape ).toLowerCase();
			return nodeNameSelector === "*" ?
				function() { return true; } :
				function( elem ) {
					return elem.nodeName && elem.nodeName.toLowerCase() === nodeName;
				};
		},

		"CLASS": function( className ) {
			var pattern = classCache[ className + " " ];

			return pattern ||
				(pattern = new RegExp( "(^|" + whitespace + ")" + className + "(" + whitespace + "|$)" )) &&
				classCache( className, function( elem ) {
					return pattern.test( typeof elem.className === "string" && elem.className || typeof elem.getAttribute !== strundefined && elem.getAttribute("class") || "" );
				});
		},

		"ATTR": function( name, operator, check ) {
			return function( elem ) {
				var result = Sizzle.attr( elem, name );

				if ( result == null ) {
					return operator === "!=";
				}
				if ( !operator ) {
					return true;
				}

				result += "";

				return operator === "=" ? result === check :
					operator === "!=" ? result !== check :
					operator === "^=" ? check && result.indexOf( check ) === 0 :
					operator === "*=" ? check && result.indexOf( check ) > -1 :
					operator === "$=" ? check && result.slice( -check.length ) === check :
					operator === "~=" ? ( " " + result + " " ).indexOf( check ) > -1 :
					operator === "|=" ? result === check || result.slice( 0, check.length + 1 ) === check + "-" :
					false;
			};
		},

		"CHILD": function( type, what, argument, first, last ) {
			var simple = type.slice( 0, 3 ) !== "nth",
				forward = type.slice( -4 ) !== "last",
				ofType = what === "of-type";

			return first === 1 && last === 0 ?

				// Shortcut for :nth-*(n)
				function( elem ) {
					return !!elem.parentNode;
				} :

				function( elem, context, xml ) {
					var cache, outerCache, node, diff, nodeIndex, start,
						dir = simple !== forward ? "nextSibling" : "previousSibling",
						parent = elem.parentNode,
						name = ofType && elem.nodeName.toLowerCase(),
						useCache = !xml && !ofType;

					if ( parent ) {

						// :(first|last|only)-(child|of-type)
						if ( simple ) {
							while ( dir ) {
								node = elem;
								while ( (node = node[ dir ]) ) {
									if ( ofType ? node.nodeName.toLowerCase() === name : node.nodeType === 1 ) {
										return false;
									}
								}
								// Reverse direction for :only-* (if we haven't yet done so)
								start = dir = type === "only" && !start && "nextSibling";
							}
							return true;
						}

						start = [ forward ? parent.firstChild : parent.lastChild ];

						// non-xml :nth-child(...) stores cache data on `parent`
						if ( forward && useCache ) {
							// Seek `elem` from a previously-cached index
							outerCache = parent[ expando ] || (parent[ expando ] = {});
							cache = outerCache[ type ] || [];
							nodeIndex = cache[0] === dirruns && cache[1];
							diff = cache[0] === dirruns && cache[2];
							node = nodeIndex && parent.childNodes[ nodeIndex ];

							while ( (node = ++nodeIndex && node && node[ dir ] ||

								// Fallback to seeking `elem` from the start
								(diff = nodeIndex = 0) || start.pop()) ) {

								// When found, cache indexes on `parent` and break
								if ( node.nodeType === 1 && ++diff && node === elem ) {
									outerCache[ type ] = [ dirruns, nodeIndex, diff ];
									break;
								}
							}

						// Use previously-cached element index if available
						} else if ( useCache && (cache = (elem[ expando ] || (elem[ expando ] = {}))[ type ]) && cache[0] === dirruns ) {
							diff = cache[1];

						// xml :nth-child(...) or :nth-last-child(...) or :nth(-last)?-of-type(...)
						} else {
							// Use the same loop as above to seek `elem` from the start
							while ( (node = ++nodeIndex && node && node[ dir ] ||
								(diff = nodeIndex = 0) || start.pop()) ) {

								if ( ( ofType ? node.nodeName.toLowerCase() === name : node.nodeType === 1 ) && ++diff ) {
									// Cache the index of each encountered element
									if ( useCache ) {
										(node[ expando ] || (node[ expando ] = {}))[ type ] = [ dirruns, diff ];
									}

									if ( node === elem ) {
										break;
									}
								}
							}
						}

						// Incorporate the offset, then check against cycle size
						diff -= last;
						return diff === first || ( diff % first === 0 && diff / first >= 0 );
					}
				};
		},

		"PSEUDO": function( pseudo, argument ) {
			// pseudo-class names are case-insensitive
			// http://www.w3.org/TR/selectors/#pseudo-classes
			// Prioritize by case sensitivity in case custom pseudos are added with uppercase letters
			// Remember that setFilters inherits from pseudos
			var args,
				fn = Expr.pseudos[ pseudo ] || Expr.setFilters[ pseudo.toLowerCase() ] ||
					Sizzle.error( "unsupported pseudo: " + pseudo );

			// The user may use createPseudo to indicate that
			// arguments are needed to create the filter function
			// just as Sizzle does
			if ( fn[ expando ] ) {
				return fn( argument );
			}

			// But maintain support for old signatures
			if ( fn.length > 1 ) {
				args = [ pseudo, pseudo, "", argument ];
				return Expr.setFilters.hasOwnProperty( pseudo.toLowerCase() ) ?
					markFunction(function( seed, matches ) {
						var idx,
							matched = fn( seed, argument ),
							i = matched.length;
						while ( i-- ) {
							idx = indexOf.call( seed, matched[i] );
							seed[ idx ] = !( matches[ idx ] = matched[i] );
						}
					}) :
					function( elem ) {
						return fn( elem, 0, args );
					};
			}

			return fn;
		}
	},

	pseudos: {
		// Potentially complex pseudos
		"not": markFunction(function( selector ) {
			// Trim the selector passed to compile
			// to avoid treating leading and trailing
			// spaces as combinators
			var input = [],
				results = [],
				matcher = compile( selector.replace( rtrim, "$1" ) );

			return matcher[ expando ] ?
				markFunction(function( seed, matches, context, xml ) {
					var elem,
						unmatched = matcher( seed, null, xml, [] ),
						i = seed.length;

					// Match elements unmatched by `matcher`
					while ( i-- ) {
						if ( (elem = unmatched[i]) ) {
							seed[i] = !(matches[i] = elem);
						}
					}
				}) :
				function( elem, context, xml ) {
					input[0] = elem;
					matcher( input, null, xml, results );
					return !results.pop();
				};
		}),

		"has": markFunction(function( selector ) {
			return function( elem ) {
				return Sizzle( selector, elem ).length > 0;
			};
		}),

		"contains": markFunction(function( text ) {
			return function( elem ) {
				return ( elem.textContent || elem.innerText || getText( elem ) ).indexOf( text ) > -1;
			};
		}),

		// "Whether an element is represented by a :lang() selector
		// is based solely on the element's language value
		// being equal to the identifier C,
		// or beginning with the identifier C immediately followed by "-".
		// The matching of C against the element's language value is performed case-insensitively.
		// The identifier C does not have to be a valid language name."
		// http://www.w3.org/TR/selectors/#lang-pseudo
		"lang": markFunction( function( lang ) {
			// lang value must be a valid identifier
			if ( !ridentifier.test(lang || "") ) {
				Sizzle.error( "unsupported lang: " + lang );
			}
			lang = lang.replace( runescape, funescape ).toLowerCase();
			return function( elem ) {
				var elemLang;
				do {
					if ( (elemLang = documentIsHTML ?
						elem.lang :
						elem.getAttribute("xml:lang") || elem.getAttribute("lang")) ) {

						elemLang = elemLang.toLowerCase();
						return elemLang === lang || elemLang.indexOf( lang + "-" ) === 0;
					}
				} while ( (elem = elem.parentNode) && elem.nodeType === 1 );
				return false;
			};
		}),

		// Miscellaneous
		"target": function( elem ) {
			var hash = window.location && window.location.hash;
			return hash && hash.slice( 1 ) === elem.id;
		},

		"root": function( elem ) {
			return elem === docElem;
		},

		"focus": function( elem ) {
			return elem === document.activeElement && (!document.hasFocus || document.hasFocus()) && !!(elem.type || elem.href || ~elem.tabIndex);
		},

		// Boolean properties
		"enabled": function( elem ) {
			return elem.disabled === false;
		},

		"disabled": function( elem ) {
			return elem.disabled === true;
		},

		"checked": function( elem ) {
			// In CSS3, :checked should return both checked and selected elements
			// http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
			var nodeName = elem.nodeName.toLowerCase();
			return (nodeName === "input" && !!elem.checked) || (nodeName === "option" && !!elem.selected);
		},

		"selected": function( elem ) {
			// Accessing this property makes selected-by-default
			// options in Safari work properly
			if ( elem.parentNode ) {
				elem.parentNode.selectedIndex;
			}

			return elem.selected === true;
		},

		// Contents
		"empty": function( elem ) {
			// http://www.w3.org/TR/selectors/#empty-pseudo
			// :empty is only affected by element nodes and content nodes(including text(3), cdata(4)),
			//   not comment, processing instructions, or others
			// Thanks to Diego Perini for the nodeName shortcut
			//   Greater than "@" means alpha characters (specifically not starting with "#" or "?")
			for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
				if ( elem.nodeName > "@" || elem.nodeType === 3 || elem.nodeType === 4 ) {
					return false;
				}
			}
			return true;
		},

		"parent": function( elem ) {
			return !Expr.pseudos["empty"]( elem );
		},

		// Element/input types
		"header": function( elem ) {
			return rheader.test( elem.nodeName );
		},

		"input": function( elem ) {
			return rinputs.test( elem.nodeName );
		},

		"button": function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return name === "input" && elem.type === "button" || name === "button";
		},

		"text": function( elem ) {
			var attr;
			// IE6 and 7 will map elem.type to 'text' for new HTML5 types (search, etc)
			// use getAttribute instead to test this case
			return elem.nodeName.toLowerCase() === "input" &&
				elem.type === "text" &&
				( (attr = elem.getAttribute("type")) == null || attr.toLowerCase() === elem.type );
		},

		// Position-in-collection
		"first": createPositionalPseudo(function() {
			return [ 0 ];
		}),

		"last": createPositionalPseudo(function( matchIndexes, length ) {
			return [ length - 1 ];
		}),

		"eq": createPositionalPseudo(function( matchIndexes, length, argument ) {
			return [ argument < 0 ? argument + length : argument ];
		}),

		"even": createPositionalPseudo(function( matchIndexes, length ) {
			var i = 0;
			for ( ; i < length; i += 2 ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"odd": createPositionalPseudo(function( matchIndexes, length ) {
			var i = 1;
			for ( ; i < length; i += 2 ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"lt": createPositionalPseudo(function( matchIndexes, length, argument ) {
			var i = argument < 0 ? argument + length : argument;
			for ( ; --i >= 0; ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"gt": createPositionalPseudo(function( matchIndexes, length, argument ) {
			var i = argument < 0 ? argument + length : argument;
			for ( ; ++i < length; ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		})
	}
};

Expr.pseudos["nth"] = Expr.pseudos["eq"];

// Add button/input type pseudos
for ( i in { radio: true, checkbox: true, file: true, password: true, image: true } ) {
	Expr.pseudos[ i ] = createInputPseudo( i );
}
for ( i in { submit: true, reset: true } ) {
	Expr.pseudos[ i ] = createButtonPseudo( i );
}

// Easy API for creating new setFilters
function setFilters() {}
setFilters.prototype = Expr.filters = Expr.pseudos;
Expr.setFilters = new setFilters();

function tokenize( selector, parseOnly ) {
	var matched, match, tokens, type,
		soFar, groups, preFilters,
		cached = tokenCache[ selector + " " ];

	if ( cached ) {
		return parseOnly ? 0 : cached.slice( 0 );
	}

	soFar = selector;
	groups = [];
	preFilters = Expr.preFilter;

	while ( soFar ) {

		// Comma and first run
		if ( !matched || (match = rcomma.exec( soFar )) ) {
			if ( match ) {
				// Don't consume trailing commas as valid
				soFar = soFar.slice( match[0].length ) || soFar;
			}
			groups.push( tokens = [] );
		}

		matched = false;

		// Combinators
		if ( (match = rcombinators.exec( soFar )) ) {
			matched = match.shift();
			tokens.push({
				value: matched,
				// Cast descendant combinators to space
				type: match[0].replace( rtrim, " " )
			});
			soFar = soFar.slice( matched.length );
		}

		// Filters
		for ( type in Expr.filter ) {
			if ( (match = matchExpr[ type ].exec( soFar )) && (!preFilters[ type ] ||
				(match = preFilters[ type ]( match ))) ) {
				matched = match.shift();
				tokens.push({
					value: matched,
					type: type,
					matches: match
				});
				soFar = soFar.slice( matched.length );
			}
		}

		if ( !matched ) {
			break;
		}
	}

	// Return the length of the invalid excess
	// if we're just parsing
	// Otherwise, throw an error or return tokens
	return parseOnly ?
		soFar.length :
		soFar ?
			Sizzle.error( selector ) :
			// Cache the tokens
			tokenCache( selector, groups ).slice( 0 );
}

function toSelector( tokens ) {
	var i = 0,
		len = tokens.length,
		selector = "";
	for ( ; i < len; i++ ) {
		selector += tokens[i].value;
	}
	return selector;
}

function addCombinator( matcher, combinator, base ) {
	var dir = combinator.dir,
		checkNonElements = base && dir === "parentNode",
		doneName = done++;

	return combinator.first ?
		// Check against closest ancestor/preceding element
		function( elem, context, xml ) {
			while ( (elem = elem[ dir ]) ) {
				if ( elem.nodeType === 1 || checkNonElements ) {
					return matcher( elem, context, xml );
				}
			}
		} :

		// Check against all ancestor/preceding elements
		function( elem, context, xml ) {
			var data, cache, outerCache,
				dirkey = dirruns + " " + doneName;

			// We can't set arbitrary data on XML nodes, so they don't benefit from dir caching
			if ( xml ) {
				while ( (elem = elem[ dir ]) ) {
					if ( elem.nodeType === 1 || checkNonElements ) {
						if ( matcher( elem, context, xml ) ) {
							return true;
						}
					}
				}
			} else {
				while ( (elem = elem[ dir ]) ) {
					if ( elem.nodeType === 1 || checkNonElements ) {
						outerCache = elem[ expando ] || (elem[ expando ] = {});
						if ( (cache = outerCache[ dir ]) && cache[0] === dirkey ) {
							if ( (data = cache[1]) === true || data === cachedruns ) {
								return data === true;
							}
						} else {
							cache = outerCache[ dir ] = [ dirkey ];
							cache[1] = matcher( elem, context, xml ) || cachedruns;
							if ( cache[1] === true ) {
								return true;
							}
						}
					}
				}
			}
		};
}

function elementMatcher( matchers ) {
	return matchers.length > 1 ?
		function( elem, context, xml ) {
			var i = matchers.length;
			while ( i-- ) {
				if ( !matchers[i]( elem, context, xml ) ) {
					return false;
				}
			}
			return true;
		} :
		matchers[0];
}

function condense( unmatched, map, filter, context, xml ) {
	var elem,
		newUnmatched = [],
		i = 0,
		len = unmatched.length,
		mapped = map != null;

	for ( ; i < len; i++ ) {
		if ( (elem = unmatched[i]) ) {
			if ( !filter || filter( elem, context, xml ) ) {
				newUnmatched.push( elem );
				if ( mapped ) {
					map.push( i );
				}
			}
		}
	}

	return newUnmatched;
}

function setMatcher( preFilter, selector, matcher, postFilter, postFinder, postSelector ) {
	if ( postFilter && !postFilter[ expando ] ) {
		postFilter = setMatcher( postFilter );
	}
	if ( postFinder && !postFinder[ expando ] ) {
		postFinder = setMatcher( postFinder, postSelector );
	}
	return markFunction(function( seed, results, context, xml ) {
		var temp, i, elem,
			preMap = [],
			postMap = [],
			preexisting = results.length,

			// Get initial elements from seed or context
			elems = seed || multipleContexts( selector || "*", context.nodeType ? [ context ] : context, [] ),

			// Prefilter to get matcher input, preserving a map for seed-results synchronization
			matcherIn = preFilter && ( seed || !selector ) ?
				condense( elems, preMap, preFilter, context, xml ) :
				elems,

			matcherOut = matcher ?
				// If we have a postFinder, or filtered seed, or non-seed postFilter or preexisting results,
				postFinder || ( seed ? preFilter : preexisting || postFilter ) ?

					// ...intermediate processing is necessary
					[] :

					// ...otherwise use results directly
					results :
				matcherIn;

		// Find primary matches
		if ( matcher ) {
			matcher( matcherIn, matcherOut, context, xml );
		}

		// Apply postFilter
		if ( postFilter ) {
			temp = condense( matcherOut, postMap );
			postFilter( temp, [], context, xml );

			// Un-match failing elements by moving them back to matcherIn
			i = temp.length;
			while ( i-- ) {
				if ( (elem = temp[i]) ) {
					matcherOut[ postMap[i] ] = !(matcherIn[ postMap[i] ] = elem);
				}
			}
		}

		if ( seed ) {
			if ( postFinder || preFilter ) {
				if ( postFinder ) {
					// Get the final matcherOut by condensing this intermediate into postFinder contexts
					temp = [];
					i = matcherOut.length;
					while ( i-- ) {
						if ( (elem = matcherOut[i]) ) {
							// Restore matcherIn since elem is not yet a final match
							temp.push( (matcherIn[i] = elem) );
						}
					}
					postFinder( null, (matcherOut = []), temp, xml );
				}

				// Move matched elements from seed to results to keep them synchronized
				i = matcherOut.length;
				while ( i-- ) {
					if ( (elem = matcherOut[i]) &&
						(temp = postFinder ? indexOf.call( seed, elem ) : preMap[i]) > -1 ) {

						seed[temp] = !(results[temp] = elem);
					}
				}
			}

		// Add elements to results, through postFinder if defined
		} else {
			matcherOut = condense(
				matcherOut === results ?
					matcherOut.splice( preexisting, matcherOut.length ) :
					matcherOut
			);
			if ( postFinder ) {
				postFinder( null, results, matcherOut, xml );
			} else {
				push.apply( results, matcherOut );
			}
		}
	});
}

function matcherFromTokens( tokens ) {
	var checkContext, matcher, j,
		len = tokens.length,
		leadingRelative = Expr.relative[ tokens[0].type ],
		implicitRelative = leadingRelative || Expr.relative[" "],
		i = leadingRelative ? 1 : 0,

		// The foundational matcher ensures that elements are reachable from top-level context(s)
		matchContext = addCombinator( function( elem ) {
			return elem === checkContext;
		}, implicitRelative, true ),
		matchAnyContext = addCombinator( function( elem ) {
			return indexOf.call( checkContext, elem ) > -1;
		}, implicitRelative, true ),
		matchers = [ function( elem, context, xml ) {
			return ( !leadingRelative && ( xml || context !== outermostContext ) ) || (
				(checkContext = context).nodeType ?
					matchContext( elem, context, xml ) :
					matchAnyContext( elem, context, xml ) );
		} ];

	for ( ; i < len; i++ ) {
		if ( (matcher = Expr.relative[ tokens[i].type ]) ) {
			matchers = [ addCombinator(elementMatcher( matchers ), matcher) ];
		} else {
			matcher = Expr.filter[ tokens[i].type ].apply( null, tokens[i].matches );

			// Return special upon seeing a positional matcher
			if ( matcher[ expando ] ) {
				// Find the next relative operator (if any) for proper handling
				j = ++i;
				for ( ; j < len; j++ ) {
					if ( Expr.relative[ tokens[j].type ] ) {
						break;
					}
				}
				return setMatcher(
					i > 1 && elementMatcher( matchers ),
					i > 1 && toSelector(
						// If the preceding token was a descendant combinator, insert an implicit any-element `*`
						tokens.slice( 0, i - 1 ).concat({ value: tokens[ i - 2 ].type === " " ? "*" : "" })
					).replace( rtrim, "$1" ),
					matcher,
					i < j && matcherFromTokens( tokens.slice( i, j ) ),
					j < len && matcherFromTokens( (tokens = tokens.slice( j )) ),
					j < len && toSelector( tokens )
				);
			}
			matchers.push( matcher );
		}
	}

	return elementMatcher( matchers );
}

function matcherFromGroupMatchers( elementMatchers, setMatchers ) {
	// A counter to specify which element is currently being matched
	var matcherCachedRuns = 0,
		bySet = setMatchers.length > 0,
		byElement = elementMatchers.length > 0,
		superMatcher = function( seed, context, xml, results, expandContext ) {
			var elem, j, matcher,
				setMatched = [],
				matchedCount = 0,
				i = "0",
				unmatched = seed && [],
				outermost = expandContext != null,
				contextBackup = outermostContext,
				// We must always have either seed elements or context
				elems = seed || byElement && Expr.find["TAG"]( "*", expandContext && context.parentNode || context ),
				// Use integer dirruns iff this is the outermost matcher
				dirrunsUnique = (dirruns += contextBackup == null ? 1 : Math.random() || 0.1);

			if ( outermost ) {
				outermostContext = context !== document && context;
				cachedruns = matcherCachedRuns;
			}

			// Add elements passing elementMatchers directly to results
			// Keep `i` a string if there are no elements so `matchedCount` will be "00" below
			for ( ; (elem = elems[i]) != null; i++ ) {
				if ( byElement && elem ) {
					j = 0;
					while ( (matcher = elementMatchers[j++]) ) {
						if ( matcher( elem, context, xml ) ) {
							results.push( elem );
							break;
						}
					}
					if ( outermost ) {
						dirruns = dirrunsUnique;
						cachedruns = ++matcherCachedRuns;
					}
				}

				// Track unmatched elements for set filters
				if ( bySet ) {
					// They will have gone through all possible matchers
					if ( (elem = !matcher && elem) ) {
						matchedCount--;
					}

					// Lengthen the array for every element, matched or not
					if ( seed ) {
						unmatched.push( elem );
					}
				}
			}

			// Apply set filters to unmatched elements
			matchedCount += i;
			if ( bySet && i !== matchedCount ) {
				j = 0;
				while ( (matcher = setMatchers[j++]) ) {
					matcher( unmatched, setMatched, context, xml );
				}

				if ( seed ) {
					// Reintegrate element matches to eliminate the need for sorting
					if ( matchedCount > 0 ) {
						while ( i-- ) {
							if ( !(unmatched[i] || setMatched[i]) ) {
								setMatched[i] = pop.call( results );
							}
						}
					}

					// Discard index placeholder values to get only actual matches
					setMatched = condense( setMatched );
				}

				// Add matches to results
				push.apply( results, setMatched );

				// Seedless set matches succeeding multiple successful matchers stipulate sorting
				if ( outermost && !seed && setMatched.length > 0 &&
					( matchedCount + setMatchers.length ) > 1 ) {

					Sizzle.uniqueSort( results );
				}
			}

			// Override manipulation of globals by nested matchers
			if ( outermost ) {
				dirruns = dirrunsUnique;
				outermostContext = contextBackup;
			}

			return unmatched;
		};

	return bySet ?
		markFunction( superMatcher ) :
		superMatcher;
}

compile = Sizzle.compile = function( selector, group /* Internal Use Only */ ) {
	var i,
		setMatchers = [],
		elementMatchers = [],
		cached = compilerCache[ selector + " " ];

	if ( !cached ) {
		// Generate a function of recursive functions that can be used to check each element
		if ( !group ) {
			group = tokenize( selector );
		}
		i = group.length;
		while ( i-- ) {
			cached = matcherFromTokens( group[i] );
			if ( cached[ expando ] ) {
				setMatchers.push( cached );
			} else {
				elementMatchers.push( cached );
			}
		}

		// Cache the compiled function
		cached = compilerCache( selector, matcherFromGroupMatchers( elementMatchers, setMatchers ) );
	}
	return cached;
};

function multipleContexts( selector, contexts, results ) {
	var i = 0,
		len = contexts.length;
	for ( ; i < len; i++ ) {
		Sizzle( selector, contexts[i], results );
	}
	return results;
}

function select( selector, context, results, seed ) {
	var i, tokens, token, type, find,
		match = tokenize( selector );

	if ( !seed ) {
		// Try to minimize operations if there is only one group
		if ( match.length === 1 ) {

			// Take a shortcut and set the context if the root selector is an ID
			tokens = match[0] = match[0].slice( 0 );
			if ( tokens.length > 2 && (token = tokens[0]).type === "ID" &&
					support.getById && context.nodeType === 9 && documentIsHTML &&
					Expr.relative[ tokens[1].type ] ) {

				context = ( Expr.find["ID"]( token.matches[0].replace(runescape, funescape), context ) || [] )[0];
				if ( !context ) {
					return results;
				}
				selector = selector.slice( tokens.shift().value.length );
			}

			// Fetch a seed set for right-to-left matching
			i = matchExpr["needsContext"].test( selector ) ? 0 : tokens.length;
			while ( i-- ) {
				token = tokens[i];

				// Abort if we hit a combinator
				if ( Expr.relative[ (type = token.type) ] ) {
					break;
				}
				if ( (find = Expr.find[ type ]) ) {
					// Search, expanding context for leading sibling combinators
					if ( (seed = find(
						token.matches[0].replace( runescape, funescape ),
						rsibling.test( tokens[0].type ) && context.parentNode || context
					)) ) {

						// If seed is empty or no tokens remain, we can return early
						tokens.splice( i, 1 );
						selector = seed.length && toSelector( tokens );
						if ( !selector ) {
							push.apply( results, seed );
							return results;
						}

						break;
					}
				}
			}
		}
	}

	// Compile and execute a filtering function
	// Provide `match` to avoid retokenization if we modified the selector above
	compile( selector, match )(
		seed,
		context,
		!documentIsHTML,
		results,
		rsibling.test( selector )
	);
	return results;
}

// One-time assignments

// Sort stability
support.sortStable = expando.split("").sort( sortOrder ).join("") === expando;

// Support: Chrome<14
// Always assume duplicates if they aren't passed to the comparison function
support.detectDuplicates = hasDuplicate;

// Initialize against the default document
setDocument();

// Support: Webkit<537.32 - Safari 6.0.3/Chrome 25 (fixed in Chrome 27)
// Detached nodes confoundingly follow *each other*
support.sortDetached = assert(function( div1 ) {
	// Should return 1, but returns 4 (following)
	return div1.compareDocumentPosition( document.createElement("div") ) & 1;
});

// Support: IE<8
// Prevent attribute/property "interpolation"
// http://msdn.microsoft.com/en-us/library/ms536429%28VS.85%29.aspx
if ( !assert(function( div ) {
	div.innerHTML = "<a href='#'></a>";
	return div.firstChild.getAttribute("href") === "#" ;
}) ) {
	addHandle( "type|href|height|width", function( elem, name, isXML ) {
		if ( !isXML ) {
			return elem.getAttribute( name, name.toLowerCase() === "type" ? 1 : 2 );
		}
	});
}

// Support: IE<9
// Use defaultValue in place of getAttribute("value")
if ( !support.attributes || !assert(function( div ) {
	div.innerHTML = "<input/>";
	div.firstChild.setAttribute( "value", "" );
	return div.firstChild.getAttribute( "value" ) === "";
}) ) {
	addHandle( "value", function( elem, name, isXML ) {
		if ( !isXML && elem.nodeName.toLowerCase() === "input" ) {
			return elem.defaultValue;
		}
	});
}

// Support: IE<9
// Use getAttributeNode to fetch booleans when getAttribute lies
if ( !assert(function( div ) {
	return div.getAttribute("disabled") == null;
}) ) {
	addHandle( booleans, function( elem, name, isXML ) {
		var val;
		if ( !isXML ) {
			return (val = elem.getAttributeNode( name )) && val.specified ?
				val.value :
				elem[ name ] === true ? name.toLowerCase() : null;
		}
	});
}

jQuery.find = Sizzle;
jQuery.expr = Sizzle.selectors;
jQuery.expr[":"] = jQuery.expr.pseudos;
jQuery.unique = Sizzle.uniqueSort;
jQuery.text = Sizzle.getText;
jQuery.isXMLDoc = Sizzle.isXML;
jQuery.contains = Sizzle.contains;


})( window );
// String to Object options format cache
var optionsCache = {};

// Convert String-formatted options into Object-formatted ones and store in cache
function createOptions( options ) {
	var object = optionsCache[ options ] = {};
	jQuery.each( options.match( core_rnotwhite ) || [], function( _, flag ) {
		object[ flag ] = true;
	});
	return object;
}

/*
 * Create a callback list using the following parameters:
 *
 *	options: an optional list of space-separated options that will change how
 *			the callback list behaves or a more traditional option object
 *
 * By default a callback list will act like an event callback list and can be
 * "fired" multiple times.
 *
 * Possible options:
 *
 *	once:			will ensure the callback list can only be fired once (like a Deferred)
 *
 *	memory:			will keep track of previous values and will call any callback added
 *					after the list has been fired right away with the latest "memorized"
 *					values (like a Deferred)
 *
 *	unique:			will ensure a callback can only be added once (no duplicate in the list)
 *
 *	stopOnFalse:	interrupt callings when a callback returns false
 *
 */
jQuery.Callbacks = function( options ) {

	// Convert options from String-formatted to Object-formatted if needed
	// (we check in cache first)
	options = typeof options === "string" ?
		( optionsCache[ options ] || createOptions( options ) ) :
		jQuery.extend( {}, options );

	var // Flag to know if list is currently firing
		firing,
		// Last fire value (for non-forgettable lists)
		memory,
		// Flag to know if list was already fired
		fired,
		// End of the loop when firing
		firingLength,
		// Index of currently firing callback (modified by remove if needed)
		firingIndex,
		// First callback to fire (used internally by add and fireWith)
		firingStart,
		// Actual callback list
		list = [],
		// Stack of fire calls for repeatable lists
		stack = !options.once && [],
		// Fire callbacks
		fire = function( data ) {
			memory = options.memory && data;
			fired = true;
			firingIndex = firingStart || 0;
			firingStart = 0;
			firingLength = list.length;
			firing = true;
			for ( ; list && firingIndex < firingLength; firingIndex++ ) {
				if ( list[ firingIndex ].apply( data[ 0 ], data[ 1 ] ) === false && options.stopOnFalse ) {
					memory = false; // To prevent further calls using add
					break;
				}
			}
			firing = false;
			if ( list ) {
				if ( stack ) {
					if ( stack.length ) {
						fire( stack.shift() );
					}
				} else if ( memory ) {
					list = [];
				} else {
					self.disable();
				}
			}
		},
		// Actual Callbacks object
		self = {
			// Add a callback or a collection of callbacks to the list
			add: function() {
				if ( list ) {
					// First, we save the current length
					var start = list.length;
					(function add( args ) {
						jQuery.each( args, function( _, arg ) {
							var type = jQuery.type( arg );
							if ( type === "function" ) {
								if ( !options.unique || !self.has( arg ) ) {
									list.push( arg );
								}
							} else if ( arg && arg.length && type !== "string" ) {
								// Inspect recursively
								add( arg );
							}
						});
					})( arguments );
					// Do we need to add the callbacks to the
					// current firing batch?
					if ( firing ) {
						firingLength = list.length;
					// With memory, if we're not firing then
					// we should call right away
					} else if ( memory ) {
						firingStart = start;
						fire( memory );
					}
				}
				return this;
			},
			// Remove a callback from the list
			remove: function() {
				if ( list ) {
					jQuery.each( arguments, function( _, arg ) {
						var index;
						while( ( index = jQuery.inArray( arg, list, index ) ) > -1 ) {
							list.splice( index, 1 );
							// Handle firing indexes
							if ( firing ) {
								if ( index <= firingLength ) {
									firingLength--;
								}
								if ( index <= firingIndex ) {
									firingIndex--;
								}
							}
						}
					});
				}
				return this;
			},
			// Check if a given callback is in the list.
			// If no argument is given, return whether or not list has callbacks attached.
			has: function( fn ) {
				return fn ? jQuery.inArray( fn, list ) > -1 : !!( list && list.length );
			},
			// Remove all callbacks from the list
			empty: function() {
				list = [];
				firingLength = 0;
				return this;
			},
			// Have the list do nothing anymore
			disable: function() {
				list = stack = memory = undefined;
				return this;
			},
			// Is it disabled?
			disabled: function() {
				return !list;
			},
			// Lock the list in its current state
			lock: function() {
				stack = undefined;
				if ( !memory ) {
					self.disable();
				}
				return this;
			},
			// Is it locked?
			locked: function() {
				return !stack;
			},
			// Call all callbacks with the given context and arguments
			fireWith: function( context, args ) {
				if ( list && ( !fired || stack ) ) {
					args = args || [];
					args = [ context, args.slice ? args.slice() : args ];
					if ( firing ) {
						stack.push( args );
					} else {
						fire( args );
					}
				}
				return this;
			},
			// Call all the callbacks with the given arguments
			fire: function() {
				self.fireWith( this, arguments );
				return this;
			},
			// To know if the callbacks have already been called at least once
			fired: function() {
				return !!fired;
			}
		};

	return self;
};
jQuery.extend({

	Deferred: function( func ) {
		var tuples = [
				// action, add listener, listener list, final state
				[ "resolve", "done", jQuery.Callbacks("once memory"), "resolved" ],
				[ "reject", "fail", jQuery.Callbacks("once memory"), "rejected" ],
				[ "notify", "progress", jQuery.Callbacks("memory") ]
			],
			state = "pending",
			promise = {
				state: function() {
					return state;
				},
				always: function() {
					deferred.done( arguments ).fail( arguments );
					return this;
				},
				then: function( /* fnDone, fnFail, fnProgress */ ) {
					var fns = arguments;
					return jQuery.Deferred(function( newDefer ) {
						jQuery.each( tuples, function( i, tuple ) {
							var action = tuple[ 0 ],
								fn = jQuery.isFunction( fns[ i ] ) && fns[ i ];
							// deferred[ done | fail | progress ] for forwarding actions to newDefer
							deferred[ tuple[1] ](function() {
								var returned = fn && fn.apply( this, arguments );
								if ( returned && jQuery.isFunction( returned.promise ) ) {
									returned.promise()
										.done( newDefer.resolve )
										.fail( newDefer.reject )
										.progress( newDefer.notify );
								} else {
									newDefer[ action + "With" ]( this === promise ? newDefer.promise() : this, fn ? [ returned ] : arguments );
								}
							});
						});
						fns = null;
					}).promise();
				},
				// Get a promise for this deferred
				// If obj is provided, the promise aspect is added to the object
				promise: function( obj ) {
					return obj != null ? jQuery.extend( obj, promise ) : promise;
				}
			},
			deferred = {};

		// Keep pipe for back-compat
		promise.pipe = promise.then;

		// Add list-specific methods
		jQuery.each( tuples, function( i, tuple ) {
			var list = tuple[ 2 ],
				stateString = tuple[ 3 ];

			// promise[ done | fail | progress ] = list.add
			promise[ tuple[1] ] = list.add;

			// Handle state
			if ( stateString ) {
				list.add(function() {
					// state = [ resolved | rejected ]
					state = stateString;

				// [ reject_list | resolve_list ].disable; progress_list.lock
				}, tuples[ i ^ 1 ][ 2 ].disable, tuples[ 2 ][ 2 ].lock );
			}

			// deferred[ resolve | reject | notify ]
			deferred[ tuple[0] ] = function() {
				deferred[ tuple[0] + "With" ]( this === deferred ? promise : this, arguments );
				return this;
			};
			deferred[ tuple[0] + "With" ] = list.fireWith;
		});

		// Make the deferred a promise
		promise.promise( deferred );

		// Call given func if any
		if ( func ) {
			func.call( deferred, deferred );
		}

		// All done!
		return deferred;
	},

	// Deferred helper
	when: function( subordinate /* , ..., subordinateN */ ) {
		var i = 0,
			resolveValues = core_slice.call( arguments ),
			length = resolveValues.length,

			// the count of uncompleted subordinates
			remaining = length !== 1 || ( subordinate && jQuery.isFunction( subordinate.promise ) ) ? length : 0,

			// the master Deferred. If resolveValues consist of only a single Deferred, just use that.
			deferred = remaining === 1 ? subordinate : jQuery.Deferred(),

			// Update function for both resolve and progress values
			updateFunc = function( i, contexts, values ) {
				return function( value ) {
					contexts[ i ] = this;
					values[ i ] = arguments.length > 1 ? core_slice.call( arguments ) : value;
					if( values === progressValues ) {
						deferred.notifyWith( contexts, values );
					} else if ( !( --remaining ) ) {
						deferred.resolveWith( contexts, values );
					}
				};
			},

			progressValues, progressContexts, resolveContexts;

		// add listeners to Deferred subordinates; treat others as resolved
		if ( length > 1 ) {
			progressValues = new Array( length );
			progressContexts = new Array( length );
			resolveContexts = new Array( length );
			for ( ; i < length; i++ ) {
				if ( resolveValues[ i ] && jQuery.isFunction( resolveValues[ i ].promise ) ) {
					resolveValues[ i ].promise()
						.done( updateFunc( i, resolveContexts, resolveValues ) )
						.fail( deferred.reject )
						.progress( updateFunc( i, progressContexts, progressValues ) );
				} else {
					--remaining;
				}
			}
		}

		// if we're not waiting on anything, resolve the master
		if ( !remaining ) {
			deferred.resolveWith( resolveContexts, resolveValues );
		}

		return deferred.promise();
	}
});
jQuery.support = (function( support ) {

	var all, a, input, select, fragment, opt, eventName, isSupported, i,
		div = document.createElement("div");

	// Setup
	div.setAttribute( "className", "t" );
	div.innerHTML = "  <link/><table></table><a href='/a'>a</a><input type='checkbox'/>";

	// Finish early in limited (non-browser) environments
	all = div.getElementsByTagName("*") || [];
	a = div.getElementsByTagName("a")[ 0 ];
	if ( !a || !a.style || !all.length ) {
		return support;
	}

	// First batch of tests
	select = document.createElement("select");
	opt = select.appendChild( document.createElement("option") );
	input = div.getElementsByTagName("input")[ 0 ];

	a.style.cssText = "top:1px;float:left;opacity:.5";

	// Test setAttribute on camelCase class. If it works, we need attrFixes when doing get/setAttribute (ie6/7)
	support.getSetAttribute = div.className !== "t";

	// IE strips leading whitespace when .innerHTML is used
	support.leadingWhitespace = div.firstChild.nodeType === 3;

	// Make sure that tbody elements aren't automatically inserted
	// IE will insert them into empty tables
	support.tbody = !div.getElementsByTagName("tbody").length;

	// Make sure that link elements get serialized correctly by innerHTML
	// This requires a wrapper element in IE
	support.htmlSerialize = !!div.getElementsByTagName("link").length;

	// Get the style information from getAttribute
	// (IE uses .cssText instead)
	support.style = /top/.test( a.getAttribute("style") );

	// Make sure that URLs aren't manipulated
	// (IE normalizes it by default)
	support.hrefNormalized = a.getAttribute("href") === "/a";

	// Make sure that element opacity exists
	// (IE uses filter instead)
	// Use a regex to work around a WebKit issue. See #5145
	support.opacity = /^0.5/.test( a.style.opacity );

	// Verify style float existence
	// (IE uses styleFloat instead of cssFloat)
	support.cssFloat = !!a.style.cssFloat;

	// Check the default checkbox/radio value ("" on WebKit; "on" elsewhere)
	support.checkOn = !!input.value;

	// Make sure that a selected-by-default option has a working selected property.
	// (WebKit defaults to false instead of true, IE too, if it's in an optgroup)
	support.optSelected = opt.selected;

	// Tests for enctype support on a form (#6743)
	support.enctype = !!document.createElement("form").enctype;

	// Makes sure cloning an html5 element does not cause problems
	// Where outerHTML is undefined, this still works
	support.html5Clone = document.createElement("nav").cloneNode( true ).outerHTML !== "<:nav></:nav>";

	// Will be defined later
	support.inlineBlockNeedsLayout = false;
	support.shrinkWrapBlocks = false;
	support.pixelPosition = false;
	support.deleteExpando = true;
	support.noCloneEvent = true;
	support.reliableMarginRight = true;
	support.boxSizingReliable = true;

	// Make sure checked status is properly cloned
	input.checked = true;
	support.noCloneChecked = input.cloneNode( true ).checked;

	// Make sure that the options inside disabled selects aren't marked as disabled
	// (WebKit marks them as disabled)
	select.disabled = true;
	support.optDisabled = !opt.disabled;

	// Support: IE<9
	try {
		delete div.test;
	} catch( e ) {
		support.deleteExpando = false;
	}

	// Check if we can trust getAttribute("value")
	input = document.createElement("input");
	input.setAttribute( "value", "" );
	support.input = input.getAttribute( "value" ) === "";

	// Check if an input maintains its value after becoming a radio
	input.value = "t";
	input.setAttribute( "type", "radio" );
	support.radioValue = input.value === "t";

	// #11217 - WebKit loses check when the name is after the checked attribute
	input.setAttribute( "checked", "t" );
	input.setAttribute( "name", "t" );

	fragment = document.createDocumentFragment();
	fragment.appendChild( input );

	// Check if a disconnected checkbox will retain its checked
	// value of true after appended to the DOM (IE6/7)
	support.appendChecked = input.checked;

	// WebKit doesn't clone checked state correctly in fragments
	support.checkClone = fragment.cloneNode( true ).cloneNode( true ).lastChild.checked;

	// Support: IE<9
	// Opera does not clone events (and typeof div.attachEvent === undefined).
	// IE9-10 clones events bound via attachEvent, but they don't trigger with .click()
	if ( div.attachEvent ) {
		div.attachEvent( "onclick", function() {
			support.noCloneEvent = false;
		});

		div.cloneNode( true ).click();
	}

	// Support: IE<9 (lack submit/change bubble), Firefox 17+ (lack focusin event)
	// Beware of CSP restrictions (https://developer.mozilla.org/en/Security/CSP)
	for ( i in { submit: true, change: true, focusin: true }) {
		div.setAttribute( eventName = "on" + i, "t" );

		support[ i + "Bubbles" ] = eventName in window || div.attributes[ eventName ].expando === false;
	}

	div.style.backgroundClip = "content-box";
	div.cloneNode( true ).style.backgroundClip = "";
	support.clearCloneStyle = div.style.backgroundClip === "content-box";

	// Support: IE<9
	// Iteration over object's inherited properties before its own.
	for ( i in jQuery( support ) ) {
		break;
	}
	support.ownLast = i !== "0";

	// Run tests that need a body at doc ready
	jQuery(function() {
		var container, marginDiv, tds,
			divReset = "padding:0;margin:0;border:0;display:block;box-sizing:content-box;-moz-box-sizing:content-box;-webkit-box-sizing:content-box;",
			body = document.getElementsByTagName("body")[0];

		if ( !body ) {
			// Return for frameset docs that don't have a body
			return;
		}

		container = document.createElement("div");
		container.style.cssText = "border:0;width:0;height:0;position:absolute;top:0;left:-9999px;margin-top:1px";

		body.appendChild( container ).appendChild( div );

		// Support: IE8
		// Check if table cells still have offsetWidth/Height when they are set
		// to display:none and there are still other visible table cells in a
		// table row; if so, offsetWidth/Height are not reliable for use when
		// determining if an element has been hidden directly using
		// display:none (it is still safe to use offsets if a parent element is
		// hidden; don safety goggles and see bug #4512 for more information).
		div.innerHTML = "<table><tr><td></td><td>t</td></tr></table>";
		tds = div.getElementsByTagName("td");
		tds[ 0 ].style.cssText = "padding:0;margin:0;border:0;display:none";
		isSupported = ( tds[ 0 ].offsetHeight === 0 );

		tds[ 0 ].style.display = "";
		tds[ 1 ].style.display = "none";

		// Support: IE8
		// Check if empty table cells still have offsetWidth/Height
		support.reliableHiddenOffsets = isSupported && ( tds[ 0 ].offsetHeight === 0 );

		// Check box-sizing and margin behavior.
		div.innerHTML = "";
		div.style.cssText = "box-sizing:border-box;-moz-box-sizing:border-box;-webkit-box-sizing:border-box;padding:1px;border:1px;display:block;width:4px;margin-top:1%;position:absolute;top:1%;";

		// Workaround failing boxSizing test due to offsetWidth returning wrong value
		// with some non-1 values of body zoom, ticket #13543
		jQuery.swap( body, body.style.zoom != null ? { zoom: 1 } : {}, function() {
			support.boxSizing = div.offsetWidth === 4;
		});

		// Use window.getComputedStyle because jsdom on node.js will break without it.
		if ( window.getComputedStyle ) {
			support.pixelPosition = ( window.getComputedStyle( div, null ) || {} ).top !== "1%";
			support.boxSizingReliable = ( window.getComputedStyle( div, null ) || { width: "4px" } ).width === "4px";

			// Check if div with explicit width and no margin-right incorrectly
			// gets computed margin-right based on width of container. (#3333)
			// Fails in WebKit before Feb 2011 nightlies
			// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
			marginDiv = div.appendChild( document.createElement("div") );
			marginDiv.style.cssText = div.style.cssText = divReset;
			marginDiv.style.marginRight = marginDiv.style.width = "0";
			div.style.width = "1px";

			support.reliableMarginRight =
				!parseFloat( ( window.getComputedStyle( marginDiv, null ) || {} ).marginRight );
		}

		if ( typeof div.style.zoom !== core_strundefined ) {
			// Support: IE<8
			// Check if natively block-level elements act like inline-block
			// elements when setting their display to 'inline' and giving
			// them layout
			div.innerHTML = "";
			div.style.cssText = divReset + "width:1px;padding:1px;display:inline;zoom:1";
			support.inlineBlockNeedsLayout = ( div.offsetWidth === 3 );

			// Support: IE6
			// Check if elements with layout shrink-wrap their children
			div.style.display = "block";
			div.innerHTML = "<div></div>";
			div.firstChild.style.width = "5px";
			support.shrinkWrapBlocks = ( div.offsetWidth !== 3 );

			if ( support.inlineBlockNeedsLayout ) {
				// Prevent IE 6 from affecting layout for positioned elements #11048
				// Prevent IE from shrinking the body in IE 7 mode #12869
				// Support: IE<8
				body.style.zoom = 1;
			}
		}

		body.removeChild( container );

		// Null elements to avoid leaks in IE
		container = div = tds = marginDiv = null;
	});

	// Null elements to avoid leaks in IE
	all = select = fragment = opt = a = input = null;

	return support;
})({});

var rbrace = /(?:\{[\s\S]*\}|\[[\s\S]*\])$/,
	rmultiDash = /([A-Z])/g;

function internalData( elem, name, data, pvt /* Internal Use Only */ ){
	if ( !jQuery.acceptData( elem ) ) {
		return;
	}

	var ret, thisCache,
		internalKey = jQuery.expando,

		// We have to handle DOM nodes and JS objects differently because IE6-7
		// can't GC object references properly across the DOM-JS boundary
		isNode = elem.nodeType,

		// Only DOM nodes need the global jQuery cache; JS object data is
		// attached directly to the object so GC can occur automatically
		cache = isNode ? jQuery.cache : elem,

		// Only defining an ID for JS objects if its cache already exists allows
		// the code to shortcut on the same path as a DOM node with no cache
		id = isNode ? elem[ internalKey ] : elem[ internalKey ] && internalKey;

	// Avoid doing any more work than we need to when trying to get data on an
	// object that has no data at all
	if ( (!id || !cache[id] || (!pvt && !cache[id].data)) && data === undefined && typeof name === "string" ) {
		return;
	}

	if ( !id ) {
		// Only DOM nodes need a new unique ID for each element since their data
		// ends up in the global cache
		if ( isNode ) {
			id = elem[ internalKey ] = core_deletedIds.pop() || jQuery.guid++;
		} else {
			id = internalKey;
		}
	}

	if ( !cache[ id ] ) {
		// Avoid exposing jQuery metadata on plain JS objects when the object
		// is serialized using JSON.stringify
		cache[ id ] = isNode ? {} : { toJSON: jQuery.noop };
	}

	// An object can be passed to jQuery.data instead of a key/value pair; this gets
	// shallow copied over onto the existing cache
	if ( typeof name === "object" || typeof name === "function" ) {
		if ( pvt ) {
			cache[ id ] = jQuery.extend( cache[ id ], name );
		} else {
			cache[ id ].data = jQuery.extend( cache[ id ].data, name );
		}
	}

	thisCache = cache[ id ];

	// jQuery data() is stored in a separate object inside the object's internal data
	// cache in order to avoid key collisions between internal data and user-defined
	// data.
	if ( !pvt ) {
		if ( !thisCache.data ) {
			thisCache.data = {};
		}

		thisCache = thisCache.data;
	}

	if ( data !== undefined ) {
		thisCache[ jQuery.camelCase( name ) ] = data;
	}

	// Check for both converted-to-camel and non-converted data property names
	// If a data property was specified
	if ( typeof name === "string" ) {

		// First Try to find as-is property data
		ret = thisCache[ name ];

		// Test for null|undefined property data
		if ( ret == null ) {

			// Try to find the camelCased property
			ret = thisCache[ jQuery.camelCase( name ) ];
		}
	} else {
		ret = thisCache;
	}

	return ret;
}

function internalRemoveData( elem, name, pvt ) {
	if ( !jQuery.acceptData( elem ) ) {
		return;
	}

	var thisCache, i,
		isNode = elem.nodeType,

		// See jQuery.data for more information
		cache = isNode ? jQuery.cache : elem,
		id = isNode ? elem[ jQuery.expando ] : jQuery.expando;

	// If there is already no cache entry for this object, there is no
	// purpose in continuing
	if ( !cache[ id ] ) {
		return;
	}

	if ( name ) {

		thisCache = pvt ? cache[ id ] : cache[ id ].data;

		if ( thisCache ) {

			// Support array or space separated string names for data keys
			if ( !jQuery.isArray( name ) ) {

				// try the string as a key before any manipulation
				if ( name in thisCache ) {
					name = [ name ];
				} else {

					// split the camel cased version by spaces unless a key with the spaces exists
					name = jQuery.camelCase( name );
					if ( name in thisCache ) {
						name = [ name ];
					} else {
						name = name.split(" ");
					}
				}
			} else {
				// If "name" is an array of keys...
				// When data is initially created, via ("key", "val") signature,
				// keys will be converted to camelCase.
				// Since there is no way to tell _how_ a key was added, remove
				// both plain key and camelCase key. #12786
				// This will only penalize the array argument path.
				name = name.concat( jQuery.map( name, jQuery.camelCase ) );
			}

			i = name.length;
			while ( i-- ) {
				delete thisCache[ name[i] ];
			}

			// If there is no data left in the cache, we want to continue
			// and let the cache object itself get destroyed
			if ( pvt ? !isEmptyDataObject(thisCache) : !jQuery.isEmptyObject(thisCache) ) {
				return;
			}
		}
	}

	// See jQuery.data for more information
	if ( !pvt ) {
		delete cache[ id ].data;

		// Don't destroy the parent cache unless the internal data object
		// had been the only thing left in it
		if ( !isEmptyDataObject( cache[ id ] ) ) {
			return;
		}
	}

	// Destroy the cache
	if ( isNode ) {
		jQuery.cleanData( [ elem ], true );

	// Use delete when supported for expandos or `cache` is not a window per isWindow (#10080)
	/* jshint eqeqeq: false */
	} else if ( jQuery.support.deleteExpando || cache != cache.window ) {
		/* jshint eqeqeq: true */
		delete cache[ id ];

	// When all else fails, null
	} else {
		cache[ id ] = null;
	}
}

jQuery.extend({
	cache: {},

	// The following elements throw uncatchable exceptions if you
	// attempt to add expando properties to them.
	noData: {
		"applet": true,
		"embed": true,
		// Ban all objects except for Flash (which handle expandos)
		"object": "clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"
	},

	hasData: function( elem ) {
		elem = elem.nodeType ? jQuery.cache[ elem[jQuery.expando] ] : elem[ jQuery.expando ];
		return !!elem && !isEmptyDataObject( elem );
	},

	data: function( elem, name, data ) {
		return internalData( elem, name, data );
	},

	removeData: function( elem, name ) {
		return internalRemoveData( elem, name );
	},

	// For internal use only.
	_data: function( elem, name, data ) {
		return internalData( elem, name, data, true );
	},

	_removeData: function( elem, name ) {
		return internalRemoveData( elem, name, true );
	},

	// A method for determining if a DOM node can handle the data expando
	acceptData: function( elem ) {
		// Do not set data on non-element because it will not be cleared (#8335).
		if ( elem.nodeType && elem.nodeType !== 1 && elem.nodeType !== 9 ) {
			return false;
		}

		var noData = elem.nodeName && jQuery.noData[ elem.nodeName.toLowerCase() ];

		// nodes accept data unless otherwise specified; rejection can be conditional
		return !noData || noData !== true && elem.getAttribute("classid") === noData;
	}
});

jQuery.fn.extend({
	data: function( key, value ) {
		var attrs, name,
			data = null,
			i = 0,
			elem = this[0];

		// Special expections of .data basically thwart jQuery.access,
		// so implement the relevant behavior ourselves

		// Gets all values
		if ( key === undefined ) {
			if ( this.length ) {
				data = jQuery.data( elem );

				if ( elem.nodeType === 1 && !jQuery._data( elem, "parsedAttrs" ) ) {
					attrs = elem.attributes;
					for ( ; i < attrs.length; i++ ) {
						name = attrs[i].name;

						if ( name.indexOf("data-") === 0 ) {
							name = jQuery.camelCase( name.slice(5) );

							dataAttr( elem, name, data[ name ] );
						}
					}
					jQuery._data( elem, "parsedAttrs", true );
				}
			}

			return data;
		}

		// Sets multiple values
		if ( typeof key === "object" ) {
			return this.each(function() {
				jQuery.data( this, key );
			});
		}

		return arguments.length > 1 ?

			// Sets one value
			this.each(function() {
				jQuery.data( this, key, value );
			}) :

			// Gets one value
			// Try to fetch any internally stored data first
			elem ? dataAttr( elem, key, jQuery.data( elem, key ) ) : null;
	},

	removeData: function( key ) {
		return this.each(function() {
			jQuery.removeData( this, key );
		});
	}
});

function dataAttr( elem, key, data ) {
	// If nothing was found internally, try to fetch any
	// data from the HTML5 data-* attribute
	if ( data === undefined && elem.nodeType === 1 ) {

		var name = "data-" + key.replace( rmultiDash, "-$1" ).toLowerCase();

		data = elem.getAttribute( name );

		if ( typeof data === "string" ) {
			try {
				data = data === "true" ? true :
					data === "false" ? false :
					data === "null" ? null :
					// Only convert to a number if it doesn't change the string
					+data + "" === data ? +data :
					rbrace.test( data ) ? jQuery.parseJSON( data ) :
						data;
			} catch( e ) {}

			// Make sure we set the data so it isn't changed later
			jQuery.data( elem, key, data );

		} else {
			data = undefined;
		}
	}

	return data;
}

// checks a cache object for emptiness
function isEmptyDataObject( obj ) {
	var name;
	for ( name in obj ) {

		// if the public data object is empty, the private is still empty
		if ( name === "data" && jQuery.isEmptyObject( obj[name] ) ) {
			continue;
		}
		if ( name !== "toJSON" ) {
			return false;
		}
	}

	return true;
}
jQuery.extend({
	queue: function( elem, type, data ) {
		var queue;

		if ( elem ) {
			type = ( type || "fx" ) + "queue";
			queue = jQuery._data( elem, type );

			// Speed up dequeue by getting out quickly if this is just a lookup
			if ( data ) {
				if ( !queue || jQuery.isArray(data) ) {
					queue = jQuery._data( elem, type, jQuery.makeArray(data) );
				} else {
					queue.push( data );
				}
			}
			return queue || [];
		}
	},

	dequeue: function( elem, type ) {
		type = type || "fx";

		var queue = jQuery.queue( elem, type ),
			startLength = queue.length,
			fn = queue.shift(),
			hooks = jQuery._queueHooks( elem, type ),
			next = function() {
				jQuery.dequeue( elem, type );
			};

		// If the fx queue is dequeued, always remove the progress sentinel
		if ( fn === "inprogress" ) {
			fn = queue.shift();
			startLength--;
		}

		if ( fn ) {

			// Add a progress sentinel to prevent the fx queue from being
			// automatically dequeued
			if ( type === "fx" ) {
				queue.unshift( "inprogress" );
			}

			// clear up the last queue stop function
			delete hooks.stop;
			fn.call( elem, next, hooks );
		}

		if ( !startLength && hooks ) {
			hooks.empty.fire();
		}
	},

	// not intended for public consumption - generates a queueHooks object, or returns the current one
	_queueHooks: function( elem, type ) {
		var key = type + "queueHooks";
		return jQuery._data( elem, key ) || jQuery._data( elem, key, {
			empty: jQuery.Callbacks("once memory").add(function() {
				jQuery._removeData( elem, type + "queue" );
				jQuery._removeData( elem, key );
			})
		});
	}
});

jQuery.fn.extend({
	queue: function( type, data ) {
		var setter = 2;

		if ( typeof type !== "string" ) {
			data = type;
			type = "fx";
			setter--;
		}

		if ( arguments.length < setter ) {
			return jQuery.queue( this[0], type );
		}

		return data === undefined ?
			this :
			this.each(function() {
				var queue = jQuery.queue( this, type, data );

				// ensure a hooks for this queue
				jQuery._queueHooks( this, type );

				if ( type === "fx" && queue[0] !== "inprogress" ) {
					jQuery.dequeue( this, type );
				}
			});
	},
	dequeue: function( type ) {
		return this.each(function() {
			jQuery.dequeue( this, type );
		});
	},
	// Based off of the plugin by Clint Helfers, with permission.
	// http://blindsignals.com/index.php/2009/07/jquery-delay/
	delay: function( time, type ) {
		time = jQuery.fx ? jQuery.fx.speeds[ time ] || time : time;
		type = type || "fx";

		return this.queue( type, function( next, hooks ) {
			var timeout = setTimeout( next, time );
			hooks.stop = function() {
				clearTimeout( timeout );
			};
		});
	},
	clearQueue: function( type ) {
		return this.queue( type || "fx", [] );
	},
	// Get a promise resolved when queues of a certain type
	// are emptied (fx is the type by default)
	promise: function( type, obj ) {
		var tmp,
			count = 1,
			defer = jQuery.Deferred(),
			elements = this,
			i = this.length,
			resolve = function() {
				if ( !( --count ) ) {
					defer.resolveWith( elements, [ elements ] );
				}
			};

		if ( typeof type !== "string" ) {
			obj = type;
			type = undefined;
		}
		type = type || "fx";

		while( i-- ) {
			tmp = jQuery._data( elements[ i ], type + "queueHooks" );
			if ( tmp && tmp.empty ) {
				count++;
				tmp.empty.add( resolve );
			}
		}
		resolve();
		return defer.promise( obj );
	}
});
var nodeHook, boolHook,
	rclass = /[\t\r\n\f]/g,
	rreturn = /\r/g,
	rfocusable = /^(?:input|select|textarea|button|object)$/i,
	rclickable = /^(?:a|area)$/i,
	ruseDefault = /^(?:checked|selected)$/i,
	getSetAttribute = jQuery.support.getSetAttribute,
	getSetInput = jQuery.support.input;

jQuery.fn.extend({
	attr: function( name, value ) {
		return jQuery.access( this, jQuery.attr, name, value, arguments.length > 1 );
	},

	removeAttr: function( name ) {
		return this.each(function() {
			jQuery.removeAttr( this, name );
		});
	},

	prop: function( name, value ) {
		return jQuery.access( this, jQuery.prop, name, value, arguments.length > 1 );
	},

	removeProp: function( name ) {
		name = jQuery.propFix[ name ] || name;
		return this.each(function() {
			// try/catch handles cases where IE balks (such as removing a property on window)
			try {
				this[ name ] = undefined;
				delete this[ name ];
			} catch( e ) {}
		});
	},

	addClass: function( value ) {
		var classes, elem, cur, clazz, j,
			i = 0,
			len = this.length,
			proceed = typeof value === "string" && value;

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( j ) {
				jQuery( this ).addClass( value.call( this, j, this.className ) );
			});
		}

		if ( proceed ) {
			// The disjunction here is for better compressibility (see removeClass)
			classes = ( value || "" ).match( core_rnotwhite ) || [];

			for ( ; i < len; i++ ) {
				elem = this[ i ];
				cur = elem.nodeType === 1 && ( elem.className ?
					( " " + elem.className + " " ).replace( rclass, " " ) :
					" "
				);

				if ( cur ) {
					j = 0;
					while ( (clazz = classes[j++]) ) {
						if ( cur.indexOf( " " + clazz + " " ) < 0 ) {
							cur += clazz + " ";
						}
					}
					elem.className = jQuery.trim( cur );

				}
			}
		}

		return this;
	},

	removeClass: function( value ) {
		var classes, elem, cur, clazz, j,
			i = 0,
			len = this.length,
			proceed = arguments.length === 0 || typeof value === "string" && value;

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( j ) {
				jQuery( this ).removeClass( value.call( this, j, this.className ) );
			});
		}
		if ( proceed ) {
			classes = ( value || "" ).match( core_rnotwhite ) || [];

			for ( ; i < len; i++ ) {
				elem = this[ i ];
				// This expression is here for better compressibility (see addClass)
				cur = elem.nodeType === 1 && ( elem.className ?
					( " " + elem.className + " " ).replace( rclass, " " ) :
					""
				);

				if ( cur ) {
					j = 0;
					while ( (clazz = classes[j++]) ) {
						// Remove *all* instances
						while ( cur.indexOf( " " + clazz + " " ) >= 0 ) {
							cur = cur.replace( " " + clazz + " ", " " );
						}
					}
					elem.className = value ? jQuery.trim( cur ) : "";
				}
			}
		}

		return this;
	},

	toggleClass: function( value, stateVal ) {
		var type = typeof value;

		if ( typeof stateVal === "boolean" && type === "string" ) {
			return stateVal ? this.addClass( value ) : this.removeClass( value );
		}

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( i ) {
				jQuery( this ).toggleClass( value.call(this, i, this.className, stateVal), stateVal );
			});
		}

		return this.each(function() {
			if ( type === "string" ) {
				// toggle individual class names
				var className,
					i = 0,
					self = jQuery( this ),
					classNames = value.match( core_rnotwhite ) || [];

				while ( (className = classNames[ i++ ]) ) {
					// check each className given, space separated list
					if ( self.hasClass( className ) ) {
						self.removeClass( className );
					} else {
						self.addClass( className );
					}
				}

			// Toggle whole class name
			} else if ( type === core_strundefined || type === "boolean" ) {
				if ( this.className ) {
					// store className if set
					jQuery._data( this, "__className__", this.className );
				}

				// If the element has a class name or if we're passed "false",
				// then remove the whole classname (if there was one, the above saved it).
				// Otherwise bring back whatever was previously saved (if anything),
				// falling back to the empty string if nothing was stored.
				this.className = this.className || value === false ? "" : jQuery._data( this, "__className__" ) || "";
			}
		});
	},

	hasClass: function( selector ) {
		var className = " " + selector + " ",
			i = 0,
			l = this.length;
		for ( ; i < l; i++ ) {
			if ( this[i].nodeType === 1 && (" " + this[i].className + " ").replace(rclass, " ").indexOf( className ) >= 0 ) {
				return true;
			}
		}

		return false;
	},

	val: function( value ) {
		var ret, hooks, isFunction,
			elem = this[0];

		if ( !arguments.length ) {
			if ( elem ) {
				hooks = jQuery.valHooks[ elem.type ] || jQuery.valHooks[ elem.nodeName.toLowerCase() ];

				if ( hooks && "get" in hooks && (ret = hooks.get( elem, "value" )) !== undefined ) {
					return ret;
				}

				ret = elem.value;

				return typeof ret === "string" ?
					// handle most common string cases
					ret.replace(rreturn, "") :
					// handle cases where value is null/undef or number
					ret == null ? "" : ret;
			}

			return;
		}

		isFunction = jQuery.isFunction( value );

		return this.each(function( i ) {
			var val;

			if ( this.nodeType !== 1 ) {
				return;
			}

			if ( isFunction ) {
				val = value.call( this, i, jQuery( this ).val() );
			} else {
				val = value;
			}

			// Treat null/undefined as ""; convert numbers to string
			if ( val == null ) {
				val = "";
			} else if ( typeof val === "number" ) {
				val += "";
			} else if ( jQuery.isArray( val ) ) {
				val = jQuery.map(val, function ( value ) {
					return value == null ? "" : value + "";
				});
			}

			hooks = jQuery.valHooks[ this.type ] || jQuery.valHooks[ this.nodeName.toLowerCase() ];

			// If set returns undefined, fall back to normal setting
			if ( !hooks || !("set" in hooks) || hooks.set( this, val, "value" ) === undefined ) {
				this.value = val;
			}
		});
	}
});

jQuery.extend({
	valHooks: {
		option: {
			get: function( elem ) {
				// Use proper attribute retrieval(#6932, #12072)
				var val = jQuery.find.attr( elem, "value" );
				return val != null ?
					val :
					elem.text;
			}
		},
		select: {
			get: function( elem ) {
				var value, option,
					options = elem.options,
					index = elem.selectedIndex,
					one = elem.type === "select-one" || index < 0,
					values = one ? null : [],
					max = one ? index + 1 : options.length,
					i = index < 0 ?
						max :
						one ? index : 0;

				// Loop through all the selected options
				for ( ; i < max; i++ ) {
					option = options[ i ];

					// oldIE doesn't update selected after form reset (#2551)
					if ( ( option.selected || i === index ) &&
							// Don't return options that are disabled or in a disabled optgroup
							( jQuery.support.optDisabled ? !option.disabled : option.getAttribute("disabled") === null ) &&
							( !option.parentNode.disabled || !jQuery.nodeName( option.parentNode, "optgroup" ) ) ) {

						// Get the specific value for the option
						value = jQuery( option ).val();

						// We don't need an array for one selects
						if ( one ) {
							return value;
						}

						// Multi-Selects return an array
						values.push( value );
					}
				}

				return values;
			},

			set: function( elem, value ) {
				var optionSet, option,
					options = elem.options,
					values = jQuery.makeArray( value ),
					i = options.length;

				while ( i-- ) {
					option = options[ i ];
					if ( (option.selected = jQuery.inArray( jQuery(option).val(), values ) >= 0) ) {
						optionSet = true;
					}
				}

				// force browsers to behave consistently when non-matching value is set
				if ( !optionSet ) {
					elem.selectedIndex = -1;
				}
				return values;
			}
		}
	},

	attr: function( elem, name, value ) {
		var hooks, ret,
			nType = elem.nodeType;

		// don't get/set attributes on text, comment and attribute nodes
		if ( !elem || nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		// Fallback to prop when attributes are not supported
		if ( typeof elem.getAttribute === core_strundefined ) {
			return jQuery.prop( elem, name, value );
		}

		// All attributes are lowercase
		// Grab necessary hook if one is defined
		if ( nType !== 1 || !jQuery.isXMLDoc( elem ) ) {
			name = name.toLowerCase();
			hooks = jQuery.attrHooks[ name ] ||
				( jQuery.expr.match.bool.test( name ) ? boolHook : nodeHook );
		}

		if ( value !== undefined ) {

			if ( value === null ) {
				jQuery.removeAttr( elem, name );

			} else if ( hooks && "set" in hooks && (ret = hooks.set( elem, value, name )) !== undefined ) {
				return ret;

			} else {
				elem.setAttribute( name, value + "" );
				return value;
			}

		} else if ( hooks && "get" in hooks && (ret = hooks.get( elem, name )) !== null ) {
			return ret;

		} else {
			ret = jQuery.find.attr( elem, name );

			// Non-existent attributes return null, we normalize to undefined
			return ret == null ?
				undefined :
				ret;
		}
	},

	removeAttr: function( elem, value ) {
		var name, propName,
			i = 0,
			attrNames = value && value.match( core_rnotwhite );

		if ( attrNames && elem.nodeType === 1 ) {
			while ( (name = attrNames[i++]) ) {
				propName = jQuery.propFix[ name ] || name;

				// Boolean attributes get special treatment (#10870)
				if ( jQuery.expr.match.bool.test( name ) ) {
					// Set corresponding property to false
					if ( getSetInput && getSetAttribute || !ruseDefault.test( name ) ) {
						elem[ propName ] = false;
					// Support: IE<9
					// Also clear defaultChecked/defaultSelected (if appropriate)
					} else {
						elem[ jQuery.camelCase( "default-" + name ) ] =
							elem[ propName ] = false;
					}

				// See #9699 for explanation of this approach (setting first, then removal)
				} else {
					jQuery.attr( elem, name, "" );
				}

				elem.removeAttribute( getSetAttribute ? name : propName );
			}
		}
	},

	attrHooks: {
		type: {
			set: function( elem, value ) {
				if ( !jQuery.support.radioValue && value === "radio" && jQuery.nodeName(elem, "input") ) {
					// Setting the type on a radio button after the value resets the value in IE6-9
					// Reset value to default in case type is set after value during creation
					var val = elem.value;
					elem.setAttribute( "type", value );
					if ( val ) {
						elem.value = val;
					}
					return value;
				}
			}
		}
	},

	propFix: {
		"for": "htmlFor",
		"class": "className"
	},

	prop: function( elem, name, value ) {
		var ret, hooks, notxml,
			nType = elem.nodeType;

		// don't get/set properties on text, comment and attribute nodes
		if ( !elem || nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		notxml = nType !== 1 || !jQuery.isXMLDoc( elem );

		if ( notxml ) {
			// Fix name and attach hooks
			name = jQuery.propFix[ name ] || name;
			hooks = jQuery.propHooks[ name ];
		}

		if ( value !== undefined ) {
			return hooks && "set" in hooks && (ret = hooks.set( elem, value, name )) !== undefined ?
				ret :
				( elem[ name ] = value );

		} else {
			return hooks && "get" in hooks && (ret = hooks.get( elem, name )) !== null ?
				ret :
				elem[ name ];
		}
	},

	propHooks: {
		tabIndex: {
			get: function( elem ) {
				// elem.tabIndex doesn't always return the correct value when it hasn't been explicitly set
				// http://fluidproject.org/blog/2008/01/09/getting-setting-and-removing-tabindex-values-with-javascript/
				// Use proper attribute retrieval(#12072)
				var tabindex = jQuery.find.attr( elem, "tabindex" );

				return tabindex ?
					parseInt( tabindex, 10 ) :
					rfocusable.test( elem.nodeName ) || rclickable.test( elem.nodeName ) && elem.href ?
						0 :
						-1;
			}
		}
	}
});

// Hooks for boolean attributes
boolHook = {
	set: function( elem, value, name ) {
		if ( value === false ) {
			// Remove boolean attributes when set to false
			jQuery.removeAttr( elem, name );
		} else if ( getSetInput && getSetAttribute || !ruseDefault.test( name ) ) {
			// IE<8 needs the *property* name
			elem.setAttribute( !getSetAttribute && jQuery.propFix[ name ] || name, name );

		// Use defaultChecked and defaultSelected for oldIE
		} else {
			elem[ jQuery.camelCase( "default-" + name ) ] = elem[ name ] = true;
		}

		return name;
	}
};
jQuery.each( jQuery.expr.match.bool.source.match( /\w+/g ), function( i, name ) {
	var getter = jQuery.expr.attrHandle[ name ] || jQuery.find.attr;

	jQuery.expr.attrHandle[ name ] = getSetInput && getSetAttribute || !ruseDefault.test( name ) ?
		function( elem, name, isXML ) {
			var fn = jQuery.expr.attrHandle[ name ],
				ret = isXML ?
					undefined :
					/* jshint eqeqeq: false */
					(jQuery.expr.attrHandle[ name ] = undefined) !=
						getter( elem, name, isXML ) ?

						name.toLowerCase() :
						null;
			jQuery.expr.attrHandle[ name ] = fn;
			return ret;
		} :
		function( elem, name, isXML ) {
			return isXML ?
				undefined :
				elem[ jQuery.camelCase( "default-" + name ) ] ?
					name.toLowerCase() :
					null;
		};
});

// fix oldIE attroperties
if ( !getSetInput || !getSetAttribute ) {
	jQuery.attrHooks.value = {
		set: function( elem, value, name ) {
			if ( jQuery.nodeName( elem, "input" ) ) {
				// Does not return so that setAttribute is also used
				elem.defaultValue = value;
			} else {
				// Use nodeHook if defined (#1954); otherwise setAttribute is fine
				return nodeHook && nodeHook.set( elem, value, name );
			}
		}
	};
}

// IE6/7 do not support getting/setting some attributes with get/setAttribute
if ( !getSetAttribute ) {

	// Use this for any attribute in IE6/7
	// This fixes almost every IE6/7 issue
	nodeHook = {
		set: function( elem, value, name ) {
			// Set the existing or create a new attribute node
			var ret = elem.getAttributeNode( name );
			if ( !ret ) {
				elem.setAttributeNode(
					(ret = elem.ownerDocument.createAttribute( name ))
				);
			}

			ret.value = value += "";

			// Break association with cloned elements by also using setAttribute (#9646)
			return name === "value" || value === elem.getAttribute( name ) ?
				value :
				undefined;
		}
	};
	jQuery.expr.attrHandle.id = jQuery.expr.attrHandle.name = jQuery.expr.attrHandle.coords =
		// Some attributes are constructed with empty-string values when not defined
		function( elem, name, isXML ) {
			var ret;
			return isXML ?
				undefined :
				(ret = elem.getAttributeNode( name )) && ret.value !== "" ?
					ret.value :
					null;
		};
	jQuery.valHooks.button = {
		get: function( elem, name ) {
			var ret = elem.getAttributeNode( name );
			return ret && ret.specified ?
				ret.value :
				undefined;
		},
		set: nodeHook.set
	};

	// Set contenteditable to false on removals(#10429)
	// Setting to empty string throws an error as an invalid value
	jQuery.attrHooks.contenteditable = {
		set: function( elem, value, name ) {
			nodeHook.set( elem, value === "" ? false : value, name );
		}
	};

	// Set width and height to auto instead of 0 on empty string( Bug #8150 )
	// This is for removals
	jQuery.each([ "width", "height" ], function( i, name ) {
		jQuery.attrHooks[ name ] = {
			set: function( elem, value ) {
				if ( value === "" ) {
					elem.setAttribute( name, "auto" );
					return value;
				}
			}
		};
	});
}


// Some attributes require a special call on IE
// http://msdn.microsoft.com/en-us/library/ms536429%28VS.85%29.aspx
if ( !jQuery.support.hrefNormalized ) {
	// href/src property should get the full normalized URL (#10299/#12915)
	jQuery.each([ "href", "src" ], function( i, name ) {
		jQuery.propHooks[ name ] = {
			get: function( elem ) {
				return elem.getAttribute( name, 4 );
			}
		};
	});
}

if ( !jQuery.support.style ) {
	jQuery.attrHooks.style = {
		get: function( elem ) {
			// Return undefined in the case of empty string
			// Note: IE uppercases css property names, but if we were to .toLowerCase()
			// .cssText, that would destroy case senstitivity in URL's, like in "background"
			return elem.style.cssText || undefined;
		},
		set: function( elem, value ) {
			return ( elem.style.cssText = value + "" );
		}
	};
}

// Safari mis-reports the default selected property of an option
// Accessing the parent's selectedIndex property fixes it
if ( !jQuery.support.optSelected ) {
	jQuery.propHooks.selected = {
		get: function( elem ) {
			var parent = elem.parentNode;

			if ( parent ) {
				parent.selectedIndex;

				// Make sure that it also works with optgroups, see #5701
				if ( parent.parentNode ) {
					parent.parentNode.selectedIndex;
				}
			}
			return null;
		}
	};
}

jQuery.each([
	"tabIndex",
	"readOnly",
	"maxLength",
	"cellSpacing",
	"cellPadding",
	"rowSpan",
	"colSpan",
	"useMap",
	"frameBorder",
	"contentEditable"
], function() {
	jQuery.propFix[ this.toLowerCase() ] = this;
});

// IE6/7 call enctype encoding
if ( !jQuery.support.enctype ) {
	jQuery.propFix.enctype = "encoding";
}

// Radios and checkboxes getter/setter
jQuery.each([ "radio", "checkbox" ], function() {
	jQuery.valHooks[ this ] = {
		set: function( elem, value ) {
			if ( jQuery.isArray( value ) ) {
				return ( elem.checked = jQuery.inArray( jQuery(elem).val(), value ) >= 0 );
			}
		}
	};
	if ( !jQuery.support.checkOn ) {
		jQuery.valHooks[ this ].get = function( elem ) {
			// Support: Webkit
			// "" is returned instead of "on" if a value isn't specified
			return elem.getAttribute("value") === null ? "on" : elem.value;
		};
	}
});
var rformElems = /^(?:input|select|textarea)$/i,
	rkeyEvent = /^key/,
	rmouseEvent = /^(?:mouse|contextmenu)|click/,
	rfocusMorph = /^(?:focusinfocus|focusoutblur)$/,
	rtypenamespace = /^([^.]*)(?:\.(.+)|)$/;

function returnTrue() {
	return true;
}

function returnFalse() {
	return false;
}

function safeActiveElement() {
	try {
		return document.activeElement;
	} catch ( err ) { }
}

/*
 * Helper functions for managing events -- not part of the public interface.
 * Props to Dean Edwards' addEvent library for many of the ideas.
 */
jQuery.event = {

	global: {},

	add: function( elem, types, handler, data, selector ) {
		var tmp, events, t, handleObjIn,
			special, eventHandle, handleObj,
			handlers, type, namespaces, origType,
			elemData = jQuery._data( elem );

		// Don't attach events to noData or text/comment nodes (but allow plain objects)
		if ( !elemData ) {
			return;
		}

		// Caller can pass in an object of custom data in lieu of the handler
		if ( handler.handler ) {
			handleObjIn = handler;
			handler = handleObjIn.handler;
			selector = handleObjIn.selector;
		}

		// Make sure that the handler has a unique ID, used to find/remove it later
		if ( !handler.guid ) {
			handler.guid = jQuery.guid++;
		}

		// Init the element's event structure and main handler, if this is the first
		if ( !(events = elemData.events) ) {
			events = elemData.events = {};
		}
		if ( !(eventHandle = elemData.handle) ) {
			eventHandle = elemData.handle = function( e ) {
				// Discard the second event of a jQuery.event.trigger() and
				// when an event is called after a page has unloaded
				return typeof jQuery !== core_strundefined && (!e || jQuery.event.triggered !== e.type) ?
					jQuery.event.dispatch.apply( eventHandle.elem, arguments ) :
					undefined;
			};
			// Add elem as a property of the handle fn to prevent a memory leak with IE non-native events
			eventHandle.elem = elem;
		}

		// Handle multiple events separated by a space
		types = ( types || "" ).match( core_rnotwhite ) || [""];
		t = types.length;
		while ( t-- ) {
			tmp = rtypenamespace.exec( types[t] ) || [];
			type = origType = tmp[1];
			namespaces = ( tmp[2] || "" ).split( "." ).sort();

			// There *must* be a type, no attaching namespace-only handlers
			if ( !type ) {
				continue;
			}

			// If event changes its type, use the special event handlers for the changed type
			special = jQuery.event.special[ type ] || {};

			// If selector defined, determine special event api type, otherwise given type
			type = ( selector ? special.delegateType : special.bindType ) || type;

			// Update special based on newly reset type
			special = jQuery.event.special[ type ] || {};

			// handleObj is passed to all event handlers
			handleObj = jQuery.extend({
				type: type,
				origType: origType,
				data: data,
				handler: handler,
				guid: handler.guid,
				selector: selector,
				needsContext: selector && jQuery.expr.match.needsContext.test( selector ),
				namespace: namespaces.join(".")
			}, handleObjIn );

			// Init the event handler queue if we're the first
			if ( !(handlers = events[ type ]) ) {
				handlers = events[ type ] = [];
				handlers.delegateCount = 0;

				// Only use addEventListener/attachEvent if the special events handler returns false
				if ( !special.setup || special.setup.call( elem, data, namespaces, eventHandle ) === false ) {
					// Bind the global event handler to the element
					if ( elem.addEventListener ) {
						elem.addEventListener( type, eventHandle, false );

					} else if ( elem.attachEvent ) {
						elem.attachEvent( "on" + type, eventHandle );
					}
				}
			}

			if ( special.add ) {
				special.add.call( elem, handleObj );

				if ( !handleObj.handler.guid ) {
					handleObj.handler.guid = handler.guid;
				}
			}

			// Add to the element's handler list, delegates in front
			if ( selector ) {
				handlers.splice( handlers.delegateCount++, 0, handleObj );
			} else {
				handlers.push( handleObj );
			}

			// Keep track of which events have ever been used, for event optimization
			jQuery.event.global[ type ] = true;
		}

		// Nullify elem to prevent memory leaks in IE
		elem = null;
	},

	// Detach an event or set of events from an element
	remove: function( elem, types, handler, selector, mappedTypes ) {
		var j, handleObj, tmp,
			origCount, t, events,
			special, handlers, type,
			namespaces, origType,
			elemData = jQuery.hasData( elem ) && jQuery._data( elem );

		if ( !elemData || !(events = elemData.events) ) {
			return;
		}

		// Once for each type.namespace in types; type may be omitted
		types = ( types || "" ).match( core_rnotwhite ) || [""];
		t = types.length;
		while ( t-- ) {
			tmp = rtypenamespace.exec( types[t] ) || [];
			type = origType = tmp[1];
			namespaces = ( tmp[2] || "" ).split( "." ).sort();

			// Unbind all events (on this namespace, if provided) for the element
			if ( !type ) {
				for ( type in events ) {
					jQuery.event.remove( elem, type + types[ t ], handler, selector, true );
				}
				continue;
			}

			special = jQuery.event.special[ type ] || {};
			type = ( selector ? special.delegateType : special.bindType ) || type;
			handlers = events[ type ] || [];
			tmp = tmp[2] && new RegExp( "(^|\\.)" + namespaces.join("\\.(?:.*\\.|)") + "(\\.|$)" );

			// Remove matching events
			origCount = j = handlers.length;
			while ( j-- ) {
				handleObj = handlers[ j ];

				if ( ( mappedTypes || origType === handleObj.origType ) &&
					( !handler || handler.guid === handleObj.guid ) &&
					( !tmp || tmp.test( handleObj.namespace ) ) &&
					( !selector || selector === handleObj.selector || selector === "**" && handleObj.selector ) ) {
					handlers.splice( j, 1 );

					if ( handleObj.selector ) {
						handlers.delegateCount--;
					}
					if ( special.remove ) {
						special.remove.call( elem, handleObj );
					}
				}
			}

			// Remove generic event handler if we removed something and no more handlers exist
			// (avoids potential for endless recursion during removal of special event handlers)
			if ( origCount && !handlers.length ) {
				if ( !special.teardown || special.teardown.call( elem, namespaces, elemData.handle ) === false ) {
					jQuery.removeEvent( elem, type, elemData.handle );
				}

				delete events[ type ];
			}
		}

		// Remove the expando if it's no longer used
		if ( jQuery.isEmptyObject( events ) ) {
			delete elemData.handle;

			// removeData also checks for emptiness and clears the expando if empty
			// so use it instead of delete
			jQuery._removeData( elem, "events" );
		}
	},

	trigger: function( event, data, elem, onlyHandlers ) {
		var handle, ontype, cur,
			bubbleType, special, tmp, i,
			eventPath = [ elem || document ],
			type = core_hasOwn.call( event, "type" ) ? event.type : event,
			namespaces = core_hasOwn.call( event, "namespace" ) ? event.namespace.split(".") : [];

		cur = tmp = elem = elem || document;

		// Don't do events on text and comment nodes
		if ( elem.nodeType === 3 || elem.nodeType === 8 ) {
			return;
		}

		// focus/blur morphs to focusin/out; ensure we're not firing them right now
		if ( rfocusMorph.test( type + jQuery.event.triggered ) ) {
			return;
		}

		if ( type.indexOf(".") >= 0 ) {
			// Namespaced trigger; create a regexp to match event type in handle()
			namespaces = type.split(".");
			type = namespaces.shift();
			namespaces.sort();
		}
		ontype = type.indexOf(":") < 0 && "on" + type;

		// Caller can pass in a jQuery.Event object, Object, or just an event type string
		event = event[ jQuery.expando ] ?
			event :
			new jQuery.Event( type, typeof event === "object" && event );

		// Trigger bitmask: & 1 for native handlers; & 2 for jQuery (always true)
		event.isTrigger = onlyHandlers ? 2 : 3;
		event.namespace = namespaces.join(".");
		event.namespace_re = event.namespace ?
			new RegExp( "(^|\\.)" + namespaces.join("\\.(?:.*\\.|)") + "(\\.|$)" ) :
			null;

		// Clean up the event in case it is being reused
		event.result = undefined;
		if ( !event.target ) {
			event.target = elem;
		}

		// Clone any incoming data and prepend the event, creating the handler arg list
		data = data == null ?
			[ event ] :
			jQuery.makeArray( data, [ event ] );

		// Allow special events to draw outside the lines
		special = jQuery.event.special[ type ] || {};
		if ( !onlyHandlers && special.trigger && special.trigger.apply( elem, data ) === false ) {
			return;
		}

		// Determine event propagation path in advance, per W3C events spec (#9951)
		// Bubble up to document, then to window; watch for a global ownerDocument var (#9724)
		if ( !onlyHandlers && !special.noBubble && !jQuery.isWindow( elem ) ) {

			bubbleType = special.delegateType || type;
			if ( !rfocusMorph.test( bubbleType + type ) ) {
				cur = cur.parentNode;
			}
			for ( ; cur; cur = cur.parentNode ) {
				eventPath.push( cur );
				tmp = cur;
			}

			// Only add window if we got to document (e.g., not plain obj or detached DOM)
			if ( tmp === (elem.ownerDocument || document) ) {
				eventPath.push( tmp.defaultView || tmp.parentWindow || window );
			}
		}

		// Fire handlers on the event path
		i = 0;
		while ( (cur = eventPath[i++]) && !event.isPropagationStopped() ) {

			event.type = i > 1 ?
				bubbleType :
				special.bindType || type;

			// jQuery handler
			handle = ( jQuery._data( cur, "events" ) || {} )[ event.type ] && jQuery._data( cur, "handle" );
			if ( handle ) {
				handle.apply( cur, data );
			}

			// Native handler
			handle = ontype && cur[ ontype ];
			if ( handle && jQuery.acceptData( cur ) && handle.apply && handle.apply( cur, data ) === false ) {
				event.preventDefault();
			}
		}
		event.type = type;

		// If nobody prevented the default action, do it now
		if ( !onlyHandlers && !event.isDefaultPrevented() ) {

			if ( (!special._default || special._default.apply( eventPath.pop(), data ) === false) &&
				jQuery.acceptData( elem ) ) {

				// Call a native DOM method on the target with the same name name as the event.
				// Can't use an .isFunction() check here because IE6/7 fails that test.
				// Don't do default actions on window, that's where global variables be (#6170)
				if ( ontype && elem[ type ] && !jQuery.isWindow( elem ) ) {

					// Don't re-trigger an onFOO event when we call its FOO() method
					tmp = elem[ ontype ];

					if ( tmp ) {
						elem[ ontype ] = null;
					}

					// Prevent re-triggering of the same event, since we already bubbled it above
					jQuery.event.triggered = type;
					try {
						elem[ type ]();
					} catch ( e ) {
						// IE<9 dies on focus/blur to hidden element (#1486,#12518)
						// only reproducible on winXP IE8 native, not IE9 in IE8 mode
					}
					jQuery.event.triggered = undefined;

					if ( tmp ) {
						elem[ ontype ] = tmp;
					}
				}
			}
		}

		return event.result;
	},

	dispatch: function( event ) {

		// Make a writable jQuery.Event from the native event object
		event = jQuery.event.fix( event );

		var i, ret, handleObj, matched, j,
			handlerQueue = [],
			args = core_slice.call( arguments ),
			handlers = ( jQuery._data( this, "events" ) || {} )[ event.type ] || [],
			special = jQuery.event.special[ event.type ] || {};

		// Use the fix-ed jQuery.Event rather than the (read-only) native event
		args[0] = event;
		event.delegateTarget = this;

		// Call the preDispatch hook for the mapped type, and let it bail if desired
		if ( special.preDispatch && special.preDispatch.call( this, event ) === false ) {
			return;
		}

		// Determine handlers
		handlerQueue = jQuery.event.handlers.call( this, event, handlers );

		// Run delegates first; they may want to stop propagation beneath us
		i = 0;
		while ( (matched = handlerQueue[ i++ ]) && !event.isPropagationStopped() ) {
			event.currentTarget = matched.elem;

			j = 0;
			while ( (handleObj = matched.handlers[ j++ ]) && !event.isImmediatePropagationStopped() ) {

				// Triggered event must either 1) have no namespace, or
				// 2) have namespace(s) a subset or equal to those in the bound event (both can have no namespace).
				if ( !event.namespace_re || event.namespace_re.test( handleObj.namespace ) ) {

					event.handleObj = handleObj;
					event.data = handleObj.data;

					ret = ( (jQuery.event.special[ handleObj.origType ] || {}).handle || handleObj.handler )
							.apply( matched.elem, args );

					if ( ret !== undefined ) {
						if ( (event.result = ret) === false ) {
							event.preventDefault();
							event.stopPropagation();
						}
					}
				}
			}
		}

		// Call the postDispatch hook for the mapped type
		if ( special.postDispatch ) {
			special.postDispatch.call( this, event );
		}

		return event.result;
	},

	handlers: function( event, handlers ) {
		var sel, handleObj, matches, i,
			handlerQueue = [],
			delegateCount = handlers.delegateCount,
			cur = event.target;

		// Find delegate handlers
		// Black-hole SVG <use> instance trees (#13180)
		// Avoid non-left-click bubbling in Firefox (#3861)
		if ( delegateCount && cur.nodeType && (!event.button || event.type !== "click") ) {

			/* jshint eqeqeq: false */
			for ( ; cur != this; cur = cur.parentNode || this ) {
				/* jshint eqeqeq: true */

				// Don't check non-elements (#13208)
				// Don't process clicks on disabled elements (#6911, #8165, #11382, #11764)
				if ( cur.nodeType === 1 && (cur.disabled !== true || event.type !== "click") ) {
					matches = [];
					for ( i = 0; i < delegateCount; i++ ) {
						handleObj = handlers[ i ];

						// Don't conflict with Object.prototype properties (#13203)
						sel = handleObj.selector + " ";

						if ( matches[ sel ] === undefined ) {
							matches[ sel ] = handleObj.needsContext ?
								jQuery( sel, this ).index( cur ) >= 0 :
								jQuery.find( sel, this, null, [ cur ] ).length;
						}
						if ( matches[ sel ] ) {
							matches.push( handleObj );
						}
					}
					if ( matches.length ) {
						handlerQueue.push({ elem: cur, handlers: matches });
					}
				}
			}
		}

		// Add the remaining (directly-bound) handlers
		if ( delegateCount < handlers.length ) {
			handlerQueue.push({ elem: this, handlers: handlers.slice( delegateCount ) });
		}

		return handlerQueue;
	},

	fix: function( event ) {
		if ( event[ jQuery.expando ] ) {
			return event;
		}

		// Create a writable copy of the event object and normalize some properties
		var i, prop, copy,
			type = event.type,
			originalEvent = event,
			fixHook = this.fixHooks[ type ];

		if ( !fixHook ) {
			this.fixHooks[ type ] = fixHook =
				rmouseEvent.test( type ) ? this.mouseHooks :
				rkeyEvent.test( type ) ? this.keyHooks :
				{};
		}
		copy = fixHook.props ? this.props.concat( fixHook.props ) : this.props;

		event = new jQuery.Event( originalEvent );

		i = copy.length;
		while ( i-- ) {
			prop = copy[ i ];
			event[ prop ] = originalEvent[ prop ];
		}

		// Support: IE<9
		// Fix target property (#1925)
		if ( !event.target ) {
			event.target = originalEvent.srcElement || document;
		}

		// Support: Chrome 23+, Safari?
		// Target should not be a text node (#504, #13143)
		if ( event.target.nodeType === 3 ) {
			event.target = event.target.parentNode;
		}

		// Support: IE<9
		// For mouse/key events, metaKey==false if it's undefined (#3368, #11328)
		event.metaKey = !!event.metaKey;

		return fixHook.filter ? fixHook.filter( event, originalEvent ) : event;
	},

	// Includes some event props shared by KeyEvent and MouseEvent
	props: "altKey bubbles cancelable ctrlKey currentTarget eventPhase metaKey relatedTarget shiftKey target timeStamp view which".split(" "),

	fixHooks: {},

	keyHooks: {
		props: "char charCode key keyCode".split(" "),
		filter: function( event, original ) {

			// Add which for key events
			if ( event.which == null ) {
				event.which = original.charCode != null ? original.charCode : original.keyCode;
			}

			return event;
		}
	},

	mouseHooks: {
		props: "button buttons clientX clientY fromElement offsetX offsetY pageX pageY screenX screenY toElement".split(" "),
		filter: function( event, original ) {
			var body, eventDoc, doc,
				button = original.button,
				fromElement = original.fromElement;

			// Calculate pageX/Y if missing and clientX/Y available
			if ( event.pageX == null && original.clientX != null ) {
				eventDoc = event.target.ownerDocument || document;
				doc = eventDoc.documentElement;
				body = eventDoc.body;

				event.pageX = original.clientX + ( doc && doc.scrollLeft || body && body.scrollLeft || 0 ) - ( doc && doc.clientLeft || body && body.clientLeft || 0 );
				event.pageY = original.clientY + ( doc && doc.scrollTop  || body && body.scrollTop  || 0 ) - ( doc && doc.clientTop  || body && body.clientTop  || 0 );
			}

			// Add relatedTarget, if necessary
			if ( !event.relatedTarget && fromElement ) {
				event.relatedTarget = fromElement === event.target ? original.toElement : fromElement;
			}

			// Add which for click: 1 === left; 2 === middle; 3 === right
			// Note: button is not normalized, so don't use it
			if ( !event.which && button !== undefined ) {
				event.which = ( button & 1 ? 1 : ( button & 2 ? 3 : ( button & 4 ? 2 : 0 ) ) );
			}

			return event;
		}
	},

	special: {
		load: {
			// Prevent triggered image.load events from bubbling to window.load
			noBubble: true
		},
		focus: {
			// Fire native event if possible so blur/focus sequence is correct
			trigger: function() {
				if ( this !== safeActiveElement() && this.focus ) {
					try {
						this.focus();
						return false;
					} catch ( e ) {
						// Support: IE<9
						// If we error on focus to hidden element (#1486, #12518),
						// let .trigger() run the handlers
					}
				}
			},
			delegateType: "focusin"
		},
		blur: {
			trigger: function() {
				if ( this === safeActiveElement() && this.blur ) {
					this.blur();
					return false;
				}
			},
			delegateType: "focusout"
		},
		click: {
			// For checkbox, fire native event so checked state will be right
			trigger: function() {
				if ( jQuery.nodeName( this, "input" ) && this.type === "checkbox" && this.click ) {
					this.click();
					return false;
				}
			},

			// For cross-browser consistency, don't fire native .click() on links
			_default: function( event ) {
				return jQuery.nodeName( event.target, "a" );
			}
		},

		beforeunload: {
			postDispatch: function( event ) {

				// Even when returnValue equals to undefined Firefox will still show alert
				if ( event.result !== undefined ) {
					event.originalEvent.returnValue = event.result;
				}
			}
		}
	},

	simulate: function( type, elem, event, bubble ) {
		// Piggyback on a donor event to simulate a different one.
		// Fake originalEvent to avoid donor's stopPropagation, but if the
		// simulated event prevents default then we do the same on the donor.
		var e = jQuery.extend(
			new jQuery.Event(),
			event,
			{
				type: type,
				isSimulated: true,
				originalEvent: {}
			}
		);
		if ( bubble ) {
			jQuery.event.trigger( e, null, elem );
		} else {
			jQuery.event.dispatch.call( elem, e );
		}
		if ( e.isDefaultPrevented() ) {
			event.preventDefault();
		}
	}
};

jQuery.removeEvent = document.removeEventListener ?
	function( elem, type, handle ) {
		if ( elem.removeEventListener ) {
			elem.removeEventListener( type, handle, false );
		}
	} :
	function( elem, type, handle ) {
		var name = "on" + type;

		if ( elem.detachEvent ) {

			// #8545, #7054, preventing memory leaks for custom events in IE6-8
			// detachEvent needed property on element, by name of that event, to properly expose it to GC
			if ( typeof elem[ name ] === core_strundefined ) {
				elem[ name ] = null;
			}

			elem.detachEvent( name, handle );
		}
	};

jQuery.Event = function( src, props ) {
	// Allow instantiation without the 'new' keyword
	if ( !(this instanceof jQuery.Event) ) {
		return new jQuery.Event( src, props );
	}

	// Event object
	if ( src && src.type ) {
		this.originalEvent = src;
		this.type = src.type;

		// Events bubbling up the document may have been marked as prevented
		// by a handler lower down the tree; reflect the correct value.
		this.isDefaultPrevented = ( src.defaultPrevented || src.returnValue === false ||
			src.getPreventDefault && src.getPreventDefault() ) ? returnTrue : returnFalse;

	// Event type
	} else {
		this.type = src;
	}

	// Put explicitly provided properties onto the event object
	if ( props ) {
		jQuery.extend( this, props );
	}

	// Create a timestamp if incoming event doesn't have one
	this.timeStamp = src && src.timeStamp || jQuery.now();

	// Mark it as fixed
	this[ jQuery.expando ] = true;
};

// jQuery.Event is based on DOM3 Events as specified by the ECMAScript Language Binding
// http://www.w3.org/TR/2003/WD-DOM-Level-3-Events-20030331/ecma-script-binding.html
jQuery.Event.prototype = {
	isDefaultPrevented: returnFalse,
	isPropagationStopped: returnFalse,
	isImmediatePropagationStopped: returnFalse,

	preventDefault: function() {
		var e = this.originalEvent;

		this.isDefaultPrevented = returnTrue;
		if ( !e ) {
			return;
		}

		// If preventDefault exists, run it on the original event
		if ( e.preventDefault ) {
			e.preventDefault();

		// Support: IE
		// Otherwise set the returnValue property of the original event to false
		} else {
			e.returnValue = false;
		}
	},
	stopPropagation: function() {
		var e = this.originalEvent;

		this.isPropagationStopped = returnTrue;
		if ( !e ) {
			return;
		}
		// If stopPropagation exists, run it on the original event
		if ( e.stopPropagation ) {
			e.stopPropagation();
		}

		// Support: IE
		// Set the cancelBubble property of the original event to true
		e.cancelBubble = true;
	},
	stopImmediatePropagation: function() {
		this.isImmediatePropagationStopped = returnTrue;
		this.stopPropagation();
	}
};

// Create mouseenter/leave events using mouseover/out and event-time checks
jQuery.each({
	mouseenter: "mouseover",
	mouseleave: "mouseout"
}, function( orig, fix ) {
	jQuery.event.special[ orig ] = {
		delegateType: fix,
		bindType: fix,

		handle: function( event ) {
			var ret,
				target = this,
				related = event.relatedTarget,
				handleObj = event.handleObj;

			// For mousenter/leave call the handler if related is outside the target.
			// NB: No relatedTarget if the mouse left/entered the browser window
			if ( !related || (related !== target && !jQuery.contains( target, related )) ) {
				event.type = handleObj.origType;
				ret = handleObj.handler.apply( this, arguments );
				event.type = fix;
			}
			return ret;
		}
	};
});

// IE submit delegation
if ( !jQuery.support.submitBubbles ) {

	jQuery.event.special.submit = {
		setup: function() {
			// Only need this for delegated form submit events
			if ( jQuery.nodeName( this, "form" ) ) {
				return false;
			}

			// Lazy-add a submit handler when a descendant form may potentially be submitted
			jQuery.event.add( this, "click._submit keypress._submit", function( e ) {
				// Node name check avoids a VML-related crash in IE (#9807)
				var elem = e.target,
					form = jQuery.nodeName( elem, "input" ) || jQuery.nodeName( elem, "button" ) ? elem.form : undefined;
				if ( form && !jQuery._data( form, "submitBubbles" ) ) {
					jQuery.event.add( form, "submit._submit", function( event ) {
						event._submit_bubble = true;
					});
					jQuery._data( form, "submitBubbles", true );
				}
			});
			// return undefined since we don't need an event listener
		},

		postDispatch: function( event ) {
			// If form was submitted by the user, bubble the event up the tree
			if ( event._submit_bubble ) {
				delete event._submit_bubble;
				if ( this.parentNode && !event.isTrigger ) {
					jQuery.event.simulate( "submit", this.parentNode, event, true );
				}
			}
		},

		teardown: function() {
			// Only need this for delegated form submit events
			if ( jQuery.nodeName( this, "form" ) ) {
				return false;
			}

			// Remove delegated handlers; cleanData eventually reaps submit handlers attached above
			jQuery.event.remove( this, "._submit" );
		}
	};
}

// IE change delegation and checkbox/radio fix
if ( !jQuery.support.changeBubbles ) {

	jQuery.event.special.change = {

		setup: function() {

			if ( rformElems.test( this.nodeName ) ) {
				// IE doesn't fire change on a check/radio until blur; trigger it on click
				// after a propertychange. Eat the blur-change in special.change.handle.
				// This still fires onchange a second time for check/radio after blur.
				if ( this.type === "checkbox" || this.type === "radio" ) {
					jQuery.event.add( this, "propertychange._change", function( event ) {
						if ( event.originalEvent.propertyName === "checked" ) {
							this._just_changed = true;
						}
					});
					jQuery.event.add( this, "click._change", function( event ) {
						if ( this._just_changed && !event.isTrigger ) {
							this._just_changed = false;
						}
						// Allow triggered, simulated change events (#11500)
						jQuery.event.simulate( "change", this, event, true );
					});
				}
				return false;
			}
			// Delegated event; lazy-add a change handler on descendant inputs
			jQuery.event.add( this, "beforeactivate._change", function( e ) {
				var elem = e.target;

				if ( rformElems.test( elem.nodeName ) && !jQuery._data( elem, "changeBubbles" ) ) {
					jQuery.event.add( elem, "change._change", function( event ) {
						if ( this.parentNode && !event.isSimulated && !event.isTrigger ) {
							jQuery.event.simulate( "change", this.parentNode, event, true );
						}
					});
					jQuery._data( elem, "changeBubbles", true );
				}
			});
		},

		handle: function( event ) {
			var elem = event.target;

			// Swallow native change events from checkbox/radio, we already triggered them above
			if ( this !== elem || event.isSimulated || event.isTrigger || (elem.type !== "radio" && elem.type !== "checkbox") ) {
				return event.handleObj.handler.apply( this, arguments );
			}
		},

		teardown: function() {
			jQuery.event.remove( this, "._change" );

			return !rformElems.test( this.nodeName );
		}
	};
}

// Create "bubbling" focus and blur events
if ( !jQuery.support.focusinBubbles ) {
	jQuery.each({ focus: "focusin", blur: "focusout" }, function( orig, fix ) {

		// Attach a single capturing handler while someone wants focusin/focusout
		var attaches = 0,
			handler = function( event ) {
				jQuery.event.simulate( fix, event.target, jQuery.event.fix( event ), true );
			};

		jQuery.event.special[ fix ] = {
			setup: function() {
				if ( attaches++ === 0 ) {
					document.addEventListener( orig, handler, true );
				}
			},
			teardown: function() {
				if ( --attaches === 0 ) {
					document.removeEventListener( orig, handler, true );
				}
			}
		};
	});
}

jQuery.fn.extend({

	on: function( types, selector, data, fn, /*INTERNAL*/ one ) {
		var type, origFn;

		// Types can be a map of types/handlers
		if ( typeof types === "object" ) {
			// ( types-Object, selector, data )
			if ( typeof selector !== "string" ) {
				// ( types-Object, data )
				data = data || selector;
				selector = undefined;
			}
			for ( type in types ) {
				this.on( type, selector, data, types[ type ], one );
			}
			return this;
		}

		if ( data == null && fn == null ) {
			// ( types, fn )
			fn = selector;
			data = selector = undefined;
		} else if ( fn == null ) {
			if ( typeof selector === "string" ) {
				// ( types, selector, fn )
				fn = data;
				data = undefined;
			} else {
				// ( types, data, fn )
				fn = data;
				data = selector;
				selector = undefined;
			}
		}
		if ( fn === false ) {
			fn = returnFalse;
		} else if ( !fn ) {
			return this;
		}

		if ( one === 1 ) {
			origFn = fn;
			fn = function( event ) {
				// Can use an empty set, since event contains the info
				jQuery().off( event );
				return origFn.apply( this, arguments );
			};
			// Use same guid so caller can remove using origFn
			fn.guid = origFn.guid || ( origFn.guid = jQuery.guid++ );
		}
		return this.each( function() {
			jQuery.event.add( this, types, fn, data, selector );
		});
	},
	one: function( types, selector, data, fn ) {
		return this.on( types, selector, data, fn, 1 );
	},
	off: function( types, selector, fn ) {
		var handleObj, type;
		if ( types && types.preventDefault && types.handleObj ) {
			// ( event )  dispatched jQuery.Event
			handleObj = types.handleObj;
			jQuery( types.delegateTarget ).off(
				handleObj.namespace ? handleObj.origType + "." + handleObj.namespace : handleObj.origType,
				handleObj.selector,
				handleObj.handler
			);
			return this;
		}
		if ( typeof types === "object" ) {
			// ( types-object [, selector] )
			for ( type in types ) {
				this.off( type, selector, types[ type ] );
			}
			return this;
		}
		if ( selector === false || typeof selector === "function" ) {
			// ( types [, fn] )
			fn = selector;
			selector = undefined;
		}
		if ( fn === false ) {
			fn = returnFalse;
		}
		return this.each(function() {
			jQuery.event.remove( this, types, fn, selector );
		});
	},

	trigger: function( type, data ) {
		return this.each(function() {
			jQuery.event.trigger( type, data, this );
		});
	},
	triggerHandler: function( type, data ) {
		var elem = this[0];
		if ( elem ) {
			return jQuery.event.trigger( type, data, elem, true );
		}
	}
});
var isSimple = /^.[^:#\[\.,]*$/,
	rparentsprev = /^(?:parents|prev(?:Until|All))/,
	rneedsContext = jQuery.expr.match.needsContext,
	// methods guaranteed to produce a unique set when starting from a unique set
	guaranteedUnique = {
		children: true,
		contents: true,
		next: true,
		prev: true
	};

jQuery.fn.extend({
	find: function( selector ) {
		var i,
			ret = [],
			self = this,
			len = self.length;

		if ( typeof selector !== "string" ) {
			return this.pushStack( jQuery( selector ).filter(function() {
				for ( i = 0; i < len; i++ ) {
					if ( jQuery.contains( self[ i ], this ) ) {
						return true;
					}
				}
			}) );
		}

		for ( i = 0; i < len; i++ ) {
			jQuery.find( selector, self[ i ], ret );
		}

		// Needed because $( selector, context ) becomes $( context ).find( selector )
		ret = this.pushStack( len > 1 ? jQuery.unique( ret ) : ret );
		ret.selector = this.selector ? this.selector + " " + selector : selector;
		return ret;
	},

	has: function( target ) {
		var i,
			targets = jQuery( target, this ),
			len = targets.length;

		return this.filter(function() {
			for ( i = 0; i < len; i++ ) {
				if ( jQuery.contains( this, targets[i] ) ) {
					return true;
				}
			}
		});
	},

	not: function( selector ) {
		return this.pushStack( winnow(this, selector || [], true) );
	},

	filter: function( selector ) {
		return this.pushStack( winnow(this, selector || [], false) );
	},

	is: function( selector ) {
		return !!winnow(
			this,

			// If this is a positional/relative selector, check membership in the returned set
			// so $("p:first").is("p:last") won't return true for a doc with two "p".
			typeof selector === "string" && rneedsContext.test( selector ) ?
				jQuery( selector ) :
				selector || [],
			false
		).length;
	},

	closest: function( selectors, context ) {
		var cur,
			i = 0,
			l = this.length,
			ret = [],
			pos = rneedsContext.test( selectors ) || typeof selectors !== "string" ?
				jQuery( selectors, context || this.context ) :
				0;

		for ( ; i < l; i++ ) {
			for ( cur = this[i]; cur && cur !== context; cur = cur.parentNode ) {
				// Always skip document fragments
				if ( cur.nodeType < 11 && (pos ?
					pos.index(cur) > -1 :

					// Don't pass non-elements to Sizzle
					cur.nodeType === 1 &&
						jQuery.find.matchesSelector(cur, selectors)) ) {

					cur = ret.push( cur );
					break;
				}
			}
		}

		return this.pushStack( ret.length > 1 ? jQuery.unique( ret ) : ret );
	},

	// Determine the position of an element within
	// the matched set of elements
	index: function( elem ) {

		// No argument, return index in parent
		if ( !elem ) {
			return ( this[0] && this[0].parentNode ) ? this.first().prevAll().length : -1;
		}

		// index in selector
		if ( typeof elem === "string" ) {
			return jQuery.inArray( this[0], jQuery( elem ) );
		}

		// Locate the position of the desired element
		return jQuery.inArray(
			// If it receives a jQuery object, the first element is used
			elem.jquery ? elem[0] : elem, this );
	},

	add: function( selector, context ) {
		var set = typeof selector === "string" ?
				jQuery( selector, context ) :
				jQuery.makeArray( selector && selector.nodeType ? [ selector ] : selector ),
			all = jQuery.merge( this.get(), set );

		return this.pushStack( jQuery.unique(all) );
	},

	addBack: function( selector ) {
		return this.add( selector == null ?
			this.prevObject : this.prevObject.filter(selector)
		);
	}
});

function sibling( cur, dir ) {
	do {
		cur = cur[ dir ];
	} while ( cur && cur.nodeType !== 1 );

	return cur;
}

jQuery.each({
	parent: function( elem ) {
		var parent = elem.parentNode;
		return parent && parent.nodeType !== 11 ? parent : null;
	},
	parents: function( elem ) {
		return jQuery.dir( elem, "parentNode" );
	},
	parentsUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "parentNode", until );
	},
	next: function( elem ) {
		return sibling( elem, "nextSibling" );
	},
	prev: function( elem ) {
		return sibling( elem, "previousSibling" );
	},
	nextAll: function( elem ) {
		return jQuery.dir( elem, "nextSibling" );
	},
	prevAll: function( elem ) {
		return jQuery.dir( elem, "previousSibling" );
	},
	nextUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "nextSibling", until );
	},
	prevUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "previousSibling", until );
	},
	siblings: function( elem ) {
		return jQuery.sibling( ( elem.parentNode || {} ).firstChild, elem );
	},
	children: function( elem ) {
		return jQuery.sibling( elem.firstChild );
	},
	contents: function( elem ) {
		return jQuery.nodeName( elem, "iframe" ) ?
			elem.contentDocument || elem.contentWindow.document :
			jQuery.merge( [], elem.childNodes );
	}
}, function( name, fn ) {
	jQuery.fn[ name ] = function( until, selector ) {
		var ret = jQuery.map( this, fn, until );

		if ( name.slice( -5 ) !== "Until" ) {
			selector = until;
		}

		if ( selector && typeof selector === "string" ) {
			ret = jQuery.filter( selector, ret );
		}

		if ( this.length > 1 ) {
			// Remove duplicates
			if ( !guaranteedUnique[ name ] ) {
				ret = jQuery.unique( ret );
			}

			// Reverse order for parents* and prev-derivatives
			if ( rparentsprev.test( name ) ) {
				ret = ret.reverse();
			}
		}

		return this.pushStack( ret );
	};
});

jQuery.extend({
	filter: function( expr, elems, not ) {
		var elem = elems[ 0 ];

		if ( not ) {
			expr = ":not(" + expr + ")";
		}

		return elems.length === 1 && elem.nodeType === 1 ?
			jQuery.find.matchesSelector( elem, expr ) ? [ elem ] : [] :
			jQuery.find.matches( expr, jQuery.grep( elems, function( elem ) {
				return elem.nodeType === 1;
			}));
	},

	dir: function( elem, dir, until ) {
		var matched = [],
			cur = elem[ dir ];

		while ( cur && cur.nodeType !== 9 && (until === undefined || cur.nodeType !== 1 || !jQuery( cur ).is( until )) ) {
			if ( cur.nodeType === 1 ) {
				matched.push( cur );
			}
			cur = cur[dir];
		}
		return matched;
	},

	sibling: function( n, elem ) {
		var r = [];

		for ( ; n; n = n.nextSibling ) {
			if ( n.nodeType === 1 && n !== elem ) {
				r.push( n );
			}
		}

		return r;
	}
});

// Implement the identical functionality for filter and not
function winnow( elements, qualifier, not ) {
	if ( jQuery.isFunction( qualifier ) ) {
		return jQuery.grep( elements, function( elem, i ) {
			/* jshint -W018 */
			return !!qualifier.call( elem, i, elem ) !== not;
		});

	}

	if ( qualifier.nodeType ) {
		return jQuery.grep( elements, function( elem ) {
			return ( elem === qualifier ) !== not;
		});

	}

	if ( typeof qualifier === "string" ) {
		if ( isSimple.test( qualifier ) ) {
			return jQuery.filter( qualifier, elements, not );
		}

		qualifier = jQuery.filter( qualifier, elements );
	}

	return jQuery.grep( elements, function( elem ) {
		return ( jQuery.inArray( elem, qualifier ) >= 0 ) !== not;
	});
}
function createSafeFragment( document ) {
	var list = nodeNames.split( "|" ),
		safeFrag = document.createDocumentFragment();

	if ( safeFrag.createElement ) {
		while ( list.length ) {
			safeFrag.createElement(
				list.pop()
			);
		}
	}
	return safeFrag;
}

var nodeNames = "abbr|article|aside|audio|bdi|canvas|data|datalist|details|figcaption|figure|footer|" +
		"header|hgroup|mark|meter|nav|output|progress|section|summary|time|video",
	rinlinejQuery = / jQuery\d+="(?:null|\d+)"/g,
	rnoshimcache = new RegExp("<(?:" + nodeNames + ")[\\s/>]", "i"),
	rleadingWhitespace = /^\s+/,
	rxhtmlTag = /<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:]+)[^>]*)\/>/gi,
	rtagName = /<([\w:]+)/,
	rtbody = /<tbody/i,
	rhtml = /<|&#?\w+;/,
	rnoInnerhtml = /<(?:script|style|link)/i,
	manipulation_rcheckableType = /^(?:checkbox|radio)$/i,
	// checked="checked" or checked
	rchecked = /checked\s*(?:[^=]|=\s*.checked.)/i,
	rscriptType = /^$|\/(?:java|ecma)script/i,
	rscriptTypeMasked = /^true\/(.*)/,
	rcleanScript = /^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g,

	// We have to close these tags to support XHTML (#13200)
	wrapMap = {
		option: [ 1, "<select multiple='multiple'>", "</select>" ],
		legend: [ 1, "<fieldset>", "</fieldset>" ],
		area: [ 1, "<map>", "</map>" ],
		param: [ 1, "<object>", "</object>" ],
		thead: [ 1, "<table>", "</table>" ],
		tr: [ 2, "<table><tbody>", "</tbody></table>" ],
		col: [ 2, "<table><tbody></tbody><colgroup>", "</colgroup></table>" ],
		td: [ 3, "<table><tbody><tr>", "</tr></tbody></table>" ],

		// IE6-8 can't serialize link, script, style, or any html5 (NoScope) tags,
		// unless wrapped in a div with non-breaking characters in front of it.
		_default: jQuery.support.htmlSerialize ? [ 0, "", "" ] : [ 1, "X<div>", "</div>"  ]
	},
	safeFragment = createSafeFragment( document ),
	fragmentDiv = safeFragment.appendChild( document.createElement("div") );

wrapMap.optgroup = wrapMap.option;
wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
wrapMap.th = wrapMap.td;

jQuery.fn.extend({
	text: function( value ) {
		return jQuery.access( this, function( value ) {
			return value === undefined ?
				jQuery.text( this ) :
				this.empty().append( ( this[0] && this[0].ownerDocument || document ).createTextNode( value ) );
		}, null, value, arguments.length );
	},

	append: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
				var target = manipulationTarget( this, elem );
				target.appendChild( elem );
			}
		});
	},

	prepend: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
				var target = manipulationTarget( this, elem );
				target.insertBefore( elem, target.firstChild );
			}
		});
	},

	before: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.parentNode ) {
				this.parentNode.insertBefore( elem, this );
			}
		});
	},

	after: function() {
		return this.domManip( arguments, function( elem ) {
			if ( this.parentNode ) {
				this.parentNode.insertBefore( elem, this.nextSibling );
			}
		});
	},

	// keepData is for internal use only--do not document
	remove: function( selector, keepData ) {
		var elem,
			elems = selector ? jQuery.filter( selector, this ) : this,
			i = 0;

		for ( ; (elem = elems[i]) != null; i++ ) {

			if ( !keepData && elem.nodeType === 1 ) {
				jQuery.cleanData( getAll( elem ) );
			}

			if ( elem.parentNode ) {
				if ( keepData && jQuery.contains( elem.ownerDocument, elem ) ) {
					setGlobalEval( getAll( elem, "script" ) );
				}
				elem.parentNode.removeChild( elem );
			}
		}

		return this;
	},

	empty: function() {
		var elem,
			i = 0;

		for ( ; (elem = this[i]) != null; i++ ) {
			// Remove element nodes and prevent memory leaks
			if ( elem.nodeType === 1 ) {
				jQuery.cleanData( getAll( elem, false ) );
			}

			// Remove any remaining nodes
			while ( elem.firstChild ) {
				elem.removeChild( elem.firstChild );
			}

			// If this is a select, ensure that it displays empty (#12336)
			// Support: IE<9
			if ( elem.options && jQuery.nodeName( elem, "select" ) ) {
				elem.options.length = 0;
			}
		}

		return this;
	},

	clone: function( dataAndEvents, deepDataAndEvents ) {
		dataAndEvents = dataAndEvents == null ? false : dataAndEvents;
		deepDataAndEvents = deepDataAndEvents == null ? dataAndEvents : deepDataAndEvents;

		return this.map( function () {
			return jQuery.clone( this, dataAndEvents, deepDataAndEvents );
		});
	},

	html: function( value ) {
		return jQuery.access( this, function( value ) {
			var elem = this[0] || {},
				i = 0,
				l = this.length;

			if ( value === undefined ) {
				return elem.nodeType === 1 ?
					elem.innerHTML.replace( rinlinejQuery, "" ) :
					undefined;
			}

			// See if we can take a shortcut and just use innerHTML
			if ( typeof value === "string" && !rnoInnerhtml.test( value ) &&
				( jQuery.support.htmlSerialize || !rnoshimcache.test( value )  ) &&
				( jQuery.support.leadingWhitespace || !rleadingWhitespace.test( value ) ) &&
				!wrapMap[ ( rtagName.exec( value ) || ["", ""] )[1].toLowerCase() ] ) {

				value = value.replace( rxhtmlTag, "<$1></$2>" );

				try {
					for (; i < l; i++ ) {
						// Remove element nodes and prevent memory leaks
						elem = this[i] || {};
						if ( elem.nodeType === 1 ) {
							jQuery.cleanData( getAll( elem, false ) );
							elem.innerHTML = value;
						}
					}

					elem = 0;

				// If using innerHTML throws an exception, use the fallback method
				} catch(e) {}
			}

			if ( elem ) {
				this.empty().append( value );
			}
		}, null, value, arguments.length );
	},

	replaceWith: function() {
		var
			// Snapshot the DOM in case .domManip sweeps something relevant into its fragment
			args = jQuery.map( this, function( elem ) {
				return [ elem.nextSibling, elem.parentNode ];
			}),
			i = 0;

		// Make the changes, replacing each context element with the new content
		this.domManip( arguments, function( elem ) {
			var next = args[ i++ ],
				parent = args[ i++ ];

			if ( parent ) {
				// Don't use the snapshot next if it has moved (#13810)
				if ( next && next.parentNode !== parent ) {
					next = this.nextSibling;
				}
				jQuery( this ).remove();
				parent.insertBefore( elem, next );
			}
		// Allow new content to include elements from the context set
		}, true );

		// Force removal if there was no new content (e.g., from empty arguments)
		return i ? this : this.remove();
	},

	detach: function( selector ) {
		return this.remove( selector, true );
	},

	domManip: function( args, callback, allowIntersection ) {

		// Flatten any nested arrays
		args = core_concat.apply( [], args );

		var first, node, hasScripts,
			scripts, doc, fragment,
			i = 0,
			l = this.length,
			set = this,
			iNoClone = l - 1,
			value = args[0],
			isFunction = jQuery.isFunction( value );

		// We can't cloneNode fragments that contain checked, in WebKit
		if ( isFunction || !( l <= 1 || typeof value !== "string" || jQuery.support.checkClone || !rchecked.test( value ) ) ) {
			return this.each(function( index ) {
				var self = set.eq( index );
				if ( isFunction ) {
					args[0] = value.call( this, index, self.html() );
				}
				self.domManip( args, callback, allowIntersection );
			});
		}

		if ( l ) {
			fragment = jQuery.buildFragment( args, this[ 0 ].ownerDocument, false, !allowIntersection && this );
			first = fragment.firstChild;

			if ( fragment.childNodes.length === 1 ) {
				fragment = first;
			}

			if ( first ) {
				scripts = jQuery.map( getAll( fragment, "script" ), disableScript );
				hasScripts = scripts.length;

				// Use the original fragment for the last item instead of the first because it can end up
				// being emptied incorrectly in certain situations (#8070).
				for ( ; i < l; i++ ) {
					node = fragment;

					if ( i !== iNoClone ) {
						node = jQuery.clone( node, true, true );

						// Keep references to cloned scripts for later restoration
						if ( hasScripts ) {
							jQuery.merge( scripts, getAll( node, "script" ) );
						}
					}

					callback.call( this[i], node, i );
				}

				if ( hasScripts ) {
					doc = scripts[ scripts.length - 1 ].ownerDocument;

					// Reenable scripts
					jQuery.map( scripts, restoreScript );

					// Evaluate executable scripts on first document insertion
					for ( i = 0; i < hasScripts; i++ ) {
						node = scripts[ i ];
						if ( rscriptType.test( node.type || "" ) &&
							!jQuery._data( node, "globalEval" ) && jQuery.contains( doc, node ) ) {

							if ( node.src ) {
								// Hope ajax is available...
								jQuery._evalUrl( node.src );
							} else {
								jQuery.globalEval( ( node.text || node.textContent || node.innerHTML || "" ).replace( rcleanScript, "" ) );
							}
						}
					}
				}

				// Fix #11809: Avoid leaking memory
				fragment = first = null;
			}
		}

		return this;
	}
});

// Support: IE<8
// Manipulating tables requires a tbody
function manipulationTarget( elem, content ) {
	return jQuery.nodeName( elem, "table" ) &&
		jQuery.nodeName( content.nodeType === 1 ? content : content.firstChild, "tr" ) ?

		elem.getElementsByTagName("tbody")[0] ||
			elem.appendChild( elem.ownerDocument.createElement("tbody") ) :
		elem;
}

// Replace/restore the type attribute of script elements for safe DOM manipulation
function disableScript( elem ) {
	elem.type = (jQuery.find.attr( elem, "type" ) !== null) + "/" + elem.type;
	return elem;
}
function restoreScript( elem ) {
	var match = rscriptTypeMasked.exec( elem.type );
	if ( match ) {
		elem.type = match[1];
	} else {
		elem.removeAttribute("type");
	}
	return elem;
}

// Mark scripts as having already been evaluated
function setGlobalEval( elems, refElements ) {
	var elem,
		i = 0;
	for ( ; (elem = elems[i]) != null; i++ ) {
		jQuery._data( elem, "globalEval", !refElements || jQuery._data( refElements[i], "globalEval" ) );
	}
}

function cloneCopyEvent( src, dest ) {

	if ( dest.nodeType !== 1 || !jQuery.hasData( src ) ) {
		return;
	}

	var type, i, l,
		oldData = jQuery._data( src ),
		curData = jQuery._data( dest, oldData ),
		events = oldData.events;

	if ( events ) {
		delete curData.handle;
		curData.events = {};

		for ( type in events ) {
			for ( i = 0, l = events[ type ].length; i < l; i++ ) {
				jQuery.event.add( dest, type, events[ type ][ i ] );
			}
		}
	}

	// make the cloned public data object a copy from the original
	if ( curData.data ) {
		curData.data = jQuery.extend( {}, curData.data );
	}
}

function fixCloneNodeIssues( src, dest ) {
	var nodeName, e, data;

	// We do not need to do anything for non-Elements
	if ( dest.nodeType !== 1 ) {
		return;
	}

	nodeName = dest.nodeName.toLowerCase();

	// IE6-8 copies events bound via attachEvent when using cloneNode.
	if ( !jQuery.support.noCloneEvent && dest[ jQuery.expando ] ) {
		data = jQuery._data( dest );

		for ( e in data.events ) {
			jQuery.removeEvent( dest, e, data.handle );
		}

		// Event data gets referenced instead of copied if the expando gets copied too
		dest.removeAttribute( jQuery.expando );
	}

	// IE blanks contents when cloning scripts, and tries to evaluate newly-set text
	if ( nodeName === "script" && dest.text !== src.text ) {
		disableScript( dest ).text = src.text;
		restoreScript( dest );

	// IE6-10 improperly clones children of object elements using classid.
	// IE10 throws NoModificationAllowedError if parent is null, #12132.
	} else if ( nodeName === "object" ) {
		if ( dest.parentNode ) {
			dest.outerHTML = src.outerHTML;
		}

		// This path appears unavoidable for IE9. When cloning an object
		// element in IE9, the outerHTML strategy above is not sufficient.
		// If the src has innerHTML and the destination does not,
		// copy the src.innerHTML into the dest.innerHTML. #10324
		if ( jQuery.support.html5Clone && ( src.innerHTML && !jQuery.trim(dest.innerHTML) ) ) {
			dest.innerHTML = src.innerHTML;
		}

	} else if ( nodeName === "input" && manipulation_rcheckableType.test( src.type ) ) {
		// IE6-8 fails to persist the checked state of a cloned checkbox
		// or radio button. Worse, IE6-7 fail to give the cloned element
		// a checked appearance if the defaultChecked value isn't also set

		dest.defaultChecked = dest.checked = src.checked;

		// IE6-7 get confused and end up setting the value of a cloned
		// checkbox/radio button to an empty string instead of "on"
		if ( dest.value !== src.value ) {
			dest.value = src.value;
		}

	// IE6-8 fails to return the selected option to the default selected
	// state when cloning options
	} else if ( nodeName === "option" ) {
		dest.defaultSelected = dest.selected = src.defaultSelected;

	// IE6-8 fails to set the defaultValue to the correct value when
	// cloning other types of input fields
	} else if ( nodeName === "input" || nodeName === "textarea" ) {
		dest.defaultValue = src.defaultValue;
	}
}

jQuery.each({
	appendTo: "append",
	prependTo: "prepend",
	insertBefore: "before",
	insertAfter: "after",
	replaceAll: "replaceWith"
}, function( name, original ) {
	jQuery.fn[ name ] = function( selector ) {
		var elems,
			i = 0,
			ret = [],
			insert = jQuery( selector ),
			last = insert.length - 1;

		for ( ; i <= last; i++ ) {
			elems = i === last ? this : this.clone(true);
			jQuery( insert[i] )[ original ]( elems );

			// Modern browsers can apply jQuery collections as arrays, but oldIE needs a .get()
			core_push.apply( ret, elems.get() );
		}

		return this.pushStack( ret );
	};
});

function getAll( context, tag ) {
	var elems, elem,
		i = 0,
		found = typeof context.getElementsByTagName !== core_strundefined ? context.getElementsByTagName( tag || "*" ) :
			typeof context.querySelectorAll !== core_strundefined ? context.querySelectorAll( tag || "*" ) :
			undefined;

	if ( !found ) {
		for ( found = [], elems = context.childNodes || context; (elem = elems[i]) != null; i++ ) {
			if ( !tag || jQuery.nodeName( elem, tag ) ) {
				found.push( elem );
			} else {
				jQuery.merge( found, getAll( elem, tag ) );
			}
		}
	}

	return tag === undefined || tag && jQuery.nodeName( context, tag ) ?
		jQuery.merge( [ context ], found ) :
		found;
}

// Used in buildFragment, fixes the defaultChecked property
function fixDefaultChecked( elem ) {
	if ( manipulation_rcheckableType.test( elem.type ) ) {
		elem.defaultChecked = elem.checked;
	}
}

jQuery.extend({
	clone: function( elem, dataAndEvents, deepDataAndEvents ) {
		var destElements, node, clone, i, srcElements,
			inPage = jQuery.contains( elem.ownerDocument, elem );

		if ( jQuery.support.html5Clone || jQuery.isXMLDoc(elem) || !rnoshimcache.test( "<" + elem.nodeName + ">" ) ) {
			clone = elem.cloneNode( true );

		// IE<=8 does not properly clone detached, unknown element nodes
		} else {
			fragmentDiv.innerHTML = elem.outerHTML;
			fragmentDiv.removeChild( clone = fragmentDiv.firstChild );
		}

		if ( (!jQuery.support.noCloneEvent || !jQuery.support.noCloneChecked) &&
				(elem.nodeType === 1 || elem.nodeType === 11) && !jQuery.isXMLDoc(elem) ) {

			// We eschew Sizzle here for performance reasons: http://jsperf.com/getall-vs-sizzle/2
			destElements = getAll( clone );
			srcElements = getAll( elem );

			// Fix all IE cloning issues
			for ( i = 0; (node = srcElements[i]) != null; ++i ) {
				// Ensure that the destination node is not null; Fixes #9587
				if ( destElements[i] ) {
					fixCloneNodeIssues( node, destElements[i] );
				}
			}
		}

		// Copy the events from the original to the clone
		if ( dataAndEvents ) {
			if ( deepDataAndEvents ) {
				srcElements = srcElements || getAll( elem );
				destElements = destElements || getAll( clone );

				for ( i = 0; (node = srcElements[i]) != null; i++ ) {
					cloneCopyEvent( node, destElements[i] );
				}
			} else {
				cloneCopyEvent( elem, clone );
			}
		}

		// Preserve script evaluation history
		destElements = getAll( clone, "script" );
		if ( destElements.length > 0 ) {
			setGlobalEval( destElements, !inPage && getAll( elem, "script" ) );
		}

		destElements = srcElements = node = null;

		// Return the cloned set
		return clone;
	},

	buildFragment: function( elems, context, scripts, selection ) {
		var j, elem, contains,
			tmp, tag, tbody, wrap,
			l = elems.length,

			// Ensure a safe fragment
			safe = createSafeFragment( context ),

			nodes = [],
			i = 0;

		for ( ; i < l; i++ ) {
			elem = elems[ i ];

			if ( elem || elem === 0 ) {

				// Add nodes directly
				if ( jQuery.type( elem ) === "object" ) {
					jQuery.merge( nodes, elem.nodeType ? [ elem ] : elem );

				// Convert non-html into a text node
				} else if ( !rhtml.test( elem ) ) {
					nodes.push( context.createTextNode( elem ) );

				// Convert html into DOM nodes
				} else {
					tmp = tmp || safe.appendChild( context.createElement("div") );

					// Deserialize a standard representation
					tag = ( rtagName.exec( elem ) || ["", ""] )[1].toLowerCase();
					wrap = wrapMap[ tag ] || wrapMap._default;

					tmp.innerHTML = wrap[1] + elem.replace( rxhtmlTag, "<$1></$2>" ) + wrap[2];

					// Descend through wrappers to the right content
					j = wrap[0];
					while ( j-- ) {
						tmp = tmp.lastChild;
					}

					// Manually add leading whitespace removed by IE
					if ( !jQuery.support.leadingWhitespace && rleadingWhitespace.test( elem ) ) {
						nodes.push( context.createTextNode( rleadingWhitespace.exec( elem )[0] ) );
					}

					// Remove IE's autoinserted <tbody> from table fragments
					if ( !jQuery.support.tbody ) {

						// String was a <table>, *may* have spurious <tbody>
						elem = tag === "table" && !rtbody.test( elem ) ?
							tmp.firstChild :

							// String was a bare <thead> or <tfoot>
							wrap[1] === "<table>" && !rtbody.test( elem ) ?
								tmp :
								0;

						j = elem && elem.childNodes.length;
						while ( j-- ) {
							if ( jQuery.nodeName( (tbody = elem.childNodes[j]), "tbody" ) && !tbody.childNodes.length ) {
								elem.removeChild( tbody );
							}
						}
					}

					jQuery.merge( nodes, tmp.childNodes );

					// Fix #12392 for WebKit and IE > 9
					tmp.textContent = "";

					// Fix #12392 for oldIE
					while ( tmp.firstChild ) {
						tmp.removeChild( tmp.firstChild );
					}

					// Remember the top-level container for proper cleanup
					tmp = safe.lastChild;
				}
			}
		}

		// Fix #11356: Clear elements from fragment
		if ( tmp ) {
			safe.removeChild( tmp );
		}

		// Reset defaultChecked for any radios and checkboxes
		// about to be appended to the DOM in IE 6/7 (#8060)
		if ( !jQuery.support.appendChecked ) {
			jQuery.grep( getAll( nodes, "input" ), fixDefaultChecked );
		}

		i = 0;
		while ( (elem = nodes[ i++ ]) ) {

			// #4087 - If origin and destination elements are the same, and this is
			// that element, do not do anything
			if ( selection && jQuery.inArray( elem, selection ) !== -1 ) {
				continue;
			}

			contains = jQuery.contains( elem.ownerDocument, elem );

			// Append to fragment
			tmp = getAll( safe.appendChild( elem ), "script" );

			// Preserve script evaluation history
			if ( contains ) {
				setGlobalEval( tmp );
			}

			// Capture executables
			if ( scripts ) {
				j = 0;
				while ( (elem = tmp[ j++ ]) ) {
					if ( rscriptType.test( elem.type || "" ) ) {
						scripts.push( elem );
					}
				}
			}
		}

		tmp = null;

		return safe;
	},

	cleanData: function( elems, /* internal */ acceptData ) {
		var elem, type, id, data,
			i = 0,
			internalKey = jQuery.expando,
			cache = jQuery.cache,
			deleteExpando = jQuery.support.deleteExpando,
			special = jQuery.event.special;

		for ( ; (elem = elems[i]) != null; i++ ) {

			if ( acceptData || jQuery.acceptData( elem ) ) {

				id = elem[ internalKey ];
				data = id && cache[ id ];

				if ( data ) {
					if ( data.events ) {
						for ( type in data.events ) {
							if ( special[ type ] ) {
								jQuery.event.remove( elem, type );

							// This is a shortcut to avoid jQuery.event.remove's overhead
							} else {
								jQuery.removeEvent( elem, type, data.handle );
							}
						}
					}

					// Remove cache only if it was not already removed by jQuery.event.remove
					if ( cache[ id ] ) {

						delete cache[ id ];

						// IE does not allow us to delete expando properties from nodes,
						// nor does it have a removeAttribute function on Document nodes;
						// we must handle all of these cases
						if ( deleteExpando ) {
							delete elem[ internalKey ];

						} else if ( typeof elem.removeAttribute !== core_strundefined ) {
							elem.removeAttribute( internalKey );

						} else {
							elem[ internalKey ] = null;
						}

						core_deletedIds.push( id );
					}
				}
			}
		}
	},

	_evalUrl: function( url ) {
		return jQuery.ajax({
			url: url,
			type: "GET",
			dataType: "script",
			async: false,
			global: false,
			"throws": true
		});
	}
});
jQuery.fn.extend({
	wrapAll: function( html ) {
		if ( jQuery.isFunction( html ) ) {
			return this.each(function(i) {
				jQuery(this).wrapAll( html.call(this, i) );
			});
		}

		if ( this[0] ) {
			// The elements to wrap the target around
			var wrap = jQuery( html, this[0].ownerDocument ).eq(0).clone(true);

			if ( this[0].parentNode ) {
				wrap.insertBefore( this[0] );
			}

			wrap.map(function() {
				var elem = this;

				while ( elem.firstChild && elem.firstChild.nodeType === 1 ) {
					elem = elem.firstChild;
				}

				return elem;
			}).append( this );
		}

		return this;
	},

	wrapInner: function( html ) {
		if ( jQuery.isFunction( html ) ) {
			return this.each(function(i) {
				jQuery(this).wrapInner( html.call(this, i) );
			});
		}

		return this.each(function() {
			var self = jQuery( this ),
				contents = self.contents();

			if ( contents.length ) {
				contents.wrapAll( html );

			} else {
				self.append( html );
			}
		});
	},

	wrap: function( html ) {
		var isFunction = jQuery.isFunction( html );

		return this.each(function(i) {
			jQuery( this ).wrapAll( isFunction ? html.call(this, i) : html );
		});
	},

	unwrap: function() {
		return this.parent().each(function() {
			if ( !jQuery.nodeName( this, "body" ) ) {
				jQuery( this ).replaceWith( this.childNodes );
			}
		}).end();
	}
});
var iframe, getStyles, curCSS,
	ralpha = /alpha\([^)]*\)/i,
	ropacity = /opacity\s*=\s*([^)]*)/,
	rposition = /^(top|right|bottom|left)$/,
	// swappable if display is none or starts with table except "table", "table-cell", or "table-caption"
	// see here for display values: https://developer.mozilla.org/en-US/docs/CSS/display
	rdisplayswap = /^(none|table(?!-c[ea]).+)/,
	rmargin = /^margin/,
	rnumsplit = new RegExp( "^(" + core_pnum + ")(.*)$", "i" ),
	rnumnonpx = new RegExp( "^(" + core_pnum + ")(?!px)[a-z%]+$", "i" ),
	rrelNum = new RegExp( "^([+-])=(" + core_pnum + ")", "i" ),
	elemdisplay = { BODY: "block" },

	cssShow = { position: "absolute", visibility: "hidden", display: "block" },
	cssNormalTransform = {
		letterSpacing: 0,
		fontWeight: 400
	},

	cssExpand = [ "Top", "Right", "Bottom", "Left" ],
	cssPrefixes = [ "Webkit", "O", "Moz", "ms" ];

// return a css property mapped to a potentially vendor prefixed property
function vendorPropName( style, name ) {

	// shortcut for names that are not vendor prefixed
	if ( name in style ) {
		return name;
	}

	// check for vendor prefixed names
	var capName = name.charAt(0).toUpperCase() + name.slice(1),
		origName = name,
		i = cssPrefixes.length;

	while ( i-- ) {
		name = cssPrefixes[ i ] + capName;
		if ( name in style ) {
			return name;
		}
	}

	return origName;
}

function isHidden( elem, el ) {
	// isHidden might be called from jQuery#filter function;
	// in that case, element will be second argument
	elem = el || elem;
	return jQuery.css( elem, "display" ) === "none" || !jQuery.contains( elem.ownerDocument, elem );
}

function showHide( elements, show ) {
	var display, elem, hidden,
		values = [],
		index = 0,
		length = elements.length;

	for ( ; index < length; index++ ) {
		elem = elements[ index ];
		if ( !elem.style ) {
			continue;
		}

		values[ index ] = jQuery._data( elem, "olddisplay" );
		display = elem.style.display;
		if ( show ) {
			// Reset the inline display of this element to learn if it is
			// being hidden by cascaded rules or not
			if ( !values[ index ] && display === "none" ) {
				elem.style.display = "";
			}

			// Set elements which have been overridden with display: none
			// in a stylesheet to whatever the default browser style is
			// for such an element
			if ( elem.style.display === "" && isHidden( elem ) ) {
				values[ index ] = jQuery._data( elem, "olddisplay", css_defaultDisplay(elem.nodeName) );
			}
		} else {

			if ( !values[ index ] ) {
				hidden = isHidden( elem );

				if ( display && display !== "none" || !hidden ) {
					jQuery._data( elem, "olddisplay", hidden ? display : jQuery.css( elem, "display" ) );
				}
			}
		}
	}

	// Set the display of most of the elements in a second loop
	// to avoid the constant reflow
	for ( index = 0; index < length; index++ ) {
		elem = elements[ index ];
		if ( !elem.style ) {
			continue;
		}
		if ( !show || elem.style.display === "none" || elem.style.display === "" ) {
			elem.style.display = show ? values[ index ] || "" : "none";
		}
	}

	return elements;
}

jQuery.fn.extend({
	css: function( name, value ) {
		return jQuery.access( this, function( elem, name, value ) {
			var len, styles,
				map = {},
				i = 0;

			if ( jQuery.isArray( name ) ) {
				styles = getStyles( elem );
				len = name.length;

				for ( ; i < len; i++ ) {
					map[ name[ i ] ] = jQuery.css( elem, name[ i ], false, styles );
				}

				return map;
			}

			return value !== undefined ?
				jQuery.style( elem, name, value ) :
				jQuery.css( elem, name );
		}, name, value, arguments.length > 1 );
	},
	show: function() {
		return showHide( this, true );
	},
	hide: function() {
		return showHide( this );
	},
	toggle: function( state ) {
		if ( typeof state === "boolean" ) {
			return state ? this.show() : this.hide();
		}

		return this.each(function() {
			if ( isHidden( this ) ) {
				jQuery( this ).show();
			} else {
				jQuery( this ).hide();
			}
		});
	}
});

jQuery.extend({
	// Add in style property hooks for overriding the default
	// behavior of getting and setting a style property
	cssHooks: {
		opacity: {
			get: function( elem, computed ) {
				if ( computed ) {
					// We should always get a number back from opacity
					var ret = curCSS( elem, "opacity" );
					return ret === "" ? "1" : ret;
				}
			}
		}
	},

	// Don't automatically add "px" to these possibly-unitless properties
	cssNumber: {
		"columnCount": true,
		"fillOpacity": true,
		"fontWeight": true,
		"lineHeight": true,
		"opacity": true,
		"order": true,
		"orphans": true,
		"widows": true,
		"zIndex": true,
		"zoom": true
	},

	// Add in properties whose names you wish to fix before
	// setting or getting the value
	cssProps: {
		// normalize float css property
		"float": jQuery.support.cssFloat ? "cssFloat" : "styleFloat"
	},

	// Get and set the style property on a DOM Node
	style: function( elem, name, value, extra ) {
		// Don't set styles on text and comment nodes
		if ( !elem || elem.nodeType === 3 || elem.nodeType === 8 || !elem.style ) {
			return;
		}

		// Make sure that we're working with the right name
		var ret, type, hooks,
			origName = jQuery.camelCase( name ),
			style = elem.style;

		name = jQuery.cssProps[ origName ] || ( jQuery.cssProps[ origName ] = vendorPropName( style, origName ) );

		// gets hook for the prefixed version
		// followed by the unprefixed version
		hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

		// Check if we're setting a value
		if ( value !== undefined ) {
			type = typeof value;

			// convert relative number strings (+= or -=) to relative numbers. #7345
			if ( type === "string" && (ret = rrelNum.exec( value )) ) {
				value = ( ret[1] + 1 ) * ret[2] + parseFloat( jQuery.css( elem, name ) );
				// Fixes bug #9237
				type = "number";
			}

			// Make sure that NaN and null values aren't set. See: #7116
			if ( value == null || type === "number" && isNaN( value ) ) {
				return;
			}

			// If a number was passed in, add 'px' to the (except for certain CSS properties)
			if ( type === "number" && !jQuery.cssNumber[ origName ] ) {
				value += "px";
			}

			// Fixes #8908, it can be done more correctly by specifing setters in cssHooks,
			// but it would mean to define eight (for every problematic property) identical functions
			if ( !jQuery.support.clearCloneStyle && value === "" && name.indexOf("background") === 0 ) {
				style[ name ] = "inherit";
			}

			// If a hook was provided, use that value, otherwise just set the specified value
			if ( !hooks || !("set" in hooks) || (value = hooks.set( elem, value, extra )) !== undefined ) {

				// Wrapped to prevent IE from throwing errors when 'invalid' values are provided
				// Fixes bug #5509
				try {
					style[ name ] = value;
				} catch(e) {}
			}

		} else {
			// If a hook was provided get the non-computed value from there
			if ( hooks && "get" in hooks && (ret = hooks.get( elem, false, extra )) !== undefined ) {
				return ret;
			}

			// Otherwise just get the value from the style object
			return style[ name ];
		}
	},

	css: function( elem, name, extra, styles ) {
		var num, val, hooks,
			origName = jQuery.camelCase( name );

		// Make sure that we're working with the right name
		name = jQuery.cssProps[ origName ] || ( jQuery.cssProps[ origName ] = vendorPropName( elem.style, origName ) );

		// gets hook for the prefixed version
		// followed by the unprefixed version
		hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

		// If a hook was provided get the computed value from there
		if ( hooks && "get" in hooks ) {
			val = hooks.get( elem, true, extra );
		}

		// Otherwise, if a way to get the computed value exists, use that
		if ( val === undefined ) {
			val = curCSS( elem, name, styles );
		}

		//convert "normal" to computed value
		if ( val === "normal" && name in cssNormalTransform ) {
			val = cssNormalTransform[ name ];
		}

		// Return, converting to number if forced or a qualifier was provided and val looks numeric
		if ( extra === "" || extra ) {
			num = parseFloat( val );
			return extra === true || jQuery.isNumeric( num ) ? num || 0 : val;
		}
		return val;
	}
});

// NOTE: we've included the "window" in window.getComputedStyle
// because jsdom on node.js will break without it.
if ( window.getComputedStyle ) {
	getStyles = function( elem ) {
		return window.getComputedStyle( elem, null );
	};

	curCSS = function( elem, name, _computed ) {
		var width, minWidth, maxWidth,
			computed = _computed || getStyles( elem ),

			// getPropertyValue is only needed for .css('filter') in IE9, see #12537
			ret = computed ? computed.getPropertyValue( name ) || computed[ name ] : undefined,
			style = elem.style;

		if ( computed ) {

			if ( ret === "" && !jQuery.contains( elem.ownerDocument, elem ) ) {
				ret = jQuery.style( elem, name );
			}

			// A tribute to the "awesome hack by Dean Edwards"
			// Chrome < 17 and Safari 5.0 uses "computed value" instead of "used value" for margin-right
			// Safari 5.1.7 (at least) returns percentage for a larger set of values, but width seems to be reliably pixels
			// this is against the CSSOM draft spec: http://dev.w3.org/csswg/cssom/#resolved-values
			if ( rnumnonpx.test( ret ) && rmargin.test( name ) ) {

				// Remember the original values
				width = style.width;
				minWidth = style.minWidth;
				maxWidth = style.maxWidth;

				// Put in the new values to get a computed value out
				style.minWidth = style.maxWidth = style.width = ret;
				ret = computed.width;

				// Revert the changed values
				style.width = width;
				style.minWidth = minWidth;
				style.maxWidth = maxWidth;
			}
		}

		return ret;
	};
} else if ( document.documentElement.currentStyle ) {
	getStyles = function( elem ) {
		return elem.currentStyle;
	};

	curCSS = function( elem, name, _computed ) {
		var left, rs, rsLeft,
			computed = _computed || getStyles( elem ),
			ret = computed ? computed[ name ] : undefined,
			style = elem.style;

		// Avoid setting ret to empty string here
		// so we don't default to auto
		if ( ret == null && style && style[ name ] ) {
			ret = style[ name ];
		}

		// From the awesome hack by Dean Edwards
		// http://erik.eae.net/archives/2007/07/27/18.54.15/#comment-102291

		// If we're not dealing with a regular pixel number
		// but a number that has a weird ending, we need to convert it to pixels
		// but not position css attributes, as those are proportional to the parent element instead
		// and we can't measure the parent instead because it might trigger a "stacking dolls" problem
		if ( rnumnonpx.test( ret ) && !rposition.test( name ) ) {

			// Remember the original values
			left = style.left;
			rs = elem.runtimeStyle;
			rsLeft = rs && rs.left;

			// Put in the new values to get a computed value out
			if ( rsLeft ) {
				rs.left = elem.currentStyle.left;
			}
			style.left = name === "fontSize" ? "1em" : ret;
			ret = style.pixelLeft + "px";

			// Revert the changed values
			style.left = left;
			if ( rsLeft ) {
				rs.left = rsLeft;
			}
		}

		return ret === "" ? "auto" : ret;
	};
}

function setPositiveNumber( elem, value, subtract ) {
	var matches = rnumsplit.exec( value );
	return matches ?
		// Guard against undefined "subtract", e.g., when used as in cssHooks
		Math.max( 0, matches[ 1 ] - ( subtract || 0 ) ) + ( matches[ 2 ] || "px" ) :
		value;
}

function augmentWidthOrHeight( elem, name, extra, isBorderBox, styles ) {
	var i = extra === ( isBorderBox ? "border" : "content" ) ?
		// If we already have the right measurement, avoid augmentation
		4 :
		// Otherwise initialize for horizontal or vertical properties
		name === "width" ? 1 : 0,

		val = 0;

	for ( ; i < 4; i += 2 ) {
		// both box models exclude margin, so add it if we want it
		if ( extra === "margin" ) {
			val += jQuery.css( elem, extra + cssExpand[ i ], true, styles );
		}

		if ( isBorderBox ) {
			// border-box includes padding, so remove it if we want content
			if ( extra === "content" ) {
				val -= jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );
			}

			// at this point, extra isn't border nor margin, so remove border
			if ( extra !== "margin" ) {
				val -= jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
			}
		} else {
			// at this point, extra isn't content, so add padding
			val += jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );

			// at this point, extra isn't content nor padding, so add border
			if ( extra !== "padding" ) {
				val += jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
			}
		}
	}

	return val;
}

function getWidthOrHeight( elem, name, extra ) {

	// Start with offset property, which is equivalent to the border-box value
	var valueIsBorderBox = true,
		val = name === "width" ? elem.offsetWidth : elem.offsetHeight,
		styles = getStyles( elem ),
		isBorderBox = jQuery.support.boxSizing && jQuery.css( elem, "boxSizing", false, styles ) === "border-box";

	// some non-html elements return undefined for offsetWidth, so check for null/undefined
	// svg - https://bugzilla.mozilla.org/show_bug.cgi?id=649285
	// MathML - https://bugzilla.mozilla.org/show_bug.cgi?id=491668
	if ( val <= 0 || val == null ) {
		// Fall back to computed then uncomputed css if necessary
		val = curCSS( elem, name, styles );
		if ( val < 0 || val == null ) {
			val = elem.style[ name ];
		}

		// Computed unit is not pixels. Stop here and return.
		if ( rnumnonpx.test(val) ) {
			return val;
		}

		// we need the check for style in case a browser which returns unreliable values
		// for getComputedStyle silently falls back to the reliable elem.style
		valueIsBorderBox = isBorderBox && ( jQuery.support.boxSizingReliable || val === elem.style[ name ] );

		// Normalize "", auto, and prepare for extra
		val = parseFloat( val ) || 0;
	}

	// use the active box-sizing model to add/subtract irrelevant styles
	return ( val +
		augmentWidthOrHeight(
			elem,
			name,
			extra || ( isBorderBox ? "border" : "content" ),
			valueIsBorderBox,
			styles
		)
	) + "px";
}

// Try to determine the default display value of an element
function css_defaultDisplay( nodeName ) {
	var doc = document,
		display = elemdisplay[ nodeName ];

	if ( !display ) {
		display = actualDisplay( nodeName, doc );

		// If the simple way fails, read from inside an iframe
		if ( display === "none" || !display ) {
			// Use the already-created iframe if possible
			iframe = ( iframe ||
				jQuery("<iframe frameborder='0' width='0' height='0'/>")
				.css( "cssText", "display:block !important" )
			).appendTo( doc.documentElement );

			// Always write a new HTML skeleton so Webkit and Firefox don't choke on reuse
			doc = ( iframe[0].contentWindow || iframe[0].contentDocument ).document;
			doc.write("<!doctype html><html><body>");
			doc.close();

			display = actualDisplay( nodeName, doc );
			iframe.detach();
		}

		// Store the correct default display
		elemdisplay[ nodeName ] = display;
	}

	return display;
}

// Called ONLY from within css_defaultDisplay
function actualDisplay( name, doc ) {
	var elem = jQuery( doc.createElement( name ) ).appendTo( doc.body ),
		display = jQuery.css( elem[0], "display" );
	elem.remove();
	return display;
}

jQuery.each([ "height", "width" ], function( i, name ) {
	jQuery.cssHooks[ name ] = {
		get: function( elem, computed, extra ) {
			if ( computed ) {
				// certain elements can have dimension info if we invisibly show them
				// however, it must have a current display style that would benefit from this
				return elem.offsetWidth === 0 && rdisplayswap.test( jQuery.css( elem, "display" ) ) ?
					jQuery.swap( elem, cssShow, function() {
						return getWidthOrHeight( elem, name, extra );
					}) :
					getWidthOrHeight( elem, name, extra );
			}
		},

		set: function( elem, value, extra ) {
			var styles = extra && getStyles( elem );
			return setPositiveNumber( elem, value, extra ?
				augmentWidthOrHeight(
					elem,
					name,
					extra,
					jQuery.support.boxSizing && jQuery.css( elem, "boxSizing", false, styles ) === "border-box",
					styles
				) : 0
			);
		}
	};
});

if ( !jQuery.support.opacity ) {
	jQuery.cssHooks.opacity = {
		get: function( elem, computed ) {
			// IE uses filters for opacity
			return ropacity.test( (computed && elem.currentStyle ? elem.currentStyle.filter : elem.style.filter) || "" ) ?
				( 0.01 * parseFloat( RegExp.$1 ) ) + "" :
				computed ? "1" : "";
		},

		set: function( elem, value ) {
			var style = elem.style,
				currentStyle = elem.currentStyle,
				opacity = jQuery.isNumeric( value ) ? "alpha(opacity=" + value * 100 + ")" : "",
				filter = currentStyle && currentStyle.filter || style.filter || "";

			// IE has trouble with opacity if it does not have layout
			// Force it by setting the zoom level
			style.zoom = 1;

			// if setting opacity to 1, and no other filters exist - attempt to remove filter attribute #6652
			// if value === "", then remove inline opacity #12685
			if ( ( value >= 1 || value === "" ) &&
					jQuery.trim( filter.replace( ralpha, "" ) ) === "" &&
					style.removeAttribute ) {

				// Setting style.filter to null, "" & " " still leave "filter:" in the cssText
				// if "filter:" is present at all, clearType is disabled, we want to avoid this
				// style.removeAttribute is IE Only, but so apparently is this code path...
				style.removeAttribute( "filter" );

				// if there is no filter style applied in a css rule or unset inline opacity, we are done
				if ( value === "" || currentStyle && !currentStyle.filter ) {
					return;
				}
			}

			// otherwise, set new filter values
			style.filter = ralpha.test( filter ) ?
				filter.replace( ralpha, opacity ) :
				filter + " " + opacity;
		}
	};
}

// These hooks cannot be added until DOM ready because the support test
// for it is not run until after DOM ready
jQuery(function() {
	if ( !jQuery.support.reliableMarginRight ) {
		jQuery.cssHooks.marginRight = {
			get: function( elem, computed ) {
				if ( computed ) {
					// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
					// Work around by temporarily setting element display to inline-block
					return jQuery.swap( elem, { "display": "inline-block" },
						curCSS, [ elem, "marginRight" ] );
				}
			}
		};
	}

	// Webkit bug: https://bugs.webkit.org/show_bug.cgi?id=29084
	// getComputedStyle returns percent when specified for top/left/bottom/right
	// rather than make the css module depend on the offset module, we just check for it here
	if ( !jQuery.support.pixelPosition && jQuery.fn.position ) {
		jQuery.each( [ "top", "left" ], function( i, prop ) {
			jQuery.cssHooks[ prop ] = {
				get: function( elem, computed ) {
					if ( computed ) {
						computed = curCSS( elem, prop );
						// if curCSS returns percentage, fallback to offset
						return rnumnonpx.test( computed ) ?
							jQuery( elem ).position()[ prop ] + "px" :
							computed;
					}
				}
			};
		});
	}

});

if ( jQuery.expr && jQuery.expr.filters ) {
	jQuery.expr.filters.hidden = function( elem ) {
		// Support: Opera <= 12.12
		// Opera reports offsetWidths and offsetHeights less than zero on some elements
		return elem.offsetWidth <= 0 && elem.offsetHeight <= 0 ||
			(!jQuery.support.reliableHiddenOffsets && ((elem.style && elem.style.display) || jQuery.css( elem, "display" )) === "none");
	};

	jQuery.expr.filters.visible = function( elem ) {
		return !jQuery.expr.filters.hidden( elem );
	};
}

// These hooks are used by animate to expand properties
jQuery.each({
	margin: "",
	padding: "",
	border: "Width"
}, function( prefix, suffix ) {
	jQuery.cssHooks[ prefix + suffix ] = {
		expand: function( value ) {
			var i = 0,
				expanded = {},

				// assumes a single number if not a string
				parts = typeof value === "string" ? value.split(" ") : [ value ];

			for ( ; i < 4; i++ ) {
				expanded[ prefix + cssExpand[ i ] + suffix ] =
					parts[ i ] || parts[ i - 2 ] || parts[ 0 ];
			}

			return expanded;
		}
	};

	if ( !rmargin.test( prefix ) ) {
		jQuery.cssHooks[ prefix + suffix ].set = setPositiveNumber;
	}
});
var r20 = /%20/g,
	rbracket = /\[\]$/,
	rCRLF = /\r?\n/g,
	rsubmitterTypes = /^(?:submit|button|image|reset|file)$/i,
	rsubmittable = /^(?:input|select|textarea|keygen)/i;

jQuery.fn.extend({
	serialize: function() {
		return jQuery.param( this.serializeArray() );
	},
	serializeArray: function() {
		return this.map(function(){
			// Can add propHook for "elements" to filter or add form elements
			var elements = jQuery.prop( this, "elements" );
			return elements ? jQuery.makeArray( elements ) : this;
		})
		.filter(function(){
			var type = this.type;
			// Use .is(":disabled") so that fieldset[disabled] works
			return this.name && !jQuery( this ).is( ":disabled" ) &&
				rsubmittable.test( this.nodeName ) && !rsubmitterTypes.test( type ) &&
				( this.checked || !manipulation_rcheckableType.test( type ) );
		})
		.map(function( i, elem ){
			var val = jQuery( this ).val();

			return val == null ?
				null :
				jQuery.isArray( val ) ?
					jQuery.map( val, function( val ){
						return { name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
					}) :
					{ name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
		}).get();
	}
});

//Serialize an array of form elements or a set of
//key/values into a query string
jQuery.param = function( a, traditional ) {
	var prefix,
		s = [],
		add = function( key, value ) {
			// If value is a function, invoke it and return its value
			value = jQuery.isFunction( value ) ? value() : ( value == null ? "" : value );
			s[ s.length ] = encodeURIComponent( key ) + "=" + encodeURIComponent( value );
		};

	// Set traditional to true for jQuery <= 1.3.2 behavior.
	if ( traditional === undefined ) {
		traditional = jQuery.ajaxSettings && jQuery.ajaxSettings.traditional;
	}

	// If an array was passed in, assume that it is an array of form elements.
	if ( jQuery.isArray( a ) || ( a.jquery && !jQuery.isPlainObject( a ) ) ) {
		// Serialize the form elements
		jQuery.each( a, function() {
			add( this.name, this.value );
		});

	} else {
		// If traditional, encode the "old" way (the way 1.3.2 or older
		// did it), otherwise encode params recursively.
		for ( prefix in a ) {
			buildParams( prefix, a[ prefix ], traditional, add );
		}
	}

	// Return the resulting serialization
	return s.join( "&" ).replace( r20, "+" );
};

function buildParams( prefix, obj, traditional, add ) {
	var name;

	if ( jQuery.isArray( obj ) ) {
		// Serialize array item.
		jQuery.each( obj, function( i, v ) {
			if ( traditional || rbracket.test( prefix ) ) {
				// Treat each array item as a scalar.
				add( prefix, v );

			} else {
				// Item is non-scalar (array or object), encode its numeric index.
				buildParams( prefix + "[" + ( typeof v === "object" ? i : "" ) + "]", v, traditional, add );
			}
		});

	} else if ( !traditional && jQuery.type( obj ) === "object" ) {
		// Serialize object item.
		for ( name in obj ) {
			buildParams( prefix + "[" + name + "]", obj[ name ], traditional, add );
		}

	} else {
		// Serialize scalar item.
		add( prefix, obj );
	}
}
jQuery.each( ("blur focus focusin focusout load resize scroll unload click dblclick " +
	"mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave " +
	"change select submit keydown keypress keyup error contextmenu").split(" "), function( i, name ) {

	// Handle event binding
	jQuery.fn[ name ] = function( data, fn ) {
		return arguments.length > 0 ?
			this.on( name, null, data, fn ) :
			this.trigger( name );
	};
});

jQuery.fn.extend({
	hover: function( fnOver, fnOut ) {
		return this.mouseenter( fnOver ).mouseleave( fnOut || fnOver );
	},

	bind: function( types, data, fn ) {
		return this.on( types, null, data, fn );
	},
	unbind: function( types, fn ) {
		return this.off( types, null, fn );
	},

	delegate: function( selector, types, data, fn ) {
		return this.on( types, selector, data, fn );
	},
	undelegate: function( selector, types, fn ) {
		// ( namespace ) or ( selector, types [, fn] )
		return arguments.length === 1 ? this.off( selector, "**" ) : this.off( types, selector || "**", fn );
	}
});
var
	// Document location
	ajaxLocParts,
	ajaxLocation,
	ajax_nonce = jQuery.now(),

	ajax_rquery = /\?/,
	rhash = /#.*$/,
	rts = /([?&])_=[^&]*/,
	rheaders = /^(.*?):[ \t]*([^\r\n]*)\r?$/mg, // IE leaves an \r character at EOL
	// #7653, #8125, #8152: local protocol detection
	rlocalProtocol = /^(?:about|app|app-storage|.+-extension|file|res|widget):$/,
	rnoContent = /^(?:GET|HEAD)$/,
	rprotocol = /^\/\//,
	rurl = /^([\w.+-]+:)(?:\/\/([^\/?#:]*)(?::(\d+)|)|)/,

	// Keep a copy of the old load method
	_load = jQuery.fn.load,

	/* Prefilters
	 * 1) They are useful to introduce custom dataTypes (see ajax/jsonp.js for an example)
	 * 2) These are called:
	 *    - BEFORE asking for a transport
	 *    - AFTER param serialization (s.data is a string if s.processData is true)
	 * 3) key is the dataType
	 * 4) the catchall symbol "*" can be used
	 * 5) execution will start with transport dataType and THEN continue down to "*" if needed
	 */
	prefilters = {},

	/* Transports bindings
	 * 1) key is the dataType
	 * 2) the catchall symbol "*" can be used
	 * 3) selection will start with transport dataType and THEN go to "*" if needed
	 */
	transports = {},

	// Avoid comment-prolog char sequence (#10098); must appease lint and evade compression
	allTypes = "*/".concat("*");

// #8138, IE may throw an exception when accessing
// a field from window.location if document.domain has been set
try {
	ajaxLocation = location.href;
} catch( e ) {
	// Use the href attribute of an A element
	// since IE will modify it given document.location
	ajaxLocation = document.createElement( "a" );
	ajaxLocation.href = "";
	ajaxLocation = ajaxLocation.href;
}

// Segment location into parts
ajaxLocParts = rurl.exec( ajaxLocation.toLowerCase() ) || [];

// Base "constructor" for jQuery.ajaxPrefilter and jQuery.ajaxTransport
function addToPrefiltersOrTransports( structure ) {

	// dataTypeExpression is optional and defaults to "*"
	return function( dataTypeExpression, func ) {

		if ( typeof dataTypeExpression !== "string" ) {
			func = dataTypeExpression;
			dataTypeExpression = "*";
		}

		var dataType,
			i = 0,
			dataTypes = dataTypeExpression.toLowerCase().match( core_rnotwhite ) || [];

		if ( jQuery.isFunction( func ) ) {
			// For each dataType in the dataTypeExpression
			while ( (dataType = dataTypes[i++]) ) {
				// Prepend if requested
				if ( dataType[0] === "+" ) {
					dataType = dataType.slice( 1 ) || "*";
					(structure[ dataType ] = structure[ dataType ] || []).unshift( func );

				// Otherwise append
				} else {
					(structure[ dataType ] = structure[ dataType ] || []).push( func );
				}
			}
		}
	};
}

// Base inspection function for prefilters and transports
function inspectPrefiltersOrTransports( structure, options, originalOptions, jqXHR ) {

	var inspected = {},
		seekingTransport = ( structure === transports );

	function inspect( dataType ) {
		var selected;
		inspected[ dataType ] = true;
		jQuery.each( structure[ dataType ] || [], function( _, prefilterOrFactory ) {
			var dataTypeOrTransport = prefilterOrFactory( options, originalOptions, jqXHR );
			if( typeof dataTypeOrTransport === "string" && !seekingTransport && !inspected[ dataTypeOrTransport ] ) {
				options.dataTypes.unshift( dataTypeOrTransport );
				inspect( dataTypeOrTransport );
				return false;
			} else if ( seekingTransport ) {
				return !( selected = dataTypeOrTransport );
			}
		});
		return selected;
	}

	return inspect( options.dataTypes[ 0 ] ) || !inspected[ "*" ] && inspect( "*" );
}

// A special extend for ajax options
// that takes "flat" options (not to be deep extended)
// Fixes #9887
function ajaxExtend( target, src ) {
	var deep, key,
		flatOptions = jQuery.ajaxSettings.flatOptions || {};

	for ( key in src ) {
		if ( src[ key ] !== undefined ) {
			( flatOptions[ key ] ? target : ( deep || (deep = {}) ) )[ key ] = src[ key ];
		}
	}
	if ( deep ) {
		jQuery.extend( true, target, deep );
	}

	return target;
}

jQuery.fn.load = function( url, params, callback ) {
	if ( typeof url !== "string" && _load ) {
		return _load.apply( this, arguments );
	}

	var selector, response, type,
		self = this,
		off = url.indexOf(" ");

	if ( off >= 0 ) {
		selector = url.slice( off, url.length );
		url = url.slice( 0, off );
	}

	// If it's a function
	if ( jQuery.isFunction( params ) ) {

		// We assume that it's the callback
		callback = params;
		params = undefined;

	// Otherwise, build a param string
	} else if ( params && typeof params === "object" ) {
		type = "POST";
	}

	// If we have elements to modify, make the request
	if ( self.length > 0 ) {
		jQuery.ajax({
			url: url,

			// if "type" variable is undefined, then "GET" method will be used
			type: type,
			dataType: "html",
			data: params
		}).done(function( responseText ) {

			// Save response for use in complete callback
			response = arguments;

			self.html( selector ?

				// If a selector was specified, locate the right elements in a dummy div
				// Exclude scripts to avoid IE 'Permission Denied' errors
				jQuery("<div>").append( jQuery.parseHTML( responseText ) ).find( selector ) :

				// Otherwise use the full result
				responseText );

		}).complete( callback && function( jqXHR, status ) {
			self.each( callback, response || [ jqXHR.responseText, status, jqXHR ] );
		});
	}

	return this;
};

// Attach a bunch of functions for handling common AJAX events
jQuery.each( [ "ajaxStart", "ajaxStop", "ajaxComplete", "ajaxError", "ajaxSuccess", "ajaxSend" ], function( i, type ){
	jQuery.fn[ type ] = function( fn ){
		return this.on( type, fn );
	};
});

jQuery.extend({

	// Counter for holding the number of active queries
	active: 0,

	// Last-Modified header cache for next request
	lastModified: {},
	etag: {},

	ajaxSettings: {
		url: ajaxLocation,
		type: "GET",
		isLocal: rlocalProtocol.test( ajaxLocParts[ 1 ] ),
		global: true,
		processData: true,
		async: true,
		contentType: "application/x-www-form-urlencoded; charset=UTF-8",
		/*
		timeout: 0,
		data: null,
		dataType: null,
		username: null,
		password: null,
		cache: null,
		throws: false,
		traditional: false,
		headers: {},
		*/

		accepts: {
			"*": allTypes,
			text: "text/plain",
			html: "text/html",
			xml: "application/xml, text/xml",
			json: "application/json, text/javascript"
		},

		contents: {
			xml: /xml/,
			html: /html/,
			json: /json/
		},

		responseFields: {
			xml: "responseXML",
			text: "responseText",
			json: "responseJSON"
		},

		// Data converters
		// Keys separate source (or catchall "*") and destination types with a single space
		converters: {

			// Convert anything to text
			"* text": String,

			// Text to html (true = no transformation)
			"text html": true,

			// Evaluate text as a json expression
			"text json": jQuery.parseJSON,

			// Parse text as xml
			"text xml": jQuery.parseXML
		},

		// For options that shouldn't be deep extended:
		// you can add your own custom options here if
		// and when you create one that shouldn't be
		// deep extended (see ajaxExtend)
		flatOptions: {
			url: true,
			context: true
		}
	},

	// Creates a full fledged settings object into target
	// with both ajaxSettings and settings fields.
	// If target is omitted, writes into ajaxSettings.
	ajaxSetup: function( target, settings ) {
		return settings ?

			// Building a settings object
			ajaxExtend( ajaxExtend( target, jQuery.ajaxSettings ), settings ) :

			// Extending ajaxSettings
			ajaxExtend( jQuery.ajaxSettings, target );
	},

	ajaxPrefilter: addToPrefiltersOrTransports( prefilters ),
	ajaxTransport: addToPrefiltersOrTransports( transports ),

	// Main method
	ajax: function( url, options ) {

		// If url is an object, simulate pre-1.5 signature
		if ( typeof url === "object" ) {
			options = url;
			url = undefined;
		}

		// Force options to be an object
		options = options || {};

		var // Cross-domain detection vars
			parts,
			// Loop variable
			i,
			// URL without anti-cache param
			cacheURL,
			// Response headers as string
			responseHeadersString,
			// timeout handle
			timeoutTimer,

			// To know if global events are to be dispatched
			fireGlobals,

			transport,
			// Response headers
			responseHeaders,
			// Create the final options object
			s = jQuery.ajaxSetup( {}, options ),
			// Callbacks context
			callbackContext = s.context || s,
			// Context for global events is callbackContext if it is a DOM node or jQuery collection
			globalEventContext = s.context && ( callbackContext.nodeType || callbackContext.jquery ) ?
				jQuery( callbackContext ) :
				jQuery.event,
			// Deferreds
			deferred = jQuery.Deferred(),
			completeDeferred = jQuery.Callbacks("once memory"),
			// Status-dependent callbacks
			statusCode = s.statusCode || {},
			// Headers (they are sent all at once)
			requestHeaders = {},
			requestHeadersNames = {},
			// The jqXHR state
			state = 0,
			// Default abort message
			strAbort = "canceled",
			// Fake xhr
			jqXHR = {
				readyState: 0,

				// Builds headers hashtable if needed
				getResponseHeader: function( key ) {
					var match;
					if ( state === 2 ) {
						if ( !responseHeaders ) {
							responseHeaders = {};
							while ( (match = rheaders.exec( responseHeadersString )) ) {
								responseHeaders[ match[1].toLowerCase() ] = match[ 2 ];
							}
						}
						match = responseHeaders[ key.toLowerCase() ];
					}
					return match == null ? null : match;
				},

				// Raw string
				getAllResponseHeaders: function() {
					return state === 2 ? responseHeadersString : null;
				},

				// Caches the header
				setRequestHeader: function( name, value ) {
					var lname = name.toLowerCase();
					if ( !state ) {
						name = requestHeadersNames[ lname ] = requestHeadersNames[ lname ] || name;
						requestHeaders[ name ] = value;
					}
					return this;
				},

				// Overrides response content-type header
				overrideMimeType: function( type ) {
					if ( !state ) {
						s.mimeType = type;
					}
					return this;
				},

				// Status-dependent callbacks
				statusCode: function( map ) {
					var code;
					if ( map ) {
						if ( state < 2 ) {
							for ( code in map ) {
								// Lazy-add the new callback in a way that preserves old ones
								statusCode[ code ] = [ statusCode[ code ], map[ code ] ];
							}
						} else {
							// Execute the appropriate callbacks
							jqXHR.always( map[ jqXHR.status ] );
						}
					}
					return this;
				},

				// Cancel the request
				abort: function( statusText ) {
					var finalText = statusText || strAbort;
					if ( transport ) {
						transport.abort( finalText );
					}
					done( 0, finalText );
					return this;
				}
			};

		// Attach deferreds
		deferred.promise( jqXHR ).complete = completeDeferred.add;
		jqXHR.success = jqXHR.done;
		jqXHR.error = jqXHR.fail;

		// Remove hash character (#7531: and string promotion)
		// Add protocol if not provided (#5866: IE7 issue with protocol-less urls)
		// Handle falsy url in the settings object (#10093: consistency with old signature)
		// We also use the url parameter if available
		s.url = ( ( url || s.url || ajaxLocation ) + "" ).replace( rhash, "" ).replace( rprotocol, ajaxLocParts[ 1 ] + "//" );

		// Alias method option to type as per ticket #12004
		s.type = options.method || options.type || s.method || s.type;

		// Extract dataTypes list
		s.dataTypes = jQuery.trim( s.dataType || "*" ).toLowerCase().match( core_rnotwhite ) || [""];

		// A cross-domain request is in order when we have a protocol:host:port mismatch
		if ( s.crossDomain == null ) {
			parts = rurl.exec( s.url.toLowerCase() );
			s.crossDomain = !!( parts &&
				( parts[ 1 ] !== ajaxLocParts[ 1 ] || parts[ 2 ] !== ajaxLocParts[ 2 ] ||
					( parts[ 3 ] || ( parts[ 1 ] === "http:" ? "80" : "443" ) ) !==
						( ajaxLocParts[ 3 ] || ( ajaxLocParts[ 1 ] === "http:" ? "80" : "443" ) ) )
			);
		}

		// Convert data if not already a string
		if ( s.data && s.processData && typeof s.data !== "string" ) {
			s.data = jQuery.param( s.data, s.traditional );
		}

		// Apply prefilters
		inspectPrefiltersOrTransports( prefilters, s, options, jqXHR );

		// If request was aborted inside a prefilter, stop there
		if ( state === 2 ) {
			return jqXHR;
		}

		// We can fire global events as of now if asked to
		fireGlobals = s.global;

		// Watch for a new set of requests
		if ( fireGlobals && jQuery.active++ === 0 ) {
			jQuery.event.trigger("ajaxStart");
		}

		// Uppercase the type
		s.type = s.type.toUpperCase();

		// Determine if request has content
		s.hasContent = !rnoContent.test( s.type );

		// Save the URL in case we're toying with the If-Modified-Since
		// and/or If-None-Match header later on
		cacheURL = s.url;

		// More options handling for requests with no content
		if ( !s.hasContent ) {

			// If data is available, append data to url
			if ( s.data ) {
				cacheURL = ( s.url += ( ajax_rquery.test( cacheURL ) ? "&" : "?" ) + s.data );
				// #9682: remove data so that it's not used in an eventual retry
				delete s.data;
			}

			// Add anti-cache in url if needed
			if ( s.cache === false ) {
				s.url = rts.test( cacheURL ) ?

					// If there is already a '_' parameter, set its value
					cacheURL.replace( rts, "$1_=" + ajax_nonce++ ) :

					// Otherwise add one to the end
					cacheURL + ( ajax_rquery.test( cacheURL ) ? "&" : "?" ) + "_=" + ajax_nonce++;
			}
		}

		// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
		if ( s.ifModified ) {
			if ( jQuery.lastModified[ cacheURL ] ) {
				jqXHR.setRequestHeader( "If-Modified-Since", jQuery.lastModified[ cacheURL ] );
			}
			if ( jQuery.etag[ cacheURL ] ) {
				jqXHR.setRequestHeader( "If-None-Match", jQuery.etag[ cacheURL ] );
			}
		}

		// Set the correct header, if data is being sent
		if ( s.data && s.hasContent && s.contentType !== false || options.contentType ) {
			jqXHR.setRequestHeader( "Content-Type", s.contentType );
		}

		// Set the Accepts header for the server, depending on the dataType
		jqXHR.setRequestHeader(
			"Accept",
			s.dataTypes[ 0 ] && s.accepts[ s.dataTypes[0] ] ?
				s.accepts[ s.dataTypes[0] ] + ( s.dataTypes[ 0 ] !== "*" ? ", " + allTypes + "; q=0.01" : "" ) :
				s.accepts[ "*" ]
		);

		// Check for headers option
		for ( i in s.headers ) {
			jqXHR.setRequestHeader( i, s.headers[ i ] );
		}

		// Allow custom headers/mimetypes and early abort
		if ( s.beforeSend && ( s.beforeSend.call( callbackContext, jqXHR, s ) === false || state === 2 ) ) {
			// Abort if not done already and return
			return jqXHR.abort();
		}

		// aborting is no longer a cancellation
		strAbort = "abort";

		// Install callbacks on deferreds
		for ( i in { success: 1, error: 1, complete: 1 } ) {
			jqXHR[ i ]( s[ i ] );
		}

		// Get transport
		transport = inspectPrefiltersOrTransports( transports, s, options, jqXHR );

		// If no transport, we auto-abort
		if ( !transport ) {
			done( -1, "No Transport" );
		} else {
			jqXHR.readyState = 1;

			// Send global event
			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxSend", [ jqXHR, s ] );
			}
			// Timeout
			if ( s.async && s.timeout > 0 ) {
				timeoutTimer = setTimeout(function() {
					jqXHR.abort("timeout");
				}, s.timeout );
			}

			try {
				state = 1;
				transport.send( requestHeaders, done );
			} catch ( e ) {
				// Propagate exception as error if not done
				if ( state < 2 ) {
					done( -1, e );
				// Simply rethrow otherwise
				} else {
					throw e;
				}
			}
		}

		// Callback for when everything is done
		function done( status, nativeStatusText, responses, headers ) {
			var isSuccess, success, error, response, modified,
				statusText = nativeStatusText;

			// Called once
			if ( state === 2 ) {
				return;
			}

			// State is "done" now
			state = 2;

			// Clear timeout if it exists
			if ( timeoutTimer ) {
				clearTimeout( timeoutTimer );
			}

			// Dereference transport for early garbage collection
			// (no matter how long the jqXHR object will be used)
			transport = undefined;

			// Cache response headers
			responseHeadersString = headers || "";

			// Set readyState
			jqXHR.readyState = status > 0 ? 4 : 0;

			// Determine if successful
			isSuccess = status >= 200 && status < 300 || status === 304;

			// Get response data
			if ( responses ) {
				response = ajaxHandleResponses( s, jqXHR, responses );
			}

			// Convert no matter what (that way responseXXX fields are always set)
			response = ajaxConvert( s, response, jqXHR, isSuccess );

			// If successful, handle type chaining
			if ( isSuccess ) {

				// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
				if ( s.ifModified ) {
					modified = jqXHR.getResponseHeader("Last-Modified");
					if ( modified ) {
						jQuery.lastModified[ cacheURL ] = modified;
					}
					modified = jqXHR.getResponseHeader("etag");
					if ( modified ) {
						jQuery.etag[ cacheURL ] = modified;
					}
				}

				// if no content
				if ( status === 204 || s.type === "HEAD" ) {
					statusText = "nocontent";

				// if not modified
				} else if ( status === 304 ) {
					statusText = "notmodified";

				// If we have data, let's convert it
				} else {
					statusText = response.state;
					success = response.data;
					error = response.error;
					isSuccess = !error;
				}
			} else {
				// We extract error from statusText
				// then normalize statusText and status for non-aborts
				error = statusText;
				if ( status || !statusText ) {
					statusText = "error";
					if ( status < 0 ) {
						status = 0;
					}
				}
			}

			// Set data for the fake xhr object
			jqXHR.status = status;
			jqXHR.statusText = ( nativeStatusText || statusText ) + "";

			// Success/Error
			if ( isSuccess ) {
				deferred.resolveWith( callbackContext, [ success, statusText, jqXHR ] );
			} else {
				deferred.rejectWith( callbackContext, [ jqXHR, statusText, error ] );
			}

			// Status-dependent callbacks
			jqXHR.statusCode( statusCode );
			statusCode = undefined;

			if ( fireGlobals ) {
				globalEventContext.trigger( isSuccess ? "ajaxSuccess" : "ajaxError",
					[ jqXHR, s, isSuccess ? success : error ] );
			}

			// Complete
			completeDeferred.fireWith( callbackContext, [ jqXHR, statusText ] );

			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxComplete", [ jqXHR, s ] );
				// Handle the global AJAX counter
				if ( !( --jQuery.active ) ) {
					jQuery.event.trigger("ajaxStop");
				}
			}
		}

		return jqXHR;
	},

	getJSON: function( url, data, callback ) {
		return jQuery.get( url, data, callback, "json" );
	},

	getScript: function( url, callback ) {
		return jQuery.get( url, undefined, callback, "script" );
	}
});

jQuery.each( [ "get", "post" ], function( i, method ) {
	jQuery[ method ] = function( url, data, callback, type ) {
		// shift arguments if data argument was omitted
		if ( jQuery.isFunction( data ) ) {
			type = type || callback;
			callback = data;
			data = undefined;
		}

		return jQuery.ajax({
			url: url,
			type: method,
			dataType: type,
			data: data,
			success: callback
		});
	};
});

/* Handles responses to an ajax request:
 * - finds the right dataType (mediates between content-type and expected dataType)
 * - returns the corresponding response
 */
function ajaxHandleResponses( s, jqXHR, responses ) {
	var firstDataType, ct, finalDataType, type,
		contents = s.contents,
		dataTypes = s.dataTypes;

	// Remove auto dataType and get content-type in the process
	while( dataTypes[ 0 ] === "*" ) {
		dataTypes.shift();
		if ( ct === undefined ) {
			ct = s.mimeType || jqXHR.getResponseHeader("Content-Type");
		}
	}

	// Check if we're dealing with a known content-type
	if ( ct ) {
		for ( type in contents ) {
			if ( contents[ type ] && contents[ type ].test( ct ) ) {
				dataTypes.unshift( type );
				break;
			}
		}
	}

	// Check to see if we have a response for the expected dataType
	if ( dataTypes[ 0 ] in responses ) {
		finalDataType = dataTypes[ 0 ];
	} else {
		// Try convertible dataTypes
		for ( type in responses ) {
			if ( !dataTypes[ 0 ] || s.converters[ type + " " + dataTypes[0] ] ) {
				finalDataType = type;
				break;
			}
			if ( !firstDataType ) {
				firstDataType = type;
			}
		}
		// Or just use first one
		finalDataType = finalDataType || firstDataType;
	}

	// If we found a dataType
	// We add the dataType to the list if needed
	// and return the corresponding response
	if ( finalDataType ) {
		if ( finalDataType !== dataTypes[ 0 ] ) {
			dataTypes.unshift( finalDataType );
		}
		return responses[ finalDataType ];
	}
}

/* Chain conversions given the request and the original response
 * Also sets the responseXXX fields on the jqXHR instance
 */
function ajaxConvert( s, response, jqXHR, isSuccess ) {
	var conv2, current, conv, tmp, prev,
		converters = {},
		// Work with a copy of dataTypes in case we need to modify it for conversion
		dataTypes = s.dataTypes.slice();

	// Create converters map with lowercased keys
	if ( dataTypes[ 1 ] ) {
		for ( conv in s.converters ) {
			converters[ conv.toLowerCase() ] = s.converters[ conv ];
		}
	}

	current = dataTypes.shift();

	// Convert to each sequential dataType
	while ( current ) {

		if ( s.responseFields[ current ] ) {
			jqXHR[ s.responseFields[ current ] ] = response;
		}

		// Apply the dataFilter if provided
		if ( !prev && isSuccess && s.dataFilter ) {
			response = s.dataFilter( response, s.dataType );
		}

		prev = current;
		current = dataTypes.shift();

		if ( current ) {

			// There's only work to do if current dataType is non-auto
			if ( current === "*" ) {

				current = prev;

			// Convert response if prev dataType is non-auto and differs from current
			} else if ( prev !== "*" && prev !== current ) {

				// Seek a direct converter
				conv = converters[ prev + " " + current ] || converters[ "* " + current ];

				// If none found, seek a pair
				if ( !conv ) {
					for ( conv2 in converters ) {

						// If conv2 outputs current
						tmp = conv2.split( " " );
						if ( tmp[ 1 ] === current ) {

							// If prev can be converted to accepted input
							conv = converters[ prev + " " + tmp[ 0 ] ] ||
								converters[ "* " + tmp[ 0 ] ];
							if ( conv ) {
								// Condense equivalence converters
								if ( conv === true ) {
									conv = converters[ conv2 ];

								// Otherwise, insert the intermediate dataType
								} else if ( converters[ conv2 ] !== true ) {
									current = tmp[ 0 ];
									dataTypes.unshift( tmp[ 1 ] );
								}
								break;
							}
						}
					}
				}

				// Apply converter (if not an equivalence)
				if ( conv !== true ) {

					// Unless errors are allowed to bubble, catch and return them
					if ( conv && s[ "throws" ] ) {
						response = conv( response );
					} else {
						try {
							response = conv( response );
						} catch ( e ) {
							return { state: "parsererror", error: conv ? e : "No conversion from " + prev + " to " + current };
						}
					}
				}
			}
		}
	}

	return { state: "success", data: response };
}
// Install script dataType
jQuery.ajaxSetup({
	accepts: {
		script: "text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"
	},
	contents: {
		script: /(?:java|ecma)script/
	},
	converters: {
		"text script": function( text ) {
			jQuery.globalEval( text );
			return text;
		}
	}
});

// Handle cache's special case and global
jQuery.ajaxPrefilter( "script", function( s ) {
	if ( s.cache === undefined ) {
		s.cache = false;
	}
	if ( s.crossDomain ) {
		s.type = "GET";
		s.global = false;
	}
});

// Bind script tag hack transport
jQuery.ajaxTransport( "script", function(s) {

	// This transport only deals with cross domain requests
	if ( s.crossDomain ) {

		var script,
			head = document.head || jQuery("head")[0] || document.documentElement;

		return {

			send: function( _, callback ) {

				script = document.createElement("script");

				script.async = true;

				if ( s.scriptCharset ) {
					script.charset = s.scriptCharset;
				}

				script.src = s.url;

				// Attach handlers for all browsers
				script.onload = script.onreadystatechange = function( _, isAbort ) {

					if ( isAbort || !script.readyState || /loaded|complete/.test( script.readyState ) ) {

						// Handle memory leak in IE
						script.onload = script.onreadystatechange = null;

						// Remove the script
						if ( script.parentNode ) {
							script.parentNode.removeChild( script );
						}

						// Dereference the script
						script = null;

						// Callback if not abort
						if ( !isAbort ) {
							callback( 200, "success" );
						}
					}
				};

				// Circumvent IE6 bugs with base elements (#2709 and #4378) by prepending
				// Use native DOM manipulation to avoid our domManip AJAX trickery
				head.insertBefore( script, head.firstChild );
			},

			abort: function() {
				if ( script ) {
					script.onload( undefined, true );
				}
			}
		};
	}
});
var oldCallbacks = [],
	rjsonp = /(=)\?(?=&|$)|\?\?/;

// Default jsonp settings
jQuery.ajaxSetup({
	jsonp: "callback",
	jsonpCallback: function() {
		var callback = oldCallbacks.pop() || ( jQuery.expando + "_" + ( ajax_nonce++ ) );
		this[ callback ] = true;
		return callback;
	}
});

// Detect, normalize options and install callbacks for jsonp requests
jQuery.ajaxPrefilter( "json jsonp", function( s, originalSettings, jqXHR ) {

	var callbackName, overwritten, responseContainer,
		jsonProp = s.jsonp !== false && ( rjsonp.test( s.url ) ?
			"url" :
			typeof s.data === "string" && !( s.contentType || "" ).indexOf("application/x-www-form-urlencoded") && rjsonp.test( s.data ) && "data"
		);

	// Handle iff the expected data type is "jsonp" or we have a parameter to set
	if ( jsonProp || s.dataTypes[ 0 ] === "jsonp" ) {

		// Get callback name, remembering preexisting value associated with it
		callbackName = s.jsonpCallback = jQuery.isFunction( s.jsonpCallback ) ?
			s.jsonpCallback() :
			s.jsonpCallback;

		// Insert callback into url or form data
		if ( jsonProp ) {
			s[ jsonProp ] = s[ jsonProp ].replace( rjsonp, "$1" + callbackName );
		} else if ( s.jsonp !== false ) {
			s.url += ( ajax_rquery.test( s.url ) ? "&" : "?" ) + s.jsonp + "=" + callbackName;
		}

		// Use data converter to retrieve json after script execution
		s.converters["script json"] = function() {
			if ( !responseContainer ) {
				jQuery.error( callbackName + " was not called" );
			}
			return responseContainer[ 0 ];
		};

		// force json dataType
		s.dataTypes[ 0 ] = "json";

		// Install callback
		overwritten = window[ callbackName ];
		window[ callbackName ] = function() {
			responseContainer = arguments;
		};

		// Clean-up function (fires after converters)
		jqXHR.always(function() {
			// Restore preexisting value
			window[ callbackName ] = overwritten;

			// Save back as free
			if ( s[ callbackName ] ) {
				// make sure that re-using the options doesn't screw things around
				s.jsonpCallback = originalSettings.jsonpCallback;

				// save the callback name for future use
				oldCallbacks.push( callbackName );
			}

			// Call if it was a function and we have a response
			if ( responseContainer && jQuery.isFunction( overwritten ) ) {
				overwritten( responseContainer[ 0 ] );
			}

			responseContainer = overwritten = undefined;
		});

		// Delegate to script
		return "script";
	}
});
var xhrCallbacks, xhrSupported,
	xhrId = 0,
	// #5280: Internet Explorer will keep connections alive if we don't abort on unload
	xhrOnUnloadAbort = window.ActiveXObject && function() {
		// Abort all pending requests
		var key;
		for ( key in xhrCallbacks ) {
			xhrCallbacks[ key ]( undefined, true );
		}
	};

// Functions to create xhrs
function createStandardXHR() {
	try {
		return new window.XMLHttpRequest();
	} catch( e ) {}
}

function createActiveXHR() {
	try {
		return new window.ActiveXObject("Microsoft.XMLHTTP");
	} catch( e ) {}
}

// Create the request object
// (This is still attached to ajaxSettings for backward compatibility)
jQuery.ajaxSettings.xhr = window.ActiveXObject ?
	/* Microsoft failed to properly
	 * implement the XMLHttpRequest in IE7 (can't request local files),
	 * so we use the ActiveXObject when it is available
	 * Additionally XMLHttpRequest can be disabled in IE7/IE8 so
	 * we need a fallback.
	 */
	function() {
		return !this.isLocal && createStandardXHR() || createActiveXHR();
	} :
	// For all other browsers, use the standard XMLHttpRequest object
	createStandardXHR;

// Determine support properties
xhrSupported = jQuery.ajaxSettings.xhr();
jQuery.support.cors = !!xhrSupported && ( "withCredentials" in xhrSupported );
xhrSupported = jQuery.support.ajax = !!xhrSupported;

// Create transport if the browser can provide an xhr
if ( xhrSupported ) {

	jQuery.ajaxTransport(function( s ) {
		// Cross domain only allowed if supported through XMLHttpRequest
		if ( !s.crossDomain || jQuery.support.cors ) {

			var callback;

			return {
				send: function( headers, complete ) {

					// Get a new xhr
					var handle, i,
						xhr = s.xhr();

					// Open the socket
					// Passing null username, generates a login popup on Opera (#2865)
					if ( s.username ) {
						xhr.open( s.type, s.url, s.async, s.username, s.password );
					} else {
						xhr.open( s.type, s.url, s.async );
					}

					// Apply custom fields if provided
					if ( s.xhrFields ) {
						for ( i in s.xhrFields ) {
							xhr[ i ] = s.xhrFields[ i ];
						}
					}

					// Override mime type if needed
					if ( s.mimeType && xhr.overrideMimeType ) {
						xhr.overrideMimeType( s.mimeType );
					}

					// X-Requested-With header
					// For cross-domain requests, seeing as conditions for a preflight are
					// akin to a jigsaw puzzle, we simply never set it to be sure.
					// (it can always be set on a per-request basis or even using ajaxSetup)
					// For same-domain requests, won't change header if already provided.
					if ( !s.crossDomain && !headers["X-Requested-With"] ) {
						headers["X-Requested-With"] = "XMLHttpRequest";
					}

					// Need an extra try/catch for cross domain requests in Firefox 3
					try {
						for ( i in headers ) {
							xhr.setRequestHeader( i, headers[ i ] );
						}
					} catch( err ) {}

					// Do send the request
					// This may raise an exception which is actually
					// handled in jQuery.ajax (so no try/catch here)
					xhr.send( ( s.hasContent && s.data ) || null );

					// Listener
					callback = function( _, isAbort ) {
						var status, responseHeaders, statusText, responses;

						// Firefox throws exceptions when accessing properties
						// of an xhr when a network error occurred
						// http://helpful.knobs-dials.com/index.php/Component_returned_failure_code:_0x80040111_(NS_ERROR_NOT_AVAILABLE)
						try {

							// Was never called and is aborted or complete
							if ( callback && ( isAbort || xhr.readyState === 4 ) ) {

								// Only called once
								callback = undefined;

								// Do not keep as active anymore
								if ( handle ) {
									xhr.onreadystatechange = jQuery.noop;
									if ( xhrOnUnloadAbort ) {
										delete xhrCallbacks[ handle ];
									}
								}

								// If it's an abort
								if ( isAbort ) {
									// Abort it manually if needed
									if ( xhr.readyState !== 4 ) {
										xhr.abort();
									}
								} else {
									responses = {};
									status = xhr.status;
									responseHeaders = xhr.getAllResponseHeaders();

									// When requesting binary data, IE6-9 will throw an exception
									// on any attempt to access responseText (#11426)
									if ( typeof xhr.responseText === "string" ) {
										responses.text = xhr.responseText;
									}

									// Firefox throws an exception when accessing
									// statusText for faulty cross-domain requests
									try {
										statusText = xhr.statusText;
									} catch( e ) {
										// We normalize with Webkit giving an empty statusText
										statusText = "";
									}

									// Filter status for non standard behaviors

									// If the request is local and we have data: assume a success
									// (success with no data won't get notified, that's the best we
									// can do given current implementations)
									if ( !status && s.isLocal && !s.crossDomain ) {
										status = responses.text ? 200 : 404;
									// IE - #1450: sometimes returns 1223 when it should be 204
									} else if ( status === 1223 ) {
										status = 204;
									}
								}
							}
						} catch( firefoxAccessException ) {
							if ( !isAbort ) {
								complete( -1, firefoxAccessException );
							}
						}

						// Call complete if needed
						if ( responses ) {
							complete( status, statusText, responses, responseHeaders );
						}
					};

					if ( !s.async ) {
						// if we're in sync mode we fire the callback
						callback();
					} else if ( xhr.readyState === 4 ) {
						// (IE6 & IE7) if it's in cache and has been
						// retrieved directly we need to fire the callback
						setTimeout( callback );
					} else {
						handle = ++xhrId;
						if ( xhrOnUnloadAbort ) {
							// Create the active xhrs callbacks list if needed
							// and attach the unload handler
							if ( !xhrCallbacks ) {
								xhrCallbacks = {};
								jQuery( window ).unload( xhrOnUnloadAbort );
							}
							// Add to list of active xhrs callbacks
							xhrCallbacks[ handle ] = callback;
						}
						xhr.onreadystatechange = callback;
					}
				},

				abort: function() {
					if ( callback ) {
						callback( undefined, true );
					}
				}
			};
		}
	});
}
var fxNow, timerId,
	rfxtypes = /^(?:toggle|show|hide)$/,
	rfxnum = new RegExp( "^(?:([+-])=|)(" + core_pnum + ")([a-z%]*)$", "i" ),
	rrun = /queueHooks$/,
	animationPrefilters = [ defaultPrefilter ],
	tweeners = {
		"*": [function( prop, value ) {
			var tween = this.createTween( prop, value ),
				target = tween.cur(),
				parts = rfxnum.exec( value ),
				unit = parts && parts[ 3 ] || ( jQuery.cssNumber[ prop ] ? "" : "px" ),

				// Starting value computation is required for potential unit mismatches
				start = ( jQuery.cssNumber[ prop ] || unit !== "px" && +target ) &&
					rfxnum.exec( jQuery.css( tween.elem, prop ) ),
				scale = 1,
				maxIterations = 20;

			if ( start && start[ 3 ] !== unit ) {
				// Trust units reported by jQuery.css
				unit = unit || start[ 3 ];

				// Make sure we update the tween properties later on
				parts = parts || [];

				// Iteratively approximate from a nonzero starting point
				start = +target || 1;

				do {
					// If previous iteration zeroed out, double until we get *something*
					// Use a string for doubling factor so we don't accidentally see scale as unchanged below
					scale = scale || ".5";

					// Adjust and apply
					start = start / scale;
					jQuery.style( tween.elem, prop, start + unit );

				// Update scale, tolerating zero or NaN from tween.cur()
				// And breaking the loop if scale is unchanged or perfect, or if we've just had enough
				} while ( scale !== (scale = tween.cur() / target) && scale !== 1 && --maxIterations );
			}

			// Update tween properties
			if ( parts ) {
				start = tween.start = +start || +target || 0;
				tween.unit = unit;
				// If a +=/-= token was provided, we're doing a relative animation
				tween.end = parts[ 1 ] ?
					start + ( parts[ 1 ] + 1 ) * parts[ 2 ] :
					+parts[ 2 ];
			}

			return tween;
		}]
	};

// Animations created synchronously will run synchronously
function createFxNow() {
	setTimeout(function() {
		fxNow = undefined;
	});
	return ( fxNow = jQuery.now() );
}

function createTween( value, prop, animation ) {
	var tween,
		collection = ( tweeners[ prop ] || [] ).concat( tweeners[ "*" ] ),
		index = 0,
		length = collection.length;
	for ( ; index < length; index++ ) {
		if ( (tween = collection[ index ].call( animation, prop, value )) ) {

			// we're done with this property
			return tween;
		}
	}
}

function Animation( elem, properties, options ) {
	var result,
		stopped,
		index = 0,
		length = animationPrefilters.length,
		deferred = jQuery.Deferred().always( function() {
			// don't match elem in the :animated selector
			delete tick.elem;
		}),
		tick = function() {
			if ( stopped ) {
				return false;
			}
			var currentTime = fxNow || createFxNow(),
				remaining = Math.max( 0, animation.startTime + animation.duration - currentTime ),
				// archaic crash bug won't allow us to use 1 - ( 0.5 || 0 ) (#12497)
				temp = remaining / animation.duration || 0,
				percent = 1 - temp,
				index = 0,
				length = animation.tweens.length;

			for ( ; index < length ; index++ ) {
				animation.tweens[ index ].run( percent );
			}

			deferred.notifyWith( elem, [ animation, percent, remaining ]);

			if ( percent < 1 && length ) {
				return remaining;
			} else {
				deferred.resolveWith( elem, [ animation ] );
				return false;
			}
		},
		animation = deferred.promise({
			elem: elem,
			props: jQuery.extend( {}, properties ),
			opts: jQuery.extend( true, { specialEasing: {} }, options ),
			originalProperties: properties,
			originalOptions: options,
			startTime: fxNow || createFxNow(),
			duration: options.duration,
			tweens: [],
			createTween: function( prop, end ) {
				var tween = jQuery.Tween( elem, animation.opts, prop, end,
						animation.opts.specialEasing[ prop ] || animation.opts.easing );
				animation.tweens.push( tween );
				return tween;
			},
			stop: function( gotoEnd ) {
				var index = 0,
					// if we are going to the end, we want to run all the tweens
					// otherwise we skip this part
					length = gotoEnd ? animation.tweens.length : 0;
				if ( stopped ) {
					return this;
				}
				stopped = true;
				for ( ; index < length ; index++ ) {
					animation.tweens[ index ].run( 1 );
				}

				// resolve when we played the last frame
				// otherwise, reject
				if ( gotoEnd ) {
					deferred.resolveWith( elem, [ animation, gotoEnd ] );
				} else {
					deferred.rejectWith( elem, [ animation, gotoEnd ] );
				}
				return this;
			}
		}),
		props = animation.props;

	propFilter( props, animation.opts.specialEasing );

	for ( ; index < length ; index++ ) {
		result = animationPrefilters[ index ].call( animation, elem, props, animation.opts );
		if ( result ) {
			return result;
		}
	}

	jQuery.map( props, createTween, animation );

	if ( jQuery.isFunction( animation.opts.start ) ) {
		animation.opts.start.call( elem, animation );
	}

	jQuery.fx.timer(
		jQuery.extend( tick, {
			elem: elem,
			anim: animation,
			queue: animation.opts.queue
		})
	);

	// attach callbacks from options
	return animation.progress( animation.opts.progress )
		.done( animation.opts.done, animation.opts.complete )
		.fail( animation.opts.fail )
		.always( animation.opts.always );
}

function propFilter( props, specialEasing ) {
	var index, name, easing, value, hooks;

	// camelCase, specialEasing and expand cssHook pass
	for ( index in props ) {
		name = jQuery.camelCase( index );
		easing = specialEasing[ name ];
		value = props[ index ];
		if ( jQuery.isArray( value ) ) {
			easing = value[ 1 ];
			value = props[ index ] = value[ 0 ];
		}

		if ( index !== name ) {
			props[ name ] = value;
			delete props[ index ];
		}

		hooks = jQuery.cssHooks[ name ];
		if ( hooks && "expand" in hooks ) {
			value = hooks.expand( value );
			delete props[ name ];

			// not quite $.extend, this wont overwrite keys already present.
			// also - reusing 'index' from above because we have the correct "name"
			for ( index in value ) {
				if ( !( index in props ) ) {
					props[ index ] = value[ index ];
					specialEasing[ index ] = easing;
				}
			}
		} else {
			specialEasing[ name ] = easing;
		}
	}
}

jQuery.Animation = jQuery.extend( Animation, {

	tweener: function( props, callback ) {
		if ( jQuery.isFunction( props ) ) {
			callback = props;
			props = [ "*" ];
		} else {
			props = props.split(" ");
		}

		var prop,
			index = 0,
			length = props.length;

		for ( ; index < length ; index++ ) {
			prop = props[ index ];
			tweeners[ prop ] = tweeners[ prop ] || [];
			tweeners[ prop ].unshift( callback );
		}
	},

	prefilter: function( callback, prepend ) {
		if ( prepend ) {
			animationPrefilters.unshift( callback );
		} else {
			animationPrefilters.push( callback );
		}
	}
});

function defaultPrefilter( elem, props, opts ) {
	/* jshint validthis: true */
	var prop, value, toggle, tween, hooks, oldfire,
		anim = this,
		orig = {},
		style = elem.style,
		hidden = elem.nodeType && isHidden( elem ),
		dataShow = jQuery._data( elem, "fxshow" );

	// handle queue: false promises
	if ( !opts.queue ) {
		hooks = jQuery._queueHooks( elem, "fx" );
		if ( hooks.unqueued == null ) {
			hooks.unqueued = 0;
			oldfire = hooks.empty.fire;
			hooks.empty.fire = function() {
				if ( !hooks.unqueued ) {
					oldfire();
				}
			};
		}
		hooks.unqueued++;

		anim.always(function() {
			// doing this makes sure that the complete handler will be called
			// before this completes
			anim.always(function() {
				hooks.unqueued--;
				if ( !jQuery.queue( elem, "fx" ).length ) {
					hooks.empty.fire();
				}
			});
		});
	}

	// height/width overflow pass
	if ( elem.nodeType === 1 && ( "height" in props || "width" in props ) ) {
		// Make sure that nothing sneaks out
		// Record all 3 overflow attributes because IE does not
		// change the overflow attribute when overflowX and
		// overflowY are set to the same value
		opts.overflow = [ style.overflow, style.overflowX, style.overflowY ];

		// Set display property to inline-block for height/width
		// animations on inline elements that are having width/height animated
		if ( jQuery.css( elem, "display" ) === "inline" &&
				jQuery.css( elem, "float" ) === "none" ) {

			// inline-level elements accept inline-block;
			// block-level elements need to be inline with layout
			if ( !jQuery.support.inlineBlockNeedsLayout || css_defaultDisplay( elem.nodeName ) === "inline" ) {
				style.display = "inline-block";

			} else {
				style.zoom = 1;
			}
		}
	}

	if ( opts.overflow ) {
		style.overflow = "hidden";
		if ( !jQuery.support.shrinkWrapBlocks ) {
			anim.always(function() {
				style.overflow = opts.overflow[ 0 ];
				style.overflowX = opts.overflow[ 1 ];
				style.overflowY = opts.overflow[ 2 ];
			});
		}
	}


	// show/hide pass
	for ( prop in props ) {
		value = props[ prop ];
		if ( rfxtypes.exec( value ) ) {
			delete props[ prop ];
			toggle = toggle || value === "toggle";
			if ( value === ( hidden ? "hide" : "show" ) ) {
				continue;
			}
			orig[ prop ] = dataShow && dataShow[ prop ] || jQuery.style( elem, prop );
		}
	}

	if ( !jQuery.isEmptyObject( orig ) ) {
		if ( dataShow ) {
			if ( "hidden" in dataShow ) {
				hidden = dataShow.hidden;
			}
		} else {
			dataShow = jQuery._data( elem, "fxshow", {} );
		}

		// store state if its toggle - enables .stop().toggle() to "reverse"
		if ( toggle ) {
			dataShow.hidden = !hidden;
		}
		if ( hidden ) {
			jQuery( elem ).show();
		} else {
			anim.done(function() {
				jQuery( elem ).hide();
			});
		}
		anim.done(function() {
			var prop;
			jQuery._removeData( elem, "fxshow" );
			for ( prop in orig ) {
				jQuery.style( elem, prop, orig[ prop ] );
			}
		});
		for ( prop in orig ) {
			tween = createTween( hidden ? dataShow[ prop ] : 0, prop, anim );

			if ( !( prop in dataShow ) ) {
				dataShow[ prop ] = tween.start;
				if ( hidden ) {
					tween.end = tween.start;
					tween.start = prop === "width" || prop === "height" ? 1 : 0;
				}
			}
		}
	}
}

function Tween( elem, options, prop, end, easing ) {
	return new Tween.prototype.init( elem, options, prop, end, easing );
}
jQuery.Tween = Tween;

Tween.prototype = {
	constructor: Tween,
	init: function( elem, options, prop, end, easing, unit ) {
		this.elem = elem;
		this.prop = prop;
		this.easing = easing || "swing";
		this.options = options;
		this.start = this.now = this.cur();
		this.end = end;
		this.unit = unit || ( jQuery.cssNumber[ prop ] ? "" : "px" );
	},
	cur: function() {
		var hooks = Tween.propHooks[ this.prop ];

		return hooks && hooks.get ?
			hooks.get( this ) :
			Tween.propHooks._default.get( this );
	},
	run: function( percent ) {
		var eased,
			hooks = Tween.propHooks[ this.prop ];

		if ( this.options.duration ) {
			this.pos = eased = jQuery.easing[ this.easing ](
				percent, this.options.duration * percent, 0, 1, this.options.duration
			);
		} else {
			this.pos = eased = percent;
		}
		this.now = ( this.end - this.start ) * eased + this.start;

		if ( this.options.step ) {
			this.options.step.call( this.elem, this.now, this );
		}

		if ( hooks && hooks.set ) {
			hooks.set( this );
		} else {
			Tween.propHooks._default.set( this );
		}
		return this;
	}
};

Tween.prototype.init.prototype = Tween.prototype;

Tween.propHooks = {
	_default: {
		get: function( tween ) {
			var result;

			if ( tween.elem[ tween.prop ] != null &&
				(!tween.elem.style || tween.elem.style[ tween.prop ] == null) ) {
				return tween.elem[ tween.prop ];
			}

			// passing an empty string as a 3rd parameter to .css will automatically
			// attempt a parseFloat and fallback to a string if the parse fails
			// so, simple values such as "10px" are parsed to Float.
			// complex values such as "rotate(1rad)" are returned as is.
			result = jQuery.css( tween.elem, tween.prop, "" );
			// Empty strings, null, undefined and "auto" are converted to 0.
			return !result || result === "auto" ? 0 : result;
		},
		set: function( tween ) {
			// use step hook for back compat - use cssHook if its there - use .style if its
			// available and use plain properties where available
			if ( jQuery.fx.step[ tween.prop ] ) {
				jQuery.fx.step[ tween.prop ]( tween );
			} else if ( tween.elem.style && ( tween.elem.style[ jQuery.cssProps[ tween.prop ] ] != null || jQuery.cssHooks[ tween.prop ] ) ) {
				jQuery.style( tween.elem, tween.prop, tween.now + tween.unit );
			} else {
				tween.elem[ tween.prop ] = tween.now;
			}
		}
	}
};

// Support: IE <=9
// Panic based approach to setting things on disconnected nodes

Tween.propHooks.scrollTop = Tween.propHooks.scrollLeft = {
	set: function( tween ) {
		if ( tween.elem.nodeType && tween.elem.parentNode ) {
			tween.elem[ tween.prop ] = tween.now;
		}
	}
};

jQuery.each([ "toggle", "show", "hide" ], function( i, name ) {
	var cssFn = jQuery.fn[ name ];
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return speed == null || typeof speed === "boolean" ?
			cssFn.apply( this, arguments ) :
			this.animate( genFx( name, true ), speed, easing, callback );
	};
});

jQuery.fn.extend({
	fadeTo: function( speed, to, easing, callback ) {

		// show any hidden elements after setting opacity to 0
		return this.filter( isHidden ).css( "opacity", 0 ).show()

			// animate to the value specified
			.end().animate({ opacity: to }, speed, easing, callback );
	},
	animate: function( prop, speed, easing, callback ) {
		var empty = jQuery.isEmptyObject( prop ),
			optall = jQuery.speed( speed, easing, callback ),
			doAnimation = function() {
				// Operate on a copy of prop so per-property easing won't be lost
				var anim = Animation( this, jQuery.extend( {}, prop ), optall );

				// Empty animations, or finishing resolves immediately
				if ( empty || jQuery._data( this, "finish" ) ) {
					anim.stop( true );
				}
			};
			doAnimation.finish = doAnimation;

		return empty || optall.queue === false ?
			this.each( doAnimation ) :
			this.queue( optall.queue, doAnimation );
	},
	stop: function( type, clearQueue, gotoEnd ) {
		var stopQueue = function( hooks ) {
			var stop = hooks.stop;
			delete hooks.stop;
			stop( gotoEnd );
		};

		if ( typeof type !== "string" ) {
			gotoEnd = clearQueue;
			clearQueue = type;
			type = undefined;
		}
		if ( clearQueue && type !== false ) {
			this.queue( type || "fx", [] );
		}

		return this.each(function() {
			var dequeue = true,
				index = type != null && type + "queueHooks",
				timers = jQuery.timers,
				data = jQuery._data( this );

			if ( index ) {
				if ( data[ index ] && data[ index ].stop ) {
					stopQueue( data[ index ] );
				}
			} else {
				for ( index in data ) {
					if ( data[ index ] && data[ index ].stop && rrun.test( index ) ) {
						stopQueue( data[ index ] );
					}
				}
			}

			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this && (type == null || timers[ index ].queue === type) ) {
					timers[ index ].anim.stop( gotoEnd );
					dequeue = false;
					timers.splice( index, 1 );
				}
			}

			// start the next in the queue if the last step wasn't forced
			// timers currently will call their complete callbacks, which will dequeue
			// but only if they were gotoEnd
			if ( dequeue || !gotoEnd ) {
				jQuery.dequeue( this, type );
			}
		});
	},
	finish: function( type ) {
		if ( type !== false ) {
			type = type || "fx";
		}
		return this.each(function() {
			var index,
				data = jQuery._data( this ),
				queue = data[ type + "queue" ],
				hooks = data[ type + "queueHooks" ],
				timers = jQuery.timers,
				length = queue ? queue.length : 0;

			// enable finishing flag on private data
			data.finish = true;

			// empty the queue first
			jQuery.queue( this, type, [] );

			if ( hooks && hooks.stop ) {
				hooks.stop.call( this, true );
			}

			// look for any active animations, and finish them
			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this && timers[ index ].queue === type ) {
					timers[ index ].anim.stop( true );
					timers.splice( index, 1 );
				}
			}

			// look for any animations in the old queue and finish them
			for ( index = 0; index < length; index++ ) {
				if ( queue[ index ] && queue[ index ].finish ) {
					queue[ index ].finish.call( this );
				}
			}

			// turn off finishing flag
			delete data.finish;
		});
	}
});

// Generate parameters to create a standard animation
function genFx( type, includeWidth ) {
	var which,
		attrs = { height: type },
		i = 0;

	// if we include width, step value is 1 to do all cssExpand values,
	// if we don't include width, step value is 2 to skip over Left and Right
	includeWidth = includeWidth? 1 : 0;
	for( ; i < 4 ; i += 2 - includeWidth ) {
		which = cssExpand[ i ];
		attrs[ "margin" + which ] = attrs[ "padding" + which ] = type;
	}

	if ( includeWidth ) {
		attrs.opacity = attrs.width = type;
	}

	return attrs;
}

// Generate shortcuts for custom animations
jQuery.each({
	slideDown: genFx("show"),
	slideUp: genFx("hide"),
	slideToggle: genFx("toggle"),
	fadeIn: { opacity: "show" },
	fadeOut: { opacity: "hide" },
	fadeToggle: { opacity: "toggle" }
}, function( name, props ) {
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return this.animate( props, speed, easing, callback );
	};
});

jQuery.speed = function( speed, easing, fn ) {
	var opt = speed && typeof speed === "object" ? jQuery.extend( {}, speed ) : {
		complete: fn || !fn && easing ||
			jQuery.isFunction( speed ) && speed,
		duration: speed,
		easing: fn && easing || easing && !jQuery.isFunction( easing ) && easing
	};

	opt.duration = jQuery.fx.off ? 0 : typeof opt.duration === "number" ? opt.duration :
		opt.duration in jQuery.fx.speeds ? jQuery.fx.speeds[ opt.duration ] : jQuery.fx.speeds._default;

	// normalize opt.queue - true/undefined/null -> "fx"
	if ( opt.queue == null || opt.queue === true ) {
		opt.queue = "fx";
	}

	// Queueing
	opt.old = opt.complete;

	opt.complete = function() {
		if ( jQuery.isFunction( opt.old ) ) {
			opt.old.call( this );
		}

		if ( opt.queue ) {
			jQuery.dequeue( this, opt.queue );
		}
	};

	return opt;
};

jQuery.easing = {
	linear: function( p ) {
		return p;
	},
	swing: function( p ) {
		return 0.5 - Math.cos( p*Math.PI ) / 2;
	}
};

jQuery.timers = [];
jQuery.fx = Tween.prototype.init;
jQuery.fx.tick = function() {
	var timer,
		timers = jQuery.timers,
		i = 0;

	fxNow = jQuery.now();

	for ( ; i < timers.length; i++ ) {
		timer = timers[ i ];
		// Checks the timer has not already been removed
		if ( !timer() && timers[ i ] === timer ) {
			timers.splice( i--, 1 );
		}
	}

	if ( !timers.length ) {
		jQuery.fx.stop();
	}
	fxNow = undefined;
};

jQuery.fx.timer = function( timer ) {
	if ( timer() && jQuery.timers.push( timer ) ) {
		jQuery.fx.start();
	}
};

jQuery.fx.interval = 13;

jQuery.fx.start = function() {
	if ( !timerId ) {
		timerId = setInterval( jQuery.fx.tick, jQuery.fx.interval );
	}
};

jQuery.fx.stop = function() {
	clearInterval( timerId );
	timerId = null;
};

jQuery.fx.speeds = {
	slow: 600,
	fast: 200,
	// Default speed
	_default: 400
};

// Back Compat <1.8 extension point
jQuery.fx.step = {};

if ( jQuery.expr && jQuery.expr.filters ) {
	jQuery.expr.filters.animated = function( elem ) {
		return jQuery.grep(jQuery.timers, function( fn ) {
			return elem === fn.elem;
		}).length;
	};
}
jQuery.fn.offset = function( options ) {
	if ( arguments.length ) {
		return options === undefined ?
			this :
			this.each(function( i ) {
				jQuery.offset.setOffset( this, options, i );
			});
	}

	var docElem, win,
		box = { top: 0, left: 0 },
		elem = this[ 0 ],
		doc = elem && elem.ownerDocument;

	if ( !doc ) {
		return;
	}

	docElem = doc.documentElement;

	// Make sure it's not a disconnected DOM node
	if ( !jQuery.contains( docElem, elem ) ) {
		return box;
	}

	// If we don't have gBCR, just use 0,0 rather than error
	// BlackBerry 5, iOS 3 (original iPhone)
	if ( typeof elem.getBoundingClientRect !== core_strundefined ) {
		box = elem.getBoundingClientRect();
	}
	win = getWindow( doc );
	return {
		top: box.top  + ( win.pageYOffset || docElem.scrollTop )  - ( docElem.clientTop  || 0 ),
		left: box.left + ( win.pageXOffset || docElem.scrollLeft ) - ( docElem.clientLeft || 0 )
	};
};

jQuery.offset = {

	setOffset: function( elem, options, i ) {
		var position = jQuery.css( elem, "position" );

		// set position first, in-case top/left are set even on static elem
		if ( position === "static" ) {
			elem.style.position = "relative";
		}

		var curElem = jQuery( elem ),
			curOffset = curElem.offset(),
			curCSSTop = jQuery.css( elem, "top" ),
			curCSSLeft = jQuery.css( elem, "left" ),
			calculatePosition = ( position === "absolute" || position === "fixed" ) && jQuery.inArray("auto", [curCSSTop, curCSSLeft]) > -1,
			props = {}, curPosition = {}, curTop, curLeft;

		// need to be able to calculate position if either top or left is auto and position is either absolute or fixed
		if ( calculatePosition ) {
			curPosition = curElem.position();
			curTop = curPosition.top;
			curLeft = curPosition.left;
		} else {
			curTop = parseFloat( curCSSTop ) || 0;
			curLeft = parseFloat( curCSSLeft ) || 0;
		}

		if ( jQuery.isFunction( options ) ) {
			options = options.call( elem, i, curOffset );
		}

		if ( options.top != null ) {
			props.top = ( options.top - curOffset.top ) + curTop;
		}
		if ( options.left != null ) {
			props.left = ( options.left - curOffset.left ) + curLeft;
		}

		if ( "using" in options ) {
			options.using.call( elem, props );
		} else {
			curElem.css( props );
		}
	}
};


jQuery.fn.extend({

	position: function() {
		if ( !this[ 0 ] ) {
			return;
		}

		var offsetParent, offset,
			parentOffset = { top: 0, left: 0 },
			elem = this[ 0 ];

		// fixed elements are offset from window (parentOffset = {top:0, left: 0}, because it is it's only offset parent
		if ( jQuery.css( elem, "position" ) === "fixed" ) {
			// we assume that getBoundingClientRect is available when computed position is fixed
			offset = elem.getBoundingClientRect();
		} else {
			// Get *real* offsetParent
			offsetParent = this.offsetParent();

			// Get correct offsets
			offset = this.offset();
			if ( !jQuery.nodeName( offsetParent[ 0 ], "html" ) ) {
				parentOffset = offsetParent.offset();
			}

			// Add offsetParent borders
			parentOffset.top  += jQuery.css( offsetParent[ 0 ], "borderTopWidth", true );
			parentOffset.left += jQuery.css( offsetParent[ 0 ], "borderLeftWidth", true );
		}

		// Subtract parent offsets and element margins
		// note: when an element has margin: auto the offsetLeft and marginLeft
		// are the same in Safari causing offset.left to incorrectly be 0
		return {
			top:  offset.top  - parentOffset.top - jQuery.css( elem, "marginTop", true ),
			left: offset.left - parentOffset.left - jQuery.css( elem, "marginLeft", true)
		};
	},

	offsetParent: function() {
		return this.map(function() {
			var offsetParent = this.offsetParent || docElem;
			while ( offsetParent && ( !jQuery.nodeName( offsetParent, "html" ) && jQuery.css( offsetParent, "position") === "static" ) ) {
				offsetParent = offsetParent.offsetParent;
			}
			return offsetParent || docElem;
		});
	}
});


// Create scrollLeft and scrollTop methods
jQuery.each( {scrollLeft: "pageXOffset", scrollTop: "pageYOffset"}, function( method, prop ) {
	var top = /Y/.test( prop );

	jQuery.fn[ method ] = function( val ) {
		return jQuery.access( this, function( elem, method, val ) {
			var win = getWindow( elem );

			if ( val === undefined ) {
				return win ? (prop in win) ? win[ prop ] :
					win.document.documentElement[ method ] :
					elem[ method ];
			}

			if ( win ) {
				win.scrollTo(
					!top ? val : jQuery( win ).scrollLeft(),
					top ? val : jQuery( win ).scrollTop()
				);

			} else {
				elem[ method ] = val;
			}
		}, method, val, arguments.length, null );
	};
});

function getWindow( elem ) {
	return jQuery.isWindow( elem ) ?
		elem :
		elem.nodeType === 9 ?
			elem.defaultView || elem.parentWindow :
			false;
}
// Create innerHeight, innerWidth, height, width, outerHeight and outerWidth methods
jQuery.each( { Height: "height", Width: "width" }, function( name, type ) {
	jQuery.each( { padding: "inner" + name, content: type, "": "outer" + name }, function( defaultExtra, funcName ) {
		// margin is only for outerHeight, outerWidth
		jQuery.fn[ funcName ] = function( margin, value ) {
			var chainable = arguments.length && ( defaultExtra || typeof margin !== "boolean" ),
				extra = defaultExtra || ( margin === true || value === true ? "margin" : "border" );

			return jQuery.access( this, function( elem, type, value ) {
				var doc;

				if ( jQuery.isWindow( elem ) ) {
					// As of 5/8/2012 this will yield incorrect results for Mobile Safari, but there
					// isn't a whole lot we can do. See pull request at this URL for discussion:
					// https://github.com/jquery/jquery/pull/764
					return elem.document.documentElement[ "client" + name ];
				}

				// Get document width or height
				if ( elem.nodeType === 9 ) {
					doc = elem.documentElement;

					// Either scroll[Width/Height] or offset[Width/Height] or client[Width/Height], whichever is greatest
					// unfortunately, this causes bug #3838 in IE6/8 only, but there is currently no good, small way to fix it.
					return Math.max(
						elem.body[ "scroll" + name ], doc[ "scroll" + name ],
						elem.body[ "offset" + name ], doc[ "offset" + name ],
						doc[ "client" + name ]
					);
				}

				return value === undefined ?
					// Get width or height on the element, requesting but not forcing parseFloat
					jQuery.css( elem, type, extra ) :

					// Set width or height on the element
					jQuery.style( elem, type, value, extra );
			}, type, chainable ? margin : undefined, chainable, null );
		};
	});
});
// Limit scope pollution from any deprecated API
// (function() {

// The number of elements contained in the matched element set
jQuery.fn.size = function() {
	return this.length;
};

jQuery.fn.andSelf = jQuery.fn.addBack;

// })();
if ( typeof module === "object" && module && typeof module.exports === "object" ) {
	// Expose jQuery as module.exports in loaders that implement the Node
	// module pattern (including browserify). Do not create the global, since
	// the user will be storing it themselves locally, and globals are frowned
	// upon in the Node module world.
	module.exports = jQuery;
} else {
	// Otherwise expose jQuery to the global object as usual
	window.jQuery = window.$ = jQuery;

	// Register as a named AMD module, since jQuery can be concatenated with other
	// files that may use define, but not via a proper concatenation script that
	// understands anonymous AMD modules. A named AMD is safest and most robust
	// way to register. Lowercase jquery is used because AMD module names are
	// derived from file names, and jQuery is normally delivered in a lowercase
	// file name. Do this after creating the global so that if an AMD module wants
	// to call noConflict to hide this version of jQuery, it will work.
	if ( typeof define === "function" && define.amd ) {
		define( "jquery", [], function () { return jQuery; } );
	}
}

})( window );

//     Underscore.js 1.5.2
//     http://underscorejs.org
//     (c) 2009-2013 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
//     Underscore may be freely distributed under the MIT license.

(function() {

  // Baseline setup
  // --------------

  // Establish the root object, `window` in the browser, or `exports` on the server.
  var root = this;

  // Save the previous value of the `_` variable.
  var previousUnderscore = root._;

  // Establish the object that gets returned to break out of a loop iteration.
  var breaker = {};

  // Save bytes in the minified (but not gzipped) version:
  var ArrayProto = Array.prototype, ObjProto = Object.prototype, FuncProto = Function.prototype;

  // Create quick reference variables for speed access to core prototypes.
  var
    push             = ArrayProto.push,
    slice            = ArrayProto.slice,
    concat           = ArrayProto.concat,
    toString         = ObjProto.toString,
    hasOwnProperty   = ObjProto.hasOwnProperty;

  // All **ECMAScript 5** native function implementations that we hope to use
  // are declared here.
  var
    nativeForEach      = ArrayProto.forEach,
    nativeMap          = ArrayProto.map,
    nativeReduce       = ArrayProto.reduce,
    nativeReduceRight  = ArrayProto.reduceRight,
    nativeFilter       = ArrayProto.filter,
    nativeEvery        = ArrayProto.every,
    nativeSome         = ArrayProto.some,
    nativeIndexOf      = ArrayProto.indexOf,
    nativeLastIndexOf  = ArrayProto.lastIndexOf,
    nativeIsArray      = Array.isArray,
    nativeKeys         = Object.keys,
    nativeBind         = FuncProto.bind;

  // Create a safe reference to the Underscore object for use below.
  var _ = function(obj) {
    if (obj instanceof _) return obj;
    if (!(this instanceof _)) return new _(obj);
    this._wrapped = obj;
  };

  // Export the Underscore object for **Node.js**, with
  // backwards-compatibility for the old `require()` API. If we're in
  // the browser, add `_` as a global object via a string identifier,
  // for Closure Compiler "advanced" mode.
  if (typeof exports !== 'undefined') {
    if (typeof module !== 'undefined' && module.exports) {
      exports = module.exports = _;
    }
    exports._ = _;
  } else {
    root._ = _;
  }

  // Current version.
  _.VERSION = '1.5.2';

  // Collection Functions
  // --------------------

  // The cornerstone, an `each` implementation, aka `forEach`.
  // Handles objects with the built-in `forEach`, arrays, and raw objects.
  // Delegates to **ECMAScript 5**'s native `forEach` if available.
  var each = _.each = _.forEach = function(obj, iterator, context) {
    if (obj == null) return;
    if (nativeForEach && obj.forEach === nativeForEach) {
      obj.forEach(iterator, context);
    } else if (obj.length === +obj.length) {
      for (var i = 0, length = obj.length; i < length; i++) {
        if (iterator.call(context, obj[i], i, obj) === breaker) return;
      }
    } else {
      var keys = _.keys(obj);
      for (var i = 0, length = keys.length; i < length; i++) {
        if (iterator.call(context, obj[keys[i]], keys[i], obj) === breaker) return;
      }
    }
  };

  // Return the results of applying the iterator to each element.
  // Delegates to **ECMAScript 5**'s native `map` if available.
  _.map = _.collect = function(obj, iterator, context) {
    var results = [];
    if (obj == null) return results;
    if (nativeMap && obj.map === nativeMap) return obj.map(iterator, context);
    each(obj, function(value, index, list) {
      results.push(iterator.call(context, value, index, list));
    });
    return results;
  };

  var reduceError = 'Reduce of empty array with no initial value';

  // **Reduce** builds up a single result from a list of values, aka `inject`,
  // or `foldl`. Delegates to **ECMAScript 5**'s native `reduce` if available.
  _.reduce = _.foldl = _.inject = function(obj, iterator, memo, context) {
    var initial = arguments.length > 2;
    if (obj == null) obj = [];
    if (nativeReduce && obj.reduce === nativeReduce) {
      if (context) iterator = _.bind(iterator, context);
      return initial ? obj.reduce(iterator, memo) : obj.reduce(iterator);
    }
    each(obj, function(value, index, list) {
      if (!initial) {
        memo = value;
        initial = true;
      } else {
        memo = iterator.call(context, memo, value, index, list);
      }
    });
    if (!initial) throw new TypeError(reduceError);
    return memo;
  };

  // The right-associative version of reduce, also known as `foldr`.
  // Delegates to **ECMAScript 5**'s native `reduceRight` if available.
  _.reduceRight = _.foldr = function(obj, iterator, memo, context) {
    var initial = arguments.length > 2;
    if (obj == null) obj = [];
    if (nativeReduceRight && obj.reduceRight === nativeReduceRight) {
      if (context) iterator = _.bind(iterator, context);
      return initial ? obj.reduceRight(iterator, memo) : obj.reduceRight(iterator);
    }
    var length = obj.length;
    if (length !== +length) {
      var keys = _.keys(obj);
      length = keys.length;
    }
    each(obj, function(value, index, list) {
      index = keys ? keys[--length] : --length;
      if (!initial) {
        memo = obj[index];
        initial = true;
      } else {
        memo = iterator.call(context, memo, obj[index], index, list);
      }
    });
    if (!initial) throw new TypeError(reduceError);
    return memo;
  };

  // Return the first value which passes a truth test. Aliased as `detect`.
  _.find = _.detect = function(obj, iterator, context) {
    var result;
    any(obj, function(value, index, list) {
      if (iterator.call(context, value, index, list)) {
        result = value;
        return true;
      }
    });
    return result;
  };

  // Return all the elements that pass a truth test.
  // Delegates to **ECMAScript 5**'s native `filter` if available.
  // Aliased as `select`.
  _.filter = _.select = function(obj, iterator, context) {
    var results = [];
    if (obj == null) return results;
    if (nativeFilter && obj.filter === nativeFilter) return obj.filter(iterator, context);
    each(obj, function(value, index, list) {
      if (iterator.call(context, value, index, list)) results.push(value);
    });
    return results;
  };

  // Return all the elements for which a truth test fails.
  _.reject = function(obj, iterator, context) {
    return _.filter(obj, function(value, index, list) {
      return !iterator.call(context, value, index, list);
    }, context);
  };

  // Determine whether all of the elements match a truth test.
  // Delegates to **ECMAScript 5**'s native `every` if available.
  // Aliased as `all`.
  _.every = _.all = function(obj, iterator, context) {
    iterator || (iterator = _.identity);
    var result = true;
    if (obj == null) return result;
    if (nativeEvery && obj.every === nativeEvery) return obj.every(iterator, context);
    each(obj, function(value, index, list) {
      if (!(result = result && iterator.call(context, value, index, list))) return breaker;
    });
    return !!result;
  };

  // Determine if at least one element in the object matches a truth test.
  // Delegates to **ECMAScript 5**'s native `some` if available.
  // Aliased as `any`.
  var any = _.some = _.any = function(obj, iterator, context) {
    iterator || (iterator = _.identity);
    var result = false;
    if (obj == null) return result;
    if (nativeSome && obj.some === nativeSome) return obj.some(iterator, context);
    each(obj, function(value, index, list) {
      if (result || (result = iterator.call(context, value, index, list))) return breaker;
    });
    return !!result;
  };

  // Determine if the array or object contains a given value (using `===`).
  // Aliased as `include`.
  _.contains = _.include = function(obj, target) {
    if (obj == null) return false;
    if (nativeIndexOf && obj.indexOf === nativeIndexOf) return obj.indexOf(target) != -1;
    return any(obj, function(value) {
      return value === target;
    });
  };

  // Invoke a method (with arguments) on every item in a collection.
  _.invoke = function(obj, method) {
    var args = slice.call(arguments, 2);
    var isFunc = _.isFunction(method);
    return _.map(obj, function(value) {
      return (isFunc ? method : value[method]).apply(value, args);
    });
  };

  // Convenience version of a common use case of `map`: fetching a property.
  _.pluck = function(obj, key) {
    return _.map(obj, function(value){ return value[key]; });
  };

  // Convenience version of a common use case of `filter`: selecting only objects
  // containing specific `key:value` pairs.
  _.where = function(obj, attrs, first) {
    if (_.isEmpty(attrs)) return first ? void 0 : [];
    return _[first ? 'find' : 'filter'](obj, function(value) {
      for (var key in attrs) {
        if (attrs[key] !== value[key]) return false;
      }
      return true;
    });
  };

  // Convenience version of a common use case of `find`: getting the first object
  // containing specific `key:value` pairs.
  _.findWhere = function(obj, attrs) {
    return _.where(obj, attrs, true);
  };

  // Return the maximum element or (element-based computation).
  // Can't optimize arrays of integers longer than 65,535 elements.
  // See [WebKit Bug 80797](https://bugs.webkit.org/show_bug.cgi?id=80797)
  _.max = function(obj, iterator, context) {
    if (!iterator && _.isArray(obj) && obj[0] === +obj[0] && obj.length < 65535) {
      return Math.max.apply(Math, obj);
    }
    if (!iterator && _.isEmpty(obj)) return -Infinity;
    var result = {computed : -Infinity, value: -Infinity};
    each(obj, function(value, index, list) {
      var computed = iterator ? iterator.call(context, value, index, list) : value;
      computed > result.computed && (result = {value : value, computed : computed});
    });
    return result.value;
  };

  // Return the minimum element (or element-based computation).
  _.min = function(obj, iterator, context) {
    if (!iterator && _.isArray(obj) && obj[0] === +obj[0] && obj.length < 65535) {
      return Math.min.apply(Math, obj);
    }
    if (!iterator && _.isEmpty(obj)) return Infinity;
    var result = {computed : Infinity, value: Infinity};
    each(obj, function(value, index, list) {
      var computed = iterator ? iterator.call(context, value, index, list) : value;
      computed < result.computed && (result = {value : value, computed : computed});
    });
    return result.value;
  };

  // Shuffle an array, using the modern version of the 
  // [Fisher-Yates shuffle](http://en.wikipedia.org/wiki/Fisher–Yates_shuffle).
  _.shuffle = function(obj) {
    var rand;
    var index = 0;
    var shuffled = [];
    each(obj, function(value) {
      rand = _.random(index++);
      shuffled[index - 1] = shuffled[rand];
      shuffled[rand] = value;
    });
    return shuffled;
  };

  // Sample **n** random values from an array.
  // If **n** is not specified, returns a single random element from the array.
  // The internal `guard` argument allows it to work with `map`.
  _.sample = function(obj, n, guard) {
    if (arguments.length < 2 || guard) {
      return obj[_.random(obj.length - 1)];
    }
    return _.shuffle(obj).slice(0, Math.max(0, n));
  };

  // An internal function to generate lookup iterators.
  var lookupIterator = function(value) {
    return _.isFunction(value) ? value : function(obj){ return obj[value]; };
  };

  // Sort the object's values by a criterion produced by an iterator.
  _.sortBy = function(obj, value, context) {
    var iterator = lookupIterator(value);
    return _.pluck(_.map(obj, function(value, index, list) {
      return {
        value: value,
        index: index,
        criteria: iterator.call(context, value, index, list)
      };
    }).sort(function(left, right) {
      var a = left.criteria;
      var b = right.criteria;
      if (a !== b) {
        if (a > b || a === void 0) return 1;
        if (a < b || b === void 0) return -1;
      }
      return left.index - right.index;
    }), 'value');
  };

  // An internal function used for aggregate "group by" operations.
  var group = function(behavior) {
    return function(obj, value, context) {
      var result = {};
      var iterator = value == null ? _.identity : lookupIterator(value);
      each(obj, function(value, index) {
        var key = iterator.call(context, value, index, obj);
        behavior(result, key, value);
      });
      return result;
    };
  };

  // Groups the object's values by a criterion. Pass either a string attribute
  // to group by, or a function that returns the criterion.
  _.groupBy = group(function(result, key, value) {
    (_.has(result, key) ? result[key] : (result[key] = [])).push(value);
  });

  // Indexes the object's values by a criterion, similar to `groupBy`, but for
  // when you know that your index values will be unique.
  _.indexBy = group(function(result, key, value) {
    result[key] = value;
  });

  // Counts instances of an object that group by a certain criterion. Pass
  // either a string attribute to count by, or a function that returns the
  // criterion.
  _.countBy = group(function(result, key) {
    _.has(result, key) ? result[key]++ : result[key] = 1;
  });

  // Use a comparator function to figure out the smallest index at which
  // an object should be inserted so as to maintain order. Uses binary search.
  _.sortedIndex = function(array, obj, iterator, context) {
    iterator = iterator == null ? _.identity : lookupIterator(iterator);
    var value = iterator.call(context, obj);
    var low = 0, high = array.length;
    while (low < high) {
      var mid = (low + high) >>> 1;
      iterator.call(context, array[mid]) < value ? low = mid + 1 : high = mid;
    }
    return low;
  };

  // Safely create a real, live array from anything iterable.
  _.toArray = function(obj) {
    if (!obj) return [];
    if (_.isArray(obj)) return slice.call(obj);
    if (obj.length === +obj.length) return _.map(obj, _.identity);
    return _.values(obj);
  };

  // Return the number of elements in an object.
  _.size = function(obj) {
    if (obj == null) return 0;
    return (obj.length === +obj.length) ? obj.length : _.keys(obj).length;
  };

  // Array Functions
  // ---------------

  // Get the first element of an array. Passing **n** will return the first N
  // values in the array. Aliased as `head` and `take`. The **guard** check
  // allows it to work with `_.map`.
  _.first = _.head = _.take = function(array, n, guard) {
    if (array == null) return void 0;
    return (n == null) || guard ? array[0] : slice.call(array, 0, n);
  };

  // Returns everything but the last entry of the array. Especially useful on
  // the arguments object. Passing **n** will return all the values in
  // the array, excluding the last N. The **guard** check allows it to work with
  // `_.map`.
  _.initial = function(array, n, guard) {
    return slice.call(array, 0, array.length - ((n == null) || guard ? 1 : n));
  };

  // Get the last element of an array. Passing **n** will return the last N
  // values in the array. The **guard** check allows it to work with `_.map`.
  _.last = function(array, n, guard) {
    if (array == null) return void 0;
    if ((n == null) || guard) {
      return array[array.length - 1];
    } else {
      return slice.call(array, Math.max(array.length - n, 0));
    }
  };

  // Returns everything but the first entry of the array. Aliased as `tail` and `drop`.
  // Especially useful on the arguments object. Passing an **n** will return
  // the rest N values in the array. The **guard**
  // check allows it to work with `_.map`.
  _.rest = _.tail = _.drop = function(array, n, guard) {
    return slice.call(array, (n == null) || guard ? 1 : n);
  };

  // Trim out all falsy values from an array.
  _.compact = function(array) {
    return _.filter(array, _.identity);
  };

  // Internal implementation of a recursive `flatten` function.
  var flatten = function(input, shallow, output) {
    if (shallow && _.every(input, _.isArray)) {
      return concat.apply(output, input);
    }
    each(input, function(value) {
      if (_.isArray(value) || _.isArguments(value)) {
        shallow ? push.apply(output, value) : flatten(value, shallow, output);
      } else {
        output.push(value);
      }
    });
    return output;
  };

  // Flatten out an array, either recursively (by default), or just one level.
  _.flatten = function(array, shallow) {
    return flatten(array, shallow, []);
  };

  // Return a version of the array that does not contain the specified value(s).
  _.without = function(array) {
    return _.difference(array, slice.call(arguments, 1));
  };

  // Produce a duplicate-free version of the array. If the array has already
  // been sorted, you have the option of using a faster algorithm.
  // Aliased as `unique`.
  _.uniq = _.unique = function(array, isSorted, iterator, context) {
    if (_.isFunction(isSorted)) {
      context = iterator;
      iterator = isSorted;
      isSorted = false;
    }
    var initial = iterator ? _.map(array, iterator, context) : array;
    var results = [];
    var seen = [];
    each(initial, function(value, index) {
      if (isSorted ? (!index || seen[seen.length - 1] !== value) : !_.contains(seen, value)) {
        seen.push(value);
        results.push(array[index]);
      }
    });
    return results;
  };

  // Produce an array that contains the union: each distinct element from all of
  // the passed-in arrays.
  _.union = function() {
    return _.uniq(_.flatten(arguments, true));
  };

  // Produce an array that contains every item shared between all the
  // passed-in arrays.
  _.intersection = function(array) {
    var rest = slice.call(arguments, 1);
    return _.filter(_.uniq(array), function(item) {
      return _.every(rest, function(other) {
        return _.indexOf(other, item) >= 0;
      });
    });
  };

  // Take the difference between one array and a number of other arrays.
  // Only the elements present in just the first array will remain.
  _.difference = function(array) {
    var rest = concat.apply(ArrayProto, slice.call(arguments, 1));
    return _.filter(array, function(value){ return !_.contains(rest, value); });
  };

  // Zip together multiple lists into a single array -- elements that share
  // an index go together.
  _.zip = function() {
    var length = _.max(_.pluck(arguments, "length").concat(0));
    var results = new Array(length);
    for (var i = 0; i < length; i++) {
      results[i] = _.pluck(arguments, '' + i);
    }
    return results;
  };

  // Converts lists into objects. Pass either a single array of `[key, value]`
  // pairs, or two parallel arrays of the same length -- one of keys, and one of
  // the corresponding values.
  _.object = function(list, values) {
    if (list == null) return {};
    var result = {};
    for (var i = 0, length = list.length; i < length; i++) {
      if (values) {
        result[list[i]] = values[i];
      } else {
        result[list[i][0]] = list[i][1];
      }
    }
    return result;
  };

  // If the browser doesn't supply us with indexOf (I'm looking at you, **MSIE**),
  // we need this function. Return the position of the first occurrence of an
  // item in an array, or -1 if the item is not included in the array.
  // Delegates to **ECMAScript 5**'s native `indexOf` if available.
  // If the array is large and already in sort order, pass `true`
  // for **isSorted** to use binary search.
  _.indexOf = function(array, item, isSorted) {
    if (array == null) return -1;
    var i = 0, length = array.length;
    if (isSorted) {
      if (typeof isSorted == 'number') {
        i = (isSorted < 0 ? Math.max(0, length + isSorted) : isSorted);
      } else {
        i = _.sortedIndex(array, item);
        return array[i] === item ? i : -1;
      }
    }
    if (nativeIndexOf && array.indexOf === nativeIndexOf) return array.indexOf(item, isSorted);
    for (; i < length; i++) if (array[i] === item) return i;
    return -1;
  };

  // Delegates to **ECMAScript 5**'s native `lastIndexOf` if available.
  _.lastIndexOf = function(array, item, from) {
    if (array == null) return -1;
    var hasIndex = from != null;
    if (nativeLastIndexOf && array.lastIndexOf === nativeLastIndexOf) {
      return hasIndex ? array.lastIndexOf(item, from) : array.lastIndexOf(item);
    }
    var i = (hasIndex ? from : array.length);
    while (i--) if (array[i] === item) return i;
    return -1;
  };

  // Generate an integer Array containing an arithmetic progression. A port of
  // the native Python `range()` function. See
  // [the Python documentation](http://docs.python.org/library/functions.html#range).
  _.range = function(start, stop, step) {
    if (arguments.length <= 1) {
      stop = start || 0;
      start = 0;
    }
    step = arguments[2] || 1;

    var length = Math.max(Math.ceil((stop - start) / step), 0);
    var idx = 0;
    var range = new Array(length);

    while(idx < length) {
      range[idx++] = start;
      start += step;
    }

    return range;
  };

  // Function (ahem) Functions
  // ------------------

  // Reusable constructor function for prototype setting.
  var ctor = function(){};

  // Create a function bound to a given object (assigning `this`, and arguments,
  // optionally). Delegates to **ECMAScript 5**'s native `Function.bind` if
  // available.
  _.bind = function(func, context) {
    var args, bound;
    if (nativeBind && func.bind === nativeBind) return nativeBind.apply(func, slice.call(arguments, 1));
    if (!_.isFunction(func)) throw new TypeError;
    args = slice.call(arguments, 2);
    return bound = function() {
      if (!(this instanceof bound)) return func.apply(context, args.concat(slice.call(arguments)));
      ctor.prototype = func.prototype;
      var self = new ctor;
      ctor.prototype = null;
      var result = func.apply(self, args.concat(slice.call(arguments)));
      if (Object(result) === result) return result;
      return self;
    };
  };

  // Partially apply a function by creating a version that has had some of its
  // arguments pre-filled, without changing its dynamic `this` context.
  _.partial = function(func) {
    var args = slice.call(arguments, 1);
    return function() {
      return func.apply(this, args.concat(slice.call(arguments)));
    };
  };

  // Bind all of an object's methods to that object. Useful for ensuring that
  // all callbacks defined on an object belong to it.
  _.bindAll = function(obj) {
    var funcs = slice.call(arguments, 1);
    if (funcs.length === 0) throw new Error("bindAll must be passed function names");
    each(funcs, function(f) { obj[f] = _.bind(obj[f], obj); });
    return obj;
  };

  // Memoize an expensive function by storing its results.
  _.memoize = function(func, hasher) {
    var memo = {};
    hasher || (hasher = _.identity);
    return function() {
      var key = hasher.apply(this, arguments);
      return _.has(memo, key) ? memo[key] : (memo[key] = func.apply(this, arguments));
    };
  };

  // Delays a function for the given number of milliseconds, and then calls
  // it with the arguments supplied.
  _.delay = function(func, wait) {
    var args = slice.call(arguments, 2);
    return setTimeout(function(){ return func.apply(null, args); }, wait);
  };

  // Defers a function, scheduling it to run after the current call stack has
  // cleared.
  _.defer = function(func) {
    return _.delay.apply(_, [func, 1].concat(slice.call(arguments, 1)));
  };

  // Returns a function, that, when invoked, will only be triggered at most once
  // during a given window of time. Normally, the throttled function will run
  // as much as it can, without ever going more than once per `wait` duration;
  // but if you'd like to disable the execution on the leading edge, pass
  // `{leading: false}`. To disable execution on the trailing edge, ditto.
  _.throttle = function(func, wait, options) {
    var context, args, result;
    var timeout = null;
    var previous = 0;
    options || (options = {});
    var later = function() {
      previous = options.leading === false ? 0 : new Date;
      timeout = null;
      result = func.apply(context, args);
    };
    return function() {
      var now = new Date;
      if (!previous && options.leading === false) previous = now;
      var remaining = wait - (now - previous);
      context = this;
      args = arguments;
      if (remaining <= 0) {
        clearTimeout(timeout);
        timeout = null;
        previous = now;
        result = func.apply(context, args);
      } else if (!timeout && options.trailing !== false) {
        timeout = setTimeout(later, remaining);
      }
      return result;
    };
  };

  // Returns a function, that, as long as it continues to be invoked, will not
  // be triggered. The function will be called after it stops being called for
  // N milliseconds. If `immediate` is passed, trigger the function on the
  // leading edge, instead of the trailing.
  _.debounce = function(func, wait, immediate) {
    var timeout, args, context, timestamp, result;
    return function() {
      context = this;
      args = arguments;
      timestamp = new Date();
      var later = function() {
        var last = (new Date()) - timestamp;
        if (last < wait) {
          timeout = setTimeout(later, wait - last);
        } else {
          timeout = null;
          if (!immediate) result = func.apply(context, args);
        }
      };
      var callNow = immediate && !timeout;
      if (!timeout) {
        timeout = setTimeout(later, wait);
      }
      if (callNow) result = func.apply(context, args);
      return result;
    };
  };

  // Returns a function that will be executed at most one time, no matter how
  // often you call it. Useful for lazy initialization.
  _.once = function(func) {
    var ran = false, memo;
    return function() {
      if (ran) return memo;
      ran = true;
      memo = func.apply(this, arguments);
      func = null;
      return memo;
    };
  };

  // Returns the first function passed as an argument to the second,
  // allowing you to adjust arguments, run code before and after, and
  // conditionally execute the original function.
  _.wrap = function(func, wrapper) {
    return function() {
      var args = [func];
      push.apply(args, arguments);
      return wrapper.apply(this, args);
    };
  };

  // Returns a function that is the composition of a list of functions, each
  // consuming the return value of the function that follows.
  _.compose = function() {
    var funcs = arguments;
    return function() {
      var args = arguments;
      for (var i = funcs.length - 1; i >= 0; i--) {
        args = [funcs[i].apply(this, args)];
      }
      return args[0];
    };
  };

  // Returns a function that will only be executed after being called N times.
  _.after = function(times, func) {
    return function() {
      if (--times < 1) {
        return func.apply(this, arguments);
      }
    };
  };

  // Object Functions
  // ----------------

  // Retrieve the names of an object's properties.
  // Delegates to **ECMAScript 5**'s native `Object.keys`
  _.keys = nativeKeys || function(obj) {
    if (obj !== Object(obj)) throw new TypeError('Invalid object');
    var keys = [];
    for (var key in obj) if (_.has(obj, key)) keys.push(key);
    return keys;
  };

  // Retrieve the values of an object's properties.
  _.values = function(obj) {
    var keys = _.keys(obj);
    var length = keys.length;
    var values = new Array(length);
    for (var i = 0; i < length; i++) {
      values[i] = obj[keys[i]];
    }
    return values;
  };

  // Convert an object into a list of `[key, value]` pairs.
  _.pairs = function(obj) {
    var keys = _.keys(obj);
    var length = keys.length;
    var pairs = new Array(length);
    for (var i = 0; i < length; i++) {
      pairs[i] = [keys[i], obj[keys[i]]];
    }
    return pairs;
  };

  // Invert the keys and values of an object. The values must be serializable.
  _.invert = function(obj) {
    var result = {};
    var keys = _.keys(obj);
    for (var i = 0, length = keys.length; i < length; i++) {
      result[obj[keys[i]]] = keys[i];
    }
    return result;
  };

  // Return a sorted list of the function names available on the object.
  // Aliased as `methods`
  _.functions = _.methods = function(obj) {
    var names = [];
    for (var key in obj) {
      if (_.isFunction(obj[key])) names.push(key);
    }
    return names.sort();
  };

  // Extend a given object with all the properties in passed-in object(s).
  _.extend = function(obj) {
    each(slice.call(arguments, 1), function(source) {
      if (source) {
        for (var prop in source) {
          obj[prop] = source[prop];
        }
      }
    });
    return obj;
  };

  // Return a copy of the object only containing the whitelisted properties.
  _.pick = function(obj) {
    var copy = {};
    var keys = concat.apply(ArrayProto, slice.call(arguments, 1));
    each(keys, function(key) {
      if (key in obj) copy[key] = obj[key];
    });
    return copy;
  };

   // Return a copy of the object without the blacklisted properties.
  _.omit = function(obj) {
    var copy = {};
    var keys = concat.apply(ArrayProto, slice.call(arguments, 1));
    for (var key in obj) {
      if (!_.contains(keys, key)) copy[key] = obj[key];
    }
    return copy;
  };

  // Fill in a given object with default properties.
  _.defaults = function(obj) {
    each(slice.call(arguments, 1), function(source) {
      if (source) {
        for (var prop in source) {
          if (obj[prop] === void 0) obj[prop] = source[prop];
        }
      }
    });
    return obj;
  };

  // Create a (shallow-cloned) duplicate of an object.
  _.clone = function(obj) {
    if (!_.isObject(obj)) return obj;
    return _.isArray(obj) ? obj.slice() : _.extend({}, obj);
  };

  // Invokes interceptor with the obj, and then returns obj.
  // The primary purpose of this method is to "tap into" a method chain, in
  // order to perform operations on intermediate results within the chain.
  _.tap = function(obj, interceptor) {
    interceptor(obj);
    return obj;
  };

  // Internal recursive comparison function for `isEqual`.
  var eq = function(a, b, aStack, bStack) {
    // Identical objects are equal. `0 === -0`, but they aren't identical.
    // See the [Harmony `egal` proposal](http://wiki.ecmascript.org/doku.php?id=harmony:egal).
    if (a === b) return a !== 0 || 1 / a == 1 / b;
    // A strict comparison is necessary because `null == undefined`.
    if (a == null || b == null) return a === b;
    // Unwrap any wrapped objects.
    if (a instanceof _) a = a._wrapped;
    if (b instanceof _) b = b._wrapped;
    // Compare `[[Class]]` names.
    var className = toString.call(a);
    if (className != toString.call(b)) return false;
    switch (className) {
      // Strings, numbers, dates, and booleans are compared by value.
      case '[object String]':
        // Primitives and their corresponding object wrappers are equivalent; thus, `"5"` is
        // equivalent to `new String("5")`.
        return a == String(b);
      case '[object Number]':
        // `NaN`s are equivalent, but non-reflexive. An `egal` comparison is performed for
        // other numeric values.
        return a != +a ? b != +b : (a == 0 ? 1 / a == 1 / b : a == +b);
      case '[object Date]':
      case '[object Boolean]':
        // Coerce dates and booleans to numeric primitive values. Dates are compared by their
        // millisecond representations. Note that invalid dates with millisecond representations
        // of `NaN` are not equivalent.
        return +a == +b;
      // RegExps are compared by their source patterns and flags.
      case '[object RegExp]':
        return a.source == b.source &&
               a.global == b.global &&
               a.multiline == b.multiline &&
               a.ignoreCase == b.ignoreCase;
    }
    if (typeof a != 'object' || typeof b != 'object') return false;
    // Assume equality for cyclic structures. The algorithm for detecting cyclic
    // structures is adapted from ES 5.1 section 15.12.3, abstract operation `JO`.
    var length = aStack.length;
    while (length--) {
      // Linear search. Performance is inversely proportional to the number of
      // unique nested structures.
      if (aStack[length] == a) return bStack[length] == b;
    }
    // Objects with different constructors are not equivalent, but `Object`s
    // from different frames are.
    var aCtor = a.constructor, bCtor = b.constructor;
    if (aCtor !== bCtor && !(_.isFunction(aCtor) && (aCtor instanceof aCtor) &&
                             _.isFunction(bCtor) && (bCtor instanceof bCtor))) {
      return false;
    }
    // Add the first object to the stack of traversed objects.
    aStack.push(a);
    bStack.push(b);
    var size = 0, result = true;
    // Recursively compare objects and arrays.
    if (className == '[object Array]') {
      // Compare array lengths to determine if a deep comparison is necessary.
      size = a.length;
      result = size == b.length;
      if (result) {
        // Deep compare the contents, ignoring non-numeric properties.
        while (size--) {
          if (!(result = eq(a[size], b[size], aStack, bStack))) break;
        }
      }
    } else {
      // Deep compare objects.
      for (var key in a) {
        if (_.has(a, key)) {
          // Count the expected number of properties.
          size++;
          // Deep compare each member.
          if (!(result = _.has(b, key) && eq(a[key], b[key], aStack, bStack))) break;
        }
      }
      // Ensure that both objects contain the same number of properties.
      if (result) {
        for (key in b) {
          if (_.has(b, key) && !(size--)) break;
        }
        result = !size;
      }
    }
    // Remove the first object from the stack of traversed objects.
    aStack.pop();
    bStack.pop();
    return result;
  };

  // Perform a deep comparison to check if two objects are equal.
  _.isEqual = function(a, b) {
    return eq(a, b, [], []);
  };

  // Is a given array, string, or object empty?
  // An "empty" object has no enumerable own-properties.
  _.isEmpty = function(obj) {
    if (obj == null) return true;
    if (_.isArray(obj) || _.isString(obj)) return obj.length === 0;
    for (var key in obj) if (_.has(obj, key)) return false;
    return true;
  };

  // Is a given value a DOM element?
  _.isElement = function(obj) {
    return !!(obj && obj.nodeType === 1);
  };

  // Is a given value an array?
  // Delegates to ECMA5's native Array.isArray
  _.isArray = nativeIsArray || function(obj) {
    return toString.call(obj) == '[object Array]';
  };

  // Is a given variable an object?
  _.isObject = function(obj) {
    return obj === Object(obj);
  };

  // Add some isType methods: isArguments, isFunction, isString, isNumber, isDate, isRegExp.
  each(['Arguments', 'Function', 'String', 'Number', 'Date', 'RegExp'], function(name) {
    _['is' + name] = function(obj) {
      return toString.call(obj) == '[object ' + name + ']';
    };
  });

  // Define a fallback version of the method in browsers (ahem, IE), where
  // there isn't any inspectable "Arguments" type.
  if (!_.isArguments(arguments)) {
    _.isArguments = function(obj) {
      return !!(obj && _.has(obj, 'callee'));
    };
  }

  // Optimize `isFunction` if appropriate.
  if (typeof (/./) !== 'function') {
    _.isFunction = function(obj) {
      return typeof obj === 'function';
    };
  }

  // Is a given object a finite number?
  _.isFinite = function(obj) {
    return isFinite(obj) && !isNaN(parseFloat(obj));
  };

  // Is the given value `NaN`? (NaN is the only number which does not equal itself).
  _.isNaN = function(obj) {
    return _.isNumber(obj) && obj != +obj;
  };

  // Is a given value a boolean?
  _.isBoolean = function(obj) {
    return obj === true || obj === false || toString.call(obj) == '[object Boolean]';
  };

  // Is a given value equal to null?
  _.isNull = function(obj) {
    return obj === null;
  };

  // Is a given variable undefined?
  _.isUndefined = function(obj) {
    return obj === void 0;
  };

  // Shortcut function for checking if an object has a given property directly
  // on itself (in other words, not on a prototype).
  _.has = function(obj, key) {
    return hasOwnProperty.call(obj, key);
  };

  // Utility Functions
  // -----------------

  // Run Underscore.js in *noConflict* mode, returning the `_` variable to its
  // previous owner. Returns a reference to the Underscore object.
  _.noConflict = function() {
    root._ = previousUnderscore;
    return this;
  };

  // Keep the identity function around for default iterators.
  _.identity = function(value) {
    return value;
  };

  // Run a function **n** times.
  _.times = function(n, iterator, context) {
    var accum = Array(Math.max(0, n));
    for (var i = 0; i < n; i++) accum[i] = iterator.call(context, i);
    return accum;
  };

  // Return a random integer between min and max (inclusive).
  _.random = function(min, max) {
    if (max == null) {
      max = min;
      min = 0;
    }
    return min + Math.floor(Math.random() * (max - min + 1));
  };

  // List of HTML entities for escaping.
  var entityMap = {
    escape: {
      '&': '&amp;',
      '<': '&lt;',
      '>': '&gt;',
      '"': '&quot;',
      "'": '&#x27;'
    }
  };
  entityMap.unescape = _.invert(entityMap.escape);

  // Regexes containing the keys and values listed immediately above.
  var entityRegexes = {
    escape:   new RegExp('[' + _.keys(entityMap.escape).join('') + ']', 'g'),
    unescape: new RegExp('(' + _.keys(entityMap.unescape).join('|') + ')', 'g')
  };

  // Functions for escaping and unescaping strings to/from HTML interpolation.
  _.each(['escape', 'unescape'], function(method) {
    _[method] = function(string) {
      if (string == null) return '';
      return ('' + string).replace(entityRegexes[method], function(match) {
        return entityMap[method][match];
      });
    };
  });

  // If the value of the named `property` is a function then invoke it with the
  // `object` as context; otherwise, return it.
  _.result = function(object, property) {
    if (object == null) return void 0;
    var value = object[property];
    return _.isFunction(value) ? value.call(object) : value;
  };

  // Add your own custom functions to the Underscore object.
  _.mixin = function(obj) {
    each(_.functions(obj), function(name) {
      var func = _[name] = obj[name];
      _.prototype[name] = function() {
        var args = [this._wrapped];
        push.apply(args, arguments);
        return result.call(this, func.apply(_, args));
      };
    });
  };

  // Generate a unique integer id (unique within the entire client session).
  // Useful for temporary DOM ids.
  var idCounter = 0;
  _.uniqueId = function(prefix) {
    var id = ++idCounter + '';
    return prefix ? prefix + id : id;
  };

  // By default, Underscore uses ERB-style template delimiters, change the
  // following template settings to use alternative delimiters.
  _.templateSettings = {
    evaluate    : /<%([\s\S]+?)%>/g,
    interpolate : /<%=([\s\S]+?)%>/g,
    escape      : /<%-([\s\S]+?)%>/g
  };

  // When customizing `templateSettings`, if you don't want to define an
  // interpolation, evaluation or escaping regex, we need one that is
  // guaranteed not to match.
  var noMatch = /(.)^/;

  // Certain characters need to be escaped so that they can be put into a
  // string literal.
  var escapes = {
    "'":      "'",
    '\\':     '\\',
    '\r':     'r',
    '\n':     'n',
    '\t':     't',
    '\u2028': 'u2028',
    '\u2029': 'u2029'
  };

  var escaper = /\\|'|\r|\n|\t|\u2028|\u2029/g;

  // JavaScript micro-templating, similar to John Resig's implementation.
  // Underscore templating handles arbitrary delimiters, preserves whitespace,
  // and correctly escapes quotes within interpolated code.
  _.template = function(text, data, settings) {
    var render;
    settings = _.defaults({}, settings, _.templateSettings);

    // Combine delimiters into one regular expression via alternation.
    var matcher = new RegExp([
      (settings.escape || noMatch).source,
      (settings.interpolate || noMatch).source,
      (settings.evaluate || noMatch).source
    ].join('|') + '|$', 'g');

    // Compile the template source, escaping string literals appropriately.
    var index = 0;
    var source = "__p+='";
    text.replace(matcher, function(match, escape, interpolate, evaluate, offset) {
      source += text.slice(index, offset)
        .replace(escaper, function(match) { return '\\' + escapes[match]; });

      if (escape) {
        source += "'+\n((__t=(" + escape + "))==null?'':_.escape(__t))+\n'";
      }
      if (interpolate) {
        source += "'+\n((__t=(" + interpolate + "))==null?'':__t)+\n'";
      }
      if (evaluate) {
        source += "';\n" + evaluate + "\n__p+='";
      }
      index = offset + match.length;
      return match;
    });
    source += "';\n";

    // If a variable is not specified, place data values in local scope.
    if (!settings.variable) source = 'with(obj||{}){\n' + source + '}\n';

    source = "var __t,__p='',__j=Array.prototype.join," +
      "print=function(){__p+=__j.call(arguments,'');};\n" +
      source + "return __p;\n";

    try {
      render = new Function(settings.variable || 'obj', '_', source);
    } catch (e) {
      e.source = source;
      throw e;
    }

    if (data) return render(data, _);
    var template = function(data) {
      return render.call(this, data, _);
    };

    // Provide the compiled function source as a convenience for precompilation.
    template.source = 'function(' + (settings.variable || 'obj') + '){\n' + source + '}';

    return template;
  };

  // Add a "chain" function, which will delegate to the wrapper.
  _.chain = function(obj) {
    return _(obj).chain();
  };

  // OOP
  // ---------------
  // If Underscore is called as a function, it returns a wrapped object that
  // can be used OO-style. This wrapper holds altered versions of all the
  // underscore functions. Wrapped objects may be chained.

  // Helper function to continue chaining intermediate results.
  var result = function(obj) {
    return this._chain ? _(obj).chain() : obj;
  };

  // Add all of the Underscore functions to the wrapper object.
  _.mixin(_);

  // Add all mutator Array functions to the wrapper.
  each(['pop', 'push', 'reverse', 'shift', 'sort', 'splice', 'unshift'], function(name) {
    var method = ArrayProto[name];
    _.prototype[name] = function() {
      var obj = this._wrapped;
      method.apply(obj, arguments);
      if ((name == 'shift' || name == 'splice') && obj.length === 0) delete obj[0];
      return result.call(this, obj);
    };
  });

  // Add all accessor Array functions to the wrapper.
  each(['concat', 'join', 'slice'], function(name) {
    var method = ArrayProto[name];
    _.prototype[name] = function() {
      return result.call(this, method.apply(this._wrapped, arguments));
    };
  });

  _.extend(_.prototype, {

    // Start chaining a wrapped Underscore object.
    chain: function() {
      this._chain = true;
      return this;
    },

    // Extracts the result from a wrapped and chained object.
    value: function() {
      return this._wrapped;
    }

  });

}).call(this);

define("underscore", (function (global) {
    return function () {
        var ret, fn;
        return ret || global._;
    };
}(this)));

//     Backbone.js 1.1.2

//     (c) 2010-2014 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
//     Backbone may be freely distributed under the MIT license.
//     For all details and documentation:
//     http://backbonejs.org

(function(root, factory) {

  // Set up Backbone appropriately for the environment. Start with AMD.
  if (typeof define === 'function' && define.amd) {
    define('backbone',['underscore', 'jquery', 'exports'], function(_, $, exports) {
      // Export global even in AMD case in case this script is loaded with
      // others that may still expect a global Backbone.
      root.Backbone = factory(root, exports, _, $);
    });

  // Next for Node.js or CommonJS. jQuery may not be needed as a module.
  } else if (typeof exports !== 'undefined') {
    var _ = require('underscore');
    factory(root, exports, _);

  // Finally, as a browser global.
  } else {
    root.Backbone = factory(root, {}, root._, (root.jQuery || root.Zepto || root.ender || root.$));
  }

}(this, function(root, Backbone, _, $) {

  // Initial Setup
  // -------------

  // Save the previous value of the `Backbone` variable, so that it can be
  // restored later on, if `noConflict` is used.
  var previousBackbone = root.Backbone;

  // Create local references to array methods we'll want to use later.
  var array = [];
  var push = array.push;
  var slice = array.slice;
  var splice = array.splice;

  // Current version of the library. Keep in sync with `package.json`.
  Backbone.VERSION = '1.1.2';

  // For Backbone's purposes, jQuery, Zepto, Ender, or My Library (kidding) owns
  // the `$` variable.
  Backbone.$ = $;

  // Runs Backbone.js in *noConflict* mode, returning the `Backbone` variable
  // to its previous owner. Returns a reference to this Backbone object.
  Backbone.noConflict = function() {
    root.Backbone = previousBackbone;
    return this;
  };

  // Turn on `emulateHTTP` to support legacy HTTP servers. Setting this option
  // will fake `"PATCH"`, `"PUT"` and `"DELETE"` requests via the `_method` parameter and
  // set a `X-Http-Method-Override` header.
  Backbone.emulateHTTP = false;

  // Turn on `emulateJSON` to support legacy servers that can't deal with direct
  // `application/json` requests ... will encode the body as
  // `application/x-www-form-urlencoded` instead and will send the model in a
  // form param named `model`.
  Backbone.emulateJSON = false;

  // Backbone.Events
  // ---------------

  // A module that can be mixed in to *any object* in order to provide it with
  // custom events. You may bind with `on` or remove with `off` callback
  // functions to an event; `trigger`-ing an event fires all callbacks in
  // succession.
  //
  //     var object = {};
  //     _.extend(object, Backbone.Events);
  //     object.on('expand', function(){ alert('expanded'); });
  //     object.trigger('expand');
  //
  var Events = Backbone.Events = {

    // Bind an event to a `callback` function. Passing `"all"` will bind
    // the callback to all events fired.
    on: function(name, callback, context) {
      if (!eventsApi(this, 'on', name, [callback, context]) || !callback) return this;
      this._events || (this._events = {});
      var events = this._events[name] || (this._events[name] = []);
      events.push({callback: callback, context: context, ctx: context || this});
      return this;
    },

    // Bind an event to only be triggered a single time. After the first time
    // the callback is invoked, it will be removed.
    once: function(name, callback, context) {
      if (!eventsApi(this, 'once', name, [callback, context]) || !callback) return this;
      var self = this;
      var once = _.once(function() {
        self.off(name, once);
        callback.apply(this, arguments);
      });
      once._callback = callback;
      return this.on(name, once, context);
    },

    // Remove one or many callbacks. If `context` is null, removes all
    // callbacks with that function. If `callback` is null, removes all
    // callbacks for the event. If `name` is null, removes all bound
    // callbacks for all events.
    off: function(name, callback, context) {
      var retain, ev, events, names, i, l, j, k;
      if (!this._events || !eventsApi(this, 'off', name, [callback, context])) return this;
      if (!name && !callback && !context) {
        this._events = void 0;
        return this;
      }
      names = name ? [name] : _.keys(this._events);
      for (i = 0, l = names.length; i < l; i++) {
        name = names[i];
        if (events = this._events[name]) {
          this._events[name] = retain = [];
          if (callback || context) {
            for (j = 0, k = events.length; j < k; j++) {
              ev = events[j];
              if ((callback && callback !== ev.callback && callback !== ev.callback._callback) ||
                  (context && context !== ev.context)) {
                retain.push(ev);
              }
            }
          }
          if (!retain.length) delete this._events[name];
        }
      }

      return this;
    },

    // Trigger one or many events, firing all bound callbacks. Callbacks are
    // passed the same arguments as `trigger` is, apart from the event name
    // (unless you're listening on `"all"`, which will cause your callback to
    // receive the true name of the event as the first argument).
    trigger: function(name) {
      if (!this._events) return this;
      var args = slice.call(arguments, 1);
      if (!eventsApi(this, 'trigger', name, args)) return this;
      var events = this._events[name];
      var allEvents = this._events.all;
      if (events) triggerEvents(events, args);
      if (allEvents) triggerEvents(allEvents, arguments);
      return this;
    },

    // Tell this object to stop listening to either specific events ... or
    // to every object it's currently listening to.
    stopListening: function(obj, name, callback) {
      var listeningTo = this._listeningTo;
      if (!listeningTo) return this;
      var remove = !name && !callback;
      if (!callback && typeof name === 'object') callback = this;
      if (obj) (listeningTo = {})[obj._listenId] = obj;
      for (var id in listeningTo) {
        obj = listeningTo[id];
        obj.off(name, callback, this);
        if (remove || _.isEmpty(obj._events)) delete this._listeningTo[id];
      }
      return this;
    }

  };

  // Regular expression used to split event strings.
  var eventSplitter = /\s+/;

  // Implement fancy features of the Events API such as multiple event
  // names `"change blur"` and jQuery-style event maps `{change: action}`
  // in terms of the existing API.
  var eventsApi = function(obj, action, name, rest) {
    if (!name) return true;

    // Handle event maps.
    if (typeof name === 'object') {
      for (var key in name) {
        obj[action].apply(obj, [key, name[key]].concat(rest));
      }
      return false;
    }

    // Handle space separated event names.
    if (eventSplitter.test(name)) {
      var names = name.split(eventSplitter);
      for (var i = 0, l = names.length; i < l; i++) {
        obj[action].apply(obj, [names[i]].concat(rest));
      }
      return false;
    }

    return true;
  };

  // A difficult-to-believe, but optimized internal dispatch function for
  // triggering events. Tries to keep the usual cases speedy (most internal
  // Backbone events have 3 arguments).
  var triggerEvents = function(events, args) {
    var ev, i = -1, l = events.length, a1 = args[0], a2 = args[1], a3 = args[2];
    switch (args.length) {
      case 0: while (++i < l) (ev = events[i]).callback.call(ev.ctx); return;
      case 1: while (++i < l) (ev = events[i]).callback.call(ev.ctx, a1); return;
      case 2: while (++i < l) (ev = events[i]).callback.call(ev.ctx, a1, a2); return;
      case 3: while (++i < l) (ev = events[i]).callback.call(ev.ctx, a1, a2, a3); return;
      default: while (++i < l) (ev = events[i]).callback.apply(ev.ctx, args); return;
    }
  };

  var listenMethods = {listenTo: 'on', listenToOnce: 'once'};

  // Inversion-of-control versions of `on` and `once`. Tell *this* object to
  // listen to an event in another object ... keeping track of what it's
  // listening to.
  _.each(listenMethods, function(implementation, method) {
    Events[method] = function(obj, name, callback) {
      var listeningTo = this._listeningTo || (this._listeningTo = {});
      var id = obj._listenId || (obj._listenId = _.uniqueId('l'));
      listeningTo[id] = obj;
      if (!callback && typeof name === 'object') callback = this;
      obj[implementation](name, callback, this);
      return this;
    };
  });

  // Aliases for backwards compatibility.
  Events.bind   = Events.on;
  Events.unbind = Events.off;

  // Allow the `Backbone` object to serve as a global event bus, for folks who
  // want global "pubsub" in a convenient place.
  _.extend(Backbone, Events);

  // Backbone.Model
  // --------------

  // Backbone **Models** are the basic data object in the framework --
  // frequently representing a row in a table in a database on your server.
  // A discrete chunk of data and a bunch of useful, related methods for
  // performing computations and transformations on that data.

  // Create a new model with the specified attributes. A client id (`cid`)
  // is automatically generated and assigned for you.
  var Model = Backbone.Model = function(attributes, options) {
    var attrs = attributes || {};
    options || (options = {});
    this.cid = _.uniqueId('c');
    this.attributes = {};
    if (options.collection) this.collection = options.collection;
    if (options.parse) attrs = this.parse(attrs, options) || {};
    attrs = _.defaults({}, attrs, _.result(this, 'defaults'));
    this.set(attrs, options);
    this.changed = {};
    this.initialize.apply(this, arguments);
  };

  // Attach all inheritable methods to the Model prototype.
  _.extend(Model.prototype, Events, {

    // A hash of attributes whose current and previous value differ.
    changed: null,

    // The value returned during the last failed validation.
    validationError: null,

    // The default name for the JSON `id` attribute is `"id"`. MongoDB and
    // CouchDB users may want to set this to `"_id"`.
    idAttribute: 'id',

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // Return a copy of the model's `attributes` object.
    toJSON: function(options) {
      return _.clone(this.attributes);
    },

    // Proxy `Backbone.sync` by default -- but override this if you need
    // custom syncing semantics for *this* particular model.
    sync: function() {
      return Backbone.sync.apply(this, arguments);
    },

    // Get the value of an attribute.
    get: function(attr) {
      return this.attributes[attr];
    },

    // Get the HTML-escaped value of an attribute.
    escape: function(attr) {
      return _.escape(this.get(attr));
    },

    // Returns `true` if the attribute contains a value that is not null
    // or undefined.
    has: function(attr) {
      return this.get(attr) != null;
    },

    // Set a hash of model attributes on the object, firing `"change"`. This is
    // the core primitive operation of a model, updating the data and notifying
    // anyone who needs to know about the change in state. The heart of the beast.
    set: function(key, val, options) {
      var attr, attrs, unset, changes, silent, changing, prev, current;
      if (key == null) return this;

      // Handle both `"key", value` and `{key: value}` -style arguments.
      if (typeof key === 'object') {
        attrs = key;
        options = val;
      } else {
        (attrs = {})[key] = val;
      }

      options || (options = {});

      // Run validation.
      if (!this._validate(attrs, options)) return false;

      // Extract attributes and options.
      unset           = options.unset;
      silent          = options.silent;
      changes         = [];
      changing        = this._changing;
      this._changing  = true;

      if (!changing) {
        this._previousAttributes = _.clone(this.attributes);
        this.changed = {};
      }
      current = this.attributes, prev = this._previousAttributes;

      // Check for changes of `id`.
      if (this.idAttribute in attrs) this.id = attrs[this.idAttribute];

      // For each `set` attribute, update or delete the current value.
      for (attr in attrs) {
        val = attrs[attr];
        if (!_.isEqual(current[attr], val)) changes.push(attr);
        if (!_.isEqual(prev[attr], val)) {
          this.changed[attr] = val;
        } else {
          delete this.changed[attr];
        }
        unset ? delete current[attr] : current[attr] = val;
      }

      // Trigger all relevant attribute changes.
      if (!silent) {
        if (changes.length) this._pending = options;
        for (var i = 0, l = changes.length; i < l; i++) {
          this.trigger('change:' + changes[i], this, current[changes[i]], options);
        }
      }

      // You might be wondering why there's a `while` loop here. Changes can
      // be recursively nested within `"change"` events.
      if (changing) return this;
      if (!silent) {
        while (this._pending) {
          options = this._pending;
          this._pending = false;
          this.trigger('change', this, options);
        }
      }
      this._pending = false;
      this._changing = false;
      return this;
    },

    // Remove an attribute from the model, firing `"change"`. `unset` is a noop
    // if the attribute doesn't exist.
    unset: function(attr, options) {
      return this.set(attr, void 0, _.extend({}, options, {unset: true}));
    },

    // Clear all attributes on the model, firing `"change"`.
    clear: function(options) {
      var attrs = {};
      for (var key in this.attributes) attrs[key] = void 0;
      return this.set(attrs, _.extend({}, options, {unset: true}));
    },

    // Determine if the model has changed since the last `"change"` event.
    // If you specify an attribute name, determine if that attribute has changed.
    hasChanged: function(attr) {
      if (attr == null) return !_.isEmpty(this.changed);
      return _.has(this.changed, attr);
    },

    // Return an object containing all the attributes that have changed, or
    // false if there are no changed attributes. Useful for determining what
    // parts of a view need to be updated and/or what attributes need to be
    // persisted to the server. Unset attributes will be set to undefined.
    // You can also pass an attributes object to diff against the model,
    // determining if there *would be* a change.
    changedAttributes: function(diff) {
      if (!diff) return this.hasChanged() ? _.clone(this.changed) : false;
      var val, changed = false;
      var old = this._changing ? this._previousAttributes : this.attributes;
      for (var attr in diff) {
        if (_.isEqual(old[attr], (val = diff[attr]))) continue;
        (changed || (changed = {}))[attr] = val;
      }
      return changed;
    },

    // Get the previous value of an attribute, recorded at the time the last
    // `"change"` event was fired.
    previous: function(attr) {
      if (attr == null || !this._previousAttributes) return null;
      return this._previousAttributes[attr];
    },

    // Get all of the attributes of the model at the time of the previous
    // `"change"` event.
    previousAttributes: function() {
      return _.clone(this._previousAttributes);
    },

    // Fetch the model from the server. If the server's representation of the
    // model differs from its current attributes, they will be overridden,
    // triggering a `"change"` event.
    fetch: function(options) {
      options = options ? _.clone(options) : {};
      if (options.parse === void 0) options.parse = true;
      var model = this;
      var success = options.success;
      options.success = function(resp) {
        if (!model.set(model.parse(resp, options), options)) return false;
        if (success) success(model, resp, options);
        model.trigger('sync', model, resp, options);
      };
      wrapError(this, options);
      return this.sync('read', this, options);
    },

    // Set a hash of model attributes, and sync the model to the server.
    // If the server returns an attributes hash that differs, the model's
    // state will be `set` again.
    save: function(key, val, options) {
      var attrs, method, xhr, attributes = this.attributes;

      // Handle both `"key", value` and `{key: value}` -style arguments.
      if (key == null || typeof key === 'object') {
        attrs = key;
        options = val;
      } else {
        (attrs = {})[key] = val;
      }

      options = _.extend({validate: true}, options);

      // If we're not waiting and attributes exist, save acts as
      // `set(attr).save(null, opts)` with validation. Otherwise, check if
      // the model will be valid when the attributes, if any, are set.
      if (attrs && !options.wait) {
        if (!this.set(attrs, options)) return false;
      } else {
        if (!this._validate(attrs, options)) return false;
      }

      // Set temporary attributes if `{wait: true}`.
      if (attrs && options.wait) {
        this.attributes = _.extend({}, attributes, attrs);
      }

      // After a successful server-side save, the client is (optionally)
      // updated with the server-side state.
      if (options.parse === void 0) options.parse = true;
      var model = this;
      var success = options.success;
      options.success = function(resp) {
        // Ensure attributes are restored during synchronous saves.
        model.attributes = attributes;
        var serverAttrs = model.parse(resp, options);
        if (options.wait) serverAttrs = _.extend(attrs || {}, serverAttrs);
        if (_.isObject(serverAttrs) && !model.set(serverAttrs, options)) {
          return false;
        }
        if (success) success(model, resp, options);
        model.trigger('sync', model, resp, options);
      };
      wrapError(this, options);

      method = this.isNew() ? 'create' : (options.patch ? 'patch' : 'update');
      if (method === 'patch') options.attrs = attrs;
      xhr = this.sync(method, this, options);

      // Restore attributes.
      if (attrs && options.wait) this.attributes = attributes;

      return xhr;
    },

    // Destroy this model on the server if it was already persisted.
    // Optimistically removes the model from its collection, if it has one.
    // If `wait: true` is passed, waits for the server to respond before removal.
    destroy: function(options) {
      options = options ? _.clone(options) : {};
      var model = this;
      var success = options.success;

      var destroy = function() {
        model.trigger('destroy', model, model.collection, options);
      };

      options.success = function(resp) {
        if (options.wait || model.isNew()) destroy();
        if (success) success(model, resp, options);
        if (!model.isNew()) model.trigger('sync', model, resp, options);
      };

      if (this.isNew()) {
        options.success();
        return false;
      }
      wrapError(this, options);

      var xhr = this.sync('delete', this, options);
      if (!options.wait) destroy();
      return xhr;
    },

    // Default URL for the model's representation on the server -- if you're
    // using Backbone's restful methods, override this to change the endpoint
    // that will be called.
    url: function() {
      var base =
        _.result(this, 'urlRoot') ||
        _.result(this.collection, 'url') ||
        urlError();
      if (this.isNew()) return base;
      return base.replace(/([^\/])$/, '$1/') + encodeURIComponent(this.id);
    },

    // **parse** converts a response into the hash of attributes to be `set` on
    // the model. The default implementation is just to pass the response along.
    parse: function(resp, options) {
      return resp;
    },

    // Create a new model with identical attributes to this one.
    clone: function() {
      return new this.constructor(this.attributes);
    },

    // A model is new if it has never been saved to the server, and lacks an id.
    isNew: function() {
      return !this.has(this.idAttribute);
    },

    // Check if the model is currently in a valid state.
    isValid: function(options) {
      return this._validate({}, _.extend(options || {}, { validate: true }));
    },

    // Run validation against the next complete set of model attributes,
    // returning `true` if all is well. Otherwise, fire an `"invalid"` event.
    _validate: function(attrs, options) {
      if (!options.validate || !this.validate) return true;
      attrs = _.extend({}, this.attributes, attrs);
      var error = this.validationError = this.validate(attrs, options) || null;
      if (!error) return true;
      this.trigger('invalid', this, error, _.extend(options, {validationError: error}));
      return false;
    }

  });

  // Underscore methods that we want to implement on the Model.
  var modelMethods = ['keys', 'values', 'pairs', 'invert', 'pick', 'omit'];

  // Mix in each Underscore method as a proxy to `Model#attributes`.
  _.each(modelMethods, function(method) {
    Model.prototype[method] = function() {
      var args = slice.call(arguments);
      args.unshift(this.attributes);
      return _[method].apply(_, args);
    };
  });

  // Backbone.Collection
  // -------------------

  // If models tend to represent a single row of data, a Backbone Collection is
  // more analagous to a table full of data ... or a small slice or page of that
  // table, or a collection of rows that belong together for a particular reason
  // -- all of the messages in this particular folder, all of the documents
  // belonging to this particular author, and so on. Collections maintain
  // indexes of their models, both in order, and for lookup by `id`.

  // Create a new **Collection**, perhaps to contain a specific type of `model`.
  // If a `comparator` is specified, the Collection will maintain
  // its models in sort order, as they're added and removed.
  var Collection = Backbone.Collection = function(models, options) {
    options || (options = {});
    if (options.model) this.model = options.model;
    if (options.comparator !== void 0) this.comparator = options.comparator;
    this._reset();
    this.initialize.apply(this, arguments);
    if (models) this.reset(models, _.extend({silent: true}, options));
  };

  // Default options for `Collection#set`.
  var setOptions = {add: true, remove: true, merge: true};
  var addOptions = {add: true, remove: false};

  // Define the Collection's inheritable methods.
  _.extend(Collection.prototype, Events, {

    // The default model for a collection is just a **Backbone.Model**.
    // This should be overridden in most cases.
    model: Model,

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // The JSON representation of a Collection is an array of the
    // models' attributes.
    toJSON: function(options) {
      return this.map(function(model){ return model.toJSON(options); });
    },

    // Proxy `Backbone.sync` by default.
    sync: function() {
      return Backbone.sync.apply(this, arguments);
    },

    // Add a model, or list of models to the set.
    add: function(models, options) {
      return this.set(models, _.extend({merge: false}, options, addOptions));
    },

    // Remove a model, or a list of models from the set.
    remove: function(models, options) {
      var singular = !_.isArray(models);
      models = singular ? [models] : _.clone(models);
      options || (options = {});
      var i, l, index, model;
      for (i = 0, l = models.length; i < l; i++) {
        model = models[i] = this.get(models[i]);
        if (!model) continue;
        delete this._byId[model.id];
        delete this._byId[model.cid];
        index = this.indexOf(model);
        this.models.splice(index, 1);
        this.length--;
        if (!options.silent) {
          options.index = index;
          model.trigger('remove', model, this, options);
        }
        this._removeReference(model, options);
      }
      return singular ? models[0] : models;
    },

    // Update a collection by `set`-ing a new list of models, adding new ones,
    // removing models that are no longer present, and merging models that
    // already exist in the collection, as necessary. Similar to **Model#set**,
    // the core operation for updating the data contained by the collection.
    set: function(models, options) {
      options = _.defaults({}, options, setOptions);
      if (options.parse) models = this.parse(models, options);
      var singular = !_.isArray(models);
      models = singular ? (models ? [models] : []) : _.clone(models);
      var i, l, id, model, attrs, existing, sort;
      var at = options.at;
      var targetModel = this.model;
      var sortable = this.comparator && (at == null) && options.sort !== false;
      var sortAttr = _.isString(this.comparator) ? this.comparator : null;
      var toAdd = [], toRemove = [], modelMap = {};
      var add = options.add, merge = options.merge, remove = options.remove;
      var order = !sortable && add && remove ? [] : false;

      // Turn bare objects into model references, and prevent invalid models
      // from being added.
      for (i = 0, l = models.length; i < l; i++) {
        attrs = models[i] || {};
        if (attrs instanceof Model) {
          id = model = attrs;
        } else {
          id = attrs[targetModel.prototype.idAttribute || 'id'];
        }

        // If a duplicate is found, prevent it from being added and
        // optionally merge it into the existing model.
        if (existing = this.get(id)) {
          if (remove) modelMap[existing.cid] = true;
          if (merge) {
            attrs = attrs === model ? model.attributes : attrs;
            if (options.parse) attrs = existing.parse(attrs, options);
            existing.set(attrs, options);
            if (sortable && !sort && existing.hasChanged(sortAttr)) sort = true;
          }
          models[i] = existing;

        // If this is a new, valid model, push it to the `toAdd` list.
        } else if (add) {
          model = models[i] = this._prepareModel(attrs, options);
          if (!model) continue;
          toAdd.push(model);
          this._addReference(model, options);
        }

        // Do not add multiple models with the same `id`.
        model = existing || model;
        if (order && (model.isNew() || !modelMap[model.id])) order.push(model);
        modelMap[model.id] = true;
      }

      // Remove nonexistent models if appropriate.
      if (remove) {
        for (i = 0, l = this.length; i < l; ++i) {
          if (!modelMap[(model = this.models[i]).cid]) toRemove.push(model);
        }
        if (toRemove.length) this.remove(toRemove, options);
      }

      // See if sorting is needed, update `length` and splice in new models.
      if (toAdd.length || (order && order.length)) {
        if (sortable) sort = true;
        this.length += toAdd.length;
        if (at != null) {
          for (i = 0, l = toAdd.length; i < l; i++) {
            this.models.splice(at + i, 0, toAdd[i]);
          }
        } else {
          if (order) this.models.length = 0;
          var orderedModels = order || toAdd;
          for (i = 0, l = orderedModels.length; i < l; i++) {
            this.models.push(orderedModels[i]);
          }
        }
      }

      // Silently sort the collection if appropriate.
      if (sort) this.sort({silent: true});

      // Unless silenced, it's time to fire all appropriate add/sort events.
      if (!options.silent) {
        for (i = 0, l = toAdd.length; i < l; i++) {
          (model = toAdd[i]).trigger('add', model, this, options);
        }
        if (sort || (order && order.length)) this.trigger('sort', this, options);
      }

      // Return the added (or merged) model (or models).
      return singular ? models[0] : models;
    },

    // When you have more items than you want to add or remove individually,
    // you can reset the entire set with a new list of models, without firing
    // any granular `add` or `remove` events. Fires `reset` when finished.
    // Useful for bulk operations and optimizations.
    reset: function(models, options) {
      options || (options = {});
      for (var i = 0, l = this.models.length; i < l; i++) {
        this._removeReference(this.models[i], options);
      }
      options.previousModels = this.models;
      this._reset();
      models = this.add(models, _.extend({silent: true}, options));
      if (!options.silent) this.trigger('reset', this, options);
      return models;
    },

    // Add a model to the end of the collection.
    push: function(model, options) {
      return this.add(model, _.extend({at: this.length}, options));
    },

    // Remove a model from the end of the collection.
    pop: function(options) {
      var model = this.at(this.length - 1);
      this.remove(model, options);
      return model;
    },

    // Add a model to the beginning of the collection.
    unshift: function(model, options) {
      return this.add(model, _.extend({at: 0}, options));
    },

    // Remove a model from the beginning of the collection.
    shift: function(options) {
      var model = this.at(0);
      this.remove(model, options);
      return model;
    },

    // Slice out a sub-array of models from the collection.
    slice: function() {
      return slice.apply(this.models, arguments);
    },

    // Get a model from the set by id.
    get: function(obj) {
      if (obj == null) return void 0;
      return this._byId[obj] || this._byId[obj.id] || this._byId[obj.cid];
    },

    // Get the model at the given index.
    at: function(index) {
      return this.models[index];
    },

    // Return models with matching attributes. Useful for simple cases of
    // `filter`.
    where: function(attrs, first) {
      if (_.isEmpty(attrs)) return first ? void 0 : [];
      return this[first ? 'find' : 'filter'](function(model) {
        for (var key in attrs) {
          if (attrs[key] !== model.get(key)) return false;
        }
        return true;
      });
    },

    // Return the first model with matching attributes. Useful for simple cases
    // of `find`.
    findWhere: function(attrs) {
      return this.where(attrs, true);
    },

    // Force the collection to re-sort itself. You don't need to call this under
    // normal circumstances, as the set will maintain sort order as each item
    // is added.
    sort: function(options) {
      if (!this.comparator) throw new Error('Cannot sort a set without a comparator');
      options || (options = {});

      // Run sort based on type of `comparator`.
      if (_.isString(this.comparator) || this.comparator.length === 1) {
        this.models = this.sortBy(this.comparator, this);
      } else {
        this.models.sort(_.bind(this.comparator, this));
      }

      if (!options.silent) this.trigger('sort', this, options);
      return this;
    },

    // Pluck an attribute from each model in the collection.
    pluck: function(attr) {
      return _.invoke(this.models, 'get', attr);
    },

    // Fetch the default set of models for this collection, resetting the
    // collection when they arrive. If `reset: true` is passed, the response
    // data will be passed through the `reset` method instead of `set`.
    fetch: function(options) {
      options = options ? _.clone(options) : {};
      if (options.parse === void 0) options.parse = true;
      var success = options.success;
      var collection = this;
      options.success = function(resp) {
        var method = options.reset ? 'reset' : 'set';
        collection[method](resp, options);
        if (success) success(collection, resp, options);
        collection.trigger('sync', collection, resp, options);
      };
      wrapError(this, options);
      return this.sync('read', this, options);
    },

    // Create a new instance of a model in this collection. Add the model to the
    // collection immediately, unless `wait: true` is passed, in which case we
    // wait for the server to agree.
    create: function(model, options) {
      options = options ? _.clone(options) : {};
      if (!(model = this._prepareModel(model, options))) return false;
      if (!options.wait) this.add(model, options);
      var collection = this;
      var success = options.success;
      options.success = function(model, resp) {
        if (options.wait) collection.add(model, options);
        if (success) success(model, resp, options);
      };
      model.save(null, options);
      return model;
    },

    // **parse** converts a response into a list of models to be added to the
    // collection. The default implementation is just to pass it through.
    parse: function(resp, options) {
      return resp;
    },

    // Create a new collection with an identical list of models as this one.
    clone: function() {
      return new this.constructor(this.models);
    },

    // Private method to reset all internal state. Called when the collection
    // is first initialized or reset.
    _reset: function() {
      this.length = 0;
      this.models = [];
      this._byId  = {};
    },

    // Prepare a hash of attributes (or other model) to be added to this
    // collection.
    _prepareModel: function(attrs, options) {
      if (attrs instanceof Model) return attrs;
      options = options ? _.clone(options) : {};
      options.collection = this;
      var model = new this.model(attrs, options);
      if (!model.validationError) return model;
      this.trigger('invalid', this, model.validationError, options);
      return false;
    },

    // Internal method to create a model's ties to a collection.
    _addReference: function(model, options) {
      this._byId[model.cid] = model;
      if (model.id != null) this._byId[model.id] = model;
      if (!model.collection) model.collection = this;
      model.on('all', this._onModelEvent, this);
    },

    // Internal method to sever a model's ties to a collection.
    _removeReference: function(model, options) {
      if (this === model.collection) delete model.collection;
      model.off('all', this._onModelEvent, this);
    },

    // Internal method called every time a model in the set fires an event.
    // Sets need to update their indexes when models change ids. All other
    // events simply proxy through. "add" and "remove" events that originate
    // in other collections are ignored.
    _onModelEvent: function(event, model, collection, options) {
      if ((event === 'add' || event === 'remove') && collection !== this) return;
      if (event === 'destroy') this.remove(model, options);
      if (model && event === 'change:' + model.idAttribute) {
        delete this._byId[model.previous(model.idAttribute)];
        if (model.id != null) this._byId[model.id] = model;
      }
      this.trigger.apply(this, arguments);
    }

  });

  // Underscore methods that we want to implement on the Collection.
  // 90% of the core usefulness of Backbone Collections is actually implemented
  // right here:
  var methods = ['forEach', 'each', 'map', 'collect', 'reduce', 'foldl',
    'inject', 'reduceRight', 'foldr', 'find', 'detect', 'filter', 'select',
    'reject', 'every', 'all', 'some', 'any', 'include', 'contains', 'invoke',
    'max', 'min', 'toArray', 'size', 'first', 'head', 'take', 'initial', 'rest',
    'tail', 'drop', 'last', 'without', 'difference', 'indexOf', 'shuffle',
    'lastIndexOf', 'isEmpty', 'chain', 'sample'];

  // Mix in each Underscore method as a proxy to `Collection#models`.
  _.each(methods, function(method) {
    Collection.prototype[method] = function() {
      var args = slice.call(arguments);
      args.unshift(this.models);
      return _[method].apply(_, args);
    };
  });

  // Underscore methods that take a property name as an argument.
  var attributeMethods = ['groupBy', 'countBy', 'sortBy', 'indexBy'];

  // Use attributes instead of properties.
  _.each(attributeMethods, function(method) {
    Collection.prototype[method] = function(value, context) {
      var iterator = _.isFunction(value) ? value : function(model) {
        return model.get(value);
      };
      return _[method](this.models, iterator, context);
    };
  });

  // Backbone.View
  // -------------

  // Backbone Views are almost more convention than they are actual code. A View
  // is simply a JavaScript object that represents a logical chunk of UI in the
  // DOM. This might be a single item, an entire list, a sidebar or panel, or
  // even the surrounding frame which wraps your whole app. Defining a chunk of
  // UI as a **View** allows you to define your DOM events declaratively, without
  // having to worry about render order ... and makes it easy for the view to
  // react to specific changes in the state of your models.

  // Creating a Backbone.View creates its initial element outside of the DOM,
  // if an existing element is not provided...
  var View = Backbone.View = function(options) {
    this.cid = _.uniqueId('view');
    options || (options = {});
    _.extend(this, _.pick(options, viewOptions));
    this._ensureElement();
    this.initialize.apply(this, arguments);
    this.delegateEvents();
  };

  // Cached regex to split keys for `delegate`.
  var delegateEventSplitter = /^(\S+)\s*(.*)$/;

  // List of view options to be merged as properties.
  var viewOptions = ['model', 'collection', 'el', 'id', 'attributes', 'className', 'tagName', 'events'];

  // Set up all inheritable **Backbone.View** properties and methods.
  _.extend(View.prototype, Events, {

    // The default `tagName` of a View's element is `"div"`.
    tagName: 'div',

    // jQuery delegate for element lookup, scoped to DOM elements within the
    // current view. This should be preferred to global lookups where possible.
    $: function(selector) {
      return this.$el.find(selector);
    },

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // **render** is the core function that your view should override, in order
    // to populate its element (`this.el`), with the appropriate HTML. The
    // convention is for **render** to always return `this`.
    render: function() {
      return this;
    },

    // Remove this view by taking the element out of the DOM, and removing any
    // applicable Backbone.Events listeners.
    remove: function() {
      this.$el.remove();
      this.stopListening();
      return this;
    },

    // Change the view's element (`this.el` property), including event
    // re-delegation.
    setElement: function(element, delegate) {
      if (this.$el) this.undelegateEvents();
      this.$el = element instanceof Backbone.$ ? element : Backbone.$(element);
      this.el = this.$el[0];
      if (delegate !== false) this.delegateEvents();
      return this;
    },

    // Set callbacks, where `this.events` is a hash of
    //
    // *{"event selector": "callback"}*
    //
    //     {
    //       'mousedown .title':  'edit',
    //       'click .button':     'save',
    //       'click .open':       function(e) { ... }
    //     }
    //
    // pairs. Callbacks will be bound to the view, with `this` set properly.
    // Uses event delegation for efficiency.
    // Omitting the selector binds the event to `this.el`.
    // This only works for delegate-able events: not `focus`, `blur`, and
    // not `change`, `submit`, and `reset` in Internet Explorer.
    delegateEvents: function(events) {
      if (!(events || (events = _.result(this, 'events')))) return this;
      this.undelegateEvents();
      for (var key in events) {
        var method = events[key];
        if (!_.isFunction(method)) method = this[events[key]];
        if (!method) continue;

        var match = key.match(delegateEventSplitter);
        var eventName = match[1], selector = match[2];
        method = _.bind(method, this);
        eventName += '.delegateEvents' + this.cid;
        if (selector === '') {
          this.$el.on(eventName, method);
        } else {
          this.$el.on(eventName, selector, method);
        }
      }
      return this;
    },

    // Clears all callbacks previously bound to the view with `delegateEvents`.
    // You usually don't need to use this, but may wish to if you have multiple
    // Backbone views attached to the same DOM element.
    undelegateEvents: function() {
      this.$el.off('.delegateEvents' + this.cid);
      return this;
    },

    // Ensure that the View has a DOM element to render into.
    // If `this.el` is a string, pass it through `$()`, take the first
    // matching element, and re-assign it to `el`. Otherwise, create
    // an element from the `id`, `className` and `tagName` properties.
    _ensureElement: function() {
      if (!this.el) {
        var attrs = _.extend({}, _.result(this, 'attributes'));
        if (this.id) attrs.id = _.result(this, 'id');
        if (this.className) attrs['class'] = _.result(this, 'className');
        var $el = Backbone.$('<' + _.result(this, 'tagName') + '>').attr(attrs);
        this.setElement($el, false);
      } else {
        this.setElement(_.result(this, 'el'), false);
      }
    }

  });

  // Backbone.sync
  // -------------

  // Override this function to change the manner in which Backbone persists
  // models to the server. You will be passed the type of request, and the
  // model in question. By default, makes a RESTful Ajax request
  // to the model's `url()`. Some possible customizations could be:
  //
  // * Use `setTimeout` to batch rapid-fire updates into a single request.
  // * Send up the models as XML instead of JSON.
  // * Persist models via WebSockets instead of Ajax.
  //
  // Turn on `Backbone.emulateHTTP` in order to send `PUT` and `DELETE` requests
  // as `POST`, with a `_method` parameter containing the true HTTP method,
  // as well as all requests with the body as `application/x-www-form-urlencoded`
  // instead of `application/json` with the model in a param named `model`.
  // Useful when interfacing with server-side languages like **PHP** that make
  // it difficult to read the body of `PUT` requests.
  Backbone.sync = function(method, model, options) {
    var type = methodMap[method];

    // Default options, unless specified.
    _.defaults(options || (options = {}), {
      emulateHTTP: Backbone.emulateHTTP,
      emulateJSON: Backbone.emulateJSON
    });

    // Default JSON-request options.
    var params = {type: type, dataType: 'json'};

    // Ensure that we have a URL.
    if (!options.url) {
      params.url = _.result(model, 'url') || urlError();
    }

    // Ensure that we have the appropriate request data.
    if (options.data == null && model && (method === 'create' || method === 'update' || method === 'patch')) {
      params.contentType = 'application/json';
      params.data = JSON.stringify(options.attrs || model.toJSON(options));
    }

    // For older servers, emulate JSON by encoding the request into an HTML-form.
    if (options.emulateJSON) {
      params.contentType = 'application/x-www-form-urlencoded';
      params.data = params.data ? {model: params.data} : {};
    }

    // For older servers, emulate HTTP by mimicking the HTTP method with `_method`
    // And an `X-HTTP-Method-Override` header.
    if (options.emulateHTTP && (type === 'PUT' || type === 'DELETE' || type === 'PATCH')) {
      params.type = 'POST';
      if (options.emulateJSON) params.data._method = type;
      var beforeSend = options.beforeSend;
      options.beforeSend = function(xhr) {
        xhr.setRequestHeader('X-HTTP-Method-Override', type);
        if (beforeSend) return beforeSend.apply(this, arguments);
      };
    }

    // Don't process data on a non-GET request.
    if (params.type !== 'GET' && !options.emulateJSON) {
      params.processData = false;
    }

    // If we're sending a `PATCH` request, and we're in an old Internet Explorer
    // that still has ActiveX enabled by default, override jQuery to use that
    // for XHR instead. Remove this line when jQuery supports `PATCH` on IE8.
    if (params.type === 'PATCH' && noXhrPatch) {
      params.xhr = function() {
        return new ActiveXObject("Microsoft.XMLHTTP");
      };
    }

    // Make the request, allowing the user to override any Ajax options.
    var xhr = options.xhr = Backbone.ajax(_.extend(params, options));
    model.trigger('request', model, xhr, options);
    return xhr;
  };

  var noXhrPatch =
    typeof window !== 'undefined' && !!window.ActiveXObject &&
      !(window.XMLHttpRequest && (new XMLHttpRequest).dispatchEvent);

  // Map from CRUD to HTTP for our default `Backbone.sync` implementation.
  var methodMap = {
    'create': 'POST',
    'update': 'PUT',
    'patch':  'PATCH',
    'delete': 'DELETE',
    'read':   'GET'
  };

  // Set the default implementation of `Backbone.ajax` to proxy through to `$`.
  // Override this if you'd like to use a different library.
  Backbone.ajax = function() {
    return Backbone.$.ajax.apply(Backbone.$, arguments);
  };

  // Backbone.Router
  // ---------------

  // Routers map faux-URLs to actions, and fire events when routes are
  // matched. Creating a new one sets its `routes` hash, if not set statically.
  var Router = Backbone.Router = function(options) {
    options || (options = {});
    if (options.routes) this.routes = options.routes;
    this._bindRoutes();
    this.initialize.apply(this, arguments);
  };

  // Cached regular expressions for matching named param parts and splatted
  // parts of route strings.
  var optionalParam = /\((.*?)\)/g;
  var namedParam    = /(\(\?)?:\w+/g;
  var splatParam    = /\*\w+/g;
  var escapeRegExp  = /[\-{}\[\]+?.,\\\^$|#\s]/g;

  // Set up all inheritable **Backbone.Router** properties and methods.
  _.extend(Router.prototype, Events, {

    // Initialize is an empty function by default. Override it with your own
    // initialization logic.
    initialize: function(){},

    // Manually bind a single named route to a callback. For example:
    //
    //     this.route('search/:query/p:num', 'search', function(query, num) {
    //       ...
    //     });
    //
    route: function(route, name, callback) {
      if (!_.isRegExp(route)) route = this._routeToRegExp(route);
      if (_.isFunction(name)) {
        callback = name;
        name = '';
      }
      if (!callback) callback = this[name];
      var router = this;
      Backbone.history.route(route, function(fragment) {
        var args = router._extractParameters(route, fragment);
        router.execute(callback, args);
        router.trigger.apply(router, ['route:' + name].concat(args));
        router.trigger('route', name, args);
        Backbone.history.trigger('route', router, name, args);
      });
      return this;
    },

    // Execute a route handler with the provided parameters.  This is an
    // excellent place to do pre-route setup or post-route cleanup.
    execute: function(callback, args) {
      if (callback) callback.apply(this, args);
    },

    // Simple proxy to `Backbone.history` to save a fragment into the history.
    navigate: function(fragment, options) {
      Backbone.history.navigate(fragment, options);
      return this;
    },

    // Bind all defined routes to `Backbone.history`. We have to reverse the
    // order of the routes here to support behavior where the most general
    // routes can be defined at the bottom of the route map.
    _bindRoutes: function() {
      if (!this.routes) return;
      this.routes = _.result(this, 'routes');
      var route, routes = _.keys(this.routes);
      while ((route = routes.pop()) != null) {
        this.route(route, this.routes[route]);
      }
    },

    // Convert a route string into a regular expression, suitable for matching
    // against the current location hash.
    _routeToRegExp: function(route) {
      route = route.replace(escapeRegExp, '\\$&')
                   .replace(optionalParam, '(?:$1)?')
                   .replace(namedParam, function(match, optional) {
                     return optional ? match : '([^/?]+)';
                   })
                   .replace(splatParam, '([^?]*?)');
      return new RegExp('^' + route + '(?:\\?([\\s\\S]*))?$');
    },

    // Given a route, and a URL fragment that it matches, return the array of
    // extracted decoded parameters. Empty or unmatched parameters will be
    // treated as `null` to normalize cross-browser behavior.
    _extractParameters: function(route, fragment) {
      var params = route.exec(fragment).slice(1);
      return _.map(params, function(param, i) {
        // Don't decode the search params.
        if (i === params.length - 1) return param || null;
        return param ? decodeURIComponent(param) : null;
      });
    }

  });

  // Backbone.History
  // ----------------

  // Handles cross-browser history management, based on either
  // [pushState](http://diveintohtml5.info/history.html) and real URLs, or
  // [onhashchange](https://developer.mozilla.org/en-US/docs/DOM/window.onhashchange)
  // and URL fragments. If the browser supports neither (old IE, natch),
  // falls back to polling.
  var History = Backbone.History = function() {
    this.handlers = [];
    _.bindAll(this, 'checkUrl');

    // Ensure that `History` can be used outside of the browser.
    if (typeof window !== 'undefined') {
      this.location = window.location;
      this.history = window.history;
    }
  };

  // Cached regex for stripping a leading hash/slash and trailing space.
  var routeStripper = /^[#\/]|\s+$/g;

  // Cached regex for stripping leading and trailing slashes.
  var rootStripper = /^\/+|\/+$/g;

  // Cached regex for detecting MSIE.
  var isExplorer = /msie [\w.]+/;

  // Cached regex for removing a trailing slash.
  var trailingSlash = /\/$/;

  // Cached regex for stripping urls of hash.
  var pathStripper = /#.*$/;

  // Has the history handling already been started?
  History.started = false;

  // Set up all inheritable **Backbone.History** properties and methods.
  _.extend(History.prototype, Events, {

    // The default interval to poll for hash changes, if necessary, is
    // twenty times a second.
    interval: 50,

    // Are we at the app root?
    atRoot: function() {
      return this.location.pathname.replace(/[^\/]$/, '$&/') === this.root;
    },

    // Gets the true hash value. Cannot use location.hash directly due to bug
    // in Firefox where location.hash will always be decoded.
    getHash: function(window) {
      var match = (window || this).location.href.match(/#(.*)$/);
      return match ? match[1] : '';
    },

    // Get the cross-browser normalized URL fragment, either from the URL,
    // the hash, or the override.
    getFragment: function(fragment, forcePushState) {
      if (fragment == null) {
        if (this._hasPushState || !this._wantsHashChange || forcePushState) {
          fragment = decodeURI(this.location.pathname + this.location.search);
          var root = this.root.replace(trailingSlash, '');
          if (!fragment.indexOf(root)) fragment = fragment.slice(root.length);
        } else {
          fragment = this.getHash();
        }
      }
      return fragment.replace(routeStripper, '');
    },

    // Start the hash change handling, returning `true` if the current URL matches
    // an existing route, and `false` otherwise.
    start: function(options) {
      if (History.started) throw new Error("Backbone.history has already been started");
      History.started = true;

      // Figure out the initial configuration. Do we need an iframe?
      // Is pushState desired ... is it available?
      this.options          = _.extend({root: '/'}, this.options, options);
      this.root             = this.options.root;
      this._wantsHashChange = this.options.hashChange !== false;
      this._wantsPushState  = !!this.options.pushState;
      this._hasPushState    = !!(this.options.pushState && this.history && this.history.pushState);
      var fragment          = this.getFragment();
      var docMode           = document.documentMode;
      var oldIE             = (isExplorer.exec(navigator.userAgent.toLowerCase()) && (!docMode || docMode <= 7));

      // Normalize root to always include a leading and trailing slash.
      this.root = ('/' + this.root + '/').replace(rootStripper, '/');

      if (oldIE && this._wantsHashChange) {
        var frame = Backbone.$('<iframe src="javascript:0" tabindex="-1">');
        this.iframe = frame.hide().appendTo('body')[0].contentWindow;
        this.navigate(fragment);
      }

      // Depending on whether we're using pushState or hashes, and whether
      // 'onhashchange' is supported, determine how we check the URL state.
      if (this._hasPushState) {
        Backbone.$(window).on('popstate', this.checkUrl);
      } else if (this._wantsHashChange && ('onhashchange' in window) && !oldIE) {
        Backbone.$(window).on('hashchange', this.checkUrl);
      } else if (this._wantsHashChange) {
        this._checkUrlInterval = setInterval(this.checkUrl, this.interval);
      }

      // Determine if we need to change the base url, for a pushState link
      // opened by a non-pushState browser.
      this.fragment = fragment;
      var loc = this.location;

      // Transition from hashChange to pushState or vice versa if both are
      // requested.
      if (this._wantsHashChange && this._wantsPushState) {

        // If we've started off with a route from a `pushState`-enabled
        // browser, but we're currently in a browser that doesn't support it...
        if (!this._hasPushState && !this.atRoot()) {
          this.fragment = this.getFragment(null, true);
          this.location.replace(this.root + '#' + this.fragment);
          // Return immediately as browser will do redirect to new url
          return true;

        // Or if we've started out with a hash-based route, but we're currently
        // in a browser where it could be `pushState`-based instead...
        } else if (this._hasPushState && this.atRoot() && loc.hash) {
          this.fragment = this.getHash().replace(routeStripper, '');
          this.history.replaceState({}, document.title, this.root + this.fragment);
        }

      }

      if (!this.options.silent) return this.loadUrl();
    },

    // Disable Backbone.history, perhaps temporarily. Not useful in a real app,
    // but possibly useful for unit testing Routers.
    stop: function() {
      Backbone.$(window).off('popstate', this.checkUrl).off('hashchange', this.checkUrl);
      if (this._checkUrlInterval) clearInterval(this._checkUrlInterval);
      History.started = false;
    },

    // Add a route to be tested when the fragment changes. Routes added later
    // may override previous routes.
    route: function(route, callback) {
      this.handlers.unshift({route: route, callback: callback});
    },

    // Checks the current URL to see if it has changed, and if it has,
    // calls `loadUrl`, normalizing across the hidden iframe.
    checkUrl: function(e) {
      var current = this.getFragment();
      if (current === this.fragment && this.iframe) {
        current = this.getFragment(this.getHash(this.iframe));
      }
      if (current === this.fragment) return false;
      if (this.iframe) this.navigate(current);
      this.loadUrl();
    },

    // Attempt to load the current URL fragment. If a route succeeds with a
    // match, returns `true`. If no defined routes matches the fragment,
    // returns `false`.
    loadUrl: function(fragment) {
      fragment = this.fragment = this.getFragment(fragment);
      return _.any(this.handlers, function(handler) {
        if (handler.route.test(fragment)) {
          handler.callback(fragment);
          return true;
        }
      });
    },

    // Save a fragment into the hash history, or replace the URL state if the
    // 'replace' option is passed. You are responsible for properly URL-encoding
    // the fragment in advance.
    //
    // The options object can contain `trigger: true` if you wish to have the
    // route callback be fired (not usually desirable), or `replace: true`, if
    // you wish to modify the current URL without adding an entry to the history.
    navigate: function(fragment, options) {
      if (!History.started) return false;
      if (!options || options === true) options = {trigger: !!options};

      var url = this.root + (fragment = this.getFragment(fragment || ''));

      // Strip the hash for matching.
      fragment = fragment.replace(pathStripper, '');

      if (this.fragment === fragment) return;
      this.fragment = fragment;

      // Don't include a trailing slash on the root.
      if (fragment === '' && url !== '/') url = url.slice(0, -1);

      // If pushState is available, we use it to set the fragment as a real URL.
      if (this._hasPushState) {
        this.history[options.replace ? 'replaceState' : 'pushState']({}, document.title, url);

      // If hash changes haven't been explicitly disabled, update the hash
      // fragment to store history.
      } else if (this._wantsHashChange) {
        this._updateHash(this.location, fragment, options.replace);
        if (this.iframe && (fragment !== this.getFragment(this.getHash(this.iframe)))) {
          // Opening and closing the iframe tricks IE7 and earlier to push a
          // history entry on hash-tag change.  When replace is true, we don't
          // want this.
          if(!options.replace) this.iframe.document.open().close();
          this._updateHash(this.iframe.location, fragment, options.replace);
        }

      // If you've told us that you explicitly don't want fallback hashchange-
      // based history, then `navigate` becomes a page refresh.
      } else {
        return this.location.assign(url);
      }
      if (options.trigger) return this.loadUrl(fragment);
    },

    // Update the hash location, either replacing the current entry, or adding
    // a new one to the browser history.
    _updateHash: function(location, fragment, replace) {
      if (replace) {
        var href = location.href.replace(/(javascript:|#).*$/, '');
        location.replace(href + '#' + fragment);
      } else {
        // Some browsers require that `hash` contains a leading #.
        location.hash = '#' + fragment;
      }
    }

  });

  // Create the default Backbone.history.
  Backbone.history = new History;

  // Helpers
  // -------

  // Helper function to correctly set up the prototype chain, for subclasses.
  // Similar to `goog.inherits`, but uses a hash of prototype properties and
  // class properties to be extended.
  var extend = function(protoProps, staticProps) {
    var parent = this;
    var child;

    // The constructor function for the new subclass is either defined by you
    // (the "constructor" property in your `extend` definition), or defaulted
    // by us to simply call the parent's constructor.
    if (protoProps && _.has(protoProps, 'constructor')) {
      child = protoProps.constructor;
    } else {
      child = function(){ return parent.apply(this, arguments); };
    }

    // Add static properties to the constructor function, if supplied.
    _.extend(child, parent, staticProps);

    // Set the prototype chain to inherit from `parent`, without calling
    // `parent`'s constructor function.
    var Surrogate = function(){ this.constructor = child; };
    Surrogate.prototype = parent.prototype;
    child.prototype = new Surrogate;

    // Add prototype properties (instance properties) to the subclass,
    // if supplied.
    if (protoProps) _.extend(child.prototype, protoProps);

    // Set a convenience property in case the parent's prototype is needed
    // later.
    child.__super__ = parent.prototype;

    return child;
  };

  // Set up inheritance for the model, collection, router, view and history.
  Model.extend = Collection.extend = Router.extend = View.extend = History.extend = extend;

  // Throw an error when a URL is needed, and none is supplied.
  var urlError = function() {
    throw new Error('A "url" property or function must be specified');
  };

  // Wrap an optional error callback with a fallback error event.
  var wrapError = function(model, options) {
    var error = options.error;
    options.error = function(resp) {
      if (error) error(model, resp, options);
      model.trigger('error', model, resp, options);
    };
  };

  return Backbone;

}));

/**
 * @license text 2.0.15 Copyright jQuery Foundation and other contributors.
 * Released under MIT license, http://github.com/requirejs/text/LICENSE
 */
/*jslint regexp: true */
/*global require, XMLHttpRequest, ActiveXObject,
  define, window, process, Packages,
  java, location, Components, FileUtils */

define('text',['module'], function (module) {
    'use strict';

    var text, fs, Cc, Ci, xpcIsWindows,
        progIds = ['Msxml2.XMLHTTP', 'Microsoft.XMLHTTP', 'Msxml2.XMLHTTP.4.0'],
        xmlRegExp = /^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im,
        bodyRegExp = /<body[^>]*>\s*([\s\S]+)\s*<\/body>/im,
        hasLocation = typeof location !== 'undefined' && location.href,
        defaultProtocol = hasLocation && location.protocol && location.protocol.replace(/\:/, ''),
        defaultHostName = hasLocation && location.hostname,
        defaultPort = hasLocation && (location.port || undefined),
        buildMap = {},
        masterConfig = (module.config && module.config()) || {};

    function useDefault(value, defaultValue) {
        return value === undefined || value === '' ? defaultValue : value;
    }

    //Allow for default ports for http and https.
    function isSamePort(protocol1, port1, protocol2, port2) {
        if (port1 === port2) {
            return true;
        } else if (protocol1 === protocol2) {
            if (protocol1 === 'http') {
                return useDefault(port1, '80') === useDefault(port2, '80');
            } else if (protocol1 === 'https') {
                return useDefault(port1, '443') === useDefault(port2, '443');
            }
        }
        return false;
    }

    text = {
        version: '2.0.15',

        strip: function (content) {
            //Strips <?xml ...?> declarations so that external SVG and XML
            //documents can be added to a document without worry. Also, if the string
            //is an HTML document, only the part inside the body tag is returned.
            if (content) {
                content = content.replace(xmlRegExp, "");
                var matches = content.match(bodyRegExp);
                if (matches) {
                    content = matches[1];
                }
            } else {
                content = "";
            }
            return content;
        },

        jsEscape: function (content) {
            return content.replace(/(['\\])/g, '\\$1')
                .replace(/[\f]/g, "\\f")
                .replace(/[\b]/g, "\\b")
                .replace(/[\n]/g, "\\n")
                .replace(/[\t]/g, "\\t")
                .replace(/[\r]/g, "\\r")
                .replace(/[\u2028]/g, "\\u2028")
                .replace(/[\u2029]/g, "\\u2029");
        },

        createXhr: masterConfig.createXhr || function () {
            //Would love to dump the ActiveX crap in here. Need IE 6 to die first.
            var xhr, i, progId;
            if (typeof XMLHttpRequest !== "undefined") {
                return new XMLHttpRequest();
            } else if (typeof ActiveXObject !== "undefined") {
                for (i = 0; i < 3; i += 1) {
                    progId = progIds[i];
                    try {
                        xhr = new ActiveXObject(progId);
                    } catch (e) {}

                    if (xhr) {
                        progIds = [progId];  // so faster next time
                        break;
                    }
                }
            }

            return xhr;
        },

        /**
         * Parses a resource name into its component parts. Resource names
         * look like: module/name.ext!strip, where the !strip part is
         * optional.
         * @param {String} name the resource name
         * @returns {Object} with properties "moduleName", "ext" and "strip"
         * where strip is a boolean.
         */
        parseName: function (name) {
            var modName, ext, temp,
                strip = false,
                index = name.lastIndexOf("."),
                isRelative = name.indexOf('./') === 0 ||
                             name.indexOf('../') === 0;

            if (index !== -1 && (!isRelative || index > 1)) {
                modName = name.substring(0, index);
                ext = name.substring(index + 1);
            } else {
                modName = name;
            }

            temp = ext || modName;
            index = temp.indexOf("!");
            if (index !== -1) {
                //Pull off the strip arg.
                strip = temp.substring(index + 1) === "strip";
                temp = temp.substring(0, index);
                if (ext) {
                    ext = temp;
                } else {
                    modName = temp;
                }
            }

            return {
                moduleName: modName,
                ext: ext,
                strip: strip
            };
        },

        xdRegExp: /^((\w+)\:)?\/\/([^\/\\]+)/,

        /**
         * Is an URL on another domain. Only works for browser use, returns
         * false in non-browser environments. Only used to know if an
         * optimized .js version of a text resource should be loaded
         * instead.
         * @param {String} url
         * @returns Boolean
         */
        useXhr: function (url, protocol, hostname, port) {
            var uProtocol, uHostName, uPort,
                match = text.xdRegExp.exec(url);
            if (!match) {
                return true;
            }
            uProtocol = match[2];
            uHostName = match[3];

            uHostName = uHostName.split(':');
            uPort = uHostName[1];
            uHostName = uHostName[0];

            return (!uProtocol || uProtocol === protocol) &&
                   (!uHostName || uHostName.toLowerCase() === hostname.toLowerCase()) &&
                   ((!uPort && !uHostName) || isSamePort(uProtocol, uPort, protocol, port));
        },

        finishLoad: function (name, strip, content, onLoad) {
            content = strip ? text.strip(content) : content;
            if (masterConfig.isBuild) {
                buildMap[name] = content;
            }
            onLoad(content);
        },

        load: function (name, req, onLoad, config) {
            //Name has format: some.module.filext!strip
            //The strip part is optional.
            //if strip is present, then that means only get the string contents
            //inside a body tag in an HTML string. For XML/SVG content it means
            //removing the <?xml ...?> declarations so the content can be inserted
            //into the current doc without problems.

            // Do not bother with the work if a build and text will
            // not be inlined.
            if (config && config.isBuild && !config.inlineText) {
                onLoad();
                return;
            }

            masterConfig.isBuild = config && config.isBuild;

            var parsed = text.parseName(name),
                nonStripName = parsed.moduleName +
                    (parsed.ext ? '.' + parsed.ext : ''),
                url = req.toUrl(nonStripName),
                useXhr = (masterConfig.useXhr) ||
                         text.useXhr;

            // Do not load if it is an empty: url
            if (url.indexOf('empty:') === 0) {
                onLoad();
                return;
            }

            //Load the text. Use XHR if possible and in a browser.
            if (!hasLocation || useXhr(url, defaultProtocol, defaultHostName, defaultPort)) {
                text.get(url, function (content) {
                    text.finishLoad(name, parsed.strip, content, onLoad);
                }, function (err) {
                    if (onLoad.error) {
                        onLoad.error(err);
                    }
                });
            } else {
                //Need to fetch the resource across domains. Assume
                //the resource has been optimized into a JS module. Fetch
                //by the module name + extension, but do not include the
                //!strip part to avoid file system issues.
                req([nonStripName], function (content) {
                    text.finishLoad(parsed.moduleName + '.' + parsed.ext,
                                    parsed.strip, content, onLoad);
                });
            }
        },

        write: function (pluginName, moduleName, write, config) {
            if (buildMap.hasOwnProperty(moduleName)) {
                var content = text.jsEscape(buildMap[moduleName]);
                write.asModule(pluginName + "!" + moduleName,
                               "define(function () { return '" +
                                   content +
                               "';});\n");
            }
        },

        writeFile: function (pluginName, moduleName, req, write, config) {
            var parsed = text.parseName(moduleName),
                extPart = parsed.ext ? '.' + parsed.ext : '',
                nonStripName = parsed.moduleName + extPart,
                //Use a '.js' file name so that it indicates it is a
                //script that can be loaded across domains.
                fileName = req.toUrl(parsed.moduleName + extPart) + '.js';

            //Leverage own load() method to load plugin value, but only
            //write out values that do not have the strip argument,
            //to avoid any potential issues with ! in file names.
            text.load(nonStripName, req, function (value) {
                //Use own write() method to construct full module value.
                //But need to create shell that translates writeFile's
                //write() to the right interface.
                var textWrite = function (contents) {
                    return write(fileName, contents);
                };
                textWrite.asModule = function (moduleName, contents) {
                    return write.asModule(moduleName, fileName, contents);
                };

                text.write(pluginName, nonStripName, textWrite, config);
            }, config);
        }
    };

    if (masterConfig.env === 'node' || (!masterConfig.env &&
            typeof process !== "undefined" &&
            process.versions &&
            !!process.versions.node &&
            !process.versions['node-webkit'] &&
            !process.versions['atom-shell'])) {
        //Using special require.nodeRequire, something added by r.js.
        fs = require.nodeRequire('fs');

        text.get = function (url, callback, errback) {
            try {
                var file = fs.readFileSync(url, 'utf8');
                //Remove BOM (Byte Mark Order) from utf8 files if it is there.
                if (file[0] === '\uFEFF') {
                    file = file.substring(1);
                }
                callback(file);
            } catch (e) {
                if (errback) {
                    errback(e);
                }
            }
        };
    } else if (masterConfig.env === 'xhr' || (!masterConfig.env &&
            text.createXhr())) {
        text.get = function (url, callback, errback, headers) {
            var xhr = text.createXhr(), header;
            xhr.open('GET', url, true);

            //Allow plugins direct access to xhr headers
            if (headers) {
                for (header in headers) {
                    if (headers.hasOwnProperty(header)) {
                        xhr.setRequestHeader(header.toLowerCase(), headers[header]);
                    }
                }
            }

            //Allow overrides specified in config
            if (masterConfig.onXhr) {
                masterConfig.onXhr(xhr, url);
            }

            xhr.onreadystatechange = function (evt) {
                var status, err;
                //Do not explicitly handle errors, those should be
                //visible via console output in the browser.
                if (xhr.readyState === 4) {
                    status = xhr.status || 0;
                    if (status > 399 && status < 600) {
                        //An http 4xx or 5xx error. Signal an error.
                        err = new Error(url + ' HTTP status: ' + status);
                        err.xhr = xhr;
                        if (errback) {
                            errback(err);
                        }
                    } else {
                        callback(xhr.responseText);
                    }

                    if (masterConfig.onXhrComplete) {
                        masterConfig.onXhrComplete(xhr, url);
                    }
                }
            };
            xhr.send(null);
        };
    } else if (masterConfig.env === 'rhino' || (!masterConfig.env &&
            typeof Packages !== 'undefined' && typeof java !== 'undefined')) {
        //Why Java, why is this so awkward?
        text.get = function (url, callback) {
            var stringBuffer, line,
                encoding = "utf-8",
                file = new java.io.File(url),
                lineSeparator = java.lang.System.getProperty("line.separator"),
                input = new java.io.BufferedReader(new java.io.InputStreamReader(new java.io.FileInputStream(file), encoding)),
                content = '';
            try {
                stringBuffer = new java.lang.StringBuffer();
                line = input.readLine();

                // Byte Order Mark (BOM) - The Unicode Standard, version 3.0, page 324
                // http://www.unicode.org/faq/utf_bom.html

                // Note that when we use utf-8, the BOM should appear as "EF BB BF", but it doesn't due to this bug in the JDK:
                // http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4508058
                if (line && line.length() && line.charAt(0) === 0xfeff) {
                    // Eat the BOM, since we've already found the encoding on this file,
                    // and we plan to concatenating this buffer with others; the BOM should
                    // only appear at the top of a file.
                    line = line.substring(1);
                }

                if (line !== null) {
                    stringBuffer.append(line);
                }

                while ((line = input.readLine()) !== null) {
                    stringBuffer.append(lineSeparator);
                    stringBuffer.append(line);
                }
                //Make sure we return a JavaScript string and not a Java string.
                content = String(stringBuffer.toString()); //String
            } finally {
                input.close();
            }
            callback(content);
        };
    } else if (masterConfig.env === 'xpconnect' || (!masterConfig.env &&
            typeof Components !== 'undefined' && Components.classes &&
            Components.interfaces)) {
        //Avert your gaze!
        Cc = Components.classes;
        Ci = Components.interfaces;
        Components.utils['import']('resource://gre/modules/FileUtils.jsm');
        xpcIsWindows = ('@mozilla.org/windows-registry-key;1' in Cc);

        text.get = function (url, callback) {
            var inStream, convertStream, fileObj,
                readData = {};

            if (xpcIsWindows) {
                url = url.replace(/\//g, '\\');
            }

            fileObj = new FileUtils.File(url);

            //XPCOM, you so crazy
            try {
                inStream = Cc['@mozilla.org/network/file-input-stream;1']
                           .createInstance(Ci.nsIFileInputStream);
                inStream.init(fileObj, 1, 0, false);

                convertStream = Cc['@mozilla.org/intl/converter-input-stream;1']
                                .createInstance(Ci.nsIConverterInputStream);
                convertStream.init(inStream, "utf-8", inStream.available(),
                Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);

                convertStream.readString(inStream.available(), readData);
                convertStream.close();
                inStream.close();
                callback(readData.value);
            } catch (e) {
                throw new Error((fileObj && fileObj.path || '') + ': ' + e);
            }
        };
    }
    return text;
});

define('aura/ext/debug', [],function() {
  'use strict';

  return {
    name: 'debug',

    initialize: function(app) {
      if (typeof window.attachDebugger === 'function') {
        app.logger.log('Attaching debugger ...');
        window.attachDebugger(app);
      }
    }
  };
});

/*!
 * EventEmitter2
 * https://github.com/hij1nx/EventEmitter2
 *
 * Copyright (c) 2013 hij1nx
 * Licensed under the MIT license.
 */
;!function(undefined) {

  var isArray = Array.isArray ? Array.isArray : function _isArray(obj) {
    return Object.prototype.toString.call(obj) === "[object Array]";
  };
  var defaultMaxListeners = 10;

  function init() {
    this._events = {};
    if (this._conf) {
      configure.call(this, this._conf);
    }
  }

  function configure(conf) {
    if (conf) {

      this._conf = conf;

      conf.delimiter && (this.delimiter = conf.delimiter);
      conf.maxListeners && (this._events.maxListeners = conf.maxListeners);
      conf.wildcard && (this.wildcard = conf.wildcard);
      conf.newListener && (this.newListener = conf.newListener);

      if (this.wildcard) {
        this.listenerTree = {};
      }
    }
  }

  function EventEmitter(conf) {
    this._events = {};
    this.newListener = false;
    configure.call(this, conf);
  }

  //
  // Attention, function return type now is array, always !
  // It has zero elements if no any matches found and one or more
  // elements (leafs) if there are matches
  //
  function searchListenerTree(handlers, type, tree, i) {
    if (!tree) {
      return [];
    }
    var listeners=[], leaf, len, branch, xTree, xxTree, isolatedBranch, endReached,
        typeLength = type.length, currentType = type[i], nextType = type[i+1];
    if (i === typeLength && tree._listeners) {
      //
      // If at the end of the event(s) list and the tree has listeners
      // invoke those listeners.
      //
      if (typeof tree._listeners === 'function') {
        handlers && handlers.push(tree._listeners);
        return [tree];
      } else {
        for (leaf = 0, len = tree._listeners.length; leaf < len; leaf++) {
          handlers && handlers.push(tree._listeners[leaf]);
        }
        return [tree];
      }
    }

    if ((currentType === '*' || currentType === '**') || tree[currentType]) {
      //
      // If the event emitted is '*' at this part
      // or there is a concrete match at this patch
      //
      if (currentType === '*') {
        for (branch in tree) {
          if (branch !== '_listeners' && tree.hasOwnProperty(branch)) {
            listeners = listeners.concat(searchListenerTree(handlers, type, tree[branch], i+1));
          }
        }
        return listeners;
      } else if(currentType === '**') {
        endReached = (i+1 === typeLength || (i+2 === typeLength && nextType === '*'));
        if(endReached && tree._listeners) {
          // The next element has a _listeners, add it to the handlers.
          listeners = listeners.concat(searchListenerTree(handlers, type, tree, typeLength));
        }

        for (branch in tree) {
          if (branch !== '_listeners' && tree.hasOwnProperty(branch)) {
            if(branch === '*' || branch === '**') {
              if(tree[branch]._listeners && !endReached) {
                listeners = listeners.concat(searchListenerTree(handlers, type, tree[branch], typeLength));
              }
              listeners = listeners.concat(searchListenerTree(handlers, type, tree[branch], i));
            } else if(branch === nextType) {
              listeners = listeners.concat(searchListenerTree(handlers, type, tree[branch], i+2));
            } else {
              // No match on this one, shift into the tree but not in the type array.
              listeners = listeners.concat(searchListenerTree(handlers, type, tree[branch], i));
            }
          }
        }
        return listeners;
      }

      listeners = listeners.concat(searchListenerTree(handlers, type, tree[currentType], i+1));
    }

    xTree = tree['*'];
    if (xTree) {
      //
      // If the listener tree will allow any match for this part,
      // then recursively explore all branches of the tree
      //
      searchListenerTree(handlers, type, xTree, i+1);
    }

    xxTree = tree['**'];
    if(xxTree) {
      if(i < typeLength) {
        if(xxTree._listeners) {
          // If we have a listener on a '**', it will catch all, so add its handler.
          searchListenerTree(handlers, type, xxTree, typeLength);
        }

        // Build arrays of matching next branches and others.
        for(branch in xxTree) {
          if(branch !== '_listeners' && xxTree.hasOwnProperty(branch)) {
            if(branch === nextType) {
              // We know the next element will match, so jump twice.
              searchListenerTree(handlers, type, xxTree[branch], i+2);
            } else if(branch === currentType) {
              // Current node matches, move into the tree.
              searchListenerTree(handlers, type, xxTree[branch], i+1);
            } else {
              isolatedBranch = {};
              isolatedBranch[branch] = xxTree[branch];
              searchListenerTree(handlers, type, { '**': isolatedBranch }, i+1);
            }
          }
        }
      } else if(xxTree._listeners) {
        // We have reached the end and still on a '**'
        searchListenerTree(handlers, type, xxTree, typeLength);
      } else if(xxTree['*'] && xxTree['*']._listeners) {
        searchListenerTree(handlers, type, xxTree['*'], typeLength);
      }
    }

    return listeners;
  }

  function growListenerTree(type, listener) {

    type = typeof type === 'string' ? type.split(this.delimiter) : type.slice();

    //
    // Looks for two consecutive '**', if so, don't add the event at all.
    //
    for(var i = 0, len = type.length; i+1 < len; i++) {
      if(type[i] === '**' && type[i+1] === '**') {
        return;
      }
    }

    var tree = this.listenerTree;
    var name = type.shift();

    while (name) {

      if (!tree[name]) {
        tree[name] = {};
      }

      tree = tree[name];

      if (type.length === 0) {

        if (!tree._listeners) {
          tree._listeners = listener;
        }
        else if(typeof tree._listeners === 'function') {
          tree._listeners = [tree._listeners, listener];
        }
        else if (isArray(tree._listeners)) {

          tree._listeners.push(listener);

          if (!tree._listeners.warned) {

            var m = defaultMaxListeners;

            if (typeof this._events.maxListeners !== 'undefined') {
              m = this._events.maxListeners;
            }

            if (m > 0 && tree._listeners.length > m) {

              tree._listeners.warned = true;
              console.error('(node) warning: possible EventEmitter memory ' +
                            'leak detected. %d listeners added. ' +
                            'Use emitter.setMaxListeners() to increase limit.',
                            tree._listeners.length);
              console.trace();
            }
          }
        }
        return true;
      }
      name = type.shift();
    }
    return true;
  }

  // By default EventEmitters will print a warning if more than
  // 10 listeners are added to it. This is a useful default which
  // helps finding memory leaks.
  //
  // Obviously not all Emitters should be limited to 10. This function allows
  // that to be increased. Set to zero for unlimited.

  EventEmitter.prototype.delimiter = '.';

  EventEmitter.prototype.setMaxListeners = function(n) {
    this._events || init.call(this);
    this._events.maxListeners = n;
    if (!this._conf) this._conf = {};
    this._conf.maxListeners = n;
  };

  EventEmitter.prototype.event = '';

  EventEmitter.prototype.once = function(event, fn) {
    this.many(event, 1, fn);
    return this;
  };

  EventEmitter.prototype.many = function(event, ttl, fn) {
    var self = this;

    if (typeof fn !== 'function') {
      throw new Error('many only accepts instances of Function');
    }

    function listener() {
      if (--ttl === 0) {
        self.off(event, listener);
      }
      fn.apply(this, arguments);
    }

    listener._origin = fn;

    this.on(event, listener);

    return self;
  };

  EventEmitter.prototype.emit = function() {

    this._events || init.call(this);

    var type = arguments[0];

    if (type === 'newListener' && !this.newListener) {
      if (!this._events.newListener) { return false; }
    }

    // Loop through the *_all* functions and invoke them.
    if (this._all) {
      var l = arguments.length;
      var args = new Array(l - 1);
      for (var i = 1; i < l; i++) args[i - 1] = arguments[i];
      for (i = 0, l = this._all.length; i < l; i++) {
        this.event = type;
        this._all[i].apply(this, args);
      }
    }

    // If there is no 'error' event listener then throw.
    if (type === 'error') {

      if (!this._all &&
        !this._events.error &&
        !(this.wildcard && this.listenerTree.error)) {

        if (arguments[1] instanceof Error) {
          throw arguments[1]; // Unhandled 'error' event
        } else {
          throw new Error("Uncaught, unspecified 'error' event.");
        }
        return false;
      }
    }

    var handler;

    if(this.wildcard) {
      handler = [];
      var ns = typeof type === 'string' ? type.split(this.delimiter) : type.slice();
      searchListenerTree.call(this, handler, ns, this.listenerTree, 0);
    }
    else {
      handler = this._events[type];
    }

    if (typeof handler === 'function') {
      this.event = type;
      if (arguments.length === 1) {
        handler.call(this);
      }
      else if (arguments.length > 1)
        switch (arguments.length) {
          case 2:
            handler.call(this, arguments[1]);
            break;
          case 3:
            handler.call(this, arguments[1], arguments[2]);
            break;
          // slower
          default:
            var l = arguments.length;
            var args = new Array(l - 1);
            for (var i = 1; i < l; i++) args[i - 1] = arguments[i];
            handler.apply(this, args);
        }
      return true;
    }
    else if (handler) {
      var l = arguments.length;
      var args = new Array(l - 1);
      for (var i = 1; i < l; i++) args[i - 1] = arguments[i];

      var listeners = handler.slice();
      for (var i = 0, l = listeners.length; i < l; i++) {
        this.event = type;
        listeners[i].apply(this, args);
      }
      return (listeners.length > 0) || !!this._all;
    }
    else {
      return !!this._all;
    }

  };

  EventEmitter.prototype.on = function(type, listener) {

    if (typeof type === 'function') {
      this.onAny(type);
      return this;
    }

    if (typeof listener !== 'function') {
      throw new Error('on only accepts instances of Function');
    }
    this._events || init.call(this);

    // To avoid recursion in the case that type == "newListeners"! Before
    // adding it to the listeners, first emit "newListeners".
    this.emit('newListener', type, listener);

    if(this.wildcard) {
      growListenerTree.call(this, type, listener);
      return this;
    }

    if (!this._events[type]) {
      // Optimize the case of one listener. Don't need the extra array object.
      this._events[type] = listener;
    }
    else if(typeof this._events[type] === 'function') {
      // Adding the second element, need to change to array.
      this._events[type] = [this._events[type], listener];
    }
    else if (isArray(this._events[type])) {
      // If we've already got an array, just append.
      this._events[type].push(listener);

      // Check for listener leak
      if (!this._events[type].warned) {

        var m = defaultMaxListeners;

        if (typeof this._events.maxListeners !== 'undefined') {
          m = this._events.maxListeners;
        }

        if (m > 0 && this._events[type].length > m) {

          this._events[type].warned = true;
          console.error('(node) warning: possible EventEmitter memory ' +
                        'leak detected. %d listeners added. ' +
                        'Use emitter.setMaxListeners() to increase limit.',
                        this._events[type].length);
          console.trace();
        }
      }
    }
    return this;
  };

  EventEmitter.prototype.onAny = function(fn) {

    if (typeof fn !== 'function') {
      throw new Error('onAny only accepts instances of Function');
    }

    if(!this._all) {
      this._all = [];
    }

    // Add the function to the event listener collection.
    this._all.push(fn);
    return this;
  };

  EventEmitter.prototype.addListener = EventEmitter.prototype.on;

  EventEmitter.prototype.off = function(type, listener) {
    if (typeof listener !== 'function') {
      throw new Error('removeListener only takes instances of Function');
    }

    var handlers,leafs=[];

    if(this.wildcard) {
      var ns = typeof type === 'string' ? type.split(this.delimiter) : type.slice();
      leafs = searchListenerTree.call(this, null, ns, this.listenerTree, 0);
    }
    else {
      // does not use listeners(), so no side effect of creating _events[type]
      if (!this._events[type]) return this;
      handlers = this._events[type];
      leafs.push({_listeners:handlers});
    }

    for (var iLeaf=0; iLeaf<leafs.length; iLeaf++) {
      var leaf = leafs[iLeaf];
      handlers = leaf._listeners;
      if (isArray(handlers)) {

        var position = -1;

        for (var i = 0, length = handlers.length; i < length; i++) {
          if (handlers[i] === listener ||
            (handlers[i].listener && handlers[i].listener === listener) ||
            (handlers[i]._origin && handlers[i]._origin === listener)) {
            position = i;
            break;
          }
        }

        if (position < 0) {
          continue;
        }

        if(this.wildcard) {
          leaf._listeners.splice(position, 1);
        }
        else {
          this._events[type].splice(position, 1);
        }

        if (handlers.length === 0) {
          if(this.wildcard) {
            delete leaf._listeners;
          }
          else {
            delete this._events[type];
          }
        }
        return this;
      }
      else if (handlers === listener ||
        (handlers.listener && handlers.listener === listener) ||
        (handlers._origin && handlers._origin === listener)) {
        if(this.wildcard) {
          delete leaf._listeners;
        }
        else {
          delete this._events[type];
        }
      }
    }

    return this;
  };

  EventEmitter.prototype.offAny = function(fn) {
    var i = 0, l = 0, fns;
    if (fn && this._all && this._all.length > 0) {
      fns = this._all;
      for(i = 0, l = fns.length; i < l; i++) {
        if(fn === fns[i]) {
          fns.splice(i, 1);
          return this;
        }
      }
    } else {
      this._all = [];
    }
    return this;
  };

  EventEmitter.prototype.removeListener = EventEmitter.prototype.off;

  EventEmitter.prototype.removeAllListeners = function(type) {
    if (arguments.length === 0) {
      !this._events || init.call(this);
      return this;
    }

    if(this.wildcard) {
      var ns = typeof type === 'string' ? type.split(this.delimiter) : type.slice();
      var leafs = searchListenerTree.call(this, null, ns, this.listenerTree, 0);

      for (var iLeaf=0; iLeaf<leafs.length; iLeaf++) {
        var leaf = leafs[iLeaf];
        leaf._listeners = null;
      }
    }
    else {
      if (!this._events[type]) return this;
      this._events[type] = null;
    }
    return this;
  };

  EventEmitter.prototype.listeners = function(type) {
    if(this.wildcard) {
      var handlers = [];
      var ns = typeof type === 'string' ? type.split(this.delimiter) : type.slice();
      searchListenerTree.call(this, handlers, ns, this.listenerTree, 0);
      return handlers;
    }

    this._events || init.call(this);

    if (!this._events[type]) this._events[type] = [];
    if (!isArray(this._events[type])) {
      this._events[type] = [this._events[type]];
    }
    return this._events[type];
  };

  EventEmitter.prototype.listenersAny = function() {

    if(this._all) {
      return this._all;
    }
    else {
      return [];
    }

  };

  if (typeof define === 'function' && define.amd) {
     // AMD. Register as an anonymous module.
    define('eventemitter',[],function() {
      return EventEmitter;
    });
  } else if (typeof exports === 'object') {
    // CommonJS
    exports.EventEmitter2 = EventEmitter;
  }
  else {
    // Browser global.
    window.EventEmitter2 = EventEmitter;
  }
}();

define('aura/ext/mediator', ['eventemitter','underscore'],function () {
  'use strict';

  return {
    name: 'mediator',

    require: {
      paths: {
        eventemitter: 'bower_components/eventemitter2/lib/eventemitter2',
        underscore: 'bower_components/underscore/underscore'
      },
      shim: {
        underscore: {
          exports: '_'
        }
      }
    },

    initialize: function (app) {
      var EventEmitter = require('eventemitter');
      var _ = require('underscore');

      app.config.mediator = _.defaults(app.config.mediator || {}, {
        wildcard: true,
        delimiter: '.',
        newListener: true,
        maxListeners: 20
      });

      var mediator = new EventEmitter(app.config.mediator);

      app.core.mediator = mediator;

      var attachListener = function(listenerType) {
        return function (name, listener, context) {
          if (this.stopped) { return; }
          if (!_.isFunction(listener) || !_.isString(name)) {
            throw new Error('Invalid arguments passed to sandbox.' + listenerType);
          }
          context = context || this;
          var callback = function() {
            var args = Array.prototype.slice.call(arguments);
            try {
              listener.apply(context, args);
            } catch(e) {
              app.logger.error("Error caught in listener '" + name + "', called with arguments: ", args, "\nError:", e.message, e, args);
            }
          };

          this._events = this._events || [];
          this._events.push({ name: name, listener: listener, callback: callback });

          mediator[listenerType](name, callback);
        };
      };

      /**
       * @class  Sandbox
       *
       */

      /**
       * @method  on
       * @param {String} name         Pattern of event to subscrbibe to.
       */
      app.sandbox.on    = attachListener('on');

      /**
       * @method  once
       * @param {String} name          Pattern of event to subscrbibe to.
       */
      app.sandbox.once  = attachListener('once');

      /**
       * @method off
       * @param {String} name         Pattern of event to subscrbibe to.
       * @param {Function} listener   Listener function to stop.
       */
      app.sandbox.off = function (name, listener) {
        if (!this._events) { return; }
        this._events = _.reject(this._events, function (evt) {
          var ret = (evt.name === name);
          if (ret) { mediator.off(name, evt.callback); }
          return ret;
        });
      };


      /**
       * @method emit
       * @param {String} name       Event name to emit
       * @param {Object} payload    Payload emitted
       */
      app.sandbox.emit = function () {
        if (this.stopped) { return; }
        var debug = app.config.debug;
        if(debug.enable && (debug.components.length === 0 || debug.components.indexOf("aura:mediator") !== -1)){
          var eventData = Array.prototype.slice.call(arguments);
          eventData.unshift('Event emitted');
          app.logger.log.apply(app.logger, eventData);
        }
        mediator.emit.apply(mediator, arguments);
      };

      /**
       * @method stopListening
       */

      app.sandbox.stopListening = function () {
        if (!this._events) { return; }
        _.each(this._events, function (evt) {
          mediator.off(evt.name, evt.callback);
        });
      };

      var eventName = ['aura', 'sandbox', 'stop'].join(app.config.mediator.delimiter);
      app.core.mediator.on(eventName, function (sandbox) {
        sandbox.stopListening();
      });
    }
  };
});

define('aura/ext/components', [],function() {

  'use strict';

  return function(app) {

    var ownProp = function(obj, key) {
      return Object.prototype.hasOwnProperty.call(obj, key);
    };

    var core = app.core;
    var _ = app.core.util._;
    core.Components = core.Components || {};

    /**
     * Components registry
     * @type {Object}
     */
    var registeredComponents = {};

    /**
     * Components Callbacks
     */
    var componentsCallbacks = {};

    function invokeCallbacks(stage, fnName, context, args) {
      var callbacks = componentsCallbacks[stage + ":" + fnName] || [];
      var dfds      = [];
      app.core.util.each(callbacks, function(i, cb) {
        if (typeof cb === 'function') {
          dfds.push(cb.apply(context, args));
        }
      });
      var ret = app.core.data.when.apply(undefined, dfds).promise();
      return ret;
    }

    function invokeWithCallbacks(fn, context) {
      var fnName;
      if (typeof fn === 'string') {
        fnName  = fn;
        fn      = context[fnName] || function() {};
      } else if (typeof fn.name === 'string') {
        fnName = fn.name;
        fn     = fn.fn || function() {};
      } else {
        throw new Error("Error invoking Component with callbacks: ", context.options.name, ". first argument should be either the name of a function or of the form : { name: 'fnName', fn: function() { ... } } ");
      }

      var dfd = app.core.data.deferred();
      var before, after, args = [].slice.call(arguments, 2);
      before = invokeCallbacks("before", fnName, context, args);

      before.then(function() {
        return fn.apply(context, args);
      }).then(function(_res) {
        return invokeCallbacks("after", fnName, context, args.concat(_res)).then(function() {
          core.data.when(_res).then(function() {
            dfd.resolve(_res);
          });
        }, dfd.reject);
      }).fail(function(err) {
        app.logger.error("Error in Component " + context.options.name + " " + fnName + " callback", err);
        dfd.reject(err);
      });

      return dfd.promise();
    }

    /**
     * The base Component constructor...
     * @class  Component
     * @constructor
     * @param {Object} options the options to init the component...
     */
    function Component(options) {
      var opts = _.clone(options);
      var dfd = core.data.deferred();
      var self = this;

      /**
       * The Components' options Object, passed to its constructor.
       * TODO: Add explanation of how options are set from HTML API + which default options are
       * set during the Component initialization.
       *
       * @property {Object} options
       */
      this.options    = _.defaults(opts || {}, this.options || {});

      /**
       * Internal, unique reference for a Component instance
       *
       * @property {String} _ref
       */
      this._ref       = opts._ref;

      /**
       * A cached jQuery object for the Components's element.
       *
       * @property {Object} $el
       */
      this.$el        = core.dom.find(opts.el);

      this.initialized = dfd.promise();

      this.invokeWithCallbacks('initialize', this.options).then(function() {
        dfd.resolve(self);
      });

      return this;
    }

    /**
     * Method called on Component's initialization.
     *
     * @method initialize
     * @param {Object} options options Object passed on Component initialization
     */
    Component.prototype.initialize = function() {};

    /**
     * A helper function to render markup and recursively start nested components.
     *
     * @method  html
     * @param  {String} markup the markup to render in the component's root el
     * @return {Component} the Component instance to allow methods chaining...
     */
    Component.prototype.html = function(markup) {
      var el = this.$el;
      el.html(markup);
      var self = this;
      _.defer(function() {
        self.sandbox.start(el, { reset: true });
      });
      return this;
    };

    /**
     * A helper function to find matching elements within the Component's root element.
     *
     * @method  $find
     * @param  {Selector | jQuery Object | Element} selector CSS selector or jQuery object.
     * @return {jQuery Object}
     */
    Component.prototype.$find = function(selector) {
      return this.$el.find(selector);
    };

    /**
     *
     * @method  invokeWithCallbacks
     * @param  {String} methodName the name of the method to invoke with callbacks
     * @return {Promise} a Promise that will resolve to the return value of the original function invoked.
     */
    Component.prototype.invokeWithCallbacks = function(methodName) {
      return invokeWithCallbacks.apply(undefined, [methodName, this].concat([].slice.call(arguments, 1)));
    };

    // Stolen from Backbone 0.9.9 !
    // Helper function to correctly set up the prototype chain, for subclasses.
    // Similar to `goog.inherits`, but uses a hash of prototype properties and
    // class properties to be extended.
    var extend = function(protoProps, staticProps) {
      var parent = this;
      var child;
      if (protoProps && ownProp(protoProps, 'constructor')) {
        child = protoProps.constructor;
      } else {
        child = function(){ parent.apply(this, arguments); };
      }
      core.util.extend(child, parent, staticProps);
      var Surrogate = function(){ this.constructor = child; };
      Surrogate.prototype = parent.prototype;
      child.prototype = new Surrogate();
      if (protoProps) { core.util.extend(child.prototype, protoProps); }
      child.__super__ = parent.prototype;
      return child;
    };

    Component.extend = extend;

    /**
     * Component loader.
     * @param  {String} name    The name of the Component to load
     * @param  {Object} options The options to pass to the new component instance.
     * @return {Promise}        A Promise that resolves to the loaded component instance.
     */
    Component.load = function(name, opts) {
      // TODO: Make it more simple / or break it down
      // in several functions...
      // it's too big !

      var dfd = core.data.deferred(),
          ref = opts.ref,
          component,
          ComponentConstructor,
          el  = opts.el;

      opts._ref  = core.util._.uniqueId(ref + '+');

      var options = _.clone(opts);

      app.logger.log("Start loading component:", name);

      dfd.fail(function(err) {
        app.logger.error("Error loading component:", name, err);
      });

      // Apply requirejs map / package configuration before the actual loading.
      require.config(options.require);

      // Here, we require the component's package definition
      require([ref], function(componentDefinition) {

        if (!componentDefinition) {
          return dfd.reject("component " + ref + " Definition is empty !");
        }

        try {

          // Ok, the component has already been loaded once, we should already have it in the registry
          if (registeredComponents[ref]) {
            ComponentConstructor = registeredComponents[ref];
          } else {

            if (componentDefinition.type) {
              // If `type` is defined, we use a constructor provided by an extension ? ex. Backbone.
              ComponentConstructor = core.Components[componentDefinition.type];
            } else {
              // Otherwise, we use the stock Component constructor.
              ComponentConstructor = Component;
            }

            if (!ComponentConstructor) {
              throw new Error("Can't find component of type '" +  componentDefinition.type + "', did you forget to include the extension that provides it ?");
            }

            if (core.util._.isObject(componentDefinition)) {
              ComponentConstructor = registeredComponents[ref] = ComponentConstructor.extend(componentDefinition);
            }
          }

          var sandbox = app.sandboxes.create(opts._ref, { el: el });

          sandbox.logger.setName("Component '" + name + "'(" + sandbox.logger.name + ')');

          // Here we inject the sandbox in the component's prototype...
          var ext = { sandbox: sandbox };

          // If the Component is just defined as a function, we use it as its `initialize` method.
          if (typeof componentDefinition === 'function') {
            ext.initialize = componentDefinition;
          }

          ComponentConstructor = ComponentConstructor.extend(ext);

          var newComponent = new ComponentConstructor(options);

          // Sandbox owns its el and vice-versa
          newComponent.$el.data('__sandbox_ref__', sandbox.ref);

          var initialized = newComponent.initialized;

          initialized.then(function(ret) { dfd.resolve(ret); });
          initialized.fail(function(err) { dfd.reject(err); });

          return initialized;
        } catch(err) {
          app.logger.error(err.message);
          dfd.reject(err);
        }
      }, function(err) { dfd.reject(err); });

      return dfd.promise();
    };

    /**
     * Parses the component's options from its element's data attributes.
     *
     * @private
     * @param  {String|DomNode} el    the element
     * @param  {String} namespace     current Component's detected namespace
     * @param  {String} opts          an Object containing the base Component's options to extend.
     * @return {Object} An object that contains the Component's options
     */
    function parseComponentOptions(el, namespace, opts) {

      var options = _.clone(opts || {});
      options.el = el;
      options.require = {};

      var name, data = core.dom.data(el);

      // Here we go through all the data attributes of the element to build the options object
      core.util.each(data, function(k, v) {
        k = k.replace(new RegExp("^" + namespace), "");
        k = k.charAt(0).toLowerCase() + k.slice(1);
        if (k !== "component" && k !== 'widget') {
          options[k] = v;
        } else {
          name = v;
        }
      });

      return buildComponentOptions(name, options);
    }

    /**
     * Parses the component's options from its element's data attributes.
     *
     * @private
     * @param  {String} name      the Component's name
     * @param  {Object} opts      an Object containing the base Component's options to extend.
     * @return {Object}           An object that contains the component's options
     */
    function buildComponentOptions(name, options) {
      var ref              = name.split("@"),
          componentName   = core.util.decamelize(ref[0]),
          componentSource = ref[1] || "default",
          requireContext  = require.s.contexts._,
          componentsPath  = app.config.sources[componentSource] || "./aura_components";

      // Register the component as a requirejs package...
      // TODO: packages are not supported by almond, should we find another way to do this ?
      options.name              = componentName;
      options.ref               = '__component__$' + componentName + "@" + componentSource;
      options.baseUrl           = componentsPath + "/" + componentName;
      options.require           = options.require || {};
      options.require.packages  = options.require.packages || [];
      options.require.packages.push({ name: options.ref, location: componentsPath + "/" + componentName });

      return options;
    }

    /**!
     * Returns a list of components.
     * If the first argument is a String, it is considered as a DomNode reference
     * We then parse its content to find aura-components inside of it.
     *
     * @static
     * @param  {Array|String} components a list of components or a reference to a root dom node
     * @return {Array}        a list of component with their options
     */
    Component.parseList = function(components) {
      var list = [];

      if (Array.isArray(components)) {
        _.map(components, function(w) {
          var options = buildComponentOptions(w.name, w.options);
          list.push({ name: w.name, options: options });
        });
      } else if (components && core.dom.find(components)) {
        var appNamespace = app.config.namespace;

        // Support for legacy data-*-widget
        var selector = ["[data-aura-component]", "[data-aura-widget]"];
        if (appNamespace) {
          selector.push("[data-" + appNamespace + "-component]");
          selector.push("[data-" + appNamespace + "-widget]");
        }
        selector = selector.join(",");
        core.dom.find(selector, components || 'body').each(function() {
          var ns = "aura";
          if (appNamespace && (this.getAttribute('data-' + appNamespace +'-component') || this.getAttribute('data-' + appNamespace +'-widget'))) {
            ns = appNamespace;
          }
          var options = parseComponentOptions(this, ns);
          list.push({ name: options.name, options: options });
        });
      }
      return list;
    };

    /**
     * Actual start method for a list of components.
     *
     * @static
     * @param  {Array|String} components cf. `Component.parseList`
     * @return {Promise} a promise that resolves to a list of started components.
     */
    Component.startAll = function(components) {
      var componentsList = Component.parseList(components);
      var list = [];
      core.util.each(componentsList, function(i, w) {
        var ret = Component.load(w.name, w.options);
        list.push(ret);
      });
      var loadedComponents = core.data.when.apply(undefined, list);
      return loadedComponents.promise();
    };

    return {
      name: 'components',

      require: { paths: { text: 'bower_components/requirejs-text/text' } },

      initialize: function(app) {

        // Components 'classes' registry...
        app.core.Components.Base = Component;

        /**
         * @class Aura
         */


        /**
         * Registers a function that will be executed before the associated
         * Component method is called.
         *
         * Once registered, every time a component method will be called with the
         * `invokeWithCallbacks` method the registered callbacks will execute accordingly.
         *
         * The callback functions can return a promise if required in order to
         * postpone the execution of the called function up to when the all
         * registered callbacks results are resolved.
         *
         * The arguments passed to the callbacks are the same than those which
         * with the component will be called. The scope of the callback is
         * the component instance itself, as per the component method.
         *
         * @method components.before
         * @param  {String}   methodName    eg. 'initialize', 'destroy'
         * @param  {Function} fn            actual function to run
         */
        app.components.before = function(methodName, fn) {
          var callbackName = "before:" + methodName;
          registerComponentCallback(callbackName, fn);
        };

        /**
         * Same as components.before, but executed after the method invocation.
         *
         * @method components.after
         * @param  {[type]}   methodName eg. 'initialize', 'destroy'
         * @param  {Function} fn         actual function to run
         */
        app.components.after = function(methodName, fn) {
          var callbackName = "after:" + methodName;
          registerComponentCallback(callbackName, fn);
        };

        // Actually registering the callbacks in the registry.
        function registerComponentCallback(callbackName, fn) {
          componentsCallbacks[callbackName] = componentsCallbacks[callbackName] || [];
          componentsCallbacks[callbackName].push(fn);
        }

        /**
         * Register a Component Type (experimental).
         * Components type ca be used to encapsulate many custom components behaviours.
         * They will be then used while declaring your components as follows:
         *
         * ```js
         * define({
         *   type: 'myComponentType',
         *   // component declaration...
         * });
         * ```
         *
         * @method components.addType
         * @param  {String}   type a string that will identify the component type.
         * @param  {Function} def A constructor the this component type
         */
        app.components.addType = function(type, def) {
          if (app.core.Components[type]) {
            throw new Error("Component type " + type + " already defined");
          }
          app.core.Components[type] = Component.extend(def);
        };


        /**
         * @class Sandbox
         */

        /**
         * Start method.
         * This method takes either an Array of Components to start or or DOM Selector to
         * target the element that will be parsed to look for Components to start.
         *
         * @method  start
         * @param  {Array|DOM Selector} list    Array of Components to start or parent node.
         * @param  {Object} options             Available options: `reset` : if true, all current children
         *                                      will be stopped before start.
         */
        app.sandbox.start = function (list, options) {
          if (this.stopped) { return; }
          var event = ['aura', 'sandbox', 'start'].join(app.config.mediator.delimiter);
          app.core.mediator.emit(event, this);
          var children = this._children || [];
          if (options && options.reset) {
            _.invoke(this._children || [], 'stop');
            children = [];
          }
          var self = this;

          return Component.startAll(list).done(function () {
            var components   = Array.prototype.slice.call(arguments);
            _.each(components, function (w) {
              w.sandbox._component = w;
              w.sandbox._parent = self;
              children.push(w.sandbox);
            });
            self._children = children;
          });
        };

      },

      /**
       * When all of an application's extensions are finally loaded, the 'extensions'
       * afterAppStart methods are then called.
       *
       * @method components.afterAppStart
       * @param  {Object}   an Aura application object
       */
      afterAppStart: function(app) {
        if (app.config.components !== false && app.startOptions.components !== false) {
          var el;
          if (Array.isArray(app.startOptions.components)) {
            el = core.dom.find('body');
          } else {
            el = core.dom.find(app.startOptions.components);
          }
          app.core.appSandbox = app.sandboxes.create(app.ref, { el: el });
          app.core.appSandbox.start(app.startOptions.components);
        }
      }
    };
  };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('form/util',[], function() {

    'use strict';

    var ignoredKeys = [
        'form',
        'validation'
    ];

    return {
        debugEnabled: false,

        // get form fields
        getFields: function(element) {
            return $(element).find('input:not([data-form="false"], [type="submit"], [type="button"], [type="checkbox"], [type="radio"]), textarea:not([data-form="false"]), select:not([data-form="false"]), *[data-form="true"], *[data-type="collection"], *[contenteditable="true"]');
        },

        findGroupedFieldsBySelector: function(element, filter) {
            var groupedFields = {}, fieldName;

            $(element).find(filter).each(function(key, field) {
                fieldName = $(field).data('mapper-property');
                if (!groupedFields[fieldName]) {
                    groupedFields[fieldName] = [];
                }
                groupedFields[fieldName].push(field);
            });

            return groupedFields;
        },

        getCheckboxes: function(element) {
            return this.findGroupedFieldsBySelector(element, 'input[type="checkbox"]');
        },

        getRadios: function(element) {
            return this.findGroupedFieldsBySelector(element, 'input[type="radio"]');
        },

        /**
         * Parses the data of a element
         * Inspired by aurajs <http://aurajs.com>
         */
        parseData: function(el, namespace, defaults) {
            var $el = $(el);
            return this.buildOptions($el.data(), namespace, '', defaults);
        },

        /**
         * Build options for given data
         * Inspired by aurajs <http://aurajs.com>
         *
         * TODO Example
         */
        buildOptions: function(data, namespace, subNamespace, defaults) {
            if (!subNamespace) {
                subNamespace = '';
            }

            if (!defaults) {
                defaults = {};
            }

            var options = $.extend({}, defaults, {}),
                fullNamespace = namespace + this.ucFirst(subNamespace);

            $.each(data, function(key, value) {
                var regExp = new RegExp('^' + fullNamespace);
                if (regExp.test(key)) {
                    if ($.inArray(key, ignoredKeys) === -1) {
                        if (key !== fullNamespace) {
                            key = key.replace(regExp, '');
                        } else {
                            key = key.replace(new RegExp('^' + namespace), '');
                        }
                        if (key !== '') {
                            key = this.lcFirst(key);
                            options[key] = value;
                        }
                    }
                }
            }.bind(this));

            return options;
        },

        /**
         * returns true if .val can be used to this dom object
         * @param el {String|Object} valid selector or dom-object
         * @returns {Boolean}
         */
        isValueField: function(el) {
            var $el = $(el);

            return $el.is('input, select, textarea, option, button');
        },

        /**
         * returns true if element is checkbox
         * @param el {String|Object} valid selector or dom-object
         * @returns {Boolean}
         */
        isCheckbox: function(el) {
            var $el = $(el);

            return $el.is(':checkbox');
        },

        /**
         * Returns input values for elements
         * @param el {String|Object} valid selector or dom-object
         * @returns {String} value or empty string
         */
        getValue: function(el) {
            var $el = $(el);
            if (this.isCheckbox($el)) {
                return $el.prop('checked');
            } else if (this.isValueField($el)) {
                return $el.val();
            } else {
                return $el.html();
            }
        },

        /**
         * Sets a value for an element
         * @param el {String|Object} valid selector or dom-object to set the value for
         * @param value {String|Number} value to insert
         */
        setValue: function(el, value) {
            var $el = $(el);
            if (this.isCheckbox($el)) {
                $el.prop('checked', value);
            } else if (this.isValueField($el)) {
                $el.val(value);
            } else {
                $el.html(value);
            }
        },

        debug: function(p1, p2, p3) {
            if (!!this.debugEnabled) {
                if (!!p1) {
                    if (!!p2) {
                        if (!!p3) {
                            console.log('Husky Validation:', p1, p2, p3);
                        } else {
                            console.log('Husky Validation:', p1, p2);
                        }
                    } else {
                        console.log('Husky Validation:', p1);
                    }
                } else {
                    console.log('Husky Validation');
                }
            }
        },

        /**
         *  JavaScript equivalent of PHP’s ucfirst
         *  inspired by http://kevin.vanzonneveld.net
         */
        ucFirst: function(str) {
            str += '';
            var f = str.charAt(0).toUpperCase();
            return f + str.substr(1);
        },

        lcFirst: function(str) {
            return str.charAt(0).toLowerCase() + str.slice(1);
        },

        startsWith: function(str, starts) {
            return str.indexOf(starts) === 0;
        },

        /**
         * Prints object
         */
        print: function(object, stage) {
            if (!stage) {
                stage = 1;
            }

            var str = '',
                oneIndent = '&nbsp;&nbsp;&nbsp;&nbsp;',
                property, value,
                indent = '',
                i = 0;

            while (i < stage) {
                indent += oneIndent;
                i++;
            }

            for (property in object) {
                if (object.hasOwnProperty(property)) {
                    value = object[property];
                    if (typeof value === 'string') {
                        if (this.isNumeric(value)) {
                            str += indent + property + ': ' + value + '; </br>';
                        } else {
                            if (value.length > 7) {
                                value = value.substring(0, 6) + ' ...';
                            }
                            str += indent + property + ': \'' + value + '\'; </br>';
                        }
                    } else {
                        str += indent + property + ': { </br>' + indent + oneIndent + print(value, stage++) + '}';
                    }
                }
            }

            return str;
        },

        isNumeric: function(str) {
            return str.match(/-?\d+(.\d+)?/);
        }
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('form/element',['form/util'], function(Util) {

    'use strict';

    return function(el, form, options) {

        var defaults = {
                type: null,
                validationTrigger: 'focusout',                     // default validate trigger
                validationAddClasses: true,                        // add error and success classes
                validationAddClassesParent: true,                  // add classes to parent element
                validationErrorClass: 'husky-validate-error',      // error class
                validationClass: 'husky-validate',                 // default class
                validation: true                                   // validation on/off
            },
            ignoredOptions = [
                'validation',
                'validationTrigger',
                'validationAddClasses',
                'validationAddClassesParent',
                'validationClass',
                'validationErrorClass',
                'validationSubmitEvent'
            ],
            valid,
            validators = {},
            type,
            lastValue = null,
            dfd = null,

            that = {
                initialize: function() {
                    dfd = $.Deferred();
                    this.requireCounter = 0;
                    this.initialized = dfd.promise();

                    this.$el = $(el);

                    // set data element
                    this.$el.data('element', this);

                    this.options = $.extend({}, defaults, options);

                    if (!!this.options.validationAddClasses) {
                        this.$el.addClass(this.options.validationClass);
                    }

                    // init validation if necessary
                    if (!!this.options.validation) {
                        that.initValidation.call(this);
                    }
                },

                resolveInitialization: function() {
                    this.requireCounter--;
                    if (this.requireCounter === 0) {
                        dfd.resolve();
                    }
                },

                initValidation: function() {
                    that.bindValidationDomEvents.call(this);

                    that.initValidators.call(this);
                    that.initType.call(this);
                },

                bindValidationDomEvents: function() {
                    // build trigger
                    var triggers = ( !this.options.validationTrigger ? '' : this.options.validationTrigger );

                    // break if no trigger is set
                    if (triggers === '' || triggers === 'none') {
                        return;
                    }

                    // always bind change event, for better UX when a select is invalid
                    if (this.$el.is('select')) {
                        triggers += new RegExp('change', 'i').test(triggers) ? '' : ' change';
                    }

                    // trim triggers to bind them correctly with .on()
                    triggers = triggers.replace(/^\s+/g, '').replace(/\s+$/g, '');

                    // bind event
                    this.$el.bind(triggers, this.validate.bind(this));
                },

                initValidators: function() {
                    var addFunction = function(name, options) {
                        this.requireCounter++;
                        require(['validator/' + name], function(Validator) {
                            validators[name] = new Validator(this.$el, form, this, options);
                            Util.debug('Element Validator', name, options);
                            that.resolveInitialization.call(this);
                        }.bind(this));
                    }.bind(this);

                    // create validators for each of the constraints
                    $.each(this.options, function(key, val) {
                        // val not false
                        // key is not ignored
                        // and key starts with validation
                        if (!!val && $.inArray(key, ignoredOptions) === -1 && Util.startsWith(key, 'validation')) {
                            // filter validation prefix
                            var name = Util.lcFirst(key.replace('validation', '')),
                                options = Util.buildOptions(this.options, 'validation', name);

                            addFunction(name, options);
                        }
                    }.bind(this));

                    // HTML 5 attributes
                    // required
                    if (this.$el.attr('required') === 'required' && !validators.required) {
                        addFunction('required', {required: true});
                    }
                    // min
                    if (!!this.$el.attr('min') && !validators.min) {
                        addFunction('min', {min: parseInt(this.$el.attr('min'), 10)});
                    }
                    // max
                    if (!!this.$el.attr('max') && !validators.max) {
                        addFunction('max', {max: parseInt(this.$el.attr('max'), 10)});
                    }
                    // regex
                    if (!!this.$el.attr('pattern') && !validators.pattern) {
                        addFunction('regex', {regex: this.$el.attr('pattern')});
                    }
                },

                initType: function() {
                    var addFunction = function(typeName, options) {
                            this.requireCounter++;
                            require(['type/' + typeName], function(Type) {
                                type = new Type(this.$el, options, form);

                                type.initialized.then(function() {
                                    Util.debug('Element Type', typeName, options);
                                    that.resolveInitialization.call(this);
                                }.bind(this));

                            }.bind(this));
                        }.bind(this),
                        options = Util.buildOptions(this.options, 'type'),
                        typeName, tmpType;

                    // FIXME date HTML5 type browser language format

                    // if type exists
                    if (!!this.options.type) {
                        typeName = this.options.type;
                    } else if (!!this.$el.attr('type')) {
                        // HTML5 type attribute
                        tmpType = this.$el.attr('type');
                        if (tmpType === 'email') {
                            typeName = 'email';
                        } else if (tmpType === 'url') {
                            typeName = 'url';
                        } else if (tmpType === 'number') {
                            typeName = 'decimal';
                        } else if (tmpType === 'date') {
                            typeName = 'date';
                            if (!!options.format) {
                                options.format = 'd';
                            }
                        } else if (tmpType === 'time') {
                            typeName = 'date';
                            if (!!options.format) {
                                options.format = 't';
                            }
                        } else if (tmpType === 'datetime') {
                            typeName = 'date';
                        } else {
                            typeName = 'string';
                        }
                    } else {
                        typeName = 'string';
                    }
                    addFunction(typeName, options);
                },

                hasConstraints: function() {
                    var typeConstraint = (!!type && type.needsValidation()),
                        validatorsConstraint = Object.keys(validators).length > 0;

                    return validatorsConstraint || typeConstraint;
                },

                needsValidation: function() {
                    return lastValue !== Util.getValue(this.$el);
                },

                reset: function() {
                    var $element = this.$el;
                    if (!!this.options.validationAddClassesParent) {
                        $element = $element.parent();
                    }
                    $element.removeClass(this.options.validationErrorClass);
                },

                setValid: function(state) {
                    valid = state;
                    if (!!this.options.validationAddClasses) {
                        that.reset.call(this);

                        var $element = this.$el;
                        if (!!this.options.validationAddClassesParent) {
                            $element = $element.parent();
                        }
                        if (!state) {
                            $element.addClass(this.options.validationErrorClass);
                        }
                    }
                }
            },

            result = {
                validate: function(force) {
                    var result = true,
                        validated = false;

                    // only if value changed or force is set
                    if (force || that.needsValidation.call(this)) {
                        if (that.hasConstraints.call(this)) {
                            // check each validator
                            $.each(validators, function (key, validator) {
                                if (!validator.validate()) {
                                    result = false;
                                    // TODO Messages
                                }
                            });
                            validated = true;
                        }
                    }

                    // check type
                    if (!!type && type.needsValidation()) {
                        if (!type.validate()) {
                            result = false;
                        }
                        validated = true;
                    }

                    // set css classes
                    if (validated === true) {
                        if (!result) {
                            Util.debug('Field validate', !!result ? 'true' : 'false', this.$el);
                        }
                        that.setValid.call(this, result);
                    }

                    return result;
                },

                update: function() {
                    if (!that.hasConstraints.call(this)) {
                        // delete state
                        //that.reset.call(this);
                        return true;
                    }

                    var result = true;
                    // check each validator
                    $.each(validators, function(key, validator) {
                        if (!validator.update()) {
                            result = false;
                            // TODO Messages
                        }
                    });

                    // check type
                    if (type !== null && !type.validate()) {
                        result = false;
                    }

                    if (!result) {
                        Util.debug('Field validate', !!result ? 'true' : 'false', this.$el);
                    }
                    that.setValid.call(this, result);

                    return result;
                },

                isValid: function() {
                    return valid;
                },

                updateConstraint: function(name, options) {
                    if ($.inArray(name, Object.keys(validators)) > -1) {
                        validators[name].updateConstraint(options);
                        this.validate();
                    } else {
                        throw 'No constraint with name: ' + name;
                    }
                },

                deleteConstraint: function(name) {
                    if ($.inArray(name, Object.keys(validators)) > -1) {
                        delete validators[name];
                        this.validate(true);
                    } else {
                        throw 'No constraint with name: ' + name;
                    }
                },

                addConstraint: function(name, options) {
                    if ($.inArray(name, Object.keys(validators)) === -1) {
                        require(['validator/' + name], function(Validator) {
                            validators[name] = new Validator(this.$el, form, this, options);
                        }.bind(this));
                    } else {
                        throw 'Constraint with name: ' + name + ' already exists';
                    }
                },

                hasConstraint: function(name) {
                    return !!validators[name];
                },

                getConstraint: function(name) {
                    if (!this.hasConstraint(name)) {
                        return false;
                    }
                    return validators[name];
                },

                fieldAdded: function(element) {
                    $.each(validators, function(key, validator) {
                        // FIXME better solution? perhaps only to interested validators?
                        validator.fieldAdded(element);
                    });
                },

                fieldRemoved: function(element) {
                    $.each(validators, function(key, validator) {
                        // FIXME better solution? perhaps only to interested validators?
                        validator.fieldRemoved(element);
                    });
                },

                setValue: function(value) {
                    var dfd = $.Deferred(),
                        result;

                    this.initialized.then(function() {
                        result = type.setValue(value);

                        // if setvalue returns a deferred wait for that
                        if (!!result) {
                            result.then(function() {
                                dfd.resolve();
                            });
                        } else {
                            dfd.resolve();
                        }
                    }.bind(this));

                    return dfd.promise();
                },

                getValue: function(data) {
                    return type.getValue(data);
                },

                getType: function() {
                    return type;
                }
            };

        that.initialize.call(result);
        return result;
    };

});

/*
 * This file is part of Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

define('form/elementGroup',[],function() {
    'use strict';

    var setMultipleValue = function(elements, values) {
            elements.forEach(function(element) {
                values.forEach(function(value) {
                    if (element.$el.val() === value) {
                        element.$el.prop('checked', true);
                    }
                });
            });
        },

        setSingleValue = function(elements, value) {
            for (var i = -1; ++i < elements.length;) {
                if (elements[i].$el.val() === value) {
                    elements[i].$el.prop('checked', true);

                    return;
                }
            }
        };

    return function(elements, isSingleValue) {
        var result = {
            getValue: function() {
                var value = [];
                elements.forEach(function(element) {
                    if (element.$el.is(':checked')) {
                        value.push(element.$el.val());
                    }
                });

                if (!!isSingleValue) {
                    if (value.length > 1) {
                        throw new Error('Single value element group cannot return more than one value');
                    }

                    return value[0];
                }

                return value;
            },

            setValue: function(values) {
                if (!!isSingleValue && !!$.isArray(values)) {
                    throw new Error('Single value element cannot be set to an array value');
                }

                if (!isSingleValue && !$.isArray(values)) {
                    throw new Error('Field with multiple values cannot be set to a single value');
                }

                if ($.isArray(values)) {
                    setMultipleValue.call(this, elements, values);
                } else {
                    setSingleValue.call(this, elements, values);
                }
            }
        };

        elements.forEach(function(element) {
            element.$el.data('elementGroup', result);
        });

        return result;
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('form/validation',[
    'form/util'
], function(Util) {

    'use strict';

    return function(form) {
        var valid,

        // private functions
            that = {
                initialize: function() {
                    that.bindValidationDomEvents.call(this);

                    Util.debug('INIT Validation');
                },

                bindValidationDomEvents: function() {
                    if (!!form.options.validationSubmitEvent) {
                        // avoid submit if not valid
                        form.$el.on('submit', function() {
                            return form.validation.validate();
                        }.bind(this));
                    }
                },

                setValid: function(state) {
                    valid = state;
                }
            },

        // define validation interface
            result = {
                validate: function(force) {
                    var result = true;
                    // validate each element
                    $.each(form.elements, function(key, element) {
                        if (!element.validate(force)) {
                            // TODO: scroll to first invalid element you can't use $.focus because an element mustn't be an input
                            result = false;
                        }
                    });

                    $.each(form.mapper.collections, function(i, collection) {
                        $.each(collection.items, function(j, item) {
                            $.each(item.data('collection').childElements, function(k, childElement) {
                                if (!childElement.validate(force)) {
                                    result = false;
                                }
                            });
                        });
                    });

                    that.setValid.call(this, result);
                    Util.debug('Validation', !!result ? 'success' : 'error');
                    return result;
                },

                isValid: function() {
                    return valid;
                },

                updateConstraint: function(selector, name, options) {
                    var $element = $(selector);
                    if (!!$element.data('element')) {
                        $(selector).data('element').updateConstraint(name, options);
                    } else {
                        throw 'No validation element';
                    }
                },

                deleteConstraint: function(selector, name) {
                    var $element = $(selector);
                    if (!!$element.data('element')) {
                        $element.data('element').deleteConstraint(name);
                    } else {
                        throw 'No validation element';
                    }
                },

                addConstraint: function(selector, name, options) {
                    var $element = $(selector), element;
                    if (!!$element.data('element')) {
                        $element.data('element').addConstraint(name, options);
                    } else {
                        // create a new one
                        element = form.addField(selector);
                        // add constraint
                        element.addConstraint(name, options);
                        form.elements.push(element);
                    }
                }
            };

        that.initialize.call(result);
        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('form/mapper',[
    'form/util'
], function(Util) {

    'use strict';

    return function(form) {

        var filters = {},

        // private functions
            that = {
                initialize: function() {
                    Util.debug('INIT Mapper');

                    this.collections = [];
                    this.collectionsSet = {};
                    this.emptyTemplates = {};
                    this.templates = {};

                    form.initialized.then(function() {
                        var selector = '*[data-type="collection"]',
                            $elements = form.$el.find(selector);

                        $elements.each(that.initCollection.bind(this));
                    }.bind(this));
                },

                initCollection: function(key, value) {
                    var $element = $(value),
                        element = $element.data('element'),
                        property = $element.data('mapper-property'),
                        newChild, collection, emptyTemplate,
                        dfd = $.Deferred(),
                        counter = 0,
                        resolve = function() {
                            counter--;
                            if (counter === 0) {
                                dfd.resolve();
                            }
                        };

                    if (!$.isArray(property)) {
                        if (typeof property === 'object') {
                            property = [property];
                            $element.data('mapper-property', property);
                        } else {
                            throw 'no valid mapper-property value';
                        }
                    }

                    // get templates
                    element.$children = $element.children().clone(true);

                    // remove children
                    $element.html('');

                    // add to collections
                    collection = {
                        id: _.uniqueId('collection_'),
                        property: property,
                        $element: $element,
                        element: element,
                        items: []
                    };
                    this.collections.push(collection);

                    counter += element.$children.length;
                    // iterate through collection
                    element.$children.each(function(i, child) {
                        var $child = $(child), propertyName, x, len,
                            propertyCount = 0,
                            resolveElement = function() {
                                propertyCount--;
                                if (propertyCount === 0) {
                                    resolve();
                                }
                            };

                        // attention: template has to be markuped as 'script'
                        if (!$child.is('script')) {
                            throw 'template has to be defined as <script>';
                        }

                        newChild = {tpl: $child.html(), id: $child.attr('id'), collection: collection};
                        element.$children[i] = newChild;

                        for (x = -1, len = property.length; ++x < len;) {
                            if (property[x].tpl === newChild.id) {
                                propertyName = property[x].data;
                                emptyTemplate = property[x]['empty-tpl'];
                            }
                            // if child has empty template, set to empty templates
                            if (property[x]['empty-tpl'] && property[x]['empty-tpl'] === newChild.id) {
                                this.emptyTemplates[property[x].data] = {
                                    id: property[x]['empty-tpl'],
                                    tpl: $child.html()
                                };
                            }
                        }
                        // check if template is set
                        if (!!propertyName) {
                            newChild.propertyName = propertyName;
                            propertyCount = collection.element.getType().getMinOccurs();
                            this.templates[propertyName] = {
                                tpl: newChild,
                                collection: collection,
                                emptyTemplate: emptyTemplate,
                            };
                            // init default children
                            for (x = collection.element.getType().getMinOccurs() + 1; --x > 0;) {
                                that.appendChildren.call(this, collection, newChild).then(function() {
                                    // set counter
                                    $('#current-counter-' + propertyName).text(collection.element.getType().getChildren(newChild.id).length);
                                    resolveElement();
                                }.bind(this));
                            }
                        } else {
                            resolveElement();
                        }
                    }.bind(this));

                    $.each(property, function(i, item) {
                        // init add button
                        form.$el.on('click', '*[data-mapper-add="' + item.data + '"]', that.addClick.bind(this));

                        // init remove button
                        form.$el.on('click', '*[data-mapper-remove="' + item.data + '"]', that.removeClick.bind(this));

                        // emit collection init event after resolve
                        dfd.then(function() {
                            that.emitInitCollectionEvent(item.data);
                        });
                    }.bind(this));

                    that.checkFullAndEmpty.call(this, property[0].data);

                    dfd.then(function() {
                        Util.debug('collection resolved');
                    });

                    return dfd.promise();
                },

                addClick: function(event) {
                    var $addButton = $(event.currentTarget),
                        propertyName = $addButton.data('mapper-add'),
                        tpl = this.templates[propertyName].tpl,
                        collection = this.templates[propertyName].collection;

                    if (collection.element.getType().canAdd(tpl.id)) {
                        that.appendChildren.call(this, collection, tpl).then(function() {
                            // set counter
                            $('#current-counter-' + propertyName).text(collection.element.getType().getChildren(tpl.id).length);
                            that.emitAddEvent(propertyName, null);
                        }.bind(this));
                    }
                    that.checkFullAndEmpty.call(this, propertyName);
                },

                addEmptyTemplate: function($element, propertyName) {
                    if (this.emptyTemplates.hasOwnProperty(propertyName)) {
                        var $emptyTemplate = $(this.emptyTemplates[propertyName].tpl);
                        $emptyTemplate.attr('id', this.emptyTemplates[propertyName].id);
                        $element.append($emptyTemplate);
                    }
                },

                removeClick: function(event) {
                    var $removeButton = $(event.currentTarget),
                        propertyName = $removeButton.data('mapper-remove'),
                        tpl = this.templates[propertyName].tpl,
                        collection = this.templates[propertyName].collection,
                        $element = $removeButton.closest('.' + propertyName + '-element');

                    if (collection.element.getType().canRemove(tpl.id)) {
                        that.remove.call(this, $element);
                        // set counter
                        $('#current-counter-' + propertyName).text(collection.element.getType().getChildren(tpl.id).length);
                        that.emitRemoveEvent(propertyName, null);
                    }
                    that.checkFullAndEmpty.call(this, propertyName);
                },

                checkFullAndEmpty: function(propertyName) {
                    var $addButton = $("[data-mapper-add='" + propertyName + "']"),
                        $removeButton = $("[data-mapper-remove='" + propertyName + "']"),
                        tpl = this.templates[propertyName].tpl,
                        collection = this.templates[propertyName].collection,
                        fullClass = collection.element.$el.data('mapper-full-class') || 'full',
                        emptyClass = collection.element.$el.data('mapper-empty-class') || 'empty';

                    $addButton.removeClass(fullClass);
                    $addButton.removeClass(emptyClass);
                    $(collection.element.$el).removeClass(fullClass);
                    $(collection.element.$el).removeClass(emptyClass);

                    if (!!$addButton.length || !!$removeButton.length) {
                        // if no add is possible add full style-classes
                        if (!collection.element.getType().canAdd(tpl.id)) {
                            $addButton.addClass(fullClass);
                            $(collection.element.$el).addClass(fullClass);

                            // else, if no remove is possible add empty style-classes
                        } else if (!collection.element.getType().canRemove(tpl.id)) {
                            $addButton.addClass(emptyClass);
                            $(collection.element.$el).addClass(emptyClass);

                        }
                    }
                },

                emitInitCollectionEvent: function(propertyName) {
                    $(form.$el).trigger('form-collection-init', [propertyName]);
                },

                emitAddEvent: function(propertyName, data) {
                    $(form.$el).trigger('form-add', [propertyName, data]);
                },

                emitRemoveEvent: function(propertyName, data) {
                    $(form.$el).trigger('form-remove', [propertyName, data]);
                },

                processData: function(el, collection, returnMapperId) {
                    // get attributes
                    var $el = $(el),
                        type = $el.data('type'),
                        property = $el.data('mapper-property'),
                        element = $el.data('element'),
                        result,
                        filtersAction;


                    if (collection && !!filters[collection.data]) {
                        filtersAction = filters[collection.data];
                    } else if (!!filters[property]) {
                        filtersAction = filters[property];
                    }

                    // if type == collection process children, else get value
                    if (type !== 'collection') {
                        if (!!element) {
                            return element.getValue();
                        } else {
                            return null;
                        }
                    } else {
                        result = [];
                        $.each($el.children(), function(key, value) {
                            if (!collection || collection.tpl === value.dataset.mapperPropertyTpl) {
                                var elements = $(value).data('collection').childElements,
                                    elementGroups = $(value).data('collection').childElementGroups,
                                    data = {};

                                elements.forEach(function(child) {
                                    that.addDataFromElement.call(this, child, data, returnMapperId);
                                });

                                for (key in elementGroups) {
                                    if (elementGroups.hasOwnProperty(key)) {
                                        data[key] = elementGroups[key].getValue();
                                    }
                                }

                                // only set mapper-id if explicitly set
                                if (!!returnMapperId) {
                                    data.mapperId = value.dataset.mapperId;
                                }

                                var keys = Object.keys(data);
                                if (keys.length === 1) { // for value only collection
                                    if (data[keys[0]] !== '') {
                                        result.push(data[keys[0]]);
                                    }
                                } else if (!filtersAction || filtersAction(data)) {
                                    result.push(data);
                                }
                            }
                        }.bind(this));
                        return result;
                    }
                },

                setCollectionData: function(data, collection) {
                    // remember first child remove the rest
                    var $element = collection.$element,
                        child = this.templates[collection.key].tpl,
                        count = data.length,
                        dfd = $.Deferred(),
                        resolve = function() {
                            count--;
                            if (count === 0) {
                                dfd.resolve();
                            }
                        },
                        x,
                        length;

                    // no element in collection
                    if (count === 0) {
                        // check if empty template exists for that element and show it
                        that.addEmptyTemplate.call(this, $element, child.propertyName);
                        dfd.resolve();
                    } else {
                        if (data.length < collection.element.getType().getMinOccurs()) {
                            for (x = collection.element.getType().getMinOccurs() + 1, length = data.length; --x > length;) {
                                data.push({});
                            }
                        }

                        // FIXME the old DOM elements should be reused, instead of generated over and over again
                        // remove all prefilled items from the collection, because the DOM elements are recreated
                        collection.items = [];

                        // foreach collection elements: create a new dom element, call setData recursively
                        $.each(data, function(key, value) {
                            that.appendChildren.call(this, collection, child, value).then(function($newElement) {
                                that.setData.call(this, value, $newElement).then(function() {
                                    resolve();
                                }.bind(this));
                            }.bind(this));
                        }.bind(this));
                    }

                    // set current length of collection
                    $('#current-counter-' + $element.attr('id')).text(data.length);
                    that.checkFullAndEmpty.call(this, collection.property[0].data);
                    return dfd.promise();
                },

                appendChildren: function(collection, child, tplOptions, data, insertAfter) {
                    var clonedChild = $.extend(true, {}, child),
                        index = clonedChild.collection.element.getType().getChildren(clonedChild.id).length,
                        options = $.extend({}, {index: index}, tplOptions || {}),
                        template = _.template(clonedChild.tpl, options, form.options.delimiter),
                        $template = $(template),
                        $newFields = Util.getFields($template),
                        $radioFields = Util.getRadios($template),
                        $checkboxFields = Util.getCheckboxes($template),
                        dfd = $.Deferred(),
                        counter = $newFields.length,
                        $element = collection.$element,
                        $lastElement,
                        element;

                    // adding
                    $template.attr('data-mapper-property-tpl', clonedChild.id);
                    $template.attr('data-mapper-id', _.uniqueId());

                    // add template to element
                    if (insertAfter && ($lastElement = $element.find('*[data-mapper-property-tpl="' + clonedChild.id + '"]').last()).length > 0) {
                        $lastElement.after($template);
                    } else {
                        $element.append($template);
                    }

                    clonedChild.collection.childElements = [];
                    clonedChild.collection.childElementGroups = {};
                    // add fields
                    if ($newFields.length > 0) {
                        $newFields.each(function(key, field) {
                            element = form.createField($(field));
                            clonedChild.collection.childElements.push(element);
                            element.initialized.then(function() {
                                counter--;
                                if (counter === 0) {
                                    dfd.resolve($template);
                                }
                            });
                        }.bind(this));
                    } else {
                        dfd.resolve($template);
                    }

                    if (_.size($radioFields) > 0) {
                        $.each($radioFields, function(key, field) {
                            clonedChild.collection.childElementGroups[key] = form.createFieldGroup(field, true);
                        });
                    }
                    if (_.size($checkboxFields) > 0) {
                        $.each($checkboxFields, function(key, field) {
                            clonedChild.collection.childElementGroups[key] = form.createFieldGroup(field, false);
                        });
                    }

                    $template.data('collection', clonedChild.collection);

                    // if automatically set data after initialization ( needed for adding elements afterwards)
                    if (!!data) {
                        dfd.then(function() {
                            that.setData.call(this, data, $newFields);
                        }.bind(this));
                    }

                    collection.items.push($template);

                    return dfd.promise();
                },

                /**
                 * Returns a collection element for a given mapper-id
                 * @param {number} mapperId
                 * @return {Object|null} the dom object or null
                 **/
                getElementByMapperId: function(mapperId) {
                    for (var i = -1, iLength = this.collections.length; ++i < iLength;) {
                        for (var j = -1, jLength = this.collections[i].items.length; ++j < jLength;) {
                            if (this.collections[i].items[j].data('mapper-id') === mapperId) {
                                return this.collections[i].items[j];
                            }
                        }
                    }

                    return null;
                },

                /**
                 * Delets an element from the DOM and the global object by a given unique-id
                 * @param {number} mapperId
                 * @return {boolean|string} if an element was found and deleted it returns its template-name, else it returns false
                 **/
                deleteElementByMapperId: function(mapperId) {
                    var templateName;

                    $.each(this.collections, function(i, collection) {
                        $.each(collection.items, function(j, item) {
                            if (item.data('mapper-id').toString() !== mapperId.toString()) {
                                return true;
                            }

                            templateName = item.attr('data-mapper-property-tpl');
                            item.remove();
                            collection.items.splice(j, 1);

                            return false;
                        });
                    }.bind(this));

                    return templateName;
                },

                remove: function($element) {
                    // remove all fields of element
                    $.each(Util.getFields($element), function(key, value) {
                        form.removeField(value);
                    }.bind(this));

                    // remove element
                    $element.remove();
                },

                setData: function(data, $el) {
                    if (!$el) {
                        $el = form.$el;
                    }

                    var dfd = $.Deferred(),
                        selector,
                        $element,
                        element,
                        count = 1,
                        resolve = function() {
                            count--;
                            if (count === 0) {
                                dfd.resolve();
                            }
                        };

                    if (typeof data !== 'object') {
                        selector = '*[data-mapper-property]';
                        $element = $el.find(selector);
                        element = $element.data('element');
                        // if element is not in form add it
                        if (!element) {
                            element = form.addField($element);
                            element.initialized.then(function() {
                                element.setValue(data).then(function() {
                                    // resolve this set data
                                    resolve();
                                });
                            }.bind(this));
                        } else {
                            element.setValue(data).then(function() {
                                // resolve this set data
                                resolve();
                            });
                        }
                    } else if (data !== null && !$.isEmptyObject(data)) {
                        count = Object.keys(data).length;
                        $.each(data, function(key, value) {
                            var $element, element, collection;

                            // if field is a collection
                            if ($.isArray(value) && this.templates.hasOwnProperty(key)) {
                                collection = this.templates[key].collection;
                                collection.key = key;

                                // if first element of collection, clear collection
                                if (!this.collectionsSet.hasOwnProperty(collection.id)) {
                                    collection.$element.children().each(function(key, value) {
                                        $(value).remove();
                                    }.bind(this));
                                }
                                this.collectionsSet[collection.id] = true;

                                that.setCollectionData.call(this, value, collection).then(function() {
                                    resolve();
                                });
                            } else if (form.elementGroups.hasOwnProperty(key)) {
                                form.elementGroups[key].setValue(value);
                                resolve();
                            } else {
                                // search field with mapper property
                                selector = '*[data-mapper-property="' + key + '"]';
                                $element = $el.andSelf().find(selector);

                                element = $element.data('element');

                                if ($element.length > 0) {
                                    // if element is not in form add it
                                    if (!element) {
                                        element = form.addField($element);
                                        element.initialized.then(function() {
                                            element.setValue(value).then(function() {
                                                resolve();
                                            });
                                        }.bind(this));
                                    } else {
                                        element.setValue(value).then(function() {
                                            resolve();
                                        });
                                    }
                                } else {
                                    resolve();
                                }
                            }
                        }.bind(this));
                    } else {
                        dfd.resolve();
                    }

                    return dfd.promise();
                },

                addDataFromElement: function(element, data, returnMapperId) {
                    var $element = element.$el,
                        property = $element.data('mapper-property'),
                        parts;

                    if (!property) {
                        return;
                    }

                    if ($.isArray(property)) {
                        $.each(property, function(i, prop) {
                            data[prop.data] = that.processData.call(this, $element, prop, returnMapperId);
                        }.bind(this));
                    } else if (property.match(/.*\..*/)) {
                        parts = property.split('.');
                        data[parts[0]] = {};
                        data[parts[0]][parts[1]] = that.processData.call(this, $element);
                    } else {
                        // process it
                        data[property] = that.processData.call(this, $element);
                    }
                },

                getDataFromElements: function(elements, elementGroups, returnMapperId) {
                    var data = {};

                    elements.forEach(function(element) {
                        that.addDataFromElement.call(this, element, data, returnMapperId);
                    }.bind(this));

                    for (var key in elementGroups) {
                        if (elementGroups.hasOwnProperty(key)) {
                            data[key] = elementGroups[key].getValue();
                        }
                    }

                    return data;
                }
            },

        // define mapper interface
            result = {

                setData: function(data, $el) {
                    this.collectionsSet = {};

                    return that.setData.call(this, data, $el);
                },

                /**
                 * extracts data from $element or default form element
                 * @param {Object} [$el=undefined] element to select data from
                 * @param {Boolean} [returnMapperId=false] returnMapperId
                 */
                getData: function($el, returnMapperId) {
                    if (!!$el && !!$el.data('mapper-id')) {
                        var collection = that.getElementByMapperId.call(this, $el.data('mapper-id')).data('collection');
                        return that.getDataFromElements(
                            collection.childElements,
                            collection.childElementGroups,
                            returnMapperId
                        );
                    } else {
                        return that.getDataFromElements(form.elements, form.elementGroups, returnMapperId);
                    }
                },

                addCollectionFilter: function(name, callback) {
                    filters[name] = callback;
                },

                removeCollectionFilter: function(name) {
                    delete filters[name];
                },

                /**
                 * adds an element to a existing collection
                 * @param {String} propertyName property defined by 'data' attribute in data-mapper-property
                 * @param {Object} [data] Possibility to set data
                 * @param {Boolean} [append=false] Define if element should be added at the end of the collection. By default items are grouped by tpl name
                 */
                addToCollection: function(propertyName, data, append) {
                    var template = this.templates[propertyName],
                        element = template.collection.$element,
                        insertAfterLast = false,
                        lastElement,
                        $emptyTpl,
                        dfd = $.Deferred();

                    // check if element exists and put it after last
                    if (!append) {
                        insertAfterLast = true;
                    }
                    // check if empty template is set and lookup in dom
                    if (template.emptyTemplate) {
                        $emptyTpl = $(element).find('#' + template.emptyTemplate);
                        if ($emptyTpl) {
                            $emptyTpl.remove();
                        }
                    }

                    that.appendChildren.call(this, template.collection, template.tpl, data, data, insertAfterLast).then(function($element) {
                        dfd.resolve($element);
                    }.bind(this));

                    return dfd;
                },

                /**
                 * Edits a field in an collection
                 * @param mapperId {Number} the unique Id of the field
                 * @param data {Object} new data to apply
                 */
                editInCollection: function(mapperId, data) {
                    var $element = that.getElementByMapperId.call(this, mapperId);
                    that.setData.call(this, data, $element);
                },

                /**
                 * Removes a field from a collection
                 * @param mapperId {Number} the unique Id of the field
                 */
                removeFromCollection: function(mapperId) {
                    var i,
                        templateName = that.deleteElementByMapperId.call(this, mapperId);

                    // check if collection still has elements with propertyName, else render empty Template
                    if (form.$el.find('*[data-mapper-property-tpl=' + templateName + ']').length < 1) {
                        // get collection with is owner of templateName
                        for (i in this.templates) {
                            // if emptyTemplates is set
                            if (this.templates[i].tpl.id === templateName) {
                                that.addEmptyTemplate.call(this, this.templates[i].collection.$element, i);
                                return;
                            }
                        }
                    }

                }
            };

        that.initialize.call(result);
        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

require.config({
    paths: {
        'form': 'js/form',
        'form/mapper': 'js/mapper',
        'form/validation': 'js/validation',
        'form/element': 'js/element',
        'form/elementGroup': 'js/elementGroup',
        'form/util': 'js/util',

        'type/default': 'js/types/default',
        'type/readonly-select': 'js/types/readonlySelect',
        'type/string': 'js/types/string',
        'type/date': 'js/types/date',
        'type/decimal': 'js/types/decimal',
        'type/hiddenData': 'js/types/hiddenData',
        'type/mappingData': 'js/types/mappingData',
        'type/email': 'js/types/email',
        'type/url': 'js/types/url',
        'type/label': 'js/types/label',
        'type/select': 'js/types/select',
        'type/collection': 'js/types/collection',
        'type/attributes': 'js/types/attributes',

        'validator/default': 'js/validators/default',
        'validator/min': 'js/validators/min',
        'validator/max': 'js/validators/max',
        'validator/minLength': 'js/validators/min-length',
        'validator/maxLength': 'js/validators/max-length',
        'validator/required': 'js/validators/required',
        'validator/unique': 'js/validators/unique',
        'validator/equal': 'js/validators/equal',
        'validator/regex': 'js/validators/regex'
    }
});

define('form',[
    'form/element',
    'form/elementGroup',
    'form/validation',
    'form/mapper',
    'form/util'
], function(Element, ElementGroup, Validation, Mapper, Util) {

    'use strict';

    return function(el, options) {
        var defaults = {
                debug: false,                     // debug on/off
                delimiter: {                      // defines which delimiter should be used for templating
                    interpolate: /<~=(.+?)~>/g,
                    escape: /<~-(.+?)~>/g,
                    evaluate: /<~(.+?)~>/g
                },
                validation: true,                 // validation on/off
                validationTrigger: 'focusout',    // default validate trigger
                validationAddClassesParent: true, // add classes to parent element
                validationAddClasses: true,       // add error and success classes
                validationSubmitEvent: true,      // avoid submit if not valid
                mapper: true                      // mapper on/off
            },

        // private functions
            that = {
                initialize: function() {
                    this.$el = $(el);
                    this.options = $.extend(defaults, this.$el.data(), options);

                    // enable / disable debug
                    Util.debugEnabled = this.options.debug;

                    this.initialized = that.initFields.call(this);

                    if (!!this.options.validation) {
                        this.validation = new Validation(this);
                    }

                    if (!!this.options.mapper) {
                        this.mapper = new Mapper(this);
                    }

                    this.$el.data('form-object', this);
                },

                // initialize field objects
                initFields: function($el) {
                    var dfd = $.Deferred(),
                        requireCounter = 0,
                        resolve = function() {
                            requireCounter--;
                            if (requireCounter === 0) {
                                dfd.resolve();
                            }
                        };

                    $.each(Util.getFields($el || this.$el), function(key, value) {
                        requireCounter++;
                        that.addField.call(this, value).initialized.then(resolve.bind(this));
                    }.bind(this));

                    that.addGroupedFields.call(this, $el);

                    return dfd.promise();
                },

                bindValidationDomEvents: function() {
                    if (!!this.options.validationSubmitEvent) {
                        // avoid submit if not valid
                        this.$el.on('submit', function() {
                            return this.validation.validate();
                        }.bind(this));
                    }
                },

                createField: function(selector) {
                    var $element = $(selector),
                        options = Util.parseData($element, '', this.options);

                    return new Element($element, result, options);
                },

                createFieldGroup: function(selectors, single) {
                    return new ElementGroup(
                        selectors.map(function(selector) {
                            var $element = $(selector),
                                options = Util.parseData($element, '', this.options);

                            return new Element($element, result, options);
                        }.bind(this)),
                        single
                    );
                },

                addField: function(selector) {
                    var element = this.createField(selector);

                    this.elements.push(element);
                    Util.debug('Element created', options);

                    return element;
                },

                addSingleGroupedField: function(key, selectors, single) {
                    this.elementGroups[key] = this.createFieldGroup(selectors, single);
                },

                addGroupedFields: function($el) {
                    $.each(Util.getCheckboxes($el || this.$el), function(key, value) {
                        if (value.length > 1) {
                            that.addSingleGroupedField.call(this, key, value, false);
                        } else {
                            // backwards compatibility: single checkbox are handled as boolean values
                            that.addField.call(this, value);
                        }
                    }.bind(this));

                    $.each(Util.getRadios($el || this.$el), function(key, value) {
                        that.addSingleGroupedField.call(this, key, value, true);
                    }.bind(this));
                }
            },

            result = {
                elements: [],
                elementGroups: {},
                options: {},
                validation: false,
                mapper: false,

                createField: function(selector) {
                    var element = that.createField(selector);

                    element.initialized.then(function() {
                        element.fieldAdded(element);
                    }.bind(this));

                    return element;
                },

                createFieldGroup: function(selectors, single) {
                    return that.createFieldGroup(selectors, single);
                },

                addField: function(selector) {
                    var element = that.addField.call(this, selector);

                    element.initialized.then(function() {
                        // say everybody I have a new field
                        // FIXME better solution?
                        $.each(this.elements, function(key, element) {
                            element.fieldAdded(element);
                        });
                    }.bind(this));

                    return element;
                },

                addGroupedFields: function($el) {
                    return that.addGroupedFields.call(this, $el);
                },

                initFields: function($el) {
                    return that.initFields.call(this, $el);
                },

                removeFields: function($el) {
                    Util.getFields($el).each(function(i, item) {
                        this.removeField(item);
                    }.bind(this));
                },

                removeField: function(selector) {
                    var $element = $(selector),
                        el = $element.data('element');

                    // say everybody I have a lost field
                    // FIXME better solution?
                    $.each(this.elements, function(key, element) {
                        element.fieldRemoved(el);
                    });

                    this.elements.splice(this.elements.indexOf(el), 1);
                }
            };

        that.initialize.call(result);
        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/default',[
    'form/util'
],function(Util) {

    'use strict';

    return function($el, defaults, options, name, typeInterface, form) {

        var that = {
                initialize: function() {
                    this.$el = $el;
                    this.options = $.extend({}, defaults, options);

                    var dfd = $.Deferred();
                    this.requireCounter = 0;
                    this.initialized = dfd.promise();

                    if (!!this.initializeSub) {
                        this.initializeSub();
                    }
                    dfd.resolve();
                }
            },

            defaultInterface = {
                name: name,

                form: form,

                needsValidation: function() {
                    return true;
                },

                updateConstraint: function(options) {
                    $.extend(this.options, options);
                },

                // mapper functionality set value into input
                setValue: function(value) {
                    Util.setValue(this.$el, this.getViewData.call(this, value));
                },

                // mapper functionality get value from input
                getValue: function() {
                    return this.getModelData.call(this, Util.getValue(this.$el));
                },

                // internationalization of view data: default none
                getViewData: function(value) {
                    return value;
                },

                // internationalization of model data: default none
                getModelData: function(value) {
                    return value;
                }
            },
            result = $.extend({}, defaultInterface, typeInterface);

        that.initialize.call(result);

        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/string',[
    'type/default'
], function(Default) {

    'use strict';

    return function($el, options) {
        var defaults = { },

            typeInterface = {
                needsValidation: function() {
                    return false;
                },

                validate: function() {
                    return true;
                }
            };

        return new Default($el, defaults, options, 'string', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/date',[
    'type/default',
    'form/util'
], function(Default, Util) {

    'use strict';

    return function($el, options) {
        var defaults = {
                format: 'd'     // possibilities f, F, t, T, d, D
            },

            getDate = function(value) {
                Util.debug(value, new Date(value));
                return new Date(value);
            },

            subType = {
                validate: function() {
                    var val = Util.getValue(this.$el), date;
                    if (val === '') {
                        return true;
                    }

                    date = Globalize.parseDate(val, this.options.format);
                    return date !== null;
                },

                needsValidation: function() {
                    var val = Util.getValue(this.$el);
                    return val !== '';
                },

                // internationalization of view data: Globalize library
                getViewData: function(value) {
                    return Globalize.format(getDate(value), this.options.format);
                },

                // internationalization of model data: Globalize library
                getModelData: function(value) {
                    if (value !== '') {
                        var date = Globalize.parseDate(value, this.options.format);
                        return date.toISOString();
                    } else {
                        return value;
                    }
                }
            };

        return new Default($el, defaults, options, 'date', subType);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/decimal',[
    'type/default'
], function(Default) {

    'use strict';

    return function($el, options) {
        var defaults = {
                format: 'n', // n, d, c, p
                regExp: /^-?(?:\d+|\d{1,3}(?:,\d{3})+)?(?:\.\d+)?$/,
                nullable: false
            },

            typeInterface = {
                initializeSub: function() {
                },

                validate: function() {
                    var val = this.getValue();

                    if (val === '' || (this.options.nullable === true && val === null)) {
                        return true;
                    }

                    return this.options.regExp.test(val);
                },

                getModelData: function(val) {
                    if(val === '') {
                        if (this.options.nullable === true) {
                            return null;
                        }
                        
                        return '';
                    }
                    return Globalize.parseFloat(val);
                },

                getViewData: function(val) {
                    if(typeof val === 'string'){
                        if(val === '') {
                            return '';
                        }
                        val = parseFloat(val);
                    }
                    return Globalize.format(val, this.options.format);
                }
            };

        return new Default($el, defaults, options, 'decimal', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/hiddenData',[
    'type/default'
], function(Default) {

    'use strict';

    return function($el, options) {
        var defaults = {
                id: 'id',
                defaultValue: null
            },

            typeInterface = {

                hiddenData: null,

                setValue: function(value) {
                    this.hiddenData = value;
                    if (!!value && typeof value === 'object' && !!value[this.options.id]) {
                        this.$el.data('id', value[this.options.id]);
                    }
                },

                getValue: function() {
                    if (this.hiddenData !== null) {
                        return this.hiddenData;
                    } else {
                        return this.options.defaultValue;
                    }
                },

                needsValidation: function() {
                    return false;
                },

                validate: function() {
                    return true;
                }
            };

        return new Default($el, defaults, options, 'hiddenData', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/mappingData',[
    'type/default'
], function(Default) {

    'use strict';

    return function($el, options) {
        var defaults = {
                defaultValue: null,
                mapping: null,
                searchProperty: 'id',
                showProperty: 'name'
            },

            typeInterface = {

                setValue: function(value) {
                    if (value !== null && typeof value !== 'object') {
                        this.$el.data('value', value);
                        this.$el.text(this.getMappingValue(value) || this.options.defaultValue);
                    }
                },

                getValue: function() {

                    var value = this.$el.data('value');

                    if (value !== null) {
                        return value;
                    } else {
                        return this.options.defaultValue;
                    }
                },

                needsValidation: function() {
                    return false;
                },

                validate: function() {
                    return true;
                },

                getMappingValue: function(val) {

                    var key, obj = this.options.mapping;

                    if (!!obj) {
                        for (key in this.options.mapping) {
                            if (!!obj.hasOwnProperty(key)) {
                                if (obj[key].hasOwnProperty(this.options.searchProperty) &&
                                    obj[key].hasOwnProperty(this.options.showProperty) &&
                                    String(obj[key][this.options.searchProperty]) === String(val)) {
                                    return obj[key][this.options.showProperty];
                                }
                            }
                        }
                    }
                }
            };

        return new Default($el, defaults, options, 'hiddenData', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/email',[
    'type/default',
    'form/util'
], function(Default, Util) {

    'use strict';

    return function($el, options) {
        var defaults = {
                regExp: /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))){2,6}$/i
            },

            typeInterface = {
                validate: function() {
                    var val = this.getValue();
                    if (val === '') {
                        return true;
                    }

                    return this.options.regExp.test(val);
                },

                needsValidation: function() {
                    var val = this.getValue();
                    return val !== '';
                }
            };

        return new Default($el, defaults, options, 'email', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/url',[
    'type/default',
    'form/util'
], function(Default, Util) {

    'use strict';

    return function($el, options) {
        var defaults = {
                regExp: /^(https?|s?ftp|git):\/\/(((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:)*@)?(((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]))|((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?)(:\d*)?)(\/((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)+(\/(([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)*)*)?)?(\?((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|[\uE000-\uF8FF]|\/|\?)*)?(#((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|\/|\?)*)?$/i
            },

            typeInterface = {
                validate: function() {
                    var val = this.getValue();
                    if (val === '') {
                        return true;
                    }

                    if (this.options['url-strict'] !== 'true') {
                        val = new RegExp('(https?|s?ftp|git)', 'i').test(val) ? val : 'http://' + val;
                    }
                    return this.options.regExp.test(val);
                },

                needsValidation: function() {
                    var val = this.getValue();
                    return val !== '';
                }
            };

        return new Default($el, defaults, options, 'email', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/label',[
    'type/default'
], function(Default) {

    'use strict';

    return function($el, options) {
        var defaults = {
                id: 'id',
                label: 'name',
                translate: true
            },

            typeInterface = {
                setValue: function(value) {
                    if (!!value[this.options.label]) {
                        var label = value[this.options.label],
                            labelValue = value[this.options.label];
                        if (!!this.options.translate) {
                            labelValue = Globalize.localize(label, Globalize.culture().name);
                            if (labelValue === undefined) {
                                labelValue = label;
                            }
                        }
                        this.$el.text(labelValue);
                    }

                    if (!!value[this.options.id]) {
                        this.$el.data(this.options.id, value[this.options.id]);
                    }
                },

                getValue: function() {
                    var result = {};
                    result[this.options.id] = this.$el.data(this.options.id);
                    result[this.options.label] = this.$el.text();
                    return result;
                },

                needsValidation: function() {
                    return false;
                },

                validate: function() {
                    return true;
                }
            };

        return new Default($el, defaults, options, 'label', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/select',[
    'type/default',
    'form/util'
], function(Default, Util) {

    'use strict';

    return function($el, options) {
        var defaults = {
                id: 'id',
                label: 'name',
                type: 'object'
            },

            typeInterface = {
                setValue: function(value) {
                    if (typeof value === 'object') {
                        this.$el.val(value[this.options.id]);
                    } else {
                        // find option where id == value and set it to selected
                        this.$el.find('option[id='+value+']').attr('selected','selected');
                        this.options.type = 'string';
                    }
                },

                getValue: function() {
                    if (this.options.type === 'object') {
                        var result = {};
                        result[this.options.id] = Util.getValue(this.$el);
                        result[this.options.label] = this.$el.find('option:selected').text();
                        return result;
                    } else {
                        // return id of selected element
                        return this.$el.children(':selected').attr('id');
                    }

                },

                needsValidation: function() {
                    return false;
                },

                validate: function() {
                    return true;
                }
            };

        return new Default($el, defaults, options, 'select', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/readonly-select',[
    'type/default'
], function(Default) {

    'use strict';

    return function($el, options) {
        var defaults = {
                id: null,
                data: [],
                idProperty: 'id',
                outputProperty: 'name'
            },

            typeInterface = {
                setValue: function(value) {
                    var data = this.options.data,
                        idProperty = this.options.idProperty,
                        i , len;

                    // check if value is an object
                    if (typeof value === 'object') {
                        if (value.hasOwnProperty(idProperty)) {
                            value = value[idProperty];
                        } else {
                            throw "value has no property named " + idProperty;
                        }
                    // if value is null continue
                    } else if (value === null) {
                        return;
                    }

                    // set data id to value
                    this.$el.data('id', value);

                    // find value in data
                    if (data.length > 0) {
                        for (i = -1, len = data.length; ++i < len;) {
                            if (data[i].hasOwnProperty(idProperty) && data[i][idProperty].toString() === value.toString()) {
                                this.$el.html(data[i][this.options.outputProperty]);
                                break;
                            }
                        }
                    }
                },

                getValue: function() {
                    var id = this.$el.data('id'),
                        i, len;

                    for (i = -1, len = this.options.data.length; ++i < len;) {
                        if (this.options.data[i][this.options.idProperty].toString() === id.toString()) {
                            return this.options.data[i];
                        }
                    }
                    return null;
                },

                needsValidation: function() {
                    return false;
                },

                validate: function() {
                    return true;
                }
            };

        return new Default($el, defaults, options, 'readonly-select', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/collection',[
    'type/default'
], function(Default) {

    'use strict';

    return function($el, options) {
        var defaults = {
                min: 0,
                max: null
            },

            subType = {
                validate: function() {
                    return true;
                },

                needsValidation: function() {
                    return false;
                },

                getChildren: function(id) {
                    return this.$el.find('*[data-mapper-property-tpl="' + id + '"]');
                },

                getMinOccurs: function() {
                    return this.options.min;
                },

                getMaxOccurs: function() {
                    return this.options.max;
                },

                canAdd: function(id) {
                    var length = this.getChildren(id).length;
                    return this.getMaxOccurs() === null || length < this.getMaxOccurs();
                },

                canRemove: function(id) {
                    var length = this.getChildren(id).length;
                    return length > this.getMinOccurs();
                }
            };

        return new Default($el, defaults, options, 'collection', subType);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/attributes',[
    'type/default'
], function(Default) {

    'use strict';

    return function($el, options) {
        var defaults = {
            },

            typeInterface = {
                setValue: function(value) {
                   this.$el.data('attributes', value);
                },

                getValue: function() {
                    return this.$el.data('attributes');
                },

                needsValidation: function() {
                    return false;
                },

                validate: function() {
                    return true;
                }
            };

        return new Default($el, defaults, options, 'attributes', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('validator/default',[],function() {

    'use strict';

    return function($el, form, defaults, options, name) {

        return {
            name: name,

            initialize: function() {
                this.$el = $el;
                this.data = $.extend(defaults, this.$el.data(), options);
                this.updateData();

                if (!!this.initializeSub) {
                    this.initializeSub();
                }
            },

            validate: function() {
                // do nothing
            },

            update: function() {
                // do nothing
                return this.validate();
            },

            updateConstraint: function(options) {
                $.extend(this.data, options);
                this.updateData();
            },

            updateData: function() {
                $.each(this.data, function(key, value) {
                    this.$el.data(key, value);
                }.bind(this));
            },

            fieldRemoved: function() {
                // do nothing
            },

            fieldAdded: function() {
                // do nothing
            }
        };

    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('validator/min',[
    'validator/default'
], function(Default) {

    'use strict';

    return function($el, form, element, options) {
        var defaults = {
                min: 0
            },

            result = $.extend(new Default($el, form, defaults, options, 'min'), {
                validate: function() {
                    var val = this.data.element.getValue();
                    return Number(val) >= this.data.min;
                }
            });

        result.initialize();
        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('validator/max',[
    'validator/default'
], function(Default) {

    'use strict';

    return function($el, form, element, options) {
        var defaults = {
                max: 999
            },

            result = $.extend(new Default($el, form, defaults, options, 'max'), {
                validate: function() {
                    var val = this.data.element.getValue();
                    return Number(val) <= this.data.max;
                }
            });

        result.initialize();
        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('validator/minLength',[
    'validator/default'
], function(Default) {

    'use strict';

    return function($el, form, element, options) {
        var defaults = {
                minLength: 0
            },

            result = $.extend(new Default($el, form, defaults, options, 'min-length'), {
                validate: function() {
                    var val = this.data.element.getValue();
                    return val.length >= this.data.minLength;
                }
            });

        result.initialize();
        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('validator/maxLength',[
    'validator/default'
], function(Default) {

    'use strict';

    return function($el, form, element, options) {
        var defaults = {
                maxLength: 999
            },

            result = $.extend(new Default($el, form, defaults, options, 'max-length'), {
                validate: function() {
                    var val = this.data.element.getValue();
                    return val.length <= this.data.maxLength;
                }
            });

        result.initialize();
        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('validator/required',[
    'validator/default'
], function(Default) {

    'use strict';

    return function($el, form, element, options) {
        var defaults = { },

            result = $.extend(new Default($el, form, defaults, options, 'required'), {
                validate: function() {
                    if (!!this.data.required) {
                        var val = this.data.element.getValue();

                        if (typeof val === 'number') {
                            return true;
                        }

                        if (!!_.isString(val)) {
                            val = val.trim();
                        }

                        // notNull && notBlank && not undefined
                        if (!val) {
                            return false;
                        }

                        // not empty array, object and string
                        return _.size(val) > 0;
                    }
                    return true;
                }
            });

        result.initialize();

        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('validator/unique',[
    'validator/default',
    'form/util'
], function(Default, Util) {

    'use strict';

    return function($el, form, element, options) {

        var defaults = {
                validationUnique: null
            },

        // elements with same group name
            relatedElements = [],

        // is the element related
            isElementRelated = function(element, group) {
                return relatedElements.indexOf(element) && !!element.options.validationUnique && element.options.validationUnique === group;
            },

        // validate all related element
            validateElements = function(val) {
                var result = true;
                $.each(relatedElements, function(key, element) {
                    if (!validateElement(val, element)) {
                        result = false;
                        return false;
                    }
                    return true;
                });
                return result;
            },

        // validate one element
            validateElement = function(val, element) {
                return val !== element.getValue();
            },

        // update all related elements
            updateRelatedElements = function() {
                $.each(relatedElements, function(key, element) {
                    element.update();
                });
            },

            result = $.extend({}, new Default($el, form, defaults, options, 'unique'), {

                initializeSub: function() {
                    // init related elements
                    element.initialized.then(function() {
                        $.each(form.elements, function(key, element) {
                            this.fieldAdded(element);
                        }.bind(this));
                    }.bind(this));
                },

                validate: function() {
                    var val = this.data.element.getValue(),
                        result;
                    if (!!this.data.unique) {
                        result = validateElements(val);
                        updateRelatedElements();
                        return result;
                    } else {
                        throw 'No option group set';
                    }
                },

                update: function() {
                    var val = this.data.element.getValue(),
                        result;
                    if (!!this.data.unique) {
                        result = validateElements(val);
                        return result;
                    } else {
                        throw 'No option group set';
                    }
                },

                fieldAdded: function(element) {
                    if (element.$el !== this.$el && isElementRelated(element, this.data.unique)) {
                        Util.debug('field added', this.$el);
                        relatedElements.push(element);
                    }
                },

                fieldRemoved: function(element) {
                    Util.debug('field removed', this.$el);
                    relatedElements = relatedElements.splice(relatedElements.indexOf(element), 1);
                }
            });

        result.initialize();
        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('validator/equal',[
    'validator/default'
], function(Default) {

    'use strict';

    return function($el, form, element, options) {
        var defaults = {
                equal: null
            },

        // elements with same group name
            relatedElements = [],

        // is the element related
            isElementRelated = function(element, group) {
                return relatedElements.indexOf(element) && !!element.options.validationEqual && element.options.validationEqual === group;
            },

        // validate all related element
            validateElements = function(val) {
                var result = true;
                $.each(relatedElements, function(key, element) {
                    if (!validateElement(val, element)) {
                        result = false;
                        return false;
                    }
                    return true;
                });
                return result;
            },

        // validate one element
            validateElement = function(val, element) {
                return val === element.getValue();
            },

        // update all related elements
            updateRelatedElements = function() {
                $.each(relatedElements, function(key, element) {
                    element.update();
                });
            },

            result = $.extend(new Default($el, form, defaults, options, 'equal'), {

                initializeSub: function() {
                    // init related elements
                    element.initialized.then(function() {
                        $.each(form.elements, function(key, element) {
                            this.fieldAdded(element);
                        }.bind(this));
                    }.bind(this));
                },

                validate: function() {
                    var val = this.$el.val(),
                        result;
                    if (!!this.data.equal) {
                        result = validateElements(val);
                        updateRelatedElements();
                        return result;
                    } else {
                        throw 'No option group set';
                    }
                },

                update: function() {
                    var val = this.data.element.getValue(),
                        result;
                    if (!!this.data.equal) {
                        result = validateElements(val);
                        return result;
                    } else {
                        throw 'No option group set';
                    }
                },

                fieldAdded: function(element) {
                    if (element.$el !== this.$el && isElementRelated(element, this.data.equal)) {
                        relatedElements.push(element);
                    }
                },

                fieldRemoved: function(element) {
                    relatedElements = relatedElements.splice(relatedElements.indexOf(element), 1);
                }
            });

        result.initialize();
        return result;
    };

});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('validator/regex',[
    'validator/default'
], function(Default) {

    'use strict';

    return function($el, form, element, options) {
        var defaults = {
                regex: /\w*/
            },

            result = $.extend(new Default($el, form, defaults, options, 'regex'), {
                validate: function() {
                    // TODO flags
                    var pattern = this.data.regex,
                        regex = new RegExp(pattern),
                        val = this.data.element.getValue();

                    if (val === '') {
                        return true;
                    }

                    return regex.test(val);
                }
            });

        result.initialize();
        return result;
    };

});


define("husky-validation", function(){});

/*!
 * Globalize
 *
 * http://github.com/jquery/globalize
 *
 * Copyright Software Freedom Conservancy, Inc.
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 */

(function( window, undefined ) {

var Globalize,
	// private variables
	regexHex,
	regexInfinity,
	regexParseFloat,
	regexTrim,
	// private JavaScript utility functions
	arrayIndexOf,
	endsWith,
	extend,
	isArray,
	isFunction,
	isObject,
	startsWith,
	trim,
	truncate,
	zeroPad,
	// private Globalization utility functions
	appendPreOrPostMatch,
	expandFormat,
	formatDate,
	formatNumber,
	getTokenRegExp,
	getEra,
	getEraYear,
	parseExact,
	parseNegativePattern;

// Global variable (Globalize) or CommonJS module (globalize)
Globalize = function( cultureSelector ) {
	return new Globalize.prototype.init( cultureSelector );
};

if ( typeof require !== "undefined" &&
	typeof exports !== "undefined" &&
	typeof module !== "undefined" ) {
	// Assume CommonJS
	module.exports = Globalize;
} else {
	// Export as global variable
	window.Globalize = Globalize;
}

Globalize.cultures = {};

Globalize.prototype = {
	constructor: Globalize,
	init: function( cultureSelector ) {
		this.cultures = Globalize.cultures;
		this.cultureSelector = cultureSelector;

		return this;
	}
};
Globalize.prototype.init.prototype = Globalize.prototype;

// 1. When defining a culture, all fields are required except the ones stated as optional.
// 2. Each culture should have a ".calendars" object with at least one calendar named "standard"
//    which serves as the default calendar in use by that culture.
// 3. Each culture should have a ".calendar" object which is the current calendar being used,
//    it may be dynamically changed at any time to one of the calendars in ".calendars".
Globalize.cultures[ "default" ] = {
	// A unique name for the culture in the form <language code>-<country/region code>
	name: "en",
	// the name of the culture in the english language
	englishName: "English",
	// the name of the culture in its own language
	nativeName: "English",
	// whether the culture uses right-to-left text
	isRTL: false,
	// "language" is used for so-called "specific" cultures.
	// For example, the culture "es-CL" means "Spanish, in Chili".
	// It represents the Spanish-speaking culture as it is in Chili,
	// which might have different formatting rules or even translations
	// than Spanish in Spain. A "neutral" culture is one that is not
	// specific to a region. For example, the culture "es" is the generic
	// Spanish culture, which may be a more generalized version of the language
	// that may or may not be what a specific culture expects.
	// For a specific culture like "es-CL", the "language" field refers to the
	// neutral, generic culture information for the language it is using.
	// This is not always a simple matter of the string before the dash.
	// For example, the "zh-Hans" culture is netural (Simplified Chinese).
	// And the "zh-SG" culture is Simplified Chinese in Singapore, whose lanugage
	// field is "zh-CHS", not "zh".
	// This field should be used to navigate from a specific culture to it's
	// more general, neutral culture. If a culture is already as general as it
	// can get, the language may refer to itself.
	language: "en",
	// numberFormat defines general number formatting rules, like the digits in
	// each grouping, the group separator, and how negative numbers are displayed.
	numberFormat: {
		// [negativePattern]
		// Note, numberFormat.pattern has no "positivePattern" unlike percent and currency,
		// but is still defined as an array for consistency with them.
		//   negativePattern: one of "(n)|-n|- n|n-|n -"
		pattern: [ "-n" ],
		// number of decimal places normally shown
		decimals: 2,
		// string that separates number groups, as in 1,000,000
		",": ",",
		// string that separates a number from the fractional portion, as in 1.99
		".": ".",
		// array of numbers indicating the size of each number group.
		// TODO: more detailed description and example
		groupSizes: [ 3 ],
		// symbol used for positive numbers
		"+": "+",
		// symbol used for negative numbers
		"-": "-",
		// symbol used for NaN (Not-A-Number)
		"NaN": "NaN",
		// symbol used for Negative Infinity
		negativeInfinity: "-Infinity",
		// symbol used for Positive Infinity
		positiveInfinity: "Infinity",
		percent: {
			// [negativePattern, positivePattern]
			//   negativePattern: one of "-n %|-n%|-%n|%-n|%n-|n-%|n%-|-% n|n %-|% n-|% -n|n- %"
			//   positivePattern: one of "n %|n%|%n|% n"
			pattern: [ "-n %", "n %" ],
			// number of decimal places normally shown
			decimals: 2,
			// array of numbers indicating the size of each number group.
			// TODO: more detailed description and example
			groupSizes: [ 3 ],
			// string that separates number groups, as in 1,000,000
			",": ",",
			// string that separates a number from the fractional portion, as in 1.99
			".": ".",
			// symbol used to represent a percentage
			symbol: "%"
		},
		currency: {
			// [negativePattern, positivePattern]
			//   negativePattern: one of "($n)|-$n|$-n|$n-|(n$)|-n$|n-$|n$-|-n $|-$ n|n $-|$ n-|$ -n|n- $|($ n)|(n $)"
			//   positivePattern: one of "$n|n$|$ n|n $"
			pattern: [ "($n)", "$n" ],
			// number of decimal places normally shown
			decimals: 2,
			// array of numbers indicating the size of each number group.
			// TODO: more detailed description and example
			groupSizes: [ 3 ],
			// string that separates number groups, as in 1,000,000
			",": ",",
			// string that separates a number from the fractional portion, as in 1.99
			".": ".",
			// symbol used to represent currency
			symbol: "$"
		}
	},
	// calendars defines all the possible calendars used by this culture.
	// There should be at least one defined with name "standard", and is the default
	// calendar used by the culture.
	// A calendar contains information about how dates are formatted, information about
	// the calendar's eras, a standard set of the date formats,
	// translations for day and month names, and if the calendar is not based on the Gregorian
	// calendar, conversion functions to and from the Gregorian calendar.
	calendars: {
		standard: {
			// name that identifies the type of calendar this is
			name: "Gregorian_USEnglish",
			// separator of parts of a date (e.g. "/" in 11/05/1955)
			"/": "/",
			// separator of parts of a time (e.g. ":" in 05:44 PM)
			":": ":",
			// the first day of the week (0 = Sunday, 1 = Monday, etc)
			firstDay: 0,
			days: {
				// full day names
				names: [ "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" ],
				// abbreviated day names
				namesAbbr: [ "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" ],
				// shortest day names
				namesShort: [ "Su", "Mo", "Tu", "We", "Th", "Fr", "Sa" ]
			},
			months: {
				// full month names (13 months for lunar calendards -- 13th month should be "" if not lunar)
				names: [ "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December", "" ],
				// abbreviated month names
				namesAbbr: [ "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "" ]
			},
			// AM and PM designators in one of these forms:
			// The usual view, and the upper and lower case versions
			//   [ standard, lowercase, uppercase ]
			// The culture does not use AM or PM (likely all standard date formats use 24 hour time)
			//   null
			AM: [ "AM", "am", "AM" ],
			PM: [ "PM", "pm", "PM" ],
			eras: [
				// eras in reverse chronological order.
				// name: the name of the era in this culture (e.g. A.D., C.E.)
				// start: when the era starts in ticks (gregorian, gmt), null if it is the earliest supported era.
				// offset: offset in years from gregorian calendar
				{
					"name": "A.D.",
					"start": null,
					"offset": 0
				}
			],
			// when a two digit year is given, it will never be parsed as a four digit
			// year greater than this year (in the appropriate era for the culture)
			// Set it as a full year (e.g. 2029) or use an offset format starting from
			// the current year: "+19" would correspond to 2029 if the current year 2010.
			twoDigitYearMax: 2029,
			// set of predefined date and time patterns used by the culture
			// these represent the format someone in this culture would expect
			// to see given the portions of the date that are shown.
			patterns: {
				// short date pattern
				d: "M/d/yyyy",
				// long date pattern
				D: "dddd, MMMM dd, yyyy",
				// short time pattern
				t: "h:mm tt",
				// long time pattern
				T: "h:mm:ss tt",
				// long date, short time pattern
				f: "dddd, MMMM dd, yyyy h:mm tt",
				// long date, long time pattern
				F: "dddd, MMMM dd, yyyy h:mm:ss tt",
				// month/day pattern
				M: "MMMM dd",
				// month/year pattern
				Y: "yyyy MMMM",
				// S is a sortable format that does not vary by culture
				S: "yyyy\u0027-\u0027MM\u0027-\u0027dd\u0027T\u0027HH\u0027:\u0027mm\u0027:\u0027ss"
			}
			// optional fields for each calendar:
			/*
			monthsGenitive:
				Same as months but used when the day preceeds the month.
				Omit if the culture has no genitive distinction in month names.
				For an explaination of genitive months, see http://blogs.msdn.com/michkap/archive/2004/12/25/332259.aspx
			convert:
				Allows for the support of non-gregorian based calendars. This convert object is used to
				to convert a date to and from a gregorian calendar date to handle parsing and formatting.
				The two functions:
					fromGregorian( date )
						Given the date as a parameter, return an array with parts [ year, month, day ]
						corresponding to the non-gregorian based year, month, and day for the calendar.
					toGregorian( year, month, day )
						Given the non-gregorian year, month, and day, return a new Date() object
						set to the corresponding date in the gregorian calendar.
			*/
		}
	},
	// For localized strings
	messages: {}
};

Globalize.cultures[ "default" ].calendar = Globalize.cultures[ "default" ].calendars.standard;

Globalize.cultures.en = Globalize.cultures[ "default" ];

Globalize.cultureSelector = "en";

//
// private variables
//

regexHex = /^0x[a-f0-9]+$/i;
regexInfinity = /^[+\-]?infinity$/i;
regexParseFloat = /^[+\-]?\d*\.?\d*(e[+\-]?\d+)?$/;
regexTrim = /^\s+|\s+$/g;

//
// private JavaScript utility functions
//

arrayIndexOf = function( array, item ) {
	if ( array.indexOf ) {
		return array.indexOf( item );
	}
	for ( var i = 0, length = array.length; i < length; i++ ) {
		if ( array[i] === item ) {
			return i;
		}
	}
	return -1;
};

endsWith = function( value, pattern ) {
	return value.substr( value.length - pattern.length ) === pattern;
};

extend = function() {
	var options, name, src, copy, copyIsArray, clone,
		target = arguments[0] || {},
		i = 1,
		length = arguments.length,
		deep = false;

	// Handle a deep copy situation
	if ( typeof target === "boolean" ) {
		deep = target;
		target = arguments[1] || {};
		// skip the boolean and the target
		i = 2;
	}

	// Handle case when target is a string or something (possible in deep copy)
	if ( typeof target !== "object" && !isFunction(target) ) {
		target = {};
	}

	for ( ; i < length; i++ ) {
		// Only deal with non-null/undefined values
		if ( (options = arguments[ i ]) != null ) {
			// Extend the base object
			for ( name in options ) {
				src = target[ name ];
				copy = options[ name ];

				// Prevent never-ending loop
				if ( target === copy ) {
					continue;
				}

				// Recurse if we're merging plain objects or arrays
				if ( deep && copy && ( isObject(copy) || (copyIsArray = isArray(copy)) ) ) {
					if ( copyIsArray ) {
						copyIsArray = false;
						clone = src && isArray(src) ? src : [];

					} else {
						clone = src && isObject(src) ? src : {};
					}

					// Never move original objects, clone them
					target[ name ] = extend( deep, clone, copy );

				// Don't bring in undefined values
				} else if ( copy !== undefined ) {
					target[ name ] = copy;
				}
			}
		}
	}

	// Return the modified object
	return target;
};

isArray = Array.isArray || function( obj ) {
	return Object.prototype.toString.call( obj ) === "[object Array]";
};

isFunction = function( obj ) {
	return Object.prototype.toString.call( obj ) === "[object Function]";
};

isObject = function( obj ) {
	return Object.prototype.toString.call( obj ) === "[object Object]";
};

startsWith = function( value, pattern ) {
	return value.indexOf( pattern ) === 0;
};

trim = function( value ) {
	return ( value + "" ).replace( regexTrim, "" );
};

truncate = function( value ) {
	if ( isNaN( value ) ) {
		return NaN;
	}
	return Math[ value < 0 ? "ceil" : "floor" ]( value );
};

zeroPad = function( str, count, left ) {
	var l;
	for ( l = str.length; l < count; l += 1 ) {
		str = ( left ? ("0" + str) : (str + "0") );
	}
	return str;
};

//
// private Globalization utility functions
//

appendPreOrPostMatch = function( preMatch, strings ) {
	// appends pre- and post- token match strings while removing escaped characters.
	// Returns a single quote count which is used to determine if the token occurs
	// in a string literal.
	var quoteCount = 0,
		escaped = false;
	for ( var i = 0, il = preMatch.length; i < il; i++ ) {
		var c = preMatch.charAt( i );
		switch ( c ) {
			case "\'":
				if ( escaped ) {
					strings.push( "\'" );
				}
				else {
					quoteCount++;
				}
				escaped = false;
				break;
			case "\\":
				if ( escaped ) {
					strings.push( "\\" );
				}
				escaped = !escaped;
				break;
			default:
				strings.push( c );
				escaped = false;
				break;
		}
	}
	return quoteCount;
};

expandFormat = function( cal, format ) {
	// expands unspecified or single character date formats into the full pattern.
	format = format || "F";
	var pattern,
		patterns = cal.patterns,
		len = format.length;
	if ( len === 1 ) {
		pattern = patterns[ format ];
		if ( !pattern ) {
			throw "Invalid date format string \'" + format + "\'.";
		}
		format = pattern;
	}
	else if ( len === 2 && format.charAt(0) === "%" ) {
		// %X escape format -- intended as a custom format string that is only one character, not a built-in format.
		format = format.charAt( 1 );
	}
	return format;
};

formatDate = function( value, format, culture ) {
	var cal = culture.calendar,
		convert = cal.convert,
		ret;

	if ( !format || !format.length || format === "i" ) {
		if ( culture && culture.name.length ) {
			if ( convert ) {
				// non-gregorian calendar, so we cannot use built-in toLocaleString()
				ret = formatDate( value, cal.patterns.F, culture );
			}
			else {
				var eraDate = new Date( value.getTime() ),
					era = getEra( value, cal.eras );
				eraDate.setFullYear( getEraYear(value, cal, era) );
				ret = eraDate.toLocaleString();
			}
		}
		else {
			ret = value.toString();
		}
		return ret;
	}

	var eras = cal.eras,
		sortable = format === "s";
	format = expandFormat( cal, format );

	// Start with an empty string
	ret = [];
	var hour,
		zeros = [ "0", "00", "000" ],
		foundDay,
		checkedDay,
		dayPartRegExp = /([^d]|^)(d|dd)([^d]|$)/g,
		quoteCount = 0,
		tokenRegExp = getTokenRegExp(),
		converted;

	function padZeros( num, c ) {
		var r, s = num + "";
		if ( c > 1 && s.length < c ) {
			r = ( zeros[c - 2] + s);
			return r.substr( r.length - c, c );
		}
		else {
			r = s;
		}
		return r;
	}

	function hasDay() {
		if ( foundDay || checkedDay ) {
			return foundDay;
		}
		foundDay = dayPartRegExp.test( format );
		checkedDay = true;
		return foundDay;
	}

	function getPart( date, part ) {
		if ( converted ) {
			return converted[ part ];
		}
		switch ( part ) {
			case 0:
				return date.getFullYear();
			case 1:
				return date.getMonth();
			case 2:
				return date.getDate();
			default:
				throw "Invalid part value " + part;
		}
	}

	if ( !sortable && convert ) {
		converted = convert.fromGregorian( value );
	}

	for ( ; ; ) {
		// Save the current index
		var index = tokenRegExp.lastIndex,
			// Look for the next pattern
			ar = tokenRegExp.exec( format );

		// Append the text before the pattern (or the end of the string if not found)
		var preMatch = format.slice( index, ar ? ar.index : format.length );
		quoteCount += appendPreOrPostMatch( preMatch, ret );

		if ( !ar ) {
			break;
		}

		// do not replace any matches that occur inside a string literal.
		if ( quoteCount % 2 ) {
			ret.push( ar[0] );
			continue;
		}

		var current = ar[ 0 ],
			clength = current.length;

		switch ( current ) {
			case "ddd":
				//Day of the week, as a three-letter abbreviation
			case "dddd":
				// Day of the week, using the full name
				var names = ( clength === 3 ) ? cal.days.namesAbbr : cal.days.names;
				ret.push( names[value.getDay()] );
				break;
			case "d":
				// Day of month, without leading zero for single-digit days
			case "dd":
				// Day of month, with leading zero for single-digit days
				foundDay = true;
				ret.push(
					padZeros( getPart(value, 2), clength )
				);
				break;
			case "MMM":
				// Month, as a three-letter abbreviation
			case "MMMM":
				// Month, using the full name
				var part = getPart( value, 1 );
				ret.push(
					( cal.monthsGenitive && hasDay() ) ?
					( cal.monthsGenitive[ clength === 3 ? "namesAbbr" : "names" ][ part ] ) :
					( cal.months[ clength === 3 ? "namesAbbr" : "names" ][ part ] )
				);
				break;
			case "M":
				// Month, as digits, with no leading zero for single-digit months
			case "MM":
				// Month, as digits, with leading zero for single-digit months
				ret.push(
					padZeros( getPart(value, 1) + 1, clength )
				);
				break;
			case "y":
				// Year, as two digits, but with no leading zero for years less than 10
			case "yy":
				// Year, as two digits, with leading zero for years less than 10
			case "yyyy":
				// Year represented by four full digits
				part = converted ? converted[ 0 ] : getEraYear( value, cal, getEra(value, eras), sortable );
				if ( clength < 4 ) {
					part = part % 100;
				}
				ret.push(
					padZeros( part, clength )
				);
				break;
			case "h":
				// Hours with no leading zero for single-digit hours, using 12-hour clock
			case "hh":
				// Hours with leading zero for single-digit hours, using 12-hour clock
				hour = value.getHours() % 12;
				if ( hour === 0 ) hour = 12;
				ret.push(
					padZeros( hour, clength )
				);
				break;
			case "H":
				// Hours with no leading zero for single-digit hours, using 24-hour clock
			case "HH":
				// Hours with leading zero for single-digit hours, using 24-hour clock
				ret.push(
					padZeros( value.getHours(), clength )
				);
				break;
			case "m":
				// Minutes with no leading zero for single-digit minutes
			case "mm":
				// Minutes with leading zero for single-digit minutes
				ret.push(
					padZeros( value.getMinutes(), clength )
				);
				break;
			case "s":
				// Seconds with no leading zero for single-digit seconds
			case "ss":
				// Seconds with leading zero for single-digit seconds
				ret.push(
					padZeros( value.getSeconds(), clength )
				);
				break;
			case "t":
				// One character am/pm indicator ("a" or "p")
			case "tt":
				// Multicharacter am/pm indicator
				part = value.getHours() < 12 ? ( cal.AM ? cal.AM[0] : " " ) : ( cal.PM ? cal.PM[0] : " " );
				ret.push( clength === 1 ? part.charAt(0) : part );
				break;
			case "f":
				// Deciseconds
			case "ff":
				// Centiseconds
			case "fff":
				// Milliseconds
				ret.push(
					padZeros( value.getMilliseconds(), 3 ).substr( 0, clength )
				);
				break;
			case "z":
				// Time zone offset, no leading zero
			case "zz":
				// Time zone offset with leading zero
				hour = value.getTimezoneOffset() / 60;
				ret.push(
					( hour <= 0 ? "+" : "-" ) + padZeros( Math.floor(Math.abs(hour)), clength )
				);
				break;
			case "zzz":
				// Time zone offset with leading zero
				hour = value.getTimezoneOffset() / 60;
				ret.push(
					( hour <= 0 ? "+" : "-" ) + padZeros( Math.floor(Math.abs(hour)), 2 ) +
					// Hard coded ":" separator, rather than using cal.TimeSeparator
					// Repeated here for consistency, plus ":" was already assumed in date parsing.
					":" + padZeros( Math.abs(value.getTimezoneOffset() % 60), 2 )
				);
				break;
			case "g":
			case "gg":
				if ( cal.eras ) {
					ret.push(
						cal.eras[ getEra(value, eras) ].name
					);
				}
				break;
		case "/":
			ret.push( cal["/"] );
			break;
		default:
			throw "Invalid date format pattern \'" + current + "\'.";
		}
	}
	return ret.join( "" );
};

// formatNumber
(function() {
	var expandNumber;

	expandNumber = function( number, precision, formatInfo ) {
		var groupSizes = formatInfo.groupSizes,
			curSize = groupSizes[ 0 ],
			curGroupIndex = 1,
			factor = Math.pow( 10, precision ),
			rounded = Math.round( number * factor ) / factor;

		if ( !isFinite(rounded) ) {
			rounded = number;
		}
		number = rounded;

		var numberString = number+"",
			right = "",
			split = numberString.split( /e/i ),
			exponent = split.length > 1 ? parseInt( split[1], 10 ) : 0;
		numberString = split[ 0 ];
		split = numberString.split( "." );
		numberString = split[ 0 ];
		right = split.length > 1 ? split[ 1 ] : "";

		var l;
		if ( exponent > 0 ) {
			right = zeroPad( right, exponent, false );
			numberString += right.slice( 0, exponent );
			right = right.substr( exponent );
		}
		else if ( exponent < 0 ) {
			exponent = -exponent;
			numberString = zeroPad( numberString, exponent + 1, true );
			right = numberString.slice( -exponent, numberString.length ) + right;
			numberString = numberString.slice( 0, -exponent );
		}

		if ( precision > 0 ) {
			right = formatInfo[ "." ] +
				( (right.length > precision) ? right.slice(0, precision) : zeroPad(right, precision) );
		}
		else {
			right = "";
		}

		var stringIndex = numberString.length - 1,
			sep = formatInfo[ "," ],
			ret = "";

		while ( stringIndex >= 0 ) {
			if ( curSize === 0 || curSize > stringIndex ) {
				return numberString.slice( 0, stringIndex + 1 ) + ( ret.length ? (sep + ret + right) : right );
			}
			ret = numberString.slice( stringIndex - curSize + 1, stringIndex + 1 ) + ( ret.length ? (sep + ret) : "" );

			stringIndex -= curSize;

			if ( curGroupIndex < groupSizes.length ) {
				curSize = groupSizes[ curGroupIndex ];
				curGroupIndex++;
			}
		}

		return numberString.slice( 0, stringIndex + 1 ) + sep + ret + right;
	};

	formatNumber = function( value, format, culture ) {
		if ( !isFinite(value) ) {
			if ( value === Infinity ) {
				return culture.numberFormat.positiveInfinity;
			}
			if ( value === -Infinity ) {
				return culture.numberFormat.negativeInfinity;
			}
			return culture.numberFormat[ "NaN" ];
		}
		if ( !format || format === "i" ) {
			return culture.name.length ? value.toLocaleString() : value.toString();
		}
		format = format || "D";

		var nf = culture.numberFormat,
			number = Math.abs( value ),
			precision = -1,
			pattern;
		if ( format.length > 1 ) precision = parseInt( format.slice(1), 10 );

		var current = format.charAt( 0 ).toUpperCase(),
			formatInfo;

		switch ( current ) {
			case "D":
				pattern = "n";
				number = truncate( number );
				if ( precision !== -1 ) {
					number = zeroPad( "" + number, precision, true );
				}
				if ( value < 0 ) number = "-" + number;
				break;
			case "N":
				formatInfo = nf;
				/* falls through */
			case "C":
				formatInfo = formatInfo || nf.currency;
				/* falls through */
			case "P":
				formatInfo = formatInfo || nf.percent;
				pattern = value < 0 ? formatInfo.pattern[ 0 ] : ( formatInfo.pattern[1] || "n" );
				if ( precision === -1 ) precision = formatInfo.decimals;
				number = expandNumber( number * (current === "P" ? 100 : 1), precision, formatInfo );
				break;
			default:
				throw "Bad number format specifier: " + current;
		}

		var patternParts = /n|\$|-|%/g,
			ret = "";
		for ( ; ; ) {
			var index = patternParts.lastIndex,
				ar = patternParts.exec( pattern );

			ret += pattern.slice( index, ar ? ar.index : pattern.length );

			if ( !ar ) {
				break;
			}

			switch ( ar[0] ) {
				case "n":
					ret += number;
					break;
				case "$":
					ret += nf.currency.symbol;
					break;
				case "-":
					// don't make 0 negative
					if ( /[1-9]/.test(number) ) {
						ret += nf[ "-" ];
					}
					break;
				case "%":
					ret += nf.percent.symbol;
					break;
			}
		}

		return ret;
	};

}());

getTokenRegExp = function() {
	// regular expression for matching date and time tokens in format strings.
	return (/\/|dddd|ddd|dd|d|MMMM|MMM|MM|M|yyyy|yy|y|hh|h|HH|H|mm|m|ss|s|tt|t|fff|ff|f|zzz|zz|z|gg|g/g);
};

getEra = function( date, eras ) {
	if ( !eras ) return 0;
	var start, ticks = date.getTime();
	for ( var i = 0, l = eras.length; i < l; i++ ) {
		start = eras[ i ].start;
		if ( start === null || ticks >= start ) {
			return i;
		}
	}
	return 0;
};

getEraYear = function( date, cal, era, sortable ) {
	var year = date.getFullYear();
	if ( !sortable && cal.eras ) {
		// convert normal gregorian year to era-shifted gregorian
		// year by subtracting the era offset
		year -= cal.eras[ era ].offset;
	}
	return year;
};

// parseExact
(function() {
	var expandYear,
		getDayIndex,
		getMonthIndex,
		getParseRegExp,
		outOfRange,
		toUpper,
		toUpperArray;

	expandYear = function( cal, year ) {
		// expands 2-digit year into 4 digits.
		if ( year < 100 ) {
			var now = new Date(),
				era = getEra( now ),
				curr = getEraYear( now, cal, era ),
				twoDigitYearMax = cal.twoDigitYearMax;
			twoDigitYearMax = typeof twoDigitYearMax === "string" ? new Date().getFullYear() % 100 + parseInt( twoDigitYearMax, 10 ) : twoDigitYearMax;
			year += curr - ( curr % 100 );
			if ( year > twoDigitYearMax ) {
				year -= 100;
			}
		}
		return year;
	};

	getDayIndex = function	( cal, value, abbr ) {
		var ret,
			days = cal.days,
			upperDays = cal._upperDays;
		if ( !upperDays ) {
			cal._upperDays = upperDays = [
				toUpperArray( days.names ),
				toUpperArray( days.namesAbbr ),
				toUpperArray( days.namesShort )
			];
		}
		value = toUpper( value );
		if ( abbr ) {
			ret = arrayIndexOf( upperDays[1], value );
			if ( ret === -1 ) {
				ret = arrayIndexOf( upperDays[2], value );
			}
		}
		else {
			ret = arrayIndexOf( upperDays[0], value );
		}
		return ret;
	};

	getMonthIndex = function( cal, value, abbr ) {
		var months = cal.months,
			monthsGen = cal.monthsGenitive || cal.months,
			upperMonths = cal._upperMonths,
			upperMonthsGen = cal._upperMonthsGen;
		if ( !upperMonths ) {
			cal._upperMonths = upperMonths = [
				toUpperArray( months.names ),
				toUpperArray( months.namesAbbr )
			];
			cal._upperMonthsGen = upperMonthsGen = [
				toUpperArray( monthsGen.names ),
				toUpperArray( monthsGen.namesAbbr )
			];
		}
		value = toUpper( value );
		var i = arrayIndexOf( abbr ? upperMonths[1] : upperMonths[0], value );
		if ( i < 0 ) {
			i = arrayIndexOf( abbr ? upperMonthsGen[1] : upperMonthsGen[0], value );
		}
		return i;
	};

	getParseRegExp = function( cal, format ) {
		// converts a format string into a regular expression with groups that
		// can be used to extract date fields from a date string.
		// check for a cached parse regex.
		var re = cal._parseRegExp;
		if ( !re ) {
			cal._parseRegExp = re = {};
		}
		else {
			var reFormat = re[ format ];
			if ( reFormat ) {
				return reFormat;
			}
		}

		// expand single digit formats, then escape regular expression characters.
		var expFormat = expandFormat( cal, format ).replace( /([\^\$\.\*\+\?\|\[\]\(\)\{\}])/g, "\\\\$1" ),
			regexp = [ "^" ],
			groups = [],
			index = 0,
			quoteCount = 0,
			tokenRegExp = getTokenRegExp(),
			match;

		// iterate through each date token found.
		while ( (match = tokenRegExp.exec(expFormat)) !== null ) {
			var preMatch = expFormat.slice( index, match.index );
			index = tokenRegExp.lastIndex;

			// don't replace any matches that occur inside a string literal.
			quoteCount += appendPreOrPostMatch( preMatch, regexp );
			if ( quoteCount % 2 ) {
				regexp.push( match[0] );
				continue;
			}

			// add a regex group for the token.
			var m = match[ 0 ],
				len = m.length,
				add;
			switch ( m ) {
				case "dddd": case "ddd":
				case "MMMM": case "MMM":
				case "gg": case "g":
					add = "(\\D+)";
					break;
				case "tt": case "t":
					add = "(\\D*)";
					break;
				case "yyyy":
				case "fff":
				case "ff":
				case "f":
					add = "(\\d{" + len + "})";
					break;
				case "dd": case "d":
				case "MM": case "M":
				case "yy": case "y":
				case "HH": case "H":
				case "hh": case "h":
				case "mm": case "m":
				case "ss": case "s":
					add = "(\\d\\d?)";
					break;
				case "zzz":
					add = "([+-]?\\d\\d?:\\d{2})";
					break;
				case "zz": case "z":
					add = "([+-]?\\d\\d?)";
					break;
				case "/":
					add = "(\\/)";
					break;
				default:
					throw "Invalid date format pattern \'" + m + "\'.";
			}
			if ( add ) {
				regexp.push( add );
			}
			groups.push( match[0] );
		}
		appendPreOrPostMatch( expFormat.slice(index), regexp );
		regexp.push( "$" );

		// allow whitespace to differ when matching formats.
		var regexpStr = regexp.join( "" ).replace( /\s+/g, "\\s+" ),
			parseRegExp = { "regExp": regexpStr, "groups": groups };

		// cache the regex for this format.
		return re[ format ] = parseRegExp;
	};

	outOfRange = function( value, low, high ) {
		return value < low || value > high;
	};

	toUpper = function( value ) {
		// "he-IL" has non-breaking space in weekday names.
		return value.split( "\u00A0" ).join( " " ).toUpperCase();
	};

	toUpperArray = function( arr ) {
		var results = [];
		for ( var i = 0, l = arr.length; i < l; i++ ) {
			results[ i ] = toUpper( arr[i] );
		}
		return results;
	};

	parseExact = function( value, format, culture ) {
		// try to parse the date string by matching against the format string
		// while using the specified culture for date field names.
		value = trim( value );
		var cal = culture.calendar,
			// convert date formats into regular expressions with groupings.
			// use the regexp to determine the input format and extract the date fields.
			parseInfo = getParseRegExp( cal, format ),
			match = new RegExp( parseInfo.regExp ).exec( value );
		if ( match === null ) {
			return null;
		}
		// found a date format that matches the input.
		var groups = parseInfo.groups,
			era = null, year = null, month = null, date = null, weekDay = null,
			hour = 0, hourOffset, min = 0, sec = 0, msec = 0, tzMinOffset = null,
			pmHour = false;
		// iterate the format groups to extract and set the date fields.
		for ( var j = 0, jl = groups.length; j < jl; j++ ) {
			var matchGroup = match[ j + 1 ];
			if ( matchGroup ) {
				var current = groups[ j ],
					clength = current.length,
					matchInt = parseInt( matchGroup, 10 );
				switch ( current ) {
					case "dd": case "d":
						// Day of month.
						date = matchInt;
						// check that date is generally in valid range, also checking overflow below.
						if ( outOfRange(date, 1, 31) ) return null;
						break;
					case "MMM": case "MMMM":
						month = getMonthIndex( cal, matchGroup, clength === 3 );
						if ( outOfRange(month, 0, 11) ) return null;
						break;
					case "M": case "MM":
						// Month.
						month = matchInt - 1;
						if ( outOfRange(month, 0, 11) ) return null;
						break;
					case "y": case "yy":
					case "yyyy":
						year = clength < 4 ? expandYear( cal, matchInt ) : matchInt;
						if ( outOfRange(year, 0, 9999) ) return null;
						break;
					case "h": case "hh":
						// Hours (12-hour clock).
						hour = matchInt;
						if ( hour === 12 ) hour = 0;
						if ( outOfRange(hour, 0, 11) ) return null;
						break;
					case "H": case "HH":
						// Hours (24-hour clock).
						hour = matchInt;
						if ( outOfRange(hour, 0, 23) ) return null;
						break;
					case "m": case "mm":
						// Minutes.
						min = matchInt;
						if ( outOfRange(min, 0, 59) ) return null;
						break;
					case "s": case "ss":
						// Seconds.
						sec = matchInt;
						if ( outOfRange(sec, 0, 59) ) return null;
						break;
					case "tt": case "t":
						// AM/PM designator.
						// see if it is standard, upper, or lower case PM. If not, ensure it is at least one of
						// the AM tokens. If not, fail the parse for this format.
						pmHour = cal.PM && ( matchGroup === cal.PM[0] || matchGroup === cal.PM[1] || matchGroup === cal.PM[2] );
						if (
							!pmHour && (
								!cal.AM || ( matchGroup !== cal.AM[0] && matchGroup !== cal.AM[1] && matchGroup !== cal.AM[2] )
							)
						) return null;
						break;
					case "f":
						// Deciseconds.
					case "ff":
						// Centiseconds.
					case "fff":
						// Milliseconds.
						msec = matchInt * Math.pow( 10, 3 - clength );
						if ( outOfRange(msec, 0, 999) ) return null;
						break;
					case "ddd":
						// Day of week.
					case "dddd":
						// Day of week.
						weekDay = getDayIndex( cal, matchGroup, clength === 3 );
						if ( outOfRange(weekDay, 0, 6) ) return null;
						break;
					case "zzz":
						// Time zone offset in +/- hours:min.
						var offsets = matchGroup.split( /:/ );
						if ( offsets.length !== 2 ) return null;
						hourOffset = parseInt( offsets[0], 10 );
						if ( outOfRange(hourOffset, -12, 13) ) return null;
						var minOffset = parseInt( offsets[1], 10 );
						if ( outOfRange(minOffset, 0, 59) ) return null;
						tzMinOffset = ( hourOffset * 60 ) + ( startsWith(matchGroup, "-") ? -minOffset : minOffset );
						break;
					case "z": case "zz":
						// Time zone offset in +/- hours.
						hourOffset = matchInt;
						if ( outOfRange(hourOffset, -12, 13) ) return null;
						tzMinOffset = hourOffset * 60;
						break;
					case "g": case "gg":
						var eraName = matchGroup;
						if ( !eraName || !cal.eras ) return null;
						eraName = trim( eraName.toLowerCase() );
						for ( var i = 0, l = cal.eras.length; i < l; i++ ) {
							if ( eraName === cal.eras[i].name.toLowerCase() ) {
								era = i;
								break;
							}
						}
						// could not find an era with that name
						if ( era === null ) return null;
						break;
				}
			}
		}
		var result = new Date(), defaultYear, convert = cal.convert;
		defaultYear = convert ? convert.fromGregorian( result )[ 0 ] : result.getFullYear();
		if ( year === null ) {
			year = defaultYear;
		}
		else if ( cal.eras ) {
			// year must be shifted to normal gregorian year
			// but not if year was not specified, its already normal gregorian
			// per the main if clause above.
			year += cal.eras[( era || 0 )].offset;
		}
		// set default day and month to 1 and January, so if unspecified, these are the defaults
		// instead of the current day/month.
		if ( month === null ) {
			month = 0;
		}
		if ( date === null ) {
			date = 1;
		}
		// now have year, month, and date, but in the culture's calendar.
		// convert to gregorian if necessary
		if ( convert ) {
			result = convert.toGregorian( year, month, date );
			// conversion failed, must be an invalid match
			if ( result === null ) return null;
		}
		else {
			// have to set year, month and date together to avoid overflow based on current date.
			result.setFullYear( year, month, date );
			// check to see if date overflowed for specified month (only checked 1-31 above).
			if ( result.getDate() !== date ) return null;
			// invalid day of week.
			if ( weekDay !== null && result.getDay() !== weekDay ) {
				return null;
			}
		}
		// if pm designator token was found make sure the hours fit the 24-hour clock.
		if ( pmHour && hour < 12 ) {
			hour += 12;
		}
		result.setHours( hour, min, sec, msec );
		if ( tzMinOffset !== null ) {
			// adjust timezone to utc before applying local offset.
			var adjustedMin = result.getMinutes() - ( tzMinOffset + result.getTimezoneOffset() );
			// Safari limits hours and minutes to the range of -127 to 127.  We need to use setHours
			// to ensure both these fields will not exceed this range.	adjustedMin will range
			// somewhere between -1440 and 1500, so we only need to split this into hours.
			result.setHours( result.getHours() + parseInt(adjustedMin / 60, 10), adjustedMin % 60 );
		}
		return result;
	};
}());

parseNegativePattern = function( value, nf, negativePattern ) {
	var neg = nf[ "-" ],
		pos = nf[ "+" ],
		ret;
	switch ( negativePattern ) {
		case "n -":
			neg = " " + neg;
			pos = " " + pos;
			/* falls through */
		case "n-":
			if ( endsWith(value, neg) ) {
				ret = [ "-", value.substr(0, value.length - neg.length) ];
			}
			else if ( endsWith(value, pos) ) {
				ret = [ "+", value.substr(0, value.length - pos.length) ];
			}
			break;
		case "- n":
			neg += " ";
			pos += " ";
			/* falls through */
		case "-n":
			if ( startsWith(value, neg) ) {
				ret = [ "-", value.substr(neg.length) ];
			}
			else if ( startsWith(value, pos) ) {
				ret = [ "+", value.substr(pos.length) ];
			}
			break;
		case "(n)":
			if ( startsWith(value, "(") && endsWith(value, ")") ) {
				ret = [ "-", value.substr(1, value.length - 2) ];
			}
			break;
	}
	return ret || [ "", value ];
};

//
// public instance functions
//

Globalize.prototype.findClosestCulture = function( cultureSelector ) {
	return Globalize.findClosestCulture.call( this, cultureSelector );
};

Globalize.prototype.format = function( value, format, cultureSelector ) {
	return Globalize.format.call( this, value, format, cultureSelector );
};

Globalize.prototype.localize = function( key, cultureSelector ) {
	return Globalize.localize.call( this, key, cultureSelector );
};

Globalize.prototype.parseInt = function( value, radix, cultureSelector ) {
	return Globalize.parseInt.call( this, value, radix, cultureSelector );
};

Globalize.prototype.parseFloat = function( value, radix, cultureSelector ) {
	return Globalize.parseFloat.call( this, value, radix, cultureSelector );
};

Globalize.prototype.culture = function( cultureSelector ) {
	return Globalize.culture.call( this, cultureSelector );
};

//
// public singleton functions
//

Globalize.addCultureInfo = function( cultureName, baseCultureName, info ) {

	var base = {},
		isNew = false;

	if ( typeof cultureName !== "string" ) {
		// cultureName argument is optional string. If not specified, assume info is first
		// and only argument. Specified info deep-extends current culture.
		info = cultureName;
		cultureName = this.culture().name;
		base = this.cultures[ cultureName ];
	} else if ( typeof baseCultureName !== "string" ) {
		// baseCultureName argument is optional string. If not specified, assume info is second
		// argument. Specified info deep-extends specified culture.
		// If specified culture does not exist, create by deep-extending default
		info = baseCultureName;
		isNew = ( this.cultures[ cultureName ] == null );
		base = this.cultures[ cultureName ] || this.cultures[ "default" ];
	} else {
		// cultureName and baseCultureName specified. Assume a new culture is being created
		// by deep-extending an specified base culture
		isNew = true;
		base = this.cultures[ baseCultureName ];
	}

	this.cultures[ cultureName ] = extend(true, {},
		base,
		info
	);
	// Make the standard calendar the current culture if it's a new culture
	if ( isNew ) {
		this.cultures[ cultureName ].calendar = this.cultures[ cultureName ].calendars.standard;
	}
};

Globalize.findClosestCulture = function( name ) {
	var match;
	if ( !name ) {
		return this.findClosestCulture( this.cultureSelector ) || this.cultures[ "default" ];
	}
	if ( typeof name === "string" ) {
		name = name.split( "," );
	}
	if ( isArray(name) ) {
		var lang,
			cultures = this.cultures,
			list = name,
			i, l = list.length,
			prioritized = [];
		for ( i = 0; i < l; i++ ) {
			name = trim( list[i] );
			var pri, parts = name.split( ";" );
			lang = trim( parts[0] );
			if ( parts.length === 1 ) {
				pri = 1;
			}
			else {
				name = trim( parts[1] );
				if ( name.indexOf("q=") === 0 ) {
					name = name.substr( 2 );
					pri = parseFloat( name );
					pri = isNaN( pri ) ? 0 : pri;
				}
				else {
					pri = 1;
				}
			}
			prioritized.push({ lang: lang, pri: pri });
		}
		prioritized.sort(function( a, b ) {
			if ( a.pri < b.pri ) {
				return 1;
			} else if ( a.pri > b.pri ) {
				return -1;
			}
			return 0;
		});
		// exact match
		for ( i = 0; i < l; i++ ) {
			lang = prioritized[ i ].lang;
			match = cultures[ lang ];
			if ( match ) {
				return match;
			}
		}

		// neutral language match
		for ( i = 0; i < l; i++ ) {
			lang = prioritized[ i ].lang;
			do {
				var index = lang.lastIndexOf( "-" );
				if ( index === -1 ) {
					break;
				}
				// strip off the last part. e.g. en-US => en
				lang = lang.substr( 0, index );
				match = cultures[ lang ];
				if ( match ) {
					return match;
				}
			}
			while ( 1 );
		}

		// last resort: match first culture using that language
		for ( i = 0; i < l; i++ ) {
			lang = prioritized[ i ].lang;
			for ( var cultureKey in cultures ) {
				var culture = cultures[ cultureKey ];
				if ( culture.language == lang ) {
					return culture;
				}
			}
		}
	}
	else if ( typeof name === "object" ) {
		return name;
	}
	return match || null;
};

Globalize.format = function( value, format, cultureSelector ) {
	var culture = this.findClosestCulture( cultureSelector );
	if ( value instanceof Date ) {
		value = formatDate( value, format, culture );
	}
	else if ( typeof value === "number" ) {
		value = formatNumber( value, format, culture );
	}
	return value;
};

Globalize.localize = function( key, cultureSelector ) {
	return this.findClosestCulture( cultureSelector ).messages[ key ] ||
		this.cultures[ "default" ].messages[ key ];
};

Globalize.parseDate = function( value, formats, culture ) {
	culture = this.findClosestCulture( culture );

	var date, prop, patterns;
	if ( formats ) {
		if ( typeof formats === "string" ) {
			formats = [ formats ];
		}
		if ( formats.length ) {
			for ( var i = 0, l = formats.length; i < l; i++ ) {
				var format = formats[ i ];
				if ( format ) {
					date = parseExact( value, format, culture );
					if ( date ) {
						break;
					}
				}
			}
		}
	} else {
		patterns = culture.calendar.patterns;
		for ( prop in patterns ) {
			date = parseExact( value, patterns[prop], culture );
			if ( date ) {
				break;
			}
		}
	}

	return date || null;
};

Globalize.parseInt = function( value, radix, cultureSelector ) {
	return truncate( Globalize.parseFloat(value, radix, cultureSelector) );
};

Globalize.parseFloat = function( value, radix, cultureSelector ) {
	// radix argument is optional
	if ( typeof radix !== "number" ) {
		cultureSelector = radix;
		radix = 10;
	}

	var culture = this.findClosestCulture( cultureSelector );
	var ret = NaN,
		nf = culture.numberFormat;

	if ( value.indexOf(culture.numberFormat.currency.symbol) > -1 ) {
		// remove currency symbol
		value = value.replace( culture.numberFormat.currency.symbol, "" );
		// replace decimal seperator
		value = value.replace( culture.numberFormat.currency["."], culture.numberFormat["."] );
	}

	//Remove percentage character from number string before parsing
	if ( value.indexOf(culture.numberFormat.percent.symbol) > -1){
		value = value.replace( culture.numberFormat.percent.symbol, "" );
	}

	// remove spaces: leading, trailing and between - and number. Used for negative currency pt-BR
	value = value.replace( / /g, "" );

	// allow infinity or hexidecimal
	if ( regexInfinity.test(value) ) {
		ret = parseFloat( value );
	}
	else if ( !radix && regexHex.test(value) ) {
		ret = parseInt( value, 16 );
	}
	else {

		// determine sign and number
		var signInfo = parseNegativePattern( value, nf, nf.pattern[0] ),
			sign = signInfo[ 0 ],
			num = signInfo[ 1 ];

		// #44 - try parsing as "(n)"
		if ( sign === "" && nf.pattern[0] !== "(n)" ) {
			signInfo = parseNegativePattern( value, nf, "(n)" );
			sign = signInfo[ 0 ];
			num = signInfo[ 1 ];
		}

		// try parsing as "-n"
		if ( sign === "" && nf.pattern[0] !== "-n" ) {
			signInfo = parseNegativePattern( value, nf, "-n" );
			sign = signInfo[ 0 ];
			num = signInfo[ 1 ];
		}

		sign = sign || "+";

		// determine exponent and number
		var exponent,
			intAndFraction,
			exponentPos = num.indexOf( "e" );
		if ( exponentPos < 0 ) exponentPos = num.indexOf( "E" );
		if ( exponentPos < 0 ) {
			intAndFraction = num;
			exponent = null;
		}
		else {
			intAndFraction = num.substr( 0, exponentPos );
			exponent = num.substr( exponentPos + 1 );
		}
		// determine decimal position
		var integer,
			fraction,
			decSep = nf[ "." ],
			decimalPos = intAndFraction.indexOf( decSep );
		if ( decimalPos < 0 ) {
			integer = intAndFraction;
			fraction = null;
		}
		else {
			integer = intAndFraction.substr( 0, decimalPos );
			fraction = intAndFraction.substr( decimalPos + decSep.length );
		}
		// handle groups (e.g. 1,000,000)
		var groupSep = nf[ "," ];
		integer = integer.split( groupSep ).join( "" );
		var altGroupSep = groupSep.replace( /\u00A0/g, " " );
		if ( groupSep !== altGroupSep ) {
			integer = integer.split( altGroupSep ).join( "" );
		}
		// build a natively parsable number string
		var p = sign + integer;
		if ( fraction !== null ) {
			p += "." + fraction;
		}
		if ( exponent !== null ) {
			// exponent itself may have a number patternd
			var expSignInfo = parseNegativePattern( exponent, nf, "-n" );
			p += "e" + ( expSignInfo[0] || "+" ) + expSignInfo[ 1 ];
		}
		if ( regexParseFloat.test(p) ) {
			ret = parseFloat( p );
		}
	}
	return ret;
};

Globalize.culture = function( cultureSelector ) {
	// setter
	if ( typeof cultureSelector !== "undefined" ) {
		this.cultureSelector = cultureSelector;
	}
	// getter
	return this.findClosestCulture( cultureSelector ) || this.cultures[ "default" ];
};

}( this ));

define("globalize_lib", function(){});

/*
Copyright (c) 2003-2015, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.md or http://ckeditor.com/license
*/
(function(){if(!window.CKEDITOR||!window.CKEDITOR.dom)window.CKEDITOR||(window.CKEDITOR=function(){var a=/(^|.*[\\\/])ckeditor\.js(?:\?.*|;.*)?$/i,e={timestamp:"F0RF",version:"4.4.7 (Standard)",revision:"3a35b3d",rnd:Math.floor(900*Math.random())+100,_:{pending:[],basePathSrcPattern:a},status:"unloaded",basePath:function(){var c=window.CKEDITOR_BASEPATH||"";if(!c)for(var d=document.getElementsByTagName("script"),b=0;b<d.length;b++){var f=d[b].src.match(a);if(f){c=f[1];break}}-1==c.indexOf(":/")&&
"//"!=c.slice(0,2)&&(c=0===c.indexOf("/")?location.href.match(/^.*?:\/\/[^\/]*/)[0]+c:location.href.match(/^[^\?]*\/(?:)/)[0]+c);if(!c)throw'The CKEditor installation path could not be automatically detected. Please set the global variable "CKEDITOR_BASEPATH" before creating editor instances.';return c}(),getUrl:function(a){-1==a.indexOf(":/")&&0!==a.indexOf("/")&&(a=this.basePath+a);this.timestamp&&("/"!=a.charAt(a.length-1)&&!/[&?]t=/.test(a))&&(a+=(0<=a.indexOf("?")?"&":"?")+"t="+this.timestamp);
return a},domReady:function(){function a(){try{document.addEventListener?(document.removeEventListener("DOMContentLoaded",a,!1),d()):document.attachEvent&&"complete"===document.readyState&&(document.detachEvent("onreadystatechange",a),d())}catch(b){}}function d(){for(var a;a=b.shift();)a()}var b=[];return function(d){function g(){try{document.documentElement.doScroll("left")}catch(k){setTimeout(g,1);return}a()}b.push(d);"complete"===document.readyState&&setTimeout(a,1);if(1==b.length)if(document.addEventListener)document.addEventListener("DOMContentLoaded",
a,!1),window.addEventListener("load",a,!1);else if(document.attachEvent){document.attachEvent("onreadystatechange",a);window.attachEvent("onload",a);d=!1;try{d=!window.frameElement}catch(e){}document.documentElement.doScroll&&d&&g()}}}()},b=window.CKEDITOR_GETURL;if(b){var g=e.getUrl;e.getUrl=function(a){return b.call(e,a)||g.call(e,a)}}return e}()),CKEDITOR.event||(CKEDITOR.event=function(){},CKEDITOR.event.implementOn=function(a){var e=CKEDITOR.event.prototype,b;for(b in e)a[b]==null&&(a[b]=e[b])},
CKEDITOR.event.prototype=function(){function a(a){var c=e(this);return c[a]||(c[a]=new b(a))}var e=function(a){a=a.getPrivate&&a.getPrivate()||a._||(a._={});return a.events||(a.events={})},b=function(a){this.name=a;this.listeners=[]};b.prototype={getListenerIndex:function(a){for(var c=0,d=this.listeners;c<d.length;c++)if(d[c].fn==a)return c;return-1}};return{define:function(b,c){var d=a.call(this,b);CKEDITOR.tools.extend(d,c,true)},on:function(b,c,d,j,f){function h(a,k,f,h){a={name:b,sender:this,
editor:a,data:k,listenerData:j,stop:f,cancel:h,removeListener:e};return c.call(d,a)===false?false:a.data}function e(){n.removeListener(b,c)}var k=a.call(this,b);if(k.getListenerIndex(c)<0){k=k.listeners;d||(d=this);isNaN(f)&&(f=10);var n=this;h.fn=c;h.priority=f;for(var o=k.length-1;o>=0;o--)if(k[o].priority<=f){k.splice(o+1,0,h);return{removeListener:e}}k.unshift(h)}return{removeListener:e}},once:function(){var a=Array.prototype.slice.call(arguments),c=a[1];a[1]=function(a){a.removeListener();return c.apply(this,
arguments)};return this.on.apply(this,a)},capture:function(){CKEDITOR.event.useCapture=1;var a=this.on.apply(this,arguments);CKEDITOR.event.useCapture=0;return a},fire:function(){var a=0,c=function(){a=1},d=0,b=function(){d=1};return function(f,h,i){var k=e(this)[f],f=a,n=d;a=d=0;if(k){var o=k.listeners;if(o.length)for(var o=o.slice(0),q,m=0;m<o.length;m++){if(k.errorProof)try{q=o[m].call(this,i,h,c,b)}catch(l){}else q=o[m].call(this,i,h,c,b);q===false?d=1:typeof q!="undefined"&&(h=q);if(a||d)break}}h=
d?false:typeof h=="undefined"?true:h;a=f;d=n;return h}}(),fireOnce:function(a,c,d){c=this.fire(a,c,d);delete e(this)[a];return c},removeListener:function(a,c){var d=e(this)[a];if(d){var b=d.getListenerIndex(c);b>=0&&d.listeners.splice(b,1)}},removeAllListeners:function(){var a=e(this),c;for(c in a)delete a[c]},hasListeners:function(a){return(a=e(this)[a])&&a.listeners.length>0}}}()),CKEDITOR.editor||(CKEDITOR.editor=function(){CKEDITOR._.pending.push([this,arguments]);CKEDITOR.event.call(this)},CKEDITOR.editor.prototype.fire=
function(a,e){a in{instanceReady:1,loaded:1}&&(this[a]=true);return CKEDITOR.event.prototype.fire.call(this,a,e,this)},CKEDITOR.editor.prototype.fireOnce=function(a,e){a in{instanceReady:1,loaded:1}&&(this[a]=true);return CKEDITOR.event.prototype.fireOnce.call(this,a,e,this)},CKEDITOR.event.implementOn(CKEDITOR.editor.prototype)),CKEDITOR.env||(CKEDITOR.env=function(){var a=navigator.userAgent.toLowerCase(),e={ie:a.indexOf("trident/")>-1,webkit:a.indexOf(" applewebkit/")>-1,air:a.indexOf(" adobeair/")>
-1,mac:a.indexOf("macintosh")>-1,quirks:document.compatMode=="BackCompat"&&(!document.documentMode||document.documentMode<10),mobile:a.indexOf("mobile")>-1,iOS:/(ipad|iphone|ipod)/.test(a),isCustomDomain:function(){if(!this.ie)return false;var a=document.domain,d=window.location.hostname;return a!=d&&a!="["+d+"]"},secure:location.protocol=="https:"};e.gecko=navigator.product=="Gecko"&&!e.webkit&&!e.ie;if(e.webkit)a.indexOf("chrome")>-1?e.chrome=true:e.safari=true;var b=0;if(e.ie){b=e.quirks||!document.documentMode?
parseFloat(a.match(/msie (\d+)/)[1]):document.documentMode;e.ie9Compat=b==9;e.ie8Compat=b==8;e.ie7Compat=b==7;e.ie6Compat=b<7||e.quirks}if(e.gecko){var g=a.match(/rv:([\d\.]+)/);if(g){g=g[1].split(".");b=g[0]*1E4+(g[1]||0)*100+(g[2]||0)*1}}e.air&&(b=parseFloat(a.match(/ adobeair\/(\d+)/)[1]));e.webkit&&(b=parseFloat(a.match(/ applewebkit\/(\d+)/)[1]));e.version=b;e.isCompatible=e.iOS&&b>=534||!e.mobile&&(e.ie&&b>6||e.gecko&&b>=2E4||e.air&&b>=1||e.webkit&&b>=522||false);e.hidpi=window.devicePixelRatio>=
2;e.needsBrFiller=e.gecko||e.webkit||e.ie&&b>10;e.needsNbspFiller=e.ie&&b<11;e.cssClass="cke_browser_"+(e.ie?"ie":e.gecko?"gecko":e.webkit?"webkit":"unknown");if(e.quirks)e.cssClass=e.cssClass+" cke_browser_quirks";if(e.ie)e.cssClass=e.cssClass+(" cke_browser_ie"+(e.quirks?"6 cke_browser_iequirks":e.version));if(e.air)e.cssClass=e.cssClass+" cke_browser_air";if(e.iOS)e.cssClass=e.cssClass+" cke_browser_ios";if(e.hidpi)e.cssClass=e.cssClass+" cke_hidpi";return e}()),"unloaded"==CKEDITOR.status&&function(){CKEDITOR.event.implementOn(CKEDITOR);
CKEDITOR.loadFullCore=function(){if(CKEDITOR.status!="basic_ready")CKEDITOR.loadFullCore._load=1;else{delete CKEDITOR.loadFullCore;var a=document.createElement("script");a.type="text/javascript";a.src=CKEDITOR.basePath+"ckeditor.js";document.getElementsByTagName("head")[0].appendChild(a)}};CKEDITOR.loadFullCoreTimeout=0;CKEDITOR.add=function(a){(this._.pending||(this._.pending=[])).push(a)};(function(){CKEDITOR.domReady(function(){var a=CKEDITOR.loadFullCore,e=CKEDITOR.loadFullCoreTimeout;if(a){CKEDITOR.status=
"basic_ready";a&&a._load?a():e&&setTimeout(function(){CKEDITOR.loadFullCore&&CKEDITOR.loadFullCore()},e*1E3)}})})();CKEDITOR.status="basic_loaded"}(),CKEDITOR.dom={},function(){var a=[],e=CKEDITOR.env.gecko?"-moz-":CKEDITOR.env.webkit?"-webkit-":CKEDITOR.env.ie?"-ms-":"",b=/&/g,g=/>/g,c=/</g,d=/"/g,j=/&amp;/g,f=/&gt;/g,h=/&lt;/g,i=/&quot;/g;CKEDITOR.on("reset",function(){a=[]});CKEDITOR.tools={arrayCompare:function(a,c){if(!a&&!c)return true;if(!a||!c||a.length!=c.length)return false;for(var d=0;d<
a.length;d++)if(a[d]!=c[d])return false;return true},clone:function(a){var c;if(a&&a instanceof Array){c=[];for(var d=0;d<a.length;d++)c[d]=CKEDITOR.tools.clone(a[d]);return c}if(a===null||typeof a!="object"||a instanceof String||a instanceof Number||a instanceof Boolean||a instanceof Date||a instanceof RegExp||a.nodeType||a.window===a)return a;c=new a.constructor;for(d in a)c[d]=CKEDITOR.tools.clone(a[d]);return c},capitalize:function(a,c){return a.charAt(0).toUpperCase()+(c?a.slice(1):a.slice(1).toLowerCase())},
extend:function(a){var c=arguments.length,d,b;if(typeof(d=arguments[c-1])=="boolean")c--;else if(typeof(d=arguments[c-2])=="boolean"){b=arguments[c-1];c=c-2}for(var f=1;f<c;f++){var g=arguments[f],j;for(j in g)if(d===true||a[j]==null)if(!b||j in b)a[j]=g[j]}return a},prototypedCopy:function(a){var c=function(){};c.prototype=a;return new c},copy:function(a){var c={},d;for(d in a)c[d]=a[d];return c},isArray:function(a){return Object.prototype.toString.call(a)=="[object Array]"},isEmpty:function(a){for(var c in a)if(a.hasOwnProperty(c))return false;
return true},cssVendorPrefix:function(a,c,d){if(d)return e+a+":"+c+";"+a+":"+c;d={};d[a]=c;d[e+a]=c;return d},cssStyleToDomStyle:function(){var a=document.createElement("div").style,c=typeof a.cssFloat!="undefined"?"cssFloat":typeof a.styleFloat!="undefined"?"styleFloat":"float";return function(a){return a=="float"?c:a.replace(/-./g,function(a){return a.substr(1).toUpperCase()})}}(),buildStyleHtml:function(a){for(var a=[].concat(a),c,d=[],b=0;b<a.length;b++)if(c=a[b])/@import|[{}]/.test(c)?d.push("<style>"+
c+"</style>"):d.push('<link type="text/css" rel=stylesheet href="'+c+'">');return d.join("")},htmlEncode:function(a){return(""+a).replace(b,"&amp;").replace(g,"&gt;").replace(c,"&lt;")},htmlDecode:function(a){return a.replace(j,"&").replace(f,">").replace(h,"<")},htmlEncodeAttr:function(a){return a.replace(d,"&quot;").replace(c,"&lt;").replace(g,"&gt;")},htmlDecodeAttr:function(a){return a.replace(i,'"').replace(h,"<").replace(f,">")},getNextNumber:function(){var a=0;return function(){return++a}}(),
getNextId:function(){return"cke_"+this.getNextNumber()},override:function(a,c){var d=c(a);d.prototype=a.prototype;return d},setTimeout:function(a,c,d,b,f){f||(f=window);d||(d=f);return f.setTimeout(function(){b?a.apply(d,[].concat(b)):a.apply(d)},c||0)},trim:function(){var a=/(?:^[ \t\n\r]+)|(?:[ \t\n\r]+$)/g;return function(c){return c.replace(a,"")}}(),ltrim:function(){var a=/^[ \t\n\r]+/g;return function(c){return c.replace(a,"")}}(),rtrim:function(){var a=/[ \t\n\r]+$/g;return function(c){return c.replace(a,
"")}}(),indexOf:function(a,c){if(typeof c=="function")for(var d=0,b=a.length;d<b;d++){if(c(a[d]))return d}else{if(a.indexOf)return a.indexOf(c);d=0;for(b=a.length;d<b;d++)if(a[d]===c)return d}return-1},search:function(a,c){var d=CKEDITOR.tools.indexOf(a,c);return d>=0?a[d]:null},bind:function(a,c){return function(){return a.apply(c,arguments)}},createClass:function(a){var c=a.$,d=a.base,b=a.privates||a._,f=a.proto,a=a.statics;!c&&(c=function(){d&&this.base.apply(this,arguments)});if(b)var g=c,c=function(){var a=
this._||(this._={}),c;for(c in b){var d=b[c];a[c]=typeof d=="function"?CKEDITOR.tools.bind(d,this):d}g.apply(this,arguments)};if(d){c.prototype=this.prototypedCopy(d.prototype);c.prototype.constructor=c;c.base=d;c.baseProto=d.prototype;c.prototype.base=function(){this.base=d.prototype.base;d.apply(this,arguments);this.base=arguments.callee}}f&&this.extend(c.prototype,f,true);a&&this.extend(c,a,true);return c},addFunction:function(c,d){return a.push(function(){return c.apply(d||this,arguments)})-1},
removeFunction:function(c){a[c]=null},callFunction:function(c){var d=a[c];return d&&d.apply(window,Array.prototype.slice.call(arguments,1))},cssLength:function(){var a=/^-?\d+\.?\d*px$/,c;return function(d){c=CKEDITOR.tools.trim(d+"")+"px";return a.test(c)?c:d||""}}(),convertToPx:function(){var a;return function(c){if(!a){a=CKEDITOR.dom.element.createFromHtml('<div style="position:absolute;left:-9999px;top:-9999px;margin:0px;padding:0px;border:0px;"></div>',CKEDITOR.document);CKEDITOR.document.getBody().append(a)}if(!/%$/.test(c)){a.setStyle("width",
c);return a.$.clientWidth}return c}}(),repeat:function(a,c){return Array(c+1).join(a)},tryThese:function(){for(var a,c=0,d=arguments.length;c<d;c++){var b=arguments[c];try{a=b();break}catch(f){}}return a},genKey:function(){return Array.prototype.slice.call(arguments).join("-")},defer:function(a){return function(){var c=arguments,d=this;window.setTimeout(function(){a.apply(d,c)},0)}},normalizeCssText:function(a,c){var d=[],b,f=CKEDITOR.tools.parseCssText(a,true,c);for(b in f)d.push(b+":"+f[b]);d.sort();
return d.length?d.join(";")+";":""},convertRgbToHex:function(a){return a.replace(/(?:rgb\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\))/gi,function(a,c,d,b){a=[c,d,b];for(c=0;c<3;c++)a[c]=("0"+parseInt(a[c],10).toString(16)).slice(-2);return"#"+a.join("")})},parseCssText:function(a,c,d){var b={};if(d){d=new CKEDITOR.dom.element("span");d.setAttribute("style",a);a=CKEDITOR.tools.convertRgbToHex(d.getAttribute("style")||"")}if(!a||a==";")return b;a.replace(/&quot;/g,'"').replace(/\s*([^:;\s]+)\s*:\s*([^;]+)\s*(?=;|$)/g,
function(a,d,f){if(c){d=d.toLowerCase();d=="font-family"&&(f=f.toLowerCase().replace(/["']/g,"").replace(/\s*,\s*/g,","));f=CKEDITOR.tools.trim(f)}b[d]=f});return b},writeCssText:function(a,c){var d,b=[];for(d in a)b.push(d+":"+a[d]);c&&b.sort();return b.join("; ")},objectCompare:function(a,c,d){var b;if(!a&&!c)return true;if(!a||!c)return false;for(b in a)if(a[b]!=c[b])return false;if(!d)for(b in c)if(a[b]!=c[b])return false;return true},objectKeys:function(a){var c=[],d;for(d in a)c.push(d);return c},
convertArrayToObject:function(a,c){var d={};arguments.length==1&&(c=true);for(var b=0,f=a.length;b<f;++b)d[a[b]]=c;return d},fixDomain:function(){for(var a;;)try{a=window.parent.document.domain;break}catch(c){a=a?a.replace(/.+?(?:\.|$)/,""):document.domain;if(!a)break;document.domain=a}return!!a},eventsBuffer:function(a,c){function d(){f=(new Date).getTime();b=false;c()}var b,f=0;return{input:function(){if(!b){var c=(new Date).getTime()-f;c<a?b=setTimeout(d,a-c):d()}},reset:function(){b&&clearTimeout(b);
b=f=0}}},enableHtml5Elements:function(a,c){for(var d=["abbr","article","aside","audio","bdi","canvas","data","datalist","details","figcaption","figure","footer","header","hgroup","mark","meter","nav","output","progress","section","summary","time","video"],b=d.length,f;b--;){f=a.createElement(d[b]);c&&a.appendChild(f)}},checkIfAnyArrayItemMatches:function(a,c){for(var d=0,b=a.length;d<b;++d)if(a[d].match(c))return true;return false},checkIfAnyObjectPropertyMatches:function(a,c){for(var d in a)if(d.match(c))return true;
return false},transparentImageData:"data:image/gif;base64,R0lGODlhAQABAPABAP///wAAACH5BAEKAAAALAAAAAABAAEAAAICRAEAOw=="}}(),CKEDITOR.dtd=function(){var a=CKEDITOR.tools.extend,e=function(a,c){for(var d=CKEDITOR.tools.clone(a),b=1;b<arguments.length;b++){var c=arguments[b],f;for(f in c)delete d[f]}return d},b={},g={},c={address:1,article:1,aside:1,blockquote:1,details:1,div:1,dl:1,fieldset:1,figure:1,footer:1,form:1,h1:1,h2:1,h3:1,h4:1,h5:1,h6:1,header:1,hgroup:1,hr:1,main:1,menu:1,nav:1,ol:1,p:1,
pre:1,section:1,table:1,ul:1},d={command:1,link:1,meta:1,noscript:1,script:1,style:1},j={},f={"#":1},h={center:1,dir:1,noframes:1};a(b,{a:1,abbr:1,area:1,audio:1,b:1,bdi:1,bdo:1,br:1,button:1,canvas:1,cite:1,code:1,command:1,datalist:1,del:1,dfn:1,em:1,embed:1,i:1,iframe:1,img:1,input:1,ins:1,kbd:1,keygen:1,label:1,map:1,mark:1,meter:1,noscript:1,object:1,output:1,progress:1,q:1,ruby:1,s:1,samp:1,script:1,select:1,small:1,span:1,strong:1,sub:1,sup:1,textarea:1,time:1,u:1,"var":1,video:1,wbr:1},f,
{acronym:1,applet:1,basefont:1,big:1,font:1,isindex:1,strike:1,style:1,tt:1});a(g,c,b,h);e={a:e(b,{a:1,button:1}),abbr:b,address:g,area:j,article:g,aside:g,audio:a({source:1,track:1},g),b:b,base:j,bdi:b,bdo:b,blockquote:g,body:g,br:j,button:e(b,{a:1,button:1}),canvas:b,caption:g,cite:b,code:b,col:j,colgroup:{col:1},command:j,datalist:a({option:1},b),dd:g,del:b,details:a({summary:1},g),dfn:b,div:g,dl:{dt:1,dd:1},dt:g,em:b,embed:j,fieldset:a({legend:1},g),figcaption:g,figure:a({figcaption:1},g),footer:g,
form:g,h1:b,h2:b,h3:b,h4:b,h5:b,h6:b,head:a({title:1,base:1},d),header:g,hgroup:{h1:1,h2:1,h3:1,h4:1,h5:1,h6:1},hr:j,html:a({head:1,body:1},g,d),i:b,iframe:f,img:j,input:j,ins:b,kbd:b,keygen:j,label:b,legend:b,li:g,link:j,main:g,map:g,mark:b,menu:a({li:1},g),meta:j,meter:e(b,{meter:1}),nav:g,noscript:a({link:1,meta:1,style:1},b),object:a({param:1},b),ol:{li:1},optgroup:{option:1},option:f,output:b,p:b,param:j,pre:b,progress:e(b,{progress:1}),q:b,rp:b,rt:b,ruby:a({rp:1,rt:1},b),s:b,samp:b,script:f,
section:g,select:{optgroup:1,option:1},small:b,source:j,span:b,strong:b,style:f,sub:b,summary:b,sup:b,table:{caption:1,colgroup:1,thead:1,tfoot:1,tbody:1,tr:1},tbody:{tr:1},td:g,textarea:f,tfoot:{tr:1},th:g,thead:{tr:1},time:e(b,{time:1}),title:f,tr:{th:1,td:1},track:j,u:b,ul:{li:1},"var":b,video:a({source:1,track:1},g),wbr:j,acronym:b,applet:a({param:1},g),basefont:j,big:b,center:g,dialog:j,dir:{li:1},font:b,isindex:j,noframes:g,strike:b,tt:b};a(e,{$block:a({audio:1,dd:1,dt:1,figcaption:1,li:1,video:1},
c,h),$blockLimit:{article:1,aside:1,audio:1,body:1,caption:1,details:1,dir:1,div:1,dl:1,fieldset:1,figcaption:1,figure:1,footer:1,form:1,header:1,hgroup:1,main:1,menu:1,nav:1,ol:1,section:1,table:1,td:1,th:1,tr:1,ul:1,video:1},$cdata:{script:1,style:1},$editable:{address:1,article:1,aside:1,blockquote:1,body:1,details:1,div:1,fieldset:1,figcaption:1,footer:1,form:1,h1:1,h2:1,h3:1,h4:1,h5:1,h6:1,header:1,hgroup:1,main:1,nav:1,p:1,pre:1,section:1},$empty:{area:1,base:1,basefont:1,br:1,col:1,command:1,
dialog:1,embed:1,hr:1,img:1,input:1,isindex:1,keygen:1,link:1,meta:1,param:1,source:1,track:1,wbr:1},$inline:b,$list:{dl:1,ol:1,ul:1},$listItem:{dd:1,dt:1,li:1},$nonBodyContent:a({body:1,head:1,html:1},e.head),$nonEditable:{applet:1,audio:1,button:1,embed:1,iframe:1,map:1,object:1,option:1,param:1,script:1,textarea:1,video:1},$object:{applet:1,audio:1,button:1,hr:1,iframe:1,img:1,input:1,object:1,select:1,table:1,textarea:1,video:1},$removeEmpty:{abbr:1,acronym:1,b:1,bdi:1,bdo:1,big:1,cite:1,code:1,
del:1,dfn:1,em:1,font:1,i:1,ins:1,label:1,kbd:1,mark:1,meter:1,output:1,q:1,ruby:1,s:1,samp:1,small:1,span:1,strike:1,strong:1,sub:1,sup:1,time:1,tt:1,u:1,"var":1},$tabIndex:{a:1,area:1,button:1,input:1,object:1,select:1,textarea:1},$tableContent:{caption:1,col:1,colgroup:1,tbody:1,td:1,tfoot:1,th:1,thead:1,tr:1},$transparent:{a:1,audio:1,canvas:1,del:1,ins:1,map:1,noscript:1,object:1,video:1},$intermediate:{caption:1,colgroup:1,dd:1,dt:1,figcaption:1,legend:1,li:1,optgroup:1,option:1,rp:1,rt:1,summary:1,
tbody:1,td:1,tfoot:1,th:1,thead:1,tr:1}});return e}(),CKEDITOR.dom.event=function(a){this.$=a},CKEDITOR.dom.event.prototype={getKey:function(){return this.$.keyCode||this.$.which},getKeystroke:function(){var a=this.getKey();if(this.$.ctrlKey||this.$.metaKey)a=a+CKEDITOR.CTRL;this.$.shiftKey&&(a=a+CKEDITOR.SHIFT);this.$.altKey&&(a=a+CKEDITOR.ALT);return a},preventDefault:function(a){var e=this.$;e.preventDefault?e.preventDefault():e.returnValue=false;a&&this.stopPropagation()},stopPropagation:function(){var a=
this.$;a.stopPropagation?a.stopPropagation():a.cancelBubble=true},getTarget:function(){var a=this.$.target||this.$.srcElement;return a?new CKEDITOR.dom.node(a):null},getPhase:function(){return this.$.eventPhase||2},getPageOffset:function(){var a=this.getTarget().getDocument().$;return{x:this.$.pageX||this.$.clientX+(a.documentElement.scrollLeft||a.body.scrollLeft),y:this.$.pageY||this.$.clientY+(a.documentElement.scrollTop||a.body.scrollTop)}}},CKEDITOR.CTRL=1114112,CKEDITOR.SHIFT=2228224,CKEDITOR.ALT=
4456448,CKEDITOR.EVENT_PHASE_CAPTURING=1,CKEDITOR.EVENT_PHASE_AT_TARGET=2,CKEDITOR.EVENT_PHASE_BUBBLING=3,CKEDITOR.dom.domObject=function(a){if(a)this.$=a},CKEDITOR.dom.domObject.prototype=function(){var a=function(a,b){return function(g){typeof CKEDITOR!="undefined"&&a.fire(b,new CKEDITOR.dom.event(g))}};return{getPrivate:function(){var a;if(!(a=this.getCustomData("_")))this.setCustomData("_",a={});return a},on:function(e){var b=this.getCustomData("_cke_nativeListeners");if(!b){b={};this.setCustomData("_cke_nativeListeners",
b)}if(!b[e]){b=b[e]=a(this,e);this.$.addEventListener?this.$.addEventListener(e,b,!!CKEDITOR.event.useCapture):this.$.attachEvent&&this.$.attachEvent("on"+e,b)}return CKEDITOR.event.prototype.on.apply(this,arguments)},removeListener:function(a){CKEDITOR.event.prototype.removeListener.apply(this,arguments);if(!this.hasListeners(a)){var b=this.getCustomData("_cke_nativeListeners"),g=b&&b[a];if(g){this.$.removeEventListener?this.$.removeEventListener(a,g,false):this.$.detachEvent&&this.$.detachEvent("on"+
a,g);delete b[a]}}},removeAllListeners:function(){var a=this.getCustomData("_cke_nativeListeners"),b;for(b in a){var g=a[b];this.$.detachEvent?this.$.detachEvent("on"+b,g):this.$.removeEventListener&&this.$.removeEventListener(b,g,false);delete a[b]}CKEDITOR.event.prototype.removeAllListeners.call(this)}}}(),function(a){var e={};CKEDITOR.on("reset",function(){e={}});a.equals=function(a){try{return a&&a.$===this.$}catch(g){return false}};a.setCustomData=function(a,g){var c=this.getUniqueId();(e[c]||
(e[c]={}))[a]=g;return this};a.getCustomData=function(a){var g=this.$["data-cke-expando"];return(g=g&&e[g])&&a in g?g[a]:null};a.removeCustomData=function(a){var g=this.$["data-cke-expando"],g=g&&e[g],c,d;if(g){c=g[a];d=a in g;delete g[a]}return d?c:null};a.clearCustomData=function(){this.removeAllListeners();var a=this.$["data-cke-expando"];a&&delete e[a]};a.getUniqueId=function(){return this.$["data-cke-expando"]||(this.$["data-cke-expando"]=CKEDITOR.tools.getNextNumber())};CKEDITOR.event.implementOn(a)}(CKEDITOR.dom.domObject.prototype),
CKEDITOR.dom.node=function(a){return a?new CKEDITOR.dom[a.nodeType==CKEDITOR.NODE_DOCUMENT?"document":a.nodeType==CKEDITOR.NODE_ELEMENT?"element":a.nodeType==CKEDITOR.NODE_TEXT?"text":a.nodeType==CKEDITOR.NODE_COMMENT?"comment":a.nodeType==CKEDITOR.NODE_DOCUMENT_FRAGMENT?"documentFragment":"domObject"](a):this},CKEDITOR.dom.node.prototype=new CKEDITOR.dom.domObject,CKEDITOR.NODE_ELEMENT=1,CKEDITOR.NODE_DOCUMENT=9,CKEDITOR.NODE_TEXT=3,CKEDITOR.NODE_COMMENT=8,CKEDITOR.NODE_DOCUMENT_FRAGMENT=11,CKEDITOR.POSITION_IDENTICAL=
0,CKEDITOR.POSITION_DISCONNECTED=1,CKEDITOR.POSITION_FOLLOWING=2,CKEDITOR.POSITION_PRECEDING=4,CKEDITOR.POSITION_IS_CONTAINED=8,CKEDITOR.POSITION_CONTAINS=16,CKEDITOR.tools.extend(CKEDITOR.dom.node.prototype,{appendTo:function(a,e){a.append(this,e);return a},clone:function(a,e){var b=this.$.cloneNode(a),g=function(c){c["data-cke-expando"]&&(c["data-cke-expando"]=false);if(c.nodeType==CKEDITOR.NODE_ELEMENT){e||c.removeAttribute("id",false);if(a)for(var c=c.childNodes,d=0;d<c.length;d++)g(c[d])}};g(b);
return new CKEDITOR.dom.node(b)},hasPrevious:function(){return!!this.$.previousSibling},hasNext:function(){return!!this.$.nextSibling},insertAfter:function(a){a.$.parentNode.insertBefore(this.$,a.$.nextSibling);return a},insertBefore:function(a){a.$.parentNode.insertBefore(this.$,a.$);return a},insertBeforeMe:function(a){this.$.parentNode.insertBefore(a.$,this.$);return a},getAddress:function(a){for(var e=[],b=this.getDocument().$.documentElement,g=this.$;g&&g!=b;){var c=g.parentNode;c&&e.unshift(this.getIndex.call({$:g},
a));g=c}return e},getDocument:function(){return new CKEDITOR.dom.document(this.$.ownerDocument||this.$.parentNode.ownerDocument)},getIndex:function(a){function e(a,c){var b=c?a.nextSibling:a.previousSibling;return!b||b.nodeType!=CKEDITOR.NODE_TEXT?null:b.nodeValue?b:e(b,c)}var b=this.$,g=-1,c;if(!this.$.parentNode||a&&b.nodeType==CKEDITOR.NODE_TEXT&&!b.nodeValue&&!e(b)&&!e(b,true))return-1;do if(!a||!(b!=this.$&&b.nodeType==CKEDITOR.NODE_TEXT&&(c||!b.nodeValue))){g++;c=b.nodeType==CKEDITOR.NODE_TEXT}while(b=
b.previousSibling);return g},getNextSourceNode:function(a,e,b){if(b&&!b.call)var g=b,b=function(a){return!a.equals(g)};var a=!a&&this.getFirst&&this.getFirst(),c;if(!a){if(this.type==CKEDITOR.NODE_ELEMENT&&b&&b(this,true)===false)return null;a=this.getNext()}for(;!a&&(c=(c||this).getParent());){if(b&&b(c,true)===false)return null;a=c.getNext()}return!a||b&&b(a)===false?null:e&&e!=a.type?a.getNextSourceNode(false,e,b):a},getPreviousSourceNode:function(a,e,b){if(b&&!b.call)var g=b,b=function(a){return!a.equals(g)};
var a=!a&&this.getLast&&this.getLast(),c;if(!a){if(this.type==CKEDITOR.NODE_ELEMENT&&b&&b(this,true)===false)return null;a=this.getPrevious()}for(;!a&&(c=(c||this).getParent());){if(b&&b(c,true)===false)return null;a=c.getPrevious()}return!a||b&&b(a)===false?null:e&&a.type!=e?a.getPreviousSourceNode(false,e,b):a},getPrevious:function(a){var e=this.$,b;do b=(e=e.previousSibling)&&e.nodeType!=10&&new CKEDITOR.dom.node(e);while(b&&a&&!a(b));return b},getNext:function(a){var e=this.$,b;do b=(e=e.nextSibling)&&
new CKEDITOR.dom.node(e);while(b&&a&&!a(b));return b},getParent:function(a){var e=this.$.parentNode;return e&&(e.nodeType==CKEDITOR.NODE_ELEMENT||a&&e.nodeType==CKEDITOR.NODE_DOCUMENT_FRAGMENT)?new CKEDITOR.dom.node(e):null},getParents:function(a){var e=this,b=[];do b[a?"push":"unshift"](e);while(e=e.getParent());return b},getCommonAncestor:function(a){if(a.equals(this))return this;if(a.contains&&a.contains(this))return a;var e=this.contains?this:this.getParent();do if(e.contains(a))return e;while(e=
e.getParent());return null},getPosition:function(a){var e=this.$,b=a.$;if(e.compareDocumentPosition)return e.compareDocumentPosition(b);if(e==b)return CKEDITOR.POSITION_IDENTICAL;if(this.type==CKEDITOR.NODE_ELEMENT&&a.type==CKEDITOR.NODE_ELEMENT){if(e.contains){if(e.contains(b))return CKEDITOR.POSITION_CONTAINS+CKEDITOR.POSITION_PRECEDING;if(b.contains(e))return CKEDITOR.POSITION_IS_CONTAINED+CKEDITOR.POSITION_FOLLOWING}if("sourceIndex"in e)return e.sourceIndex<0||b.sourceIndex<0?CKEDITOR.POSITION_DISCONNECTED:
e.sourceIndex<b.sourceIndex?CKEDITOR.POSITION_PRECEDING:CKEDITOR.POSITION_FOLLOWING}for(var e=this.getAddress(),a=a.getAddress(),b=Math.min(e.length,a.length),g=0;g<=b-1;g++)if(e[g]!=a[g]){if(g<b)return e[g]<a[g]?CKEDITOR.POSITION_PRECEDING:CKEDITOR.POSITION_FOLLOWING;break}return e.length<a.length?CKEDITOR.POSITION_CONTAINS+CKEDITOR.POSITION_PRECEDING:CKEDITOR.POSITION_IS_CONTAINED+CKEDITOR.POSITION_FOLLOWING},getAscendant:function(a,e){var b=this.$,g,c;if(!e)b=b.parentNode;if(typeof a=="function"){c=
true;g=a}else{c=false;g=function(c){c=typeof c.nodeName=="string"?c.nodeName.toLowerCase():"";return typeof a=="string"?c==a:c in a}}for(;b;){if(g(c?new CKEDITOR.dom.node(b):b))return new CKEDITOR.dom.node(b);try{b=b.parentNode}catch(d){b=null}}return null},hasAscendant:function(a,e){var b=this.$;if(!e)b=b.parentNode;for(;b;){if(b.nodeName&&b.nodeName.toLowerCase()==a)return true;b=b.parentNode}return false},move:function(a,e){a.append(this.remove(),e)},remove:function(a){var e=this.$,b=e.parentNode;
if(b){if(a)for(;a=e.firstChild;)b.insertBefore(e.removeChild(a),e);b.removeChild(e)}return this},replace:function(a){this.insertBefore(a);a.remove()},trim:function(){this.ltrim();this.rtrim()},ltrim:function(){for(var a;this.getFirst&&(a=this.getFirst());){if(a.type==CKEDITOR.NODE_TEXT){var e=CKEDITOR.tools.ltrim(a.getText()),b=a.getLength();if(e){if(e.length<b){a.split(b-e.length);this.$.removeChild(this.$.firstChild)}}else{a.remove();continue}}break}},rtrim:function(){for(var a;this.getLast&&(a=
this.getLast());){if(a.type==CKEDITOR.NODE_TEXT){var e=CKEDITOR.tools.rtrim(a.getText()),b=a.getLength();if(e){if(e.length<b){a.split(e.length);this.$.lastChild.parentNode.removeChild(this.$.lastChild)}}else{a.remove();continue}}break}if(CKEDITOR.env.needsBrFiller)(a=this.$.lastChild)&&(a.type==1&&a.nodeName.toLowerCase()=="br")&&a.parentNode.removeChild(a)},isReadOnly:function(){var a=this;this.type!=CKEDITOR.NODE_ELEMENT&&(a=this.getParent());if(a&&typeof a.$.isContentEditable!="undefined")return!(a.$.isContentEditable||
a.data("cke-editable"));for(;a;){if(a.data("cke-editable"))break;if(a.getAttribute("contentEditable")=="false")return true;if(a.getAttribute("contentEditable")=="true")break;a=a.getParent()}return!a}}),CKEDITOR.dom.window=function(a){CKEDITOR.dom.domObject.call(this,a)},CKEDITOR.dom.window.prototype=new CKEDITOR.dom.domObject,CKEDITOR.tools.extend(CKEDITOR.dom.window.prototype,{focus:function(){this.$.focus()},getViewPaneSize:function(){var a=this.$.document,e=a.compatMode=="CSS1Compat";return{width:(e?
a.documentElement.clientWidth:a.body.clientWidth)||0,height:(e?a.documentElement.clientHeight:a.body.clientHeight)||0}},getScrollPosition:function(){var a=this.$;if("pageXOffset"in a)return{x:a.pageXOffset||0,y:a.pageYOffset||0};a=a.document;return{x:a.documentElement.scrollLeft||a.body.scrollLeft||0,y:a.documentElement.scrollTop||a.body.scrollTop||0}},getFrame:function(){var a=this.$.frameElement;return a?new CKEDITOR.dom.element.get(a):null}}),CKEDITOR.dom.document=function(a){CKEDITOR.dom.domObject.call(this,
a)},CKEDITOR.dom.document.prototype=new CKEDITOR.dom.domObject,CKEDITOR.tools.extend(CKEDITOR.dom.document.prototype,{type:CKEDITOR.NODE_DOCUMENT,appendStyleSheet:function(a){if(this.$.createStyleSheet)this.$.createStyleSheet(a);else{var e=new CKEDITOR.dom.element("link");e.setAttributes({rel:"stylesheet",type:"text/css",href:a});this.getHead().append(e)}},appendStyleText:function(a){if(this.$.createStyleSheet){var e=this.$.createStyleSheet("");e.cssText=a}else{var b=new CKEDITOR.dom.element("style",
this);b.append(new CKEDITOR.dom.text(a,this));this.getHead().append(b)}return e||b.$.sheet},createElement:function(a,e){var b=new CKEDITOR.dom.element(a,this);if(e){e.attributes&&b.setAttributes(e.attributes);e.styles&&b.setStyles(e.styles)}return b},createText:function(a){return new CKEDITOR.dom.text(a,this)},focus:function(){this.getWindow().focus()},getActive:function(){var a;try{a=this.$.activeElement}catch(e){return null}return new CKEDITOR.dom.element(a)},getById:function(a){return(a=this.$.getElementById(a))?
new CKEDITOR.dom.element(a):null},getByAddress:function(a,e){for(var b=this.$.documentElement,g=0;b&&g<a.length;g++){var c=a[g];if(e)for(var d=-1,j=0;j<b.childNodes.length;j++){var f=b.childNodes[j];if(!(e===true&&f.nodeType==3&&f.previousSibling&&f.previousSibling.nodeType==3)){d++;if(d==c){b=f;break}}}else b=b.childNodes[c]}return b?new CKEDITOR.dom.node(b):null},getElementsByTag:function(a,e){!(CKEDITOR.env.ie&&document.documentMode<=8)&&e&&(a=e+":"+a);return new CKEDITOR.dom.nodeList(this.$.getElementsByTagName(a))},
getHead:function(){var a=this.$.getElementsByTagName("head")[0];return a=a?new CKEDITOR.dom.element(a):this.getDocumentElement().append(new CKEDITOR.dom.element("head"),true)},getBody:function(){return new CKEDITOR.dom.element(this.$.body)},getDocumentElement:function(){return new CKEDITOR.dom.element(this.$.documentElement)},getWindow:function(){return new CKEDITOR.dom.window(this.$.parentWindow||this.$.defaultView)},write:function(a){this.$.open("text/html","replace");CKEDITOR.env.ie&&(a=a.replace(/(?:^\s*<!DOCTYPE[^>]*?>)|^/i,
'$&\n<script data-cke-temp="1">('+CKEDITOR.tools.fixDomain+")();<\/script>"));this.$.write(a);this.$.close()},find:function(a){return new CKEDITOR.dom.nodeList(this.$.querySelectorAll(a))},findOne:function(a){return(a=this.$.querySelector(a))?new CKEDITOR.dom.element(a):null},_getHtml5ShivFrag:function(){var a=this.getCustomData("html5ShivFrag");if(!a){a=this.$.createDocumentFragment();CKEDITOR.tools.enableHtml5Elements(a,true);this.setCustomData("html5ShivFrag",a)}return a}}),CKEDITOR.dom.nodeList=
function(a){this.$=a},CKEDITOR.dom.nodeList.prototype={count:function(){return this.$.length},getItem:function(a){if(a<0||a>=this.$.length)return null;return(a=this.$[a])?new CKEDITOR.dom.node(a):null}},CKEDITOR.dom.element=function(a,e){typeof a=="string"&&(a=(e?e.$:document).createElement(a));CKEDITOR.dom.domObject.call(this,a)},CKEDITOR.dom.element.get=function(a){return(a=typeof a=="string"?document.getElementById(a)||document.getElementsByName(a)[0]:a)&&(a.$?a:new CKEDITOR.dom.element(a))},CKEDITOR.dom.element.prototype=
new CKEDITOR.dom.node,CKEDITOR.dom.element.createFromHtml=function(a,e){var b=new CKEDITOR.dom.element("div",e);b.setHtml(a);return b.getFirst().remove()},CKEDITOR.dom.element.setMarker=function(a,e,b,g){var c=e.getCustomData("list_marker_id")||e.setCustomData("list_marker_id",CKEDITOR.tools.getNextNumber()).getCustomData("list_marker_id"),d=e.getCustomData("list_marker_names")||e.setCustomData("list_marker_names",{}).getCustomData("list_marker_names");a[c]=e;d[b]=1;return e.setCustomData(b,g)},CKEDITOR.dom.element.clearAllMarkers=
function(a){for(var e in a)CKEDITOR.dom.element.clearMarkers(a,a[e],1)},CKEDITOR.dom.element.clearMarkers=function(a,e,b){var g=e.getCustomData("list_marker_names"),c=e.getCustomData("list_marker_id"),d;for(d in g)e.removeCustomData(d);e.removeCustomData("list_marker_names");if(b){e.removeCustomData("list_marker_id");delete a[c]}},function(){function a(a){var d=true;if(!a.$.id){a.$.id="cke_tmp_"+CKEDITOR.tools.getNextNumber();d=false}return function(){d||a.removeAttribute("id")}}function e(a,d){return"#"+
a.$.id+" "+d.split(/,\s*/).join(", #"+a.$.id+" ")}function b(a){for(var d=0,b=0,f=g[a].length;b<f;b++)d=d+(parseInt(this.getComputedStyle(g[a][b])||0,10)||0);return d}CKEDITOR.tools.extend(CKEDITOR.dom.element.prototype,{type:CKEDITOR.NODE_ELEMENT,addClass:function(a){var d=this.$.className;d&&(RegExp("(?:^|\\s)"+a+"(?:\\s|$)","").test(d)||(d=d+(" "+a)));this.$.className=d||a;return this},removeClass:function(a){var d=this.getAttribute("class");if(d){a=RegExp("(?:^|\\s+)"+a+"(?=\\s|$)","i");if(a.test(d))(d=
d.replace(a,"").replace(/^\s+/,""))?this.setAttribute("class",d):this.removeAttribute("class")}return this},hasClass:function(a){return RegExp("(?:^|\\s+)"+a+"(?=\\s|$)","").test(this.getAttribute("class"))},append:function(a,d){typeof a=="string"&&(a=this.getDocument().createElement(a));d?this.$.insertBefore(a.$,this.$.firstChild):this.$.appendChild(a.$);return a},appendHtml:function(a){if(this.$.childNodes.length){var d=new CKEDITOR.dom.element("div",this.getDocument());d.setHtml(a);d.moveChildren(this)}else this.setHtml(a)},
appendText:function(a){this.$.text!=null?this.$.text=this.$.text+a:this.append(new CKEDITOR.dom.text(a))},appendBogus:function(a){if(a||CKEDITOR.env.needsBrFiller){for(a=this.getLast();a&&a.type==CKEDITOR.NODE_TEXT&&!CKEDITOR.tools.rtrim(a.getText());)a=a.getPrevious();if(!a||!a.is||!a.is("br")){a=this.getDocument().createElement("br");CKEDITOR.env.gecko&&a.setAttribute("type","_moz");this.append(a)}}},breakParent:function(a){var d=new CKEDITOR.dom.range(this.getDocument());d.setStartAfter(this);
d.setEndAfter(a);a=d.extractContents();d.insertNode(this.remove());a.insertAfterNode(this)},contains:CKEDITOR.env.ie||CKEDITOR.env.webkit?function(a){var d=this.$;return a.type!=CKEDITOR.NODE_ELEMENT?d.contains(a.getParent().$):d!=a.$&&d.contains(a.$)}:function(a){return!!(this.$.compareDocumentPosition(a.$)&16)},focus:function(){function a(){try{this.$.focus()}catch(c){}}return function(d){d?CKEDITOR.tools.setTimeout(a,100,this):a.call(this)}}(),getHtml:function(){var a=this.$.innerHTML;return CKEDITOR.env.ie?
a.replace(/<\?[^>]*>/g,""):a},getOuterHtml:function(){if(this.$.outerHTML)return this.$.outerHTML.replace(/<\?[^>]*>/,"");var a=this.$.ownerDocument.createElement("div");a.appendChild(this.$.cloneNode(true));return a.innerHTML},getClientRect:function(){var a=CKEDITOR.tools.extend({},this.$.getBoundingClientRect());!a.width&&(a.width=a.right-a.left);!a.height&&(a.height=a.bottom-a.top);return a},setHtml:CKEDITOR.env.ie&&CKEDITOR.env.version<9?function(a){try{var d=this.$;if(this.getParent())return d.innerHTML=
a;var b=this.getDocument()._getHtml5ShivFrag();b.appendChild(d);d.innerHTML=a;b.removeChild(d);return a}catch(f){this.$.innerHTML="";d=new CKEDITOR.dom.element("body",this.getDocument());d.$.innerHTML=a;for(d=d.getChildren();d.count();)this.append(d.getItem(0));return a}}:function(a){return this.$.innerHTML=a},setText:function(){var a=document.createElement("p");a.innerHTML="x";a=a.textContent;return function(d){this.$[a?"textContent":"innerText"]=d}}(),getAttribute:function(){var a=function(a){return this.$.getAttribute(a,
2)};return CKEDITOR.env.ie&&(CKEDITOR.env.ie7Compat||CKEDITOR.env.quirks)?function(a){switch(a){case "class":a="className";break;case "http-equiv":a="httpEquiv";break;case "name":return this.$.name;case "tabindex":a=this.$.getAttribute(a,2);a!==0&&this.$.tabIndex===0&&(a=null);return a;case "checked":a=this.$.attributes.getNamedItem(a);return(a.specified?a.nodeValue:this.$.checked)?"checked":null;case "hspace":case "value":return this.$[a];case "style":return this.$.style.cssText;case "contenteditable":case "contentEditable":return this.$.attributes.getNamedItem("contentEditable").specified?
this.$.getAttribute("contentEditable"):null}return this.$.getAttribute(a,2)}:a}(),getChildren:function(){return new CKEDITOR.dom.nodeList(this.$.childNodes)},getComputedStyle:CKEDITOR.env.ie?function(a){return this.$.currentStyle[CKEDITOR.tools.cssStyleToDomStyle(a)]}:function(a){var d=this.getWindow().$.getComputedStyle(this.$,null);return d?d.getPropertyValue(a):""},getDtd:function(){var a=CKEDITOR.dtd[this.getName()];this.getDtd=function(){return a};return a},getElementsByTag:CKEDITOR.dom.document.prototype.getElementsByTag,
getTabIndex:CKEDITOR.env.ie?function(){var a=this.$.tabIndex;a===0&&(!CKEDITOR.dtd.$tabIndex[this.getName()]&&parseInt(this.getAttribute("tabindex"),10)!==0)&&(a=-1);return a}:CKEDITOR.env.webkit?function(){var a=this.$.tabIndex;if(a===void 0){a=parseInt(this.getAttribute("tabindex"),10);isNaN(a)&&(a=-1)}return a}:function(){return this.$.tabIndex},getText:function(){return this.$.textContent||this.$.innerText||""},getWindow:function(){return this.getDocument().getWindow()},getId:function(){return this.$.id||
null},getNameAtt:function(){return this.$.name||null},getName:function(){var a=this.$.nodeName.toLowerCase();if(CKEDITOR.env.ie&&document.documentMode<=8){var d=this.$.scopeName;d!="HTML"&&(a=d.toLowerCase()+":"+a)}this.getName=function(){return a};return this.getName()},getValue:function(){return this.$.value},getFirst:function(a){var d=this.$.firstChild;(d=d&&new CKEDITOR.dom.node(d))&&(a&&!a(d))&&(d=d.getNext(a));return d},getLast:function(a){var d=this.$.lastChild;(d=d&&new CKEDITOR.dom.node(d))&&
(a&&!a(d))&&(d=d.getPrevious(a));return d},getStyle:function(a){return this.$.style[CKEDITOR.tools.cssStyleToDomStyle(a)]},is:function(){var a=this.getName();if(typeof arguments[0]=="object")return!!arguments[0][a];for(var d=0;d<arguments.length;d++)if(arguments[d]==a)return true;return false},isEditable:function(a){var d=this.getName();if(this.isReadOnly()||this.getComputedStyle("display")=="none"||this.getComputedStyle("visibility")=="hidden"||CKEDITOR.dtd.$nonEditable[d]||CKEDITOR.dtd.$empty[d]||
this.is("a")&&(this.data("cke-saved-name")||this.hasAttribute("name"))&&!this.getChildCount())return false;if(a!==false){a=CKEDITOR.dtd[d]||CKEDITOR.dtd.span;return!(!a||!a["#"])}return true},isIdentical:function(a){var d=this.clone(0,1),a=a.clone(0,1);d.removeAttributes(["_moz_dirty","data-cke-expando","data-cke-saved-href","data-cke-saved-name"]);a.removeAttributes(["_moz_dirty","data-cke-expando","data-cke-saved-href","data-cke-saved-name"]);if(d.$.isEqualNode){d.$.style.cssText=CKEDITOR.tools.normalizeCssText(d.$.style.cssText);
a.$.style.cssText=CKEDITOR.tools.normalizeCssText(a.$.style.cssText);return d.$.isEqualNode(a.$)}d=d.getOuterHtml();a=a.getOuterHtml();if(CKEDITOR.env.ie&&CKEDITOR.env.version<9&&this.is("a")){var b=this.getParent();if(b.type==CKEDITOR.NODE_ELEMENT){b=b.clone();b.setHtml(d);d=b.getHtml();b.setHtml(a);a=b.getHtml()}}return d==a},isVisible:function(){var a=(this.$.offsetHeight||this.$.offsetWidth)&&this.getComputedStyle("visibility")!="hidden",d,b;if(a&&CKEDITOR.env.webkit){d=this.getWindow();if(!d.equals(CKEDITOR.document.getWindow())&&
(b=d.$.frameElement))a=(new CKEDITOR.dom.element(b)).isVisible()}return!!a},isEmptyInlineRemoveable:function(){if(!CKEDITOR.dtd.$removeEmpty[this.getName()])return false;for(var a=this.getChildren(),d=0,b=a.count();d<b;d++){var f=a.getItem(d);if(!(f.type==CKEDITOR.NODE_ELEMENT&&f.data("cke-bookmark"))&&(f.type==CKEDITOR.NODE_ELEMENT&&!f.isEmptyInlineRemoveable()||f.type==CKEDITOR.NODE_TEXT&&CKEDITOR.tools.trim(f.getText())))return false}return true},hasAttributes:CKEDITOR.env.ie&&(CKEDITOR.env.ie7Compat||
CKEDITOR.env.quirks)?function(){for(var a=this.$.attributes,d=0;d<a.length;d++){var b=a[d];switch(b.nodeName){case "class":if(this.getAttribute("class"))return true;case "data-cke-expando":continue;default:if(b.specified)return true}}return false}:function(){var a=this.$.attributes,d=a.length,b={"data-cke-expando":1,_moz_dirty:1};return d>0&&(d>2||!b[a[0].nodeName]||d==2&&!b[a[1].nodeName])},hasAttribute:function(){function a(c){var b=this.$.attributes.getNamedItem(c);if(this.getName()=="input")switch(c){case "class":return this.$.className.length>
0;case "checked":return!!this.$.checked;case "value":c=this.getAttribute("type");return c=="checkbox"||c=="radio"?this.$.value!="on":!!this.$.value}return!b?false:b.specified}return CKEDITOR.env.ie?CKEDITOR.env.version<8?function(d){return d=="name"?!!this.$.name:a.call(this,d)}:a:function(a){return!!this.$.attributes.getNamedItem(a)}}(),hide:function(){this.setStyle("display","none")},moveChildren:function(a,d){var b=this.$,a=a.$;if(b!=a){var f;if(d)for(;f=b.lastChild;)a.insertBefore(b.removeChild(f),
a.firstChild);else for(;f=b.firstChild;)a.appendChild(b.removeChild(f))}},mergeSiblings:function(){function a(c,b,f){if(b&&b.type==CKEDITOR.NODE_ELEMENT){for(var g=[];b.data("cke-bookmark")||b.isEmptyInlineRemoveable();){g.push(b);b=f?b.getNext():b.getPrevious();if(!b||b.type!=CKEDITOR.NODE_ELEMENT)return}if(c.isIdentical(b)){for(var e=f?c.getLast():c.getFirst();g.length;)g.shift().move(c,!f);b.moveChildren(c,!f);b.remove();e&&e.type==CKEDITOR.NODE_ELEMENT&&e.mergeSiblings()}}}return function(d){if(d===
false||CKEDITOR.dtd.$removeEmpty[this.getName()]||this.is("a")){a(this,this.getNext(),true);a(this,this.getPrevious())}}}(),show:function(){this.setStyles({display:"",visibility:""})},setAttribute:function(){var a=function(a,c){this.$.setAttribute(a,c);return this};return CKEDITOR.env.ie&&(CKEDITOR.env.ie7Compat||CKEDITOR.env.quirks)?function(d,b){d=="class"?this.$.className=b:d=="style"?this.$.style.cssText=b:d=="tabindex"?this.$.tabIndex=b:d=="checked"?this.$.checked=b:d=="contenteditable"?a.call(this,
"contentEditable",b):a.apply(this,arguments);return this}:CKEDITOR.env.ie8Compat&&CKEDITOR.env.secure?function(b,g){if(b=="src"&&g.match(/^http:\/\//))try{a.apply(this,arguments)}catch(f){}else a.apply(this,arguments);return this}:a}(),setAttributes:function(a){for(var b in a)this.setAttribute(b,a[b]);return this},setValue:function(a){this.$.value=a;return this},removeAttribute:function(){var a=function(a){this.$.removeAttribute(a)};return CKEDITOR.env.ie&&(CKEDITOR.env.ie7Compat||CKEDITOR.env.quirks)?
function(a){a=="class"?a="className":a=="tabindex"?a="tabIndex":a=="contenteditable"&&(a="contentEditable");this.$.removeAttribute(a)}:a}(),removeAttributes:function(a){if(CKEDITOR.tools.isArray(a))for(var b=0;b<a.length;b++)this.removeAttribute(a[b]);else for(b in a)a.hasOwnProperty(b)&&this.removeAttribute(b)},removeStyle:function(a){var b=this.$.style;if(!b.removeProperty&&(a=="border"||a=="margin"||a=="padding")){var g=["top","left","right","bottom"],f;a=="border"&&(f=["color","style","width"]);
for(var b=[],h=0;h<g.length;h++)if(f)for(var e=0;e<f.length;e++)b.push([a,g[h],f[e]].join("-"));else b.push([a,g[h]].join("-"));for(a=0;a<b.length;a++)this.removeStyle(b[a])}else{b.removeProperty?b.removeProperty(a):b.removeAttribute(CKEDITOR.tools.cssStyleToDomStyle(a));this.$.style.cssText||this.removeAttribute("style")}},setStyle:function(a,b){this.$.style[CKEDITOR.tools.cssStyleToDomStyle(a)]=b;return this},setStyles:function(a){for(var b in a)this.setStyle(b,a[b]);return this},setOpacity:function(a){if(CKEDITOR.env.ie&&
CKEDITOR.env.version<9){a=Math.round(a*100);this.setStyle("filter",a>=100?"":"progid:DXImageTransform.Microsoft.Alpha(opacity="+a+")")}else this.setStyle("opacity",a)},unselectable:function(){this.setStyles(CKEDITOR.tools.cssVendorPrefix("user-select","none"));if(CKEDITOR.env.ie){this.setAttribute("unselectable","on");for(var a,b=this.getElementsByTag("*"),g=0,f=b.count();g<f;g++){a=b.getItem(g);a.setAttribute("unselectable","on")}}},getPositionedAncestor:function(){for(var a=this;a.getName()!="html";){if(a.getComputedStyle("position")!=
"static")return a;a=a.getParent()}return null},getDocumentPosition:function(a){var b=0,g=0,f=this.getDocument(),h=f.getBody(),e=CKEDITOR.env.quirks;if(document.documentElement.getBoundingClientRect){var k=this.$.getBoundingClientRect(),n=f.$.documentElement,o=n.clientTop||h.$.clientTop||0,q=n.clientLeft||h.$.clientLeft||0,m=true;if(CKEDITOR.env.ie){m=f.getDocumentElement().contains(this);f=f.getBody().contains(this);m=e&&f||!e&&m}if(m){if(CKEDITOR.env.webkit){b=h.$.scrollLeft||n.scrollLeft;g=h.$.scrollTop||
n.scrollTop}else{g=e?h.$:n;b=g.scrollLeft;g=g.scrollTop}b=k.left+b-q;g=k.top+g-o}}else{o=this;for(q=null;o&&!(o.getName()=="body"||o.getName()=="html");){b=b+(o.$.offsetLeft-o.$.scrollLeft);g=g+(o.$.offsetTop-o.$.scrollTop);if(!o.equals(this)){b=b+(o.$.clientLeft||0);g=g+(o.$.clientTop||0)}for(;q&&!q.equals(o);){b=b-q.$.scrollLeft;g=g-q.$.scrollTop;q=q.getParent()}q=o;o=(k=o.$.offsetParent)?new CKEDITOR.dom.element(k):null}}if(a){k=this.getWindow();o=a.getWindow();if(!k.equals(o)&&k.$.frameElement){a=
(new CKEDITOR.dom.element(k.$.frameElement)).getDocumentPosition(a);b=b+a.x;g=g+a.y}}if(!document.documentElement.getBoundingClientRect&&CKEDITOR.env.gecko&&!e){b=b+(this.$.clientLeft?1:0);g=g+(this.$.clientTop?1:0)}return{x:b,y:g}},scrollIntoView:function(a){var b=this.getParent();if(b){do{(b.$.clientWidth&&b.$.clientWidth<b.$.scrollWidth||b.$.clientHeight&&b.$.clientHeight<b.$.scrollHeight)&&!b.is("body")&&this.scrollIntoParent(b,a,1);if(b.is("html")){var g=b.getWindow();try{var f=g.$.frameElement;
f&&(b=new CKEDITOR.dom.element(f))}catch(h){}}}while(b=b.getParent())}},scrollIntoParent:function(a,b,g){var f,h,e,k;function n(b,d){if(/body|html/.test(a.getName()))a.getWindow().$.scrollBy(b,d);else{a.$.scrollLeft=a.$.scrollLeft+b;a.$.scrollTop=a.$.scrollTop+d}}function o(a,b){var c={x:0,y:0};if(!a.is(m?"body":"html")){var d=a.$.getBoundingClientRect();c.x=d.left;c.y=d.top}d=a.getWindow();if(!d.equals(b)){d=o(CKEDITOR.dom.element.get(d.$.frameElement),b);c.x=c.x+d.x;c.y=c.y+d.y}return c}function q(a,
b){return parseInt(a.getComputedStyle("margin-"+b)||0,10)||0}!a&&(a=this.getWindow());e=a.getDocument();var m=e.$.compatMode=="BackCompat";a instanceof CKEDITOR.dom.window&&(a=m?e.getBody():e.getDocumentElement());e=a.getWindow();h=o(this,e);var l=o(a,e),p=this.$.offsetHeight;f=this.$.offsetWidth;var t=a.$.clientHeight,s=a.$.clientWidth;e=h.x-q(this,"left")-l.x||0;k=h.y-q(this,"top")-l.y||0;f=h.x+f+q(this,"right")-(l.x+s)||0;h=h.y+p+q(this,"bottom")-(l.y+t)||0;if(k<0||h>0)n(0,b===true?k:b===false?
h:k<0?k:h);if(g&&(e<0||f>0))n(e<0?e:f,0)},setState:function(a,b,g){b=b||"cke";switch(a){case CKEDITOR.TRISTATE_ON:this.addClass(b+"_on");this.removeClass(b+"_off");this.removeClass(b+"_disabled");g&&this.setAttribute("aria-pressed",true);g&&this.removeAttribute("aria-disabled");break;case CKEDITOR.TRISTATE_DISABLED:this.addClass(b+"_disabled");this.removeClass(b+"_off");this.removeClass(b+"_on");g&&this.setAttribute("aria-disabled",true);g&&this.removeAttribute("aria-pressed");break;default:this.addClass(b+
"_off");this.removeClass(b+"_on");this.removeClass(b+"_disabled");g&&this.removeAttribute("aria-pressed");g&&this.removeAttribute("aria-disabled")}},getFrameDocument:function(){var a=this.$;try{a.contentWindow.document}catch(b){a.src=a.src}return a&&new CKEDITOR.dom.document(a.contentWindow.document)},copyAttributes:function(a,b){for(var g=this.$.attributes,b=b||{},f=0;f<g.length;f++){var h=g[f],e=h.nodeName.toLowerCase(),k;if(!(e in b))if(e=="checked"&&(k=this.getAttribute(e)))a.setAttribute(e,k);
else if(!CKEDITOR.env.ie||this.hasAttribute(e)){k=this.getAttribute(e);if(k===null)k=h.nodeValue;a.setAttribute(e,k)}}if(this.$.style.cssText!=="")a.$.style.cssText=this.$.style.cssText},renameNode:function(a){if(this.getName()!=a){var b=this.getDocument(),a=new CKEDITOR.dom.element(a,b);this.copyAttributes(a);this.moveChildren(a);this.getParent()&&this.$.parentNode.replaceChild(a.$,this.$);a.$["data-cke-expando"]=this.$["data-cke-expando"];this.$=a.$;delete this.getName}},getChild:function(){function a(b,
c){var f=b.childNodes;if(c>=0&&c<f.length)return f[c]}return function(b){var g=this.$;if(b.slice)for(;b.length>0&&g;)g=a(g,b.shift());else g=a(g,b);return g?new CKEDITOR.dom.node(g):null}}(),getChildCount:function(){return this.$.childNodes.length},disableContextMenu:function(){this.on("contextmenu",function(a){a.data.getTarget().hasClass("cke_enable_context_menu")||a.data.preventDefault()})},getDirection:function(a){return a?this.getComputedStyle("direction")||this.getDirection()||this.getParent()&&
this.getParent().getDirection(1)||this.getDocument().$.dir||"ltr":this.getStyle("direction")||this.getAttribute("dir")},data:function(a,b){a="data-"+a;if(b===void 0)return this.getAttribute(a);b===false?this.removeAttribute(a):this.setAttribute(a,b);return null},getEditor:function(){var a=CKEDITOR.instances,b,g;for(b in a){g=a[b];if(g.element.equals(this)&&g.elementMode!=CKEDITOR.ELEMENT_MODE_APPENDTO)return g}return null},find:function(b){var d=a(this),b=new CKEDITOR.dom.nodeList(this.$.querySelectorAll(e(this,
b)));d();return b},findOne:function(b){var d=a(this),b=this.$.querySelector(e(this,b));d();return b?new CKEDITOR.dom.element(b):null},forEach:function(a,b,g){if(!g&&(!b||this.type==b))var f=a(this);if(f!==false)for(var g=this.getChildren(),h=0;h<g.count();h++){f=g.getItem(h);f.type==CKEDITOR.NODE_ELEMENT?f.forEach(a,b):(!b||f.type==b)&&a(f)}}});var g={width:["border-left-width","border-right-width","padding-left","padding-right"],height:["border-top-width","border-bottom-width","padding-top","padding-bottom"]};
CKEDITOR.dom.element.prototype.setSize=function(a,d,g){if(typeof d=="number"){if(g&&(!CKEDITOR.env.ie||!CKEDITOR.env.quirks))d=d-b.call(this,a);this.setStyle(a,d+"px")}};CKEDITOR.dom.element.prototype.getSize=function(a,d){var g=Math.max(this.$["offset"+CKEDITOR.tools.capitalize(a)],this.$["client"+CKEDITOR.tools.capitalize(a)])||0;d&&(g=g-b.call(this,a));return g}}(),CKEDITOR.dom.documentFragment=function(a){a=a||CKEDITOR.document;this.$=a.type==CKEDITOR.NODE_DOCUMENT?a.$.createDocumentFragment():
a},CKEDITOR.tools.extend(CKEDITOR.dom.documentFragment.prototype,CKEDITOR.dom.element.prototype,{type:CKEDITOR.NODE_DOCUMENT_FRAGMENT,insertAfterNode:function(a){a=a.$;a.parentNode.insertBefore(this.$,a.nextSibling)}},!0,{append:1,appendBogus:1,getFirst:1,getLast:1,getParent:1,getNext:1,getPrevious:1,appendTo:1,moveChildren:1,insertBefore:1,insertAfterNode:1,replace:1,trim:1,type:1,ltrim:1,rtrim:1,getDocument:1,getChildCount:1,getChild:1,getChildren:1}),function(){function a(a,b){var c=this.range;
if(this._.end)return null;if(!this._.start){this._.start=1;if(c.collapsed){this.end();return null}c.optimize()}var d,f=c.startContainer;d=c.endContainer;var g=c.startOffset,k=c.endOffset,h,e=this.guard,j=this.type,i=a?"getPreviousSourceNode":"getNextSourceNode";if(!a&&!this._.guardLTR){var v=d.type==CKEDITOR.NODE_ELEMENT?d:d.getParent(),r=d.type==CKEDITOR.NODE_ELEMENT?d.getChild(k):d.getNext();this._.guardLTR=function(a,b){return(!b||!v.equals(a))&&(!r||!a.equals(r))&&(a.type!=CKEDITOR.NODE_ELEMENT||
!b||!a.equals(c.root))}}if(a&&!this._.guardRTL){var y=f.type==CKEDITOR.NODE_ELEMENT?f:f.getParent(),z=f.type==CKEDITOR.NODE_ELEMENT?g?f.getChild(g-1):null:f.getPrevious();this._.guardRTL=function(a,b){return(!b||!y.equals(a))&&(!z||!a.equals(z))&&(a.type!=CKEDITOR.NODE_ELEMENT||!b||!a.equals(c.root))}}var B=a?this._.guardRTL:this._.guardLTR;h=e?function(a,b){return B(a,b)===false?false:e(a,b)}:B;if(this.current)d=this.current[i](false,j,h);else{if(a)d.type==CKEDITOR.NODE_ELEMENT&&(d=k>0?d.getChild(k-
1):h(d,true)===false?null:d.getPreviousSourceNode(true,j,h));else{d=f;if(d.type==CKEDITOR.NODE_ELEMENT&&!(d=d.getChild(g)))d=h(f,true)===false?null:f.getNextSourceNode(true,j,h)}d&&h(d)===false&&(d=null)}for(;d&&!this._.end;){this.current=d;if(!this.evaluator||this.evaluator(d)!==false){if(!b)return d}else if(b&&this.evaluator)return false;d=d[i](false,j,h)}this.end();return this.current=null}function e(b){for(var c,d=null;c=a.call(this,b);)d=c;return d}function b(a){if(i(a))return false;if(a.type==
CKEDITOR.NODE_TEXT)return true;if(a.type==CKEDITOR.NODE_ELEMENT){if(a.is(CKEDITOR.dtd.$inline)||a.is("hr")||a.getAttribute("contenteditable")=="false")return true;var b;if(b=!CKEDITOR.env.needsBrFiller)if(b=a.is(k))a:{b=0;for(var c=a.getChildCount();b<c;++b)if(!i(a.getChild(b))){b=false;break a}b=true}if(b)return true}return false}CKEDITOR.dom.walker=CKEDITOR.tools.createClass({$:function(a){this.range=a;this._={}},proto:{end:function(){this._.end=1},next:function(){return a.call(this)},previous:function(){return a.call(this,
1)},checkForward:function(){return a.call(this,0,1)!==false},checkBackward:function(){return a.call(this,1,1)!==false},lastForward:function(){return e.call(this)},lastBackward:function(){return e.call(this,1)},reset:function(){delete this.current;this._={}}}});var g={block:1,"list-item":1,table:1,"table-row-group":1,"table-header-group":1,"table-footer-group":1,"table-row":1,"table-column-group":1,"table-column":1,"table-cell":1,"table-caption":1},c={absolute:1,fixed:1};CKEDITOR.dom.element.prototype.isBlockBoundary=
function(a){return this.getComputedStyle("float")=="none"&&!(this.getComputedStyle("position")in c)&&g[this.getComputedStyle("display")]?true:!!(this.is(CKEDITOR.dtd.$block)||a&&this.is(a))};CKEDITOR.dom.walker.blockBoundary=function(a){return function(b){return!(b.type==CKEDITOR.NODE_ELEMENT&&b.isBlockBoundary(a))}};CKEDITOR.dom.walker.listItemBoundary=function(){return this.blockBoundary({br:1})};CKEDITOR.dom.walker.bookmark=function(a,b){function c(a){return a&&a.getName&&a.getName()=="span"&&
a.data("cke-bookmark")}return function(d){var f,g;f=d&&d.type!=CKEDITOR.NODE_ELEMENT&&(g=d.getParent())&&c(g);f=a?f:f||c(d);return!!(b^f)}};CKEDITOR.dom.walker.whitespaces=function(a){return function(b){var c;b&&b.type==CKEDITOR.NODE_TEXT&&(c=!CKEDITOR.tools.trim(b.getText())||CKEDITOR.env.webkit&&b.getText()=="​");return!!(a^c)}};CKEDITOR.dom.walker.invisible=function(a){var b=CKEDITOR.dom.walker.whitespaces(),c=CKEDITOR.env.webkit?1:0;return function(d){if(b(d))d=1;else{d.type==CKEDITOR.NODE_TEXT&&
(d=d.getParent());d=d.$.offsetWidth<=c}return!!(a^d)}};CKEDITOR.dom.walker.nodeType=function(a,b){return function(c){return!!(b^c.type==a)}};CKEDITOR.dom.walker.bogus=function(a){function b(a){return!j(a)&&!f(a)}return function(c){var f=CKEDITOR.env.needsBrFiller?c.is&&c.is("br"):c.getText&&d.test(c.getText());if(f){f=c.getParent();c=c.getNext(b);f=f.isBlockBoundary()&&(!c||c.type==CKEDITOR.NODE_ELEMENT&&c.isBlockBoundary())}return!!(a^f)}};CKEDITOR.dom.walker.temp=function(a){return function(b){b.type!=
CKEDITOR.NODE_ELEMENT&&(b=b.getParent());b=b&&b.hasAttribute("data-cke-temp");return!!(a^b)}};var d=/^[\t\r\n ]*(?:&nbsp;|\xa0)$/,j=CKEDITOR.dom.walker.whitespaces(),f=CKEDITOR.dom.walker.bookmark(),h=CKEDITOR.dom.walker.temp();CKEDITOR.dom.walker.ignored=function(a){return function(b){b=j(b)||f(b)||h(b);return!!(a^b)}};var i=CKEDITOR.dom.walker.ignored(),k=function(a){var b={},c;for(c in a)CKEDITOR.dtd[c]["#"]&&(b[c]=1);return b}(CKEDITOR.dtd.$block);CKEDITOR.dom.walker.editable=function(a){return function(c){return!!(a^
b(c))}};CKEDITOR.dom.element.prototype.getBogus=function(){var a=this;do a=a.getPreviousSourceNode();while(f(a)||j(a)||a.type==CKEDITOR.NODE_ELEMENT&&a.is(CKEDITOR.dtd.$inline)&&!a.is(CKEDITOR.dtd.$empty));return a&&(CKEDITOR.env.needsBrFiller?a.is&&a.is("br"):a.getText&&d.test(a.getText()))?a:false}}(),CKEDITOR.dom.range=function(a){this.endOffset=this.endContainer=this.startOffset=this.startContainer=null;this.collapsed=true;var e=a instanceof CKEDITOR.dom.document;this.document=e?a:a.getDocument();
this.root=e?a.getBody():a},function(){function a(){var a=false,b=CKEDITOR.dom.walker.whitespaces(),c=CKEDITOR.dom.walker.bookmark(true),f=CKEDITOR.dom.walker.bogus();return function(g){if(c(g)||b(g))return true;if(f(g)&&!a)return a=true;return g.type==CKEDITOR.NODE_TEXT&&(g.hasAscendant("pre")||CKEDITOR.tools.trim(g.getText()).length)||g.type==CKEDITOR.NODE_ELEMENT&&!g.is(d)?false:true}}function e(a){var b=CKEDITOR.dom.walker.whitespaces(),c=CKEDITOR.dom.walker.bookmark(1);return function(d){return c(d)||
b(d)?true:!a&&j(d)||d.type==CKEDITOR.NODE_ELEMENT&&d.is(CKEDITOR.dtd.$removeEmpty)}}function b(a){return function(){var b;return this[a?"getPreviousNode":"getNextNode"](function(a){!b&&i(a)&&(b=a);return h(a)&&!(j(a)&&a.equals(b))})}}var g=function(a){a.collapsed=a.startContainer&&a.endContainer&&a.startContainer.equals(a.endContainer)&&a.startOffset==a.endOffset},c=function(a,b,c,d){a.optimizeBookmark();var f=a.startContainer,g=a.endContainer,h=a.startOffset,e=a.endOffset,j,i;if(g.type==CKEDITOR.NODE_TEXT)g=
g.split(e);else if(g.getChildCount()>0)if(e>=g.getChildCount()){g=g.append(a.document.createText(""));i=true}else g=g.getChild(e);if(f.type==CKEDITOR.NODE_TEXT){f.split(h);f.equals(g)&&(g=f.getNext())}else if(h)if(h>=f.getChildCount()){f=f.append(a.document.createText(""));j=true}else f=f.getChild(h).getPrevious();else{f=f.append(a.document.createText(""),1);j=true}var h=f.getParents(),e=g.getParents(),w,x,v;for(w=0;w<h.length;w++){x=h[w];v=e[w];if(!x.equals(v))break}for(var r=c,y,z,B,A=w;A<h.length;A++){y=
h[A];r&&!y.equals(f)&&(z=r.append(y.clone()));for(y=y.getNext();y;){if(y.equals(e[A])||y.equals(g))break;B=y.getNext();if(b==2)r.append(y.clone(true));else{y.remove();b==1&&r.append(y)}y=B}r&&(r=z)}r=c;for(c=w;c<e.length;c++){y=e[c];b>0&&!y.equals(g)&&(z=r.append(y.clone()));if(!h[c]||y.$.parentNode!=h[c].$.parentNode)for(y=y.getPrevious();y;){if(y.equals(h[c])||y.equals(f))break;B=y.getPrevious();if(b==2)r.$.insertBefore(y.$.cloneNode(true),r.$.firstChild);else{y.remove();b==1&&r.$.insertBefore(y.$,
r.$.firstChild)}y=B}r&&(r=z)}if(b==2){x=a.startContainer;if(x.type==CKEDITOR.NODE_TEXT){x.$.data=x.$.data+x.$.nextSibling.data;x.$.parentNode.removeChild(x.$.nextSibling)}a=a.endContainer;if(a.type==CKEDITOR.NODE_TEXT&&a.$.nextSibling){a.$.data=a.$.data+a.$.nextSibling.data;a.$.parentNode.removeChild(a.$.nextSibling)}}else{if(x&&v&&(f.$.parentNode!=x.$.parentNode||g.$.parentNode!=v.$.parentNode)){b=v.getIndex();j&&v.$.parentNode==f.$.parentNode&&b--;if(d&&x.type==CKEDITOR.NODE_ELEMENT){d=CKEDITOR.dom.element.createFromHtml('<span data-cke-bookmark="1" style="display:none">&nbsp;</span>',
a.document);d.insertAfter(x);x.mergeSiblings(false);a.moveToBookmark({startNode:d})}else a.setStart(v.getParent(),b)}a.collapse(true)}j&&f.remove();i&&g.$.parentNode&&g.remove()},d={abbr:1,acronym:1,b:1,bdo:1,big:1,cite:1,code:1,del:1,dfn:1,em:1,font:1,i:1,ins:1,label:1,kbd:1,q:1,samp:1,small:1,span:1,strike:1,strong:1,sub:1,sup:1,tt:1,u:1,"var":1},j=CKEDITOR.dom.walker.bogus(),f=/^[\t\r\n ]*(?:&nbsp;|\xa0)$/,h=CKEDITOR.dom.walker.editable(),i=CKEDITOR.dom.walker.ignored(true);CKEDITOR.dom.range.prototype=
{clone:function(){var a=new CKEDITOR.dom.range(this.root);a._setStartContainer(this.startContainer);a.startOffset=this.startOffset;a._setEndContainer(this.endContainer);a.endOffset=this.endOffset;a.collapsed=this.collapsed;return a},collapse:function(a){if(a){this._setEndContainer(this.startContainer);this.endOffset=this.startOffset}else{this._setStartContainer(this.endContainer);this.startOffset=this.endOffset}this.collapsed=true},cloneContents:function(){var a=new CKEDITOR.dom.documentFragment(this.document);
this.collapsed||c(this,2,a);return a},deleteContents:function(a){this.collapsed||c(this,0,null,a)},extractContents:function(a){var b=new CKEDITOR.dom.documentFragment(this.document);this.collapsed||c(this,1,b,a);return b},createBookmark:function(a){var b,c,d,f,g=this.collapsed;b=this.document.createElement("span");b.data("cke-bookmark",1);b.setStyle("display","none");b.setHtml("&nbsp;");if(a){d="cke_bm_"+CKEDITOR.tools.getNextNumber();b.setAttribute("id",d+(g?"C":"S"))}if(!g){c=b.clone();c.setHtml("&nbsp;");
a&&c.setAttribute("id",d+"E");f=this.clone();f.collapse();f.insertNode(c)}f=this.clone();f.collapse(true);f.insertNode(b);if(c){this.setStartAfter(b);this.setEndBefore(c)}else this.moveToPosition(b,CKEDITOR.POSITION_AFTER_END);return{startNode:a?d+(g?"C":"S"):b,endNode:a?d+"E":c,serializable:a,collapsed:g}},createBookmark2:function(){function a(c){var d=c.container,f=c.offset,g;g=d;var h=f;g=g.type!=CKEDITOR.NODE_ELEMENT||h===0||h==g.getChildCount()?0:g.getChild(h-1).type==CKEDITOR.NODE_TEXT&&g.getChild(h).type==
CKEDITOR.NODE_TEXT;if(g){d=d.getChild(f-1);f=d.getLength()}d.type==CKEDITOR.NODE_ELEMENT&&f>1&&(f=d.getChild(f-1).getIndex(true)+1);if(d.type==CKEDITOR.NODE_TEXT){g=d;for(h=0;(g=g.getPrevious())&&g.type==CKEDITOR.NODE_TEXT;)h=h+g.getLength();g=h;if(d.getText())f=f+g;else{h=d.getPrevious(b);if(g){f=g;d=h?h.getNext():d.getParent().getFirst()}else{d=d.getParent();f=h?h.getIndex(true)+1:0}}}c.container=d;c.offset=f}var b=CKEDITOR.dom.walker.nodeType(CKEDITOR.NODE_TEXT,true);return function(b){var c=this.collapsed,
d={container:this.startContainer,offset:this.startOffset},f={container:this.endContainer,offset:this.endOffset};if(b){a(d);c||a(f)}return{start:d.container.getAddress(b),end:c?null:f.container.getAddress(b),startOffset:d.offset,endOffset:f.offset,normalized:b,collapsed:c,is2:true}}}(),moveToBookmark:function(a){if(a.is2){var b=this.document.getByAddress(a.start,a.normalized),c=a.startOffset,d=a.end&&this.document.getByAddress(a.end,a.normalized),a=a.endOffset;this.setStart(b,c);d?this.setEnd(d,a):
this.collapse(true)}else{b=(c=a.serializable)?this.document.getById(a.startNode):a.startNode;a=c?this.document.getById(a.endNode):a.endNode;this.setStartBefore(b);b.remove();if(a){this.setEndBefore(a);a.remove()}else this.collapse(true)}},getBoundaryNodes:function(){var a=this.startContainer,b=this.endContainer,c=this.startOffset,d=this.endOffset,f;if(a.type==CKEDITOR.NODE_ELEMENT){f=a.getChildCount();if(f>c)a=a.getChild(c);else if(f<1)a=a.getPreviousSourceNode();else{for(a=a.$;a.lastChild;)a=a.lastChild;
a=new CKEDITOR.dom.node(a);a=a.getNextSourceNode()||a}}if(b.type==CKEDITOR.NODE_ELEMENT){f=b.getChildCount();if(f>d)b=b.getChild(d).getPreviousSourceNode(true);else if(f<1)b=b.getPreviousSourceNode();else{for(b=b.$;b.lastChild;)b=b.lastChild;b=new CKEDITOR.dom.node(b)}}a.getPosition(b)&CKEDITOR.POSITION_FOLLOWING&&(a=b);return{startNode:a,endNode:b}},getCommonAncestor:function(a,b){var c=this.startContainer,d=this.endContainer,c=c.equals(d)?a&&c.type==CKEDITOR.NODE_ELEMENT&&this.startOffset==this.endOffset-
1?c.getChild(this.startOffset):c:c.getCommonAncestor(d);return b&&!c.is?c.getParent():c},optimize:function(){var a=this.startContainer,b=this.startOffset;a.type!=CKEDITOR.NODE_ELEMENT&&(b?b>=a.getLength()&&this.setStartAfter(a):this.setStartBefore(a));a=this.endContainer;b=this.endOffset;a.type!=CKEDITOR.NODE_ELEMENT&&(b?b>=a.getLength()&&this.setEndAfter(a):this.setEndBefore(a))},optimizeBookmark:function(){var a=this.startContainer,b=this.endContainer;a.is&&(a.is("span")&&a.data("cke-bookmark"))&&
this.setStartAt(a,CKEDITOR.POSITION_BEFORE_START);b&&(b.is&&b.is("span")&&b.data("cke-bookmark"))&&this.setEndAt(b,CKEDITOR.POSITION_AFTER_END)},trim:function(a,b){var c=this.startContainer,d=this.startOffset,f=this.collapsed;if((!a||f)&&c&&c.type==CKEDITOR.NODE_TEXT){if(d)if(d>=c.getLength()){d=c.getIndex()+1;c=c.getParent()}else{var g=c.split(d),d=c.getIndex()+1,c=c.getParent();if(this.startContainer.equals(this.endContainer))this.setEnd(g,this.endOffset-this.startOffset);else if(c.equals(this.endContainer))this.endOffset=
this.endOffset+1}else{d=c.getIndex();c=c.getParent()}this.setStart(c,d);if(f){this.collapse(true);return}}c=this.endContainer;d=this.endOffset;if(!b&&!f&&c&&c.type==CKEDITOR.NODE_TEXT){if(d){d>=c.getLength()||c.split(d);d=c.getIndex()+1}else d=c.getIndex();c=c.getParent();this.setEnd(c,d)}},enlarge:function(a,b){function c(a){return a&&a.type==CKEDITOR.NODE_ELEMENT&&a.hasAttribute("contenteditable")?null:a}var d=RegExp(/[^\s\ufeff]/);switch(a){case CKEDITOR.ENLARGE_INLINE:var f=1;case CKEDITOR.ENLARGE_ELEMENT:if(this.collapsed)break;
var g=this.getCommonAncestor(),h=this.root,e,j,i,w,x,v=false,r,y;r=this.startContainer;var z=this.startOffset;if(r.type==CKEDITOR.NODE_TEXT){if(z){r=!CKEDITOR.tools.trim(r.substring(0,z)).length&&r;v=!!r}if(r&&!(w=r.getPrevious()))i=r.getParent()}else{z&&(w=r.getChild(z-1)||r.getLast());w||(i=r)}for(i=c(i);i||w;){if(i&&!w){!x&&i.equals(g)&&(x=true);if(f?i.isBlockBoundary():!h.contains(i))break;if(!v||i.getComputedStyle("display")!="inline"){v=false;x?e=i:this.setStartBefore(i)}w=i.getPrevious()}for(;w;){r=
false;if(w.type==CKEDITOR.NODE_COMMENT)w=w.getPrevious();else{if(w.type==CKEDITOR.NODE_TEXT){y=w.getText();d.test(y)&&(w=null);r=/[\s\ufeff]$/.test(y)}else if((w.$.offsetWidth>(CKEDITOR.env.webkit?1:0)||b&&w.is("br"))&&!w.data("cke-bookmark"))if(v&&CKEDITOR.dtd.$removeEmpty[w.getName()]){y=w.getText();if(d.test(y))w=null;else for(var z=w.$.getElementsByTagName("*"),B=0,A;A=z[B++];)if(!CKEDITOR.dtd.$removeEmpty[A.nodeName.toLowerCase()]){w=null;break}w&&(r=!!y.length)}else w=null;r&&(v?x?e=i:i&&this.setStartBefore(i):
v=true);if(w){r=w.getPrevious();if(!i&&!r){i=w;w=null;break}w=r}else i=null}}i&&(i=c(i.getParent()))}r=this.endContainer;z=this.endOffset;i=w=null;x=v=false;var G=function(a,b){var c=new CKEDITOR.dom.range(h);c.setStart(a,b);c.setEndAt(h,CKEDITOR.POSITION_BEFORE_END);var c=new CKEDITOR.dom.walker(c),f;for(c.guard=function(a){return!(a.type==CKEDITOR.NODE_ELEMENT&&a.isBlockBoundary())};f=c.next();){if(f.type!=CKEDITOR.NODE_TEXT)return false;y=f!=a?f.getText():f.substring(b);if(d.test(y))return false}return true};
if(r.type==CKEDITOR.NODE_TEXT)if(CKEDITOR.tools.trim(r.substring(z)).length)v=true;else{v=!r.getLength();if(z==r.getLength()){if(!(w=r.getNext()))i=r.getParent()}else G(r,z)&&(i=r.getParent())}else(w=r.getChild(z))||(i=r);for(;i||w;){if(i&&!w){!x&&i.equals(g)&&(x=true);if(f?i.isBlockBoundary():!h.contains(i))break;if(!v||i.getComputedStyle("display")!="inline"){v=false;x?j=i:i&&this.setEndAfter(i)}w=i.getNext()}for(;w;){r=false;if(w.type==CKEDITOR.NODE_TEXT){y=w.getText();G(w,0)||(w=null);r=/^[\s\ufeff]/.test(y)}else if(w.type==
CKEDITOR.NODE_ELEMENT){if((w.$.offsetWidth>0||b&&w.is("br"))&&!w.data("cke-bookmark"))if(v&&CKEDITOR.dtd.$removeEmpty[w.getName()]){y=w.getText();if(d.test(y))w=null;else{z=w.$.getElementsByTagName("*");for(B=0;A=z[B++];)if(!CKEDITOR.dtd.$removeEmpty[A.nodeName.toLowerCase()]){w=null;break}}w&&(r=!!y.length)}else w=null}else r=1;r&&v&&(x?j=i:this.setEndAfter(i));if(w){r=w.getNext();if(!i&&!r){i=w;w=null;break}w=r}else i=null}i&&(i=c(i.getParent()))}if(e&&j){g=e.contains(j)?j:e;this.setStartBefore(g);
this.setEndAfter(g)}break;case CKEDITOR.ENLARGE_BLOCK_CONTENTS:case CKEDITOR.ENLARGE_LIST_ITEM_CONTENTS:i=new CKEDITOR.dom.range(this.root);h=this.root;i.setStartAt(h,CKEDITOR.POSITION_AFTER_START);i.setEnd(this.startContainer,this.startOffset);i=new CKEDITOR.dom.walker(i);var D,J,C=CKEDITOR.dom.walker.blockBoundary(a==CKEDITOR.ENLARGE_LIST_ITEM_CONTENTS?{br:1}:null),F=null,E=function(a){if(a.type==CKEDITOR.NODE_ELEMENT&&a.getAttribute("contenteditable")=="false")if(F){if(F.equals(a)){F=null;return}}else F=
a;else if(F)return;var b=C(a);b||(D=a);return b},f=function(a){var b=E(a);!b&&(a.is&&a.is("br"))&&(J=a);return b};i.guard=E;i=i.lastBackward();D=D||h;this.setStartAt(D,!D.is("br")&&(!i&&this.checkStartOfBlock()||i&&D.contains(i))?CKEDITOR.POSITION_AFTER_START:CKEDITOR.POSITION_AFTER_END);if(a==CKEDITOR.ENLARGE_LIST_ITEM_CONTENTS){i=this.clone();i=new CKEDITOR.dom.walker(i);var I=CKEDITOR.dom.walker.whitespaces(),L=CKEDITOR.dom.walker.bookmark();i.evaluator=function(a){return!I(a)&&!L(a)};if((i=i.previous())&&
i.type==CKEDITOR.NODE_ELEMENT&&i.is("br"))break}i=this.clone();i.collapse();i.setEndAt(h,CKEDITOR.POSITION_BEFORE_END);i=new CKEDITOR.dom.walker(i);i.guard=a==CKEDITOR.ENLARGE_LIST_ITEM_CONTENTS?f:E;D=F=J=null;i=i.lastForward();D=D||h;this.setEndAt(D,!i&&this.checkEndOfBlock()||i&&D.contains(i)?CKEDITOR.POSITION_BEFORE_END:CKEDITOR.POSITION_BEFORE_START);J&&this.setEndAfter(J)}},shrink:function(a,b,c){if(!this.collapsed){var a=a||CKEDITOR.SHRINK_TEXT,d=this.clone(),f=this.startContainer,g=this.endContainer,
h=this.startOffset,e=this.endOffset,i=1,j=1;if(f&&f.type==CKEDITOR.NODE_TEXT)if(h)if(h>=f.getLength())d.setStartAfter(f);else{d.setStartBefore(f);i=0}else d.setStartBefore(f);if(g&&g.type==CKEDITOR.NODE_TEXT)if(e)if(e>=g.getLength())d.setEndAfter(g);else{d.setEndAfter(g);j=0}else d.setEndBefore(g);var d=new CKEDITOR.dom.walker(d),w=CKEDITOR.dom.walker.bookmark();d.evaluator=function(b){return b.type==(a==CKEDITOR.SHRINK_ELEMENT?CKEDITOR.NODE_ELEMENT:CKEDITOR.NODE_TEXT)};var x;d.guard=function(b,d){if(w(b))return true;
if(a==CKEDITOR.SHRINK_ELEMENT&&b.type==CKEDITOR.NODE_TEXT||d&&b.equals(x)||c===false&&b.type==CKEDITOR.NODE_ELEMENT&&b.isBlockBoundary()||b.type==CKEDITOR.NODE_ELEMENT&&b.hasAttribute("contenteditable"))return false;!d&&b.type==CKEDITOR.NODE_ELEMENT&&(x=b);return true};if(i)(f=d[a==CKEDITOR.SHRINK_ELEMENT?"lastForward":"next"]())&&this.setStartAt(f,b?CKEDITOR.POSITION_AFTER_START:CKEDITOR.POSITION_BEFORE_START);if(j){d.reset();(d=d[a==CKEDITOR.SHRINK_ELEMENT?"lastBackward":"previous"]())&&this.setEndAt(d,
b?CKEDITOR.POSITION_BEFORE_END:CKEDITOR.POSITION_AFTER_END)}return!(!i&&!j)}},insertNode:function(a){this.optimizeBookmark();this.trim(false,true);var b=this.startContainer,c=b.getChild(this.startOffset);c?a.insertBefore(c):b.append(a);a.getParent()&&a.getParent().equals(this.endContainer)&&this.endOffset++;this.setStartBefore(a)},moveToPosition:function(a,b){this.setStartAt(a,b);this.collapse(true)},moveToRange:function(a){this.setStart(a.startContainer,a.startOffset);this.setEnd(a.endContainer,
a.endOffset)},selectNodeContents:function(a){this.setStart(a,0);this.setEnd(a,a.type==CKEDITOR.NODE_TEXT?a.getLength():a.getChildCount())},setStart:function(a,b){if(a.type==CKEDITOR.NODE_ELEMENT&&CKEDITOR.dtd.$empty[a.getName()]){b=a.getIndex();a=a.getParent()}this._setStartContainer(a);this.startOffset=b;if(!this.endContainer){this._setEndContainer(a);this.endOffset=b}g(this)},setEnd:function(a,b){if(a.type==CKEDITOR.NODE_ELEMENT&&CKEDITOR.dtd.$empty[a.getName()]){b=a.getIndex()+1;a=a.getParent()}this._setEndContainer(a);
this.endOffset=b;if(!this.startContainer){this._setStartContainer(a);this.startOffset=b}g(this)},setStartAfter:function(a){this.setStart(a.getParent(),a.getIndex()+1)},setStartBefore:function(a){this.setStart(a.getParent(),a.getIndex())},setEndAfter:function(a){this.setEnd(a.getParent(),a.getIndex()+1)},setEndBefore:function(a){this.setEnd(a.getParent(),a.getIndex())},setStartAt:function(a,b){switch(b){case CKEDITOR.POSITION_AFTER_START:this.setStart(a,0);break;case CKEDITOR.POSITION_BEFORE_END:a.type==
CKEDITOR.NODE_TEXT?this.setStart(a,a.getLength()):this.setStart(a,a.getChildCount());break;case CKEDITOR.POSITION_BEFORE_START:this.setStartBefore(a);break;case CKEDITOR.POSITION_AFTER_END:this.setStartAfter(a)}g(this)},setEndAt:function(a,b){switch(b){case CKEDITOR.POSITION_AFTER_START:this.setEnd(a,0);break;case CKEDITOR.POSITION_BEFORE_END:a.type==CKEDITOR.NODE_TEXT?this.setEnd(a,a.getLength()):this.setEnd(a,a.getChildCount());break;case CKEDITOR.POSITION_BEFORE_START:this.setEndBefore(a);break;
case CKEDITOR.POSITION_AFTER_END:this.setEndAfter(a)}g(this)},fixBlock:function(a,b){var c=this.createBookmark(),d=this.document.createElement(b);this.collapse(a);this.enlarge(CKEDITOR.ENLARGE_BLOCK_CONTENTS);this.extractContents().appendTo(d);d.trim();d.appendBogus();this.insertNode(d);this.moveToBookmark(c);return d},splitBlock:function(a){var b=new CKEDITOR.dom.elementPath(this.startContainer,this.root),c=new CKEDITOR.dom.elementPath(this.endContainer,this.root),d=b.block,f=c.block,g=null;if(!b.blockLimit.equals(c.blockLimit))return null;
if(a!="br"){if(!d){d=this.fixBlock(true,a);f=(new CKEDITOR.dom.elementPath(this.endContainer,this.root)).block}f||(f=this.fixBlock(false,a))}a=d&&this.checkStartOfBlock();b=f&&this.checkEndOfBlock();this.deleteContents();if(d&&d.equals(f))if(b){g=new CKEDITOR.dom.elementPath(this.startContainer,this.root);this.moveToPosition(f,CKEDITOR.POSITION_AFTER_END);f=null}else if(a){g=new CKEDITOR.dom.elementPath(this.startContainer,this.root);this.moveToPosition(d,CKEDITOR.POSITION_BEFORE_START);d=null}else{f=
this.splitElement(d);d.is("ul","ol")||d.appendBogus()}return{previousBlock:d,nextBlock:f,wasStartOfBlock:a,wasEndOfBlock:b,elementPath:g}},splitElement:function(a){if(!this.collapsed)return null;this.setEndAt(a,CKEDITOR.POSITION_BEFORE_END);var b=this.extractContents(),c=a.clone(false);b.appendTo(c);c.insertAfter(a);this.moveToPosition(a,CKEDITOR.POSITION_AFTER_END);return c},removeEmptyBlocksAtEnd:function(){function a(d){return function(a){return b(a)||(c(a)||a.type==CKEDITOR.NODE_ELEMENT&&a.isEmptyInlineRemoveable())||
d.is("table")&&a.is("caption")?false:true}}var b=CKEDITOR.dom.walker.whitespaces(),c=CKEDITOR.dom.walker.bookmark(false);return function(b){for(var c=this.createBookmark(),d=this[b?"endPath":"startPath"](),f=d.block||d.blockLimit,g;f&&!f.equals(d.root)&&!f.getFirst(a(f));){g=f.getParent();this[b?"setEndAt":"setStartAt"](f,CKEDITOR.POSITION_AFTER_END);f.remove(1);f=g}this.moveToBookmark(c)}}(),startPath:function(){return new CKEDITOR.dom.elementPath(this.startContainer,this.root)},endPath:function(){return new CKEDITOR.dom.elementPath(this.endContainer,
this.root)},checkBoundaryOfElement:function(a,b){var c=b==CKEDITOR.START,d=this.clone();d.collapse(c);d[c?"setStartAt":"setEndAt"](a,c?CKEDITOR.POSITION_AFTER_START:CKEDITOR.POSITION_BEFORE_END);d=new CKEDITOR.dom.walker(d);d.evaluator=e(c);return d[c?"checkBackward":"checkForward"]()},checkStartOfBlock:function(){var b=this.startContainer,c=this.startOffset;if(CKEDITOR.env.ie&&c&&b.type==CKEDITOR.NODE_TEXT){b=CKEDITOR.tools.ltrim(b.substring(0,c));f.test(b)&&this.trim(0,1)}this.trim();b=new CKEDITOR.dom.elementPath(this.startContainer,
this.root);c=this.clone();c.collapse(true);c.setStartAt(b.block||b.blockLimit,CKEDITOR.POSITION_AFTER_START);b=new CKEDITOR.dom.walker(c);b.evaluator=a();return b.checkBackward()},checkEndOfBlock:function(){var b=this.endContainer,c=this.endOffset;if(CKEDITOR.env.ie&&b.type==CKEDITOR.NODE_TEXT){b=CKEDITOR.tools.rtrim(b.substring(c));f.test(b)&&this.trim(1,0)}this.trim();b=new CKEDITOR.dom.elementPath(this.endContainer,this.root);c=this.clone();c.collapse(false);c.setEndAt(b.block||b.blockLimit,CKEDITOR.POSITION_BEFORE_END);
b=new CKEDITOR.dom.walker(c);b.evaluator=a();return b.checkForward()},getPreviousNode:function(a,b,c){var d=this.clone();d.collapse(1);d.setStartAt(c||this.root,CKEDITOR.POSITION_AFTER_START);c=new CKEDITOR.dom.walker(d);c.evaluator=a;c.guard=b;return c.previous()},getNextNode:function(a,b,c){var d=this.clone();d.collapse();d.setEndAt(c||this.root,CKEDITOR.POSITION_BEFORE_END);c=new CKEDITOR.dom.walker(d);c.evaluator=a;c.guard=b;return c.next()},checkReadOnly:function(){function a(b,c){for(;b;){if(b.type==
CKEDITOR.NODE_ELEMENT){if(b.getAttribute("contentEditable")=="false"&&!b.data("cke-editable"))return 0;if(b.is("html")||b.getAttribute("contentEditable")=="true"&&(b.contains(c)||b.equals(c)))break}b=b.getParent()}return 1}return function(){var b=this.startContainer,c=this.endContainer;return!(a(b,c)&&a(c,b))}}(),moveToElementEditablePosition:function(a,b){if(a.type==CKEDITOR.NODE_ELEMENT&&!a.isEditable(false)){this.moveToPosition(a,b?CKEDITOR.POSITION_AFTER_END:CKEDITOR.POSITION_BEFORE_START);return true}for(var c=
0;a;){if(a.type==CKEDITOR.NODE_TEXT){b&&this.endContainer&&this.checkEndOfBlock()&&f.test(a.getText())?this.moveToPosition(a,CKEDITOR.POSITION_BEFORE_START):this.moveToPosition(a,b?CKEDITOR.POSITION_AFTER_END:CKEDITOR.POSITION_BEFORE_START);c=1;break}if(a.type==CKEDITOR.NODE_ELEMENT)if(a.isEditable()){this.moveToPosition(a,b?CKEDITOR.POSITION_BEFORE_END:CKEDITOR.POSITION_AFTER_START);c=1}else if(b&&a.is("br")&&this.endContainer&&this.checkEndOfBlock())this.moveToPosition(a,CKEDITOR.POSITION_BEFORE_START);
else if(a.getAttribute("contenteditable")=="false"&&a.is(CKEDITOR.dtd.$block)){this.setStartBefore(a);this.setEndAfter(a);return true}var d=a,g=c,h=void 0;d.type==CKEDITOR.NODE_ELEMENT&&d.isEditable(false)&&(h=d[b?"getLast":"getFirst"](i));!g&&!h&&(h=d[b?"getPrevious":"getNext"](i));a=h}return!!c},moveToClosestEditablePosition:function(a,b){var c=new CKEDITOR.dom.range(this.root),d=0,f,g=[CKEDITOR.POSITION_AFTER_END,CKEDITOR.POSITION_BEFORE_START];c.moveToPosition(a,g[b?0:1]);if(a.is(CKEDITOR.dtd.$block)){if(f=
c[b?"getNextEditableNode":"getPreviousEditableNode"]()){d=1;if(f.type==CKEDITOR.NODE_ELEMENT&&f.is(CKEDITOR.dtd.$block)&&f.getAttribute("contenteditable")=="false"){c.setStartAt(f,CKEDITOR.POSITION_BEFORE_START);c.setEndAt(f,CKEDITOR.POSITION_AFTER_END)}else c.moveToPosition(f,g[b?1:0])}}else d=1;d&&this.moveToRange(c);return!!d},moveToElementEditStart:function(a){return this.moveToElementEditablePosition(a)},moveToElementEditEnd:function(a){return this.moveToElementEditablePosition(a,true)},getEnclosedNode:function(){var a=
this.clone();a.optimize();if(a.startContainer.type!=CKEDITOR.NODE_ELEMENT||a.endContainer.type!=CKEDITOR.NODE_ELEMENT)return null;var a=new CKEDITOR.dom.walker(a),b=CKEDITOR.dom.walker.bookmark(false,true),c=CKEDITOR.dom.walker.whitespaces(true);a.evaluator=function(a){return c(a)&&b(a)};var d=a.next();a.reset();return d&&d.equals(a.previous())?d:null},getTouchedStartNode:function(){var a=this.startContainer;return this.collapsed||a.type!=CKEDITOR.NODE_ELEMENT?a:a.getChild(this.startOffset)||a},getTouchedEndNode:function(){var a=
this.endContainer;return this.collapsed||a.type!=CKEDITOR.NODE_ELEMENT?a:a.getChild(this.endOffset-1)||a},getNextEditableNode:b(),getPreviousEditableNode:b(1),scrollIntoView:function(){var a=new CKEDITOR.dom.element.createFromHtml("<span>&nbsp;</span>",this.document),b,c,d,f=this.clone();f.optimize();if(d=f.startContainer.type==CKEDITOR.NODE_TEXT){c=f.startContainer.getText();b=f.startContainer.split(f.startOffset);a.insertAfter(f.startContainer)}else f.insertNode(a);a.scrollIntoView();if(d){f.startContainer.setText(c);
b.remove()}a.remove()},_setStartContainer:function(a){this.startContainer=a},_setEndContainer:function(a){this.endContainer=a}}}(),CKEDITOR.POSITION_AFTER_START=1,CKEDITOR.POSITION_BEFORE_END=2,CKEDITOR.POSITION_BEFORE_START=3,CKEDITOR.POSITION_AFTER_END=4,CKEDITOR.ENLARGE_ELEMENT=1,CKEDITOR.ENLARGE_BLOCK_CONTENTS=2,CKEDITOR.ENLARGE_LIST_ITEM_CONTENTS=3,CKEDITOR.ENLARGE_INLINE=4,CKEDITOR.START=1,CKEDITOR.END=2,CKEDITOR.SHRINK_ELEMENT=1,CKEDITOR.SHRINK_TEXT=2,"use strict",function(){function a(a){if(!(arguments.length<
1)){this.range=a;this.forceBrBreak=0;this.enlargeBr=1;this.enforceRealBlocks=0;this._||(this._={})}}function e(a){var b=[];a.forEach(function(a){if(a.getAttribute("contenteditable")=="true"){b.push(a);return false}},CKEDITOR.NODE_ELEMENT,true);return b}function b(a,c,d,f){a:{f==null&&(f=e(d));for(var g;g=f.shift();)if(g.getDtd().p){f={element:g,remaining:f};break a}f=null}if(!f)return 0;if((g=CKEDITOR.filter.instances[f.element.data("cke-filter")])&&!g.check(c))return b(a,c,d,f.remaining);c=new CKEDITOR.dom.range(f.element);
c.selectNodeContents(f.element);c=c.createIterator();c.enlargeBr=a.enlargeBr;c.enforceRealBlocks=a.enforceRealBlocks;c.activeFilter=c.filter=g;a._.nestedEditable={element:f.element,container:d,remaining:f.remaining,iterator:c};return 1}function g(a,b,c){if(!b)return false;a=a.clone();a.collapse(!c);return a.checkBoundaryOfElement(b,c?CKEDITOR.START:CKEDITOR.END)}var c=/^[\r\n\t ]+$/,d=CKEDITOR.dom.walker.bookmark(false,true),j=CKEDITOR.dom.walker.whitespaces(true),f=function(a){return d(a)&&j(a)},
h={dd:1,dt:1,li:1};a.prototype={getNextParagraph:function(a){var e,j,o,q,m,a=a||"p";if(this._.nestedEditable){if(e=this._.nestedEditable.iterator.getNextParagraph(a)){this.activeFilter=this._.nestedEditable.iterator.activeFilter;return e}this.activeFilter=this.filter;if(b(this,a,this._.nestedEditable.container,this._.nestedEditable.remaining)){this.activeFilter=this._.nestedEditable.iterator.activeFilter;return this._.nestedEditable.iterator.getNextParagraph(a)}this._.nestedEditable=null}if(!this.range.root.getDtd()[a])return null;
if(!this._.started){var l=this.range.clone();j=l.startPath();var p=l.endPath(),t=!l.collapsed&&g(l,j.block),s=!l.collapsed&&g(l,p.block,1);l.shrink(CKEDITOR.SHRINK_ELEMENT,true);t&&l.setStartAt(j.block,CKEDITOR.POSITION_BEFORE_END);s&&l.setEndAt(p.block,CKEDITOR.POSITION_AFTER_START);j=l.endContainer.hasAscendant("pre",true)||l.startContainer.hasAscendant("pre",true);l.enlarge(this.forceBrBreak&&!j||!this.enlargeBr?CKEDITOR.ENLARGE_LIST_ITEM_CONTENTS:CKEDITOR.ENLARGE_BLOCK_CONTENTS);if(!l.collapsed){j=
new CKEDITOR.dom.walker(l.clone());p=CKEDITOR.dom.walker.bookmark(true,true);j.evaluator=p;this._.nextNode=j.next();j=new CKEDITOR.dom.walker(l.clone());j.evaluator=p;j=j.previous();this._.lastNode=j.getNextSourceNode(true,null,l.root);if(this._.lastNode&&this._.lastNode.type==CKEDITOR.NODE_TEXT&&!CKEDITOR.tools.trim(this._.lastNode.getText())&&this._.lastNode.getParent().isBlockBoundary()){p=this.range.clone();p.moveToPosition(this._.lastNode,CKEDITOR.POSITION_AFTER_END);if(p.checkEndOfBlock()){p=
new CKEDITOR.dom.elementPath(p.endContainer,p.root);this._.lastNode=(p.block||p.blockLimit).getNextSourceNode(true)}}if(!this._.lastNode||!l.root.contains(this._.lastNode)){this._.lastNode=this._.docEndMarker=l.document.createText("");this._.lastNode.insertAfter(j)}l=null}this._.started=1;j=l}p=this._.nextNode;l=this._.lastNode;for(this._.nextNode=null;p;){var t=0,s=p.hasAscendant("pre"),u=p.type!=CKEDITOR.NODE_ELEMENT,w=0;if(u)p.type==CKEDITOR.NODE_TEXT&&c.test(p.getText())&&(u=0);else{var x=p.getName();
if(CKEDITOR.dtd.$block[x]&&p.getAttribute("contenteditable")=="false"){e=p;b(this,a,e);break}else if(p.isBlockBoundary(this.forceBrBreak&&!s&&{br:1})){if(x=="br")u=1;else if(!j&&!p.getChildCount()&&x!="hr"){e=p;o=p.equals(l);break}if(j){j.setEndAt(p,CKEDITOR.POSITION_BEFORE_START);if(x!="br")this._.nextNode=p}t=1}else{if(p.getFirst()){if(!j){j=this.range.clone();j.setStartAt(p,CKEDITOR.POSITION_BEFORE_START)}p=p.getFirst();continue}u=1}}if(u&&!j){j=this.range.clone();j.setStartAt(p,CKEDITOR.POSITION_BEFORE_START)}o=
(!t||u)&&p.equals(l);if(j&&!t)for(;!p.getNext(f)&&!o;){x=p.getParent();if(x.isBlockBoundary(this.forceBrBreak&&!s&&{br:1})){t=1;u=0;o||x.equals(l);j.setEndAt(x,CKEDITOR.POSITION_BEFORE_END);break}p=x;u=1;o=p.equals(l);w=1}u&&j.setEndAt(p,CKEDITOR.POSITION_AFTER_END);p=this._getNextSourceNode(p,w,l);if((o=!p)||t&&j)break}if(!e){if(!j){this._.docEndMarker&&this._.docEndMarker.remove();return this._.nextNode=null}e=new CKEDITOR.dom.elementPath(j.startContainer,j.root);p=e.blockLimit;t={div:1,th:1,td:1};
e=e.block;if(!e&&p&&!this.enforceRealBlocks&&t[p.getName()]&&j.checkStartOfBlock()&&j.checkEndOfBlock()&&!p.equals(j.root))e=p;else if(!e||this.enforceRealBlocks&&e.is(h)){e=this.range.document.createElement(a);j.extractContents().appendTo(e);e.trim();j.insertNode(e);q=m=true}else if(e.getName()!="li"){if(!j.checkStartOfBlock()||!j.checkEndOfBlock()){e=e.clone(false);j.extractContents().appendTo(e);e.trim();m=j.splitBlock();q=!m.wasStartOfBlock;m=!m.wasEndOfBlock;j.insertNode(e)}}else if(!o)this._.nextNode=
e.equals(l)?null:this._getNextSourceNode(j.getBoundaryNodes().endNode,1,l)}if(q)(q=e.getPrevious())&&q.type==CKEDITOR.NODE_ELEMENT&&(q.getName()=="br"?q.remove():q.getLast()&&q.getLast().$.nodeName.toLowerCase()=="br"&&q.getLast().remove());if(m)(q=e.getLast())&&q.type==CKEDITOR.NODE_ELEMENT&&q.getName()=="br"&&(!CKEDITOR.env.needsBrFiller||q.getPrevious(d)||q.getNext(d))&&q.remove();if(!this._.nextNode)this._.nextNode=o||e.equals(l)||!l?null:this._getNextSourceNode(e,1,l);return e},_getNextSourceNode:function(a,
b,c){function f(a){return!(a.equals(c)||a.equals(g))}for(var g=this.range.root,a=a.getNextSourceNode(b,null,f);!d(a);)a=a.getNextSourceNode(b,null,f);return a}};CKEDITOR.dom.range.prototype.createIterator=function(){return new a(this)}}(),CKEDITOR.command=function(a,e){this.uiItems=[];this.exec=function(b){if(this.state==CKEDITOR.TRISTATE_DISABLED||!this.checkAllowed())return false;this.editorFocus&&a.focus();return this.fire("exec")===false?true:e.exec.call(this,a,b)!==false};this.refresh=function(a,
b){if(!this.readOnly&&a.readOnly)return true;if(this.context&&!b.isContextFor(this.context)){this.disable();return true}if(!this.checkAllowed(true)){this.disable();return true}this.startDisabled||this.enable();this.modes&&!this.modes[a.mode]&&this.disable();return this.fire("refresh",{editor:a,path:b})===false?true:e.refresh&&e.refresh.apply(this,arguments)!==false};var b;this.checkAllowed=function(g){return!g&&typeof b=="boolean"?b:b=a.activeFilter.checkFeature(this)};CKEDITOR.tools.extend(this,
e,{modes:{wysiwyg:1},editorFocus:1,contextSensitive:!!e.context,state:CKEDITOR.TRISTATE_DISABLED});CKEDITOR.event.call(this)},CKEDITOR.command.prototype={enable:function(){this.state==CKEDITOR.TRISTATE_DISABLED&&this.checkAllowed()&&this.setState(!this.preserveState||typeof this.previousState=="undefined"?CKEDITOR.TRISTATE_OFF:this.previousState)},disable:function(){this.setState(CKEDITOR.TRISTATE_DISABLED)},setState:function(a){if(this.state==a||a!=CKEDITOR.TRISTATE_DISABLED&&!this.checkAllowed())return false;
this.previousState=this.state;this.state=a;this.fire("state");return true},toggleState:function(){this.state==CKEDITOR.TRISTATE_OFF?this.setState(CKEDITOR.TRISTATE_ON):this.state==CKEDITOR.TRISTATE_ON&&this.setState(CKEDITOR.TRISTATE_OFF)}},CKEDITOR.event.implementOn(CKEDITOR.command.prototype),CKEDITOR.ENTER_P=1,CKEDITOR.ENTER_BR=2,CKEDITOR.ENTER_DIV=3,CKEDITOR.config={customConfig:"config.js",autoUpdateElement:!0,language:"",defaultLanguage:"en",contentsLangDirection:"",enterMode:CKEDITOR.ENTER_P,
forceEnterMode:!1,shiftEnterMode:CKEDITOR.ENTER_BR,docType:"<!DOCTYPE html>",bodyId:"",bodyClass:"",fullPage:!1,height:200,extraPlugins:"",removePlugins:"",protectedSource:[],tabIndex:0,width:"",baseFloatZIndex:1E4,blockedKeystrokes:[CKEDITOR.CTRL+66,CKEDITOR.CTRL+73,CKEDITOR.CTRL+85]},function(){function a(a,b,c,d,f){var g,h,a=[];for(g in b){h=b[g];h=typeof h=="boolean"?{}:typeof h=="function"?{match:h}:G(h);if(g.charAt(0)!="$")h.elements=g;if(c)h.featureName=c.toLowerCase();var e=h;e.elements=j(e.elements,
/\s+/)||null;e.propertiesOnly=e.propertiesOnly||e.elements===true;var i=/\s*,\s*/,p=void 0;for(p in F){e[p]=j(e[p],i)||null;var m=e,k=E[p],x=j(e[E[p]],i),s=e[p],w=[],l=true,C=void 0;x?l=false:x={};for(C in s)if(C.charAt(0)=="!"){C=C.slice(1);w.push(C);x[C]=true;l=false}for(;C=w.pop();){s[C]=s["!"+C];delete s["!"+C]}m[k]=(l?false:x)||null}e.match=e.match||null;d.push(h);a.push(h)}for(var b=f.elements,f=f.generic,n,c=0,d=a.length;c<d;++c){g=G(a[c]);h=g.classes===true||g.styles===true||g.attributes===
true;e=g;p=k=i=void 0;for(i in F)e[i]=t(e[i]);m=true;for(p in E){i=E[p];k=e[i];x=[];s=void 0;for(s in k)s.indexOf("*")>-1?x.push(RegExp("^"+s.replace(/\*/g,".*")+"$")):x.push(s);k=x;if(k.length){e[i]=k;m=false}}e.nothingRequired=m;e.noProperties=!(e.attributes||e.classes||e.styles);if(g.elements===true||g.elements===null)f[h?"unshift":"push"](g);else{e=g.elements;delete g.elements;for(n in e)if(b[n])b[n][h?"unshift":"push"](g);else b[n]=[g]}}}function e(a,c,d,g){if(!a.match||a.match(c))if(g||f(a,
c)){if(!a.propertiesOnly)d.valid=true;if(!d.allAttributes)d.allAttributes=b(a.attributes,c.attributes,d.validAttributes);if(!d.allStyles)d.allStyles=b(a.styles,c.styles,d.validStyles);if(!d.allClasses){a=a.classes;c=c.classes;g=d.validClasses;if(a)if(a===true)a=true;else{for(var h=0,e=c.length,j;h<e;++h){j=c[h];g[j]||(g[j]=a(j))}a=false}else a=false;d.allClasses=a}}}function b(a,b,c){if(!a)return false;if(a===true)return true;for(var d in b)c[d]||(c[d]=a(d));return false}function g(a,b,d){if(!a.match||
a.match(b)){if(a.noProperties)return false;d.hadInvalidAttribute=c(a.attributes,b.attributes)||d.hadInvalidAttribute;d.hadInvalidStyle=c(a.styles,b.styles)||d.hadInvalidStyle;a=a.classes;b=b.classes;if(a){for(var f=false,g=a===true,h=b.length;h--;)if(g||a(b[h])){b.splice(h,1);f=true}a=f}else a=false;d.hadInvalidClass=a||d.hadInvalidClass}}function c(a,b){if(!a)return false;var c=false,d=a===true,f;for(f in b)if(d||a(f)){delete b[f];c=true}return c}function d(a,b,c){if(a.disabled||a.customConfig&&
!c||!b)return false;a._.cachedChecks={};return true}function j(a,b){if(!a)return false;if(a===true)return a;if(typeof a=="string"){a=D(a);return a=="*"?true:CKEDITOR.tools.convertArrayToObject(a.split(b))}if(CKEDITOR.tools.isArray(a))return a.length?CKEDITOR.tools.convertArrayToObject(a):false;var c={},d=0,f;for(f in a){c[f]=a[f];d++}return d?c:false}function f(a,b){if(a.nothingRequired)return true;var c,d,f,g;if(f=a.requiredClasses){g=b.classes;for(c=0;c<f.length;++c){d=f[c];if(typeof d=="string"){if(CKEDITOR.tools.indexOf(g,
d)==-1)return false}else if(!CKEDITOR.tools.checkIfAnyArrayItemMatches(g,d))return false}}return h(b.styles,a.requiredStyles)&&h(b.attributes,a.requiredAttributes)}function h(a,b){if(!b)return true;for(var c=0,d;c<b.length;++c){d=b[c];if(typeof d=="string"){if(!(d in a))return false}else if(!CKEDITOR.tools.checkIfAnyObjectPropertyMatches(a,d))return false}return true}function i(a){if(!a)return{};for(var a=a.split(/\s*,\s*/).sort(),b={};a.length;)b[a.shift()]=J;return b}function k(a){for(var b,c,d,
f,g={},h=1,a=D(a);b=a.match(I);){if(c=b[2]){d=n(c,"styles");f=n(c,"attrs");c=n(c,"classes")}else d=f=c=null;g["$"+h++]={elements:b[1],classes:c,styles:d,attributes:f};a=a.slice(b[0].length)}return g}function n(a,b){var c=a.match(L[b]);return c?D(c[1]):null}function o(a){var b=a.styleBackup=a.attributes.style,c=a.classBackup=a.attributes["class"];if(!a.styles)a.styles=CKEDITOR.tools.parseCssText(b||"",1);if(!a.classes)a.classes=c?c.split(/\s+/):[]}function q(a,b,c,d){var f=0,h;if(d.toHtml)b.name=b.name.replace(H,
"$1");if(d.doCallbacks&&a.elementCallbacks){a:for(var j=a.elementCallbacks,i=0,m=j.length,t;i<m;++i)if(t=j[i](b)){h=t;break a}if(h)return h}if(d.doTransform)if(h=a._.transformations[b.name]){o(b);for(j=0;j<h.length;++j)x(a,b,h[j]);l(b)}if(d.doFilter){a:{j=b.name;i=a._;a=i.allowedRules.elements[j];h=i.allowedRules.generic;j=i.disallowedRules.elements[j];i=i.disallowedRules.generic;m=d.skipRequired;t={valid:false,validAttributes:{},validClasses:{},validStyles:{},allAttributes:false,allClasses:false,
allStyles:false,hadInvalidAttribute:false,hadInvalidClass:false,hadInvalidStyle:false};var k,s;if(!a&&!h)a=null;else{o(b);if(j){k=0;for(s=j.length;k<s;++k)if(g(j[k],b,t)===false){a=null;break a}}if(i){k=0;for(s=i.length;k<s;++k)g(i[k],b,t)}if(a){k=0;for(s=a.length;k<s;++k)e(a[k],b,t,m)}if(h){k=0;for(s=h.length;k<s;++k)e(h[k],b,t,m)}a=t}}if(!a){c.push(b);return A}if(!a.valid){c.push(b);return A}s=a.validAttributes;var w=a.validStyles;h=a.validClasses;var j=b.attributes,C=b.styles,i=b.classes,m=b.classBackup,
n=b.styleBackup,v,F,r=[];t=[];var u=/^data-cke-/;k=false;delete j.style;delete j["class"];delete b.classBackup;delete b.styleBackup;if(!a.allAttributes)for(v in j)if(!s[v])if(u.test(v)){if(v!=(F=v.replace(/^data-cke-saved-/,""))&&!s[F]){delete j[v];k=true}}else{delete j[v];k=true}if(!a.allStyles||a.hadInvalidStyle){for(v in C)a.allStyles||w[v]?r.push(v+":"+C[v]):k=true;if(r.length)j.style=r.sort().join("; ")}else if(n)j.style=n;if(!a.allClasses||a.hadInvalidClass){for(v=0;v<i.length;++v)(a.allClasses||
h[i[v]])&&t.push(i[v]);t.length&&(j["class"]=t.sort().join(" "));m&&t.length<m.split(/\s+/).length&&(k=true)}else m&&(j["class"]=m);k&&(f=A);if(!d.skipFinalValidation&&!p(b)){c.push(b);return A}}if(d.toHtml)b.name=b.name.replace(Q,"cke:$1");return f}function m(a){var b=[],c;for(c in a)c.indexOf("*")>-1&&b.push(c.replace(/\*/g,".*"));return b.length?RegExp("^(?:"+b.join("|")+")$"):null}function l(a){var b=a.attributes,c;delete b.style;delete b["class"];if(c=CKEDITOR.tools.writeCssText(a.styles,true))b.style=
c;a.classes.length&&(b["class"]=a.classes.sort().join(" "))}function p(a){switch(a.name){case "a":if(!a.children.length&&!a.attributes.name)return false;break;case "img":if(!a.attributes.src)return false}return true}function t(a){if(!a)return false;if(a===true)return true;var b=m(a);return function(c){return c in a||b&&c.match(b)}}function s(){return new CKEDITOR.htmlParser.element("br")}function u(a){return a.type==CKEDITOR.NODE_ELEMENT&&(a.name=="br"||B.$block[a.name])}function w(a,b,c){var d=a.name;
if(B.$empty[d]||!a.children.length)if(d=="hr"&&b=="br")a.replaceWith(s());else{a.parent&&c.push({check:"it",el:a.parent});a.remove()}else if(B.$block[d]||d=="tr")if(b=="br"){if(a.previous&&!u(a.previous)){b=s();b.insertBefore(a)}if(a.next&&!u(a.next)){b=s();b.insertAfter(a)}a.replaceWithChildren()}else{var d=a.children,f;b:{f=B[b];for(var g=0,h=d.length,e;g<h;++g){e=d[g];if(e.type==CKEDITOR.NODE_ELEMENT&&!f[e.name]){f=false;break b}}f=true}if(f){a.name=b;a.attributes={};c.push({check:"parent-down",
el:a})}else{f=a.parent;for(var g=f.type==CKEDITOR.NODE_DOCUMENT_FRAGMENT||f.name=="body",j,i,h=d.length;h>0;){e=d[--h];if(g&&(e.type==CKEDITOR.NODE_TEXT||e.type==CKEDITOR.NODE_ELEMENT&&B.$inline[e.name])){if(!j){j=new CKEDITOR.htmlParser.element(b);j.insertAfter(a);c.push({check:"parent-down",el:j})}j.add(e,0)}else{j=null;i=B[f.name]||B.span;e.insertAfter(a);f.type!=CKEDITOR.NODE_DOCUMENT_FRAGMENT&&(e.type==CKEDITOR.NODE_ELEMENT&&!i[e.name])&&c.push({check:"el-up",el:e})}}a.remove()}}else if(d=="style")a.remove();
else{a.parent&&c.push({check:"it",el:a.parent});a.replaceWithChildren()}}function x(a,b,c){var d,f;for(d=0;d<c.length;++d){f=c[d];if((!f.check||a.check(f.check,false))&&(!f.left||f.left(b))){f.right(b,O);break}}}function v(a,b){var c=b.getDefinition(),d=c.attributes,f=c.styles,g,h,e,j;if(a.name!=c.element)return false;for(g in d)if(g=="class"){c=d[g].split(/\s+/);for(e=a.classes.join("|");j=c.pop();)if(e.indexOf(j)==-1)return false}else if(a.attributes[g]!=d[g])return false;for(h in f)if(a.styles[h]!=
f[h])return false;return true}function r(a,b){var c,d;if(typeof a=="string")c=a;else if(a instanceof CKEDITOR.style)d=a;else{c=a[0];d=a[1]}return[{element:c,left:d,right:function(a,c){c.transform(a,b)}}]}function y(a){return function(b){return v(b,a)}}function z(a){return function(b,c){c[a](b)}}var B=CKEDITOR.dtd,A=1,G=CKEDITOR.tools.copy,D=CKEDITOR.tools.trim,J="cke-test",C=["","p","br","div"];CKEDITOR.FILTER_SKIP_TREE=2;CKEDITOR.filter=function(a){this.allowedContent=[];this.disallowedContent=[];
this.elementCallbacks=null;this.disabled=false;this.editor=null;this.id=CKEDITOR.tools.getNextNumber();this._={allowedRules:{elements:{},generic:[]},disallowedRules:{elements:{},generic:[]},transformations:{},cachedTests:{}};CKEDITOR.filter.instances[this.id]=this;if(a instanceof CKEDITOR.editor){a=this.editor=a;this.customConfig=true;var b=a.config.allowedContent;if(b===true)this.disabled=true;else{if(!b)this.customConfig=false;this.allow(b,"config",1);this.allow(a.config.extraAllowedContent,"extra",
1);this.allow(C[a.enterMode]+" "+C[a.shiftEnterMode],"default",1);this.disallow(a.config.disallowedContent)}}else{this.customConfig=false;this.allow(a,"default",1)}};CKEDITOR.filter.instances={};CKEDITOR.filter.prototype={allow:function(b,c,f){if(!d(this,b,f))return false;var g,h;if(typeof b=="string")b=k(b);else if(b instanceof CKEDITOR.style){if(b.toAllowedContentRules)return this.allow(b.toAllowedContentRules(this.editor),c,f);g=b.getDefinition();b={};f=g.attributes;b[g.element]=g={styles:g.styles,
requiredStyles:g.styles&&CKEDITOR.tools.objectKeys(g.styles)};if(f){f=G(f);g.classes=f["class"]?f["class"].split(/\s+/):null;g.requiredClasses=g.classes;delete f["class"];g.attributes=f;g.requiredAttributes=f&&CKEDITOR.tools.objectKeys(f)}}else if(CKEDITOR.tools.isArray(b)){for(g=0;g<b.length;++g)h=this.allow(b[g],c,f);return h}a(this,b,c,this.allowedContent,this._.allowedRules);return true},applyTo:function(a,b,c,d){if(this.disabled)return false;var f=this,g=[],h=this.editor&&this.editor.config.protectedSource,
e,j=false,i={doFilter:!c,doTransform:true,doCallbacks:true,toHtml:b};a.forEach(function(a){if(a.type==CKEDITOR.NODE_ELEMENT){if(a.attributes["data-cke-filter"]=="off")return false;if(!b||!(a.name=="span"&&~CKEDITOR.tools.objectKeys(a.attributes).join("|").indexOf("data-cke-"))){e=q(f,a,g,i);if(e&A)j=true;else if(e&2)return false}}else if(a.type==CKEDITOR.NODE_COMMENT&&a.value.match(/^\{cke_protected\}(?!\{C\})/)){var c;a:{var d=decodeURIComponent(a.value.replace(/^\{cke_protected\}/,""));c=[];var p,
m,t;if(h)for(m=0;m<h.length;++m)if((t=d.match(h[m]))&&t[0].length==d.length){c=true;break a}d=CKEDITOR.htmlParser.fragment.fromHtml(d);d.children.length==1&&(p=d.children[0]).type==CKEDITOR.NODE_ELEMENT&&q(f,p,c,i);c=!c.length}c||g.push(a)}},null,true);g.length&&(j=true);for(var m,a=[],d=C[d||(this.editor?this.editor.enterMode:CKEDITOR.ENTER_P)],t;c=g.pop();)c.type==CKEDITOR.NODE_ELEMENT?w(c,d,a):c.remove();for(;m=a.pop();){c=m.el;if(c.parent){t=B[c.parent.name]||B.span;switch(m.check){case "it":B.$removeEmpty[c.name]&&
!c.children.length?w(c,d,a):p(c)||w(c,d,a);break;case "el-up":c.parent.type!=CKEDITOR.NODE_DOCUMENT_FRAGMENT&&!t[c.name]&&w(c,d,a);break;case "parent-down":c.parent.type!=CKEDITOR.NODE_DOCUMENT_FRAGMENT&&!t[c.name]&&w(c.parent,d,a)}}}return j},checkFeature:function(a){if(this.disabled||!a)return true;a.toFeature&&(a=a.toFeature(this.editor));return!a.requiredContent||this.check(a.requiredContent)},disable:function(){this.disabled=true},disallow:function(b){if(!d(this,b,true))return false;typeof b==
"string"&&(b=k(b));a(this,b,null,this.disallowedContent,this._.disallowedRules);return true},addContentForms:function(a){if(!this.disabled&&a){var b,c,d=[],f;for(b=0;b<a.length&&!f;++b){c=a[b];if((typeof c=="string"||c instanceof CKEDITOR.style)&&this.check(c))f=c}if(f){for(b=0;b<a.length;++b)d.push(r(a[b],f));this.addTransformations(d)}}},addElementCallback:function(a){if(!this.elementCallbacks)this.elementCallbacks=[];this.elementCallbacks.push(a)},addFeature:function(a){if(this.disabled||!a)return true;
a.toFeature&&(a=a.toFeature(this.editor));this.allow(a.allowedContent,a.name);this.addTransformations(a.contentTransformations);this.addContentForms(a.contentForms);return a.requiredContent&&(this.customConfig||this.disallowedContent.length)?this.check(a.requiredContent):true},addTransformations:function(a){var b,c;if(!this.disabled&&a){var d=this._.transformations,f;for(f=0;f<a.length;++f){b=a[f];var g=void 0,h=void 0,e=void 0,j=void 0,i=void 0,p=void 0;c=[];for(h=0;h<b.length;++h){e=b[h];if(typeof e==
"string"){e=e.split(/\s*:\s*/);j=e[0];i=null;p=e[1]}else{j=e.check;i=e.left;p=e.right}if(!g){g=e;g=g.element?g.element:j?j.match(/^([a-z0-9]+)/i)[0]:g.left.getDefinition().element}i instanceof CKEDITOR.style&&(i=y(i));c.push({check:j==g?null:j,left:i,right:typeof p=="string"?z(p):p})}b=g;d[b]||(d[b]=[]);d[b].push(c)}}},check:function(a,b,c){if(this.disabled)return true;if(CKEDITOR.tools.isArray(a)){for(var d=a.length;d--;)if(this.check(a[d],b,c))return true;return false}var f,g;if(typeof a=="string"){g=
a+"<"+(b===false?"0":"1")+(c?"1":"0")+">";if(g in this._.cachedChecks)return this._.cachedChecks[g];d=k(a).$1;f=d.styles;var h=d.classes;d.name=d.elements;d.classes=h=h?h.split(/\s*,\s*/):[];d.styles=i(f);d.attributes=i(d.attributes);d.children=[];h.length&&(d.attributes["class"]=h.join(" "));if(f)d.attributes.style=CKEDITOR.tools.writeCssText(d.styles);f=d}else{d=a.getDefinition();f=d.styles;h=d.attributes||{};if(f){f=G(f);h.style=CKEDITOR.tools.writeCssText(f,true)}else f={};f={name:d.element,attributes:h,
classes:h["class"]?h["class"].split(/\s+/):[],styles:f,children:[]}}var h=CKEDITOR.tools.clone(f),e=[],j;if(b!==false&&(j=this._.transformations[f.name])){for(d=0;d<j.length;++d)x(this,f,j[d]);l(f)}q(this,h,e,{doFilter:true,doTransform:b!==false,skipRequired:!c,skipFinalValidation:!c});b=e.length>0?false:CKEDITOR.tools.objectCompare(f.attributes,h.attributes,true)?true:false;typeof a=="string"&&(this._.cachedChecks[g]=b);return b},getAllowedEnterMode:function(){var a=["p","div","br"],b={p:CKEDITOR.ENTER_P,
div:CKEDITOR.ENTER_DIV,br:CKEDITOR.ENTER_BR};return function(c,d){var f=a.slice(),g;if(this.check(C[c]))return c;for(d||(f=f.reverse());g=f.pop();)if(this.check(g))return b[g];return CKEDITOR.ENTER_BR}}(),destroy:function(){delete CKEDITOR.filter.instances[this.id];delete this._;delete this.allowedContent;delete this.disallowedContent}};var F={styles:1,attributes:1,classes:1},E={styles:"requiredStyles",attributes:"requiredAttributes",classes:"requiredClasses"},I=/^([a-z0-9\-*\s]+)((?:\s*\{[!\w\-,\s\*]+\}\s*|\s*\[[!\w\-,\s\*]+\]\s*|\s*\([!\w\-,\s\*]+\)\s*){0,3})(?:;\s*|$)/i,
L={styles:/{([^}]+)}/,attrs:/\[([^\]]+)\]/,classes:/\(([^\)]+)\)/},H=/^cke:(object|embed|param)$/,Q=/^(object|embed|param)$/,O=CKEDITOR.filter.transformationsTools={sizeToStyle:function(a){this.lengthToStyle(a,"width");this.lengthToStyle(a,"height")},sizeToAttribute:function(a){this.lengthToAttribute(a,"width");this.lengthToAttribute(a,"height")},lengthToStyle:function(a,b,c){c=c||b;if(!(c in a.styles)){var d=a.attributes[b];if(d){/^\d+$/.test(d)&&(d=d+"px");a.styles[c]=d}}delete a.attributes[b]},
lengthToAttribute:function(a,b,c){c=c||b;if(!(c in a.attributes)){var d=a.styles[b],f=d&&d.match(/^(\d+)(?:\.\d*)?px$/);f?a.attributes[c]=f[1]:d==J&&(a.attributes[c]=J)}delete a.styles[b]},alignmentToStyle:function(a){if(!("float"in a.styles)){var b=a.attributes.align;if(b=="left"||b=="right")a.styles["float"]=b}delete a.attributes.align},alignmentToAttribute:function(a){if(!("align"in a.attributes)){var b=a.styles["float"];if(b=="left"||b=="right")a.attributes.align=b}delete a.styles["float"]},matchesStyle:v,
transform:function(a,b){if(typeof b=="string")a.name=b;else{var c=b.getDefinition(),d=c.styles,f=c.attributes,g,h,e,j;a.name=c.element;for(g in f)if(g=="class"){c=a.classes.join("|");for(e=f[g].split(/\s+/);j=e.pop();)c.indexOf(j)==-1&&a.classes.push(j)}else a.attributes[g]=f[g];for(h in d)a.styles[h]=d[h]}}}}(),function(){CKEDITOR.focusManager=function(a){if(a.focusManager)return a.focusManager;this.hasFocus=false;this.currentActive=null;this._={editor:a};return this};CKEDITOR.focusManager._={blurDelay:200};
CKEDITOR.focusManager.prototype={focus:function(a){this._.timer&&clearTimeout(this._.timer);if(a)this.currentActive=a;if(!this.hasFocus&&!this._.locked){(a=CKEDITOR.currentInstance)&&a.focusManager.blur(1);this.hasFocus=true;(a=this._.editor.container)&&a.addClass("cke_focus");this._.editor.fire("focus")}},lock:function(){this._.locked=1},unlock:function(){delete this._.locked},blur:function(a){function e(){if(this.hasFocus){this.hasFocus=false;var a=this._.editor.container;a&&a.removeClass("cke_focus");
this._.editor.fire("blur")}}if(!this._.locked){this._.timer&&clearTimeout(this._.timer);var b=CKEDITOR.focusManager._.blurDelay;a||!b?e.call(this):this._.timer=CKEDITOR.tools.setTimeout(function(){delete this._.timer;e.call(this)},b,this)}},add:function(a,e){var b=a.getCustomData("focusmanager");if(!b||b!=this){b&&b.remove(a);var b="focus",g="blur";if(e)if(CKEDITOR.env.ie){b="focusin";g="focusout"}else CKEDITOR.event.useCapture=1;var c={blur:function(){a.equals(this.currentActive)&&this.blur()},focus:function(){this.focus(a)}};
a.on(b,c.focus,this);a.on(g,c.blur,this);if(e)CKEDITOR.event.useCapture=0;a.setCustomData("focusmanager",this);a.setCustomData("focusmanager_handlers",c)}},remove:function(a){a.removeCustomData("focusmanager");var e=a.removeCustomData("focusmanager_handlers");a.removeListener("blur",e.blur);a.removeListener("focus",e.focus)}}}(),CKEDITOR.keystrokeHandler=function(a){if(a.keystrokeHandler)return a.keystrokeHandler;this.keystrokes={};this.blockedKeystrokes={};this._={editor:a};return this},function(){var a,
e=function(b){var b=b.data,c=b.getKeystroke(),d=this.keystrokes[c],e=this._.editor;a=e.fire("key",{keyCode:c,domEvent:b})===false;if(!a){d&&(a=e.execCommand(d,{from:"keystrokeHandler"})!==false);a||(a=!!this.blockedKeystrokes[c])}a&&b.preventDefault(true);return!a},b=function(b){if(a){a=false;b.data.preventDefault(true)}};CKEDITOR.keystrokeHandler.prototype={attach:function(a){a.on("keydown",e,this);if(CKEDITOR.env.gecko&&CKEDITOR.env.mac)a.on("keypress",b,this)}}}(),function(){CKEDITOR.lang={languages:{af:1,
ar:1,bg:1,bn:1,bs:1,ca:1,cs:1,cy:1,da:1,de:1,el:1,"en-au":1,"en-ca":1,"en-gb":1,en:1,eo:1,es:1,et:1,eu:1,fa:1,fi:1,fo:1,"fr-ca":1,fr:1,gl:1,gu:1,he:1,hi:1,hr:1,hu:1,id:1,is:1,it:1,ja:1,ka:1,km:1,ko:1,ku:1,lt:1,lv:1,mk:1,mn:1,ms:1,nb:1,nl:1,no:1,pl:1,"pt-br":1,pt:1,ro:1,ru:1,si:1,sk:1,sl:1,sq:1,"sr-latn":1,sr:1,sv:1,th:1,tr:1,tt:1,ug:1,uk:1,vi:1,"zh-cn":1,zh:1},rtl:{ar:1,fa:1,he:1,ku:1,ug:1},load:function(a,e,b){if(!a||!CKEDITOR.lang.languages[a])a=this.detect(e,a);var g=this,e=function(){g[a].dir=
g.rtl[a]?"rtl":"ltr";b(a,g[a])};this[a]?e():CKEDITOR.scriptLoader.load(CKEDITOR.getUrl("lang/"+a+".js"),e,this)},detect:function(a,e){var b=this.languages,e=e||navigator.userLanguage||navigator.language||a,g=e.toLowerCase().match(/([a-z]+)(?:-([a-z]+))?/),c=g[1],g=g[2];b[c+"-"+g]?c=c+"-"+g:b[c]||(c=null);CKEDITOR.lang.detect=c?function(){return c}:function(a){return a};return c||a}}}(),CKEDITOR.scriptLoader=function(){var a={},e={};return{load:function(b,g,c,d){var j=typeof b=="string";j&&(b=[b]);
c||(c=CKEDITOR);var f=b.length,h=[],i=[],k=function(a){g&&(j?g.call(c,a):g.call(c,h,i))};if(f===0)k(true);else{var n=function(a,b){(b?h:i).push(a);if(--f<=0){d&&CKEDITOR.document.getDocumentElement().removeStyle("cursor");k(b)}},o=function(b,c){a[b]=1;var d=e[b];delete e[b];for(var f=0;f<d.length;f++)d[f](b,c)},q=function(b){if(a[b])n(b,true);else{var c=e[b]||(e[b]=[]);c.push(n);if(!(c.length>1)){var d=new CKEDITOR.dom.element("script");d.setAttributes({type:"text/javascript",src:b});if(g)if(CKEDITOR.env.ie&&
CKEDITOR.env.version<11)d.$.onreadystatechange=function(){if(d.$.readyState=="loaded"||d.$.readyState=="complete"){d.$.onreadystatechange=null;o(b,true)}};else{d.$.onload=function(){setTimeout(function(){o(b,true)},0)};d.$.onerror=function(){o(b,false)}}d.appendTo(CKEDITOR.document.getHead())}}};d&&CKEDITOR.document.getDocumentElement().setStyle("cursor","wait");for(var m=0;m<f;m++)q(b[m])}},queue:function(){function a(){var b;(b=g[0])&&this.load(b.scriptUrl,b.callback,CKEDITOR,0)}var g=[];return function(c,
d){var e=this;g.push({scriptUrl:c,callback:function(){d&&d.apply(this,arguments);g.shift();a.call(e)}});g.length==1&&a.call(this)}}()}}(),CKEDITOR.resourceManager=function(a,e){this.basePath=a;this.fileName=e;this.registered={};this.loaded={};this.externals={};this._={waitingList:{}}},CKEDITOR.resourceManager.prototype={add:function(a,e){if(this.registered[a])throw'[CKEDITOR.resourceManager.add] The resource name "'+a+'" is already registered.';var b=this.registered[a]=e||{};b.name=a;b.path=this.getPath(a);
CKEDITOR.fire(a+CKEDITOR.tools.capitalize(this.fileName)+"Ready",b);return this.get(a)},get:function(a){return this.registered[a]||null},getPath:function(a){var e=this.externals[a];return CKEDITOR.getUrl(e&&e.dir||this.basePath+a+"/")},getFilePath:function(a){var e=this.externals[a];return CKEDITOR.getUrl(this.getPath(a)+(e?e.file:this.fileName+".js"))},addExternal:function(a,e,b){for(var a=a.split(","),g=0;g<a.length;g++){var c=a[g];b||(e=e.replace(/[^\/]+$/,function(a){b=a;return""}));this.externals[c]=
{dir:e,file:b||this.fileName+".js"}}},load:function(a,e,b){CKEDITOR.tools.isArray(a)||(a=a?[a]:[]);for(var g=this.loaded,c=this.registered,d=[],j={},f={},h=0;h<a.length;h++){var i=a[h];if(i)if(!g[i]&&!c[i]){var k=this.getFilePath(i);d.push(k);k in j||(j[k]=[]);j[k].push(i)}else f[i]=this.get(i)}CKEDITOR.scriptLoader.load(d,function(a,c){if(c.length)throw'[CKEDITOR.resourceManager.load] Resource name "'+j[c[0]].join(",")+'" was not found at "'+c[0]+'".';for(var d=0;d<a.length;d++)for(var h=j[a[d]],
i=0;i<h.length;i++){var p=h[i];f[p]=this.get(p);g[p]=1}e.call(b,f)},this)}},CKEDITOR.plugins=new CKEDITOR.resourceManager("plugins/","plugin"),CKEDITOR.plugins.load=CKEDITOR.tools.override(CKEDITOR.plugins.load,function(a){var e={};return function(b,g,c){var d={},j=function(b){a.call(this,b,function(a){CKEDITOR.tools.extend(d,a);var b=[],f;for(f in a){var n=a[f],o=n&&n.requires;if(!e[f]){if(n.icons)for(var q=n.icons.split(","),m=q.length;m--;)CKEDITOR.skin.addIcon(q[m],n.path+"icons/"+(CKEDITOR.env.hidpi&&
n.hidpi?"hidpi/":"")+q[m]+".png");e[f]=1}if(o){o.split&&(o=o.split(","));for(n=0;n<o.length;n++)d[o[n]]||b.push(o[n])}}if(b.length)j.call(this,b);else{for(f in d){n=d[f];if(n.onLoad&&!n.onLoad._called){n.onLoad()===false&&delete d[f];n.onLoad._called=1}}g&&g.call(c||window,d)}},this)};j.call(this,b)}}),CKEDITOR.plugins.setLang=function(a,e,b){var g=this.get(a),a=g.langEntries||(g.langEntries={}),g=g.lang||(g.lang=[]);g.split&&(g=g.split(","));CKEDITOR.tools.indexOf(g,e)==-1&&g.push(e);a[e]=b},CKEDITOR.ui=
function(a){if(a.ui)return a.ui;this.items={};this.instances={};this.editor=a;this._={handlers:{}};return this},CKEDITOR.ui.prototype={add:function(a,e,b){b.name=a.toLowerCase();var g=this.items[a]={type:e,command:b.command||null,args:Array.prototype.slice.call(arguments,2)};CKEDITOR.tools.extend(g,b)},get:function(a){return this.instances[a]},create:function(a){var e=this.items[a],b=e&&this._.handlers[e.type],g=e&&e.command&&this.editor.getCommand(e.command),b=b&&b.create.apply(this,e.args);this.instances[a]=
b;g&&g.uiItems.push(b);if(b&&!b.type)b.type=e.type;return b},addHandler:function(a,e){this._.handlers[a]=e},space:function(a){return CKEDITOR.document.getById(this.spaceId(a))},spaceId:function(a){return this.editor.id+"_"+a}},CKEDITOR.event.implementOn(CKEDITOR.ui),function(){function a(a,d,f){CKEDITOR.event.call(this);a=a&&CKEDITOR.tools.clone(a);if(d!==void 0){if(d instanceof CKEDITOR.dom.element){if(!f)throw Error("One of the element modes must be specified.");}else throw Error("Expect element of type CKEDITOR.dom.element.");
if(CKEDITOR.env.ie&&CKEDITOR.env.quirks&&f==CKEDITOR.ELEMENT_MODE_INLINE)throw Error("Inline element mode is not supported on IE quirks.");if(!(f==CKEDITOR.ELEMENT_MODE_INLINE?d.is(CKEDITOR.dtd.$editable)||d.is("textarea"):f==CKEDITOR.ELEMENT_MODE_REPLACE?!d.is(CKEDITOR.dtd.$nonBodyContent):1))throw Error('The specified element mode is not supported on element: "'+d.getName()+'".');this.element=d;this.elementMode=f;this.name=this.elementMode!=CKEDITOR.ELEMENT_MODE_APPENDTO&&(d.getId()||d.getNameAtt())}else this.elementMode=
CKEDITOR.ELEMENT_MODE_NONE;this._={};this.commands={};this.templates={};this.name=this.name||e();this.id=CKEDITOR.tools.getNextId();this.status="unloaded";this.config=CKEDITOR.tools.prototypedCopy(CKEDITOR.config);this.ui=new CKEDITOR.ui(this);this.focusManager=new CKEDITOR.focusManager(this);this.keystrokeHandler=new CKEDITOR.keystrokeHandler(this);this.on("readOnly",b);this.on("selectionChange",function(a){c(this,a.data.path)});this.on("activeFilterChange",function(){c(this,this.elementPath(),true)});
this.on("mode",b);this.on("instanceReady",function(){this.config.startupFocus&&this.focus()});CKEDITOR.fire("instanceCreated",null,this);CKEDITOR.add(this);CKEDITOR.tools.setTimeout(function(){j(this,a)},0,this)}function e(){do var a="editor"+ ++o;while(CKEDITOR.instances[a]);return a}function b(){var a=this.commands,b;for(b in a)g(this,a[b])}function g(a,b){b[b.startDisabled?"disable":a.readOnly&&!b.readOnly?"disable":b.modes[a.mode]?"enable":"disable"]()}function c(a,b,c){if(b){var d,f,g=a.commands;
for(f in g){d=g[f];(c||d.contextSensitive)&&d.refresh(a,b)}}}function d(a){var b=a.config.customConfig;if(!b)return false;var b=CKEDITOR.getUrl(b),c=q[b]||(q[b]={});if(c.fn){c.fn.call(a,a.config);(CKEDITOR.getUrl(a.config.customConfig)==b||!d(a))&&a.fireOnce("customConfigLoaded")}else CKEDITOR.scriptLoader.queue(b,function(){c.fn=CKEDITOR.editorConfig?CKEDITOR.editorConfig:function(){};d(a)});return true}function j(a,b){a.on("customConfigLoaded",function(){if(b){if(b.on)for(var c in b.on)a.on(c,b.on[c]);
CKEDITOR.tools.extend(a.config,b,true);delete a.config.on}c=a.config;a.readOnly=!(!c.readOnly&&!(a.elementMode==CKEDITOR.ELEMENT_MODE_INLINE?a.element.is("textarea")?a.element.hasAttribute("disabled"):a.element.isReadOnly():a.elementMode==CKEDITOR.ELEMENT_MODE_REPLACE&&a.element.hasAttribute("disabled")));a.blockless=a.elementMode==CKEDITOR.ELEMENT_MODE_INLINE?!(a.element.is("textarea")||CKEDITOR.dtd[a.element.getName()].p):false;a.tabIndex=c.tabIndex||a.element&&a.element.getAttribute("tabindex")||
0;a.activeEnterMode=a.enterMode=a.blockless?CKEDITOR.ENTER_BR:c.enterMode;a.activeShiftEnterMode=a.shiftEnterMode=a.blockless?CKEDITOR.ENTER_BR:c.shiftEnterMode;if(c.skin)CKEDITOR.skinName=c.skin;a.fireOnce("configLoaded");a.dataProcessor=new CKEDITOR.htmlDataProcessor(a);a.filter=a.activeFilter=new CKEDITOR.filter(a);f(a)});if(b&&b.customConfig!=null)a.config.customConfig=b.customConfig;d(a)||a.fireOnce("customConfigLoaded")}function f(a){CKEDITOR.skin.loadPart("editor",function(){h(a)})}function h(a){CKEDITOR.lang.load(a.config.language,
a.config.defaultLanguage,function(b,c){var d=a.config.title;a.langCode=b;a.lang=CKEDITOR.tools.prototypedCopy(c);a.title=typeof d=="string"||d===false?d:[a.lang.editor,a.name].join(", ");if(!a.config.contentsLangDirection)a.config.contentsLangDirection=a.elementMode==CKEDITOR.ELEMENT_MODE_INLINE?a.element.getDirection(1):a.lang.dir;a.fire("langLoaded");i(a)})}function i(a){a.getStylesSet(function(b){a.once("loaded",function(){a.fire("stylesSet",{styles:b})},null,null,1);k(a)})}function k(a){var b=
a.config,c=b.plugins,d=b.extraPlugins,f=b.removePlugins;if(d)var g=RegExp("(?:^|,)(?:"+d.replace(/\s*,\s*/g,"|")+")(?=,|$)","g"),c=c.replace(g,""),c=c+(","+d);if(f)var h=RegExp("(?:^|,)(?:"+f.replace(/\s*,\s*/g,"|")+")(?=,|$)","g"),c=c.replace(h,"");CKEDITOR.env.air&&(c=c+",adobeair");CKEDITOR.plugins.load(c.split(","),function(c){var d=[],f=[],g=[];a.plugins=c;for(var e in c){var j=c[e],i=j.lang,p=null,t=j.requires,k;CKEDITOR.tools.isArray(t)&&(t=t.join(","));if(t&&(k=t.match(h)))for(;t=k.pop();)CKEDITOR.tools.setTimeout(function(a,
b){throw Error('Plugin "'+a.replace(",","")+'" cannot be removed from the plugins list, because it\'s required by "'+b+'" plugin.');},0,null,[t,e]);if(i&&!a.lang[e]){i.split&&(i=i.split(","));if(CKEDITOR.tools.indexOf(i,a.langCode)>=0)p=a.langCode;else{p=a.langCode.replace(/-.*/,"");p=p!=a.langCode&&CKEDITOR.tools.indexOf(i,p)>=0?p:CKEDITOR.tools.indexOf(i,"en")>=0?"en":i[0]}if(!j.langEntries||!j.langEntries[p])g.push(CKEDITOR.getUrl(j.path+"lang/"+p+".js"));else{a.lang[e]=j.langEntries[p];p=null}}f.push(p);
d.push(j)}CKEDITOR.scriptLoader.load(g,function(){for(var c=["beforeInit","init","afterInit"],g=0;g<c.length;g++)for(var h=0;h<d.length;h++){var e=d[h];g===0&&(f[h]&&e.lang&&e.langEntries)&&(a.lang[e.name]=e.langEntries[f[h]]);if(e[c[g]])e[c[g]](a)}a.fireOnce("pluginsLoaded");b.keystrokes&&a.setKeystroke(a.config.keystrokes);for(h=0;h<a.config.blockedKeystrokes.length;h++)a.keystrokeHandler.blockedKeystrokes[a.config.blockedKeystrokes[h]]=1;a.status="loaded";a.fireOnce("loaded");CKEDITOR.fire("instanceLoaded",
null,a)})})}function n(){var a=this.element;if(a&&this.elementMode!=CKEDITOR.ELEMENT_MODE_APPENDTO){var b=this.getData();this.config.htmlEncodeOutput&&(b=CKEDITOR.tools.htmlEncode(b));a.is("textarea")?a.setValue(b):a.setHtml(b);return true}return false}a.prototype=CKEDITOR.editor.prototype;CKEDITOR.editor=a;var o=0,q={};CKEDITOR.tools.extend(CKEDITOR.editor.prototype,{addCommand:function(a,b){b.name=a.toLowerCase();var c=new CKEDITOR.command(this,b);this.mode&&g(this,c);return this.commands[a]=c},
_attachToForm:function(){function a(d){b.updateElement();b._.required&&(!c.getValue()&&b.fire("required")===false)&&d.data.preventDefault()}var b=this,c=b.element,d=new CKEDITOR.dom.element(c.$.form);if(c.is("textarea")&&d){d.on("submit",a);if(d.$.submit&&d.$.submit.call&&d.$.submit.apply)d.$.submit=CKEDITOR.tools.override(d.$.submit,function(b){return function(){a();b.apply?b.apply(this):b()}});b.on("destroy",function(){d.removeListener("submit",a)})}},destroy:function(a){this.fire("beforeDestroy");
!a&&n.call(this);this.editable(null);this.filter.destroy();delete this.filter;delete this.activeFilter;this.status="destroyed";this.fire("destroy");this.removeAllListeners();CKEDITOR.remove(this);CKEDITOR.fire("instanceDestroyed",null,this)},elementPath:function(a){if(!a){a=this.getSelection();if(!a)return null;a=a.getStartElement()}return a?new CKEDITOR.dom.elementPath(a,this.editable()):null},createRange:function(){var a=this.editable();return a?new CKEDITOR.dom.range(a):null},execCommand:function(a,
b){var c=this.getCommand(a),d={name:a,commandData:b,command:c};if(c&&c.state!=CKEDITOR.TRISTATE_DISABLED&&this.fire("beforeCommandExec",d)!==false){d.returnValue=c.exec(d.commandData);if(!c.async&&this.fire("afterCommandExec",d)!==false)return d.returnValue}return false},getCommand:function(a){return this.commands[a]},getData:function(a){!a&&this.fire("beforeGetData");var b=this._.data;if(typeof b!="string")b=(b=this.element)&&this.elementMode==CKEDITOR.ELEMENT_MODE_REPLACE?b.is("textarea")?b.getValue():
b.getHtml():"";b={dataValue:b};!a&&this.fire("getData",b);return b.dataValue},getSnapshot:function(){var a=this.fire("getSnapshot");if(typeof a!="string"){var b=this.element;b&&this.elementMode==CKEDITOR.ELEMENT_MODE_REPLACE&&(a=b.is("textarea")?b.getValue():b.getHtml())}return a},loadSnapshot:function(a){this.fire("loadSnapshot",a)},setData:function(a,b,c){var d=true,f=b;if(b&&typeof b=="object"){c=b.internal;f=b.callback;d=!b.noSnapshot}!c&&d&&this.fire("saveSnapshot");if(f||!c)this.once("dataReady",
function(a){!c&&d&&this.fire("saveSnapshot");f&&f.call(a.editor)});a={dataValue:a};!c&&this.fire("setData",a);this._.data=a.dataValue;!c&&this.fire("afterSetData",a)},setReadOnly:function(a){a=a==null||a;if(this.readOnly!=a){this.readOnly=a;this.keystrokeHandler.blockedKeystrokes[8]=+a;this.editable().setReadOnly(a);this.fire("readOnly")}},insertHtml:function(a,b){this.fire("insertHtml",{dataValue:a,mode:b})},insertText:function(a){this.fire("insertText",a)},insertElement:function(a){this.fire("insertElement",
a)},focus:function(){this.fire("beforeFocus")},checkDirty:function(){return this.status=="ready"&&this._.previousValue!==this.getSnapshot()},resetDirty:function(){this._.previousValue=this.getSnapshot()},updateElement:function(){return n.call(this)},setKeystroke:function(){for(var a=this.keystrokeHandler.keystrokes,b=CKEDITOR.tools.isArray(arguments[0])?arguments[0]:[[].slice.call(arguments,0)],c,d,f=b.length;f--;){c=b[f];d=0;if(CKEDITOR.tools.isArray(c)){d=c[1];c=c[0]}d?a[c]=d:delete a[c]}},addFeature:function(a){return this.filter.addFeature(a)},
setActiveFilter:function(a){if(!a)a=this.filter;if(this.activeFilter!==a){this.activeFilter=a;this.fire("activeFilterChange");a===this.filter?this.setActiveEnterMode(null,null):this.setActiveEnterMode(a.getAllowedEnterMode(this.enterMode),a.getAllowedEnterMode(this.shiftEnterMode,true))}},setActiveEnterMode:function(a,b){a=a?this.blockless?CKEDITOR.ENTER_BR:a:this.enterMode;b=b?this.blockless?CKEDITOR.ENTER_BR:b:this.shiftEnterMode;if(this.activeEnterMode!=a||this.activeShiftEnterMode!=b){this.activeEnterMode=
a;this.activeShiftEnterMode=b;this.fire("activeEnterModeChange")}}})}(),CKEDITOR.ELEMENT_MODE_NONE=0,CKEDITOR.ELEMENT_MODE_REPLACE=1,CKEDITOR.ELEMENT_MODE_APPENDTO=2,CKEDITOR.ELEMENT_MODE_INLINE=3,CKEDITOR.htmlParser=function(){this._={htmlPartsRegex:/<(?:(?:\/([^>]+)>)|(?:!--([\S|\s]*?)--\>)|(?:([^\/\s>]+)((?:\s+[\w\-:.]+(?:\s*=\s*?(?:(?:"[^"]*")|(?:'[^']*')|[^\s"'\/>]+))?)*)[\S\s]*?(\/?)>))/g}},function(){var a=/([\w\-:.]+)(?:(?:\s*=\s*(?:(?:"([^"]*)")|(?:'([^']*)')|([^\s>]+)))|(?=\s|$))/g,e={checked:1,
compact:1,declare:1,defer:1,disabled:1,ismap:1,multiple:1,nohref:1,noresize:1,noshade:1,nowrap:1,readonly:1,selected:1};CKEDITOR.htmlParser.prototype={onTagOpen:function(){},onTagClose:function(){},onText:function(){},onCDATA:function(){},onComment:function(){},parse:function(b){for(var g,c,d=0,j;g=this._.htmlPartsRegex.exec(b);){c=g.index;if(c>d){d=b.substring(d,c);if(j)j.push(d);else this.onText(d)}d=this._.htmlPartsRegex.lastIndex;if(c=g[1]){c=c.toLowerCase();if(j&&CKEDITOR.dtd.$cdata[c]){this.onCDATA(j.join(""));
j=null}if(!j){this.onTagClose(c);continue}}if(j)j.push(g[0]);else if(c=g[3]){c=c.toLowerCase();if(!/="/.test(c)){var f={},h,i=g[4];g=!!g[5];if(i)for(;h=a.exec(i);){var k=h[1].toLowerCase();h=h[2]||h[3]||h[4]||"";f[k]=!h&&e[k]?k:CKEDITOR.tools.htmlDecodeAttr(h)}this.onTagOpen(c,f,g);!j&&CKEDITOR.dtd.$cdata[c]&&(j=[])}}else if(c=g[2])this.onComment(c)}if(b.length>d)this.onText(b.substring(d,b.length))}}}(),CKEDITOR.htmlParser.basicWriter=CKEDITOR.tools.createClass({$:function(){this._={output:[]}},
proto:{openTag:function(a){this._.output.push("<",a)},openTagClose:function(a,e){e?this._.output.push(" />"):this._.output.push(">")},attribute:function(a,e){typeof e=="string"&&(e=CKEDITOR.tools.htmlEncodeAttr(e));this._.output.push(" ",a,'="',e,'"')},closeTag:function(a){this._.output.push("</",a,">")},text:function(a){this._.output.push(a)},comment:function(a){this._.output.push("<\!--",a,"--\>")},write:function(a){this._.output.push(a)},reset:function(){this._.output=[];this._.indent=false},getHtml:function(a){var e=
this._.output.join("");a&&this.reset();return e}}}),"use strict",function(){CKEDITOR.htmlParser.node=function(){};CKEDITOR.htmlParser.node.prototype={remove:function(){var a=this.parent.children,e=CKEDITOR.tools.indexOf(a,this),b=this.previous,g=this.next;b&&(b.next=g);g&&(g.previous=b);a.splice(e,1);this.parent=null},replaceWith:function(a){var e=this.parent.children,b=CKEDITOR.tools.indexOf(e,this),g=a.previous=this.previous,c=a.next=this.next;g&&(g.next=a);c&&(c.previous=a);e[b]=a;a.parent=this.parent;
this.parent=null},insertAfter:function(a){var e=a.parent.children,b=CKEDITOR.tools.indexOf(e,a),g=a.next;e.splice(b+1,0,this);this.next=a.next;this.previous=a;a.next=this;g&&(g.previous=this);this.parent=a.parent},insertBefore:function(a){var e=a.parent.children,b=CKEDITOR.tools.indexOf(e,a);e.splice(b,0,this);this.next=a;(this.previous=a.previous)&&(a.previous.next=this);a.previous=this;this.parent=a.parent},getAscendant:function(a){var e=typeof a=="function"?a:typeof a=="string"?function(b){return b.name==
a}:function(b){return b.name in a},b=this.parent;for(;b&&b.type==CKEDITOR.NODE_ELEMENT;){if(e(b))return b;b=b.parent}return null},wrapWith:function(a){this.replaceWith(a);a.add(this);return a},getIndex:function(){return CKEDITOR.tools.indexOf(this.parent.children,this)},getFilterContext:function(a){return a||{}}}}(),"use strict",CKEDITOR.htmlParser.comment=function(a){this.value=a;this._={isBlockLike:false}},CKEDITOR.htmlParser.comment.prototype=CKEDITOR.tools.extend(new CKEDITOR.htmlParser.node,
{type:CKEDITOR.NODE_COMMENT,filter:function(a,e){var b=this.value;if(!(b=a.onComment(e,b,this))){this.remove();return false}if(typeof b!="string"){this.replaceWith(b);return false}this.value=b;return true},writeHtml:function(a,e){e&&this.filter(e);a.comment(this.value)}}),"use strict",function(){CKEDITOR.htmlParser.text=function(a){this.value=a;this._={isBlockLike:false}};CKEDITOR.htmlParser.text.prototype=CKEDITOR.tools.extend(new CKEDITOR.htmlParser.node,{type:CKEDITOR.NODE_TEXT,filter:function(a,
e){if(!(this.value=a.onText(e,this.value,this))){this.remove();return false}},writeHtml:function(a,e){e&&this.filter(e);a.text(this.value)}})}(),"use strict",function(){CKEDITOR.htmlParser.cdata=function(a){this.value=a};CKEDITOR.htmlParser.cdata.prototype=CKEDITOR.tools.extend(new CKEDITOR.htmlParser.node,{type:CKEDITOR.NODE_TEXT,filter:function(){},writeHtml:function(a){a.write(this.value)}})}(),"use strict",CKEDITOR.htmlParser.fragment=function(){this.children=[];this.parent=null;this._={isBlockLike:true,
hasInlineStarted:false}},function(){function a(a){return a.attributes["data-cke-survive"]?false:a.name=="a"&&a.attributes.href||CKEDITOR.dtd.$removeEmpty[a.name]}var e=CKEDITOR.tools.extend({table:1,ul:1,ol:1,dl:1},CKEDITOR.dtd.table,CKEDITOR.dtd.ul,CKEDITOR.dtd.ol,CKEDITOR.dtd.dl),b={ol:1,ul:1},g=CKEDITOR.tools.extend({},{html:1},CKEDITOR.dtd.html,CKEDITOR.dtd.body,CKEDITOR.dtd.head,{style:1,script:1}),c={ul:"li",ol:"li",dl:"dd",table:"tbody",tbody:"tr",thead:"tr",tfoot:"tr",tr:"td"};CKEDITOR.htmlParser.fragment.fromHtml=
function(d,j,f){function h(a){var b;if(p.length>0)for(var c=0;c<p.length;c++){var d=p[c],f=d.name,g=CKEDITOR.dtd[f],h=s.name&&CKEDITOR.dtd[s.name];if((!h||h[f])&&(!a||!g||g[a]||!CKEDITOR.dtd[a])){if(!b){i();b=1}d=d.clone();d.parent=s;s=d;p.splice(c,1);c--}else if(f==s.name){n(s,s.parent,1);c--}}}function i(){for(;t.length;)n(t.shift(),s)}function k(a){if(a._.isBlockLike&&a.name!="pre"&&a.name!="textarea"){var b=a.children.length,c=a.children[b-1],d;if(c&&c.type==CKEDITOR.NODE_TEXT)(d=CKEDITOR.tools.rtrim(c.value))?
c.value=d:a.children.length=b-1}}function n(b,c,d){var c=c||s||l,g=s;if(b.previous===void 0){if(o(c,b)){s=c;m.onTagOpen(f,{});b.returnPoint=c=s}k(b);(!a(b)||b.children.length)&&c.add(b);b.name=="pre"&&(w=false);b.name=="textarea"&&(u=false)}if(b.returnPoint){s=b.returnPoint;delete b.returnPoint}else s=d?c:g}function o(a,b){if((a==l||a.name=="body")&&f&&(!a.name||CKEDITOR.dtd[a.name][f])){var c,d;return(c=b.attributes&&(d=b.attributes["data-cke-real-element-type"])?d:b.name)&&c in CKEDITOR.dtd.$inline&&
!(c in CKEDITOR.dtd.head)&&!b.isOrphan||b.type==CKEDITOR.NODE_TEXT}}function q(a,b){return a in CKEDITOR.dtd.$listItem||a in CKEDITOR.dtd.$tableContent?a==b||a=="dt"&&b=="dd"||a=="dd"&&b=="dt":false}var m=new CKEDITOR.htmlParser,l=j instanceof CKEDITOR.htmlParser.element?j:typeof j=="string"?new CKEDITOR.htmlParser.element(j):new CKEDITOR.htmlParser.fragment,p=[],t=[],s=l,u=l.name=="textarea",w=l.name=="pre";m.onTagOpen=function(c,d,f,j){d=new CKEDITOR.htmlParser.element(c,d);if(d.isUnknown&&f)d.isEmpty=
true;d.isOptionalClose=j;if(a(d))p.push(d);else{if(c=="pre")w=true;else{if(c=="br"&&w){s.add(new CKEDITOR.htmlParser.text("\n"));return}c=="textarea"&&(u=true)}if(c=="br")t.push(d);else{for(;;){j=(f=s.name)?CKEDITOR.dtd[f]||(s._.isBlockLike?CKEDITOR.dtd.div:CKEDITOR.dtd.span):g;if(!d.isUnknown&&!s.isUnknown&&!j[c])if(s.isOptionalClose)m.onTagClose(f);else if(c in b&&f in b){f=s.children;(f=f[f.length-1])&&f.name=="li"||n(f=new CKEDITOR.htmlParser.element("li"),s);!d.returnPoint&&(d.returnPoint=s);
s=f}else if(c in CKEDITOR.dtd.$listItem&&!q(c,f))m.onTagOpen(c=="li"?"ul":"dl",{},0,1);else if(f in e&&!q(c,f)){!d.returnPoint&&(d.returnPoint=s);s=s.parent}else{f in CKEDITOR.dtd.$inline&&p.unshift(s);if(s.parent)n(s,s.parent,1);else{d.isOrphan=1;break}}else break}h(c);i();d.parent=s;d.isEmpty?n(d):s=d}}};m.onTagClose=function(a){for(var b=p.length-1;b>=0;b--)if(a==p[b].name){p.splice(b,1);return}for(var c=[],d=[],g=s;g!=l&&g.name!=a;){g._.isBlockLike||d.unshift(g);c.push(g);g=g.returnPoint||g.parent}if(g!=
l){for(b=0;b<c.length;b++){var h=c[b];n(h,h.parent)}s=g;g._.isBlockLike&&i();n(g,g.parent);if(g==s)s=s.parent;p=p.concat(d)}a=="body"&&(f=false)};m.onText=function(a){if((!s._.hasInlineStarted||t.length)&&!w&&!u){a=CKEDITOR.tools.ltrim(a);if(a.length===0)return}var b=s.name,d=b?CKEDITOR.dtd[b]||(s._.isBlockLike?CKEDITOR.dtd.div:CKEDITOR.dtd.span):g;if(!u&&!d["#"]&&b in e){m.onTagOpen(c[b]||"");m.onText(a)}else{i();h();!w&&!u&&(a=a.replace(/[\t\r\n ]{2,}|[\t\r\n]/g," "));a=new CKEDITOR.htmlParser.text(a);
if(o(s,a))this.onTagOpen(f,{},0,1);s.add(a)}};m.onCDATA=function(a){s.add(new CKEDITOR.htmlParser.cdata(a))};m.onComment=function(a){i();h();s.add(new CKEDITOR.htmlParser.comment(a))};m.parse(d);for(i();s!=l;)n(s,s.parent,1);k(l);return l};CKEDITOR.htmlParser.fragment.prototype={type:CKEDITOR.NODE_DOCUMENT_FRAGMENT,add:function(a,b){isNaN(b)&&(b=this.children.length);var c=b>0?this.children[b-1]:null;if(c){if(a._.isBlockLike&&c.type==CKEDITOR.NODE_TEXT){c.value=CKEDITOR.tools.rtrim(c.value);if(c.value.length===
0){this.children.pop();this.add(a);return}}c.next=a}a.previous=c;a.parent=this;this.children.splice(b,0,a);if(!this._.hasInlineStarted)this._.hasInlineStarted=a.type==CKEDITOR.NODE_TEXT||a.type==CKEDITOR.NODE_ELEMENT&&!a._.isBlockLike},filter:function(a,b){b=this.getFilterContext(b);a.onRoot(b,this);this.filterChildren(a,false,b)},filterChildren:function(a,b,c){if(this.childrenFilteredBy!=a.id){c=this.getFilterContext(c);if(b&&!this.parent)a.onRoot(c,this);this.childrenFilteredBy=a.id;for(b=0;b<this.children.length;b++)this.children[b].filter(a,
c)===false&&b--}},writeHtml:function(a,b){b&&this.filter(b);this.writeChildrenHtml(a)},writeChildrenHtml:function(a,b,c){var g=this.getFilterContext();if(c&&!this.parent&&b)b.onRoot(g,this);b&&this.filterChildren(b,false,g);b=0;c=this.children;for(g=c.length;b<g;b++)c[b].writeHtml(a)},forEach:function(a,b,c){if(!c&&(!b||this.type==b))var g=a(this);if(g!==false)for(var c=this.children,e=0;e<c.length;e++){g=c[e];g.type==CKEDITOR.NODE_ELEMENT?g.forEach(a,b):(!b||g.type==b)&&a(g)}},getFilterContext:function(a){return a||
{}}}}(),"use strict",function(){function a(){this.rules=[]}function e(b,g,c,d){var e,f;for(e in g){(f=b[e])||(f=b[e]=new a);f.add(g[e],c,d)}}CKEDITOR.htmlParser.filter=CKEDITOR.tools.createClass({$:function(b){this.id=CKEDITOR.tools.getNextNumber();this.elementNameRules=new a;this.attributeNameRules=new a;this.elementsRules={};this.attributesRules={};this.textRules=new a;this.commentRules=new a;this.rootRules=new a;b&&this.addRules(b,10)},proto:{addRules:function(a,g){var c;if(typeof g=="number")c=
g;else if(g&&"priority"in g)c=g.priority;typeof c!="number"&&(c=10);typeof g!="object"&&(g={});a.elementNames&&this.elementNameRules.addMany(a.elementNames,c,g);a.attributeNames&&this.attributeNameRules.addMany(a.attributeNames,c,g);a.elements&&e(this.elementsRules,a.elements,c,g);a.attributes&&e(this.attributesRules,a.attributes,c,g);a.text&&this.textRules.add(a.text,c,g);a.comment&&this.commentRules.add(a.comment,c,g);a.root&&this.rootRules.add(a.root,c,g)},applyTo:function(a){a.filter(this)},onElementName:function(a,
g){return this.elementNameRules.execOnName(a,g)},onAttributeName:function(a,g){return this.attributeNameRules.execOnName(a,g)},onText:function(a,g,c){return this.textRules.exec(a,g,c)},onComment:function(a,g,c){return this.commentRules.exec(a,g,c)},onRoot:function(a,g){return this.rootRules.exec(a,g)},onElement:function(a,g){for(var c=[this.elementsRules["^"],this.elementsRules[g.name],this.elementsRules.$],d,e=0;e<3;e++)if(d=c[e]){d=d.exec(a,g,this);if(d===false)return null;if(d&&d!=g)return this.onNode(a,
d);if(g.parent&&!g.name)break}return g},onNode:function(a,g){var c=g.type;return c==CKEDITOR.NODE_ELEMENT?this.onElement(a,g):c==CKEDITOR.NODE_TEXT?new CKEDITOR.htmlParser.text(this.onText(a,g.value)):c==CKEDITOR.NODE_COMMENT?new CKEDITOR.htmlParser.comment(this.onComment(a,g.value)):null},onAttribute:function(a,g,c,d){return(c=this.attributesRules[c])?c.exec(a,d,g,this):d}}});CKEDITOR.htmlParser.filterRulesGroup=a;a.prototype={add:function(a,g,c){this.rules.splice(this.findIndex(g),0,{value:a,priority:g,
options:c})},addMany:function(a,g,c){for(var d=[this.findIndex(g),0],e=0,f=a.length;e<f;e++)d.push({value:a[e],priority:g,options:c});this.rules.splice.apply(this.rules,d)},findIndex:function(a){for(var g=this.rules,c=g.length-1;c>=0&&a<g[c].priority;)c--;return c+1},exec:function(a,g){var c=g instanceof CKEDITOR.htmlParser.node||g instanceof CKEDITOR.htmlParser.fragment,d=Array.prototype.slice.call(arguments,1),e=this.rules,f=e.length,h,i,k,n;for(n=0;n<f;n++){if(c){h=g.type;i=g.name}k=e[n];if(!(a.nonEditable&&
!k.options.applyToAll||a.nestedEditable&&k.options.excludeNestedEditable)){k=k.value.apply(null,d);if(k===false||c&&k&&(k.name!=i||k.type!=h))return k;k!=null&&(d[0]=g=k)}}return g},execOnName:function(a,g){for(var c=0,d=this.rules,e=d.length,f;g&&c<e;c++){f=d[c];!(a.nonEditable&&!f.options.applyToAll||a.nestedEditable&&f.options.excludeNestedEditable)&&(g=g.replace(f.value[0],f.value[1]))}return g}}}(),function(){function a(a,f){function h(a){return a||CKEDITOR.env.needsNbspFiller?new CKEDITOR.htmlParser.text(" "):
new CKEDITOR.htmlParser.element("br",{"data-cke-bogus":1})}function e(a,c){return function(f){if(f.type!=CKEDITOR.NODE_DOCUMENT_FRAGMENT){var e=[],j=b(f),p,k;if(j)for(i(j,1)&&e.push(j);j;){if(d(j)&&(p=g(j))&&i(p))if((k=g(p))&&!d(k))e.push(p);else{h(t).insertAfter(p);p.remove()}j=j.previous}for(j=0;j<e.length;j++)e[j].remove();if(e=!a||(typeof c=="function"?c(f):c)!==false)if(!t&&!CKEDITOR.env.needsBrFiller&&f.type==CKEDITOR.NODE_DOCUMENT_FRAGMENT)e=false;else if(!t&&!CKEDITOR.env.needsBrFiller&&(document.documentMode>
7||f.name in CKEDITOR.dtd.tr||f.name in CKEDITOR.dtd.$listItem))e=false;else{e=b(f);e=!e||f.name=="form"&&e.name=="input"}e&&f.add(h(a))}}}function i(a,b){if((!t||CKEDITOR.env.needsBrFiller)&&a.type==CKEDITOR.NODE_ELEMENT&&a.name=="br"&&!a.attributes["data-cke-eol"])return true;var c;if(a.type==CKEDITOR.NODE_TEXT&&(c=a.value.match(p))){if(c.index){(new CKEDITOR.htmlParser.text(a.value.substring(0,c.index))).insertBefore(a);a.value=c[0]}if(!CKEDITOR.env.needsBrFiller&&t&&(!b||a.parent.name in x))return true;
if(!t)if((c=a.previous)&&c.name=="br"||!c||d(c))return true}return false}var k={elements:{}},t=f=="html",x=CKEDITOR.tools.extend({},w),C;for(C in x)"#"in s[C]||delete x[C];for(C in x)k.elements[C]=e(t,a.config.fillEmptyBlocks);k.root=e(t,false);k.elements.br=function(a){return function(b){if(b.parent.type!=CKEDITOR.NODE_DOCUMENT_FRAGMENT){var f=b.attributes;if("data-cke-bogus"in f||"data-cke-eol"in f)delete f["data-cke-bogus"];else{for(f=b.next;f&&c(f);)f=f.next;var e=g(b);!f&&d(b.parent)?j(b.parent,
h(a)):d(f)&&(e&&!d(e))&&h(a).insertBefore(f)}}}}(t);return k}function e(a,b){return a!=CKEDITOR.ENTER_BR&&b!==false?a==CKEDITOR.ENTER_DIV?"div":"p":false}function b(a){for(a=a.children[a.children.length-1];a&&c(a);)a=a.previous;return a}function g(a){for(a=a.previous;a&&c(a);)a=a.previous;return a}function c(a){return a.type==CKEDITOR.NODE_TEXT&&!CKEDITOR.tools.trim(a.value)||a.type==CKEDITOR.NODE_ELEMENT&&a.attributes["data-cke-bookmark"]}function d(a){return a&&(a.type==CKEDITOR.NODE_ELEMENT&&a.name in
w||a.type==CKEDITOR.NODE_DOCUMENT_FRAGMENT)}function j(a,b){var c=a.children[a.children.length-1];a.children.push(b);b.parent=a;if(c){c.next=b;b.previous=c}}function f(a){a=a.attributes;a.contenteditable!="false"&&(a["data-cke-editable"]=a.contenteditable?"true":1);a.contenteditable="false"}function h(a){a=a.attributes;switch(a["data-cke-editable"]){case "true":a.contenteditable="true";break;case "1":delete a.contenteditable}}function i(a){return a.replace(z,function(a,b,c){return"<"+b+c.replace(B,
function(a,b){return A.test(b)&&c.indexOf("data-cke-saved-"+b)==-1?" data-cke-saved-"+a+" data-cke-"+CKEDITOR.rnd+"-"+a:a})+">"})}function k(a,b){return a.replace(b,function(a,b,c){a.indexOf("<textarea")===0&&(a=b+q(c).replace(/</g,"&lt;").replace(/>/g,"&gt;")+"</textarea>");return"<cke:encoded>"+encodeURIComponent(a)+"</cke:encoded>"})}function n(a){return a.replace(J,function(a,b){return decodeURIComponent(b)})}function o(a){return a.replace(/<\!--(?!{cke_protected})[\s\S]+?--\>/g,function(a){return"<\!--"+
t+"{C}"+encodeURIComponent(a).replace(/--/g,"%2D%2D")+"--\>"})}function q(a){return a.replace(/<\!--\{cke_protected\}\{C\}([\s\S]+?)--\>/g,function(a,b){return decodeURIComponent(b)})}function m(a,b){var c=b._.dataStore;return a.replace(/<\!--\{cke_protected\}([\s\S]+?)--\>/g,function(a,b){return decodeURIComponent(b)}).replace(/\{cke_protected_(\d+)\}/g,function(a,b){return c&&c[b]||""})}function l(a,b){for(var c=[],d=b.config.protectedSource,f=b._.dataStore||(b._.dataStore={id:1}),g=/<\!--\{cke_temp(comment)?\}(\d*?)--\>/g,
d=[/<script[\s\S]*?<\/script>/gi,/<noscript[\s\S]*?<\/noscript>/gi,/<meta[\s\S]*?\/?>/gi].concat(d),a=a.replace(/<\!--[\s\S]*?--\>/g,function(a){return"<\!--{cke_tempcomment}"+(c.push(a)-1)+"--\>"}),h=0;h<d.length;h++)a=a.replace(d[h],function(a){a=a.replace(g,function(a,b,d){return c[d]});return/cke_temp(comment)?/.test(a)?a:"<\!--{cke_temp}"+(c.push(a)-1)+"--\>"});a=a.replace(g,function(a,b,d){return"<\!--"+t+(b?"{C}":"")+encodeURIComponent(c[d]).replace(/--/g,"%2D%2D")+"--\>"});a=a.replace(/<\w+(?:\s+(?:(?:[^\s=>]+\s*=\s*(?:[^'"\s>]+|'[^']*'|"[^"]*"))|[^\s=>]+))+\s*>/g,
function(a){return a.replace(/<\!--\{cke_protected\}([^>]*)--\>/g,function(a,b){f[f.id]=decodeURIComponent(b);return"{cke_protected_"+f.id++ +"}"})});return a=a.replace(/<(title|iframe|textarea)([^>]*)>([\s\S]*?)<\/\1>/g,function(a,c,d,f){return"<"+c+d+">"+m(q(f),b)+"</"+c+">"})}CKEDITOR.htmlDataProcessor=function(b){var c,d,f=this;this.editor=b;this.dataFilter=c=new CKEDITOR.htmlParser.filter;this.htmlFilter=d=new CKEDITOR.htmlParser.filter;this.writer=new CKEDITOR.htmlParser.basicWriter;c.addRules(x);
c.addRules(v,{applyToAll:true});c.addRules(a(b,"data"),{applyToAll:true});d.addRules(r);d.addRules(y,{applyToAll:true});d.addRules(a(b,"html"),{applyToAll:true});b.on("toHtml",function(a){var a=a.data,c=a.dataValue,d,c=l(c,b),c=k(c,D),c=i(c),c=k(c,G),c=c.replace(C,"$1cke:$2"),c=c.replace(E,"<cke:$1$2></cke:$1>"),c=c.replace(/(<pre\b[^>]*>)(\r\n|\n)/g,"$1$2$2"),c=c.replace(/([^a-z0-9<\-])(on\w{3,})(?!>)/gi,"$1data-cke-"+CKEDITOR.rnd+"-$2");d=a.context||b.editable().getName();var f;if(CKEDITOR.env.ie&&
CKEDITOR.env.version<9&&d=="pre"){d="div";c="<pre>"+c+"</pre>";f=1}d=b.document.createElement(d);d.setHtml("a"+c);c=d.getHtml().substr(1);c=c.replace(RegExp("data-cke-"+CKEDITOR.rnd+"-","ig"),"");f&&(c=c.replace(/^<pre>|<\/pre>$/gi,""));c=c.replace(F,"$1$2");c=n(c);c=q(c);d=a.fixForBody===false?false:e(a.enterMode,b.config.autoParagraph);c=CKEDITOR.htmlParser.fragment.fromHtml(c,a.context,d);if(d){f=c;if(!f.children.length&&CKEDITOR.dtd[f.name][d]){d=new CKEDITOR.htmlParser.element(d);f.add(d)}}a.dataValue=
c},null,null,5);b.on("toHtml",function(a){a.data.filter.applyTo(a.data.dataValue,true,a.data.dontFilter,a.data.enterMode)&&b.fire("dataFiltered")},null,null,6);b.on("toHtml",function(a){a.data.dataValue.filterChildren(f.dataFilter,true)},null,null,10);b.on("toHtml",function(a){var a=a.data,b=a.dataValue,c=new CKEDITOR.htmlParser.basicWriter;b.writeChildrenHtml(c);b=c.getHtml(true);a.dataValue=o(b)},null,null,15);b.on("toDataFormat",function(a){var c=a.data.dataValue;a.data.enterMode!=CKEDITOR.ENTER_BR&&
(c=c.replace(/^<br *\/?>/i,""));a.data.dataValue=CKEDITOR.htmlParser.fragment.fromHtml(c,a.data.context,e(a.data.enterMode,b.config.autoParagraph))},null,null,5);b.on("toDataFormat",function(a){a.data.dataValue.filterChildren(f.htmlFilter,true)},null,null,10);b.on("toDataFormat",function(a){a.data.filter.applyTo(a.data.dataValue,false,true)},null,null,11);b.on("toDataFormat",function(a){var c=a.data.dataValue,d=f.writer;d.reset();c.writeChildrenHtml(d);c=d.getHtml(true);c=q(c);c=m(c,b);a.data.dataValue=
c},null,null,15)};CKEDITOR.htmlDataProcessor.prototype={toHtml:function(a,b,c,d){var f=this.editor,g,h,e;if(b&&typeof b=="object"){g=b.context;c=b.fixForBody;d=b.dontFilter;h=b.filter;e=b.enterMode}else g=b;!g&&g!==null&&(g=f.editable().getName());return f.fire("toHtml",{dataValue:a,context:g,fixForBody:c,dontFilter:d,filter:h||f.filter,enterMode:e||f.enterMode}).dataValue},toDataFormat:function(a,b){var c,d,f;if(b){c=b.context;d=b.filter;f=b.enterMode}!c&&c!==null&&(c=this.editor.editable().getName());
return this.editor.fire("toDataFormat",{dataValue:a,filter:d||this.editor.filter,context:c,enterMode:f||this.editor.enterMode}).dataValue}};var p=/(?:&nbsp;|\xa0)$/,t="{cke_protected}",s=CKEDITOR.dtd,u=["caption","colgroup","col","thead","tfoot","tbody"],w=CKEDITOR.tools.extend({},s.$blockLimit,s.$block),x={elements:{input:f,textarea:f}},v={attributeNames:[[/^on/,"data-cke-pa-on"],[/^data-cke-expando$/,""]]},r={elements:{embed:function(a){var b=a.parent;if(b&&b.name=="object"){var c=b.attributes.width,
b=b.attributes.height;if(c)a.attributes.width=c;if(b)a.attributes.height=b}},a:function(a){if(!a.children.length&&!a.attributes.name&&!a.attributes["data-cke-saved-name"])return false}}},y={elementNames:[[/^cke:/,""],[/^\?xml:namespace$/,""]],attributeNames:[[/^data-cke-(saved|pa)-/,""],[/^data-cke-.*/,""],["hidefocus",""]],elements:{$:function(a){var b=a.attributes;if(b){if(b["data-cke-temp"])return false;for(var c=["name","href","src"],d,f=0;f<c.length;f++){d="data-cke-saved-"+c[f];d in b&&delete b[c[f]]}}return a},
table:function(a){a.children.slice(0).sort(function(a,b){var c,d;if(a.type==CKEDITOR.NODE_ELEMENT&&b.type==a.type){c=CKEDITOR.tools.indexOf(u,a.name);d=CKEDITOR.tools.indexOf(u,b.name)}if(!(c>-1&&d>-1&&c!=d)){c=a.parent?a.getIndex():-1;d=b.parent?b.getIndex():-1}return c>d?1:-1})},param:function(a){a.children=[];a.isEmpty=true;return a},span:function(a){a.attributes["class"]=="Apple-style-span"&&delete a.name},html:function(a){delete a.attributes.contenteditable;delete a.attributes["class"]},body:function(a){delete a.attributes.spellcheck;
delete a.attributes.contenteditable},style:function(a){var b=a.children[0];if(b&&b.value)b.value=CKEDITOR.tools.trim(b.value);if(!a.attributes.type)a.attributes.type="text/css"},title:function(a){var b=a.children[0];!b&&j(a,b=new CKEDITOR.htmlParser.text);b.value=a.attributes["data-cke-title"]||""},input:h,textarea:h},attributes:{"class":function(a){return CKEDITOR.tools.ltrim(a.replace(/(?:^|\s+)cke_[^\s]*/g,""))||false}}};if(CKEDITOR.env.ie)y.attributes.style=function(a){return a.replace(/(^|;)([^\:]+)/g,
function(a){return a.toLowerCase()})};var z=/<(a|area|img|input|source)\b([^>]*)>/gi,B=/([\w-]+)\s*=\s*(?:(?:"[^"]*")|(?:'[^']*')|(?:[^ "'>]+))/gi,A=/^(href|src|name)$/i,G=/(?:<style(?=[ >])[^>]*>[\s\S]*?<\/style>)|(?:<(:?link|meta|base)[^>]*>)/gi,D=/(<textarea(?=[ >])[^>]*>)([\s\S]*?)(?:<\/textarea>)/gi,J=/<cke:encoded>([^<]*)<\/cke:encoded>/gi,C=/(<\/?)((?:object|embed|param|html|body|head|title)[^>]*>)/gi,F=/(<\/?)cke:((?:html|body|head|title)[^>]*>)/gi,E=/<cke:(param|embed)([^>]*?)\/?>(?!\s*<\/cke:\1)/gi}(),
"use strict",CKEDITOR.htmlParser.element=function(a,e){this.name=a;this.attributes=e||{};this.children=[];var b=a||"",g=b.match(/^cke:(.*)/);g&&(b=g[1]);b=!(!CKEDITOR.dtd.$nonBodyContent[b]&&!CKEDITOR.dtd.$block[b]&&!CKEDITOR.dtd.$listItem[b]&&!CKEDITOR.dtd.$tableContent[b]&&!(CKEDITOR.dtd.$nonEditable[b]||b=="br"));this.isEmpty=!!CKEDITOR.dtd.$empty[a];this.isUnknown=!CKEDITOR.dtd[a];this._={isBlockLike:b,hasInlineStarted:this.isEmpty||!b}},CKEDITOR.htmlParser.cssStyle=function(a){var e={};((a instanceof
CKEDITOR.htmlParser.element?a.attributes.style:a)||"").replace(/&quot;/g,'"').replace(/\s*([^ :;]+)\s*:\s*([^;]+)\s*(?=;|$)/g,function(a,g,c){g=="font-family"&&(c=c.replace(/["']/g,""));e[g.toLowerCase()]=c});return{rules:e,populate:function(a){var g=this.toString();if(g)a instanceof CKEDITOR.dom.element?a.setAttribute("style",g):a instanceof CKEDITOR.htmlParser.element?a.attributes.style=g:a.style=g},toString:function(){var a=[],g;for(g in e)e[g]&&a.push(g,":",e[g],";");return a.join("")}}},function(){function a(a){return function(b){return b.type==
CKEDITOR.NODE_ELEMENT&&(typeof a=="string"?b.name==a:b.name in a)}}var e=function(a,b){a=a[0];b=b[0];return a<b?-1:a>b?1:0},b=CKEDITOR.htmlParser.fragment.prototype;CKEDITOR.htmlParser.element.prototype=CKEDITOR.tools.extend(new CKEDITOR.htmlParser.node,{type:CKEDITOR.NODE_ELEMENT,add:b.add,clone:function(){return new CKEDITOR.htmlParser.element(this.name,this.attributes)},filter:function(a,b){var d=this,e,f,b=d.getFilterContext(b);if(b.off)return true;if(!d.parent)a.onRoot(b,d);for(;;){e=d.name;
if(!(f=a.onElementName(b,e))){this.remove();return false}d.name=f;if(!(d=a.onElement(b,d))){this.remove();return false}if(d!==this){this.replaceWith(d);return false}if(d.name==e)break;if(d.type!=CKEDITOR.NODE_ELEMENT){this.replaceWith(d);return false}if(!d.name){this.replaceWithChildren();return false}}e=d.attributes;var h,i;for(h in e){i=h;for(f=e[h];;)if(i=a.onAttributeName(b,h))if(i!=h){delete e[h];h=i}else break;else{delete e[h];break}i&&((f=a.onAttribute(b,d,i,f))===false?delete e[i]:e[i]=f)}d.isEmpty||
this.filterChildren(a,false,b);return true},filterChildren:b.filterChildren,writeHtml:function(a,b){b&&this.filter(b);var d=this.name,j=[],f=this.attributes,h,i;a.openTag(d,f);for(h in f)j.push([h,f[h]]);a.sortAttributes&&j.sort(e);h=0;for(i=j.length;h<i;h++){f=j[h];a.attribute(f[0],f[1])}a.openTagClose(d,this.isEmpty);this.writeChildrenHtml(a);this.isEmpty||a.closeTag(d)},writeChildrenHtml:b.writeChildrenHtml,replaceWithChildren:function(){for(var a=this.children,b=a.length;b;)a[--b].insertAfter(this);
this.remove()},forEach:b.forEach,getFirst:function(b){if(!b)return this.children.length?this.children[0]:null;typeof b!="function"&&(b=a(b));for(var c=0,d=this.children.length;c<d;++c)if(b(this.children[c]))return this.children[c];return null},getHtml:function(){var a=new CKEDITOR.htmlParser.basicWriter;this.writeChildrenHtml(a);return a.getHtml()},setHtml:function(a){for(var a=this.children=CKEDITOR.htmlParser.fragment.fromHtml(a).children,b=0,d=a.length;b<d;++b)a[b].parent=this},getOuterHtml:function(){var a=
new CKEDITOR.htmlParser.basicWriter;this.writeHtml(a);return a.getHtml()},split:function(a){for(var b=this.children.splice(a,this.children.length-a),d=this.clone(),e=0;e<b.length;++e)b[e].parent=d;d.children=b;if(b[0])b[0].previous=null;if(a>0)this.children[a-1].next=null;this.parent.add(d,this.getIndex()+1);return d},addClass:function(a){if(!this.hasClass(a)){var b=this.attributes["class"]||"";this.attributes["class"]=b+(b?" ":"")+a}},removeClass:function(a){var b=this.attributes["class"];if(b)(b=
CKEDITOR.tools.trim(b.replace(RegExp("(?:\\s+|^)"+a+"(?:\\s+|$)")," ")))?this.attributes["class"]=b:delete this.attributes["class"]},hasClass:function(a){var b=this.attributes["class"];return!b?false:RegExp("(?:^|\\s)"+a+"(?=\\s|$)").test(b)},getFilterContext:function(a){var b=[];a||(a={off:false,nonEditable:false,nestedEditable:false});!a.off&&this.attributes["data-cke-processor"]=="off"&&b.push("off",true);!a.nonEditable&&this.attributes.contenteditable=="false"?b.push("nonEditable",true):a.nonEditable&&
(!a.nestedEditable&&this.attributes.contenteditable=="true")&&b.push("nestedEditable",true);if(b.length)for(var a=CKEDITOR.tools.copy(a),d=0;d<b.length;d=d+2)a[b[d]]=b[d+1];return a}},true)}(),function(){var a={},e=/{([^}]+)}/g,b=/([\\'])/g,g=/\n/g,c=/\r/g;CKEDITOR.template=function(d){if(a[d])this.output=a[d];else{var j=d.replace(b,"\\$1").replace(g,"\\n").replace(c,"\\r").replace(e,function(a,b){return"',data['"+b+"']==undefined?'{"+b+"}':data['"+b+"'],'"});this.output=a[d]=Function("data","buffer",
"return buffer?buffer.push('"+j+"'):['"+j+"'].join('');")}}}(),delete CKEDITOR.loadFullCore,CKEDITOR.instances={},CKEDITOR.document=new CKEDITOR.dom.document(document),CKEDITOR.add=function(a){CKEDITOR.instances[a.name]=a;a.on("focus",function(){if(CKEDITOR.currentInstance!=a){CKEDITOR.currentInstance=a;CKEDITOR.fire("currentInstance")}});a.on("blur",function(){if(CKEDITOR.currentInstance==a){CKEDITOR.currentInstance=null;CKEDITOR.fire("currentInstance")}});CKEDITOR.fire("instance",null,a)},CKEDITOR.remove=
function(a){delete CKEDITOR.instances[a.name]},function(){var a={};CKEDITOR.addTemplate=function(e,b){var g=a[e];if(g)return g;g={name:e,source:b};CKEDITOR.fire("template",g);return a[e]=new CKEDITOR.template(g.source)};CKEDITOR.getTemplate=function(e){return a[e]}}(),function(){var a=[];CKEDITOR.addCss=function(e){a.push(e)};CKEDITOR.getCss=function(){return a.join("\n")}}(),CKEDITOR.on("instanceDestroyed",function(){CKEDITOR.tools.isEmpty(this.instances)&&CKEDITOR.fire("reset")}),CKEDITOR.TRISTATE_ON=
1,CKEDITOR.TRISTATE_OFF=2,CKEDITOR.TRISTATE_DISABLED=0,function(){CKEDITOR.inline=function(a,e){if(!CKEDITOR.env.isCompatible)return null;a=CKEDITOR.dom.element.get(a);if(a.getEditor())throw'The editor instance "'+a.getEditor().name+'" is already attached to the provided element.';var b=new CKEDITOR.editor(e,a,CKEDITOR.ELEMENT_MODE_INLINE),g=a.is("textarea")?a:null;if(g){b.setData(g.getValue(),null,true);a=CKEDITOR.dom.element.createFromHtml('<div contenteditable="'+!!b.readOnly+'" class="cke_textarea_inline">'+
g.getValue()+"</div>",CKEDITOR.document);a.insertAfter(g);g.hide();g.$.form&&b._attachToForm()}else b.setData(a.getHtml(),null,true);b.on("loaded",function(){b.fire("uiReady");b.editable(a);b.container=a;b.setData(b.getData(1));b.resetDirty();b.fire("contentDom");b.mode="wysiwyg";b.fire("mode");b.status="ready";b.fireOnce("instanceReady");CKEDITOR.fire("instanceReady",null,b)},null,null,1E4);b.on("destroy",function(){if(g){b.container.clearCustomData();b.container.remove();g.show()}b.element.clearCustomData();
delete b.element});return b};CKEDITOR.inlineAll=function(){var a,e,b;for(b in CKEDITOR.dtd.$editable)for(var g=CKEDITOR.document.getElementsByTag(b),c=0,d=g.count();c<d;c++){a=g.getItem(c);if(a.getAttribute("contenteditable")=="true"){e={element:a,config:{}};CKEDITOR.fire("inline",e)!==false&&CKEDITOR.inline(a,e.config)}}};CKEDITOR.domReady(function(){!CKEDITOR.disableAutoInline&&CKEDITOR.inlineAll()})}(),CKEDITOR.replaceClass="ckeditor",function(){function a(a,c,d,j){if(!CKEDITOR.env.isCompatible)return null;
a=CKEDITOR.dom.element.get(a);if(a.getEditor())throw'The editor instance "'+a.getEditor().name+'" is already attached to the provided element.';var f=new CKEDITOR.editor(c,a,j);if(j==CKEDITOR.ELEMENT_MODE_REPLACE){a.setStyle("visibility","hidden");f._.required=a.hasAttribute("required");a.removeAttribute("required")}d&&f.setData(d,null,true);f.on("loaded",function(){b(f);j==CKEDITOR.ELEMENT_MODE_REPLACE&&(f.config.autoUpdateElement&&a.$.form)&&f._attachToForm();f.setMode(f.config.startupMode,function(){f.resetDirty();
f.status="ready";f.fireOnce("instanceReady");CKEDITOR.fire("instanceReady",null,f)})});f.on("destroy",e);return f}function e(){var a=this.container,b=this.element;if(a){a.clearCustomData();a.remove()}if(b){b.clearCustomData();if(this.elementMode==CKEDITOR.ELEMENT_MODE_REPLACE){b.show();this._.required&&b.setAttribute("required","required")}delete this.element}}function b(a){var b=a.name,d=a.element,e=a.elementMode,f=a.fire("uiSpace",{space:"top",html:""}).html,h=a.fire("uiSpace",{space:"bottom",html:""}).html,
i=new CKEDITOR.template('<{outerEl} id="cke_{name}" class="{id} cke cke_reset cke_chrome cke_editor_{name} cke_{langDir} '+CKEDITOR.env.cssClass+'"  dir="{langDir}" lang="{langCode}" role="application"'+(a.title?' aria-labelledby="cke_{name}_arialbl"':"")+">"+(a.title?'<span id="cke_{name}_arialbl" class="cke_voice_label">{voiceLabel}</span>':"")+'<{outerEl} class="cke_inner cke_reset" role="presentation">{topHtml}<{outerEl} id="{contentId}" class="cke_contents cke_reset" role="presentation"></{outerEl}>{bottomHtml}</{outerEl}></{outerEl}>'),
b=CKEDITOR.dom.element.createFromHtml(i.output({id:a.id,name:b,langDir:a.lang.dir,langCode:a.langCode,voiceLabel:a.title,topHtml:f?'<span id="'+a.ui.spaceId("top")+'" class="cke_top cke_reset_all" role="presentation" style="height:auto">'+f+"</span>":"",contentId:a.ui.spaceId("contents"),bottomHtml:h?'<span id="'+a.ui.spaceId("bottom")+'" class="cke_bottom cke_reset_all" role="presentation">'+h+"</span>":"",outerEl:CKEDITOR.env.ie?"span":"div"}));if(e==CKEDITOR.ELEMENT_MODE_REPLACE){d.hide();b.insertAfter(d)}else d.append(b);
a.container=b;f&&a.ui.space("top").unselectable();h&&a.ui.space("bottom").unselectable();d=a.config.width;e=a.config.height;d&&b.setStyle("width",CKEDITOR.tools.cssLength(d));e&&a.ui.space("contents").setStyle("height",CKEDITOR.tools.cssLength(e));b.disableContextMenu();CKEDITOR.env.webkit&&b.on("focus",function(){a.focus()});a.fireOnce("uiReady")}CKEDITOR.replace=function(b,c){return a(b,c,null,CKEDITOR.ELEMENT_MODE_REPLACE)};CKEDITOR.appendTo=function(b,c,d){return a(b,c,d,CKEDITOR.ELEMENT_MODE_APPENDTO)};
CKEDITOR.replaceAll=function(){for(var a=document.getElementsByTagName("textarea"),b=0;b<a.length;b++){var d=null,e=a[b];if(e.name||e.id){if(typeof arguments[0]=="string"){if(!RegExp("(?:^|\\s)"+arguments[0]+"(?:$|\\s)").test(e.className))continue}else if(typeof arguments[0]=="function"){d={};if(arguments[0](e,d)===false)continue}this.replace(e,d)}}};CKEDITOR.editor.prototype.addMode=function(a,b){(this._.modes||(this._.modes={}))[a]=b};CKEDITOR.editor.prototype.setMode=function(a,b){var d=this,e=
this._.modes;if(!(a==d.mode||!e||!e[a])){d.fire("beforeSetMode",a);if(d.mode){var f=d.checkDirty(),e=d._.previousModeData,h,i=0;d.fire("beforeModeUnload");d.editable(0);d._.previousMode=d.mode;d._.previousModeData=h=d.getData(1);if(d.mode=="source"&&e==h){d.fire("lockSnapshot",{forceUpdate:true});i=1}d.ui.space("contents").setHtml("");d.mode=""}else d._.previousModeData=d.getData(1);this._.modes[a](function(){d.mode=a;f!==void 0&&!f&&d.resetDirty();i?d.fire("unlockSnapshot"):a=="wysiwyg"&&d.fire("saveSnapshot");
setTimeout(function(){d.fire("mode");b&&b.call(d)},0)})}};CKEDITOR.editor.prototype.resize=function(a,b,d,e){var f=this.container,h=this.ui.space("contents"),i=CKEDITOR.env.webkit&&this.document&&this.document.getWindow().$.frameElement,e=e?this.container.getFirst(function(a){return a.type==CKEDITOR.NODE_ELEMENT&&a.hasClass("cke_inner")}):f;e.setSize("width",a,true);i&&(i.style.width="1%");h.setStyle("height",Math.max(b-(d?0:(e.$.offsetHeight||0)-(h.$.clientHeight||0)),0)+"px");i&&(i.style.width=
"100%");this.fire("resize")};CKEDITOR.editor.prototype.getResizable=function(a){return a?this.ui.space("contents"):this.container};CKEDITOR.domReady(function(){CKEDITOR.replaceClass&&CKEDITOR.replaceAll(CKEDITOR.replaceClass)})}(),CKEDITOR.config.startupMode="wysiwyg",function(){function a(a){var b=a.editor,c=a.data.path,d=c.blockLimit,f=a.data.selection,h=f.getRanges()[0],i;if(CKEDITOR.env.gecko||CKEDITOR.env.ie&&CKEDITOR.env.needsBrFiller)if(f=e(f,c)){f.appendBogus();i=CKEDITOR.env.ie}if(j(b,c.block,
d)&&h.collapsed&&!h.getCommonAncestor().isReadOnly()){c=h.clone();c.enlarge(CKEDITOR.ENLARGE_BLOCK_CONTENTS);d=new CKEDITOR.dom.walker(c);d.guard=function(a){return!g(a)||a.type==CKEDITOR.NODE_COMMENT||a.isReadOnly()};if(!d.checkForward()||c.checkStartOfBlock()&&c.checkEndOfBlock()){b=h.fixBlock(true,b.activeEnterMode==CKEDITOR.ENTER_DIV?"div":"p");if(!CKEDITOR.env.needsBrFiller)(b=b.getFirst(g))&&(b.type==CKEDITOR.NODE_TEXT&&CKEDITOR.tools.trim(b.getText()).match(/^(?:&nbsp;|\xa0)$/))&&b.remove();
i=1;a.cancel()}}i&&h.select()}function e(a,b){if(a.isFake)return 0;var c=b.block||b.blockLimit,d=c&&c.getLast(g);if(c&&c.isBlockBoundary()&&(!d||!(d.type==CKEDITOR.NODE_ELEMENT&&d.isBlockBoundary()))&&!c.is("pre")&&!c.getBogus())return c}function b(a){var b=a.data.getTarget();if(b.is("input")){b=b.getAttribute("type");(b=="submit"||b=="reset")&&a.data.preventDefault()}}function g(a){return o(a)&&q(a)}function c(a,b){return function(c){var d=CKEDITOR.dom.element.get(c.data.$.toElement||c.data.$.fromElement||
c.data.$.relatedTarget);(!d||!b.equals(d)&&!b.contains(d))&&a.call(this,c)}}function d(a){function b(a){return function(b,d){d&&(b.type==CKEDITOR.NODE_ELEMENT&&b.is(f))&&(c=b);if(!d&&g(b)&&(!a||!k(b)))return false}}var c,d=a.getRanges()[0],a=a.root,f={table:1,ul:1,ol:1,dl:1};if(d.startPath().contains(f)){var e=d.clone();e.collapse(1);e.setStartAt(a,CKEDITOR.POSITION_AFTER_START);a=new CKEDITOR.dom.walker(e);a.guard=b();a.checkBackward();if(c){e=d.clone();e.collapse();e.setEndAt(c,CKEDITOR.POSITION_AFTER_END);
a=new CKEDITOR.dom.walker(e);a.guard=b(true);c=false;a.checkForward();return c}}return null}function j(a,b,c){return a.config.autoParagraph!==false&&a.activeEnterMode!=CKEDITOR.ENTER_BR&&a.editable().equals(c)&&!b||b&&b.getAttribute("contenteditable")=="true"}function f(a){a.editor.focus();a.editor.fire("saveSnapshot")}function h(a){var b=a.editor;b.getSelection().scrollIntoView();setTimeout(function(){b.fire("saveSnapshot")},0)}function i(a,b,c){for(var d=a.getCommonAncestor(b),b=a=c?b:a;(a=a.getParent())&&
!d.equals(a)&&a.getChildCount()==1;)b=a;b.remove()}CKEDITOR.editable=CKEDITOR.tools.createClass({base:CKEDITOR.dom.element,$:function(a,b){this.base(b.$||b);this.editor=a;this.status="unloaded";this.hasFocus=false;this.setup()},proto:{focus:function(){var a;if(CKEDITOR.env.webkit&&!this.hasFocus){a=this.editor._.previousActive||this.getDocument().getActive();if(this.contains(a)){a.focus();return}}try{this.$[CKEDITOR.env.ie&&this.getDocument().equals(CKEDITOR.document)?"setActive":"focus"]()}catch(b){if(!CKEDITOR.env.ie)throw b;
}if(CKEDITOR.env.safari&&!this.isInline()){a=CKEDITOR.document.getActive();a.equals(this.getWindow().getFrame())||this.getWindow().focus()}},on:function(a,b){var d=Array.prototype.slice.call(arguments,0);if(CKEDITOR.env.ie&&/^focus|blur$/.exec(a)){a=a=="focus"?"focusin":"focusout";b=c(b,this);d[0]=a;d[1]=b}return CKEDITOR.dom.element.prototype.on.apply(this,d)},attachListener:function(a){!this._.listeners&&(this._.listeners=[]);var b=Array.prototype.slice.call(arguments,1),b=a.on.apply(a,b);this._.listeners.push(b);
return b},clearListeners:function(){var a=this._.listeners;try{for(;a.length;)a.pop().removeListener()}catch(b){}},restoreAttrs:function(){var a=this._.attrChanges,b,c;for(c in a)if(a.hasOwnProperty(c)){b=a[c];b!==null?this.setAttribute(c,b):this.removeAttribute(c)}},attachClass:function(a){var b=this.getCustomData("classes");if(!this.hasClass(a)){!b&&(b=[]);b.push(a);this.setCustomData("classes",b);this.addClass(a)}},changeAttr:function(a,b){var c=this.getAttribute(a);if(b!==c){!this._.attrChanges&&
(this._.attrChanges={});a in this._.attrChanges||(this._.attrChanges[a]=c);this.setAttribute(a,b)}},insertHtml:function(a,b){f(this);m(this,b||"html",a)},insertText:function(a){f(this);var b=this.editor,c=b.getSelection().getStartElement().hasAscendant("pre",true)?CKEDITOR.ENTER_BR:b.activeEnterMode,b=c==CKEDITOR.ENTER_BR,d=CKEDITOR.tools,a=d.htmlEncode(a.replace(/\r\n/g,"\n")),a=a.replace(/\t/g,"&nbsp;&nbsp; &nbsp;"),c=c==CKEDITOR.ENTER_P?"p":"div";if(!b){var g=/\n{2}/g;if(g.test(a))var e="<"+c+
">",h="</"+c+">",a=e+a.replace(g,function(){return h+e})+h}a=a.replace(/\n/g,"<br>");b||(a=a.replace(RegExp("<br>(?=</"+c+">)"),function(a){return d.repeat(a,2)}));a=a.replace(/^ | $/g,"&nbsp;");a=a.replace(/(>|\s) /g,function(a,b){return b+"&nbsp;"}).replace(/ (?=<)/g,"&nbsp;");m(this,"text",a)},insertElement:function(a,b){b?this.insertElementIntoRange(a,b):this.insertElementIntoSelection(a)},insertElementIntoRange:function(a,b){var c=this.editor,d=c.config.enterMode,f=a.getName(),g=CKEDITOR.dtd.$block[f];
if(b.checkReadOnly())return false;b.deleteContents(1);b.startContainer.type==CKEDITOR.NODE_ELEMENT&&b.startContainer.is({tr:1,table:1,tbody:1,thead:1,tfoot:1})&&l(b);var e,h;if(g)for(;(e=b.getCommonAncestor(0,1))&&(h=CKEDITOR.dtd[e.getName()])&&(!h||!h[f]);)if(e.getName()in CKEDITOR.dtd.span)b.splitElement(e);else if(b.checkStartOfBlock()&&b.checkEndOfBlock()){b.setStartBefore(e);b.collapse(true);e.remove()}else b.splitBlock(d==CKEDITOR.ENTER_DIV?"div":"p",c.editable());b.insertNode(a);return true},
insertElementIntoSelection:function(a){f(this);var b=this.editor,c=b.activeEnterMode,b=b.getSelection(),d=b.getRanges()[0],e=a.getName(),e=CKEDITOR.dtd.$block[e];if(this.insertElementIntoRange(a,d)){d.moveToPosition(a,CKEDITOR.POSITION_AFTER_END);if(e)if((e=a.getNext(function(a){return g(a)&&!k(a)}))&&e.type==CKEDITOR.NODE_ELEMENT&&e.is(CKEDITOR.dtd.$block))e.getDtd()["#"]?d.moveToElementEditStart(e):d.moveToElementEditEnd(a);else if(!e&&c!=CKEDITOR.ENTER_BR){e=d.fixBlock(true,c==CKEDITOR.ENTER_DIV?
"div":"p");d.moveToElementEditStart(e)}}b.selectRanges([d]);h(this)},setData:function(a,b){b||(a=this.editor.dataProcessor.toHtml(a));this.setHtml(a);this.fixInitialSelection();if(this.status=="unloaded")this.status="ready";this.editor.fire("dataReady")},getData:function(a){var b=this.getHtml();a||(b=this.editor.dataProcessor.toDataFormat(b));return b},setReadOnly:function(a){this.setAttribute("contenteditable",!a)},detach:function(){this.removeClass("cke_editable");this.status="detached";var a=this.editor;
this._.detach();delete a.document;delete a.window},isInline:function(){return this.getDocument().equals(CKEDITOR.document)},fixInitialSelection:function(){function a(){var b=c.getDocument().$,d=b.getSelection(),f;if(d.anchorNode&&d.anchorNode==c.$)f=true;else if(CKEDITOR.env.webkit){var g=c.getDocument().getActive();g&&(g.equals(c)&&!d.anchorNode)&&(f=true)}if(f){f=new CKEDITOR.dom.range(c);f.moveToElementEditStart(c);b=b.createRange();b.setStart(f.startContainer.$,f.startOffset);b.collapse(true);
d.removeAllRanges();d.addRange(b)}}function b(){var a=c.getDocument().$,d=a.selection,f=c.getDocument().getActive();if(d.type=="None"&&f.equals(c)){d=new CKEDITOR.dom.range(c);a=a.body.createTextRange();d.moveToElementEditStart(c);d=d.startContainer;d.type!=CKEDITOR.NODE_ELEMENT&&(d=d.getParent());a.moveToElementText(d.$);a.collapse(true);a.select()}}var c=this;if(CKEDITOR.env.ie&&(CKEDITOR.env.version<9||CKEDITOR.env.quirks)){if(this.hasFocus){this.focus();b()}}else if(this.hasFocus){this.focus();
a()}else this.once("focus",function(){a()},null,null,-999)},setup:function(){var a=this.editor;this.attachListener(a,"beforeGetData",function(){var b=this.getData();this.is("textarea")||a.config.ignoreEmptyParagraph!==false&&(b=b.replace(n,function(a,b){return b}));a.setData(b,null,1)},this);this.attachListener(a,"getSnapshot",function(a){a.data=this.getData(1)},this);this.attachListener(a,"afterSetData",function(){this.setData(a.getData(1))},this);this.attachListener(a,"loadSnapshot",function(a){this.setData(a.data,
1)},this);this.attachListener(a,"beforeFocus",function(){var b=a.getSelection();(b=b&&b.getNative())&&b.type=="Control"||this.focus()},this);this.attachListener(a,"insertHtml",function(a){this.insertHtml(a.data.dataValue,a.data.mode)},this);this.attachListener(a,"insertElement",function(a){this.insertElement(a.data)},this);this.attachListener(a,"insertText",function(a){this.insertText(a.data)},this);this.setReadOnly(a.readOnly);this.attachClass("cke_editable");this.attachClass(a.elementMode==CKEDITOR.ELEMENT_MODE_INLINE?
"cke_editable_inline":a.elementMode==CKEDITOR.ELEMENT_MODE_REPLACE||a.elementMode==CKEDITOR.ELEMENT_MODE_APPENDTO?"cke_editable_themed":"");this.attachClass("cke_contents_"+a.config.contentsLangDirection);a.keystrokeHandler.blockedKeystrokes[8]=+a.readOnly;a.keystrokeHandler.attach(this);this.on("blur",function(){this.hasFocus=false},null,null,-1);this.on("focus",function(){this.hasFocus=true},null,null,-1);a.focusManager.add(this);if(this.equals(CKEDITOR.document.getActive())){this.hasFocus=true;
a.once("contentDom",function(){a.focusManager.focus(this)},this)}this.isInline()&&this.changeAttr("tabindex",a.tabIndex);if(!this.is("textarea")){a.document=this.getDocument();a.window=this.getWindow();var c=a.document;this.changeAttr("spellcheck",!a.config.disableNativeSpellChecker);var f=a.config.contentsLangDirection;this.getDirection(1)!=f&&this.changeAttr("dir",f);var e=CKEDITOR.getCss();if(e){f=c.getHead();if(!f.getCustomData("stylesheet")){e=c.appendStyleText(e);e=new CKEDITOR.dom.element(e.ownerNode||
e.owningElement);f.setCustomData("stylesheet",e);e.data("cke-temp",1)}}f=c.getCustomData("stylesheet_ref")||0;c.setCustomData("stylesheet_ref",f+1);this.setCustomData("cke_includeReadonly",!a.config.disableReadonlyStyling);this.attachListener(this,"click",function(a){var a=a.data,b=(new CKEDITOR.dom.elementPath(a.getTarget(),this)).contains("a");b&&(a.$.button!=2&&b.isReadOnly())&&a.preventDefault()});var h={8:1,46:1};this.attachListener(a,"key",function(b){if(a.readOnly)return true;var c=b.data.domEvent.getKey(),
f;if(c in h){var b=a.getSelection(),e,g=b.getRanges()[0],i=g.startPath(),j,k,t,c=c==8;if(CKEDITOR.env.ie&&CKEDITOR.env.version<11&&(e=b.getSelectedElement())||(e=d(b))){a.fire("saveSnapshot");g.moveToPosition(e,CKEDITOR.POSITION_BEFORE_START);e.remove();g.select();a.fire("saveSnapshot");f=1}else if(g.collapsed)if((j=i.block)&&(t=j[c?"getPrevious":"getNext"](o))&&t.type==CKEDITOR.NODE_ELEMENT&&t.is("table")&&g[c?"checkStartOfBlock":"checkEndOfBlock"]()){a.fire("saveSnapshot");g[c?"checkEndOfBlock":
"checkStartOfBlock"]()&&j.remove();g["moveToElementEdit"+(c?"End":"Start")](t);g.select();a.fire("saveSnapshot");f=1}else if(i.blockLimit&&i.blockLimit.is("td")&&(k=i.blockLimit.getAscendant("table"))&&g.checkBoundaryOfElement(k,c?CKEDITOR.START:CKEDITOR.END)&&(t=k[c?"getPrevious":"getNext"](o))){a.fire("saveSnapshot");g["moveToElementEdit"+(c?"End":"Start")](t);g.checkStartOfBlock()&&g.checkEndOfBlock()?t.remove():g.select();a.fire("saveSnapshot");f=1}else if((k=i.contains(["td","th","caption"]))&&
g.checkBoundaryOfElement(k,c?CKEDITOR.START:CKEDITOR.END))f=1}return!f});a.blockless&&(CKEDITOR.env.ie&&CKEDITOR.env.needsBrFiller)&&this.attachListener(this,"keyup",function(b){if(b.data.getKeystroke()in h&&!this.getFirst(g)){this.appendBogus();b=a.createRange();b.moveToPosition(this,CKEDITOR.POSITION_AFTER_START);b.select()}});this.attachListener(this,"dblclick",function(b){if(a.readOnly)return false;b={element:b.data.getTarget()};a.fire("doubleclick",b)});CKEDITOR.env.ie&&this.attachListener(this,
"click",b);CKEDITOR.env.ie||this.attachListener(this,"mousedown",function(b){var c=b.data.getTarget();if(c.is("img","hr","input","textarea","select")&&!c.isReadOnly()){a.getSelection().selectElement(c);c.is("input","textarea","select")&&b.data.preventDefault()}});CKEDITOR.env.gecko&&this.attachListener(this,"mouseup",function(b){if(b.data.$.button==2){b=b.data.getTarget();if(!b.getOuterHtml().replace(n,"")){var c=a.createRange();c.moveToElementEditStart(b);c.select(true)}}});if(CKEDITOR.env.webkit){this.attachListener(this,
"click",function(a){a.data.getTarget().is("input","select")&&a.data.preventDefault()});this.attachListener(this,"mouseup",function(a){a.data.getTarget().is("input","textarea")&&a.data.preventDefault()})}CKEDITOR.env.webkit&&this.attachListener(a,"key",function(b){b=b.data.domEvent.getKey();if(b in h){var c=b==8,d=a.getSelection().getRanges()[0],b=d.startPath();if(d.collapsed){var f;a:{var g=b.block;if(g)if(d[c?"checkStartOfBlock":"checkEndOfBlock"]())if(!d.moveToClosestEditablePosition(g,!c)||!d.collapsed)f=
false;else{if(d.startContainer.type==CKEDITOR.NODE_ELEMENT){var e=d.startContainer.getChild(d.startOffset-(c?1:0));if(e&&e.type==CKEDITOR.NODE_ELEMENT&&e.is("hr")){a.fire("saveSnapshot");e.remove();f=true;break a}}if((d=d.startPath().block)&&(!d||!d.contains(g))){a.fire("saveSnapshot");var j;(j=(c?d:g).getBogus())&&j.remove();f=a.getSelection();j=f.createBookmarks();(c?g:d).moveChildren(c?d:g,false);b.lastElement.mergeSiblings();i(g,d,!c);f.selectBookmarks(j);f=true}}else f=false;else f=false}if(!f)return}else{c=
d;f=b.block;j=c.endPath().block;if(!f||!j||f.equals(j))b=false;else{a.fire("saveSnapshot");(g=f.getBogus())&&g.remove();c.deleteContents();if(j.getParent()){j.moveChildren(f,false);b.lastElement.mergeSiblings();i(f,j,true)}c=a.getSelection().getRanges()[0];c.collapse(1);c.select();b=true}if(!b)return}a.getSelection().scrollIntoView();a.fire("saveSnapshot");return false}},this,null,100)}}},_:{detach:function(){this.editor.setData(this.editor.getData(),0,1);this.clearListeners();this.restoreAttrs();
var a;if(a=this.removeCustomData("classes"))for(;a.length;)this.removeClass(a.pop());if(!this.is("textarea")){a=this.getDocument();var b=a.getHead();if(b.getCustomData("stylesheet")){var c=a.getCustomData("stylesheet_ref");if(--c)a.setCustomData("stylesheet_ref",c);else{a.removeCustomData("stylesheet_ref");b.removeCustomData("stylesheet").remove()}}}this.editor.fire("contentDomUnload");delete this.editor}}});CKEDITOR.editor.prototype.editable=function(a){var b=this._.editable;if(b&&a)return 0;if(arguments.length)b=
this._.editable=a?a instanceof CKEDITOR.editable?a:new CKEDITOR.editable(this,a):(b&&b.detach(),null);return b};var k=CKEDITOR.dom.walker.bogus(),n=/(^|<body\b[^>]*>)\s*<(p|div|address|h\d|center|pre)[^>]*>\s*(?:<br[^>]*>|&nbsp;|\u00A0|&#160;)?\s*(:?<\/\2>)?\s*(?=$|<\/body>)/gi,o=CKEDITOR.dom.walker.whitespaces(true),q=CKEDITOR.dom.walker.bookmark(false,true);CKEDITOR.on("instanceLoaded",function(b){var c=b.editor;c.on("insertElement",function(a){a=a.data;if(a.type==CKEDITOR.NODE_ELEMENT&&(a.is("input")||
a.is("textarea"))){a.getAttribute("contentEditable")!="false"&&a.data("cke-editable",a.hasAttribute("contenteditable")?"true":"1");a.setAttribute("contentEditable",false)}});c.on("selectionChange",function(b){if(!c.readOnly){var d=c.getSelection();if(d&&!d.isLocked){d=c.checkDirty();c.fire("lockSnapshot");a(b);c.fire("unlockSnapshot");!d&&c.resetDirty()}}})});CKEDITOR.on("instanceCreated",function(a){var b=a.editor;b.on("mode",function(){var a=b.editable();if(a&&a.isInline()){var c=b.title;a.changeAttr("role",
"textbox");a.changeAttr("aria-label",c);c&&a.changeAttr("title",c);var d=b.fire("ariaEditorHelpLabel",{}).label;if(d)if(c=this.ui.space(this.elementMode==CKEDITOR.ELEMENT_MODE_INLINE?"top":"contents")){var f=CKEDITOR.tools.getNextId(),d=CKEDITOR.dom.element.createFromHtml('<span id="'+f+'" class="cke_voice_label">'+d+"</span>");c.append(d);a.changeAttr("aria-describedby",f)}}})});CKEDITOR.addCss(".cke_editable{cursor:text}.cke_editable img,.cke_editable input,.cke_editable textarea{cursor:default}");
var m=function(){function a(b){return b.type==CKEDITOR.NODE_ELEMENT}function b(c,d){var f,g,e,h,j=[],k=d.range.startContainer;f=d.range.startPath();for(var k=i[k.getName()],m=0,n=c.getChildren(),w=n.count(),s=-1,x=-1,l=0,r=f.contains(i.$list);m<w;++m){f=n.getItem(m);if(a(f)){e=f.getName();if(r&&e in CKEDITOR.dtd.$list)j=j.concat(b(f,d));else{h=!!k[e];if(e=="br"&&f.data("cke-eol")&&(!m||m==w-1)){l=(g=m?j[m-1].node:n.getItem(m+1))&&(!a(g)||!g.is("br"));g=g&&a(g)&&i.$block[g.getName()]}s==-1&&!h&&(s=
m);h||(x=m);j.push({isElement:1,isLineBreak:l,isBlock:f.isBlockBoundary(),hasBlockSibling:g,node:f,name:e,allowed:h});g=l=0}}else j.push({isElement:0,node:f,allowed:1})}if(s>-1)j[s].firstNotAllowed=1;if(x>-1)j[x].lastNotAllowed=1;return j}function c(b,d){var f=[],g=b.getChildren(),e=g.count(),h,j=0,k=i[d],m=!b.is(i.$inline)||b.is("br");for(m&&f.push(" ");j<e;j++){h=g.getItem(j);a(h)&&!h.is(k)?f=f.concat(c(h,d)):f.push(h)}m&&f.push(" ");return f}function d(b){return b&&a(b)&&(b.is(i.$removeEmpty)||
b.is("a")&&!b.isBlockBoundary())}function f(b,c,d,g){var e=b.clone(),h,j;e.setEndAt(c,CKEDITOR.POSITION_BEFORE_END);if((h=(new CKEDITOR.dom.walker(e)).next())&&a(h)&&k[h.getName()]&&(j=h.getPrevious())&&a(j)&&!j.getParent().equals(b.startContainer)&&d.contains(j)&&g.contains(h)&&h.isIdentical(j)){h.moveChildren(j);h.remove();f(b,c,d,g)}}function e(b,c){function d(b,c){if(c.isBlock&&c.isElement&&!c.node.is("br")&&a(b)&&b.is("br")){b.remove();return 1}}var f=c.endContainer.getChild(c.endOffset),g=c.endContainer.getChild(c.endOffset-
1);f&&d(f,b[b.length-1]);if(g&&d(g,b[0])){c.setEnd(c.endContainer,c.endOffset-1);c.collapse()}}var i=CKEDITOR.dtd,k={p:1,div:1,h1:1,h2:1,h3:1,h4:1,h5:1,h6:1,ul:1,ol:1,li:1,pre:1,dl:1,blockquote:1},m={p:1,div:1,h1:1,h2:1,h3:1,h4:1,h5:1,h6:1},n=CKEDITOR.tools.extend({},i.$inline);delete n.br;return function(k,l,r){var q=k.editor,o=q.getSelection().getRanges()[0],C=false;if(l=="unfiltered_html"){l="html";C=true}if(!o.checkReadOnly()){var F=(new CKEDITOR.dom.elementPath(o.startContainer,o.root)).blockLimit||
o.root,l={type:l,dontFilter:C,editable:k,editor:q,range:o,blockLimit:F,mergeCandidates:[],zombies:[]},q=l.range,C=l.mergeCandidates,E,I,L,H;if(l.type=="text"&&q.shrink(CKEDITOR.SHRINK_ELEMENT,true,false)){E=CKEDITOR.dom.element.createFromHtml("<span>&nbsp;</span>",q.document);q.insertNode(E);q.setStartAfter(E)}I=new CKEDITOR.dom.elementPath(q.startContainer);l.endPath=L=new CKEDITOR.dom.elementPath(q.endContainer);if(!q.collapsed){var F=L.block||L.blockLimit,Q=q.getCommonAncestor();F&&(!F.equals(Q)&&
!F.contains(Q)&&q.checkEndOfBlock())&&l.zombies.push(F);q.deleteContents()}for(;(H=a(q.startContainer)&&q.startContainer.getChild(q.startOffset-1))&&a(H)&&H.isBlockBoundary()&&I.contains(H);)q.moveToPosition(H,CKEDITOR.POSITION_BEFORE_END);f(q,l.blockLimit,I,L);if(E){q.setEndBefore(E);q.collapse();E.remove()}E=q.startPath();if(F=E.contains(d,false,1)){q.splitElement(F);l.inlineStylesRoot=F;l.inlineStylesPeak=E.lastElement}E=q.createBookmark();(F=E.startNode.getPrevious(g))&&a(F)&&d(F)&&C.push(F);
(F=E.startNode.getNext(g))&&a(F)&&d(F)&&C.push(F);for(F=E.startNode;(F=F.getParent())&&d(F);)C.push(F);q.moveToBookmark(E);if(E=r){E=l.range;if(l.type=="text"&&l.inlineStylesRoot){H=l.inlineStylesPeak;q=H.getDocument().createText("{cke-peak}");for(C=l.inlineStylesRoot.getParent();!H.equals(C);){q=q.appendTo(H.clone());H=H.getParent()}r=q.getOuterHtml().split("{cke-peak}").join(r)}H=l.blockLimit.getName();if(/^\s+|\s+$/.test(r)&&"span"in CKEDITOR.dtd[H])var O='<span data-cke-marker="1">&nbsp;</span>',
r=O+r+O;r=l.editor.dataProcessor.toHtml(r,{context:null,fixForBody:false,dontFilter:l.dontFilter,filter:l.editor.activeFilter,enterMode:l.editor.activeEnterMode});H=E.document.createElement("body");H.setHtml(r);if(O){H.getFirst().remove();H.getLast().remove()}if((O=E.startPath().block)&&!(O.getChildCount()==1&&O.getBogus()))a:{var M;if(H.getChildCount()==1&&a(M=H.getFirst())&&M.is(m)){O=M.getElementsByTag("*");E=0;for(C=O.count();E<C;E++){q=O.getItem(E);if(!q.is(n))break a}M.moveChildren(M.getParent(1));
M.remove()}}l.dataWrapper=H;E=r}if(E){M=l.range;var O=M.document,K,r=l.blockLimit;E=0;var R;H=[];var N,T,C=q=0,P,V;I=M.startContainer;var F=l.endPath.elements[0],W;L=F.getPosition(I);Q=!!F.getCommonAncestor(I)&&L!=CKEDITOR.POSITION_IDENTICAL&&!(L&CKEDITOR.POSITION_CONTAINS+CKEDITOR.POSITION_IS_CONTAINED);I=b(l.dataWrapper,l);for(e(I,M);E<I.length;E++){L=I[E];if(K=L.isLineBreak){K=M;P=r;var S=void 0,Y=void 0;if(L.hasBlockSibling)K=1;else{S=K.startContainer.getAscendant(i.$block,1);if(!S||!S.is({div:1,
p:1}))K=0;else{Y=S.getPosition(P);if(Y==CKEDITOR.POSITION_IDENTICAL||Y==CKEDITOR.POSITION_CONTAINS)K=0;else{P=K.splitElement(S);K.moveToPosition(P,CKEDITOR.POSITION_AFTER_START);K=1}}}}if(K)C=E>0;else{K=M.startPath();if(!L.isBlock&&j(l.editor,K.block,K.blockLimit)&&(T=l.editor.activeEnterMode!=CKEDITOR.ENTER_BR&&l.editor.config.autoParagraph!==false?l.editor.activeEnterMode==CKEDITOR.ENTER_DIV?"div":"p":false)){T=O.createElement(T);T.appendBogus();M.insertNode(T);CKEDITOR.env.needsBrFiller&&(R=T.getBogus())&&
R.remove();M.moveToPosition(T,CKEDITOR.POSITION_BEFORE_END)}if((K=M.startPath().block)&&!K.equals(N)){if(R=K.getBogus()){R.remove();H.push(K)}N=K}L.firstNotAllowed&&(q=1);if(q&&L.isElement){K=M.startContainer;for(P=null;K&&!i[K.getName()][L.name];){if(K.equals(r)){K=null;break}P=K;K=K.getParent()}if(K){if(P){V=M.splitElement(P);l.zombies.push(V);l.zombies.push(P)}}else{P=r.getName();W=!E;K=E==I.length-1;P=c(L.node,P);for(var S=[],Y=P.length,Z=0,ba=void 0,X=0,aa=-1;Z<Y;Z++){ba=P[Z];if(ba==" "){if(!X&&
(!W||Z)){S.push(new CKEDITOR.dom.text(" "));aa=S.length}X=1}else{S.push(ba);X=0}}K&&aa==S.length&&S.pop();W=S}}if(W){for(;K=W.pop();)M.insertNode(K);W=0}else M.insertNode(L.node);if(L.lastNotAllowed&&E<I.length-1){(V=Q?F:V)&&M.setEndAt(V,CKEDITOR.POSITION_AFTER_START);q=0}M.collapse()}}l.dontMoveCaret=C;l.bogusNeededBlocks=H}R=l.range;var U;V=l.bogusNeededBlocks;for(W=R.createBookmark();N=l.zombies.pop();)if(N.getParent()){T=R.clone();T.moveToElementEditStart(N);T.removeEmptyBlocksAtEnd()}if(V)for(;N=
V.pop();)CKEDITOR.env.needsBrFiller?N.appendBogus():N.append(R.document.createText(" "));for(;N=l.mergeCandidates.pop();)N.mergeSiblings();R.moveToBookmark(W);if(!l.dontMoveCaret){for(N=a(R.startContainer)&&R.startContainer.getChild(R.startOffset-1);N&&a(N)&&!N.is(i.$empty);){if(N.isBlockBoundary())R.moveToPosition(N,CKEDITOR.POSITION_BEFORE_END);else{if(d(N)&&N.getHtml().match(/(\s|&nbsp;)$/g)){U=null;break}U=R.clone();U.moveToPosition(N,CKEDITOR.POSITION_BEFORE_END)}N=N.getLast(g)}U&&R.moveToRange(U)}o.select();
h(k)}}}(),l=function(){function a(b){b=new CKEDITOR.dom.walker(b);b.guard=function(a,b){if(b)return false;if(a.type==CKEDITOR.NODE_ELEMENT)return a.is(CKEDITOR.dtd.$tableContent)};b.evaluator=function(a){return a.type==CKEDITOR.NODE_ELEMENT};return b}function b(a,c,d){c=a.getDocument().createElement(c);a.append(c,d);return c}function c(a){var b=a.count(),d;for(b;b-- >0;){d=a.getItem(b);if(!CKEDITOR.tools.trim(d.getHtml())){d.appendBogus();CKEDITOR.env.ie&&(CKEDITOR.env.version<9&&d.getChildCount())&&
d.getFirst().remove()}}}return function(d){var f=d.startContainer,g=f.getAscendant("table",1),e=false;c(g.getElementsByTag("td"));c(g.getElementsByTag("th"));g=d.clone();g.setStart(f,0);g=a(g).lastBackward();if(!g){g=d.clone();g.setEndAt(f,CKEDITOR.POSITION_BEFORE_END);g=a(g).lastForward();e=true}g||(g=f);if(g.is("table")){d.setStartAt(g,CKEDITOR.POSITION_BEFORE_START);d.collapse(true);g.remove()}else{g.is({tbody:1,thead:1,tfoot:1})&&(g=b(g,"tr",e));g.is("tr")&&(g=b(g,g.getParent().is("thead")?"th":
"td",e));(f=g.getBogus())&&f.remove();d.moveToPosition(g,e?CKEDITOR.POSITION_AFTER_START:CKEDITOR.POSITION_BEFORE_END)}}}()}(),function(){function a(){var a=this._.fakeSelection,b;if(a){b=this.getSelection(1);if(!b||!b.isHidden()){a.reset();a=0}}if(!a){a=b||this.getSelection(1);if(!a||a.getType()==CKEDITOR.SELECTION_NONE)return}this.fire("selectionCheck",a);b=this.elementPath();if(!b.compare(this._.selectionPreviousPath)){if(CKEDITOR.env.webkit)this._.previousActive=this.document.getActive();this._.selectionPreviousPath=
b;this.fire("selectionChange",{selection:a,path:b})}}function e(){m=true;if(!q){b.call(this);q=CKEDITOR.tools.setTimeout(b,200,this)}}function b(){q=null;if(m){CKEDITOR.tools.setTimeout(a,0,this);m=false}}function g(a){return l(a)||a.type==CKEDITOR.NODE_ELEMENT&&!a.is(CKEDITOR.dtd.$empty)?true:false}function c(a){function b(c,d){return!c||c.type==CKEDITOR.NODE_TEXT?false:a.clone()["moveToElementEdit"+(d?"End":"Start")](c)}if(!(a.root instanceof CKEDITOR.editable))return false;var c=a.startContainer,
d=a.getPreviousNode(g,null,c),f=a.getNextNode(g,null,c);return b(d)||b(f,1)||!d&&!f&&!(c.type==CKEDITOR.NODE_ELEMENT&&c.isBlockBoundary()&&c.getBogus())?true:false}function d(a){return a.getCustomData("cke-fillingChar")}function j(a,b){var c=a&&a.removeCustomData("cke-fillingChar");if(c){if(b!==false){var d,g=a.getDocument().getSelection().getNative(),e=g&&g.type!="None"&&g.getRangeAt(0);if(c.getLength()>1&&e&&e.intersectsNode(c.$)){d=h(g);e=g.focusNode==c.$&&g.focusOffset>0;g.anchorNode==c.$&&g.anchorOffset>
0&&d[0].offset--;e&&d[1].offset--}}c.setText(f(c.getText()));d&&i(a.getDocument().$,d)}}function f(a){return a.replace(/\u200B( )?/g,function(a){return a[1]?" ":""})}function h(a){return[{node:a.anchorNode,offset:a.anchorOffset},{node:a.focusNode,offset:a.focusOffset}]}function i(a,b){var c=a.getSelection(),d=a.createRange();d.setStart(b[0].node,b[0].offset);d.collapse(true);c.removeAllRanges();c.addRange(d);c.extend(b[1].node,b[1].offset)}function k(a){var b=CKEDITOR.dom.element.createFromHtml('<div data-cke-hidden-sel="1" data-cke-temp="1" style="'+
(CKEDITOR.env.ie?"display:none":"position:fixed;top:0;left:-1000px")+'">&nbsp;</div>',a.document);a.fire("lockSnapshot");a.editable().append(b);var c=a.getSelection(1),d=a.createRange(),f=c.root.on("selectionchange",function(a){a.cancel()},null,null,0);d.setStartAt(b,CKEDITOR.POSITION_AFTER_START);d.setEndAt(b,CKEDITOR.POSITION_BEFORE_END);c.selectRanges([d]);f.removeListener();a.fire("unlockSnapshot");a._.hiddenSelectionContainer=b}function n(a){var b={37:1,39:1,8:1,46:1};return function(c){var d=
c.data.getKeystroke();if(b[d]){var f=a.getSelection().getRanges(),g=f[0];if(f.length==1&&g.collapsed)if((d=g[d<38?"getPreviousEditableNode":"getNextEditableNode"]())&&d.type==CKEDITOR.NODE_ELEMENT&&d.getAttribute("contenteditable")=="false"){a.getSelection().fake(d);c.data.preventDefault();c.cancel()}}}}function o(a){for(var b=0;b<a.length;b++){var c=a[b];c.getCommonAncestor().isReadOnly()&&a.splice(b,1);if(!c.collapsed){if(c.startContainer.isReadOnly())for(var d=c.startContainer,f;d;){if((f=d.type==
CKEDITOR.NODE_ELEMENT)&&d.is("body")||!d.isReadOnly())break;f&&d.getAttribute("contentEditable")=="false"&&c.setStartAfter(d);d=d.getParent()}d=c.startContainer;f=c.endContainer;var g=c.startOffset,e=c.endOffset,h=c.clone();d&&d.type==CKEDITOR.NODE_TEXT&&(g>=d.getLength()?h.setStartAfter(d):h.setStartBefore(d));f&&f.type==CKEDITOR.NODE_TEXT&&(e?h.setEndAfter(f):h.setEndBefore(f));d=new CKEDITOR.dom.walker(h);d.evaluator=function(d){if(d.type==CKEDITOR.NODE_ELEMENT&&d.isReadOnly()){var f=c.clone();
c.setEndBefore(d);c.collapsed&&a.splice(b--,1);if(!(d.getPosition(h.endContainer)&CKEDITOR.POSITION_CONTAINS)){f.setStartAfter(d);f.collapsed||a.splice(b+1,0,f)}return true}return false};d.next()}}return a}var q,m,l=CKEDITOR.dom.walker.invisible(1),p=function(){function a(b){return function(a){var c=a.editor.createRange();c.moveToClosestEditablePosition(a.selected,b)&&a.editor.getSelection().selectRanges([c]);return false}}function b(a){return function(b){var c=b.editor,d=c.createRange(),f;if(!(f=
d.moveToClosestEditablePosition(b.selected,a)))f=d.moveToClosestEditablePosition(b.selected,!a);f&&c.getSelection().selectRanges([d]);c.fire("saveSnapshot");b.selected.remove();if(!f){d.moveToElementEditablePosition(c.editable());c.getSelection().selectRanges([d])}c.fire("saveSnapshot");return false}}var c=a(),d=a(1);return{37:c,38:c,39:d,40:d,8:b(),46:b(1)}}();CKEDITOR.on("instanceCreated",function(b){function c(){var a=d.getSelection();a&&a.removeAllRanges()}var d=b.editor;d.on("contentDom",function(){function b(){p=
new CKEDITOR.dom.selection(d.getSelection());p.lock()}function c(){g.removeListener("mouseup",c);k.removeListener("mouseup",c);var a=CKEDITOR.document.$.selection,b=a.createRange();a.type!="None"&&b.parentElement().ownerDocument==f.$&&b.select()}var f=d.document,g=CKEDITOR.document,h=d.editable(),i=f.getBody(),k=f.getDocumentElement(),m=h.isInline(),C,p;CKEDITOR.env.gecko&&h.attachListener(h,"focus",function(a){a.removeListener();if(C!==0)if((a=d.getSelection().getNative())&&a.isCollapsed&&a.anchorNode==
h.$){a=d.createRange();a.moveToElementEditStart(h);a.select()}},null,null,-2);h.attachListener(h,CKEDITOR.env.webkit?"DOMFocusIn":"focus",function(){C&&CKEDITOR.env.webkit&&(C=d._.previousActive&&d._.previousActive.equals(f.getActive()));d.unlockSelection(C);C=0},null,null,-1);h.attachListener(h,"mousedown",function(){C=0});if(CKEDITOR.env.ie||m){t?h.attachListener(h,"beforedeactivate",b,null,null,-1):h.attachListener(d,"selectionCheck",b,null,null,-1);h.attachListener(h,CKEDITOR.env.webkit?"DOMFocusOut":
"blur",function(){d.lockSelection(p);C=1},null,null,-1);h.attachListener(h,"mousedown",function(){C=0})}if(CKEDITOR.env.ie&&!m){var s;h.attachListener(h,"mousedown",function(a){if(a.data.$.button==2){a=d.document.getSelection();if(!a||a.getType()==CKEDITOR.SELECTION_NONE)s=d.window.getScrollPosition()}});h.attachListener(h,"mouseup",function(a){if(a.data.$.button==2&&s){d.document.$.documentElement.scrollLeft=s.x;d.document.$.documentElement.scrollTop=s.y}s=null});if(f.$.compatMode!="BackCompat"){if(CKEDITOR.env.ie7Compat||
CKEDITOR.env.ie6Compat)k.on("mousedown",function(a){function b(a){a=a.data.$;if(d){var c=i.$.createTextRange();try{c.moveToPoint(a.clientX,a.clientY)}catch(f){}d.setEndPoint(e.compareEndPoints("StartToStart",c)<0?"EndToEnd":"StartToStart",c);d.select()}}function c(){k.removeListener("mousemove",b);g.removeListener("mouseup",c);k.removeListener("mouseup",c);d.select()}a=a.data;if(a.getTarget().is("html")&&a.$.y<k.$.clientHeight&&a.$.x<k.$.clientWidth){var d=i.$.createTextRange();try{d.moveToPoint(a.$.clientX,
a.$.clientY)}catch(f){}var e=d.duplicate();k.on("mousemove",b);g.on("mouseup",c);k.on("mouseup",c)}});if(CKEDITOR.env.version>7&&CKEDITOR.env.version<11)k.on("mousedown",function(a){if(a.data.getTarget().is("html")){g.on("mouseup",c);k.on("mouseup",c)}})}}h.attachListener(h,"selectionchange",a,d);h.attachListener(h,"keyup",e,d);h.attachListener(h,CKEDITOR.env.webkit?"DOMFocusIn":"focus",function(){d.forceNextSelectionCheck();d.selectionChange(1)});if(m&&(CKEDITOR.env.webkit||CKEDITOR.env.gecko)){var l;
h.attachListener(h,"mousedown",function(){l=1});h.attachListener(f.getDocumentElement(),"mouseup",function(){l&&e.call(d);l=0})}else h.attachListener(CKEDITOR.env.ie?h:f.getDocumentElement(),"mouseup",e,d);CKEDITOR.env.webkit&&h.attachListener(f,"keydown",function(a){switch(a.data.getKey()){case 13:case 33:case 34:case 35:case 36:case 37:case 39:case 8:case 45:case 46:j(h)}},null,null,-1);h.attachListener(h,"keydown",n(d),null,null,-1)});d.on("setData",function(){d.unlockSelection();CKEDITOR.env.webkit&&
c()});d.on("contentDomUnload",function(){d.unlockSelection()});if(CKEDITOR.env.ie9Compat)d.on("beforeDestroy",c,null,null,9);d.on("dataReady",function(){delete d._.fakeSelection;delete d._.hiddenSelectionContainer;d.selectionChange(1)});d.on("loadSnapshot",function(){var a=CKEDITOR.dom.walker.nodeType(CKEDITOR.NODE_ELEMENT),b=d.editable().getLast(a);if(b&&b.hasAttribute("data-cke-hidden-sel")){b.remove();if(CKEDITOR.env.gecko)(a=d.editable().getFirst(a))&&(a.is("br")&&a.getAttribute("_moz_editor_bogus_node"))&&
a.remove()}},null,null,100);d.on("key",function(a){if(d.mode=="wysiwyg"){var b=d.getSelection();if(b.isFake){var c=p[a.data.keyCode];if(c)return c({editor:d,selected:b.getSelectedElement(),selection:b,keyEvent:a})}}})});CKEDITOR.on("instanceReady",function(a){function b(){var a=g.editable();if(a)if(a=d(a)){var c=g.document.$.getSelection();if(c.type!="None"&&(c.anchorNode==a.$||c.focusNode==a.$))k=h(c);e=a.getText();a.setText(f(e))}}function c(){var a=g.editable();if(a)if(a=d(a)){a.setText(e);if(k){i(g.document.$,
k);k=null}}}var g=a.editor,e,k;if(CKEDITOR.env.webkit){g.on("selectionChange",function(){var a=g.editable(),b=d(a);b&&(b.getCustomData("ready")?j(a):b.setCustomData("ready",1))},null,null,-1);g.on("beforeSetMode",function(){j(g.editable())},null,null,-1);g.on("beforeUndoImage",b);g.on("afterUndoImage",c);g.on("beforeGetData",b,null,null,0);g.on("getData",c)}});CKEDITOR.editor.prototype.selectionChange=function(b){(b?a:e).call(this)};CKEDITOR.editor.prototype.getSelection=function(a){if((this._.savedSelection||
this._.fakeSelection)&&!a)return this._.savedSelection||this._.fakeSelection;return(a=this.editable())&&this.mode=="wysiwyg"?new CKEDITOR.dom.selection(a):null};CKEDITOR.editor.prototype.lockSelection=function(a){a=a||this.getSelection(1);if(a.getType()!=CKEDITOR.SELECTION_NONE){!a.isLocked&&a.lock();this._.savedSelection=a;return true}return false};CKEDITOR.editor.prototype.unlockSelection=function(a){var b=this._.savedSelection;if(b){b.unlock(a);delete this._.savedSelection;return true}return false};
CKEDITOR.editor.prototype.forceNextSelectionCheck=function(){delete this._.selectionPreviousPath};CKEDITOR.dom.document.prototype.getSelection=function(){return new CKEDITOR.dom.selection(this)};CKEDITOR.dom.range.prototype.select=function(){var a=this.root instanceof CKEDITOR.editable?this.root.editor.getSelection():new CKEDITOR.dom.selection(this.root);a.selectRanges([this]);return a};CKEDITOR.SELECTION_NONE=1;CKEDITOR.SELECTION_TEXT=2;CKEDITOR.SELECTION_ELEMENT=3;var t=typeof window.getSelection!=
"function",s=1;CKEDITOR.dom.selection=function(a){if(a instanceof CKEDITOR.dom.selection)var b=a,a=a.root;var c=a instanceof CKEDITOR.dom.element;this.rev=b?b.rev:s++;this.document=a instanceof CKEDITOR.dom.document?a:a.getDocument();this.root=c?a:this.document.getBody();this.isLocked=0;this._={cache:{}};if(b){CKEDITOR.tools.extend(this._.cache,b._.cache);this.isFake=b.isFake;this.isLocked=b.isLocked;return this}var a=this.getNative(),d,f;if(a)if(a.getRangeAt)d=(f=a.rangeCount&&a.getRangeAt(0))&&
new CKEDITOR.dom.node(f.commonAncestorContainer);else{try{f=a.createRange()}catch(g){}d=f&&CKEDITOR.dom.element.get(f.item&&f.item(0)||f.parentElement())}if(!d||!(d.type==CKEDITOR.NODE_ELEMENT||d.type==CKEDITOR.NODE_TEXT)||!this.root.equals(d)&&!this.root.contains(d)){this._.cache.type=CKEDITOR.SELECTION_NONE;this._.cache.startElement=null;this._.cache.selectedElement=null;this._.cache.selectedText="";this._.cache.ranges=new CKEDITOR.dom.rangeList}return this};var u={img:1,hr:1,li:1,table:1,tr:1,
td:1,th:1,embed:1,object:1,ol:1,ul:1,a:1,input:1,form:1,select:1,textarea:1,button:1,fieldset:1,thead:1,tfoot:1};CKEDITOR.dom.selection.prototype={getNative:function(){return this._.cache.nativeSel!==void 0?this._.cache.nativeSel:this._.cache.nativeSel=t?this.document.$.selection:this.document.getWindow().$.getSelection()},getType:t?function(){var a=this._.cache;if(a.type)return a.type;var b=CKEDITOR.SELECTION_NONE;try{var c=this.getNative(),d=c.type;if(d=="Text")b=CKEDITOR.SELECTION_TEXT;if(d=="Control")b=
CKEDITOR.SELECTION_ELEMENT;if(c.createRange().parentElement())b=CKEDITOR.SELECTION_TEXT}catch(f){}return a.type=b}:function(){var a=this._.cache;if(a.type)return a.type;var b=CKEDITOR.SELECTION_TEXT,c=this.getNative();if(!c||!c.rangeCount)b=CKEDITOR.SELECTION_NONE;else if(c.rangeCount==1){var c=c.getRangeAt(0),d=c.startContainer;if(d==c.endContainer&&d.nodeType==1&&c.endOffset-c.startOffset==1&&u[d.childNodes[c.startOffset].nodeName.toLowerCase()])b=CKEDITOR.SELECTION_ELEMENT}return a.type=b},getRanges:function(){var a=
t?function(){function a(b){return(new CKEDITOR.dom.node(b)).getIndex()}var b=function(b,c){b=b.duplicate();b.collapse(c);var d=b.parentElement();if(!d.hasChildNodes())return{container:d,offset:0};for(var f=d.children,g,e,h=b.duplicate(),j=0,i=f.length-1,k=-1,m,p;j<=i;){k=Math.floor((j+i)/2);g=f[k];h.moveToElementText(g);m=h.compareEndPoints("StartToStart",b);if(m>0)i=k-1;else if(m<0)j=k+1;else return{container:d,offset:a(g)}}if(k==-1||k==f.length-1&&m<0){h.moveToElementText(d);h.setEndPoint("StartToStart",
b);h=h.text.replace(/(\r\n|\r)/g,"\n").length;f=d.childNodes;if(!h){g=f[f.length-1];return g.nodeType!=CKEDITOR.NODE_TEXT?{container:d,offset:f.length}:{container:g,offset:g.nodeValue.length}}for(d=f.length;h>0&&d>0;){e=f[--d];if(e.nodeType==CKEDITOR.NODE_TEXT){p=e;h=h-e.nodeValue.length}}return{container:p,offset:-h}}h.collapse(m>0?true:false);h.setEndPoint(m>0?"StartToStart":"EndToStart",b);h=h.text.replace(/(\r\n|\r)/g,"\n").length;if(!h)return{container:d,offset:a(g)+(m>0?0:1)};for(;h>0;)try{e=
g[m>0?"previousSibling":"nextSibling"];if(e.nodeType==CKEDITOR.NODE_TEXT){h=h-e.nodeValue.length;p=e}g=e}catch(t){return{container:d,offset:a(g)}}return{container:p,offset:m>0?-h:p.nodeValue.length+h}};return function(){var a=this.getNative(),c=a&&a.createRange(),d=this.getType();if(!a)return[];if(d==CKEDITOR.SELECTION_TEXT){a=new CKEDITOR.dom.range(this.root);d=b(c,true);a.setStart(new CKEDITOR.dom.node(d.container),d.offset);d=b(c);a.setEnd(new CKEDITOR.dom.node(d.container),d.offset);a.endContainer.getPosition(a.startContainer)&
CKEDITOR.POSITION_PRECEDING&&a.endOffset<=a.startContainer.getIndex()&&a.collapse();return[a]}if(d==CKEDITOR.SELECTION_ELEMENT){for(var d=[],f=0;f<c.length;f++){for(var g=c.item(f),e=g.parentNode,h=0,a=new CKEDITOR.dom.range(this.root);h<e.childNodes.length&&e.childNodes[h]!=g;h++);a.setStart(new CKEDITOR.dom.node(e),h);a.setEnd(new CKEDITOR.dom.node(e),h+1);d.push(a)}return d}return[]}}():function(){var a=[],b,c=this.getNative();if(!c)return a;for(var d=0;d<c.rangeCount;d++){var f=c.getRangeAt(d);
b=new CKEDITOR.dom.range(this.root);b.setStart(new CKEDITOR.dom.node(f.startContainer),f.startOffset);b.setEnd(new CKEDITOR.dom.node(f.endContainer),f.endOffset);a.push(b)}return a};return function(b){var c=this._.cache,d=c.ranges;if(!d)c.ranges=d=new CKEDITOR.dom.rangeList(a.call(this));return!b?d:o(new CKEDITOR.dom.rangeList(d.slice()))}}(),getStartElement:function(){var a=this._.cache;if(a.startElement!==void 0)return a.startElement;var b;switch(this.getType()){case CKEDITOR.SELECTION_ELEMENT:return this.getSelectedElement();
case CKEDITOR.SELECTION_TEXT:var c=this.getRanges()[0];if(c){if(c.collapsed){b=c.startContainer;b.type!=CKEDITOR.NODE_ELEMENT&&(b=b.getParent())}else{for(c.optimize();;){b=c.startContainer;if(c.startOffset==(b.getChildCount?b.getChildCount():b.getLength())&&!b.isBlockBoundary())c.setStartAfter(b);else break}b=c.startContainer;if(b.type!=CKEDITOR.NODE_ELEMENT)return b.getParent();b=b.getChild(c.startOffset);if(!b||b.type!=CKEDITOR.NODE_ELEMENT)b=c.startContainer;else for(c=b.getFirst();c&&c.type==
CKEDITOR.NODE_ELEMENT;){b=c;c=c.getFirst()}}b=b.$}}return a.startElement=b?new CKEDITOR.dom.element(b):null},getSelectedElement:function(){var a=this._.cache;if(a.selectedElement!==void 0)return a.selectedElement;var b=this,c=CKEDITOR.tools.tryThese(function(){return b.getNative().createRange().item(0)},function(){for(var a=b.getRanges()[0].clone(),c,d,f=2;f&&(!(c=a.getEnclosedNode())||!(c.type==CKEDITOR.NODE_ELEMENT&&u[c.getName()]&&(d=c)));f--)a.shrink(CKEDITOR.SHRINK_ELEMENT);return d&&d.$});return a.selectedElement=
c?new CKEDITOR.dom.element(c):null},getSelectedText:function(){var a=this._.cache;if(a.selectedText!==void 0)return a.selectedText;var b=this.getNative(),b=t?b.type=="Control"?"":b.createRange().text:b.toString();return a.selectedText=b},lock:function(){this.getRanges();this.getStartElement();this.getSelectedElement();this.getSelectedText();this._.cache.nativeSel=null;this.isLocked=1},unlock:function(a){if(this.isLocked){if(a)var b=this.getSelectedElement(),c=!b&&this.getRanges(),d=this.isFake;this.isLocked=
0;this.reset();if(a)(a=b||c[0]&&c[0].getCommonAncestor())&&a.getAscendant("body",1)&&(d?this.fake(b):b?this.selectElement(b):this.selectRanges(c))}},reset:function(){this._.cache={};this.isFake=0;var a=this.root.editor;if(a&&a._.fakeSelection&&this.rev==a._.fakeSelection.rev){delete a._.fakeSelection;var b=a._.hiddenSelectionContainer;if(b){var c=a.checkDirty();a.fire("lockSnapshot");b.remove();a.fire("unlockSnapshot");!c&&a.resetDirty()}delete a._.hiddenSelectionContainer}this.rev=s++},selectElement:function(a){var b=
new CKEDITOR.dom.range(this.root);b.setStartBefore(a);b.setEndAfter(a);this.selectRanges([b])},selectRanges:function(a){var b=this.root.editor,b=b&&b._.hiddenSelectionContainer;this.reset();if(b)for(var b=this.root,d,f=0;f<a.length;++f){d=a[f];if(d.endContainer.equals(b))d.endOffset=Math.min(d.endOffset,b.getChildCount())}if(a.length)if(this.isLocked){var g=CKEDITOR.document.getActive();this.unlock();this.selectRanges(a);this.lock();g&&!g.equals(this.root)&&g.focus()}else{var e;a:{var h,i;if(a.length==
1&&!(i=a[0]).collapsed&&(e=i.getEnclosedNode())&&e.type==CKEDITOR.NODE_ELEMENT){i=i.clone();i.shrink(CKEDITOR.SHRINK_ELEMENT,true);if((h=i.getEnclosedNode())&&h.type==CKEDITOR.NODE_ELEMENT)e=h;if(e.getAttribute("contenteditable")=="false")break a}e=void 0}if(e)this.fake(e);else{if(t){i=CKEDITOR.dom.walker.whitespaces(true);h=/\ufeff|\u00a0/;b={table:1,tbody:1,tr:1};if(a.length>1){e=a[a.length-1];a[0].setEnd(e.endContainer,e.endOffset)}e=a[0];var a=e.collapsed,k,m,p;if((d=e.getEnclosedNode())&&d.type==
CKEDITOR.NODE_ELEMENT&&d.getName()in u&&(!d.is("a")||!d.getText()))try{p=d.$.createControlRange();p.addElement(d.$);p.select();return}catch(C){}if(e.startContainer.type==CKEDITOR.NODE_ELEMENT&&e.startContainer.getName()in b||e.endContainer.type==CKEDITOR.NODE_ELEMENT&&e.endContainer.getName()in b){e.shrink(CKEDITOR.NODE_ELEMENT,true);a=e.collapsed}p=e.createBookmark();b=p.startNode;if(!a)g=p.endNode;p=e.document.$.body.createTextRange();p.moveToElementText(b.$);p.moveStart("character",1);if(g){h=
e.document.$.body.createTextRange();h.moveToElementText(g.$);p.setEndPoint("EndToEnd",h);p.moveEnd("character",-1)}else{k=b.getNext(i);m=b.hasAscendant("pre");k=!(k&&k.getText&&k.getText().match(h))&&(m||!b.hasPrevious()||b.getPrevious().is&&b.getPrevious().is("br"));m=e.document.createElement("span");m.setHtml("&#65279;");m.insertBefore(b);k&&e.document.createText("﻿").insertBefore(b)}e.setStartBefore(b);b.remove();if(a){if(k){p.moveStart("character",-1);p.select();e.document.$.selection.clear()}else p.select();
e.moveToPosition(m,CKEDITOR.POSITION_BEFORE_START);m.remove()}else{e.setEndBefore(g);g.remove();p.select()}}else{g=this.getNative();if(!g)return;this.removeAllRanges();for(p=0;p<a.length;p++){if(p<a.length-1){k=a[p];m=a[p+1];h=k.clone();h.setStart(k.endContainer,k.endOffset);h.setEnd(m.startContainer,m.startOffset);if(!h.collapsed){h.shrink(CKEDITOR.NODE_ELEMENT,true);e=h.getCommonAncestor();h=h.getEnclosedNode();if(e.isReadOnly()||h&&h.isReadOnly()){m.setStart(k.startContainer,k.startOffset);a.splice(p--,
1);continue}}}e=a[p];m=this.document.$.createRange();if(e.collapsed&&CKEDITOR.env.webkit&&c(e)){k=this.root;j(k,false);h=k.getDocument().createText("​");k.setCustomData("cke-fillingChar",h);e.insertNode(h);if((k=h.getNext())&&!h.getPrevious()&&k.type==CKEDITOR.NODE_ELEMENT&&k.getName()=="br"){j(this.root);e.moveToPosition(k,CKEDITOR.POSITION_BEFORE_START)}else e.moveToPosition(h,CKEDITOR.POSITION_AFTER_END)}m.setStart(e.startContainer.$,e.startOffset);try{m.setEnd(e.endContainer.$,e.endOffset)}catch(n){if(n.toString().indexOf("NS_ERROR_ILLEGAL_VALUE")>=
0){e.collapse(1);m.setEnd(e.endContainer.$,e.endOffset)}else throw n;}g.addRange(m)}}this.reset();this.root.fire("selectionchange")}}},fake:function(a){var b=this.root.editor;this.reset();k(b);var c=this._.cache,d=new CKEDITOR.dom.range(this.root);d.setStartBefore(a);d.setEndAfter(a);c.ranges=new CKEDITOR.dom.rangeList(d);c.selectedElement=c.startElement=a;c.type=CKEDITOR.SELECTION_ELEMENT;c.selectedText=c.nativeSel=null;this.isFake=1;this.rev=s++;b._.fakeSelection=this;this.root.fire("selectionchange")},
isHidden:function(){var a=this.getCommonAncestor();a&&a.type==CKEDITOR.NODE_TEXT&&(a=a.getParent());return!(!a||!a.data("cke-hidden-sel"))},createBookmarks:function(a){a=this.getRanges().createBookmarks(a);this.isFake&&(a.isFake=1);return a},createBookmarks2:function(a){a=this.getRanges().createBookmarks2(a);this.isFake&&(a.isFake=1);return a},selectBookmarks:function(a){for(var b=[],c=0;c<a.length;c++){var d=new CKEDITOR.dom.range(this.root);d.moveToBookmark(a[c]);b.push(d)}a.isFake?this.fake(b[0].getEnclosedNode()):
this.selectRanges(b);return this},getCommonAncestor:function(){var a=this.getRanges();return!a.length?null:a[0].startContainer.getCommonAncestor(a[a.length-1].endContainer)},scrollIntoView:function(){this.type!=CKEDITOR.SELECTION_NONE&&this.getRanges()[0].scrollIntoView()},removeAllRanges:function(){if(this.getType()!=CKEDITOR.SELECTION_NONE){var a=this.getNative();try{a&&a[t?"empty":"removeAllRanges"]()}catch(b){}this.reset()}}}}(),"use strict",CKEDITOR.STYLE_BLOCK=1,CKEDITOR.STYLE_INLINE=2,CKEDITOR.STYLE_OBJECT=
3,function(){function a(a,b){for(var c,d;a=a.getParent();){if(a.equals(b))break;if(a.getAttribute("data-nostyle"))c=a;else if(!d){var f=a.getAttribute("contentEditable");f=="false"?c=a:f=="true"&&(d=1)}}return c}function e(b){var c=b.document;if(b.collapsed){c=p(this,c);b.insertNode(c);b.moveToPosition(c,CKEDITOR.POSITION_BEFORE_END)}else{var d=this.element,f=this._.definition,h,i=f.ignoreReadonly,j=i||f.includeReadonly;j==null&&(j=b.root.getCustomData("cke_includeReadonly"));var k=CKEDITOR.dtd[d];
if(!k){h=true;k=CKEDITOR.dtd.span}b.enlarge(CKEDITOR.ENLARGE_INLINE,1);b.trim();var m=b.createBookmark(),t=m.startNode,n=m.endNode,s=t,l;if(!i){var o=b.getCommonAncestor(),i=a(t,o),o=a(n,o);i&&(s=i.getNextSourceNode(true));o&&(n=o)}for(s.getPosition(n)==CKEDITOR.POSITION_FOLLOWING&&(s=0);s;){i=false;if(s.equals(n)){s=null;i=true}else{var u=s.type==CKEDITOR.NODE_ELEMENT?s.getName():null,o=u&&s.getAttribute("contentEditable")=="false",w=u&&s.getAttribute("data-nostyle");if(u&&s.data("cke-bookmark")){s=
s.getNextSourceNode(true);continue}if(o&&j&&CKEDITOR.dtd.$block[u])for(var x=s,r=g(x),v=void 0,z=r.length,y=0,x=z&&new CKEDITOR.dom.range(x.getDocument());y<z;++y){var v=r[y],A=CKEDITOR.filter.instances[v.data("cke-filter")];if(A?A.check(this):1){x.selectNodeContents(v);e.call(this,x)}}r=u?!k[u]||w?0:o&&!j?0:(s.getPosition(n)|G)==G&&(!f.childRule||f.childRule(s)):1;if(r)if((r=s.getParent())&&((r.getDtd()||CKEDITOR.dtd.span)[d]||h)&&(!f.parentRule||f.parentRule(r))){if(!l&&(!u||!CKEDITOR.dtd.$removeEmpty[u]||
(s.getPosition(n)|G)==G)){l=b.clone();l.setStartBefore(s)}u=s.type;if(u==CKEDITOR.NODE_TEXT||o||u==CKEDITOR.NODE_ELEMENT&&!s.getChildCount()){for(var u=s,aa;(i=!u.getNext(B))&&(aa=u.getParent(),k[aa.getName()])&&(aa.getPosition(t)|D)==D&&(!f.childRule||f.childRule(aa));)u=aa;l.setEndAfter(u)}}else i=true;else i=true;s=s.getNextSourceNode(w||o)}if(i&&l&&!l.collapsed){for(var i=p(this,c),o=i.hasAttributes(),w=l.getCommonAncestor(),u={},r={},v={},z={},U,$,ca;i&&w;){if(w.getName()==d){for(U in f.attributes)if(!z[U]&&
(ca=w.getAttribute($)))i.getAttribute(U)==ca?r[U]=1:z[U]=1;for($ in f.styles)if(!v[$]&&(ca=w.getStyle($)))i.getStyle($)==ca?u[$]=1:v[$]=1}w=w.getParent()}for(U in r)i.removeAttribute(U);for($ in u)i.removeStyle($);o&&!i.hasAttributes()&&(i=null);if(i){l.extractContents().appendTo(i);l.insertNode(i);q.call(this,i);i.mergeSiblings();CKEDITOR.env.ie||i.$.normalize()}else{i=new CKEDITOR.dom.element("span");l.extractContents().appendTo(i);l.insertNode(i);q.call(this,i);i.remove(true)}l=null}}b.moveToBookmark(m);
b.shrink(CKEDITOR.SHRINK_TEXT);b.shrink(CKEDITOR.NODE_ELEMENT,true)}}function b(a){function b(){for(var a=new CKEDITOR.dom.elementPath(d.getParent()),c=new CKEDITOR.dom.elementPath(j.getParent()),f=null,g=null,e=0;e<a.elements.length;e++){var h=a.elements[e];if(h==a.block||h==a.blockLimit)break;k.checkElementRemovable(h,true)&&(f=h)}for(e=0;e<c.elements.length;e++){h=c.elements[e];if(h==c.block||h==c.blockLimit)break;k.checkElementRemovable(h,true)&&(g=h)}g&&j.breakParent(g);f&&d.breakParent(f)}a.enlarge(CKEDITOR.ENLARGE_INLINE,
1);var c=a.createBookmark(),d=c.startNode;if(a.collapsed){for(var f=new CKEDITOR.dom.elementPath(d.getParent(),a.root),g,e=0,h;e<f.elements.length&&(h=f.elements[e]);e++){if(h==f.block||h==f.blockLimit)break;if(this.checkElementRemovable(h)){var i;if(a.collapsed&&(a.checkBoundaryOfElement(h,CKEDITOR.END)||(i=a.checkBoundaryOfElement(h,CKEDITOR.START)))){g=h;g.match=i?"start":"end"}else{h.mergeSiblings();h.is(this.element)?o.call(this,h):m(h,u(this)[h.getName()])}}}if(g){h=d;for(e=0;;e++){i=f.elements[e];
if(i.equals(g))break;else if(i.match)continue;else i=i.clone();i.append(h);h=i}h[g.match=="start"?"insertBefore":"insertAfter"](g)}}else{var j=c.endNode,k=this;b();for(f=d;!f.equals(j);){g=f.getNextSourceNode();if(f.type==CKEDITOR.NODE_ELEMENT&&this.checkElementRemovable(f)){f.getName()==this.element?o.call(this,f):m(f,u(this)[f.getName()]);if(g.type==CKEDITOR.NODE_ELEMENT&&g.contains(d)){b();g=d.getNext()}}f=g}}a.moveToBookmark(c);a.shrink(CKEDITOR.NODE_ELEMENT,true)}function g(a){var b=[];a.forEach(function(a){if(a.getAttribute("contenteditable")==
"true"){b.push(a);return false}},CKEDITOR.NODE_ELEMENT,true);return b}function c(a){var b=a.getEnclosedNode()||a.getCommonAncestor(false,true);(a=(new CKEDITOR.dom.elementPath(b,a.root)).contains(this.element,1))&&!a.isReadOnly()&&t(a,this)}function d(a){var b=a.getCommonAncestor(true,true);if(a=(new CKEDITOR.dom.elementPath(b,a.root)).contains(this.element,1)){var b=this._.definition,c=b.attributes;if(c)for(var d in c)a.removeAttribute(d,c[d]);if(b.styles)for(var f in b.styles)b.styles.hasOwnProperty(f)&&
a.removeStyle(f)}}function j(a){var b=a.createBookmark(true),c=a.createIterator();c.enforceRealBlocks=true;if(this._.enterMode)c.enlargeBr=this._.enterMode!=CKEDITOR.ENTER_BR;for(var d,f=a.document,g;d=c.getNextParagraph();)if(!d.isReadOnly()&&(c.activeFilter?c.activeFilter.check(this):1)){g=p(this,f,d);h(d,g)}a.moveToBookmark(b)}function f(a){var b=a.createBookmark(1),c=a.createIterator();c.enforceRealBlocks=true;c.enlargeBr=this._.enterMode!=CKEDITOR.ENTER_BR;for(var d,f;d=c.getNextParagraph();)if(this.checkElementRemovable(d))if(d.is("pre")){(f=
this._.enterMode==CKEDITOR.ENTER_BR?null:a.document.createElement(this._.enterMode==CKEDITOR.ENTER_P?"p":"div"))&&d.copyAttributes(f);h(d,f)}else o.call(this,d);a.moveToBookmark(b)}function h(a,b){var c=!b;if(c){b=a.getDocument().createElement("div");a.copyAttributes(b)}var d=b&&b.is("pre"),f=a.is("pre"),g=!d&&f;if(d&&!f){f=b;(g=a.getBogus())&&g.remove();g=a.getHtml();g=k(g,/(?:^[ \t\n\r]+)|(?:[ \t\n\r]+$)/g,"");g=g.replace(/[ \t\r\n]*(<br[^>]*>)[ \t\r\n]*/gi,"$1");g=g.replace(/([ \t\n\r]+|&nbsp;)/g,
" ");g=g.replace(/<br\b[^>]*>/gi,"\n");if(CKEDITOR.env.ie){var e=a.getDocument().createElement("div");e.append(f);f.$.outerHTML="<pre>"+g+"</pre>";f.copyAttributes(e.getFirst());f=e.getFirst().remove()}else f.setHtml(g);b=f}else g?b=n(c?[a.getHtml()]:i(a),b):a.moveChildren(b);b.replace(a);if(d){var c=b,h;if((h=c.getPrevious(A))&&h.type==CKEDITOR.NODE_ELEMENT&&h.is("pre")){d=k(h.getHtml(),/\n$/,"")+"\n\n"+k(c.getHtml(),/^\n/,"");CKEDITOR.env.ie?c.$.outerHTML="<pre>"+d+"</pre>":c.setHtml(d);h.remove()}}else c&&
l(b)}function i(a){var b=[];k(a.getOuterHtml(),/(\S\s*)\n(?:\s|(<span[^>]+data-cke-bookmark.*?\/span>))*\n(?!$)/gi,function(a,b,c){return b+"</pre>"+c+"<pre>"}).replace(/<pre\b.*?>([\s\S]*?)<\/pre>/gi,function(a,c){b.push(c)});return b}function k(a,b,c){var d="",f="",a=a.replace(/(^<span[^>]+data-cke-bookmark.*?\/span>)|(<span[^>]+data-cke-bookmark.*?\/span>$)/gi,function(a,b,c){b&&(d=b);c&&(f=c);return""});return d+a.replace(b,c)+f}function n(a,b){var c;a.length>1&&(c=new CKEDITOR.dom.documentFragment(b.getDocument()));
for(var d=0;d<a.length;d++){var f=a[d],f=f.replace(/(\r\n|\r)/g,"\n"),f=k(f,/^[ \t]*\n/,""),f=k(f,/\n$/,""),f=k(f,/^[ \t]+|[ \t]+$/g,function(a,b){return a.length==1?"&nbsp;":b?" "+CKEDITOR.tools.repeat("&nbsp;",a.length-1):CKEDITOR.tools.repeat("&nbsp;",a.length-1)+" "}),f=f.replace(/\n/g,"<br>"),f=f.replace(/[ \t]{2,}/g,function(a){return CKEDITOR.tools.repeat("&nbsp;",a.length-1)+" "});if(c){var g=b.clone();g.setHtml(f);c.append(g)}else b.setHtml(f)}return c||b}function o(a,b){var c=this._.definition,
d=c.attributes,c=c.styles,f=u(this)[a.getName()],g=CKEDITOR.tools.isEmpty(d)&&CKEDITOR.tools.isEmpty(c),e;for(e in d)if(!((e=="class"||this._.definition.fullMatch)&&a.getAttribute(e)!=w(e,d[e]))&&!(b&&e.slice(0,5)=="data-")){g=a.hasAttribute(e);a.removeAttribute(e)}for(var h in c)if(!(this._.definition.fullMatch&&a.getStyle(h)!=w(h,c[h],true))){g=g||!!a.getStyle(h);a.removeStyle(h)}m(a,f,v[a.getName()]);g&&(this._.definition.alwaysRemoveElement?l(a,1):!CKEDITOR.dtd.$block[a.getName()]||this._.enterMode==
CKEDITOR.ENTER_BR&&!a.hasAttributes()?l(a):a.renameNode(this._.enterMode==CKEDITOR.ENTER_P?"p":"div"))}function q(a){for(var b=u(this),c=a.getElementsByTag(this.element),d,f=c.count();--f>=0;){d=c.getItem(f);d.isReadOnly()||o.call(this,d,true)}for(var g in b)if(g!=this.element){c=a.getElementsByTag(g);for(f=c.count()-1;f>=0;f--){d=c.getItem(f);d.isReadOnly()||m(d,b[g])}}}function m(a,b,c){if(b=b&&b.attributes)for(var d=0;d<b.length;d++){var f=b[d][0],g;if(g=a.getAttribute(f)){var e=b[d][1];(e===null||
e.test&&e.test(g)||typeof e=="string"&&g==e)&&a.removeAttribute(f)}}c||l(a)}function l(a,b){if(!a.hasAttributes()||b)if(CKEDITOR.dtd.$block[a.getName()]){var c=a.getPrevious(A),d=a.getNext(A);c&&(c.type==CKEDITOR.NODE_TEXT||!c.isBlockBoundary({br:1}))&&a.append("br",1);d&&(d.type==CKEDITOR.NODE_TEXT||!d.isBlockBoundary({br:1}))&&a.append("br");a.remove(true)}else{c=a.getFirst();d=a.getLast();a.remove(true);if(c){c.type==CKEDITOR.NODE_ELEMENT&&c.mergeSiblings();d&&(!c.equals(d)&&d.type==CKEDITOR.NODE_ELEMENT)&&
d.mergeSiblings()}}}function p(a,b,c){var d;d=a.element;d=="*"&&(d="span");d=new CKEDITOR.dom.element(d,b);c&&c.copyAttributes(d);d=t(d,a);b.getCustomData("doc_processing_style")&&d.hasAttribute("id")?d.removeAttribute("id"):b.setCustomData("doc_processing_style",1);return d}function t(a,b){var c=b._.definition,d=c.attributes,c=CKEDITOR.style.getStyleText(c);if(d)for(var f in d)a.setAttribute(f,d[f]);c&&a.setAttribute("style",c);return a}function s(a,b){for(var c in a)a[c]=a[c].replace(z,function(a,
c){return b[c]})}function u(a){if(a._.overrides)return a._.overrides;var b=a._.overrides={},c=a._.definition.overrides;if(c){CKEDITOR.tools.isArray(c)||(c=[c]);for(var d=0;d<c.length;d++){var f=c[d],g,e;if(typeof f=="string")g=f.toLowerCase();else{g=f.element?f.element.toLowerCase():a.element;e=f.attributes}f=b[g]||(b[g]={});if(e){var f=f.attributes=f.attributes||[],h;for(h in e)f.push([h.toLowerCase(),e[h]])}}}return b}function w(a,b,c){var d=new CKEDITOR.dom.element("span");d[c?"setStyle":"setAttribute"](a,
b);return d[c?"getStyle":"getAttribute"](a)}function x(a,b,c){for(var d=a.document,f=a.getRanges(),b=b?this.removeFromRange:this.applyToRange,g,e=f.createIterator();g=e.getNextRange();)b.call(this,g,c);a.selectRanges(f);d.removeCustomData("doc_processing_style")}var v={address:1,div:1,h1:1,h2:1,h3:1,h4:1,h5:1,h6:1,p:1,pre:1,section:1,header:1,footer:1,nav:1,article:1,aside:1,figure:1,dialog:1,hgroup:1,time:1,meter:1,menu:1,command:1,keygen:1,output:1,progress:1,details:1,datagrid:1,datalist:1},r=
{a:1,blockquote:1,embed:1,hr:1,img:1,li:1,object:1,ol:1,table:1,td:1,tr:1,th:1,ul:1,dl:1,dt:1,dd:1,form:1,audio:1,video:1},y=/\s*(?:;\s*|$)/,z=/#\((.+?)\)/g,B=CKEDITOR.dom.walker.bookmark(0,1),A=CKEDITOR.dom.walker.whitespaces(1);CKEDITOR.style=function(a,b){if(typeof a.type=="string")return new CKEDITOR.style.customHandlers[a.type](a);var c=a.attributes;if(c&&c.style){a.styles=CKEDITOR.tools.extend({},a.styles,CKEDITOR.tools.parseCssText(c.style));delete c.style}if(b){a=CKEDITOR.tools.clone(a);s(a.attributes,
b);s(a.styles,b)}c=this.element=a.element?typeof a.element=="string"?a.element.toLowerCase():a.element:"*";this.type=a.type||(v[c]?CKEDITOR.STYLE_BLOCK:r[c]?CKEDITOR.STYLE_OBJECT:CKEDITOR.STYLE_INLINE);if(typeof this.element=="object")this.type=CKEDITOR.STYLE_OBJECT;this._={definition:a}};CKEDITOR.style.prototype={apply:function(a){if(a instanceof CKEDITOR.dom.document)return x.call(this,a.getSelection());if(this.checkApplicable(a.elementPath(),a)){var b=this._.enterMode;if(!b)this._.enterMode=a.activeEnterMode;
x.call(this,a.getSelection(),0,a);this._.enterMode=b}},remove:function(a){if(a instanceof CKEDITOR.dom.document)return x.call(this,a.getSelection(),1);if(this.checkApplicable(a.elementPath(),a)){var b=this._.enterMode;if(!b)this._.enterMode=a.activeEnterMode;x.call(this,a.getSelection(),1,a);this._.enterMode=b}},applyToRange:function(a){this.applyToRange=this.type==CKEDITOR.STYLE_INLINE?e:this.type==CKEDITOR.STYLE_BLOCK?j:this.type==CKEDITOR.STYLE_OBJECT?c:null;return this.applyToRange(a)},removeFromRange:function(a){this.removeFromRange=
this.type==CKEDITOR.STYLE_INLINE?b:this.type==CKEDITOR.STYLE_BLOCK?f:this.type==CKEDITOR.STYLE_OBJECT?d:null;return this.removeFromRange(a)},applyToObject:function(a){t(a,this)},checkActive:function(a,b){switch(this.type){case CKEDITOR.STYLE_BLOCK:return this.checkElementRemovable(a.block||a.blockLimit,true,b);case CKEDITOR.STYLE_OBJECT:case CKEDITOR.STYLE_INLINE:for(var c=a.elements,d=0,f;d<c.length;d++){f=c[d];if(!(this.type==CKEDITOR.STYLE_INLINE&&(f==a.block||f==a.blockLimit))){if(this.type==
CKEDITOR.STYLE_OBJECT){var g=f.getName();if(!(typeof this.element=="string"?g==this.element:g in this.element))continue}if(this.checkElementRemovable(f,true,b))return true}}}return false},checkApplicable:function(a,b,c){b&&b instanceof CKEDITOR.filter&&(c=b);if(c&&!c.check(this))return false;switch(this.type){case CKEDITOR.STYLE_OBJECT:return!!a.contains(this.element);case CKEDITOR.STYLE_BLOCK:return!!a.blockLimit.getDtd()[this.element]}return true},checkElementMatch:function(a,b){var c=this._.definition;
if(!a||!c.ignoreReadonly&&a.isReadOnly())return false;var d=a.getName();if(typeof this.element=="string"?d==this.element:d in this.element){if(!b&&!a.hasAttributes())return true;if(d=c._AC)c=d;else{var d={},f=0,g=c.attributes;if(g)for(var e in g){f++;d[e]=g[e]}if(e=CKEDITOR.style.getStyleText(c)){d.style||f++;d.style=e}d._length=f;c=c._AC=d}if(c._length){for(var h in c)if(h!="_length"){f=a.getAttribute(h)||"";if(h=="style")a:{d=c[h];typeof d=="string"&&(d=CKEDITOR.tools.parseCssText(d));typeof f==
"string"&&(f=CKEDITOR.tools.parseCssText(f,true));e=void 0;for(e in d)if(!(e in f&&(f[e]==d[e]||d[e]=="inherit"||f[e]=="inherit"))){d=false;break a}d=true}else d=c[h]==f;if(d){if(!b)return true}else if(b)return false}if(b)return true}else return true}return false},checkElementRemovable:function(a,b,c){if(this.checkElementMatch(a,b,c))return true;if(b=u(this)[a.getName()]){var d;if(!(b=b.attributes))return true;for(c=0;c<b.length;c++){d=b[c][0];if(d=a.getAttribute(d)){var f=b[c][1];if(f===null)return true;
if(typeof f=="string"){if(d==f)return true}else if(f.test(d))return true}}}return false},buildPreview:function(a){var b=this._.definition,c=[],d=b.element;d=="bdo"&&(d="span");var c=["<",d],f=b.attributes;if(f)for(var g in f)c.push(" ",g,'="',f[g],'"');(f=CKEDITOR.style.getStyleText(b))&&c.push(' style="',f,'"');c.push(">",a||b.name,"</",d,">");return c.join("")},getDefinition:function(){return this._.definition}};CKEDITOR.style.getStyleText=function(a){var b=a._ST;if(b)return b;var b=a.styles,c=
a.attributes&&a.attributes.style||"",d="";c.length&&(c=c.replace(y,";"));for(var f in b){var g=b[f],e=(f+":"+g).replace(y,";");g=="inherit"?d=d+e:c=c+e}c.length&&(c=CKEDITOR.tools.normalizeCssText(c,true));return a._ST=c+d};CKEDITOR.style.customHandlers={};CKEDITOR.style.addCustomHandler=function(a){var b=function(a){this._={definition:a};this.setup&&this.setup(a)};b.prototype=CKEDITOR.tools.extend(CKEDITOR.tools.prototypedCopy(CKEDITOR.style.prototype),{assignedTo:CKEDITOR.STYLE_OBJECT},a,true);
return this.customHandlers[a.type]=b};var G=CKEDITOR.POSITION_PRECEDING|CKEDITOR.POSITION_IDENTICAL|CKEDITOR.POSITION_IS_CONTAINED,D=CKEDITOR.POSITION_FOLLOWING|CKEDITOR.POSITION_IDENTICAL|CKEDITOR.POSITION_IS_CONTAINED}(),CKEDITOR.styleCommand=function(a,e){this.requiredContent=this.allowedContent=this.style=a;CKEDITOR.tools.extend(this,e,true)},CKEDITOR.styleCommand.prototype.exec=function(a){a.focus();this.state==CKEDITOR.TRISTATE_OFF?a.applyStyle(this.style):this.state==CKEDITOR.TRISTATE_ON&&
a.removeStyle(this.style)},CKEDITOR.stylesSet=new CKEDITOR.resourceManager("","stylesSet"),CKEDITOR.addStylesSet=CKEDITOR.tools.bind(CKEDITOR.stylesSet.add,CKEDITOR.stylesSet),CKEDITOR.loadStylesSet=function(a,e,b){CKEDITOR.stylesSet.addExternal(a,e,"");CKEDITOR.stylesSet.load(a,b)},CKEDITOR.tools.extend(CKEDITOR.editor.prototype,{attachStyleStateChange:function(a,e){var b=this._.styleStateChangeCallbacks;if(!b){b=this._.styleStateChangeCallbacks=[];this.on("selectionChange",function(a){for(var c=
0;c<b.length;c++){var d=b[c],e=d.style.checkActive(a.data.path,this)?CKEDITOR.TRISTATE_ON:CKEDITOR.TRISTATE_OFF;d.fn.call(this,e)}})}b.push({style:a,fn:e})},applyStyle:function(a){a.apply(this)},removeStyle:function(a){a.remove(this)},getStylesSet:function(a){if(this._.stylesDefinitions)a(this._.stylesDefinitions);else{var e=this,b=e.config.stylesCombo_stylesSet||e.config.stylesSet;if(b===false)a(null);else if(b instanceof Array){e._.stylesDefinitions=b;a(b)}else{b||(b="default");var b=b.split(":"),
g=b[0];CKEDITOR.stylesSet.addExternal(g,b[1]?b.slice(1).join(":"):CKEDITOR.getUrl("styles.js"),"");CKEDITOR.stylesSet.load(g,function(b){e._.stylesDefinitions=b[g];a(e._.stylesDefinitions)})}}}}),CKEDITOR.dom.comment=function(a,e){typeof a=="string"&&(a=(e?e.$:document).createComment(a));CKEDITOR.dom.domObject.call(this,a)},CKEDITOR.dom.comment.prototype=new CKEDITOR.dom.node,CKEDITOR.tools.extend(CKEDITOR.dom.comment.prototype,{type:CKEDITOR.NODE_COMMENT,getOuterHtml:function(){return"<\!--"+this.$.nodeValue+
"--\>"}}),"use strict",function(){var a={},e={},b;for(b in CKEDITOR.dtd.$blockLimit)b in CKEDITOR.dtd.$list||(a[b]=1);for(b in CKEDITOR.dtd.$block)b in CKEDITOR.dtd.$blockLimit||b in CKEDITOR.dtd.$empty||(e[b]=1);CKEDITOR.dom.elementPath=function(b,c){var d=null,j=null,f=[],h=b,i,c=c||b.getDocument().getBody();do if(h.type==CKEDITOR.NODE_ELEMENT){f.push(h);if(!this.lastElement){this.lastElement=h;if(h.is(CKEDITOR.dtd.$object)||h.getAttribute("contenteditable")=="false")continue}if(h.equals(c))break;
if(!j){i=h.getName();h.getAttribute("contenteditable")=="true"?j=h:!d&&e[i]&&(d=h);if(a[i]){var k;if(k=!d){if(i=i=="div"){a:{i=h.getChildren();k=0;for(var n=i.count();k<n;k++){var o=i.getItem(k);if(o.type==CKEDITOR.NODE_ELEMENT&&CKEDITOR.dtd.$block[o.getName()]){i=true;break a}}i=false}i=!i}k=i}k?d=h:j=h}}}while(h=h.getParent());j||(j=c);this.block=d;this.blockLimit=j;this.root=c;this.elements=f}}(),CKEDITOR.dom.elementPath.prototype={compare:function(a){var e=this.elements,a=a&&a.elements;if(!a||
e.length!=a.length)return false;for(var b=0;b<e.length;b++)if(!e[b].equals(a[b]))return false;return true},contains:function(a,e,b){var g;typeof a=="string"&&(g=function(b){return b.getName()==a});a instanceof CKEDITOR.dom.element?g=function(b){return b.equals(a)}:CKEDITOR.tools.isArray(a)?g=function(b){return CKEDITOR.tools.indexOf(a,b.getName())>-1}:typeof a=="function"?g=a:typeof a=="object"&&(g=function(b){return b.getName()in a});var c=this.elements,d=c.length;e&&d--;if(b){c=Array.prototype.slice.call(c,
0);c.reverse()}for(e=0;e<d;e++)if(g(c[e]))return c[e];return null},isContextFor:function(a){var e;if(a in CKEDITOR.dtd.$block){e=this.contains(CKEDITOR.dtd.$intermediate)||this.root.equals(this.block)&&this.block||this.blockLimit;return!!e.getDtd()[a]}return true},direction:function(){return(this.block||this.blockLimit||this.root).getDirection(1)}},CKEDITOR.dom.text=function(a,e){typeof a=="string"&&(a=(e?e.$:document).createTextNode(a));this.$=a},CKEDITOR.dom.text.prototype=new CKEDITOR.dom.node,
CKEDITOR.tools.extend(CKEDITOR.dom.text.prototype,{type:CKEDITOR.NODE_TEXT,getLength:function(){return this.$.nodeValue.length},getText:function(){return this.$.nodeValue},setText:function(a){this.$.nodeValue=a},split:function(a){var e=this.$.parentNode,b=e.childNodes.length,g=this.getLength(),c=this.getDocument(),d=new CKEDITOR.dom.text(this.$.splitText(a),c);if(e.childNodes.length==b)if(a>=g){d=c.createText("");d.insertAfter(this)}else{a=c.createText("");a.insertAfter(d);a.remove()}return d},substring:function(a,
e){return typeof e!="number"?this.$.nodeValue.substr(a):this.$.nodeValue.substring(a,e)}}),function(){function a(a,g,c){var d=a.serializable,e=g[c?"endContainer":"startContainer"],f=c?"endOffset":"startOffset",h=d?g.document.getById(a.startNode):a.startNode,a=d?g.document.getById(a.endNode):a.endNode;if(e.equals(h.getPrevious())){g.startOffset=g.startOffset-e.getLength()-a.getPrevious().getLength();e=a.getNext()}else if(e.equals(a.getPrevious())){g.startOffset=g.startOffset-e.getLength();e=a.getNext()}e.equals(h.getParent())&&
g[f]++;e.equals(a.getParent())&&g[f]++;g[c?"endContainer":"startContainer"]=e;return g}CKEDITOR.dom.rangeList=function(a){if(a instanceof CKEDITOR.dom.rangeList)return a;a?a instanceof CKEDITOR.dom.range&&(a=[a]):a=[];return CKEDITOR.tools.extend(a,e)};var e={createIterator:function(){var a=this,g=CKEDITOR.dom.walker.bookmark(),c=[],d;return{getNextRange:function(e){d=d===void 0?0:d+1;var f=a[d];if(f&&a.length>1){if(!d)for(var h=a.length-1;h>=0;h--)c.unshift(a[h].createBookmark(true));if(e)for(var i=
0;a[d+i+1];){for(var k=f.document,e=0,h=k.getById(c[i].endNode),k=k.getById(c[i+1].startNode);;){h=h.getNextSourceNode(false);if(k.equals(h))e=1;else if(g(h)||h.type==CKEDITOR.NODE_ELEMENT&&h.isBlockBoundary())continue;break}if(!e)break;i++}for(f.moveToBookmark(c.shift());i--;){h=a[++d];h.moveToBookmark(c.shift());f.setEnd(h.endContainer,h.endOffset)}}return f}}},createBookmarks:function(b){for(var g=[],c,d=0;d<this.length;d++){g.push(c=this[d].createBookmark(b,true));for(var e=d+1;e<this.length;e++){this[e]=
a(c,this[e]);this[e]=a(c,this[e],true)}}return g},createBookmarks2:function(a){for(var g=[],c=0;c<this.length;c++)g.push(this[c].createBookmark2(a));return g},moveToBookmarks:function(a){for(var g=0;g<this.length;g++)this[g].moveToBookmark(a[g])}}}(),function(){function a(){return CKEDITOR.getUrl(CKEDITOR.skinName.split(",")[1]||"skins/"+CKEDITOR.skinName.split(",")[0]+"/")}function e(b){var c=CKEDITOR.skin["ua_"+b],d=CKEDITOR.env;if(c)for(var c=c.split(",").sort(function(a,b){return a>b?-1:1}),f=
0,g;f<c.length;f++){g=c[f];if(d.ie&&(g.replace(/^ie/,"")==d.version||d.quirks&&g=="iequirks"))g="ie";if(d[g]){b=b+("_"+c[f]);break}}return CKEDITOR.getUrl(a()+b+".css")}function b(a,b){if(!d[a]){CKEDITOR.document.appendStyleSheet(e(a));d[a]=1}b&&b()}function g(a){var b=a.getById(j);if(!b){b=a.getHead().append("style");b.setAttribute("id",j);b.setAttribute("type","text/css")}return b}function c(a,b,c){var d,f,g;if(CKEDITOR.env.webkit){b=b.split("}").slice(0,-1);for(f=0;f<b.length;f++)b[f]=b[f].split("{")}for(var e=
0;e<a.length;e++)if(CKEDITOR.env.webkit)for(f=0;f<b.length;f++){g=b[f][1];for(d=0;d<c.length;d++)g=g.replace(c[d][0],c[d][1]);a[e].$.sheet.addRule(b[f][0],g)}else{g=b;for(d=0;d<c.length;d++)g=g.replace(c[d][0],c[d][1]);CKEDITOR.env.ie&&CKEDITOR.env.version<11?a[e].$.styleSheet.cssText=a[e].$.styleSheet.cssText+g:a[e].$.innerHTML=a[e].$.innerHTML+g}}var d={};CKEDITOR.skin={path:a,loadPart:function(c,d){CKEDITOR.skin.name!=CKEDITOR.skinName.split(",")[0]?CKEDITOR.scriptLoader.load(CKEDITOR.getUrl(a()+
"skin.js"),function(){b(c,d)}):b(c,d)},getPath:function(a){return CKEDITOR.getUrl(e(a))},icons:{},addIcon:function(a,b,c,d){a=a.toLowerCase();this.icons[a]||(this.icons[a]={path:b,offset:c||0,bgsize:d||"16px"})},getIconStyle:function(a,b,c,d,f){var g;if(a){a=a.toLowerCase();b&&(g=this.icons[a+"-rtl"]);g||(g=this.icons[a])}a=c||g&&g.path||"";d=d||g&&g.offset;f=f||g&&g.bgsize||"16px";return a&&"background-image:url("+CKEDITOR.getUrl(a)+");background-position:0 "+d+"px;background-size:"+f+";"}};CKEDITOR.tools.extend(CKEDITOR.editor.prototype,
{getUiColor:function(){return this.uiColor},setUiColor:function(a){var b=g(CKEDITOR.document);return(this.setUiColor=function(a){this.uiColor=a;var d=CKEDITOR.skin.chameleon,g="",e="";if(typeof d=="function"){g=d(this,"editor");e=d(this,"panel")}a=[[h,a]];c([b],g,a);c(f,e,a)}).call(this,a)}});var j="cke_ui_color",f=[],h=/\$color/g;CKEDITOR.on("instanceLoaded",function(a){if(!CKEDITOR.env.ie||!CKEDITOR.env.quirks){var b=a.editor,a=function(a){a=(a.data[0]||a.data).element.getElementsByTag("iframe").getItem(0).getFrameDocument();
if(!a.getById("cke_ui_color")){a=g(a);f.push(a);var d=b.getUiColor();d&&c([a],CKEDITOR.skin.chameleon(b,"panel"),[[h,d]])}};b.on("panelShow",a);b.on("menuShow",a);b.config.uiColor&&b.setUiColor(b.config.uiColor)}})}(),function(){if(CKEDITOR.env.webkit)CKEDITOR.env.hc=false;else{var a=CKEDITOR.dom.element.createFromHtml('<div style="width:0;height:0;position:absolute;left:-10000px;border:1px solid;border-color:red blue"></div>',CKEDITOR.document);a.appendTo(CKEDITOR.document.getHead());try{var e=a.getComputedStyle("border-top-color"),
b=a.getComputedStyle("border-right-color");CKEDITOR.env.hc=!!(e&&e==b)}catch(g){CKEDITOR.env.hc=false}a.remove()}if(CKEDITOR.env.hc)CKEDITOR.env.cssClass=CKEDITOR.env.cssClass+" cke_hc";CKEDITOR.document.appendStyleText(".cke{visibility:hidden;}");CKEDITOR.status="loaded";CKEDITOR.fireOnce("loaded");if(a=CKEDITOR._.pending){delete CKEDITOR._.pending;for(e=0;e<a.length;e++){CKEDITOR.editor.prototype.constructor.apply(a[e][0],a[e][1]);CKEDITOR.add(a[e][0])}}}(),CKEDITOR.skin.name="moono",CKEDITOR.skin.ua_editor=
"ie,iequirks,ie7,ie8,gecko",CKEDITOR.skin.ua_dialog="ie,iequirks,ie7,ie8",CKEDITOR.skin.chameleon=function(){var a=function(){return function(a,b){for(var d=a.match(/[^#]./g),e=0;e<3;e++){var f=d,h=e,i;i=parseInt(d[e],16);i=("0"+(b<0?0|i*(1+b):0|i+(255-i)*b).toString(16)).slice(-2);f[h]=i}return"#"+d.join("")}}(),e=function(){var a=new CKEDITOR.template("background:#{to};background-image:-webkit-gradient(linear,lefttop,leftbottom,from({from}),to({to}));background-image:-moz-linear-gradient(top,{from},{to});background-image:-webkit-linear-gradient(top,{from},{to});background-image:-o-linear-gradient(top,{from},{to});background-image:-ms-linear-gradient(top,{from},{to});background-image:linear-gradient(top,{from},{to});filter:progid:DXImageTransform.Microsoft.gradient(gradientType=0,startColorstr='{from}',endColorstr='{to}');");
return function(b,d){return a.output({from:b,to:d})}}(),b={editor:new CKEDITOR.template("{id}.cke_chrome [border-color:{defaultBorder};] {id} .cke_top [ {defaultGradient}border-bottom-color:{defaultBorder};] {id} .cke_bottom [{defaultGradient}border-top-color:{defaultBorder};] {id} .cke_resizer [border-right-color:{ckeResizer}] {id} .cke_dialog_title [{defaultGradient}border-bottom-color:{defaultBorder};] {id} .cke_dialog_footer [{defaultGradient}outline-color:{defaultBorder};border-top-color:{defaultBorder};] {id} .cke_dialog_tab [{lightGradient}border-color:{defaultBorder};] {id} .cke_dialog_tab:hover [{mediumGradient}] {id} .cke_dialog_contents [border-top-color:{defaultBorder};] {id} .cke_dialog_tab_selected, {id} .cke_dialog_tab_selected:hover [background:{dialogTabSelected};border-bottom-color:{dialogTabSelectedBorder};] {id} .cke_dialog_body [background:{dialogBody};border-color:{defaultBorder};] {id} .cke_toolgroup [{lightGradient}border-color:{defaultBorder};] {id} a.cke_button_off:hover, {id} a.cke_button_off:focus, {id} a.cke_button_off:active [{mediumGradient}] {id} .cke_button_on [{ckeButtonOn}] {id} .cke_toolbar_separator [background-color: {ckeToolbarSeparator};] {id} .cke_combo_button [border-color:{defaultBorder};{lightGradient}] {id} a.cke_combo_button:hover, {id} a.cke_combo_button:focus, {id} .cke_combo_on a.cke_combo_button [border-color:{defaultBorder};{mediumGradient}] {id} .cke_path_item [color:{elementsPathColor};] {id} a.cke_path_item:hover, {id} a.cke_path_item:focus, {id} a.cke_path_item:active [background-color:{elementsPathBg};] {id}.cke_panel [border-color:{defaultBorder};] "),
panel:new CKEDITOR.template(".cke_panel_grouptitle [{lightGradient}border-color:{defaultBorder};] .cke_menubutton_icon [background-color:{menubuttonIcon};] .cke_menubutton:hover .cke_menubutton_icon, .cke_menubutton:focus .cke_menubutton_icon, .cke_menubutton:active .cke_menubutton_icon [background-color:{menubuttonIconHover};] .cke_menuseparator [background-color:{menubuttonIcon};] a:hover.cke_colorbox, a:focus.cke_colorbox, a:active.cke_colorbox [border-color:{defaultBorder};] a:hover.cke_colorauto, a:hover.cke_colormore, a:focus.cke_colorauto, a:focus.cke_colormore, a:active.cke_colorauto, a:active.cke_colormore [background-color:{ckeColorauto};border-color:{defaultBorder};] ")};
return function(g,c){var d=g.uiColor,d={id:"."+g.id,defaultBorder:a(d,-0.1),defaultGradient:e(a(d,0.9),d),lightGradient:e(a(d,1),a(d,0.7)),mediumGradient:e(a(d,0.8),a(d,0.5)),ckeButtonOn:e(a(d,0.6),a(d,0.7)),ckeResizer:a(d,-0.4),ckeToolbarSeparator:a(d,0.5),ckeColorauto:a(d,0.8),dialogBody:a(d,0.7),dialogTabSelected:e("#FFFFFF","#FFFFFF"),dialogTabSelectedBorder:"#FFF",elementsPathColor:a(d,-0.6),elementsPathBg:d,menubuttonIcon:a(d,0.5),menubuttonIconHover:a(d,0.3)};return b[c].output(d).replace(/\[/g,
"{").replace(/\]/g,"}")}}(),CKEDITOR.plugins.add("dialogui",{onLoad:function(){var a=function(a){this._||(this._={});this._["default"]=this._.initValue=a["default"]||"";this._.required=a.required||false;for(var b=[this._],c=1;c<arguments.length;c++)b.push(arguments[c]);b.push(true);CKEDITOR.tools.extend.apply(CKEDITOR.tools,b);return this._},e={build:function(a,b,c){return new CKEDITOR.ui.dialog.textInput(a,b,c)}},b={build:function(a,b,c){return new CKEDITOR.ui.dialog[b.type](a,b,c)}},g={isChanged:function(){return this.getValue()!=
this.getInitValue()},reset:function(a){this.setValue(this.getInitValue(),a)},setInitValue:function(){this._.initValue=this.getValue()},resetInitValue:function(){this._.initValue=this._["default"]},getInitValue:function(){return this._.initValue}},c=CKEDITOR.tools.extend({},CKEDITOR.ui.dialog.uiElement.prototype.eventProcessors,{onChange:function(a,b){if(!this._.domOnChangeRegistered){a.on("load",function(){this.getInputElement().on("change",function(){a.parts.dialog.isVisible()&&this.fire("change",
{value:this.getValue()})},this)},this);this._.domOnChangeRegistered=true}this.on("change",b)}},true),d=/^on([A-Z]\w+)/,j=function(a){for(var b in a)(d.test(b)||b=="title"||b=="type")&&delete a[b];return a};CKEDITOR.tools.extend(CKEDITOR.ui.dialog,{labeledElement:function(b,c,d,g){if(!(arguments.length<4)){var e=a.call(this,c);e.labelId=CKEDITOR.tools.getNextId()+"_label";this._.children=[];var j={role:c.role||"presentation"};if(c.includeLabel)j["aria-labelledby"]=e.labelId;CKEDITOR.ui.dialog.uiElement.call(this,
b,c,d,"div",null,j,function(){var a=[],d=c.required?" cke_required":"";if(c.labelLayout!="horizontal")a.push('<label class="cke_dialog_ui_labeled_label'+d+'" ',' id="'+e.labelId+'"',e.inputId?' for="'+e.inputId+'"':"",(c.labelStyle?' style="'+c.labelStyle+'"':"")+">",c.label,"</label>",'<div class="cke_dialog_ui_labeled_content"',c.controlStyle?' style="'+c.controlStyle+'"':"",' role="presentation">',g.call(this,b,c),"</div>");else{d={type:"hbox",widths:c.widths,padding:0,children:[{type:"html",html:'<label class="cke_dialog_ui_labeled_label'+
d+'" id="'+e.labelId+'" for="'+e.inputId+'"'+(c.labelStyle?' style="'+c.labelStyle+'"':"")+">"+CKEDITOR.tools.htmlEncode(c.label)+"</span>"},{type:"html",html:'<span class="cke_dialog_ui_labeled_content"'+(c.controlStyle?' style="'+c.controlStyle+'"':"")+">"+g.call(this,b,c)+"</span>"}]};CKEDITOR.dialog._.uiElementBuilders.hbox.build(b,d,a)}return a.join("")})}},textInput:function(b,c,d){if(!(arguments.length<3)){a.call(this,c);var g=this._.inputId=CKEDITOR.tools.getNextId()+"_textInput",e={"class":"cke_dialog_ui_input_"+
c.type,id:g,type:c.type};if(c.validate)this.validate=c.validate;if(c.maxLength)e.maxlength=c.maxLength;if(c.size)e.size=c.size;if(c.inputStyle)e.style=c.inputStyle;var j=this,q=false;b.on("load",function(){j.getInputElement().on("keydown",function(a){a.data.getKeystroke()==13&&(q=true)});j.getInputElement().on("keyup",function(a){if(a.data.getKeystroke()==13&&q){b.getButton("ok")&&setTimeout(function(){b.getButton("ok").click()},0);q=false}},null,null,1E3)});CKEDITOR.ui.dialog.labeledElement.call(this,
b,c,d,function(){var a=['<div class="cke_dialog_ui_input_',c.type,'" role="presentation"'];c.width&&a.push('style="width:'+c.width+'" ');a.push("><input ");e["aria-labelledby"]=this._.labelId;this._.required&&(e["aria-required"]=this._.required);for(var b in e)a.push(b+'="'+e[b]+'" ');a.push(" /></div>");return a.join("")})}},textarea:function(b,c,d){if(!(arguments.length<3)){a.call(this,c);var g=this,e=this._.inputId=CKEDITOR.tools.getNextId()+"_textarea",j={};if(c.validate)this.validate=c.validate;
j.rows=c.rows||5;j.cols=c.cols||20;j["class"]="cke_dialog_ui_input_textarea "+(c["class"]||"");if(typeof c.inputStyle!="undefined")j.style=c.inputStyle;if(c.dir)j.dir=c.dir;CKEDITOR.ui.dialog.labeledElement.call(this,b,c,d,function(){j["aria-labelledby"]=this._.labelId;this._.required&&(j["aria-required"]=this._.required);var a=['<div class="cke_dialog_ui_input_textarea" role="presentation"><textarea id="',e,'" '],b;for(b in j)a.push(b+'="'+CKEDITOR.tools.htmlEncode(j[b])+'" ');a.push(">",CKEDITOR.tools.htmlEncode(g._["default"]),
"</textarea></div>");return a.join("")})}},checkbox:function(b,c,d){if(!(arguments.length<3)){var g=a.call(this,c,{"default":!!c["default"]});if(c.validate)this.validate=c.validate;CKEDITOR.ui.dialog.uiElement.call(this,b,c,d,"span",null,null,function(){var a=CKEDITOR.tools.extend({},c,{id:c.id?c.id+"_checkbox":CKEDITOR.tools.getNextId()+"_checkbox"},true),d=[],e=CKEDITOR.tools.getNextId()+"_label",i={"class":"cke_dialog_ui_checkbox_input",type:"checkbox","aria-labelledby":e};j(a);if(c["default"])i.checked=
"checked";if(typeof a.inputStyle!="undefined")a.style=a.inputStyle;g.checkbox=new CKEDITOR.ui.dialog.uiElement(b,a,d,"input",null,i);d.push(' <label id="',e,'" for="',i.id,'"'+(c.labelStyle?' style="'+c.labelStyle+'"':"")+">",CKEDITOR.tools.htmlEncode(c.label),"</label>");return d.join("")})}},radio:function(b,c,d){if(!(arguments.length<3)){a.call(this,c);if(!this._["default"])this._["default"]=this._.initValue=c.items[0][1];if(c.validate)this.validate=c.valdiate;var g=[],e=this;c.role="radiogroup";
c.includeLabel=true;CKEDITOR.ui.dialog.labeledElement.call(this,b,c,d,function(){for(var a=[],d=[],i=(c.id?c.id:CKEDITOR.tools.getNextId())+"_radio",l=0;l<c.items.length;l++){var p=c.items[l],t=p[2]!==void 0?p[2]:p[0],s=p[1]!==void 0?p[1]:p[0],u=CKEDITOR.tools.getNextId()+"_radio_input",w=u+"_label",u=CKEDITOR.tools.extend({},c,{id:u,title:null,type:null},true),t=CKEDITOR.tools.extend({},u,{title:t},true),x={type:"radio","class":"cke_dialog_ui_radio_input",name:i,value:s,"aria-labelledby":w},v=[];
if(e._["default"]==s)x.checked="checked";j(u);j(t);if(typeof u.inputStyle!="undefined")u.style=u.inputStyle;u.keyboardFocusable=true;g.push(new CKEDITOR.ui.dialog.uiElement(b,u,v,"input",null,x));v.push(" ");new CKEDITOR.ui.dialog.uiElement(b,t,v,"label",null,{id:w,"for":x.id},p[0]);a.push(v.join(""))}new CKEDITOR.ui.dialog.hbox(b,g,a,d);return d.join("")});this._.children=g}},button:function(b,c,d){if(arguments.length){typeof c=="function"&&(c=c(b.getParentEditor()));a.call(this,c,{disabled:c.disabled||
false});CKEDITOR.event.implementOn(this);var g=this;b.on("load",function(){var a=this.getElement();(function(){a.on("click",function(a){g.click();a.data.preventDefault()});a.on("keydown",function(a){if(a.data.getKeystroke()in{32:1}){g.click();a.data.preventDefault()}})})();a.unselectable()},this);var e=CKEDITOR.tools.extend({},c);delete e.style;var j=CKEDITOR.tools.getNextId()+"_label";CKEDITOR.ui.dialog.uiElement.call(this,b,e,d,"a",null,{style:c.style,href:"javascript:void(0)",title:c.label,hidefocus:"true",
"class":c["class"],role:"button","aria-labelledby":j},'<span id="'+j+'" class="cke_dialog_ui_button">'+CKEDITOR.tools.htmlEncode(c.label)+"</span>")}},select:function(b,c,d){if(!(arguments.length<3)){var g=a.call(this,c);if(c.validate)this.validate=c.validate;g.inputId=CKEDITOR.tools.getNextId()+"_select";CKEDITOR.ui.dialog.labeledElement.call(this,b,c,d,function(){var a=CKEDITOR.tools.extend({},c,{id:c.id?c.id+"_select":CKEDITOR.tools.getNextId()+"_select"},true),d=[],e=[],i={id:g.inputId,"class":"cke_dialog_ui_input_select",
"aria-labelledby":this._.labelId};d.push('<div class="cke_dialog_ui_input_',c.type,'" role="presentation"');c.width&&d.push('style="width:'+c.width+'" ');d.push(">");if(c.size!==void 0)i.size=c.size;if(c.multiple!==void 0)i.multiple=c.multiple;j(a);for(var l=0,p;l<c.items.length&&(p=c.items[l]);l++)e.push('<option value="',CKEDITOR.tools.htmlEncode(p[1]!==void 0?p[1]:p[0]).replace(/"/g,"&quot;"),'" /> ',CKEDITOR.tools.htmlEncode(p[0]));if(typeof a.inputStyle!="undefined")a.style=a.inputStyle;g.select=
new CKEDITOR.ui.dialog.uiElement(b,a,d,"select",null,i,e.join(""));d.push("</div>");return d.join("")})}},file:function(b,c,d){if(!(arguments.length<3)){c["default"]===void 0&&(c["default"]="");var g=CKEDITOR.tools.extend(a.call(this,c),{definition:c,buttons:[]});if(c.validate)this.validate=c.validate;b.on("load",function(){CKEDITOR.document.getById(g.frameId).getParent().addClass("cke_dialog_ui_input_file")});CKEDITOR.ui.dialog.labeledElement.call(this,b,c,d,function(){g.frameId=CKEDITOR.tools.getNextId()+
"_fileInput";var a=['<iframe frameborder="0" allowtransparency="0" class="cke_dialog_ui_input_file" role="presentation" id="',g.frameId,'" title="',c.label,'" src="javascript:void('];a.push(CKEDITOR.env.ie?"(function(){"+encodeURIComponent("document.open();("+CKEDITOR.tools.fixDomain+")();document.close();")+"})()":"0");a.push(')"></iframe>');return a.join("")})}},fileButton:function(b,c,d){var g=this;if(!(arguments.length<3)){a.call(this,c);if(c.validate)this.validate=c.validate;var e=CKEDITOR.tools.extend({},
c),j=e.onClick;e.className=(e.className?e.className+" ":"")+"cke_dialog_ui_button";e.onClick=function(a){var d=c["for"];if(!j||j.call(this,a)!==false){b.getContentElement(d[0],d[1]).submit();this.disable()}};b.on("load",function(){b.getContentElement(c["for"][0],c["for"][1])._.buttons.push(g)});CKEDITOR.ui.dialog.button.call(this,b,e,d)}},html:function(){var a=/^\s*<[\w:]+\s+([^>]*)?>/,b=/^(\s*<[\w:]+(?:\s+[^>]*)?)((?:.|\r|\n)+)$/,c=/\/$/;return function(d,g,e){if(!(arguments.length<3)){var j=[],
m=g.html;m.charAt(0)!="<"&&(m="<span>"+m+"</span>");var l=g.focus;if(l){var p=this.focus;this.focus=function(){(typeof l=="function"?l:p).call(this);this.fire("focus")};if(g.isFocusable)this.isFocusable=this.isFocusable;this.keyboardFocusable=true}CKEDITOR.ui.dialog.uiElement.call(this,d,g,j,"span",null,null,"");j=j.join("").match(a);m=m.match(b)||["","",""];if(c.test(m[1])){m[1]=m[1].slice(0,-1);m[2]="/"+m[2]}e.push([m[1]," ",j[1]||"",m[2]].join(""))}}}(),fieldset:function(a,b,c,d,g){var e=g.label;
this._={children:b};CKEDITOR.ui.dialog.uiElement.call(this,a,g,d,"fieldset",null,null,function(){var a=[];e&&a.push("<legend"+(g.labelStyle?' style="'+g.labelStyle+'"':"")+">"+e+"</legend>");for(var b=0;b<c.length;b++)a.push(c[b]);return a.join("")})}},true);CKEDITOR.ui.dialog.html.prototype=new CKEDITOR.ui.dialog.uiElement;CKEDITOR.ui.dialog.labeledElement.prototype=CKEDITOR.tools.extend(new CKEDITOR.ui.dialog.uiElement,{setLabel:function(a){var b=CKEDITOR.document.getById(this._.labelId);b.getChildCount()<
1?(new CKEDITOR.dom.text(a,CKEDITOR.document)).appendTo(b):b.getChild(0).$.nodeValue=a;return this},getLabel:function(){var a=CKEDITOR.document.getById(this._.labelId);return!a||a.getChildCount()<1?"":a.getChild(0).getText()},eventProcessors:c},true);CKEDITOR.ui.dialog.button.prototype=CKEDITOR.tools.extend(new CKEDITOR.ui.dialog.uiElement,{click:function(){return!this._.disabled?this.fire("click",{dialog:this._.dialog}):false},enable:function(){this._.disabled=false;var a=this.getElement();a&&a.removeClass("cke_disabled")},
disable:function(){this._.disabled=true;this.getElement().addClass("cke_disabled")},isVisible:function(){return this.getElement().getFirst().isVisible()},isEnabled:function(){return!this._.disabled},eventProcessors:CKEDITOR.tools.extend({},CKEDITOR.ui.dialog.uiElement.prototype.eventProcessors,{onClick:function(a,b){this.on("click",function(){b.apply(this,arguments)})}},true),accessKeyUp:function(){this.click()},accessKeyDown:function(){this.focus()},keyboardFocusable:true},true);CKEDITOR.ui.dialog.textInput.prototype=
CKEDITOR.tools.extend(new CKEDITOR.ui.dialog.labeledElement,{getInputElement:function(){return CKEDITOR.document.getById(this._.inputId)},focus:function(){var a=this.selectParentTab();setTimeout(function(){var b=a.getInputElement();b&&b.$.focus()},0)},select:function(){var a=this.selectParentTab();setTimeout(function(){var b=a.getInputElement();if(b){b.$.focus();b.$.select()}},0)},accessKeyUp:function(){this.select()},setValue:function(a){!a&&(a="");return CKEDITOR.ui.dialog.uiElement.prototype.setValue.apply(this,
arguments)},keyboardFocusable:true},g,true);CKEDITOR.ui.dialog.textarea.prototype=new CKEDITOR.ui.dialog.textInput;CKEDITOR.ui.dialog.select.prototype=CKEDITOR.tools.extend(new CKEDITOR.ui.dialog.labeledElement,{getInputElement:function(){return this._.select.getElement()},add:function(a,b,c){var d=new CKEDITOR.dom.element("option",this.getDialog().getParentEditor().document),g=this.getInputElement().$;d.$.text=a;d.$.value=b===void 0||b===null?a:b;c===void 0||c===null?CKEDITOR.env.ie?g.add(d.$):g.add(d.$,
null):g.add(d.$,c);return this},remove:function(a){this.getInputElement().$.remove(a);return this},clear:function(){for(var a=this.getInputElement().$;a.length>0;)a.remove(0);return this},keyboardFocusable:true},g,true);CKEDITOR.ui.dialog.checkbox.prototype=CKEDITOR.tools.extend(new CKEDITOR.ui.dialog.uiElement,{getInputElement:function(){return this._.checkbox.getElement()},setValue:function(a,b){this.getInputElement().$.checked=a;!b&&this.fire("change",{value:a})},getValue:function(){return this.getInputElement().$.checked},
accessKeyUp:function(){this.setValue(!this.getValue())},eventProcessors:{onChange:function(a,b){if(!CKEDITOR.env.ie||CKEDITOR.env.version>8)return c.onChange.apply(this,arguments);a.on("load",function(){var a=this._.checkbox.getElement();a.on("propertychange",function(b){b=b.data.$;b.propertyName=="checked"&&this.fire("change",{value:a.$.checked})},this)},this);this.on("change",b);return null}},keyboardFocusable:true},g,true);CKEDITOR.ui.dialog.radio.prototype=CKEDITOR.tools.extend(new CKEDITOR.ui.dialog.uiElement,
{setValue:function(a,b){for(var c=this._.children,d,g=0;g<c.length&&(d=c[g]);g++)d.getElement().$.checked=d.getValue()==a;!b&&this.fire("change",{value:a})},getValue:function(){for(var a=this._.children,b=0;b<a.length;b++)if(a[b].getElement().$.checked)return a[b].getValue();return null},accessKeyUp:function(){var a=this._.children,b;for(b=0;b<a.length;b++)if(a[b].getElement().$.checked){a[b].getElement().focus();return}a[0].getElement().focus()},eventProcessors:{onChange:function(a,b){if(CKEDITOR.env.ie){a.on("load",
function(){for(var a=this._.children,b=this,c=0;c<a.length;c++)a[c].getElement().on("propertychange",function(a){a=a.data.$;a.propertyName=="checked"&&this.$.checked&&b.fire("change",{value:this.getAttribute("value")})})},this);this.on("change",b)}else return c.onChange.apply(this,arguments);return null}}},g,true);CKEDITOR.ui.dialog.file.prototype=CKEDITOR.tools.extend(new CKEDITOR.ui.dialog.labeledElement,g,{getInputElement:function(){var a=CKEDITOR.document.getById(this._.frameId).getFrameDocument();
return a.$.forms.length>0?new CKEDITOR.dom.element(a.$.forms[0].elements[0]):this.getElement()},submit:function(){this.getInputElement().getParent().$.submit();return this},getAction:function(){return this.getInputElement().getParent().$.action},registerEvents:function(a){var b=/^on([A-Z]\w+)/,c,d=function(a,b,c,d){a.on("formLoaded",function(){a.getInputElement().on(c,d,a)})},g;for(g in a)if(c=g.match(b))this.eventProcessors[g]?this.eventProcessors[g].call(this,this._.dialog,a[g]):d(this,this._.dialog,
c[1].toLowerCase(),a[g]);return this},reset:function(){function a(){c.$.open();var f="";d.size&&(f=d.size-(CKEDITOR.env.ie?7:0));var t=b.frameId+"_input";c.$.write(['<html dir="'+m+'" lang="'+l+'"><head><title></title></head><body style="margin: 0; overflow: hidden; background: transparent;">','<form enctype="multipart/form-data" method="POST" dir="'+m+'" lang="'+l+'" action="',CKEDITOR.tools.htmlEncode(d.action),'"><label id="',b.labelId,'" for="',t,'" style="display:none">',CKEDITOR.tools.htmlEncode(d.label),
'</label><input style="width:100%" id="',t,'" aria-labelledby="',b.labelId,'" type="file" name="',CKEDITOR.tools.htmlEncode(d.id||"cke_upload"),'" size="',CKEDITOR.tools.htmlEncode(f>0?f:""),'" /></form></body></html><script>',CKEDITOR.env.ie?"("+CKEDITOR.tools.fixDomain+")();":"","window.parent.CKEDITOR.tools.callFunction("+e+");","window.onbeforeunload = function() {window.parent.CKEDITOR.tools.callFunction("+j+")}","<\/script>"].join(""));c.$.close();for(f=0;f<g.length;f++)g[f].enable()}var b=
this._,c=CKEDITOR.document.getById(b.frameId).getFrameDocument(),d=b.definition,g=b.buttons,e=this.formLoadedNumber,j=this.formUnloadNumber,m=b.dialog._.editor.lang.dir,l=b.dialog._.editor.langCode;if(!e){e=this.formLoadedNumber=CKEDITOR.tools.addFunction(function(){this.fire("formLoaded")},this);j=this.formUnloadNumber=CKEDITOR.tools.addFunction(function(){this.getInputElement().clearCustomData()},this);this.getDialog()._.editor.on("destroy",function(){CKEDITOR.tools.removeFunction(e);CKEDITOR.tools.removeFunction(j)})}CKEDITOR.env.gecko?
setTimeout(a,500):a()},getValue:function(){return this.getInputElement().$.value||""},setInitValue:function(){this._.initValue=""},eventProcessors:{onChange:function(a,b){if(!this._.domOnChangeRegistered){this.on("formLoaded",function(){this.getInputElement().on("change",function(){this.fire("change",{value:this.getValue()})},this)},this);this._.domOnChangeRegistered=true}this.on("change",b)}},keyboardFocusable:true},true);CKEDITOR.ui.dialog.fileButton.prototype=new CKEDITOR.ui.dialog.button;CKEDITOR.ui.dialog.fieldset.prototype=
CKEDITOR.tools.clone(CKEDITOR.ui.dialog.hbox.prototype);CKEDITOR.dialog.addUIElement("text",e);CKEDITOR.dialog.addUIElement("password",e);CKEDITOR.dialog.addUIElement("textarea",b);CKEDITOR.dialog.addUIElement("checkbox",b);CKEDITOR.dialog.addUIElement("radio",b);CKEDITOR.dialog.addUIElement("button",b);CKEDITOR.dialog.addUIElement("select",b);CKEDITOR.dialog.addUIElement("file",b);CKEDITOR.dialog.addUIElement("fileButton",b);CKEDITOR.dialog.addUIElement("html",b);CKEDITOR.dialog.addUIElement("fieldset",
{build:function(a,b,c){for(var d=b.children,g,e=[],j=[],m=0;m<d.length&&(g=d[m]);m++){var l=[];e.push(l);j.push(CKEDITOR.dialog._.uiElementBuilders[g.type].build(a,g,l))}return new CKEDITOR.ui.dialog[b.type](a,j,e,c,b)}})}}),CKEDITOR.DIALOG_RESIZE_NONE=0,CKEDITOR.DIALOG_RESIZE_WIDTH=1,CKEDITOR.DIALOG_RESIZE_HEIGHT=2,CKEDITOR.DIALOG_RESIZE_BOTH=3,function(){function a(){for(var a=this._.tabIdList.length,b=CKEDITOR.tools.indexOf(this._.tabIdList,this._.currentTabId)+a,c=b-1;c>b-a;c--)if(this._.tabs[this._.tabIdList[c%
a]][0].$.offsetHeight)return this._.tabIdList[c%a];return null}function e(){for(var a=this._.tabIdList.length,b=CKEDITOR.tools.indexOf(this._.tabIdList,this._.currentTabId),c=b+1;c<b+a;c++)if(this._.tabs[this._.tabIdList[c%a]][0].$.offsetHeight)return this._.tabIdList[c%a];return null}function b(a,b){for(var c=a.$.getElementsByTagName("input"),d=0,f=c.length;d<f;d++){var g=new CKEDITOR.dom.element(c[d]);if(g.getAttribute("type").toLowerCase()=="text")if(b){g.setAttribute("value",g.getCustomData("fake_value")||
"");g.removeCustomData("fake_value")}else{g.setCustomData("fake_value",g.getAttribute("value"));g.setAttribute("value","")}}}function g(a,b){var c=this.getInputElement();c&&(a?c.removeAttribute("aria-invalid"):c.setAttribute("aria-invalid",true));a||(this.select?this.select():this.focus());b&&alert(b);this.fire("validated",{valid:a,msg:b})}function c(){var a=this.getInputElement();a&&a.removeAttribute("aria-invalid")}function d(a){var a=CKEDITOR.dom.element.createFromHtml(CKEDITOR.addTemplate("dialog",
l).output({id:CKEDITOR.tools.getNextNumber(),editorId:a.id,langDir:a.lang.dir,langCode:a.langCode,editorDialogClass:"cke_editor_"+a.name.replace(/\./g,"\\.")+"_dialog",closeTitle:a.lang.common.close,hidpi:CKEDITOR.env.hidpi?"cke_hidpi":""})),b=a.getChild([0,0,0,0,0]),c=b.getChild(0),d=b.getChild(1);if(CKEDITOR.env.ie&&!CKEDITOR.env.quirks){var f="javascript:void(function(){"+encodeURIComponent("document.open();("+CKEDITOR.tools.fixDomain+")();document.close();")+"}())";CKEDITOR.dom.element.createFromHtml('<iframe frameBorder="0" class="cke_iframe_shim" src="'+
f+'" tabIndex="-1"></iframe>').appendTo(b.getParent())}c.unselectable();d.unselectable();return{element:a,parts:{dialog:a.getChild(0),title:c,close:d,tabs:b.getChild(2),contents:b.getChild([3,0,0,0]),footer:b.getChild([3,0,1,0])}}}function j(a,b,c){this.element=b;this.focusIndex=c;this.tabIndex=0;this.isFocusable=function(){return!b.getAttribute("disabled")&&b.isVisible()};this.focus=function(){a._.currentFocusIndex=this.focusIndex;this.element.focus()};b.on("keydown",function(a){a.data.getKeystroke()in
{32:1,13:1}&&this.fire("click")});b.on("focus",function(){this.fire("mouseover")});b.on("blur",function(){this.fire("mouseout")})}function f(a){function b(){a.layout()}var c=CKEDITOR.document.getWindow();c.on("resize",b);a.on("hide",function(){c.removeListener("resize",b)})}function h(a,b){this._={dialog:a};CKEDITOR.tools.extend(this,b)}function i(a){function b(c){var j=a.getSize(),i=CKEDITOR.document.getWindow().getViewPaneSize(),k=c.data.$.screenX,p=c.data.$.screenY,s=k-d.x,t=p-d.y;d={x:k,y:p};
f.x=f.x+s;f.y=f.y+t;a.move(f.x+h[3]<e?-h[3]:f.x-h[1]>i.width-j.width-e?i.width-j.width+(g.lang.dir=="rtl"?0:h[1]):f.x,f.y+h[0]<e?-h[0]:f.y-h[2]>i.height-j.height-e?i.height-j.height+h[2]:f.y,1);c.data.preventDefault()}function c(){CKEDITOR.document.removeListener("mousemove",b);CKEDITOR.document.removeListener("mouseup",c);if(CKEDITOR.env.ie6Compat){var a=r.getChild(0).getFrameDocument();a.removeListener("mousemove",b);a.removeListener("mouseup",c)}}var d=null,f=null,g=a.getParentEditor(),e=g.config.dialog_magnetDistance,
h=CKEDITOR.skin.margins||[0,0,0,0];typeof e=="undefined"&&(e=20);a.parts.title.on("mousedown",function(g){d={x:g.data.$.screenX,y:g.data.$.screenY};CKEDITOR.document.on("mousemove",b);CKEDITOR.document.on("mouseup",c);f=a.getPosition();if(CKEDITOR.env.ie6Compat){var e=r.getChild(0).getFrameDocument();e.on("mousemove",b);e.on("mouseup",c)}g.data.preventDefault()},a)}function k(a){var b,c;function d(f){var s=h.lang.dir=="rtl",t=p.width,m=p.height,l=t+(f.data.$.screenX-b)*(s?-1:1)*(a._.moved?1:2),n=
m+(f.data.$.screenY-c)*(a._.moved?1:2),u=a._.element.getFirst(),u=s&&u.getComputedStyle("right"),q=a.getPosition();q.y+n>k.height&&(n=k.height-q.y);if((s?u:q.x)+l>k.width)l=k.width-(s?u:q.x);if(e==CKEDITOR.DIALOG_RESIZE_WIDTH||e==CKEDITOR.DIALOG_RESIZE_BOTH)t=Math.max(g.minWidth||0,l-j);if(e==CKEDITOR.DIALOG_RESIZE_HEIGHT||e==CKEDITOR.DIALOG_RESIZE_BOTH)m=Math.max(g.minHeight||0,n-i);a.resize(t,m);a._.moved||a.layout();f.data.preventDefault()}function f(){CKEDITOR.document.removeListener("mouseup",
f);CKEDITOR.document.removeListener("mousemove",d);if(s){s.remove();s=null}if(CKEDITOR.env.ie6Compat){var a=r.getChild(0).getFrameDocument();a.removeListener("mouseup",f);a.removeListener("mousemove",d)}}var g=a.definition,e=g.resizable;if(e!=CKEDITOR.DIALOG_RESIZE_NONE){var h=a.getParentEditor(),j,i,k,p,s,t=CKEDITOR.tools.addFunction(function(g){p=a.getSize();var e=a.parts.contents;if(e.$.getElementsByTagName("iframe").length){s=CKEDITOR.dom.element.createFromHtml('<div class="cke_dialog_resize_cover" style="height: 100%; position: absolute; width: 100%;"></div>');
e.append(s)}i=p.height-a.parts.contents.getSize("height",!(CKEDITOR.env.gecko||CKEDITOR.env.ie&&CKEDITOR.env.quirks));j=p.width-a.parts.contents.getSize("width",1);b=g.screenX;c=g.screenY;k=CKEDITOR.document.getWindow().getViewPaneSize();CKEDITOR.document.on("mousemove",d);CKEDITOR.document.on("mouseup",f);if(CKEDITOR.env.ie6Compat){e=r.getChild(0).getFrameDocument();e.on("mousemove",d);e.on("mouseup",f)}g.preventDefault&&g.preventDefault()});a.on("load",function(){var b="";e==CKEDITOR.DIALOG_RESIZE_WIDTH?
b=" cke_resizer_horizontal":e==CKEDITOR.DIALOG_RESIZE_HEIGHT&&(b=" cke_resizer_vertical");b=CKEDITOR.dom.element.createFromHtml('<div class="cke_resizer'+b+" cke_resizer_"+h.lang.dir+'" title="'+CKEDITOR.tools.htmlEncode(h.lang.common.resize)+'" onmousedown="CKEDITOR.tools.callFunction('+t+', event )">'+(h.lang.dir=="ltr"?"◢":"◣")+"</div>");a.parts.footer.append(b,1)});h.on("destroy",function(){CKEDITOR.tools.removeFunction(t)})}}function n(a){a.data.preventDefault(1)}function o(a){var b=CKEDITOR.document.getWindow(),
c=a.config,d=c.dialog_backgroundCoverColor||"white",f=c.dialog_backgroundCoverOpacity,g=c.baseFloatZIndex,c=CKEDITOR.tools.genKey(d,f,g),e=v[c];if(e)e.show();else{g=['<div tabIndex="-1" style="position: ',CKEDITOR.env.ie6Compat?"absolute":"fixed","; z-index: ",g,"; top: 0px; left: 0px; ",!CKEDITOR.env.ie6Compat?"background-color: "+d:"",'" class="cke_dialog_background_cover">'];if(CKEDITOR.env.ie6Compat){d="<html><body style=\\'background-color:"+d+";\\'></body></html>";g.push('<iframe hidefocus="true" frameborder="0" id="cke_dialog_background_iframe" src="javascript:');
g.push("void((function(){"+encodeURIComponent("document.open();("+CKEDITOR.tools.fixDomain+")();document.write( '"+d+"' );document.close();")+"})())");g.push('" style="position:absolute;left:0;top:0;width:100%;height: 100%;filter: progid:DXImageTransform.Microsoft.Alpha(opacity=0)"></iframe>')}g.push("</div>");e=CKEDITOR.dom.element.createFromHtml(g.join(""));e.setOpacity(f!==void 0?f:0.5);e.on("keydown",n);e.on("keypress",n);e.on("keyup",n);e.appendTo(CKEDITOR.document.getBody());v[c]=e}a.focusManager.add(e);
r=e;var a=function(){var a=b.getViewPaneSize();e.setStyles({width:a.width+"px",height:a.height+"px"})},h=function(){var a=b.getScrollPosition(),c=CKEDITOR.dialog._.currentTop;e.setStyles({left:a.x+"px",top:a.y+"px"});if(c){do{a=c.getPosition();c.move(a.x,a.y)}while(c=c._.parentDialog)}};x=a;b.on("resize",a);a();(!CKEDITOR.env.mac||!CKEDITOR.env.webkit)&&e.focus();if(CKEDITOR.env.ie6Compat){var j=function(){h();arguments.callee.prevScrollHandler.apply(this,arguments)};b.$.setTimeout(function(){j.prevScrollHandler=
window.onscroll||function(){};window.onscroll=j},0);h()}}function q(a){if(r){a.focusManager.remove(r);a=CKEDITOR.document.getWindow();r.hide();a.removeListener("resize",x);CKEDITOR.env.ie6Compat&&a.$.setTimeout(function(){window.onscroll=window.onscroll&&window.onscroll.prevScrollHandler||null},0);x=null}}var m=CKEDITOR.tools.cssLength,l='<div class="cke_reset_all {editorId} {editorDialogClass} {hidpi}" dir="{langDir}" lang="{langCode}" role="dialog" aria-labelledby="cke_dialog_title_{id}"><table class="cke_dialog '+
CKEDITOR.env.cssClass+' cke_{langDir}" style="position:absolute" role="presentation"><tr><td role="presentation"><div class="cke_dialog_body" role="presentation"><div id="cke_dialog_title_{id}" class="cke_dialog_title" role="presentation"></div><a id="cke_dialog_close_button_{id}" class="cke_dialog_close_button" href="javascript:void(0)" title="{closeTitle}" role="button"><span class="cke_label">X</span></a><div id="cke_dialog_tabs_{id}" class="cke_dialog_tabs" role="tablist"></div><table class="cke_dialog_contents" role="presentation"><tr><td id="cke_dialog_contents_{id}" class="cke_dialog_contents_body" role="presentation"></td></tr><tr><td id="cke_dialog_footer_{id}" class="cke_dialog_footer" role="presentation"></td></tr></table></div></td></tr></table></div>';
CKEDITOR.dialog=function(b,f){function h(){var a=v._.focusList;a.sort(function(a,b){return a.tabIndex!=b.tabIndex?b.tabIndex-a.tabIndex:a.focusIndex-b.focusIndex});for(var b=a.length,c=0;c<b;c++)a[c].focusIndex=c}function j(a){var b=v._.focusList,a=a||0;if(!(b.length<1)){var c=v._.currentFocusIndex;try{b[c].getInputElement().$.blur()}catch(d){}for(var f=c=(c+a+b.length)%b.length;a&&!b[f].isFocusable();){f=(f+a+b.length)%b.length;if(f==c)break}b[f].focus();b[f].type=="text"&&b[f].select()}}function s(c){if(v==
CKEDITOR.dialog._.currentTop){var d=c.data.getKeystroke(),f=b.lang.dir=="rtl";o=x=0;if(d==9||d==CKEDITOR.SHIFT+9){d=d==CKEDITOR.SHIFT+9;if(v._.tabBarMode){d=d?a.call(v):e.call(v);v.selectPage(d);v._.tabs[d][0].focus()}else j(d?-1:1);o=1}else if(d==CKEDITOR.ALT+121&&!v._.tabBarMode&&v.getPageCount()>1){v._.tabBarMode=true;v._.tabs[v._.currentTabId][0].focus();o=1}else if((d==37||d==39)&&v._.tabBarMode){d=d==(f?39:37)?a.call(v):e.call(v);v.selectPage(d);v._.tabs[d][0].focus();o=1}else if((d==13||d==
32)&&v._.tabBarMode){this.selectPage(this._.currentTabId);this._.tabBarMode=false;this._.currentFocusIndex=-1;j(1);o=1}else if(d==13){d=c.data.getTarget();if(!d.is("a","button","select","textarea")&&(!d.is("input")||d.$.type!="button")){(d=this.getButton("ok"))&&CKEDITOR.tools.setTimeout(d.click,0,d);o=1}x=1}else if(d==27){(d=this.getButton("cancel"))?CKEDITOR.tools.setTimeout(d.click,0,d):this.fire("cancel",{hide:true}).hide!==false&&this.hide();x=1}else return;t(c)}}function t(a){o?a.data.preventDefault(1):
x&&a.data.stopPropagation()}var m=CKEDITOR.dialog._.dialogDefinitions[f],l=CKEDITOR.tools.clone(p),n=b.config.dialog_buttonsOrder||"OS",u=b.lang.dir,q={},o,x;(n=="OS"&&CKEDITOR.env.mac||n=="rtl"&&u=="ltr"||n=="ltr"&&u=="rtl")&&l.buttons.reverse();m=CKEDITOR.tools.extend(m(b),l);m=CKEDITOR.tools.clone(m);m=new w(this,m);l=d(b);this._={editor:b,element:l.element,name:f,contentSize:{width:0,height:0},size:{width:0,height:0},contents:{},buttons:{},accessKeyMap:{},tabs:{},tabIdList:[],currentTabId:null,
currentTabIndex:null,pageCount:0,lastTab:null,tabBarMode:false,focusList:[],currentFocusIndex:0,hasFocus:false};this.parts=l.parts;CKEDITOR.tools.setTimeout(function(){b.fire("ariaWidget",this.parts.contents)},0,this);l={position:CKEDITOR.env.ie6Compat?"absolute":"fixed",top:0,visibility:"hidden"};l[u=="rtl"?"right":"left"]=0;this.parts.dialog.setStyles(l);CKEDITOR.event.call(this);this.definition=m=CKEDITOR.fire("dialogDefinition",{name:f,definition:m},b).definition;if(!("removeDialogTabs"in b._)&&
b.config.removeDialogTabs){l=b.config.removeDialogTabs.split(";");for(u=0;u<l.length;u++){n=l[u].split(":");if(n.length==2){var r=n[0];q[r]||(q[r]=[]);q[r].push(n[1])}}b._.removeDialogTabs=q}if(b._.removeDialogTabs&&(q=b._.removeDialogTabs[f]))for(u=0;u<q.length;u++)m.removeContents(q[u]);if(m.onLoad)this.on("load",m.onLoad);if(m.onShow)this.on("show",m.onShow);if(m.onHide)this.on("hide",m.onHide);if(m.onOk)this.on("ok",function(a){b.fire("saveSnapshot");setTimeout(function(){b.fire("saveSnapshot")},
0);if(m.onOk.call(this,a)===false)a.data.hide=false});if(m.onCancel)this.on("cancel",function(a){if(m.onCancel.call(this,a)===false)a.data.hide=false});var v=this,z=function(a){var b=v._.contents,c=false,d;for(d in b)for(var f in b[d])if(c=a.call(this,b[d][f]))return};this.on("ok",function(a){z(function(b){if(b.validate){var c=b.validate(this),d=typeof c=="string"||c===false;if(d){a.data.hide=false;a.stop()}g.call(b,!d,typeof c=="string"?c:void 0);return d}})},this,null,0);this.on("cancel",function(a){z(function(c){if(c.isChanged()){if(!b.config.dialog_noConfirmCancel&&
!confirm(b.lang.common.confirmCancel))a.data.hide=false;return true}})},this,null,0);this.parts.close.on("click",function(a){this.fire("cancel",{hide:true}).hide!==false&&this.hide();a.data.preventDefault()},this);this.changeFocus=j;var y=this._.element;b.focusManager.add(y,1);this.on("show",function(){y.on("keydown",s,this);if(CKEDITOR.env.gecko)y.on("keypress",t,this)});this.on("hide",function(){y.removeListener("keydown",s);CKEDITOR.env.gecko&&y.removeListener("keypress",t);z(function(a){c.apply(a)})});
this.on("iframeAdded",function(a){(new CKEDITOR.dom.document(a.data.iframe.$.contentWindow.document)).on("keydown",s,this,null,0)});this.on("show",function(){h();if(b.config.dialog_startupFocusTab&&v._.pageCount>1){v._.tabBarMode=true;v._.tabs[v._.currentTabId][0].focus()}else if(!this._.hasFocus){this._.currentFocusIndex=-1;if(m.onFocus){var a=m.onFocus.call(this);a&&a.focus()}else j(1)}},this,null,4294967295);if(CKEDITOR.env.ie6Compat)this.on("load",function(){var a=this.getElement(),b=a.getFirst();
b.remove();b.appendTo(a)},this);i(this);k(this);(new CKEDITOR.dom.text(m.title,CKEDITOR.document)).appendTo(this.parts.title);for(u=0;u<m.contents.length;u++)(q=m.contents[u])&&this.addPage(q);this.parts.tabs.on("click",function(a){var b=a.data.getTarget();if(b.hasClass("cke_dialog_tab")){b=b.$.id;this.selectPage(b.substring(4,b.lastIndexOf("_")));if(this._.tabBarMode){this._.tabBarMode=false;this._.currentFocusIndex=-1;j(1)}a.data.preventDefault()}},this);u=[];q=CKEDITOR.dialog._.uiElementBuilders.hbox.build(this,
{type:"hbox",className:"cke_dialog_footer_buttons",widths:[],children:m.buttons},u).getChild();this.parts.footer.setHtml(u.join(""));for(u=0;u<q.length;u++)this._.buttons[q[u].id]=q[u]};CKEDITOR.dialog.prototype={destroy:function(){this.hide();this._.element.remove()},resize:function(){return function(a,b){if(!this._.contentSize||!(this._.contentSize.width==a&&this._.contentSize.height==b)){CKEDITOR.dialog.fire("resize",{dialog:this,width:a,height:b},this._.editor);this.fire("resize",{width:a,height:b},
this._.editor);this.parts.contents.setStyles({width:a+"px",height:b+"px"});if(this._.editor.lang.dir=="rtl"&&this._.position)this._.position.x=CKEDITOR.document.getWindow().getViewPaneSize().width-this._.contentSize.width-parseInt(this._.element.getFirst().getStyle("right"),10);this._.contentSize={width:a,height:b}}}}(),getSize:function(){var a=this._.element.getFirst();return{width:a.$.offsetWidth||0,height:a.$.offsetHeight||0}},move:function(a,b,c){var d=this._.element.getFirst(),f=this._.editor.lang.dir==
"rtl",g=d.getComputedStyle("position")=="fixed";CKEDITOR.env.ie&&d.setStyle("zoom","100%");if(!g||!this._.position||!(this._.position.x==a&&this._.position.y==b)){this._.position={x:a,y:b};if(!g){g=CKEDITOR.document.getWindow().getScrollPosition();a=a+g.x;b=b+g.y}if(f){g=this.getSize();a=CKEDITOR.document.getWindow().getViewPaneSize().width-g.width-a}b={top:(b>0?b:0)+"px"};b[f?"right":"left"]=(a>0?a:0)+"px";d.setStyles(b);c&&(this._.moved=1)}},getPosition:function(){return CKEDITOR.tools.extend({},
this._.position)},show:function(){var a=this._.element,b=this.definition;!a.getParent()||!a.getParent().equals(CKEDITOR.document.getBody())?a.appendTo(CKEDITOR.document.getBody()):a.setStyle("display","block");this.resize(this._.contentSize&&this._.contentSize.width||b.width||b.minWidth,this._.contentSize&&this._.contentSize.height||b.height||b.minHeight);this.reset();this.selectPage(this.definition.contents[0].id);if(CKEDITOR.dialog._.currentZIndex===null)CKEDITOR.dialog._.currentZIndex=this._.editor.config.baseFloatZIndex;
this._.element.getFirst().setStyle("z-index",CKEDITOR.dialog._.currentZIndex=CKEDITOR.dialog._.currentZIndex+10);if(CKEDITOR.dialog._.currentTop===null){CKEDITOR.dialog._.currentTop=this;this._.parentDialog=null;o(this._.editor)}else{this._.parentDialog=CKEDITOR.dialog._.currentTop;this._.parentDialog.getElement().getFirst().$.style.zIndex-=Math.floor(this._.editor.config.baseFloatZIndex/2);CKEDITOR.dialog._.currentTop=this}a.on("keydown",z);a.on("keyup",B);this._.hasFocus=false;for(var c in b.contents)if(b.contents[c]){var a=
b.contents[c],d=this._.tabs[a.id],g=a.requiredContent,e=0;if(d){for(var h in this._.contents[a.id]){var j=this._.contents[a.id][h];if(!(j.type=="hbox"||j.type=="vbox"||!j.getInputElement()))if(j.requiredContent&&!this._.editor.activeFilter.check(j.requiredContent))j.disable();else{j.enable();e++}}!e||g&&!this._.editor.activeFilter.check(g)?d[0].addClass("cke_dialog_tab_disabled"):d[0].removeClass("cke_dialog_tab_disabled")}}CKEDITOR.tools.setTimeout(function(){this.layout();f(this);this.parts.dialog.setStyle("visibility",
"");this.fireOnce("load",{});CKEDITOR.ui.fire("ready",this);this.fire("show",{});this._.editor.fire("dialogShow",this);this._.parentDialog||this._.editor.focusManager.lock();this.foreach(function(a){a.setInitValue&&a.setInitValue()})},100,this)},layout:function(){var a=this.parts.dialog,b=this.getSize(),c=CKEDITOR.document.getWindow().getViewPaneSize(),d=(c.width-b.width)/2,f=(c.height-b.height)/2;CKEDITOR.env.ie6Compat||(b.height+(f>0?f:0)>c.height||b.width+(d>0?d:0)>c.width?a.setStyle("position",
"absolute"):a.setStyle("position","fixed"));this.move(this._.moved?this._.position.x:d,this._.moved?this._.position.y:f)},foreach:function(a){for(var b in this._.contents)for(var c in this._.contents[b])a.call(this,this._.contents[b][c]);return this},reset:function(){var a=function(a){a.reset&&a.reset(1)};return function(){this.foreach(a);return this}}(),setupContent:function(){var a=arguments;this.foreach(function(b){b.setup&&b.setup.apply(b,a)})},commitContent:function(){var a=arguments;this.foreach(function(b){CKEDITOR.env.ie&&
this._.currentFocusIndex==b.focusIndex&&b.getInputElement().$.blur();b.commit&&b.commit.apply(b,a)})},hide:function(){if(this.parts.dialog.isVisible()){this.fire("hide",{});this._.editor.fire("dialogHide",this);this.selectPage(this._.tabIdList[0]);var a=this._.element;a.setStyle("display","none");this.parts.dialog.setStyle("visibility","hidden");for(G(this);CKEDITOR.dialog._.currentTop!=this;)CKEDITOR.dialog._.currentTop.hide();if(this._.parentDialog){var b=this._.parentDialog.getElement().getFirst();
b.setStyle("z-index",parseInt(b.$.style.zIndex,10)+Math.floor(this._.editor.config.baseFloatZIndex/2))}else q(this._.editor);if(CKEDITOR.dialog._.currentTop=this._.parentDialog)CKEDITOR.dialog._.currentZIndex=CKEDITOR.dialog._.currentZIndex-10;else{CKEDITOR.dialog._.currentZIndex=null;a.removeListener("keydown",z);a.removeListener("keyup",B);var c=this._.editor;c.focus();setTimeout(function(){c.focusManager.unlock();CKEDITOR.env.iOS&&c.window.focus()},0)}delete this._.parentDialog;this.foreach(function(a){a.resetInitValue&&
a.resetInitValue()})}},addPage:function(a){if(!a.requiredContent||this._.editor.filter.check(a.requiredContent)){for(var b=[],c=a.label?' title="'+CKEDITOR.tools.htmlEncode(a.label)+'"':"",d=CKEDITOR.dialog._.uiElementBuilders.vbox.build(this,{type:"vbox",className:"cke_dialog_page_contents",children:a.elements,expand:!!a.expand,padding:a.padding,style:a.style||"width: 100%;"},b),f=this._.contents[a.id]={},g=d.getChild(),e=0;d=g.shift();){!d.notAllowed&&(d.type!="hbox"&&d.type!="vbox")&&e++;f[d.id]=
d;typeof d.getChild=="function"&&g.push.apply(g,d.getChild())}if(!e)a.hidden=true;b=CKEDITOR.dom.element.createFromHtml(b.join(""));b.setAttribute("role","tabpanel");d=CKEDITOR.env;f="cke_"+a.id+"_"+CKEDITOR.tools.getNextNumber();c=CKEDITOR.dom.element.createFromHtml(['<a class="cke_dialog_tab"',this._.pageCount>0?" cke_last":"cke_first",c,a.hidden?' style="display:none"':"",' id="',f,'"',d.gecko&&!d.hc?"":' href="javascript:void(0)"',' tabIndex="-1" hidefocus="true" role="tab">',a.label,"</a>"].join(""));
b.setAttribute("aria-labelledby",f);this._.tabs[a.id]=[c,b];this._.tabIdList.push(a.id);!a.hidden&&this._.pageCount++;this._.lastTab=c;this.updateStyle();b.setAttribute("name",a.id);b.appendTo(this.parts.contents);c.unselectable();this.parts.tabs.append(c);if(a.accessKey){A(this,this,"CTRL+"+a.accessKey,J,D);this._.accessKeyMap["CTRL+"+a.accessKey]=a.id}}},selectPage:function(a){if(this._.currentTabId!=a&&!this._.tabs[a][0].hasClass("cke_dialog_tab_disabled")&&this.fire("selectPage",{page:a,currentPage:this._.currentTabId})!==
false){for(var c in this._.tabs){var d=this._.tabs[c][0],f=this._.tabs[c][1];if(c!=a){d.removeClass("cke_dialog_tab_selected");f.hide()}f.setAttribute("aria-hidden",c!=a)}var g=this._.tabs[a];g[0].addClass("cke_dialog_tab_selected");if(CKEDITOR.env.ie6Compat||CKEDITOR.env.ie7Compat){b(g[1]);g[1].show();setTimeout(function(){b(g[1],1)},0)}else g[1].show();this._.currentTabId=a;this._.currentTabIndex=CKEDITOR.tools.indexOf(this._.tabIdList,a)}},updateStyle:function(){this.parts.dialog[(this._.pageCount===
1?"add":"remove")+"Class"]("cke_single_page")},hidePage:function(b){var c=this._.tabs[b]&&this._.tabs[b][0];if(c&&this._.pageCount!=1&&c.isVisible()){b==this._.currentTabId&&this.selectPage(a.call(this));c.hide();this._.pageCount--;this.updateStyle()}},showPage:function(a){if(a=this._.tabs[a]&&this._.tabs[a][0]){a.show();this._.pageCount++;this.updateStyle()}},getElement:function(){return this._.element},getName:function(){return this._.name},getContentElement:function(a,b){var c=this._.contents[a];
return c&&c[b]},getValueOf:function(a,b){return this.getContentElement(a,b).getValue()},setValueOf:function(a,b,c){return this.getContentElement(a,b).setValue(c)},getButton:function(a){return this._.buttons[a]},click:function(a){return this._.buttons[a].click()},disableButton:function(a){return this._.buttons[a].disable()},enableButton:function(a){return this._.buttons[a].enable()},getPageCount:function(){return this._.pageCount},getParentEditor:function(){return this._.editor},getSelectedElement:function(){return this.getParentEditor().getSelection().getSelectedElement()},
addFocusable:function(a,b){if(typeof b=="undefined"){b=this._.focusList.length;this._.focusList.push(new j(this,a,b))}else{this._.focusList.splice(b,0,new j(this,a,b));for(var c=b+1;c<this._.focusList.length;c++)this._.focusList[c].focusIndex++}}};CKEDITOR.tools.extend(CKEDITOR.dialog,{add:function(a,b){if(!this._.dialogDefinitions[a]||typeof b=="function")this._.dialogDefinitions[a]=b},exists:function(a){return!!this._.dialogDefinitions[a]},getCurrent:function(){return CKEDITOR.dialog._.currentTop},
isTabEnabled:function(a,b,c){a=a.config.removeDialogTabs;return!(a&&a.match(RegExp("(?:^|;)"+b+":"+c+"(?:$|;)","i")))},okButton:function(){var a=function(a,b){b=b||{};return CKEDITOR.tools.extend({id:"ok",type:"button",label:a.lang.common.ok,"class":"cke_dialog_ui_button_ok",onClick:function(a){a=a.data.dialog;a.fire("ok",{hide:true}).hide!==false&&a.hide()}},b,true)};a.type="button";a.override=function(b){return CKEDITOR.tools.extend(function(c){return a(c,b)},{type:"button"},true)};return a}(),
cancelButton:function(){var a=function(a,b){b=b||{};return CKEDITOR.tools.extend({id:"cancel",type:"button",label:a.lang.common.cancel,"class":"cke_dialog_ui_button_cancel",onClick:function(a){a=a.data.dialog;a.fire("cancel",{hide:true}).hide!==false&&a.hide()}},b,true)};a.type="button";a.override=function(b){return CKEDITOR.tools.extend(function(c){return a(c,b)},{type:"button"},true)};return a}(),addUIElement:function(a,b){this._.uiElementBuilders[a]=b}});CKEDITOR.dialog._={uiElementBuilders:{},
dialogDefinitions:{},currentTop:null,currentZIndex:null};CKEDITOR.event.implementOn(CKEDITOR.dialog);CKEDITOR.event.implementOn(CKEDITOR.dialog.prototype);var p={resizable:CKEDITOR.DIALOG_RESIZE_BOTH,minWidth:600,minHeight:400,buttons:[CKEDITOR.dialog.okButton,CKEDITOR.dialog.cancelButton]},t=function(a,b,c){for(var d=0,f;f=a[d];d++){if(f.id==b)return f;if(c&&f[c])if(f=t(f[c],b,c))return f}return null},s=function(a,b,c,d,f){if(c){for(var g=0,e;e=a[g];g++){if(e.id==c){a.splice(g,0,b);return b}if(d&&
e[d])if(e=s(e[d],b,c,d,true))return e}if(f)return null}a.push(b);return b},u=function(a,b,c){for(var d=0,f;f=a[d];d++){if(f.id==b)return a.splice(d,1);if(c&&f[c])if(f=u(f[c],b,c))return f}return null},w=function(a,b){this.dialog=a;for(var c=b.contents,d=0,f;f=c[d];d++)c[d]=f&&new h(a,f);CKEDITOR.tools.extend(this,b)};w.prototype={getContents:function(a){return t(this.contents,a)},getButton:function(a){return t(this.buttons,a)},addContents:function(a,b){return s(this.contents,a,b)},addButton:function(a,
b){return s(this.buttons,a,b)},removeContents:function(a){u(this.contents,a)},removeButton:function(a){u(this.buttons,a)}};h.prototype={get:function(a){return t(this.elements,a,"children")},add:function(a,b){return s(this.elements,a,b,"children")},remove:function(a){u(this.elements,a,"children")}};var x,v={},r,y={},z=function(a){var b=a.data.$.ctrlKey||a.data.$.metaKey,c=a.data.$.altKey,d=a.data.$.shiftKey,f=String.fromCharCode(a.data.$.keyCode);if((b=y[(b?"CTRL+":"")+(c?"ALT+":"")+(d?"SHIFT+":"")+
f])&&b.length){b=b[b.length-1];b.keydown&&b.keydown.call(b.uiElement,b.dialog,b.key);a.data.preventDefault()}},B=function(a){var b=a.data.$.ctrlKey||a.data.$.metaKey,c=a.data.$.altKey,d=a.data.$.shiftKey,f=String.fromCharCode(a.data.$.keyCode);if((b=y[(b?"CTRL+":"")+(c?"ALT+":"")+(d?"SHIFT+":"")+f])&&b.length){b=b[b.length-1];if(b.keyup){b.keyup.call(b.uiElement,b.dialog,b.key);a.data.preventDefault()}}},A=function(a,b,c,d,f){(y[c]||(y[c]=[])).push({uiElement:a,dialog:b,key:c,keyup:f||a.accessKeyUp,
keydown:d||a.accessKeyDown})},G=function(a){for(var b in y){for(var c=y[b],d=c.length-1;d>=0;d--)(c[d].dialog==a||c[d].uiElement==a)&&c.splice(d,1);c.length===0&&delete y[b]}},D=function(a,b){a._.accessKeyMap[b]&&a.selectPage(a._.accessKeyMap[b])},J=function(){};(function(){CKEDITOR.ui.dialog={uiElement:function(a,b,c,d,f,g,e){if(!(arguments.length<4)){var h=(d.call?d(b):d)||"div",j=["<",h," "],i=(f&&f.call?f(b):f)||{},k=(g&&g.call?g(b):g)||{},p=(e&&e.call?e.call(this,a,b):e)||"",s=this.domId=k.id||
CKEDITOR.tools.getNextId()+"_uiElement";if(b.requiredContent&&!a.getParentEditor().filter.check(b.requiredContent)){i.display="none";this.notAllowed=true}k.id=s;var m={};b.type&&(m["cke_dialog_ui_"+b.type]=1);b.className&&(m[b.className]=1);if(b.disabled)m.cke_disabled=1;for(var t=k["class"]&&k["class"].split?k["class"].split(" "):[],s=0;s<t.length;s++)t[s]&&(m[t[s]]=1);t=[];for(s in m)t.push(s);k["class"]=t.join(" ");if(b.title)k.title=b.title;m=(b.style||"").split(";");if(b.align){t=b.align;i["margin-left"]=
t=="left"?0:"auto";i["margin-right"]=t=="right"?0:"auto"}for(s in i)m.push(s+":"+i[s]);b.hidden&&m.push("display:none");for(s=m.length-1;s>=0;s--)m[s]===""&&m.splice(s,1);if(m.length>0)k.style=(k.style?k.style+"; ":"")+m.join("; ");for(s in k)j.push(s+'="'+CKEDITOR.tools.htmlEncode(k[s])+'" ');j.push(">",p,"</",h,">");c.push(j.join(""));(this._||(this._={})).dialog=a;if(typeof b.isChanged=="boolean")this.isChanged=function(){return b.isChanged};if(typeof b.isChanged=="function")this.isChanged=b.isChanged;
if(typeof b.setValue=="function")this.setValue=CKEDITOR.tools.override(this.setValue,function(a){return function(c){a.call(this,b.setValue.call(this,c))}});if(typeof b.getValue=="function")this.getValue=CKEDITOR.tools.override(this.getValue,function(a){return function(){return b.getValue.call(this,a.call(this))}});CKEDITOR.event.implementOn(this);this.registerEvents(b);this.accessKeyUp&&(this.accessKeyDown&&b.accessKey)&&A(this,a,"CTRL+"+b.accessKey);var l=this;a.on("load",function(){var b=l.getInputElement();
if(b){var c=l.type in{checkbox:1,ratio:1}&&CKEDITOR.env.ie&&CKEDITOR.env.version<8?"cke_dialog_ui_focused":"";b.on("focus",function(){a._.tabBarMode=false;a._.hasFocus=true;l.fire("focus");c&&this.addClass(c)});b.on("blur",function(){l.fire("blur");c&&this.removeClass(c)})}});CKEDITOR.tools.extend(this,b);if(this.keyboardFocusable){this.tabIndex=b.tabIndex||0;this.focusIndex=a._.focusList.push(this)-1;this.on("focus",function(){a._.currentFocusIndex=l.focusIndex})}}},hbox:function(a,b,c,d,f){if(!(arguments.length<
4)){this._||(this._={});var g=this._.children=b,e=f&&f.widths||null,h=f&&f.height||null,j,i={role:"presentation"};f&&f.align&&(i.align=f.align);CKEDITOR.ui.dialog.uiElement.call(this,a,f||{type:"hbox"},d,"table",{},i,function(){var a=['<tbody><tr class="cke_dialog_ui_hbox">'];for(j=0;j<c.length;j++){var b="cke_dialog_ui_hbox_child",d=[];j===0&&(b="cke_dialog_ui_hbox_first");j==c.length-1&&(b="cke_dialog_ui_hbox_last");a.push('<td class="',b,'" role="presentation" ');e?e[j]&&d.push("width:"+m(e[j])):
d.push("width:"+Math.floor(100/c.length)+"%");h&&d.push("height:"+m(h));f&&f.padding!==void 0&&d.push("padding:"+m(f.padding));CKEDITOR.env.ie&&(CKEDITOR.env.quirks&&g[j].align)&&d.push("text-align:"+g[j].align);d.length>0&&a.push('style="'+d.join("; ")+'" ');a.push(">",c[j],"</td>")}a.push("</tr></tbody>");return a.join("")})}},vbox:function(a,b,c,d,f){if(!(arguments.length<3)){this._||(this._={});var g=this._.children=b,e=f&&f.width||null,h=f&&f.heights||null;CKEDITOR.ui.dialog.uiElement.call(this,
a,f||{type:"vbox"},d,"div",null,{role:"presentation"},function(){var b=['<table role="presentation" cellspacing="0" border="0" '];b.push('style="');f&&f.expand&&b.push("height:100%;");b.push("width:"+m(e||"100%"),";");CKEDITOR.env.webkit&&b.push("float:none;");b.push('"');b.push('align="',CKEDITOR.tools.htmlEncode(f&&f.align||(a.getParentEditor().lang.dir=="ltr"?"left":"right")),'" ');b.push("><tbody>");for(var d=0;d<c.length;d++){var j=[];b.push('<tr><td role="presentation" ');e&&j.push("width:"+
m(e||"100%"));h?j.push("height:"+m(h[d])):f&&f.expand&&j.push("height:"+Math.floor(100/c.length)+"%");f&&f.padding!==void 0&&j.push("padding:"+m(f.padding));CKEDITOR.env.ie&&(CKEDITOR.env.quirks&&g[d].align)&&j.push("text-align:"+g[d].align);j.length>0&&b.push('style="',j.join("; "),'" ');b.push(' class="cke_dialog_ui_vbox_child">',c[d],"</td></tr>")}b.push("</tbody></table>");return b.join("")})}}}})();CKEDITOR.ui.dialog.uiElement.prototype={getElement:function(){return CKEDITOR.document.getById(this.domId)},
getInputElement:function(){return this.getElement()},getDialog:function(){return this._.dialog},setValue:function(a,b){this.getInputElement().setValue(a);!b&&this.fire("change",{value:a});return this},getValue:function(){return this.getInputElement().getValue()},isChanged:function(){return false},selectParentTab:function(){for(var a=this.getInputElement();(a=a.getParent())&&a.$.className.search("cke_dialog_page_contents")==-1;);if(!a)return this;a=a.getAttribute("name");this._.dialog._.currentTabId!=
a&&this._.dialog.selectPage(a);return this},focus:function(){this.selectParentTab().getInputElement().focus();return this},registerEvents:function(a){var b=/^on([A-Z]\w+)/,c,d=function(a,b,c,d){b.on("load",function(){a.getInputElement().on(c,d,a)})},f;for(f in a)if(c=f.match(b))this.eventProcessors[f]?this.eventProcessors[f].call(this,this._.dialog,a[f]):d(this,this._.dialog,c[1].toLowerCase(),a[f]);return this},eventProcessors:{onLoad:function(a,b){a.on("load",b,this)},onShow:function(a,b){a.on("show",
b,this)},onHide:function(a,b){a.on("hide",b,this)}},accessKeyDown:function(){this.focus()},accessKeyUp:function(){},disable:function(){var a=this.getElement();this.getInputElement().setAttribute("disabled","true");a.addClass("cke_disabled")},enable:function(){var a=this.getElement();this.getInputElement().removeAttribute("disabled");a.removeClass("cke_disabled")},isEnabled:function(){return!this.getElement().hasClass("cke_disabled")},isVisible:function(){return this.getInputElement().isVisible()},
isFocusable:function(){return!this.isEnabled()||!this.isVisible()?false:true}};CKEDITOR.ui.dialog.hbox.prototype=CKEDITOR.tools.extend(new CKEDITOR.ui.dialog.uiElement,{getChild:function(a){if(arguments.length<1)return this._.children.concat();a.splice||(a=[a]);return a.length<2?this._.children[a[0]]:this._.children[a[0]]&&this._.children[a[0]].getChild?this._.children[a[0]].getChild(a.slice(1,a.length)):null}},true);CKEDITOR.ui.dialog.vbox.prototype=new CKEDITOR.ui.dialog.hbox;(function(){var a=
{build:function(a,b,c){for(var d=b.children,f,g=[],e=[],h=0;h<d.length&&(f=d[h]);h++){var j=[];g.push(j);e.push(CKEDITOR.dialog._.uiElementBuilders[f.type].build(a,f,j))}return new CKEDITOR.ui.dialog[b.type](a,e,g,c,b)}};CKEDITOR.dialog.addUIElement("hbox",a);CKEDITOR.dialog.addUIElement("vbox",a)})();CKEDITOR.dialogCommand=function(a,b){this.dialogName=a;CKEDITOR.tools.extend(this,b,true)};CKEDITOR.dialogCommand.prototype={exec:function(a){a.openDialog(this.dialogName)},canUndo:false,editorFocus:1};
(function(){var a=/^([a]|[^a])+$/,b=/^\d*$/,c=/^\d*(?:\.\d+)?$/,d=/^(((\d*(\.\d+))|(\d*))(px|\%)?)?$/,f=/^(((\d*(\.\d+))|(\d*))(px|em|ex|in|cm|mm|pt|pc|\%)?)?$/i,g=/^(\s*[\w-]+\s*:\s*[^:;]+(?:;|$))*$/;CKEDITOR.VALIDATE_OR=1;CKEDITOR.VALIDATE_AND=2;CKEDITOR.dialog.validate={functions:function(){var a=arguments;return function(){var b=this&&this.getValue?this.getValue():a[0],c,d=CKEDITOR.VALIDATE_AND,f=[],g;for(g=0;g<a.length;g++)if(typeof a[g]=="function")f.push(a[g]);else break;if(g<a.length&&typeof a[g]==
"string"){c=a[g];g++}g<a.length&&typeof a[g]=="number"&&(d=a[g]);var e=d==CKEDITOR.VALIDATE_AND?true:false;for(g=0;g<f.length;g++)e=d==CKEDITOR.VALIDATE_AND?e&&f[g](b):e||f[g](b);return!e?c:true}},regex:function(a,b){return function(c){c=this&&this.getValue?this.getValue():c;return!a.test(c)?b:true}},notEmpty:function(b){return this.regex(a,b)},integer:function(a){return this.regex(b,a)},number:function(a){return this.regex(c,a)},cssLength:function(a){return this.functions(function(a){return f.test(CKEDITOR.tools.trim(a))},
a)},htmlLength:function(a){return this.functions(function(a){return d.test(CKEDITOR.tools.trim(a))},a)},inlineStyle:function(a){return this.functions(function(a){return g.test(CKEDITOR.tools.trim(a))},a)},equals:function(a,b){return this.functions(function(b){return b==a},b)},notEqual:function(a,b){return this.functions(function(b){return b!=a},b)}};CKEDITOR.on("instanceDestroyed",function(a){if(CKEDITOR.tools.isEmpty(CKEDITOR.instances)){for(var b;b=CKEDITOR.dialog._.currentTop;)b.hide();for(var c in v)v[c].remove();
v={}}var a=a.editor._.storedDialogs,d;for(d in a)a[d].destroy()})})();CKEDITOR.tools.extend(CKEDITOR.editor.prototype,{openDialog:function(a,b){var c=null,d=CKEDITOR.dialog._.dialogDefinitions[a];CKEDITOR.dialog._.currentTop===null&&o(this);if(typeof d=="function"){c=this._.storedDialogs||(this._.storedDialogs={});c=c[a]||(c[a]=new CKEDITOR.dialog(this,a));b&&b.call(c,c);c.show()}else{if(d=="failed"){q(this);throw Error('[CKEDITOR.dialog.openDialog] Dialog "'+a+'" failed when loading definition.');
}typeof d=="string"&&CKEDITOR.scriptLoader.load(CKEDITOR.getUrl(d),function(){typeof CKEDITOR.dialog._.dialogDefinitions[a]!="function"&&(CKEDITOR.dialog._.dialogDefinitions[a]="failed");this.openDialog(a,b)},this,0,1)}CKEDITOR.skin.loadPart("dialog");return c}})}(),CKEDITOR.plugins.add("dialog",{requires:"dialogui",init:function(a){a.on("doubleclick",function(e){e.data.dialog&&a.openDialog(e.data.dialog)},null,null,999)}}),function(){CKEDITOR.plugins.add("a11yhelp",{requires:"dialog",availableLangs:{af:1,
ar:1,bg:1,ca:1,cs:1,cy:1,da:1,de:1,el:1,en:1,"en-gb":1,eo:1,es:1,et:1,fa:1,fi:1,fr:1,"fr-ca":1,gl:1,gu:1,he:1,hi:1,hr:1,hu:1,id:1,it:1,ja:1,km:1,ko:1,ku:1,lt:1,lv:1,mk:1,mn:1,nb:1,nl:1,no:1,pl:1,pt:1,"pt-br":1,ro:1,ru:1,si:1,sk:1,sl:1,sq:1,sr:1,"sr-latn":1,sv:1,th:1,tr:1,tt:1,ug:1,uk:1,vi:1,zh:1,"zh-cn":1},init:function(a){var e=this;a.addCommand("a11yHelp",{exec:function(){var b=a.langCode,b=e.availableLangs[b]?b:e.availableLangs[b.replace(/-.*/,"")]?b.replace(/-.*/,""):"en";CKEDITOR.scriptLoader.load(CKEDITOR.getUrl(e.path+
"dialogs/lang/"+b+".js"),function(){a.lang.a11yhelp=e.langEntries[b];a.openDialog("a11yHelp")})},modes:{wysiwyg:1,source:1},readOnly:1,canUndo:false});a.setKeystroke(CKEDITOR.ALT+48,"a11yHelp");CKEDITOR.dialog.add("a11yHelp",this.path+"dialogs/a11yhelp.js");a.on("ariaEditorHelpLabel",function(b){b.data.label=a.lang.common.editorHelp})}})}(),CKEDITOR.plugins.add("about",{requires:"dialog",init:function(a){var e=a.addCommand("about",new CKEDITOR.dialogCommand("about"));e.modes={wysiwyg:1,source:1};
e.canUndo=false;e.readOnly=1;a.ui.addButton&&a.ui.addButton("About",{label:a.lang.about.title,command:"about",toolbar:"about"});CKEDITOR.dialog.add("about",this.path+"dialogs/about.js")}}),CKEDITOR.plugins.add("basicstyles",{init:function(a){var e=0,b=function(b,c,d,i){if(i){var i=new CKEDITOR.style(i),k=g[d];k.unshift(i);a.attachStyleStateChange(i,function(b){!a.readOnly&&a.getCommand(d).setState(b)});a.addCommand(d,new CKEDITOR.styleCommand(i,{contentForms:k}));a.ui.addButton&&a.ui.addButton(b,
{label:c,command:d,toolbar:"basicstyles,"+(e=e+10)})}},g={bold:["strong","b",["span",function(a){a=a.styles["font-weight"];return a=="bold"||+a>=700}]],italic:["em","i",["span",function(a){return a.styles["font-style"]=="italic"}]],underline:["u",["span",function(a){return a.styles["text-decoration"]=="underline"}]],strike:["s","strike",["span",function(a){return a.styles["text-decoration"]=="line-through"}]],subscript:["sub"],superscript:["sup"]},c=a.config,d=a.lang.basicstyles;b("Bold",d.bold,"bold",
c.coreStyles_bold);b("Italic",d.italic,"italic",c.coreStyles_italic);b("Underline",d.underline,"underline",c.coreStyles_underline);b("Strike",d.strike,"strike",c.coreStyles_strike);b("Subscript",d.subscript,"subscript",c.coreStyles_subscript);b("Superscript",d.superscript,"superscript",c.coreStyles_superscript);a.setKeystroke([[CKEDITOR.CTRL+66,"bold"],[CKEDITOR.CTRL+73,"italic"],[CKEDITOR.CTRL+85,"underline"]])}}),CKEDITOR.config.coreStyles_bold={element:"strong",overrides:"b"},CKEDITOR.config.coreStyles_italic=
{element:"em",overrides:"i"},CKEDITOR.config.coreStyles_underline={element:"u"},CKEDITOR.config.coreStyles_strike={element:"s",overrides:"strike"},CKEDITOR.config.coreStyles_subscript={element:"sub"},CKEDITOR.config.coreStyles_superscript={element:"sup"},function(){var a={exec:function(a){var b=a.getCommand("blockquote").state,g=a.getSelection(),c=g&&g.getRanges()[0];if(c){var d=g.createBookmarks();if(CKEDITOR.env.ie){var j=d[0].startNode,f=d[0].endNode,h;if(j&&j.getParent().getName()=="blockquote")for(h=
j;h=h.getNext();)if(h.type==CKEDITOR.NODE_ELEMENT&&h.isBlockBoundary()){j.move(h,true);break}if(f&&f.getParent().getName()=="blockquote")for(h=f;h=h.getPrevious();)if(h.type==CKEDITOR.NODE_ELEMENT&&h.isBlockBoundary()){f.move(h);break}}var i=c.createIterator();i.enlargeBr=a.config.enterMode!=CKEDITOR.ENTER_BR;if(b==CKEDITOR.TRISTATE_OFF){for(j=[];b=i.getNextParagraph();)j.push(b);if(j.length<1){b=a.document.createElement(a.config.enterMode==CKEDITOR.ENTER_P?"p":"div");f=d.shift();c.insertNode(b);
b.append(new CKEDITOR.dom.text("﻿",a.document));c.moveToBookmark(f);c.selectNodeContents(b);c.collapse(true);f=c.createBookmark();j.push(b);d.unshift(f)}h=j[0].getParent();c=[];for(f=0;f<j.length;f++){b=j[f];h=h.getCommonAncestor(b.getParent())}for(b={table:1,tbody:1,tr:1,ol:1,ul:1};b[h.getName()];)h=h.getParent();for(f=null;j.length>0;){for(b=j.shift();!b.getParent().equals(h);)b=b.getParent();b.equals(f)||c.push(b);f=b}for(;c.length>0;){b=c.shift();if(b.getName()=="blockquote"){for(f=new CKEDITOR.dom.documentFragment(a.document);b.getFirst();){f.append(b.getFirst().remove());
j.push(f.getLast())}f.replace(b)}else j.push(b)}c=a.document.createElement("blockquote");for(c.insertBefore(j[0]);j.length>0;){b=j.shift();c.append(b)}}else if(b==CKEDITOR.TRISTATE_ON){f=[];for(h={};b=i.getNextParagraph();){for(j=c=null;b.getParent();){if(b.getParent().getName()=="blockquote"){c=b.getParent();j=b;break}b=b.getParent()}if(c&&j&&!j.getCustomData("blockquote_moveout")){f.push(j);CKEDITOR.dom.element.setMarker(h,j,"blockquote_moveout",true)}}CKEDITOR.dom.element.clearAllMarkers(h);b=
[];j=[];for(h={};f.length>0;){i=f.shift();c=i.getParent();if(i.getPrevious())if(i.getNext()){i.breakParent(i.getParent());j.push(i.getNext())}else i.remove().insertAfter(c);else i.remove().insertBefore(c);if(!c.getCustomData("blockquote_processed")){j.push(c);CKEDITOR.dom.element.setMarker(h,c,"blockquote_processed",true)}b.push(i)}CKEDITOR.dom.element.clearAllMarkers(h);for(f=j.length-1;f>=0;f--){c=j[f];a:{h=c;for(var i=0,k=h.getChildCount(),n=void 0;i<k&&(n=h.getChild(i));i++)if(n.type==CKEDITOR.NODE_ELEMENT&&
n.isBlockBoundary()){h=false;break a}h=true}h&&c.remove()}if(a.config.enterMode==CKEDITOR.ENTER_BR)for(c=true;b.length;){i=b.shift();if(i.getName()=="div"){f=new CKEDITOR.dom.documentFragment(a.document);c&&(i.getPrevious()&&!(i.getPrevious().type==CKEDITOR.NODE_ELEMENT&&i.getPrevious().isBlockBoundary()))&&f.append(a.document.createElement("br"));for(c=i.getNext()&&!(i.getNext().type==CKEDITOR.NODE_ELEMENT&&i.getNext().isBlockBoundary());i.getFirst();)i.getFirst().remove().appendTo(f);c&&f.append(a.document.createElement("br"));
f.replace(i);c=false}}}g.selectBookmarks(d);a.focus()}},refresh:function(a,b){this.setState(a.elementPath(b.block||b.blockLimit).contains("blockquote",1)?CKEDITOR.TRISTATE_ON:CKEDITOR.TRISTATE_OFF)},context:"blockquote",allowedContent:"blockquote",requiredContent:"blockquote"};CKEDITOR.plugins.add("blockquote",{init:function(e){if(!e.blockless){e.addCommand("blockquote",a);e.ui.addButton&&e.ui.addButton("Blockquote",{label:e.lang.blockquote.toolbar,command:"blockquote",toolbar:"blocks,10"})}}})}(),
"use strict",function(){function a(a){function b(){var c=a.editable();c.on(y,function(a){(!CKEDITOR.env.ie||!x)&&s(a)});CKEDITOR.env.ie&&c.on("paste",function(b){if(!v){g();b.data.preventDefault();s(b);o("paste")||a.openDialog("paste")}});if(CKEDITOR.env.ie){c.on("contextmenu",e,null,null,0);c.on("beforepaste",function(a){a.data&&(!a.data.$.ctrlKey&&!a.data.$.shiftKey)&&e()},null,null,0)}c.on("beforecut",function(){!x&&m(a)});var d;c.attachListener(CKEDITOR.env.ie?c:a.document.getDocumentElement(),
"mouseup",function(){d=setTimeout(function(){u()},0)});a.on("destroy",function(){clearTimeout(d)});c.on("keyup",u)}function c(b){return{type:b,canUndo:b=="cut",startDisabled:true,exec:function(){this.type=="cut"&&m();var b;var c=this.type;if(CKEDITOR.env.ie)b=o(c);else try{b=a.document.$.execCommand(c,false,null)}catch(d){b=false}b||alert(a.lang.clipboard[this.type+"Error"]);return b}}}function d(){return{canUndo:false,async:true,exec:function(a,b){var c=function(b,c){b&&q(b.type,b.dataValue,!!c);
a.fire("afterCommandExec",{name:"paste",command:d,returnValue:!!b})},d=this;typeof b=="string"?c({type:"auto",dataValue:b},1):a.getClipboardData(c)}}}function g(){v=1;setTimeout(function(){v=0},100)}function e(){x=1;setTimeout(function(){x=0},10)}function o(b){var c=a.document,d=c.getBody(),f=false,g=function(){f=true};d.on(b,g);(CKEDITOR.env.version>7?c.$:c.$.selection.createRange()).execCommand(b);d.removeListener(b,g);return f}function q(b,c,d){b={type:b};if(d&&a.fire("beforePaste",b)===false||
!c)return false;b.dataValue=c;return a.fire("paste",b)}function m(){if(CKEDITOR.env.ie&&!CKEDITOR.env.quirks){var b=a.getSelection(),c,d,f;if(b.getType()==CKEDITOR.SELECTION_ELEMENT&&(c=b.getSelectedElement())){d=b.getRanges()[0];f=a.document.createText("");f.insertBefore(c);d.setStartBefore(f);d.setEndAfter(c);b.selectRanges([d]);setTimeout(function(){if(c.getParent()){f.remove();b.selectElement(c)}},0)}}}function l(b,c){var d=a.document,f=a.editable(),g=function(a){a.cancel()},e;if(!d.getById("cke_pastebin")){var h=
a.getSelection(),i=h.createBookmarks();CKEDITOR.env.ie&&h.root.fire("selectionchange");var k=new CKEDITOR.dom.element((CKEDITOR.env.webkit||f.is("body"))&&!CKEDITOR.env.ie?"body":"div",d);k.setAttributes({id:"cke_pastebin","data-cke-temp":"1"});var s=0,d=d.getWindow();if(CKEDITOR.env.webkit){f.append(k);k.addClass("cke_editable");if(!f.is("body")){s=f.getComputedStyle("position")!="static"?f:CKEDITOR.dom.element.get(f.$.offsetParent);s=s.getDocumentPosition().y}}else f.getAscendant(CKEDITOR.env.ie?
"body":"html",1).append(k);k.setStyles({position:"absolute",top:d.getScrollPosition().y-s+10+"px",width:"1px",height:Math.max(1,d.getViewPaneSize().height-20)+"px",overflow:"hidden",margin:0,padding:0});CKEDITOR.env.safari&&k.setStyles(CKEDITOR.tools.cssVendorPrefix("user-select","text"));if(s=k.getParent().isReadOnly()){k.setOpacity(0);k.setAttribute("contenteditable",true)}else k.setStyle(a.config.contentsLangDirection=="ltr"?"left":"right","-1000px");a.on("selectionChange",g,null,null,0);if(CKEDITOR.env.webkit||
CKEDITOR.env.gecko)e=f.once("blur",g,null,null,-100);s&&k.focus();s=new CKEDITOR.dom.range(k);s.selectNodeContents(k);var p=s.select();CKEDITOR.env.ie&&(e=f.once("blur",function(){a.lockSelection(p)}));var m=CKEDITOR.document.getWindow().getScrollPosition().y;setTimeout(function(){if(CKEDITOR.env.webkit)CKEDITOR.document.getBody().$.scrollTop=m;e&&e.removeListener();CKEDITOR.env.ie&&f.focus();h.selectBookmarks(i);k.remove();var b;if(CKEDITOR.env.webkit&&(b=k.getFirst())&&b.is&&b.hasClass("Apple-style-span"))k=
b;a.removeListener("selectionChange",g);c(k.getHtml())},0)}}function p(){if(CKEDITOR.env.ie){a.focus();g();var b=a.focusManager;b.lock();if(a.editable().fire(y)&&!o("paste")){b.unlock();return false}b.unlock()}else try{if(a.editable().fire(y)&&!a.document.$.execCommand("Paste",false,null))throw 0;}catch(c){return false}return true}function t(b){if(a.mode=="wysiwyg")switch(b.data.keyCode){case CKEDITOR.CTRL+86:case CKEDITOR.SHIFT+45:b=a.editable();g();!CKEDITOR.env.ie&&b.fire("beforepaste");break;
case CKEDITOR.CTRL+88:case CKEDITOR.SHIFT+46:a.fire("saveSnapshot");setTimeout(function(){a.fire("saveSnapshot")},50)}}function s(b){var c={type:"auto"},d=a.fire("beforePaste",c);l(b,function(a){a=a.replace(/<span[^>]+data-cke-bookmark[^<]*?<\/span>/ig,"");d&&q(c.type,a,0,1)})}function u(){if(a.mode=="wysiwyg"){var b=w("paste");a.getCommand("cut").setState(w("cut"));a.getCommand("copy").setState(w("copy"));a.getCommand("paste").setState(b);a.fire("pasteState",b)}}function w(b){if(r&&b in{paste:1,
cut:1})return CKEDITOR.TRISTATE_DISABLED;if(b=="paste")return CKEDITOR.TRISTATE_OFF;var b=a.getSelection(),c=b.getRanges();return b.getType()==CKEDITOR.SELECTION_NONE||c.length==1&&c[0].collapsed?CKEDITOR.TRISTATE_DISABLED:CKEDITOR.TRISTATE_OFF}var x=0,v=0,r=0,y=CKEDITOR.env.ie?"beforepaste":"paste";(function(){a.on("key",t);a.on("contentDom",b);a.on("selectionChange",function(a){r=a.data.selection.getRanges()[0].checkReadOnly();u()});a.contextMenu&&a.contextMenu.addListener(function(a,b){r=b.getRanges()[0].checkReadOnly();
return{cut:w("cut"),copy:w("copy"),paste:w("paste")}})})();(function(){function b(c,d,f,g,e){var h=a.lang.clipboard[d];a.addCommand(d,f);a.ui.addButton&&a.ui.addButton(c,{label:h,command:d,toolbar:"clipboard,"+g});a.addMenuItems&&a.addMenuItem(d,{label:h,command:d,group:"clipboard",order:e})}b("Cut","cut",c("cut"),10,1);b("Copy","copy",c("copy"),20,4);b("Paste","paste",d(),30,8)})();a.getClipboardData=function(b,c){function d(a){a.removeListener();a.cancel();c(a.data)}function f(a){a.removeListener();
a.cancel();i=true;c({type:h,dataValue:a.data})}function g(){this.customTitle=b&&b.title}var e=false,h="auto",i=false;if(!c){c=b;b=null}a.on("paste",d,null,null,0);a.on("beforePaste",function(a){a.removeListener();e=true;h=a.data.type},null,null,1E3);if(p()===false){a.removeListener("paste",d);if(e&&a.fire("pasteDialog",g)){a.on("pasteDialogCommit",f);a.on("dialogHide",function(a){a.removeListener();a.data.removeListener("pasteDialogCommit",f);setTimeout(function(){i||c(null)},10)})}else c(null)}}}
function e(a){if(CKEDITOR.env.webkit){if(!a.match(/^[^<]*$/g)&&!a.match(/^(<div><br( ?\/)?><\/div>|<div>[^<]*<\/div>)*$/gi))return"html"}else if(CKEDITOR.env.ie){if(!a.match(/^([^<]|<br( ?\/)?>)*$/gi)&&!a.match(/^(<p>([^<]|<br( ?\/)?>)*<\/p>|(\r\n))*$/gi))return"html"}else if(CKEDITOR.env.gecko){if(!a.match(/^([^<]|<br( ?\/)?>)*$/gi))return"html"}else return"html";return"htmlifiedtext"}function b(a,b){function c(a){return CKEDITOR.tools.repeat("</p><p>",~~(a/2))+(a%2==1?"<br>":"")}b=b.replace(/\s+/g,
" ").replace(/> +</g,"><").replace(/<br ?\/>/gi,"<br>");b=b.replace(/<\/?[A-Z]+>/g,function(a){return a.toLowerCase()});if(b.match(/^[^<]$/))return b;if(CKEDITOR.env.webkit&&b.indexOf("<div>")>-1){b=b.replace(/^(<div>(<br>|)<\/div>)(?!$|(<div>(<br>|)<\/div>))/g,"<br>").replace(/^(<div>(<br>|)<\/div>){2}(?!$)/g,"<div></div>");b.match(/<div>(<br>|)<\/div>/)&&(b="<p>"+b.replace(/(<div>(<br>|)<\/div>)+/g,function(a){return c(a.split("</div><div>").length+1)})+"</p>");b=b.replace(/<\/div><div>/g,"<br>");
b=b.replace(/<\/?div>/g,"")}if(CKEDITOR.env.gecko&&a.enterMode!=CKEDITOR.ENTER_BR){CKEDITOR.env.gecko&&(b=b.replace(/^<br><br>$/,"<br>"));b.indexOf("<br><br>")>-1&&(b="<p>"+b.replace(/(<br>){2,}/g,function(a){return c(a.length/4)})+"</p>")}return d(a,b)}function g(){var a=new CKEDITOR.htmlParser.filter,b={blockquote:1,dl:1,fieldset:1,h1:1,h2:1,h3:1,h4:1,h5:1,h6:1,ol:1,p:1,table:1,ul:1},c=CKEDITOR.tools.extend({br:0},CKEDITOR.dtd.$inline),d={p:1,br:1,"cke:br":1},g=CKEDITOR.dtd,e=CKEDITOR.tools.extend({area:1,
basefont:1,embed:1,iframe:1,map:1,object:1,param:1},CKEDITOR.dtd.$nonBodyContent,CKEDITOR.dtd.$cdata),o=function(a){delete a.name;a.add(new CKEDITOR.htmlParser.text(" "))},q=function(a){for(var b=a,c;(b=b.next)&&b.name&&b.name.match(/^h\d$/);){c=new CKEDITOR.htmlParser.element("cke:br");c.isEmpty=true;for(a.add(c);c=b.children.shift();)a.add(c)}};a.addRules({elements:{h1:q,h2:q,h3:q,h4:q,h5:q,h6:q,img:function(a){var a=CKEDITOR.tools.trim(a.attributes.alt||""),b=" ";a&&!a.match(/(^http|\.(jpe?g|gif|png))/i)&&
(b=" ["+a+"] ");return new CKEDITOR.htmlParser.text(b)},td:o,th:o,$:function(a){var j=a.name,p;if(e[j])return false;a.attributes={};if(j=="br")return a;if(b[j])a.name="p";else if(c[j])delete a.name;else if(g[j]){p=new CKEDITOR.htmlParser.element("cke:br");p.isEmpty=true;if(CKEDITOR.dtd.$empty[j])return p;a.add(p,0);p=p.clone();p.isEmpty=true;a.add(p);delete a.name}d[a.name]||delete a.name;return a}}},{applyToAll:true});return a}function c(a,b,c){var b=new CKEDITOR.htmlParser.fragment.fromHtml(b),
g=new CKEDITOR.htmlParser.basicWriter;b.writeHtml(g,c);var b=g.getHtml(),b=b.replace(/\s*(<\/?[a-z:]+ ?\/?>)\s*/g,"$1").replace(/(<cke:br \/>){2,}/g,"<cke:br />").replace(/(<cke:br \/>)(<\/?p>|<br \/>)/g,"$2").replace(/(<\/?p>|<br \/>)(<cke:br \/>)/g,"$1").replace(/<(cke:)?br( \/)?>/g,"<br>").replace(/<p><\/p>/g,""),e=0,b=b.replace(/<\/?p>/g,function(a){if(a=="<p>"){if(++e>1)return"</p><p>"}else if(--e>0)return"</p><p>";return a}).replace(/<p><\/p>/g,"");return d(a,b)}function d(a,b){a.enterMode==
CKEDITOR.ENTER_BR?b=b.replace(/(<\/p><p>)+/g,function(a){return CKEDITOR.tools.repeat("<br>",a.length/7*2)}).replace(/<\/?p>/g,""):a.enterMode==CKEDITOR.ENTER_DIV&&(b=b.replace(/<(\/)?p>/g,"<$1div>"));return b}CKEDITOR.plugins.add("clipboard",{requires:"dialog",init:function(d){var f;a(d);CKEDITOR.dialog.add("paste",CKEDITOR.getUrl(this.path+"dialogs/paste.js"));d.on("paste",function(a){var b=a.data.dataValue,c=CKEDITOR.dtd.$block;if(b.indexOf("Apple-")>-1){b=b.replace(/<span class="Apple-converted-space">&nbsp;<\/span>/gi,
" ");a.data.type!="html"&&(b=b.replace(/<span class="Apple-tab-span"[^>]*>([^<]*)<\/span>/gi,function(a,b){return b.replace(/\t/g,"&nbsp;&nbsp; &nbsp;")}));if(b.indexOf('<br class="Apple-interchange-newline">')>-1){a.data.startsWithEOL=1;a.data.preSniffing="html";b=b.replace(/<br class="Apple-interchange-newline">/,"")}b=b.replace(/(<[^>]+) class="Apple-[^"]*"/gi,"$1")}if(b.match(/^<[^<]+cke_(editable|contents)/i)){var d,f,g=new CKEDITOR.dom.element("div");for(g.setHtml(b);g.getChildCount()==1&&(d=
g.getFirst())&&d.type==CKEDITOR.NODE_ELEMENT&&(d.hasClass("cke_editable")||d.hasClass("cke_contents"));)g=f=d;f&&(b=f.getHtml().replace(/<br>$/i,""))}CKEDITOR.env.ie?b=b.replace(/^&nbsp;(?: |\r\n)?<(\w+)/g,function(b,d){if(d.toLowerCase()in c){a.data.preSniffing="html";return"<"+d}return b}):CKEDITOR.env.webkit?b=b.replace(/<\/(\w+)><div><br><\/div>$/,function(b,d){if(d in c){a.data.endsWithEOL=1;return"</"+d+">"}return b}):CKEDITOR.env.gecko&&(b=b.replace(/(\s)<br>$/,"$1"));a.data.dataValue=b},null,
null,3);d.on("paste",function(a){var a=a.data,i=a.type,k=a.dataValue,n,o=d.config.clipboard_defaultContentType||"html";n=i=="html"||a.preSniffing=="html"?"html":e(k);n=="htmlifiedtext"?k=b(d.config,k):i=="text"&&n=="html"&&(k=c(d.config,k,f||(f=g(d))));a.startsWithEOL&&(k='<br data-cke-eol="1">'+k);a.endsWithEOL&&(k=k+'<br data-cke-eol="1">');i=="auto"&&(i=n=="html"||o=="html"?"html":"text");a.type=i;a.dataValue=k;delete a.preSniffing;delete a.startsWithEOL;delete a.endsWithEOL},null,null,6);d.on("paste",
function(a){a=a.data;d.insertHtml(a.dataValue,a.type);setTimeout(function(){d.fire("afterPaste")},0)},null,null,1E3);d.on("pasteDialog",function(a){setTimeout(function(){d.openDialog("paste",a.data)},0)})}})}(),function(){CKEDITOR.plugins.add("panel",{beforeInit:function(a){a.ui.addHandler(CKEDITOR.UI_PANEL,CKEDITOR.ui.panel.handler)}});CKEDITOR.UI_PANEL="panel";CKEDITOR.ui.panel=function(a,b){b&&CKEDITOR.tools.extend(this,b);CKEDITOR.tools.extend(this,{className:"",css:[]});this.id=CKEDITOR.tools.getNextId();
this.document=a;this.isFramed=this.forceIFrame||this.css.length;this._={blocks:{}}};CKEDITOR.ui.panel.handler={create:function(a){return new CKEDITOR.ui.panel(a)}};var a=CKEDITOR.addTemplate("panel",'<div lang="{langCode}" id="{id}" dir={dir} class="cke cke_reset_all {editorId} cke_panel cke_panel {cls} cke_{dir}" style="z-index:{z-index}" role="presentation">{frame}</div>'),e=CKEDITOR.addTemplate("panel-frame",'<iframe id="{id}" class="cke_panel_frame" role="presentation" frameborder="0" src="{src}"></iframe>'),
b=CKEDITOR.addTemplate("panel-frame-inner",'<!DOCTYPE html><html class="cke_panel_container {env}" dir="{dir}" lang="{langCode}"><head>{css}</head><body class="cke_{dir}" style="margin:0;padding:0" onload="{onload}"></body></html>');CKEDITOR.ui.panel.prototype={render:function(g,c){this.getHolderElement=function(){var a=this._.holder;if(!a){if(this.isFramed){var a=this.document.getById(this.id+"_frame"),c=a.getParent(),a=a.getFrameDocument();CKEDITOR.env.iOS&&c.setStyles({overflow:"scroll","-webkit-overflow-scrolling":"touch"});
c=CKEDITOR.tools.addFunction(CKEDITOR.tools.bind(function(){this.isLoaded=true;if(this.onLoad)this.onLoad()},this));a.write(b.output(CKEDITOR.tools.extend({css:CKEDITOR.tools.buildStyleHtml(this.css),onload:"window.parent.CKEDITOR.tools.callFunction("+c+");"},d)));a.getWindow().$.CKEDITOR=CKEDITOR;a.on("keydown",function(a){var b=a.data.getKeystroke(),c=this.document.getById(this.id).getAttribute("dir");this._.onKeyDown&&this._.onKeyDown(b)===false?a.data.preventDefault():(b==27||b==(c=="rtl"?39:
37))&&this.onEscape&&this.onEscape(b)===false&&a.data.preventDefault()},this);a=a.getBody();a.unselectable();CKEDITOR.env.air&&CKEDITOR.tools.callFunction(c)}else a=this.document.getById(this.id);this._.holder=a}return a};var d={editorId:g.id,id:this.id,langCode:g.langCode,dir:g.lang.dir,cls:this.className,frame:"",env:CKEDITOR.env.cssClass,"z-index":g.config.baseFloatZIndex+1};if(this.isFramed){var j=CKEDITOR.env.air?"javascript:void(0)":CKEDITOR.env.ie?"javascript:void(function(){"+encodeURIComponent("document.open();("+
CKEDITOR.tools.fixDomain+")();document.close();")+"}())":"";d.frame=e.output({id:this.id+"_frame",src:j})}j=a.output(d);c&&c.push(j);return j},addBlock:function(a,b){b=this._.blocks[a]=b instanceof CKEDITOR.ui.panel.block?b:new CKEDITOR.ui.panel.block(this.getHolderElement(),b);this._.currentBlock||this.showBlock(a);return b},getBlock:function(a){return this._.blocks[a]},showBlock:function(a){var a=this._.blocks[a],b=this._.currentBlock,d=!this.forceIFrame||CKEDITOR.env.ie?this._.holder:this.document.getById(this.id+
"_frame");b&&b.hide();this._.currentBlock=a;CKEDITOR.fire("ariaWidget",d);a._.focusIndex=-1;this._.onKeyDown=a.onKeyDown&&CKEDITOR.tools.bind(a.onKeyDown,a);a.show();return a},destroy:function(){this.element&&this.element.remove()}};CKEDITOR.ui.panel.block=CKEDITOR.tools.createClass({$:function(a,b){this.element=a.append(a.getDocument().createElement("div",{attributes:{tabindex:-1,"class":"cke_panel_block"},styles:{display:"none"}}));b&&CKEDITOR.tools.extend(this,b);this.element.setAttributes({role:this.attributes.role||
"presentation","aria-label":this.attributes["aria-label"],title:this.attributes.title||this.attributes["aria-label"]});this.keys={};this._.focusIndex=-1;this.element.disableContextMenu()},_:{markItem:function(a){if(a!=-1){a=this.element.getElementsByTag("a").getItem(this._.focusIndex=a);CKEDITOR.env.webkit&&a.getDocument().getWindow().focus();a.focus();this.onMark&&this.onMark(a)}}},proto:{show:function(){this.element.setStyle("display","")},hide:function(){(!this.onHide||this.onHide.call(this)!==
true)&&this.element.setStyle("display","none")},onKeyDown:function(a,b){var d=this.keys[a];switch(d){case "next":for(var e=this._.focusIndex,d=this.element.getElementsByTag("a"),f;f=d.getItem(++e);)if(f.getAttribute("_cke_focus")&&f.$.offsetWidth){this._.focusIndex=e;f.focus();break}if(!f&&!b){this._.focusIndex=-1;return this.onKeyDown(a,1)}return false;case "prev":e=this._.focusIndex;for(d=this.element.getElementsByTag("a");e>0&&(f=d.getItem(--e));){if(f.getAttribute("_cke_focus")&&f.$.offsetWidth){this._.focusIndex=
e;f.focus();break}f=null}if(!f&&!b){this._.focusIndex=d.count();return this.onKeyDown(a,1)}return false;case "click":case "mouseup":e=this._.focusIndex;(f=e>=0&&this.element.getElementsByTag("a").getItem(e))&&(f.$[d]?f.$[d]():f.$["on"+d]());return false}return true}}})}(),CKEDITOR.plugins.add("floatpanel",{requires:"panel"}),function(){function a(a,g,c,d,j){var j=CKEDITOR.tools.genKey(g.getUniqueId(),c.getUniqueId(),a.lang.dir,a.uiColor||"",d.css||"",j||""),f=e[j];if(!f){f=e[j]=new CKEDITOR.ui.panel(g,
d);f.element=c.append(CKEDITOR.dom.element.createFromHtml(f.render(a),g));f.element.setStyles({display:"none",position:"absolute"})}return f}var e={};CKEDITOR.ui.floatPanel=CKEDITOR.tools.createClass({$:function(b,g,c,d){function e(){k.hide()}c.forceIFrame=1;c.toolbarRelated&&b.elementMode==CKEDITOR.ELEMENT_MODE_INLINE&&(g=CKEDITOR.document.getById("cke_"+b.name));var f=g.getDocument(),d=a(b,f,g,c,d||0),h=d.element,i=h.getFirst(),k=this;h.disableContextMenu();this.element=h;this._={editor:b,panel:d,
parentElement:g,definition:c,document:f,iframe:i,children:[],dir:b.lang.dir};b.on("mode",e);b.on("resize",e);if(!CKEDITOR.env.iOS)f.getWindow().on("resize",e)},proto:{addBlock:function(a,g){return this._.panel.addBlock(a,g)},addListBlock:function(a,g){return this._.panel.addListBlock(a,g)},getBlock:function(a){return this._.panel.getBlock(a)},showBlock:function(a,g,c,d,e,f){var h=this._.panel,i=h.showBlock(a);this.allowBlur(false);a=this._.editor.editable();this._.returnFocus=a.hasFocus?a:new CKEDITOR.dom.element(CKEDITOR.document.$.activeElement);
this._.hideTimeout=0;var k=this.element,a=this._.iframe,a=CKEDITOR.env.ie?a:new CKEDITOR.dom.window(a.$.contentWindow),n=k.getDocument(),o=this._.parentElement.getPositionedAncestor(),q=g.getDocumentPosition(n),n=o?o.getDocumentPosition(n):{x:0,y:0},m=this._.dir=="rtl",l=q.x+(d||0)-n.x,p=q.y+(e||0)-n.y;if(m&&(c==1||c==4))l=l+g.$.offsetWidth;else if(!m&&(c==2||c==3))l=l+(g.$.offsetWidth-1);if(c==3||c==4)p=p+(g.$.offsetHeight-1);this._.panel._.offsetParentId=g.getId();k.setStyles({top:p+"px",left:0,
display:""});k.setOpacity(0);k.getFirst().removeStyle("width");this._.editor.focusManager.add(a);if(!this._.blurSet){CKEDITOR.event.useCapture=true;a.on("blur",function(a){function b(){delete this._.returnFocus;this.hide()}if(this.allowBlur()&&a.data.getPhase()==CKEDITOR.EVENT_PHASE_AT_TARGET&&this.visible&&!this._.activeChild)if(CKEDITOR.env.iOS){if(!this._.hideTimeout)this._.hideTimeout=CKEDITOR.tools.setTimeout(b,0,this)}else b.call(this)},this);a.on("focus",function(){this._.focused=true;this.hideChild();
this.allowBlur(true)},this);if(CKEDITOR.env.iOS){a.on("touchstart",function(){clearTimeout(this._.hideTimeout)},this);a.on("touchend",function(){this._.hideTimeout=0;this.focus()},this)}CKEDITOR.event.useCapture=false;this._.blurSet=1}h.onEscape=CKEDITOR.tools.bind(function(a){if(this.onEscape&&this.onEscape(a)===false)return false},this);CKEDITOR.tools.setTimeout(function(){var a=CKEDITOR.tools.bind(function(){k.removeStyle("width");if(i.autoSize){var a=i.element.getDocument(),a=(CKEDITOR.env.webkit?
i.element:a.getBody()).$.scrollWidth;CKEDITOR.env.ie&&(CKEDITOR.env.quirks&&a>0)&&(a=a+((k.$.offsetWidth||0)-(k.$.clientWidth||0)+3));k.setStyle("width",a+10+"px");a=i.element.$.scrollHeight;CKEDITOR.env.ie&&(CKEDITOR.env.quirks&&a>0)&&(a=a+((k.$.offsetHeight||0)-(k.$.clientHeight||0)+3));k.setStyle("height",a+"px");h._.currentBlock.element.setStyle("display","none").removeStyle("display")}else k.removeStyle("height");m&&(l=l-k.$.offsetWidth);k.setStyle("left",l+"px");var b=h.element.getWindow(),
a=k.$.getBoundingClientRect(),b=b.getViewPaneSize(),c=a.width||a.right-a.left,d=a.height||a.bottom-a.top,g=m?a.right:b.width-a.left,e=m?b.width-a.right:a.left;m?g<c&&(l=e>c?l+c:b.width>c?l-a.left:l-a.right+b.width):g<c&&(l=e>c?l-c:b.width>c?l-a.right+b.width:l-a.left);c=a.top;b.height-a.top<d&&(p=c>d?p-d:b.height>d?p-a.bottom+b.height:p-a.top);if(CKEDITOR.env.ie){b=a=new CKEDITOR.dom.element(k.$.offsetParent);b.getName()=="html"&&(b=b.getDocument().getBody());b.getComputedStyle("direction")=="rtl"&&
(l=CKEDITOR.env.ie8Compat?l-k.getDocument().getDocumentElement().$.scrollLeft*2:l-(a.$.scrollWidth-a.$.clientWidth))}var a=k.getFirst(),j;(j=a.getCustomData("activePanel"))&&j.onHide&&j.onHide.call(this,1);a.setCustomData("activePanel",this);k.setStyles({top:p+"px",left:l+"px"});k.setOpacity(1);f&&f()},this);h.isLoaded?a():h.onLoad=a;CKEDITOR.tools.setTimeout(function(){var a=CKEDITOR.env.webkit&&CKEDITOR.document.getWindow().getScrollPosition().y;this.focus();i.element.focus();if(CKEDITOR.env.webkit)CKEDITOR.document.getBody().$.scrollTop=
a;this.allowBlur(true);this._.editor.fire("panelShow",this)},0,this)},CKEDITOR.env.air?200:0,this);this.visible=1;this.onShow&&this.onShow.call(this)},focus:function(){if(CKEDITOR.env.webkit){var a=CKEDITOR.document.getActive();a&&!a.equals(this._.iframe)&&a.$.blur()}(this._.lastFocused||this._.iframe.getFrameDocument().getWindow()).focus()},blur:function(){var a=this._.iframe.getFrameDocument().getActive();a&&a.is("a")&&(this._.lastFocused=a)},hide:function(a){if(this.visible&&(!this.onHide||this.onHide.call(this)!==
true)){this.hideChild();CKEDITOR.env.gecko&&this._.iframe.getFrameDocument().$.activeElement.blur();this.element.setStyle("display","none");this.visible=0;this.element.getFirst().removeCustomData("activePanel");if(a=a&&this._.returnFocus){CKEDITOR.env.webkit&&a.type&&a.getWindow().$.focus();a.focus()}delete this._.lastFocused;this._.editor.fire("panelHide",this)}},allowBlur:function(a){var g=this._.panel;if(a!==void 0)g.allowBlur=a;return g.allowBlur},showAsChild:function(a,g,c,d,e,f){if(!(this._.activeChild==
a&&a._.panel._.offsetParentId==c.getId())){this.hideChild();a.onHide=CKEDITOR.tools.bind(function(){CKEDITOR.tools.setTimeout(function(){this._.focused||this.hide()},0,this)},this);this._.activeChild=a;this._.focused=false;a.showBlock(g,c,d,e,f);this.blur();(CKEDITOR.env.ie7Compat||CKEDITOR.env.ie6Compat)&&setTimeout(function(){a.element.getChild(0).$.style.cssText+=""},100)}},hideChild:function(a){var g=this._.activeChild;if(g){delete g.onHide;delete this._.activeChild;g.hide();a&&this.focus()}}}});
CKEDITOR.on("instanceDestroyed",function(){var a=CKEDITOR.tools.isEmpty(CKEDITOR.instances),g;for(g in e){var c=e[g];a?c.destroy():c.element.hide()}a&&(e={})})}(),CKEDITOR.plugins.add("menu",{requires:"floatpanel",beforeInit:function(a){for(var e=a.config.menu_groups.split(","),b=a._.menuGroups={},g=a._.menuItems={},c=0;c<e.length;c++)b[e[c]]=c+1;a.addMenuGroup=function(a,c){b[a]=c||100};a.addMenuItem=function(a,c){b[c.group]&&(g[a]=new CKEDITOR.menuItem(this,a,c))};a.addMenuItems=function(a){for(var b in a)this.addMenuItem(b,
a[b])};a.getMenuItem=function(a){return g[a]};a.removeMenuItem=function(a){delete g[a]}}}),function(){function a(a){a.sort(function(a,b){return a.group<b.group?-1:a.group>b.group?1:a.order<b.order?-1:a.order>b.order?1:0})}var e='<span class="cke_menuitem"><a id="{id}" class="cke_menubutton cke_menubutton__{name} cke_menubutton_{state} {cls}" href="{href}" title="{title}" tabindex="-1"_cke_focus=1 hidefocus="true" role="{role}" aria-haspopup="{hasPopup}" aria-disabled="{disabled}" {ariaChecked}';CKEDITOR.env.gecko&&
CKEDITOR.env.mac&&(e=e+' onkeypress="return false;"');CKEDITOR.env.gecko&&(e=e+' onblur="this.style.cssText = this.style.cssText;"');var e=e+(' onmouseover="CKEDITOR.tools.callFunction({hoverFn},{index});" onmouseout="CKEDITOR.tools.callFunction({moveOutFn},{index});" '+(CKEDITOR.env.ie?'onclick="return false;" onmouseup':"onclick")+'="CKEDITOR.tools.callFunction({clickFn},{index}); return false;">'),b=CKEDITOR.addTemplate("menuItem",e+'<span class="cke_menubutton_inner"><span class="cke_menubutton_icon"><span class="cke_button_icon cke_button__{iconName}_icon" style="{iconStyle}"></span></span><span class="cke_menubutton_label">{label}</span>{arrowHtml}</span></a></span>'),
g=CKEDITOR.addTemplate("menuArrow",'<span class="cke_menuarrow"><span>{label}</span></span>');CKEDITOR.menu=CKEDITOR.tools.createClass({$:function(a,b){b=this._.definition=b||{};this.id=CKEDITOR.tools.getNextId();this.editor=a;this.items=[];this._.listeners=[];this._.level=b.level||1;var g=CKEDITOR.tools.extend({},b.panel,{css:[CKEDITOR.skin.getPath("editor")],level:this._.level-1,block:{}}),f=g.block.attributes=g.attributes||{};!f.role&&(f.role="menu");this._.panelDefinition=g},_:{onShow:function(){var a=
this.editor.getSelection(),b=a&&a.getStartElement(),g=this.editor.elementPath(),f=this._.listeners;this.removeAll();for(var e=0;e<f.length;e++){var i=f[e](b,a,g);if(i)for(var k in i){var n=this.editor.getMenuItem(k);if(n&&(!n.command||this.editor.getCommand(n.command).state)){n.state=i[k];this.add(n)}}}},onClick:function(a){this.hide();if(a.onClick)a.onClick();else a.command&&this.editor.execCommand(a.command)},onEscape:function(a){var b=this.parent;b?b._.panel.hideChild(1):a==27&&this.hide(1);return false},
onHide:function(){this.onHide&&this.onHide()},showSubMenu:function(a){var b=this._.subMenu,g=this.items[a];if(g=g.getItems&&g.getItems()){if(b)b.removeAll();else{b=this._.subMenu=new CKEDITOR.menu(this.editor,CKEDITOR.tools.extend({},this._.definition,{level:this._.level+1},true));b.parent=this;b._.onClick=CKEDITOR.tools.bind(this._.onClick,this)}for(var f in g){var e=this.editor.getMenuItem(f);if(e){e.state=g[f];b.add(e)}}var i=this._.panel.getBlock(this.id).element.getDocument().getById(this.id+
(""+a));setTimeout(function(){b.show(i,2)},0)}else this._.panel.hideChild(1)}},proto:{add:function(a){if(!a.order)a.order=this.items.length;this.items.push(a)},removeAll:function(){this.items=[]},show:function(b,d,g,f){if(!this.parent){this._.onShow();if(!this.items.length)return}var d=d||(this.editor.lang.dir=="rtl"?2:1),e=this.items,i=this.editor,k=this._.panel,n=this._.element;if(!k){k=this._.panel=new CKEDITOR.ui.floatPanel(this.editor,CKEDITOR.document.getBody(),this._.panelDefinition,this._.level);
k.onEscape=CKEDITOR.tools.bind(function(a){if(this._.onEscape(a)===false)return false},this);k.onShow=function(){k._.panel.getHolderElement().getParent().addClass("cke cke_reset_all")};k.onHide=CKEDITOR.tools.bind(function(){this._.onHide&&this._.onHide()},this);n=k.addBlock(this.id,this._.panelDefinition.block);n.autoSize=true;var o=n.keys;o[40]="next";o[9]="next";o[38]="prev";o[CKEDITOR.SHIFT+9]="prev";o[i.lang.dir=="rtl"?37:39]=CKEDITOR.env.ie?"mouseup":"click";o[32]=CKEDITOR.env.ie?"mouseup":
"click";CKEDITOR.env.ie&&(o[13]="mouseup");n=this._.element=n.element;o=n.getDocument();o.getBody().setStyle("overflow","hidden");o.getElementsByTag("html").getItem(0).setStyle("overflow","hidden");this._.itemOverFn=CKEDITOR.tools.addFunction(function(a){clearTimeout(this._.showSubTimeout);this._.showSubTimeout=CKEDITOR.tools.setTimeout(this._.showSubMenu,i.config.menu_subMenuDelay||400,this,[a])},this);this._.itemOutFn=CKEDITOR.tools.addFunction(function(){clearTimeout(this._.showSubTimeout)},this);
this._.itemClickFn=CKEDITOR.tools.addFunction(function(a){var b=this.items[a];if(b.state==CKEDITOR.TRISTATE_DISABLED)this.hide(1);else if(b.getItems)this._.showSubMenu(a);else this._.onClick(b)},this)}a(e);for(var o=i.elementPath(),o=['<div class="cke_menu'+(o&&o.direction()!=i.lang.dir?" cke_mixed_dir_content":"")+'" role="presentation">'],q=e.length,m=q&&e[0].group,l=0;l<q;l++){var p=e[l];if(m!=p.group){o.push('<div class="cke_menuseparator" role="separator"></div>');m=p.group}p.render(this,l,o)}o.push("</div>");
n.setHtml(o.join(""));CKEDITOR.ui.fire("ready",this);this.parent?this.parent._.panel.showAsChild(k,this.id,b,d,g,f):k.showBlock(this.id,b,d,g,f);i.fire("menuShow",[k])},addListener:function(a){this._.listeners.push(a)},hide:function(a){this._.onHide&&this._.onHide();this._.panel&&this._.panel.hide(a)}}});CKEDITOR.menuItem=CKEDITOR.tools.createClass({$:function(a,b,g){CKEDITOR.tools.extend(this,g,{order:0,className:"cke_menubutton__"+b});this.group=a._.menuGroups[this.group];this.editor=a;this.name=
b},proto:{render:function(a,d,e){var f=a.id+(""+d),h=typeof this.state=="undefined"?CKEDITOR.TRISTATE_OFF:this.state,i="",k=h==CKEDITOR.TRISTATE_ON?"on":h==CKEDITOR.TRISTATE_DISABLED?"disabled":"off";this.role in{menuitemcheckbox:1,menuitemradio:1}&&(i=' aria-checked="'+(h==CKEDITOR.TRISTATE_ON?"true":"false")+'"');var n=this.getItems,o="&#"+(this.editor.lang.dir=="rtl"?"9668":"9658")+";",q=this.name;if(this.icon&&!/\./.test(this.icon))q=this.icon;a={id:f,name:this.name,iconName:q,label:this.label,
cls:this.className||"",state:k,hasPopup:n?"true":"false",disabled:h==CKEDITOR.TRISTATE_DISABLED,title:this.label,href:"javascript:void('"+(this.label||"").replace("'")+"')",hoverFn:a._.itemOverFn,moveOutFn:a._.itemOutFn,clickFn:a._.itemClickFn,index:d,iconStyle:CKEDITOR.skin.getIconStyle(q,this.editor.lang.dir=="rtl",q==this.icon?null:this.icon,this.iconOffset),arrowHtml:n?g.output({label:o}):"",role:this.role?this.role:"menuitem",ariaChecked:i};b.output(a,e)}}})}(),CKEDITOR.config.menu_groups="clipboard,form,tablecell,tablecellproperties,tablerow,tablecolumn,table,anchor,link,image,flash,checkbox,radio,textfield,hiddenfield,imagebutton,button,select,textarea,div",
CKEDITOR.plugins.add("contextmenu",{requires:"menu",onLoad:function(){CKEDITOR.plugins.contextMenu=CKEDITOR.tools.createClass({base:CKEDITOR.menu,$:function(a){this.base.call(this,a,{panel:{className:"cke_menu_panel",attributes:{"aria-label":a.lang.contextmenu.options}}})},proto:{addTarget:function(a,e){a.on("contextmenu",function(a){var a=a.data,d=CKEDITOR.env.webkit?b:CKEDITOR.env.mac?a.$.metaKey:a.$.ctrlKey;if(!e||!d){a.preventDefault();if(CKEDITOR.env.mac&&CKEDITOR.env.webkit){var d=this.editor,
g=(new CKEDITOR.dom.elementPath(a.getTarget(),d.editable())).contains(function(a){return a.hasAttribute("contenteditable")},true);g&&g.getAttribute("contenteditable")=="false"&&d.getSelection().fake(g)}var g=a.getTarget().getDocument(),f=a.getTarget().getDocument().getDocumentElement(),d=!g.equals(CKEDITOR.document),g=g.getWindow().getScrollPosition(),h=d?a.$.clientX:a.$.pageX||g.x+a.$.clientX,i=d?a.$.clientY:a.$.pageY||g.y+a.$.clientY;CKEDITOR.tools.setTimeout(function(){this.open(f,null,h,i)},CKEDITOR.env.ie?
200:0,this)}},this);if(CKEDITOR.env.webkit){var b,g=function(){b=0};a.on("keydown",function(a){b=CKEDITOR.env.mac?a.data.$.metaKey:a.data.$.ctrlKey});a.on("keyup",g);a.on("contextmenu",g)}},open:function(a,e,b,g){this.editor.focus();a=a||CKEDITOR.document.getDocumentElement();this.editor.selectionChange(1);this.show(a,e,b,g)}}})},beforeInit:function(a){var e=a.contextMenu=new CKEDITOR.plugins.contextMenu(a);a.on("contentDom",function(){e.addTarget(a.editable(),a.config.browserContextMenuOnCtrl!==
false)});a.addCommand("contextMenu",{exec:function(){a.contextMenu.open(a.document.getBody())}});a.setKeystroke(CKEDITOR.SHIFT+121,"contextMenu");a.setKeystroke(CKEDITOR.CTRL+CKEDITOR.SHIFT+121,"contextMenu")}}),function(){var a;function e(d,g){function f(a){a=n.list[a];if(a.equals(d.editable())||a.getAttribute("contenteditable")=="true"){var b=d.createRange();b.selectNodeContents(a);b.select()}else d.getSelection().selectElement(a);d.focus()}function e(){k&&k.setHtml(b);delete n.list}var i=d.ui.spaceId("path"),
k,n=d._.elementsPath,o=n.idBase;g.html=g.html+('<span id="'+i+'_label" class="cke_voice_label">'+d.lang.elementspath.eleLabel+'</span><span id="'+i+'" class="cke_path" role="group" aria-labelledby="'+i+'_label">'+b+"</span>");d.on("uiReady",function(){var a=d.ui.space("path");a&&d.focusManager.add(a,1)});n.onClick=f;var q=CKEDITOR.tools.addFunction(f),m=CKEDITOR.tools.addFunction(function(a,b){var c=n.idBase,g,b=new CKEDITOR.dom.event(b);g=d.lang.dir=="rtl";switch(b.getKeystroke()){case g?39:37:case 9:(g=
CKEDITOR.document.getById(c+(a+1)))||(g=CKEDITOR.document.getById(c+"0"));g.focus();return false;case g?37:39:case CKEDITOR.SHIFT+9:(g=CKEDITOR.document.getById(c+(a-1)))||(g=CKEDITOR.document.getById(c+(n.list.length-1)));g.focus();return false;case 27:d.focus();return false;case 13:case 32:f(a);return false}return true});d.on("selectionChange",function(){for(var a=[],f=n.list=[],g=[],e=n.filters,h=true,j=d.elementPath().elements,x,v=j.length;v--;){var r=j[v],y=0;x=r.data("cke-display-name")?r.data("cke-display-name"):
r.data("cke-real-element-type")?r.data("cke-real-element-type"):r.getName();h=r.hasAttribute("contenteditable")?r.getAttribute("contenteditable")=="true":h;!h&&!r.hasAttribute("contenteditable")&&(y=1);for(var z=0;z<e.length;z++){var B=e[z](r,x);if(B===false){y=1;break}x=B||x}if(!y){f.unshift(r);g.unshift(x)}}f=f.length;for(e=0;e<f;e++){x=g[e];h=d.lang.elementspath.eleTitle.replace(/%1/,x);x=c.output({id:o+e,label:h,text:x,jsTitle:"javascript:void('"+x+"')",index:e,keyDownFn:m,clickFn:q});a.unshift(x)}k||
(k=CKEDITOR.document.getById(i));g=k;g.setHtml(a.join("")+b);d.fire("elementsPathUpdate",{space:g})});d.on("readOnly",e);d.on("contentDomUnload",e);d.addCommand("elementsPathFocus",a);d.setKeystroke(CKEDITOR.ALT+122,"elementsPathFocus")}a={editorFocus:false,readOnly:1,exec:function(a){(a=CKEDITOR.document.getById(a._.elementsPath.idBase+"0"))&&a.focus(CKEDITOR.env.ie||CKEDITOR.env.air)}};var b='<span class="cke_path_empty">&nbsp;</span>',g="";CKEDITOR.env.gecko&&CKEDITOR.env.mac&&(g=g+' onkeypress="return false;"');
CKEDITOR.env.gecko&&(g=g+' onblur="this.style.cssText = this.style.cssText;"');var c=CKEDITOR.addTemplate("pathItem",'<a id="{id}" href="{jsTitle}" tabindex="-1" class="cke_path_item" title="{label}"'+g+' hidefocus="true"  onkeydown="return CKEDITOR.tools.callFunction({keyDownFn},{index}, event );" onclick="CKEDITOR.tools.callFunction({clickFn},{index}); return false;" role="button" aria-label="{label}">{text}</a>');CKEDITOR.plugins.add("elementspath",{init:function(a){a._.elementsPath={idBase:"cke_elementspath_"+
CKEDITOR.tools.getNextNumber()+"_",filters:[]};a.on("uiSpace",function(b){b.data.space=="bottom"&&e(a,b.data)})}})}(),function(){function a(a,c){var d,j;c.on("refresh",function(a){var c=[e],d;for(d in a.data.states)c.push(a.data.states[d]);this.setState(CKEDITOR.tools.search(c,b)?b:e)},c,null,100);c.on("exec",function(b){d=a.getSelection();j=d.createBookmarks(1);if(!b.data)b.data={};b.data.done=false},c,null,0);c.on("exec",function(){a.forceNextSelectionCheck();d.selectBookmarks(j)},c,null,100)}var e=
CKEDITOR.TRISTATE_DISABLED,b=CKEDITOR.TRISTATE_OFF;CKEDITOR.plugins.add("indent",{init:function(b){var c=CKEDITOR.plugins.indent.genericDefinition;a(b,b.addCommand("indent",new c(true)));a(b,b.addCommand("outdent",new c));if(b.ui.addButton){b.ui.addButton("Indent",{label:b.lang.indent.indent,command:"indent",directional:true,toolbar:"indent,20"});b.ui.addButton("Outdent",{label:b.lang.indent.outdent,command:"outdent",directional:true,toolbar:"indent,10"})}b.on("dirChanged",function(a){var c=b.createRange(),
f=a.data.node;c.setStartBefore(f);c.setEndAfter(f);for(var e=new CKEDITOR.dom.walker(c),i;i=e.next();)if(i.type==CKEDITOR.NODE_ELEMENT)if(!i.equals(f)&&i.getDirection()){c.setStartAfter(i);e=new CKEDITOR.dom.walker(c)}else{var k=b.config.indentClasses;if(k)for(var n=a.data.dir=="ltr"?["_rtl",""]:["","_rtl"],o=0;o<k.length;o++)if(i.hasClass(k[o]+n[0])){i.removeClass(k[o]+n[0]);i.addClass(k[o]+n[1])}k=i.getStyle("margin-right");n=i.getStyle("margin-left");k?i.setStyle("margin-left",k):i.removeStyle("margin-left");
n?i.setStyle("margin-right",n):i.removeStyle("margin-right")}})}});CKEDITOR.plugins.indent={genericDefinition:function(a){this.isIndent=!!a;this.startDisabled=!this.isIndent},specificDefinition:function(a,b,d){this.name=b;this.editor=a;this.jobs={};this.enterBr=a.config.enterMode==CKEDITOR.ENTER_BR;this.isIndent=!!d;this.relatedGlobal=d?"indent":"outdent";this.indentKey=d?9:CKEDITOR.SHIFT+9;this.database={}},registerCommands:function(a,b){a.on("pluginsLoaded",function(){for(var a in b)(function(a,
b){var c=a.getCommand(b.relatedGlobal),d;for(d in b.jobs){c.on("exec",function(c){if(!c.data.done){a.fire("lockSnapshot");if(b.execJob(a,d))c.data.done=true;a.fire("unlockSnapshot");CKEDITOR.dom.element.clearAllMarkers(b.database)}},this,null,d);c.on("refresh",function(c){if(!c.data.states)c.data.states={};c.data.states[b.name+"@"+d]=b.refreshJob(a,d,c.data.path)},this,null,d)}a.addFeature(b)})(this,b[a])})}};CKEDITOR.plugins.indent.genericDefinition.prototype={context:"p",exec:function(){}};CKEDITOR.plugins.indent.specificDefinition.prototype=
{execJob:function(a,b){var d=this.jobs[b];if(d.state!=e)return d.exec.call(this,a)},refreshJob:function(a,b,d){b=this.jobs[b];b.state=a.activeFilter.checkFeature(this)?b.refresh.call(this,a,d):e;return b.state},getContext:function(a){return a.contains(this.context)}}}(),function(){function a(a){function c(e){for(var h=q.startContainer,m=q.endContainer;h&&!h.getParent().equals(e);)h=h.getParent();for(;m&&!m.getParent().equals(e);)m=m.getParent();if(!h||!m)return false;for(var l=h,h=[],o=false;!o;){l.equals(m)&&
(o=true);h.push(l);l=l.getNext()}if(h.length<1)return false;l=e.getParents(true);for(m=0;m<l.length;m++)if(l[m].getName&&j[l[m].getName()]){e=l[m];break}for(var l=d.isIndent?1:-1,m=h[0],h=h[h.length-1],o=CKEDITOR.plugins.list.listToArray(e,g),x=o[h.getCustomData("listarray_index")].indent,m=m.getCustomData("listarray_index");m<=h.getCustomData("listarray_index");m++){o[m].indent=o[m].indent+l;if(l>0){var v=o[m].parent;o[m].parent=new CKEDITOR.dom.element(v.getName(),v.getDocument())}}for(m=h.getCustomData("listarray_index")+
1;m<o.length&&o[m].indent>x;m++)o[m].indent=o[m].indent+l;h=CKEDITOR.plugins.list.arrayToList(o,g,null,a.config.enterMode,e.getDirection());if(!d.isIndent){var r;if((r=e.getParent())&&r.is("li"))for(var l=h.listNode.getChildren(),y=[],z,m=l.count()-1;m>=0;m--)(z=l.getItem(m))&&(z.is&&z.is("li"))&&y.push(z)}h&&h.listNode.replace(e);if(y&&y.length)for(m=0;m<y.length;m++){for(z=e=y[m];(z=z.getNext())&&z.is&&z.getName()in j;){CKEDITOR.env.needsNbspFiller&&!e.getFirst(b)&&e.append(q.document.createText(" "));
e.append(z)}e.insertAfter(r)}h&&a.fire("contentDomInvalidated");return true}for(var d=this,g=this.database,j=this.context,o=a.getSelection(),o=(o&&o.getRanges()).createIterator(),q;q=o.getNextRange();){for(var m=q.getCommonAncestor();m&&!(m.type==CKEDITOR.NODE_ELEMENT&&j[m.getName()]);)m=m.getParent();m||(m=q.startPath().contains(j))&&q.setEndAt(m,CKEDITOR.POSITION_BEFORE_END);if(!m){var l=q.getEnclosedNode();if(l&&l.type==CKEDITOR.NODE_ELEMENT&&l.getName()in j){q.setStartAt(l,CKEDITOR.POSITION_AFTER_START);
q.setEndAt(l,CKEDITOR.POSITION_BEFORE_END);m=l}}if(m&&q.startContainer.type==CKEDITOR.NODE_ELEMENT&&q.startContainer.getName()in j){l=new CKEDITOR.dom.walker(q);l.evaluator=e;q.startContainer=l.next()}if(m&&q.endContainer.type==CKEDITOR.NODE_ELEMENT&&q.endContainer.getName()in j){l=new CKEDITOR.dom.walker(q);l.evaluator=e;q.endContainer=l.previous()}if(m)return c(m)}return 0}function e(a){return a.type==CKEDITOR.NODE_ELEMENT&&a.is("li")}function b(a){return g(a)&&c(a)}var g=CKEDITOR.dom.walker.whitespaces(true),
c=CKEDITOR.dom.walker.bookmark(false,true),d=CKEDITOR.TRISTATE_DISABLED,j=CKEDITOR.TRISTATE_OFF;CKEDITOR.plugins.add("indentlist",{requires:"indent",init:function(b){function c(b){e.specificDefinition.apply(this,arguments);this.requiredContent=["ul","ol"];b.on("key",function(a){if(b.mode=="wysiwyg"&&a.data.keyCode==this.indentKey){var c=this.getContext(b.elementPath());if(c&&(!this.isIndent||!CKEDITOR.plugins.indentList.firstItemInPath(this.context,b.elementPath(),c))){b.execCommand(this.relatedGlobal);
a.cancel()}}},this);this.jobs[this.isIndent?10:30]={refresh:this.isIndent?function(a,b){var c=this.getContext(b),f=CKEDITOR.plugins.indentList.firstItemInPath(this.context,b,c);return!c||!this.isIndent||f?d:j}:function(a,b){return!this.getContext(b)||this.isIndent?d:j},exec:CKEDITOR.tools.bind(a,this)}}var e=CKEDITOR.plugins.indent;e.registerCommands(b,{indentlist:new c(b,"indentlist",true),outdentlist:new c(b,"outdentlist")});CKEDITOR.tools.extend(c.prototype,e.specificDefinition.prototype,{context:{ol:1,
ul:1}})}});CKEDITOR.plugins.indentList={};CKEDITOR.plugins.indentList.firstItemInPath=function(a,b,c){var d=b.contains(e);c||(c=b.contains(a));return c&&d&&d.equals(c.getFirst(e))}}(),function(){function a(a,b,c){function d(c){if((j=i[c?"getFirst":"getLast"]())&&(!j.is||!j.isBlockBoundary())&&(k=b.root[c?"getPrevious":"getNext"](CKEDITOR.dom.walker.invisible(true)))&&(!k.is||!k.isBlockBoundary({br:1})))a.document.createElement("br")[c?"insertBefore":"insertAfter"](j)}for(var f=CKEDITOR.plugins.list.listToArray(b.root,
c),e=[],g=0;g<b.contents.length;g++){var h=b.contents[g];if((h=h.getAscendant("li",true))&&!h.getCustomData("list_item_processed")){e.push(h);CKEDITOR.dom.element.setMarker(c,h,"list_item_processed",true)}}h=null;for(g=0;g<e.length;g++){h=e[g].getCustomData("listarray_index");f[h].indent=-1}for(g=h+1;g<f.length;g++)if(f[g].indent>f[g-1].indent+1){e=f[g-1].indent+1-f[g].indent;for(h=f[g].indent;f[g]&&f[g].indent>=h;){f[g].indent=f[g].indent+e;g++}g--}var i=CKEDITOR.plugins.list.arrayToList(f,c,null,
a.config.enterMode,b.root.getAttribute("dir")).listNode,j,k;d(true);d();i.replace(b.root);a.fire("contentDomInvalidated")}function e(a,b){this.name=a;this.context=this.type=b;this.allowedContent=b+" li";this.requiredContent=b}function b(a,b,c,d){for(var f,g;f=a[d?"getLast":"getFirst"](q);){(g=f.getDirection(1))!==b.getDirection(1)&&f.setAttribute("dir",g);f.remove();c?f[d?"insertBefore":"insertAfter"](c):b.append(f,d)}}function g(a){function c(d){var f=a[d?"getPrevious":"getNext"](k);if(f&&f.type==
CKEDITOR.NODE_ELEMENT&&f.is(a.getName())){b(a,f,null,!d);a.remove();a=f}}c();c(1)}function c(a){return a.type==CKEDITOR.NODE_ELEMENT&&(a.getName()in CKEDITOR.dtd.$block||a.getName()in CKEDITOR.dtd.$listItem)&&CKEDITOR.dtd[a.getName()]["#"]}function d(a,c,d){a.fire("saveSnapshot");d.enlarge(CKEDITOR.ENLARGE_LIST_ITEM_CONTENTS);var f=d.extractContents();c.trim(false,true);var e=c.createBookmark(),h=new CKEDITOR.dom.elementPath(c.startContainer),i=h.block,h=h.lastElement.getAscendant("li",1)||i,q=new CKEDITOR.dom.elementPath(d.startContainer),
o=q.contains(CKEDITOR.dtd.$listItem),q=q.contains(CKEDITOR.dtd.$list);if(i)(i=i.getBogus())&&i.remove();else if(q)(i=q.getPrevious(k))&&n(i)&&i.remove();(i=f.getLast())&&(i.type==CKEDITOR.NODE_ELEMENT&&i.is("br"))&&i.remove();(i=c.startContainer.getChild(c.startOffset))?f.insertBefore(i):c.startContainer.append(f);if(o)if(f=j(o))if(h.contains(o)){b(f,o.getParent(),o);f.remove()}else h.append(f);for(;d.checkStartOfBlock()&&d.checkEndOfBlock();){q=d.startPath();f=q.block;if(!f)break;if(f.is("li")){h=
f.getParent();f.equals(h.getLast(k))&&f.equals(h.getFirst(k))&&(f=h)}d.moveToPosition(f,CKEDITOR.POSITION_BEFORE_START);f.remove()}d=d.clone();f=a.editable();d.setEndAt(f,CKEDITOR.POSITION_BEFORE_END);d=new CKEDITOR.dom.walker(d);d.evaluator=function(a){return k(a)&&!n(a)};(d=d.next())&&(d.type==CKEDITOR.NODE_ELEMENT&&d.getName()in CKEDITOR.dtd.$list)&&g(d);c.moveToBookmark(e);c.select();a.fire("saveSnapshot")}function j(a){return(a=a.getLast(k))&&a.type==CKEDITOR.NODE_ELEMENT&&a.getName()in f?a:
null}var f={ol:1,ul:1},h=CKEDITOR.dom.walker.whitespaces(),i=CKEDITOR.dom.walker.bookmark(),k=function(a){return!(h(a)||i(a))},n=CKEDITOR.dom.walker.bogus();CKEDITOR.plugins.list={listToArray:function(a,b,c,d,g){if(!f[a.getName()])return[];d||(d=0);c||(c=[]);for(var e=0,h=a.getChildCount();e<h;e++){var i=a.getChild(e);i.type==CKEDITOR.NODE_ELEMENT&&i.getName()in CKEDITOR.dtd.$list&&CKEDITOR.plugins.list.listToArray(i,b,c,d+1);if(i.$.nodeName.toLowerCase()=="li"){var j={parent:a,indent:d,element:i,
contents:[]};if(g)j.grandparent=g;else{j.grandparent=a.getParent();if(j.grandparent&&j.grandparent.$.nodeName.toLowerCase()=="li")j.grandparent=j.grandparent.getParent()}b&&CKEDITOR.dom.element.setMarker(b,i,"listarray_index",c.length);c.push(j);for(var k=0,n=i.getChildCount(),q;k<n;k++){q=i.getChild(k);q.type==CKEDITOR.NODE_ELEMENT&&f[q.getName()]?CKEDITOR.plugins.list.listToArray(q,b,c,d+1,j.grandparent):j.contents.push(q)}}}return c},arrayToList:function(a,b,c,d,g){c||(c=0);if(!a||a.length<c+1)return null;
for(var e,h=a[c].parent.getDocument(),j=new CKEDITOR.dom.documentFragment(h),n=null,q=c,o=Math.max(a[c].indent,0),z=null,B,A,G=d==CKEDITOR.ENTER_P?"p":"div";;){var D=a[q];e=D.grandparent;B=D.element.getDirection(1);if(D.indent==o){if(!n||a[q].parent.getName()!=n.getName()){n=a[q].parent.clone(false,1);g&&n.setAttribute("dir",g);j.append(n)}z=n.append(D.element.clone(0,1));B!=n.getDirection(1)&&z.setAttribute("dir",B);for(e=0;e<D.contents.length;e++)z.append(D.contents[e].clone(1,1));q++}else if(D.indent==
Math.max(o,0)+1){D=a[q-1].element.getDirection(1);q=CKEDITOR.plugins.list.arrayToList(a,null,q,d,D!=B?B:null);!z.getChildCount()&&(CKEDITOR.env.needsNbspFiller&&h.$.documentMode<=7)&&z.append(h.createText(" "));z.append(q.listNode);q=q.nextIndex}else if(D.indent==-1&&!c&&e){if(f[e.getName()]){z=D.element.clone(false,true);B!=e.getDirection(1)&&z.setAttribute("dir",B)}else z=new CKEDITOR.dom.documentFragment(h);var n=e.getDirection(1)!=B,J=D.element,C=J.getAttribute("class"),F=J.getAttribute("style"),
E=z.type==CKEDITOR.NODE_DOCUMENT_FRAGMENT&&(d!=CKEDITOR.ENTER_BR||n||F||C),I,L=D.contents.length,H;for(e=0;e<L;e++){I=D.contents[e];if(i(I)&&L>1)E?H=I.clone(1,1):z.append(I.clone(1,1));else if(I.type==CKEDITOR.NODE_ELEMENT&&I.isBlockBoundary()){n&&!I.getDirection()&&I.setAttribute("dir",B);A=I;var Q=J.getAttribute("style");Q&&A.setAttribute("style",Q.replace(/([^;])$/,"$1;")+(A.getAttribute("style")||""));C&&I.addClass(C);A=null;if(H){z.append(H);H=null}z.append(I.clone(1,1))}else if(E){if(!A){A=
h.createElement(G);z.append(A);n&&A.setAttribute("dir",B)}F&&A.setAttribute("style",F);C&&A.setAttribute("class",C);if(H){A.append(H);H=null}A.append(I.clone(1,1))}else z.append(I.clone(1,1))}if(H){(A||z).append(H);H=null}if(z.type==CKEDITOR.NODE_DOCUMENT_FRAGMENT&&q!=a.length-1){if(CKEDITOR.env.needsBrFiller)(B=z.getLast())&&(B.type==CKEDITOR.NODE_ELEMENT&&B.is("br"))&&B.remove();B=z.getLast(k);(!B||!(B.type==CKEDITOR.NODE_ELEMENT&&B.is(CKEDITOR.dtd.$block)))&&z.append(h.createElement("br"))}B=z.$.nodeName.toLowerCase();
(B=="div"||B=="p")&&z.appendBogus();j.append(z);n=null;q++}else return null;A=null;if(a.length<=q||Math.max(a[q].indent,0)<o)break}if(b)for(a=j.getFirst();a;){if(a.type==CKEDITOR.NODE_ELEMENT){CKEDITOR.dom.element.clearMarkers(b,a);if(a.getName()in CKEDITOR.dtd.$listItem){c=a;h=g=d=void 0;if(d=c.getDirection()){for(g=c.getParent();g&&!(h=g.getDirection());)g=g.getParent();d==h&&c.removeAttribute("dir")}}}a=a.getNextSourceNode()}return{listNode:j,nextIndex:q}}};var o=/^h[1-6]$/,q=CKEDITOR.dom.walker.nodeType(CKEDITOR.NODE_ELEMENT);
e.prototype={exec:function(b){this.refresh(b,b.elementPath());var c=b.config,d=b.getSelection(),e=d&&d.getRanges();if(this.state==CKEDITOR.TRISTATE_OFF){var h=b.editable();if(h.getFirst(k)){var i=e.length==1&&e[0];(c=i&&i.getEnclosedNode())&&(c.is&&this.type==c.getName())&&this.setState(CKEDITOR.TRISTATE_ON)}else{c.enterMode==CKEDITOR.ENTER_BR?h.appendBogus():e[0].fixBlock(1,c.enterMode==CKEDITOR.ENTER_P?"p":"div");d.selectRanges(e)}}for(var c=d.createBookmarks(true),h=[],j={},e=e.createIterator(),
n=0;(i=e.getNextRange())&&++n;){var q=i.getBoundaryNodes(),r=q.startNode,y=q.endNode;r.type==CKEDITOR.NODE_ELEMENT&&r.getName()=="td"&&i.setStartAt(q.startNode,CKEDITOR.POSITION_AFTER_START);y.type==CKEDITOR.NODE_ELEMENT&&y.getName()=="td"&&i.setEndAt(q.endNode,CKEDITOR.POSITION_BEFORE_END);i=i.createIterator();for(i.forceBrBreak=this.state==CKEDITOR.TRISTATE_OFF;q=i.getNextParagraph();)if(!q.getCustomData("list_block")){CKEDITOR.dom.element.setMarker(j,q,"list_block",1);for(var z=b.elementPath(q),
r=z.elements,y=0,z=z.blockLimit,B,A=r.length-1;A>=0&&(B=r[A]);A--)if(f[B.getName()]&&z.contains(B)){z.removeCustomData("list_group_object_"+n);if(r=B.getCustomData("list_group_object"))r.contents.push(q);else{r={root:B,contents:[q]};h.push(r);CKEDITOR.dom.element.setMarker(j,B,"list_group_object",r)}y=1;break}if(!y){y=z;if(y.getCustomData("list_group_object_"+n))y.getCustomData("list_group_object_"+n).contents.push(q);else{r={root:y,contents:[q]};CKEDITOR.dom.element.setMarker(j,y,"list_group_object_"+
n,r);h.push(r)}}}}for(B=[];h.length>0;){r=h.shift();if(this.state==CKEDITOR.TRISTATE_OFF)if(f[r.root.getName()]){e=b;n=r;r=j;i=B;y=CKEDITOR.plugins.list.listToArray(n.root,r);z=[];for(q=0;q<n.contents.length;q++){A=n.contents[q];if((A=A.getAscendant("li",true))&&!A.getCustomData("list_item_processed")){z.push(A);CKEDITOR.dom.element.setMarker(r,A,"list_item_processed",true)}}for(var A=n.root.getDocument(),G=void 0,D=void 0,q=0;q<z.length;q++){var J=z[q].getCustomData("listarray_index"),G=y[J].parent;
if(!G.is(this.type)){D=A.createElement(this.type);G.copyAttributes(D,{start:1,type:1});D.removeStyle("list-style-type");y[J].parent=D}}r=CKEDITOR.plugins.list.arrayToList(y,r,null,e.config.enterMode);y=void 0;z=r.listNode.getChildCount();for(q=0;q<z&&(y=r.listNode.getChild(q));q++)y.getName()==this.type&&i.push(y);r.listNode.replace(n.root);e.fire("contentDomInvalidated")}else{y=b;q=r;i=B;z=q.contents;e=q.root.getDocument();n=[];if(z.length==1&&z[0].equals(q.root)){r=e.createElement("div");z[0].moveChildren&&
z[0].moveChildren(r);z[0].append(r);z[0]=r}q=q.contents[0].getParent();for(A=0;A<z.length;A++)q=q.getCommonAncestor(z[A].getParent());G=y.config.useComputedState;y=r=void 0;G=G===void 0||G;for(A=0;A<z.length;A++)for(D=z[A];J=D.getParent();){if(J.equals(q)){n.push(D);!y&&D.getDirection()&&(y=1);D=D.getDirection(G);r!==null&&(r=r&&r!=D?null:D);break}D=J}if(!(n.length<1)){z=n[n.length-1].getNext();A=e.createElement(this.type);i.push(A);for(G=i=void 0;n.length;){i=n.shift();G=e.createElement("li");if(i.is("pre")||
o.test(i.getName())||i.getAttribute("contenteditable")=="false")i.appendTo(G);else{i.copyAttributes(G);if(r&&i.getDirection()){G.removeStyle("direction");G.removeAttribute("dir")}i.moveChildren(G);i.remove()}G.appendTo(A)}r&&y&&A.setAttribute("dir",r);z?A.insertBefore(z):A.appendTo(q)}}else this.state==CKEDITOR.TRISTATE_ON&&f[r.root.getName()]&&a.call(this,b,r,j)}for(A=0;A<B.length;A++)g(B[A]);CKEDITOR.dom.element.clearAllMarkers(j);d.selectBookmarks(c);b.focus()},refresh:function(a,b){var c=b.contains(f,
1),d=b.blockLimit||b.root;c&&d.contains(c)?this.setState(c.is(this.type)?CKEDITOR.TRISTATE_ON:CKEDITOR.TRISTATE_OFF):this.setState(CKEDITOR.TRISTATE_OFF)}};CKEDITOR.plugins.add("list",{requires:"indentlist",init:function(a){if(!a.blockless){a.addCommand("numberedlist",new e("numberedlist","ol"));a.addCommand("bulletedlist",new e("bulletedlist","ul"));if(a.ui.addButton){a.ui.addButton("NumberedList",{label:a.lang.list.numberedlist,command:"numberedlist",directional:true,toolbar:"list,10"});a.ui.addButton("BulletedList",
{label:a.lang.list.bulletedlist,command:"bulletedlist",directional:true,toolbar:"list,20"})}a.on("key",function(b){var e=b.data.domEvent.getKey(),g;if(a.mode=="wysiwyg"&&e in{8:1,46:1}){var h=a.getSelection().getRanges()[0],i=h&&h.startPath();if(h&&h.collapsed){var q=e==8,o=a.editable(),v=new CKEDITOR.dom.walker(h.clone());v.evaluator=function(a){return k(a)&&!n(a)};v.guard=function(a,b){return!(b&&a.type==CKEDITOR.NODE_ELEMENT&&a.is("table"))};e=h.clone();if(q){var r;if((r=i.contains(f))&&h.checkBoundaryOfElement(r,
CKEDITOR.START)&&(r=r.getParent())&&r.is("li")&&(r=j(r))){g=r;r=r.getPrevious(k);e.moveToPosition(r&&n(r)?r:g,CKEDITOR.POSITION_BEFORE_START)}else{v.range.setStartAt(o,CKEDITOR.POSITION_AFTER_START);v.range.setEnd(h.startContainer,h.startOffset);if((r=v.previous())&&r.type==CKEDITOR.NODE_ELEMENT&&(r.getName()in f||r.is("li"))){if(!r.is("li")){v.range.selectNodeContents(r);v.reset();v.evaluator=c;r=v.previous()}g=r;e.moveToElementEditEnd(g)}}if(g){d(a,e,h);b.cancel()}else if((e=i.contains(f))&&h.checkBoundaryOfElement(e,
CKEDITOR.START)){g=e.getFirst(k);if(h.checkBoundaryOfElement(g,CKEDITOR.START)){r=e.getPrevious(k);if(j(g)){if(r){h.moveToElementEditEnd(r);h.select()}}else a.execCommand("outdent");b.cancel()}}}else if(g=i.contains("li")){v.range.setEndAt(o,CKEDITOR.POSITION_BEFORE_END);g=(i=g.getLast(k))&&c(i)?i:g;o=0;if((r=v.next())&&r.type==CKEDITOR.NODE_ELEMENT&&r.getName()in f&&r.equals(i)){o=1;r=v.next()}else h.checkBoundaryOfElement(g,CKEDITOR.END)&&(o=1);if(o&&r){h=h.clone();h.moveToElementEditStart(r);d(a,
e,h);b.cancel()}}else{v.range.setEndAt(o,CKEDITOR.POSITION_BEFORE_END);if((r=v.next())&&r.type==CKEDITOR.NODE_ELEMENT&&r.is(f)){r=r.getFirst(k);if(i.block&&h.checkStartOfBlock()&&h.checkEndOfBlock()){i.block.remove();h.moveToElementEditStart(r);h.select()}else if(j(r)){h.moveToElementEditStart(r);h.select()}else{h=h.clone();h.moveToElementEditStart(r);d(a,e,h)}b.cancel()}}setTimeout(function(){a.selectionChange(1)})}}})}}})}(),function(){function a(a,b,c){c=a.config.forceEnterMode||c;if(a.mode=="wysiwyg"){if(!b)b=
a.activeEnterMode;if(!a.elementPath().isContextFor("p")){b=CKEDITOR.ENTER_BR;c=1}a.fire("saveSnapshot");b==CKEDITOR.ENTER_BR?j(a,b,null,c):f(a,b,null,c);a.fire("saveSnapshot")}}function e(a){for(var a=a.getSelection().getRanges(true),b=a.length-1;b>0;b--)a[b].deleteContents();return a[0]}function b(a){var b=a.startContainer.getAscendant(function(a){return a.type==CKEDITOR.NODE_ELEMENT&&a.getAttribute("contenteditable")=="true"},true);if(a.root.equals(b))return a;b=new CKEDITOR.dom.range(b);b.moveToRange(a);
return b}CKEDITOR.plugins.add("enterkey",{init:function(b){b.addCommand("enter",{modes:{wysiwyg:1},editorFocus:false,exec:function(b){a(b)}});b.addCommand("shiftEnter",{modes:{wysiwyg:1},editorFocus:false,exec:function(b){a(b,b.activeShiftEnterMode,1)}});b.setKeystroke([[13,"enter"],[CKEDITOR.SHIFT+13,"shiftEnter"]])}});var g=CKEDITOR.dom.walker.whitespaces(),c=CKEDITOR.dom.walker.bookmark();CKEDITOR.plugins.enterkey={enterBlock:function(a,d,f,o){if(f=f||e(a)){var f=b(f),q=f.document,m=f.checkStartOfBlock(),
l=f.checkEndOfBlock(),p=a.elementPath(f.startContainer),t=p.block,s=d==CKEDITOR.ENTER_DIV?"div":"p",u;if(m&&l){if(t&&(t.is("li")||t.getParent().is("li"))){t.is("li")||(t=t.getParent());f=t.getParent();u=f.getParent();var o=!t.hasPrevious(),w=!t.hasNext(),s=a.getSelection(),x=s.createBookmarks(),m=t.getDirection(1),l=t.getAttribute("class"),v=t.getAttribute("style"),r=u.getDirection(1)!=m,a=a.enterMode!=CKEDITOR.ENTER_BR||r||v||l;if(u.is("li"))if(o||w)t[o?"insertBefore":"insertAfter"](u);else t.breakParent(u);
else{if(a){if(p.block.is("li")){u=q.createElement(d==CKEDITOR.ENTER_P?"p":"div");r&&u.setAttribute("dir",m);v&&u.setAttribute("style",v);l&&u.setAttribute("class",l);t.moveChildren(u)}else u=p.block;if(o||w)u[o?"insertBefore":"insertAfter"](f);else{t.breakParent(f);u.insertAfter(f)}}else{t.appendBogus(true);if(o||w)for(;q=t[o?"getFirst":"getLast"]();)q[o?"insertBefore":"insertAfter"](f);else for(t.breakParent(f);q=t.getLast();)q.insertAfter(f)}t.remove()}s.selectBookmarks(x);return}if(t&&t.getParent().is("blockquote")){t.breakParent(t.getParent());
t.getPrevious().getFirst(CKEDITOR.dom.walker.invisible(1))||t.getPrevious().remove();t.getNext().getFirst(CKEDITOR.dom.walker.invisible(1))||t.getNext().remove();f.moveToElementEditStart(t);f.select();return}}else if(t&&t.is("pre")&&!l){j(a,d,f,o);return}if(m=f.splitBlock(s)){d=m.previousBlock;t=m.nextBlock;p=m.wasStartOfBlock;a=m.wasEndOfBlock;if(t){x=t.getParent();if(x.is("li")){t.breakParent(x);t.move(t.getNext(),1)}}else if(d&&(x=d.getParent())&&x.is("li")){d.breakParent(x);x=d.getNext();f.moveToElementEditStart(x);
d.move(d.getPrevious())}if(!p&&!a){if(t.is("li")){u=f.clone();u.selectNodeContents(t);u=new CKEDITOR.dom.walker(u);u.evaluator=function(a){return!(c(a)||g(a)||a.type==CKEDITOR.NODE_ELEMENT&&a.getName()in CKEDITOR.dtd.$inline&&!(a.getName()in CKEDITOR.dtd.$empty))};(x=u.next())&&(x.type==CKEDITOR.NODE_ELEMENT&&x.is("ul","ol"))&&(CKEDITOR.env.needsBrFiller?q.createElement("br"):q.createText(" ")).insertBefore(x)}t&&f.moveToElementEditStart(t)}else{if(d){if(d.is("li")||!h.test(d.getName())&&!d.is("pre"))u=
d.clone()}else t&&(u=t.clone());if(u)o&&!u.is("li")&&u.renameNode(s);else if(x&&x.is("li"))u=x;else{u=q.createElement(s);d&&(w=d.getDirection())&&u.setAttribute("dir",w)}if(q=m.elementPath){o=0;for(s=q.elements.length;o<s;o++){x=q.elements[o];if(x.equals(q.block)||x.equals(q.blockLimit))break;if(CKEDITOR.dtd.$removeEmpty[x.getName()]){x=x.clone();u.moveChildren(x);u.append(x)}}}u.appendBogus();u.getParent()||f.insertNode(u);u.is("li")&&u.removeAttribute("value");if(CKEDITOR.env.ie&&p&&(!a||!d.getChildCount())){f.moveToElementEditStart(a?
d:u);f.select()}f.moveToElementEditStart(p&&!a?t:u)}f.select();f.scrollIntoView()}}},enterBr:function(a,b,c,d){if(c=c||e(a)){var g=c.document,j=c.checkEndOfBlock(),l=new CKEDITOR.dom.elementPath(a.getSelection().getStartElement()),p=l.block,t=p&&l.block.getName();if(!d&&t=="li")f(a,b,c,d);else{if(!d&&j&&h.test(t))if(j=p.getDirection()){g=g.createElement("div");g.setAttribute("dir",j);g.insertAfter(p);c.setStart(g,0)}else{g.createElement("br").insertAfter(p);CKEDITOR.env.gecko&&g.createText("").insertAfter(p);
c.setStartAt(p.getNext(),CKEDITOR.env.ie?CKEDITOR.POSITION_BEFORE_START:CKEDITOR.POSITION_AFTER_START)}else{a=t=="pre"&&CKEDITOR.env.ie&&CKEDITOR.env.version<8?g.createText("\r"):g.createElement("br");c.deleteContents();c.insertNode(a);if(CKEDITOR.env.needsBrFiller){g.createText("﻿").insertAfter(a);j&&(p||l.blockLimit).appendBogus();a.getNext().$.nodeValue="";c.setStartAt(a.getNext(),CKEDITOR.POSITION_AFTER_START)}else c.setStartAt(a,CKEDITOR.POSITION_AFTER_END)}c.collapse(true);c.select();c.scrollIntoView()}}}};
var d=CKEDITOR.plugins.enterkey,j=d.enterBr,f=d.enterBlock,h=/^h[1-6]$/}(),function(){function a(a,b){var g={},c=[],d={nbsp:" ",shy:"­",gt:">",lt:"<",amp:"&",apos:"'",quot:'"'},a=a.replace(/\b(nbsp|shy|gt|lt|amp|apos|quot)(?:,|$)/g,function(a,f){var e=b?"&"+f+";":d[f];g[e]=b?d[f]:"&"+f+";";c.push(e);return""});if(!b&&a){var a=a.split(","),j=document.createElement("div"),f;j.innerHTML="&"+a.join(";&")+";";f=j.innerHTML;j=null;for(j=0;j<f.length;j++){var h=f.charAt(j);g[h]="&"+a[j]+";";c.push(h)}}g.regex=
c.join(b?"|":"");return g}CKEDITOR.plugins.add("entities",{afterInit:function(e){function b(a){return h[a]}function g(a){return c.entities_processNumerical=="force"||!j[a]?"&#"+a.charCodeAt(0)+";":j[a]}var c=e.config;if(e=(e=e.dataProcessor)&&e.htmlFilter){var d=[];c.basicEntities!==false&&d.push("nbsp,gt,lt,amp");if(c.entities){d.length&&d.push("quot,iexcl,cent,pound,curren,yen,brvbar,sect,uml,copy,ordf,laquo,not,shy,reg,macr,deg,plusmn,sup2,sup3,acute,micro,para,middot,cedil,sup1,ordm,raquo,frac14,frac12,frac34,iquest,times,divide,fnof,bull,hellip,prime,Prime,oline,frasl,weierp,image,real,trade,alefsym,larr,uarr,rarr,darr,harr,crarr,lArr,uArr,rArr,dArr,hArr,forall,part,exist,empty,nabla,isin,notin,ni,prod,sum,minus,lowast,radic,prop,infin,ang,and,or,cap,cup,int,there4,sim,cong,asymp,ne,equiv,le,ge,sub,sup,nsub,sube,supe,oplus,otimes,perp,sdot,lceil,rceil,lfloor,rfloor,lang,rang,loz,spades,clubs,hearts,diams,circ,tilde,ensp,emsp,thinsp,zwnj,zwj,lrm,rlm,ndash,mdash,lsquo,rsquo,sbquo,ldquo,rdquo,bdquo,dagger,Dagger,permil,lsaquo,rsaquo,euro");
c.entities_latin&&d.push("Agrave,Aacute,Acirc,Atilde,Auml,Aring,AElig,Ccedil,Egrave,Eacute,Ecirc,Euml,Igrave,Iacute,Icirc,Iuml,ETH,Ntilde,Ograve,Oacute,Ocirc,Otilde,Ouml,Oslash,Ugrave,Uacute,Ucirc,Uuml,Yacute,THORN,szlig,agrave,aacute,acirc,atilde,auml,aring,aelig,ccedil,egrave,eacute,ecirc,euml,igrave,iacute,icirc,iuml,eth,ntilde,ograve,oacute,ocirc,otilde,ouml,oslash,ugrave,uacute,ucirc,uuml,yacute,thorn,yuml,OElig,oelig,Scaron,scaron,Yuml");c.entities_greek&&d.push("Alpha,Beta,Gamma,Delta,Epsilon,Zeta,Eta,Theta,Iota,Kappa,Lambda,Mu,Nu,Xi,Omicron,Pi,Rho,Sigma,Tau,Upsilon,Phi,Chi,Psi,Omega,alpha,beta,gamma,delta,epsilon,zeta,eta,theta,iota,kappa,lambda,mu,nu,xi,omicron,pi,rho,sigmaf,sigma,tau,upsilon,phi,chi,psi,omega,thetasym,upsih,piv");
c.entities_additional&&d.push(c.entities_additional)}var j=a(d.join(",")),f=j.regex?"["+j.regex+"]":"a^";delete j.regex;c.entities&&c.entities_processNumerical&&(f="[^ -~]|"+f);var f=RegExp(f,"g"),h=a("nbsp,gt,lt,amp,shy",true),i=RegExp(h.regex,"g");e.addRules({text:function(a){return a.replace(i,b).replace(f,g)}},{applyToAll:true,excludeNestedEditable:true})}}})}(),CKEDITOR.config.basicEntities=!0,CKEDITOR.config.entities=!0,CKEDITOR.config.entities_latin=!0,CKEDITOR.config.entities_greek=!0,CKEDITOR.config.entities_additional=
"#39",CKEDITOR.plugins.add("popup"),CKEDITOR.tools.extend(CKEDITOR.editor.prototype,{popup:function(a,e,b,g){e=e||"80%";b=b||"70%";typeof e=="string"&&(e.length>1&&e.substr(e.length-1,1)=="%")&&(e=parseInt(window.screen.width*parseInt(e,10)/100,10));typeof b=="string"&&(b.length>1&&b.substr(b.length-1,1)=="%")&&(b=parseInt(window.screen.height*parseInt(b,10)/100,10));e<640&&(e=640);b<420&&(b=420);var c=parseInt((window.screen.height-b)/2,10),d=parseInt((window.screen.width-e)/2,10),g=(g||"location=no,menubar=no,toolbar=no,dependent=yes,minimizable=no,modal=yes,alwaysRaised=yes,resizable=yes,scrollbars=yes")+
",width="+e+",height="+b+",top="+c+",left="+d,j=window.open("",null,g,true);if(!j)return false;try{if(navigator.userAgent.toLowerCase().indexOf(" chrome/")==-1){j.moveTo(d,c);j.resizeTo(e,b)}j.focus();j.location.href=a}catch(f){window.open(a,null,g,true)}return true}}),function(){function a(a,b){var c=[];if(b)for(var d in b)c.push(d+"="+encodeURIComponent(b[d]));else return a;return a+(a.indexOf("?")!=-1?"&":"?")+c.join("&")}function e(a){a=a+"";return a.charAt(0).toUpperCase()+a.substr(1)}function b(){var b=
this.getDialog(),c=b.getParentEditor();c._.filebrowserSe=this;var d=c.config["filebrowser"+e(b.getName())+"WindowWidth"]||c.config.filebrowserWindowWidth||"80%",b=c.config["filebrowser"+e(b.getName())+"WindowHeight"]||c.config.filebrowserWindowHeight||"70%",f=this.filebrowser.params||{};f.CKEditor=c.name;f.CKEditorFuncNum=c._.filebrowserFn;if(!f.langCode)f.langCode=c.langCode;f=a(this.filebrowser.url,f);c.popup(f,d,b,c.config.filebrowserWindowFeatures||c.config.fileBrowserWindowFeatures)}function g(){var a=
this.getDialog();a.getParentEditor()._.filebrowserSe=this;return!a.getContentElement(this["for"][0],this["for"][1]).getInputElement().$.value||!a.getContentElement(this["for"][0],this["for"][1]).getAction()?false:true}function c(b,c,d){var f=d.params||{};f.CKEditor=b.name;f.CKEditorFuncNum=b._.filebrowserFn;if(!f.langCode)f.langCode=b.langCode;c.action=a(d.url,f);c.filebrowser=d}function d(a,f,j,n){if(n&&n.length)for(var o,q=n.length;q--;){o=n[q];(o.type=="hbox"||o.type=="vbox"||o.type=="fieldset")&&
d(a,f,j,o.children);if(o.filebrowser){if(typeof o.filebrowser=="string")o.filebrowser={action:o.type=="fileButton"?"QuickUpload":"Browse",target:o.filebrowser};if(o.filebrowser.action=="Browse"){var m=o.filebrowser.url;if(m===void 0){m=a.config["filebrowser"+e(f)+"BrowseUrl"];if(m===void 0)m=a.config.filebrowserBrowseUrl}if(m){o.onClick=b;o.filebrowser.url=m;o.hidden=false}}else if(o.filebrowser.action=="QuickUpload"&&o["for"]){m=o.filebrowser.url;if(m===void 0){m=a.config["filebrowser"+e(f)+"UploadUrl"];
if(m===void 0)m=a.config.filebrowserUploadUrl}if(m){var l=o.onClick;o.onClick=function(a){var b=a.sender;return l&&l.call(b,a)===false?false:g.call(b,a)};o.filebrowser.url=m;o.hidden=false;c(a,j.getContents(o["for"][0]).get(o["for"][1]),o.filebrowser)}}}}}function j(a,b,c){if(c.indexOf(";")!==-1){for(var c=c.split(";"),d=0;d<c.length;d++)if(j(a,b,c[d]))return true;return false}return(a=a.getContents(b).get(c).filebrowser)&&a.url}function f(a,b){var c=this._.filebrowserSe.getDialog(),d=this._.filebrowserSe["for"],
f=this._.filebrowserSe.filebrowser.onSelect;d&&c.getContentElement(d[0],d[1]).reset();if(!(typeof b=="function"&&b.call(this._.filebrowserSe)===false)&&!(f&&f.call(this._.filebrowserSe,a,b)===false)){typeof b=="string"&&b&&alert(b);if(a){d=this._.filebrowserSe;c=d.getDialog();if(d=d.filebrowser.target||null){d=d.split(":");if(f=c.getContentElement(d[0],d[1])){f.setValue(a);c.selectPage(d[0])}}}}}CKEDITOR.plugins.add("filebrowser",{requires:"popup",init:function(a){a._.filebrowserFn=CKEDITOR.tools.addFunction(f,
a);a.on("destroy",function(){CKEDITOR.tools.removeFunction(this._.filebrowserFn)})}});CKEDITOR.on("dialogDefinition",function(a){if(a.editor.plugins.filebrowser)for(var b=a.data.definition,c,f=0;f<b.contents.length;++f)if(c=b.contents[f]){d(a.editor,a.data.name,b,c.elements);if(c.hidden&&c.filebrowser)c.hidden=!j(b,c.id,c.filebrowser)}})}(),function(){function a(a){var c=a.config,d=a.fire("uiSpace",{space:"top",html:""}).html,j=function(){function d(a,c,f){h.setStyle(c,b(f));h.setStyle("position",
a)}function f(a){var b=k.getDocumentPosition();switch(a){case "top":d("absolute","top",b.y-s-x);break;case "pin":d("fixed","top",r);break;case "bottom":d("absolute","top",b.y+(p.height||p.bottom-p.top)+x)}i=a}var i,k,l,p,t,s,u,w=c.floatSpaceDockedOffsetX||0,x=c.floatSpaceDockedOffsetY||0,v=c.floatSpacePinnedOffsetX||0,r=c.floatSpacePinnedOffsetY||0;return function(c){if(k=a.editable()){c&&c.name=="focus"&&h.show();h.removeStyle("left");h.removeStyle("right");l=h.getClientRect();p=k.getClientRect();
t=e.getViewPaneSize();s=l.height;u="pageXOffset"in e.$?e.$.pageXOffset:CKEDITOR.document.$.documentElement.scrollLeft;if(i){s+x<=p.top?f("top"):s+x>t.height-p.bottom?f("pin"):f("bottom");var c=t.width/2,c=p.left>0&&p.right<t.width&&p.width>l.width?a.config.contentsLangDirection=="rtl"?"right":"left":c-p.left>p.right-c?"left":"right",d;if(l.width>t.width){c="left";d=0}else{d=c=="left"?p.left>0?p.left:0:p.right<t.width?t.width-p.right:0;if(d+l.width>t.width){c=c=="left"?"right":"left";d=0}}h.setStyle(c,
b((i=="pin"?v:w)+d+(i=="pin"?0:c=="left"?u:-u)))}else{i="pin";f("pin");j(c)}}}}();if(d){var f=new CKEDITOR.template('<div id="cke_{name}" class="cke {id} cke_reset_all cke_chrome cke_editor_{name} cke_float cke_{langDir} '+CKEDITOR.env.cssClass+'" dir="{langDir}" title="'+(CKEDITOR.env.gecko?" ":"")+'" lang="{langCode}" role="application" style="{style}"'+(a.title?' aria-labelledby="cke_{name}_arialbl"':" ")+">"+(a.title?'<span id="cke_{name}_arialbl" class="cke_voice_label">{voiceLabel}</span>':
" ")+'<div class="cke_inner"><div id="{topId}" class="cke_top" role="presentation">{content}</div></div></div>'),h=CKEDITOR.document.getBody().append(CKEDITOR.dom.element.createFromHtml(f.output({content:d,id:a.id,langDir:a.lang.dir,langCode:a.langCode,name:a.name,style:"display:none;z-index:"+(c.baseFloatZIndex-1),topId:a.ui.spaceId("top"),voiceLabel:a.title}))),i=CKEDITOR.tools.eventsBuffer(500,j),k=CKEDITOR.tools.eventsBuffer(100,j);h.unselectable();h.on("mousedown",function(a){a=a.data;a.getTarget().hasAscendant("a",
1)||a.preventDefault()});a.on("focus",function(b){j(b);a.on("change",i.input);e.on("scroll",k.input);e.on("resize",k.input)});a.on("blur",function(){h.hide();a.removeListener("change",i.input);e.removeListener("scroll",k.input);e.removeListener("resize",k.input)});a.on("destroy",function(){e.removeListener("scroll",k.input);e.removeListener("resize",k.input);h.clearCustomData();h.remove()});a.focusManager.hasFocus&&h.show();a.focusManager.add(h,1)}}var e=CKEDITOR.document.getWindow(),b=CKEDITOR.tools.cssLength;
CKEDITOR.plugins.add("floatingspace",{init:function(b){b.on("loaded",function(){a(this)},null,null,20)}})}(),CKEDITOR.plugins.add("listblock",{requires:"panel",onLoad:function(){var a=CKEDITOR.addTemplate("panel-list",'<ul role="presentation" class="cke_panel_list">{items}</ul>'),e=CKEDITOR.addTemplate("panel-list-item",'<li id="{id}" class="cke_panel_listItem" role=presentation><a id="{id}_option" _cke_focus=1 hidefocus=true title="{title}" href="javascript:void(\'{val}\')"  {onclick}="CKEDITOR.tools.callFunction({clickFn},\'{val}\'); return false;" role="option">{text}</a></li>'),
b=CKEDITOR.addTemplate("panel-list-group",'<h1 id="{id}" class="cke_panel_grouptitle" role="presentation" >{label}</h1>'),g=/\'/g;CKEDITOR.ui.panel.prototype.addListBlock=function(a,b){return this.addBlock(a,new CKEDITOR.ui.listBlock(this.getHolderElement(),b))};CKEDITOR.ui.listBlock=CKEDITOR.tools.createClass({base:CKEDITOR.ui.panel.block,$:function(a,b){var b=b||{},e=b.attributes||(b.attributes={});(this.multiSelect=!!b.multiSelect)&&(e["aria-multiselectable"]=true);!e.role&&(e.role="listbox");
this.base.apply(this,arguments);this.element.setAttribute("role",e.role);e=this.keys;e[40]="next";e[9]="next";e[38]="prev";e[CKEDITOR.SHIFT+9]="prev";e[32]=CKEDITOR.env.ie?"mouseup":"click";CKEDITOR.env.ie&&(e[13]="mouseup");this._.pendingHtml=[];this._.pendingList=[];this._.items={};this._.groups={}},_:{close:function(){if(this._.started){var b=a.output({items:this._.pendingList.join("")});this._.pendingList=[];this._.pendingHtml.push(b);delete this._.started}},getClick:function(){if(!this._.click)this._.click=
CKEDITOR.tools.addFunction(function(a){var b=this.toggle(a);if(this.onClick)this.onClick(a,b)},this);return this._.click}},proto:{add:function(a,b,j){var f=CKEDITOR.tools.getNextId();if(!this._.started){this._.started=1;this._.size=this._.size||0}this._.items[a]=f;var h;h=CKEDITOR.tools.htmlEncodeAttr(a).replace(g,"\\'");a={id:f,val:h,onclick:CKEDITOR.env.ie?'onclick="return false;" onmouseup':"onclick",clickFn:this._.getClick(),title:CKEDITOR.tools.htmlEncodeAttr(j||a),text:b||a};this._.pendingList.push(e.output(a))},
startGroup:function(a){this._.close();var d=CKEDITOR.tools.getNextId();this._.groups[a]=d;this._.pendingHtml.push(b.output({id:d,label:a}))},commit:function(){this._.close();this.element.appendHtml(this._.pendingHtml.join(""));delete this._.size;this._.pendingHtml=[]},toggle:function(a){var b=this.isMarked(a);b?this.unmark(a):this.mark(a);return!b},hideGroup:function(a){var b=(a=this.element.getDocument().getById(this._.groups[a]))&&a.getNext();if(a){a.setStyle("display","none");b&&b.getName()=="ul"&&
b.setStyle("display","none")}},hideItem:function(a){this.element.getDocument().getById(this._.items[a]).setStyle("display","none")},showAll:function(){var a=this._.items,b=this._.groups,e=this.element.getDocument(),f;for(f in a)e.getById(a[f]).setStyle("display","");for(var g in b){a=e.getById(b[g]);f=a.getNext();a.setStyle("display","");f&&f.getName()=="ul"&&f.setStyle("display","")}},mark:function(a){this.multiSelect||this.unmarkAll();var a=this._.items[a],b=this.element.getDocument().getById(a);
b.addClass("cke_selected");this.element.getDocument().getById(a+"_option").setAttribute("aria-selected",true);this.onMark&&this.onMark(b)},unmark:function(a){var b=this.element.getDocument(),a=this._.items[a],e=b.getById(a);e.removeClass("cke_selected");b.getById(a+"_option").removeAttribute("aria-selected");this.onUnmark&&this.onUnmark(e)},unmarkAll:function(){var a=this._.items,b=this.element.getDocument(),e;for(e in a){var f=a[e];b.getById(f).removeClass("cke_selected");b.getById(f+"_option").removeAttribute("aria-selected")}this.onUnmark&&
this.onUnmark()},isMarked:function(a){return this.element.getDocument().getById(this._.items[a]).hasClass("cke_selected")},focus:function(a){this._.focusIndex=-1;var b=this.element.getElementsByTag("a"),e,f=-1;if(a)for(e=this.element.getDocument().getById(this._.items[a]).getFirst();a=b.getItem(++f);){if(a.equals(e)){this._.focusIndex=f;break}}else this.element.focus();e&&setTimeout(function(){e.focus()},0)}}})}}),function(){var a='<a id="{id}" class="cke_button cke_button__{name} cke_button_{state} {cls}"'+
(CKEDITOR.env.gecko&&!CKEDITOR.env.hc?"":" href=\"javascript:void('{titleJs}')\"")+' title="{title}" tabindex="-1" hidefocus="true" role="button" aria-labelledby="{id}_label" aria-haspopup="{hasArrow}" aria-disabled="{ariaDisabled}"';CKEDITOR.env.gecko&&CKEDITOR.env.mac&&(a=a+' onkeypress="return false;"');CKEDITOR.env.gecko&&(a=a+' onblur="this.style.cssText = this.style.cssText;"');var a=a+(' onkeydown="return CKEDITOR.tools.callFunction({keydownFn},event);" onfocus="return CKEDITOR.tools.callFunction({focusFn},event);" '+
(CKEDITOR.env.ie?'onclick="return false;" onmouseup':"onclick")+'="CKEDITOR.tools.callFunction({clickFn},this);return false;"><span class="cke_button_icon cke_button__{iconName}_icon" style="{style}"'),a=a+'>&nbsp;</span><span id="{id}_label" class="cke_button_label cke_button__{name}_label" aria-hidden="false">{label}</span>{arrowHtml}</a>',e=CKEDITOR.addTemplate("buttonArrow",'<span class="cke_button_arrow">'+(CKEDITOR.env.hc?"&#9660;":"")+"</span>"),b=CKEDITOR.addTemplate("button",a);CKEDITOR.plugins.add("button",
{beforeInit:function(a){a.ui.addHandler(CKEDITOR.UI_BUTTON,CKEDITOR.ui.button.handler)}});CKEDITOR.UI_BUTTON="button";CKEDITOR.ui.button=function(a){CKEDITOR.tools.extend(this,a,{title:a.label,click:a.click||function(b){b.execCommand(a.command)}});this._={}};CKEDITOR.ui.button.handler={create:function(a){return new CKEDITOR.ui.button(a)}};CKEDITOR.ui.button.prototype={render:function(a,c){function d(){var b=a.mode;if(b){b=this.modes[b]?l[b]!==void 0?l[b]:CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED;
b=a.readOnly&&!this.readOnly?CKEDITOR.TRISTATE_DISABLED:b;this.setState(b);this.refresh&&this.refresh()}}var j=CKEDITOR.env,f=this._.id=CKEDITOR.tools.getNextId(),h="",i=this.command,k;this._.editor=a;var n={id:f,button:this,editor:a,focus:function(){CKEDITOR.document.getById(f).focus()},execute:function(){this.button.click(a)},attach:function(a){this.button.attach(a)}},o=CKEDITOR.tools.addFunction(function(a){if(n.onkey){a=new CKEDITOR.dom.event(a);return n.onkey(n,a.getKeystroke())!==false}}),q=
CKEDITOR.tools.addFunction(function(a){var b;n.onfocus&&(b=n.onfocus(n,new CKEDITOR.dom.event(a))!==false);return b}),m=0;n.clickFn=k=CKEDITOR.tools.addFunction(function(){if(m){a.unlockSelection(1);m=0}n.execute();j.iOS&&a.focus()});if(this.modes){var l={};a.on("beforeModeUnload",function(){if(a.mode&&this._.state!=CKEDITOR.TRISTATE_DISABLED)l[a.mode]=this._.state},this);a.on("activeFilterChange",d,this);a.on("mode",d,this);!this.readOnly&&a.on("readOnly",d,this)}else if(i)if(i=a.getCommand(i)){i.on("state",
function(){this.setState(i.state)},this);h=h+(i.state==CKEDITOR.TRISTATE_ON?"on":i.state==CKEDITOR.TRISTATE_DISABLED?"disabled":"off")}if(this.directional)a.on("contentDirChanged",function(b){var c=CKEDITOR.document.getById(this._.id),d=c.getFirst(),b=b.data;b!=a.lang.dir?c.addClass("cke_"+b):c.removeClass("cke_ltr").removeClass("cke_rtl");d.setAttribute("style",CKEDITOR.skin.getIconStyle(t,b=="rtl",this.icon,this.iconOffset))},this);i||(h=h+"off");var p=this.name||this.command,t=p;if(this.icon&&
!/\./.test(this.icon)){t=this.icon;this.icon=null}h={id:f,name:p,iconName:t,label:this.label,cls:this.className||"",state:h,ariaDisabled:h=="disabled"?"true":"false",title:this.title,titleJs:j.gecko&&!j.hc?"":(this.title||"").replace("'",""),hasArrow:this.hasArrow?"true":"false",keydownFn:o,focusFn:q,clickFn:k,style:CKEDITOR.skin.getIconStyle(t,a.lang.dir=="rtl",this.icon,this.iconOffset),arrowHtml:this.hasArrow?e.output():""};b.output(h,c);if(this.onRender)this.onRender();return n},setState:function(a){if(this._.state==
a)return false;this._.state=a;var b=CKEDITOR.document.getById(this._.id);if(b){b.setState(a,"cke_button");a==CKEDITOR.TRISTATE_DISABLED?b.setAttribute("aria-disabled",true):b.removeAttribute("aria-disabled");if(this.hasArrow){a=a==CKEDITOR.TRISTATE_ON?this._.editor.lang.button.selectedLabel.replace(/%1/g,this.label):this.label;CKEDITOR.document.getById(this._.id+"_label").setText(a)}else a==CKEDITOR.TRISTATE_ON?b.setAttribute("aria-pressed",true):b.removeAttribute("aria-pressed");return true}return false},
getState:function(){return this._.state},toFeature:function(a){if(this._.feature)return this._.feature;var b=this;!this.allowedContent&&(!this.requiredContent&&this.command)&&(b=a.getCommand(this.command)||b);return this._.feature=b}};CKEDITOR.ui.prototype.addButton=function(a,b){this.add(a,CKEDITOR.UI_BUTTON,b)}}(),CKEDITOR.plugins.add("richcombo",{requires:"floatpanel,listblock,button",beforeInit:function(a){a.ui.addHandler(CKEDITOR.UI_RICHCOMBO,CKEDITOR.ui.richCombo.handler)}}),function(){var a=
'<span id="{id}" class="cke_combo cke_combo__{name} {cls}" role="presentation"><span id="{id}_label" class="cke_combo_label">{label}</span><a class="cke_combo_button" title="{title}" tabindex="-1"'+(CKEDITOR.env.gecko&&!CKEDITOR.env.hc?"":" href=\"javascript:void('{titleJs}')\"")+' hidefocus="true" role="button" aria-labelledby="{id}_label" aria-haspopup="true"';CKEDITOR.env.gecko&&CKEDITOR.env.mac&&(a=a+' onkeypress="return false;"');CKEDITOR.env.gecko&&(a=a+' onblur="this.style.cssText = this.style.cssText;"');
var a=a+(' onkeydown="return CKEDITOR.tools.callFunction({keydownFn},event,this);" onfocus="return CKEDITOR.tools.callFunction({focusFn},event);" '+(CKEDITOR.env.ie?'onclick="return false;" onmouseup':"onclick")+'="CKEDITOR.tools.callFunction({clickFn},this);return false;"><span id="{id}_text" class="cke_combo_text cke_combo_inlinelabel">{label}</span><span class="cke_combo_open"><span class="cke_combo_arrow">'+(CKEDITOR.env.hc?"&#9660;":CKEDITOR.env.air?"&nbsp;":"")+"</span></span></a></span>"),
e=CKEDITOR.addTemplate("combo",a);CKEDITOR.UI_RICHCOMBO="richcombo";CKEDITOR.ui.richCombo=CKEDITOR.tools.createClass({$:function(a){CKEDITOR.tools.extend(this,a,{canGroup:false,title:a.label,modes:{wysiwyg:1},editorFocus:1});a=this.panel||{};delete this.panel;this.id=CKEDITOR.tools.getNextNumber();this.document=a.parent&&a.parent.getDocument()||CKEDITOR.document;a.className="cke_combopanel";a.block={multiSelect:a.multiSelect,attributes:a.attributes};a.toolbarRelated=true;this._={panelDefinition:a,
items:{}}},proto:{renderHtml:function(a){var e=[];this.render(a,e);return e.join("")},render:function(a,g){function c(){if(this.getState()!=CKEDITOR.TRISTATE_ON){var c=this.modes[a.mode]?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED;if(a.readOnly&&!this.readOnly)c=CKEDITOR.TRISTATE_DISABLED;this.setState(c);this.setValue("");c!=CKEDITOR.TRISTATE_DISABLED&&this.refresh&&this.refresh()}}var d=CKEDITOR.env,j="cke_"+this.id,f=CKEDITOR.tools.addFunction(function(c){if(o){a.unlockSelection(1);o=0}i.execute(c)},
this),h=this,i={id:j,combo:this,focus:function(){CKEDITOR.document.getById(j).getChild(1).focus()},execute:function(c){var d=h._;if(d.state!=CKEDITOR.TRISTATE_DISABLED){h.createPanel(a);if(d.on)d.panel.hide();else{h.commit();var f=h.getValue();f?d.list.mark(f):d.list.unmarkAll();d.panel.showBlock(h.id,new CKEDITOR.dom.element(c),4)}}},clickFn:f};a.on("activeFilterChange",c,this);a.on("mode",c,this);a.on("selectionChange",c,this);!this.readOnly&&a.on("readOnly",c,this);var k=CKEDITOR.tools.addFunction(function(c,
d){var c=new CKEDITOR.dom.event(c),e=c.getKeystroke();if(e==40)a.once("panelShow",function(a){a.data._.panel._.currentBlock.onKeyDown(40)});switch(e){case 13:case 32:case 40:CKEDITOR.tools.callFunction(f,d);break;default:i.onkey(i,e)}c.preventDefault()}),n=CKEDITOR.tools.addFunction(function(){i.onfocus&&i.onfocus()}),o=0;i.keyDownFn=k;d={id:j,name:this.name||this.command,label:this.label,title:this.title,cls:this.className||"",titleJs:d.gecko&&!d.hc?"":(this.title||"").replace("'",""),keydownFn:k,
focusFn:n,clickFn:f};e.output(d,g);if(this.onRender)this.onRender();return i},createPanel:function(a){if(!this._.panel){var e=this._.panelDefinition,c=this._.panelDefinition.block,d=e.parent||CKEDITOR.document.getBody(),j="cke_combopanel__"+this.name,f=new CKEDITOR.ui.floatPanel(a,d,e),h=f.addListBlock(this.id,c),i=this;f.onShow=function(){this.element.addClass(j);i.setState(CKEDITOR.TRISTATE_ON);i._.on=1;i.editorFocus&&!a.focusManager.hasFocus&&a.focus();if(i.onOpen)i.onOpen();a.once("panelShow",
function(){h.focus(!h.multiSelect&&i.getValue())})};f.onHide=function(c){this.element.removeClass(j);i.setState(i.modes&&i.modes[a.mode]?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED);i._.on=0;if(!c&&i.onClose)i.onClose()};f.onEscape=function(){f.hide(1)};h.onClick=function(a,b){i.onClick&&i.onClick.call(i,a,b);f.hide()};this._.panel=f;this._.list=h;f.getBlock(this.id).onHide=function(){i._.on=0;i.setState(CKEDITOR.TRISTATE_OFF)};this.init&&this.init()}},setValue:function(a,e){this._.value=a;var c=
this.document.getById("cke_"+this.id+"_text");if(c){if(!a&&!e){e=this.label;c.addClass("cke_combo_inlinelabel")}else c.removeClass("cke_combo_inlinelabel");c.setText(typeof e!="undefined"?e:a)}},getValue:function(){return this._.value||""},unmarkAll:function(){this._.list.unmarkAll()},mark:function(a){this._.list.mark(a)},hideItem:function(a){this._.list.hideItem(a)},hideGroup:function(a){this._.list.hideGroup(a)},showAll:function(){this._.list.showAll()},add:function(a,e,c){this._.items[a]=c||a;
this._.list.add(a,e,c)},startGroup:function(a){this._.list.startGroup(a)},commit:function(){if(!this._.committed){this._.list.commit();this._.committed=1;CKEDITOR.ui.fire("ready",this)}this._.committed=1},setState:function(a){if(this._.state!=a){var e=this.document.getById("cke_"+this.id);e.setState(a,"cke_combo");a==CKEDITOR.TRISTATE_DISABLED?e.setAttribute("aria-disabled",true):e.removeAttribute("aria-disabled");this._.state=a}},getState:function(){return this._.state},enable:function(){this._.state==
CKEDITOR.TRISTATE_DISABLED&&this.setState(this._.lastState)},disable:function(){if(this._.state!=CKEDITOR.TRISTATE_DISABLED){this._.lastState=this._.state;this.setState(CKEDITOR.TRISTATE_DISABLED)}}},statics:{handler:{create:function(a){return new CKEDITOR.ui.richCombo(a)}}}});CKEDITOR.ui.prototype.addRichCombo=function(a,e){this.add(a,CKEDITOR.UI_RICHCOMBO,e)}}(),CKEDITOR.plugins.add("format",{requires:"richcombo",init:function(a){if(!a.blockless){for(var e=a.config,b=a.lang.format,g=e.format_tags.split(";"),
c={},d=0,j=[],f=0;f<g.length;f++){var h=g[f],i=new CKEDITOR.style(e["format_"+h]);if(!a.filter.customConfig||a.filter.check(i)){d++;c[h]=i;c[h]._.enterMode=a.config.enterMode;j.push(i)}}d!==0&&a.ui.addRichCombo("Format",{label:b.label,title:b.panelTitle,toolbar:"styles,20",allowedContent:j,panel:{css:[CKEDITOR.skin.getPath("editor")].concat(e.contentsCss),multiSelect:false,attributes:{"aria-label":b.panelTitle}},init:function(){this.startGroup(b.panelTitle);for(var a in c){var d=b["tag_"+a];this.add(a,
c[a].buildPreview(d),d)}},onClick:function(b){a.focus();a.fire("saveSnapshot");var b=c[b],d=a.elementPath();a[b.checkActive(d,a)?"removeStyle":"applyStyle"](b);setTimeout(function(){a.fire("saveSnapshot")},0)},onRender:function(){a.on("selectionChange",function(b){var d=this.getValue(),b=b.data.path;this.refresh();for(var f in c)if(c[f].checkActive(b,a)){f!=d&&this.setValue(f,a.lang.format["tag_"+f]);return}this.setValue("")},this)},onOpen:function(){this.showAll();for(var b in c)a.activeFilter.check(c[b])||
this.hideItem(b)},refresh:function(){var b=a.elementPath();if(b){if(b.isContextFor("p"))for(var d in c)if(a.activeFilter.check(c[d]))return;this.setState(CKEDITOR.TRISTATE_DISABLED)}}})}}}),CKEDITOR.config.format_tags="p;h1;h2;h3;h4;h5;h6;pre;address;div",CKEDITOR.config.format_p={element:"p"},CKEDITOR.config.format_div={element:"div"},CKEDITOR.config.format_pre={element:"pre"},CKEDITOR.config.format_address={element:"address"},CKEDITOR.config.format_h1={element:"h1"},CKEDITOR.config.format_h2={element:"h2"},
CKEDITOR.config.format_h3={element:"h3"},CKEDITOR.config.format_h4={element:"h4"},CKEDITOR.config.format_h5={element:"h5"},CKEDITOR.config.format_h6={element:"h6"},function(){var a={canUndo:false,exec:function(a){var b=a.document.createElement("hr");a.insertElement(b)},allowedContent:"hr",requiredContent:"hr"};CKEDITOR.plugins.add("horizontalrule",{init:function(e){if(!e.blockless){e.addCommand("horizontalrule",a);e.ui.addButton&&e.ui.addButton("HorizontalRule",{label:e.lang.horizontalrule.toolbar,
command:"horizontalrule",toolbar:"insert,40"})}}})}(),CKEDITOR.plugins.add("htmlwriter",{init:function(a){var e=new CKEDITOR.htmlWriter;e.forceSimpleAmpersand=a.config.forceSimpleAmpersand;e.indentationChars=a.config.dataIndentationChars||"\t";a.dataProcessor.writer=e}}),CKEDITOR.htmlWriter=CKEDITOR.tools.createClass({base:CKEDITOR.htmlParser.basicWriter,$:function(){this.base();this.indentationChars="\t";this.selfClosingEnd=" />";this.lineBreakChars="\n";this.sortAttributes=1;this._.indent=0;this._.indentation=
"";this._.inPre=0;this._.rules={};var a=CKEDITOR.dtd,e;for(e in CKEDITOR.tools.extend({},a.$nonBodyContent,a.$block,a.$listItem,a.$tableContent))this.setRules(e,{indent:!a[e]["#"],breakBeforeOpen:1,breakBeforeClose:!a[e]["#"],breakAfterClose:1,needsSpace:e in a.$block&&!(e in{li:1,dt:1,dd:1})});this.setRules("br",{breakAfterOpen:1});this.setRules("title",{indent:0,breakAfterOpen:0});this.setRules("style",{indent:0,breakBeforeClose:1});this.setRules("pre",{breakAfterOpen:1,indent:0})},proto:{openTag:function(a){var e=
this._.rules[a];this._.afterCloser&&(e&&e.needsSpace&&this._.needsSpace)&&this._.output.push("\n");if(this._.indent)this.indentation();else if(e&&e.breakBeforeOpen){this.lineBreak();this.indentation()}this._.output.push("<",a);this._.afterCloser=0},openTagClose:function(a,e){var b=this._.rules[a];if(e){this._.output.push(this.selfClosingEnd);if(b&&b.breakAfterClose)this._.needsSpace=b.needsSpace}else{this._.output.push(">");if(b&&b.indent)this._.indentation=this._.indentation+this.indentationChars}b&&
b.breakAfterOpen&&this.lineBreak();a=="pre"&&(this._.inPre=1)},attribute:function(a,e){if(typeof e=="string"){this.forceSimpleAmpersand&&(e=e.replace(/&amp;/g,"&"));e=CKEDITOR.tools.htmlEncodeAttr(e)}this._.output.push(" ",a,'="',e,'"')},closeTag:function(a){var e=this._.rules[a];if(e&&e.indent)this._.indentation=this._.indentation.substr(this.indentationChars.length);if(this._.indent)this.indentation();else if(e&&e.breakBeforeClose){this.lineBreak();this.indentation()}this._.output.push("</",a,">");
a=="pre"&&(this._.inPre=0);if(e&&e.breakAfterClose){this.lineBreak();this._.needsSpace=e.needsSpace}this._.afterCloser=1},text:function(a){if(this._.indent){this.indentation();!this._.inPre&&(a=CKEDITOR.tools.ltrim(a))}this._.output.push(a)},comment:function(a){this._.indent&&this.indentation();this._.output.push("<\!--",a,"--\>")},lineBreak:function(){!this._.inPre&&this._.output.length>0&&this._.output.push(this.lineBreakChars);this._.indent=1},indentation:function(){!this._.inPre&&this._.indentation&&
this._.output.push(this._.indentation);this._.indent=0},reset:function(){this._.output=[];this._.indent=0;this._.indentation="";this._.afterCloser=0;this._.inPre=0},setRules:function(a,e){var b=this._.rules[a];b?CKEDITOR.tools.extend(b,e,true):this._.rules[a]=e}}}),function(){function a(a,e){e||(e=a.getSelection().getSelectedElement());if(e&&e.is("img")&&!e.data("cke-realelement")&&!e.isReadOnly())return e}function e(a){var e=a.getStyle("float");if(e=="inherit"||e=="none")e=0;e||(e=a.getAttribute("align"));
return e}CKEDITOR.plugins.add("image",{requires:"dialog",init:function(b){if(!b.plugins.image2){CKEDITOR.dialog.add("image",this.path+"dialogs/image.js");var e="img[alt,!src]{border-style,border-width,float,height,margin,margin-bottom,margin-left,margin-right,margin-top,width}";CKEDITOR.dialog.isTabEnabled(b,"image","advanced")&&(e="img[alt,dir,id,lang,longdesc,!src,title]{*}(*)");b.addCommand("image",new CKEDITOR.dialogCommand("image",{allowedContent:e,requiredContent:"img[alt,src]",contentTransformations:[["img{width}: sizeToStyle",
"img[width]: sizeToAttribute"],["img{float}: alignmentToStyle","img[align]: alignmentToAttribute"]]}));b.ui.addButton&&b.ui.addButton("Image",{label:b.lang.common.image,command:"image",toolbar:"insert,10"});b.on("doubleclick",function(a){var b=a.data.element;if(b.is("img")&&!b.data("cke-realelement")&&!b.isReadOnly())a.data.dialog="image"});b.addMenuItems&&b.addMenuItems({image:{label:b.lang.image.menu,command:"image",group:"image"}});b.contextMenu&&b.contextMenu.addListener(function(c){if(a(b,c))return{image:CKEDITOR.TRISTATE_OFF}})}},
afterInit:function(b){function g(c){var d=b.getCommand("justify"+c);if(d){if(c=="left"||c=="right")d.on("exec",function(d){var f=a(b),g;if(f){g=e(f);if(g==c){f.removeStyle("float");c==e(f)&&f.removeAttribute("align")}else f.setStyle("float",c);d.cancel()}});d.on("refresh",function(d){var f=a(b);if(f){f=e(f);this.setState(f==c?CKEDITOR.TRISTATE_ON:c=="right"||c=="left"?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED);d.cancel()}})}}if(!b.plugins.image2){g("left");g("right");g("center");g("block")}}})}(),
CKEDITOR.config.image_removeLinkByEmptyURL=!0,function(){function a(a,b){var c=g.exec(a),e=g.exec(b);if(c){if(!c[2]&&e[2]=="px")return e[1];if(c[2]=="px"&&!e[2])return e[1]+"px"}return b}var e=CKEDITOR.htmlParser.cssStyle,b=CKEDITOR.tools.cssLength,g=/^((?:\d*(?:\.\d+))|(?:\d+))(.*)?$/i,c={elements:{$:function(b){var c=b.attributes;if((c=(c=(c=c&&c["data-cke-realelement"])&&new CKEDITOR.htmlParser.fragment.fromHtml(decodeURIComponent(c)))&&c.children[0])&&b.attributes["data-cke-resizable"]){var f=
(new e(b)).rules,b=c.attributes,g=f.width,f=f.height;g&&(b.width=a(b.width,g));f&&(b.height=a(b.height,f))}return c}}};CKEDITOR.plugins.add("fakeobjects",{init:function(a){a.filter.allow("img[!data-cke-realelement,src,alt,title](*){*}","fakeobjects")},afterInit:function(a){(a=(a=a.dataProcessor)&&a.htmlFilter)&&a.addRules(c,{applyToAll:true})}});CKEDITOR.editor.prototype.createFakeElement=function(a,c,f,g){var i=this.lang.fakeobjects,i=i[f]||i.unknown,c={"class":c,"data-cke-realelement":encodeURIComponent(a.getOuterHtml()),
"data-cke-real-node-type":a.type,alt:i,title:i,align:a.getAttribute("align")||""};if(!CKEDITOR.env.hc)c.src=CKEDITOR.tools.transparentImageData;f&&(c["data-cke-real-element-type"]=f);if(g){c["data-cke-resizable"]=g;f=new e;g=a.getAttribute("width");a=a.getAttribute("height");g&&(f.rules.width=b(g));a&&(f.rules.height=b(a));f.populate(c)}return this.document.createElement("img",{attributes:c})};CKEDITOR.editor.prototype.createFakeParserElement=function(a,c,f,g){var i=this.lang.fakeobjects,i=i[f]||
i.unknown,k;k=new CKEDITOR.htmlParser.basicWriter;a.writeHtml(k);k=k.getHtml();c={"class":c,"data-cke-realelement":encodeURIComponent(k),"data-cke-real-node-type":a.type,alt:i,title:i,align:a.attributes.align||""};if(!CKEDITOR.env.hc)c.src=CKEDITOR.tools.transparentImageData;f&&(c["data-cke-real-element-type"]=f);if(g){c["data-cke-resizable"]=g;g=a.attributes;a=new e;f=g.width;g=g.height;f!==void 0&&(a.rules.width=b(f));g!==void 0&&(a.rules.height=b(g));a.populate(c)}return new CKEDITOR.htmlParser.element("img",
c)};CKEDITOR.editor.prototype.restoreRealElement=function(b){if(b.data("cke-real-node-type")!=CKEDITOR.NODE_ELEMENT)return null;var c=CKEDITOR.dom.element.createFromHtml(decodeURIComponent(b.data("cke-realelement")),this.document);if(b.data("cke-resizable")){var f=b.getStyle("width"),b=b.getStyle("height");f&&c.setAttribute("width",a(c.getAttribute("width"),f));b&&c.setAttribute("height",a(c.getAttribute("height"),b))}return c}}(),"use strict",function(){function a(a){return a.replace(/'/g,"\\$&")}
function e(a){for(var b,c=a.length,d=[],f=0;f<c;f++){b=a.charCodeAt(f);d.push(b)}return"String.fromCharCode("+d.join(",")+")"}function b(b,c){var d=b.plugins.link,f=d.compiledProtectionFunction.params,e,g;g=[d.compiledProtectionFunction.name,"("];for(var h=0;h<f.length;h++){d=f[h].toLowerCase();e=c[d];h>0&&g.push(",");g.push("'",e?a(encodeURIComponent(c[d])):"","'")}g.push(")");return g.join("")}function g(a){var a=a.config.emailProtection||"",b;if(a&&a!="encode"){b={};a.replace(/^([^(]+)\(([^)]+)\)$/,
function(a,c,d){b.name=c;b.params=[];d.replace(/[^,\s]+/g,function(a){b.params.push(a)})})}return b}CKEDITOR.plugins.add("link",{requires:"dialog,fakeobjects",onLoad:function(){function a(b){return c.replace(/%1/g,b=="rtl"?"right":"left").replace(/%2/g,"cke_contents_"+b)}var b="background:url("+CKEDITOR.getUrl(this.path+"images"+(CKEDITOR.env.hidpi?"/hidpi":"")+"/anchor.png")+") no-repeat %1 center;border:1px dotted #00f;background-size:16px;",c=".%2 a.cke_anchor,.%2 a.cke_anchor_empty,.cke_editable.%2 a[name],.cke_editable.%2 a[data-cke-saved-name]{"+
b+"padding-%1:18px;cursor:auto;}.%2 img.cke_anchor{"+b+"width:16px;min-height:15px;height:1.15em;vertical-align:text-bottom;}";CKEDITOR.addCss(a("ltr")+a("rtl"))},init:function(a){var b="a[!href]";CKEDITOR.dialog.isTabEnabled(a,"link","advanced")&&(b=b.replace("]",",accesskey,charset,dir,id,lang,name,rel,tabindex,title,type]{*}(*)"));CKEDITOR.dialog.isTabEnabled(a,"link","target")&&(b=b.replace("]",",target,onclick]"));a.addCommand("link",new CKEDITOR.dialogCommand("link",{allowedContent:b,requiredContent:"a[href]"}));
a.addCommand("anchor",new CKEDITOR.dialogCommand("anchor",{allowedContent:"a[!name,id]",requiredContent:"a[name]"}));a.addCommand("unlink",new CKEDITOR.unlinkCommand);a.addCommand("removeAnchor",new CKEDITOR.removeAnchorCommand);a.setKeystroke(CKEDITOR.CTRL+76,"link");if(a.ui.addButton){a.ui.addButton("Link",{label:a.lang.link.toolbar,command:"link",toolbar:"links,10"});a.ui.addButton("Unlink",{label:a.lang.link.unlink,command:"unlink",toolbar:"links,20"});a.ui.addButton("Anchor",{label:a.lang.link.anchor.toolbar,
command:"anchor",toolbar:"links,30"})}CKEDITOR.dialog.add("link",this.path+"dialogs/link.js");CKEDITOR.dialog.add("anchor",this.path+"dialogs/anchor.js");a.on("doubleclick",function(b){var c=CKEDITOR.plugins.link.getSelectedLink(a)||b.data.element;if(!c.isReadOnly())if(c.is("a")){b.data.dialog=c.getAttribute("name")&&(!c.getAttribute("href")||!c.getChildCount())?"anchor":"link";b.data.link=c}else if(CKEDITOR.plugins.link.tryRestoreFakeAnchor(a,c))b.data.dialog="anchor"},null,null,0);a.on("doubleclick",
function(b){b.data.dialog in{link:1,anchor:1}&&b.data.link&&a.getSelection().selectElement(b.data.link)},null,null,20);a.addMenuItems&&a.addMenuItems({anchor:{label:a.lang.link.anchor.menu,command:"anchor",group:"anchor",order:1},removeAnchor:{label:a.lang.link.anchor.remove,command:"removeAnchor",group:"anchor",order:5},link:{label:a.lang.link.menu,command:"link",group:"link",order:1},unlink:{label:a.lang.link.unlink,command:"unlink",group:"link",order:5}});a.contextMenu&&a.contextMenu.addListener(function(b){if(!b||
b.isReadOnly())return null;b=CKEDITOR.plugins.link.tryRestoreFakeAnchor(a,b);if(!b&&!(b=CKEDITOR.plugins.link.getSelectedLink(a)))return null;var c={};b.getAttribute("href")&&b.getChildCount()&&(c={link:CKEDITOR.TRISTATE_OFF,unlink:CKEDITOR.TRISTATE_OFF});if(b&&b.hasAttribute("name"))c.anchor=c.removeAnchor=CKEDITOR.TRISTATE_OFF;return c});this.compiledProtectionFunction=g(a)},afterInit:function(a){a.dataProcessor.dataFilter.addRules({elements:{a:function(b){return!b.attributes.name?null:!b.children.length?
a.createFakeParserElement(b,"cke_anchor","anchor"):null}}});var b=a._.elementsPath&&a._.elementsPath.filters;b&&b.push(function(b,c){if(c=="a"&&(CKEDITOR.plugins.link.tryRestoreFakeAnchor(a,b)||b.getAttribute("name")&&(!b.getAttribute("href")||!b.getChildCount())))return"anchor"})}});var c=/^javascript:/,d=/^mailto:([^?]+)(?:\?(.+))?$/,j=/subject=([^;?:@&=$,\/]*)/,f=/body=([^;?:@&=$,\/]*)/,h=/^#(.*)$/,i=/^((?:http|https|ftp|news):\/\/)?(.*)$/,k=/^(_(?:self|top|parent|blank))$/,n=/^javascript:void\(location\.href='mailto:'\+String\.fromCharCode\(([^)]+)\)(?:\+'(.*)')?\)$/,
o=/^javascript:([^(]+)\(([^)]+)\)$/,q=/\s*window.open\(\s*this\.href\s*,\s*(?:'([^']*)'|null)\s*,\s*'([^']*)'\s*\)\s*;\s*return\s*false;*\s*/,m=/(?:^|,)([^=]+)=(\d+|yes|no)/gi,l={id:"advId",dir:"advLangDir",accessKey:"advAccessKey",name:"advName",lang:"advLangCode",tabindex:"advTabIndex",title:"advTitle",type:"advContentType","class":"advCSSClasses",charset:"advCharset",style:"advStyles",rel:"advRel"};CKEDITOR.plugins.link={getSelectedLink:function(a){var b=a.getSelection(),c=b.getSelectedElement();
if(c&&c.is("a"))return c;if(b=b.getRanges()[0]){b.shrink(CKEDITOR.SHRINK_TEXT);return a.elementPath(b.getCommonAncestor()).contains("a",1)}return null},getEditorAnchors:function(a){for(var b=a.editable(),c=b.isInline()&&!a.plugins.divarea?a.document:b,b=c.getElementsByTag("a"),c=c.getElementsByTag("img"),d=[],f=0,e;e=b.getItem(f++);)if(e.data("cke-saved-name")||e.hasAttribute("name"))d.push({name:e.data("cke-saved-name")||e.getAttribute("name"),id:e.getAttribute("id")});for(f=0;e=c.getItem(f++);)(e=
this.tryRestoreFakeAnchor(a,e))&&d.push({name:e.getAttribute("name"),id:e.getAttribute("id")});return d},fakeAnchor:true,tryRestoreFakeAnchor:function(a,b){if(b&&b.data("cke-real-element-type")&&b.data("cke-real-element-type")=="anchor"){var c=a.restoreRealElement(b);if(c.data("cke-saved-name"))return c}},parseLinkAttributes:function(a,b){var e=b&&(b.data("cke-saved-href")||b.getAttribute("href"))||"",g=a.plugins.link.compiledProtectionFunction,w=a.config.emailProtection,x,v={};e.match(c)&&(w=="encode"?
e=e.replace(n,function(a,b,c){return"mailto:"+String.fromCharCode.apply(String,b.split(","))+(c&&c.replace(/\\'/g,"'"))}):w&&e.replace(o,function(a,b,c){if(b==g.name){v.type="email";for(var a=v.email={},b=/(^')|('$)/g,c=c.match(/[^,\s]+/g),d=c.length,f,e,h=0;h<d;h++){f=decodeURIComponent;e=c[h].replace(b,"").replace(/\\'/g,"'");e=f(e);f=g.params[h].toLowerCase();a[f]=e}a.address=[a.name,a.domain].join("@")}}));if(!v.type)if(w=e.match(h)){v.type="anchor";v.anchor={};v.anchor.name=v.anchor.id=w[1]}else if(w=
e.match(d)){x=e.match(j);e=e.match(f);v.type="email";var r=v.email={};r.address=w[1];x&&(r.subject=decodeURIComponent(x[1]));e&&(r.body=decodeURIComponent(e[1]))}else if(e&&(x=e.match(i))){v.type="url";v.url={};v.url.protocol=x[1];v.url.url=x[2]}if(b){if(e=b.getAttribute("target"))v.target={type:e.match(k)?e:"frame",name:e};else if(e=(e=b.data("cke-pa-onclick")||b.getAttribute("onclick"))&&e.match(q))for(v.target={type:"popup",name:e[1]};w=m.exec(e[2]);)(w[2]=="yes"||w[2]=="1")&&!(w[1]in{height:1,
width:1,top:1,left:1})?v.target[w[1]]=true:isFinite(w[2])&&(v.target[w[1]]=w[2]);var e={},y;for(y in l)(w=b.getAttribute(y))&&(e[l[y]]=w);if(y=b.data("cke-saved-name")||e.advName)e.advName=y;if(!CKEDITOR.tools.isEmpty(e))v.advanced=e}return v},getLinkAttributes:function(c,d){var f=c.config.emailProtection||"",g={};switch(d.type){case "url":var f=d.url&&d.url.protocol!==void 0?d.url.protocol:"http://",h=d.url&&CKEDITOR.tools.trim(d.url.url)||"";g["data-cke-saved-href"]=h.indexOf("/")===0?h:f+h;break;
case "anchor":f=d.anchor&&d.anchor.id;g["data-cke-saved-href"]="#"+(d.anchor&&d.anchor.name||f||"");break;case "email":var i=d.email,h=i.address;switch(f){case "":case "encode":var j=encodeURIComponent(i.subject||""),k=encodeURIComponent(i.body||""),i=[];j&&i.push("subject="+j);k&&i.push("body="+k);i=i.length?"?"+i.join("&"):"";if(f=="encode"){f=["javascript:void(location.href='mailto:'+",e(h)];i&&f.push("+'",a(i),"'");f.push(")")}else f=["mailto:",h,i];break;default:f=h.split("@",2);i.name=f[0];
i.domain=f[1];f=["javascript:",b(c,i)]}g["data-cke-saved-href"]=f.join("")}if(d.target)if(d.target.type=="popup"){for(var f=["window.open(this.href, '",d.target.name||"","', '"],m=["resizable","status","location","toolbar","menubar","fullscreen","scrollbars","dependent"],h=m.length,j=function(a){d.target[a]&&m.push(a+"="+d.target[a])},i=0;i<h;i++)m[i]=m[i]+(d.target[m[i]]?"=yes":"=no");j("width");j("left");j("height");j("top");f.push(m.join(","),"'); return false;");g["data-cke-pa-onclick"]=f.join("")}else if(d.target.type!=
"notSet"&&d.target.name)g.target=d.target.name;if(d.advanced){for(var q in l)(f=d.advanced[l[q]])&&(g[q]=f);if(g.name)g["data-cke-saved-name"]=g.name}if(g["data-cke-saved-href"])g.href=g["data-cke-saved-href"];q=CKEDITOR.tools.extend({target:1,onclick:1,"data-cke-pa-onclick":1,"data-cke-saved-name":1},l);for(var n in g)delete q[n];return{set:g,removed:CKEDITOR.tools.objectKeys(q)}}};CKEDITOR.unlinkCommand=function(){};CKEDITOR.unlinkCommand.prototype={exec:function(a){var b=new CKEDITOR.style({element:"a",
type:CKEDITOR.STYLE_INLINE,alwaysRemoveElement:1});a.removeStyle(b)},refresh:function(a,b){var c=b.lastElement&&b.lastElement.getAscendant("a",true);c&&c.getName()=="a"&&c.getAttribute("href")&&c.getChildCount()?this.setState(CKEDITOR.TRISTATE_OFF):this.setState(CKEDITOR.TRISTATE_DISABLED)},contextSensitive:1,startDisabled:1,requiredContent:"a[href]"};CKEDITOR.removeAnchorCommand=function(){};CKEDITOR.removeAnchorCommand.prototype={exec:function(a){var b=a.getSelection(),c=b.createBookmarks(),d;if(b&&
(d=b.getSelectedElement())&&(!d.getChildCount()?CKEDITOR.plugins.link.tryRestoreFakeAnchor(a,d):d.is("a")))d.remove(1);else if(d=CKEDITOR.plugins.link.getSelectedLink(a))if(d.hasAttribute("href")){d.removeAttributes({name:1,"data-cke-saved-name":1});d.removeClass("cke_anchor")}else d.remove(1);b.selectBookmarks(c)},requiredContent:"a[name]"};CKEDITOR.tools.extend(CKEDITOR.config,{linkShowAdvancedTab:true,linkShowTargetTab:true})}(),"use strict",function(){function a(a,b,c){return k(b)&&k(c)&&c.equals(b.getNext(function(a){return!(Y(a)||
Z(a)||n(a))}))}function e(a){this.upper=a[0];this.lower=a[1];this.set.apply(this,a.slice(2))}function b(a){var b=a.element;if(b&&k(b))if((b=b.getAscendant(a.triggers,true))&&a.editable.contains(b)){var c=d(b);if(c.getAttribute("contenteditable")=="true")return b;if(c.is(a.triggers))return c}return null}function g(a,b,c){u(a,b);u(a,c);a=b.size.bottom;c=c.size.top;return a&&c?0|(a+c)/2:a||c}function c(a,b,c){return b=b[c?"getPrevious":"getNext"](function(b){return b&&b.type==CKEDITOR.NODE_TEXT&&!Y(b)||
k(b)&&!n(b)&&!i(a,b)})}function d(a,b){if(a.data("cke-editable"))return null;for(b||(a=a.getParent());a;){if(a.data("cke-editable"))break;if(a.hasAttribute("contenteditable"))return a;a=a.getParent()}return null}function j(a){var b=a.doc,c=z('<span contenteditable="false" style="'+P+"position:absolute;border-top:1px dashed "+a.boxColor+'"></span>',b),d=CKEDITOR.getUrl(this.path+"images/"+(B.hidpi?"hidpi/":"")+"icon"+(a.rtl?"-rtl":"")+".png");r(c,{attach:function(){this.wrap.getParent()||this.wrap.appendTo(a.editable,
true);return this},lineChildren:[r(z('<span title="'+a.editor.lang.magicline.title+'" contenteditable="false">&#8629;</span>',b),{base:P+"height:17px;width:17px;"+(a.rtl?"left":"right")+":17px;background:url("+d+") center no-repeat "+a.boxColor+";cursor:pointer;"+(B.hc?"font-size: 15px;line-height:14px;border:1px solid #fff;text-align:center;":"")+(B.hidpi?"background-size: 9px 10px;":""),looks:["top:-8px;"+CKEDITOR.tools.cssVendorPrefix("border-radius","2px",1),"top:-17px;"+CKEDITOR.tools.cssVendorPrefix("border-radius",
"2px 2px 0px 0px",1),"top:-1px;"+CKEDITOR.tools.cssVendorPrefix("border-radius","0px 0px 2px 2px",1)]}),r(z(W,b),{base:V+"left:0px;border-left-color:"+a.boxColor+";",looks:["border-width:8px 0 8px 8px;top:-8px","border-width:8px 0 0 8px;top:-8px","border-width:0 0 8px 8px;top:0px"]}),r(z(W,b),{base:V+"right:0px;border-right-color:"+a.boxColor+";",looks:["border-width:8px 8px 8px 0;top:-8px","border-width:8px 8px 0 0;top:-8px","border-width:0 8px 8px 0;top:0px"]})],detach:function(){this.wrap.getParent()&&
this.wrap.remove();return this},mouseNear:function(){u(a,this);var b=a.holdDistance,c=this.size;return c&&a.mouse.y>c.top-b&&a.mouse.y<c.bottom+b&&a.mouse.x>c.left-b&&a.mouse.x<c.right+b?true:false},place:function(){var b=a.view,c=a.editable,d=a.trigger,f=d.upper,e=d.lower,g=f||e,h=g.getParent(),i={};this.trigger=d;f&&u(a,f,true);e&&u(a,e,true);u(a,h,true);a.inInlineMode&&w(a,true);if(h.equals(c)){i.left=b.scroll.x;i.right=-b.scroll.x;i.width=""}else{i.left=g.size.left-g.size.margin.left+b.scroll.x-
(a.inInlineMode?b.editable.left+b.editable.border.left:0);i.width=g.size.outerWidth+g.size.margin.left+g.size.margin.right+b.scroll.x;i.right=""}if(f&&e)i.top=f.size.margin.bottom===e.size.margin.top?0|f.size.bottom+f.size.margin.bottom/2:f.size.margin.bottom<e.size.margin.top?f.size.bottom+f.size.margin.bottom:f.size.bottom+f.size.margin.bottom-e.size.margin.top;else if(f){if(!e)i.top=f.size.bottom+f.size.margin.bottom}else i.top=e.size.top-e.size.margin.top;if(d.is(L)||i.top>b.scroll.y-15&&i.top<
b.scroll.y+5){i.top=a.inInlineMode?0:b.scroll.y;this.look(L)}else if(d.is(H)||i.top>b.pane.bottom-5&&i.top<b.pane.bottom+15){i.top=a.inInlineMode?b.editable.height+b.editable.padding.top+b.editable.padding.bottom:b.pane.bottom-1;this.look(H)}else{if(a.inInlineMode)i.top=i.top-(b.editable.top+b.editable.border.top);this.look(Q)}if(a.inInlineMode){i.top--;i.top=i.top+b.editable.scroll.top;i.left=i.left+b.editable.scroll.left}for(var j in i)i[j]=CKEDITOR.tools.cssLength(i[j]);this.setStyles(i)},look:function(a){if(this.oldLook!=
a){for(var b=this.lineChildren.length,c;b--;)(c=this.lineChildren[b]).setAttribute("style",c.base+c.looks[0|a/2]);this.oldLook=a}},wrap:new y("span",a.doc)});for(b=c.lineChildren.length;b--;)c.lineChildren[b].appendTo(c);c.look(Q);c.appendTo(c.wrap);c.unselectable();c.lineChildren[0].on("mouseup",function(b){c.detach();f(a,function(b){var c=a.line.trigger;b[c.is(J)?"insertBefore":"insertAfter"](c.is(J)?c.lower:c.upper)},true);a.editor.focus();!B.ie&&a.enterMode!=CKEDITOR.ENTER_BR&&a.hotNode.scrollIntoView();
b.data.preventDefault(true)});c.on("mousedown",function(a){a.data.preventDefault(true)});a.line=c}function f(a,b,c){var f=new CKEDITOR.dom.range(a.doc),e=a.editor,g;if(B.ie&&a.enterMode==CKEDITOR.ENTER_BR)g=a.doc.createText(O);else{g=(g=d(a.element,true))&&g.data("cke-enter-mode")||a.enterMode;g=new y(D[g],a.doc);g.is("br")||a.doc.createText(O).appendTo(g)}c&&e.fire("saveSnapshot");b(g);f.moveToPosition(g,CKEDITOR.POSITION_AFTER_START);e.getSelection().selectRanges([f]);a.hotNode=g;c&&e.fire("saveSnapshot")}
function h(a,e){return{canUndo:true,modes:{wysiwyg:1},exec:function(){function g(b){var c=B.ie&&B.version<9?" ":O,d=a.hotNode&&a.hotNode.getText()==c&&a.element.equals(a.hotNode)&&a.lastCmdDirection===!!e;f(a,function(c){d&&a.hotNode&&a.hotNode.remove();c[e?"insertAfter":"insertBefore"](b);c.setAttributes({"data-cke-magicline-hot":1,"data-cke-magicline-dir":!!e});a.lastCmdDirection=!!e});!B.ie&&a.enterMode!=CKEDITOR.ENTER_BR&&a.hotNode.scrollIntoView();a.line.detach()}return function(f){var f=f.getSelection().getStartElement(),
h,f=f.getAscendant(N,1);if(!m(a,f)&&f&&!f.equals(a.editable)&&!f.contains(a.editable)){if((h=d(f))&&h.getAttribute("contenteditable")=="false")f=h;a.element=f;h=c(a,f,!e);var i;if(k(h)&&h.is(a.triggers)&&h.is(R)&&(!c(a,h,!e)||(i=c(a,h,!e))&&k(i)&&i.is(a.triggers)))g(h);else{i=b(a,f);if(k(i))if(c(a,i,!e))(f=c(a,i,!e))&&(k(f)&&f.is(a.triggers))&&g(i);else g(i)}}}}()}}function i(a,b){if(!b||!(b.type==CKEDITOR.NODE_ELEMENT&&b.$))return false;var c=a.line;return c.wrap.equals(b)||c.wrap.contains(b)}function k(a){return a&&
a.type==CKEDITOR.NODE_ELEMENT&&a.$}function n(a){if(!k(a))return false;var b;if(!(b=o(a)))if(k(a)){b={left:1,right:1,center:1};b=!(!b[a.getComputedStyle("float")]&&!b[a.getAttribute("align")])}else b=false;return b}function o(a){return!!{absolute:1,fixed:1}[a.getComputedStyle("position")]}function q(a,b){return k(b)?b.is(a.triggers):null}function m(a,b){if(!b)return false;for(var c=b.getParents(1),d=c.length;d--;)for(var f=a.tabuList.length;f--;)if(c[d].hasAttribute(a.tabuList[f]))return true;return false}
function l(a,b,c){b=b[c?"getLast":"getFirst"](function(b){return a.isRelevant(b)&&!b.is(K)});if(!b)return false;u(a,b);return c?b.size.top>a.mouse.y:b.size.bottom<a.mouse.y}function p(a){var b=a.editable,c=a.mouse,d=a.view,f=a.triggerOffset;w(a);var g=c.y>(a.inInlineMode?d.editable.top+d.editable.height/2:Math.min(d.editable.height,d.pane.height)/2),b=b[g?"getLast":"getFirst"](function(a){return!(Y(a)||Z(a))});if(!b)return null;i(a,b)&&(b=a.line.wrap[g?"getPrevious":"getNext"](function(a){return!(Y(a)||
Z(a))}));if(!k(b)||n(b)||!q(a,b))return null;u(a,b);if(!g&&b.size.top>=0&&c.y>0&&c.y<b.size.top+f){a=a.inInlineMode||d.scroll.y===0?L:Q;return new e([null,b,J,E,a])}if(g&&b.size.bottom<=d.pane.height&&c.y>b.size.bottom-f&&c.y<d.pane.height){a=a.inInlineMode||b.size.bottom>d.pane.height-f&&b.size.bottom<d.pane.height?H:Q;return new e([b,null,C,E,a])}return null}function t(a){var d=a.mouse,f=a.view,g=a.triggerOffset,h=b(a);if(!h)return null;u(a,h);var g=Math.min(g,0|h.size.outerHeight/2),i=[],j,m;if(d.y>
h.size.top-1&&d.y<h.size.top+g)m=false;else if(d.y>h.size.bottom-g&&d.y<h.size.bottom+1)m=true;else return null;if(n(h)||l(a,h,m)||h.getParent().is(M))return null;var p=c(a,h,!m);if(p){if(p&&p.type==CKEDITOR.NODE_TEXT)return null;if(k(p)){if(n(p)||!q(a,p)||p.getParent().is(M))return null;i=[p,h][m?"reverse":"concat"]().concat([F,E])}}else{if(h.equals(a.editable[m?"getLast":"getFirst"](a.isRelevant))){w(a);m&&d.y>h.size.bottom-g&&d.y<f.pane.height&&h.size.bottom>f.pane.height-g&&h.size.bottom<f.pane.height?
j=H:d.y>0&&d.y<h.size.top+g&&(j=L)}else j=Q;i=[null,h][m?"reverse":"concat"]().concat([m?C:J,E,j,h.equals(a.editable[m?"getLast":"getFirst"](a.isRelevant))?m?H:L:Q])}return 0 in i?new e(i):null}function s(a,b,c,d){for(var f=function(){var c=B.ie?b.$.currentStyle:a.win.$.getComputedStyle(b.$,"");return B.ie?function(a){return c[CKEDITOR.tools.cssStyleToDomStyle(a)]}:function(a){return c.getPropertyValue(a)}}(),e=b.getDocumentPosition(),g={},h={},i={},j={},k=X.length;k--;){g[X[k]]=parseInt(f("border-"+
X[k]+"-width"),10)||0;i[X[k]]=parseInt(f("padding-"+X[k]),10)||0;h[X[k]]=parseInt(f("margin-"+X[k]),10)||0}(!c||d)&&x(a,d);j.top=e.y-(c?0:a.view.scroll.y);j.left=e.x-(c?0:a.view.scroll.x);j.outerWidth=b.$.offsetWidth;j.outerHeight=b.$.offsetHeight;j.height=j.outerHeight-(i.top+i.bottom+g.top+g.bottom);j.width=j.outerWidth-(i.left+i.right+g.left+g.right);j.bottom=j.top+j.outerHeight;j.right=j.left+j.outerWidth;if(a.inInlineMode)j.scroll={top:b.$.scrollTop,left:b.$.scrollLeft};return r({border:g,padding:i,
margin:h,ignoreScroll:c},j,true)}function u(a,b,c){if(!k(b))return b.size=null;if(b.size){if(b.size.ignoreScroll==c&&b.size.date>new Date-T)return null}else b.size={};return r(b.size,s(a,b,c),{date:+new Date},true)}function w(a,b){a.view.editable=s(a,a.editable,b,true)}function x(a,b){if(!a.view)a.view={};var c=a.view;if(b||!(c&&c.date>new Date-T)){var d=a.win,c=d.getScrollPosition(),d=d.getViewPaneSize();r(a.view,{scroll:{x:c.x,y:c.y,width:a.doc.$.documentElement.scrollWidth-d.width,height:a.doc.$.documentElement.scrollHeight-
d.height},pane:{width:d.width,height:d.height,bottom:d.height+c.y},date:+new Date},true)}}function v(a,b,c,d){for(var f=d,g=d,h=0,i=false,j=false,k=a.view.pane.height,m=a.mouse;m.y+h<k&&m.y-h>0;){i||(i=b(f,d));j||(j=b(g,d));!i&&m.y-h>0&&(f=c(a,{x:m.x,y:m.y-h}));!j&&m.y+h<k&&(g=c(a,{x:m.x,y:m.y+h}));if(i&&j)break;h=h+2}return new e([f,g,null,null])}CKEDITOR.plugins.add("magicline",{init:function(a){var d=a.config,g=d.magicline_triggerOffset||30,l={editor:a,enterMode:d.enterMode,triggerOffset:g,holdDistance:0|
g*(d.magicline_holdDistance||0.5),boxColor:d.magicline_color||"#ff0000",rtl:d.contentsLangDirection=="rtl",tabuList:["data-cke-hidden-sel"].concat(d.magicline_tabuList||[]),triggers:d.magicline_everywhere?N:{table:1,hr:1,div:1,ul:1,ol:1,dl:1,form:1,blockquote:1}},q,u,v;l.isRelevant=function(a){return k(a)&&!i(l,a)&&!n(a)};a.on("contentDom",function(){var g=a.editable(),k=a.document,n=a.window;r(l,{editable:g,inInlineMode:g.isInline(),doc:k,win:n,hotNode:null},true);l.boundary=l.inInlineMode?l.editable:
l.doc.getDocumentElement();if(!g.is(G.$inline)){l.inInlineMode&&!o(g)&&g.setStyles({position:"relative",top:null,left:null});j.call(this,l);x(l);g.attachListener(a,"beforeUndoImage",function(){l.line.detach()});g.attachListener(a,"beforeGetData",function(){if(l.line.wrap.getParent()){l.line.detach();a.once("getData",function(){l.line.attach()},null,null,1E3)}},null,null,0);g.attachListener(l.inInlineMode?k:k.getWindow().getFrame(),"mouseout",function(b){if(a.mode=="wysiwyg")if(l.inInlineMode){var c=
b.data.$.clientX,b=b.data.$.clientY;x(l);w(l,true);var d=l.view.editable,f=l.view.scroll;if(!(c>d.left-f.x&&c<d.right-f.x)||!(b>d.top-f.y&&b<d.bottom-f.y)){clearTimeout(v);v=null;l.line.detach()}}else{clearTimeout(v);v=null;l.line.detach()}});g.attachListener(g,"keyup",function(){l.hiddenMode=0});g.attachListener(g,"keydown",function(b){if(a.mode=="wysiwyg")switch(b.data.getKeystroke()){case 2228240:case 16:l.hiddenMode=1;l.line.detach()}});g.attachListener(l.inInlineMode?g:k,"mousemove",function(b){u=
true;if(!(a.mode!="wysiwyg"||a.readOnly||v)){var c={x:b.data.$.clientX,y:b.data.$.clientY};v=setTimeout(function(){l.mouse=c;v=l.trigger=null;x(l);if(u&&!l.hiddenMode&&a.focusManager.hasFocus&&!l.line.mouseNear()&&(l.element=S(l,true))){if((l.trigger=p(l)||t(l)||ba(l))&&!m(l,l.trigger.upper||l.trigger.lower))l.line.attach().place();else{l.trigger=null;l.line.detach()}u=false}},30)}});g.attachListener(n,"scroll",function(){if(a.mode=="wysiwyg"){l.line.detach();if(B.webkit){l.hiddenMode=1;clearTimeout(q);
q=setTimeout(function(){if(!l.mouseDown)l.hiddenMode=0},50)}}});g.attachListener(A?k:n,"mousedown",function(){if(a.mode=="wysiwyg"){l.line.detach();l.hiddenMode=1;l.mouseDown=1}});g.attachListener(A?k:n,"mouseup",function(){l.hiddenMode=0;l.mouseDown=0});a.addCommand("accessPreviousSpace",h(l));a.addCommand("accessNextSpace",h(l,true));a.setKeystroke([[d.magicline_keystrokePrevious,"accessPreviousSpace"],[d.magicline_keystrokeNext,"accessNextSpace"]]);a.on("loadSnapshot",function(){var b,c,d,f;for(f in{p:1,
br:1,div:1}){b=a.document.getElementsByTag(f);for(d=b.count();d--;)if((c=b.getItem(d)).data("cke-magicline-hot")){l.hotNode=c;l.lastCmdDirection=c.data("cke-magicline-dir")==="true"?true:false;return}}});this.backdoor={accessFocusSpace:f,boxTrigger:e,isLine:i,getAscendantTrigger:b,getNonEmptyNeighbour:c,getSize:s,that:l,triggerEdge:t,triggerEditable:p,triggerExpand:ba}}},this)}});var r=CKEDITOR.tools.extend,y=CKEDITOR.dom.element,z=y.createFromHtml,B=CKEDITOR.env,A=CKEDITOR.env.ie&&CKEDITOR.env.version<
9,G=CKEDITOR.dtd,D={},J=128,C=64,F=32,E=16,I=8,L=4,H=2,Q=1,O=" ",M=G.$listItem,K=G.$tableContent,R=r({},G.$nonEditable,G.$empty),N=G.$block,T=100,P="width:0px;height:0px;padding:0px;margin:0px;display:block;z-index:9999;color:#fff;position:absolute;font-size: 0px;line-height:0px;",V=P+"border-color:transparent;display:block;border-style:solid;",W="<span>"+O+"</span>";D[CKEDITOR.ENTER_BR]="br";D[CKEDITOR.ENTER_P]="p";D[CKEDITOR.ENTER_DIV]="div";e.prototype={set:function(a,b,c){this.properties=a+b+
(c||Q);return this},is:function(a){return(this.properties&a)==a}};var S=function(){function a(b,c){var d=b.$.elementFromPoint(c.x,c.y);return d&&d.nodeType?new CKEDITOR.dom.element(d):null}return function(b,c,d){if(!b.mouse)return null;var f=b.doc,e=b.line.wrap,d=d||b.mouse,g=a(f,d);if(c&&i(b,g)){e.hide();g=a(f,d);e.show()}return!g||!(g.type==CKEDITOR.NODE_ELEMENT&&g.$)||B.ie&&B.version<9&&!b.boundary.equals(g)&&!b.boundary.contains(g)?null:g}}(),Y=CKEDITOR.dom.walker.whitespaces(),Z=CKEDITOR.dom.walker.nodeType(CKEDITOR.NODE_COMMENT),
ba=function(){function b(d){var f=d.element,e,h,i;if(!k(f)||f.contains(d.editable)||f.isReadOnly())return null;i=v(d,function(a,b){return!b.equals(a)},function(a,b){return S(a,true,b)},f);e=i.upper;h=i.lower;if(a(d,e,h))return i.set(F,I);if(e&&f.contains(e))for(;!e.getParent().equals(f);)e=e.getParent();else e=f.getFirst(function(a){return c(d,a)});if(h&&f.contains(h))for(;!h.getParent().equals(f);)h=h.getParent();else h=f.getLast(function(a){return c(d,a)});if(!e||!h)return null;u(d,e);u(d,h);if(!(d.mouse.y>
e.size.top&&d.mouse.y<h.size.bottom))return null;for(var f=Number.MAX_VALUE,j,m,l,q;h&&!h.equals(e);){if(!(m=e.getNext(d.isRelevant)))break;j=Math.abs(g(d,e,m)-d.mouse.y);if(j<f){f=j;l=e;q=m}e=m;u(d,e)}if(!l||!q||!(d.mouse.y>l.size.top&&d.mouse.y<q.size.bottom))return null;i.upper=l;i.lower=q;return i.set(F,I)}function c(a,b){return!(b&&b.type==CKEDITOR.NODE_TEXT||Z(b)||n(b)||i(a,b)||b.type==CKEDITOR.NODE_ELEMENT&&b.$&&b.is("br"))}return function(c){var d=b(c),f;if(f=d){f=d.upper;var e=d.lower;f=
!f||!e||n(e)||n(f)||e.equals(f)||f.equals(e)||e.contains(f)||f.contains(e)?false:q(c,f)&&q(c,e)&&a(c,f,e)?true:false}return f?d:null}}(),X=["top","left","right","bottom"]}(),CKEDITOR.config.magicline_keystrokePrevious=CKEDITOR.CTRL+CKEDITOR.SHIFT+51,CKEDITOR.config.magicline_keystrokeNext=CKEDITOR.CTRL+CKEDITOR.SHIFT+52,function(){function a(a){if(!a||a.type!=CKEDITOR.NODE_ELEMENT||a.getName()!="form")return[];for(var b=[],c=["style","className"],e=0;e<c.length;e++){var g=a.$.elements.namedItem(c[e]);
if(g){g=new CKEDITOR.dom.element(g);b.push([g,g.nextSibling]);g.remove()}}return b}function e(a,b){if(a&&!(a.type!=CKEDITOR.NODE_ELEMENT||a.getName()!="form")&&b.length>0)for(var c=b.length-1;c>=0;c--){var e=b[c][0],g=b[c][1];g?e.insertBefore(g):e.appendTo(a)}}function b(b,c){var f=a(b),g={},i=b.$;if(!c){g["class"]=i.className||"";i.className=""}g.inline=i.style.cssText||"";if(!c)i.style.cssText="position: static; overflow: visible";e(f);return g}function g(b,c){var f=a(b),g=b.$;if("class"in c)g.className=
c["class"];if("inline"in c)g.style.cssText=c.inline;e(f)}function c(a){if(!a.editable().isInline()){var b=CKEDITOR.instances,c;for(c in b){var e=b[c];if(e.mode=="wysiwyg"&&!e.readOnly){e=e.document.getBody();e.setAttribute("contentEditable",false);e.setAttribute("contentEditable",true)}}if(a.editable().hasFocus){a.toolbox.focus();a.focus()}}}CKEDITOR.plugins.add("maximize",{init:function(a){function e(){var b=i.getViewPaneSize();a.resize(b.width,b.height,null,true)}if(a.elementMode!=CKEDITOR.ELEMENT_MODE_INLINE){var f=
a.lang,h=CKEDITOR.document,i=h.getWindow(),k,n,o,q=CKEDITOR.TRISTATE_OFF;a.addCommand("maximize",{modes:{wysiwyg:!CKEDITOR.env.iOS,source:!CKEDITOR.env.iOS},readOnly:1,editorFocus:false,exec:function(){var m=a.container.getFirst(function(a){return a.type==CKEDITOR.NODE_ELEMENT&&a.hasClass("cke_inner")}),l=a.ui.space("contents");if(a.mode=="wysiwyg"){var p=a.getSelection();k=p&&p.getRanges();n=i.getScrollPosition()}else{var t=a.editable().$;k=!CKEDITOR.env.ie&&[t.selectionStart,t.selectionEnd];n=[t.scrollLeft,
t.scrollTop]}if(this.state==CKEDITOR.TRISTATE_OFF){i.on("resize",e);o=i.getScrollPosition();for(p=a.container;p=p.getParent();){p.setCustomData("maximize_saved_styles",b(p));p.setStyle("z-index",a.config.baseFloatZIndex-5)}l.setCustomData("maximize_saved_styles",b(l,true));m.setCustomData("maximize_saved_styles",b(m,true));l={overflow:CKEDITOR.env.webkit?"":"hidden",width:0,height:0};h.getDocumentElement().setStyles(l);!CKEDITOR.env.gecko&&h.getDocumentElement().setStyle("position","fixed");(!CKEDITOR.env.gecko||
!CKEDITOR.env.quirks)&&h.getBody().setStyles(l);CKEDITOR.env.ie?setTimeout(function(){i.$.scrollTo(0,0)},0):i.$.scrollTo(0,0);m.setStyle("position",CKEDITOR.env.gecko&&CKEDITOR.env.quirks?"fixed":"absolute");m.$.offsetLeft;m.setStyles({"z-index":a.config.baseFloatZIndex-5,left:"0px",top:"0px"});m.addClass("cke_maximized");e();l=m.getDocumentPosition();m.setStyles({left:-1*l.x+"px",top:-1*l.y+"px"});CKEDITOR.env.gecko&&c(a)}else if(this.state==CKEDITOR.TRISTATE_ON){i.removeListener("resize",e);l=[l,
m];for(p=0;p<l.length;p++){g(l[p],l[p].getCustomData("maximize_saved_styles"));l[p].removeCustomData("maximize_saved_styles")}for(p=a.container;p=p.getParent();){g(p,p.getCustomData("maximize_saved_styles"));p.removeCustomData("maximize_saved_styles")}CKEDITOR.env.ie?setTimeout(function(){i.$.scrollTo(o.x,o.y)},0):i.$.scrollTo(o.x,o.y);m.removeClass("cke_maximized");if(CKEDITOR.env.webkit){m.setStyle("display","inline");setTimeout(function(){m.setStyle("display","block")},0)}a.fire("resize")}this.toggleState();
if(p=this.uiItems[0]){l=this.state==CKEDITOR.TRISTATE_OFF?f.maximize.maximize:f.maximize.minimize;p=CKEDITOR.document.getById(p._.id);p.getChild(1).setHtml(l);p.setAttribute("title",l);p.setAttribute("href",'javascript:void("'+l+'");')}if(a.mode=="wysiwyg")if(k){CKEDITOR.env.gecko&&c(a);a.getSelection().selectRanges(k);(t=a.getSelection().getStartElement())&&t.scrollIntoView(true)}else i.$.scrollTo(n.x,n.y);else{if(k){t.selectionStart=k[0];t.selectionEnd=k[1]}t.scrollLeft=n[0];t.scrollTop=n[1]}k=
n=null;q=this.state;a.fire("maximize",this.state)},canUndo:false});a.ui.addButton&&a.ui.addButton("Maximize",{label:f.maximize.maximize,command:"maximize",toolbar:"tools,10"});a.on("mode",function(){var b=a.getCommand("maximize");b.setState(b.state==CKEDITOR.TRISTATE_DISABLED?CKEDITOR.TRISTATE_DISABLED:q)},null,null,100)}}})}(),function(){function a(a,e,c){var d=CKEDITOR.cleanWord;if(d)c();else{a=CKEDITOR.getUrl(a.config.pasteFromWordCleanupFile||e+"filter/default.js");CKEDITOR.scriptLoader.load(a,
c,null,true)}return!d}function e(a){a.data.type="html"}CKEDITOR.plugins.add("pastefromword",{requires:"clipboard",init:function(b){var g=0,c=this.path;b.addCommand("pastefromword",{canUndo:false,async:true,exec:function(a){var b=this;g=1;a.once("beforePaste",e);a.getClipboardData({title:a.lang.pastefromword.title},function(c){c&&a.fire("paste",{type:"html",dataValue:c.dataValue});a.fire("afterCommandExec",{name:"pastefromword",command:b,returnValue:!!c})})}});b.ui.addButton&&b.ui.addButton("PasteFromWord",
{label:b.lang.pastefromword.toolbar,command:"pastefromword",toolbar:"clipboard,50"});b.on("pasteState",function(a){b.getCommand("pastefromword").setState(a.data)});b.on("paste",function(d){var e=d.data,f=e.dataValue;if(f&&(g||/(class=\"?Mso|style=\"[^\"]*\bmso\-|w:WordDocument)/.test(f))){var h=a(b,c,function(){if(h)b.fire("paste",e);else if(!b.config.pasteFromWordPromptCleanup||g||confirm(b.lang.pastefromword.confirmCleanup))e.dataValue=CKEDITOR.cleanWord(f,b);g=0});h&&d.cancel()}},null,null,3)}})}(),
function(){var a={canUndo:false,async:true,exec:function(e){e.getClipboardData({title:e.lang.pastetext.title},function(b){b&&e.fire("paste",{type:"text",dataValue:b.dataValue});e.fire("afterCommandExec",{name:"pastetext",command:a,returnValue:!!b})})}};CKEDITOR.plugins.add("pastetext",{requires:"clipboard",init:function(e){e.addCommand("pastetext",a);e.ui.addButton&&e.ui.addButton("PasteText",{label:e.lang.pastetext.button,command:"pastetext",toolbar:"clipboard,40"});if(e.config.forcePasteAsPlainText)e.on("beforePaste",
function(a){if(a.data.type!="html")a.data.type="text"});e.on("pasteState",function(a){e.getCommand("pastetext").setState(a.data)})}})}(),CKEDITOR.plugins.add("removeformat",{init:function(a){a.addCommand("removeFormat",CKEDITOR.plugins.removeformat.commands.removeformat);a.ui.addButton&&a.ui.addButton("RemoveFormat",{label:a.lang.removeformat.toolbar,command:"removeFormat",toolbar:"cleanup,10"})}}),CKEDITOR.plugins.removeformat={commands:{removeformat:{exec:function(a){for(var e=a._.removeFormatRegex||
(a._.removeFormatRegex=RegExp("^(?:"+a.config.removeFormatTags.replace(/,/g,"|")+")$","i")),b=a._.removeAttributes||(a._.removeAttributes=a.config.removeFormatAttributes.split(",")),g=CKEDITOR.plugins.removeformat.filter,c=a.getSelection().getRanges(),d=c.createIterator(),j=function(a){return a.type==CKEDITOR.NODE_ELEMENT},f;f=d.getNextRange();){f.collapsed||f.enlarge(CKEDITOR.ENLARGE_ELEMENT);var h=f.createBookmark(),i=h.startNode,k=h.endNode,n=function(b){for(var c=a.elementPath(b),d=c.elements,
f=1,h;h=d[f];f++){if(h.equals(c.block)||h.equals(c.blockLimit))break;e.test(h.getName())&&g(a,h)&&b.breakParent(h)}};n(i);if(k){n(k);for(i=i.getNextSourceNode(true,CKEDITOR.NODE_ELEMENT);i;){if(i.equals(k))break;if(i.isReadOnly()){if(i.getPosition(k)&CKEDITOR.POSITION_CONTAINS)break;i=i.getNext(j)}else{n=i.getNextSourceNode(false,CKEDITOR.NODE_ELEMENT);if(!(i.getName()=="img"&&i.data("cke-realelement"))&&g(a,i))if(e.test(i.getName()))i.remove(1);else{i.removeAttributes(b);a.fire("removeFormatCleanup",
i)}i=n}}}f.moveToBookmark(h)}a.forceNextSelectionCheck();a.getSelection().selectRanges(c)}}},filter:function(a,e){for(var b=a._.removeFormatFilters||[],g=0;g<b.length;g++)if(b[g](e)===false)return false;return true}},CKEDITOR.editor.prototype.addRemoveFormatFilter=function(a){if(!this._.removeFormatFilters)this._.removeFormatFilters=[];this._.removeFormatFilters.push(a)},CKEDITOR.config.removeFormatTags="b,big,cite,code,del,dfn,em,font,i,ins,kbd,q,s,samp,small,span,strike,strong,sub,sup,tt,u,var",
CKEDITOR.config.removeFormatAttributes="class,style,lang,width,height,align,hspace,valign",CKEDITOR.plugins.add("resize",{init:function(a){var e,b,g,c;function d(d){var h=e,j=b,k=h+(d.data.$.screenX-g)*(i=="rtl"?-1:1),d=j+(d.data.$.screenY-c);n&&(h=Math.max(f.resize_minWidth,Math.min(k,f.resize_maxWidth)));o&&(j=Math.max(f.resize_minHeight,Math.min(d,f.resize_maxHeight)));a.resize(n?h:null,j)}function j(){CKEDITOR.document.removeListener("mousemove",d);CKEDITOR.document.removeListener("mouseup",j);
if(a.document){a.document.removeListener("mousemove",d);a.document.removeListener("mouseup",j)}}var f=a.config,h=a.ui.spaceId("resizer"),i=a.element?a.element.getDirection(1):"ltr";!f.resize_dir&&(f.resize_dir="vertical");f.resize_maxWidth===void 0&&(f.resize_maxWidth=3E3);f.resize_maxHeight===void 0&&(f.resize_maxHeight=3E3);f.resize_minWidth===void 0&&(f.resize_minWidth=750);f.resize_minHeight===void 0&&(f.resize_minHeight=250);if(f.resize_enabled!==false){var k=null,n=(f.resize_dir=="both"||f.resize_dir==
"horizontal")&&f.resize_minWidth!=f.resize_maxWidth,o=(f.resize_dir=="both"||f.resize_dir=="vertical")&&f.resize_minHeight!=f.resize_maxHeight,q=CKEDITOR.tools.addFunction(function(h){k||(k=a.getResizable());e=k.$.offsetWidth||0;b=k.$.offsetHeight||0;g=h.screenX;c=h.screenY;f.resize_minWidth>e&&(f.resize_minWidth=e);f.resize_minHeight>b&&(f.resize_minHeight=b);CKEDITOR.document.on("mousemove",d);CKEDITOR.document.on("mouseup",j);if(a.document){a.document.on("mousemove",d);a.document.on("mouseup",
j)}h.preventDefault&&h.preventDefault()});a.on("destroy",function(){CKEDITOR.tools.removeFunction(q)});a.on("uiSpace",function(b){if(b.data.space=="bottom"){var c="";n&&!o&&(c=" cke_resizer_horizontal");!n&&o&&(c=" cke_resizer_vertical");var d='<span id="'+h+'" class="cke_resizer'+c+" cke_resizer_"+i+'" title="'+CKEDITOR.tools.htmlEncode(a.lang.common.resize)+'" onmousedown="CKEDITOR.tools.callFunction('+q+', event)">'+(i=="ltr"?"◢":"◣")+"</span>";i=="ltr"&&c=="ltr"?b.data.html=b.data.html+d:b.data.html=
d+b.data.html}},a,null,100);a.on("maximize",function(b){a.ui.space("resizer")[b.data==CKEDITOR.TRISTATE_ON?"hide":"show"]()})}}}),CKEDITOR.plugins.add("menubutton",{requires:"button,menu",onLoad:function(){var a=function(a){var b=this._,g=b.menu;if(b.state!==CKEDITOR.TRISTATE_DISABLED)if(b.on&&g)g.hide();else{b.previousState=b.state;if(!g){g=b.menu=new CKEDITOR.menu(a,{panel:{className:"cke_menu_panel",attributes:{"aria-label":a.lang.common.options}}});g.onHide=CKEDITOR.tools.bind(function(){var c=
this.command?a.getCommand(this.command).modes:this.modes;this.setState(!c||c[a.mode]?b.previousState:CKEDITOR.TRISTATE_DISABLED);b.on=0},this);this.onMenu&&g.addListener(this.onMenu)}this.setState(CKEDITOR.TRISTATE_ON);b.on=1;setTimeout(function(){g.show(CKEDITOR.document.getById(b.id),4)},0)}};CKEDITOR.ui.menuButton=CKEDITOR.tools.createClass({base:CKEDITOR.ui.button,$:function(e){delete e.panel;this.base(e);this.hasArrow=true;this.click=a},statics:{handler:{create:function(a){return new CKEDITOR.ui.menuButton(a)}}}})},
beforeInit:function(a){a.ui.addHandler(CKEDITOR.UI_MENUBUTTON,CKEDITOR.ui.menuButton.handler)}}),CKEDITOR.UI_MENUBUTTON="menubutton","use strict",CKEDITOR.plugins.add("scayt",{requires:"menubutton,dialog",tabToOpen:null,dialogName:"scaytDialog",init:function(a){var e=this,b=CKEDITOR.plugins.scayt;this.bindEvents(a);this.parseConfig(a);this.addRule(a);CKEDITOR.dialog.add(this.dialogName,CKEDITOR.getUrl(this.path+"dialogs/options.js"));this.addMenuItems(a);var g=a.lang.scayt,c=CKEDITOR.env;a.ui.add("Scayt",
CKEDITOR.UI_MENUBUTTON,{label:g.text_title,title:a.plugins.wsc?a.lang.wsc.title:g.text_title,modes:{wysiwyg:!(c.ie&&(c.version<8||c.quirks))},toolbar:"spellchecker,20",refresh:function(){var c=a.ui.instances.Scayt.getState();a.scayt&&(c=b.state[a.name]?CKEDITOR.TRISTATE_ON:CKEDITOR.TRISTATE_OFF);a.fire("scaytButtonState",c)},onRender:function(){var b=this;a.on("scaytButtonState",function(a){typeof a.data!==void 0&&b.setState(a.data)})},onMenu:function(){var c=a.scayt;a.getMenuItem("scaytToggle").label=
a.lang.scayt[c&&b.state[a.name]?"btn_disable":"btn_enable"];c={scaytToggle:CKEDITOR.TRISTATE_OFF,scaytOptions:c?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED,scaytLangs:c?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED,scaytDict:c?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED,scaytAbout:c?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED,WSC:a.plugins.wsc?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED};a.config.scayt_uiTabs[0]||delete c.scaytOptions;a.config.scayt_uiTabs[1]||delete c.scaytLangs;
a.config.scayt_uiTabs[2]||delete c.scaytDict;return c}});if(a.contextMenu&&a.addMenuItems){a.contextMenu.addListener(function(){var b=a.scayt,c;if(b){var f=b.getSelectionNode();if(f=f?f.getAttribute(b.getNodeAttribute()):f){c=e.menuGenerator(a,f,e);b.showBanner("."+a.contextMenu._.definition.panel.className.split(" ").join(" ."))}}return c});a.contextMenu._.onHide=CKEDITOR.tools.override(a.contextMenu._.onHide,function(b){return function(){var c=a.scayt;c&&c.hideBanner();return b.apply(this)}})}},
addMenuItems:function(a){var e=this,b=CKEDITOR.plugins.scayt;a.addMenuGroup("scaytButton");var g=a.config.scayt_contextMenuItemsOrder.split("|");if(g&&g.length)for(var c=0;c<g.length;c++)a.addMenuGroup("scayt_"+g[c],c-10);g={scaytToggle:{label:a.lang.scayt.btn_enable,group:"scaytButton",onClick:function(){var c=a.scayt;b.state[a.name]=!b.state[a.name];b.state[a.name]===true?c||b.createScayt(a):c&&b.destroy(a)}},scaytAbout:{label:a.lang.scayt.btn_about,group:"scaytButton",onClick:function(){a.scayt.tabToOpen=
"about";a.lockSelection();a.openDialog(e.dialogName)}},scaytOptions:{label:a.lang.scayt.btn_options,group:"scaytButton",onClick:function(){a.scayt.tabToOpen="options";a.lockSelection();a.openDialog(e.dialogName)}},scaytLangs:{label:a.lang.scayt.btn_langs,group:"scaytButton",onClick:function(){a.scayt.tabToOpen="langs";a.lockSelection();a.openDialog(e.dialogName)}},scaytDict:{label:a.lang.scayt.btn_dictionaries,group:"scaytButton",onClick:function(){a.scayt.tabToOpen="dictionaries";a.lockSelection();
a.openDialog(e.dialogName)}}};if(a.plugins.wsc)g.WSC={label:a.lang.wsc.toolbar,group:"scaytButton",onClick:function(){var b=CKEDITOR.plugins.scayt,c=a.scayt,f=a.elementMode==CKEDITOR.ELEMENT_MODE_INLINE?a.container.getText():a.document.getBody().getText();if(f=f.replace(/\s/g,"")){c&&(b.state[a.name]&&c.setMarkupPaused)&&c.setMarkupPaused(true);a.lockSelection();a.execCommand("checkspell")}else alert("Nothing to check!")}};a.addMenuItems(g)},bindEvents:function(a){function e(){var b=a.scayt;if(b){b.removeMarkupInSelectionNode();
b.fire("startSpellCheck")}}var b=CKEDITOR.plugins.scayt,g=a.elementMode==CKEDITOR.ELEMENT_MODE_INLINE;CKEDITOR.on("dialogDefinition",function(a){if(a.data.name==="scaytDialog")a.data.definition.dialog.on("cancel",function(){return false},this,null,-1)});var c=function(){a.scayt&&b.destroy(a)},d=function(){b.state[a.name]&&!a.readOnly&&b.createScayt(a)},j=function(){var b=a.editable();b.attachListener(b,"focus",function(){var b=CKEDITOR.plugins.scayt&&CKEDITOR.plugins.scayt.state[a.name]&&a.scayt,
c,d;if((g||b)&&a._.savedSelection)for(var b=a._.savedSelection.getSelectedElement(),b=!b&&a._.savedSelection.getRanges(),f=0;f<b.length;f++){d=b[f];c=d.startContainer.getText().length;(c<d.startOffset||c<d.endOffset)&&a.unlockSelection(false)}},this,null,-10)},f=function(){if(g){a.on("blur",c);a.on("focus",d);a.focusManager.hasFocus&&d()}else d();j()};a.on("contentDom",f);a.on("beforeCommandExec",function(c){var d;if(c.data.name in b.options.disablingCommandExec&&a.mode=="wysiwyg"){if(d=a.scayt){b.destroy(a);
a.fire("scaytButtonState",CKEDITOR.TRISTATE_DISABLED)}}else if(c.data.name==="bold"||c.data.name==="italic"||c.data.name==="underline"||c.data.name==="strike"||c.data.name==="subscript"||c.data.name==="superscript")if(d=a.scayt){d.removeMarkupInSelectionNode();setTimeout(function(){d.fire("startSpellCheck")},0)}});a.on("beforeSetMode",function(c){if(c.data=="source"){if(c=a.scayt){b.destroy(a);a.fire("scaytButtonState",CKEDITOR.TRISTATE_DISABLED)}if(a.document){a.document.getBody().removeAttribute("_jquid");
a.document.getBody().removeAttribute("dir")}}});a.on("afterCommandExec",function(b){var c;if(a.mode=="wysiwyg"&&(b.data.name=="undo"||b.data.name=="redo"))(c=a.scayt)&&setTimeout(function(){c.fire("startSpellCheck")},250)});a.on("readOnly",function(c){var d;if(c){d=a.scayt;if(c.editor.readOnly===true)d&&d.fire("removeMarkupInDocument",{});else if(d)d.fire("startSpellCheck");else if(c.editor.mode=="wysiwyg"&&b.state[c.editor.name]===true){b.createScayt(a);c.editor.fire("scaytButtonState",CKEDITOR.TRISTATE_ON)}}});
a.on("beforeDestroy",c);a.on("setData",function(){c();a.elementMode==CKEDITOR.ELEMENT_MODE_INLINE&&f()},this,null,50);a.on("insertElement",function(){CKEDITOR.env.ie?setTimeout(function(){e()},50):e()},this,null,50);a.on("insertHtml",function(){e()},this,null,50);a.on("insertText",function(){e()},this,null,50);a.on("scaytDialogShown",function(b){b.data.selectPage(a.scayt.tabToOpen)})},parseConfig:function(a){var e=CKEDITOR.plugins.scayt;e.replaceOldOptionsNames(a.config);if(typeof a.config.scayt_autoStartup!==
"boolean")a.config.scayt_autoStartup=false;e.state[a.name]=a.config.scayt_autoStartup;if(!a.config.scayt_contextCommands)a.config.scayt_contextCommands="ignore|ignoreall|add";if(!a.config.scayt_contextMenuItemsOrder)a.config.scayt_contextMenuItemsOrder="suggest|moresuggest|control";if(!a.config.scayt_sLang)a.config.scayt_sLang="en_US";if(a.config.scayt_maxSuggestions===void 0||typeof a.config.scayt_maxSuggestions!="number"||a.config.scayt_maxSuggestions<0)a.config.scayt_maxSuggestions=5;if(a.config.scayt_customDictionaryIds===
void 0||typeof a.config.scayt_customDictionaryIds!=="string")a.config.scayt_customDictionaryIds="";if(a.config.scayt_userDictionaryName===void 0||typeof a.config.scayt_userDictionaryName!=="string")a.config.scayt_userDictionaryName=null;if(typeof a.config.scayt_uiTabs==="string"&&a.config.scayt_uiTabs.split(",").length===3){var b=[],g=[];a.config.scayt_uiTabs=a.config.scayt_uiTabs.split(",");CKEDITOR.tools.search(a.config.scayt_uiTabs,function(a){if(Number(a)===1||Number(a)===0){g.push(true);b.push(Number(a))}else g.push(false)});
a.config.scayt_uiTabs=CKEDITOR.tools.search(g,false)===null?b:[1,1,1]}else a.config.scayt_uiTabs=[1,1,1];if(typeof a.config.scayt_serviceProtocol!="string")a.config.scayt_serviceProtocol=null;if(typeof a.config.scayt_serviceHost!="string")a.config.scayt_serviceHost=null;if(typeof a.config.scayt_servicePort!="string")a.config.scayt_servicePort=null;if(typeof a.config.scayt_servicePath!="string")a.config.scayt_servicePath=null;if(!a.config.scayt_moreSuggestions)a.config.scayt_moreSuggestions="on";if(typeof a.config.scayt_customerId!==
"string")a.config.scayt_customerId="1:WvF0D4-UtPqN1-43nkD4-NKvUm2-daQqk3-LmNiI-z7Ysb4-mwry24-T8YrS3-Q2tpq2";if(typeof a.config.scayt_srcUrl!=="string"){e=document.location.protocol;e=e.search(/https?:/)!=-1?e:"http:";a.config.scayt_srcUrl=e+"//svc.webspellchecker.net/spellcheck31/lf/scayt3/ckscayt/ckscayt.js"}if(typeof CKEDITOR.config.scayt_handleCheckDirty!=="boolean")CKEDITOR.config.scayt_handleCheckDirty=true;if(typeof CKEDITOR.config.scayt_handleUndoRedo!=="boolean")CKEDITOR.config.scayt_handleUndoRedo=
true;if(a.config.scayt_disableOptionsStorage){var e=CKEDITOR.tools.isArray(a.config.scayt_disableOptionsStorage)?a.config.scayt_disableOptionsStorage:typeof a.config.scayt_disableOptionsStorage==="string"?[a.config.scayt_disableOptionsStorage]:void 0,c=["all","options","lang","ignore-all-caps-words","ignore-domain-names","ignore-words-with-mixed-cases","ignore-words-with-numbers"],d=["lang","ignore-all-caps-words","ignore-domain-names","ignore-words-with-mixed-cases","ignore-words-with-numbers"],
j=CKEDITOR.tools.search,f=CKEDITOR.tools.indexOf;a.config.scayt_disableOptionsStorage=function(a){for(var b=[],e=0;e<a.length;e++){var g=a[e],o=!!j(a,"options");if(!j(c,g)||o&&j(d,function(a){if(a==="lang")return false}))return;j(d,g)&&d.splice(f(d,g),1);if(g==="all"||o&&j(a,"lang"))return[];g==="options"&&(d=["lang"])}return b=b.concat(d)}(e)}},addRule:function(a){var e=a.dataProcessor,b=e&&e.htmlFilter,g=a._.elementsPath&&a._.elementsPath.filters,e=e&&e.dataFilter,c=a.addRemoveFormatFilter,d=function(b){var c=
CKEDITOR.plugins.scayt;if(a.scayt&&b.hasAttribute(c.options.data_attribute_name))return false},j=function(b){var c=CKEDITOR.plugins.scayt,d=true;a.scayt&&b.hasAttribute(c.options.data_attribute_name)&&(d=false);return d};g&&g.push(d);e&&e.addRules({elements:{span:function(b){var c=CKEDITOR.plugins.scayt;if(c&&c.state[a.name]&&b.classes&&CKEDITOR.tools.search(b.classes,c.options.misspelled_word_class))if(b.classes&&b.parent.type===CKEDITOR.NODE_DOCUMENT_FRAGMENT){delete b.attributes.style;delete b.name}else delete b.classes[CKEDITOR.tools.indexOf(b.classes,
c.options.misspelled_word_class)];return b}}});b&&b.addRules({elements:{span:function(b){var c=CKEDITOR.plugins.scayt;if(c&&c.state[a.name]&&b.hasClass(c.options.misspelled_word_class)&&b.attributes[c.options.data_attribute_name]){b.removeClass(c.options.misspelled_word_class);delete b.attributes[c.options.data_attribute_name];delete b.name}return b}}});c&&c.call(a,j)},scaytMenuDefinition:function(a){var e=this,a=a.scayt;return{scayt_ignore:{label:a.getLocal("btn_ignore"),group:"scayt_control",order:1,
exec:function(a){a.scayt.ignoreWord()}},scayt_ignoreall:{label:a.getLocal("btn_ignoreAll"),group:"scayt_control",order:2,exec:function(a){a.scayt.ignoreAllWords()}},scayt_add:{label:a.getLocal("btn_addWord"),group:"scayt_control",order:3,exec:function(a){var e=a.scayt;setTimeout(function(){e.addWordToUserDictionary()},10)}},option:{label:a.getLocal("btn_options"),group:"scayt_control",order:4,exec:function(a){a.scayt.tabToOpen="options";a.lockSelection();a.openDialog(e.dialogName)},verification:function(a){return a.config.scayt_uiTabs[0]==
1?true:false}},language:{label:a.getLocal("btn_langs"),group:"scayt_control",order:5,exec:function(a){a.scayt.tabToOpen="langs";a.lockSelection();a.openDialog(e.dialogName)},verification:function(a){return a.config.scayt_uiTabs[1]==1?true:false}},dictionary:{label:a.getLocal("btn_dictionaries"),group:"scayt_control",order:6,exec:function(a){a.scayt.tabToOpen="dictionaries";a.lockSelection();a.openDialog(e.dialogName)},verification:function(a){return a.config.scayt_uiTabs[2]==1?true:false}},about:{label:a.getLocal("btn_about"),
group:"scayt_control",order:7,exec:function(a){a.scayt.tabToOpen="about";a.lockSelection();a.openDialog(e.dialogName)}}}},buildSuggestionMenuItems:function(a,e){var b={},g={},c=a.scayt;if(e.length>0&&e[0]!=="no_any_suggestions")for(var d=0;d<e.length;d++){var j="scayt_suggest_"+CKEDITOR.plugins.scayt.suggestions[d].replace(" ","_");a.addCommand(j,this.createCommand(CKEDITOR.plugins.scayt.suggestions[d]));if(d<a.config.scayt_maxSuggestions){a.addMenuItem(j,{label:e[d],command:j,group:"scayt_suggest",
order:d+1});b[j]=CKEDITOR.TRISTATE_OFF}else{a.addMenuItem(j,{label:e[d],command:j,group:"scayt_moresuggest",order:d+1});g[j]=CKEDITOR.TRISTATE_OFF;if(a.config.scayt_moreSuggestions==="on"){a.addMenuItem("scayt_moresuggest",{label:c.getLocal("btn_moreSuggestions"),group:"scayt_moresuggest",order:10,getItems:function(){return g}});b.scayt_moresuggest=CKEDITOR.TRISTATE_OFF}}}else{b.no_scayt_suggest=CKEDITOR.TRISTATE_DISABLED;a.addCommand("no_scayt_suggest",{exec:function(){}});a.addMenuItem("no_scayt_suggest",
{label:c.getLocal("btn_noSuggestions")||"no_scayt_suggest",command:"no_scayt_suggest",group:"scayt_suggest",order:0})}return b},menuGenerator:function(a,e){var b=a.scayt,g=this.scaytMenuDefinition(a),c={},d=a.config.scayt_contextCommands.split("|");b.fire("getSuggestionsList",{lang:b.getLang(),word:e});c=this.buildSuggestionMenuItems(a,CKEDITOR.plugins.scayt.suggestions);if(a.config.scayt_contextCommands=="off")return c;for(var j in g)if(!(CKEDITOR.tools.indexOf(d,j.replace("scayt_",""))==-1&&a.config.scayt_contextCommands!=
"all")){c[j]=CKEDITOR.TRISTATE_OFF;typeof g[j].verification==="function"&&!g[j].verification(a)&&delete c[j];a.addCommand(j,{exec:g[j].exec});a.addMenuItem(j,{label:a.lang.scayt[g[j].label]||g[j].label,command:j,group:g[j].group,order:g[j].order})}return c},createCommand:function(a){return{exec:function(e){e.scayt.replaceSelectionNode({word:a})}}}}),CKEDITOR.plugins.scayt={state:{},suggestions:[],loadingHelper:{loadOrder:[]},isLoading:!1,options:{disablingCommandExec:{source:!0,newpage:!0,templates:!0},
data_attribute_name:"data-scayt-word",misspelled_word_class:"scayt-misspell-word"},backCompatibilityMap:{scayt_service_protocol:"scayt_serviceProtocol",scayt_service_host:"scayt_serviceHost",scayt_service_port:"scayt_servicePort",scayt_service_path:"scayt_servicePath",scayt_customerid:"scayt_customerId"},replaceOldOptionsNames:function(a){for(var e in a)if(e in this.backCompatibilityMap){a[this.backCompatibilityMap[e]]=a[e];delete a[e]}},createScayt:function(a){var e=this;this.loadScaytLibrary(a,
function(a){var g={lang:a.config.scayt_sLang,container:a.editable().$.nodeName=="BODY"?a.document.getWindow().$.frameElement:a.editable().$,customDictionary:a.config.scayt_customDictionaryIds,userDictionaryName:a.config.scayt_userDictionaryName,localization:a.langCode,customer_id:a.config.scayt_customerId,debug:a.config.scayt_debug,data_attribute_name:e.options.data_attribute_name,misspelled_word_class:e.options.misspelled_word_class,"options-to-restore":a.config.scayt_disableOptionsStorage,focused:a.editable().hasFocus,
ignoreElementsRegex:a.config.scayt_elementsToIgnore};if(a.config.scayt_serviceProtocol)g.service_protocol=a.config.scayt_serviceProtocol;if(a.config.scayt_serviceHost)g.service_host=a.config.scayt_serviceHost;if(a.config.scayt_servicePort)g.service_port=a.config.scayt_servicePort;if(a.config.scayt_servicePath)g.service_path=a.config.scayt_servicePath;g=new SCAYT.CKSCAYT(g,function(){},function(){});g.subscribe("suggestionListSend",function(a){for(var b={},e=[],f=0;f<a.suggestionList.length;f++)if(!b["word_"+
a.suggestionList[f]]){b["word_"+a.suggestionList[f]]=a.suggestionList[f];e.push(a.suggestionList[f])}CKEDITOR.plugins.scayt.suggestions=e});a.scayt=g;a.fire("scaytButtonState",a.readOnly?CKEDITOR.TRISTATE_DISABLED:CKEDITOR.TRISTATE_ON)})},destroy:function(a){a.scayt&&a.scayt.destroy();delete a.scayt;a.fire("scaytButtonState",CKEDITOR.TRISTATE_OFF)},loadScaytLibrary:function(a,e){var b=this;if(typeof window.SCAYT==="undefined"||typeof window.SCAYT.CKSCAYT!=="function"){this.loadingHelper[a.name]=e;
this.loadingHelper.loadOrder.push(a.name);CKEDITOR.scriptLoader.load(a.config.scayt_srcUrl,function(){var a;CKEDITOR.fireOnce("scaytReady");for(var c=0;c<b.loadingHelper.loadOrder.length;c++){a=b.loadingHelper.loadOrder[c];if(typeof b.loadingHelper[a]==="function")b.loadingHelper[a](CKEDITOR.instances[a]);delete b.loadingHelper[a]}b.loadingHelper.loadOrder=[]})}else if(window.SCAYT&&typeof window.SCAYT.CKSCAYT==="function"){CKEDITOR.fireOnce("scaytReady");a.scayt||typeof e==="function"&&e(a)}}},CKEDITOR.on("scaytReady",
function(){if(CKEDITOR.config.scayt_handleCheckDirty===true){var a=CKEDITOR.editor.prototype;a.checkDirty=CKEDITOR.tools.override(a.checkDirty,function(a){return function(){var e=null,c=this.scayt;if(!CKEDITOR.plugins.scayt||!CKEDITOR.plugins.scayt.state[this.name]||!this.scayt)e=a.call(this);else if(e=this.status=="ready")var d=c.removeMarkupFromString(this.getSnapshot()),c=c.removeMarkupFromString(this._.previousValue),e=e&&c!==d;return e}});a.resetDirty=CKEDITOR.tools.override(a.resetDirty,function(a){return function(){var e=
this.scayt;!CKEDITOR.plugins.scayt||!CKEDITOR.plugins.scayt.state[this.name]||!this.scayt?a.call(this):this._.previousValue=e.removeMarkupFromString(this.getSnapshot())}})}if(CKEDITOR.config.scayt_handleUndoRedo===true){var a=CKEDITOR.plugins.undo.Image.prototype,e=typeof a.equalsContent=="function"?"equalsContent":"equals";a[e]=CKEDITOR.tools.override(a[e],function(a){return function(e){var c=e.editor.scayt,d=this.contents,j=e.contents,f=null;if(CKEDITOR.plugins.scayt&&CKEDITOR.plugins.scayt.state[e.editor.name]&&
e.editor.scayt){this.contents=c.removeMarkupFromString(d)||"";e.contents=c.removeMarkupFromString(j)||""}f=a.apply(this,arguments);this.contents=d;e.contents=j;return f}})}}),function(){var a={preserveState:true,editorFocus:false,readOnly:1,exec:function(a){this.toggleState();this.refresh(a)},refresh:function(a){if(a.document){var b=this.state==CKEDITOR.TRISTATE_ON?"attachClass":"removeClass";a.editable()[b]("cke_show_borders")}}};CKEDITOR.plugins.add("showborders",{modes:{wysiwyg:1},onLoad:function(){var a;
a=(CKEDITOR.env.ie6Compat?[".%1 table.%2,",".%1 table.%2 td, .%1 table.%2 th","{","border : #d3d3d3 1px dotted","}"]:[".%1 table.%2,",".%1 table.%2 > tr > td, .%1 table.%2 > tr > th,",".%1 table.%2 > tbody > tr > td, .%1 table.%2 > tbody > tr > th,",".%1 table.%2 > thead > tr > td, .%1 table.%2 > thead > tr > th,",".%1 table.%2 > tfoot > tr > td, .%1 table.%2 > tfoot > tr > th","{","border : #d3d3d3 1px dotted","}"]).join("").replace(/%2/g,"cke_show_border").replace(/%1/g,"cke_show_borders ");CKEDITOR.addCss(a)},
init:function(e){var b=e.addCommand("showborders",a);b.canUndo=false;e.config.startupShowBorders!==false&&b.setState(CKEDITOR.TRISTATE_ON);e.on("mode",function(){b.state!=CKEDITOR.TRISTATE_DISABLED&&b.refresh(e)},null,null,100);e.on("contentDom",function(){b.state!=CKEDITOR.TRISTATE_DISABLED&&b.refresh(e)});e.on("removeFormatCleanup",function(a){a=a.data;e.getCommand("showborders").state==CKEDITOR.TRISTATE_ON&&(a.is("table")&&(!a.hasAttribute("border")||parseInt(a.getAttribute("border"),10)<=0))&&
a.addClass("cke_show_border")})},afterInit:function(a){var b=a.dataProcessor,a=b&&b.dataFilter,b=b&&b.htmlFilter;a&&a.addRules({elements:{table:function(a){var a=a.attributes,b=a["class"],d=parseInt(a.border,10);if((!d||d<=0)&&(!b||b.indexOf("cke_show_border")==-1))a["class"]=(b||"")+" cke_show_border"}}});b&&b.addRules({elements:{table:function(a){var a=a.attributes,b=a["class"];b&&(a["class"]=b.replace("cke_show_border","").replace(/\s{2}/," ").replace(/^\s+|\s+$/,""))}}})}});CKEDITOR.on("dialogDefinition",
function(a){var b=a.data.name;if(b=="table"||b=="tableProperties"){a=a.data.definition;b=a.getContents("info").get("txtBorder");b.commit=CKEDITOR.tools.override(b.commit,function(a){return function(b,d){a.apply(this,arguments);var e=parseInt(this.getValue(),10);d[!e||e<=0?"addClass":"removeClass"]("cke_show_border")}});if(a=(a=a.getContents("advanced"))&&a.get("advCSSClasses")){a.setup=CKEDITOR.tools.override(a.setup,function(a){return function(){a.apply(this,arguments);this.setValue(this.getValue().replace(/cke_show_border/,
""))}});a.commit=CKEDITOR.tools.override(a.commit,function(a){return function(b,d){a.apply(this,arguments);parseInt(d.getAttribute("border"),10)||d.addClass("cke_show_border")}})}}})}(),function(){CKEDITOR.plugins.add("sourcearea",{init:function(e){function b(){var a=c&&this.equals(CKEDITOR.document.getActive());this.hide();this.setStyle("height",this.getParent().$.clientHeight+"px");this.setStyle("width",this.getParent().$.clientWidth+"px");this.show();a&&this.focus()}if(e.elementMode!=CKEDITOR.ELEMENT_MODE_INLINE){var g=
CKEDITOR.plugins.sourcearea;e.addMode("source",function(c){var g=e.ui.space("contents").getDocument().createElement("textarea");g.setStyles(CKEDITOR.tools.extend({width:CKEDITOR.env.ie7Compat?"99%":"100%",height:"100%",resize:"none",outline:"none","text-align":"left"},CKEDITOR.tools.cssVendorPrefix("tab-size",e.config.sourceAreaTabSize||4)));g.setAttribute("dir","ltr");g.addClass("cke_source cke_reset cke_enable_context_menu");e.ui.space("contents").append(g);g=e.editable(new a(e,g));g.setData(e.getData(1));
if(CKEDITOR.env.ie){g.attachListener(e,"resize",b,g);g.attachListener(CKEDITOR.document.getWindow(),"resize",b,g);CKEDITOR.tools.setTimeout(b,0,g)}e.fire("ariaWidget",this);c()});e.addCommand("source",g.commands.source);e.ui.addButton&&e.ui.addButton("Source",{label:e.lang.sourcearea.toolbar,command:"source",toolbar:"mode,10"});e.on("mode",function(){e.getCommand("source").setState(e.mode=="source"?CKEDITOR.TRISTATE_ON:CKEDITOR.TRISTATE_OFF)});var c=CKEDITOR.env.ie&&CKEDITOR.env.version==9}}});var a=
CKEDITOR.tools.createClass({base:CKEDITOR.editable,proto:{setData:function(a){this.setValue(a);this.status="ready";this.editor.fire("dataReady")},getData:function(){return this.getValue()},insertHtml:function(){},insertElement:function(){},insertText:function(){},setReadOnly:function(a){this[(a?"set":"remove")+"Attribute"]("readOnly","readonly")},detach:function(){a.baseProto.detach.call(this);this.clearCustomData();this.remove()}}})}(),CKEDITOR.plugins.sourcearea={commands:{source:{modes:{wysiwyg:1,
source:1},editorFocus:!1,readOnly:1,exec:function(a){a.mode=="wysiwyg"&&a.fire("saveSnapshot");a.getCommand("source").setState(CKEDITOR.TRISTATE_DISABLED);a.setMode(a.mode=="source"?"wysiwyg":"source")},canUndo:!1}}},CKEDITOR.plugins.add("specialchar",{availableLangs:{af:1,ar:1,bg:1,ca:1,cs:1,cy:1,da:1,de:1,el:1,en:1,"en-gb":1,eo:1,es:1,et:1,fa:1,fi:1,fr:1,"fr-ca":1,gl:1,he:1,hr:1,hu:1,id:1,it:1,ja:1,km:1,ku:1,lt:1,lv:1,nb:1,nl:1,no:1,pl:1,pt:1,"pt-br":1,ru:1,si:1,sk:1,sl:1,sq:1,sv:1,th:1,tr:1,tt:1,
ug:1,uk:1,vi:1,zh:1,"zh-cn":1},requires:"dialog",init:function(a){var e=this;CKEDITOR.dialog.add("specialchar",this.path+"dialogs/specialchar.js");a.addCommand("specialchar",{exec:function(){var b=a.langCode,b=e.availableLangs[b]?b:e.availableLangs[b.replace(/-.*/,"")]?b.replace(/-.*/,""):"en";CKEDITOR.scriptLoader.load(CKEDITOR.getUrl(e.path+"dialogs/lang/"+b+".js"),function(){CKEDITOR.tools.extend(a.lang.specialchar,e.langEntries[b]);a.openDialog("specialchar")})},modes:{wysiwyg:1},canUndo:false});
a.ui.addButton&&a.ui.addButton("SpecialChar",{label:a.lang.specialchar.toolbar,command:"specialchar",toolbar:"insert,50"})}}),CKEDITOR.config.specialChars="! &quot; # $ % &amp; ' ( ) * + - . / 0 1 2 3 4 5 6 7 8 9 : ; &lt; = &gt; ? @ A B C D E F G H I J K L M N O P Q R S T U V W X Y Z [ ] ^ _ ` a b c d e f g h i j k l m n o p q r s t u v w x y z { | } ~ &euro; &lsquo; &rsquo; &ldquo; &rdquo; &ndash; &mdash; &iexcl; &cent; &pound; &curren; &yen; &brvbar; &sect; &uml; &copy; &ordf; &laquo; &not; &reg; &macr; &deg; &sup2; &sup3; &acute; &micro; &para; &middot; &cedil; &sup1; &ordm; &raquo; &frac14; &frac12; &frac34; &iquest; &Agrave; &Aacute; &Acirc; &Atilde; &Auml; &Aring; &AElig; &Ccedil; &Egrave; &Eacute; &Ecirc; &Euml; &Igrave; &Iacute; &Icirc; &Iuml; &ETH; &Ntilde; &Ograve; &Oacute; &Ocirc; &Otilde; &Ouml; &times; &Oslash; &Ugrave; &Uacute; &Ucirc; &Uuml; &Yacute; &THORN; &szlig; &agrave; &aacute; &acirc; &atilde; &auml; &aring; &aelig; &ccedil; &egrave; &eacute; &ecirc; &euml; &igrave; &iacute; &icirc; &iuml; &eth; &ntilde; &ograve; &oacute; &ocirc; &otilde; &ouml; &divide; &oslash; &ugrave; &uacute; &ucirc; &uuml; &yacute; &thorn; &yuml; &OElig; &oelig; &#372; &#374 &#373 &#375; &sbquo; &#8219; &bdquo; &hellip; &trade; &#9658; &bull; &rarr; &rArr; &hArr; &diams; &asymp;".split(" "),
function(){CKEDITOR.plugins.add("stylescombo",{requires:"richcombo",init:function(a){var e=a.config,b=a.lang.stylescombo,g={},c=[],d=[];a.on("stylesSet",function(b){if(b=b.data.styles){for(var f,h,i,k=0,n=b.length;k<n;k++){f=b[k];if(!(a.blockless&&f.element in CKEDITOR.dtd.$block)){h=f.name;f=new CKEDITOR.style(f);if(!a.filter.customConfig||a.filter.check(f)){f._name=h;f._.enterMode=e.enterMode;f._.type=i=f.assignedTo||f.type;f._.weight=k+(i==CKEDITOR.STYLE_OBJECT?1:i==CKEDITOR.STYLE_BLOCK?2:3)*1E3;
g[h]=f;c.push(f);d.push(f)}}}c.sort(function(a,b){return a._.weight-b._.weight})}});a.ui.addRichCombo("Styles",{label:b.label,title:b.panelTitle,toolbar:"styles,10",allowedContent:d,panel:{css:[CKEDITOR.skin.getPath("editor")].concat(e.contentsCss),multiSelect:true,attributes:{"aria-label":b.panelTitle}},init:function(){var a,d,e,g,k,n;k=0;for(n=c.length;k<n;k++){a=c[k];d=a._name;g=a._.type;if(g!=e){this.startGroup(b["panelTitle"+g]);e=g}this.add(d,a.type==CKEDITOR.STYLE_OBJECT?d:a.buildPreview(),
d)}this.commit()},onClick:function(b){a.focus();a.fire("saveSnapshot");var b=g[b],c=a.elementPath();a[b.checkActive(c,a)?"removeStyle":"applyStyle"](b);a.fire("saveSnapshot")},onRender:function(){a.on("selectionChange",function(b){for(var c=this.getValue(),b=b.data.path.elements,d=0,e=b.length,k;d<e;d++){k=b[d];for(var n in g)if(g[n].checkElementRemovable(k,true,a)){n!=c&&this.setValue(n);return}}this.setValue("")},this)},onOpen:function(){var c=a.getSelection().getSelectedElement(),c=a.elementPath(c),
d=[0,0,0,0];this.showAll();this.unmarkAll();for(var e in g){var i=g[e],k=i._.type;i.checkApplicable(c,a,a.activeFilter)?d[k]++:this.hideItem(e);i.checkActive(c,a)&&this.mark(e)}d[CKEDITOR.STYLE_BLOCK]||this.hideGroup(b["panelTitle"+CKEDITOR.STYLE_BLOCK]);d[CKEDITOR.STYLE_INLINE]||this.hideGroup(b["panelTitle"+CKEDITOR.STYLE_INLINE]);d[CKEDITOR.STYLE_OBJECT]||this.hideGroup(b["panelTitle"+CKEDITOR.STYLE_OBJECT])},refresh:function(){var b=a.elementPath();if(b){for(var c in g)if(g[c].checkApplicable(b,
a,a.activeFilter))return;this.setState(CKEDITOR.TRISTATE_DISABLED)}},reset:function(){g={};c=[]}})}})}(),function(){function a(a){return{editorFocus:false,canUndo:false,modes:{wysiwyg:1},exec:function(b){if(b.editable().hasFocus){var e=b.getSelection(),f;if(f=(new CKEDITOR.dom.elementPath(e.getCommonAncestor(),e.root)).contains({td:1,th:1},1)){var e=b.createRange(),g=CKEDITOR.tools.tryThese(function(){var b=f.getParent().$.cells[f.$.cellIndex+(a?-1:1)];b.parentNode.parentNode;return b},function(){var b=
f.getParent(),b=b.getAscendant("table").$.rows[b.$.rowIndex+(a?-1:1)];return b.cells[a?b.cells.length-1:0]});if(!g&&!a){for(var i=f.getAscendant("table").$,g=f.getParent().$.cells,i=new CKEDITOR.dom.element(i.insertRow(-1),b.document),k=0,n=g.length;k<n;k++)i.append((new CKEDITOR.dom.element(g[k],b.document)).clone(false,false)).appendBogus();e.moveToElementEditStart(i)}else if(g){g=new CKEDITOR.dom.element(g);e.moveToElementEditStart(g);(!e.checkStartOfBlock()||!e.checkEndOfBlock())&&e.selectNodeContents(g)}else return true;
e.select(true);return true}}return false}}}var e={editorFocus:false,modes:{wysiwyg:1,source:1}},b={exec:function(a){a.container.focusNext(true,a.tabIndex)}},g={exec:function(a){a.container.focusPrevious(true,a.tabIndex)}};CKEDITOR.plugins.add("tab",{init:function(c){for(var d=c.config.enableTabKeyTools!==false,j=c.config.tabSpaces||0,f="";j--;)f=f+" ";if(f)c.on("key",function(a){if(a.data.keyCode==9){c.insertText(f);a.cancel()}});if(d)c.on("key",function(a){(a.data.keyCode==9&&c.execCommand("selectNextCell")||
a.data.keyCode==CKEDITOR.SHIFT+9&&c.execCommand("selectPreviousCell"))&&a.cancel()});c.addCommand("blur",CKEDITOR.tools.extend(b,e));c.addCommand("blurBack",CKEDITOR.tools.extend(g,e));c.addCommand("selectNextCell",a());c.addCommand("selectPreviousCell",a(true))}})}(),CKEDITOR.dom.element.prototype.focusNext=function(a,e){var b=e===void 0?this.getTabIndex():e,g,c,d,j,f,h;if(b<=0)for(f=this.getNextSourceNode(a,CKEDITOR.NODE_ELEMENT);f;){if(f.isVisible()&&f.getTabIndex()===0){d=f;break}f=f.getNextSourceNode(false,
CKEDITOR.NODE_ELEMENT)}else for(f=this.getDocument().getBody().getFirst();f=f.getNextSourceNode(false,CKEDITOR.NODE_ELEMENT);){if(!g)if(!c&&f.equals(this)){c=true;if(a){if(!(f=f.getNextSourceNode(true,CKEDITOR.NODE_ELEMENT)))break;g=1}}else c&&!this.contains(f)&&(g=1);if(f.isVisible()&&!((h=f.getTabIndex())<0)){if(g&&h==b){d=f;break}if(h>b&&(!d||!j||h<j)){d=f;j=h}else if(!d&&h===0){d=f;j=h}}}d&&d.focus()},CKEDITOR.dom.element.prototype.focusPrevious=function(a,e){for(var b=e===void 0?this.getTabIndex():
e,g,c,d,j=0,f,h=this.getDocument().getBody().getLast();h=h.getPreviousSourceNode(false,CKEDITOR.NODE_ELEMENT);){if(!g)if(!c&&h.equals(this)){c=true;if(a){if(!(h=h.getPreviousSourceNode(true,CKEDITOR.NODE_ELEMENT)))break;g=1}}else c&&!this.contains(h)&&(g=1);if(h.isVisible()&&!((f=h.getTabIndex())<0))if(b<=0){if(g&&f===0){d=h;break}if(f>j){d=h;j=f}}else{if(g&&f==b){d=h;break}if(f<b&&(!d||f>j)){d=h;j=f}}}d&&d.focus()},CKEDITOR.plugins.add("table",{requires:"dialog",init:function(a){function e(a){return CKEDITOR.tools.extend(a||
{},{contextSensitive:1,refresh:function(a,b){this.setState(b.contains("table",1)?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED)}})}if(!a.blockless){var b=a.lang.table;a.addCommand("table",new CKEDITOR.dialogCommand("table",{context:"table",allowedContent:"table{width,height}[align,border,cellpadding,cellspacing,summary];caption tbody thead tfoot;th td tr[scope];"+(a.plugins.dialogadvtab?"table"+a.plugins.dialogadvtab.allowedContent():""),requiredContent:"table",contentTransformations:[["table{width}: sizeToStyle",
"table[width]: sizeToAttribute"]]}));a.addCommand("tableProperties",new CKEDITOR.dialogCommand("tableProperties",e()));a.addCommand("tableDelete",e({exec:function(a){var b=a.elementPath().contains("table",1);if(b){var d=b.getParent(),e=a.editable();d.getChildCount()==1&&(!d.is("td","th")&&!d.equals(e))&&(b=d);a=a.createRange();a.moveToPosition(b,CKEDITOR.POSITION_BEFORE_START);b.remove();a.select()}}}));a.ui.addButton&&a.ui.addButton("Table",{label:b.toolbar,command:"table",toolbar:"insert,30"});
CKEDITOR.dialog.add("table",this.path+"dialogs/table.js");CKEDITOR.dialog.add("tableProperties",this.path+"dialogs/table.js");a.addMenuItems&&a.addMenuItems({table:{label:b.menu,command:"tableProperties",group:"table",order:5},tabledelete:{label:b.deleteTable,command:"tableDelete",group:"table",order:1}});a.on("doubleclick",function(a){if(a.data.element.is("table"))a.data.dialog="tableProperties"});a.contextMenu&&a.contextMenu.addListener(function(){return{tabledelete:CKEDITOR.TRISTATE_OFF,table:CKEDITOR.TRISTATE_OFF}})}}}),
function(){function a(a){function b(a){if(!(c.length>0)&&a.type==CKEDITOR.NODE_ELEMENT&&o.test(a.getName())&&!a.getCustomData("selected_cell")){CKEDITOR.dom.element.setMarker(d,a,"selected_cell",true);c.push(a)}}for(var a=a.getRanges(),c=[],d={},f=0;f<a.length;f++){var e=a[f];if(e.collapsed){e=e.getCommonAncestor();(e=e.getAscendant("td",true)||e.getAscendant("th",true))&&c.push(e)}else{var e=new CKEDITOR.dom.walker(e),g;for(e.guard=b;g=e.next();)if(g.type!=CKEDITOR.NODE_ELEMENT||!g.is(CKEDITOR.dtd.table))if((g=
g.getAscendant("td",true)||g.getAscendant("th",true))&&!g.getCustomData("selected_cell")){CKEDITOR.dom.element.setMarker(d,g,"selected_cell",true);c.push(g)}}}CKEDITOR.dom.element.clearAllMarkers(d);return c}function e(b,c){for(var d=a(b),f=d[0],e=f.getAscendant("table"),f=f.getDocument(),g=d[0].getParent(),h=g.$.rowIndex,d=d[d.length-1],i=d.getParent().$.rowIndex+d.$.rowSpan-1,d=new CKEDITOR.dom.element(e.$.rows[i]),h=c?h:i,g=c?g:d,d=CKEDITOR.tools.buildTableMap(e),e=d[h],h=c?d[h-1]:d[h+1],d=d[0].length,
f=f.createElement("tr"),i=0;e[i]&&i<d;i++){var j;if(e[i].rowSpan>1&&h&&e[i]==h[i]){j=e[i];j.rowSpan=j.rowSpan+1}else{j=(new CKEDITOR.dom.element(e[i])).clone();j.removeAttribute("rowSpan");j.appendBogus();f.append(j);j=j.$}i=i+(j.colSpan-1)}c?f.insertBefore(g):f.insertAfter(g)}function b(c){if(c instanceof CKEDITOR.dom.selection){for(var d=a(c),f=d[0].getAscendant("table"),e=CKEDITOR.tools.buildTableMap(f),c=d[0].getParent().$.rowIndex,d=d[d.length-1],g=d.getParent().$.rowIndex+d.$.rowSpan-1,d=[],
h=c;h<=g;h++){for(var i=e[h],j=new CKEDITOR.dom.element(f.$.rows[h]),k=0;k<i.length;k++){var n=new CKEDITOR.dom.element(i[k]),o=n.getParent().$.rowIndex;if(n.$.rowSpan==1)n.remove();else{n.$.rowSpan=n.$.rowSpan-1;if(o==h){o=e[h+1];o[k-1]?n.insertAfter(new CKEDITOR.dom.element(o[k-1])):(new CKEDITOR.dom.element(f.$.rows[h+1])).append(n,1)}}k=k+(n.$.colSpan-1)}d.push(j)}e=f.$.rows;f=new CKEDITOR.dom.element(e[g+1]||(c>0?e[c-1]:null)||f.$.parentNode);for(h=d.length;h>=0;h--)b(d[h]);return f}if(c instanceof
CKEDITOR.dom.element){f=c.getAscendant("table");f.$.rows.length==1?f.remove():c.remove()}return null}function g(a,b){for(var c=b?Infinity:0,d=0;d<a.length;d++){var f;f=a[d];for(var e=b,g=f.getParent().$.cells,h=0,i=0;i<g.length;i++){var j=g[i],h=h+(e?1:j.colSpan);if(j==f.$)break}f=h-1;if(b?f<c:f>c)c=f}return c}function c(b,c){for(var d=a(b),f=d[0].getAscendant("table"),e=g(d,1),d=g(d),e=c?e:d,h=CKEDITOR.tools.buildTableMap(f),f=[],d=[],i=h.length,j=0;j<i;j++){f.push(h[j][e]);d.push(c?h[j][e-1]:h[j][e+
1])}for(j=0;j<i;j++)if(f[j]){if(f[j].colSpan>1&&d[j]==f[j]){e=f[j];e.colSpan=e.colSpan+1}else{e=(new CKEDITOR.dom.element(f[j])).clone();e.removeAttribute("colSpan");e.appendBogus();e[c?"insertBefore":"insertAfter"].call(e,new CKEDITOR.dom.element(f[j]));e=e.$}j=j+(e.rowSpan-1)}}function d(a,b){var c=a.getStartElement();if(c=c.getAscendant("td",1)||c.getAscendant("th",1)){var d=c.clone();d.appendBogus();b?d.insertBefore(c):d.insertAfter(c)}}function j(b){if(b instanceof CKEDITOR.dom.selection){var b=
a(b),c=b[0]&&b[0].getAscendant("table"),d;a:{var e=0;d=b.length-1;for(var g={},h,i;h=b[e++];)CKEDITOR.dom.element.setMarker(g,h,"delete_cell",true);for(e=0;h=b[e++];)if((i=h.getPrevious())&&!i.getCustomData("delete_cell")||(i=h.getNext())&&!i.getCustomData("delete_cell")){CKEDITOR.dom.element.clearAllMarkers(g);d=i;break a}CKEDITOR.dom.element.clearAllMarkers(g);i=b[0].getParent();if(i=i.getPrevious())d=i.getLast();else{i=b[d].getParent();d=(i=i.getNext())?i.getChild(0):null}}for(i=b.length-1;i>=
0;i--)j(b[i]);d?f(d,true):c&&c.remove()}else if(b instanceof CKEDITOR.dom.element){c=b.getParent();c.getChildCount()==1?c.remove():b.remove()}}function f(a,b){var c=a.getDocument(),d=CKEDITOR.document;if(CKEDITOR.env.ie&&CKEDITOR.env.version==10){d.focus();c.focus()}c=new CKEDITOR.dom.range(c);if(!c["moveToElementEdit"+(b?"End":"Start")](a)){c.selectNodeContents(a);c.collapse(b?false:true)}c.select(true)}function h(a,b,c){a=a[b];if(typeof c=="undefined")return a;for(b=0;a&&b<a.length;b++){if(c.is&&
a[b]==c.$)return b;if(b==c)return new CKEDITOR.dom.element(a[b])}return c.is?-1:null}function i(b,c,d){var f=a(b),e;if((c?f.length!=1:f.length<2)||(e=b.getCommonAncestor())&&e.type==CKEDITOR.NODE_ELEMENT&&e.is("table"))return false;var g,b=f[0];e=b.getAscendant("table");var i=CKEDITOR.tools.buildTableMap(e),j=i.length,k=i[0].length,n=b.getParent().$.rowIndex,o=h(i,n,b);if(c){var y;try{var z=parseInt(b.getAttribute("rowspan"),10)||1;g=parseInt(b.getAttribute("colspan"),10)||1;y=i[c=="up"?n-z:c=="down"?
n+z:n][c=="left"?o-g:c=="right"?o+g:o]}catch(B){return false}if(!y||b.$==y)return false;f[c=="up"||c=="left"?"unshift":"push"](new CKEDITOR.dom.element(y))}for(var c=b.getDocument(),A=n,z=y=0,G=!d&&new CKEDITOR.dom.documentFragment(c),D=0,c=0;c<f.length;c++){g=f[c];var J=g.getParent(),C=g.getFirst(),F=g.$.colSpan,E=g.$.rowSpan,J=J.$.rowIndex,I=h(i,J,g),D=D+F*E,z=Math.max(z,I-o+F);y=Math.max(y,J-n+E);if(!d){F=g;(E=F.getBogus())&&E.remove();F.trim();if(g.getChildren().count()){if(J!=A&&C&&(!C.isBlockBoundary||
!C.isBlockBoundary({br:1})))(A=G.getLast(CKEDITOR.dom.walker.whitespaces(true)))&&(!A.is||!A.is("br"))&&G.append("br");g.moveChildren(G)}c?g.remove():g.setHtml("")}A=J}if(d)return y*z==D;G.moveChildren(b);b.appendBogus();z>=k?b.removeAttribute("rowSpan"):b.$.rowSpan=y;y>=j?b.removeAttribute("colSpan"):b.$.colSpan=z;d=new CKEDITOR.dom.nodeList(e.$.rows);f=d.count();for(c=f-1;c>=0;c--){e=d.getItem(c);if(!e.$.cells.length){e.remove();f++}}return b}function k(b,c){var d=a(b);if(d.length>1)return false;
if(c)return true;var d=d[0],f=d.getParent(),e=f.getAscendant("table"),g=CKEDITOR.tools.buildTableMap(e),i=f.$.rowIndex,j=h(g,i,d),k=d.$.rowSpan,n;if(k>1){n=Math.ceil(k/2);for(var k=Math.floor(k/2),f=i+n,e=new CKEDITOR.dom.element(e.$.rows[f]),g=h(g,f),o,f=d.clone(),i=0;i<g.length;i++){o=g[i];if(o.parentNode==e.$&&i>j){f.insertBefore(new CKEDITOR.dom.element(o));break}else o=null}o||e.append(f)}else{k=n=1;e=f.clone();e.insertAfter(f);e.append(f=d.clone());o=h(g,i);for(j=0;j<o.length;j++)o[j].rowSpan++}f.appendBogus();
d.$.rowSpan=n;f.$.rowSpan=k;n==1&&d.removeAttribute("rowSpan");k==1&&f.removeAttribute("rowSpan");return f}function n(b,c){var d=a(b);if(d.length>1)return false;if(c)return true;var d=d[0],f=d.getParent(),e=f.getAscendant("table"),e=CKEDITOR.tools.buildTableMap(e),g=h(e,f.$.rowIndex,d),i=d.$.colSpan;if(i>1){f=Math.ceil(i/2);i=Math.floor(i/2)}else{for(var i=f=1,j=[],k=0;k<e.length;k++){var n=e[k];j.push(n[g]);n[g].rowSpan>1&&(k=k+(n[g].rowSpan-1))}for(e=0;e<j.length;e++)j[e].colSpan++}e=d.clone();
e.insertAfter(d);e.appendBogus();d.$.colSpan=f;e.$.colSpan=i;f==1&&d.removeAttribute("colSpan");i==1&&e.removeAttribute("colSpan");return e}var o=/^(?:td|th)$/;CKEDITOR.plugins.tabletools={requires:"table,dialog,contextmenu",init:function(g){function h(a){return CKEDITOR.tools.extend(a||{},{contextSensitive:1,refresh:function(a,b){this.setState(b.contains({td:1,th:1},1)?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED)}})}function l(a,b){var c=g.addCommand(a,b);g.addFeature(c)}var o=g.lang.table;
l("cellProperties",new CKEDITOR.dialogCommand("cellProperties",h({allowedContent:"td th{width,height,border-color,background-color,white-space,vertical-align,text-align}[colspan,rowspan]",requiredContent:"table"})));CKEDITOR.dialog.add("cellProperties",this.path+"dialogs/tableCell.js");l("rowDelete",h({requiredContent:"table",exec:function(a){a=a.getSelection();f(b(a))}}));l("rowInsertBefore",h({requiredContent:"table",exec:function(a){a=a.getSelection();e(a,true)}}));l("rowInsertAfter",h({requiredContent:"table",
exec:function(a){a=a.getSelection();e(a)}}));l("columnDelete",h({requiredContent:"table",exec:function(b){for(var b=b.getSelection(),b=a(b),c=b[0],d=b[b.length-1],b=c.getAscendant("table"),e=CKEDITOR.tools.buildTableMap(b),g,h,i=[],j=0,k=e.length;j<k;j++)for(var m=0,l=e[j].length;m<l;m++){e[j][m]==c.$&&(g=m);e[j][m]==d.$&&(h=m)}for(j=g;j<=h;j++)for(m=0;m<e.length;m++){d=e[m];c=new CKEDITOR.dom.element(b.$.rows[m]);d=new CKEDITOR.dom.element(d[j]);if(d.$){d.$.colSpan==1?d.remove():d.$.colSpan=d.$.colSpan-
1;m=m+(d.$.rowSpan-1);c.$.cells.length||i.push(c)}}h=b.$.rows[0]&&b.$.rows[0].cells;g=new CKEDITOR.dom.element(h[g]||(g?h[g-1]:b.$.parentNode));i.length==k&&b.remove();g&&f(g,true)}}));l("columnInsertBefore",h({requiredContent:"table",exec:function(a){a=a.getSelection();c(a,true)}}));l("columnInsertAfter",h({requiredContent:"table",exec:function(a){a=a.getSelection();c(a)}}));l("cellDelete",h({requiredContent:"table",exec:function(a){a=a.getSelection();j(a)}}));l("cellMerge",h({allowedContent:"td[colspan,rowspan]",
requiredContent:"td[colspan,rowspan]",exec:function(a){f(i(a.getSelection()),true)}}));l("cellMergeRight",h({allowedContent:"td[colspan]",requiredContent:"td[colspan]",exec:function(a){f(i(a.getSelection(),"right"),true)}}));l("cellMergeDown",h({allowedContent:"td[rowspan]",requiredContent:"td[rowspan]",exec:function(a){f(i(a.getSelection(),"down"),true)}}));l("cellVerticalSplit",h({allowedContent:"td[rowspan]",requiredContent:"td[rowspan]",exec:function(a){f(k(a.getSelection()))}}));l("cellHorizontalSplit",
h({allowedContent:"td[colspan]",requiredContent:"td[colspan]",exec:function(a){f(n(a.getSelection()))}}));l("cellInsertBefore",h({requiredContent:"table",exec:function(a){a=a.getSelection();d(a,true)}}));l("cellInsertAfter",h({requiredContent:"table",exec:function(a){a=a.getSelection();d(a)}}));g.addMenuItems&&g.addMenuItems({tablecell:{label:o.cell.menu,group:"tablecell",order:1,getItems:function(){var b=g.getSelection(),c=a(b);return{tablecell_insertBefore:CKEDITOR.TRISTATE_OFF,tablecell_insertAfter:CKEDITOR.TRISTATE_OFF,
tablecell_delete:CKEDITOR.TRISTATE_OFF,tablecell_merge:i(b,null,true)?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED,tablecell_merge_right:i(b,"right",true)?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED,tablecell_merge_down:i(b,"down",true)?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED,tablecell_split_vertical:k(b,true)?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED,tablecell_split_horizontal:n(b,true)?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED,tablecell_properties:c.length>0?CKEDITOR.TRISTATE_OFF:
CKEDITOR.TRISTATE_DISABLED}}},tablecell_insertBefore:{label:o.cell.insertBefore,group:"tablecell",command:"cellInsertBefore",order:5},tablecell_insertAfter:{label:o.cell.insertAfter,group:"tablecell",command:"cellInsertAfter",order:10},tablecell_delete:{label:o.cell.deleteCell,group:"tablecell",command:"cellDelete",order:15},tablecell_merge:{label:o.cell.merge,group:"tablecell",command:"cellMerge",order:16},tablecell_merge_right:{label:o.cell.mergeRight,group:"tablecell",command:"cellMergeRight",
order:17},tablecell_merge_down:{label:o.cell.mergeDown,group:"tablecell",command:"cellMergeDown",order:18},tablecell_split_horizontal:{label:o.cell.splitHorizontal,group:"tablecell",command:"cellHorizontalSplit",order:19},tablecell_split_vertical:{label:o.cell.splitVertical,group:"tablecell",command:"cellVerticalSplit",order:20},tablecell_properties:{label:o.cell.title,group:"tablecellproperties",command:"cellProperties",order:21},tablerow:{label:o.row.menu,group:"tablerow",order:1,getItems:function(){return{tablerow_insertBefore:CKEDITOR.TRISTATE_OFF,
tablerow_insertAfter:CKEDITOR.TRISTATE_OFF,tablerow_delete:CKEDITOR.TRISTATE_OFF}}},tablerow_insertBefore:{label:o.row.insertBefore,group:"tablerow",command:"rowInsertBefore",order:5},tablerow_insertAfter:{label:o.row.insertAfter,group:"tablerow",command:"rowInsertAfter",order:10},tablerow_delete:{label:o.row.deleteRow,group:"tablerow",command:"rowDelete",order:15},tablecolumn:{label:o.column.menu,group:"tablecolumn",order:1,getItems:function(){return{tablecolumn_insertBefore:CKEDITOR.TRISTATE_OFF,
tablecolumn_insertAfter:CKEDITOR.TRISTATE_OFF,tablecolumn_delete:CKEDITOR.TRISTATE_OFF}}},tablecolumn_insertBefore:{label:o.column.insertBefore,group:"tablecolumn",command:"columnInsertBefore",order:5},tablecolumn_insertAfter:{label:o.column.insertAfter,group:"tablecolumn",command:"columnInsertAfter",order:10},tablecolumn_delete:{label:o.column.deleteColumn,group:"tablecolumn",command:"columnDelete",order:15}});g.contextMenu&&g.contextMenu.addListener(function(a,b,c){return(a=c.contains({td:1,th:1},
1))&&!a.isReadOnly()?{tablecell:CKEDITOR.TRISTATE_OFF,tablerow:CKEDITOR.TRISTATE_OFF,tablecolumn:CKEDITOR.TRISTATE_OFF}:null})},getSelectedCells:a};CKEDITOR.plugins.add("tabletools",CKEDITOR.plugins.tabletools)}(),CKEDITOR.tools.buildTableMap=function(a){for(var a=a.$.rows,e=-1,b=[],g=0;g<a.length;g++){e++;!b[e]&&(b[e]=[]);for(var c=-1,d=0;d<a[g].cells.length;d++){var j=a[g].cells[d];for(c++;b[e][c];)c++;for(var f=isNaN(j.colSpan)?1:j.colSpan,j=isNaN(j.rowSpan)?1:j.rowSpan,h=0;h<j;h++){b[e+h]||(b[e+
h]=[]);for(var i=0;i<f;i++)b[e+h][c+i]=a[g].cells[d]}c=c+(f-1)}}return b},function(){function a(a){function b(){for(var d=g(),h=CKEDITOR.tools.clone(a.config.toolbarGroups)||e(a),i=0;i<h.length;i++){var k=h[i];if(k!="/"){typeof k=="string"&&(k=h[i]={name:k});var l,p=k.groups;if(p)for(var t=0;t<p.length;t++){l=p[t];(l=d[l])&&f(k,l)}(l=d[k.name])&&f(k,l)}}return h}function g(){var b={},d,f,e;for(d in a.ui.items){f=a.ui.items[d];e=f.toolbar||"others";e=e.split(",");f=e[0];e=parseInt(e[1]||-1,10);b[f]||
(b[f]=[]);b[f].push({name:d,order:e})}for(f in b)b[f]=b[f].sort(function(a,b){return a.order==b.order?0:b.order<0?-1:a.order<0?1:a.order<b.order?-1:1});return b}function f(b,d){if(d.length){b.items?b.items.push(a.ui.create("-")):b.items=[];for(var f;f=d.shift();){f=typeof f=="string"?f:f.name;if(!i||CKEDITOR.tools.indexOf(i,f)==-1)(f=a.ui.create(f))&&a.addFeature(f)&&b.items.push(f)}}}function h(a){var b=[],c,d,e;for(c=0;c<a.length;++c){d=a[c];e={};if(d=="/")b.push(d);else if(CKEDITOR.tools.isArray(d)){f(e,
CKEDITOR.tools.clone(d));b.push(e)}else if(d.items){f(e,CKEDITOR.tools.clone(d.items));e.name=d.name;b.push(e)}}return b}var i=a.config.removeButtons,i=i&&i.split(","),k=a.config.toolbar;typeof k=="string"&&(k=a.config["toolbar_"+k]);return a.toolbar=k?h(k):b()}function e(a){return a._.toolbarGroups||(a._.toolbarGroups=[{name:"document",groups:["mode","document","doctools"]},{name:"clipboard",groups:["clipboard","undo"]},{name:"editing",groups:["find","selection","spellchecker"]},{name:"forms"},"/",
{name:"basicstyles",groups:["basicstyles","cleanup"]},{name:"paragraph",groups:["list","indent","blocks","align","bidi"]},{name:"links"},{name:"insert"},"/",{name:"styles"},{name:"colors"},{name:"tools"},{name:"others"},{name:"about"}])}var b=function(){this.toolbars=[];this.focusCommandExecuted=false};b.prototype.focus=function(){for(var a=0,b;b=this.toolbars[a++];)for(var e=0,f;f=b.items[e++];)if(f.focus){f.focus();return}};var g={modes:{wysiwyg:1,source:1},readOnly:1,exec:function(a){if(a.toolbox){a.toolbox.focusCommandExecuted=
true;CKEDITOR.env.ie||CKEDITOR.env.air?setTimeout(function(){a.toolbox.focus()},100):a.toolbox.focus()}}};CKEDITOR.plugins.add("toolbar",{requires:"button",init:function(c){var d,e=function(a,b){var g,k=c.lang.dir=="rtl",n=c.config.toolbarGroupCycling,o=k?37:39,k=k?39:37,n=n===void 0||n;switch(b){case 9:case CKEDITOR.SHIFT+9:for(;!g||!g.items.length;){g=b==9?(g?g.next:a.toolbar.next)||c.toolbox.toolbars[0]:(g?g.previous:a.toolbar.previous)||c.toolbox.toolbars[c.toolbox.toolbars.length-1];if(g.items.length)for(a=
g.items[d?g.items.length-1:0];a&&!a.focus;)(a=d?a.previous:a.next)||(g=0)}a&&a.focus();return false;case o:g=a;do{g=g.next;!g&&n&&(g=a.toolbar.items[0])}while(g&&!g.focus);g?g.focus():e(a,9);return false;case 40:if(a.button&&a.button.hasArrow){c.once("panelShow",function(a){a.data._.panel._.currentBlock.onKeyDown(40)});a.execute()}else e(a,b==40?o:k);return false;case k:case 38:g=a;do{g=g.previous;!g&&n&&(g=a.toolbar.items[a.toolbar.items.length-1])}while(g&&!g.focus);if(g)g.focus();else{d=1;e(a,
CKEDITOR.SHIFT+9);d=0}return false;case 27:c.focus();return false;case 13:case 32:a.execute();return false}return true};c.on("uiSpace",function(d){if(d.data.space==c.config.toolbarLocation){d.removeListener();c.toolbox=new b;var g=CKEDITOR.tools.getNextId(),i=['<span id="',g,'" class="cke_voice_label">',c.lang.toolbar.toolbars,"</span>",'<span id="'+c.ui.spaceId("toolbox")+'" class="cke_toolbox" role="group" aria-labelledby="',g,'" onmousedown="return false;">'],g=c.config.toolbarStartupExpanded!==
false,k,n;c.config.toolbarCanCollapse&&c.elementMode!=CKEDITOR.ELEMENT_MODE_INLINE&&i.push('<span class="cke_toolbox_main"'+(g?">":' style="display:none">'));for(var o=c.toolbox.toolbars,q=a(c),m=0;m<q.length;m++){var l,p=0,t,s=q[m],u;if(s){if(k){i.push("</span>");n=k=0}if(s==="/")i.push('<span class="cke_toolbar_break"></span>');else{u=s.items||s;for(var w=0;w<u.length;w++){var x=u[w],v;if(x)if(x.type==CKEDITOR.UI_SEPARATOR)n=k&&x;else{v=x.canGroup!==false;if(!p){l=CKEDITOR.tools.getNextId();p={id:l,
items:[]};t=s.name&&(c.lang.toolbar.toolbarGroups[s.name]||s.name);i.push('<span id="',l,'" class="cke_toolbar"',t?' aria-labelledby="'+l+'_label"':"",' role="toolbar">');t&&i.push('<span id="',l,'_label" class="cke_voice_label">',t,"</span>");i.push('<span class="cke_toolbar_start"></span>');var r=o.push(p)-1;if(r>0){p.previous=o[r-1];p.previous.next=p}}if(v){if(!k){i.push('<span class="cke_toolgroup" role="presentation">');k=1}}else if(k){i.push("</span>");k=0}l=function(a){a=a.render(c,i);r=p.items.push(a)-
1;if(r>0){a.previous=p.items[r-1];a.previous.next=a}a.toolbar=p;a.onkey=e;a.onfocus=function(){c.toolbox.focusCommandExecuted||c.focus()}};if(n){l(n);n=0}l(x)}}if(k){i.push("</span>");n=k=0}p&&i.push('<span class="cke_toolbar_end"></span></span>')}}}c.config.toolbarCanCollapse&&i.push("</span>");if(c.config.toolbarCanCollapse&&c.elementMode!=CKEDITOR.ELEMENT_MODE_INLINE){var y=CKEDITOR.tools.addFunction(function(){c.execCommand("toolbarCollapse")});c.on("destroy",function(){CKEDITOR.tools.removeFunction(y)});
c.addCommand("toolbarCollapse",{readOnly:1,exec:function(a){var b=a.ui.space("toolbar_collapser"),c=b.getPrevious(),d=a.ui.space("contents"),f=c.getParent(),e=parseInt(d.$.style.height,10),g=f.$.offsetHeight,h=b.hasClass("cke_toolbox_collapser_min");if(h){c.show();b.removeClass("cke_toolbox_collapser_min");b.setAttribute("title",a.lang.toolbar.toolbarCollapse)}else{c.hide();b.addClass("cke_toolbox_collapser_min");b.setAttribute("title",a.lang.toolbar.toolbarExpand)}b.getFirst().setText(h?"▲":"◀");
d.setStyle("height",e-(f.$.offsetHeight-g)+"px");a.fire("resize")},modes:{wysiwyg:1,source:1}});c.setKeystroke(CKEDITOR.ALT+(CKEDITOR.env.ie||CKEDITOR.env.webkit?189:109),"toolbarCollapse");i.push('<a title="'+(g?c.lang.toolbar.toolbarCollapse:c.lang.toolbar.toolbarExpand)+'" id="'+c.ui.spaceId("toolbar_collapser")+'" tabIndex="-1" class="cke_toolbox_collapser');g||i.push(" cke_toolbox_collapser_min");i.push('" onclick="CKEDITOR.tools.callFunction('+y+')">','<span class="cke_arrow">&#9650;</span>',
"</a>")}i.push("</span>");d.data.html=d.data.html+i.join("")}});c.on("destroy",function(){if(this.toolbox){var a,b=0,c,d,e;for(a=this.toolbox.toolbars;b<a.length;b++){d=a[b].items;for(c=0;c<d.length;c++){e=d[c];e.clickFn&&CKEDITOR.tools.removeFunction(e.clickFn);e.keyDownFn&&CKEDITOR.tools.removeFunction(e.keyDownFn)}}}});c.on("uiReady",function(){var a=c.ui.space("toolbox");a&&c.focusManager.add(a,1)});c.addCommand("toolbarFocus",g);c.setKeystroke(CKEDITOR.ALT+121,"toolbarFocus");c.ui.add("-",CKEDITOR.UI_SEPARATOR,
{});c.ui.addHandler(CKEDITOR.UI_SEPARATOR,{create:function(){return{render:function(a,b){b.push('<span class="cke_toolbar_separator" role="separator"></span>');return{}}}}})}});CKEDITOR.ui.prototype.addToolbarGroup=function(a,b,g){var f=e(this.editor),h=b===0,i={name:a};if(g){if(g=CKEDITOR.tools.search(f,function(a){return a.name==g})){!g.groups&&(g.groups=[]);if(b){b=CKEDITOR.tools.indexOf(g.groups,b);if(b>=0){g.groups.splice(b+1,0,a);return}}h?g.groups.splice(0,0,a):g.groups.push(a);return}b=null}b&&
(b=CKEDITOR.tools.indexOf(f,function(a){return a.name==b}));h?f.splice(0,0,a):typeof b=="number"?f.splice(b+1,0,i):f.push(a)}}(),CKEDITOR.UI_SEPARATOR="separator",CKEDITOR.config.toolbarLocation="top","use strict",function(){var a=[CKEDITOR.CTRL+90,CKEDITOR.CTRL+89,CKEDITOR.CTRL+CKEDITOR.SHIFT+90],e={8:1,46:1};CKEDITOR.plugins.add("undo",{init:function(c){function e(a){j.enabled&&a.data.command.canUndo!==false&&j.save()}function g(){j.enabled=c.readOnly?false:c.mode=="wysiwyg";j.onChange()}var j=
c.undoManager=new b(c),n=j.editingHandler=new d(j),o=c.addCommand("undo",{exec:function(){if(j.undo()){c.selectionChange();this.fire("afterUndo")}},startDisabled:true,canUndo:false}),q=c.addCommand("redo",{exec:function(){if(j.redo()){c.selectionChange();this.fire("afterRedo")}},startDisabled:true,canUndo:false});c.setKeystroke([[a[0],"undo"],[a[1],"redo"],[a[2],"redo"]]);j.onChange=function(){o.setState(j.undoable()?CKEDITOR.TRISTATE_OFF:CKEDITOR.TRISTATE_DISABLED);q.setState(j.redoable()?CKEDITOR.TRISTATE_OFF:
CKEDITOR.TRISTATE_DISABLED)};c.on("beforeCommandExec",e);c.on("afterCommandExec",e);c.on("saveSnapshot",function(a){j.save(a.data&&a.data.contentOnly)});c.on("contentDom",n.attachListeners,n);c.on("instanceReady",function(){c.fire("saveSnapshot")});c.on("beforeModeUnload",function(){c.mode=="wysiwyg"&&j.save(true)});c.on("mode",g);c.on("readOnly",g);if(c.ui.addButton){c.ui.addButton("Undo",{label:c.lang.undo.undo,command:"undo",toolbar:"undo,10"});c.ui.addButton("Redo",{label:c.lang.undo.redo,command:"redo",
toolbar:"undo,20"})}c.resetUndo=function(){j.reset();c.fire("saveSnapshot")};c.on("updateSnapshot",function(){j.currentImage&&j.update()});c.on("lockSnapshot",function(a){a=a.data;j.lock(a&&a.dontUpdate,a&&a.forceUpdate)});c.on("unlockSnapshot",j.unlock,j)}});CKEDITOR.plugins.undo={};var b=CKEDITOR.plugins.undo.UndoManager=function(a){this.strokesRecorded=[0,0];this.locked=null;this.previousKeyGroup=-1;this.limit=a.config.undoStackSize||20;this.strokesLimit=25;this.editor=a;this.reset()};b.prototype=
{type:function(a,c){var d=b.getKeyGroup(a),e=this.strokesRecorded[d]+1,c=c||e>=this.strokesLimit;if(!this.typing){this.hasUndo=this.typing=true;this.hasRedo=false;this.onChange()}if(c){e=0;this.editor.fire("saveSnapshot")}else this.editor.fire("change");this.strokesRecorded[d]=e;this.previousKeyGroup=d},keyGroupChanged:function(a){return b.getKeyGroup(a)!=this.previousKeyGroup},reset:function(){this.snapshots=[];this.index=-1;this.currentImage=null;this.hasRedo=this.hasUndo=false;this.locked=null;
this.resetType()},resetType:function(){this.strokesRecorded=[0,0];this.typing=false;this.previousKeyGroup=-1},refreshState:function(){this.hasUndo=!!this.getNextImage(true);this.hasRedo=!!this.getNextImage(false);this.resetType();this.onChange()},save:function(a,b,c){var d=this.editor;if(this.locked||d.status!="ready"||d.mode!="wysiwyg")return false;var e=d.editable();if(!e||e.status!="ready")return false;e=this.snapshots;b||(b=new g(d));if(b.contents===false)return false;if(this.currentImage)if(b.equalsContent(this.currentImage)){if(a||
b.equalsSelection(this.currentImage))return false}else c!==false&&d.fire("change");e.splice(this.index+1,e.length-this.index-1);e.length==this.limit&&e.shift();this.index=e.push(b)-1;this.currentImage=b;c!==false&&this.refreshState();return true},restoreImage:function(a){var b=this.editor,c;if(a.bookmarks){b.focus();c=b.getSelection()}this.locked={level:999};this.editor.loadSnapshot(a.contents);if(a.bookmarks)c.selectBookmarks(a.bookmarks);else if(CKEDITOR.env.ie){c=this.editor.document.getBody().$.createTextRange();
c.collapse(true);c.select()}this.locked=null;this.index=a.index;this.currentImage=this.snapshots[this.index];this.update();this.refreshState();b.fire("change")},getNextImage:function(a){var b=this.snapshots,c=this.currentImage,d;if(c)if(a)for(d=this.index-1;d>=0;d--){a=b[d];if(!c.equalsContent(a)){a.index=d;return a}}else for(d=this.index+1;d<b.length;d++){a=b[d];if(!c.equalsContent(a)){a.index=d;return a}}return null},redoable:function(){return this.enabled&&this.hasRedo},undoable:function(){return this.enabled&&
this.hasUndo},undo:function(){if(this.undoable()){this.save(true);var a=this.getNextImage(true);if(a)return this.restoreImage(a),true}return false},redo:function(){if(this.redoable()){this.save(true);if(this.redoable()){var a=this.getNextImage(false);if(a)return this.restoreImage(a),true}}return false},update:function(a){if(!this.locked){a||(a=new g(this.editor));for(var b=this.index,c=this.snapshots;b>0&&this.currentImage.equalsContent(c[b-1]);)b=b-1;c.splice(b,this.index-b+1,a);this.index=b;this.currentImage=
a}},updateSelection:function(a){if(!this.snapshots.length)return false;var b=this.snapshots,c=b[b.length-1];if(c.equalsContent(a)&&!c.equalsSelection(a)){this.currentImage=b[b.length-1]=a;return true}return false},lock:function(a,b){if(this.locked)this.locked.level++;else if(a)this.locked={level:1};else{var c=null;if(b)c=true;else{var d=new g(this.editor,true);this.currentImage&&this.currentImage.equalsContent(d)&&(c=d)}this.locked={update:c,level:1}}},unlock:function(){if(this.locked&&!--this.locked.level){var a=
this.locked.update;this.locked=null;if(a===true)this.update();else if(a){var b=new g(this.editor,true);a.equalsContent(b)||this.update()}}}};b.navigationKeyCodes={37:1,38:1,39:1,40:1,36:1,35:1,33:1,34:1};b.keyGroups={PRINTABLE:0,FUNCTIONAL:1};b.isNavigationKey=function(a){return!!b.navigationKeyCodes[a]};b.getKeyGroup=function(a){var c=b.keyGroups;return e[a]?c.FUNCTIONAL:c.PRINTABLE};b.getOppositeKeyGroup=function(a){var c=b.keyGroups;return a==c.FUNCTIONAL?c.PRINTABLE:c.FUNCTIONAL};b.ieFunctionalKeysBug=
function(a){return CKEDITOR.env.ie&&b.getKeyGroup(a)==b.keyGroups.FUNCTIONAL};var g=CKEDITOR.plugins.undo.Image=function(a,b){this.editor=a;a.fire("beforeUndoImage");var c=a.getSnapshot();CKEDITOR.env.ie&&c&&(c=c.replace(/\s+data-cke-expando=".*?"/g,""));this.contents=c;if(!b)this.bookmarks=(c=c&&a.getSelection())&&c.createBookmarks2(true);a.fire("afterUndoImage")},c=/\b(?:href|src|name)="[^"]*?"/gi;g.prototype={equalsContent:function(a){var b=this.contents,a=a.contents;if(CKEDITOR.env.ie&&(CKEDITOR.env.ie7Compat||
CKEDITOR.env.quirks)){b=b.replace(c,"");a=a.replace(c,"")}return b!=a?false:true},equalsSelection:function(a){var b=this.bookmarks,a=a.bookmarks;if(b||a){if(!b||!a||b.length!=a.length)return false;for(var c=0;c<b.length;c++){var d=b[c],e=a[c];if(d.startOffset!=e.startOffset||d.endOffset!=e.endOffset||!CKEDITOR.tools.arrayCompare(d.start,e.start)||!CKEDITOR.tools.arrayCompare(d.end,e.end))return false}}return true}};var d=CKEDITOR.plugins.undo.NativeEditingHandler=function(a){this.undoManager=a;this.ignoreInputEvent=
false;this.keyEventsStack=new j;this.lastKeydownImage=null};d.prototype={onKeydown:function(c){var d=c.data.getKey();if(d!==229)if(CKEDITOR.tools.indexOf(a,c.data.getKeystroke())>-1)c.data.preventDefault();else{this.keyEventsStack.cleanUp(c);c=this.undoManager;this.keyEventsStack.getLast(d)||this.keyEventsStack.push(d);this.lastKeydownImage=new g(c.editor);if(b.isNavigationKey(d)||this.undoManager.keyGroupChanged(d))if(c.strokesRecorded[0]||c.strokesRecorded[1]){c.save(false,this.lastKeydownImage,
false);c.resetType()}}},onInput:function(){if(this.ignoreInputEvent)this.ignoreInputEvent=false;else{var a=this.keyEventsStack.getLast();a||(a=this.keyEventsStack.push(0));this.keyEventsStack.increment(a.keyCode);if(this.keyEventsStack.getTotalInputs()>=this.undoManager.strokesLimit){this.undoManager.type(a.keyCode,true);this.keyEventsStack.resetInputs()}}},onKeyup:function(a){var c=this.undoManager,a=a.data.getKey(),d=this.keyEventsStack.getTotalInputs();this.keyEventsStack.remove(a);if(!b.ieFunctionalKeysBug(a)||
!this.lastKeydownImage||!this.lastKeydownImage.equalsContent(new g(c.editor,true)))if(d>0)c.type(a);else if(b.isNavigationKey(a))this.onNavigationKey(true)},onNavigationKey:function(a){var b=this.undoManager;(a||!b.save(true,null,false))&&b.updateSelection(new g(b.editor));b.resetType()},ignoreInputEventListener:function(){this.ignoreInputEvent=true},attachListeners:function(){var a=this.undoManager.editor,c=a.editable(),d=this;c.attachListener(c,"keydown",function(a){d.onKeydown(a);if(b.ieFunctionalKeysBug(a.data.getKey()))d.onInput()},
null,null,999);c.attachListener(c,CKEDITOR.env.ie?"keypress":"input",d.onInput,d,null,999);c.attachListener(c,"keyup",d.onKeyup,d,null,999);c.attachListener(c,"paste",d.ignoreInputEventListener,d,null,999);c.attachListener(c,"drop",d.ignoreInputEventListener,d,null,999);c.attachListener(c.isInline()?c:a.document.getDocumentElement(),"click",function(){d.onNavigationKey()},null,null,999);c.attachListener(this.undoManager.editor,"blur",function(){d.keyEventsStack.remove(9)},null,null,999)}};var j=CKEDITOR.plugins.undo.KeyEventsStack=
function(){this.stack=[]};j.prototype={push:function(a){return this.stack[this.stack.push({keyCode:a,inputs:0})-1]},getLastIndex:function(a){if(typeof a!="number")return this.stack.length-1;for(var b=this.stack.length;b--;)if(this.stack[b].keyCode==a)return b;return-1},getLast:function(a){a=this.getLastIndex(a);return a!=-1?this.stack[a]:null},increment:function(a){this.getLast(a).inputs++},remove:function(a){a=this.getLastIndex(a);a!=-1&&this.stack.splice(a,1)},resetInputs:function(a){if(typeof a==
"number")this.getLast(a).inputs=0;else for(a=this.stack.length;a--;)this.stack[a].inputs=0},getTotalInputs:function(){for(var a=this.stack.length,b=0;a--;)b=b+this.stack[a].inputs;return b},cleanUp:function(a){a=a.data.$;!a.ctrlKey&&!a.metaKey&&this.remove(17);a.shiftKey||this.remove(16);a.altKey||this.remove(18)}}}(),CKEDITOR.plugins.add("wsc",{requires:"dialog",parseApi:function(a){a.config.wsc_onFinish=typeof a.config.wsc_onFinish==="function"?a.config.wsc_onFinish:function(){};a.config.wsc_onClose=
typeof a.config.wsc_onClose==="function"?a.config.wsc_onClose:function(){}},parseConfig:function(a){a.config.wsc_customerId=a.config.wsc_customerId||CKEDITOR.config.wsc_customerId||"1:ua3xw1-2XyGJ3-GWruD3-6OFNT1-oXcuB1-nR6Bp4-hgQHc-EcYng3-sdRXG3-NOfFk";a.config.wsc_customDictionaryIds=a.config.wsc_customDictionaryIds||CKEDITOR.config.wsc_customDictionaryIds||"";a.config.wsc_userDictionaryName=a.config.wsc_userDictionaryName||CKEDITOR.config.wsc_userDictionaryName||"";a.config.wsc_customLoaderScript=
a.config.wsc_customLoaderScript||CKEDITOR.config.wsc_customLoaderScript;CKEDITOR.config.wsc_cmd=a.config.wsc_cmd||CKEDITOR.config.wsc_cmd||"spell";CKEDITOR.config.wsc_version="v4.3.0-26-g5bcf855";CKEDITOR.config.wsc_removeGlobalVariable=true},init:function(a){var e=CKEDITOR.env;this.parseConfig(a);this.parseApi(a);a.addCommand("checkspell",new CKEDITOR.dialogCommand("checkspell")).modes={wysiwyg:!CKEDITOR.env.opera&&!CKEDITOR.env.air&&document.domain==window.location.hostname&&!(e.ie&&(e.version<
8||e.quirks))};typeof a.plugins.scayt=="undefined"&&a.ui.addButton&&a.ui.addButton("SpellChecker",{label:a.lang.wsc.toolbar,click:function(a){var e=a.elementMode==CKEDITOR.ELEMENT_MODE_INLINE?a.container.getText():a.document.getBody().getText();(e=e.replace(/\s/g,""))?a.execCommand("checkspell"):alert("Nothing to check!")},toolbar:"spellchecker,10"});CKEDITOR.dialog.add("checkspell",this.path+(CKEDITOR.env.ie&&CKEDITOR.env.version<=7?"dialogs/wsc_ie.js":window.postMessage?"dialogs/wsc.js":"dialogs/wsc_ie.js"))}}),
function(){function a(a){var b=this.editor,g=a.document,f=g.body,h=g.getElementById("cke_actscrpt");h&&h.parentNode.removeChild(h);(h=g.getElementById("cke_shimscrpt"))&&h.parentNode.removeChild(h);(h=g.getElementById("cke_basetagscrpt"))&&h.parentNode.removeChild(h);f.contentEditable=true;if(CKEDITOR.env.ie){f.hideFocus=true;f.disabled=true;f.removeAttribute("disabled")}delete this._.isLoadingData;this.$=f;g=new CKEDITOR.dom.document(g);this.setup();this.fixInitialSelection();if(CKEDITOR.env.ie){g.getDocumentElement().addClass(g.$.compatMode);
b.config.enterMode!=CKEDITOR.ENTER_P&&this.attachListener(g,"selectionchange",function(){var a=g.getBody(),c=b.getSelection(),e=c&&c.getRanges()[0];e&&(a.getHtml().match(/^<p>(?:&nbsp;|<br>)<\/p>$/i)&&e.startContainer.equals(a))&&setTimeout(function(){e=b.getSelection().getRanges()[0];if(!e.startContainer.equals("body")){a.getFirst().remove(1);e.moveToElementEditEnd(a);e.select()}},0)})}if(CKEDITOR.env.webkit||CKEDITOR.env.ie&&CKEDITOR.env.version>10)g.getDocumentElement().on("mousedown",function(a){a.data.getTarget().is("html")&&
setTimeout(function(){b.editable().focus()})});e(b);try{b.document.$.execCommand("2D-position",false,true)}catch(i){}(CKEDITOR.env.gecko||CKEDITOR.env.ie&&b.document.$.compatMode=="CSS1Compat")&&this.attachListener(this,"keydown",function(a){var c=a.data.getKeystroke();if(c==33||c==34)if(CKEDITOR.env.ie)setTimeout(function(){b.getSelection().scrollIntoView()},0);else if(b.window.$.innerHeight>this.$.offsetHeight){var e=b.createRange();e[c==33?"moveToElementEditStart":"moveToElementEditEnd"](this);
e.select();a.data.preventDefault()}});CKEDITOR.env.ie&&this.attachListener(g,"blur",function(){try{g.$.selection.empty()}catch(a){}});CKEDITOR.env.iOS&&this.attachListener(g,"touchend",function(){a.focus()});f=b.document.getElementsByTag("title").getItem(0);f.data("cke-title",f.getText());if(CKEDITOR.env.ie)b.document.$.title=this._.docTitle;CKEDITOR.tools.setTimeout(function(){if(this.status=="unloaded")this.status="ready";b.fire("contentDom");if(this._.isPendingFocus){b.focus();this._.isPendingFocus=
false}setTimeout(function(){b.fire("dataReady")},0);CKEDITOR.env.ie&&setTimeout(function(){if(b.document){var a=b.document.$.body;a.runtimeStyle.marginBottom="0px";a.runtimeStyle.marginBottom=""}},1E3)},0,this)}function e(a){function b(){var d;a.editable().attachListener(a,"selectionChange",function(){var b=a.getSelection().getSelectedElement();if(b){if(d){d.detachEvent("onresizestart",e);d=null}b.$.attachEvent("onresizestart",e);d=b.$}})}function e(a){a.returnValue=false}if(CKEDITOR.env.gecko)try{var f=
a.document.$;f.execCommand("enableObjectResizing",false,!a.config.disableObjectResizing);f.execCommand("enableInlineTableEditing",false,!a.config.disableNativeTableHandles)}catch(g){}else CKEDITOR.env.ie&&(CKEDITOR.env.version<11&&a.config.disableObjectResizing)&&b(a)}function b(){var a=[];if(CKEDITOR.document.$.documentMode>=8){a.push("html.CSS1Compat [contenteditable=false]{min-height:0 !important}");var b=[],e;for(e in CKEDITOR.dtd.$removeEmpty)b.push("html.CSS1Compat "+e+"[contenteditable=false]");
a.push(b.join(",")+"{display:inline-block}")}else if(CKEDITOR.env.gecko){a.push("html{height:100% !important}");a.push("img:-moz-broken{-moz-force-broken-image-icon:1;min-width:24px;min-height:24px}")}a.push("html{cursor:text;*cursor:auto}");a.push("img,input,textarea{cursor:default}");return a.join("\n")}CKEDITOR.plugins.add("wysiwygarea",{init:function(a){a.config.fullPage&&a.addFeature({allowedContent:"html head title; style [media,type]; body (*)[id]; meta link [*]",requiredContent:"body"});a.addMode("wysiwyg",
function(b){function e(f){f&&f.removeListener();a.editable(new g(a,h.$.contentWindow.document.body));a.setData(a.getData(1),b)}var f="document.open();"+(CKEDITOR.env.ie?"("+CKEDITOR.tools.fixDomain+")();":"")+"document.close();",f=CKEDITOR.env.air?"javascript:void(0)":CKEDITOR.env.ie?"javascript:void(function(){"+encodeURIComponent(f)+"}())":"",h=CKEDITOR.dom.element.createFromHtml('<iframe src="'+f+'" frameBorder="0"></iframe>');h.setStyles({width:"100%",height:"100%"});h.addClass("cke_wysiwyg_frame cke_reset");
var i=a.ui.space("contents");i.append(h);if(f=CKEDITOR.env.ie||CKEDITOR.env.gecko)h.on("load",e);var k=a.title,n=a.fire("ariaEditorHelpLabel",{}).label;if(k){CKEDITOR.env.ie&&n&&(k=k+(", "+n));h.setAttribute("title",k)}if(n){var k=CKEDITOR.tools.getNextId(),o=CKEDITOR.dom.element.createFromHtml('<span id="'+k+'" class="cke_voice_label">'+n+"</span>");i.append(o,1);h.setAttribute("aria-describedby",k)}a.on("beforeModeUnload",function(a){a.removeListener();o&&o.remove()});h.setAttributes({tabIndex:a.tabIndex,
allowTransparency:"true"});!f&&e();if(CKEDITOR.env.webkit){f=function(){i.setStyle("width","100%");h.hide();h.setSize("width",i.getSize("width"));i.removeStyle("width");h.show()};h.setCustomData("onResize",f);CKEDITOR.document.getWindow().on("resize",f)}a.fire("ariaWidget",h)})}});CKEDITOR.editor.prototype.addContentsCss=function(a){var b=this.config,e=b.contentsCss;if(!CKEDITOR.tools.isArray(e))b.contentsCss=e?[e]:[];b.contentsCss.push(a)};var g=CKEDITOR.tools.createClass({$:function(){this.base.apply(this,
arguments);this._.frameLoadedHandler=CKEDITOR.tools.addFunction(function(b){CKEDITOR.tools.setTimeout(a,0,this,b)},this);this._.docTitle=this.getWindow().getFrame().getAttribute("title")},base:CKEDITOR.editable,proto:{setData:function(a,d){var e=this.editor;if(d){this.setHtml(a);this.fixInitialSelection();e.fire("dataReady")}else{this._.isLoadingData=true;e._.dataStore={id:1};var f=e.config,g=f.fullPage,i=f.docType,k=CKEDITOR.tools.buildStyleHtml(b()).replace(/<style>/,'<style data-cke-temp="1">');
g||(k=k+CKEDITOR.tools.buildStyleHtml(e.config.contentsCss));var n=f.baseHref?'<base href="'+f.baseHref+'" data-cke-temp="1" />':"";g&&(a=a.replace(/<!DOCTYPE[^>]*>/i,function(a){e.docType=i=a;return""}).replace(/<\?xml\s[^\?]*\?>/i,function(a){e.xmlDeclaration=a;return""}));a=e.dataProcessor.toHtml(a);if(g){/<body[\s|>]/.test(a)||(a="<body>"+a);/<html[\s|>]/.test(a)||(a="<html>"+a+"</html>");/<head[\s|>]/.test(a)?/<title[\s|>]/.test(a)||(a=a.replace(/<head[^>]*>/,"$&<title></title>")):a=a.replace(/<html[^>]*>/,
"$&<head><title></title></head>");n&&(a=a.replace(/<head[^>]*?>/,"$&"+n));a=a.replace(/<\/head\s*>/,k+"$&");a=i+a}else a=f.docType+'<html dir="'+f.contentsLangDirection+'" lang="'+(f.contentsLanguage||e.langCode)+'"><head><title>'+this._.docTitle+"</title>"+n+k+"</head><body"+(f.bodyId?' id="'+f.bodyId+'"':"")+(f.bodyClass?' class="'+f.bodyClass+'"':"")+">"+a+"</body></html>";if(CKEDITOR.env.gecko){a=a.replace(/<body/,'<body contenteditable="true" ');CKEDITOR.env.version<2E4&&(a=a.replace(/<body[^>]*>/,
"$&<\!-- cke-content-start --\>"))}f='<script id="cke_actscrpt" type="text/javascript"'+(CKEDITOR.env.ie?' defer="defer" ':"")+">var wasLoaded=0;function onload(){if(!wasLoaded)window.parent.CKEDITOR.tools.callFunction("+this._.frameLoadedHandler+",window);wasLoaded=1;}"+(CKEDITOR.env.ie?"onload();":'document.addEventListener("DOMContentLoaded", onload, false );')+"<\/script>";CKEDITOR.env.ie&&CKEDITOR.env.version<9&&(f=f+'<script id="cke_shimscrpt">window.parent.CKEDITOR.tools.enableHtml5Elements(document)<\/script>');
n&&(CKEDITOR.env.ie&&CKEDITOR.env.version<10)&&(f=f+'<script id="cke_basetagscrpt">var baseTag = document.querySelector( "base" );baseTag.href = baseTag.href;<\/script>');a=a.replace(/(?=\s*<\/(:?head)>)/,f);this.clearCustomData();this.clearListeners();e.fire("contentDomUnload");var o=this.getDocument();try{o.write(a)}catch(q){setTimeout(function(){o.write(a)},0)}}},getData:function(a){if(a)return this.getHtml();var a=this.editor,b=a.config,e=b.fullPage,f=e&&a.docType,g=e&&a.xmlDeclaration,i=this.getDocument(),
e=e?i.getDocumentElement().getOuterHtml():i.getBody().getHtml();CKEDITOR.env.gecko&&b.enterMode!=CKEDITOR.ENTER_BR&&(e=e.replace(/<br>(?=\s*(:?$|<\/body>))/,""));e=a.dataProcessor.toDataFormat(e);g&&(e=g+"\n"+e);f&&(e=f+"\n"+e);return e},focus:function(){this._.isLoadingData?this._.isPendingFocus=true:g.baseProto.focus.call(this)},detach:function(){var a=this.editor,b=a.document,a=a.window.getFrame();g.baseProto.detach.call(this);this.clearCustomData();b.getDocumentElement().clearCustomData();a.clearCustomData();
CKEDITOR.tools.removeFunction(this._.frameLoadedHandler);(b=a.removeCustomData("onResize"))&&b.removeListener();a.remove()}}})}(),CKEDITOR.config.disableObjectResizing=!1,CKEDITOR.config.disableNativeTableHandles=!0,CKEDITOR.config.disableNativeSpellChecker=!0,CKEDITOR.config.contentsCss=CKEDITOR.getUrl("contents.css"),CKEDITOR.config.plugins="dialogui,dialog,a11yhelp,about,basicstyles,blockquote,clipboard,panel,floatpanel,menu,contextmenu,elementspath,indent,indentlist,list,enterkey,entities,popup,filebrowser,floatingspace,listblock,button,richcombo,format,horizontalrule,htmlwriter,image,fakeobjects,link,magicline,maximize,pastefromword,pastetext,removeformat,resize,menubutton,scayt,showborders,sourcearea,specialchar,stylescombo,tab,table,tabletools,toolbar,undo,wsc,wysiwygarea",
CKEDITOR.config.skin="moono",function(){var a=function(a,b){for(var g=CKEDITOR.getUrl("plugins/"+b),a=a.split(","),c=0;c<a.length;c++)CKEDITOR.skin.icons[a[c]]={path:g,offset:-a[++c],bgsize:a[++c]}};CKEDITOR.env.hidpi?a("about,0,,bold,24,,italic,48,,strike,72,,subscript,96,,superscript,120,,underline,144,,bidiltr,168,,bidirtl,192,,blockquote,216,,copy-rtl,240,,copy,264,,cut-rtl,288,,cut,312,,paste-rtl,336,,paste,360,,codesnippet,384,,bgcolor,408,,textcolor,432,,creatediv,456,,docprops-rtl,480,,docprops,504,,find-rtl,528,,find,552,,replace,576,,flash,600,,button,624,,checkbox,648,,form,672,,hiddenfield,696,,imagebutton,720,,radio,744,,select-rtl,768,,select,792,,textarea-rtl,816,,textarea,840,,textfield-rtl,864,,textfield,888,,horizontalrule,912,,iframe,936,,image,960,,indent-rtl,984,,indent,1008,,outdent-rtl,1032,,outdent,1056,,justifyblock,1080,,justifycenter,1104,,justifyleft,1128,,justifyright,1152,,language,1176,,anchor-rtl,1200,,anchor,1224,,link,1248,,unlink,1272,,bulletedlist-rtl,1296,,bulletedlist,1320,,numberedlist-rtl,1344,,numberedlist,1368,,mathjax,1392,,maximize,1416,,newpage-rtl,1440,,newpage,1464,,pagebreak-rtl,1488,,pagebreak,1512,,pastefromword-rtl,1536,,pastefromword,1560,,pastetext-rtl,1584,,pastetext,1608,,placeholder,1632,,preview-rtl,1656,,preview,1680,,print,1704,,removeformat,1728,,save,1752,,scayt,1776,,selectall,1800,,showblocks-rtl,1824,,showblocks,1848,,smiley,1872,,source-rtl,1896,,source,1920,,sourcedialog-rtl,1944,,sourcedialog,1968,,specialchar,1992,,table,2016,,templates-rtl,2040,,templates,2064,,uicolor,2088,,redo-rtl,2112,,redo,2136,,undo-rtl,2160,,undo,2184,,simplebox,4416,auto,spellchecker,2232,",
"icons_hidpi.png"):a("about,0,auto,bold,24,auto,italic,48,auto,strike,72,auto,subscript,96,auto,superscript,120,auto,underline,144,auto,bidiltr,168,auto,bidirtl,192,auto,blockquote,216,auto,copy-rtl,240,auto,copy,264,auto,cut-rtl,288,auto,cut,312,auto,paste-rtl,336,auto,paste,360,auto,codesnippet,384,auto,bgcolor,408,auto,textcolor,432,auto,creatediv,456,auto,docprops-rtl,480,auto,docprops,504,auto,find-rtl,528,auto,find,552,auto,replace,576,auto,flash,600,auto,button,624,auto,checkbox,648,auto,form,672,auto,hiddenfield,696,auto,imagebutton,720,auto,radio,744,auto,select-rtl,768,auto,select,792,auto,textarea-rtl,816,auto,textarea,840,auto,textfield-rtl,864,auto,textfield,888,auto,horizontalrule,912,auto,iframe,936,auto,image,960,auto,indent-rtl,984,auto,indent,1008,auto,outdent-rtl,1032,auto,outdent,1056,auto,justifyblock,1080,auto,justifycenter,1104,auto,justifyleft,1128,auto,justifyright,1152,auto,language,1176,auto,anchor-rtl,1200,auto,anchor,1224,auto,link,1248,auto,unlink,1272,auto,bulletedlist-rtl,1296,auto,bulletedlist,1320,auto,numberedlist-rtl,1344,auto,numberedlist,1368,auto,mathjax,1392,auto,maximize,1416,auto,newpage-rtl,1440,auto,newpage,1464,auto,pagebreak-rtl,1488,auto,pagebreak,1512,auto,pastefromword-rtl,1536,auto,pastefromword,1560,auto,pastetext-rtl,1584,auto,pastetext,1608,auto,placeholder,1632,auto,preview-rtl,1656,auto,preview,1680,auto,print,1704,auto,removeformat,1728,auto,save,1752,auto,scayt,1776,auto,selectall,1800,auto,showblocks-rtl,1824,auto,showblocks,1848,auto,smiley,1872,auto,source-rtl,1896,auto,source,1920,auto,sourcedialog-rtl,1944,auto,sourcedialog,1968,auto,specialchar,1992,auto,table,2016,auto,templates-rtl,2040,auto,templates,2064,auto,uicolor,2088,auto,redo-rtl,2112,auto,redo,2136,auto,undo-rtl,2160,auto,undo,2184,auto,simplebox,2208,auto,spellchecker,2232,auto",
"icons.png")}()})();
define("ckeditor", function(){});

(function(window) {
    var re = {
        not_string: /[^s]/,
        number: /[diefg]/,
        json: /[j]/,
        not_json: /[^j]/,
        text: /^[^\x25]+/,
        modulo: /^\x25{2}/,
        placeholder: /^\x25(?:([1-9]\d*)\$|\(([^\)]+)\))?(\+)?(0|'[^$])?(-)?(\d+)?(?:\.(\d+))?([b-gijosuxX])/,
        key: /^([a-z_][a-z_\d]*)/i,
        key_access: /^\.([a-z_][a-z_\d]*)/i,
        index_access: /^\[(\d+)\]/,
        sign: /^[\+\-]/
    }

    function sprintf() {
        var key = arguments[0], cache = sprintf.cache
        if (!(cache[key] && cache.hasOwnProperty(key))) {
            cache[key] = sprintf.parse(key)
        }
        return sprintf.format.call(null, cache[key], arguments)
    }

    sprintf.format = function(parse_tree, argv) {
        var cursor = 1, tree_length = parse_tree.length, node_type = "", arg, output = [], i, k, match, pad, pad_character, pad_length, is_positive = true, sign = ""
        for (i = 0; i < tree_length; i++) {
            node_type = get_type(parse_tree[i])
            if (node_type === "string") {
                output[output.length] = parse_tree[i]
            }
            else if (node_type === "array") {
                match = parse_tree[i] // convenience purposes only
                if (match[2]) { // keyword argument
                    arg = argv[cursor]
                    for (k = 0; k < match[2].length; k++) {
                        if (!arg.hasOwnProperty(match[2][k])) {
                            throw new Error(sprintf("[sprintf] property '%s' does not exist", match[2][k]))
                        }
                        arg = arg[match[2][k]]
                    }
                }
                else if (match[1]) { // positional argument (explicit)
                    arg = argv[match[1]]
                }
                else { // positional argument (implicit)
                    arg = argv[cursor++]
                }

                if (get_type(arg) == "function") {
                    arg = arg()
                }

                if (re.not_string.test(match[8]) && re.not_json.test(match[8]) && (get_type(arg) != "number" && isNaN(arg))) {
                    throw new TypeError(sprintf("[sprintf] expecting number but found %s", get_type(arg)))
                }

                if (re.number.test(match[8])) {
                    is_positive = arg >= 0
                }

                switch (match[8]) {
                    case "b":
                        arg = arg.toString(2)
                    break
                    case "c":
                        arg = String.fromCharCode(arg)
                    break
                    case "d":
                    case "i":
                        arg = parseInt(arg, 10)
                    break
                    case "j":
                        arg = JSON.stringify(arg, null, match[6] ? parseInt(match[6]) : 0)
                    break
                    case "e":
                        arg = match[7] ? arg.toExponential(match[7]) : arg.toExponential()
                    break
                    case "f":
                        arg = match[7] ? parseFloat(arg).toFixed(match[7]) : parseFloat(arg)
                    break
                    case "g":
                        arg = match[7] ? parseFloat(arg).toPrecision(match[7]) : parseFloat(arg)
                    break
                    case "o":
                        arg = arg.toString(8)
                    break
                    case "s":
                        arg = ((arg = String(arg)) && match[7] ? arg.substring(0, match[7]) : arg)
                    break
                    case "u":
                        arg = arg >>> 0
                    break
                    case "x":
                        arg = arg.toString(16)
                    break
                    case "X":
                        arg = arg.toString(16).toUpperCase()
                    break
                }
                if (re.json.test(match[8])) {
                    output[output.length] = arg
                }
                else {
                    if (re.number.test(match[8]) && (!is_positive || match[3])) {
                        sign = is_positive ? "+" : "-"
                        arg = arg.toString().replace(re.sign, "")
                    }
                    else {
                        sign = ""
                    }
                    pad_character = match[4] ? match[4] === "0" ? "0" : match[4].charAt(1) : " "
                    pad_length = match[6] - (sign + arg).length
                    pad = match[6] ? (pad_length > 0 ? str_repeat(pad_character, pad_length) : "") : ""
                    output[output.length] = match[5] ? sign + arg + pad : (pad_character === "0" ? sign + pad + arg : pad + sign + arg)
                }
            }
        }
        return output.join("")
    }

    sprintf.cache = {}

    sprintf.parse = function(fmt) {
        var _fmt = fmt, match = [], parse_tree = [], arg_names = 0
        while (_fmt) {
            if ((match = re.text.exec(_fmt)) !== null) {
                parse_tree[parse_tree.length] = match[0]
            }
            else if ((match = re.modulo.exec(_fmt)) !== null) {
                parse_tree[parse_tree.length] = "%"
            }
            else if ((match = re.placeholder.exec(_fmt)) !== null) {
                if (match[2]) {
                    arg_names |= 1
                    var field_list = [], replacement_field = match[2], field_match = []
                    if ((field_match = re.key.exec(replacement_field)) !== null) {
                        field_list[field_list.length] = field_match[1]
                        while ((replacement_field = replacement_field.substring(field_match[0].length)) !== "") {
                            if ((field_match = re.key_access.exec(replacement_field)) !== null) {
                                field_list[field_list.length] = field_match[1]
                            }
                            else if ((field_match = re.index_access.exec(replacement_field)) !== null) {
                                field_list[field_list.length] = field_match[1]
                            }
                            else {
                                throw new SyntaxError("[sprintf] failed to parse named argument key")
                            }
                        }
                    }
                    else {
                        throw new SyntaxError("[sprintf] failed to parse named argument key")
                    }
                    match[2] = field_list
                }
                else {
                    arg_names |= 2
                }
                if (arg_names === 3) {
                    throw new Error("[sprintf] mixing positional and named placeholders is not (yet) supported")
                }
                parse_tree[parse_tree.length] = match
            }
            else {
                throw new SyntaxError("[sprintf] unexpected placeholder")
            }
            _fmt = _fmt.substring(match[0].length)
        }
        return parse_tree
    }

    var vsprintf = function(fmt, argv, _argv) {
        _argv = (argv || []).slice(0)
        _argv.splice(0, 0, fmt)
        return sprintf.apply(null, _argv)
    }

    /**
     * helpers
     */
    function get_type(variable) {
        return Object.prototype.toString.call(variable).slice(8, -1).toLowerCase()
    }

    function str_repeat(input, multiplier) {
        return Array(multiplier + 1).join(input)
    }

    /**
     * export to either browser or node.js
     */
    if (typeof exports !== "undefined") {
        exports.sprintf = sprintf
        exports.vsprintf = vsprintf
    }
    else {
        window.sprintf = sprintf
        window.vsprintf = vsprintf

        if (typeof define === "function" && define.amd) {
            define('sprintf',[],function() {
                return {
                    sprintf: sprintf,
                    vsprintf: vsprintf
                }
            })
        }
    }
})(typeof window === "undefined" ? this : window);

/*
 Copyright (c) 2003-2015, CKSource - Frederico Knabben. All rights reserved.
 For licensing, see LICENSE.md or http://ckeditor.com/license
*/
(function(a){CKEDITOR.config.jqueryOverrideVal="undefined"==typeof CKEDITOR.config.jqueryOverrideVal?!0:CKEDITOR.config.jqueryOverrideVal;"undefined"!=typeof a&&(a.extend(a.fn,{ckeditorGet:function(){var a=this.eq(0).data("ckeditorInstance");if(!a)throw"CKEditor is not initialized yet, use ckeditor() with a callback.";return a},ckeditor:function(g,d){if(!CKEDITOR.env.isCompatible)throw Error("The environment is incompatible.");if(!a.isFunction(g))var k=d,d=g,g=k;var i=[],d=d||{};this.each(function(){var b=
a(this),c=b.data("ckeditorInstance"),f=b.data("_ckeditorInstanceLock"),h=this,j=new a.Deferred;i.push(j.promise());if(c&&!f)g&&g.apply(c,[this]),j.resolve();else if(f)c.once("instanceReady",function(){setTimeout(function(){c.element?(c.element.$==h&&g&&g.apply(c,[h]),j.resolve()):setTimeout(arguments.callee,100)},0)},null,null,9999);else{if(d.autoUpdateElement||"undefined"==typeof d.autoUpdateElement&&CKEDITOR.config.autoUpdateElement)d.autoUpdateElementJquery=!0;d.autoUpdateElement=!1;b.data("_ckeditorInstanceLock",
!0);c=a(this).is("textarea")?CKEDITOR.replace(h,d):CKEDITOR.inline(h,d);b.data("ckeditorInstance",c);c.on("instanceReady",function(d){var e=d.editor;setTimeout(function(){if(e.element){d.removeListener();e.on("dataReady",function(){b.trigger("dataReady.ckeditor",[e])});e.on("setData",function(a){b.trigger("setData.ckeditor",[e,a.data])});e.on("getData",function(a){b.trigger("getData.ckeditor",[e,a.data])},999);e.on("destroy",function(){b.trigger("destroy.ckeditor",[e])});e.on("save",function(){a(h.form).submit();
return!1},null,null,20);if(e.config.autoUpdateElementJquery&&b.is("textarea")&&a(h.form).length){var c=function(){b.ckeditor(function(){e.updateElement()})};a(h.form).submit(c);a(h.form).bind("form-pre-serialize",c);b.bind("destroy.ckeditor",function(){a(h.form).unbind("submit",c);a(h.form).unbind("form-pre-serialize",c)})}e.on("destroy",function(){b.removeData("ckeditorInstance")});b.removeData("_ckeditorInstanceLock");b.trigger("instanceReady.ckeditor",[e]);g&&g.apply(e,[h]);j.resolve()}else setTimeout(arguments.callee,
100)},0)},null,null,9999)}});var f=new a.Deferred;this.promise=f.promise();a.when.apply(this,i).then(function(){f.resolve()});this.editor=this.eq(0).data("ckeditorInstance");return this}}),CKEDITOR.config.jqueryOverrideVal&&(a.fn.val=CKEDITOR.tools.override(a.fn.val,function(g){return function(d){if(arguments.length){var k=this,i=[],f=this.each(function(){var b=a(this),c=b.data("ckeditorInstance");if(b.is("textarea")&&c){var f=new a.Deferred;c.setData(d,function(){f.resolve()});i.push(f.promise());
return!0}return g.call(b,d)});if(i.length){var b=new a.Deferred;a.when.apply(this,i).done(function(){b.resolveWith(k)});return b.promise()}return f}var f=a(this).eq(0),c=f.data("ckeditorInstance");return f.is("textarea")&&c?c.getData():g.call(f)}})))})(window.jQuery);
define("jqueryAdapter", function(){});

;(function(root, factory) {
  if (typeof define === 'function' && define.amd) {
    define('html.sortable',['jquery'], factory);
  } else if (typeof exports === 'object') {
    module.exports = factory(require('jquery'));
  } else {
    root.sortable = factory(root.jQuery);
  }
}(this, function($) {
/*
 * HTML5 Sortable jQuery Plugin
 * https://github.com/voidberg/html5sortable
 *
 * Original code copyright 2012 Ali Farhadi.
 * This version is mantained by Alexandru Badiu <andu@ctrlz.ro> & Lukas Oppermann <lukas@vea.re>
 *
 *
 * Released under the MIT license.
 */
'use strict';
/*
 * variables global to the plugin
 */
var dragging;
var draggingHeight;
var placeholders = $();
var sortables = [];
/*
 * remove event handlers from items
 * @param [jquery Collection] items
 * @info event.h5s (jquery way of namespacing events, to bind multiple handlers to the event)
 */
var _removeItemEvents = function(items) {
  items.off('dragstart.h5s');
  items.off('dragend.h5s');
  items.off('selectstart.h5s');
  items.off('dragover.h5s');
  items.off('dragenter.h5s');
  items.off('drop.h5s');
};
/*
 * remove event handlers from sortable
 * @param [jquery Collection] sortable
 * @info event.h5s (jquery way of namespacing events, to bind multiple handlers to the event)
 */
var _removeSortableEvents = function(sortable) {
  sortable.off('dragover.h5s');
  sortable.off('dragenter.h5s');
  sortable.off('drop.h5s');
};
/*
 * attache ghost to dataTransfer object
 * @param [event] original event
 * @param [object] ghost-object with item, x and y coordinates
 */
var _attachGhost = function(event, ghost) {
  // this needs to be set for HTML5 drag & drop to work
  event.dataTransfer.effectAllowed = 'move';
  event.dataTransfer.setData('text', '');

  // check if setDragImage method is available
  if (event.dataTransfer.setDragImage) {
    event.dataTransfer.setDragImage(ghost.item, ghost.x, ghost.y);
  }
};
/**
 * _addGhostPos clones the dragged item and adds it as a Ghost item
 * @param [object] event - the event fired when dragstart is triggered
 * @param [object] ghost - .item = node, draggedItem = jQuery collection
 */
var _addGhostPos = function(e, ghost) {
  if (!ghost.x) {
    ghost.x = parseInt(e.pageX - ghost.draggedItem.offset().left);
  }
  if (!ghost.y) {
    ghost.y = parseInt(e.pageY - ghost.draggedItem.offset().top);
  }
  return ghost;
};
/**
 * _makeGhost decides which way to make a ghost and passes it to attachGhost
 * @param [jQuery selection] $draggedItem - the item that the user drags
 */
var _makeGhost = function($draggedItem) {
  return {
    item: $draggedItem[0],
    draggedItem: $draggedItem
  };
};
/**
 * _getGhost constructs ghost and attaches it to dataTransfer
 * @param [event] event - the original drag event object
 * @param [jQuery selection] $draggedItem - the item that the user drags
 * @param [object] ghostOpt - the ghost options
 */
// TODO: could $draggedItem be replaced by event.target in all instances
var _getGhost = function(event, $draggedItem) {
  // add ghost item & draggedItem to ghost object
  var ghost = _makeGhost($draggedItem);
  // attach ghost position
  ghost = _addGhostPos(event, ghost);
  // attach ghost to dataTransfer
  _attachGhost(event, ghost);
};
/*
 * return options if not set on sortable already
 * @param [object] soptions
 * @param [object] options
 */
var _getOptions = function(soptions, options) {
  if (typeof soptions === 'undefined') {
    return options;
  }
  return soptions;
};
/*
 * remove data from sortable
 * @param [jquery Collection] a single sortable
 */
var _removeSortableData = function(sortable) {
  sortable.removeData('opts');
  sortable.removeData('connectWith');
  sortable.removeData('items');
  sortable.removeAttr('aria-dropeffect');
};
/*
 * remove data from items
 * @param [jquery Collection] items
 */
var _removeItemData = function(items) {
  items.removeAttr('aria-grabbed');
  items.removeAttr('draggable');
  items.removeAttr('role');
};
/*
 * check if two lists are connected
 * @param [jquery Collection] items
 */
var _listsConnected = function(curList, destList) {
  if (curList[0] === destList[0]) {
    return true;
  }
  if (curList.data('connectWith') !== undefined) {
    return curList.data('connectWith') === destList.data('connectWith');
  }
  return false;
};
/*
 * destroy the sortable
 * @param [jquery Collection] a single sortable
 */
var _destroySortable = function(sortable) {
  var opts = sortable.data('opts') || {};
  var items = sortable.children(opts.items);
  var handles = opts.handle ? items.find(opts.handle) : items;
  // remove event handlers & data from sortable
  _removeSortableEvents(sortable);
  _removeSortableData(sortable);
  // remove event handlers & data from items
  handles.off('mousedown.h5s');
  _removeItemEvents(items);
  _removeItemData(items);
};
/*
 * enable the sortable
 * @param [jquery Collection] a single sortable
 */
var _enableSortable = function(sortable) {
  var opts = sortable.data('opts');
  var items = sortable.children(opts.items);
  var handles = opts.handle ? items.find(opts.handle) : items;
  sortable.attr('aria-dropeffect', 'move');
  handles.attr('draggable', 'true');
  // IE FIX for ghost
  // can be disabled as it has the side effect that other events
  // (e.g. click) will be ignored
  var spanEl = (document || window.document).createElement('span');
  if (typeof spanEl.dragDrop === 'function' && !opts.disableIEFix) {
    handles.on('mousedown.h5s', function() {
      if (items.index(this) !== -1) {
        this.dragDrop();
      } else {
        $(this).parents(opts.items)[0].dragDrop();
      }
    });
  }
};
/*
 * disable the sortable
 * @param [jquery Collection] a single sortable
 */
var _disableSortable = function(sortable) {
  var opts = sortable.data('opts');
  var items = sortable.children(opts.items);
  var handles = opts.handle ? items.find(opts.handle) : items;
  sortable.attr('aria-dropeffect', 'none');
  handles.attr('draggable', false);
  handles.off('mousedown.h5s');
};
/*
 * reload the sortable
 * @param [jquery Collection] a single sortable
 * @description events need to be removed to not be double bound
 */
var _reloadSortable = function(sortable) {
  var opts = sortable.data('opts');
  var items = sortable.children(opts.items);
  var handles = opts.handle ? items.find(opts.handle) : items;
  // remove event handlers from items
  _removeItemEvents(items);
  handles.off('mousedown.h5s');
  // remove event handlers from sortable
  _removeSortableEvents(sortable);
};
/*
 * public sortable object
 * @param [object|string] options|method
 */
var sortable = function(selector, options) {

  var $sortables = $(selector);
  var method = String(options);

  options = $.extend({
    connectWith: false,
    placeholder: null,
    // dragImage can be null or a jQuery element
    dragImage: null,
    disableIEFix: false,
    placeholderClass: 'sortable-placeholder',
    draggingClass: 'sortable-dragging',
    hoverClass: false
  }, options);

  /* TODO: maxstatements should be 25, fix and remove line below */
  /*jshint maxstatements:false */
  return $sortables.each(function() {

    var $sortable = $(this);

    if (/enable|disable|destroy/.test(method)) {
      sortable[method]($sortable);
      return;
    }

    // get options & set options on sortable
    options = _getOptions($sortable.data('opts'), options);
    $sortable.data('opts', options);
    // reset sortable
    _reloadSortable($sortable);
    // initialize
    var items = $sortable.children(options.items);
    var index;
    var startParent;
    var newParent;
    var placeholder = (options.placeholder === null) ? $('<' + (/^ul|ol$/i.test(this.tagName) ? 'li' : 'div') + ' class="' + options.placeholderClass + '"/>') : $(options.placeholder).addClass(options.placeholderClass);

    // setup sortable ids
    if (!$sortable.attr('data-sortable-id')) {
      var id = sortables.length;
      sortables[id] = $sortable;
      $sortable.attr('data-sortable-id', id);
      items.attr('data-item-sortable-id', id);
    }

    $sortable.data('items', options.items);
    placeholders = placeholders.add(placeholder);
    if (options.connectWith) {
      $sortable.data('connectWith', options.connectWith);
    }

    _enableSortable($sortable);
    items.attr('role', 'option');
    items.attr('aria-grabbed', 'false');

    // Mouse over class
    if (options.hoverClass) {
      var hoverClass = 'sortable-over';
      if (typeof options.hoverClass === 'string') {
        hoverClass = options.hoverClass;
      }

      items.hover(function() {
        $(this).addClass(hoverClass);
      }, function() {
        $(this).removeClass(hoverClass);
      });
    }

    // Handle drag events on draggable items
    items.on('dragstart.h5s', function(e) {
      e.stopImmediatePropagation();

      if (options.dragImage) {
        _attachGhost(e.originalEvent, {
          item: options.dragImage,
          x: 0,
          y: 0
        });
        console.log('WARNING: dragImage option is deprecated' +
        ' and will be removed in the future!');
      } else {
        // add transparent clone or other ghost to cursor
        _getGhost(e.originalEvent, $(this), options.dragImage);
      }
      // cache selsection & add attr for dragging
      dragging = $(this);
      dragging.addClass(options.draggingClass);
      dragging.attr('aria-grabbed', 'true');
      // grab values
      index = dragging.index();
      draggingHeight = dragging.height();
      startParent = $(this).parent();
      // trigger sortstar update
      dragging.parent().triggerHandler('sortstart', {
        item: dragging,
        placeholder: placeholder,
        startparent: startParent
      });
    });
    // Handle drag events on draggable items
    items.on('dragend.h5s', function() {
      if (!dragging) {
        return;
      }
      // remove dragging attributes and show item
      dragging.removeClass(options.draggingClass);
      dragging.attr('aria-grabbed', 'false');
      dragging.show();

      placeholders.detach();
      newParent = $(this).parent();
      dragging.parent().triggerHandler('sortstop', {
        item: dragging,
        startparent: startParent,
      });
      if (index !== dragging.index() ||
          startParent.get(0) !== newParent.get(0)) {
        dragging.parent().triggerHandler('sortupdate', {
          item: dragging,
          index: newParent.children(newParent.data('items')).index(dragging),
          oldindex: items.index(dragging),
          elementIndex: dragging.index(),
          oldElementIndex: index,
          startparent: startParent,
          endparent: newParent
        });
      }
      dragging = null;
      draggingHeight = null;
    });
    // Handle drop event on sortable & placeholder
    // TODO: REMOVE placeholder?????
    $(this).add([placeholder]).on('drop.h5s', function(e) {
      if (!_listsConnected($sortable, $(dragging).parent())) {
        return;
      }

      e.stopPropagation();
      placeholders.filter(':visible').after(dragging);
      dragging.trigger('dragend.h5s');
      return false;
    });

    // Handle dragover and dragenter events on draggable items
    items.add([this]).on('dragover.h5s dragenter.h5s', function(e) {
      if (!_listsConnected($sortable, $(dragging).parent())) {
        return;
      }

      e.preventDefault();
      e.originalEvent.dataTransfer.dropEffect = 'move';
      if (items.is(this)) {
        var thisHeight = $(this).height();
        if (options.forcePlaceholderSize) {
          placeholder.height(draggingHeight);
        }

        // Check if $(this) is bigger than the draggable. If it is, we have to define a dead zone to prevent flickering
        if (thisHeight > draggingHeight) {
          // Dead zone?
          var deadZone = thisHeight - draggingHeight;
          var offsetTop = $(this).offset().top;
          if (placeholder.index() < $(this).index() &&
              e.originalEvent.pageY < offsetTop + deadZone) {
            return false;
          }
          if (placeholder.index() > $(this).index() &&
              e.originalEvent.pageY > offsetTop + thisHeight - deadZone) {
            return false;
          }
        }

        dragging.hide();
        if (placeholder.index() < $(this).index()) {
          $(this).after(placeholder);
        } else {
          $(this).before(placeholder);
        }
        placeholders.not(placeholder).detach();
      } else {
        if (!placeholders.is(this) && !$(this).children(options.items).length) {
          placeholders.detach();
          $(this).append(placeholder);
        }
      }
      return false;
    });
  });
};

sortable.destroy = function(sortable) {
  _destroySortable(sortable);
};

sortable.enable = function(sortable) {
  _enableSortable(sortable);
};

sortable.disable = function(sortable) {
  _disableSortable(sortable);
};

$.fn.sortable = function(options) {
  return sortable(this, options);
};

return sortable;
}));

/*global unescape, module, define, window, global*/

/*
 UriTemplate Copyright (c) 2012-2013 Franz Antesberger. All Rights Reserved.
 Available via the MIT license.
*/

(function (exportCallback) {
    "use strict";

var UriTemplateError = (function () {

    function UriTemplateError (options) {
        this.options = options;
    }

    UriTemplateError.prototype.toString = function () {
        if (JSON && JSON.stringify) {
            return JSON.stringify(this.options);
        }
        else {
            return this.options;
        }
    };

    return UriTemplateError;
}());

var objectHelper = (function () {
    function isArray (value) {
        return Object.prototype.toString.apply(value) === '[object Array]';
    }

    function isString (value) {
        return Object.prototype.toString.apply(value) === '[object String]';
    }
    
    function isNumber (value) {
        return Object.prototype.toString.apply(value) === '[object Number]';
    }
    
    function isBoolean (value) {
        return Object.prototype.toString.apply(value) === '[object Boolean]';
    }
    
    function join (arr, separator) {
        var
            result = '',
            first = true,
            index;
        for (index = 0; index < arr.length; index += 1) {
            if (first) {
                first = false;
            }
            else {
                result += separator;
            }
            result += arr[index];
        }
        return result;
    }

    function map (arr, mapper) {
        var
            result = [],
            index = 0;
        for (; index < arr.length; index += 1) {
            result.push(mapper(arr[index]));
        }
        return result;
    }

    function filter (arr, predicate) {
        var
            result = [],
            index = 0;
        for (; index < arr.length; index += 1) {
            if (predicate(arr[index])) {
                result.push(arr[index]);
            }
        }
        return result;
    }

    function deepFreezeUsingObjectFreeze (object) {
        if (typeof object !== "object" || object === null) {
            return object;
        }
        Object.freeze(object);
        var property, propertyName;
        for (propertyName in object) {
            if (object.hasOwnProperty(propertyName)) {
                property = object[propertyName];
                // be aware, arrays are 'object', too
                if (typeof property === "object") {
                    deepFreeze(property);
                }
            }
        }
        return object;
    }

    function deepFreeze (object) {
        if (typeof Object.freeze === 'function') {
            return deepFreezeUsingObjectFreeze(object);
        }
        return object;
    }


    return {
        isArray: isArray,
        isString: isString,
        isNumber: isNumber,
        isBoolean: isBoolean,
        join: join,
        map: map,
        filter: filter,
        deepFreeze: deepFreeze
    };
}());

var charHelper = (function () {

    function isAlpha (chr) {
        return (chr >= 'a' && chr <= 'z') || ((chr >= 'A' && chr <= 'Z'));
    }

    function isDigit (chr) {
        return chr >= '0' && chr <= '9';
    }

    function isHexDigit (chr) {
        return isDigit(chr) || (chr >= 'a' && chr <= 'f') || (chr >= 'A' && chr <= 'F');
    }

    return {
        isAlpha: isAlpha,
        isDigit: isDigit,
        isHexDigit: isHexDigit
    };
}());

var pctEncoder = (function () {
    var utf8 = {
        encode: function (chr) {
            // see http://ecmanaut.blogspot.de/2006/07/encoding-decoding-utf8-in-javascript.html
            return unescape(encodeURIComponent(chr));
        },
        numBytes: function (firstCharCode) {
            if (firstCharCode <= 0x7F) {
                return 1;
            }
            else if (0xC2 <= firstCharCode && firstCharCode <= 0xDF) {
                return 2;
            }
            else if (0xE0 <= firstCharCode && firstCharCode <= 0xEF) {
                return 3;
            }
            else if (0xF0 <= firstCharCode && firstCharCode <= 0xF4) {
                return 4;
            }
            // no valid first octet
            return 0;
        },
        isValidFollowingCharCode: function (charCode) {
            return 0x80 <= charCode && charCode <= 0xBF;
        }
    };

    /**
     * encodes a character, if needed or not.
     * @param chr
     * @return pct-encoded character
     */
    function encodeCharacter (chr) {
        var
            result = '',
            octets = utf8.encode(chr),
            octet,
            index;
        for (index = 0; index < octets.length; index += 1) {
            octet = octets.charCodeAt(index);
            result += '%' + (octet < 0x10 ? '0' : '') + octet.toString(16).toUpperCase();
        }
        return result;
    }

    /**
     * Returns, whether the given text at start is in the form 'percent hex-digit hex-digit', like '%3F'
     * @param text
     * @param start
     * @return {boolean|*|*}
     */
    function isPercentDigitDigit (text, start) {
        return text.charAt(start) === '%' && charHelper.isHexDigit(text.charAt(start + 1)) && charHelper.isHexDigit(text.charAt(start + 2));
    }

    /**
     * Parses a hex number from start with length 2.
     * @param text a string
     * @param start the start index of the 2-digit hex number
     * @return {Number}
     */
    function parseHex2 (text, start) {
        return parseInt(text.substr(start, 2), 16);
    }

    /**
     * Returns whether or not the given char sequence is a correctly pct-encoded sequence.
     * @param chr
     * @return {boolean}
     */
    function isPctEncoded (chr) {
        if (!isPercentDigitDigit(chr, 0)) {
            return false;
        }
        var firstCharCode = parseHex2(chr, 1);
        var numBytes = utf8.numBytes(firstCharCode);
        if (numBytes === 0) {
            return false;
        }
        for (var byteNumber = 1; byteNumber < numBytes; byteNumber += 1) {
            if (!isPercentDigitDigit(chr, 3*byteNumber) || !utf8.isValidFollowingCharCode(parseHex2(chr, 3*byteNumber + 1))) {
                return false;
            }
        }
        return true;
    }

    /**
     * Reads as much as needed from the text, e.g. '%20' or '%C3%B6'. It does not decode!
     * @param text
     * @param startIndex
     * @return the character or pct-string of the text at startIndex
     */
    function pctCharAt(text, startIndex) {
        var chr = text.charAt(startIndex);
        if (!isPercentDigitDigit(text, startIndex)) {
            return chr;
        }
        var utf8CharCode = parseHex2(text, startIndex + 1);
        var numBytes = utf8.numBytes(utf8CharCode);
        if (numBytes === 0) {
            return chr;
        }
        for (var byteNumber = 1; byteNumber < numBytes; byteNumber += 1) {
            if (!isPercentDigitDigit(text, startIndex + 3 * byteNumber) || !utf8.isValidFollowingCharCode(parseHex2(text, startIndex + 3 * byteNumber + 1))) {
                return chr;
            }
        }
        return text.substr(startIndex, 3 * numBytes);
    }

    return {
        encodeCharacter: encodeCharacter,
        isPctEncoded: isPctEncoded,
        pctCharAt: pctCharAt
    };
}());

var rfcCharHelper = (function () {

    /**
     * Returns if an character is an varchar character according 2.3 of rfc 6570
     * @param chr
     * @return (Boolean)
     */
    function isVarchar (chr) {
        return charHelper.isAlpha(chr) || charHelper.isDigit(chr) || chr === '_' || pctEncoder.isPctEncoded(chr);
    }

    /**
     * Returns if chr is an unreserved character according 1.5 of rfc 6570
     * @param chr
     * @return {Boolean}
     */
    function isUnreserved (chr) {
        return charHelper.isAlpha(chr) || charHelper.isDigit(chr) || chr === '-' || chr === '.' || chr === '_' || chr === '~';
    }

    /**
     * Returns if chr is an reserved character according 1.5 of rfc 6570
     * or the percent character mentioned in 3.2.1.
     * @param chr
     * @return {Boolean}
     */
    function isReserved (chr) {
        return chr === ':' || chr === '/' || chr === '?' || chr === '#' || chr === '[' || chr === ']' || chr === '@' || chr === '!' || chr === '$' || chr === '&' || chr === '(' ||
            chr === ')' || chr === '*' || chr === '+' || chr === ',' || chr === ';' || chr === '=' || chr === "'";
    }

    return {
        isVarchar: isVarchar,
        isUnreserved: isUnreserved,
        isReserved: isReserved
    };

}());

/**
 * encoding of rfc 6570
 */
var encodingHelper = (function () {

    function encode (text, passReserved) {
        var
            result = '',
            index,
            chr = '';
        if (typeof text === "number" || typeof text === "boolean") {
            text = text.toString();
        }
        for (index = 0; index < text.length; index += chr.length) {
            chr = text.charAt(index);
            result += rfcCharHelper.isUnreserved(chr) || (passReserved && rfcCharHelper.isReserved(chr)) ? chr : pctEncoder.encodeCharacter(chr);
        }
        return result;
    }

    function encodePassReserved (text) {
        return encode(text, true);
    }

    function encodeLiteralCharacter (literal, index) {
        var chr = pctEncoder.pctCharAt(literal, index);
        if (chr.length > 1) {
            return chr;
        }
        else {
            return rfcCharHelper.isReserved(chr) || rfcCharHelper.isUnreserved(chr) ? chr : pctEncoder.encodeCharacter(chr);
        }
    }

    function encodeLiteral (literal) {
        var
            result = '',
            index,
            chr = '';
        for (index = 0; index < literal.length; index += chr.length) {
            chr = pctEncoder.pctCharAt(literal, index);
            if (chr.length > 1) {
                result += chr;
            }
            else {
                result += rfcCharHelper.isReserved(chr) || rfcCharHelper.isUnreserved(chr) ? chr : pctEncoder.encodeCharacter(chr);
            }
        }
        return result;
    }

    return {
        encode: encode,
        encodePassReserved: encodePassReserved,
        encodeLiteral: encodeLiteral,
        encodeLiteralCharacter: encodeLiteralCharacter
    };

}());


// the operators defined by rfc 6570
var operators = (function () {

    var
        bySymbol = {};

    function create (symbol) {
        bySymbol[symbol] = {
            symbol: symbol,
            separator: (symbol === '?') ? '&' : (symbol === '' || symbol === '+' || symbol === '#') ? ',' : symbol,
            named: symbol === ';' || symbol === '&' || symbol === '?',
            ifEmpty: (symbol === '&' || symbol === '?') ? '=' : '',
            first: (symbol === '+' ) ? '' : symbol,
            encode: (symbol === '+' || symbol === '#') ? encodingHelper.encodePassReserved : encodingHelper.encode,
            toString: function () {
                return this.symbol;
            }
        };
    }

    create('');
    create('+');
    create('#');
    create('.');
    create('/');
    create(';');
    create('?');
    create('&');
    return {
        valueOf: function (chr) {
            if (bySymbol[chr]) {
                return bySymbol[chr];
            }
            if ("=,!@|".indexOf(chr) >= 0) {
                return null;
            }
            return bySymbol[''];
        }
    };
}());


/**
 * Detects, whether a given element is defined in the sense of rfc 6570
 * Section 2.3 of the RFC makes clear defintions:
 * * undefined and null are not defined.
 * * the empty string is defined
 * * an array ("list") is defined, if it is not empty (even if all elements are not defined)
 * * an object ("map") is defined, if it contains at least one property with defined value
 * @param object
 * @return {Boolean}
 */
function isDefined (object) {
    var
        propertyName;
    if (object === null || object === undefined) {
        return false;
    }
    if (objectHelper.isArray(object)) {
        // Section 2.3: A variable defined as a list value is considered undefined if the list contains zero members
        return object.length > 0;
    }
    if (typeof object === "string" || typeof object === "number" || typeof object === "boolean") {
        // falsy values like empty strings, false or 0 are "defined"
        return true;
    }
    // else Object
    for (propertyName in object) {
        if (object.hasOwnProperty(propertyName) && isDefined(object[propertyName])) {
            return true;
        }
    }
    return false;
}

var LiteralExpression = (function () {
    function LiteralExpression (literal) {
        this.literal = encodingHelper.encodeLiteral(literal);
    }

    LiteralExpression.prototype.expand = function () {
        return this.literal;
    };

    LiteralExpression.prototype.toString = LiteralExpression.prototype.expand;

    return LiteralExpression;
}());

var parse = (function () {

    function parseExpression (expressionText) {
        var
            operator,
            varspecs = [],
            varspec = null,
            varnameStart = null,
            maxLengthStart = null,
            index,
            chr = '';

        function closeVarname () {
            var varname = expressionText.substring(varnameStart, index);
            if (varname.length === 0) {
                throw new UriTemplateError({expressionText: expressionText, message: "a varname must be specified", position: index});
            }
            varspec = {varname: varname, exploded: false, maxLength: null};
            varnameStart = null;
        }

        function closeMaxLength () {
            if (maxLengthStart === index) {
                throw new UriTemplateError({expressionText: expressionText, message: "after a ':' you have to specify the length", position: index});
            }
            varspec.maxLength = parseInt(expressionText.substring(maxLengthStart, index), 10);
            maxLengthStart = null;
        }

        operator = (function (operatorText) {
            var op = operators.valueOf(operatorText);
            if (op === null) {
                throw new UriTemplateError({expressionText: expressionText, message: "illegal use of reserved operator", position: index, operator: operatorText});
            }
            return op;
        }(expressionText.charAt(0)));
        index = operator.symbol.length;

        varnameStart = index;

        for (; index < expressionText.length; index += chr.length) {
            chr = pctEncoder.pctCharAt(expressionText, index);

            if (varnameStart !== null) {
                // the spec says: varname =  varchar *( ["."] varchar )
                // so a dot is allowed except for the first char
                if (chr === '.') {
                    if (varnameStart === index) {
                        throw new UriTemplateError({expressionText: expressionText, message: "a varname MUST NOT start with a dot", position: index});
                    }
                    continue;
                }
                if (rfcCharHelper.isVarchar(chr)) {
                    continue;
                }
                closeVarname();
            }
            if (maxLengthStart !== null) {
                if (index === maxLengthStart && chr === '0') {
                    throw new UriTemplateError({expressionText: expressionText, message: "A :prefix must not start with digit 0", position: index});
                }
                if (charHelper.isDigit(chr)) {
                    if (index - maxLengthStart >= 4) {
                        throw new UriTemplateError({expressionText: expressionText, message: "A :prefix must have max 4 digits", position: index});
                    }
                    continue;
                }
                closeMaxLength();
            }
            if (chr === ':') {
                if (varspec.maxLength !== null) {
                    throw new UriTemplateError({expressionText: expressionText, message: "only one :maxLength is allowed per varspec", position: index});
                }
                if (varspec.exploded) {
                    throw new UriTemplateError({expressionText: expressionText, message: "an exploeded varspec MUST NOT be varspeced", position: index});
                }
                maxLengthStart = index + 1;
                continue;
            }
            if (chr === '*') {
                if (varspec === null) {
                    throw new UriTemplateError({expressionText: expressionText, message: "exploded without varspec", position: index});
                }
                if (varspec.exploded) {
                    throw new UriTemplateError({expressionText: expressionText, message: "exploded twice", position: index});
                }
                if (varspec.maxLength) {
                    throw new UriTemplateError({expressionText: expressionText, message: "an explode (*) MUST NOT follow to a prefix", position: index});
                }
                varspec.exploded = true;
                continue;
            }
            // the only legal character now is the comma
            if (chr === ',') {
                varspecs.push(varspec);
                varspec = null;
                varnameStart = index + 1;
                continue;
            }
            throw new UriTemplateError({expressionText: expressionText, message: "illegal character", character: chr, position: index});
        } // for chr
        if (varnameStart !== null) {
            closeVarname();
        }
        if (maxLengthStart !== null) {
            closeMaxLength();
        }
        varspecs.push(varspec);
        return new VariableExpression(expressionText, operator, varspecs);
    }

    function parse (uriTemplateText) {
        // assert filled string
        var
            index,
            chr,
            expressions = [],
            braceOpenIndex = null,
            literalStart = 0;
        for (index = 0; index < uriTemplateText.length; index += 1) {
            chr = uriTemplateText.charAt(index);
            if (literalStart !== null) {
                if (chr === '}') {
                    throw new UriTemplateError({templateText: uriTemplateText, message: "unopened brace closed", position: index});
                }
                if (chr === '{') {
                    if (literalStart < index) {
                        expressions.push(new LiteralExpression(uriTemplateText.substring(literalStart, index)));
                    }
                    literalStart = null;
                    braceOpenIndex = index;
                }
                continue;
            }

            if (braceOpenIndex !== null) {
                // here just { is forbidden
                if (chr === '{') {
                    throw new UriTemplateError({templateText: uriTemplateText, message: "brace already opened", position: index});
                }
                if (chr === '}') {
                    if (braceOpenIndex + 1 === index) {
                        throw new UriTemplateError({templateText: uriTemplateText, message: "empty braces", position: braceOpenIndex});
                    }
                    try {
                        expressions.push(parseExpression(uriTemplateText.substring(braceOpenIndex + 1, index)));
                    }
                    catch (error) {
                        if (error.prototype === UriTemplateError.prototype) {
                            throw new UriTemplateError({templateText: uriTemplateText, message: error.options.message, position: braceOpenIndex + error.options.position, details: error.options});
                        }
                        throw error;
                    }
                    braceOpenIndex = null;
                    literalStart = index + 1;
                }
                continue;
            }
            throw new Error('reached unreachable code');
        }
        if (braceOpenIndex !== null) {
            throw new UriTemplateError({templateText: uriTemplateText, message: "unclosed brace", position: braceOpenIndex});
        }
        if (literalStart < uriTemplateText.length) {
            expressions.push(new LiteralExpression(uriTemplateText.substr(literalStart)));
        }
        return new UriTemplate(uriTemplateText, expressions);
    }

    return parse;
}());

var VariableExpression = (function () {
    // helper function if JSON is not available
    function prettyPrint (value) {
        return (JSON && JSON.stringify) ? JSON.stringify(value) : value;
    }

    function isEmpty (value) {
        if (!isDefined(value)) {
            return true;
        }
        if (objectHelper.isString(value)) {
            return value === '';
        }
        if (objectHelper.isNumber(value) || objectHelper.isBoolean(value)) {
            return false;
        }
        if (objectHelper.isArray(value)) {
            return value.length === 0;
        }
        for (var propertyName in value) {
            if (value.hasOwnProperty(propertyName)) {
                return false;
            }
        }
        return true;
    }

    function propertyArray (object) {
        var
            result = [],
            propertyName;
        for (propertyName in object) {
            if (object.hasOwnProperty(propertyName)) {
                result.push({name: propertyName, value: object[propertyName]});
            }
        }
        return result;
    }

    function VariableExpression (templateText, operator, varspecs) {
        this.templateText = templateText;
        this.operator = operator;
        this.varspecs = varspecs;
    }

    VariableExpression.prototype.toString = function () {
        return this.templateText;
    };

    function expandSimpleValue(varspec, operator, value) {
        var result = '';
        value = value.toString();
        if (operator.named) {
            result += encodingHelper.encodeLiteral(varspec.varname);
            if (value === '') {
                result += operator.ifEmpty;
                return result;
            }
            result += '=';
        }
        if (varspec.maxLength !== null) {
            value = value.substr(0, varspec.maxLength);
        }
        result += operator.encode(value);
        return result;
    }

    function valueDefined (nameValue) {
        return isDefined(nameValue.value);
    }

    function expandNotExploded(varspec, operator, value) {
        var
            arr = [],
            result = '';
        if (operator.named) {
            result += encodingHelper.encodeLiteral(varspec.varname);
            if (isEmpty(value)) {
                result += operator.ifEmpty;
                return result;
            }
            result += '=';
        }
        if (objectHelper.isArray(value)) {
            arr = value;
            arr = objectHelper.filter(arr, isDefined);
            arr = objectHelper.map(arr, operator.encode);
            result += objectHelper.join(arr, ',');
        }
        else {
            arr = propertyArray(value);
            arr = objectHelper.filter(arr, valueDefined);
            arr = objectHelper.map(arr, function (nameValue) {
                return operator.encode(nameValue.name) + ',' + operator.encode(nameValue.value);
            });
            result += objectHelper.join(arr, ',');
        }
        return result;
    }

    function expandExplodedNamed (varspec, operator, value) {
        var
            isArray = objectHelper.isArray(value),
            arr = [];
        if (isArray) {
            arr = value;
            arr = objectHelper.filter(arr, isDefined);
            arr = objectHelper.map(arr, function (listElement) {
                var tmp = encodingHelper.encodeLiteral(varspec.varname);
                if (isEmpty(listElement)) {
                    tmp += operator.ifEmpty;
                }
                else {
                    tmp += '=' + operator.encode(listElement);
                }
                return tmp;
            });
        }
        else {
            arr = propertyArray(value);
            arr = objectHelper.filter(arr, valueDefined);
            arr = objectHelper.map(arr, function (nameValue) {
                var tmp = encodingHelper.encodeLiteral(nameValue.name);
                if (isEmpty(nameValue.value)) {
                    tmp += operator.ifEmpty;
                }
                else {
                    tmp += '=' + operator.encode(nameValue.value);
                }
                return tmp;
            });
        }
        return objectHelper.join(arr, operator.separator);
    }

    function expandExplodedUnnamed (operator, value) {
        var
            arr = [],
            result = '';
        if (objectHelper.isArray(value)) {
            arr = value;
            arr = objectHelper.filter(arr, isDefined);
            arr = objectHelper.map(arr, operator.encode);
            result += objectHelper.join(arr, operator.separator);
        }
        else {
            arr = propertyArray(value);
            arr = objectHelper.filter(arr, function (nameValue) {
                return isDefined(nameValue.value);
            });
            arr = objectHelper.map(arr, function (nameValue) {
                return operator.encode(nameValue.name) + '=' + operator.encode(nameValue.value);
            });
            result += objectHelper.join(arr, operator.separator);
        }
        return result;
    }


    VariableExpression.prototype.expand = function (variables) {
        var
            expanded = [],
            index,
            varspec,
            value,
            valueIsArr,
            oneExploded = false,
            operator = this.operator;

        // expand each varspec and join with operator's separator
        for (index = 0; index < this.varspecs.length; index += 1) {
            varspec = this.varspecs[index];
            value = variables[varspec.varname];
            // if (!isDefined(value)) {
            // if (variables.hasOwnProperty(varspec.name)) {
            if (value === null || value === undefined) {
                continue;
            }
            if (varspec.exploded) {
                oneExploded = true;
            }
            valueIsArr = objectHelper.isArray(value);
            if (typeof value === "string" || typeof value === "number" || typeof value === "boolean") {
                expanded.push(expandSimpleValue(varspec, operator, value));
            }
            else if (varspec.maxLength && isDefined(value)) {
                // 2.4.1 of the spec says: "Prefix modifiers are not applicable to variables that have composite values."
                throw new Error('Prefix modifiers are not applicable to variables that have composite values. You tried to expand ' + this + " with " + prettyPrint(value));
            }
            else if (!varspec.exploded) {
                if (operator.named || !isEmpty(value)) {
                    expanded.push(expandNotExploded(varspec, operator, value));
                }
            }
            else if (isDefined(value)) {
                if (operator.named) {
                    expanded.push(expandExplodedNamed(varspec, operator, value));
                }
                else {
                    expanded.push(expandExplodedUnnamed(operator, value));
                }
            }
        }

        if (expanded.length === 0) {
            return "";
        }
        else {
            return operator.first + objectHelper.join(expanded, operator.separator);
        }
    };

    return VariableExpression;
}());

var UriTemplate = (function () {
    function UriTemplate (templateText, expressions) {
        this.templateText = templateText;
        this.expressions = expressions;
        objectHelper.deepFreeze(this);
    }

    UriTemplate.prototype.toString = function () {
        return this.templateText;
    };

    UriTemplate.prototype.expand = function (variables) {
        // this.expressions.map(function (expression) {return expression.expand(variables);}).join('');
        var
            index,
            result = '';
        for (index = 0; index < this.expressions.length; index += 1) {
            result += this.expressions[index].expand(variables);
        }
        return result;
    };

    UriTemplate.parse = parse;
    UriTemplate.UriTemplateError = UriTemplateError;
    return UriTemplate;
}());

    exportCallback(UriTemplate);

}(function (UriTemplate) {
        "use strict";
        // export UriTemplate, when module is present, or pass it to window or global
        if (typeof module !== "undefined") {
            module.exports = UriTemplate;
        }
        else if (typeof define === "function") {
            define('uri-template',[],function() {
                return UriTemplate;
            });
        }
        else if (typeof window !== "undefined") {
            window.UriTemplate = UriTemplate;
        }
        else {
            global.UriTemplate = UriTemplate;
        }
    }
));

/*!
 * typeahead.js 0.10.5
 * https://github.com/twitter/typeahead.js
 * Copyright 2013-2014 Twitter, Inc. and other contributors; Licensed MIT
 */

(function($) {
    var _ = function() {
        "use strict";
        return {
            isMsie: function() {
                return /(msie|trident)/i.test(navigator.userAgent) ? navigator.userAgent.match(/(msie |rv:)(\d+(.\d+)?)/i)[2] : false;
            },
            isBlankString: function(str) {
                return !str || /^\s*$/.test(str);
            },
            escapeRegExChars: function(str) {
                return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
            },
            isString: function(obj) {
                return typeof obj === "string";
            },
            isNumber: function(obj) {
                return typeof obj === "number";
            },
            isArray: $.isArray,
            isFunction: $.isFunction,
            isObject: $.isPlainObject,
            isUndefined: function(obj) {
                return typeof obj === "undefined";
            },
            toStr: function toStr(s) {
                return _.isUndefined(s) || s === null ? "" : s + "";
            },
            bind: $.proxy,
            each: function(collection, cb) {
                $.each(collection, reverseArgs);
                function reverseArgs(index, value) {
                    return cb(value, index);
                }
            },
            map: $.map,
            filter: $.grep,
            every: function(obj, test) {
                var result = true;
                if (!obj) {
                    return result;
                }
                $.each(obj, function(key, val) {
                    if (!(result = test.call(null, val, key, obj))) {
                        return false;
                    }
                });
                return !!result;
            },
            some: function(obj, test) {
                var result = false;
                if (!obj) {
                    return result;
                }
                $.each(obj, function(key, val) {
                    if (result = test.call(null, val, key, obj)) {
                        return false;
                    }
                });
                return !!result;
            },
            mixin: $.extend,
            getUniqueId: function() {
                var counter = 0;
                return function() {
                    return counter++;
                };
            }(),
            templatify: function templatify(obj) {
                return $.isFunction(obj) ? obj : template;
                function template() {
                    return String(obj);
                }
            },
            defer: function(fn) {
                setTimeout(fn, 0);
            },
            debounce: function(func, wait, immediate) {
                var timeout, result;
                return function() {
                    var context = this, args = arguments, later, callNow;
                    later = function() {
                        timeout = null;
                        if (!immediate) {
                            result = func.apply(context, args);
                        }
                    };
                    callNow = immediate && !timeout;
                    clearTimeout(timeout);
                    timeout = setTimeout(later, wait);
                    if (callNow) {
                        result = func.apply(context, args);
                    }
                    return result;
                };
            },
            throttle: function(func, wait) {
                var context, args, timeout, result, previous, later;
                previous = 0;
                later = function() {
                    previous = new Date();
                    timeout = null;
                    result = func.apply(context, args);
                };
                return function() {
                    var now = new Date(), remaining = wait - (now - previous);
                    context = this;
                    args = arguments;
                    if (remaining <= 0) {
                        clearTimeout(timeout);
                        timeout = null;
                        previous = now;
                        result = func.apply(context, args);
                    } else if (!timeout) {
                        timeout = setTimeout(later, remaining);
                    }
                    return result;
                };
            },
            noop: function() {}
        };
    }();
    var VERSION = "0.10.5";
    var tokenizers = function() {
        "use strict";
        return {
            nonword: nonword,
            whitespace: whitespace,
            obj: {
                nonword: getObjTokenizer(nonword),
                whitespace: getObjTokenizer(whitespace)
            }
        };
        function whitespace(str) {
            str = _.toStr(str);
            return str ? str.split(/\s+/) : [];
        }
        function nonword(str) {
            str = _.toStr(str);
            return str ? str.split(/\W+/) : [];
        }
        function getObjTokenizer(tokenizer) {
            return function setKey() {
                var args = [].slice.call(arguments, 0);
                return function tokenize(o) {
                    var tokens = [];
                    _.each(args, function(k) {
                        tokens = tokens.concat(tokenizer(_.toStr(o[k])));
                    });
                    return tokens;
                };
            };
        }
    }();
    var LruCache = function() {
        "use strict";
        function LruCache(maxSize) {
            this.maxSize = _.isNumber(maxSize) ? maxSize : 100;
            this.reset();
            if (this.maxSize <= 0) {
                this.set = this.get = $.noop;
            }
        }
        _.mixin(LruCache.prototype, {
            set: function set(key, val) {
                var tailItem = this.list.tail, node;
                if (this.size >= this.maxSize) {
                    this.list.remove(tailItem);
                    delete this.hash[tailItem.key];
                }
                if (node = this.hash[key]) {
                    node.val = val;
                    this.list.moveToFront(node);
                } else {
                    node = new Node(key, val);
                    this.list.add(node);
                    this.hash[key] = node;
                    this.size++;
                }
            },
            get: function get(key) {
                var node = this.hash[key];
                if (node) {
                    this.list.moveToFront(node);
                    return node.val;
                }
            },
            reset: function reset() {
                this.size = 0;
                this.hash = {};
                this.list = new List();
            }
        });
        function List() {
            this.head = this.tail = null;
        }
        _.mixin(List.prototype, {
            add: function add(node) {
                if (this.head) {
                    node.next = this.head;
                    this.head.prev = node;
                }
                this.head = node;
                this.tail = this.tail || node;
            },
            remove: function remove(node) {
                node.prev ? node.prev.next = node.next : this.head = node.next;
                node.next ? node.next.prev = node.prev : this.tail = node.prev;
            },
            moveToFront: function(node) {
                this.remove(node);
                this.add(node);
            }
        });
        function Node(key, val) {
            this.key = key;
            this.val = val;
            this.prev = this.next = null;
        }
        return LruCache;
    }();
    var PersistentStorage = function() {
        "use strict";
        var ls, methods;
        try {
            ls = window.localStorage;
            ls.setItem("~~~", "!");
            ls.removeItem("~~~");
        } catch (err) {
            ls = null;
        }
        function PersistentStorage(namespace) {
            this.prefix = [ "__", namespace, "__" ].join("");
            this.ttlKey = "__ttl__";
            this.keyMatcher = new RegExp("^" + _.escapeRegExChars(this.prefix));
        }
        if (ls && window.JSON) {
            methods = {
                _prefix: function(key) {
                    return this.prefix + key;
                },
                _ttlKey: function(key) {
                    return this._prefix(key) + this.ttlKey;
                },
                get: function(key) {
                    if (this.isExpired(key)) {
                        this.remove(key);
                    }
                    return decode(ls.getItem(this._prefix(key)));
                },
                set: function(key, val, ttl) {
                    if (_.isNumber(ttl)) {
                        ls.setItem(this._ttlKey(key), encode(now() + ttl));
                    } else {
                        ls.removeItem(this._ttlKey(key));
                    }
                    return ls.setItem(this._prefix(key), encode(val));
                },
                remove: function(key) {
                    ls.removeItem(this._ttlKey(key));
                    ls.removeItem(this._prefix(key));
                    return this;
                },
                clear: function() {
                    var i, key, keys = [], len = ls.length;
                    for (i = 0; i < len; i++) {
                        if ((key = ls.key(i)).match(this.keyMatcher)) {
                            keys.push(key.replace(this.keyMatcher, ""));
                        }
                    }
                    for (i = keys.length; i--; ) {
                        this.remove(keys[i]);
                    }
                    return this;
                },
                isExpired: function(key) {
                    var ttl = decode(ls.getItem(this._ttlKey(key)));
                    return _.isNumber(ttl) && now() > ttl ? true : false;
                }
            };
        } else {
            methods = {
                get: _.noop,
                set: _.noop,
                remove: _.noop,
                clear: _.noop,
                isExpired: _.noop
            };
        }
        _.mixin(PersistentStorage.prototype, methods);
        return PersistentStorage;
        function now() {
            return new Date().getTime();
        }
        function encode(val) {
            return JSON.stringify(_.isUndefined(val) ? null : val);
        }
        function decode(val) {
            return JSON.parse(val);
        }
    }();
    var Transport = function() {
        "use strict";
        var pendingRequestsCount = 0, pendingRequests = {}, maxPendingRequests = 6, sharedCache = new LruCache(10);
        function Transport(o) {
            o = o || {};
            this.cancelled = false;
            this.lastUrl = null;
            this._send = o.transport ? callbackToDeferred(o.transport) : $.ajax;
            this._get = o.rateLimiter ? o.rateLimiter(this._get) : this._get;
            this._cache = o.cache === false ? new LruCache(0) : sharedCache;
        }
        Transport.setMaxPendingRequests = function setMaxPendingRequests(num) {
            maxPendingRequests = num;
        };
        Transport.resetCache = function resetCache() {
            sharedCache.reset();
        };
        _.mixin(Transport.prototype, {
            _get: function(url, o, cb) {
                var that = this, jqXhr;
                if (this.cancelled || url !== this.lastUrl) {
                    return;
                }
                if (jqXhr = pendingRequests[url]) {
                    jqXhr.done(done).fail(fail);
                } else if (pendingRequestsCount < maxPendingRequests) {
                    pendingRequestsCount++;
                    pendingRequests[url] = this._send(url, o).done(done).fail(fail).always(always);
                } else {
                    this.onDeckRequestArgs = [].slice.call(arguments, 0);
                }
                function done(resp) {
                    cb && cb(null, resp);
                    that._cache.set(url, resp);
                }
                function fail() {
                    cb && cb(true);
                }
                function always() {
                    pendingRequestsCount--;
                    delete pendingRequests[url];
                    if (that.onDeckRequestArgs) {
                        that._get.apply(that, that.onDeckRequestArgs);
                        that.onDeckRequestArgs = null;
                    }
                }
            },
            get: function(url, o, cb) {
                var resp;
                if (_.isFunction(o)) {
                    cb = o;
                    o = {};
                }
                this.cancelled = false;
                this.lastUrl = url;
                if (resp = this._cache.get(url)) {
                    _.defer(function() {
                        cb && cb(null, resp);
                    });
                } else {
                    this._get(url, o, cb);
                }
                return !!resp;
            },
            cancel: function() {
                this.cancelled = true;
            }
        });
        return Transport;
        function callbackToDeferred(fn) {
            return function customSendWrapper(url, o) {
                var deferred = $.Deferred();
                fn(url, o, onSuccess, onError);
                return deferred;
                function onSuccess(resp) {
                    _.defer(function() {
                        deferred.resolve(resp);
                    });
                }
                function onError(err) {
                    _.defer(function() {
                        deferred.reject(err);
                    });
                }
            };
        }
    }();
    var SearchIndex = function() {
        "use strict";
        function SearchIndex(o) {
            o = o || {};
            if (!o.datumTokenizer || !o.queryTokenizer) {
                $.error("datumTokenizer and queryTokenizer are both required");
            }
            this.datumTokenizer = o.datumTokenizer;
            this.queryTokenizer = o.queryTokenizer;
            this.reset();
        }
        _.mixin(SearchIndex.prototype, {
            bootstrap: function bootstrap(o) {
                this.datums = o.datums;
                this.trie = o.trie;
            },
            add: function(data) {
                var that = this;
                data = _.isArray(data) ? data : [ data ];
                _.each(data, function(datum) {
                    var id, tokens;
                    id = that.datums.push(datum) - 1;
                    tokens = normalizeTokens(that.datumTokenizer(datum));
                    _.each(tokens, function(token) {
                        var node, chars, ch;
                        node = that.trie;
                        chars = token.split("");
                        while (ch = chars.shift()) {
                            node = node.children[ch] || (node.children[ch] = newNode());
                            node.ids.push(id);
                        }
                    });
                });
            },
            get: function get(query) {
                var that = this, tokens, matches;
                tokens = normalizeTokens(this.queryTokenizer(query));
                _.each(tokens, function(token) {
                    var node, chars, ch, ids;
                    if (matches && matches.length === 0) {
                        return false;
                    }
                    node = that.trie;
                    chars = token.split("");
                    while (node && (ch = chars.shift())) {
                        node = node.children[ch];
                    }
                    if (node && chars.length === 0) {
                        ids = node.ids.slice(0);
                        matches = matches ? getIntersection(matches, ids) : ids;
                    } else {
                        matches = [];
                        return false;
                    }
                });
                return matches ? _.map(unique(matches), function(id) {
                    return that.datums[id];
                }) : [];
            },
            reset: function reset() {
                this.datums = [];
                this.trie = newNode();
            },
            serialize: function serialize() {
                return {
                    datums: this.datums,
                    trie: this.trie
                };
            }
        });
        return SearchIndex;
        function normalizeTokens(tokens) {
            tokens = _.filter(tokens, function(token) {
                return !!token;
            });
            tokens = _.map(tokens, function(token) {
                return token.toLowerCase();
            });
            return tokens;
        }
        function newNode() {
            return {
                ids: [],
                children: {}
            };
        }
        function unique(array) {
            var seen = {}, uniques = [];
            for (var i = 0, len = array.length; i < len; i++) {
                if (!seen[array[i]]) {
                    seen[array[i]] = true;
                    uniques.push(array[i]);
                }
            }
            return uniques;
        }
        function getIntersection(arrayA, arrayB) {
            var ai = 0, bi = 0, intersection = [];
            arrayA = arrayA.sort(compare);
            arrayB = arrayB.sort(compare);
            var lenArrayA = arrayA.length, lenArrayB = arrayB.length;
            while (ai < lenArrayA && bi < lenArrayB) {
                if (arrayA[ai] < arrayB[bi]) {
                    ai++;
                } else if (arrayA[ai] > arrayB[bi]) {
                    bi++;
                } else {
                    intersection.push(arrayA[ai]);
                    ai++;
                    bi++;
                }
            }
            return intersection;
            function compare(a, b) {
                return a - b;
            }
        }
    }();
    var oParser = function() {
        "use strict";
        return {
            local: getLocal,
            prefetch: getPrefetch,
            remote: getRemote
        };
        function getLocal(o) {
            return o.local || null;
        }
        function getPrefetch(o) {
            var prefetch, defaults;
            defaults = {
                url: null,
                thumbprint: "",
                ttl: 24 * 60 * 60 * 1e3,
                filter: null,
                ajax: {}
            };
            if (prefetch = o.prefetch || null) {
                prefetch = _.isString(prefetch) ? {
                    url: prefetch
                } : prefetch;
                prefetch = _.mixin(defaults, prefetch);
                prefetch.thumbprint = VERSION + prefetch.thumbprint;
                prefetch.ajax.type = prefetch.ajax.type || "GET";
                prefetch.ajax.dataType = prefetch.ajax.dataType || "json";
                !prefetch.url && $.error("prefetch requires url to be set");
            }
            return prefetch;
        }
        function getRemote(o) {
            var remote, defaults;
            defaults = {
                url: null,
                cache: true,
                wildcard: "%QUERY",
                replace: null,
                rateLimitBy: "debounce",
                rateLimitWait: 300,
                send: null,
                filter: null,
                ajax: {}
            };
            if (remote = o.remote || null) {
                remote = _.isString(remote) ? {
                    url: remote
                } : remote;
                remote = _.mixin(defaults, remote);
                remote.rateLimiter = /^throttle$/i.test(remote.rateLimitBy) ? byThrottle(remote.rateLimitWait) : byDebounce(remote.rateLimitWait);
                remote.ajax.type = remote.ajax.type || "GET";
                remote.ajax.dataType = remote.ajax.dataType || "json";
                delete remote.rateLimitBy;
                delete remote.rateLimitWait;
                !remote.url && $.error("remote requires url to be set");
            }
            return remote;
            function byDebounce(wait) {
                return function(fn) {
                    return _.debounce(fn, wait);
                };
            }
            function byThrottle(wait) {
                return function(fn) {
                    return _.throttle(fn, wait);
                };
            }
        }
    }();
    (function(root) {
        "use strict";
        var old, keys;
        old = root.Bloodhound;
        keys = {
            data: "data",
            protocol: "protocol",
            thumbprint: "thumbprint"
        };
        root.Bloodhound = Bloodhound;
        function Bloodhound(o) {
            if (!o || !o.local && !o.prefetch && !o.remote) {
                $.error("one of local, prefetch, or remote is required");
            }
            this.limit = o.limit || 5;
            this.sorter = getSorter(o.sorter);
            this.dupDetector = o.dupDetector || ignoreDuplicates;
            this.local = oParser.local(o);
            this.prefetch = oParser.prefetch(o);
            this.remote = oParser.remote(o);
            this.cacheKey = this.prefetch ? this.prefetch.cacheKey || this.prefetch.url : null;
            this.index = new SearchIndex({
                datumTokenizer: o.datumTokenizer,
                queryTokenizer: o.queryTokenizer
            });
            this.storage = this.cacheKey ? new PersistentStorage(this.cacheKey) : null;
        }
        Bloodhound.noConflict = function noConflict() {
            root.Bloodhound = old;
            return Bloodhound;
        };
        Bloodhound.tokenizers = tokenizers;
        _.mixin(Bloodhound.prototype, {
            _loadPrefetch: function loadPrefetch(o) {
                var that = this, serialized, deferred;
                if (serialized = this._readFromStorage(o.thumbprint)) {
                    this.index.bootstrap(serialized);
                    deferred = $.Deferred().resolve();
                } else {
                    deferred = $.ajax(o.url, o.ajax).done(handlePrefetchResponse);
                }
                return deferred;
                function handlePrefetchResponse(resp) {
                    that.clear();
                    that.add(o.filter ? o.filter(resp) : resp);
                    that._saveToStorage(that.index.serialize(), o.thumbprint, o.ttl);
                }
            },
            _getFromRemote: function getFromRemote(query, cb) {
                var that = this, url, uriEncodedQuery;
                if (!this.transport) {
                    return;
                }
                query = query || "";
                uriEncodedQuery = encodeURIComponent(query);
                url = this.remote.replace ? this.remote.replace(this.remote.url, query) : this.remote.url.replace(this.remote.wildcard, uriEncodedQuery);
                return this.transport.get(url, this.remote.ajax, handleRemoteResponse);
                function handleRemoteResponse(err, resp) {
                    err ? cb([]) : cb(that.remote.filter ? that.remote.filter(resp) : resp);
                }
            },
            _cancelLastRemoteRequest: function cancelLastRemoteRequest() {
                this.transport && this.transport.cancel();
            },
            _saveToStorage: function saveToStorage(data, thumbprint, ttl) {
                if (this.storage) {
                    this.storage.set(keys.data, data, ttl);
                    this.storage.set(keys.protocol, location.protocol, ttl);
                    this.storage.set(keys.thumbprint, thumbprint, ttl);
                }
            },
            _readFromStorage: function readFromStorage(thumbprint) {
                var stored = {}, isExpired;
                if (this.storage) {
                    stored.data = this.storage.get(keys.data);
                    stored.protocol = this.storage.get(keys.protocol);
                    stored.thumbprint = this.storage.get(keys.thumbprint);
                }
                isExpired = stored.thumbprint !== thumbprint || stored.protocol !== location.protocol;
                return stored.data && !isExpired ? stored.data : null;
            },
            _initialize: function initialize() {
                var that = this, local = this.local, deferred;
                deferred = this.prefetch ? this._loadPrefetch(this.prefetch) : $.Deferred().resolve();
                local && deferred.done(addLocalToIndex);
                this.transport = this.remote ? new Transport(this.remote) : null;
                return this.initPromise = deferred.promise();
                function addLocalToIndex() {
                    that.add(_.isFunction(local) ? local() : local);
                }
            },
            initialize: function initialize(force) {
                return !this.initPromise || force ? this._initialize() : this.initPromise;
            },
            add: function add(data) {
                this.index.add(data);
            },
            get: function get(query, cb) {
                var that = this, matches = [], cacheHit = false;
                matches = this.index.get(query);
                matches = this.sorter(matches).slice(0, this.limit);
                matches.length < this.limit ? cacheHit = this._getFromRemote(query, returnRemoteMatches) : this._cancelLastRemoteRequest();
                if (!cacheHit) {
                    (matches.length > 0 || !this.transport) && cb && cb(matches);
                }
                function returnRemoteMatches(remoteMatches) {
                    var matchesWithBackfill = matches.slice(0);
                    _.each(remoteMatches, function(remoteMatch) {
                        var isDuplicate;
                        isDuplicate = _.some(matchesWithBackfill, function(match) {
                            return that.dupDetector(remoteMatch, match);
                        });
                        !isDuplicate && matchesWithBackfill.push(remoteMatch);
                        return matchesWithBackfill.length < that.limit;
                    });
                    cb && cb(that.sorter(matchesWithBackfill));
                }
            },
            clear: function clear() {
                this.index.reset();
            },
            clearPrefetchCache: function clearPrefetchCache() {
                this.storage && this.storage.clear();
            },
            clearRemoteCache: function clearRemoteCache() {
                this.transport && Transport.resetCache();
            },
            ttAdapter: function ttAdapter() {
                return _.bind(this.get, this);
            }
        });
        return Bloodhound;
        function getSorter(sortFn) {
            return _.isFunction(sortFn) ? sort : noSort;
            function sort(array) {
                return array.sort(sortFn);
            }
            function noSort(array) {
                return array;
            }
        }
        function ignoreDuplicates() {
            return false;
        }
    })(this);
    var html = function() {
        return {
            wrapper: '<span class="twitter-typeahead"></span>',
            dropdown: '<span class="tt-dropdown-menu"></span>',
            dataset: '<div class="tt-dataset-%CLASS%"></div>',
            suggestions: '<span class="tt-suggestions"></span>',
            suggestion: '<div class="tt-suggestion"></div>'
        };
    }();
    var css = function() {
        "use strict";
        var css = {
            wrapper: {
                position: "relative",
                display: "inline-block"
            },
            hint: {
                position: "absolute",
                top: "0",
                left: "0",
                borderColor: "transparent",
                boxShadow: "none",
                opacity: "1"
            },
            input: {
                position: "relative",
                verticalAlign: "top",
                backgroundColor: "transparent"
            },
            inputWithNoHint: {
                position: "relative",
                verticalAlign: "top"
            },
            dropdown: {
                position: "absolute",
                top: "100%",
                left: "0",
                zIndex: "100",
                display: "none"
            },
            suggestions: {
                display: "block"
            },
            suggestion: {
                whiteSpace: "nowrap",
                cursor: "pointer"
            },
            suggestionChild: {
                whiteSpace: "normal"
            },
            ltr: {
                left: "0",
                right: "auto"
            },
            rtl: {
                left: "auto",
                right: " 0"
            }
        };
        if (_.isMsie()) {
            _.mixin(css.input, {
                backgroundImage: "url(data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7)"
            });
        }
        if (_.isMsie() && _.isMsie() <= 7) {
            _.mixin(css.input, {
                marginTop: "-1px"
            });
        }
        return css;
    }();
    var EventBus = function() {
        "use strict";
        var namespace = "typeahead:";
        function EventBus(o) {
            if (!o || !o.el) {
                $.error("EventBus initialized without el");
            }
            this.$el = $(o.el);
        }
        _.mixin(EventBus.prototype, {
            trigger: function(type) {
                var args = [].slice.call(arguments, 1);
                this.$el.trigger(namespace + type, args);
            }
        });
        return EventBus;
    }();
    var EventEmitter = function() {
        "use strict";
        var splitter = /\s+/, nextTick = getNextTick();
        return {
            onSync: onSync,
            onAsync: onAsync,
            off: off,
            trigger: trigger
        };
        function on(method, types, cb, context) {
            var type;
            if (!cb) {
                return this;
            }
            types = types.split(splitter);
            cb = context ? bindContext(cb, context) : cb;
            this._callbacks = this._callbacks || {};
            while (type = types.shift()) {
                this._callbacks[type] = this._callbacks[type] || {
                    sync: [],
                    async: []
                };
                this._callbacks[type][method].push(cb);
            }
            return this;
        }
        function onAsync(types, cb, context) {
            return on.call(this, "async", types, cb, context);
        }
        function onSync(types, cb, context) {
            return on.call(this, "sync", types, cb, context);
        }
        function off(types) {
            var type;
            if (!this._callbacks) {
                return this;
            }
            types = types.split(splitter);
            while (type = types.shift()) {
                delete this._callbacks[type];
            }
            return this;
        }
        function trigger(types) {
            var type, callbacks, args, syncFlush, asyncFlush;
            if (!this._callbacks) {
                return this;
            }
            types = types.split(splitter);
            args = [].slice.call(arguments, 1);
            while ((type = types.shift()) && (callbacks = this._callbacks[type])) {
                syncFlush = getFlush(callbacks.sync, this, [ type ].concat(args));
                asyncFlush = getFlush(callbacks.async, this, [ type ].concat(args));
                syncFlush() && nextTick(asyncFlush);
            }
            return this;
        }
        function getFlush(callbacks, context, args) {
            return flush;
            function flush() {
                var cancelled;
                for (var i = 0, len = callbacks.length; !cancelled && i < len; i += 1) {
                    cancelled = callbacks[i].apply(context, args) === false;
                }
                return !cancelled;
            }
        }
        function getNextTick() {
            var nextTickFn;
            if (window.setImmediate) {
                nextTickFn = function nextTickSetImmediate(fn) {
                    setImmediate(function() {
                        fn();
                    });
                };
            } else {
                nextTickFn = function nextTickSetTimeout(fn) {
                    setTimeout(function() {
                        fn();
                    }, 0);
                };
            }
            return nextTickFn;
        }
        function bindContext(fn, context) {
            return fn.bind ? fn.bind(context) : function() {
                fn.apply(context, [].slice.call(arguments, 0));
            };
        }
    }();
    var highlight = function(doc) {
        "use strict";
        var defaults = {
            node: null,
            pattern: null,
            tagName: "strong",
            className: null,
            wordsOnly: false,
            caseSensitive: false
        };
        return function hightlight(o) {
            var regex;
            o = _.mixin({}, defaults, o);
            if (!o.node || !o.pattern) {
                return;
            }
            o.pattern = _.isArray(o.pattern) ? o.pattern : [ o.pattern ];
            regex = getRegex(o.pattern, o.caseSensitive, o.wordsOnly);
            traverse(o.node, hightlightTextNode);
            function hightlightTextNode(textNode) {
                var match, patternNode, wrapperNode;
                if (match = regex.exec(textNode.data)) {
                    wrapperNode = doc.createElement(o.tagName);
                    o.className && (wrapperNode.className = o.className);
                    patternNode = textNode.splitText(match.index);
                    patternNode.splitText(match[0].length);
                    wrapperNode.appendChild(patternNode.cloneNode(true));
                    textNode.parentNode.replaceChild(wrapperNode, patternNode);
                }
                return !!match;
            }
            function traverse(el, hightlightTextNode) {
                var childNode, TEXT_NODE_TYPE = 3;
                for (var i = 0; i < el.childNodes.length; i++) {
                    childNode = el.childNodes[i];
                    if (childNode.nodeType === TEXT_NODE_TYPE) {
                        i += hightlightTextNode(childNode) ? 1 : 0;
                    } else {
                        traverse(childNode, hightlightTextNode);
                    }
                }
            }
        };
        function getRegex(patterns, caseSensitive, wordsOnly) {
            var escapedPatterns = [], regexStr;
            for (var i = 0, len = patterns.length; i < len; i++) {
                escapedPatterns.push(_.escapeRegExChars(patterns[i]));
            }
            regexStr = wordsOnly ? "\\b(" + escapedPatterns.join("|") + ")\\b" : "(" + escapedPatterns.join("|") + ")";
            return caseSensitive ? new RegExp(regexStr) : new RegExp(regexStr, "i");
        }
    }(window.document);
    var Input = function() {
        "use strict";
        var specialKeyCodeMap;
        specialKeyCodeMap = {
            9: "tab",
            27: "esc",
            37: "left",
            39: "right",
            13: "enter",
            38: "up",
            40: "down"
        };
        function Input(o) {
            var that = this, onBlur, onFocus, onKeydown, onInput;
            o = o || {};
            if (!o.input) {
                $.error("input is missing");
            }
            onBlur = _.bind(this._onBlur, this);
            onFocus = _.bind(this._onFocus, this);
            onKeydown = _.bind(this._onKeydown, this);
            onInput = _.bind(this._onInput, this);
            this.$hint = $(o.hint);
            this.$input = $(o.input).on("blur.tt", onBlur).on("focus.tt", onFocus).on("keydown.tt", onKeydown);
            if (this.$hint.length === 0) {
                this.setHint = this.getHint = this.clearHint = this.clearHintIfInvalid = _.noop;
            }
            if (!_.isMsie()) {
                this.$input.on("input.tt", onInput);
            } else {
                this.$input.on("keydown.tt keypress.tt cut.tt paste.tt", function($e) {
                    if (specialKeyCodeMap[$e.which || $e.keyCode]) {
                        return;
                    }
                    _.defer(_.bind(that._onInput, that, $e));
                });
            }
            this.query = this.$input.val();
            this.$overflowHelper = buildOverflowHelper(this.$input);
        }
        Input.normalizeQuery = function(str) {
            return (str || "").replace(/^\s*/g, "").replace(/\s{2,}/g, " ");
        };
        _.mixin(Input.prototype, EventEmitter, {
            _onBlur: function onBlur() {
                this.resetInputValue();
                this.trigger("blurred");
            },
            _onFocus: function onFocus() {
                this.trigger("focused");
            },
            _onKeydown: function onKeydown($e) {
                var keyName = specialKeyCodeMap[$e.which || $e.keyCode];
                this._managePreventDefault(keyName, $e);
                if (keyName && this._shouldTrigger(keyName, $e)) {
                    this.trigger(keyName + "Keyed", $e);
                }
            },
            _onInput: function onInput() {
                this._checkInputValue();
            },
            _managePreventDefault: function managePreventDefault(keyName, $e) {
                var preventDefault, hintValue, inputValue;
                switch (keyName) {
                  case "tab":
                    hintValue = this.getHint();
                    inputValue = this.getInputValue();
                    preventDefault = hintValue && hintValue !== inputValue && !withModifier($e);
                    break;

                  case "up":
                  case "down":
                    preventDefault = !withModifier($e);
                    break;

                  default:
                    preventDefault = false;
                }
                preventDefault && $e.preventDefault();
            },
            _shouldTrigger: function shouldTrigger(keyName, $e) {
                var trigger;
                switch (keyName) {
                  case "tab":
                    trigger = !withModifier($e);
                    break;

                  default:
                    trigger = true;
                }
                return trigger;
            },
            _checkInputValue: function checkInputValue() {
                var inputValue, areEquivalent, hasDifferentWhitespace;
                inputValue = this.getInputValue();
                areEquivalent = areQueriesEquivalent(inputValue, this.query);
                hasDifferentWhitespace = areEquivalent ? this.query.length !== inputValue.length : false;
                this.query = inputValue;
                if (!areEquivalent) {
                    this.trigger("queryChanged", this.query);
                } else if (hasDifferentWhitespace) {
                    this.trigger("whitespaceChanged", this.query);
                }
            },
            focus: function focus() {
                this.$input.focus();
            },
            blur: function blur() {
                this.$input.blur();
            },
            getQuery: function getQuery() {
                return this.query;
            },
            setQuery: function setQuery(query) {
                this.query = query;
            },
            getInputValue: function getInputValue() {
                return this.$input.val();
            },
            setInputValue: function setInputValue(value, silent) {
                this.$input.val(value);
                silent ? this.clearHint() : this._checkInputValue();
            },
            resetInputValue: function resetInputValue() {
                this.setInputValue(this.query, true);
            },
            getHint: function getHint() {
                return this.$hint.val();
            },
            setHint: function setHint(value) {
                this.$hint.val(value);
            },
            clearHint: function clearHint() {
                this.setHint("");
            },
            clearHintIfInvalid: function clearHintIfInvalid() {
                var val, hint, valIsPrefixOfHint, isValid;
                val = this.getInputValue();
                hint = this.getHint();
                valIsPrefixOfHint = val !== hint && hint.indexOf(val) === 0;
                isValid = val !== "" && valIsPrefixOfHint && !this.hasOverflow();
                !isValid && this.clearHint();
            },
            getLanguageDirection: function getLanguageDirection() {
                return (this.$input.css("direction") || "ltr").toLowerCase();
            },
            hasOverflow: function hasOverflow() {
                var constraint = this.$input.width() - 2;
                this.$overflowHelper.text(this.getInputValue());
                return this.$overflowHelper.width() >= constraint;
            },
            isCursorAtEnd: function() {
                var valueLength, selectionStart, range;
                valueLength = this.$input.val().length;
                selectionStart = this.$input[0].selectionStart;
                if (_.isNumber(selectionStart)) {
                    return selectionStart === valueLength;
                } else if (document.selection) {
                    range = document.selection.createRange();
                    range.moveStart("character", -valueLength);
                    return valueLength === range.text.length;
                }
                return true;
            },
            destroy: function destroy() {
                this.$hint.off(".tt");
                this.$input.off(".tt");
                this.$hint = this.$input = this.$overflowHelper = null;
            }
        });
        return Input;
        function buildOverflowHelper($input) {
            return $('<pre aria-hidden="true"></pre>').css({
                position: "absolute",
                visibility: "hidden",
                whiteSpace: "pre",
                fontFamily: $input.css("font-family"),
                fontSize: $input.css("font-size"),
                fontStyle: $input.css("font-style"),
                fontVariant: $input.css("font-variant"),
                fontWeight: $input.css("font-weight"),
                wordSpacing: $input.css("word-spacing"),
                letterSpacing: $input.css("letter-spacing"),
                textIndent: $input.css("text-indent"),
                textRendering: $input.css("text-rendering"),
                textTransform: $input.css("text-transform")
            }).insertAfter($input);
        }
        function areQueriesEquivalent(a, b) {
            return Input.normalizeQuery(a) === Input.normalizeQuery(b);
        }
        function withModifier($e) {
            return $e.altKey || $e.ctrlKey || $e.metaKey || $e.shiftKey;
        }
    }();
    var Dataset = function() {
        "use strict";
        var datasetKey = "ttDataset", valueKey = "ttValue", datumKey = "ttDatum";
        function Dataset(o) {
            o = o || {};
            o.templates = o.templates || {};
            if (!o.source) {
                $.error("missing source");
            }
            if (o.name && !isValidName(o.name)) {
                $.error("invalid dataset name: " + o.name);
            }
            this.query = null;
            this.highlight = !!o.highlight;
            this.name = o.name || _.getUniqueId();
            this.source = o.source;
            this.displayFn = getDisplayFn(o.display || o.displayKey);
            this.templates = getTemplates(o.templates, this.displayFn);
            this.$el = $(html.dataset.replace("%CLASS%", this.name));
        }
        Dataset.extractDatasetName = function extractDatasetName(el) {
            return $(el).data(datasetKey);
        };
        Dataset.extractValue = function extractDatum(el) {
            return $(el).data(valueKey);
        };
        Dataset.extractDatum = function extractDatum(el) {
            return $(el).data(datumKey);
        };
        _.mixin(Dataset.prototype, EventEmitter, {
            _render: function render(query, suggestions) {
                if (!this.$el) {
                    return;
                }
                var that = this, hasSuggestions;
                this.$el.empty();
                hasSuggestions = suggestions && suggestions.length;
                if (!hasSuggestions && this.templates.empty) {
                    this.$el.html(getEmptyHtml()).prepend(that.templates.header ? getHeaderHtml() : null).append(that.templates.footer ? getFooterHtml() : null);
                } else if (hasSuggestions) {
                    this.$el.html(getSuggestionsHtml()).prepend(that.templates.header ? getHeaderHtml() : null).append(that.templates.footer ? getFooterHtml() : null);
                }
                this.trigger("rendered");
                function getEmptyHtml() {
                    return that.templates.empty({
                        query: query,
                        isEmpty: true
                    });
                }
                function getSuggestionsHtml() {
                    var $suggestions, nodes;
                    $suggestions = $(html.suggestions).css(css.suggestions);
                    nodes = _.map(suggestions, getSuggestionNode);
                    $suggestions.append.apply($suggestions, nodes);
                    that.highlight && highlight({
                        className: "tt-highlight",
                        node: $suggestions[0],
                        pattern: query
                    });
                    return $suggestions;
                    function getSuggestionNode(suggestion) {
                        var $el;
                        $el = $(html.suggestion).append(that.templates.suggestion(suggestion)).data(datasetKey, that.name).data(valueKey, that.displayFn(suggestion)).data(datumKey, suggestion);
                        $el.children().each(function() {
                            $(this).css(css.suggestionChild);
                        });
                        return $el;
                    }
                }
                function getHeaderHtml() {
                    return that.templates.header({
                        query: query,
                        isEmpty: !hasSuggestions
                    });
                }
                function getFooterHtml() {
                    return that.templates.footer({
                        query: query,
                        isEmpty: !hasSuggestions
                    });
                }
            },
            getRoot: function getRoot() {
                return this.$el;
            },
            update: function update(query) {
                var that = this;
                this.query = query;
                this.canceled = false;
                this.source(query, render);
                function render(suggestions) {
                    if (!that.canceled && query === that.query) {
                        that._render(query, suggestions);
                    }
                }
            },
            cancel: function cancel() {
                this.canceled = true;
            },
            clear: function clear() {
                this.cancel();
                this.$el.empty();
                this.trigger("rendered");
            },
            isEmpty: function isEmpty() {
                return this.$el.is(":empty");
            },
            destroy: function destroy() {
                this.$el = null;
            }
        });
        return Dataset;
        function getDisplayFn(display) {
            display = display || "value";
            return _.isFunction(display) ? display : displayFn;
            function displayFn(obj) {
                return obj[display];
            }
        }
        function getTemplates(templates, displayFn) {
            return {
                empty: templates.empty && _.templatify(templates.empty),
                header: templates.header && _.templatify(templates.header),
                footer: templates.footer && _.templatify(templates.footer),
                suggestion: templates.suggestion || suggestionTemplate
            };
            function suggestionTemplate(context) {
                return "<p>" + displayFn(context) + "</p>";
            }
        }
        function isValidName(str) {
            return /^[_a-zA-Z0-9-]+$/.test(str);
        }
    }();
    var Dropdown = function() {
        "use strict";
        function Dropdown(o) {
            var that = this, onSuggestionClick, onSuggestionMouseEnter, onSuggestionMouseLeave;
            o = o || {};
            if (!o.menu) {
                $.error("menu is required");
            }
            this.isOpen = false;
            this.isEmpty = true;
            this.datasets = _.map(o.datasets, initializeDataset);
            onSuggestionClick = _.bind(this._onSuggestionClick, this);
            onSuggestionMouseEnter = _.bind(this._onSuggestionMouseEnter, this);
            onSuggestionMouseLeave = _.bind(this._onSuggestionMouseLeave, this);
            this.$menu = $(o.menu).on("click.tt", ".tt-suggestion", onSuggestionClick).on("mouseenter.tt", ".tt-suggestion", onSuggestionMouseEnter).on("mouseleave.tt", ".tt-suggestion", onSuggestionMouseLeave);
            _.each(this.datasets, function(dataset) {
                that.$menu.append(dataset.getRoot());
                dataset.onSync("rendered", that._onRendered, that);
            });
        }
        _.mixin(Dropdown.prototype, EventEmitter, {
            _onSuggestionClick: function onSuggestionClick($e) {
                this.trigger("suggestionClicked", $($e.currentTarget));
            },
            _onSuggestionMouseEnter: function onSuggestionMouseEnter($e) {
                this._removeCursor();
                this._setCursor($($e.currentTarget), true);
            },
            _onSuggestionMouseLeave: function onSuggestionMouseLeave() {
                this._removeCursor();
            },
            _onRendered: function onRendered() {
                this.isEmpty = _.every(this.datasets, isDatasetEmpty);
                this.isEmpty ? this._hide() : this.isOpen && this._show();
                this.trigger("datasetRendered");
                function isDatasetEmpty(dataset) {
                    return dataset.isEmpty();
                }
            },
            _hide: function() {
                this.$menu.hide();
            },
            _show: function() {
                this.$menu.css("display", "block");
            },
            _getSuggestions: function getSuggestions() {
                return this.$menu.find(".tt-suggestion");
            },
            _getCursor: function getCursor() {
                return this.$menu.find(".tt-cursor").first();
            },
            _setCursor: function setCursor($el, silent) {
                $el.first().addClass("tt-cursor");
                !silent && this.trigger("cursorMoved");
            },
            _removeCursor: function removeCursor() {
                this._getCursor().removeClass("tt-cursor");
            },
            _moveCursor: function moveCursor(increment) {
                var $suggestions, $oldCursor, newCursorIndex, $newCursor;
                if (!this.isOpen) {
                    return;
                }
                $oldCursor = this._getCursor();
                $suggestions = this._getSuggestions();
                this._removeCursor();
                newCursorIndex = $suggestions.index($oldCursor) + increment;
                newCursorIndex = (newCursorIndex + 1) % ($suggestions.length + 1) - 1;
                if (newCursorIndex === -1) {
                    this.trigger("cursorRemoved");
                    return;
                } else if (newCursorIndex < -1) {
                    newCursorIndex = $suggestions.length - 1;
                }
                this._setCursor($newCursor = $suggestions.eq(newCursorIndex));
                this._ensureVisible($newCursor);
            },
            _ensureVisible: function ensureVisible($el) {
                var elTop, elBottom, menuScrollTop, menuHeight;
                elTop = $el.position().top;
                elBottom = elTop + $el.outerHeight(true);
                menuScrollTop = this.$menu.scrollTop();
                menuHeight = this.$menu.height() + parseInt(this.$menu.css("paddingTop"), 10) + parseInt(this.$menu.css("paddingBottom"), 10);
                if (elTop < 0) {
                    this.$menu.scrollTop(menuScrollTop + elTop);
                } else if (menuHeight < elBottom) {
                    this.$menu.scrollTop(menuScrollTop + (elBottom - menuHeight));
                }
            },
            close: function close() {
                if (this.isOpen) {
                    this.isOpen = false;
                    this._removeCursor();
                    this._hide();
                    this.trigger("closed");
                }
            },
            open: function open() {
                if (!this.isOpen) {
                    this.isOpen = true;
                    !this.isEmpty && this._show();
                    this.trigger("opened");
                }
            },
            setLanguageDirection: function setLanguageDirection(dir) {
                this.$menu.css(dir === "ltr" ? css.ltr : css.rtl);
            },
            moveCursorUp: function moveCursorUp() {
                this._moveCursor(-1);
            },
            moveCursorDown: function moveCursorDown() {
                this._moveCursor(+1);
            },
            getDatumForSuggestion: function getDatumForSuggestion($el) {
                var datum = null;
                if ($el.length) {
                    datum = {
                        raw: Dataset.extractDatum($el),
                        value: Dataset.extractValue($el),
                        datasetName: Dataset.extractDatasetName($el)
                    };
                }
                return datum;
            },
            getDatumForCursor: function getDatumForCursor() {
                return this.getDatumForSuggestion(this._getCursor().first());
            },
            getDatumForTopSuggestion: function getDatumForTopSuggestion() {
                return this.getDatumForSuggestion(this._getSuggestions().first());
            },
            update: function update(query) {
                _.each(this.datasets, updateDataset);
                function updateDataset(dataset) {
                    dataset.update(query);
                }
            },
            empty: function empty() {
                _.each(this.datasets, clearDataset);
                this.isEmpty = true;
                function clearDataset(dataset) {
                    dataset.clear();
                }
            },
            isVisible: function isVisible() {
                return this.isOpen && !this.isEmpty;
            },
            destroy: function destroy() {
                this.$menu.off(".tt");
                this.$menu = null;
                _.each(this.datasets, destroyDataset);
                function destroyDataset(dataset) {
                    dataset.destroy();
                }
            }
        });
        return Dropdown;
        function initializeDataset(oDataset) {
            return new Dataset(oDataset);
        }
    }();
    var Typeahead = function() {
        "use strict";
        var attrsKey = "ttAttrs";
        function Typeahead(o) {
            var $menu, $input, $hint;
            o = o || {};
            if (!o.input) {
                $.error("missing input");
            }
            this.isActivated = false;
            this.autoselect = !!o.autoselect;
            this.minLength = _.isNumber(o.minLength) ? o.minLength : 1;
            this.$node = buildDom(o.input, o.withHint);
            $menu = this.$node.find(".tt-dropdown-menu");
            $input = this.$node.find(".tt-input");
            $hint = this.$node.find(".tt-hint");
            $input.on("blur.tt", function($e) {
                var active, isActive, hasActive;
                active = document.activeElement;
                isActive = $menu.is(active);
                hasActive = $menu.has(active).length > 0;
                if (_.isMsie() && (isActive || hasActive)) {
                    $e.preventDefault();
                    $e.stopImmediatePropagation();
                    _.defer(function() {
                        $input.focus();
                    });
                }
            });
            $menu.on("mousedown.tt", function($e) {
                $e.preventDefault();
            });
            this.eventBus = o.eventBus || new EventBus({
                el: $input
            });
            this.dropdown = new Dropdown({
                menu: $menu,
                datasets: o.datasets
            }).onSync("suggestionClicked", this._onSuggestionClicked, this).onSync("cursorMoved", this._onCursorMoved, this).onSync("cursorRemoved", this._onCursorRemoved, this).onSync("opened", this._onOpened, this).onSync("closed", this._onClosed, this).onAsync("datasetRendered", this._onDatasetRendered, this);
            this.input = new Input({
                input: $input,
                hint: $hint
            }).onSync("focused", this._onFocused, this).onSync("blurred", this._onBlurred, this).onSync("enterKeyed", this._onEnterKeyed, this).onSync("tabKeyed", this._onTabKeyed, this).onSync("escKeyed", this._onEscKeyed, this).onSync("upKeyed", this._onUpKeyed, this).onSync("downKeyed", this._onDownKeyed, this).onSync("leftKeyed", this._onLeftKeyed, this).onSync("rightKeyed", this._onRightKeyed, this).onSync("queryChanged", this._onQueryChanged, this).onSync("whitespaceChanged", this._onWhitespaceChanged, this);
            this._setLanguageDirection();
        }
        _.mixin(Typeahead.prototype, {
            _onSuggestionClicked: function onSuggestionClicked(type, $el) {
                var datum;
                if (datum = this.dropdown.getDatumForSuggestion($el)) {
                    this._select(datum);
                }
            },
            _onCursorMoved: function onCursorMoved() {
                var datum = this.dropdown.getDatumForCursor();
                this.input.setInputValue(datum.value, true);
                this.eventBus.trigger("cursorchanged", datum.raw, datum.datasetName);
            },
            _onCursorRemoved: function onCursorRemoved() {
                this.input.resetInputValue();
                this._updateHint();
            },
            _onDatasetRendered: function onDatasetRendered() {
                this._updateHint();
            },
            _onOpened: function onOpened() {
                this._updateHint();
                this.eventBus.trigger("opened");
            },
            _onClosed: function onClosed() {
                this.input.clearHint();
                this.eventBus.trigger("closed");
            },
            _onFocused: function onFocused() {
                this.isActivated = true;
                this.dropdown.open();
            },
            _onBlurred: function onBlurred() {
                this.isActivated = false;
                this.dropdown.empty();
                this.dropdown.close();
            },
            _onEnterKeyed: function onEnterKeyed(type, $e) {
                var cursorDatum, topSuggestionDatum;
                cursorDatum = this.dropdown.getDatumForCursor();
                topSuggestionDatum = this.dropdown.getDatumForTopSuggestion();
                if (cursorDatum) {
                    this._select(cursorDatum);
                    $e.preventDefault();
                } else if (this.autoselect && topSuggestionDatum) {
                    this._select(topSuggestionDatum);
                    $e.preventDefault();
                }
            },
            _onTabKeyed: function onTabKeyed(type, $e) {
                var datum;
                if (datum = this.dropdown.getDatumForCursor()) {
                    this._select(datum);
                    $e.preventDefault();
                } else {
                    this._autocomplete(true);
                }
            },
            _onEscKeyed: function onEscKeyed() {
                this.dropdown.close();
                this.input.resetInputValue();
            },
            _onUpKeyed: function onUpKeyed() {
                var query = this.input.getQuery();
                this.dropdown.isEmpty && query.length >= this.minLength ? this.dropdown.update(query) : this.dropdown.moveCursorUp();
                this.dropdown.open();
            },
            _onDownKeyed: function onDownKeyed() {
                var query = this.input.getQuery();
                this.dropdown.isEmpty && query.length >= this.minLength ? this.dropdown.update(query) : this.dropdown.moveCursorDown();
                this.dropdown.open();
            },
            _onLeftKeyed: function onLeftKeyed() {
                this.dir === "rtl" && this._autocomplete();
            },
            _onRightKeyed: function onRightKeyed() {
                this.dir === "ltr" && this._autocomplete();
            },
            _onQueryChanged: function onQueryChanged(e, query) {
                this.input.clearHintIfInvalid();
                query.length >= this.minLength ? this.dropdown.update(query) : this.dropdown.empty();
                this.dropdown.open();
                this._setLanguageDirection();
            },
            _onWhitespaceChanged: function onWhitespaceChanged() {
                this._updateHint();
                this.dropdown.open();
            },
            _setLanguageDirection: function setLanguageDirection() {
                var dir;
                if (this.dir !== (dir = this.input.getLanguageDirection())) {
                    this.dir = dir;
                    this.$node.css("direction", dir);
                    this.dropdown.setLanguageDirection(dir);
                }
            },
            _updateHint: function updateHint() {
                var datum, val, query, escapedQuery, frontMatchRegEx, match;
                datum = this.dropdown.getDatumForTopSuggestion();
                if (datum && this.dropdown.isVisible() && !this.input.hasOverflow()) {
                    val = this.input.getInputValue();
                    query = Input.normalizeQuery(val);
                    escapedQuery = _.escapeRegExChars(query);
                    frontMatchRegEx = new RegExp("^(?:" + escapedQuery + ")(.+$)", "i");
                    match = frontMatchRegEx.exec(datum.value);
                    match ? this.input.setHint(val + match[1]) : this.input.clearHint();
                } else {
                    this.input.clearHint();
                }
            },
            _autocomplete: function autocomplete(laxCursor) {
                var hint, query, isCursorAtEnd, datum;
                hint = this.input.getHint();
                query = this.input.getQuery();
                isCursorAtEnd = laxCursor || this.input.isCursorAtEnd();
                if (hint && query !== hint && isCursorAtEnd) {
                    datum = this.dropdown.getDatumForTopSuggestion();
                    datum && this.input.setInputValue(datum.value);
                    this.eventBus.trigger("autocompleted", datum.raw, datum.datasetName);
                }
            },
            _select: function select(datum) {
                this.input.setQuery(datum.value);
                this.input.setInputValue(datum.value, true);
                this._setLanguageDirection();
                this.eventBus.trigger("selected", datum.raw, datum.datasetName);
                this.dropdown.close();
                _.defer(_.bind(this.dropdown.empty, this.dropdown));
            },
            open: function open() {
                this.dropdown.open();
            },
            close: function close() {
                this.dropdown.close();
            },
            setVal: function setVal(val) {
                val = _.toStr(val);
                if (this.isActivated) {
                    this.input.setInputValue(val);
                } else {
                    this.input.setQuery(val);
                    this.input.setInputValue(val, true);
                }
                this._setLanguageDirection();
            },
            getVal: function getVal() {
                return this.input.getQuery();
            },
            destroy: function destroy() {
                this.input.destroy();
                this.dropdown.destroy();
                destroyDomStructure(this.$node);
                this.$node = null;
            }
        });
        return Typeahead;
        function buildDom(input, withHint) {
            var $input, $wrapper, $dropdown, $hint;
            $input = $(input);
            $wrapper = $(html.wrapper).css(css.wrapper);
            $dropdown = $(html.dropdown).css(css.dropdown);
            $hint = $input.clone().css(css.hint).css(getBackgroundStyles($input));
            $hint.val("").removeData().addClass("tt-hint").removeAttr("id name placeholder required").prop("readonly", true).attr({
                autocomplete: "off",
                spellcheck: "false",
                tabindex: -1
            });
            $input.data(attrsKey, {
                dir: $input.attr("dir"),
                autocomplete: $input.attr("autocomplete"),
                spellcheck: $input.attr("spellcheck"),
                style: $input.attr("style")
            });
            $input.addClass("tt-input").attr({
                autocomplete: "off",
                spellcheck: false
            }).css(withHint ? css.input : css.inputWithNoHint);
            try {
                !$input.attr("dir") && $input.attr("dir", "auto");
            } catch (e) {}
            return $input.wrap($wrapper).parent().prepend(withHint ? $hint : null).append($dropdown);
        }
        function getBackgroundStyles($el) {
            return {
                backgroundAttachment: $el.css("background-attachment"),
                backgroundClip: $el.css("background-clip"),
                backgroundColor: $el.css("background-color"),
                backgroundImage: $el.css("background-image"),
                backgroundOrigin: $el.css("background-origin"),
                backgroundPosition: $el.css("background-position"),
                backgroundRepeat: $el.css("background-repeat"),
                backgroundSize: $el.css("background-size")
            };
        }
        function destroyDomStructure($node) {
            var $input = $node.find(".tt-input");
            _.each($input.data(attrsKey), function(val, key) {
                _.isUndefined(val) ? $input.removeAttr(key) : $input.attr(key, val);
            });
            $input.detach().removeData(attrsKey).removeClass("tt-input").insertAfter($node);
            $node.remove();
        }
    }();
    (function() {
        "use strict";
        var old, typeaheadKey, methods;
        old = $.fn.typeahead;
        typeaheadKey = "ttTypeahead";
        methods = {
            initialize: function initialize(o, datasets) {
                datasets = _.isArray(datasets) ? datasets : [].slice.call(arguments, 1);
                o = o || {};
                return this.each(attach);
                function attach() {
                    var $input = $(this), eventBus, typeahead;
                    _.each(datasets, function(d) {
                        d.highlight = !!o.highlight;
                    });
                    typeahead = new Typeahead({
                        input: $input,
                        eventBus: eventBus = new EventBus({
                            el: $input
                        }),
                        withHint: _.isUndefined(o.hint) ? true : !!o.hint,
                        minLength: o.minLength,
                        autoselect: o.autoselect,
                        datasets: datasets
                    });
                    $input.data(typeaheadKey, typeahead);
                }
            },
            open: function open() {
                return this.each(openTypeahead);
                function openTypeahead() {
                    var $input = $(this), typeahead;
                    if (typeahead = $input.data(typeaheadKey)) {
                        typeahead.open();
                    }
                }
            },
            close: function close() {
                return this.each(closeTypeahead);
                function closeTypeahead() {
                    var $input = $(this), typeahead;
                    if (typeahead = $input.data(typeaheadKey)) {
                        typeahead.close();
                    }
                }
            },
            val: function val(newVal) {
                return !arguments.length ? getVal(this.first()) : this.each(setVal);
                function setVal() {
                    var $input = $(this), typeahead;
                    if (typeahead = $input.data(typeaheadKey)) {
                        typeahead.setVal(newVal);
                    }
                }
                function getVal($input) {
                    var typeahead, query;
                    if (typeahead = $input.data(typeaheadKey)) {
                        query = typeahead.getVal();
                    }
                    return query;
                }
            },
            destroy: function destroy() {
                return this.each(unattach);
                function unattach() {
                    var $input = $(this), typeahead;
                    if (typeahead = $input.data(typeaheadKey)) {
                        typeahead.destroy();
                        $input.removeData(typeaheadKey);
                    }
                }
            }
        };
        $.fn.typeahead = function(method) {
            var tts;
            if (methods[method] && method !== "initialize") {
                tts = this.filter(function() {
                    return !!$(this).data(typeaheadKey);
                });
                return methods[method].apply(tts, [].slice.call(arguments, 1));
            } else {
                return methods.initialize.apply(this, arguments);
            }
        };
        $.fn.typeahead.noConflict = function noConflict() {
            $.fn.typeahead = old;
            return this;
        };
    })();
})(window.jQuery);
define("typeahead", function(){});


/*
 *
 * More info at [www.dropzonejs.com](http://www.dropzonejs.com)
 *
 * Copyright (c) 2012, Matias Meno
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

(function() {
  var Dropzone, Emitter, camelize, contentLoaded, detectVerticalSquash, drawImageIOSFix, noop, without,
    __slice = [].slice,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  noop = function() {};

  Emitter = (function() {
    function Emitter() {}

    Emitter.prototype.addEventListener = Emitter.prototype.on;

    Emitter.prototype.on = function(event, fn) {
      this._callbacks = this._callbacks || {};
      if (!this._callbacks[event]) {
        this._callbacks[event] = [];
      }
      this._callbacks[event].push(fn);
      return this;
    };

    Emitter.prototype.emit = function() {
      var args, callback, callbacks, event, _i, _len;
      event = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      this._callbacks = this._callbacks || {};
      callbacks = this._callbacks[event];
      if (callbacks) {
        for (_i = 0, _len = callbacks.length; _i < _len; _i++) {
          callback = callbacks[_i];
          callback.apply(this, args);
        }
      }
      return this;
    };

    Emitter.prototype.removeListener = Emitter.prototype.off;

    Emitter.prototype.removeAllListeners = Emitter.prototype.off;

    Emitter.prototype.removeEventListener = Emitter.prototype.off;

    Emitter.prototype.off = function(event, fn) {
      var callback, callbacks, i, _i, _len;
      if (!this._callbacks || arguments.length === 0) {
        this._callbacks = {};
        return this;
      }
      callbacks = this._callbacks[event];
      if (!callbacks) {
        return this;
      }
      if (arguments.length === 1) {
        delete this._callbacks[event];
        return this;
      }
      for (i = _i = 0, _len = callbacks.length; _i < _len; i = ++_i) {
        callback = callbacks[i];
        if (callback === fn) {
          callbacks.splice(i, 1);
          break;
        }
      }
      return this;
    };

    return Emitter;

  })();

  Dropzone = (function(_super) {
    var extend, resolveOption;

    __extends(Dropzone, _super);

    Dropzone.prototype.Emitter = Emitter;


    /*
    This is a list of all available events you can register on a dropzone object.
    
    You can register an event handler like this:
    
        dropzone.on("dragEnter", function() { });
     */

    Dropzone.prototype.events = ["drop", "dragstart", "dragend", "dragenter", "dragover", "dragleave", "addedfile", "removedfile", "thumbnail", "error", "errormultiple", "processing", "processingmultiple", "uploadprogress", "totaluploadprogress", "sending", "sendingmultiple", "success", "successmultiple", "canceled", "canceledmultiple", "complete", "completemultiple", "reset", "maxfilesexceeded", "maxfilesreached", "queuecomplete"];

    Dropzone.prototype.defaultOptions = {
      url: null,
      method: "post",
      withCredentials: false,
      parallelUploads: 2,
      uploadMultiple: false,
      maxFilesize: 256,
      paramName: "file",
      createImageThumbnails: true,
      maxThumbnailFilesize: 10,
      thumbnailWidth: 120,
      thumbnailHeight: 120,
      filesizeBase: 1000,
      maxFiles: null,
      filesizeBase: 1000,
      params: {},
      clickable: true,
      ignoreHiddenFiles: true,
      acceptedFiles: null,
      acceptedMimeTypes: null,
      autoProcessQueue: true,
      autoQueue: true,
      addRemoveLinks: false,
      previewsContainer: null,
      capture: null,
      dictDefaultMessage: "Drop files here to upload",
      dictFallbackMessage: "Your browser does not support drag'n'drop file uploads.",
      dictFallbackText: "Please use the fallback form below to upload your files like in the olden days.",
      dictFileTooBig: "File is too big ({{filesize}}MiB). Max filesize: {{maxFilesize}}MiB.",
      dictInvalidFileType: "You can't upload files of this type.",
      dictResponseError: "Server responded with {{statusCode}} code.",
      dictCancelUpload: "Cancel upload",
      dictCancelUploadConfirmation: "Are you sure you want to cancel this upload?",
      dictRemoveFile: "Remove file",
      dictRemoveFileConfirmation: null,
      dictMaxFilesExceeded: "You can not upload any more files.",
      accept: function(file, done) {
        return done();
      },
      init: function() {
        return noop;
      },
      forceFallback: false,
      fallback: function() {
        var child, messageElement, span, _i, _len, _ref;
        this.element.className = "" + this.element.className + " dz-browser-not-supported";
        _ref = this.element.getElementsByTagName("div");
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          child = _ref[_i];
          if (/(^| )dz-message($| )/.test(child.className)) {
            messageElement = child;
            child.className = "dz-message";
            continue;
          }
        }
        if (!messageElement) {
          messageElement = Dropzone.createElement("<div class=\"dz-message\"><span></span></div>");
          this.element.appendChild(messageElement);
        }
        span = messageElement.getElementsByTagName("span")[0];
        if (span) {
          span.textContent = this.options.dictFallbackMessage;
        }
        return this.element.appendChild(this.getFallbackForm());
      },
      resize: function(file) {
        var info, srcRatio, trgRatio;
        info = {
          srcX: 0,
          srcY: 0,
          srcWidth: file.width,
          srcHeight: file.height
        };
        srcRatio = file.width / file.height;
        info.optWidth = this.options.thumbnailWidth;
        info.optHeight = this.options.thumbnailHeight;
        if ((info.optWidth == null) && (info.optHeight == null)) {
          info.optWidth = info.srcWidth;
          info.optHeight = info.srcHeight;
        } else if (info.optWidth == null) {
          info.optWidth = srcRatio * info.optHeight;
        } else if (info.optHeight == null) {
          info.optHeight = (1 / srcRatio) * info.optWidth;
        }
        trgRatio = info.optWidth / info.optHeight;
        if (file.height < info.optHeight || file.width < info.optWidth) {
          info.trgHeight = info.srcHeight;
          info.trgWidth = info.srcWidth;
        } else {
          if (srcRatio > trgRatio) {
            info.srcHeight = file.height;
            info.srcWidth = info.srcHeight * trgRatio;
          } else {
            info.srcWidth = file.width;
            info.srcHeight = info.srcWidth / trgRatio;
          }
        }
        info.srcX = (file.width - info.srcWidth) / 2;
        info.srcY = (file.height - info.srcHeight) / 2;
        return info;
      },

      /*
      Those functions register themselves to the events on init and handle all
      the user interface specific stuff. Overwriting them won't break the upload
      but can break the way it's displayed.
      You can overwrite them if you don't like the default behavior. If you just
      want to add an additional event handler, register it on the dropzone object
      and don't overwrite those options.
       */
      drop: function(e) {
        return this.element.classList.remove("dz-drag-hover");
      },
      dragstart: noop,
      dragend: function(e) {
        return this.element.classList.remove("dz-drag-hover");
      },
      dragenter: function(e) {
        return this.element.classList.add("dz-drag-hover");
      },
      dragover: function(e) {
        return this.element.classList.add("dz-drag-hover");
      },
      dragleave: function(e) {
        return this.element.classList.remove("dz-drag-hover");
      },
      paste: noop,
      reset: function() {
        return this.element.classList.remove("dz-started");
      },
      addedfile: function(file) {
        var node, removeFileEvent, removeLink, _i, _j, _k, _len, _len1, _len2, _ref, _ref1, _ref2, _results;
        if (this.element === this.previewsContainer) {
          this.element.classList.add("dz-started");
        }
        if (this.previewsContainer) {
          file.previewElement = Dropzone.createElement(this.options.previewTemplate.trim());
          file.previewTemplate = file.previewElement;
          this.previewsContainer.appendChild(file.previewElement);
          _ref = file.previewElement.querySelectorAll("[data-dz-name]");
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            node = _ref[_i];
            node.textContent = file.name;
          }
          _ref1 = file.previewElement.querySelectorAll("[data-dz-size]");
          for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
            node = _ref1[_j];
            node.innerHTML = this.filesize(file.size);
          }
          if (this.options.addRemoveLinks) {
            file._removeLink = Dropzone.createElement("<a class=\"dz-remove\" href=\"javascript:undefined;\" data-dz-remove>" + this.options.dictRemoveFile + "</a>");
            file.previewElement.appendChild(file._removeLink);
          }
          removeFileEvent = (function(_this) {
            return function(e) {
              e.preventDefault();
              e.stopPropagation();
              if (file.status === Dropzone.UPLOADING) {
                return Dropzone.confirm(_this.options.dictCancelUploadConfirmation, function() {
                  return _this.removeFile(file);
                });
              } else {
                if (_this.options.dictRemoveFileConfirmation) {
                  return Dropzone.confirm(_this.options.dictRemoveFileConfirmation, function() {
                    return _this.removeFile(file);
                  });
                } else {
                  return _this.removeFile(file);
                }
              }
            };
          })(this);
          _ref2 = file.previewElement.querySelectorAll("[data-dz-remove]");
          _results = [];
          for (_k = 0, _len2 = _ref2.length; _k < _len2; _k++) {
            removeLink = _ref2[_k];
            _results.push(removeLink.addEventListener("click", removeFileEvent));
          }
          return _results;
        }
      },
      removedfile: function(file) {
        var _ref;
        if (file.previewElement) {
          if ((_ref = file.previewElement) != null) {
            _ref.parentNode.removeChild(file.previewElement);
          }
        }
        return this._updateMaxFilesReachedClass();
      },
      thumbnail: function(file, dataUrl) {
        var thumbnailElement, _i, _len, _ref;
        if (file.previewElement) {
          file.previewElement.classList.remove("dz-file-preview");
          _ref = file.previewElement.querySelectorAll("[data-dz-thumbnail]");
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            thumbnailElement = _ref[_i];
            thumbnailElement.alt = file.name;
            thumbnailElement.src = dataUrl;
          }
          return setTimeout(((function(_this) {
            return function() {
              return file.previewElement.classList.add("dz-image-preview");
            };
          })(this)), 1);
        }
      },
      error: function(file, message) {
        var node, _i, _len, _ref, _results;
        if (file.previewElement) {
          file.previewElement.classList.add("dz-error");
          if (typeof message !== "String" && message.error) {
            message = message.error;
          }
          _ref = file.previewElement.querySelectorAll("[data-dz-errormessage]");
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            node = _ref[_i];
            _results.push(node.textContent = message);
          }
          return _results;
        }
      },
      errormultiple: noop,
      processing: function(file) {
        if (file.previewElement) {
          file.previewElement.classList.add("dz-processing");
          if (file._removeLink) {
            return file._removeLink.textContent = this.options.dictCancelUpload;
          }
        }
      },
      processingmultiple: noop,
      uploadprogress: function(file, progress, bytesSent) {
        var node, _i, _len, _ref, _results;
        if (file.previewElement) {
          _ref = file.previewElement.querySelectorAll("[data-dz-uploadprogress]");
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            node = _ref[_i];
            if (node.nodeName === 'PROGRESS') {
              _results.push(node.value = progress);
            } else {
              _results.push(node.style.width = "" + progress + "%");
            }
          }
          return _results;
        }
      },
      totaluploadprogress: noop,
      sending: noop,
      sendingmultiple: noop,
      success: function(file) {
        if (file.previewElement) {
          return file.previewElement.classList.add("dz-success");
        }
      },
      successmultiple: noop,
      canceled: function(file) {
        return this.emit("error", file, "Upload canceled.");
      },
      canceledmultiple: noop,
      complete: function(file) {
        if (file._removeLink) {
          file._removeLink.textContent = this.options.dictRemoveFile;
        }
        if (file.previewElement) {
          return file.previewElement.classList.add("dz-complete");
        }
      },
      completemultiple: noop,
      maxfilesexceeded: noop,
      maxfilesreached: noop,
      queuecomplete: noop,
      previewTemplate: "<div class=\"dz-preview dz-file-preview\">\n  <div class=\"dz-image\"><img data-dz-thumbnail /></div>\n  <div class=\"dz-details\">\n    <div class=\"dz-size\"><span data-dz-size></span></div>\n    <div class=\"dz-filename\"><span data-dz-name></span></div>\n  </div>\n  <div class=\"dz-progress\"><span class=\"dz-upload\" data-dz-uploadprogress></span></div>\n  <div class=\"dz-error-message\"><span data-dz-errormessage></span></div>\n  <div class=\"dz-success-mark\">\n    <svg width=\"54px\" height=\"54px\" viewBox=\"0 0 54 54\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:sketch=\"http://www.bohemiancoding.com/sketch/ns\">\n      <title>Check</title>\n      <defs></defs>\n      <g id=\"Page-1\" stroke=\"none\" stroke-width=\"1\" fill=\"none\" fill-rule=\"evenodd\" sketch:type=\"MSPage\">\n        <path d=\"M23.5,31.8431458 L17.5852419,25.9283877 C16.0248253,24.3679711 13.4910294,24.366835 11.9289322,25.9289322 C10.3700136,27.4878508 10.3665912,30.0234455 11.9283877,31.5852419 L20.4147581,40.0716123 C20.5133999,40.1702541 20.6159315,40.2626649 20.7218615,40.3488435 C22.2835669,41.8725651 24.794234,41.8626202 26.3461564,40.3106978 L43.3106978,23.3461564 C44.8771021,21.7797521 44.8758057,19.2483887 43.3137085,17.6862915 C41.7547899,16.1273729 39.2176035,16.1255422 37.6538436,17.6893022 L23.5,31.8431458 Z M27,53 C41.3594035,53 53,41.3594035 53,27 C53,12.6405965 41.3594035,1 27,1 C12.6405965,1 1,12.6405965 1,27 C1,41.3594035 12.6405965,53 27,53 Z\" id=\"Oval-2\" stroke-opacity=\"0.198794158\" stroke=\"#747474\" fill-opacity=\"0.816519475\" fill=\"#FFFFFF\" sketch:type=\"MSShapeGroup\"></path>\n      </g>\n    </svg>\n  </div>\n  <div class=\"dz-error-mark\">\n    <svg width=\"54px\" height=\"54px\" viewBox=\"0 0 54 54\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:sketch=\"http://www.bohemiancoding.com/sketch/ns\">\n      <title>Error</title>\n      <defs></defs>\n      <g id=\"Page-1\" stroke=\"none\" stroke-width=\"1\" fill=\"none\" fill-rule=\"evenodd\" sketch:type=\"MSPage\">\n        <g id=\"Check-+-Oval-2\" sketch:type=\"MSLayerGroup\" stroke=\"#747474\" stroke-opacity=\"0.198794158\" fill=\"#FFFFFF\" fill-opacity=\"0.816519475\">\n          <path d=\"M32.6568542,29 L38.3106978,23.3461564 C39.8771021,21.7797521 39.8758057,19.2483887 38.3137085,17.6862915 C36.7547899,16.1273729 34.2176035,16.1255422 32.6538436,17.6893022 L27,23.3431458 L21.3461564,17.6893022 C19.7823965,16.1255422 17.2452101,16.1273729 15.6862915,17.6862915 C14.1241943,19.2483887 14.1228979,21.7797521 15.6893022,23.3461564 L21.3431458,29 L15.6893022,34.6538436 C14.1228979,36.2202479 14.1241943,38.7516113 15.6862915,40.3137085 C17.2452101,41.8726271 19.7823965,41.8744578 21.3461564,40.3106978 L27,34.6568542 L32.6538436,40.3106978 C34.2176035,41.8744578 36.7547899,41.8726271 38.3137085,40.3137085 C39.8758057,38.7516113 39.8771021,36.2202479 38.3106978,34.6538436 L32.6568542,29 Z M27,53 C41.3594035,53 53,41.3594035 53,27 C53,12.6405965 41.3594035,1 27,1 C12.6405965,1 1,12.6405965 1,27 C1,41.3594035 12.6405965,53 27,53 Z\" id=\"Oval-2\" sketch:type=\"MSShapeGroup\"></path>\n        </g>\n      </g>\n    </svg>\n  </div>\n</div>"
    };

    extend = function() {
      var key, object, objects, target, val, _i, _len;
      target = arguments[0], objects = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      for (_i = 0, _len = objects.length; _i < _len; _i++) {
        object = objects[_i];
        for (key in object) {
          val = object[key];
          target[key] = val;
        }
      }
      return target;
    };

    function Dropzone(element, options) {
      var elementOptions, fallback, _ref;
      this.element = element;
      this.version = Dropzone.version;
      this.defaultOptions.previewTemplate = this.defaultOptions.previewTemplate.replace(/\n*/g, "");
      this.clickableElements = [];
      this.listeners = [];
      this.files = [];
      if (typeof this.element === "string") {
        this.element = document.querySelector(this.element);
      }
      if (!(this.element && (this.element.nodeType != null))) {
        throw new Error("Invalid dropzone element.");
      }
      if (this.element.dropzone) {
        throw new Error("Dropzone already attached.");
      }
      Dropzone.instances.push(this);
      this.element.dropzone = this;
      elementOptions = (_ref = Dropzone.optionsForElement(this.element)) != null ? _ref : {};
      this.options = extend({}, this.defaultOptions, elementOptions, options != null ? options : {});
      if (this.options.forceFallback || !Dropzone.isBrowserSupported()) {
        return this.options.fallback.call(this);
      }
      if (this.options.url == null) {
        this.options.url = this.element.getAttribute("action");
      }
      if (!this.options.url) {
        throw new Error("No URL provided.");
      }
      if (this.options.acceptedFiles && this.options.acceptedMimeTypes) {
        throw new Error("You can't provide both 'acceptedFiles' and 'acceptedMimeTypes'. 'acceptedMimeTypes' is deprecated.");
      }
      if (this.options.acceptedMimeTypes) {
        this.options.acceptedFiles = this.options.acceptedMimeTypes;
        delete this.options.acceptedMimeTypes;
      }
      this.options.method = this.options.method.toUpperCase();
      if ((fallback = this.getExistingFallback()) && fallback.parentNode) {
        fallback.parentNode.removeChild(fallback);
      }
      if (this.options.previewsContainer !== false) {
        if (this.options.previewsContainer) {
          this.previewsContainer = Dropzone.getElement(this.options.previewsContainer, "previewsContainer");
        } else {
          this.previewsContainer = this.element;
        }
      }
      if (this.options.clickable) {
        if (this.options.clickable === true) {
          this.clickableElements = [this.element];
        } else {
          this.clickableElements = Dropzone.getElements(this.options.clickable, "clickable");
        }
      }
      this.init();
    }

    Dropzone.prototype.getAcceptedFiles = function() {
      var file, _i, _len, _ref, _results;
      _ref = this.files;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        file = _ref[_i];
        if (file.accepted) {
          _results.push(file);
        }
      }
      return _results;
    };

    Dropzone.prototype.getRejectedFiles = function() {
      var file, _i, _len, _ref, _results;
      _ref = this.files;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        file = _ref[_i];
        if (!file.accepted) {
          _results.push(file);
        }
      }
      return _results;
    };

    Dropzone.prototype.getFilesWithStatus = function(status) {
      var file, _i, _len, _ref, _results;
      _ref = this.files;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        file = _ref[_i];
        if (file.status === status) {
          _results.push(file);
        }
      }
      return _results;
    };

    Dropzone.prototype.getQueuedFiles = function() {
      return this.getFilesWithStatus(Dropzone.QUEUED);
    };

    Dropzone.prototype.getUploadingFiles = function() {
      return this.getFilesWithStatus(Dropzone.UPLOADING);
    };

    Dropzone.prototype.getActiveFiles = function() {
      var file, _i, _len, _ref, _results;
      _ref = this.files;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        file = _ref[_i];
        if (file.status === Dropzone.UPLOADING || file.status === Dropzone.QUEUED) {
          _results.push(file);
        }
      }
      return _results;
    };

    Dropzone.prototype.init = function() {
      var eventName, noPropagation, setupHiddenFileInput, _i, _len, _ref, _ref1;
      if (this.element.tagName === "form") {
        this.element.setAttribute("enctype", "multipart/form-data");
      }
      if (this.element.classList.contains("dropzone") && !this.element.querySelector(".dz-message")) {
        this.element.appendChild(Dropzone.createElement("<div class=\"dz-default dz-message\"><span>" + this.options.dictDefaultMessage + "</span></div>"));
      }
      if (this.clickableElements.length) {
        setupHiddenFileInput = (function(_this) {
          return function() {
            if (_this.hiddenFileInput) {
              document.body.removeChild(_this.hiddenFileInput);
            }
            _this.hiddenFileInput = document.createElement("input");
            _this.hiddenFileInput.setAttribute("type", "file");
            if ((_this.options.maxFiles == null) || _this.options.maxFiles > 1) {
              _this.hiddenFileInput.setAttribute("multiple", "multiple");
            }
            _this.hiddenFileInput.className = "dz-hidden-input";
            if (_this.options.acceptedFiles != null) {
              _this.hiddenFileInput.setAttribute("accept", _this.options.acceptedFiles);
            }
            if (_this.options.capture != null) {
              _this.hiddenFileInput.setAttribute("capture", _this.options.capture);
            }
            _this.hiddenFileInput.style.visibility = "hidden";
            _this.hiddenFileInput.style.position = "absolute";
            _this.hiddenFileInput.style.top = "0";
            _this.hiddenFileInput.style.left = "0";
            _this.hiddenFileInput.style.height = "0";
            _this.hiddenFileInput.style.width = "0";
            document.body.appendChild(_this.hiddenFileInput);
            return _this.hiddenFileInput.addEventListener("change", function() {
              var file, files, _i, _len;
              files = _this.hiddenFileInput.files;
              if (files.length) {
                for (_i = 0, _len = files.length; _i < _len; _i++) {
                  file = files[_i];
                  _this.addFile(file);
                }
              }
              return setupHiddenFileInput();
            });
          };
        })(this);
        setupHiddenFileInput();
      }
      this.URL = (_ref = window.URL) != null ? _ref : window.webkitURL;
      _ref1 = this.events;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        eventName = _ref1[_i];
        this.on(eventName, this.options[eventName]);
      }
      this.on("uploadprogress", (function(_this) {
        return function() {
          return _this.updateTotalUploadProgress();
        };
      })(this));
      this.on("removedfile", (function(_this) {
        return function() {
          return _this.updateTotalUploadProgress();
        };
      })(this));
      this.on("canceled", (function(_this) {
        return function(file) {
          return _this.emit("complete", file);
        };
      })(this));
      this.on("complete", (function(_this) {
        return function(file) {
          if (_this.getUploadingFiles().length === 0 && _this.getQueuedFiles().length === 0) {
            return setTimeout((function() {
              return _this.emit("queuecomplete");
            }), 0);
          }
        };
      })(this));
      noPropagation = function(e) {
        e.stopPropagation();
        if (e.preventDefault) {
          return e.preventDefault();
        } else {
          return e.returnValue = false;
        }
      };
      this.listeners = [
        {
          element: this.element,
          events: {
            "dragstart": (function(_this) {
              return function(e) {
                return _this.emit("dragstart", e);
              };
            })(this),
            "dragenter": (function(_this) {
              return function(e) {
                noPropagation(e);
                return _this.emit("dragenter", e);
              };
            })(this),
            "dragover": (function(_this) {
              return function(e) {
                var efct;
                try {
                  efct = e.dataTransfer.effectAllowed;
                } catch (_error) {}
                e.dataTransfer.dropEffect = 'move' === efct || 'linkMove' === efct ? 'move' : 'copy';
                noPropagation(e);
                return _this.emit("dragover", e);
              };
            })(this),
            "dragleave": (function(_this) {
              return function(e) {
                return _this.emit("dragleave", e);
              };
            })(this),
            "drop": (function(_this) {
              return function(e) {
                noPropagation(e);
                return _this.drop(e);
              };
            })(this),
            "dragend": (function(_this) {
              return function(e) {
                return _this.emit("dragend", e);
              };
            })(this)
          }
        }
      ];
      this.clickableElements.forEach((function(_this) {
        return function(clickableElement) {
          return _this.listeners.push({
            element: clickableElement,
            events: {
              "click": function(evt) {
                if ((clickableElement !== _this.element) || (evt.target === _this.element || Dropzone.elementInside(evt.target, _this.element.querySelector(".dz-message")))) {
                  return _this.hiddenFileInput.click();
                }
              }
            }
          });
        };
      })(this));
      this.enable();
      return this.options.init.call(this);
    };

    Dropzone.prototype.destroy = function() {
      var _ref;
      this.disable();
      this.removeAllFiles(true);
      if ((_ref = this.hiddenFileInput) != null ? _ref.parentNode : void 0) {
        this.hiddenFileInput.parentNode.removeChild(this.hiddenFileInput);
        this.hiddenFileInput = null;
      }
      delete this.element.dropzone;
      return Dropzone.instances.splice(Dropzone.instances.indexOf(this), 1);
    };

    Dropzone.prototype.updateTotalUploadProgress = function() {
      var activeFiles, file, totalBytes, totalBytesSent, totalUploadProgress, _i, _len, _ref;
      totalBytesSent = 0;
      totalBytes = 0;
      activeFiles = this.getActiveFiles();
      if (activeFiles.length) {
        _ref = this.getActiveFiles();
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          file = _ref[_i];
          totalBytesSent += file.upload.bytesSent;
          totalBytes += file.upload.total;
        }
        totalUploadProgress = 100 * totalBytesSent / totalBytes;
      } else {
        totalUploadProgress = 100;
      }
      return this.emit("totaluploadprogress", totalUploadProgress, totalBytes, totalBytesSent);
    };

    Dropzone.prototype._getParamName = function(n) {
      if (typeof this.options.paramName === "function") {
        return this.options.paramName(n);
      } else {
        return "" + this.options.paramName + (this.options.uploadMultiple ? "[" + n + "]" : "");
      }
    };

    Dropzone.prototype.getFallbackForm = function() {
      var existingFallback, fields, fieldsString, form;
      if (existingFallback = this.getExistingFallback()) {
        return existingFallback;
      }
      fieldsString = "<div class=\"dz-fallback\">";
      if (this.options.dictFallbackText) {
        fieldsString += "<p>" + this.options.dictFallbackText + "</p>";
      }
      fieldsString += "<input type=\"file\" name=\"" + (this._getParamName(0)) + "\" " + (this.options.uploadMultiple ? 'multiple="multiple"' : void 0) + " /><input type=\"submit\" value=\"Upload!\"></div>";
      fields = Dropzone.createElement(fieldsString);
      if (this.element.tagName !== "FORM") {
        form = Dropzone.createElement("<form action=\"" + this.options.url + "\" enctype=\"multipart/form-data\" method=\"" + this.options.method + "\"></form>");
        form.appendChild(fields);
      } else {
        this.element.setAttribute("enctype", "multipart/form-data");
        this.element.setAttribute("method", this.options.method);
      }
      return form != null ? form : fields;
    };

    Dropzone.prototype.getExistingFallback = function() {
      var fallback, getFallback, tagName, _i, _len, _ref;
      getFallback = function(elements) {
        var el, _i, _len;
        for (_i = 0, _len = elements.length; _i < _len; _i++) {
          el = elements[_i];
          if (/(^| )fallback($| )/.test(el.className)) {
            return el;
          }
        }
      };
      _ref = ["div", "form"];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        tagName = _ref[_i];
        if (fallback = getFallback(this.element.getElementsByTagName(tagName))) {
          return fallback;
        }
      }
    };

    Dropzone.prototype.setupEventListeners = function() {
      var elementListeners, event, listener, _i, _len, _ref, _results;
      _ref = this.listeners;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        elementListeners = _ref[_i];
        _results.push((function() {
          var _ref1, _results1;
          _ref1 = elementListeners.events;
          _results1 = [];
          for (event in _ref1) {
            listener = _ref1[event];
            _results1.push(elementListeners.element.addEventListener(event, listener, false));
          }
          return _results1;
        })());
      }
      return _results;
    };

    Dropzone.prototype.removeEventListeners = function() {
      var elementListeners, event, listener, _i, _len, _ref, _results;
      _ref = this.listeners;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        elementListeners = _ref[_i];
        _results.push((function() {
          var _ref1, _results1;
          _ref1 = elementListeners.events;
          _results1 = [];
          for (event in _ref1) {
            listener = _ref1[event];
            _results1.push(elementListeners.element.removeEventListener(event, listener, false));
          }
          return _results1;
        })());
      }
      return _results;
    };

    Dropzone.prototype.disable = function() {
      var file, _i, _len, _ref, _results;
      this.clickableElements.forEach(function(element) {
        return element.classList.remove("dz-clickable");
      });
      this.removeEventListeners();
      _ref = this.files;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        file = _ref[_i];
        _results.push(this.cancelUpload(file));
      }
      return _results;
    };

    Dropzone.prototype.enable = function() {
      this.clickableElements.forEach(function(element) {
        return element.classList.add("dz-clickable");
      });
      return this.setupEventListeners();
    };

    Dropzone.prototype.filesize = function(size) {
      var cutoff, i, selectedSize, selectedUnit, unit, units, _i, _len;
      units = ['TB', 'GB', 'MB', 'KB', 'b'];
      selectedSize = selectedUnit = null;
      for (i = _i = 0, _len = units.length; _i < _len; i = ++_i) {
        unit = units[i];
        cutoff = Math.pow(this.options.filesizeBase, 4 - i) / 10;
        if (size >= cutoff) {
          selectedSize = size / Math.pow(this.options.filesizeBase, 4 - i);
          selectedUnit = unit;
          break;
        }
      }
      selectedSize = Math.round(10 * selectedSize) / 10;
      return "<strong>" + selectedSize + "</strong> " + selectedUnit;
    };

    Dropzone.prototype._updateMaxFilesReachedClass = function() {
      if ((this.options.maxFiles != null) && this.getAcceptedFiles().length >= this.options.maxFiles) {
        if (this.getAcceptedFiles().length === this.options.maxFiles) {
          this.emit('maxfilesreached', this.files);
        }
        return this.element.classList.add("dz-max-files-reached");
      } else {
        return this.element.classList.remove("dz-max-files-reached");
      }
    };

    Dropzone.prototype.drop = function(e) {
      var files, items;
      if (!e.dataTransfer) {
        return;
      }
      this.emit("drop", e);
      files = e.dataTransfer.files;
      if (files.length) {
        items = e.dataTransfer.items;
        if (items && items.length && (items[0].webkitGetAsEntry != null)) {
          this._addFilesFromItems(items);
        } else {
          this.handleFiles(files);
        }
      }
    };

    Dropzone.prototype.paste = function(e) {
      var items, _ref;
      if ((e != null ? (_ref = e.clipboardData) != null ? _ref.items : void 0 : void 0) == null) {
        return;
      }
      this.emit("paste", e);
      items = e.clipboardData.items;
      if (items.length) {
        return this._addFilesFromItems(items);
      }
    };

    Dropzone.prototype.handleFiles = function(files) {
      var file, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        _results.push(this.addFile(file));
      }
      return _results;
    };

    Dropzone.prototype._addFilesFromItems = function(items) {
      var entry, item, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        if ((item.webkitGetAsEntry != null) && (entry = item.webkitGetAsEntry())) {
          if (entry.isFile) {
            _results.push(this.addFile(item.getAsFile()));
          } else if (entry.isDirectory) {
            _results.push(this._addFilesFromDirectory(entry, entry.name));
          } else {
            _results.push(void 0);
          }
        } else if (item.getAsFile != null) {
          if ((item.kind == null) || item.kind === "file") {
            _results.push(this.addFile(item.getAsFile()));
          } else {
            _results.push(void 0);
          }
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    Dropzone.prototype._addFilesFromDirectory = function(directory, path) {
      var dirReader, entriesReader;
      dirReader = directory.createReader();
      entriesReader = (function(_this) {
        return function(entries) {
          var entry, _i, _len;
          for (_i = 0, _len = entries.length; _i < _len; _i++) {
            entry = entries[_i];
            if (entry.isFile) {
              entry.file(function(file) {
                if (_this.options.ignoreHiddenFiles && file.name.substring(0, 1) === '.') {
                  return;
                }
                file.fullPath = "" + path + "/" + file.name;
                return _this.addFile(file);
              });
            } else if (entry.isDirectory) {
              _this._addFilesFromDirectory(entry, "" + path + "/" + entry.name);
            }
          }
        };
      })(this);
      return dirReader.readEntries(entriesReader, function(error) {
        return typeof console !== "undefined" && console !== null ? typeof console.log === "function" ? console.log(error) : void 0 : void 0;
      });
    };

    Dropzone.prototype.accept = function(file, done) {
      if (file.size > this.options.maxFilesize * 1024 * 1024) {
        return done(this.options.dictFileTooBig.replace("{{filesize}}", Math.round(file.size / 1024 / 10.24) / 100).replace("{{maxFilesize}}", this.options.maxFilesize));
      } else if (!Dropzone.isValidFile(file, this.options.acceptedFiles)) {
        return done(this.options.dictInvalidFileType);
      } else if ((this.options.maxFiles != null) && this.getAcceptedFiles().length >= this.options.maxFiles) {
        done(this.options.dictMaxFilesExceeded.replace("{{maxFiles}}", this.options.maxFiles));
        return this.emit("maxfilesexceeded", file);
      } else {
        return this.options.accept.call(this, file, done);
      }
    };

    Dropzone.prototype.addFile = function(file) {
      file.upload = {
        progress: 0,
        total: file.size,
        bytesSent: 0
      };
      this.files.push(file);
      file.status = Dropzone.ADDED;
      this.emit("addedfile", file);
      this._enqueueThumbnail(file);
      return this.accept(file, (function(_this) {
        return function(error) {
          if (error) {
            file.accepted = false;
            _this._errorProcessing([file], error);
          } else {
            file.accepted = true;
            if (_this.options.autoQueue) {
              _this.enqueueFile(file);
            }
          }
          return _this._updateMaxFilesReachedClass();
        };
      })(this));
    };

    Dropzone.prototype.enqueueFiles = function(files) {
      var file, _i, _len;
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        this.enqueueFile(file);
      }
      return null;
    };

    Dropzone.prototype.enqueueFile = function(file) {
      if (file.status === Dropzone.ADDED && file.accepted === true) {
        file.status = Dropzone.QUEUED;
        if (this.options.autoProcessQueue) {
          return setTimeout(((function(_this) {
            return function() {
              return _this.processQueue();
            };
          })(this)), 0);
        }
      } else {
        throw new Error("This file can't be queued because it has already been processed or was rejected.");
      }
    };

    Dropzone.prototype._thumbnailQueue = [];

    Dropzone.prototype._processingThumbnail = false;

    Dropzone.prototype._enqueueThumbnail = function(file) {
      if (this.options.createImageThumbnails && file.type.match(/image.*/) && file.size <= this.options.maxThumbnailFilesize * 1024 * 1024) {
        this._thumbnailQueue.push(file);
        return setTimeout(((function(_this) {
          return function() {
            return _this._processThumbnailQueue();
          };
        })(this)), 0);
      }
    };

    Dropzone.prototype._processThumbnailQueue = function() {
      if (this._processingThumbnail || this._thumbnailQueue.length === 0) {
        return;
      }
      this._processingThumbnail = true;
      return this.createThumbnail(this._thumbnailQueue.shift(), (function(_this) {
        return function() {
          _this._processingThumbnail = false;
          return _this._processThumbnailQueue();
        };
      })(this));
    };

    Dropzone.prototype.removeFile = function(file) {
      if (file.status === Dropzone.UPLOADING) {
        this.cancelUpload(file);
      }
      this.files = without(this.files, file);
      this.emit("removedfile", file);
      if (this.files.length === 0) {
        return this.emit("reset");
      }
    };

    Dropzone.prototype.removeAllFiles = function(cancelIfNecessary) {
      var file, _i, _len, _ref;
      if (cancelIfNecessary == null) {
        cancelIfNecessary = false;
      }
      _ref = this.files.slice();
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        file = _ref[_i];
        if (file.status !== Dropzone.UPLOADING || cancelIfNecessary) {
          this.removeFile(file);
        }
      }
      return null;
    };

    Dropzone.prototype.createThumbnail = function(file, callback) {
      var fileReader;
      fileReader = new FileReader;
      fileReader.onload = (function(_this) {
        return function() {
          if (file.type === "image/svg+xml") {
            _this.emit("thumbnail", file, fileReader.result);
            if (callback != null) {
              callback();
            }
            return;
          }
          return _this.createThumbnailFromUrl(file, fileReader.result, callback);
        };
      })(this);
      return fileReader.readAsDataURL(file);
    };

    Dropzone.prototype.createThumbnailFromUrl = function(file, imageUrl, callback) {
      var img;
      img = document.createElement("img");
      img.onload = (function(_this) {
        return function() {
          var canvas, ctx, resizeInfo, thumbnail, _ref, _ref1, _ref2, _ref3;
          file.width = img.width;
          file.height = img.height;
          resizeInfo = _this.options.resize.call(_this, file);
          if (resizeInfo.trgWidth == null) {
            resizeInfo.trgWidth = resizeInfo.optWidth;
          }
          if (resizeInfo.trgHeight == null) {
            resizeInfo.trgHeight = resizeInfo.optHeight;
          }
          canvas = document.createElement("canvas");
          ctx = canvas.getContext("2d");
          canvas.width = resizeInfo.trgWidth;
          canvas.height = resizeInfo.trgHeight;
          drawImageIOSFix(ctx, img, (_ref = resizeInfo.srcX) != null ? _ref : 0, (_ref1 = resizeInfo.srcY) != null ? _ref1 : 0, resizeInfo.srcWidth, resizeInfo.srcHeight, (_ref2 = resizeInfo.trgX) != null ? _ref2 : 0, (_ref3 = resizeInfo.trgY) != null ? _ref3 : 0, resizeInfo.trgWidth, resizeInfo.trgHeight);
          thumbnail = canvas.toDataURL("image/png");
          _this.emit("thumbnail", file, thumbnail);
          if (callback != null) {
            return callback();
          }
        };
      })(this);
      if (callback != null) {
        img.onerror = callback;
      }
      return img.src = imageUrl;
    };

    Dropzone.prototype.processQueue = function() {
      var i, parallelUploads, processingLength, queuedFiles;
      parallelUploads = this.options.parallelUploads;
      processingLength = this.getUploadingFiles().length;
      i = processingLength;
      if (processingLength >= parallelUploads) {
        return;
      }
      queuedFiles = this.getQueuedFiles();
      if (!(queuedFiles.length > 0)) {
        return;
      }
      if (this.options.uploadMultiple) {
        return this.processFiles(queuedFiles.slice(0, parallelUploads - processingLength));
      } else {
        while (i < parallelUploads) {
          if (!queuedFiles.length) {
            return;
          }
          this.processFile(queuedFiles.shift());
          i++;
        }
      }
    };

    Dropzone.prototype.processFile = function(file) {
      return this.processFiles([file]);
    };

    Dropzone.prototype.processFiles = function(files) {
      var file, _i, _len;
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        file.processing = true;
        file.status = Dropzone.UPLOADING;
        this.emit("processing", file);
      }
      if (this.options.uploadMultiple) {
        this.emit("processingmultiple", files);
      }
      return this.uploadFiles(files);
    };

    Dropzone.prototype._getFilesWithXhr = function(xhr) {
      var file, files;
      return files = (function() {
        var _i, _len, _ref, _results;
        _ref = this.files;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          file = _ref[_i];
          if (file.xhr === xhr) {
            _results.push(file);
          }
        }
        return _results;
      }).call(this);
    };

    Dropzone.prototype.cancelUpload = function(file) {
      var groupedFile, groupedFiles, _i, _j, _len, _len1, _ref;
      if (file.status === Dropzone.UPLOADING) {
        groupedFiles = this._getFilesWithXhr(file.xhr);
        for (_i = 0, _len = groupedFiles.length; _i < _len; _i++) {
          groupedFile = groupedFiles[_i];
          groupedFile.status = Dropzone.CANCELED;
        }
        file.xhr.abort();
        for (_j = 0, _len1 = groupedFiles.length; _j < _len1; _j++) {
          groupedFile = groupedFiles[_j];
          this.emit("canceled", groupedFile);
        }
        if (this.options.uploadMultiple) {
          this.emit("canceledmultiple", groupedFiles);
        }
      } else if ((_ref = file.status) === Dropzone.ADDED || _ref === Dropzone.QUEUED) {
        file.status = Dropzone.CANCELED;
        this.emit("canceled", file);
        if (this.options.uploadMultiple) {
          this.emit("canceledmultiple", [file]);
        }
      }
      if (this.options.autoProcessQueue) {
        return this.processQueue();
      }
    };

    resolveOption = function() {
      var args, option;
      option = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      if (typeof option === 'function') {
        return option.apply(this, args);
      }
      return option;
    };

    Dropzone.prototype.uploadFile = function(file) {
      return this.uploadFiles([file]);
    };

    Dropzone.prototype.uploadFiles = function(files) {
      var file, formData, handleError, headerName, headerValue, headers, i, input, inputName, inputType, key, method, option, progressObj, response, updateProgress, url, value, xhr, _i, _j, _k, _l, _len, _len1, _len2, _len3, _m, _ref, _ref1, _ref2, _ref3, _ref4, _ref5;
      xhr = new XMLHttpRequest();
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        file.xhr = xhr;
      }
      method = resolveOption(this.options.method, files);
      url = resolveOption(this.options.url, files);
      xhr.open(method, url, true);
      xhr.withCredentials = !!this.options.withCredentials;
      response = null;
      handleError = (function(_this) {
        return function() {
          var _j, _len1, _results;
          _results = [];
          for (_j = 0, _len1 = files.length; _j < _len1; _j++) {
            file = files[_j];
            _results.push(_this._errorProcessing(files, response || _this.options.dictResponseError.replace("{{statusCode}}", xhr.status), xhr));
          }
          return _results;
        };
      })(this);
      updateProgress = (function(_this) {
        return function(e) {
          var allFilesFinished, progress, _j, _k, _l, _len1, _len2, _len3, _results;
          if (e != null) {
            progress = 100 * e.loaded / e.total;
            for (_j = 0, _len1 = files.length; _j < _len1; _j++) {
              file = files[_j];
              file.upload = {
                progress: progress,
                total: e.total,
                bytesSent: e.loaded
              };
            }
          } else {
            allFilesFinished = true;
            progress = 100;
            for (_k = 0, _len2 = files.length; _k < _len2; _k++) {
              file = files[_k];
              if (!(file.upload.progress === 100 && file.upload.bytesSent === file.upload.total)) {
                allFilesFinished = false;
              }
              file.upload.progress = progress;
              file.upload.bytesSent = file.upload.total;
            }
            if (allFilesFinished) {
              return;
            }
          }
          _results = [];
          for (_l = 0, _len3 = files.length; _l < _len3; _l++) {
            file = files[_l];
            _results.push(_this.emit("uploadprogress", file, progress, file.upload.bytesSent));
          }
          return _results;
        };
      })(this);
      xhr.onload = (function(_this) {
        return function(e) {
          var _ref;
          if (files[0].status === Dropzone.CANCELED) {
            return;
          }
          if (xhr.readyState !== 4) {
            return;
          }
          response = xhr.responseText;
          if (xhr.getResponseHeader("content-type") && ~xhr.getResponseHeader("content-type").indexOf("application/json")) {
            try {
              response = JSON.parse(response);
            } catch (_error) {
              e = _error;
              response = "Invalid JSON response from server.";
            }
          }
          updateProgress();
          if (!((200 <= (_ref = xhr.status) && _ref < 300))) {
            return handleError();
          } else {
            return _this._finished(files, response, e);
          }
        };
      })(this);
      xhr.onerror = (function(_this) {
        return function() {
          if (files[0].status === Dropzone.CANCELED) {
            return;
          }
          return handleError();
        };
      })(this);
      progressObj = (_ref = xhr.upload) != null ? _ref : xhr;
      progressObj.onprogress = updateProgress;
      headers = {
        "Accept": "application/json",
        "Cache-Control": "no-cache",
        "X-Requested-With": "XMLHttpRequest"
      };
      if (this.options.headers) {
        extend(headers, this.options.headers);
      }
      for (headerName in headers) {
        headerValue = headers[headerName];
        xhr.setRequestHeader(headerName, headerValue);
      }
      formData = new FormData();
      if (this.options.params) {
        _ref1 = this.options.params;
        for (key in _ref1) {
          value = _ref1[key];
          formData.append(key, value);
        }
      }
      for (_j = 0, _len1 = files.length; _j < _len1; _j++) {
        file = files[_j];
        this.emit("sending", file, xhr, formData);
      }
      if (this.options.uploadMultiple) {
        this.emit("sendingmultiple", files, xhr, formData);
      }
      if (this.element.tagName === "FORM") {
        _ref2 = this.element.querySelectorAll("input, textarea, select, button");
        for (_k = 0, _len2 = _ref2.length; _k < _len2; _k++) {
          input = _ref2[_k];
          inputName = input.getAttribute("name");
          inputType = input.getAttribute("type");
          if (input.tagName === "SELECT" && input.hasAttribute("multiple")) {
            _ref3 = input.options;
            for (_l = 0, _len3 = _ref3.length; _l < _len3; _l++) {
              option = _ref3[_l];
              if (option.selected) {
                formData.append(inputName, option.value);
              }
            }
          } else if (!inputType || ((_ref4 = inputType.toLowerCase()) !== "checkbox" && _ref4 !== "radio") || input.checked) {
            formData.append(inputName, input.value);
          }
        }
      }
      for (i = _m = 0, _ref5 = files.length - 1; 0 <= _ref5 ? _m <= _ref5 : _m >= _ref5; i = 0 <= _ref5 ? ++_m : --_m) {
        formData.append(this._getParamName(i), files[i], files[i].name);
      }
      return xhr.send(formData);
    };

    Dropzone.prototype._finished = function(files, responseText, e) {
      var file, _i, _len;
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        file.status = Dropzone.SUCCESS;
        this.emit("success", file, responseText, e);
        this.emit("complete", file);
      }
      if (this.options.uploadMultiple) {
        this.emit("successmultiple", files, responseText, e);
        this.emit("completemultiple", files);
      }
      if (this.options.autoProcessQueue) {
        return this.processQueue();
      }
    };

    Dropzone.prototype._errorProcessing = function(files, message, xhr) {
      var file, _i, _len;
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        file.status = Dropzone.ERROR;
        this.emit("error", file, message, xhr);
        this.emit("complete", file);
      }
      if (this.options.uploadMultiple) {
        this.emit("errormultiple", files, message, xhr);
        this.emit("completemultiple", files);
      }
      if (this.options.autoProcessQueue) {
        return this.processQueue();
      }
    };

    return Dropzone;

  })(Emitter);

  Dropzone.version = "4.0.1";

  Dropzone.options = {};

  Dropzone.optionsForElement = function(element) {
    if (element.getAttribute("id")) {
      return Dropzone.options[camelize(element.getAttribute("id"))];
    } else {
      return void 0;
    }
  };

  Dropzone.instances = [];

  Dropzone.forElement = function(element) {
    if (typeof element === "string") {
      element = document.querySelector(element);
    }
    if ((element != null ? element.dropzone : void 0) == null) {
      throw new Error("No Dropzone found for given element. This is probably because you're trying to access it before Dropzone had the time to initialize. Use the `init` option to setup any additional observers on your Dropzone.");
    }
    return element.dropzone;
  };

  Dropzone.autoDiscover = true;

  Dropzone.discover = function() {
    var checkElements, dropzone, dropzones, _i, _len, _results;
    if (document.querySelectorAll) {
      dropzones = document.querySelectorAll(".dropzone");
    } else {
      dropzones = [];
      checkElements = function(elements) {
        var el, _i, _len, _results;
        _results = [];
        for (_i = 0, _len = elements.length; _i < _len; _i++) {
          el = elements[_i];
          if (/(^| )dropzone($| )/.test(el.className)) {
            _results.push(dropzones.push(el));
          } else {
            _results.push(void 0);
          }
        }
        return _results;
      };
      checkElements(document.getElementsByTagName("div"));
      checkElements(document.getElementsByTagName("form"));
    }
    _results = [];
    for (_i = 0, _len = dropzones.length; _i < _len; _i++) {
      dropzone = dropzones[_i];
      if (Dropzone.optionsForElement(dropzone) !== false) {
        _results.push(new Dropzone(dropzone));
      } else {
        _results.push(void 0);
      }
    }
    return _results;
  };

  Dropzone.blacklistedBrowsers = [/opera.*Macintosh.*version\/12/i];

  Dropzone.isBrowserSupported = function() {
    var capableBrowser, regex, _i, _len, _ref;
    capableBrowser = true;
    if (window.File && window.FileReader && window.FileList && window.Blob && window.FormData && document.querySelector) {
      if (!("classList" in document.createElement("a"))) {
        capableBrowser = false;
      } else {
        _ref = Dropzone.blacklistedBrowsers;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          regex = _ref[_i];
          if (regex.test(navigator.userAgent)) {
            capableBrowser = false;
            continue;
          }
        }
      }
    } else {
      capableBrowser = false;
    }
    return capableBrowser;
  };

  without = function(list, rejectedItem) {
    var item, _i, _len, _results;
    _results = [];
    for (_i = 0, _len = list.length; _i < _len; _i++) {
      item = list[_i];
      if (item !== rejectedItem) {
        _results.push(item);
      }
    }
    return _results;
  };

  camelize = function(str) {
    return str.replace(/[\-_](\w)/g, function(match) {
      return match.charAt(1).toUpperCase();
    });
  };

  Dropzone.createElement = function(string) {
    var div;
    div = document.createElement("div");
    div.innerHTML = string;
    return div.childNodes[0];
  };

  Dropzone.elementInside = function(element, container) {
    if (element === container) {
      return true;
    }
    while (element = element.parentNode) {
      if (element === container) {
        return true;
      }
    }
    return false;
  };

  Dropzone.getElement = function(el, name) {
    var element;
    if (typeof el === "string") {
      element = document.querySelector(el);
    } else if (el.nodeType != null) {
      element = el;
    }
    if (element == null) {
      throw new Error("Invalid `" + name + "` option provided. Please provide a CSS selector or a plain HTML element.");
    }
    return element;
  };

  Dropzone.getElements = function(els, name) {
    var e, el, elements, _i, _j, _len, _len1, _ref;
    if (els instanceof Array) {
      elements = [];
      try {
        for (_i = 0, _len = els.length; _i < _len; _i++) {
          el = els[_i];
          elements.push(this.getElement(el, name));
        }
      } catch (_error) {
        e = _error;
        elements = null;
      }
    } else if (typeof els === "string") {
      elements = [];
      _ref = document.querySelectorAll(els);
      for (_j = 0, _len1 = _ref.length; _j < _len1; _j++) {
        el = _ref[_j];
        elements.push(el);
      }
    } else if (els.nodeType != null) {
      elements = [els];
    }
    if (!((elements != null) && elements.length)) {
      throw new Error("Invalid `" + name + "` option provided. Please provide a CSS selector, a plain HTML element or a list of those.");
    }
    return elements;
  };

  Dropzone.confirm = function(question, accepted, rejected) {
    if (window.confirm(question)) {
      return accepted();
    } else if (rejected != null) {
      return rejected();
    }
  };

  Dropzone.isValidFile = function(file, acceptedFiles) {
    var baseMimeType, mimeType, validType, _i, _len;
    if (!acceptedFiles) {
      return true;
    }
    acceptedFiles = acceptedFiles.split(",");
    mimeType = file.type;
    baseMimeType = mimeType.replace(/\/.*$/, "");
    for (_i = 0, _len = acceptedFiles.length; _i < _len; _i++) {
      validType = acceptedFiles[_i];
      validType = validType.trim();
      if (validType.charAt(0) === ".") {
        if (file.name.toLowerCase().indexOf(validType.toLowerCase(), file.name.length - validType.length) !== -1) {
          return true;
        }
      } else if (/\/\*$/.test(validType)) {
        if (baseMimeType === validType.replace(/\/.*$/, "")) {
          return true;
        }
      } else {
        if (mimeType === validType) {
          return true;
        }
      }
    }
    return false;
  };

  if (typeof jQuery !== "undefined" && jQuery !== null) {
    jQuery.fn.dropzone = function(options) {
      return this.each(function() {
        return new Dropzone(this, options);
      });
    };
  }

  if (typeof module !== "undefined" && module !== null) {
    module.exports = Dropzone;
  } else {
    window.Dropzone = Dropzone;
  }

  Dropzone.ADDED = "added";

  Dropzone.QUEUED = "queued";

  Dropzone.ACCEPTED = Dropzone.QUEUED;

  Dropzone.UPLOADING = "uploading";

  Dropzone.PROCESSING = Dropzone.UPLOADING;

  Dropzone.CANCELED = "canceled";

  Dropzone.ERROR = "error";

  Dropzone.SUCCESS = "success";


  /*
  
  Bugfix for iOS 6 and 7
  Source: http://stackoverflow.com/questions/11929099/html5-canvas-drawimage-ratio-bug-ios
  based on the work of https://github.com/stomita/ios-imagefile-megapixel
   */

  detectVerticalSquash = function(img) {
    var alpha, canvas, ctx, data, ey, ih, iw, py, ratio, sy;
    iw = img.naturalWidth;
    ih = img.naturalHeight;
    canvas = document.createElement("canvas");
    canvas.width = 1;
    canvas.height = ih;
    ctx = canvas.getContext("2d");
    ctx.drawImage(img, 0, 0);
    data = ctx.getImageData(0, 0, 1, ih).data;
    sy = 0;
    ey = ih;
    py = ih;
    while (py > sy) {
      alpha = data[(py - 1) * 4 + 3];
      if (alpha === 0) {
        ey = py;
      } else {
        sy = py;
      }
      py = (ey + sy) >> 1;
    }
    ratio = py / ih;
    if (ratio === 0) {
      return 1;
    } else {
      return ratio;
    }
  };

  drawImageIOSFix = function(ctx, img, sx, sy, sw, sh, dx, dy, dw, dh) {
    var vertSquashRatio;
    vertSquashRatio = detectVerticalSquash(img);
    return ctx.drawImage(img, sx, sy, sw, sh, dx, dy, dw, dh / vertSquashRatio);
  };


  /*
   * contentloaded.js
   *
   * Author: Diego Perini (diego.perini at gmail.com)
   * Summary: cross-browser wrapper for DOMContentLoaded
   * Updated: 20101020
   * License: MIT
   * Version: 1.2
   *
   * URL:
   * http://javascript.nwbox.com/ContentLoaded/
   * http://javascript.nwbox.com/ContentLoaded/MIT-LICENSE
   */

  contentLoaded = function(win, fn) {
    var add, doc, done, init, poll, pre, rem, root, top;
    done = false;
    top = true;
    doc = win.document;
    root = doc.documentElement;
    add = (doc.addEventListener ? "addEventListener" : "attachEvent");
    rem = (doc.addEventListener ? "removeEventListener" : "detachEvent");
    pre = (doc.addEventListener ? "" : "on");
    init = function(e) {
      if (e.type === "readystatechange" && doc.readyState !== "complete") {
        return;
      }
      (e.type === "load" ? win : doc)[rem](pre + e.type, init, false);
      if (!done && (done = true)) {
        return fn.call(win, e.type || e);
      }
    };
    poll = function() {
      var e;
      try {
        root.doScroll("left");
      } catch (_error) {
        e = _error;
        setTimeout(poll, 50);
        return;
      }
      return init("poll");
    };
    if (doc.readyState !== "complete") {
      if (doc.createEventObject && root.doScroll) {
        try {
          top = !win.frameElement;
        } catch (_error) {}
        if (top) {
          poll();
        }
      }
      doc[add](pre + "DOMContentLoaded", init, false);
      doc[add](pre + "readystatechange", init, false);
      return win[add](pre + "load", init, false);
    }
  };

  Dropzone._autoDiscoverFunction = function() {
    if (Dropzone.autoDiscover) {
      return Dropzone.discover();
    }
  };

  contentLoaded(window, Dropzone._autoDiscoverFunction);

}).call(this);

define("dropzone", ["jquery"], (function (global) {
    return function () {
        var ret, fn;
        return ret || global.Dropzone;
    };
}(this)));

/* ===================================================
 * tagmanager.js v3.0.1
 * http://welldonethings.com/tags/manager
 * ===================================================
 * Copyright 2012 Max Favilli
 *
 * Licensed under the Mozilla Public License, Version 2.0 You may not use this work except in compliance with the License.
 *
 * http://www.mozilla.org/MPL/2.0/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================== */
(function($) {

    "use strict";

    var defaults = {
        prefilled: null,
        CapitalizeFirstLetter: false,
        preventSubmitOnEnter: true,     // deprecated
        isClearInputOnEsc: true,        // deprecated
        externalTagId: false,
        prefillIdFieldName: 'Id',
        prefillValueFieldName: 'Value',
        AjaxPush: null,
        AjaxPushAllTags: null,
        AjaxPushParameters: null,
        delimiters: [9, 13, 44],        // tab, enter, comma
        backspace: [8],
        maxTags: 0,
        hiddenTagListName: null,        // deprecated
        hiddenTagListId: null,          // deprecated
        replace: true,
        output: null,
        deleteTagsOnBackspace: true,    // deprecated
        tagsContainer: null,
        tagCloseIcon: 'x',
        tagClass: '',
        validator: null,
        onlyTagList: false,
        tagList: null,
        fillInputOnTagRemove: false
    },

    publicMethods = {
        pushTag : function (tag, ignoreEvents, externalTagId) {
            var $self = $(this), opts = $self.data('opts'), alreadyInList, tlisLowerCase, max, tagId,
            tlis = $self.data("tlis"), tlid = $self.data("tlid"), idx, newTagId, newTagRemoveId, escaped,
            html, $el, lastTagId, lastTagObj;

            tag = privateMethods.trimTag(tag, opts.delimiterChars);

            if (!tag || tag.length <= 0) { return; }

            // check if restricted only to the tagList suggestions
            if (opts.onlyTagList && undefined !== opts.tagList ){

                //if the list has been updated by look pushed tag in the tagList. if not found return
                if (opts.tagList){
                    var $tagList = opts.tagList;

                    // change each array item to lower case
                    $.each($tagList, function(index, item) {
                        $tagList[index] = item.toLowerCase();
                    });
                    var suggestion = $.inArray(tag.toLowerCase(), $tagList);

                    if ( -1 === suggestion ) {
                        //console.log("tag:" + tag + " not in tagList, not adding it");
                        return;
                    } 
                }

            }

            if (opts.CapitalizeFirstLetter && tag.length > 1) {
                tag = tag.charAt(0).toUpperCase() + tag.slice(1).toLowerCase();
            }

            // call the validator (if any) and do not let the tag pass if invalid
            if (opts.validator && !opts.validator(tag)) {
                $self.trigger('tm:invalid', tag)
                return;
            }

            // dont accept new tags beyond the defined maximum
            if (opts.maxTags > 0 && tlis.length >= opts.maxTags) { return; }

            alreadyInList = false;
            //use jQuery.map to make this work in IE8 (pure JS map is JS 1.6 but IE8 only supports JS 1.5)
            tlisLowerCase = jQuery.map(tlis, function(elem) {
                return elem.toLowerCase();
            });

            idx = $.inArray(tag.toLowerCase(), tlisLowerCase);

            if (-1 !== idx) {
                // console.log("tag:" + tag + " !!already in list!!");
                alreadyInList = true;
            }

            if (alreadyInList) {
                $self.trigger('tm:duplicated', tag);
                if (opts.blinkClass) {
                    for (var i = 0; i < 6; ++i) {
                        $("#" + $self.data("tm_rndid") + "_" + tlid[idx]).queue(function(next) {
                            $(this).toggleClass(opts.blinkClass);
                            next();
                        }).delay(100);
                    }
                } else {
                    $("#" + $self.data("tm_rndid") + "_" + tlid[idx]).stop()
                        .animate({backgroundColor: opts.blinkBGColor_1}, 100)
                        .animate({backgroundColor: opts.blinkBGColor_2}, 100)
                        .animate({backgroundColor: opts.blinkBGColor_1}, 100)
                        .animate({backgroundColor: opts.blinkBGColor_2}, 100)
                        .animate({backgroundColor: opts.blinkBGColor_1}, 100)
                        .animate({backgroundColor: opts.blinkBGColor_2}, 100);
                }
            } else {
                if (opts.externalTagId === true) {
                    if (externalTagId === undefined) {
                        $.error('externalTagId is not passed for tag -' + tag);
                    }
                    tagId = externalTagId;
                } else {
                    max = Math.max.apply(null, tlid);
                    max = max === -Infinity ? 0 : max;

                    tagId = ++max;
                }
                if (!ignoreEvents) { $self.trigger('tm:pushing', [tag, tagId]); }
                tlis.push(tag);
                tlid.push(tagId);

                if (!ignoreEvents)
                    if (opts.AjaxPush !== null && opts.AjaxPushAllTags == null) {
                        if ($.inArray(tag, opts.prefilled) === -1) {
                            $.post(opts.AjaxPush, $.extend({tag: tag}, opts.AjaxPushParameters));
                        }
                    }

                // console.log("tagList: " + tlis);

                newTagId = $self.data("tm_rndid") + '_' + tagId;
                newTagRemoveId = $self.data("tm_rndid") + '_Remover_' + tagId;
                escaped = $("<span/>").text(tag).html();

                html = '<span class="' + privateMethods.tagClasses.call($self) + '" id="' + newTagId + '">';
                html+= '<span>' + escaped + '</span>';
                html+= '<a href="#" class="tm-tag-remove" id="' + newTagRemoveId + '" TagIdToRemove="' + tagId + '">';
                html+= opts.tagCloseIcon + '</a></span> ';
                $el = $(html);

                if (opts.tagsContainer !== null) {
                    $(opts.tagsContainer).append($el);
                } else {
                    if (tlid.length > 1) {
                        lastTagObj = $self.siblings("#" + $self.data("tm_rndid") + "_" + tlid[tlid.length - 2]);
                        lastTagObj.after($el);
                    } else {
                        $self.before($el);
                    }
                }

                $el.find("#" + newTagRemoveId).on("click", $self, function(e) {
                    e.preventDefault();
                    var TagIdToRemove = parseInt($(this).attr("TagIdToRemove"));
                    privateMethods.spliceTag.call($self, TagIdToRemove, e.data);
                });

                privateMethods.refreshHiddenTagList.call($self);

                if (!ignoreEvents) { $self.trigger('tm:pushed', [tag, tagId]); }

                privateMethods.showOrHide.call($self);
                //if (tagManagerOptions.maxTags > 0 && tlis.length >= tagManagerOptions.maxTags) {
                //  obj.hide();
                //}
            }
            $self.val("");
        },

        popTag : function () {
            var $self = $(this), tagId, tagBeingRemoved,
            tlis = $self.data("tlis"),
            tlid = $self.data("tlid");

            if (tlid.length > 0) {
              tagId = tlid.pop();

              tagBeingRemoved = tlis[tlis.length - 1];
              $self.trigger('tm:popping', [tagBeingRemoved, tagId]);
              tlis.pop();

              // console.log("TagIdToRemove: " + tagId);
              $("#" + $self.data("tm_rndid") + "_" + tagId).remove();
              privateMethods.refreshHiddenTagList.call($self);
              $self.trigger('tm:popped', [tagBeingRemoved, tagId]);
              // console.log(tlis);
            }
        },

        empty : function() {
            var $self = $(this), tlis = $self.data("tlis"), tlid = $self.data("tlid"), tagId;

            while (tlid.length > 0) {
                tagId = tlid.pop();
                tlis.pop();
                // console.log("TagIdToRemove: " + tagId);
                $("#" + $self.data("tm_rndid") + "_" + tagId).remove();
                privateMethods.refreshHiddenTagList.call($self);
                // console.log(tlis);
            }
            $self.trigger('tm:emptied', null);

            privateMethods.showOrHide.call($self);
            //if (tagManagerOptions.maxTags > 0 && tlis.length < tagManagerOptions.maxTags) {
            //  obj.show();
            //}
        },

        tags : function() {
            var $self = this, tlis = $self.data("tlis");
            return tlis;
        }
    },

    privateMethods = {
        showOrHide : function () {
            var $self = this, opts = $self.data('opts'), tlis = $self.data("tlis");

            if (opts.maxTags > 0 && tlis.length < opts.maxTags) {
                $self.show();
                $self.trigger('tm:show');
            }

            if (opts.maxTags > 0 && tlis.length >= opts.maxTags) {
                $self.hide();
                $self.trigger('tm:hide');
            }
        },

        tagClasses : function () {
            var $self = $(this), opts = $self.data('opts'), tagBaseClass = opts.tagBaseClass,
            inputBaseClass = opts.inputBaseClass, cl;
            // 1) default class (tm-tag)
            cl = tagBaseClass;
            // 2) interpolate from input class: tm-input-xxx --> tm-tag-xxx
            if ($self.attr('class')) {
                $.each($self.attr('class').split(' '), function (index, value) {
                    if (value.indexOf(inputBaseClass + '-') !== -1) {
                        cl += ' ' + tagBaseClass + value.substring(inputBaseClass.length);
                    }
                });
            }
            // 3) tags from tagClass option
            cl += (opts.tagClass ? ' ' + opts.tagClass : '');
            return cl;
        },

        trimTag : function (tag, delimiterChars) {
            var i;
            tag = $.trim(tag);
            // truncate at the first delimiter char
            i = 0;
            for (i; i < tag.length; i++) {
                if ($.inArray(tag.charCodeAt(i), delimiterChars) !== -1) { break; }
            }
            return tag.substring(0, i);
        },

        refreshHiddenTagList : function () {
            var $self = $(this), tlis = $self.data("tlis"), lhiddenTagList = $self.data("lhiddenTagList");

            if (lhiddenTagList) {
                $(lhiddenTagList).val(tlis.join($self.data('opts').baseDelimiter)).change();
            }

            $self.trigger('tm:refresh', tlis.join($self.data('opts').baseDelimiter));
        },

        killEvent : function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            e.stopPropagation();
            e.preventDefault();
        },

        keyInArray : function (e, ary) {
            return $.inArray(e.which, ary) !== -1;
        },

        applyDelimiter : function (e) {
            var $self = $(this);
            publicMethods.pushTag.call($self,$(this).val());
            e.preventDefault();
        },

        prefill: function (pta) {
            var $self = $(this);
            var opts = $self.data('opts')
            $.each(pta, function (key, val) {
                if (opts.externalTagId === true) {
                    publicMethods.pushTag.call($self, val[opts.prefillValueFieldName], true, val[opts.prefillIdFieldName]);
                } else {
                    publicMethods.pushTag.call($self, val, true);
                }
            });
        },

        pushAllTags : function (e, tag) {
            var $self = $(this), opts = $self.data('opts'), tlis = $self.data("tlis");
            if (opts.AjaxPushAllTags) {
                if (e.type !== 'tm:pushed' || $.inArray(tag, opts.prefilled) === -1) {
                    $.post(opts.AjaxPush, $.extend({ tags: tlis.join(opts.baseDelimiter) }, opts.AjaxPushParameters));
                }
            }
        },

        spliceTag : function (tagId) {
            var $self = this, tlis = $self.data("tlis"), tlid = $self.data("tlid"), idx = $.inArray(tagId, tlid),
            tagBeingRemoved;

            // console.log("TagIdToRemove: " + tagId);
            // console.log("position: " + idx);

            if (-1 !== idx) {
                tagBeingRemoved = tlis[idx];
                $self.trigger('tm:splicing', [tagBeingRemoved, tagId]);
                $("#" + $self.data("tm_rndid") + "_" + tagId).remove();
                tlis.splice(idx, 1);
                tlid.splice(idx, 1);
                privateMethods.refreshHiddenTagList.call($self);
                $self.trigger('tm:spliced', [tagBeingRemoved, tagId]);
                // console.log(tlis);
            }

            privateMethods.showOrHide.call($self);
            //if (tagManagerOptions.maxTags > 0 && tlis.length < tagManagerOptions.maxTags) {
            //  obj.show();
            //}
        },

        init : function (options) {
            var opts = $.extend({}, defaults, options), delimiters, keyNums;

            opts.hiddenTagListName = (opts.hiddenTagListName === null)
                ? 'hidden-' + this.attr('name')
                : opts.hiddenTagListName;

            delimiters = opts.delimeters || opts.delimiters; // 'delimeter' is deprecated
            keyNums = [9, 13, 17, 18, 19, 37, 38, 39, 40]; // delimiter values to be handled as key codes
            opts.delimiterChars = [];
            opts.delimiterKeys = [];

            $.each(delimiters, function (i, v) {
                if ($.inArray(v, keyNums) !== -1) {
                    opts.delimiterKeys.push(v);
                } else {
                    opts.delimiterChars.push(v);
                }
            });

            opts.baseDelimiter = String.fromCharCode(opts.delimiterChars[0] || 44);
            opts.tagBaseClass = 'tm-tag';
            opts.inputBaseClass = 'tm-input';

            if (!$.isFunction(opts.validator)) { opts.validator = null; }

            this.each(function() {
                var $self = $(this), hiddenObj ='', rndid ='', albet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

                // prevent double-initialization of TagManager
                if ($self.data('tagManager')) { return false; }
                $self.data('tagManager', true);

                for (var i = 0; i < 5; i++) {
                  rndid += albet.charAt(Math.floor(Math.random() * albet.length));
                }

                $self.data("tm_rndid", rndid);

                // store instance-specific data in the DOM object
                $self.data('opts',opts)
                    .data('tlis', []) //list of string tags
                    .data('tlid', []); //list of ID of the string tags

                if (opts.output === null) {
                    hiddenObj = $('<input/>', {
                        type: 'hidden',
                        name: opts.hiddenTagListName
                    });
                    $self.after(hiddenObj);
                    $self.data("lhiddenTagList", hiddenObj);
                } else {
                    $self.data("lhiddenTagList", $(opts.output));
                }

                if (opts.AjaxPushAllTags) {
                    $self.on('tm:spliced', privateMethods.pushAllTags);
                    $self.on('tm:popped', privateMethods.pushAllTags);
                    $self.on('tm:pushed', privateMethods.pushAllTags);
                }

                // hide popovers on focus and keypress events
                $self.on('focus keypress', function(e) {
                    if ($(this).popover) { $(this).popover('hide'); }
                });

                // handle ESC (keyup used for browser compatibility)
                if (opts.isClearInputOnEsc) {
                    $self.on('keyup', function(e) {
                        if (e.which === 27) {
                            // console.log('esc detected');
                            $(this).val('');
                            privateMethods.killEvent(e);
                        }
                    });
                }

                $self.on('keypress', function(e) {
                    // push ASCII-based delimiters
                    if (privateMethods.keyInArray(e, opts.delimiterChars)) {
                        privateMethods.applyDelimiter.call($self, e);
                    }
                });

                $self.on('keydown', function(e) {
                    // disable ENTER
                    if (e.which === 13) {
                        if (opts.preventSubmitOnEnter) {
                            privateMethods.killEvent(e);
                        }
                    }

                    // push key-based delimiters (includes <enter> by default)
                    if (privateMethods.keyInArray(e, opts.delimiterKeys)) {
                        privateMethods.applyDelimiter.call($self, e);
                    }
                });

                // BACKSPACE (keydown used for browser compatibility)
                if (opts.deleteTagsOnBackspace) {
                    $self.on('keydown', function(e) {
                        if (privateMethods.keyInArray(e, opts.backspace)) {
                            // console.log("backspace detected");
                            if ($(this).val().length <= 0) {
                                publicMethods.popTag.call($self);
                                privateMethods.killEvent(e);
                            }
                        }
                    });
                }

                // on tag pop fill back the tag's content to the input field
                if (opts.fillInputOnTagRemove) {
                    $self.on('tm:popped', function(e, tag) {
                        $(this).val(tag);
                    });
                }

                $self.change(function(e) {
                    if (!/webkit/.test(navigator.userAgent.toLowerCase())) {
                        $self.focus();
                    } // why?

                    /* unimplemented mode to push tag on blur
                     else if (tagManagerOptions.pushTagOnBlur) {
                     console.log('change: pushTagOnBlur ' + tag);
                     pushTag($(this).val());
                     } */
                    privateMethods.killEvent(e);
                });

                if (opts.prefilled !== null) {
                    if (typeof (opts.prefilled) === "object") {
                        privateMethods.prefill.call($self, opts.prefilled);
                    } else if (typeof (opts.prefilled) === "string") {
                        privateMethods.prefill.call($self, opts.prefilled.split(opts.baseDelimiter));
                    } else if (typeof (opts.prefilled) === "function") {
                        privateMethods.prefill.call($self, opts.prefilled());
                    }
                } else if (opts.output !== null) {
                    if ($(opts.output) && $(opts.output).val()) { var existing_tags = $(opts.output); }
                    privateMethods.prefill.call($self,$(opts.output).val().split(opts.baseDelimiter));
                }

            });

            return this;
        }
    };

    $.fn.tagsManager = function(method) {
        var $self = $(this);

        if (!(0 in this)) { return this; }

        if ( publicMethods[method] ) {
            return publicMethods[method].apply( $self, Array.prototype.slice.call(arguments, 1) );
        } else if ( typeof method === 'object' || ! method ) {
            return privateMethods.init.apply( this, arguments );
        } else {
            $.error( 'Method ' +  method + ' does not exist.' );
            return false;
        }
    };

}(jQuery));

define("tagsManager", function(){});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/husky-select',[
    'type/default',
    'form/util'
], function(Default) {

    'use strict';

    return function($el, options) {
        var defaults = {
                id: 'id',
                label: 'value',
                required: false
            },

            typeInterface = {
                setValue: function(data) {

                    if (data === undefined || data === '' || data === null) {
                        return;
                    }

                    if (typeof data === 'object') {
                        this.$el.data({
                            'selection': data[this.options.id],
                            'selectionValues': data[this.options.label]
                        }).trigger('data-changed');
                    } else {
                        this.$el.data({
                            'selection': data
                        }).trigger('data-changed');
                    }
                },

                getValue: function() {
                    // For single select
                    var data = {},
                        ids = this.$el.data('selection'),
                        values = this.$el.data('selection-values'),
                        returnValue = this.$el.attr('data-mapper-return-value');

                    if (!ids || ids.length === 0) {
                        return undefined;
                    }

                    // check if 'data-mapper-return-value' is defined
                    if (typeof returnValue !== 'undefined') {
                        if (returnValue === 'id') {
                            return Array.isArray(ids) ? ids[0] : ids;
                        } else if(returnValue === 'value'){
                            return Array.isArray(values) ? values[0] : values;
                        }
                    }
                    // return value if property type is set to string
                    if (this.$el.attr('data-mapper-property-type') === 'string') {
                        return Array.isArray(values) ? values[0] : values;
                    }

                    data[this.options.label] = Array.isArray(values) ? values[0] : values;
                    data[this.options.id] = Array.isArray(ids) ? ids[0] : ids;

                    return data;
                },

                needsValidation: function() {
                    var val = this.getValue();
                    return !!val;
                },

                validate: function() {
                    var value = this.getValue();
                    if (typeof value === 'object' && value.hasOwnProperty('id')) {
                        return !!value.id;
                    } else {
                        return value !== '' && typeof value !== 'undefined';
                    }
                }
            };

        return new Default($el, defaults, options, 'husky-select', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

define('type/husky-auto-complete',[
    'type/default',
    'form/util'
], function(Default) {

    'use strict';

    return function($el, options) {
        var defaults = {
                id: 'id',
                name: 'name',
                required: false
            },

            typeInterface = {
                setValue: function(data) {
                    var $input = $el.find('input');

                    if (!$input.length) {
                        $el.data('value', data);
                        $el.attr('data-value', JSON.stringify(data));
                    }

                    $input.data('id', data[this.options.id]);
                    $input.attr('data-id', data[this.options.id]);
                    $input.typeahead('val', data[this.options.name]);
                },

                getValue: function() {
                    var $input = $el.find('input');

                    if (!$input.data('id') || $input.data('id') === 'null') {
                        return;
                    }

                    var value = {};

                    value[this.options.id] = $input.data('id');
                    value[this.options.name] = $input.val();

                    return value;
                },

                needsValidation: function() {
                    var val = this.getValue();
                    return !!val;
                },

                validate: function() {
                    var value = this.getValue();
                    if (typeof value === 'object' && value.hasOwnProperty('id')) {
                        return !!value.id;
                    } else {
                        return value !== '' && typeof value !== 'undefined';
                    }
                }
            };

        return new Default($el, defaults, options, 'husky-auto-complete', typeInterface);
    };
});

/*
 * This file is part of the Sulu CMS.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

define('services/husky/url-validator',[],function() {

    'use strict';

    var constants = {
            regex: "(?:\\S+(?::\\S*)?@)?(?:(?!10(?:\\.\\d{1,3}){3})(?!127(?:\\.\\d{1,3}){3})(?!169\\.254(?:\\.\\d{1,3}){2})(?!192\\.168(?:\\.\\d{1,3}){2})(?!172\\.(?:1[6-9]|2\\d|3[0-1])(?:\\.\\d{1,3}){2})(?:[1-9]\\d?|1\\d\\d|2[01]\\d|22[0-3])(?:\\.(?:1?\\d{1,2}|2[0-4]\\d|25[0-5])){2}(?:\\.(?:[1-9]\\d?|1\\d\\d|2[0-4]\\d|25[0-4]))|(?:(?:[a-z\\u00A1-\\uFFFF0-9]+-?)*[a-z\\u00A1-\\uFFFF0-9]+)(?:\\.(?:[a-z\\u00A1-\\uFFFF0-9]+-?)*[a-z\\u00A1-\\uFFFF0-9]+)*(?:\\.(?:[a-z\\u00A1-\\uFFFF]{2,})))(?::\\d{2,5})?(?:\/[^\\s]*)?",
            regexOptions: 'im',
            defaultProtocols: ['http://', 'https://', 'ftp://', 'ftps://'],
            specificPartKey: 3,
            schemeKey: 1,
            urlKey: 0
        },

        getRegexp = function(schemes) {
            var schemeNames = _.map(schemes, function(scheme) {
                    return scheme.replace('://', '');
                }),
                hasSameScheme = schemes.indexOf('//') > -1,
                schemeRegex = [
                    (hasSameScheme ? '((' : '('),
                    schemeNames.join('|'),
                    (hasSameScheme ? '):)?' : '):'),
                    '\\/{2}'
                ].join(''),
                regexParts = [];

            regexParts.push('^(');
            if (schemes.length > 0) {
                regexParts.push(schemeRegex);
                regexParts.push(')(');
            }
            regexParts.push(constants.regex);
            regexParts.push(')$');

            return new RegExp(regexParts.join(''), constants.regexOptions);
        };

    function Validator() {
    }

    /**
     * Returns default schemes.
     *
     * @returns {Array}
     */
    Validator.prototype.getDefaultSchemes = function() {
        return constants.defaultProtocols;
    };

    /**
     * Tests given URL against internal regex.
     *
     * @param {String} url
     * @param {Array} schemes if NULL default schemes will be used
     *
     * @returns {Boolean}
     */
    Validator.prototype.test = function(url, schemes) {
        if (!schemes) {
            schemes = this.getDefaultSchemes();
        }

        return getRegexp(schemes).test(url);
    };

    /**
     * Returns parts of URL.
     *
     * @param {String} url
     * @param {Array} schemes if NULL default schemes will be used
     *
     * @returns {{url, scheme, specificPart}}
     */
    Validator.prototype.match = function(url, schemes) {
        if (!schemes) {
            schemes = this.getDefaultSchemes();
        }

        var match = url.match(getRegexp(schemes));

        if (!match) {
            return match;
        }

        return {
            scheme: match[constants.schemeKey],
            specificPart: match[constants.specificPartKey]
        };
    };

    return new Validator();
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/husky-input',[
    'type/default',
    'services/husky/url-validator'
], function(Default, urlValidator) {

    'use strict';

    return function($el, options) {
        var defaults = {
                id: 'id',
                label: 'value',
                required: false,
                dateFormat: 'd',
                timeFormat: 't'
            },

            typeValidators = {
                phone: function(value) {
                    var regex = /^[0-9 \\(\\)/+-]*$/;
                    return regex.test(value);
                },
                password: function(value) {
                    var regex = /^.{6,30}$/;
                    return regex.test(value);
                },
                email: function(value) {
                    var regex = /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))){2,6}$/i;
                    return regex.test(value);
                },
                url: function(value) {
                    return urlValidator.test(value, []);
                },
                time: function(value) {
                    return Globalize.parseDate(value, this.options.timeFormat) !== null;
                },
                color: function(value) {
                    // hex color with leading #
                    var regex = /^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/;
                    return regex.test(value);
                }
            },

            typeSetter = {
                default: function(data) {
                    this.$el.data({
                        value: data
                    }).trigger('data-changed');
                },

                time: function(data) {
                    var dateTime = Globalize.parseDate(data, 'HH:mm:ss');

                    typeSetter.default.call(this, Globalize.format(dateTime, this.options.timeFormat));
                }
            },

            typeGetter = {
                default: function() {
                    return this.$el.find('input').val();
                },
                time: function() {
                    var value = typeGetter.default.call(this),
                        dateTime = Globalize.parseDate(value, this.options.timeFormat);

                    if (value === '') {
                        return value;
                    }

                    return Globalize.format(dateTime, 'HH:mm:ss');
                },
                date: function() {
                    return this.$el.data('value');
                }
            },

            typeInterface = {
                setValue: function(data) {
                    var type = this.$el.data('auraSkin'),
                        setter = typeSetter[type] || typeSetter.default;

                    setter.call(this, data);
                },

                getValue: function() {
                    var type = this.$el.data('auraSkin'),
                        getter = typeGetter[type] || typeGetter.default;

                    return getter.call(this);
                },

                needsValidation: function() {
                    var val = this.getInputValue();

                    return val !== '';
                },

                getInputValue: function() {
                    var type = this.$el.data('auraSkin');

                    if (type === 'data') {
                        return typeGetter.date.call(this);
                    }

                    return typeGetter.default.call(this);
                },

                validate: function() {
                    var value = this.getInputValue(),
                        type = this.$el.data('auraSkin');

                    if (!!type && !!typeValidators[type]) {
                        return typeValidators[type].call(this, value);
                    } else {
                        return true;
                    }
                }
            };

        return new Default($el, defaults, options, 'husky-input', typeInterface);
    };
});

/*
 * This file is part of the Husky Validation.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

define('type/url-input',[
    'type/default',
    'services/husky/url-validator'
], function(Default, urlValidator) {

    'use strict';

    return function($el, options) {
        var defaults = {
                schemes: urlValidator.getDefaultSchemes()
            },

            constants = {
                dataKey: 'url-data',
                dataChangedEvent: 'data-changed'
            },

            typeInterface = {
                setValue: function(data) {
                    var match = urlValidator.match(data, this.options.schemes);

                    if (!!match) {
                        this.$el.data(
                            constants.dataKey,
                            match
                        );
                        this.$el.trigger(constants.dataChangedEvent)
                    }
                },

                getValue: function() {
                    var data = this.$el.data(constants.dataKey);

                    if (!!data && !!data.scheme && !!data.specificPart) {
                        return data.scheme + data.specificPart;
                    }

                    return null
                },

                needsValidation: function() {
                    return this.getValue() !== null;
                },

                validate: function() {
                    var url = this.getValue();

                    return urlValidator.test(url, this.options.schemes);
                }
            };

        return new Default($el, defaults, options, 'url-input', typeInterface);
    };
});

/*
 * This file is part of the Sulu CMS.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

require.config({
    paths: {
        'sprintf': 'bower_components/sprintf/sprintf'
    }
});

define('services/husky/util',['sprintf'], function(sprintf) {

    'use strict';

    var instance = null,

        /**
         * Replace rules for escape html function
         * @type {{}}
         */
        entityMap = {
            "&": "&amp;",
            "<": "&lt;",
            ">": "&gt;",
            '"': '&quot;',
            "'": '&#39;',
            "/": '&#x2F;'
        },

        /**
         * From a given image-tag, with an already loaded image. This function returns the
         * base64 string for that image
         *
         * @param {Object} $img The image dom element
         * @param {string} mimeType The mime type of the image
         *
         * @returns {string} the base64 string
         */
        getBlobFromImageTag = function($img, mimeType) {
            var deferred = $.Deferred(), canvas = document.createElement('canvas'), ctx;

            canvas.width = $img.get(0).width;
            canvas.height = $img.get(0).height;
            ctx = canvas.getContext('2d');
            ctx.drawImage($img.get(0), 0, 0);

            canvas.toBlob(function(blob) {
                deferred.resolve(blob);
            }, mimeType);

            return deferred.promise();
        },

        /**
         * Calculates the number of matches parts of the first array
         * have with parts of the second array
         *
         * @param {Array} parts1
         * @param {Array} parts2
         *
         * @returns {number}
         */
        getSuccessfulMatches = function(parts1, parts2) {
            var successfulMatches = 0, lastMatchedIndex = -1, partMatched, i, j;

            // all parts of the first array are tried to match against a single part of the second array
            for (i = 0; i < parts1.length; i++) {
                partMatched = false;

                // for each not matched part in the second array
                for (j = lastMatchedIndex + 1; j < parts2.length; j++) {
                    // try to match it with the current part of the first array
                    if (parts1[i] === parts2[j]) {
                        successfulMatches += 1;
                        lastMatchedIndex = i;
                        partMatched = true;
                        break;
                    }
                }
                // after a part is not matched the counting is finished
                if (!partMatched) {
                    break;
                }
            }

            return successfulMatches;
        };

    function Util() {
    }

    // for comparing arrays
    Util.prototype.compare = function(a, b) {
        if (typeof a === 'object' && typeof b === 'object') {
            return JSON.stringify(a) === JSON.stringify(b);
        }
    };

    // Crockfords better typeof
    Util.prototype.typeOf = function(value) {
        var s = typeof value;
        if (s === 'object') {
            if (value) {
                if (value instanceof Array) {
                    s = 'array';
                }
            } else {
                s = 'null';
            }
        }

        return s;
    };

    Util.prototype.isEqual = _.isEqual;

    Util.prototype.isEmpty = _.isEmpty;

    /**
     * cool guy loop implementation of foreach: http://jsperf.com/loops3/2
     * returns -> callback(value, index)
     */
    Util.prototype.foreach = function(array, callbackValue) {
        if (array.length && array.length > 0) {
            for (var i = -1, length = array.length; ++i < length;) {
                callbackValue(array[i], i);
            }
        }
    };

    Util.prototype.load = function(url, data, dataType) {
        var deferred = new $.Deferred(),
            settings = {
                url: url,
                data: data || null,
                dataType: 'json',

                success: function(data, textStatus) {
                    deferred.resolve(data, textStatus);
                }.bind(this),

                error: function(jqXHR, textStatus, error) {
                    deferred.reject(textStatus, error);
                }
            };

        if (typeof(dataType) !== 'undefined') {
            settings.dataType = dataType;
        }
        this.ajax(settings);

        return deferred.promise();
    };

    Util.prototype.save = function(url, type, data) {
        var deferred = $.Deferred();

        this.ajax({

            headers: {
                'Content-Type': 'application/json'
            },

            url: url,
            type: type,
            data: JSON.stringify(data),

            success: function(data, textStatus) {
                deferred.resolve(data, textStatus);
            }.bind(this),

            error: function(jqXHR, textStatus, error) {
                deferred.reject(jqXHR, textStatus, error);
            }
        });

        return deferred.promise();
    };

    Util.prototype.cropMiddle = function(text, maxLength, delimiter) {
        var substrLength;

        // return text if it doesn't need to be cropped
        if (!text || text.length <= maxLength || !text.slice) {
            return text;
        }

        // default delimiter
        if (!delimiter) {
            delimiter = '...';
        }

        substrLength = Math.floor((maxLength - delimiter.length) / 2);

        return text.slice(0, substrLength) + delimiter + text.slice(-substrLength);
    };

    Util.prototype.cropFront = function(text, maxLength, delimiter) {
        if (!text || text.length <= maxLength || !text.slice) {
            return text;
        }

        delimiter = delimiter || '...';

        return delimiter + text.slice(-(maxLength - delimiter.length));
    };

    Util.prototype.cropTail = function(text, maxLength, delimiter) {
        if (!text || text.length <= maxLength || !text.slice) {
            return text;
        }

        delimiter = delimiter || '...';

        return text.slice(0, (maxLength - delimiter.length)) + delimiter;
    };

    Util.prototype.contains = function(list, value) {
        return _.contains(list, value);
    };

    Util.prototype.isAlphaNumeric = function(str) {
        var code, i, len;

        for (i = 0, len = str.length; i < len; i++) {
            code = str.charCodeAt(i);
            if (!(code > 47 && code < 58) && // numeric (0-9)
                !(code > 64 && code < 91) && // upper alpha (A-Z)
                !(code > 96 && code < 123)) { // lower alpha (a-z)
                return false;
            }
        }

        return true;
    };

    Util.prototype.uniqueId = function(prefix) {
        return _.uniqueId(prefix);
    };

    Util.prototype.delay = function(delay, callback) {
        return _.delay(delay, callback);
    };

    Util.prototype.union = function() {
        return _.union.apply(this, arguments);
    };

    Util.prototype.each = $.each;

    Util.prototype.ajax = $.ajax;

    Util.prototype.ajaxError = function(callback) {
        $(document).ajaxError(callback);
    };

    Util.prototype.when = function() {
        return $.when.apply(null, arguments);
    };

    Util.prototype.deepCopy = function(object) {
        var parent = {};

        if ($.isArray(object)) {
            parent = [];
        }

        return $.extend(true, parent, object);
    };

    /**
     * Takes a two dimensional array and returns a given column as a one-dimensional array
     * @param data {Array}
     * @param propertyName {String}
     * @returns {Array}
     */
    Util.prototype.arrayGetColumn = function(data, propertyName) {
        if (Util.prototype.typeOf(data) === 'array' &&
            data.length > 0 &&
            Util.prototype.typeOf(data[0]) === 'object') {
            var values = [];
            Util.prototype.foreach(data, function(el) {
                values.push(el[propertyName]);
            }.bind(this));
            return values;
        } else {
            return data;
        }
    };

    /**
     * Removes elements from one array from another array and returns the result
     * @param array {Array} The array to remove from
     * @param remove {Array} containing the elements which sould be removed
     * @returns {Array}
     */
    Util.prototype.removeFromArray = function(array, remove) {
        return $.grep(array, function(value) {
            return remove.indexOf(value) == -1;
        });
    };

    /**
     * Returns a parameter value from a given url
     * Found at http://stackoverflow.com/a/901144
     * Has limitations e.g. for parameters like a[asf]=value
     * @param name {string} name of the parameter to search for
     * @param url {string}
     * @returns {string}
     */
    Util.prototype.getParameterByName = function(name, url) {
        name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
        var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
            results = regex.exec(url);

        return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
    };

    Util.prototype.template = _.template;

    Util.prototype.extend = $.extend;

    Util.prototype.Deferred = $.Deferred;

    Util.prototype.arrayMap = _.map;

    Util.prototype.object = _.object;

    /**
     * Escapes special html character
     * @param string
     * @returns {string}
     */
    Util.prototype.escapeHtml = function(string) {
        return String(string).replace(/[&<>"'\/]/g, function(s) {
            return entityMap[s];
        });
    };

    /**
     * Makes the first letter uppercase
     * @param string
     * @returns {string}
     */
    Util.prototype.capitalizeFirstLetter = function(string) {
        if (!string.slice) {
            return string;
        }

        return string.charAt(0).toUpperCase() + string.slice(1);
    };

    /**
     * Returns human readable byte string.
     *
     * @param {int} bytes
     * @returns {String}
     */
    Util.prototype.formatBytes = function(bytes){
        if (bytes === 0) {
            return '0 Byte';
        }
        var k = 1000,
            sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'],
            i = Math.floor(Math.log(bytes) / Math.log(k));

        return (bytes / Math.pow(k, i)).toFixed(2) + ' ' + sizes[i];
    };

    /**
     * Compares two url, using their individual parts (sections between "/")
     * The function tries to match each part of the first url to a part
     * in the second url.
     *
     * @param {String} matchUrl The url where each part is tried to match
     * @param {String} toMatchUrl The url against which parts are matched
     * @returns {number} The difference between the number of matched parts and not matched parts.
     */
    Util.prototype.compareURIs = function(matchUrl, toMatchUrl) {
        var matchParts = matchUrl.split('/'),
            toMatchParts = toMatchUrl.split('/'),
            successfulMatches,
            countNotMatchedParts;

        successfulMatches = getSuccessfulMatches(matchParts, toMatchParts);
        countNotMatchedParts = matchParts.length - successfulMatches;

        return successfulMatches - countNotMatchedParts;
    };

    /**
     * Return a formatted string.
     *
     * @param {String} format
     * @param {String...} arguments
     *
     * @return {String}
     */
    Util.prototype.sprintf = sprintf.sprintf;

    /**
     * Loads an image from a given url and converts it into a blob.
     *
     * @param {String} imageUrl The url to load the image from
     *
     * @returns {Object} a promise which gets resolved with the image blob
     */
    Util.prototype.loadImageAsBlob = function(imageUrl, mimeType) {
        var $container = $('<div/>'),
            $img = $('<img crossOrigin="Anonymous" src="' + imageUrl + '"/>'),
            whenImageLoaded = $.Deferred();

        $container.width(0);
        $container.height(0);
        $container.css('visibility', 'hidden');
        $('body').append($container);

        $img.on('load', function() {
            getBlobFromImageTag($img, mimeType).then(function(blob) {
                $container.remove();
                whenImageLoaded.resolve(blob);
            });
        });
        $container.append($img);

        return whenImageLoaded;
    };

    Util.getInstance = function() {
        if (instance == null) {
            instance = new Util();
        }

        return instance;
    };

    return Util.getInstance();
});

/*
 * This file is part of the Sulu CMS.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

define('services/husky/mediator',[],function() {

    'use strict';

    var instance = null;

    function Mediator() {}

    Mediator.getInstance = function() {
        if (instance == null) {
            instance = new Mediator();
        }
        return instance;
    };

    Mediator.prototype.on = window.Husky.on;

    Mediator.prototype.once = window.Husky.once;

    Mediator.prototype.off = window.Husky.off;

    Mediator.prototype.emit = window.Husky.emit;

    return Mediator.getInstance();
});

/*
 * This file is part of the Sulu CMS.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

define('services/husky/translator',[],function() {

    'use strict';

    var instance = null;

    function Translator() {}

    Translator.getInstance = function() {
        if (instance == null) {
            instance = new Translator();
        }
        return instance;
    };

    Translator.prototype.translate = window.Husky.translate;

    return Translator.getInstance();
});

define('bower_components/aura/lib/platform',[],function() {
  // The bind method is used for callbacks.
  //
  // * (bind)[https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/Function/bind]
  // * (You don't need to use $.proxy)[http://www.aaron-powell.com/javascript/you-dont-need-jquery-proxy]
  // * credits: taken from bind_even_never in this discussion: https://prototype.lighthouseapp.com/projects/8886/tickets/215-optimize-bind-bindaseventlistener#ticket-215-9
  if (typeof Function.prototype.bind !== "function") {
    Function.prototype.bind = function(context) {
       var fn = this, args = Array.prototype.slice.call(arguments, 1);
       return function(){
          return fn.apply(context, Array.prototype.concat.apply(args, arguments));
       };
    };
  }

  // Returns true if an object is an array, false if it is not.
  //
  // * (isArray)[https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/Array/isArray]
  if (typeof Array.isArray !== "function") {
    Array.isArray = function(vArg) {
      return Object.prototype.toString.call(vArg) === "[object Array]";
    };
  }

  // Creates a new object with the specified prototype object and properties.
  //
  // * (create)[https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/Object/create]

  if (!Object.create) {
    Object.create = function (o) {
      if (arguments.length > 1) {
        throw new Error('Object.create implementation only accepts the first parameter.');
      }
      function F() {}
      F.prototype = o;
      return new F();
    };
  }
  // Returns an array of a given object's own enumerable properties, in the same order as that provided by a for-in loop
  // (the difference being that a for-in loop enumerates properties in the prototype chain as well).
  //
  // (keys)[https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/Object/keys]
  if (!Object.keys) {
    Object.keys = (function () {
      var ownProp = Object.prototype.hasOwnProperty,
          hasDontEnumBug = !({toString: null}).propertyIsEnumerable('toString'),
          dontEnums = [
            'toString',
            'toLocaleString',
            'valueOf',
            'hasOwnProperty',
            'isPrototypeOf',
            'propertyIsEnumerable',
            'constructor'
          ],
          dontEnumsLength = dontEnums.length;

      return function (obj) {
        if (typeof obj !== 'object' && typeof obj !== 'function' || obj === null) {
          throw new TypeError('Object.keys called on non-object');
        }

        var result = [];

        for (var prop in obj) {
          if (ownProp.call(obj, prop)) {
            result.push(prop);
          }
        }

        if (hasDontEnumBug) {
          for (var i=0; i < dontEnumsLength; i++) {
            if (ownProp.call(obj, dontEnums[i])) {
              result.push(dontEnums[i]);
            }
          }
        }
        return result;
      };
    })();
  }


});

(function() {
  if (window.jQuery) {
    define('jquery', [], function () {
      return window.jQuery;
    });
  } else {
    require.config({
      paths: {
        jquery:     'bower_components/jquery/jquery'
      },
      shim: {
        jquery:     { exports: '$' }
      }
    });
  }

  if (window._) {
    define('underscore', [], function () {
      return window._;
    });
  } else {
    require.config({
      paths: {
        underscore: 'bower_components/underscore/underscore'
      },
      shim: {
        underscore: { exports: '_' }
      }
    });
  }

  define('bower_components/aura/lib/base',['module', 'underscore', 'jquery', './platform'], function(module, _, $, platform) {

    // Auto configure aura path... if not set yet...
    if (!require.s.contexts._.config.paths.aura) {
      require.config({ paths: { aura: module.id.replace(/base$/, '') }});
    }

    var base = {};

    base.dom = {
      find: function(selector, context) {
        context = context || document;
        return $(context).find(selector);
      },
      contains: function(selector, context) {
        return $.contains(context || document, selector);
      },
      data: function(selector, attribute) {
        return $(selector).data(attribute);
      }
    };

    base.data = {
      deferred: $.Deferred,
      when: $.when
    };

    base.util = {
      each: $.each,
      extend: $.extend,
      uniq: _.uniq,
      _: _,
      decamelize: function(camelCase, delimiter) {
        delimiter = (delimiter === undefined) ? '_' : delimiter;
        return camelCase.replace(/([A-Z])/g, delimiter + '$1').toLowerCase();
      }
    };

    base.events = {
      listen: function(context, events, selector, callback) {
        return $(context).on(events, selector, callback);
      },
      bindAll: function() {
        return _.bindAll.apply(this, arguments);
      }
    };

    base.template = {
      parse: _.template
    };

    return base;

  });

})();

define('bower_components/aura/lib/logger',[], function() {
  'use strict';

  var noop    = function() {},
      console = window.console || {};

  function Logger(name) {
    this.name   = name;
    this._log   = noop;
    this._warn  = noop;
    this._error = noop;
    this._enabled = false;
    return this;
  }

  Logger.prototype.isEnabled = function () {
    return this._enabled;
  };

  Logger.prototype.setName = function(name){
    this.name = name;
  };

  Logger.prototype.enable = function() {
    if (Function.prototype.bind && typeof console === "object") {
      var logFns = ["log", "warn", "error"];
      for (var i = 0; i < logFns.length; i++) {
        console[logFns[i]] = Function.prototype.call.bind(console[logFns[i]], console);
      }
    }

    this._log     = (console.log   || noop);
    this._warn    = (console.warn  || this._log);
    this._error   = (console.error || this._log);
    this._enabled = true;

    return this;
  };

  Logger.prototype.write = function(output, args){
    var parameters = Array.prototype.slice.call(args);
    parameters.unshift(this.name + ":");
    output.apply(console, parameters);
  };

  Logger.prototype.log = function() {
    this.write(this._log, arguments);
  };

  Logger.prototype.warn = function() {
    this.write(this._warn, arguments);
  };

  Logger.prototype.error = function() {
    this.write(this._error, arguments);
  };

  return Logger;
});

define('bower_components/aura/lib/aura.extensions',['./base', './logger'], function(base, Logger) {

  var _ = base.util._,
      slice = Array.prototype.slice,
      deferred = base.data.deferred,
      when     = base.data.when,
      logger   = new Logger('Extensions').enable();

  function ExtManager() {
    this._extensions  = [];
    this.initStatus   = deferred();
    return this;
  }

  //---------------------------------------------------------------------------
  // Public API
  //---------------------------------------------------------------------------

  ExtManager.prototype.add = function(ext) {
    if (_.include(this._extensions, ext)) {
      var msg =  ext.ref.toString() + " is already registered.";
          msg += "Extensions can only be added once.";
      throw new Error(msg);
    }

    if (this.initStarted) {
      throw new Error("Init extensions already called");
    }

    this._extensions.push(ext);
    return this;
  };

  ExtManager.prototype.onReady = function(fn) {
    this.initStatus.then(fn);
    return this;
  };

  ExtManager.prototype.onFailure = function(fn) {
    this.initStatus.fail(fn);
    return this;
  };

  ExtManager.prototype.init = function() {

    if (this.initStarted) {
      throw new Error("Init extensions already called");
    }

    this.initStarted = true;

    var extensions    = _.compact(this._extensions.slice(0)),
        initialized   = [],
        initStatus    = this.initStatus;

    // Enforce sequencial loading of extensions.
    // The `initStatus` promise resolves to the
    // actually resolved and loaded extensions.
    (function _init(extDef) {
      if (extDef) {
        var ext = initExtension(extDef);
        initialized.push(ext);
        ext.done(function () { _init(extensions.shift()); });
        ext.fail(function (err) {
          if (!err) {
            err = "Unknown error while loading an extension";
          }
          if (!(err instanceof Error)) {
            err = new Error(err);
          }
          initStatus.reject(err);
        });
      } else if (extensions.length === 0) {
        when.apply(undefined, initialized).done(function () {
          initStatus.resolve(Array.prototype.slice.call(arguments));
        });
      }
    })(extensions.shift());

    return initStatus.promise();
  };

  //---------------------------------------------------------------------------
  // Private API
  //---------------------------------------------------------------------------

  /*!
   * Helper function that returns the first function found among its arguments.
   * If no function if found, it return a noop (empty function).
   *
   * @return {[type]} [description]
   */
  function getFn() {
    var funcs = slice.call(arguments), fn;
    for (var f = 0, l = funcs.length; f < l; f++) {
      fn = funcs[f];
      if (typeof(fn) === 'function') { return fn; }
    }
    return function () {};
  }

  /*!
   * If the value of the first argument is a function then invoke
   * it with the rest of the args, otherwise, return it.
   */
  function getVal(val) {
    if (typeof val === 'function') {
      return val.apply(undefined, slice.call(arguments, 1));
    } else {
      return val;
    }
  }

  /*!
  * Actual extension loading.
  *
  * The sequence is:
  *
  * * resolves the extension reference
  * * register and requires its dependencies if any
  * * init the extension
  *
  * This method also returns a promise that allows
  * to keep track of the app's loading sequence.
  *
  * If the extension provides a `afterAppStart` method,
  * the promise will resolve to that function that
  * will be called at the end of the app loading sequence.
  *
  * @param {String|Object|Function} extDef the reference and context of the extension
  */

  function initExtension(extDef) {
    var dfd       = deferred(),
        ref       = extDef.ref,
        context   = extDef.context;

    var req = requireExtension(ref, context);
    req.fail(dfd.reject);
    req.done(function (ext) {

      // The extension did not return anything,
      // but probably already did what it had to do.
      if (!ext) { return dfd.resolve(); }

      // Let's initialize it then...
      // If ext is a function, call it
      // Else If ext has a init method, call it
      var init = when(getFn(ext, ext.initialize)(context));
      init.done(function () { dfd.resolve(ext); });
      init.fail(dfd.reject);
    });
    return dfd.promise();
  }

  /*!
  * Extension resolution before actual loading.
  * If `ext` is a String, it is considered as a reference
  * to an AMD module that has to be loaded.
  *
  * This method returns a promise that resolves to the actual extension,
  * With all its dependencies already required' too.
  *
  * @param {String|Object|Function} ext the reference of the extension
  * @param {Object} context the thing this extension is supposed to extend
  */

  function requireExtension(ext, context) {
    var dfd = deferred();

    var resolve = function (ext) {
      ext = getVal(ext, context);
      if (ext && ext.require && ext.require.paths) {
        var deps = Object.keys(ext.require.paths) || [];
        require.config(ext.require);
        require(deps, function() {
          dfd.resolve(ext);
        }, reject);
      } else {
        dfd.resolve(ext);
      }
    };

    var reject = function (err) {
      logger.error("Error loading ext:", ext, err);
      dfd.reject(err);
    };

    if (typeof ext === 'string') {
      require([ext], resolve, reject);
    } else {
      resolve(ext);
    }

    return dfd;
  }

  return ExtManager;
});

define('bower_components/aura/lib/aura',[
  './base',
  './aura.extensions',
  './logger'
  ], function(base, ExtManager, Logger) {

  var _ = base.util._,
      noop     = function() {},
      freeze   = Object.freeze || noop;

  /**
  * Aura constructor and main entry point
  *
  * Every instance of Aura defines an Aura application.
  * An Aura application is in charge of loading the various
  * extensions that will apply to it (defined either
  * programmatically or by way of configuration).
  *
  * An Aura application is the glue between all the extensions
  * and components inside its instance.
  *
  * Internally an Aura application wraps 4 important objects:
  *
  * - `config` is the object passed as the first param of the apps constructor
  * - `core`   is a container where the extensions add new features
  * - `sandbox` is an object that will be used as a prototype, to create fresh sandboxes to the components
  * - `extensions` An instance of the ExtensionManager. It contains all the extensions that will be loaded in the app.
  *
  * Extensions are here to provide features that will be used by the components...
  * They are meant to extend the apps' core & sandbox.
  * They also have access to the apps's config.
  *
  * Example of a creation of an Aura Application:
  *
  *     var app = aura({ key: 'value' });
  *     app.use('ext1').use('ext2');
  *     app.components.addSource('supercomponents', 'https://my.extern.al/components/store');
  *     app.start('body');
  *
  * @class Aura
  * @param {Object} [config] Main App config.
  * @method constructor
  */
  function Aura(config) {

    if (!(this instanceof Aura)) {
      return new Aura(config);
    }

    var extManager = new ExtManager(),
        app = this;

    /**
     * The App's internal unique reference.
     *
     * @property {String} ref
     */
    app.ref = _.uniqueId('aura_');

    /**
     * The App's config object
     *
     * @property {Object} config
     */
    app.config = config = config || {};
    app.config.sources = app.config.sources || { "default" : "./aura_components" };


    /*!
     * App Sandboxes
     */

    var appSandboxes = {};
    var baseSandbox = Object.create(base);

    /**
     * Namespace for sanboxes related methods.
     *
     * @type {Object}
     */
    app.sandboxes = {};

    /**
     * Creates a brand new sandbox, using the App's `sandbox` object as a prototype.
     *
     * @method sandboxes.create
     * @param   {String} [ref]      the Sandbox unique ref
     * @param   {Object} [options]  an object to that directly extends the Sandbox
     * @return  {Sandbox}
     */
    app.sandboxes.create = function (ref, options) {

      // Making shure we have a unique ref
      ref = ref || _.uniqueId('sandbox-');
      if (appSandboxes[ref]) {
        throw new Error("Sandbox with ref " + ref + " already exists.");
      }

      // Create a brand new sandbox based on the baseSandbox
      var sandbox = Object.create(baseSandbox);

      // Give it a ref
      sandbox.ref = ref || _.uniqueId('sandbox-');

      // Attach a Logger
      sandbox.logger = new Logger(sandbox.ref);

      // Register it in the app's sandboxes registry
      appSandboxes[sandbox.ref] = sandbox;

      var debug = config.debug;
      if(debug === true || (debug.enable && (debug.components.length === 0 || debug.components.indexOf(ref) !== -1))){
        sandbox.logger.enable();
      }

      // Extend if we have some options
      _.extend(sandbox, options || {});

      return sandbox;
    };

    /**
     * Get a sandbox by reference.
     *
     * @method sandboxes.get
     * @param  {String} ref  the Sandbox ref to retreive
     * @return {Sandbox}
     */
    app.sandboxes.get = function(ref) {
      return appSandboxes[ref];
    };

    /**
     * Tells the app to load the given extension.
     *
     * Aura extensions are loaded in the app during init.
     * they are responsible for :
     *
     * - resolving & loading external dependencies via requirejs
     * - they have direct access to the app's internals
     * - they are here to add new features to the app... that are made available through the sandboxes to the components.
     *
     * This method can only be called before the App is actually started.
     * Note that the App is started when its `start` method is called.
     *
     * @method use
     * @param  {String | Object | Function} ref the reference of the extension
     * @return {Aura} the Aura app object
     * @chainable
     */
    app.use = function(ref) {
      extManager.add({ ref: ref, context: app });
      return app;
    };

    /**
     * Namespace for components related methods.
     *
     * @type {Object}
     */
    app.components = {};

    /**
     * Adds a new source for components.
     * A Component source is an endpoint (basically a URL)
     * that is the root of a component repository.
     *
     * @method  components.addSource
     * @param {String} name    the name of the source.
     * @param {String} baseUrl the base url for those components.
     */
    app.components.addSource = function(name, baseUrl) {
      if (config.sources[name]) {
        throw new Error("Components source '" + name + "' is already registered");
      }
      config.sources[name] = baseUrl;
      return app;
    };

    /**
     * `core` is just a namespace used by the extensions to add features to the App.
     *
     * @property {Object} core
     */
    app.core = Object.create(base);

    /**
     * Application start.
     *
     * Bootstraps the extensions loading process.
     * All the extensions are resolved and loaded when `start` id called.
     * Start returns a promise that shall fail if any of the
     * extensions fails to load.
     *
     * The app's responsibility is to load its extensions,
     * make sure they are properly loaded when the app starts
     * and eventually make sure they are properly cleaned up when the app stops
     *
     * @method start
     * @param  {Object | String | Array} options start options.
     * @return {Promise} a promise that resolves when the app is started.
     */
    app.start = function(options) {

      if (app.started) {
        app.logger.error("Aura already started!");
        return extManager.initStatus;
      }

      app.logger.log("Starting Aura");

      var startOptions = options || {};

      // Support for different types of options (string, DOM Selector or Object)
      if (typeof options === 'string') {
        startOptions = { components: app.core.dom.find(options) };
      } else if (Array.isArray(options)) {
        startOptions = { components: options };
      } else if (options && options.widgets && !options.components) {
        startOptions.components = options.widgets;
      } else if (startOptions.components === undefined) {
        startOptions.components = app.core.dom.find(app.config.components || 'body');
      }

      extManager.onReady(function(exts) {
        // Then we call all the `afterAppStart` provided by the extensions
        base.util.each(exts, function(i, ext) {
          if (ext && typeof(ext.afterAppStart) === 'function') {
            ext.afterAppStart(app);
          }
        });
      });

      // If there was an error in the boot sequence we
      // reject every body and perform a cleanup
      // TODO: Provide a better error message to the user.
      extManager.onFailure(function() {
        app.logger.error("Error initializing app:", app.config.name, arguments);
        app.stop();
      });

      app.startOptions = startOptions;
      app.started = true;

      // Finally... we return a promise that allows
      // to keep track of the loading process...
      return extManager.init();
    };

    /**
     * Stops the application and unregister its loaded dependencies.
     *
     * @method stop
     * @return {void}
     */
    app.stop = function() {
      // TODO: We ne to actually do some cleanup here.
      app.started = false;
    };

    /**
     * Check if any component is not stopped correctly and still referenced by aura.
     * If that is the case we will stop the component and remove it.
     * TODO: Consider using 'MutationObserver' for cleanup
     *
     * @method cleanUp
     * @return {void}
     */
    app.cleanUp = function () {
      _.defer(function() {    
        _.each(appSandboxes, function(sandbox) {
          if (!!sandbox && !!sandbox.el && !app.core.dom.contains($(sandbox.el)[0], document)) {
            sandbox.stop();
          }
        });
      });
    };

    /**
     * Sandboxes are a way to implement the facade pattern on top of the features provided by `core`.
     *
     * The `sandbox` property of an Aura App is just an Object that will be used
     * as a blueprint, once the app is started, to create new instances
     * of sandboxed environments for the Components.
     * @property {Object} sandbox
     */
    app.sandbox = baseSandbox;

    /**
     * App Logger
     *
     * @property {Logger} logger
     */
    app.logger     = new Logger(app.ref);

    /**
     * @class Sandbox
     */

    /**
     * Stop method for a Sandbox.
     * If no arguments provided, the sandbox itself and all its children are stopped.
     * If a DOM Selector is provided, all matching children will be stopped.
     *
     * @param  {undefined|String} DOM Selector
     */
    app.sandbox.stop = function(selector) {
      // Stop sandbox directly if the selector is a ref and which _ref can be
      // found
      var sandbox = app.sandboxes.get(selector);
      
      if (sandbox) {
        stopSandbox(sandbox);
      }

      // Return early if selector is invalid
      try {
        $.find(selector);
      } catch (err) {
        return err;
      }

      if (selector) {
        app.core.dom.find(selector, this.el).each(function(i, el) {
          var ref = app.core.dom.find(el).data('__sandbox_ref__');
          stopSandbox(ref);
        });
      } else {
        stopSandbox(this);
      }
    };

    function stopSandbox(sandbox) {
      if (typeof sandbox === 'string') {
        sandbox = app.sandboxes.get(sandbox);
      }
      if (sandbox) {
        var event = ['aura', 'sandbox', 'stop'].join(app.config.mediator.delimiter);
        _.invoke(sandbox._children, 'stop');
        app.core.mediator.emit(event, sandbox);
        if (sandbox._component) {
          // remove is deprecated 
          sandbox._component.invokeWithCallbacks('remove');
          sandbox._component.invokeWithCallbacks('destroy');
        }
        sandbox.stopped  = true;
        sandbox.el && app.core.dom.find(sandbox.el).remove();
        appSandboxes[sandbox.ref] = null;
        delete appSandboxes[sandbox.ref]; // do we need to actually call `delete` ?
        return sandbox;
      }
    }

    // Register core extensions : debug, mediator and components.
    config.debug = config.debug || {};
    var debug = config.debug;
    if (debug === true) {
      config.debug = debug = {
        enable: true
      };
    }
    if (debug.enable) {
      if(debug.components){
        debug.components = debug.components.split(' ');
      }else{
        debug.components = [];
      }
      app.logger.enable();
      app.use('aura/ext/debug');
    }

    app.use('aura/ext/mediator');
    app.use('aura/ext/components');

    return app;
  }

  return Aura;
});

define('husky',[
    'bower_components/aura/lib/aura'
], function(Aura) {

    'use strict';

    // husky - subclass of aurajs
    function Husky(config) {

        if (!(this instanceof Husky)) {
            return new Husky(config);
        }

        Aura.call(this, config);  // call super constructor.

        var app = this;

        app.components.addSource('husky', './husky_components');

        app.use('./husky_extensions/jquery');
        app.use('./husky_extensions/backbone');
        app.use('./husky_extensions/collection');
        app.use('./husky_extensions/model');
        app.use('./husky_extensions/html.sortable');
        app.use('./husky_extensions/husky-validation');
        app.use('./husky_extensions/util');
        app.use('./husky_extensions/confirm');
        app.use('./husky_extensions/template');
        app.use('./husky_extensions/globalize');
        app.use('./husky_extensions/uri-template');
        app.use('./husky_extensions/ckeditor-extension');
		app.use('./husky_extensions/typeahead');
        app.use('./husky_extensions/tagmanager');
        app.use('./husky_extensions/dropzone');
        app.use('./husky_extensions/colorpicker');
        app.use('./husky_extensions/datepicker');
        app.use('./husky_extensions/itembox');
        app.use('./husky_extensions/itemgrid');
        app.use('./husky_extensions/cache-factory');
        app.use('./husky_extensions/infinite-scroll');

        app.use(function(app) {
            window.Husky = app.sandboxes.create('husky-sandbox');
        });
    }

    // subclass extends superclass
    Husky.prototype = Object.create(Aura.prototype);
    Husky.prototype.constructor = Husky;

    // Husky version flag
    Husky.prototype.version = '0.1.0';

    return Husky;
});

/*
 * This file is part of the Sulu CMS.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * Name: navigation
 *
 * Description:
 *  Navigation Element
 *
 * Options:
 *  footerTemplate - html template to define the footer
 *
 * Provides Events:
 *  husky.navigation.footer.set - set template
 *
 * Triggers Events:
 *  husky.navigation.item.select - {item}
 *  husky.navigation.item.toggle - isExpanded, {item}
 *  husky.navigation.item.settings - {item}
 *
 */

// TODO: arrow keys
// TODO: action as link for ux -> footer where it will link
// TODO: cogwheel at dashboard in the middle
// TODO: events as specified
// TODO: move functions into private space
// TODO: cleanup css-classes
// TODO: complete yui-doc

define('__component__$navigation@husky',[],function() {

    'use strict';

    var templates = {
            /** component skeleton */
            skeleton: [
                '<nav class="navigation<% if (collapsed === "true") {%> collapsed<% } %>">',
                '   <div class="navigation-content">',
                '       <div class="fa-chevron-left navigation-close-icon"></div>',
                '       <div class="wrapper">',
                '           <header class="navigation-header">',
                '               <div class="logo"></div>',
                '               <div class="navigation-header-title"><% if (data.title) { %> <%= translate(data.title) %><% } %></div>',
                '           </header>',
                '           <div id="navigation-item-container" class="navigation-item-container"></div>',
                '       </div>',
                '       <footer>',
                '       </footer>',
                '   </div>',
                '</nav>'].join(''),
            /** main navigation items (with icons)*/
            mainItem: [
                '<li class="js-navigation-items navigation-items" id="<%= item.id %>" data-id="<%= item.id %>">',
                '   <div <% if (!!item.items && item.items.length > 0) { %> class="navigation-items-toggle" <% } %> >',
                '       <a class="<% if (!!item.action || !!item.event) { %>js-navigation-item <% } %>navigation-item" href="#">',
                '           <span class="<%= icon %> navigation-item-icon"></span>',
                '           <span class="navigation-item-title"><%= translate(item.title) %></span>',
                '       </a>',
                '       <% if (!!item.items && item.items.length > 0) { %>',
                '           <a class="fa-chevron-right navigation-toggle-icon" href="#"></a>',
                '       <% } %>',
                '   </div>',
                '</li>'].join(''),
            /** sub navigation items */
            subToggleItem: [
                '   <li class="js-navigation-items navigation-subitems" id="<%= item.id %>" data-id="<%= item.id %>">',
                '       <div class="navigation-subitems-toggle">',
                '           <a class="<% if (!!item.action || !!item.event) { %>js-navigation-item <% } %> navigation-item" href="#"><%= translate(item.title) %></a>',
                '           <a class="fa-chevron-right navigation-toggle-icon" href="#"></a>',
                '       </div>',
                '</li>'].join(''),
            /** siple sub item */
            subItem: [
                '<li class="js-navigation-sub-item" id="<%= item.id %>" data-id="<%= item.id %>">',
                '   <a href="#"><%= translate(item.title) %></a>',
                '</li>'
            ].join(''),
            //footer template
            footer: [
                '<div class="user">',
                '   <span class="fa-user pic"></span>',
                '   <div class="name"><%= userName %></div>',
                '</div>',
                '<div class="options">',
                '   <div class="locale-dropdown"></div>',
                '       <a href="<%= logoutRoute %>" title="Logout" class="fa-lock logout"></a>',
                '   </div>',
                '<div class="version"><%= system %> (<%= version %>, <a href="#"><%= versionHistory %></a>)</div>'
            ].join('')
        },
        defaults = {
            footerTemplate: '',
            collapsed: false,
            resizeWidth: 1300,
            forceCollapse: false,
            systemName: 'Sulu 2.0',
            footer: true,
            logoutRoute: '/logout',
            translations: {
                versionHistory: 'navigation.version-history',
                user: 'navigation.user'
            }
        },
        CONSTANTS = {
            COMPONENT_CLASS: 'husky-navigation',
            UNCOLLAPSED_WIDTH: 250, //px
            COLLAPSED_WIDTH: 50, //px
            ITEM_LABEL_HEIGHT: 50, //px
            TRANSITIONEND_EVENT: 'transitionend webkitTransitionEnd oTransitionEnd otransitionend MSTransitionEnd',
            HIDDEN_CLASS: 'disappeared'
        },

        namespace = 'husky.navigation.',

        /**
         * raised when header was clicked
         * @event husky.navigation.header.clicked
         */
        EVENT_HEADER_CLICKED = namespace + 'header.clicked',

        /**
         * raised when item is selected
         * @event husky.navigation.item.select
         * @param {Object} item The selected item
         */
        EVENT_ITEM_SELECT = namespace + 'item.select',

        /**
         * listens on and activates the matching navigation-item
         * @event husky.navigation.select-item
         * @param {string} selectAction The select-action to match the navigation-item for
         */
        EVENT_SELECT_ITEM = namespace + 'select-item',

        /**
         * listens on and activates the matching navigation-item
         * @event husky.navigation.select-id
         * @param {string} id The id of item which should be selected
         * @param {Object} options Additional options
         */
        EVENT_SELECT_ID = namespace + 'select-id',

        /**
         * raised when navigation was collapsed
         * @event husky.navigation.collapsed
         * @param {Number} width The width of the collapsed navigation
         */
        EVENT_COLLAPSED = namespace + 'collapsed',

        /**
         * raised when navigation was un-collapsed
         * @event husky.navigation.uncollapsed
         * @param {Number} width The width of thewidth un-collapsed navigation
         * @param {Boolean} true iff the uncollapsing-process was forced
         */
        EVENT_UNCOLLAPSED = namespace + 'uncollapsed',

        /**
         * forces navigation to collapse
         * @event husky.navigation.collapse
         * @param {Boolean} stayCollapsed - if true the navigation stays collapsed till the custom-uncollapse event is emited
         */
        EVENT_COLLAPSE = namespace + 'collapse',

        /**
         * forces navigation to uncollapse
         * @event husky.navigation.uncollapse
         * @param {Bool} force If true, the navigation will act as overflow and collapse again after interaction
         */
        EVENT_UNCOLLAPSE = namespace + 'uncollapse',


        /**
         * raised when navigation collapse / uncollapse of navigation is triggered. called before transition starts
         * only raised when not forced
         * @event husky.navigation.size.change
         */
        EVENT_SIZE_CHANGE = namespace + 'size.change',

        /**
         * raised when navigation was un / collapsed. called when transition is finished. only raised when not forced
         * @event husky.navigation.size.changed
         */
        EVENT_SIZE_CHANGED = namespace + 'size.changed',

        /**
         * raised when the user locale is changed
         * @event husky.navigation.user-locale.changed
         */
        USER_LOCALE_CHANGED = namespace + 'user-locale.changed',

        /**
         * raised when the version history is clicked
         * @event husky.navigation.version-history.clicked
         */
        VERSION_HISTORY_CLICKED = namespace + 'version-history.clicked',

        /**
         * raised when the username gets clicked
         * @event husky.navigation.username.clicked
         */
        USERNAME_CLICKED = namespace + 'username.clicked',

        /**
         * show the navigation when it was hidden before
         * @event husky.navigation.show
         */
        EVENT_SHOW = namespace + 'show',

        /**
         * triggers the update of the navigation size
         * @event husky.navigation.show.size
         */
        EVENT_SIZE_UPDATE = namespace + 'size.update',

        /**
         * hides the navigation completely
         * @event husky.navigation.hide
         */
        EVENT_HIDE = namespace + 'hide';


    return {

        initialize: function() {

            this.sandbox.logger.log('Initialized Navigation');

            // merging options
            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

            this.stayCollapsed = false;
            this.hidden = false;
            this.tooltipsEnabled = true;
            this.animating = false;
            this.items = []; // array to save all navigation items

            // binding dom events
            this.bindDOMEvents();

            this.bindCustomEvents();

            // load Data
            if (!!this.options.url) {
                this.sandbox.util.load(this.options.url, null, 'json')
                    .then(function(data) {
                        this.options.data = data;
                        this.render();
                        this.sandbox.emit('husky.navigation.initialized');
                    }.bind(this))
                    .fail(function(data) {
                        this.sandbox.logger.log("data could not be loaded:", data);
                    }.bind(this));
            }
        },

        /**
         * renders the navigation
         * gets called after navigation data was loaded
         */
        render: function() {
            this.sandbox.dom.addClass(this.$el, CONSTANTS.COMPONENT_CLASS);

            this.renderSkeleton()
            this.renderNavigationItems(this.options.data);
            this.renderFooter();

            // collapse if necessary
            this.resizeListener();

            // preselect item based on url
            if (!!this.options.selectAction && this.options.selectAction.length > 0) {
                this.preselectItem(this.options.selectAction);
            }
        },

        /**
         * Renders the main skeleton of the navigation and sets assigns global
         * variables with dom-objects rendered with the skeleton.
         */
        renderSkeleton: function() {
            // render skeleton
            this.sandbox.dom.html(
                this.$el,
                this.sandbox.template.parse(
                    templates.skeleton,
                    this.sandbox.util.extend(true, {}, this.options, {translate: this.sandbox.translate})
                )
            );

            this.$navigation = this.$find('.navigation');
            this.$navigationContent = this.$find('.navigation-content');
        },

        /**
         *  renders main navigation elements
         */
        renderNavigationItems: function(data) {
            var $elem, $sectionDiv, $sectionList;
            // iterate through sections
            this.sandbox.util.foreach(data.items, function(section) {
                if (!section.items || !section.items.length) {
                    return;
                }

                $sectionDiv = this.sandbox.dom.createElement('<div class="section">');
                $sectionList = this.sandbox.dom.createElement('<ul class="section-items">');

                var title;
                if (!!section.title && !!(title = this.sandbox.translate(section.title))) {
                    this.sandbox.dom.append($sectionDiv, '<div class="section-headline"><span class="section-headline-title">' + title.toUpperCase() + '</span></div>');
                } else if (!!$elem && $elem.children().length > 0) {
                    this.sandbox.dom.append($sectionDiv, '<hr class="section-headline-divider"/>');
                }

                this.sandbox.dom.append($sectionDiv, $sectionList);
                this.sandbox.dom.append('#navigation-item-container', $sectionDiv);

                // iterate through section items
                this.sandbox.util.foreach(section.items, function(item) {
                    // create item
                    $elem = this.sandbox.dom.createElement(this.sandbox.template.parse(templates.mainItem, {
                        item: item,
                        icon: item.icon ? 'fa-' + item.icon : '',
                        translate: this.sandbox.translate
                    }));
                    this.sandbox.dom.append($sectionList, $elem);
                    //render sub-items
                    if (item.items && item.items.length > 0) {
                        this.renderSubNavigationItems(item, $elem);
                    }
                    item.domObject = $elem;
                    this.items.push(item);
                }.bind(this));
            }.bind(this));
        },

        /**
         * renders sub-navigation elements
         */
        renderSubNavigationItems: function(data, $parentList) {
            var elem,
                textCont,
                list = this.sandbox.dom.createElement('<ul class="navigation-items-list" />');

            this.sandbox.dom.append($parentList, list);

            this.sandbox.util.foreach(data.items, function(item) {
                item.parentTitle = data.title;
                this.items.push(item);
                if (!!item.items && item.items.length > 0) {
                    elem = this.sandbox.dom.createElement(this.sandbox.template.parse(templates.subToggleItem, {
                        item: item,
                        translate: this.sandbox.translate
                    }));
                    this.renderSubNavigationItems(item, this.sandbox.dom.find('div', elem));
                } else {
                    elem = this.sandbox.dom.createElement(this.sandbox.template.parse(templates.subItem, {
                        item: item,
                        translate: this.sandbox.translate
                    }));
                }
                this.sandbox.dom.append(list, elem);

                // add tooltip
                textCont = this.sandbox.dom.find('a', elem);
                if (this.sandbox.dom.width(textCont) + 20 < this.sandbox.dom.get(textCont, 0).scrollWidth) {
                    this.sandbox.dom.attr(textCont, {'title': this.sandbox.dom.html(textCont)});
                }

                item.domObject = elem;
            }.bind(this));
        },

        /**
         * Renders the footer of the navigation
         */
        renderFooter: function() {
            if (this.options.footer === true) {
                var $footer = this.sandbox.dom.find('footer', this.$el);
                this.sandbox.dom.html($footer, _.template(templates.footer)({
                    userName: this.options.userName,
                    system: this.options.systemName,
                    version: this.options.systemVersion,
                    versionHistory: this.sandbox.translate(this.options.translations.versionHistory),
                    logoutRoute: this.options.logoutRoute
                }));

                this.sandbox.start([
                    {
                        name: 'select@husky',
                        options: {
                            el: this.sandbox.dom.find('.locale-dropdown', $footer),
                            instanceName: 'navigation-locale',
                            value: 'name',
                            data: this.options.userLocales,
                            preSelectedElements: [this.options.userLocale],
                            style: 'small',
                            emitValues: true
                        }
                    }
                ]);
            }
        },

        /**
         * Handles the event when the collapsed Footer is clicked
         */
        collapsedFooterClickHandler: function() {
            this.unCollapse(true);

            // scroll to footer
            this.sandbox.dom.one(this.sandbox.dom.$window, CONSTANTS.TRANSITIONEND_EVENT,
                this.checkBottomHit.bind(this, null, this.sandbox.dom.find('footer', this.$el)));
        },

        /**
         * Interaction
         */
        bindDOMEvents: function() {
            this.sandbox.dom.on(this.$el, 'click', this.toggleItems.bind(this), '.navigation-items-toggle, .navigation-subitems-toggle');

            this.sandbox.dom.on(this.$el, 'click', function(event) {
                event.preventDefault();
                this.selectMenuItem(event.currentTarget);
                _.delay(this.resizeListener.bind(this), 700); // TODO: is this really neccessary?
            }.bind(this), '.js-navigation-sub-item, .js-navigation-item');

            this.sandbox.dom.on(this.$el, 'click', function() {
                this.sandbox.emit(EVENT_HEADER_CLICKED);
            }.bind(this), '.navigation-header div');

            // collapse events
            this.sandbox.dom.on(this.sandbox.dom.window, 'resize', this.resizeListener.bind(this));
            this.sandbox.dom.on(this.$el, 'click', this.collapse.bind(this), '.navigation.collapseIcon .navigation-close-icon');
            this.sandbox.dom.on(this.$el, 'click', this.collapsedFooterClickHandler.bind(this), '.navigation.collapsed footer');

            // tooltip events
            this.sandbox.dom.on(this.$el, 'mouseenter', function(event) {
                this.showToolTip.call(this, this.sandbox.translate(this.options.translations.user), event);
            }.bind(this), '.navigation.collapsed footer');
            this.sandbox.dom.on(this.$el, 'mouseenter', this.showToolTip.bind(this, ''), '.navigation.collapsed .navigation-items');
            this.sandbox.dom.on(this.$el, 'mouseleave', this.hideToolTip.bind(this), '.navigation.collapsed .navigation-items, .navigation.collapsed, .navigation.collapsed footer');

            this.sandbox.dom.on(this.$el, CONSTANTS.TRANSITIONEND_EVENT, function() {
                this.sandbox.emit(EVENT_SIZE_CHANGED, this.sandbox.dom.width(this.$navigation));
                this.tooltipsEnabled = true;
            }.bind(this));
            this.sandbox.dom.on(this.$el, CONSTANTS.TRANSITIONEND_EVENT, function(event) {
                event.stopPropagation();
            }.bind(this), '.navigation *');

            // version history clicked
            this.sandbox.dom.on(this.$el, 'click', function() {
                this.sandbox.emit(VERSION_HISTORY_CLICKED);
            }.bind(this), 'footer .version a');

            // user clicked
            this.sandbox.dom.on(this.$el, 'click', function() {
                this.sandbox.emit(USERNAME_CLICKED);
            }.bind(this), 'footer .user');
        },

        /**
         * event listener
         */
        bindCustomEvents: function() {
            // merge options with newly passed options and rerender footer
            this.sandbox.on('husky.navigation.footer.set', function(options) {
                this.options = this.sandbox.util.extend(true, {}, this.options, options);
                this.renderFooter();
            }.bind(this));

            //user locales dropdown
            this.sandbox.on('husky.select.navigation-locale.selected.item', function(locale) {
                this.sandbox.emit(USER_LOCALE_CHANGED, locale);
            }.bind(this));

            this.sandbox.on(EVENT_COLLAPSE, function(stayCollapsed) {
                this.stayCollapsed = (typeof stayCollapsed === 'boolean') ? stayCollapsed : false;
                this.collapse();
            }.bind(this));
            this.sandbox.on(EVENT_UNCOLLAPSE, function(force) {
                this.stayCollapsed = false;
                this.unCollapse(force);
            }.bind(this));

            this.sandbox.on(EVENT_HIDE, this.hide.bind(this));
            this.sandbox.on(EVENT_SHOW, this.show.bind(this));

            this.sandbox.on(EVENT_SIZE_UPDATE, this.resizeListener.bind(this));

            this.sandbox.on(EVENT_SELECT_ITEM, this.preselectItem.bind(this));
            this.sandbox.on(EVENT_SELECT_ID, this.selectItem.bind(this));
        },

        resizeListener: function() {
            var windowWidth = this.sandbox.dom.width(this.sandbox.dom.window);

            if (windowWidth <= this.options.resizeWidth || this.stayCollapsed === true) {
                this.collapse();
            } else if (this.sandbox.dom.hasClass(this.$navigation, 'collapsed')) {
                this.unCollapse();
            }
        },

        showToolTip: function(title, event) {
            if (this.tooltipsEnabled === true) {
                var offset,
                    target = event.currentTarget;

                if (!title) {
                    title = this.sandbox.dom.html(this.sandbox.dom.find('.navigation-item-title', event.currentTarget));
                }
                if (!this.$tooltip) {
                    this.$tooltip = this.sandbox.dom.createElement('<div class="navigation-tooltip">' + title + '</div>');
                    this.sandbox.dom.append('body', this.$tooltip);
                }
                offset = this.sandbox.dom.offset(target);
                this.sandbox.dom.css(this.$tooltip, {
                    top: offset.top + (this.sandbox.dom.outerHeight(target) - 30) / 2
                });
            }
        },

        hideToolTip: function() {
            if (!!this.$tooltip) {
                this.sandbox.dom.remove(this.$tooltip);
                this.$tooltip = null;
            }
        },

        /**
         * Selects a navigation item by a given action. It selects the navigation point
         * which's action is most similar to the given action. If no satisfiable navigation item
         * can be found it deselects the currently selected action.
         * @param selectAction {String} the action for which a navigation item should be selected
         */
        preselectItem: function(selectAction) {
            var maxMatchValue = 0, matchValue, matchItem, parent, match;
            this.sandbox.util.foreach(this.items, function(item) {
                if (!item || !item.action) {
                    return;
                }
                matchValue = this.sandbox.util.compareURIs(item.action, selectAction);
                if (matchValue > maxMatchValue) {
                    matchItem = item;
                    maxMatchValue = matchValue;
                }
            }.bind(this));

            if (!!matchItem && maxMatchValue > 0) {
                match = matchItem.domObject;

                if (this.sandbox.dom.hasClass(match, 'js-navigation-sub-item')) {
                    parent = this.sandbox.dom.closest(match, '.navigation-items');

                    // toggle parent only when it is not expaneded
                    if (!this.sandbox.dom.hasClass(parent, 'is-expanded')) {
                        this.toggleItems(null, parent);
                    }
                }
                this.selectMenuItem(match, false);
                this.checkBottomHit(null, match);
            } else {
                this.unmarkSelectedItem();
            }
        },


        /**
         * Select item with given id
         * @param id {string}
         * @param options {options}
         */
        selectItem: function(id, options) {
            var item = this.getItemById(id),
                domObject = item.domObject,
                parent;

            if ($(domObject).hasClass('is-selected')) {
                return;
            }

            if (this.sandbox.dom.hasClass(domObject, 'js-navigation-sub-item')) {
                parent = this.sandbox.dom.closest(domObject, '.navigation-items');

                // toggle parent only when it is not expanded
                if (!this.sandbox.dom.hasClass(parent, 'is-expanded')) {
                    this.toggleItems(null, parent);
                }
            }
            this.selectMenuItem(domObject, false, options);
            this.checkBottomHit(null, domObject);
        },

        /**
         * Toggles menu element with submenu
         * Raises navigation.toggle
         * @param event
         * @param customTarget
         */
        toggleItems: function(event, customTarget) {

            if (event) {
                event.preventDefault();
            } else {
                event = {
                    currentTarget: customTarget
                };
            }

            var $items = this.sandbox.dom.closest(event.currentTarget, '.js-navigation-items'),
                $childList = this.sandbox.dom.find('ul:first', $items),
                item, $toggle,
                isExpanded = this.sandbox.dom.hasClass($items, 'is-expanded'),
                navWasCollapsed;

            // only check collapse if event was fired
            if (!customTarget) {
                navWasCollapsed = this.showIfCollapsed();
            }

            if (isExpanded && !navWasCollapsed) {
                $toggle = this.sandbox.dom.find('.fa-chevron-down', event.currentTarget);
                this.animateSlideUp($items, $toggle);
            } else if (this.collapsed !== true) {
                $toggle = this.sandbox.dom.find('.fa-chevron-right', event.currentTarget);
                this.animateSlideDown($items, $toggle, $childList);
            }

            // emit event
            item = this.getItemById(this.sandbox.dom.data($items, 'id'));
            this.sandbox.emit('husky.navigation.item.toggle', !isExpanded, item);
        },

        /**
         * Handles the slide-up animation
         * @param $items
         * @param $toggle
         */
        animateSlideUp: function($items, $toggle) {
            if (this.animating === false) {
                this.animating = true;

                this.sandbox.dom.animate($items, {
                    height: CONSTANTS.ITEM_LABEL_HEIGHT
                }, {
                    duration: 180,
                    complete: function() {
                        this.animating = false;
                    }.bind(this)
                });

                this.sandbox.dom.removeClass($items, 'is-expanded');

                // change toggle item
                this.sandbox.dom.removeClass($toggle, 'fa-chevron-down');
                this.sandbox.dom.prependClass($toggle, 'fa-chevron-right');
            }
        },

        /**
         * Handles the slide-down animation
         * @param $items
         * @param $toggle
         * @param $childList
         */
        animateSlideDown: function($items, $toggle, $childList) {
            if (this.animating === false) {
                this.animating = true;
                var expandedHeight = CONSTANTS.ITEM_LABEL_HEIGHT + this.sandbox.dom.outerHeight($childList);

                this.sandbox.dom.stop($items);
                this.sandbox.dom.animate($items, {
                    height: (expandedHeight + 30)
                }, {
                    duration: 120
                });

                this.sandbox.dom.animate($items, {
                    height: expandedHeight
                }, {
                    duration: 100,
                    complete: function() {
                        this.animating = false;
                    }.bind(this)
                });

                this.sandbox.dom.addClass($items, 'is-expanded');

                // change toggle item
                this.sandbox.dom.removeClass($toggle, 'fa-chevron-right');
                this.sandbox.dom.prependClass($toggle, 'fa-chevron-down');

                this.sandbox.dom.one($items, CONSTANTS.TRANSITIONEND_EVENT, this.checkBottomHit.bind(this));
            }
        },

        checkBottomHit: function(event, customTarget) {
            var xBottom, windowHeight, itemHeight, itemTop, scrollTop,
                $items;

            if (event) {
                $items = event.currentTarget;
            } else {
                $items = customTarget;
            }
            // check if collapsed element overlaps browser border
            itemTop = this.sandbox.dom.offset($items).top;
            itemHeight = this.sandbox.dom.height($items);
            xBottom = itemTop + itemHeight;
            windowHeight = this.sandbox.dom.height(this.sandbox.dom.window);
            scrollTop = this.sandbox.dom.scrollTop(this.$navigationContent);
            if (xBottom > windowHeight) {
                if (itemHeight < windowHeight) {
                    this.sandbox.dom.scrollAnimate(scrollTop + (xBottom - windowHeight + 40), this.$navigationContent);
                } else {
                    this.sandbox.dom.scrollAnimate(itemTop, this.$navigationContent);
                }
            }
        },

        /**
         * Removes the inline-style for expanded items
         */
        removeHeightforExpanded: function() {
            var $expandedItems = this.sandbox.dom.find('.is-expanded', this.$el), i, length;
            for (i = -1, length = $expandedItems.length; ++i < length;) {
                this.sandbox.dom.removeAttr($expandedItems[i], 'style');
            }
        },

        /**
         * Resets the inline-style height for expanded items
         */
        setHeightForExpanded: function() {
            var $expandedItems = this.sandbox.dom.find('.is-expanded', this.$el),
                height, i, length;
            for (i = -1, length = $expandedItems.length; ++i < length;) {
                height = CONSTANTS.ITEM_LABEL_HEIGHT + this.sandbox.dom.outerHeight(
                    this.sandbox.dom.find('.navigation-items-list', $expandedItems[i]));

                this.sandbox.dom.css($expandedItems[i], {
                    'height': height
                });
            }
        },

        // returns if nav was collapsed
        showIfCollapsed: function() {

            if (this.sandbox.dom.hasClass(this.$navigation, 'collapsed')) {
                this.unCollapse(true);
                return true;
            }
            return false;
        },

        collapse: function() {
            if (this.hidden === false) {
                this.sandbox.dom.addClass(this.$navigation, 'collapsed');
                this.sandbox.dom.removeClass(this.$navigation, 'collapseIcon');
                this.removeHeightforExpanded();
                if (!this.collapsed) {
                    this.sandbox.dom.css(this.$el, {'width': ''});
                    this.sandbox.emit(EVENT_COLLAPSED, CONSTANTS.COLLAPSED_WIDTH);
                    this.sandbox.emit(EVENT_SIZE_CHANGE, CONSTANTS.COLLAPSED_WIDTH);
                    this.collapsed = true;
                    this.tooltipsEnabled = false;
                }
            }
        },

        unCollapse: function(forced) {
            if ((this.stayCollapsed === false || forced === true) && this.hidden === false) {
                if (forced) {
                    // freeze width of parent so that the navigation overlaps the content
                    this.sandbox.dom.width(this.$el, this.sandbox.dom.width(this.$navigation));
                    this.sandbox.dom.addClass(this.$navigation, 'collapseIcon');
                } else {
                    this.sandbox.dom.removeClass(this.$navigation, 'collapseIcon');
                    this.sandbox.dom.css(this.$el, {'width': ''});
                }
                this.sandbox.dom.removeClass(this.$navigation, 'collapsed');
                this.hideToolTip();
                this.setHeightForExpanded();
                if (this.collapsed) {
                    this.sandbox.emit(EVENT_UNCOLLAPSED, CONSTANTS.UNCOLLAPSED_WIDTH, forced);
                    if (!forced) {
                        this.sandbox.emit(EVENT_SIZE_CHANGE, CONSTANTS.UNCOLLAPSED_WIDTH);
                    }
                    this.collapsed = !this.collapsed;
                    this.tooltipsEnabled = false;
                }
            }
        },


        /**
         * Selects menu element without submenu
         * Raises navigation.select
         * @param event
         * @param [customTarget] if event is undefined, the target must be passed customly
         * @param emit
         * @param {Object} options Extends the item
         */
        selectMenuItem: function(item, emit, options) {
            var $selectItem = $(item), itemData, extendedItem;

            // if a main navigation point should be selected, select its first sub-point
            if ($selectItem.hasClass('js-navigation-item')) {
                $selectItem = $(item).closest('li');
            }

            // if clicked item is already selected do nothing
            if ($selectItem.hasClass('is-selected')) {
                return;
            }
            // if a toggelable item was clicked nothing should be selected, it just toggles
            if ($(item).parent().hasClass('navigation-items-toggle') || $(item).parent().hasClass('navigation-subitems-toggle')) {
                return;
            }

            itemData = this.getItemById(this.sandbox.dom.data($selectItem, 'id'));
            extendedItem = this.sandbox.util.extend(true, {}, itemData, options);

            // If it is a navigation-point with a custom configured event (like actions), the event gets emitted and nothing else.
            if (!!itemData.event && emit !== false) {
                this.emitItemSelectEvent(extendedItem);
                return;
            }

            if (emit !== false) {
                this.emitItemSelectEvent(extendedItem);
            }

            this.markItemSelected($selectItem);
        },

        /**
         * Marks a given navigation item as selected. The main navigation-point corresponding
         * to the given item gets marked as active
         * @param $item The item to select
         */
        markItemSelected: function($item) {
            this.unmarkSelectedItem();
            $item.addClass('is-selected');
            $item.closest('.js-navigation-items').addClass('is-active');
        },

        /**
         * Unmarks the currently selected navigation item
         */
        unmarkSelectedItem: function() {
            this.sandbox.dom.removeClass(this.$find('.is-selected'), 'is-selected');
            this.sandbox.dom.removeClass(this.$find('.is-active'), 'is-active');
        },

        /**
         * Emits a select event for a given item. If the item has a custom event configured, the
         * custom event gets emitted, otherwise a default aura event gets emitted.
         * @param itemData The item data
         */
        emitItemSelectEvent: function(itemData) {
            if (itemData.event) {
                this.sandbox.emit('husky.navigation.item.event.' + itemData.event, itemData.eventArgs);
            } else {
                this.sandbox.emit(EVENT_ITEM_SELECT, itemData);
            }
        },

        /**
         * Takes an id and returns the matching item
         * @param id {number|string} id of the item
         * @returns {object|null}
         */
        getItemById: function(id) {
            for (var i = -1, length = this.items.length; ++i < length;) {
                // api can return string values as id so here "no" typesafe comparison
                if (id == this.items[i].id) {
                    return this.items[i];
                }
            }
            return null;
        },

        /**
         * Shows the navigation
         */
        show: function() {
            this.sandbox.dom.removeClass(this.$navigation, CONSTANTS.HIDDEN_CLASS);
            if (!!this.currentNavigationWidth) {
                this.sandbox.dom.removeAttr(this.$navigation, 'style');
                this.currentNavigationWidth = null;
                this.hidden = false;
            }
        },

        /**
         * Hides the navigation
         */
        hide: function() {
            this.sandbox.dom.addClass(this.$navigation, CONSTANTS.HIDDEN_CLASS);
            this.currentNavigationWidth = this.sandbox.dom.width(this.$navigation);
            this.sandbox.dom.width(this.$navigation, 0);
            this.hidden = true;
        }
    };
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/column-options
 */


/**
 * @class Column-Options
 * @constructor
 *
 * @param {Object} [options] Configuration object
 * @param {String} [options.url] url to fetch data from
 * @param {String} [options.instanceName] name of the instance - will affect events name
 * @param {Object} [options.data] if no url is provided
 * @param {String} [options.trigger] the dom item which opens the component
 * @param {Object} [options.header] Configuration object for the header
 * @param {Boolean} [options.header.disabled] defines if header should be shown
 * @param {String} [options.header.title] headline of the header
 * @param {Object} [options.footer] Configuration object for the header
 * @param {Boolean} [options.footer.disabled] defines if footer should be shown
 * @param {Boolean} [options.hidden] defines if component should be hidden when component is initialized
 */
define('__component__$column-options@husky',[],function() {

    'use strict';

    var defaults = {
            url: null,
            data: [],
            trigger: null,
            instanceName: null,
            header: {
                disabled: false,
                title: 'Column Options'
            },
            footer: {
                disabled: false
            },
            hidden: false
        },

        templates = {
            listItem: [
                '<li class="column-options-list-item" data-id="<%= id %>" draggable="true">',
                '   <span class="move">&#8942;</span>',
                '   <span class="text"><%= title %></span>',
                '   <span class="<%= toggleIcon %> visibility-toggle"></span>',
                '</li>'].join(''),
            header: [
                '<div class="column-options-header">',
                '   <span class="title"><%= title %></span>',
                '   <a href="#" class="fa-times close-button"></a>',
                '</div>'
            ].join('')
        },

        iconClasses = {
            visible: 'fa-eye',
            hidden: 'fa-eye-slash'
        },


        /**
         * triggered when component is completely initialized
         * @event husky.column-options[.INSTANCE_NAME].initialized
         */
        INITIALIZED = function() {
            return getEventName.call(this, 'initialized');

        },

        /**
         * triggered when item was enabled
         * @event husky.column-options[.INSTANCE_NAME].item.enabled
         * @param {Object} item that was enabled
         */
        ENABLED = function() {
            return getEventName.call(this, 'item.enabled');

        },

        /**
         * triggered when item was disabled
         * @event husky.column-options[.INSTANCE_NAME].item.disabled
         * @param {Object} item that was disabled
         */
        DISABLED = function() {
            return getEventName.call(this, 'item.disabled');

        },

        /**
         * triggered when save was clicked
         * @event husky.column-options[.INSTANCE_NAME].saved
         * @param {Array} Contains all visible items
         */
        SAVED = function() {
            return getEventName.call(this, 'saved');

        },

        /**
         * used for receiving all visible columns
         * @event husky.column-options[.INSTANCE_NAME].get-selected
         */
        GET_SELECTED = function() {
            return getEventName.call(this, 'get-selected');

        },

        /**
         * used for receiving all columns
         * @event husky.column-options[.INSTANCE_NAME].get-all
         */
        GET_ALL = function() {
            return getEventName.call(this, 'get-all');

        },


        /**
         * DOM events
         */
        bindDOMEvents = function() {
            this.sandbox.dom.on(this.$el, 'click', customStopPropagation.bind(this), this.$container); // prevent from unwanted events
            this.sandbox.dom.on(this.$el, 'mouseover', onMouseOver.bind(this), 'li');
            this.sandbox.dom.on(this.$el, 'mouseout', onMouseOut.bind(this), 'li');
            this.sandbox.dom.on(this.$el, 'click', toggleVisibility.bind(this), '.visibility-toggle');
        },


        /**
         * custom events
         */
        bindCustomEvents = function() {
            this.sandbox.on(GET_SELECTED.call(this), getSelectedItems.bind(this));
            this.sandbox.on(GET_ALL.call(this), getAllItems.bind(this));
        },

        getEventName = function(postFix) {
            return 'husky.column-options.' + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
        },

        /**
         * returns all items that are visible
         * @param callbackFunction
         */
        getSelectedItems = function(callbackFunction) {
            var id, items,
                $visibleItems = this.sandbox.dom.find('li:not(.disabled)', this.$el);

            items = [];
            this.sandbox.util.foreach($visibleItems, function($domItem) {
                id = this.sandbox.dom.data($domItem, 'id');
                items.push(this.items[id]);
            }.bind(this));

            callbackFunction(items);
        },

        /**
         * returns all items that are visible
         * @param callbackFunction
         */
        getAllItems = function(callbackFunction) {
            var id, items,
                $visibleItems = this.sandbox.dom.find('li', this.$el);

            items = [];
            this.sandbox.util.foreach($visibleItems, function($domItem) {
                id = this.sandbox.dom.data($domItem, 'id');
                items.push(this.items[id]);
            }.bind(this));

            callbackFunction(items);
        },


        onMouseOver = function(event) {
            this.sandbox.dom.addClass(event.currentTarget, 'hover-style');
        },

        onMouseOut = function(event) {
            this.sandbox.dom.removeClass(event.currentTarget, 'hover-style');
        },


        /**
         * function checks if id is set and unique among all items
         * otherwise a new id is generated for the element
         * @param item
         */
        checkItemId = function(item) {
            // if item has no id, generate random id
            if (!item.id || !!this.items[item.id]) {
                do {
                    item.id = this.sandbox.util._.uniqueId();
                } while (!!this.items[item.id]);
            }
        },


        /**
         * gets called when toggle item is clicked
         * opens dropdown submenu
         * @param event
         */
        customStopPropagation = function(event) {
            event.preventDefault();
            event.stopPropagation();
        },

        /**
         * called when save was clicked
         */
        submit = function() {
            var $items = this.sandbox.dom.find('.column-options-list-item', this.$list),
                items = [],
                id;


            // resort array
            this.sandbox.util.foreach($items, function($item) {
                id = this.sandbox.dom.data($item, 'id');
                items.push(this.items[id]);
            }.bind(this));
            this.data = items;

            // make permanent
            this.options.data = this.data;

            getAllItems.call(this, function(items) {
                this.sandbox.emit(SAVED.call(this), items);
            }.bind(this));

        },

        /**
         * close dropdown - callback
         */
        close = function() {
            this.sandbox.dom.remove(this.$el);
        },

        /**
         * renders list items
         */
        renderItems = function() {
            var $listItem;

            // create items array
            this.items = [];

            this.sandbox.util.foreach(this.data, function(item) {
                checkItemId.call(this, item);
                // save to items array
                this.items[item.id] = item;

                // append to list
                $listItem = this.sandbox.dom.createElement(this.sandbox.template.parse(templates.listItem, {
                    id: item.id,
                    toggleIcon: (item.default === true || item.default === 'true') ? '' : iconClasses.visible,
                    title: this.sandbox.translate(item.translation)}));
                this.sandbox.dom.append(this.$list, $listItem);

                // set to disabled
                if (typeof item.disabled !== 'undefined' && item.disabled !== "false" && item.disabled !== false) {
                    toggleVisibility.call(this, {currentTarget: this.sandbox.dom.find('.visibility-toggle', $listItem), doNotEmitEvents: true, preventDefault: function() {
                    }});
                }
            }.bind(this));
        },

        startOverlay = function() {
            if (this.overlayLoaded === false) {
                var $element = this.sandbox.dom.createElement('<div/>');
                this.sandbox.dom.append(this.$el, $element);

                this.sandbox.start([
                    {
                        name: 'overlay@husky',
                        options: {
                            el: $element,
                            container: this.$el,
                            data: this.$list,
                            okCallback: function() {
                                submit.call(this);
                                close.call(this);
                            }.bind(this),
                            closeCallback: function() {
                                close.call(this);
                            }.bind(this),
                            instanceName: 'column-options-' + this.options.instanceName,
                            title: this.options.header.title,
                            openOnStart: !this.options.hidden,
                            removeOnClose: true
                        }
                    }
                ]);

                this.overlayLoaded = true;
            }
        },

        /**
         * toggles the classes of an item
         * @param event
         */
        toggleVisibility = function(event) {
            event.preventDefault();

            var $listItem = this.sandbox.dom.parent(event.currentTarget),
                isDisabled = this.sandbox.dom.hasClass($listItem, 'disabled'),
                id = this.sandbox.dom.data($listItem, 'id'),
                item = this.items[id],
                visible = iconClasses.visible,
                hidden = iconClasses.hidden;

            if (isDisabled) {
                // enable
                this.numVisible++;
                this.sandbox.dom.removeClass(event.currentTarget, hidden);
                this.sandbox.dom.prependClass(event.currentTarget, visible);
            } else {
                // disable
                // one column must stay visible
                if (this.numVisible === 1) {
                    return;
                }

                this.numVisible--;
                this.sandbox.dom.prependClass(event.currentTarget, hidden);
                this.sandbox.dom.removeClass(event.currentTarget, visible);
            }

            this.sandbox.dom.toggleClass($listItem, 'disabled');
            item.disabled = !isDisabled;

            if (!event.doNotEmitEvents) {
                this.sandbox.emit(isDisabled ? ENABLED.call(this) : DISABLED.call(this), item);
            }
        };


    return {

        initialize: function() {
            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);
            this.overlayLoaded = false;

            // define handle object to trigger
            this.options.trigger = this.options.trigger || this.$el;

            // load data and call render
            if (!!this.options.url) {
                this.sandbox.util.load(this.options.url)
                    .then(this.render.bind(this))
                    .fail(function(data) {
                        this.sandbox.logger.log('data could not be loaded:', data);
                    }.bind(this));
            } else if (!!this.options.data) {
                this.render((this.options.data));
            } else {
                this.sandbox.logger.log('no data provided for tabs!');
            }

            bindDOMEvents.call(this);

            bindCustomEvents.call(this);
        },

        /**
         * renders list
         * @param data
         */
        render: function(data) {

            this.numVisible = data.length;

            this.options.data = data;
            // temporary data save
            this.data = this.sandbox.util.extend(true, [], this.options.data);

            this.sandbox.dom.addClass(this.$el, 'column-options-parent');

            // init list
            this.$list = this.sandbox.dom.createElement('<ul class="column-options-list" />');

            // render list items
            renderItems.call(this);

            // make list sortables
            this.sandbox.dom.sortable(this.$list, {handle: '.move'});
            startOverlay.call(this);

            // initialization finished
            this.sandbox.emit(INITIALIZED.call(this));
        },

        /**
         * rerenders list items (reset)
         */
        rerender: function() {
            this.data = this.sandbox.util.extend(true, [], this.options.data);
            this.sandbox.dom.html(this.$list, '');
            renderItems.call(this);
            // make list sortable
            this.sandbox.dom.sortable(this.$list, {handle: '.move'});
        }
    };
});

/**
 * @class TableView (Datagrid Decorator)
 * @constructor
 *
 * @param {Object} [viewOptions] Configuration object
 * @param {Boolean} [options.editable] will not set class is-selectable to prevent hover effect for complete rows
 * @param {Boolean} [options.removeRow] displays in the last column an icon to remove a row
 * @param {Object} [options.selectItem] Configuration object of select item (column)
 * @param {Boolean} [options.selectItem.inFirstCell] If true checkbox is in the first cell. If true checkbox gets its own cell
 * @param {String} [options.selectItem.type] Type of select [checkbox, radio]
 * @param {Boolean} [options.addRowTop] adds row to the top of the table when add row is triggered
 * @param {Array} [options.excludeFields=['id']] array of fields to exclude by the view
 * @param {Boolean} [options.showHead] if TRUE head would be showed
 * @param {Array} [options.icons] array of icons to display
 * @param {String} [options.icons[].icon] the actual icon which sould be displayed
 * @param {String} [options.icons[].column] the id of the column in which the icon should be displayed
 * @param {String} [options.icons[].align] the align of the icon. 'left' org 'right'
 * @param {String} [options.icons[].disableCallback] callback to determine if icon is displayed
 * @param {Function} [options.icons.callback] a callback to execute if the icon got clicked. Gets the id of the data-record as first argument
 * @param {Boolean} [options.hideChildrenAtBeginning] if true children get hidden, if all children are loaded at the beginning
 * @param {String|Number|Null} [options.openChildId] the id of the children to open all parents for. (only relevant in a child-list)
 * @param {String|Number} [options.cssClass] css-class to give the the components element
 * @param {Boolean} [options.highlightSelected] highlights the clicked row when selected
 * @param {String} [options.removeIcon] icon to use for the remove-row item
 * @param {Number} [options.croppedMaxLength] the length to which croppable cells will be cropped on overflow
 * @param {Boolean} [options.openPathToSelectedChildren] true to show path to selected children
 * @param {String} [options.actionIcon] the icon which gets shown in the action click button
 * @param {String|Number} [options.actionIconColumn] the column to add the action icon to. Null for firs column
 * @param {Boolean} [rendered] property used by the datagrid-main class
 * @param {Function} [initialize] function which gets called once at the start of the view
 * @param {Function} [render] function to render data
 * @param {Function} [destroy] function to destroy the view and unbind events
 * @param {Function} [onResize] function which gets automatically executed on window resize
 * @param {Function} [unbindCustomEvents] function to unbind the custom events of this object
 * @param {Boolean} [cropContents] Define if cell contents should be cropped, when options.croppedMaxSize is exceeded
 */
define('husky_components/datagrid/decorators/table-view',[],function() {

    'use strict';

    var defaults = {
            editable: false,
            removeRow: false,
            selectItem: {
                type: 'checkbox',
                inFirstCell: false
            },
            noItemsText: 'This list is empty',
            addRowTop: true,
            excludeFields: [''],
            cssClass: '',
            thumbnailFormat: '50x50',
            showHead: true,
            hideChildrenAtBeginning: true,
            openChildId: null,
            highlightSelected: false,
            icons: [],
            cssClasses: [],
            removeIcon: 'trash-o',
            emptyIcon: 'fa-coffee',
            noImgIcon: 'fa-coffee',
            actionIcon: 'pencil',
            actionIconColumn: null,
            croppedMaxLength: 35,
            openPathToSelectedChildren: true,
            cropContents: true,
            editableOptions: {}
        },

        constants = {
            selectedRowClass: 'selected',
            isSelectableClass: 'is-selectable',
            sortableClass: 'is-sortable',
            skeletonClass: 'husky-table',
            containerClass: 'table-container',
            overflowClass: 'overflow',
            emptyListElementClass: 'empty-list',
            rowRemoverClass: 'row-remover',
            checkboxClass: 'checkbox',
            radioClass: 'radio',
            cellFitClass: 'fit',
            tableClass: 'table',
            rowClass: 'row',
            thumbSrcKey: 'url',
            thumbAltKey: 'alt',
            evenClass: 'even',
            oddClass: 'odd',
            headerCellClass: 'header-cell',
            ascSortedClass: 'sorted-asc',
            descSortedClass: 'sorted-desc',
            headerCellLoaderClass: 'header-loader',
            headerLoadingClass: 'is-loading',
            editableItemClass: 'editable',
            editableInputClass: 'editable-input',
            inputWrapperClass: 'input-wrapper',
            editedErrorClass: 'server-validation-error',
            newRecordId: 'newrecord',
            gridIconClass: 'grid-icon',
            gridBadgeClass: 'badge',
            gridImageClass: 'grid-image',
            childWrapperClass: 'child-wrapper',
            parentClass: 'children-toggler',
            noChildrenClass: 'no-children',
            toggleIconClass: 'toggle-icon',
            collapsedIcon: 'fa-caret-right',
            expandedIcon: 'fa-caret-down',
            checkboxCellClass: 'checkbox-cell',
            thumbnailCellClass: 'thumbnail-cell',
            textContainerClass: 'cell-content',
            renderingClass: 'rendering',
            actionClass: 'has-action',
            inactiveClass: 'inactive',
            childIndent: 28 //px
        },

        selectItems = {
            CHECKBOX: 'checkbox',
            RADIO: 'radio'
        },

        /**
         * Templates used by this class
         */
        templates = {
            skeleton: [
                '<div class="' + constants.skeletonClass + '">',
                '   <div class="' + constants.containerClass + '"></div>',
                '</div>'
            ].join(''),
            table: '<table class="' + constants.tableClass + '"></table>',
            header: '<thead></thead>',
            body: '<tbody></tbody>',
            row: '<tr class="' + constants.rowClass + '"></tr>',
            headerCell: '<th class="' + constants.headerCellClass + '"></th>',
            cell: '<td></td>',
            textContainer: '<span class="' + constants.textContainerClass + '"><%= content %></span>',
            headerCellLoader: '<div class="' + constants.headerCellLoaderClass + '"></div>',
            removeCellContent: '<span class="fa-<%= icon %> ' + constants.rowRemoverClass + '"></span>',
            editableCellContent: {
                string: [
                    '<span class="' + constants.editableItemClass + '"><%= value %></span>',
                    '<div class="' + constants.inputWrapperClass + '">',
                    '   <input type="text" class="form-element husky-validate ' + constants.editableInputClass + '" value="<%= value %>">',
                    '</div>'
                ].join(''),
                select: [
                    '<select class="form-element husky-validate ' + constants.editableInputClass + '">',
                    '<% _.each(options.values, function(title, index) { %>',
                    '   <option value="<%= index %>" <% if (value === index) { %>selected<% } %>><%= translate(title) %></option>',
                    '<% }); %>',
                    '</select>'
                ].join('')
            },
            img: [
                '<div class="' + constants.gridImageClass + '">',
                '<% if (!!src) { %>',
                '   <img alt="<%= alt %>" src="<%= src %>"/>',
                '<% } else { %>',
                '   <div class="<%= noImgIcon %> empty"></div>',
                '<% } %>',
                '</div>'
            ].join(''),
            childWrapper: '<div class="' + constants.childWrapperClass + '"></div>',
            toggleIcon: '<span class="' + constants.toggleIconClass + '"></span>',
            icon: [
                '<span class="', constants.gridIconClass, ' <%= cssClass %> <%= align %>" data-icon-index="<%= index %>">',
                '   <span class="fa-<%= icon %>"></span>',
                '</span>'
            ].join(''),
            badge: [
                '<span class="' + constants.gridBadgeClass + ' <%= cssClass %>">',
                '   <% if(!!icon) { %><span class="fa-<%= icon %>"></span><% } %>',
                '   <% if(!!title) { %><%= title %><% } %>',
                '</span> '
            ].join(''),
            checkbox: [
                '<div class="custom-checkbox">',
                '   <input class="' + constants.checkboxClass + '" type="checkbox" data-form="false"/>',
                '   <span class="icon"></span>',
                '</div>'
            ].join(''),
            radio: [
                '<div class="custom-radio">',
                '    <input class="' + constants.radioClass + '" name="<%= name %>" type="radio" data-form="false"/>',
                '    <span class="icon"></span>',
                '</div>'
            ].join(''),
            empty: [
                '<div class="' + constants.emptyListElementClass + '">',
                '   <div class="<%= emptyIcon %> icon"></div>',
                '   <span><%= text %></span>',
                '</div>'
            ].join('')
        },

        /**
         * used to update the table width and its containers due to responsiveness
         * @event husky.datagrid.update.table
         */
        UPDATE_TABLE = function() {
            return this.datagrid.createEventName.call(this.datagrid, 'update.table');
        },

        /**
         * used to update the table width and its containers due to responsiveness
         * @event husky.datagrid.table.open-child
         * @param {Number|String} id The id of the data-record to open the parents for
         */
        OPEN_PARENTS = function() {
            return this.datagrid.createEventName.call(this.datagrid, 'table.open-parents');
        },

        /**
         * triggered when a radio button inside the datagrid is clicked
         * @event husky.datagrid.table.open-child
         * @param {Number|String} id The id of the data-record to open the parents for
         * @param {String} columnName column name
         */
        RADIO_SELECTED = function() {
            return this.datagrid.createEventName.call(this.datagrid, 'radio.selected');
        },

        /**
         * triggered when children were collapsed
         * @event husky.datagrid.table.children.collapsed
         */
        CHILDREN_COLLAPSED = function() {
            return this.datagrid.createEventName.call(this.datagrid, 'children.collapsed');
        },

        /**
         * triggered when children were expanded
         * @event husky.datagrid.table.children.expanded
         */
        CHILDREN_EXPANDED = function() {
            return this.datagrid.createEventName.call(this.datagrid, 'children.expanded');
        };

    return {

        /**
         * Public methods used by the main datagrid class (start)
         * -------------------------------------------------------------------- */

        /**
         * Initializes the view, gets called only once
         * @param {Object} context The context of the datagrid class
         * @param {Object} options The options used by the view
         */
        initialize: function(context, options) {
            // store context of the datagrid-component
            this.datagrid = context;
            this.keys = {id: this.datagrid.options.idKey};

            // make sandbox available in this-context
            this.sandbox = this.datagrid.sandbox;

            // merge defaults with options
            this.options = this.sandbox.util.extend(true, {}, defaults, options);

            this.editStatuses = {};

            this.bindCustomEvents();
            this.setVariables();
        },

        /**
         * Binds custom related events
         */
        bindCustomEvents: function() {
            this.sandbox.on(UPDATE_TABLE.call(this), this.onResize.bind(this));
            this.sandbox.on(OPEN_PARENTS.call(this), this.openParents.bind(this));

            if (!!this.options.openPathToSelectedChildren) {
                var eventName = '';
                if (!!this.datagrid.options.instanceName) {
                    eventName = 'husky.datagrid.' + this.datagrid.options.instanceName + '.view.rendered';
                } else {
                    eventName = 'husky.datagrid.view.rendered';
                }
                this.sandbox.on(eventName, this.openPathToSelectedChildren.bind(this));
            }
        },

        /**
         * Opens path to all selected children
         */
        openPathToSelectedChildren: function() {
            this.sandbox.util.each(this.datagrid.selectedItems, function(idx, id) {
                this.openParents(id);
            }.bind(this));
        },

        /**
         * Unbinds custom events
         */
        unbindCustomEvents: function() {
            this.sandbox.off(UPDATE_TABLE.call(this), this.onResize.bind(this));
            this.sandbox.off(OPEN_PARENTS.call(this), this.openParents.bind(this));
        },

        /**
         * Method to render data in table view
         */
        render: function(data, $container) {
            this.$el = this.sandbox.dom.createElement(templates.skeleton);
            this.sandbox.dom.append($container, this.$el);
            this.addViewClasses();
            this.addActionIcon();
            this.data = data;
            this.renderTable();
            this.bindDomEvents();
            if (this.datagrid.options.resizeListeners === true) {
                this.onResize();
            }
            if (!!this.options.openChildId) {
                this.openParents(this.options.openChildId);
                this.options.openChildId = null;
            }
            this.renderChildrenHidden = false;
            this.sandbox.dom.removeClass(this.$el, constants.renderingClass);
            this.rendered = true;
        },

        /**
         * Destroys the view
         */
        destroy: function() {
            this.sandbox.stop(this.sandbox.dom.find('*', this.$el));
            this.sandbox.dom.remove(this.$el);
            this.setVariables();
        },

        /**
         * Adds a row to the datagrid
         * @param record {Object} the new record to add
         */
        addRecord: function(record, appendAtBottom) {
            var prepend = (!!appendAtBottom) ? false : this.options.addRowTop;

            this.removeEmptyIndicator();
            this.renderBodyRow(record, prepend);
            this.setAlternateClasses();
        },

        /**
         * Removes a record from the view
         * If the record has child-records, the child records are removed too.
         * @param recordId {Number|String}
         */
        removeRecord: function(recordId) {
            // remove children of record
            this.table.rows[recordId].$el.siblings().each(function(key, sibling) {
                if ($(sibling).data('parent') === recordId) {
                    this.removeRecord($(sibling).data('id'));
                }
            }.bind(this));

            // deselect record to adjust parent-checkboxes
            this.deselectRecord(recordId);

            this.datagrid.removeRecord.call(this.datagrid, recordId);
            this.sandbox.dom.remove(this.table.rows[recordId].$el);
            delete this.table.rows[recordId];
            if (Object.keys(this.table.rows).length === 0) {
                this.toggleSelectAllItem(false);
                this.renderEmptyIndicator();
            }
        },

        /**
         * Handles the responsiveness
         */
        onResize: function() {
            if (this.containerIsOverflown() === true) {
                this.overflowHandler();
            } else {
                this.underflowHandler();
            }
        },

        /**
         * Public methods used by the main datagrid class (end)
         * -------------------------------------------------------------------- */

        /**
         * Sets the components starting properties
         */
        setVariables: function() {
            this.rendered = false;
            this.$el = null;
            this.table = {};
            this.data = null;
            this.rowClicked = {};
            this.isFocusoutHandlerEnabled = this.options.editable === true || this.options.editable === 'true';
            this.renderChildrenHidden = this.options.hideChildrenAtBeginning;
            this.tableCropped = false;
            this.cropBreakPoint = null;
            this.icons = this.sandbox.util.extend(true, [], this.options.icons);
            this.badges = this.sandbox.util.extend(true, [], this.options.badges);
            this.cssClasses = this.sandbox.util.extend(true, [], this.options.cssClasses);
        },

        /**
         * Adds css classes to the view element
         */
        addViewClasses: function() {
            this.sandbox.dom.addClass(this.$el, this.options.cssClass);
            this.sandbox.dom.addClass(this.$el, constants.renderingClass);
            if ((this.options.highlightSelected === true || !!this.options.selectItem) && this.options.editable !== true) {
                this.sandbox.dom.addClass(this.$el, constants.isSelectableClass);
            }
        },

        /**
         * Adds an action-icon to into the first column
         */
        addActionIcon: function() {
            if (typeof this.datagrid.options.actionCallback === 'function'
                && !!this.datagrid.matchings
                && this.datagrid.matchings.length > 0
                && !!this.options.actionIcon
            ) {
                this.icons.push({
                    icon: this.options.actionIcon,
                    column: this.options.actionIconColumn || this.datagrid.matchings[0].attribute,
                    align: 'left',
                    callback: this.datagrid.options.actionCallback,
                    actionIcon: true
                });
            }
        },

        /**
         * Render methods (start)
         * -------------------------------------------------------------------- */

        /**
         * Renders the table
         */
        renderTable: function() {
            this.table.$el = this.sandbox.dom.createElement(templates.table);
            this.table.$container = this.sandbox.dom.find('.' + constants.containerClass, this.$el);
            if (this.options.showHead === true) {
                this.renderHeader();
            }
            this.renderBody();
            this.sandbox.dom.append(this.table.$container, this.table.$el);
            this.setAlternateClasses();
        },

        /**
         * Renders the table header
         */
        renderHeader: function() {
            this.table.header = {};
            this.table.header.$el = this.sandbox.dom.createElement(templates.header);
            this.table.header.$row = this.sandbox.dom.createElement(templates.row);
            this.sandbox.dom.append(this.table.header.$el, this.table.header.$row);
            this.renderHeaderSelectItem();
            this.renderHeaderCells();
            this.renderHeaderRemoveItem();
            this.sandbox.dom.append(this.table.$el, this.table.header.$el);
        },

        /**
         * Renders the select-all checkbox of the header
         */
        renderHeaderSelectItem: function() {
            if (!!this.options.selectItem && !!this.options.selectItem.type) {
                var $cell = this.sandbox.dom.createElement(templates.headerCell);
                this.sandbox.dom.addClass($cell, constants.checkboxCellClass);
                if (this.options.selectItem.type === selectItems.CHECKBOX) {
                    this.sandbox.dom.html($cell, templates.checkbox);
                }
                this.sandbox.dom.prepend(this.table.header.$row, $cell);
            }
        },

        /**
         * Renderes the cells in the header
         */
        renderHeaderCells: function() {
            var $headerCell;
            this.table.header.cells = {};

            this.sandbox.util.foreach(this.datagrid.matchings, function(column) {
                $headerCell = this.sandbox.dom.createElement(templates.headerCell);

                if (!!column.class && typeof column.class === 'string') {
                    this.sandbox.dom.addClass($headerCell, column.class);
                }

                this.sandbox.dom.html($headerCell, this.sandbox.util.template(templates.textContainer)({
                    content: this.sandbox.translate(column.content)
                }));
                this.sandbox.dom.data($headerCell, 'attribute', column.attribute);
                this.table.header.cells[column.attribute] = {
                    $el: $headerCell,
                    sortable: column.sortable
                };
                this.sandbox.dom.append(this.table.header.$row, this.table.header.cells[column.attribute].$el);
                this.setHeaderCellClasses(column.attribute);
            }.bind(this));
        },

        /**
         * Sets css classes on a header cell
         * @param column {String} the column attribute of the column to set the classes for
         */
        setHeaderCellClasses: function(column) {
            if (this.datagrid.options.sortable === true) {
                var $element = this.table.header.cells[column].$el,
                    sortedClass;
                if (this.table.header.cells[column].sortable === true) {
                    this.sandbox.dom.addClass($element, constants.sortableClass);
                    if (column === this.datagrid.sort.attribute) {
                        sortedClass = (this.datagrid.sort.direction === 'asc') ? constants.ascSortedClass : constants.descSortedClass;
                        this.sandbox.dom.addClass($element, sortedClass);
                    }
                }
            }
        },

        /**
         * Renderes an empty remove-row cell into the header
         */
        renderHeaderRemoveItem: function() {
            if (this.options.removeRow === true) {
                var $cell = this.sandbox.dom.createElement(templates.headerCell);
                this.sandbox.dom.addClass($cell, constants.cellFitClass);
                this.sandbox.dom.append(this.table.header.$row, $cell);
            }
        },

        /**
         * Renders the table body
         */
        renderBody: function() {
            this.table.$body = this.sandbox.dom.createElement(templates.body);
            this.table.rows = {};
            if (this.data.embedded.length > 0) {
                this.sandbox.util.foreach(this.data.embedded, function(record) {
                    this.renderBodyRow(record);
                }.bind(this));
            } else {
                this.renderEmptyIndicator();
            }
            this.sandbox.dom.append(this.table.$el, this.table.$body);
        },

        /**
         * Renders the checkbox or radio button for a row in the tbody
         * @param id {Number|String} the id of the row to add the select-item for
         * @param norender {Boolean} if true the select-item doesn't get rendered as a cell but returned as string
         * @returns {String} html if no render is set to true
         */
        renderRowSelectItem: function(id, norender) {
            if (this.datagrid.options.onlySelectLeaves === true && !!this.table.rows[id].hasChildren) {
                return '';
            }
            if (!!this.options.selectItem && !!this.options.selectItem.type &&
                (this.options.selectItem.inFirstCell === false || norender === true)) {
                var $cell = this.sandbox.dom.createElement(templates.cell);
                this.sandbox.dom.addClass($cell, constants.cellFitClass);
                if (this.options.selectItem.type === selectItems.CHECKBOX) {
                    this.sandbox.dom.html($cell, templates.checkbox);
                } else if (this.options.selectItem.type === selectItems.RADIO) {
                    this.sandbox.dom.html($cell, this.sandbox.util.template(templates.radio)({
                        name: 'datagrid-' + this.datagrid.options.instanceName
                    }));
                }
                if (norender === true) {
                    return this.sandbox.dom.html($cell);
                }
                this.sandbox.dom.addClass($cell, constants.checkboxCellClass);
                this.sandbox.dom.prepend(this.table.rows[id].$el, $cell);
            }
        },

        /**
         * Renders a single table row. If the row already exists it replaces the exiting one
         * @param record {Object} the record
         * @param prepend {Boolean} if true row gets prepended
         */
        renderBodyRow: function(record, prepend) {
            this.removeNewRecordRow();

            var $row = this.sandbox.dom.createElement(templates.row),
                $overrideElement = (!!this.table.rows[record.id]) ? this.table.rows[record.id].$el : null,
                hasParent = this.hasParent(record);

            record.id = (!!record[this.keys.id]) ? record[this.keys.id] : constants.newRecordId;
            this.sandbox.dom.data($row, 'id', record.id);
            if (!!record.parent) {
                this.sandbox.dom.data($row, 'parent', record.parent);
            }

            // render the parents before rendering the children
            if (hasParent) {
                if (!this.table.rows[record.parent]) {
                    this.renderBodyRow(this.data.embedded[this.datagrid.getRecordIndexById(record.parent)], prepend);
                }
                this.table.rows[record.parent].childrenLoaded = true;
            }

            this.table.rows[record.id] = {
                $el: $row,
                cells: {},
                childrenLoaded: !!this.table.rows[record.id] ? this.table.rows[record.id].childrenLoaded : false,
                childrenExpanded: false,
                parent: hasParent ? record.parent : null,
                hasChildren: (!!record[this.datagrid.options.childrenPropertyName]) ? record[this.datagrid.options.childrenPropertyName] : false,
                selectedChildren: 0,
                level: 1
            };

            if (typeof this.datagrid.options.clickCallback === 'function' ||
                typeof this.datagrid.options.actionCallback === 'function' ||
                this.table.rows[record.id].hasChildren === true) {
                this.sandbox.dom.addClass($row, constants.actionClass);
            }
            if (typeof this.datagrid.options.clickCallback !== 'function' &&
                this.table.rows[record.id].hasChildren === false) {
                this.sandbox.dom.addClass($row, constants.inactiveClass);
            }

            this.renderRowSelectItem(record.id);
            this.renderBodyCellsForRow(record);
            this.renderRowRemoveItem(record.id);
            this.insertBodyRow(record, $overrideElement, prepend);
            this.executeRowPostRenderActions(record);
        },

        /**
         * Inserts a body row into the dom. Looks if a row needs to be overriden, or if a parent exists etc.
         * @param record {Object} the data object of the record
         * @param $overrideElement {Object}
         * @param prepend {Boolean} true to prepend
         */
        insertBodyRow: function(record, $overrideElement, prepend) {
            var $parentElement = (!!this.table.rows[record.parent]) ? this.table.rows[record.parent].$el : null,
                insertMethod = (prepend === true) ? this.sandbox.dom.prepend : this.sandbox.dom.append;

            // if there already was a row with the same id, override it with the new one
            if (!!$overrideElement && !!$overrideElement.length) {
                this.sandbox.dom.after($overrideElement, this.table.rows[record.id].$el);
                this.sandbox.dom.remove($overrideElement);
                // if there is a parent insert it after the parent row
            } else if (!!$parentElement && !!$parentElement.length) {
                if (this.renderChildrenHidden === true) {
                    this.sandbox.dom.hide(this.table.rows[record.id].$el);
                    this.changeChildrenToggleIcon(record.parent, false);
                } else {
                    this.table.rows[record.parent].childrenExpanded = true;
                    this.changeChildrenToggleIcon(record.parent, true);
                }
                // insert the row after the last child element of the parent, or directly after the parent
                // if no such child exists
                this.sandbox.dom.after(
                    this.getLastChildElementOfParent($parentElement) || $parentElement,
                    this.table.rows[record.id].$el
                );
            } else {
                insertMethod(this.table.$body, this.table.rows[record.id].$el);
            }
        },

        /**
         * Given a parent element the method returns the last child element. If no child element
         * exsits null is returned
         * @param $parent {Object} the valid parent element
         * @returns {Object} the last child element or null
         */
        getLastChildElementOfParent: function($parent) {
            var $children = $parent.siblings().filter(function() {
                return $(this).data('parent') === $parent.data('id');
            });
            if ($children.length > 0) {
                return $children.last();
            }

            return null;
        },

        /**
         * Manipulates a row of a rendered after it has been rendered. For examples checks the checkbox or focuses an input
         * @param record {Object} the data of the record
         */
        executeRowPostRenderActions: function(record) {
            if (!!this.datagrid.itemIsSelected.call(this.datagrid, record.id)) {
                this.toggleSelectRecord(record.id, true);
            } else {
                this.toggleSelectAllItem(false);
            }
            // select first input if record is new and editable is true
            if (this.options.editable === true && record.id === constants.newRecordId) {
                this.showInput(record.id);
            }
        },

        /**
         * Renderes the all the content cells in a body row
         * @param record {Object} the data for the row
         */
        renderBodyCellsForRow: function(record) {
            // foreach matching grab the corresponding data and render the cell with it
            this.sandbox.util.foreach(this.datagrid.matchings, function(column, index) {
                if (this.options.excludeFields.indexOf(column.attribute) === -1) {
                    this.renderBodyCell(record, column, index);
                }
            }.bind(this));
        },

        /**
         * Renders the remove item for a row in the tbody
         * @param id {Number|String} the id of the row to add the select-item for
         */
        renderRowRemoveItem: function(id) {
            if (this.options.removeRow === true) {
                var $cell = this.sandbox.dom.createElement(templates.cell);
                this.sandbox.dom.html($cell, this.sandbox.util.template(templates.removeCellContent)({
                    icon: this.options.removeIcon
                }));
                this.sandbox.dom.addClass($cell, constants.cellFitClass);
                this.sandbox.dom.append(this.table.rows[id].$el, $cell);
            }
        },

        /**
         * Renders a single cell
         * @param record {Object} the record to render the cell for
         * @param column {Object} the column which should be rendered
         * @param index {Number} the index of the cell in the row
         */
        renderBodyCell: function(record, column, index) {
            var $cell = this.sandbox.dom.createElement(templates.cell),
                content = this.getCellContent(record, column, $cell),
                selectItem, isCroppable = false;
            if (!!this.datagrid.options.childrenPropertyName && index === 0) {
                content = this.wrapChildrenCellContent(content, record);
            }
            if (!!this.options.selectItem && this.options.selectItem.inFirstCell === true && index === 0) {
                this.sandbox.dom.attr($cell, 'colspan', 2);
                selectItem = this.renderRowSelectItem(record.id, true);
                if (typeof content === 'string') {
                    content = selectItem + content;
                } else {
                    this.sandbox.dom.prepend(content, selectItem);
                }
            }

            if (!!column.class && typeof column.class === 'string') {
                this.sandbox.dom.addClass($cell, column.class);
            }
            if (column.type === this.datagrid.types.THUMBNAILS) {
                this.sandbox.dom.addClass($cell, constants.thumbnailCellClass);
                this.sandbox.dom.addClass($cell, constants.cellFitClass);
            }

            this.sandbox.dom.html($cell, content);
            this.sandbox.dom.data($cell, 'attribute', column.attribute);

            if (!!this.sandbox.dom.find('.' + constants.textContainerClass, $cell).length &&
                this.sandbox.dom.children(this.sandbox.dom.find('.' + constants.textContainerClass, $cell)).length === 0) {
                isCroppable = true;
            }

            this.table.rows[record.id].cells[column.attribute] = {
                $el: $cell,
                originalData: record[column.attribute],
                originalContent: this.sandbox.dom.html(content),
                editable: !!column.editable,
                croppable: isCroppable
            };
            // append cell to corresponding row
            this.sandbox.dom.append(
                this.table.rows[record.id].$el,
                this.table.rows[record.id].cells[column.attribute].$el
            );
        },

        /**
         * Gets the actual content for a cell
         * @param record {Object} the record to get the content for
         * @param column {Object} the column for which the content should be returned
         * @param $cell {Object} the cell-dom-element
         * @returns {String|Object} the dom object for the cell content or html
         */
        getCellContent: function(record, column, $cell) {
            var content = record[column.attribute];
            if (!!column.type && column.type === this.datagrid.types.THUMBNAILS) {
                content = this.datagrid.manipulateContent(content, column.type, this.options.thumbnailFormat);
                content = this.sandbox.util.template(templates.img)({
                    alt: content[constants.thumbAltKey],
                    src: content[constants.thumbSrcKey],
                    noImgIcon: typeof this.options.noImgIcon === 'function' ?
                        this.options.noImgIcon(record) : this.options.noImgIcon
                });
            } else {
                content = this.datagrid.processContentFilter(
                    column.attribute,
                    content,
                    column.type,
                    Object.keys(this.table.rows).length,
                    record.hasOwnProperty('id') ? record.id : null
                );
            }
            if (this.options.editable === true && column.editable === true) {
                content = this.getEditableCellContent(content, column.attribute, column.type);
            } else {
                content = this.sandbox.util.template(templates.textContainer)({
                    content: content
                });
            }

            if (!!this.icons) {
                content = this.addIconsToCellContent(content, column, $cell, record);
            }

            if (!!this.badges) {
                content = this.addBadgesToCellContent(content, column, record);
            }

            if (!!this.cssClasses) {
                content = this.addCssClassesToCellContent(content, column, record);
            }

            return content;
        },

        /**
         * Wraps content into an additional container an indent and a toggler icon (for children and parents)
         * @param content {String|Object} the content to wrap. html or dom object
         * @param record {Object} the record data object
         */
        wrapChildrenCellContent: function(content, record) {
            var $wrappedContent = this.sandbox.dom.createElement(templates.childWrapper),
                $icon, paddingLeft = 0;
            // if has children
            if (!!record[this.datagrid.options.childrenPropertyName]) {
                this.sandbox.dom.addClass($wrappedContent, constants.parentClass);
                $icon = this.sandbox.dom.createElement(templates.toggleIcon);
                this.sandbox.dom.prependClass($icon, constants.collapsedIcon);
                this.sandbox.dom.prepend($wrappedContent, $icon);
            } else {
                this.sandbox.dom.addClass($wrappedContent, constants.noChildrenClass);
            }
            // if has parent
            if (this.hasParent(record)) {
                this.table.rows[record.id].level = this.table.rows[record.parent].level + 1;
                paddingLeft = constants.childIndent * (this.table.rows[record.id].level - 1);
            }
            if (this.datagrid.options.onlySelectLeaves == true && this.table.rows[record.id].hasChildren == true) {
                paddingLeft += 40;
            }
            // give that child an indent, children love indents
            if (!!paddingLeft) {
                this.sandbox.dom.css($wrappedContent, {
                    'padding-left': paddingLeft + 'px'
                });
            }

            this.sandbox.dom.append($wrappedContent, content);
            return $wrappedContent;
        },

        hasParent: function(record) {
            return (!!record.parent && !!this.data.embedded[this.datagrid.getRecordIndexById(record.parent)]);
        },

        /**
         * Takes a string and retruns the markup for an editable cell
         * @param content {String} the original value
         * @param type {String} the type of value
         * @param columnName {String} the name of value
         * @returns {String|Object} html or a dom object
         */
        getEditableCellContent: function(content, columnName, type) {
            type = !!type ? type : 'string';
            var options = !!this.options.editableOptions[columnName] ? this.options.editableOptions[columnName] : {};

            return this.sandbox.util.template(templates.editableCellContent[type], {
                value: content,
                options: options,
                translate: this.sandbox.translate
            });
        },

        /**
         * Adds icons to a cell content
         * @param content {String|Object} html or a dom object. If its a string icons get added to the string, if its an object it gets appended
         * @param column {Object} the column data object
         * @param $cell {Object} the cell-dom-element
         * @param record {Object} the record object
         * @returns content {String|Object} html or a dom object
         */
        addIconsToCellContent: function(content, column, $cell, record) {
            var iconStr;
            this.sandbox.util.foreach(this.icons, function(icon, index) {
                if (icon.column === column.attribute && (!icon.disableCallback || icon.disableCallback(record))) {
                    iconStr = this.sandbox.util.template(templates.icon)({
                        icon: icon.icon,
                        align: icon.align,
                        cssClass: icon.cssClass,
                        index: index
                    });
                    if (typeof content === 'object') {
                        this.sandbox.dom.append(content, iconStr);
                    } else if (typeof content === 'string') {
                        content += iconStr;
                    }
                    if (icon.actionIcon === true) {
                        this.sandbox.dom.addClass($cell, constants.actionClass);
                    }
                }
            }.bind(this));

            return content;
        },

        /**
         * Adds badges to a cell content
         * @param content {String|Object} html or a dom object. If its a string icons get added to the string, if its an object it gets appended
         * @param column {Object} the column data object
         * @param record {Object} record which will be rendered
         * @returns content {String|Object} html or a dom object
         */
        addBadgesToCellContent: function(content, column, record) {
            var badgeStr;
            this.sandbox.util.foreach(this.badges, function(badge) {
                if (badge.column === column.attribute) {
                    if (!!badge.callback) {
                        badge = badge.callback(record, badge);
                    }

                    if (!badge) {
                        return;
                    }

                    badgeStr = this.sandbox.util.template(templates.badge)(
                        this.sandbox.util.extend(true, {cssClass: null, title: null, icon: null}, badge)
                    );
                    if (typeof content === 'object') {
                        this.sandbox.dom.prepend(content, badgeStr);
                    } else if (typeof content === 'string') {
                        content = badgeStr + content;
                    }
                }
            }.bind(this));

            return content;
        },

        /**
         * Adds css-classes to a cell content
         * @param content {String|Object} html or a dom object
         * @param column {Object} the column data object
         * @param record {Object} record which will be rendered
         * @returns content {String|Object} html or a dom object
         */
        addCssClassesToCellContent: function(content, column, record) {
            this.sandbox.util.foreach(this.cssClasses, function(cssClass) {
                if (cssClass.column === column.attribute) {
                    if (!!cssClass.callback) {
                        cssClass = cssClass.callback(record, cssClass);
                    }

                    if (!cssClass) {
                        return;
                    }

                    if (typeof content === 'object') {
                        $(content).wrap(['<span class="', cssClass, '" />'].join(''));
                    } else if (typeof content === 'string') {
                        content = ['<span class="', cssClass, '">', content, "</span>"].join('');
                    }
                }
            }.bind(this));

            return content;
        },

        /**
         * Renders the empty list element
         */
        renderEmptyIndicator: function() {
            this.sandbox.dom.append(this.$el, this.sandbox.util.template(templates.empty)({
                text: this.sandbox.translate(this.options.noItemsText),
                emptyIcon: this.options.emptyIcon
            }));
        },

        /**
         * Removes the "new" record row
         */
        removeNewRecordRow: function() {
            if (!!this.table.rows[constants.newRecordId]) {
                this.sandbox.dom.remove(this.table.rows[constants.newRecordId].$el);
                delete this.table.rows[constants.newRecordId];
            }
        },

        /**
         * Render methods (end)
         * -------------------------------------------------------------------- */

        /**
         * Returns true or false whether the container is overflown or not
         * @returns {boolean}
         */
        containerIsOverflown: function() {
            if (!this.table.$container) {
                return false;
            }

            return this.sandbox.dom.get(this.table.$container, 0).scrollWidth > this.sandbox.dom.width(this.table.$container);
        },

        /**
         * Gets executed when the table width is bigger than its container width
         */
        overflowHandler: function() {
            if (!!this.options.cropContents) {
                this.toggleCropTable(true);
            }

            // if still overflown add a css class
            if (this.containerIsOverflown() === true) {
                this.sandbox.dom.addClass(this.$el, constants.overflowClass);
            }
        },

        /**
         * Gets executed when the table width is not bigger than its container width
         */
        underflowHandler: function() {
            if (this.tableCropped === true) {
                if (!!this.options.cropContents && this.sandbox.dom.width(this.table.$container) > this.cropBreakPoint) {
                    this.toggleCropTable(false);
                }
                if (this.containerIsOverflown() === false) {
                    this.sandbox.dom.removeClass(this.$el, constants.overflowClass);
                }
            }
        },

        /**
         * Crops or uncropps all croppable cells in the table
         */
        toggleCropTable: function(crop) {
            if (this.tableCropped !== crop) {
                var $contentContainer,
                    content;
                this.sandbox.util.each(this.table.rows, function(rowId, row) {
                    this.sandbox.util.each(row.cells, function(cellId, cell) {
                        if (cell.croppable === true) {
                            $contentContainer = this.sandbox.dom.find('.' + constants.textContainerClass, cell.$el);
                            if (crop === true) {
                                content = this.sandbox.util.cropMiddle(cell.originalContent, this.options.croppedMaxLength);
                                this.sandbox.dom.attr($contentContainer, 'title', cell.originalContent);
                                this.tableCropped = true;
                                this.cropBreakPoint = this.sandbox.dom.width(this.table.$container);
                            } else {
                                content = cell.originalContent;
                                this.sandbox.dom.removeAttr($contentContainer, 'title');
                                this.tableCropped = false;
                            }
                            this.sandbox.dom.html($contentContainer, content);
                        }
                    }.bind(this));
                }.bind(this));
            }
        },

        /**
         * Bindes dom related events
         */
        bindDomEvents: function() {
            // select or deselect items if the body recognizes a change event
            this.sandbox.dom.on(
                this.table.$body, 'click', this.selectItemChangeHandler.bind(this),
                '.' + constants.checkboxClass + ', .' + constants.radioClass
            );
            // handle click on body row
            this.sandbox.dom.on(this.table.$body, 'click', this.bodyRowClickHandler.bind(this), '.' + constants.rowClass);
            // handle dblclick on body row
            this.sandbox.dom.on(this.table.$body, 'dblclick', this.bodyRowDblClickHandler.bind(this), '.' + constants.rowClass);
            // action callback on click on table-cell with has-action class
            this.sandbox.dom.on(this.table.$body, 'click', this.cellActionCallback.bind(this), 'td.' + constants.actionClass);

            // remove row event
            if (this.options.removeRow === true) {
                this.sandbox.dom.on(this.table.$body, 'click', this.removeItemClickHandler.bind(this), '.' + constants.rowRemoverClass);
            }
            if (!!this.table.header) {
                // select all
                this.sandbox.dom.on(this.table.header.$el, 'change', this.allSelectItemChangeHandler.bind(this));

                // click on sortable item
                if (this.datagrid.options.sortable === true) {
                    this.sandbox.dom.on(
                        this.table.header.$el, 'click', this.sortItemClickHandler.bind(this),
                        '.' + constants.headerCellClass + '.' + constants.sortableClass
                    );
                }
            }
            // click on editable item
            if (this.options.editable === true) {
                this.sandbox.dom.on(this.table.$body, 'click', this.editableItemClickHandler.bind(this), '.' + constants.editableItemClass);
                this.sandbox.dom.on(this.table.$body, 'focusout', this.editableInputFocusoutHandler.bind(this), '.' + constants.editableInputClass);
                this.sandbox.dom.on(this.table.$body, 'keydown', this.editableInputKeyHandler.bind(this), '.' + constants.editableInputClass);
                this.sandbox.dom.on(this.table.$body, 'change', this.editableInputEventHandler.bind(this), '.' + constants.editableInputClass);
            }
            if (!!this.icons) {
                this.sandbox.dom.on(this.table.$body, 'click', this.iconClickHandler.bind(this), '.' + constants.gridIconClass);
            }
            this.sandbox.dom.on(this.table.$body.find('img'), 'error', function() {
                $(this).remove();
            });

            this.sandbox.dom.on(this.table.$body.find('img'), 'load', function() {
                $(this).parent().find('.fa-coffee').remove();
            });
            this.sandbox.dom.on(this.table.$body, 'click', this.radioButtonClickHandler.bind(this), 'input[type="radio"]');
        },

        /**
         * Gets executed if a radio button in the table body gets clicked
         * @param event
         */
        radioButtonClickHandler: function(event) {
            this.sandbox.dom.stopPropagation(event);
            var recordId = this.sandbox.dom.data(this.sandbox.dom.parents(event.currentTarget, '.' + constants.rowClass), 'id'),
                attribute = this.sandbox.dom.data(this.sandbox.dom.parents(event.currentTarget, 'td'), 'attribute');
            this.sandbox.emit(RADIO_SELECTED.call(this), recordId, attribute);
        },

        /**
         * Handles the click on grid-icons
         * @param event {Object} the event object
         */
        iconClickHandler: function(event) {
            var icon = this.icons[this.sandbox.dom.data(event.currentTarget, 'icon-index')],
                recordId = this.sandbox.dom.data(this.sandbox.dom.parents(event.currentTarget, '.' + constants.rowClass), 'id');

            if (typeof recordId !== 'undefined' && !!icon && typeof icon.callback === 'function') {
                event.stopPropagation();
                icon.callback(recordId, this.datagrid.getRecordById(recordId));
            }
        },

        /**
         * Handles the click on a sortable item
         * @param event {Object} the event object
         */
        sortItemClickHandler: function(event) {
            this.sandbox.dom.stopPropagation(event);
            if (this.sandbox.dom.hasClass(event.currentTarget, constants.headerLoadingClass)) {
                return false;
            }
            var attribute = this.sandbox.dom.data(event.currentTarget, 'attribute'),
                direction = 'asc';
            if (this.datagrid.sort.attribute === attribute && this.datagrid.sort.direction === direction) {
                direction = 'desc';
            }
            this.startHeaderCellLoader(attribute);
            // delegate sorting to datagrid
            this.datagrid.sortGrid.call(this.datagrid, attribute, direction);
        },

        /**
         * Handles the click on an editable item
         * @param event {Object} the event object
         */
        editableItemClickHandler: function(event) {
            this.sandbox.dom.stopPropagation(event);
            var recordId = this.sandbox.dom.data(this.sandbox.dom.parents(event.currentTarget, '.' + constants.rowClass), 'id'),
                attribute = this.sandbox.dom.data(this.sandbox.dom.parents(event.currentTarget, 'td'), 'attribute');
            this.showInput(recordId, attribute);
        },

        /**
         * Shows a edit-input for a row and an attribute if no attribute passed shows the first input in row
         * @param recordId {Number|String}
         * @param attribute {String}
         */
        showInput: function(recordId, attribute) {
            var $cell, $inputs, $inputField, $inputWrapper;

            if (!attribute) {
                $inputs = this.sandbox.dom.find('.' + constants.editableInputClass, this.table.rows[recordId].$el);
                attribute = this.sandbox.dom.data(this.sandbox.dom.parents($inputs[0], 'td'), 'attribute');
            }

            $cell = this.table.rows[recordId].cells[attribute].$el;
            $inputField = this.sandbox.dom.find('.' + constants.editableInputClass, $cell);
            $inputWrapper = this.sandbox.dom.find('.' + constants.inputWrapperClass, $cell);

            this.sandbox.dom.show($inputWrapper);
            this.sandbox.dom.focus($inputField);
            this.sandbox.dom.select($inputField);

            this.editStatuses[recordId] = true;
        },

        /**
         * Handles keypress events of the editable inputs
         * @param event {Object} the event object
         */
        editableInputKeyHandler: function(event) {
            // remove server error class if text has changed again
            this.removeEditedErrorCallback($(event.currentTarget).parents('.' + constants.rowClass).data('id'));

            // on enter
            if (event.keyCode === 13) {
                this.editableInputEventHandler(event);
            }
        },

        /**
         * Handles the focusout of an editable input
         * @param event {Object} the event object
         */
        editableInputFocusoutHandler: function(event) {
            if (!!this.isFocusoutHandlerEnabled) {
                this.editableInputEventHandler(event);
            }
        },

        editableInputEventHandler: function(event) {
            this.sandbox.dom.stopPropagation(event);

            var $parent = this.sandbox.dom.parents(event.currentTarget, '.' + constants.rowClass),
                recordId = this.sandbox.dom.data($parent, 'id');

            if (!!this.editStatuses[recordId] || $(event.currentTarget).is('select')) {
                this.editRow(recordId);
            }

            this.editStatuses[recordId] = false;
        },

        /**
         * Gets the data of all edit-inputs in a row and saves their values
         * @param recordId {String|Number} the id of the row to edit
         */
        editRow: function(recordId) {
            var modifiedRecord = {};
            // build new record object out of the inputs in the row
            this.sandbox.util.each(this.table.rows[recordId].cells, function(attribute, cell) {
                if (!!this.sandbox.dom.find('.' + constants.editableInputClass, cell.$el).length) {
                    modifiedRecord[attribute] = this.sandbox.dom.val(
                        this.sandbox.dom.find('.' + constants.editableInputClass, cell.$el)
                    );
                }
            }.bind(this));
            this.saveRow(recordId, modifiedRecord, this.editedSuccessCallback.bind(this), this.editedErrorCallback.bind(this, recordId));
        },

        /**
         * Clears everything up after a row was edited. (hides the input and updates the values)
         * @param record {Object} the changed data record
         */
        editedSuccessCallback: function(record) {
            var $row;
            if (!!record.id && !!this.table.rows[record.id]) {
                $row = this.table.rows[record.id].$el;
            } else if (!!this.table.rows[constants.newRecordId]) {
                $row = this.table.rows[constants.newRecordId].$el;
            }
            if (!!$row && !!$row.length) {
                this.sandbox.dom.hide(this.sandbox.dom.find('.' + constants.inputWrapperClass, $row));
                this.renderBodyRow(record, this.options.addRowTop);
            }
        },

        /**
         * Adds a css class to all inputs in a row, if the editing request returned with an error
         * @param recordId
         */
        editedErrorCallback: function(recordId) {
            var $row = this.table.rows[recordId].$el;
            this.sandbox.dom.addClass(
                this.sandbox.dom.find('.' + constants.inputWrapperClass, $row),
                constants.editedErrorClass
            );
        },

        removeEditedErrorCallback: function(recordId) {
            var $row = this.table.rows[recordId].$el;
            $row.find('.' + constants.inputWrapperClass).removeClass(constants.editedErrorClass);
            this.editStatuses[recordId] = true;
        },

        /**
         * Replaces a record with an existing one and sends it to a server
         * @param recordId {Number|String} the id of the record to override
         * @param newRecordData {Object} the new record data
         * @param successCallback {Function} gets executed after success
         * @param errorCallback {Function} gets executed after error
         */
        saveRow: function(recordId, newRecordData, successCallback, errorCallback) {
            var hasChanged = false,
                record;
            this.sandbox.util.each(this.table.rows[recordId].cells, function(attribute, cell) {
                if (cell.editable === true && cell.originalData !== newRecordData[attribute]) {
                    hasChanged = true;
                }
            }.bind(this));
            // merge record data
            record = this.sandbox.util.extend(
                true, {}, this.data.embedded[this.datagrid.getRecordIndexById(recordId)], newRecordData
            );
            if (recordId === constants.newRecordId) {
                delete record.id;
            }
            if (hasChanged === true) {
                // pass data to datagrid to save it
                this.datagrid.saveGrid.call(this.datagrid,
                    record, this.datagrid.getUrlWithoutParams.call(this.datagrid),
                    successCallback, errorCallback, this.options.addRowTop);
            } else {
                typeof successCallback === 'function' && successCallback(record);
            }
        },

        /**
         * Starts a loader in a cell in the header
         * @param column {String} the attribute of the header cell to insert the loader in
         */
        startHeaderCellLoader: function(column) {
            var $container = this.sandbox.dom.createElement(templates.headerCellLoader);
            this.sandbox.dom.addClass(this.table.header.cells[column].$el, constants.headerLoadingClass);
            this.sandbox.dom.append(this.sandbox.dom.find('.' + constants.textContainerClass, this.table.header.cells[column].$el), $container);
            this.sandbox.start([
                {
                    name: 'loader@husky',
                    options: {
                        el: $container,
                        size: '10px',
                        color: '#999999'
                    }
                }
            ]);
        },

        /**
         * Handles the click on a body row
         * @param event {Object} the event object
         */
        bodyRowClickHandler: function(event) {
            this.sandbox.dom.stopPropagation(event);
            var recordId = this.sandbox.dom.data(event.currentTarget, 'id');
            if (!!recordId && !!this.table.rows && !!this.table.rows[recordId]) {
                if (!!this.table.rows[recordId].hasChildren) {
                    this.toggleChildren(recordId);
                }
            }
            this.rowClickCallback(event);
        },

        /**
         * Handles the dblclick on a body row
         * @param event {Object} the event object
         */
        bodyRowDblClickHandler: function(event) {
            this.sandbox.dom.stopPropagation(event);
            this.rowActionCallback(event);
        },

        /**
         * Calls the item-clicked callback
         * @param event {Object} the original click event
         */
        rowClickCallback: function(event) {
            var recordId = this.sandbox.dom.data(event.currentTarget, 'id');

            if (!this.rowClicked[recordId]) {
                this.rowClicked[recordId] = true;

                this.datagrid.itemClicked.call(this.datagrid, recordId);
                // delay to prevent multiple emits on double click
                this.sandbox.util.delay(function() {
                    this.rowClicked[recordId] = false;
                }.bind(this), 500);
            }
        },

        /**
         * Calls the row-action callback
         * @param event {Object} the original event
         */
        rowActionCallback: function(event) {
            var recordId = this.sandbox.dom.data(event.currentTarget, 'id');
            this.datagrid.itemAction.call(this.datagrid, recordId);
        },

        /**
         * Calls the row-action callback
         * @param event {Object} the original event
         */
        cellActionCallback: function(event) {
            var recordId = this.sandbox.dom.data(this.sandbox.dom.parent(event.currentTarget), 'id');
            if (!!this.table.rows[recordId] && !this.table.rows[recordId].hasChildren) {
                this.datagrid.itemAction.call(this.datagrid, recordId);
            }
        },

        /**
         * Handles the click on the remove item
         * @param event {Object} the event object
         */
        removeItemClickHandler: function(event) {
            this.sandbox.dom.stopPropagation(event);
            var recordId = this.sandbox.dom.data(this.sandbox.dom.parents(event.currentTarget, '.' + constants.rowClass), 'id');
            this.removeRecord(recordId);
        },

        /**
         * Handles the change event of the select items
         * @param event {Object} the event object
         */
        selectItemChangeHandler: function(event) {
            if (!!event.type) {
                this.sandbox.dom.stopPropagation(event);
            }
            var recordId = this.sandbox.dom.data(this.sandbox.dom.parents(event.target, '.' + constants.rowClass), 'id'),
                isChecked = this.sandbox.dom.is(event.target, ':checked');
            if (this.options.selectItem.type === selectItems.CHECKBOX) {
                this.toggleSelectRecord(recordId, isChecked);
            } else if (this.options.selectItem.type === selectItems.RADIO) {
                this.uniqueSelectRecord(recordId);
            }
        },

        /**
         * Handles the change event of a select item in the header
         * @param event {Object} the event object
         */
        allSelectItemChangeHandler: function(event) {
            if (this.data.embedded.length == 0) {
                this.sandbox.dom.prop(event.target, 'checked', false);
                return false;
            }
            this.sandbox.dom.stopPropagation(event);
            var isChecked = this.sandbox.dom.is(event.target, ':checked');
            if (isChecked === true) {
                this.selectAllRecords();
            } else {
                this.deselectAllRecords();
            }
        },

        /**
         * Selejcts all records
         */
        selectAllRecords: function() {
            this.datagrid.selectAllItems.call(this.datagrid);
            this.sandbox.dom.prop(this.sandbox.dom.find('.' + constants.checkboxClass, this.table.$body), 'checked', true);
            this.sandbox.dom.addClass(this.sandbox.dom.find('.' + constants.rowClass, this.table.$body), constants.selectedRowClass);
        },

        /**
         * Deselects all records
         */
        deselectAllRecords: function() {
            this.datagrid.deselectAllItems.call(this.datagrid);
            this.sandbox.dom.prop(this.sandbox.dom.find('.' + constants.checkboxClass, this.table.$body), 'checked', false);
            this.sandbox.dom.removeClass(this.sandbox.dom.find('.' + constants.rowClass, this.table.$body), constants.selectedRowClass);
            this.updateSelectAll();
        },

        /**
         * Selects a record with a given id
         * @param recordId {Number|String} the id of the record to select or deselect
         */
        selectRecord: function(recordId) {
            this.toggleSelectRecord(recordId, true);
        },

        /**
         * Deselects a record with a given id
         * @param recordId {Number|String} the id of the record to select or deselect
         */
        deselectRecord: function(recordId) {
            this.toggleSelectRecord(recordId, false);
        },

        /**
         * Selects or deselects a record with a given id
         * @param id {Number|String} the id of the record to select or deselect
         * @param select {Boolean} true to select false to deselect
         */
        toggleSelectRecord: function(id, select) {
            if (select === true) {
                this.datagrid.setItemSelected.call(this.datagrid, id);

                if (this.table.rows[id]) {
                    // ensure that checkboxes are checked
                    this.sandbox.dom.prop(
                        this.sandbox.dom.find('.' + constants.checkboxClass, this.table.rows[id].$el), 'checked', true
                    );
                    this.sandbox.dom.addClass(this.table.rows[id].$el, constants.selectedRowClass);
                    this.indeterminateSelectParents(id);
                }
            } else {
                this.datagrid.setItemUnselected.call(this.datagrid, id);

                if (this.table.rows[id]) {
                    // ensure that checkboxes are unchecked
                    this.sandbox.dom.prop(
                        this.sandbox.dom.find('.' + constants.checkboxClass, this.table.rows[id].$el), 'checked', false
                    );
                    if (this.table.rows[id].selectedChildren > 0) {
                        this.sandbox.dom.prop(
                            this.sandbox.dom.find('.' + constants.checkboxClass, this.table.rows[id].$el),
                            'indeterminate',
                            true
                        );
                    }
                    this.sandbox.dom.removeClass(this.table.rows[id].$el, constants.selectedRowClass);
                    this.indeterminateUnselectParents(id);
                }
            }

            this.updateSelectAll();
        },

        /**
         * Selects or deselects the select all item
         * @param select {Boolean} true to select false to deselect the select all item
         */
        toggleSelectAllItem: function(select) {
            if (!!this.table.header) {
                this.sandbox.dom.prop(
                    this.sandbox.dom.find('.' + constants.checkboxClass, this.table.header.$el), 'checked', select
                );
            }
        },

        /**
         * Updates the select all item depending on the given data and selections
         */
        updateSelectAll: function() {
            if (!!this.table.header) {
                this.toggleSelectAllItem(
                    this.datagrid.getSelectedItemIds.call(this.datagrid).length === this.data.embedded.length
                );
            }
        },

        /**
         * Selects a record and deselects all other records
         * @param id {Number|String} the id of the record to select
         */
        uniqueSelectRecord: function(id) {
            this.datagrid.deselectAllItems.call(this.datagrid);
            this.datagrid.setItemSelected.call(this.datagrid, id);
        },

        /**
         * Removes the empty list element
         */
        removeEmptyIndicator: function() {
            this.sandbox.dom.remove(this.sandbox.dom.find('.' + constants.emptyListElementClass, this.$el));
        },

        /**
         * Changes the toggle-icon of a parent row
         * @param recordId {String|Number} the id of the parent row
         * @param isExpanded {Boolean} the state for which the icon should be set
         */
        changeChildrenToggleIcon: function(recordId, isExpanded) {
            var $icon = this.sandbox.dom.find('.' + constants.toggleIconClass, this.table.rows[recordId].$el);
            if (!!$icon && !!$icon.length) {
                if (isExpanded === true) {
                    this.sandbox.dom.removeClass($icon, constants.collapsedIcon);
                    this.sandbox.dom.prependClass($icon, constants.expandedIcon);
                } else {
                    this.sandbox.dom.removeClass($icon, constants.expandedIcon);
                    this.sandbox.dom.prependClass($icon, constants.collapsedIcon);
                }
            }
        },

        /**
         * Toggles the children of a parent
         * @param recordId {Number|String} the id of the parent to toggle the children for
         */
        toggleChildren: function(recordId) {
            if (this.table.rows[recordId].childrenLoaded === true) {
                if (this.table.rows[recordId].childrenExpanded === true) {
                    this.hideChildren(recordId);
                    this.sandbox.emit(CHILDREN_COLLAPSED.call(this));
                } else {
                    this.showChildren(recordId);
                    this.sandbox.emit(CHILDREN_EXPANDED.call(this));
                }
            } else {
                this.loadChildren(recordId);
            }
        },

        /**
         * Hides the children of a parent
         * @param recordId {Number|String} the id of the parent to hide the children for
         */
        hideChildren: function(recordId) {
            this.sandbox.util.each(this.table.rows, function(rowId, row) {
                if (row.parent == recordId) {
                    if (!!row.hasChildren) {
                        this.hideChildren(rowId);
                    }
                    this.sandbox.dom.hide(row.$el);
                }
            }.bind(this));
            this.table.rows[recordId].childrenExpanded = false;
            this.changeChildrenToggleIcon(recordId, false);
            this.setAlternateClasses();
        },

        /**
         * Shows the children of a parent
         * @param recordId {Number|String} the id of the parent to show the children for
         */
        showChildren: function(recordId) {
            this.sandbox.util.each(this.table.rows, function(rowId, row) {
                if (row.parent == recordId) {
                    this.sandbox.dom.show(row.$el);
                }
            }.bind(this));
            this.table.rows[recordId].childrenExpanded = true;
            this.changeChildrenToggleIcon(recordId, true);
            this.setAlternateClasses();
        },

        /**
         * Loads the children of a parent and displays them
         * @param recordId {Number|String} the id of the parent to load the children for
         */
        loadChildren: function(recordId) {
            var $el = $('<div style="margin-right: -3px"/>'),
                $icon = $(this.table.rows[recordId].$el).find('.' + constants.toggleIconClass);

            // replace icon with loader while loading children
            this.sandbox.dom.append($icon, $el);
            this.sandbox.dom.removeClass($icon, constants.collapsedIcon);
            this.sandbox.start([
                {
                    name: 'loader@husky',
                    options: {
                        el: $el,
                        size: '8px',
                        color: '#999999'
                    }
                }
            ]);

            this.datagrid.loadChildren.call(this.datagrid, recordId).then(function () {
                this.sandbox.stop($el);
            }.bind(this))

            this.table.rows[recordId].childrenLoaded = true;
        },

        /**
         * Opens all parents of a record
         * @param id {Number|String} the id of the record
         */
        openParents: function(recordId) {
            if (!!this.table.rows[recordId]) {
                var parentId = this.table.rows[recordId].parent;
                if (!!parentId) {
                    if (!!this.table.rows[parentId].parent) {
                        this.openParents(parentId);
                    }
                    this.showChildren(parentId);
                }
            }
        },

        /**
         * Marks the checkboxes of all parents as indeterminate (if they are not already checked)
         * @param recordId {Number|String} the id of the record
         */
        indeterminateSelectParents: function(recordId) {
            var parentId = this.table.rows[recordId].parent,
                $checkbox;
            if (!!parentId) {
                this.table.rows[parentId].selectedChildren += 1;
                $checkbox = this.table.rows[parentId].$el.find('.' + constants.checkboxClass);
                if (!$checkbox.prop('checked')) {
                    $checkbox.prop('indeterminate', true);
                }
                this.indeterminateSelectParents(parentId);
            }
        },

        /**
         * Unmarks the checkboxes of all parents as indeterminate
         * @param recordId {Number|String} the id of the record
         */
        indeterminateUnselectParents: function(recordId) {
            var parentId = this.table.rows[recordId].parent,
                $checkbox;
            if (!!parentId) {
                this.table.rows[parentId].selectedChildren -= 1;
                $checkbox = this.table.rows[parentId].$el.find('.' + constants.checkboxClass);
                if (this.table.rows[parentId].selectedChildren === 0 && !!$checkbox.prop('indeterminate')) {
                    $checkbox.prop('indeterminate', false);
                }
                this.indeterminateUnselectParents(parentId);
            }
        },

        /**
         * Sets an even- and odd-class alternatly to all visible rows
         */
        setAlternateClasses: function() {
            var $rows = this.sandbox.dom.find('.' + constants.rowClass, this.table.$body);
            this.sandbox.dom.removeClass($rows, constants.evenClass);
            this.sandbox.dom.removeClass($rows, constants.oddClass);
            this.sandbox.dom.addClass(
                this.sandbox.dom.filter($rows, ':visible:even'),
                constants.evenClass
            );
            this.sandbox.dom.addClass(
                this.sandbox.dom.filter($rows, ':visible:odd'),
                constants.oddClass
            );
        },

        /**
         * Collapses all child nodes.
         */
        collapseAllChildren: function() {
            for (var i = 0, length = this.data.embedded.length; i < length; i++) {
                this.hideChildren(this.data.embedded[i].id);
            }
        },

        /**
         * Show only selected items.
         *
         * @param {Boolean} show
         */
        showSelected: function(show) {
            // TODO this is a really basic implementation
            // - here should all selected be loaded
            // - currently only current page will be filtered
            var $items = this.datagrid.$find('tbody .row:not(.selected)');

            if (!!show) {
                $items.hide();
            } else {
                $items.show();
            }
        }
    };
});

/**
 * @class Tiles (Datagrid Decorator)
 * @constructor
 *
 * @param {String} [icon] The icon to display in the rendered tiles
 * @param {Object} [fields] Describes the fields from which the title and the description get constructed
 * @param {Array} [fields.title] The fields from which the title gets constructed
 * @param {Array} [fields.description] The fields from which the description gets constructed
 * @param {Object} [translations] Contains the translation keys used by this view
 * @param {Boolean} [showAddNewTile] True to show a tile which emits an add new
 * @param {Boolean} [addNewIcon] The icon to show in the "add new"-tile
 *
 * @param {Boolean} [rendered] property used by the datagrid-main class
 * @param {Function} [initialize] function which gets called once at the start of the view
 * @param {Function} [render] function to render data
 * @param {Function} [destroy] function to destroy the view and unbind events
 */
define('husky_components/datagrid/decorators/tiles-view',[],function() {

    'use strict';

    var defaults = {
        icon: 'fa-folder',
        showAddNewTile: true,
        addNewIcon: 'fa-plus-circle',
        fields: {
            title: ['title'],
            description: ['elementsCount']
        },
        translations: {
            items: 'public.items',
            addNew: 'public.add-new'
        }
    },

    templates = {
        tile: [
            '<div class="tile">',
            '   <span class="<%= icon %> icon"></span>',
            '   <span title="<%= title %>" class="title"><%= title %></span>',
            '   <span class="description"><%= description %></span>',
            '</div>'
        ].join(''),
        addNewTile: [
            '<div class="add-new-tile">',
            '   <span class="<%= icon %> icon"></span>',
            '   <span class="title"><%= title %></span>',
            '</div>'
        ].join('')
    },

    /**
     * Apply datagrid-content-filters on the given record column by column.
     * Datagrid-content-filters are used to format the raw database-values (for the number of elements)
     *
     * @param {Object} record The record data
     * @returns {Object} the modified record
     */
    processContentFilters = function(record) {
        var modifiedRecord = this.sandbox.util.extend(false, {}, record), argument;
        this.datagrid.matchings.forEach(function(matching) {
            argument = null;
            if (matching.type === this.datagrid.types.COUNT) {
                argument = this.sandbox.translate(this.options.translations.items);
            }

            modifiedRecord[matching.attribute] = this.datagrid.processContentFilter.call(
                this.datagrid,
                matching.attribute,
                modifiedRecord[matching.attribute],
                matching.type,
                argument
            );
        }.bind(this));

        return modifiedRecord;
    },

    /**
     * Emitted when the add tile gets clicked
     *
     * @event husky.datagrid.tiles.add-clicked
     */
    ADD_CLICKED = function() {
        return this.datagrid.createEventName.call(this.datagrid, 'tiles.add-clicked');
    };

    return {

        /**
         * Initializes the view, gets called only once
         * @param {Object} context The context of the datagrid class
         * @param {Object} options The options used by the view
         */
        initialize: function(context, options) {
            // context of the datagrid-component
            this.datagrid = context;

            // make sandbox available in this-context
            this.sandbox = this.datagrid.sandbox;

            // merge defaults with options
            this.options = this.sandbox.util.extend(true, {}, defaults, options);
        },

        /**
         * Method to render this view.
         *
         * @param data object containing the data which is rendered
         * @param $container dom-element to render datagrid in
         */
        render: function(data, $container) {
            this.rendered = false;
            this.$el = null;
            this.$tiles = {};

            this.renderContainer($container);
            data.embedded.map(this.renderRecord.bind(this));
            if (!!this.options.showAddNewTile) {
                this.renderAddNewTile();
            }
            this.bindDomEvents();
            this.rendered = true;
        },

        /**
         * Binds dom related events for the view
         */
        bindDomEvents: function() {
            this.$el.on('click', '.tile', function(event) {
                this.datagrid.itemAction.call(this.datagrid, $(event.currentTarget).data('id'));
            }.bind(this));
        },

        /**
         * Renders the main container of the decorator
         *
         * @param {Object} $container The element to render the decorator into.
         */
        renderContainer: function($container) {
            this.$el = $('<div class="tiles-view"/>');
            $container.append(this.$el);
        },

        /**
         * Renders the add new tile and binds its click event
         */
        renderAddNewTile: function() {
            var $addNew = $(_.template(templates.addNewTile, {
                icon: this.options.addNewIcon,
                title: this.sandbox.translate(this.options.translations.addNew)
            }));

            $addNew.on('click', function() {
                this.sandbox.emit(ADD_CLICKED.call(this));
            }.bind(this));

            this.$el.append($addNew);
        },

        /**
         * Renders a single record into the view
         *
         * @param {Object} record The data of the record
         */
        renderRecord: function(record) {
            record = processContentFilters.call(this, record);
            var title = this.getTitle(record),
                description = this.getDescription(record),
                $title, lineHeight, countLinesOfTitle, isOverflownHorizontally;

            this.$tiles[record.id] = $(_.template(templates.tile, {
                icon: this.options.icon,
                title: title,
                description: this.sandbox.util.cropMiddle(description, 18)
            }));
            this.$tiles[record.id].data('id', record.id);

            this.$el.append(this.$tiles[record.id]);

            $title = this.$tiles[record.id].find('.title');
            lineHeight = parseInt($title.css('line-height').replace('px', ''));
            countLinesOfTitle = $title.height() / lineHeight;
            isOverflownHorizontally = $title.get(0).scrollWidth > $title.innerWidth();
            if (countLinesOfTitle > 2 || !!isOverflownHorizontally) {
                this.$tiles[record.id].find('.title').html(this.sandbox.util.cropMiddle(title, 18));
            }
        },

        /**
         * Constructs and returns the title string to render
         *
         * @param {Object} record the data to construct the title from
         * @return {String} the constructed title
         */
        getTitle: function(record) {
            var title = '';
            this.options.fields.title.forEach(function(titleField) {
                title += record[titleField] + ' ';
            });

            return title.trim();
        },

        /**
         * Constructs and returns the description string to render
         *
         * @param {Object} record the data to construct the description from
         * @return {String} the constructed description
         */
        getDescription: function(record) {
            var description = '';
            this.options.fields.description.forEach(function(descriptionField) {
                description += record[descriptionField] + ' ';
            });

            return description.trim();
        },

        /**
         * Destroys the view
         */
        destroy: function() {
            this.$el.remove();
        }
    };
});

/**
 * @class DropdownPagination (Datagrid Decorator)
 * @constructor
 *
 * @param {Object} [paginationOptions] Configuration object
 * @param {Array} [options.showElementsSteps] Array which contains the steps for the Show-Elements-dropdown as integers
 * @param {Number} [options.limit] Data records per page
 *
 * @param {Function} [initialize] function which gets called once at the start of the view
 * @param {Function} [render] function to render data
 * @param {Function} [destroy] function to destroy the pagination and unbind events
 * @param {Function} [getHeight] function which returns the height of the pagination
 */
define('husky_components/datagrid/decorators/dropdown-pagination',[],function() {

    'use strict';

    var defaults = {
            showElementsSteps: [10, 20, 50, 100, 200],
            limit: 20
        },

        constants = {
            paginationClass: 'pagination-wrapper dropdowns',
            paginationElementClass: 'pagination',
            prevClass: 'previous',
            nextClass: 'next',
            inputClass: 'page-input',
            pageLabelClass: 'page-label',
            sizeChangeClass: 'size-change',
            loaderClass: 'pagination-loader'
        },

        /**
         * Templates used by this class
         */
        templates = {
            pageLabel: [
                '<div class="', constants.pageLabelClass, '">',
                '    <%= translate("pagination.page") %>',
                '    <input type="text" class="form-element ', constants.inputClass, '" value="<%= i %>" />',
                '    <%= translate("pagination.of") %> <%= pages %>',
                '</div>'
            ].join(''),

            pageChanger: [
                '<span class="inline-block"><%= label %></span>',
                '<div class="', constants.nextClass, ' pagination-prev pull-right pointer <%= prevButtonClass %>"></div>',
                '<div class="' + constants.prevClass + ' pagination-next pull-right pointer <%= nextButtonClass %>"></div>'
            ].join(''),

            loader: [
                '<div class="', constants.loaderClass, '"></div>'
            ].join(''),

            showElements: [
                '<div class="show-elements">',
                '<span class="text"><%= translate(text) %>:</span>',
                '<div class="' + constants.sizeChangeClass + ' dropdown-trigger">',
                '<%= desc %>',
                '<span class="dropdown-toggle"></span>',
                '</div>',
                '</div>'
            ].join('')
        },

        /**
         * Translation keys used by this class
         */
        translations = {
            show: 'pagination.show',
            elementsOf: 'pagination.elements-of',
            elementsPerPage: 'pagination.elements-per-page'
        };

    return {

        /**
         * Initializes the pagination
         * @param {Object} context The context of the datagrid
         * @param {Object} options The options used by this pagination
         */
        initialize: function(context, options) {
            // context of the datagrid-component
            this.datagrid = context;

            // make sandbox available in this-context
            this.sandbox = this.datagrid.sandbox;

            // merge defaults with pagination options
            this.options = this.sandbox.util.extend(true, {}, defaults, options);

            this.setVariables();
            this.bindCustomEvents();
        },

        /**
         * Returns the total height of the pagination
         * @returns {number}
         */
        getHeight: function() {
            return this.sandbox.dom.outerHeight(this.$paginationContainer, true);
        },

        /**
         * Renders the pagination
         */
        render: function(data, $container) {
            this.$el = $container;
            this.data = data;

            this.$paginationContainer = this.sandbox.dom.createElement('<div class="' + constants.paginationClass + '"/>');
            this.preparePagination();
            this.sandbox.dom.append(this.$el, this.$paginationContainer);
            this.bindDomEvents();
        },

        /**
         * Returns the pagination page size
         * @returns {Number} current limit
         */
        getLimit: function() {
            return this.options.limit;
        },

        /**
         * Sets the classes properties
         */
        setVariables: function() {
            this.$el = null;
            this.$paginationContainer = null;
            this.data = null;
        },

        /**
         * Destroys the pagination
         */
        destroy: function() {
            this.sandbox.stop(this.sandbox.dom.find('*', this.$paginationContainer));
            this.unbindDomEvents();
            this.sandbox.dom.remove(this.$paginationContainer);
        },

        /**
         * Binds custom related events
         */
        bindCustomEvents: function() {
            // show-elements dropdown item clicked
            this.sandbox.on('husky.dropdown.' + this.datagrid.options.instanceName + '-pagination-dropdown-show.item.click', function(item) {
                if (this.data.limit !== item.id || this.data.embedded.length === this.data.total) {
                    this.startLoader();
                    // always jump to the first page
                    this.datagrid.changePage.call(this.datagrid, null, 1, item.id);
                }
            }.bind(this));
        },

        /**
         * Starts a small loader
         */
        startLoader: function() {
            var $container = this.sandbox.dom.createElement(templates.loader);
            this.sandbox.dom.append(this.$paginationContainer, $container);
            this.sandbox.start([
                {
                    name: 'loader@husky',
                    options: {
                        el: $container,
                        size: '20px',
                        color: '#999999'
                    }
                }
            ]);
        },

        /**
         * Binds dom related events
         */
        bindDomEvents: function() {
            var $element = this.sandbox.dom.find('input.' + constants.inputClass, this.$el),
                fontSize = parseInt($element.css('font-size'), 10),
                minWidth = parseInt($element.css('min-width'), 10),
                maxWidth = parseInt($element.css('max-width'), 10);

            // next page
            this.sandbox.dom.on(
                this.sandbox.dom.find('.' + constants.nextClass, this.$el),
                'click',
                this.nextPage.bind(this)
            );

            // previous page
            this.sandbox.dom.on(
                this.sandbox.dom.find('.' + constants.prevClass, this.$el),
                'click',
                this.prevPage.bind(this)
            );

            this.sandbox.dom.on(
                $element,
                'focusout',
                this.inputHandler.bind(this)
            );

            this.sandbox.dom.on(
                $element,
                'keydown',
                function() {
                    if (event.which === 13) {
                        this.inputHandler(event);
                        return;
                    }

                    var width = 50 + (($element.val().length * fontSize) / 2);

                    if (width > minWidth && width <= maxWidth) {
                        $element.css({width: width + 'px'});
                    }
                }.bind(this)
            );
        },

        inputHandler: function(event) {
            var $element = this.sandbox.dom.find(event.currentTarget),
                page = parseInt($element.val(), 10);

            if (isNaN(page)) {
                $element.val(this.data.page);
                return;
            }

            if (page < 1) {
                page = 1;
            }

            if (page > this.data.pages) {
                page = this.data.pages;
            }

            if (page === this.data.page) {
                $element.val(this.data.page);
                return;
            }

            this.startLoader();
            this.datagrid.changePage.call(this.datagrid, null, page);
        },

        /**
         * Triggers a page change to the next change
         */
        nextPage: function() {
            if (!!this.data.links.next) {
                this.startLoader();
                this.datagrid.changePage.call(this.datagrid, this.data.links.next.href);
            }
        },

        /**
         * Triggers a page change to the previous page
         */
        prevPage: function() {
            if (!!this.data.links.previous) {
                this.startLoader();
                this.datagrid.changePage.call(this.datagrid, this.data.links.previous.href);
            }
        },

        /**
         * Unbinds all events from the class
         */
        unbindDomEvents: function() {
            this.sandbox.dom.unbind(this.sandbox.dom.find('*', this.$paginationContainer));
            this.sandbox.dom.unbind(this.$paginationContainer);
        },

        /**
         * Delegates the rendering of the pagination when pagination is needed
         * @returns {*}
         */
        preparePagination: function() {
            var $pagination,
                $showElements,
                paginationLabel,
                description;

            // if first defined step is bigger than the number of all elements don't display show-elements dropdown
            if (this.data.total > this.options.showElementsSteps[0]) {
                description = this.data.limit;
                $showElements = this.sandbox.dom.createElement(this.sandbox.util.template(templates.showElements)({
                    desc: description,
                    text: translations.elementsPerPage,
                    translate: this.sandbox.translate
                }));
                this.sandbox.dom.append(this.$paginationContainer, '<span></span>');
                this.sandbox.dom.append(this.$paginationContainer, $showElements);

                this.prepareShowElementsDropdown();
            }

            if (parseInt(this.data.pages, 10) > 1) {
                $pagination = this.sandbox.dom.createElement('<div/>');
                $pagination.addClass(constants.paginationElementClass);

                this.sandbox.dom.append(this.$paginationContainer, $pagination);

                paginationLabel = this.renderPaginationRow(this.data.page, this.data.pages);

                this.sandbox.dom.append($pagination, this.sandbox.util.template(templates.pageChanger, {
                    label: paginationLabel,
                    prevButtonClass: (this.data.page < this.data.pages) ? '' : 'disabled',
                    nextButtonClass: (this.data.page > 1) ? '' : 'disabled'
                }));
            }
        },

        /**
         * Renders template for one row in the pagination
         * @param i current page number
         * @param pages total number of pages
         */
        renderPaginationRow: function(i, pages) {
            var defaults = {
                translate: this.sandbox.translate,
                i: i,
                pages: pages
            };

            return this.sandbox.util.template(templates.pageLabel, defaults);
        },

        /**
         * Prepares and initializes the dropdown for showing elements
         */
        prepareShowElementsDropdown: function() {
            var i, length, data = [];

            for (i = -1, length = this.options.showElementsSteps.length; ++i < length;) {
                data.push({
                    id: this.options.showElementsSteps[i],
                    name: this.options.showElementsSteps[i]
                });
            }

            this.sandbox.start([
                {
                    name: 'dropdown@husky',
                    options: {
                        el: this.sandbox.dom.find('.' + constants.sizeChangeClass, this.$paginationContainer),
                        setParentDropDown: true,
                        instanceName: this.datagrid.options.instanceName + '-pagination-dropdown-show',
                        alignment: 'left',
                        verticalAlignment: this.options.verticalAlignment,
                        data: data
                    }
                }
            ]);
        }
    };
});

/**
 * @class InfiniteScrollPagination (Datagrid Decorator)
 * @constructor
 *
 * @param {Object} [paginationOptions] Configuration object
 * @param {Number} [options.limit] Data records per page
 * @param {Number} [options.scrollContainer] CSS selector of the scrollable container which contains the records
 * @param {Number} [options.reachedBottomMessage] message or translation key to display when all items are loaded
 * @param {Number} [options.scrollOffset] Defines an offset to the bottom, new pages are loaded when reaching this offset
 *
 * @param {Function} [initialize] function which gets called once at the start of the view
 * @param {Function} [render] function to render data
 * @param {Function} [destroy] function to destroy the pagination and unbind events
 * @param {Function} [getHeight] function which returns the height of the pagination
 */
define('husky_components/datagrid/decorators/infinite-scroll-pagination',[],function() {

    'use strict';

    var defaults = {
            scrollContainer: '.page',
            reachedBottomMessage: 'you reached the end of the list',
            scrollOffset: 0,
            limit: 50
        },

        constants = {
            paginationClass: 'pagination-wrapper infinite-scroll',
            paginationElementClass: 'pagination',
            loaderClass: 'pagination-loader'
        },

        /**
         * Templates used by this class
         */
        templates = {
            paginationContainer: [
                '<div class="' + constants.paginationClass + '">',
                '   <div class="' + constants.loaderClass + '"/>',
                '   <span class="pagination-message"><%= reachedBottomMessage %></span>',
                '</div>'
            ].join('')
        };

    return {

        /**
         * Initializes the pagination
         * @param {Object} context The context of the datagrid
         * @param {Object} options The options used by this pagination
         */
        initialize: function(context, options) {
            // context of the datagrid-component
            this.datagrid = context;

            // make sandbox available in this-context
            this.sandbox = this.datagrid.sandbox;

            // merge defaults with pagination options
            this.options = this.sandbox.util.extend(true, {}, defaults, options);

            this.setVariables();
        },

        /**
         * Sets the classes properties
         */
        setVariables: function() {
            this.$el = null;
            this.$paginationContainer = null;
            this.$loader = null;
            this.data = null;
        },

        /**
         * Returns the total height of the pagination
         * @returns {number}
         */
        getHeight: function() {
            return this.sandbox.dom.outerHeight(this.$paginationContainer, true);
        },

        /**
         * Renders the pagination
         */
        render: function(data, $container) {
            this.$el = $container;
            this.data = data;

            this.$paginationContainer = this.sandbox.dom.createElement(
                this.sandbox.util.template(templates.paginationContainer, {
                    reachedBottomMessage: this.sandbox.translate(this.options.reachedBottomMessage)
                })
            );

            this.$loader = $(this.$paginationContainer).find('.' + constants.loaderClass);
            this.sandbox.dom.append(this.$el, this.$paginationContainer);

            this.startLoader();
            this.bindInfiniteScroll();
            this.fillScrollContainer();
        },

        /**
         * Starts a small loader in pagination container. loader is hidde per css
         */
        startLoader: function() {
            this.sandbox.start([
                {
                    name: 'loader@husky',
                    options: {
                        el: this.$loader,
                        size: '40px',
                        color: '#cccccc'
                    }
                }
            ]);
        },

        /**
         * Returns the pagination page size
         * @returns {Number} current limit
         */
        getLimit: function() {
            return this.options.limit;
        },

        /**
         * Destroys the pagination
         */
        destroy: function() {
            // stop components
            this.sandbox.stop(this.sandbox.dom.find('*', this.$paginationContainer));
            // unbind events
            this.sandbox.dom.unbind(this.sandbox.dom.find('*', this.$paginationContainer));
            this.sandbox.infiniteScroll.destroy(this.options.scrollContainer);
            // remove container
            this.sandbox.dom.remove(this.$paginationContainer);
        },

        /**
         * Binds the scroll event in the given scrollContainer
         */
        bindInfiniteScroll: function() {
            this.sandbox.infiniteScroll.initialize(
                this.options.scrollContainer, this.appendNextPage.bind(this), this.options.scrollOffset
            );
        },

        /**
         * Add records to the datagrid until scroll-container is filled or all records are displayed
         */
        fillScrollContainer: function() {
            var $scrollContainer = $(this.options.scrollContainer);

            // check if scroll-container has an scroll-bar
            if ($scrollContainer[0].clientHeight >= $scrollContainer[0].scrollHeight) {
                // check if there are unrendered items
                if (!!this.datagrid.data.links && !!this.datagrid.data.links.next) {
                    this.appendNextPage().then(function() {
                        this.fillScrollContainer();
                    }.bind(this));
                }
            }
        },

        /**
         * Load next page and append records at the bottom of the datagrid
         * If there isn't a next page, render a reached-bottom message
         * @returns {*}
         */
        appendNextPage: function() {
            var promise = $.Deferred();

            if (!!this.datagrid.data.links && !!this.datagrid.data.links.next) {
                this.$paginationContainer.addClass('loading');

                this.sandbox.util.load(this.datagrid.data.links.next.href).then(function(data) {
                    this.updateDatagrid(data);
                    this.$paginationContainer.data('showReachedEnd', true);
                    this.$paginationContainer.removeClass('loading');

                    promise.resolve();
                }.bind(this))
            } else {
                if (!!this.$paginationContainer.data('showReachedEnd')) {
                    this.$paginationContainer.addClass('reached-end');
                }
                promise.resolve();
            }

            return promise;
        },

        /**
         * Update datagrid properties to the given data
         * Append embedded records to the datagrid
         * @param data
         */
        updateDatagrid: function(data) {
            this.datagrid.data.links = this.datagrid.parseLinks(data._links);
            this.datagrid.data.total = data.total;
            this.datagrid.data.page = data.page;
            this.datagrid.data.pages = data.pages;
            this.datagrid.data.limit = data.limit;

            this.addRecordsToDatagrid(data._embedded[this.datagrid.options.resultKey]);
        },

        /**
         * Append given records tothe bottom of the datagrid
         * @param records
         */
        addRecordsToDatagrid: function(records) {
            var recordsToAdd = [];
            records.forEach(function(record) {
                if (!!record.id) {
                    recordsToAdd.push(record);
                    this.datagrid.data.embedded.push(record);
                }
            }.bind(this));

            // Add records all at once if possible
            if (!!this.datagrid.gridViews[this.datagrid.viewId].addRecords) {
                this.datagrid.gridViews[this.datagrid.viewId].addRecords(recordsToAdd, true);
            } else {
                recordsToAdd.forEach(function(record) {
                    this.datagrid.gridViews[this.datagrid.viewId].addRecord(record, true);
                }.bind(this));
            }
        }
    };
});

/**
 * @class DataGrid
 * @constructor
 *
 * @param {Object} [options] Configuration object
 * @param {Object} [options.data] if no url is provided (some functionality like search & sort will not work)
 * @param {String} [options.resultKey] the name of the data-array in the embedded in the response
 * @param {String} [options.defaultMeasureUnit] the unit that should be taken
 * @param {String} [options.view] name of the view to use.
 *                  external views can be used by configuring the path to decorator-js with require.config.paths
 * @param {Object} [options.viewOptions] Configuration Object for the view
 * @param {Boolean|String} [options.pagination] name the the pagination to use. If false no pagination will be initialized
 *                  external paginations can be used by configuring path to decorator-js with require.config.paths
 * @param {Object} [options.paginationOptions] Configuration Object for the pagination
 *
 * @param {Boolean} [options.sortable] Defines if records are sortable
 * @param {String} [options.searchInstanceName] if set, a listener will be set for the corresponding search events
 * @param {String} [options.columnOptionsInstanceName] if set, a listener will be set for listening for column changes
 * @param {String} [options.url] url to fetch data from
 * @param {String} [options.instanceName] name of the datagrid instance
 * @param {Array} [options.expandIds] array of ids which is appended to expandIds-parameter when loading list items
 * @param {Array} [options.preselected] preselected ids
 * @param {Boolean|String} [options.childrenPropertyName] name of the property which contains the number of children. False to indaticate that list is flat
 * @param {Boolean} [options.onlySelectLeaves] If true only the outermost children can be selected
 * @param {Boolean} [options.resizeListeners] If true a resize-listener will be instantiated, which is responsible for responsiveness
 * @param {String|Function} [options.contentFilters] Used for filtering data at a specifig attribute / column.
 *        If defined as callback the rows value will be passed. If defined as a string, an existing filter template will be applied (see matching types).
 *        Passed will be (content, arguments, recordId).
 * @param {Array} [options.matchings] configuration array of columns if fieldsData isn't set
 * @param {String} [options.matchings.content] column title
 * @param {String} [options.matchings.width] width of column (used by the table view)
 * @param {String} [options.matchings.class] css class of the column
 * @param {String} [options.matchings.type] type of the column. Used to manipulate its content (e.g. 'date')
 * @param {String} [options.matchings.attribute] mapping information to data (if not set it will just iterate through attributes)
 * @param {Boolean} [options.selectedCounter] If true a counter will be displayed which shows how much elements have been selected
 * @param {String} [options.selectedCounterText] translation key or text used in the selected-counter
 * @param {Function} [options.clickCallback] callback for clicking an item - first parameter item id, second parameter the dataset of the clicked item
 * @param {Function} [options.actionCallback] action callback. E.g. executed on double-click in table-view - first parameter item id, second parameter the dataset of the clicked item
 * @param {Function} [options.idKey] the name of the id property
 * @param {Function} [options.childrenKey] the name of the children property
 */
(function() {

    'use strict';

    define('__component__$datagrid@husky',[
        'husky_components/datagrid/decorators/table-view',
        'husky_components/datagrid/decorators/tiles-view',
        'husky_components/datagrid/decorators/dropdown-pagination',
        'husky_components/datagrid/decorators/infinite-scroll-pagination'
    ], function(decoratorTableView, decoratorTilesView, decoratorDropdownPagination, infiniteScrollPagination) {

        /* Default values for options */

        var defaults = {
                view: 'table',
                viewOptions: {
                    table: {},
                    thumbnail: {}
                },
                pagination: 'dropdown',
                paginationOptions: {
                    dropdown: {},
                    'infinite-scroll': {}
                },

                contentFilters: null,
                sortable: true,
                matchings: [],
                url: null,
                saveParams: {},
                data: null,
                instanceName: '',
                searchInstanceName: null,
                searchFields: [],
                columnOptionsInstanceName: null,
                defaultMeasureUnit: 'px',
                preselected: [],
                onlySelectLeaves: false,
                childrenPropertyName: false,
                resizeListeners: true,
                resultKey: 'items',
                selectedCounter: false,
                selectedCounterText: 'public.elements-selected',
                selectedCounterShowSelectedText: 'public.show-selected',
                selectedCounterShowAllText: 'public.show-all',
                viewSpacingBottom: 110,
                clickCallback: null,
                actionCallback: null,
                idKey: 'id',
                childrenKey: 'children'
            },

            types = {
                DATE: 'date',
                DATETIME: 'datetime',
                THUMBNAILS: 'thumbnails',
                TITLE: 'title',
                BYTES: 'bytes',
                RADIO: 'radio',
                CHECKBOX: 'checkbox',
                CHECKBOX_READONLY: 'checkbox_readonly',
                COUNT: 'count',
                TRANSLATION: 'translation',
                NUMBER: 'number',
                CURRENCY: 'currency'
            },

            decorators = {
                views: {
                    table: decoratorTableView,
                    tiles: decoratorTilesView
                },
                paginations: {
                    'dropdown': decoratorDropdownPagination,
                    'infinite-scroll': infiniteScrollPagination
                }
            },

            filters = {
                /**
                 * Takes bytes and returns a more readable string
                 * @param bytes {Number}
                 * @returns {string}
                 */
                bytes: function(bytes) {
                    return this.sandbox.util.formatBytes(bytes);
                },

                title: function(content) {
                    return content;
                },

                /**
                 * Brings a date into the right format
                 * @param date {String} the date to parse
                 * @returns {String}
                 */
                date: function(date) {
                    var parsedDate = this.sandbox.date.format(date);
                    if (parsedDate !== null) {
                        return parsedDate;
                    }
                    return date;
                },

                /**
                 * Brings a datetime into the right format
                 * @param datetime {String} the date to parse
                 * @returns {String}
                 */
                datetime: function(datetime) {
                    var parsedDate = this.sandbox.date.format(datetime, true);
                    if (parsedDate !== null) {
                        return parsedDate;
                    }
                    return datetime;
                },

                /**
                 * Translates a string
                 * @param val {String} the string to translate
                 * @returns {String}
                 */
                translation: function(val) {
                    return this.sandbox.translate(val);
                },

                /**
                 * Formats a float as culture specific number
                 * @param val {String} the string to format
                 * @returns {String}
                 */
                number: function(val) {
                    return this.sandbox.numberFormat(val, 'n');
                },


                /**
                 * Formats a float as culture specific currency
                 * @param val {String} the string to format
                 * @returns {String}
                 */
                currency: function(val) {
                    return this.sandbox.numberFormat(val, 'c');
                },

                /**
                 * Attaches a postfix to a number
                 * @param number
                 * @param postfix
                 */
                count: function(number, postfix) {
                    return (!!postfix) ? number + ' ' + postfix : number;
                },

                /**
                 * Takes an array of thumbnails and returns an object with url and and alt
                 * @param thumbnails {Array} array of thumbnails
                 * @param format {String} the format of the thumbnail
                 * @returns {Object} with url and alt property
                 */
                thumbnails: function(thumbnails, format) {
                    var thumbnail = {
                        url: null,
                        alt: null
                    };
                    if (!!thumbnails && !!thumbnails[format]) {
                        if (typeof thumbnails[format] === 'object') {
                            thumbnail.url = thumbnails[format].url;
                            thumbnail.alt = thumbnails[format].alt;
                        } else {
                            thumbnail.url = thumbnails[format];
                            thumbnail.alt = '';
                        }
                    }
                    return thumbnail;
                },

                /**
                 * checks for bool value and sets radio to true
                 */
                radio: function(content, index, columnName) {
                    return this.sandbox.util.template(templates.radio, {checked: !!content, columnName: columnName});
                },

                /**
                 * checks for bool value and renders a checkbox with value.
                 */
                checkbox: function(content, index, columnName) {
                    return this.sandbox.util.template(templates.checkbox, {
                        checked: !!content,
                        columnName: columnName,
                        readonly: false
                    });
                },

                /**
                 * checks for bool value and renders a readonly checkbox with value.
                 */
                checkbox_readonly: function(content, index, columnName) {
                    return this.sandbox.util.template(templates.checkbox, {
                        checked: !!content,
                        columnName: columnName,
                        readonly: true
                    });
                }
            },

            templates = {
                radio: [
                    '<div class="custom-radio custom-filter">',
                    '   <input name="radio-<%= columnName %>" type="radio" class="form-element" <% if (checked) { print("checked")} %>/>',
                    '   <span class="icon"></span>',
                    '</div>'
                ].join(''),
                checkbox: [
                    '<div class="custom-checkbox custom-filter">',
                    '   <input name="radio-<%= columnName %>" type="checkbox" class="form-element"',
                    '       <% if (readonly) { %>readonly="readonly"<% } %>',
                    '       <% if (checked) { %>checked<% } %>/>',
                    '   <span class="icon"></span>',
                    '</div>'
                ].join(''),
                selectedCounter: [
                    '<span class="selected-elements invisible smaller-font grey-font">',
                    '    <span class="number">0</span>',
                    '    &nbsp;<%= text %>',
                    '    <span class="show-selected-container" <% if (!showSelectedLink) { %>style="display: none;"<% } %>>',
                    '        &nbsp;(<span class="show-selected"><%= showText %></span>)',
                    '    </span>',
                    '</span>'
                ].join('')
            },

            namespace = 'husky.datagrid.',


        /* TRIGGERS EVENTS */

            /**
             * raised after initialization has finished
             * @event husky.datagrid.initialized
             */
            INITIALIZED = function() {
                return this.createEventName('initialized');
            },

            /**
             * raised after a view has been rendered
             * @event husky.datagrid.initialized
             */
            VIEW_RENDERED = function() {
                return this.createEventName('view.rendered');
            },

            /**
             * raised when the the current page changes
             * @event husky.datagrid.page.change
             */
            PAGE_CHANGE = function() {
                return this.createEventName('page.change');
            },

            /**
             * raised when the data is updated
             * @event husky.datagrid.updated
             */
            UPDATED = function() {
                return this.createEventName('updated');
            },

            /**
             * raised when the data is initially loaded
             * @event husky.datagrid.loaded
             */
            LOADED = function() {
                return this.createEventName('loaded');
            },

            /**
             * raised when item is deselected
             * @event husky.datagrid.item.deselect
             * @param {String} id of deselected item
             */
            ITEM_DESELECT = function() {
                return this.createEventName('item.deselect');
            },

            /**
             * raised when selection of items changes
             * @event husky.datagrid.number.selections
             */
            NUMBER_SELECTIONS = function() {
                return this.createEventName('number.selections');
            },

            /**
             * raised when item is selected
             * @event husky.datagrid.item.select
             * @param {String} if of selected item
             */
            ITEM_SELECT = function() {
                return this.createEventName('item.select');
            },

            /**
             * raised when clicked on an item
             * @event husky.datagrid.item.click
             * @param {String} id of item that was clicked
             */
            ITEM_CLICK = function() {
                return this.createEventName('item.click');
            },

            /**
             * raised when all items get deselected via the header checkbox
             * @event husky.datagrid.all.deselect
             */
            ALL_DESELECT = function() {
                return this.createEventName('all.deselect');
            },

            /**
             * raised when all items get deselected via the header checkbox
             * @event husky.datagrid.all.select
             * @param {Array} ids of all items that have been clicked
             */
            ALL_SELECT = function() {
                return this.createEventName('all.select');
            },

            /**
             * listens on and update the selected state of the given items
             * @event husky.datagrid.selected.update
             * @param {Array} ids of all items that should be selected
             */
            SELECTED_UPDATE = function() {
                return this.createEventName('selected.update');
            },

            /**
             * raised when data was saved
             * @event husky.datagrid.data.saved
             * @param {Object} data returned
             */
            DATA_SAVED = function() {
                return this.createEventName('updated');
            },

            /**
             * Raised when save of data failed.
             *
             * @event husky.datagrid.data.save.failed
             *
             * @param {String} jqXHR status
             * @param {String} text status
             * @param {String} error thrown
             */
            DATA_SAVE_FAILED = function() {
                return this.createEventName('data.save.failed');
            },

            /**
             * Raised when loading of data failed.
             *
             * @event husky.datagrid.loading.failed
             *
             * @param {String} jqXHR
             * @param {String} text status
             * @param {String} error thrown
             *
             */
            LOADING_FAILED = function() {
                return this.createEventName('loading.failed');
            },

            /**
             * raised when editable table is changed
             * @event husky.datagrid.data.save
             */
            DATA_CHANGED = function() {
                return this.createEventName('data.changed');
            },

        /* PROVIDED EVENTS */

            /**
             * raised when husky.datagrid.data.get is triggered
             * @event husky.datagrid.data.provide
             */
            DATA_PROVIDE = function() {
                return this.createEventName('data.provide');
            },

            /**
             * listens on and changes the view of the datagrid
             * @event husky.datagrid.view.change
             * @param {String} viewId The identifier of the view
             * @param {Object} Options to merge with the current view options
             */
            CHANGE_VIEW = function() {
                return this.createEventName('view.change');
            },

            /**
             * listens on and changes the view, pagination, page and limit of the datagrid
             * @event husky.datagrid.change
             * @param {Number} page The page
             * @param {Number} limit The limit
             * @param {String} viewId The identifier of the view
             * @param {Object} Options to merge with the current view options
             * @param {String} paginationId The identifier of the pagination
             */
            CHANGE = function() {
                return this.createEventName('change');
            },

            /**
             * listens on and changes the pagination of the datagrid
             * @event husky.datagrid.pagination.change
             * @param {String} paginationId The identifier of the pagination
             */
            CHANGE_PAGINATION = function() {
                return this.createEventName('pagination.change');
            },

            /**
             * listens on and changes the current rendered page
             * @event husky.datagrid.change.page
             * @param {Integer} page page to render
             */
            CHANGE_PAGE = function() {
                return this.createEventName('change.page');
            },

            /**
             * used to add a data record
             * @event husky.datagrid.record.add
             * @param {Object} the data of the new record
             */
            RECORD_ADD = function() {
                return this.createEventName('record.add');
            },

            /**
             * used to add multiple data records
             * @event husky.datagrid.records.add
             * @param {Object} the array of new records
             * @param callback {Function} callback to execute after process has been finished
             */
            RECORDS_ADD = function() {
                return this.createEventName('records.add');
            },

            /**
             * used to remove a data-record
             * @event husky.datagrid.record.remove
             * @param {String} id of the record to be removed
             */
            RECORD_REMOVE = function() {
                return this.createEventName('record.remove');
            },

            /**
             * used to remove multiple data-record
             * @event husky.datagrid.records.remove
             * @param {String} array of ids of the records to be removed
             */
            RECORDS_REMOVE = function() {
                return this.createEventName('records.remove');
            },


            /**
             * listens on and merges one or more data-records with a given ones
             * @event husky.datagrid.records.change
             * @param {Object|Array} the new data-record. Must at least contain an id-property. Can also be an array of data-records
             */
            RECORDS_CHANGE = function() {
                return this.createEventName('records.change');
            },

            /**
             * Deletes all records, sets given records and updates the view
             * @event husky.datagrid.records.set
             * @param {Array} array of data-records
             */
            RECORDS_SET = function() {
                return this.createEventName('records.set');
            },

            /**
             * used to get the currently displayed items
             * @event husky.datagrid.records.get
             * @param {Function} callback function receives array of records
             */
            RECORDS_GET = function() {
                return this.createEventName('records.get');
            },

            /**
             * raised when limit of request changed
             * @event husky.datagrid.page-size.changed
             * @param {Integer} pageSize new size
             * @param {String} paginationId current pagination
             */
            PAGE_SIZE_CHANGED = function() {
                return this.createEventName('page-size.changed');
            },

            /**
             * used to trigger an update of the data
             * @event husky.datagrid.update
             */
            UPDATE = function() {
                return this.createEventName('update');
            },

            /**
             * used to filter data by search
             * @event husky.datagrid.data.filter
             * @param {String} searchField
             * @param {String} searchString
             */
            DATA_SEARCH = function() {
                return this.createEventName('data.search');
            },

            /**
             * raised when data is sorted
             * @event husky.datagrid.data.sort
             */
            DATA_SORT = function() {
                return this.createEventName('data.sort');
            },

            /**
             * used to filter data by updating an url parameter
             * @event husky.datagrid.url.update
             * @param {Object} url parameter : key
             */
            URL_UPDATE = function() {
                return this.createEventName('url.update');
            },

            /**
             * triggers husky.datagrid.data.provide
             * @event husky.datagrid.data.get
             */
            DATA_GET = function() {
                return this.createEventName('data.get');
            },

            /**
             * sets new data to datagrid
             * @event husky.datagrid.data.set
             */
            DATA_SET = function() {
                return this.createEventName('data.set');
            },

            /**
             * triggers husky.datagrid.items.selected event, which returns all selected item ids
             * @event husky.datagrid.items.get-selected
             * @param  {Function} callback function receives array of selected items
             */
            ITEMS_GET_SELECTED = function() {
                return this.createEventName('items.get-selected');
            },

            /**
             * listens on and shows the medium loader above the view
             * @event husky.datagrid.medium-loader.show
             */
            MEDIUM_LOADER_SHOW = function() {
                return this.createEventName('medium-loader.show');
            },

            /**
             * listens on and shows the medium loader above the view
             * @event husky.datagrid.medium-loader.show
             */
            MEDIUM_LOADER_HIDE = function() {
                return this.createEventName('medium-loader.hide');
            },

            /**
             * selects an item with a given id
             * @event husky.datagrid.select.item
             * @param {String|Number} The id of the item to select
             */
            SELECT_ITEM = function() {
                return this.createEventName('select.item');
            },

            /**
             * deselects an item with a given id
             * @event husky.datagrid.deselect.item
             * @param {String|Number} The id of the item to deselect
             */
            DESELECT_ITEM = function() {
                return this.createEventName('deselect.item');
            },

            /**
             * toggle an item with a given id
             * @event husky.datagrid.toggle.item
             * @param {String|Number} The id of the item to toggle
             */
            TOGGLE_ITEM = function() {
                return this.createEventName('toggle.item');
            },

            /**
             * deselect all items
             * @event husky.datagrid.items.deselect
             */
            ITEMS_DESELECT = function() {
                return this.createEventName('items.deselect');
            },

        /**
         * Private Methods
         * --------------------------------------------------------------------
         */

            /**
             * function updates an url by a given parameter name and value and returns it. The parameter is either added or updated.
             * If value is not set, the parameter will be removed from url
             * @param {String} url Url string to be updated
             * @param {String} paramName Parameter which should be added / updated / removed
             * @param {String|Null} paramValue Value of the parameter. If not set, parameter will be removed from url
             * @returns {String} updated url
             */
            setGetParameter = function(url, paramName, paramValue) {
                if (url.indexOf(paramName + "=") >= 0) {
                    var prefix = url.substring(0, url.indexOf(paramName + "=")),
                        suffix = url.substring(url.indexOf(paramName + "="));
                    suffix = suffix.substring(suffix.indexOf('=') + 1);
                    suffix = (suffix.indexOf('&') >= 0) ? suffix.substring(suffix.indexOf('&')) : '';
                    if (!!paramValue) {
                        url = prefix + paramName + '=' + paramValue + suffix;
                    } else {
                        if (url.substr(url.indexOf(paramName + '=') - 1, 1) === '&') {
                            url = url.substring(0, prefix.length - 1) + suffix;
                        } else {
                            url = prefix + suffix.substring(1, suffix.length);
                        }
                    }
                }
                else if (!!paramValue) {
                    if (url.indexOf("?") < 0) {
                        url += "?" + paramName + "=" + paramValue;
                    }
                    else {
                        url += "&" + paramName + "=" + paramValue;
                    }
                }
                return url;
            };

        return {

            /**
             * Creates the eventnames
             * @param postfix {String} event name to append
             */
            createEventName: function(postfix) {
                return namespace + ((!!this.options.instanceName) ? this.options.instanceName + '.' : '') + postfix;
            },

            /**
             * Initialize the datagrid component
             */
            initialize: function() {
                this.sandbox.logger.log('initialized datagrid');

                // extend default options and set variables
                this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

                this.matchings = [];
                this.requestFields = [];
                this.selectedItems = [];

                // make a copy of the decorators for each datagrid instance
                // if you directly access the decorators variable the datagrid-context in the decorators will be overwritten
                this.decorators = this.sandbox.util.extend(true, {}, decorators);

                this.types = types;

                this.gridViews = {};
                this.viewId = this.options.view;

                this.paginations = {};
                this.paginationId = this.options.pagination;

                this.$loader = null;
                this.$mediumLoader = null;
                this.isLoading = false;

                // append datagrid to html element
                this.$element = this.sandbox.dom.$('<div class="husky-datagrid"/>');
                this.$el.append(this.$element);

                this.dataGridWindowResize = null;

                this.sort = {
                    attribute: null,
                    direction: null
                };

                this.bindCustomEvents(); // Should only be be called once
                this.bindDOMEvents();
                this.renderMediumLoader();

                this.loadAndInitializeDecorators().then(function() {
                    if (this.options.selectedCounter === true) {
                        this.renderSelectedCounter();
                    }
                    this.loadAndEvaluateMatchings().then(function() {
                        // merge dom-selected and option-preselected items
                        this.preSelectItems();
                        this.loadAndRenderData();
                        this.sandbox.emit(INITIALIZED.call(this));
                    }.bind(this));
                }.bind(this));

            },

            remove: function() {
                this.unbindWindowResize();
                this.destroy();
            },

            /**
             * Gets the data either via the url or the array
             */
            loadAndRenderData: function() {
                var url;
                if (!!this.options.url) {
                    url = this.options.url;

                    if (this.requestFields.length > 0) {
                        url += (url.indexOf('?') === -1) ? '?' : '&';
                        url += 'fields=' + this.requestFields.join(',');
                    }

                    this.sandbox.logger.log('load data from url');

                    this.loading();
                    this.load({
                        url: url,
                        success: function(data) {
                            this.sandbox.emit(LOADED.call(this), data);
                        }.bind(this)
                    });
                } else if (!!this.options.data) {
                    this.sandbox.logger.log('load data from array');
                    this.data = {};
                    if (!!this.options.resultKey && !!this.options.data[this.options.resultKey]) {
                        this.data.embedded = this.parseEmbedded(this.options.data[this.options.resultKey]);
                    } else {
                        this.data.embedded = this.parseEmbedded(this.options.data);
                    }

                    this.render();
                    if (!!this.paginations[this.paginationId]) {
                        this.paginations[this.paginationId].render(this.data, this.$element);
                    }
                }
            },

            /**
             * Checks if matchings/fields are given as url or array.
             * If a url is given the appropriate fields are fetched.
             */
            loadAndEvaluateMatchings: function() {
                var def = this.sandbox.data.deferred();

                var matchings = this.options.matchings;
                if (typeof(matchings) === 'string') {
                    // Load matchings/fields from url
                    this.loading();
                    this.loadMatchings({
                        url: matchings,
                        success: function(response) {
                            this.filterMatchings(response);
                            def.resolve();
                        }.bind(this)
                    });
                } else {
                    this.filterMatchings(matchings);
                    def.resolve();
                }
                return def;
            },

            /**
             * Loads matchings via ajax
             * @param params url
             */
            loadMatchings: function(params) {
                this.sandbox.dom.addClass(this.$find('.selected-elements'), 'invisible');
                this.sandbox.util.load(params.url)
                    .then(function(response) {
                        this.updateSelectedCounter();
                        if (this.isLoading === true) {
                            this.stopLoading();
                        }
                        this.destroy();
                        if (!!params.success && typeof params.success === 'function') {
                            params.success(response);
                        }
                    }.bind(this))
                    .fail(function(jqXHR, textStatus, error) {
                        this.sandbox.emit(LOADING_FAILED.call(this), jqXHR, textStatus, error);
                    }.bind(this));
            },

            /**
             * Takes an array of matchings and filters disabled matchings out of it
             * Moreover it constructs the array of fields which should be requested from a server
             * The filtered matchings are available in this.matchings
             * The request-fields are available in this.requestFields
             *
             * @param {Array} matchings array with matchings
             */
            filterMatchings: function(matchings) {
                var matchingObject;
                this.matchings = [];
                this.requestFields = [];

                this.sandbox.util.foreach(matchings, function(matching) {
                    matchingObject = {};

                    // only add matching if it's not disabled
                    if ((matching.disabled !== 'true' && matching.disabled !== true)) {

                        // build up the matching with the keys of the passed matching
                        for (var key in matching) {
                            if (key === 'translation') {
                                matchingObject.content = this.sandbox.translate(matching.translation);
                            } else if (key === 'name') {
                                matchingObject.attribute = matching.name;
                            } else if (key === 'sortable') {
                                matchingObject.sortable = matching.sortable;
                                if (typeof matching.sortable === 'string') {
                                    matchingObject.sortable = JSON.parse(matching.sortable);
                                }
                            } else {
                                matchingObject[key] = matching[key];
                            }
                        }
                        // push the constructed matching to the global matchings array
                        this.matchings.push(matchingObject);
                        this.requestFields.push(matching.name);
                    } else if (matching.name === this.options.idKey) {
                        this.requestFields.push(matching.name);
                    }
                    // always load the id (never ever even think about not loading the id)
                }.bind(this));
            },

            /**
             * Renders the data of the datagrid
             */
            render: function() {
                this.renderView();
                if (!!this.paginations[this.paginationId]) {
                    this.paginations[this.paginationId].render(this.data, this.$element);
                }
                if (this.options.resizeListeners === true) {
                    this.windowResizeListener();
                }
            },

            /**
             * Renderes the counter which shows how many elements have been selected
             */
            renderSelectedCounter: function() {
                this.sandbox.dom.append(
                    this.$element,
                    this.sandbox.util.template(templates.selectedCounter, {
                            text: this.sandbox.translate(this.options.selectedCounterText),
                            showText: this.sandbox.translate(this.options.selectedCounterShowSelectedText),
                            showSelectedLink: !!this.gridViews[this.viewId].showSelected
                        }
                    )
                );
                this.updateSelectedCounter();
            },

            /**
             * Renderes the current view
             */
            renderView: function() {
                this.gridViews[this.viewId].render(this.data, this.$element);
                this.updateSelectedCounter();
                this.sandbox.emit(VIEW_RENDERED.call(this));
            },

            /**
             * Preselects items because of passed options via javascript and the dom
             */
            preSelectItems: function() {
                var dataSelected = this.sandbox.dom.data(this.$el, 'selected'),
                    convertedArray = [];
                if (!!dataSelected) {
                    this.options.preselected = this.sandbox.util.union(this.options.preselected, dataSelected);
                }
                // convert to an array which only contains ids as integers
                this.sandbox.util.foreach(this.options.preselected, function(element) {
                    if (typeof element === 'object') {
                        convertedArray.push(element.id);
                    } else {
                        convertedArray.push(element);
                    }
                }.bind(this));
                this.options.preselected = convertedArray;
                this.setSelectedItems(this.options.preselected);
                this.setSelectedItemsToData();
            },

            /**
             * Sets the ids of slected records into the dom
             */
            setSelectedItemsToData: function() {
                this.sandbox.dom.removeAttr(this.$el, 'data-selected');
                this.sandbox.dom.data(this.$el, 'selected', this.getSelectedItemIds());
            },

            /**
             * Destroys the view and the pagination
             */
            destroy: function() {
                if (!!this.gridViews[this.viewId] && !!this.gridViews[this.viewId].rendered) {
                    this.gridViews[this.viewId].destroy();
                    if (!!this.paginations[this.paginationId]) {
                        this.paginations[this.paginationId].destroy();
                    }
                }
            },

            /**
             * Rerenders the view
             */
            rerenderView: function() {
                this.gridViews[this.viewId].destroy();
                this.renderView();
            },

            /**
             * Rerenders the pagination
             */
            rerenderPagination: function() {
                if (!!this.paginations[this.paginationId]) {
                    this.paginations[this.paginationId].destroy();
                    this.paginations[this.paginationId].render(this.data, this.$element);
                }
            },

            /**
             * Loads contents via ajax
             * @param params url
             */
            load: function(params) {
                var expandIds;

                this.currentUrl = this.getUrl(params);

                if (!params.data) params.data = {};
                expandIds = this.getSelectedItemIds();
                expandIds = (!!this.options.expandIds) ? expandIds.concat(this.options.expandIds) : expandIds;
                params.data['expandIds'] = expandIds.join(',');

                this.sandbox.dom.addClass(this.$find('.selected-elements'), 'invisible');
                return this.sandbox.util.load(this.currentUrl, params.data)
                    .then(function(response) {
                        this.sandbox.dom.removeClass(this.$find('.selected-elements'), 'invisible');
                        if (this.isLoading === true) {
                            this.stopLoading();
                        }
                        this.destroy();
                        this.parseData(response);
                        this.getSortingFromUrl(this.currentUrl);
                        this.render();
                        if (!!params.success && typeof params.success === 'function') {
                            params.success(response);
                        }
                    }.bind(this))
                    .fail(function(jqXHR, textStatus, error) {
                        this.sandbox.emit(LOADING_FAILED.call(this), jqXHR, textStatus, error);
                    }.bind(this));
            },

            /**
             * Changes data in grid from array
             * @param data array
             */
            setData: function(data) {
                this.destroy();

                if (!!data.resultKey && !!data[data.resultKey]) {
                    this.data.embedded = data[data.resultKey];
                } else {
                    this.data.embedded = data;
                }

                if (!!this.paginations[this.paginationId]) {
                    this.paginations[this.paginationId].render(this.data, this.$element);
                }

                this.render();
            },

            /**
             * Parses the sorting params from the url
             * @param url
             */
            getSortingFromUrl: function(url) {
                if (url.indexOf('sortOrder') > -1 && url.indexOf('sortBy') > -1) {
                    var attribute = this.sandbox.util.getParameterByName('sortBy', url),
                        direction = this.sandbox.util.getParameterByName('sortOrder', url);

                    if (!!attribute && !!direction) {
                        this.sort.attribute = attribute;
                        this.sort.direction = direction;
                    }
                }
            },

            /**
             * Displays a loading icon
             */
            loading: function() {
                if (this.sandbox.dom.is(this.$element, ':visible')) {
                    this.sandbox.dom.height(this.$element, this.sandbox.dom.outerHeight(this.$element));
                    this.sandbox.dom.width(this.$element, this.sandbox.dom.outerWidth(this.$element));
                }

                this.sandbox.dom.addClass(this.$element, 'loading');

                if (!this.$loader) {
                    this.$loader = this.sandbox.dom.createElement('<div class="datagrid-loader"/>');
                    this.sandbox.dom.hide(this.$loader);
                    this.sandbox.dom.append(this.$element, this.$loader);

                    this.sandbox.start([
                        {
                            name: 'loader@husky',
                            options: {
                                el: this.$loader,
                                size: '100px',
                                color: '#cccccc'
                            }
                        }
                    ]);
                }

                this.sandbox.dom.show(this.$loader);
                this.isLoading = true;
            },

            /**
             * Hides the loading icon
             */
            stopLoading: function() {
                this.isLoading = false;
                this.sandbox.dom.hide(this.$loader);
                this.sandbox.dom.removeClass(this.$element, 'loading');

                this.sandbox.dom.height(this.$element, '');
                this.sandbox.dom.width(this.$element, '');
                return this.$element;
            },

            /**
             * Gets the view and a load to get data and render it
             */
            loadAndInitializeDecorators: function() {
                var def = this.sandbox.data.deferred();
                this.loadAndInitializePagination(this.paginationId).then(function() {
                    this.loadAndInitializeView(this.viewId).then(function() {
                        def.resolve();
                    }.bind(this));
                }.bind(this));
                return def;
            },

            /**
             * Renderes and starts the medium loader, which can
             * be displayed above a view with events
             */
            renderMediumLoader: function() {
                this.$mediumLoader = this.sandbox.dom.createElement('<div class="medium-loader"/>');
                this.sandbox.dom.append(this.$element, this.$mediumLoader);

                this.sandbox.start([
                    {
                        name: 'loader@husky',
                        options: {
                            el: this.$mediumLoader,
                            size: '50px',
                            color: '#cccccc'
                        }
                    }
                ]);
            },

            /**
             * Displays the medium loader above the view
             */
            showMediumLoader: function() {
                this.sandbox.dom.addClass(this.$mediumLoader, 'show');
            },

            /**
             * Hides the medium loader
             */
            hideMediumLoader: function() {
                this.sandbox.dom.removeClass(this.$mediumLoader, 'show');
            },

            /**
             * Takes a data object and sets it to the global data-property
             * @param data {Object} data property
             */
            parseData: function(data) {
                this.data = {};
                this.data.links = this.parseLinks(data._links);
                this.data.embedded = this.parseEmbedded(data._embedded[this.options.resultKey]);
                this.data.total = data.total;
                this.data.page = data.page;
                this.data.pages = data.pages;
                this.data.limit = data.limit;
            },

            /**
             * Parses data and returns normalized embedded items.
             *
             * @param {Array} data
             * @param {Object} parent
             */
            parseEmbedded: function(data, parent) {
                var embedded = [];
                for (var i = 0, len = data.length; i < len; i++) {
                    var children = [];
                    if (!!parent) {
                        data[i].parent = parent;
                    }

                    data[i].id = data[i][this.options.idKey];
                    if (!data[i][this.options.childrenPropertyName]) {
                        data[i][this.options.childrenPropertyName] = false;
                    }

                    if (data[i].hasOwnProperty(this.options.childrenKey)) {
                        children = this.parseEmbedded(data[i][this.options.childrenKey], data[i].id);
                        data[i][this.options.childrenPropertyName] = (children.length > 0);
                    }

                    embedded.push(data[i]);
                    embedded = embedded.concat(children);
                }

                return embedded;
            },

            /**
             * Unescapes all passed links
             * @param links {Object}
             * @returns {Object}
             */
            parseLinks: function(links) {
                var linksObj = {};
                this.sandbox.util.each(links, function(index, link) {
                    linksObj[index] = {
                        href: decodeURI(link.href)
                    };
                }.bind(this));
                return linksObj;
            },

            /**
             * Load view decorator to given viewId and initialize it
             * @param viewId {String} the identifier of the decorator
             */
            loadAndInitializeView: function(viewId) {
                this.viewId = viewId;
                var def = this.sandbox.data.deferred();

                // view allready loaded
                if (!!this.gridViews[this.viewId]) {
                    def.resolve();
                }

                // load husky decorator
                else if (!!this.decorators.views[this.viewId]) {
                    this.gridViews[this.viewId] = this.decorators.views[this.viewId];
                    this.initializeViewDecorator();
                    def.resolve();
                }

                // not an husky decorator, try to load external decorator
                else {
                    require([viewId], function(view) {
                        // this will keep BC
                        if (typeof view === 'function') {
                            view = new view();
                        }

                        this.gridViews[this.viewId] = view;
                        this.initializeViewDecorator();
                        def.resolve();
                    }.bind(this));
                }

                return def;
            },

            /**
             * Initialize the current set view decorator. Log an error message to console if the view is not valid
             */
            initializeViewDecorator: function() {
                if (this.isViewValid(this.gridViews[this.viewId])) {
                    // merge view options with passed ones
                    this.gridViews[this.viewId].initialize(this, this.options.viewOptions[this.viewId]);
                } else {
                    this.sandbox.logger.log('Error: View does not meet the configured requirements. See the documentation');
                }
            },

            /**
             * Validates a given view
             * @param view {Object} the view to validate
             * @returns {boolean} returns true if the view is usable
             */
            isViewValid: function(view) {
                if (!view ||
                    typeof view.initialize === 'undefined' ||
                    typeof view.render === 'undefined' ||
                    typeof view.destroy === 'undefined') {
                    return false;
                }
                return true;
            },

            /**
             * Load pagination decorator to given paginationId and initialize it
             * @param {String} paginationId the identifier of the pagination
             */
            loadAndInitializePagination: function(paginationId) {
                this.paginationId = paginationId;
                var def = this.sandbox.data.deferred();

                // no pagination set or pagination allready loaded
                if (!paginationId || !!this.paginations[this.paginationId]) {
                    def.resolve();
                }

                // load husky decorator
                else if (!!this.decorators.paginations[this.paginationId]) {
                    this.paginations[this.paginationId] = this.decorators.paginations[this.paginationId];
                    this.initializePaginationDecorator();
                    def.resolve();
                }

                // not an husky decorator, try to load external decorator
                else {
                    require([paginationId], function(pagination) {
                        this.decorators.paginations[this.paginationId] = pagination;
                        this.initializePaginationDecorator();
                        def.resolve();
                    }.bind(this));
                }

                return def;
            },

            /**
             * Initialize the current set pagination decorator. Log an error message to console if the view is not valid
             */
            initializePaginationDecorator: function() {
                if (this.isPaginationValid(this.paginations[this.paginationId])) {
                    this.paginations[this.paginationId]
                        .initialize(this, this.options.paginationOptions[this.paginationId]);
                } else {
                    this.sandbox.logger
                        .log('Error: Pagination does not meet the configured requirements. See the documentation');
                }
            },

            /**
             * Changes the view of the datagrid
             * @param view {String} identifier of the new view to use
             * @param options {Object} an object with options to merge with the current view options for the view
             */
            changeView: function(view, options) {
                // only change if view or if options are passed (could be passed to the same view)
                if (view !== this.viewId || !!options) {
                    this.destroy();
                    return this.loadAndInitializeView(view).then(function() {
                        this.extendViewOptions(options);
                        this.render();
                    }.bind(this));
                }
            },

            /**
             * Changes the pagination of the datagrid
             * @param pagination {String} identifier of the new pagination to use
             */
            changePagination: function(pagination) {
                if (pagination !== this.paginationId) {
                    this.destroy();
                    this.loadAndInitializePagination(pagination).then(function() {
                        this.render();
                    }.bind(this));
                }
            },

            /**
             * Changes style and page of datagrid
             * @param {Number} page The page
             * @param {Number} limit The limit
             * @param {String} viewId The identifier of the view
             * @param {Object} options to merge with the current view options
             * @param {String} paginationId The identifier of the pagination
             */
            change: function(page, limit, viewId, options, paginationId) {
                if (paginationId === this.paginationId && view === this.viewId) {
                    return;
                }

                this.showMediumLoader();
                this.changePage(null, page, limit).then(function() {
                    this.changeView(viewId, options).then(function() {
                        this.changePagination(paginationId);
                        this.hideMediumLoader();
                    }.bind(this));
                }.bind(this));
            },

            /**
             * Takes an object with options and passes them to the view,
             * so the view can extend its current ones with them
             * @param options {Object} mew options
             */
            extendViewOptions: function(options) {
                if (!!options && !!this.gridViews[this.viewId].extendOptions) {
                    this.gridViews[this.viewId].extendOptions(options);
                }
            },

            /**
             * Validates a given pagination
             * @param pagination {Object} the pagination to validate
             * @returns {boolean} returns true if the pagination is usable
             */
            isPaginationValid: function(pagination) {
                if (!pagination ||
                    typeof pagination.initialize === 'undefined' ||
                    typeof pagination.render === 'undefined' ||
                    typeof pagination.getLimit === 'undefined' ||
                    typeof pagination.destroy === 'undefined') {
                    return false;
                }
                return true;
            },

            /**
             * Returns url with page size and page param at the end
             * @param params
             * @returns {string}
             */
            getUrl: function(params) {
                if (!!this.data && !!this.data.links) {
                    return params.url;
                }

                var delimiter = '?', url = params.url;
                if (!!this.options.pagination && !!this.paginations[this.paginationId].getLimit()) {
                    if (params.url.indexOf('?') !== -1) {
                        delimiter = '&';
                    }
                    url = params.url + delimiter + 'limit=' + this.paginations[this.paginationId].getLimit();
                    if (params.page > 1) {
                        url += '&page=' + params.page;
                    }
                }

                return url;
            },

            /**
             * returns number and unit
             * @param numberUnit
             * @returns {{number: Number, unit: *}}
             */
            getNumberAndUnit: function(numberUnit) {
                numberUnit = String(numberUnit);
                var regex = numberUnit.match(/(\d+)\s*(.*)/);
                // no unit , set default
                if (!regex[2]) {
                    regex[2] = this.options.defaultMeasureUnit;
                }
                return {number: parseInt(regex[1], 10), unit: regex[2]};
            },

            /**
             * Manipulates the content of a cell with a process realted to the columns type
             * @param content {String} the content of the cell
             * @param type {String} the columns type
             * @param argument {Number|String} argument to pass to the processor
             * @param columnName {String} Name of the column
             * @returns {String} the manipulated content
             */
            manipulateContent: function(content, type, argument, columnName) {
                if (typeof type === 'function') {
                    return type(content, argument, columnName);
                } else if (filters.hasOwnProperty(type)) {
                    return filters[type].call(this, content, argument, columnName);
                }
                return content;
            },

            /**
             * Checks if a filter was set for specified column and processes it
             * @param {String} attributeName name of processed attribute
             * @param {String} content the content which is processed
             * @param {String} [type] if no filter is set, the type is processed
             * @param {Number|String} [argument] argument to pass to the processor
             * @returns {String} the manipulated content
             */
            processContentFilter: function(attributeName, content, type, argument, recordId) {
                // check if filter is set for current column
                if (!!this.options.contentFilters && this.options.contentFilters.hasOwnProperty(attributeName)) {
                    // check if filter is function or string and call filter
                    if (typeof this.options.contentFilters[attributeName] === 'function') {
                        return this.options.contentFilters[attributeName].call(this, content, argument, recordId);
                    } else if (typeof this.options.contentFilters[attributeName] === 'string') {
                        type = this.options.contentFilters[attributeName];
                        return this.manipulateContent(content, type, argument, attributeName);
                    }
                }
                // if no filter was set, check if type is set and calls
                else if (!!type) {
                    return this.manipulateContent(content, type, argument);
                }
                return content;
            },

            /**
             * Reset the sorting properties
             */
            resetSortingOptions: function() {
                this.sort.attribute = null;
                this.sort.direction = null;
            },

            /**
             * Binds Dom-related events
             */
            bindDOMEvents: function() {
                if (this.options.resizeListeners === true) {
                    this.dataGridWindowResize = this.windowResizeListener.bind(this);
                    this.sandbox.dom.on(
                        this.sandbox.dom.$window,
                        'resize.' + this.createEventName('dom'),
                        this.dataGridWindowResize
                    );
                }

                if (this.options.selectedCounter) {
                    this.sandbox.dom.on(this.$el,
                        'click',
                        this.toggleSelectedClickHandler.bind(this),
                        '.selected-elements .show-selected'
                    );
                }
            },

            /**
             * Handlers the click on the "show selected" and "show all" button.
             *
             * @param {Object} event The click event
             */
            toggleSelectedClickHandler: function(event) {
                this.sandbox.dom.stopPropagation(event);
                if (!this.gridViews[this.viewId].showSelected) {
                    return;
                }

                this.show = !this.show;
                this.gridViews[this.viewId].showSelected(this.show);
                if (!!this.show && !!this.paginations[this.paginationId]) {
                    this.paginations[this.paginationId].destroy();
                } else {
                    this.rerenderPagination();
                }

                this.updateSelectedCounter();
            },

            unbindWindowResize: function() {
                this.sandbox.dom.off(this.sandbox.dom.$window, 'resize.' + this.createEventName('dom'));
            },

            /**
             * Bind custom-related events
             */
            bindCustomEvents: function() {
                if (this.options.resizeListeners === true) {
                    this.sandbox.on('husky.navigation.size.changed', this.windowResizeListener.bind(this));
                }

                this.sandbox.on(UPDATE.call(this), this.updateGrid.bind(this));
                this.sandbox.on(DATA_SET.call(this), this.setData.bind(this));
                this.sandbox.on(DATA_GET.call(this), this.provideData.bind(this));
                this.sandbox.on(DATA_SEARCH.call(this), this.searchGrid.bind(this));
                this.sandbox.on(URL_UPDATE.call(this), this.updateUrl.bind(this));
                this.sandbox.on(CHANGE_VIEW.call(this), this.changeView.bind(this));
                this.sandbox.on(CHANGE.call(this), this.change.bind(this));
                this.sandbox.on(CHANGE_PAGINATION.call(this), this.changePagination.bind(this));
                this.sandbox.on(RECORD_ADD.call(this), this.addRecordHandler.bind(this));
                this.sandbox.on(RECORDS_ADD.call(this), this.addRecordsHandler.bind(this));
                this.sandbox.on(RECORD_REMOVE.call(this), this.removeRecordHandler.bind(this));
                this.sandbox.on(RECORDS_REMOVE.call(this), this.removeRecordsHandler.bind(this));
                this.sandbox.on(RECORDS_CHANGE.call(this), this.changeRecordsHandler.bind(this));
                this.sandbox.on(RECORDS_SET.call(this), this.setRecordsHandler.bind(this));
                this.sandbox.on(RECORDS_GET.call(this), this.getRecordsHandler.bind(this));
                this.sandbox.on(NUMBER_SELECTIONS.call(this), this.updateSelectedCounter.bind(this));
                this.sandbox.on(MEDIUM_LOADER_SHOW.call(this), this.showMediumLoader.bind(this));
                this.sandbox.on(MEDIUM_LOADER_HIDE.call(this), this.hideMediumLoader.bind(this));
                this.sandbox.on(SELECTED_UPDATE.call(this), this.updateSelection.bind(this));
                this.sandbox.on(SELECT_ITEM.call(this), this.selectItem.bind(this));
                this.sandbox.on(DESELECT_ITEM.call(this), this.deselectItem.bind(this));
                this.sandbox.on(TOGGLE_ITEM.call(this), this.toggleItem.bind(this));
                this.sandbox.on(ITEMS_DESELECT.call(this), function() {
                    this.gridViews[this.viewId].deselectAllRecords();
                }.bind(this));
                this.sandbox.on(ITEMS_GET_SELECTED.call(this), function(callback, returnItems) {
                    if (!!returnItems) {
                        var ids = this.getSelectedItemIds(),
                            items = [];

                        this.sandbox.util.foreach(ids, function(id) {
                            items.push(this.getRecordById(id));
                        }.bind(this));

                        callback(ids, items);
                    } else {
                        callback(this.sandbox.util.deepCopy(this.getSelectedItemIds()));
                    }
                }.bind(this));
                this.sandbox.on(CHANGE_PAGE.call(this), function(page, limit) {
                    if (!isNaN(page)) {
                        this.changePage(null, page, limit);
                    }
                }.bind(this));

                this.startColumnOptionsListener();
                this.startSearchListener();
            },

            /**
             * Listens on search events to trigger a grid search
             */
            startSearchListener: function() {
                if (this.options.searchInstanceName !== null) {
                    var searchInstanceName = '.' + this.options.searchInstanceName;
                    this.sandbox.on('husky.search' + searchInstanceName, this.searchGrid.bind(this));
                    this.sandbox.on('husky.search' + searchInstanceName + '.reset', this.searchGrid.bind(this, ''));
                }
            },

            /**
             * Starts a listener on column options to trigger a grid filter
             */
            startColumnOptionsListener: function() {
                // listen to events from column options
                if (this.options.columnOptionsInstanceName !== null) {
                    var columnOptionsInstanceName = (this.options.columnOptionsInstanceName !== '') ? '.' + this.options.columnOptionsInstanceName : '';
                    this.sandbox.on('husky.column-options' + columnOptionsInstanceName + '.saved', this.filterGrid.bind(this));
                }
            },

            /**
             * Returns url without params
             */
            getUrlWithoutParams: function() {
                var url = this.data.links.self.href;

                if (url.indexOf('?') !== -1) {
                    return url.substring(0, url.indexOf('?'));
                }

                return url;
            },

            /**
             * Returns the index of a data record for a given id
             * @param id {Number|String} the id to search for
             * @returns {Number|String} the index of the found record
             */
            getRecordIndexById: function(id) {
                if (!!this.data && !!this.data.embedded) {
                    for (var i = -1, length = this.data.embedded.length; ++i < length;) {
                        if (this.data.embedded[i].id === id) {
                            return i;
                        }
                    }
                }

                return null;
            },

            /**
             * Returns the item for a given id
             * @param id {Number|String} the id to search for
             * @returns {Number|String} the index of the found record
             */
            getRecordById: function(id) {
                for (var i = -1, length = this.data.embedded.length; ++i < length;) {
                    if (this.data.embedded[i].id === id) {
                        return this.data.embedded[i];
                    }
                }

                return null;
            },

            /**
             * Provides data of the list to the caller
             */
            provideData: function() {
                this.sandbox.emit(DATA_PROVIDE.call(this), this.data);
            },

            /**
             * calls the funciton of the view responsible for the responsiveness
             */
            windowResizeListener: function() {
                if (!!this.gridViews[this.viewId] && !!this.gridViews[this.viewId].onResize) {
                    this.gridViews[this.viewId].onResize();
                }
            },

            /**
             * Handles the record add event
             */
            addRecordHandler: function(recordData) {
                if (!this.gridViews[this.viewId].addRecord) {
                    return;
                }

                if (!!recordData[this.options.idKey]) {
                    this.pushRecords([recordData]);
                }
                this.gridViews[this.viewId].addRecord(recordData, false);
                this.sandbox.emit(NUMBER_SELECTIONS.call(this), this.getSelectedItemIds().length);
            },

            /**
             * Handles the event for adding multiple records
             * to a view
             * @param records {Array} array with new records to add
             * @param callback {Function} callback to execute after process has been finished
             */
            addRecordsHandler: function(records, callback) {
                if (!this.gridViews[this.viewId].addRecord) {
                    return;
                }

                this.sandbox.util.foreach(records, function(record) {
                    if (!!record[this.options.idKey]) {
                        this.pushRecords([record]);
                        this.gridViews[this.viewId].addRecord(record, false);
                    }
                }.bind(this));
                if (typeof callback === 'function') {
                    callback();
                }
                this.sandbox.emit(NUMBER_SELECTIONS.call(this), this.getSelectedItemIds().length);
            },

            /**
             * Handles the row remove event
             */
            removeRecordHandler: function(recordId) {
                if (!this.gridViews[this.viewId].removeRecord || !recordId) {
                    return;
                }

                this.gridViews[this.viewId].removeRecord(recordId);
                this.removeRecordFromSelected(recordId);
                this.sandbox.emit(NUMBER_SELECTIONS.call(this), this.getSelectedItemIds().length);
            },

            /**
             * Handles the row remove event
             */
            removeRecordsHandler: function(recordIds) {
                if (!this.gridViews[this.viewId].removeRecord || !recordIds) {
                    return;
                }

                this.sandbox.util.foreach(recordIds, function(recordId) {
                    this.gridViews[this.viewId].removeRecord(recordId);
                    this.removeRecordFromSelected(recordId);
                }.bind(this));
                this.sandbox.emit(NUMBER_SELECTIONS.call(this), this.getSelectedItemIds().length);
            },

            /**
             * Removes a record from the selected items
             * @param recordId
             */
            removeRecordFromSelected: function(recordId) {
                var index = this.selectedItems.indexOf(recordId);
                if (index > -1) {
                    this.selectedItems.splice(index, 1);
                }
            },

            /**
             * Merges one or more data-records with a given ones and updates the view
             * @param {Object|Array} records the new data-record or an array of data-records
             */
            changeRecordsHandler: function(records) {
                if (!this.sandbox.dom.isArray(records)) {
                    records = [records];
                }
                this.sandbox.util.foreach(records, function(record) {
                    this.changeRecord(record);
                }.bind(this));

                this.rerenderView();
                this.rerenderPagination();
            },

            /**
             * Deletes all records, sets given records and updates the view
             * @param {Array} records array of data-records
             */
            setRecordsHandler: function(records) {
                // Delete all records.
                this.data.embedded = [];

                // Add the records.
                this.pushRecords(records);

                this.rerenderView();
                this.rerenderPagination();
            },

            /**
             * Calls the given callback-function with all displayed records.
             * @param {Array} records array of data-records
             */
            getRecordsHandler: function(callback) {
                callback(this.sandbox.util.deepCopy(this.data.embedded));
            },

            /**
             * calls the clickCallback for an item
             * @param {Number|String} id the id of the item
             */
            itemClicked: function(id) {
                if (typeof this.options.clickCallback === 'function') {
                    this.options.clickCallback(id, this.data.embedded[this.getRecordIndexById(id)]);
                }
                this.sandbox.emit(ITEM_CLICK.call(this), id, this.data.embedded[this.getRecordIndexById(id)]);
            },

            /**
             * calls the actionCallback for an item
             * @param id {Number|String} the id of the item
             */
            itemAction: function(id) {
                if (typeof this.options.actionCallback === 'function') {
                    this.options.actionCallback(id, this.data.embedded[this.getRecordIndexById(id)]);
                }
            },

            /**
             * Returns the maximum height for the view to fit on screen
             * @returns {number}
             */
            getRemainingViewHeight: function() {
                var height = this.sandbox.dom.height(this.sandbox.dom.window) - this.sandbox.dom.offset(this.$element).top;
                if (!!this.paginations[this.paginationId] && !!this.paginations[this.paginationId].getHeight) {
                    height -= this.paginations[this.paginationId].getHeight();
                }
                height -= this.options.viewSpacingBottom;

                return height;
            },

            /**
             * Updates the selected-elements counter
             */
            updateSelectedCounter: function() {
                if (this.options.selectedCounter === true) {
                    this.sandbox.dom.html(this.$find('.selected-elements .number'), this.getSelectedItemIds().length);
                    if (this.getSelectedItemIds().length === 0) {
                        this.sandbox.dom.addClass(this.$find('.selected-elements'), 'invisible');
                    } else {
                        this.sandbox.dom.removeClass(this.$find('.selected-elements'), 'invisible');
                    }

                    if (!!this.gridViews[this.viewId] && !!this.gridViews[this.viewId].showSelected &&
                        this.getSelectedItemIds().length === 0
                    ) {
                        if (this.show === true && !!this.gridViews[this.viewId].rendered) {
                            this.gridViews[this.viewId].showSelected(false);
                        }
                        this.show = false;
                    }

                    if (!!this.show) {
                        this.sandbox.dom.text(
                            this.$find('.show-selected'),
                            this.sandbox.translate(this.options.selectedCounterShowAllText)
                        );
                    } else {
                        this.sandbox.dom.text(
                            this.$find('.show-selected'),
                            this.sandbox.translate(this.options.selectedCounterShowSelectedText)
                        );
                    }

                    if (!!this.gridViews[this.viewId].showSelected) {
                        this.sandbox.dom.show(this.$find('.show-selected-container'));
                    } else {
                        this.sandbox.dom.hide(this.$find('.show-selected-container'));
                    }

                }
            },

            /**
             * Methods for data manipulation
             * --------------------------------------------------------------------
             */

            /**
             * Sets all data records unselected
             */
            deselectAllItems: function() {
                this.selectedItems = [];
                // emit events with selected data
                this.sandbox.emit(ALL_DESELECT.call(this));
                this.sandbox.emit(NUMBER_SELECTIONS.call(this), 0);
                this.setSelectedItemsToData();
            },

            /**
             * Sets all data records selected
             */
            selectAllItems: function() {
                var i, length;
                for (i = -1, length = this.data.embedded.length; ++i < length;) {
                    this.setItemSelected(this.data.embedded[i].id);
                }
                this.sandbox.emit(ALL_SELECT.call(this), this.selectedItems);
                // emit events with selected data
                this.sandbox.emit(NUMBER_SELECTIONS.call(this), this.selectedItems.length);
                this.setSelectedItemsToData();
            },

            /**
             * Returns the ids of all selected items
             * @param selection {Array} list of selected items
             */
            updateSelection: function(selection) {
                var i, length;

                this.gridViews[this.viewId].deselectAllRecords();

                for (i = -1, length = selection.length; ++i < length;) {
                    this.gridViews[this.viewId].selectRecord(selection[i]);
                }
            },

            /**
             * Selects an item with a given id
             * @param itemId {Number|String} the id of the item to select
             */
            selectItem: function(itemId) {
                this.gridViews[this.viewId].selectRecord(itemId);
            },

            /**
             * Deselects an item with a given id
             * @param itemId {Number|String} the id of the item to select
             */
            deselectItem: function(itemId) {
                if (!!this.gridViews[this.viewId].deselectRecord) {
                    this.gridViews[this.viewId].deselectRecord(itemId);
                }
            },

            /**
             * Toogle an item with a given id
             * @param itemId {Number|String} the id of the item to select
             */
            toggleItem: function(itemId) {
                if (this.itemIsSelected(itemId)) {
                    this.deselectItem(itemId);
                } else {
                    this.selectItem(itemId);
                }
            },

            /**
             * Returns the ids of all selected items
             * @return {Array} array with all ids
             */
            getSelectedItemIds: function() {
                return this.selectedItems;
            },

            /**
             * Sets all data records with their ids contained in the passed array selected and
             * deselects all data records not contained.
             * @param items {Array} array with all items that should be selected
             */
            setSelectedItems: function(items) {
                var count = 0, i, length, position;
                for (i = -1, length = items.length; ++i < length;) {
                    if (this.selectedItems.indexOf(items[i]) === -1 && this.selectingAllowed(items[i])) {
                        this.selectedItems.push(items[i]);
                        count++;
                    } else if ((position = this.selectedItems.indexOf(items[i])) !== -1) {
                        this.selectedItems.splice(position, 1);
                    }
                }
                this.sandbox.emit(NUMBER_SELECTIONS.call(this), count);
            },

            /**
             * Returns true if an item with a given id is selected
             * @param id {Number|String} id of the item
             * @returns {Boolean} returns true if item is selected
             */
            itemIsSelected: function(id) {
                return this.selectedItems.indexOf(id) !== -1;
            },

            /**
             * Sets a data record with a given id selected
             * @param id {String|Number} id of the item
             * @return {Boolean} true of operation was successfull
             */
            setItemSelected: function(id) {
                if (this.selectedItems.indexOf(id) === -1) {
                    this.selectedItems.push(id);
                    // emit events with selected data
                    this.sandbox.emit(ITEM_SELECT.call(this), id, this.getRecordById(id));
                    this.sandbox.emit(NUMBER_SELECTIONS.call(this), this.getSelectedItemIds().length);
                    this.setSelectedItemsToData();
                    return true;
                }
                return false;
            },

            /**
             * Sets a data record with a given id unselected
             * @param id
             * @return {Boolean} true of operation was succesfull
             */
            setItemUnselected: function(id) {
                var position;

                if ((position = this.selectedItems.indexOf(id)) !== -1) {
                    this.selectedItems.splice(position, 1);
                    // emit events with selected data
                    this.sandbox.emit(ITEM_DESELECT.call(this), id);
                    this.sandbox.emit(NUMBER_SELECTIONS.call(this), this.getSelectedItemIds().length);
                    this.setSelectedItemsToData();
                    return true;
                }
                return false;
            },

            /**
             * Returns true if selection for the data-record is allowed
             * @param id {Number|String} the id of the data-record
             */
            selectingAllowed: function(id) {
                var itemIndex = this.getRecordIndexById(id);
                if (this.options.onlySelectLeaves === true && this.data.embedded[itemIndex][this.options.childrenPropertyName]) {
                    return false;
                }
                return true;
            },

            /**
             * updates the current url by given parameter object
             * @param {Object} parameters Object key is used as parameter name, value as parameter value
             */
            updateUrl: function(parameters) {
                var url, key;

                url = this.currentUrl;

                for (key in parameters) {
                    url = setGetParameter.call(this, url, key, parameters[key]);
                }

                this.destroy();
                this.loading();
                this.load({
                    url: url,
                    success: function(data) {
                        this.sandbox.emit(UPDATED.call(this), data);
                    }.bind(this)
                });
            },

            /**
             * Takes a record and overrides the one with the same id
             * @param record {Object} new record to set. Needs at leas an id property
             * @returns {boolean} true if record got successfully overridden
             */
            updateRecord: function(record) {
                for (var i = -1, length = this.data.embedded.length; ++i < length;) {
                    if (record[this.options.idKey] === this.data.embedded[i].id) {
                        this.data.embedded[i] = record;
                        return true;
                    }
                }
                return false;
            },

            /**
             * Merges a data-record with a given one
             * @param record {Object} the new data-record. Must contain an id-property
             * @returns {Boolean} returns true if changed successfully
             */
            changeRecord: function(record) {
                if (!!record[this.options.idKey]) {
                    for (var i = -1, length = this.data.embedded.length; ++i < length;) {
                        if (record[this.options.idKey] === this.data.embedded[i].id) {
                            this.data.embedded[i] = this.sandbox.util.extend(true, {}, this.data.embedded[i], record);
                            return true;
                        }
                    }
                } else {
                    this.sandbox.logger.log('Error: Failed changing a record. Must contain id-property');
                    return false;
                }
            },

            /**
             * Deletes a record with a given id
             * @param recordId {Number|String} the id of the record to delete
             * @returns {boolean} true if record got successfully deleted
             */
            removeRecord: function(recordId) {
                if (!!recordId) {
                    var i, length;
                    for (i = -1, length = this.data.embedded.length; ++i < length;) {
                        if (recordId === this.data.embedded[i].id) {
                            this.removeRecordFromSelected(recordId);
                            this.data.embedded.splice(i, 1);
                            this.rerenderPagination();
                            return true;
                        }
                    }
                }
                return false;
            },

            /**
             * Takes an array of records and pushes them to the global array
             * @param records {Array} records to push
             */
            pushRecords: function(records) {
                for (var i = -1, length = records.length; ++i < length;) {
                    this.data.embedded.push(records[i]);
                }
                this.rerenderPagination();
            },

            /**
             * Takes an array of records and unshifts them to the global array
             * @param records {Array} records to push
             */
            unshiftRecords: function(records) {
                for (var i = -1, length = records.length; ++i < length;) {
                    this.data.embedded.unshift(records[i]);
                    this.rerenderPagination();
                }
            },

            /**
             * Called when the current page should change
             * Emits husky.datagrid.updated event on success
             * @param uri {String} Url to load the new data from
             * @param page {Number} the page to change to
             * @param limit {Number} new page size. Has to be set if no Uri is passed
             */
            changePage: function(uri, page, limit) {
                if (!!this.data.links.pagination || !!uri) {
                    var def = this.sandbox.data.deferred();
                    this.show = false;
                    this.updateSelectedCounter();

                    var url, uriTemplate;

                    // if a url is passed load the data from this url
                    if (!!uri) {
                        url = uri;

                        // else generate an own uri
                    } else {
                        // return if no page is passed or passed invalidly
                        if (!page || page > this.data.pages || page < 1) {
                            this.sandbox.logger.log("invalid page number or reached start/end!");
                            def.resolve();
                            return def;
                        }
                        // if no limit is passed keep current limit
                        if (!limit) {
                            limit = this.data.limit;
                        } else if (this.data.limit !== limit) {
                            this.sandbox.emit(PAGE_SIZE_CHANGED.call(this), limit, this.paginationId);
                        }

                        // generate uri for loading
                        uriTemplate = this.sandbox.uritemplate.parse(this.data.links.pagination.href);
                        url = this.sandbox.uritemplate.expand(uriTemplate, {page: page, limit: limit});
                    }

                    this.sandbox.emit(PAGE_CHANGE.call(this), url);
                    this.load({
                        url: url,
                        success: function() {
                            def.resolve();
                            this.sandbox.emit(UPDATED.call(this), 'changed page');
                        }.bind(this)
                    });

                    return def;
                }
            },

            /**
             * Updates data in datagrid
             * Called when husky.datagrid.update event emitted
             * Emits husky.datagrid.updated event on success
             */
            updateGrid: function() {
                if (!!this.data.links.self) {
                    this.resetSortingOptions();

                    this.load({
                        url: this.data.links.self.href,
                        success: function() {
                            this.sandbox.emit(UPDATED.call(this));
                        }.bind(this)
                    });
                }
            },

            /**
             * Filters fields out of the data passed to the view
             * So its possible to hide fields/columns
             * Note that the also the arrangement of fields/columns can be changed and the view can react to it
             * @param {Array} matchings
             */
            filterGrid: function(matchings) {
                if (!!this.data.links.filter) {
                    var uriTemplate, url;

                    this.filterMatchings(matchings);

                    uriTemplate = this.sandbox.uritemplate.parse(this.data.links.filter.href);
                    url = this.sandbox.uritemplate.expand(uriTemplate, {fieldsList: this.requestFields.join(',')});

                    this.destroy();
                    this.loading();
                    this.load({
                        url: url,
                        success: function() {
                            this.sandbox.emit(UPDATED.call(this));
                        }.bind(this)
                    });
                }
            },

            /**
             * Constructs the url to get sorted data and sets the request for it
             * @param attribute {String} The attribute to sort by
             * @param direction {String} the sort method to use 'asc' or 'desc'
             */
            sortGrid: function(attribute, direction) {
                if (this.options.sortable === true && !!this.data.links.sortable) {
                    var template, url;

                    // if passed attribute is sortable
                    if (!!attribute) {

                        this.sort.attribute = attribute;
                        this.sort.direction = direction;

                        template = this.sandbox.uritemplate.parse(this.data.links.sortable.href);
                        url = this.sandbox.uritemplate.expand(template, {sortBy: attribute, sortOrder: direction});

                        this.sandbox.emit(DATA_SORT.call(this), {attribute: attribute, direction: direction});

                        this.load({
                            url: url,
                            success: function(data) {
                                this.sandbox.emit(UPDATED.call(this), data);
                            }.bind(this)
                        });
                    }
                }
            },

            /**
             * Loads the children of a record
             * @param recordId {Number|String}
             */
            loadChildren: function(recordId) {
                var deferred = this.sandbox.data.deferred();

                if (!!this.data.links.children) {
                    var template = this.sandbox.uritemplate.parse(this.data.links.children.href),
                        url = this.sandbox.uritemplate.expand(template, {parentId: recordId});

                    this.sandbox.util.load(this.getUrl({url: url}))
                        .then(function(response) {
                            this.addRecordsHandler(response._embedded[this.options.resultKey]);
                            deferred.resolve();
                        }.bind(this))
                        .fail(function(status, error) {
                            this.sandbox.logger.error(status, error);
                            deferred.fail();
                        }.bind(this));
                } else {
                    deferred.resolve();
                }

                return deferred;
            },

            /**
             * triggers an api search
             * @param {String} searchString The String that will be searched
             * @param {String|Array} searchFields Fields that will be included into the search
             */
            searchGrid: function(searchString, searchFields) {
                if (!!this.data.links.find) {
                    var template, url;

                    template = this.sandbox.uritemplate.parse(this.data.links.find.href);

                    if (!!searchFields) {
                        url = this.sandbox.uritemplate.expand(template, {
                            searchString: searchString,
                            searchFields: searchFields
                        });
                    } else {
                        url = this.sandbox.uritemplate.expand(template, {
                            searchString: searchString,
                            searchFields: this.options.searchFields.join(',')
                        });
                    }

                    this.destroy();
                    this.loading();
                    this.load({
                        url: url,
                        success: function() {
                            this.sandbox.emit(UPDATED.call(this));
                        }.bind(this)
                    });
                }
            },

            /**
             * Takes a data-record and sends it to a url
             * @param data {Object} the data object to save
             * @param url {String} Url to send the data to
             * @param success {Function} callback to call after saving
             * @param fail {Function} callback to call if saving has failed
             * @param unshift {Boolean} If true newly added records will be unshifted instead of pushed
             */
            saveGrid: function(data, url, success, fail, unshift) {
                var method = 'POST',
                    isNewRecord = true;

                if (!!data.id) {
                    method = 'PUT';
                    isNewRecord = false;
                    url = url + '/' + data.id;
                }

                for (var key in this.options.saveParams) {
                    url = setGetParameter.call(this, url, key, this.options.saveParams[key]);
                }

                this.sandbox.emit(DATA_CHANGED.call(this));

                this.sandbox.util.save(url, method, data)
                    .then(function(data, textStatus) {
                        this.sandbox.emit(DATA_SAVED.call(this), data, textStatus);

                        if (typeof success === 'function') {
                            success(data, textStatus);
                        }

                        // update the global data array
                        if (isNewRecord === false) {
                            this.updateRecord(data);
                        } else if (unshift === true) {
                            this.unshiftRecords([data]);
                        } else {
                            this.pushRecords([data]);
                        }
                    }.bind(this))
                    .fail(function(jqXHR, textStatus, error) {
                        this.sandbox.emit(DATA_SAVE_FAILED.call(this), jqXHR, textStatus, error, data);

                        if (typeof fail === 'function') {
                            fail(jqXHR, textStatus, error);
                        }

                    }.bind(this));
            }
        };
    });
})();

/*
 * This file is part of the Sulu CMS.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * Name: dropdown
 * Options:
 *
 *
 * Provided Events:
 * husky.dropdown.<<instanceName>>.item.click   - triggered when item clicked
 * husky.dropdown.<<instanceName>>.data.load    - triggered when data loaded
 *
 * Use Events
 * husky.dropdown.<<instanceName>>.toggle       - toggles (show/hide) dropdown menu
 * husky.dropdown.<<instanceName>>.show       - show dropdown menu
 * husky.dropdown.<<instanceName>>.hide       - hide dropdown menu
 */

define('__component__$dropdown@husky',['underscore'], function(_) {

    'use strict';

    var moduleName = 'Husky.Ui.DropDown',
        defaults = {
            url: '',     // url for lazy loading
            data: [],    // data array
            trigger: '',  // trigger for click event
            triggerOutside: false, // if true, trigger is searched in whole dom, otherwise just in this.$el
            shadow: true,  // if box-shadow should be shown
            toggleClassOn: null, // container to set is-active class
            valueName: 'name', // name of text property
            infoName: 'info', // name of info property (used to display hover info)
            clickedName: 'clickedInfo', // name of info property (used to display clicked info)
            setParentDropDown: false, // set class dropdown for parent dom object
            excludeItems: [], // items to filter,
            instanceName: 'undefined',  // instance name
            alignment: 'left',  // alignment of the arrow and the box
            verticalAlignment: 'bottom', // alignment of the box
            translateLabels: true,   // translate labels withe globalize
            clickDelay: 1000
        };

    return {
        initialize: function() {
            this.name = moduleName;

            this.options = this.sandbox.util.extend({}, defaults, this.options);

            this.$element = this.sandbox.dom.createElement('<div/>', {
                'class': 'husky-drop-down'
            });

            this.sandbox.dom.append(this.options.el, this.$element);

            this.render();
            this.sandbox.emit('husky.dropdown.' + this.options.instanceName + '.initialized');
        },

        render: function() {

            var dropdownClasses = ['dropdown-menu'];

            if (this.options.shadow === true) {
                dropdownClasses.push('dropdown-shadow');
            }

            this.$dropDown = this.sandbox.dom.createElement('<div/>', {
                'class': dropdownClasses.join(' ')
            });
            this.$dropDownList = this.sandbox.dom.createElement('<ul/>');

            this.sandbox.dom.append(this.$element, this.$dropDown);
            this.sandbox.dom.append(this.$dropDown, this.$dropDownList);
            this.hideDropDown();

            if (this.options.setParentDropDown) {
                // add class dropdown to parent
                this.sandbox.dom.addClass(this.sandbox.dom.parent(this.$element), 'dropdown');
            }

            // check alignment
            if (this.options.alignment === 'right') {
                this.sandbox.dom.addClass(this.$dropDown, 'dropdown-align-right');
            }

            // check vertical alignment
            if (this.options.verticalAlignment === 'top') {
                this.sandbox.dom.addClass(this.$dropDown, 'dropdown-align-top');
            }

            // bind dom elements
            this.bindDOMEvents();
            this.bindCustomEvents();

            // load data
            this.prepareData();
        },

        // bind dom elements
        bindDOMEvents: function() {

            // turn off all events
            this.sandbox.dom.off(this.$element);

            // ------------------------------------------------------------
            // DOM events
            // ------------------------------------------------------------

            // init drop-down
            if (this.options.trigger !== '') {
                if (this.options.triggerOutside) {
                    $(this.options.trigger).addClass('husky-dropdown-trigger');
                    this.sandbox.dom.on(this.options.trigger, 'click', this.triggerClick.bind(this));
                } else {
                    $(this.options.el).addClass('husky-dropdown-trigger');
                    this.sandbox.dom.on(this.options.el, 'click', this.triggerClick.bind(this), this.options.trigger);
                }
            } else {
                $(this.options.el).addClass('husky-dropdown-trigger');
                this.sandbox.dom.on(this.options.el, 'click', this.triggerClick.bind(this));
            }

            // on click on list item
            this.sandbox.dom.on(this.$dropDownList, 'click', function(event) {
                if (!this.sandbox.dom.hasClass(event.currentTarget, 'disabled')) {
                    this.clickItem(this.sandbox.dom.data(event.currentTarget, 'id'));
                }
            }.bind(this), 'li:not(".divider")');
        },

        bindCustomEvents: function() {
            this.sandbox.on(this.getEvent('toggle'), this.triggerClick.bind(this));
            this.sandbox.on(this.getEvent('show'), this.showDropDown.bind(this));
            this.sandbox.on(this.getEvent('hide'), this.hideDropDown.bind(this));
            this.sandbox.on(this.getEvent('item.enable'), this.itemEnable.bind(this));
            this.sandbox.on(this.getEvent('item.disable'), this.itemDisable.bind(this));
            this.sandbox.on(this.getEvent('item.toggle'), this.itemToggle.bind(this));
        },

        getEvent: function(append) {
            return 'husky.dropdown.' + this.options.instanceName + '.' + append;
        },

        itemToggle: function(id, enable) {
            if (enable === true) {
                this.itemEnable(id);
            } else {
                this.itemDisable(id);
            }
        },

        itemEnable: function(id) {
            this.sandbox.dom.removeClass(this.$find('li[data-id="' + id + '"]'), 'disabled');
        },

        itemDisable: function(id) {
            this.sandbox.dom.addClass(this.$find('li[data-id="' + id + '"]'), 'disabled');
        },

        // trigger event with clicked item
        clickItem: function(id) {
            var item = this.getItemById(id),
                $item = this.$el.find('*[data-id="' + id + '"]'),
                $infos;

            if (!item) {
                return;
            }

            if (!!item.callback && typeof item.callback === 'function') {
                item.callback.call(this);
            } else if (!!this.options.clickCallback && typeof this.options.clickCallback === 'function') {
                this.options.clickCallback(item, this.$el);
            } else {
                this.sandbox.emit(this.getEvent('item.click'), item, this.$el);
            }

            if (!item[this.options.clickedName]) {
                return this.hideDropDown();
            }

            $infos = $item.find('.info, .info-clicked');
            $item.addClass('highlight-animation');
            $infos.addClass('clicked');

            _.delay(function() {
                $item.removeClass('highlight-animation');
                $infos.removeClass('clicked');

                return this.hideDropDown();
            }.bind(this), this.options.clickDelay);
        },

        getItemById: function(id) {
            for (var i = 0, length = this.options.data.length, item; i < length; ++i) {
                item = this.options.data[i];
                if (typeof item.id !== 'undefined' && item.id.toString() === id.toString()) {
                    return item;
                }
            }
        },

        // trigger click event handler toggles the dropDown
        triggerClick: function(event) {
            var $target = $(event.target);
            if ($target.hasClass('husky-dropdown-item')
                || $target.parents().hasClass('husky-dropdown-item')
            ) {
                return;
            }

            this.toggleDropDown();
        },

        // prepares data for dropDown, if options.data not set load with ajax
        prepareData: function() {
            if (this.options.data.length > 0) {
                this.generateDropDown(this.options.data);
            } else {
                this.loadData();
            }
        },

        // load data with ajax
        loadData: function() {
            var url = this.getUrl();
            this.sandbox.logger.log(this.name, 'load: ' + url);

            this.sandbox.util.ajax({
                url: url,
                success: function(response) {
                    this.sandbox.logger.log(this.name, 'load', 'success');

                    if (response.total > 0 && response.items.length === response.total) {
                        this.options.data = response.items;
                    } else {
                        this.options.data = [];
                    }
                    this.generateDropDown(this.options.data);
                }.bind(this),
                error: function() {
                    this.sandbox.logger.log(this.name, 'load', 'error');

                    this.options.data = [];
                    this.generateDropDown(this.options.data);
                }.bind(this)
            });

            // FIXME event will be bound later
            setTimeout(function() {
                this.sandbox.emit(this.getEvent('data.load'));
            }.bind(this), 200);
        },

        // generate dropDown with given items
        generateDropDown: function(items) {
            this.clearDropDown();
            if (items.length > 0) {
                items.forEach(function(item) {
                    if (item.divider !== true) {
                        if (this.isVisible(item)) {
                            var label = item[this.options.valueName],
                                info = item[this.options.infoName],
                                clicked = item[this.options.clickedName],
                                template;
                            if (this.options.translateLabels) {
                                label = this.sandbox.translate(label);
                                info = this.sandbox.translate(info);
                                clicked = this.sandbox.translate(clicked);
                            }

                            label = this.sandbox.util.cropMiddle(label, 35);
                            template = [
                                '<li class="husky-dropdown-item" data-id="', item.id, '">',
                                '<span class="label">', label, '</span>',
                                (info !== undefined ? ('<span class="info">' + info + '</span>') : ''),
                                (clicked !== undefined ? ('<span class="info-clicked">' + clicked + '</span>') : ''),
                                '</li>'
                            ].join('');

                            this.sandbox.dom.append(this.$dropDownList, template);
                        }
                    } else {
                        this.sandbox.dom.append(this.$dropDownList, '<li class="divider"/>');
                    }
                }.bind(this));
            } else {
                this.sandbox.dom.append(this.$dropDownList, '<li>No data received</li>');
            }

            this.sandbox.emit(this.getEvent('rendered'));
        },

        // is item visible (filter)
        isVisible: function(item) {
            var result = true;
            this.options.excludeItems.forEach(function(testItem) {
                if (item.id === testItem.id) {
                    result = false;
                }
            }.bind(this));

            return result;
        },

        // clear childs of list
        clearDropDown: function() {
            // FIXME make it easier
            this.sandbox.dom.remove(this.sandbox.dom.children(this.sandbox.dom.children(this.$dropDown, 'ul'), 'li'));
        },

        // toggle dropDown visible
        toggleDropDown: function() {
            this.sandbox.logger.log(this.name, 'toggle dropdown');
            if (this.sandbox.dom.is(this.$dropDown, ':visible')) {
                this.hideDropDown();
            } else {
                this.showDropDown();
            }
        },

        // make dropDown visible
        showDropDown: function() {
            this.sandbox.logger.log(this.name, 'show dropdown');
            this.sandbox.dom.removeClass(this.$dropDown, 'top');
            // on click on trigger outside check
            this.sandbox.dom.one(this.sandbox.dom.window, 'click', this.hideDropDown.bind(this));
            this.sandbox.dom.show(this.$dropDown);
            this.sandbox.emit('husky.dropdown.' + this.options.instanceName + '.showing');
            if (!!this.options.toggleClassOn) {
                this.sandbox.dom.addClass(this.options.toggleClassOn, 'is-active');
            }
            // add up class if dropdown would pass the screen borders
            if ((this.sandbox.dom.offset(this.$dropDown).top - this.sandbox.dom.scrollTop(this.sandbox.dom.window) +
                this.sandbox.dom.outerHeight(this.$dropDown)) > this.sandbox.dom.height(this.sandbox.dom.window)) {
                this.sandbox.dom.addClass(this.$dropDown, 'top');
            }
        },

        // hide dropDown
        hideDropDown: function(event) {
            // do not hide if the target is children of trigger but not children of item or the element was highlighted
            // reapply the hide listener
            if (!!event) {
                var $target = $(event.target);

                if ($target.is('.husky-dropdown-trigger:not(.husky-dropdown-item), .highlight-animation')
                    || $target.parents().is('.husky-dropdown-trigger:not(.husky-dropdown-item), .highlight-animation')
                ) {
                    this.sandbox.dom.one(this.sandbox.dom.window, 'click', this.hideDropDown.bind(this));

                    return;
                }
            }

            this.sandbox.logger.log(this.name, 'hide dropdown');
            // remove global click event
            this.sandbox.dom.off(this.sandbox.dom.window, 'click', this.hideDropDown.bind(this));
            this.sandbox.dom.hide(this.$dropDown);
            if (!!this.options.toggleClassOn) {
                this.sandbox.dom.removeClass(this.options.toggleClassOn, 'is-active');
            }
        },

        // get url for pattern
        getUrl: function() {
            return this.options.url;
        }

    };

});

/*
 * This file is part of the Sulu CMS.
 *
 * (c) MASSIVE ART Webservices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

define('__component__$matrix@husky',[],function() {

    'use strict';

    var activeClass = 'is-active';

    return {
        initialize: function() {
            this.$element = this.sandbox.dom.createElement('<div class="husky-matrix"/>');

            this.prepare();

            this.sandbox.dom.append(this.$el, this.$element);

            this.bindDOMEvents();
            this.bindCustomEvents();
        },

        bindDOMEvents: function() {
            this.sandbox.dom.on(
                this.$element,
                'click',
                this.toggleIcon.bind(this),
                'tbody > tr > td.value > span[class^="fa-"]'
            );
            this.sandbox.dom.on(this.$element, 'click', this.toggleRow.bind(this), 'tbody > tr > td:last-child');
        },

        bindCustomEvents: function() {
            this.sandbox.on('husky.matrix.set-all', this.setAllActive.bind(this));
            this.sandbox.on('husky.matrix.unset-all', this.unsetAllActive.bind(this));
        },

        toggleIcon: function(event) {
            var $target = event.currentTarget,
                $tr = this.sandbox.dom.parent($target),
                $allTargets = this.sandbox.dom.find('span[class^="fa-"]', $tr),
                $activeTargets,
                $link = this.sandbox.dom.find('td:last-child span', $tr);

            this.sandbox.dom.toggleClass($target, activeClass);

            $activeTargets = this.sandbox.dom.find('span[class^="fa-"].' + activeClass, $tr);
            if ($activeTargets.length < $allTargets.length) {
                this.sandbox.dom.html($link, this.options.captions.all);
            } else {
                this.sandbox.dom.html($link, this.options.captions.none);
            }

            // emit events for communication with the outside
            this.sandbox.emit('husky.matrix.changed', {
                section: this.sandbox.dom.data($target, 'section'),
                value: this.sandbox.dom.data($target, 'value'),
                activated: this.sandbox.dom.hasClass($target, activeClass)
            });
        },

        toggleRow: function(event) {
            var $tr = this.sandbox.dom.parent(event.currentTarget),
                $targets = this.sandbox.dom.find('span[class^="fa-"]', $tr),
                $activeTargets = this.sandbox.dom.find('span[class^="fa-"].' + activeClass, $tr),
                $link = this.sandbox.dom.find('td:last-child span', $tr), activated;

            if ($activeTargets.length < $targets.length) {
                this.sandbox.dom.addClass($targets, activeClass);
                this.sandbox.dom.html($link, this.options.captions.none);
                activated = true;
            } else {
                this.sandbox.dom.removeClass($targets, activeClass);
                this.sandbox.dom.html($link, this.options.captions.all);
                activated = false;
            }

            // emit events for communication with the outside
            this.sandbox.emit('husky.matrix.changed', {
                section: this.sandbox.dom.data($targets, 'section'),
                value: this.options.values.horizontal[$tr.data('row-count')],
                activated: activated
            });
        },

        setAllActive: function() {
            var $targets = this.sandbox.dom.find('span[class^="fa-"]', this.$element),
                $trs = this.sandbox.dom.find('tbody > tr', this.$element);
            this.sandbox.dom.addClass($targets, activeClass);

            // emit events for communication with the outside
            this.sandbox.dom.each($trs, function(key, tr) {
                this.sandbox.emit('husky.matrix.changed', {
                    section: this.sandbox.dom.data(this.sandbox.dom.find('td.section', tr), 'section'),
                    value: this.options.values.horizontal[$(tr).data('row-count')],
                    activated: true
                });
            }.bind(this));
        },

        unsetAllActive: function() {
            var $targets = this.sandbox.dom.find('span[class^="fa-"]', this.$element),
                $trs = this.sandbox.dom.find('tbody > tr', this.$element);
            this.sandbox.dom.removeClass($targets, activeClass);

            // emit events for communication with the outsite
            this.sandbox.dom.each($trs, function(key, tr) {
                this.sandbox.emit('husky.matrix.changed', {
                    section: this.sandbox.dom.data(this.sandbox.dom.find('td.section', tr), 'section'),
                    value: this.options.values.horizontal[$(tr).data('row-count')],
                    activated: false
                });
            }.bind(this));
        },

        prepare: function() {
            this.$element.append(this.prepareTable());
        },

        prepareTable: function() {
            var $table;

            $table = this.sandbox.dom.createElement('<table class="table matrix"/>');
            this.sandbox.dom.append($table, this.prepareTableHead());

            if (!!this.options.captions.vertical) {
                this.sandbox.dom.append($table, this.prepareTableBody());
            }

            return $table;
        },

        prepareTableHead: function() {
            var $thead = this.sandbox.dom.createElement('<thead/>'),
                $tr = this.sandbox.dom.createElement('<tr/>'),
                $thSection = this.sandbox.dom.createElement('<th class="section"/>'),
                $thGeneral, $th;

            if (!!this.options.captions.general) {
                $thGeneral = this.sandbox.dom.createElement('<th class="general"/>');
                this.sandbox.dom.html($thGeneral, this.options.captions.general);
                this.sandbox.dom.append($tr, $thGeneral);
            }

            this.sandbox.dom.html($thSection, this.options.captions.type);

            this.sandbox.dom.append($tr, $thSection);

            if (typeof(this.options.captions.horizontal) === 'string') {
                // insert a header for all values, if the horizontal caption is a string
                $th = this.sandbox.dom.createElement('<th/>');
                this.sandbox.dom.html($th, this.options.captions.horizontal);
                this.sandbox.dom.append($tr, $th);
            } else {
                // insert the corresponding headers, if the horizontal caption is an array
                this.options.captions.horizontal.forEach(function(caption) {
                    var $th = this.sandbox.dom.createElement('<th/>');
                    this.sandbox.dom.html($th, caption);
                    this.sandbox.dom.append($tr, $th);
                }.bind(this));
            }

            // add empty th for all link
            this.sandbox.dom.append($tr, this.sandbox.dom.createElement('<th/>'));

            this.sandbox.dom.append($thead, $tr);

            return $thead;
        },

        prepareTableBody: function() {
            var $tbody = this.sandbox.dom.createElement('<tbody/>'),
                rowCount;

            for (rowCount = 0; rowCount < this.options.captions.vertical.length; rowCount++) {
                this.prepareTableRow($tbody, rowCount);
            }

            return $tbody;
        },

        prepareTableRow: function($tbody, rowCount) {
            var $tr = this.sandbox.dom.createElement('<tr/>'),
                $tdHead = this.sandbox.dom.createElement('<td class="section"/>'),
                $tdValue = this.sandbox.dom.createElement('<td class="value"/>'),
                $tdAll = this.sandbox.dom.createElement('<td class="all"/>'),
                allActive;

            $tr.data('row-count', rowCount);

            // insert empty line, if there is a general caption
            if (!!this.options.captions.general) {
                this.sandbox.dom.append($tr, this.sandbox.dom.createElement('<td/>'));
            }

            // insert vertical headlines as first element
            this.sandbox.dom.html($tdHead, this.options.captions.vertical[rowCount]);
            this.sandbox.dom.data($tdHead, 'section', this.options.values.vertical[rowCount]);
            this.sandbox.dom.append($tr, $tdHead);

            // flag for checking if every flag is true
            // insert values of matrix
            allActive = this.prepareTableColumn(
                $tdValue,
                this.options.values.horizontal[rowCount],
                this.options.values.vertical[rowCount],
                this.options.data[rowCount]
            );

            this.sandbox.dom.append($tr, $tdValue);

            //add all link
            this.sandbox.dom.html(
                $tdAll,
                [
                    '<span class="pointer">',
                    (!!allActive) ? this.options.captions.none : this.options.captions.all,
                    '</span>'
                ].join('')
            );
            this.sandbox.dom.append($tr, $tdAll);

            this.sandbox.dom.append($tbody, $tr);
        },

        prepareTableColumn: function($tdValue, columnData, section, value) {
            var allActive = true,
                title,
                $span,
                i;

            for (i = 0; i < columnData.length; i++) {
                if (columnData[i].title) {
                    title = 'title="' + this.sandbox.translate(columnData[i].title) + '"';
                } else {
                    title = '';
                }
                $span = this.sandbox.dom.createElement(
                    '<span ' + title +
                    ' class="fa-' + columnData[i].icon + ' matrix-icon pointer"/>'
                );
                this.sandbox.dom.data($span, 'value', columnData[i].value);
                this.sandbox.dom.data($span, 'section', section);

                // set activated if set in delivered data
                if (!!value && !!value[i]) {
                    this.sandbox.dom.addClass($span, activeClass);
                } else {
                    // set the flag to false if there is one
                    allActive = false;
                }

                this.sandbox.dom.append($tdValue, $span);
            }

            return allActive;
        }
    };
});

/**
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

/**
 * @class Search
 * @constructor
 *
 * @param {Object} [options] Configuration object
 * @param {String} [options.instanceName=null] allows to createa custom events (in case of multiple tabs on one page)
 * @param {String} [options.placeholderText=Search...] the text to be shown as placeholder
 * @param {String} [options.appearance=gray] appearance can be 'gray', 'white' or 'small'
 */
define('__component__$search@husky',[], function() {

    'use strict';

    var templates = {
            skeleton: [
                '<a href="#" class="fa-search fa-flip-horizontal search-icon" />',
                '<a href="#" class="fa-times-circle remove-icon" />',
                '<input id="search-input" type="text" class="form-element input-round search-input" placeholder="<%= placeholderText %>"/>'
            ].join('')
        },
        defaults = {
            instanceName: null,
            placeholderText: 'public.search',
            appearance: 'gray',
            slide: false
        },
        constants = {
            slideClass: 'search-slide',
            slideCollapsedClass: 'slide-collapsed',
            slideExpandedClass: 'slide-expanded'
        },

        /**
         * triggered when user clicked on search icon or pressed enter
         * @event husky.search[.INSTANCE_NAME]
         * @param {String} the string thats been looked for
         */
        SEARCH = function() {
            return this.getEventName();
        },

        /**
         * triggered when user clicks on reset button of search
         * @event husky.search[.INSTANCE_NAME].reset
         */
        RESET = function() {
            return this.getEventName('reset');
        },

        /**
         * clear value if search field
         * @event husky.search[.INSTANCE_NAME].clear
         */
        CLEAR = function() {
            return this.getEventName('clear');
        },

        /**
         * triggered when user clicks on search icon or pressed enter
         * @event husky.search[.INSTANCE_NAME].initialized
         */
        INITIALIZED = function() {
            return this.getEventName('initialized');
        };

    return {

        initialize: function() {
            this.sandbox.logger.log('initialize', this);
            this.options = this.sandbox.util.extend({}, defaults, this.options);

            this.render();

            this.bindDOMEvents();
            this.bindCustomEvents();

            this.sandbox.emit(INITIALIZED.call(this));
        },

        bindCustomEvents: function() {
            this.sandbox.on(CLEAR.call(this), function() {
                this.removeSearch(null, true);
            }.bind(this));
        },

        render: function() {
            this.sandbox.dom.addClass(this.$el, 'search-container');
            this.sandbox.dom.addClass(this.$el, this.options.appearance);
            if (this.options.slide === true) {
                this.sandbox.dom.addClass(this.$el, constants.slideClass);
                this.collapse();
            }

            this.sandbox.dom.html(this.$el, this.sandbox.template.parse(templates.skeleton, {placeholderText: this.sandbox.translate(this.options.placeholderText)}));
        },

        // bind dom elements
        bindDOMEvents: function() {
            this.sandbox.dom.on(this.$el, 'click', this.selectInput.bind(this), 'input');
            this.sandbox.dom.on(this.$el, 'keyup onchange', this.checkKeyPressed.bind(this), '#search-input');
            this.sandbox.dom.on(this.$find('input'), 'focus', function() {
                this.sandbox.dom.addClass(this.$el, 'focus');
            }.bind(this));

            this.sandbox.dom.on(this.$el, 'click', this.submitSearch.bind(this), '.search-icon');
            this.sandbox.dom.on(this.$el, 'click', this.removeSearch.bind(this), '.remove-icon');

            this.sandbox.dom.on(this.$find('input'), 'blur', function() {
                this.sandbox.dom.removeClass(this.$el, 'focus');
            }.bind(this));

            if (this.options.slide === true) {
                this.sandbox.dom.on(this.$find('input'), 'focus', this.expand.bind(this));
                this.sandbox.dom.on(this.$find('input'), 'blur', this.collapse.bind(this));
            }

            this.$el.on('keydown keyup', function(event) {
                event.stopPropagation();
            }.bind(this));
        },

        collapse: function() {
            this.sandbox.dom.removeClass(this.$el, constants.slideExpandedClass);
            this.sandbox.dom.addClass(this.$el, constants.slideCollapsedClass);
        },

        expand: function() {
            this.sandbox.dom.removeClass(this.$el, constants.slideCollapsedClass);
            this.sandbox.dom.addClass(this.$el, constants.slideExpandedClass);
        },

        resetSearch: function(noEmit) {
            if(!noEmit) {
                this.sandbox.emit(RESET.call(this));
            }

            this.searchSubmitted = false;
        },

        checkKeyPressed: function(event) {

            var $removeIcon;

            // when search contains text show remove icon
            $removeIcon = this.sandbox.dom.prev(event.currentTarget, '.remove-icon');
            if (this.sandbox.dom.val(event.currentTarget).length > 0) {
                this.sandbox.dom.show($removeIcon);
            } else {
                this.sandbox.dom.hide($removeIcon);
            }

            if (event.keyCode === 13) {
                // enter pressed
                this.submitSearch();
            } else if (event.keyCode === 27) {
                // escape pressed
                this.removeSearch();
            }
        },

        submitSearch: function(event) {
            if (!!event) {
                this.sandbox.dom.preventDefault(event);
            }

            // get search value
            var searchString = this.sandbox.dom.val(this.sandbox.dom.find('#search-input', this.$el));

            // if searchstring is emtpy, emit reset
            if (searchString === '') {
                if (this.searchSubmitted) {
                    this.resetSearch();
                }

                return;
            }

            this.searchSubmitted = true;

            // emit event
            this.sandbox.emit(SEARCH.call(this), searchString);
        },

        removeSearch: function(event, noEmit) {
            if (!!event) {
                this.sandbox.dom.preventDefault(event);
            } else {
                event = {
                    target: this.sandbox.dom.find('.remove-icon', this.$el),
                    currentTarget: this.sandbox.dom.find('.remove-icon', this.$el)
                };
            }
            var $input = this.sandbox.dom.next(event.currentTarget, 'input');

            this.sandbox.dom.hide(event.target);
            this.sandbox.dom.val($input, '');
            if (this.searchSubmitted) {
                this.resetSearch(noEmit);
            }
        },

        selectInput: function(event) {
            this.sandbox.dom.trigger(event.currentTarget, 'select');
        },


        /**
         * function emits event based on options.name
         * @param postfix
         */
        getEventName: function(postfix) {
            var event = 'husky.search';

            if (!!postfix) {
                postfix = '.' + postfix;
            } else {
                postfix = '';
            }

            if (this.options.instanceName) {
                event += '.' + this.options.instanceName;
            }

            return event + postfix;
        }

    };

});

/*
 * This file is part of the Sulu CMS.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

define('services/husky/logger',[],function() {

    'use strict';

    function Logger() {
    }

    Logger.prototype = window.Husky.logger;

    return new Logger();
});

/*
 * This file is part of the Sulu CMS.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

define('services/husky/expression',['underscore', 'services/husky/logger'], function(_, Logger) {

    'use strict';

    function Expression() {
    }

    /**
     * Evaluates the given display-conditions against the condition-data.
     *
     * @param {[{property, operator, value}]} displayConditions
     * @param {Object} values
     *
     * @returns {boolean}
     */
    Expression.prototype.evaluate = function(displayConditions, values) {
        for (var i = 0, length = displayConditions.length; i < length; i++) {
            var item = _.extend(
                {
                    property: null,
                    operator: null,
                    value: null
                },
                displayConditions[i]
            );

            if (!values.hasOwnProperty(item.property)) {
                Logger.warn('property "' + item.property + '" does not exists in data');

                return false;
            }

            switch (item.operator) {
                case 'eq':
                    if (values[item.property] !== item.value) {
                        return false;
                    }
                    break;
                case 'neq':
                    if (values[item.property] === item.value) {
                        return false;
                    }
                    break;
                default:
                    Logger.error('operator "' + item.operator + '" is not implemented.');

                    return false;
            }
        }

        return true;
    };

    return new Expression();
});

/**
 * @class Tabs
 * @constructor
 *
 * @param {Object} [options] Configuration object
 */
// TODO: complete yuidoc

/*****************************************************************************
 *
 *  Tabs
 *
 *  Options (defaults)
 *      - url: url to fetch data from
 *      - data: if no url is provided
 *      - selected: the item that's selected on initialize
 *      - instanceName - enables custom events (in case of multiple tabs on one page)
 *      - preselect - either true (for url) or position / name  (see preselector for more information)
 *      - preselector:
 *          - url: defines if actions are going to be checked against current URL and preselected (current URL mus be provided by options.fragment) - preselector itself is not going to be taken into account in this case
 *          - position: compares items position against whats defined in options.preselect
 *          - name: compares items name against whats defined in options.preselect
 *      - forceReload - defines if tabs are forcing page to reload
 *      - forceSelect - forces tabs to select first item, if no selected item has been found
 *      - preSelectEvent.enabled - when enabled triggers the item pre select event
 *      - preSelectEvent.triggerSelectItem - when previous options and this options is enabled it triggers the item select event right after the preselect
 *      - fragment - current url to choose preselected item
 *  Provides Events
 *      - husky.tabs.<<instanceName>>.getSelected [callback(item)] - returns item with callback
 *  Triggers Events
 *      - husky.tabs.<<instanceName>>.item.select [item] - triggered when item was clicked
 *      - husky.tabs.<<instanceName>>.initialized [selectedItem]- triggered when tabs have been initialized
 *
 *  Data options
 *      - items
 *          - forceReload: overwrites default-setting for certain item
 * *
 *****************************************************************************/

define('__component__$tabs@husky',['services/husky/expression'], function(Expression) {

    'use strict';

    var defaults = {
            url: null,
            data: [],
            instanceName: '',
            preselect: true,
            preselector: 'url',
            forceReload: false,
            callback: null,
            forceSelect: true,
            values: {},
            preSelectEvent: {
                enabled: false,
                triggerSelectItem: true
            }
        },

        /**
         * enable tabs
         * @event husky.tabs.activate
         */
        ACTIVATE = function() {
            return this.createEventName('activate');
        },

        /**
         * disable tabs
         * @event husky.tabs.deactivate
         */
        DEACTIVATE = function() {
            return this.createEventName('deactivate');
        },

        /**
         * used to show a certain item
         * @event husky.tabs.item.show
         * @param {String} id Id of item to show
         */
        ITEM_SHOW = function() {
            return this.createEventName('item.show');
        },

        /**
         * used to hide a certain item
         * @event husky.tabs.item.hide
         * @param {String} id Id of item to hide
         */
        ITEM_HIDE = function() {
            return this.createEventName('item.hide');
        },

        /**
         * used to select a certain item
         * @event husky.tabs.item.select
         * @param {String} id Id of item to enable
         */
        ITEM_SELECT = function() {
            return this.createEventName('item.select');
        },

        /**
         * used before selecting a certain item
         * @event husky.tabs.item.preselect
         */
        ITEM_PRE_SELECT = function() {
            return this.createEventName('item.preselect');
        },

        /**
         * used to select a certain item manually
         * @event husky.tabs.item.clicked
         * @param {Object} event object
         */
        ITEM_CLICKED = function() {
            return this.createEventName('item.clicked');
        },

        /**
         * used to get selected items
         * @event husky.tabs.item.getSelected
         */
        GET_SELECTED = function() {
            return this.createEventName('getSelected');
        },

        /**
         * used to update the tab-component.
         * @event husky.tabs.initialized
         * @param {Object} values object
         */
        UPDATE = function() {
            return this.createEventName('update');
        },

        /**
         * used to update the tab-component.
         * @event husky.tabs.initialized
         * @param {Number} visibleTabs object
         */
        UPDATED = function() {
            return this.createEventName('updated');
        },

        /**
         * triggered when component was initialized
         * @event husky.tabs.initialized
         * @param {Object} selectedItem
         * @param {Number} visibleTabs
         */
        INITIALIZED = function() {
            return this.createEventName('initialized');
        },

        /**
         * Triggered when a tab is clicked and it is enabled in the options
         * Lets you handle the click event before the tab gets changed
         * Will trigger selectItem when enabled otherwise you have to trigger ITEM_CLICKED to trigger it
         * @param event
         */
        preSelectEvent = function(event) {
            event.preventDefault();
            this.sandbox.emit(ITEM_PRE_SELECT.call(this), event);

            if (!!this.options.preSelectEvent.triggerSelectItem) {
                selectItem.call(this, event);
            }
        },

        selectItem = function(event) {
            event.preventDefault();
            if (this.active === true && this.sandbox.dom.hasClass(event.currentTarget, 'is-selected') !== true) {
                var item = this.items[this.sandbox.dom.data(event.currentTarget, 'id')];

                if (!!item) {
                    this.sandbox.dom.removeClass(this.sandbox.dom.find('.is-selected', this.$el), 'is-selected');
                    this.sandbox.dom.addClass(event.currentTarget, 'is-selected');
                    this.sandbox.dom.addClass(this.$marker, 'animate');
                    setMarker.call(this);
                    // callback
                    if (item.hasOwnProperty('callback') && typeof item.callback === 'function') {
                        item.callback.call(this, item);
                    } else if (!!this.options.callback && typeof this.options.callback === 'function') {
                        this.options.callback.call(this, item);
                    } else {
                        triggerSelectEvent.call(this, item);
                    }
                }
            } else {
                return false;
            }
        },

        setMarker = function() {
            var $selected = this.$find('li.is-selected');
            if (!!$selected.length) {
                this.sandbox.dom.width(this.$marker, this.sandbox.dom.outerWidth($selected));
                this.sandbox.dom.css(this.$marker, {'left': this.sandbox.dom.position($selected).left + 'px'});
            }
        },

        triggerSelectEvent = function(item) {
            item.forceReload = (item.forceReload && typeof item.forceReload !== "undefined") ? item.forceReload : this.options.forceReload;
            this.sandbox.emit(ITEM_SELECT.call(this), item);
        },

        showItem = function(item) {
            this.sandbox.dom.show(this.domItems[item]);
            this.sandbox.dom.removeClass(this.$marker, 'animate');
            setMarker.call(this);
        },

        hideItem = function(item) {
            this.sandbox.dom.hide(this.domItems[item]);
            this.sandbox.dom.removeClass(this.$marker, 'animate');
            setMarker.call(this);
        },

        update = function(values) {
            var visibleTabs = 0;
            this.sandbox.util.foreach(this.data, function(item) {
                var $item = this.$find('li[data-id="' + item.id + '"]');

                if (!!item.displayConditions && !Expression.evaluate(item.displayConditions, values)) {
                    this.sandbox.dom.hide($item);
                } else {
                    this.sandbox.dom.show($item);
                    ++visibleTabs;
                }
            }.bind(this));

            if (visibleTabs >= 2) {
                this.$el.show();
            } else {
                this.$el.hide();
            }

            this.sandbox.emit(UPDATED.call(this), visibleTabs);

            setMarker.call(this);
        },

        bindDOMEvents = function() {
            if (!!this.options.preSelectEvent.enabled) {
                this.sandbox.dom.on(this.$el, 'click', preSelectEvent.bind(this), 'li');
            } else {
                this.sandbox.dom.on(this.$el, 'click', selectItem.bind(this), 'li');
            }
        },

        bindCustomEvents = function() {
            this.sandbox.on(GET_SELECTED.call(this), function(callback) {
                var selection = this.sandbox.dom.find('.is-selected', this.options.el);
                callback.call(this.items[this.sandbox.dom.data(selection, 'id')]);
            }.bind(this));

            this.sandbox.on(ACTIVATE.call(this), this.activate.bind(this));
            this.sandbox.on(DEACTIVATE.call(this), this.deactivate.bind(this));
            this.sandbox.on(ITEM_SHOW.call(this), showItem.bind(this));
            this.sandbox.on(ITEM_HIDE.call(this), hideItem.bind(this));
            this.sandbox.on(ITEM_CLICKED.call(this), selectItem.bind(this));
            this.sandbox.on(UPDATE.call(this), update.bind(this));
        };

    return {

        initialize: function() {
            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);
            this.active = true;
            this.$marker = null;

            this.items = [];
            this.domItems = {};

            bindDOMEvents.call(this);
            bindCustomEvents.call(this);

            // load data and call render
            if (!!this.options.url) {
                this.sandbox.util.load(this.options.url)
                    .then(function(data) {
                        this.render(data, this.options.values);
                    }.bind(this))
                    .fail(function(data) {
                        this.sandbox.logger.log('data could not be loaded:', data);
                    }.bind(this));
            } else if (!!this.options.data) {
                this.render(this.options.data, this.options.values);
            } else {
                this.sandbox.logger.log('no data provided for tabs!');
            }
        },

        createEventName: function(ending) {
            var instanceName = this.options.instanceName ? this.options.instanceName + '.' : '';

            return 'husky.tabs.' + instanceName + ending;
        },

        /**
         * Sets the min-width of an item to its width in selected state
         * (to avoid bumping)
         * @param $item
         */
        setMinWidth: function($item) {
            this.sandbox.dom.addClass($item, 'is-selected');
            this.sandbox.dom.css($item, {
                'min-width': this.sandbox.dom.outerWidth($item) + 'px'
            });
            this.sandbox.dom.removeClass($item, 'is-selected');
        },

        deactivate: function() {
            this.active = false;
            this.sandbox.dom.addClass(this.sandbox.dom.find('.tabs-container', this.$el), 'deactivated');
        },

        activate: function() {
            this.active = true;
            this.sandbox.dom.removeClass(this.sandbox.dom.find('.tabs-container', this.$el), 'deactivated');
        },

        generateIds: function(data) {
            this.sandbox.util.foreach(data, function(item) {
                if (!item.id) {
                    item.id = this.getRandId();
                }
            }.bind(this));
            return data;
        },

        getRandId: function() {
            return Math.floor((Math.random() * 1677721500000000)).toString(16);
        },

        render: function(data, values) {
            this.data = this.generateIds(data);

            var $element = this.sandbox.dom.createElement('<div class="tabs-container"></div>'),
                $list = this.sandbox.dom.createElement('<ul/>'),
                selectedItem = null,
                $item = null;

            this.$marker = this.sandbox.dom.createElement('<div class="marker"></div>');
            this.sandbox.dom.append($element, this.$marker);

            this.sandbox.dom.append(this.$el, $element);
            this.sandbox.dom.append($element, $list);

            this.items = [];
            this.domItems = {};

            var visibleTabs = 0;
            this.sandbox.util.foreach(this.data, function(item, index) {
                this.items[item.id] = item;
                $item = this.sandbox.dom.createElement(
                    '<li data-id="' + item.id + '"><a href="#">' + this.sandbox.translate(item.name) + '</a></li>'
                );
                this.sandbox.dom.append($list, $item);

                if ((!!item.disabled && item.disabled.toString() === 'true')
                    || (!!item.displayConditions && !Expression.evaluate(item.displayConditions, values))
                ) {
                    this.sandbox.dom.hide($item);
                } else {
                    // count only visible items
                    ++visibleTabs;
                }

                // set min-width of element
                this.setMinWidth($item);

                // check if item got selected
                if (!!this.options.preselect) {
                    if ((this.options.preselector === 'url' && !!this.options.fragment && this.options.fragment === item.action) ||
                        (this.options.preselector === 'position' && (index + 1).toString() === this.options.preselect.toString()) ||
                        (this.options.preselector === 'name' && item.name === this.options.preselect)) {
                        this.sandbox.dom.addClass($item, 'is-selected');
                        selectedItem = item;
                    }
                }
                this.domItems[item.id] = $item;
            }.bind(this));

            // force selection of first element
            if (!selectedItem && this.options.forceSelect) {
                selectedItem = this.options.data[0];
                this.sandbox.dom.addClass(this.sandbox.dom.find('li', $list).eq(0), 'is-selected');
            }
            setMarker.call(this);

            // initialization finished
            this.sandbox.emit(INITIALIZED.call(this), selectedItem, visibleTabs);

            if (visibleTabs < 2) {
                this.$el.hide();
            }
        }
    };
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/toolbar
 */

/**
 * @class Toolbar
 * @constructor
 *
 * @param {Object} [options] Configuration object
 * @param {String} [options.url] url to fetch data from
 * @param {String} [options.instanceName] enables custom events (in case of multiple tabs on one page)
 * @param {String} [options.dropdownItemsKey] key with result-array for requested dropdown items
 * @param {Object} [options.searchOptions] options to pass to search component
 * @param {Object} [options.small] if true the toolbar is displayed smaller
 * @param {boolean} [options.hasSearch] if true a search item gets inserted in its own group at the end. A search item can also be added manually through the data
 * @param {String} [options.skin] custom skin-class to add to the component
 * @param {Boolean} [options.showTitleAsTooltip] shows the title of the button only as tooltip
 * @param {Boolean} [options.showTitle] if false doesn't display the title
 * @param {Boolean} [options.responsive] iff true to toolbar gets collapsed/expanded if it overflows it's element
 *
 * @param {Object} [options.groups] array of groups with id and align to specify groups to put items in
 * @param {String|Number} [options.groups.id] the id of the group
 * @param {String} [options.groups.id] the group of button will be aligned "left" or "right"
 *
 * @param {Array} [options.buttons] if no url is provided
 * @param {String} [options.buttons.title]
 * @param {String} [options.buttons.id]
 * @param {Boolean} [options.buttons.disabled] is item disabled or enabled
 * @param {String} [options.buttons.class] highlight/highlight-gray/highlight-white
 * @param {String} [options.buttons.group] id of the a group specified in the options
 * @param {Integer} [options.buttons.position] integer to sort the items - default 9000
 * @param {Function} [options.buttons.callback] callback function
 * @param {Boolean} [options.buttons.hidden] if true button gets hidden form the beginning on
 *
 * @param {Array} [options.buttons.dropdownItems]
 * @param {String} [options.buttons.dropdownItems.title]
 * @param {String} [options.buttons.dropdownItems.icon] false will remove icon
 * @param {Function} [options.buttons.dropdownItems.callback]
 * @param {Boolean} [options.buttons.dropdownItems.divider] if true takes item as divider element
 */
define('__component__$toolbar@husky',[],function() {

    'use strict';

    var defaults = {
            url: null,
            buttons: [],
            instanceName: '',
            dropdownItemsKey: '_embedded',
            searchOptions: null,
            hasSearch: false,
            groups: [
                {
                    id: 1,
                    align: 'left'
                }
            ],
            skin: 'default',
            small: false,
            showTitleAsTooltip: false,
            showTitle: true,
            responsive: false
        },

        itemDefaults = {
            dropdownOptions: {}
        },

        constants = {
            collapsedWidth: 50,
            dropdownToggleWidth: 5, //px
            loaderWhiteColor: 'white',
            loaderDarkColor: '#cccccc',
            markedClass: 'marked'
        },

        selectors = {
            dropdownMenu: '.toolbar-dropdown-menu'
        },

        /** templates container */
        templates = {
            skeleton: [
                '<div class="husky-toolbar">',
                '</div>'
            ].join('')
        },


        /**
         * triggered when toolbar is initialized and ready to use
         *
         * @event husky.toolbar.[INSTANCE_NAME].initialized
         */
        INITIALIZED = function() {
            return createEventName.call(this, 'initialized');
        },


        /**
         * triggered when when a dropdown gets opened
         *
         * @event husky.toolbar.[INSTANCE_NAME].dropdown.opened
         */
        DROPDOWN_OPENED = function() {
            return createEventName.call(this, 'dropdown.opened');
        },

        /**
         * triggered when when a dropdown gets opened
         *
         * @event husky.toolbar.[INSTANCE_NAME].dropdown.closed
         */
        DROPDOWN_CLOSED = function() {
            return createEventName.call(this, 'dropdown.closed');
        },

        /**
         * triggered when the icon or the title of a button gets changed
         *
         * @event husky.toolbar.[INSTANCE_NAME].title.changed
         */
        BUTTON_CHANGED = function() {
            return createEventName.call(this, 'button.changed');
        },

        /**
         * raised when an item with no specified callback is clicked
         *
         * @event husky.toolbar.[INSTANCE_NAME].item.select
         * @param {Object} item Clicked item
         */
        ITEM_SELECT = function() {
            return createEventName.call(this, 'item.select');
        },

        /**
         * enable a button
         *
         * @event husky.toolbar.[INSTANCE_NAME.]item.enable
         * @param {string} id The id of the button which should be enabled
         */
        ITEM_ENABLE = function() {
            return createEventName.call(this, 'item.enable');
        },

        /**
         * disable a button
         *
         * @event husky.toolbar.[INSTANCE_NAME.]item.disable
         * @param {string} id The id of the button which should be disabled
         */
        ITEM_DISABLE = function() {
            return createEventName.call(this, 'item.disable');
        },

        /**
         * event to set button into loading state
         *
         * @event husky.toolbar.[INSTANCE_NAME.]item.loading
         * @param {string} id The id of the button
         */
        ITEM_LOADING = function() {
            return createEventName.call(this, 'item.loading');
        },

        /**
         * event to change a buttons selected dropdown-item
         *
         * @event husky.toolbar.[INSTANCE_NAME.]item.change
         * @param {string} buttonId The id of the button
         * @param {string} itemId The id or the index of the dropdown-item
         * @param {boolean} executeCallback If true callback of dropdown item gets executed
         */
        ITEM_CHANGE = function() {
            return createEventName.call(this, 'item.change');
        },

        /**
         * Event to reset a buttons and it's dropdown-items into original state.
         *
         * @event husky.toolbar.[INSTANCE_NAME.]item.reset
         * @param {string} buttonId The id of the button
         */
        ITEM_RESET = function() {
            return createEventName.call(this, 'item.reset');
        },

        /**
         * event to hide a button
         *
         * @event husky.toolbar.[INSTANCE_NAME.]item.hide
         * @param {string} button The id of the button
         */
        ITEM_HIDE = function() {
            return createEventName.call(this, 'item.hide');
        },

        /**
         * event to show a button
         *
         * @event husky.toolbar.[INSTANCE_NAME.]item.show
         * @param {string} button The id of the button
         */
        ITEM_SHOW = function() {
            return createEventName.call(this, 'item.show');
        },

        /**
         * event to mark a subitem
         *
         * @event husky.toolbar.[INSTANCE_NAME.]item.mark
         * @param {string} button The id of the button
         */
        ITEM_MARK = function() {
            return createEventName.call(this, 'item.mark');
        },

        /**
         * event to unmark a subitem
         *
         * @event husky.toolbar.[INSTANCE_NAME.]item.unmark
         * @param {string} button The id of the button
         */
        ITEM_UNMARK = function() {
            return createEventName.call(this, 'item.unmark');
        },

        /**
         * event to change a buttons default title and default icon
         *
         * @event husky.toolbar.[INSTANCE_NAME.]button.set
         * @param {string} button The id of the button
         * @param {object} object with a icon and title
         */
        BUTTON_SET = function() {
            return createEventName.call(this, 'button.set');
        },

        /**
         * event to set new dropdown-items for a button
         *
         * @event husky.toolbar.[INSTANCE_NAME.]items.set
         * @param {string} button The id of the button
         * @param {array} items The items to set
         * @param {integer} itemId The id or the index of the item to set selected - optional
         */
        ITEMS_SET = function() {
            return createEventName.call(this, 'items.set');
        },

        /**
         * event to collapse the toolbar
         *
         * @event husky.toolbar.[INSTANCE_NAME.].collapse
         *  @param {function} callback to execute after collapsing
         */
        COLLAPSE = function() {
            return createEventName.call(this, 'collapse');
        },

        /**
         * event to expand the toolbar
         *
         * @event husky.toolbar.[INSTANCE_NAME.].expand
         * @param {function} callback to execute after expanding
         */
        EXPAND = function() {
            return createEventName.call(this, 'expand');
        },

        /** events bound to dom */
        bindDOMEvents = function() {
            this.sandbox.dom.on(this.$el, 'click', function(event) {
                this.sandbox.dom.stopPropagation(event);
            }.bind(this), 'li .content');
            this.sandbox.dom.on(this.$el, 'click', toggleItem.bind(this), 'li');
            this.sandbox.dom.on(this.$el, 'click', selectItem.bind(this), 'li');
            if (this.options.responsive === true) {
                $(window).on('resize.husky-toolbar-' + this.options.instanceName, updateOverflow.bind(this));
            }
        },

        /** events bound to sandbox */
        bindCustomEvents = function() {
            this.sandbox.on(ITEM_DISABLE.call(this), function(id, highlight) {
                if (!!this.items[id]) {
                    toggleEnabled.call(this, false, id, highlight);
                }
            }.bind(this));

            this.sandbox.on(ITEM_ENABLE.call(this), function(id, highlight) {
                if (!!this.items[id]) {
                    toggleEnabled.call(this, true, id, highlight);
                }
            }.bind(this));

            this.sandbox.on(ITEM_LOADING.call(this), function(id) {
                if (!!this.items[id]) {
                    itemLoading.call(this, id);
                }
            }.bind(this));

            this.sandbox.on(ITEM_HIDE.call(this), function(id) {
                if (!!this.items[id]) {
                    hideItem.call(this, this.items[id].$el);
                }
            }.bind(this));

            this.sandbox.on(ITEM_SHOW.call(this), function(id) {
                if (!!this.items[id]) {
                    showItem.call(this, this.items[id].$el);
                }
            }.bind(this));

            this.sandbox.on(COLLAPSE.call(this), function(callback) {
                collapseAll.call(this);
                if (typeof callback === 'function') {
                    callback();
                }
            }.bind(this));

            this.sandbox.on(EXPAND.call(this), function(callback) {
                expandAll.call(this);
                if (typeof callback === 'function') {
                    callback();
                }
            }.bind(this));

            this.sandbox.on(ITEM_MARK.call(this), uniqueMarkItem.bind(this));

            this.sandbox.on(ITEM_UNMARK.call(this), unmarkItem.bind(this));

            this.sandbox.on(ITEM_CHANGE.call(this), changeButton.bind(this));

            this.sandbox.on(ITEM_RESET.call(this), resetButton.bind(this));

            this.sandbox.on(BUTTON_SET.call(this), function(button, newData) {
                changeMainListItem.call(this, this.items[button].$el, newData);
            }.bind(this));

            this.sandbox.on(ITEMS_SET.call(this), function(button, items, itemId) {
                if (!!this.items[button]) {
                    if (items.length > 0) {
                        deleteDropdown.call(this, this.items[button]);
                        this.sandbox.dom.removeAttr(this.sandbox.dom.find('.dropdown-toggle', this.items[button].$el), 'style');
                        this.items[button].dropdownItems = items;
                        createDropdownMenu.call(this, this.items[button].$el, this.items[button]);
                        if (!!itemId) {
                            changeButton.call(this, this.items[button].id, itemId).then(function(item) {
                                if (!this.items[button].dropdownOptions
                                    || typeof this.items[button].dropdownOptions.preSelectedCallback !== 'function'
                                ) {
                                    return;
                                }

                                this.items[button].dropdownOptions.preSelectedCallback(item);
                            }.bind(this));
                        }
                    } else {
                        deleteDropdown.call(this, this.items[button]);
                        this.sandbox.dom.hide(this.sandbox.dom.find('.dropdown-toggle', this.items[button].$el));
                    }
                }
            }.bind(this));
        },

        /**
         * Changes a button text and icon.
         *
         * @param {String} button
         * @param {String|Number} itemId
         * @param {Bool} executeCallback
         */
        changeButton = function(buttonId, itemId, executeCallback) {
            // check if button exists
            if (!this.items[buttonId]) {
                return;
            }

            var button = this.items[buttonId],
                deferred = $.Deferred();

            button.initialized.then(function() {
                // update icon
                var index = getItemIndexById.call(this, itemId, button);
                if (index === true) {
                    index = 0;
                }

                changeMainListItem.call(this, button.$el, button.dropdownItems[index]);
                this.sandbox.emit(ITEM_MARK.call(this), button.dropdownItems[index].id);
                if (executeCallback === true || !!button.dropdownItems[index].callback
                    && typeof button.dropdownItems[index].callback === 'function'
                ) {
                    button.dropdownItems[index].callback();
                }

                deferred.resolve(button.dropdownItems[index]);
            }.bind(this));

            return deferred.promise();
        },

        /**
         * Resets button to its original state.
         *
         * @param {String} buttonId
         */
        resetButton = function(buttonId) {
            // check if button exists
            if (!this.items[buttonId]) {
                return;
            }

            this.items[buttonId].initialized.then(function() {
                resetMainListItem.call(this, this.items[buttonId].$el);
            }.bind(this));
        },

        /**
         * Depending on if the toolbar overflows or not collapses or expands the toolbar
         * collapsing - if the toolbar is expanded and overflown
         * expanding - if the toolbar is underflown and collapsed and the expanded version has enough space
         */
        updateOverflow = function() {
            if (this.$el.width() < this.$el[0].scrollWidth) {
                if (!this.collapsed) {
                    this.expandedWidth = this.$find('.husky-toolbar').outerWidth();
                    collapseAll.call(this);
                    updatedOverflowClass.call(this);
                } else {
                    updatedOverflowClass.call(this);
                }
            } else {
                if (this.collapsed && this.$el.width() >= this.expandedWidth) {
                    expandAll.call(this);
                    this.expandedWidth = this.$find('.husky-toolbar').outerWidth();
                    updatedOverflowClass.call(this);
                } else {
                    updatedOverflowClass.call(this);
                }
            }
        },

        /**
         * Sets an overflow-class on the element, depending on whether or ot
         * the toolbar overflows
         */
        updatedOverflowClass = function() {
            if (this.$el.width() < this.$el[0].scrollWidth) {
                this.$el.addClass('overflown');
            } else {
                this.$el.removeClass('overflown');
            }
        },

        /**
         * Makes the toolbar unscrollable and makes the toolbar-overflow's overflow visible
         * so the dropdown can be seen
         */
        lockToolbarScroll = function() {
            var scrollPos = this.$el.scrollLeft();
            this.$el.css({overflow: 'visible'});
            this.$find('.husky-toolbar').css({
                'margin-left': ((-1) * scrollPos) + 'px'
            });
        },

        /**
         * Makes the toolbar-container's overflow hidden and the wrapper itself scrollable
         */
        unlockToolbarScroll = function() {
            this.$el.removeAttr('style');
            this.$find('.husky-toolbar').removeAttr('style');
        },

        /**
         * Sets an item enabled or disabled
         * @param enabled {boolean} If true button gets enabled if false button gets disabled
         * @param id {Number|String} id The id of the button
         * @param highlight {boolean} if true a highlight effect is played
         */
        toggleEnabled = function(enabled, id, highlight) {
            // check if toolbar has an item with specified id
            if (!this.items[id]) {
                return;
            }

            var item = this.items[id],
                $item = this.sandbox.dom.find('[data-id="' + id + '"]', this.$el);

            this.items[id].disabled = !enabled;

            // in case of item has state loading, restore original state
            if (item.loading) {
                item.loading = false;
                this.sandbox.stop(this.sandbox.dom.find('.item-loader', $item));
                this.sandbox.dom.remove(this.sandbox.dom.find('.item-loader', $item));
                this.sandbox.dom.removeClass($item, 'is-loading');
                this.sandbox.dom.width($item, '');
            }

            if (highlight !== false) {
                // add color fading effect
                this.sandbox.dom.addClass($item, 'highlight-animation');

                // remove class after effect has finished
                this.sandbox.dom.on($item, 'animationend webkitAnimationEnd oanimationend MSAnimationEnd', function(ev) {
                    this.sandbox.dom.removeClass(ev.currentTarget, 'highlight-animation');
                }.bind(this));
            }

            if (!!enabled) {
                this.sandbox.dom.removeClass($item, 'disabled');
            } else {
                this.sandbox.dom.addClass($item, 'disabled');
            }

            if (this.options.responsive === true) {
                updateOverflow.call(this);
            }
        },

        /**
         * Hides a button
         * @param $button
         */
        hideItem = function($button) {
            var $list = $button.parent('.toolbar-dropdown-menu');
            this.sandbox.dom.addClass($button, 'hidden');

            if ($list.length > 0) {
                updateDropdownArrow.call(this, $list);
            }
        },

        /**
         * Shows a button
         * @param $button
         */
        showItem = function($button) {
            var $list = $button.parent('.toolbar-dropdown-menu');
            this.sandbox.dom.removeClass($button, 'hidden');

            if ($list.length > 0) {
                updateDropdownArrow.call(this, $list);
            }
        },

        /**
         * Sets a button into loading state
         * @param id {Number|String} id The id of the button
         */
        itemLoading = function(id) {
            var item = this.items[id],
                $item = this.sandbox.dom.find('[data-id="' + id + '"]', this.$el),
                $loader,
                color = constants.loaderWhiteColor;

            if (item.loading) {
                return;
            }

            item.loading = true;
            // fix width
            this.sandbox.dom.width($item, this.sandbox.dom.width($item));
            this.sandbox.dom.addClass($item, 'is-loading');

            $loader = this.sandbox.dom.createElement('<span class="item-loader"></span>');
            this.sandbox.dom.append($item, $loader);

            if (this.sandbox.dom.hasClass($item, 'highlight-white')) {
                color = constants.loaderDarkColor;
            }

            this.sandbox.start([{
                name: 'loader@husky',
                options: {
                    el: $loader,
                    size: this.options.small ? '14px' : '20px',
                    color: color
                }
            }]);

            if (this.options.responsive === true) {
                updateOverflow.call(this);
            }
        },


        /**
         * gets called when toggle item is clicked
         * opens dropdown submenu
         * @param event
         */
        toggleItem = function(event) {

            event.preventDefault();
            event.stopPropagation();

            var $list = this.sandbox.dom.$(event.currentTarget),
                id = this.sandbox.dom.data($list, 'id'),
                item = this.items[id],
                visible;
            if (!!this.sandbox.dom.find('.dropdown-toggle', $list).length) {
                // abort if disabled or dropdown-arrow wasn't clicked and but the onlyOnClickOnArrow option was true
                if (!item
                    || item.disabled
                    || (item.dropdownOptions.onlyOnClickOnArrow === true && !this.sandbox.dom.hasClass(event.target, 'dropdown-toggle'))
                    || !countVisibleItemsInDropdown($list.children('.toolbar-dropdown-menu'))
                ) {
                    return false;
                }

                if (this.sandbox.dom.hasClass($list, 'is-expanded')) {
                    visible = true;
                }
                hideDropdowns.call(this);

                if (!visible) {
                    // TODO: check if dropdown overlaps screen: set ul to .right-aligned

                    // on every click remove sub-menu
                    this.sandbox.dom.one('body', 'click', hideDropdowns.bind(this));

                    if (this.options.responsive === true) {
                        lockToolbarScroll.call(this);
                    }

                    // IMPORTANT (FIX FOR IE11/EDGE rendering Bug): The following
                    // two lines must remain at end of the method. More specifically
                    // they must come after "lockToolbarScroll"
                    this.sandbox.dom.addClass($list, 'is-expanded');
                    this.sandbox.dom.show(this.sandbox.dom.find(selectors.dropdownMenu, $list));

                    this.sandbox.emit(DROPDOWN_OPENED.call(this));
                }
            }
        },

        /**
         * Determines the index of a dropdown-item by its id
         * @param id {integer} id of the dropdown-item
         * @param button {object} parent object of the item
         * @return {integer} the index of the item
         */
        getItemIndexById = function(id, button) {
            for (var i = -1, length = button.dropdownItems.length; ++i < length;) {
                if (button.dropdownItems[i].id === id) {
                    return i;
                }
            }
            return id;
        },

        /**
         * hides dropdowns of this instance
         */
        hideDropdowns = function() {
            this.sandbox.dom.removeClass(this.sandbox.dom.find('.is-expanded', this.$el), 'is-expanded');
            this.sandbox.dom.hide(this.$find(selectors.dropdownMenu));
            if (this.options.responsive === true) {
                unlockToolbarScroll.call(this);
            }
            this.sandbox.emit(DROPDOWN_CLOSED.call(this));
        },

        /**
         * when an element gets selected
         * gets item from items array and delegates to triggerSelectEvent function
         * @param event
         */
        selectItem = function(event) {
            this.sandbox.dom.stopPropagation(event);
            this.sandbox.dom.preventDefault(event);

            var item = this.items[this.sandbox.dom.data(event.currentTarget, 'id')],
                $parent = (!!this.items[item.parentId]) ? this.items[item.parentId].$el : null;

            // stop if loading or the dropdown gets opened
            if (item.loading || (
                    !!item.dropdownItems
                    && item.dropdownOptions.onlyOnClickOnArrow !== true
                    && !!countVisibleItemsInDropdown($(event.currentTarget).children('.toolbar-dropdown-menu'))
                ) || this.sandbox.dom.hasClass(event.target, 'dropdown-toggle')
            ) {
                return false;
            }
            hideDropdowns.call(this);
            if (!!item.parentId && !!this.items[item.parentId].dropdownOptions &&
                this.items[item.parentId].dropdownOptions.markSelected === true) {
                uniqueMarkItem.call(this, item.id);
            }
            if (!item.disabled) {
                triggerSelectEvent.call(this, item, $parent);
            } else {
                return false;
            }
        },

        /**
         * either calls items callback (if set) or triggers select event
         * @param item
         * @param $parent
         */
        triggerSelectEvent = function(item, $parent) {
            var parentItem,
                original = item._original || item,
                $content = this.sandbox.dom.find('.content', this.items[item.id].$el);

            // forward click to content
            if (!!$content.length) {
                this.sandbox.dom.click(this.sandbox.dom.children($content));
            }

            // check if has parent and type of parent
            if (item.parentId) {
                parentItem = this.items[item.parentId];
                if (!!parentItem.dropdownOptions.changeButton) {
                    changeMainListItem.call(this, $parent, item);
                }

                //check if dropdownOptions is set and pass clicked item to the callback
                if (!!parentItem.dropdownOptions) {
                    if (typeof parentItem.dropdownOptions.callback === 'function') {
                        parentItem.dropdownOptions.callback(original);
                    }
                }
            }

            // if callback is set call it, else trigger event
            if (item.callback) {
                item.callback();
            } else {
                this.sandbox.emit(ITEM_SELECT.call(this), item);
            }
        },

        /**
         * Resets the list items icon and title. Tries to set to default, otherwise to null.
         *
         * @param listElement
         */
        resetMainListItem = function(listElement) {
            var listItems = this.sandbox.dom.find('span', listElement),
                itemId = this.sandbox.dom.data(listElement).id,
                item = this.items[itemId];

            // reset icon
            this.sandbox.dom.removeClass(listItems.eq(0), '');
            item.icon = item.defaultIcon;
            if (!!item.defaultIcon) {
                this.sandbox.dom.addClass(listItems.eq(0), createIconSupportClass.call(this, item));
            }

            // reset title
            item.title = this.sandbox.translate(item.defaultTitle);
            this.sandbox.dom.html(listItems.eq(1), item.title);
            this.items[itemId].title = item.title;

            // remove marked class from dropdown-item
            if (!!item.dropdownItems) {
                this.sandbox.dom.removeClass(selectors.dropdownMenu + ' li', 'marked');
            }

            if (this.options.responsive === true) {
                updateOverflow.call(this);
            }

            this.sandbox.emit(BUTTON_CHANGED.call(this));
        },

        /**
         * Changes the list items icon and title.
         *
         * @param listElement
         * @param item
         */
        changeMainListItem = function(listElement, item) {
            var listItems = this.sandbox.dom.find('span', listElement),
                itemId = this.sandbox.dom.data(listElement).id;
            if (!!item.icon) {
                this.sandbox.dom.removeClass(listItems.eq(0), '');
                this.sandbox.dom.addClass(listItems.eq(0), createIconSupportClass.call(this, item));
                this.items[itemId].icon = item.icon;
            }
            if (!!item.title) {
                item.title = this.sandbox.translate(item.title);
                this.sandbox.dom.html(listItems.eq(1), item.title);
                this.items[itemId].icon = item.title;
            }
            if (this.options.responsive === true) {
                updateOverflow.call(this);
            }
            this.sandbox.emit(BUTTON_CHANGED.call(this));
        },

        /**
         * creates the class string of an icon
         * @param item
         * @returns {string}
         */
        createIconSupportClass = function(item) {
            var classArray,
                classString = '',
                icon = createIconClass.call(this, item);

            // create icon class
            if (item.icon) {
                classArray = [];
                classArray.push(icon);
                classArray.push('icon');

                classString = classArray.join(' ');
            }

            return classString;
        },

        /**
         * returns valid class for item and state
         * @param item
         * @param enabled
         */
        createIconClass = function(item) {
            return 'fa-' + item.icon;
        },

        /**
         * created dropdown menu
         * @param listItem
         * @param parent
         */
        createDropdownMenu = function(listItem, parent) {
            var $list = this.sandbox.dom.createElement('<ul class="toolbar-dropdown-menu" />'),
                $item;

            if (!!parent.dropdownOptions && !!parent.dropdownOptions.maxHeight) {
                $list.css('maxHeight', parent.dropdownOptions.maxHeight);
            }

            this.sandbox.dom.append(listItem, $list);
            this.sandbox.util.foreach(parent.dropdownItems, function(dropdownItem) {
                dropdownItem.title = this.sandbox.translate(dropdownItem.title);
                if (dropdownItem.divider) {
                    // prevent divider when not enough items
                    if (this.items[parent.id].dropdownItems.length <= 2) {
                        return
                    }
                    this.sandbox.dom.append($list, '<li class="divider"></li>');
                    return;
                }

                dropdownItem.parentId = parent.id;
                // check id for uniqueness
                checkItemId.call(this, dropdownItem);
                $item = this.sandbox.dom.createElement(
                    '<li data-id="' + dropdownItem.id + '"><a href="#">' + dropdownItem.title + '</a></li>'
                );
                dropdownItem.$el = $item;
                this.items[dropdownItem.id] = dropdownItem;

                if (dropdownItem.disabled === true) {
                    this.sandbox.dom.addClass($item, 'disabled');
                }
                if (dropdownItem.marked === true) {
                    uniqueMarkItem.call(this, dropdownItem.id);
                }
                if (!!dropdownItem.styleClass) {
                    $item.addClass(dropdownItem.styleClass);
                }
                this.sandbox.dom.append($list, $item);
            }.bind(this));

            updateDropdownArrow.call(this, $list);
        },

        countVisibleItemsInDropdown = function($list) {
            return $list.children('li:not(.hidden)').length;
        },

        updateDropdownArrow = function($list) {
            var $dropdownToggle = $list.siblings('.dropdown-toggle');
            if (!!countVisibleItemsInDropdown($list)) {
                $dropdownToggle.css('display', '');
            } else {
                $dropdownToggle.css('display', 'none');
            }
        },

        /**
         * Marks an item with a css class und unmarks all other items with the same parent
         * @param itemId {Number|String} the id of the item
         */
        uniqueMarkItem = function(itemId) {
            if (!!this.items[itemId] && !!this.items[itemId].parentId && !this.items[itemId].disabled) {
                // unmark all items with the same parent
                this.sandbox.util.each(this.items, function(id, item) {
                    if (item.parentId === this.items[itemId].parentId) {
                        this.sandbox.dom.removeClass(item.$el, constants.markedClass);
                    }
                }.bind(this));
                // mark passed element
                this.sandbox.dom.addClass(this.items[itemId].$el, constants.markedClass);
            }
        },

        /**
         * Unmark an item by removing the marked class from the item
         * @param itemId {Number|String} the id of the item
         */
        unmarkItem = function(itemId) {
            if (!!this.items[itemId] && !!this.items[itemId].parentId) {
                this.sandbox.dom.removeClass(this.items[itemId].$el, constants.markedClass);
            }
        },

        /**
         * Handles requested items
         * @param requestedItems
         * @param buttonId
         */
        handleRequestedItems = function(requestedItems, buttonId) {
            var id, title, icon, callback, divider, i, length;
            this.items[buttonId].dropdownItems = [];

            //for loop sets the the items[button].dropdownItems - array together
            for (i = -1, length = requestedItems.length; ++i < length;) {

                if (!!this.items[buttonId].dropdownOptions.idAttribute) {
                    id = requestedItems[i][this.items[buttonId].dropdownOptions.idAttribute];
                } else if (!!requestedItems[i].id) {
                    id = requestedItems[i].id;
                }

                if (!!this.items[buttonId].dropdownOptions.titleAttribute) {
                    title = requestedItems[i][this.items[buttonId].dropdownOptions.titleAttribute];
                } else if (!!requestedItems[i].title) {
                    title = requestedItems[i].title;
                }
                if (!!this.items[buttonId].dropdownOptions.languageNamespace) {
                    title += this.items[buttonId].dropdownOptions.languageNamespace + title;
                }
                title = this.sandbox.translate(title);

                if (!!requestedItems[i].icon) {
                    icon = requestedItems[i].icon;
                }

                if (!!requestedItems[i].callback) {
                    callback = requestedItems[i].callback;
                }

                if (!!requestedItems[i].divider) {
                    divider = requestedItems[i].divider;
                } else {
                    divider = false;
                }

                this.items[buttonId].dropdownItems[i] = {
                    id: id,
                    title: title,
                    icon: icon,
                    callback: callback,
                    divider: divider,
                    _original: requestedItems[i]
                };
            }

            //now generate the dropdown
            this.sandbox.emit(
                ITEMS_SET.call(this),
                buttonId,
                this.items[buttonId].dropdownItems,
                this.items[buttonId].dropdownOptions.preSelected
            );
        },

        /**
         * Collapses all buttons
         */
        collapseAll = function() {
            for (var key in this.items) {
                if (this.items.hasOwnProperty(key)) {
                    toggleCollapseButton.call(this, this.items[key], true);
                }
            }
            this.collapsed = true;
        },

        /**
         * Expands all buttons
         */
        expandAll = function() {
            for (var key in this.items) {
                if (this.items.hasOwnProperty(key)) {
                    toggleCollapseButton.call(this, this.items[key], false);
                }
            }
            this.collapsed = false;
        },


        /**
         * Expands or collapses a given button
         * @param button {Object}
         * @param collapse {Boolean} if true button gets collapsed
         */
        toggleCollapseButton = function(button, collapse) {
            if (!button.parentId && !!button.icon) {
                // show title
                if (collapse === true) {
                    this.sandbox.dom.hide(this.sandbox.dom.find('.title', button.$el));
                } else {
                    this.sandbox.dom.removeAttr(this.sandbox.dom.find('.title', button.$el), 'style');
                }
            }
        },

        /**
         * Sorts all items with their position-property
         * @param {array} data The list of items to sort
         * @return {array} returns the sorted array
         */
        sortData = function(data) {
            for (var i = -1, length = data.length; ++i < length;) {
                if (typeof data[i].position !== 'number') {
                    data[i].position = 9000;
                }
            }
            data = data.sort(function(a, b) {
                return a.position - b.position;
            });

            return data;
        },

        /**
         * function checks if id is set and unique among all items
         * otherwise a new id is generated for the element
         * @param item
         */
        checkItemId = function(item) {
            // if item has no id, generate random id
            if (!item.id || !!this.items[item.id]) {
                do {
                    item.id = this.sandbox.util.uniqueId();
                } while (!!this.items[item.id]);
            }
            // set enabled defaults
            if (!item.disabled) {
                item.disabled = false;
            }
        },

        /**
         * Removes the dropdown of a button
         * @param button
         */
        deleteDropdown = function(button) {
            if (!!button.dropdownItems) {
                // remove the related stuff
                this.sandbox.dom.remove(this.sandbox.dom.find(selectors.dropdownMenu, button.$el));
                this.sandbox.dom.hide(this.sandbox.dom.find('.dropdown-toggle', button.$el));

                // delete JS related stuff
                for (var i = -1, length = button.dropdownItems.length; ++i < length;) {
                    delete this.items[button.dropdownItems[i].id];
                }
                button.dropdownItems = [];
            }
        },

        /**
         * Inserts a search into an item
         * @param $item
         */
        insertSearch = function($item) {
            var searchOptions;

            this.sandbox.dom.removeClass($item);
            this.sandbox.dom.addClass($item, 'toolbar-search');
            this.sandbox.dom.append($item, '<div id="' + this.options.instanceName + '-toolbar-search"/>');

            searchOptions = {
                el: '#' + this.options.instanceName + '-toolbar-search',
                instanceName: this.options.instanceName,
                appearance: 'white small',
                slide: true
            };
            searchOptions = this.sandbox.util.extend(true, {}, searchOptions, this.options.searchOptions);
            // start search component
            this.sandbox.start([
                {
                    name: 'search@husky',
                    options: searchOptions
                }
            ]);
        },

        /**
         * Add the skin-classes to the component-element
         * @param $element {Object}
         */
        addSkinClass = function($element) {
            if (this.options.skin !== 'default') {
                this.sandbox.dom.addClass($element, this.options.skin);
            }
        },

        /**
         * Pushes a search item to a passed data array and creates a group where the new search item gets inserted to
         * @param data {Object} data Array of data to push the item to
         * @returns {Object} the data array
         */
        pushSearch = function(data) {
            // add own group for search item
            var searchGroup = {
                id: 'search',
                align: 'right'
            };

            this.options.groups.unshift(searchGroup);

            // push search item
            data.push({
                id: 'search',
                hasSearch: true,
                group: 'search'
            });

            return data;
        },

        /** returns normalized event names */
        createEventName = function(postFix) {
            return 'husky.toolbar.' + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
        };

    return {

        /**
         * initialize component
         */
        initialize: function() {

            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

            this.collapsed = false;
            this.itemGroups = {};
            this.items = {};
            this.expandedWidth = 0;

            // load data and call render
            if (!!this.options.url) {
                this.sandbox.util.load(this.options.url)
                    .then(this.render.bind(this))
                    .fail(function(data) {
                        this.sandbox.logger.log('data could not be loaded:', data);
                    }.bind(this));
            } else if (!!this.options.buttons) {
                this.render((this.options.buttons));
            } else {
                this.sandbox.logger.log('no data provided for tabs!');
            }

            bindDOMEvents.call(this);
            bindCustomEvents.call(this);

            if (this.options.responsive === true) {
                updateOverflow.call(this);
            }

            // initialization finished
            this.sandbox.emit(INITIALIZED.call(this));
        },

        /**
         * Destroys the component
         */
        destroy: function() {
            $(window).off('resize.husky-toolbar-' + this.options.instanceName);
        },

        /**
         * renders the toolbar
         * @param {Object} data
         */
        render: function(data) {

            var classArray, addTo,
                $listItem,
                title;

            // if has search is set in options push a search item and it's own group
            if (this.options.hasSearch === true) {
                data = pushSearch.call(this, data);
            }

            //render container and groups
            this.renderSkeleton();

            data = sortData.call(this, data);

            // create all elements
            this.sandbox.util.foreach(data, function(item) {
                item = this.sandbox.util.extend(true, {}, item, itemDefaults);
                item.title = this.sandbox.translate(item.title);

                // check id for uniqueness
                checkItemId.call(this, item);

                var dfd = this.sandbox.data.deferred();

                // set default title and icon
                item.defaultTitle = item.title;
                item.defaultIcon = item.icon;

                // save to items array
                this.items[item.id] = item;
                this.items[item.id].initialized = dfd.promise();

                // create class array
                classArray = ['toolbar-item'];
                if (!!item['class']) {
                    classArray.push(item['class']);
                }

                if (item.hidden) {
                    classArray.push('hidden');
                }

                if (item.disabled) {
                    classArray.push('disabled');
                }

                // if group is set to and exists add it to that group else take the first group
                if (!!item.group && !!this.itemGroups[item.group]) {
                    addTo = this.itemGroups[item.group];
                } else {
                    addTo = this.itemGroups[Object.keys(this.itemGroups)[0]];
                }

                $listItem = this.sandbox.dom.createElement('<li class="' + classArray.join(' ') + '" data-id="' + item.id + '"/>');

                // if has-search is true render a search bar, else render the item normally
                if (item.hasSearch === true) {
                    insertSearch.call(this, $listItem);
                } else {
                    if (!!item.icon) {
                        // create icon span
                        this.sandbox.dom.append($listItem, '<span class="' + createIconSupportClass.call(this, item) + '" />');
                    }

                    // create content span
                    if (!!item.content) {
                        var $contentSpan = this.sandbox.dom.createElement('<span class="content"/>');
                        this.sandbox.dom.append($contentSpan, item.content);
                        this.sandbox.dom.append($listItem, $contentSpan);
                        this.sandbox.start($contentSpan);
                    }

                    // create title span
                    title = '';
                    if (!!item.title && (this.options.showTitle === true || item.showTitle === true)) {
                        title = item.title;
                    }
                    this.sandbox.dom.append($listItem, '<span class="title">' + title + '</span>');

                    // add dropdown-toggle element (hidden at default)
                    this.sandbox.dom.append($listItem, this.sandbox.dom.createElement('<span class="dropdown-toggle" />'));
                    this.sandbox.dom.hide(this.sandbox.dom.find('.dropdown-toggle', $listItem));

                    //add tooltip to item
                    if (this.options.showTitleAsTooltip === true) {
                        this.sandbox.dom.attr($listItem, {'title': title});
                    }

                    //hide the item if hidden true
                    if (item.hidden === true) {
                        hideItem.call(this, $listItem);
                    }

                    if (!!item.dropdownOptions && !!item.dropdownOptions.url) {
                        this.sandbox.util.load(item.dropdownOptions.url)
                            .then(function(result) {
                                var data = result[this.options.dropdownItemsKey];
                                if (!!item.dropdownOptions.resultKey) {
                                    data = data[item.dropdownOptions.resultKey];
                                }

                                // add items if present
                                if (!!item.dropdownItems) {
                                    data = data.concat(item.dropdownItems);
                                }

                                handleRequestedItems.call(this, data, item.id);
                                dfd.resolve();
                            }.bind(this))
                            .fail(function(result) {
                                this.sandbox.logger.log('dorpdown-items could not be loaded', result);
                            }.bind(this));
                    } else {
                        // now create subitems
                        if (!!item.dropdownItems) {
                            this.sandbox.dom.removeAttr(this.sandbox.dom.find('.dropdown-toggle', $listItem), 'style');
                            createDropdownMenu.call(this, $listItem, item);
                        }
                        dfd.resolve();
                    }
                }

                // create button
                this.sandbox.dom.append(addTo, $listItem);

                //set width for buttons with dropdowns
                this.items[item.id].$el = $listItem;

            }.bind(this));
        },

        /**
         * Renders the main container and the groups
         */
        renderSkeleton: function() {
            var i, length, $group, $skeleton;

            $skeleton = this.sandbox.dom.createElement(templates.skeleton);

            // add small class to skeleton
            if (this.options.small === true) {
                this.sandbox.dom.addClass($skeleton, 'small');
            }

            // add skin class
            addSkinClass.call(this, $skeleton);

            // render groups
            for (i = -1, length = this.options.groups.length; ++i < length;) {
                $group = this.sandbox.dom.createElement('<ul class="group"/>');

                if (!!this.options.groups[i].align) {
                    this.sandbox.dom.addClass($group, this.options.groups[i].align);
                } else {
                    this.sandbox.dom.addClass($group, 'left');
                }

                this.sandbox.dom.append($skeleton, $group);
                this.itemGroups[this.options.groups[i].id] = $group;
            }

            // append skeleton to component element
            this.sandbox.dom.append(this.$el, $skeleton);

            //add last class
            this.sandbox.dom.addClass(
                this.sandbox.dom.last(this.sandbox.dom.find('.group.left', this.$el)),
                'last'
            );

            //add responsive class
            if (this.options.responsive === true) {
                this.$el.addClass('husky-toolbar-responsive');
            }
        }
    };

});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/auto-complete
 */

/**
 * @class AutoComplete
 * @constructor
 *
 * @param {Object} [options] Configuration object
 * @param {String} [options.prefetchUrl] url to prefetch data
 * @param {Array} [options.localData] array of local data
 * @param {String} [options.remoteUrl] url to fetch data on input
 * @param {String} [options.getParameter] name for GET-parameter in remote query
 * @param {String} [options.valueKey] Name of value-property in suggestion
 * @param {String} [options.resultKey] Key for suggestions-array in JSON result
 * @param {object} [options.value] with name (value of the input box), id (data-id of the input box)
 * @param {String} [options.instanceName] name of the component instance
 * @param {Boolean} [options.noNewValues] if true input value must have been suggested by auto-complete
 * @param {String} [options.suggestionClass] CSS-class for auto-complete suggestions
 * @param {String} [options.suggestionIcon] Icon Class - Image gets rendered before every suggestion
 * @param {String} [options.autoCompleteIcon] Icon Class - Icon in auto complete input
 * @param {Boolean} [options.stickToInput] If true suggestions are always under the input field
 * @param {Boolean} [options.hint] if false typeahead hint-field will be removed
 * @param {Boolean} [options.emptyOnBlur] If true input field value gets deleted on blur
 * @param {Array} [options.excludes] Array of suggestions to exclude from the suggestion dropdown
 * @param {Function} [options.selectCallback] function which will be called when element is selected
 * @param {Array} [options.fields] A list of the fields to show inside the dropdown
 * @param {String} [options.dropdownSizeClass] The styling class for the dropdown. Defined inside the autocomplete stylesheet
 * @param {String} [options.footerContent] Could be a template or just text
 * @param {Integer} [options.limit] Max number of items displayed in the list. Defaults to 5.
 */

define('__component__$auto-complete@husky',[], function() {

    'use strict';

    /**
     * Default values for options
     */
    var defaults = {
        prefetchUrl: '',
        localData: [],
        remoteUrl: '',
        getParameter: 'query',
        valueKey: 'name',
        resultKey: 'countries',
        value: null,
        instanceName: 'undefined',
        noNewValues: false,
        suggestionClass: 'suggestion',
        suggestionIcon: '',
        autoCompleteIcon: 'search',
        stickToInput: false,
        hint: false,
        emptyOnBlur: false,
        excludes: [],
        selectCallback: null,
        fields: [],
        dropdownSizeClass: '',
        footerContent: '',
        limit: 5
    },

    templates = {
        main: [
            '<div class="husky-auto-complete <%= dropdownSizeClass %>">',
            '    <div class="front">',
            '       <a class="fa-<%= autoCompleteIcon %>"></a>',
            '    </div>',
            '    <div class="input"></div>',
            '    <div class="back">',
            '       <div class="loader" style="display: none;"></div>',
            '    </div>',
            '</div>'
        ].join('')
    },

    eventNamespace = 'husky.auto-complete.',

    /**
     * raised after initialization
     * @event husky.auto-complete.initialized
     */
    INITIALIZED = function() {
        return createEventName.call(this, 'initialized');
    },

    /**
     * raised after prefetched data is retrieved
     * @event husky.auto-complete.prefetch-data
     */
    PREFETCH_LOAD = function() {
        return createEventName.call(this, 'prefetch-data');
    },

    /**
     * raised before remoted data is loaded
     * @event husky.auto-complete.remote-data-load
     */
    REMOTE_LOAD = function() {
        return createEventName.call(this, 'remote-data-load');
    },

    /**
     * raised after remoted data is retrieved
     * @event husky.auto-complete.remote-data
     */
    REMOTE_RETRIEVE = function() {
        return createEventName.call(this, 'remote-data');
    },

    /**
     * raised before the component tries to request a match after blur
     * @event husky.auto-complete.request-match
     */
    REQUEST_MATCH = function() {
        return createEventName.call(this, 'request-match');
    },

    /**
     * raised after autocomplete suggestion is selected
     * @event husky.auto-complete.select
     * @param {object} selected datum with id and name
     */
    SELECT = function() {
        return createEventName.call(this, 'select');
    },

    /**
     * raised after selection has been removed
     * @event husky.auto-complete.selection-removed
     */
    SELECTION_REMOVED = function() {
        return createEventName.call(this, 'selection-removed');
    },

    /**
     * raised after autocomplete suggestion is selected
     * @event husky.auto-complete.set-excludes
     * @param {array} array of objects to exclude from suggestions
     */
    SET_EXCLUDES = function() {
        return createEventName.call(this, 'set-excludes');
    },

    /**
     * listens on and passes boolean to callback if input is matched exactly
     * @event husky.auto-complete.is-matched
     * @param {Function} Callback which gets the booloan passed
     */
    IS_MATCHED = function() {
        return createEventName.call(this, 'is-matched');
    },

    /**
     * raised after autocomplete footer was clicked
     * @event husky.auto-complete.footer.clicked
     */
    FOOTER_CLICKED = function() {
        return createEventName.call(this, 'footer.clicked');
    },

    /** returns normalized event names */
    createEventName = function(postFix) {
        return eventNamespace + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
    };

    return {

        /**
         * Returns the id of the options.value object
         * @returns {Integer|null}
         */
        getValueID: function() {
            if (!!this.options.value) {
                return this.options.value.id;
            } else {
                return null;
            }
        },

        /**
         * Returns the value of the options.value object
         * @returns {String}
         */
        getValueName: function() {
            if (!!this.options.value) {
                return this.sandbox.util.escapeHtml(this.options.value[this.options.valueKey]);
            } else {
                return '';
            }
        },

        initialize: function() {
            this.sandbox.logger.log('initialize', this);
            this.sandbox.logger.log(arguments);

            // extend default options
            this.options = this.sandbox.util.extend({}, defaults, this.options);

            this.suggestionTpl = null;
            this.footerTpl = null;
            this.data = null;
            this.matched = true;
            this.matches = [];
            this.excludes = this.parseExcludes(this.options.excludes);
            this.localData = {};
            this.localData._embedded = {};
            this.localData._embedded[this.options.resultKey] = this.options.localData;

            this.setTemplates();

            this.render();
            this.bindDomEvents();
            this.setCustomEvents();

            this.sandbox.emit(INITIALIZED.call(this), this.$valueField);
        },

        /**
         * Initializes the templates
         */
        setTemplates: function() {
            var iconHTML = '';
            if (this.options.suggestionIcon !== '') {
                iconHTML = '<span class="fa-' + this.options.suggestionIcon + ' icon"></span>';
            }

            // suggestions
            if (this.options.fields.length) {
                this.suggestionTpl = this.sandbox.util.template('' +
                    '<div class="' + this.options.suggestionClass + '" data-id="<%= context[\'id \']%>">' +
                    '   <div class="border">' +
                    '       <div class="text">' +
                    '           <% _.each(fields, function(field, idx) { %>' +
                    '           <div class="suggestion-column" style="width: <%= field.width %>;"><%= context[field.id] %></div>' +
                    '           <% }) %>' +
                    '       </div>' +
                    '   </div>' +
                    '</div>');
            } else {
                this.suggestionTpl = this.sandbox.util.template('' +
                    '<div class="' + this.options.suggestionClass + '" data-id="<%= context[\'id \']%>">' +
                    '   <div class="border">' +
                            iconHTML +
                    '       <div class="text"><%= context[this.options.valueKey] %></div>' +
                    '   </div>' +
                    '</div>');
            }

            if (!!this.options.footerContent) {
                this.footerTpl = this.sandbox.util.template(
                    '<div class="auto-complete-footer">' +
                        this.options.footerContent +
                    '</div>'
                );
            }
        },

        /**
         * @param context {object} context for template - id, name
         * @returns {String} html of suggestion element
         */
        buildSuggestionTemplate: function(context) {
            var domObj;
            if (this.suggestionTpl !== null) {
                domObj = this.sandbox.dom.createElement(this.suggestionTpl({ context: context, fields: this.options.fields }));
                if (this.isExcluded(context)) {
                    this.sandbox.dom.addClass(domObj, 'disabled');
                }
                return this.sandbox.dom.html(this.sandbox.dom.append(this.sandbox.dom.$('<div/>'), domObj));
            }
        },

        /**
         * the main-template gets rendered and displayed
         */
        renderMain: function() {
            this.sandbox.dom.html(this.$el, this.sandbox.template.parse(templates.main, {
                autoCompleteIcon: this.options.autoCompleteIcon,
                dropdownSizeClass: this.options.dropdownSizeClass
            }));
        },

        /**
         * Initializes and appends the input, starts the typeahead-auto-complete plugin
         */
        render: function() {
            this.renderMain();
            this.initValueField();
            this.appendValueField();

            this.sandbox.start([
                {
                    name: 'loader@husky',
                    options: {
                        el: this.$el.find('.loader'),
                        color: '#ccc',
                        size: '20px'
                    }
                }
            ]);

            this.bindTypeahead();
        },

        /**
         * Assigns an input box to an object property
         */
        initValueField: function() {
            this.$valueField = this.sandbox.dom.createElement('<input id="' + this.options.instanceName + '" ' +
                'class="husky-validate" ' +
                'type="text" ' +
                'autofill="false" ' +
                'autocomplete="off"' +
                'data-id="' + this.getValueID() + '" ' +
                'value="' + this.getValueName() + '"/>');
        },

        /**
         * Appends the input box to the component container
         */
        appendValueField: function() {
            if (!!this.$valueField.length) {
                this.sandbox.dom.append(this.sandbox.dom.find('.input', this.$el), this.$valueField);
            }
        },

        /**
         * Starts the typeahead auto-complete plugin
         */
        bindTypeahead: function() {
            var delimiter = (this.options.remoteUrl.indexOf('?') === -1) ? '?' : '&',
                configs = {
                    name: this.options.instanceName,
                    local: this.handleData(this.localData),
                    limit: this.options.limit,
                    displayKey: this.options.valueKey,
                    templates: {
                        suggestion: function(context) {
                            //saves the fact that the current input has matches
                            this.matches.push(context);
                            this.matched = true;
                            return this.buildSuggestionTemplate(context);
                        }.bind(this),

                        footer: function() {
                            if (!!this.footerTpl) {
                                return this.footerTpl();
                            }
                            return null;
                        }.bind(this)
                    }
                };

            if (!!this.options.prefetchUrl) {
                configs.prefetch = {
                    url: this.options.prefetchUrl,
                    ttl: 1,
                    filter: function(data) {
                        this.sandbox.emit(PREFETCH_LOAD.call(this), this.handleData(data));

                        return this.data;
                    }.bind(this)
                };
            }

            if (!!this.options.remoteUrl) {
                configs.remote = {
                    url: this.options.remoteUrl + delimiter + this.options.getParameter + '=%QUERY',
                    ajax: {
                        beforeSend: function() {
                            this.sandbox.emit(REMOTE_LOAD.call(this));
                            this.showLoader();
                        }.bind(this)
                    },
                    filter: function(data) {
                        this.sandbox.emit(REMOTE_RETRIEVE.call(this), this.handleData(data));
                        this.hideLoader();

                        return this.data;
                    }.bind(this)
                };
            }

            this.sandbox.autocomplete.init(this.$valueField, configs);

            //looses the dropdown from the input box
            if (this.options.stickToInput === false) {
                this.sandbox.dom.css(this.sandbox.dom.find('.twitter-typeahead', this.$el), 'position', 'static');
            }

            //removes the typeahead hint box
            if (this.options.hint === false) {
                this.sandbox.dom.remove(this.sandbox.dom.find('.tt-hint', this.$el));
            }
        },

        /**
         * Show loader to indicate loading suggestions.
         */
        showLoader: function() {
            this.$el.find('.loader').show();
        },

        /**
         * Hide loader after loading suggestions.
         */
        hideLoader: function() {
            this.$el.find('.loader').hide();
        },

        /**
         * Returns true if id or name of context is contained within the excluded array
         * @param context {object} context with id and name
         * @returns {Boolean}
         */
        isExcluded: function(context) {
            for (var i = -1, length = this.excludes.length; ++i < length;) {
                if (context.id !== null && context.id === this.excludes[i].id) {
                    return true;
                } else if (context[this.options.valueKey] !== null &&
                    context[this.options.valueKey] === this.excludes[i][this.options.valueKey]) {
                    return true;
                }
            }
            return false;
        },

        /**
         * Binds custom events
         */
        setCustomEvents: function() {
            this.sandbox.on(SET_EXCLUDES.call(this), function(excludes) {
                this.excludes = this.parseExcludes(excludes);
            }.bind(this));

            this.sandbox.on(IS_MATCHED.call(this), function(callback) {
                if (this.isMatchedExactly() === true && this.getClosestMatch() !== null) {
                    callback(true);
                } else {
                    this.checkMatches(this.getValueFieldValue(), callback, true);
                }
            }.bind(this));

            this.sandbox.on('husky.auto-complete.' + this.options.instanceName + '.select', function(data) {
                this.selectedElement = data;
            }.bind(this));
        },

        /**
         * Brings an array of suggestions to exclude into the right format
         * @param excludes
         */
        parseExcludes: function(excludes) {
            var arrayReturn = [];

            if (!!excludes.length) {
                this.sandbox.util.foreach(excludes, function(exclude) {
                    if (typeof exclude !== 'object') {
                        arrayReturn.push({
                            id: null,
                            name: exclude
                        });
                    } else {
                        arrayReturn.push(exclude);
                    }
                }.bind(this));
            }
            return arrayReturn;
        },

        /**
         * sets several events
         */
        bindDomEvents: function() {
            this.sandbox.dom.on(this.$valueField, 'typeahead:selected', function(event, datum) {
                if (this.isExcluded(datum)) {
                    this.sandbox.dom.stopPropagation(event);
                    this.clearValueFieldValue();
                } else {
                    this.sandbox.emit(SELECT.call(this), datum);
                    this.setValueFieldId(datum.id);
                    if (typeof this.options.selectCallback === 'function') {
                        this.options.selectCallback.call(this, datum, event);
                    }
                }
            }.bind(this));

            //remove state and matches on new input
            this.sandbox.dom.on(this.$valueField, 'keypress', function(event) {
                if (event.keyCode !== 13) {
                    this.matched = false;
                    this.matches = [];
                }
            }.bind(this));

            this.sandbox.dom.on(this.sandbox.dom.find('.tt-dropdown-menu', this.$el), 'click', function() {
                return false;
            }.bind(this), '.disabled');

            this.sandbox.dom.on(this.$valueField, 'blur', function() {
                //don't do anything if the dropdown is clicked on
                if (this.options.emptyOnBlur === false) {
                    this.handleBlur();
                } else {
                    this.clearValueFieldValue();
                }
            }.bind(this));

            this.sandbox.dom.on(this.$el, 'click', function() {
                this.sandbox.dom.blur(this.$valueField);
                this.clearValueFieldValue();
                this.sandbox.emit(FOOTER_CLICKED.call(this));
            }.bind(this), '.auto-complete-footer');

            // clear data attribute when input is empty
            this.sandbox.dom.on(this.$valueField, 'focusout', function() {
                if (this.sandbox.dom.val(this.$valueField) === '') {
                    var dataId = this.sandbox.dom.attr(this.$valueField, 'data-id');
                    if (dataId != null && dataId !== 'null') {
                        this.sandbox.dom.removeAttr(this.$valueField, 'data-id');
                        this.sandbox.dom.data(this.$valueField, 'id', 'null');
                        this.sandbox.emit(SELECTION_REMOVED.call(this));
                    }
                }
            }.bind(this));
        },

        /**
         * Gets called when the input box triggers the blur event
         */
        handleBlur: function() {
            if (!!this.selectedElement) { // selected via dropdown
                this.selectedElement = null;
            } else if (this.options.noNewValues === true) {
                //check input matches an auto-complete suggestion
                if (this.isMatched() === true && this.getClosestMatch() !== null) {
                    //set value o field to the closes match
                    this.setValueFieldValue(this.getClosestMatch().name);
                    this.setValueFieldId(this.getClosestMatch().id);
                } else {
                    //request to check if a match exists
                    if (this.getValueFieldValue() !== '') {
                        this.checkMatches(this.getValueFieldValue(), function(isMatched) {
                            if (isMatched === true) {
                                this.setValueFieldValue(this.getClosestMatch().name);
                                this.setValueFieldId(this.getClosestMatch().id);
                            } else {
                                this.clearValueFieldValue();
                            }
                        }.bind(this), false);
                    }
                }
            } else {
                //check if new input or already contained in auto-complete suggestions
                if (this.isMatchedExactly() === true && this.getClosestMatch() !== null) {
                    this.setValueFieldValue(this.getClosestMatch().name);
                    this.setValueFieldId(this.getClosestMatch().id);
                }
            }
        },

        /**
         * Tries to request matches via the remoteUrl
         * and emits an event if matches do exist
         * @param {String} string The string to check if matches exist
         * @param {Function} callback to pass a boolean to
         * @param {Boolean} exactly If true string must be have an identical match
         */
        checkMatches: function(string, callback, exactly) {
            var delimiter = (this.options.remoteUrl.indexOf('?') === -1) ? '?' : '&';
            this.sandbox.emit(REQUEST_MATCH.call(this));
            this.sandbox.util.ajax({
                url: this.options.remoteUrl + delimiter + this.options.getParameter + '=' + string,

                success: function(data) {
                    this.matches = this.handleData(data);

                    if (exactly !== true) {
                        if (this.matches.length > 0) {
                            callback(true);
                        } else {
                            callback(false);
                        }
                    } else {
                        if (this.isMatchedExactly() === true) {
                            callback(true);
                        } else {
                            callback(false);
                        }
                    }
                }.bind(this),

                error: function(error) {
                    this.sandbox.logger.log('Error requesting auto-complete-matches', error);
                    callback(false);
                }.bind(this)
            });
        },

        /**
         * Returns the closest match for an input
         * @returns {object} closest match with id and name
         */
        getClosestMatch: function() {
            if (!!this.matches.length && this.getValueFieldValue() !== '') {
                return this.matches[0];
            }
            return null;
        },

        /**
         * Returns the trimed value of the input field
         * @returns {String}
         */
        getValueFieldValue: function() {
            return this.sandbox.dom.val(this.$valueField);
        },

        /**
         * Sets the input box value
         * @param value {String} new input value
         */
        setValueFieldValue: function(value) {
            this.sandbox.autocomplete.setValue(this.$valueField, value);
        },

        /**
         * Deletes the input box value
         */
        clearValueFieldValue: function() {
            this.sandbox.autocomplete.setValue(this.$valueField, '');
        },

        /**
         * Sets the data-id attribute on the input box
         * @param id {Integer} new data-id attribute value
         */
        setValueFieldId: function(id) {
            this.sandbox.dom.attr(this.$valueField, {'data-id': id});
            this.sandbox.dom.data(this.$valueField, 'id',  id);
        },

        /**
         * returns the matched property (true if auto-complete suggestion exist)
         * @returns {boolean}
         */
        isMatched: function() {
            return this.matched;
        },

        /**
         * Returns true if input matches an auto-complete suggestion exactly
         * case-insensitive
         * @returns {boolean}
         */
        isMatchedExactly: function() {
            if (this.getClosestMatch() !== null) {
                if (this.getValueFieldValue().toLowerCase() === this.getClosestMatch().name.toLowerCase()) {
                    return true;
                }
            }
            return false;
        },

        /**
         * Assigns loaded data to properties
         * @param data {object} with data array
         */
        handleData: function(data) {
            if (typeof data === 'object') {
                this.data = [];

                this.sandbox.util.foreach(data._embedded[this.options.resultKey], function(key) {
                    if (this.isExcluded(key) === false) {
                        this.data.push(key);
                    }
                }.bind(this));
                return this.data;
            }
            return false;
        }
    };
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/auto-complete-list
 */

/**
 * @class AutoCompleteList
 * @constructor
 *
 * @param {Object} [options] Configuration object
 * @param {String} [options.instanceName] name of the component instance
 * @param {Array} [options.items] preloaded items
 * @param {String} [options.itemsUrl] url to load items
 * @param {String} [options.resultKey] Key for AJAX response
 * @param {Array} [options.suggestions] suggestions for suggestions box
 * @param {String} [options.suggestionsHeadline] Headline for suggestions bxo
 * @param {String} [options.suggestionsUrl] url to load suggestions
 * @param {String} [options.suggestionsKey] Key for AJAX response
 * @param {String} [options.inputSelector] CSS-selector for input wrapper div
 * @param {String} [options.label] label (headline)
 * @param {Boolean} [options.autocomplete] enable/disable autocomplete
 * @param {Array} [options.localData] local data passed to the autocomplete component
 * @param {String} [options.prefetchUrl] url to prefetch data for the autocomplete component
 * @param {String} [options.remoteUrl] url to fetch data  for the autocomplete component from on input
 * @param {Object} [options.autocompleteOptions] options to pass to the autocomplete component
 * @param {String} [options.autocompleteParameter] name of parameter which contains the user's input (for auto-complete)
 * @param {Integer} [options.maxListItems] maximum amount of list items accepted (0 = no limit)
 * @param {Boolean} [options.CapitalizeFirstLetter] if true the first letter of each item gets capitalized
 * @param {String} [options.listItemClass] CSS-class for list items
 * @param {String} [options.suggestionDeactivatedClass] CSS-class for suggestion-element if suggestion is already used
 * @param {String} [options.AjaxPush] url to which added list items get send via ajax POST
 * @param {Boolean} [options.AjaxPushAllItems] if true all list items get sent if an item is added
 * @param {Object} [options.AjaxPushParameters] additional parameter payload to push with each AjaxPush
 * @param {String} [options.togglerSelector] CSS-selector for suggestion-toggler
 * @param {String} [options.arrowDownClass] CSS-class for arrow down icon
 * @param {String} [options.arrowUpClass] CSS-class for arrow up icon
 * @param {Integer} [options.slideDuration] ms - duration for sliding suggestinos up/down
 * @param {String} [options.elementTagDataName] attribute name to store list of tags on element
 * @param {String} [options.autoCompleteIcon] Icon Class-suffix for autocomplete-icon
 * @param {String} [options.suggestionIcon] Icon Class-suffix for autocomplete-suggestion-icon
 * @param {Array} [options.delimiters] Array of key-codes which trigger a tag input
 * @param {Boolean} [options.noNewTags] If true only auto-completed tags are accepted
 * @param {String} [options.footerContent] Could be a template or just text
 * @param {String} [options.valueKey] Name of value-property in suggestion
 */
define('__component__$auto-complete-list@husky',[], function() {

        'use strict';

        /**
         * Default values for options
         */
        var defaults = {
                instanceName: 'undefined',
                items: [],
                itemsUrl: '',
                resultKey: 'tags',
                suggestions: [],
                suggestionsHeadline: '',
                suggestionsUrl: '',
                suggestionsKey: 'suggestions',
                label: '',
                inputSelector: '.husky-auto-complete-container',
                autocomplete: true,
                localData: [],
                prefetchUrl: '',
                remoteUrl: '',
                autocompleteOptions: {},
                getParameter: 'query',
                maxListItems: 0,
                CapitalizeFirstLetter: false,
                listItemClass: 'auto-complete-list-selection',
                suggestionDeactivatedClass: 'deactivated',
                AjaxPush: '',
                AjaxPushAllItems: false,
                AjaxPushParameters: null,
                togglerSelector: '.toggler',
                arrowDownClass: 'fa-caret-down',
                arrowUpClass: 'fa-caret-up',
                slideDuration: 500,
                elementTagDataName: 'tags',
                autoCompleteIcon: 'search',
                suggestionIcon: 'tag',
                delimiters: [9, 188, 13],
                noNewTags: false,
                valueKey: 'name'
            },

            templates = {
                main: [
                    '<div class="auto-complete-list-container">',
                    '    <label>',
                    '        <%= label %>',
                    '        <div class="auto-complete-list">',
                    '           <div class="husky-auto-complete-container"></div>',
                    '           <div class="toggler"></div>',
                    '        </div>',
                    '    </label>',
                    '</div>'
                ].join(''),
                suggestion: [
                    '<div class="auto-complete-list-suggestions">',
                    '    <h5><%= headline %></h5>',
                    '    <ul></ul>',
                    '</div>'
                ].join('')
            },

            /** Position values for toggling suggestions */
            togglerPosUp = 'up',
            togglerPosDown = 'down',

            eventNamespace = 'husky.auto-complete-list.',

            /**
             * raised after initialization
             * @event husky.auto-complete-list.initialized
             */
            INITIALIZED = function() {
                return createEventName.call(this, 'initialized');
            },

            /**
             * used for setting tags
             * @event husky.auto-complete-list.set-tags
             * @param {Array} tags Array of strings
             */
            SET_TAGS = function() {
                return createEventName.call(this, 'set-tags');
            },

            /**
             * used for receiving tags
             * @event husky.auto-complete-list.get-tags
             */
            GET_TAGS = function() {
                return createEventName.call(this, 'get-tags');
            },

            /**
             * raised after AJAX request for loading items is sent
             * @event husky.auto-complete-list.items-request
             */
            ITEM_REQUEST = function() {
                return createEventName.call(this, 'item-request');
            },

            /**
             * raised after AJAX request for loading suggestions is sent
             * @event husky.auto-complete-list.sug-request
             */
            SUGGESTION_REQUEST = function() {
                return createEventName.call(this, 'sug-request');
            },

            /**
             * raised after a suggestion element is clicked and added to the list
             * @event husky.auto-complete-list.sug-added
             * @param {object} suggestion - the suggestion element with id, name, DOM-object
             */
            SUGGESTION_ADDED = function() {
                return createEventName.call(this, 'sug-added');
            },

            /**
             * raised after an item is deleted
             * @event husky.auto-complete-list.item-deleted
             */
            ITEM_DELETED = function() {
                return createEventName.call(this, 'item-deleted');
            },

            /**
             * raised after data was loaded
             * @event husky.auto-complete-list.data-loaded
             */
            DATA_LOADED = function() {
                return createEventName.call(this, 'data-loaded');
            },

            /**
             * raised after an item is added
             * @event husky.auto-complete-list.item-added
             * @param {string} item value
             */
            ITEM_ADDED = function() {
                return createEventName.call(this, 'item-added');
            },

            /**
             * raised after all item were added
             * @event husky.auto-complete-list.items-added
             */
            ITEMS_ADDED = function() {
                return createEventName.call(this, 'items-added');
            },

            /**
             * raised when the suggestion container is closed
             * @event husky.auto-complete-list.sug-closed
             */
            SUGGESTIONS_CLOSED = function() {
                return createEventName.call(this, 'sug-closed');
            },

            /**
             * raised when the suggestion container is opened
             * @event husky.auto-complete-list.sug-opened
             */
            SUGGESTIONS_OPENED = function() {
                return createEventName.call(this, 'sug-opened');
            },

            /** returns normalized event names */
            createEventName = function(postFix) {
                return eventNamespace + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
            };


        return {

            initialize: function() {
                this.sandbox.logger.log('initialize', this);

                this.setVars();

                // extend default options
                this.options = this.sandbox.util.extend({}, defaults, this.options);

                this.renderMain();
                this.initInputCont();
                this.initSuggestions();
                this.initItems();

                if (this.options.autocomplete === true) {
                    this.sandbox.on('husky.auto-complete.' + this.options.instanceName + '.initialized', function() {
                        this.sandbox.emit(INITIALIZED.call(this));
                    }.bind(this));
                } else {
                    this.sandbox.emit(INITIALIZED.call(this));
                }
            },

            /**
             * object properties get initialized
             */
            setVars: function() {
                this.suggestions = [];
                this.$suggestions = null;
                this.$input = null;
                this.tagApi = null;
                this.toggler = null;
                this.$inputCont = null;
            },

            /**
             * the main-template gets rendered and displayed
             */
            renderMain: function() {
                this.sandbox.dom.html(this.$el,
                    _.template(templates.main)({
                        label: this.options.label
                    })
                );
            },

            /**
             * the container for the input field gets assigned to a object property
             * @returns {boolean}
             */
            initInputCont: function() {
                this.$inputCont = this.sandbox.dom.find(this.options.inputSelector, this.$el);
                if (!this.$inputCont.length) {
                    this.sandbox.logger.log('Initializing input-container failed.');
                    return false;
                }
                return true;
            },

            /**
             * Autocomplete component and/or tagmanager plugin get started
             */
            startPlugins: function() {
                if (this.options.autocomplete === true) {
                    this.bindStartTmEvent();
                    this.startAutocomplete();
                } else {
                    //if autocomplete component is disabled, input field needs to be inserted
                    this.initInput();
                    this.appendInput();

                    this.startTagmanager();
                    this.bindEvents();
                }
            },

            /**
             * Input DOM-object gets assigned to an object property
             */
            initInput: function() {
                this.$input = this.sandbox.dom.createElement('<input type="text" autocomplete="off"/>');
            },

            /**
             * The input field gets inserted into the input-container
             */
            appendInput: function() {
                if (!!this.$input.length) {
                    this.sandbox.dom.append(this.$inputCont, this.$input);
                }
            },

            /**
             * husky-auto-complete component gets started
             */
            startAutocomplete: function() {
                this.sandbox.start([
                    {
                        name: 'auto-complete@husky',
                        options: this.sandbox.util.extend(
                            {
                                el: this.$inputCont,
                                emptyOnBlur: true,
                                instanceName: this.options.instanceName,
                                localData: this.options.localData,
                                prefetchUrl: this.options.prefetchUrl,
                                remoteUrl: this.options.remoteUrl,
                                getParameter: this.options.getParameter,
                                suggestionIcon: this.options.suggestionIcon,
                                autoCompleteIcon: this.options.autoCompleteIcon,
                                resultKey: this.options.resultKey,
                                footerContent: this.options.footerContent,
                                valueKey: this.options.valueKey
                            },
                            this.options.autocompleteOptions
                        )
                    }
                ]);
            },

            /**
             * Tagmanager plugin gets started
             */
            startTagmanager: function() {
                this.tagApi = this.sandbox.autocompleteList.init(this.$input, {
                    tagClass: this.options.listItemClass,
                    prefilled: this.options.items,
                    tagCloseIcon: '',
                    maxTags: this.options.maxListItems,
                    AjaxPush: this.options.AjaxPush,
                    AjaxPushAllTags: this.options.AjaxPushAllItems,
                    AjaxPushParameters: this.options.AjaxPushParameters,
                    CapitalizeFirstLetter: this.options.CapitalizeFirstLetter,
                    delimiters: []
                });
                this.setElementDataTags();
            },

            /**
             * Binds the start of the tagmanager plugin, to the initialize event of the auto-complete component
             * (autocomplete-component needs to be running to start tagmanager)
             */
            bindStartTmEvent: function() {
                this.sandbox.once('husky.auto-complete.' + this.options.instanceName + '.initialized', function(data) {
                    this.$input = data;
                    this.startTagmanager();
                    this.bindEvents();
                }.bind(this));
            },

            /**
             * Bind several events
             */
            bindEvents: function() {
                this.sandbox.on(GET_TAGS.call(this), function(callback) {
                    if (typeof callback === 'function') {
                        callback(this.getTags());
                    } else {
                        this.sandbox.logger.log('Error: Callback is not a function');
                    }
                }.bind(this));

                this.sandbox.on(SET_TAGS.call(this), function(tags) {
                    this.pushTags(tags);
                }.bind(this));

                this.sandbox.on(ITEM_ADDED.call(this), function() {
                    this.setElementDataTags();
                    this.sandbox.emit('husky.auto-complete.' + this.options.instanceName + '.set-excludes', this.getTags());
                }.bind(this));

                this.sandbox.on(ITEM_DELETED.call(this), function() {
                    this.sandbox.emit('husky.auto-complete.' + this.options.instanceName + '.set-excludes', this.getTags());
                }.bind(this));

                //if an autocomplete-suggestion gets clicked on, it gets added to the list
                this.sandbox.on('husky.auto-complete.' + this.options.instanceName + '.select', function(d) {
                    this.pushTag(d[this.options.valueKey]);
                }.bind(this));

                this.sandbox.dom.on(this.$input, 'keydown', function(event) {
                    if (event.keyCode === 8 && this.sandbox.dom.val(this.$input).trim() === '') {
                        this.itemDeleteHandler();
                        this.setElementDataTags();
                    }

                    if (this.options.delimiters.indexOf(event.keyCode) !== -1) {
                        this.pushTagHandler(event, this.sandbox.dom.val(this.$input).trim());
                    }

                }.bind(this));

                this.sandbox.dom.on(this.$el, 'click', function(event) {
                    if (this.sandbox.dom.hasClass(event.target, 'tm-tag-remove') === true) {
                        this.itemDeleteHandler();
                        this.setElementDataTags();
                    }
                }.bind(this));

                if (this.toggler !== null) {
                    this.sandbox.dom.on(this.toggler.$el, 'click', function(event) {
                        this.sandbox.dom.preventDefault(event);
                        this.changeToggler();
                        this.toggleSuggestions();
                    }.bind(this));
                }
            },

            /**
             * Gets executed if the user presses a defined delimiter
             * @param {Object} the keydown event
             * @param {String} tag The new Tag to push
             */
            pushTagHandler: function(event, tag) {
                if (event.keyCode !== 9) {
                    this.sandbox.dom.preventDefault(event);
                }

                if (this.options.noNewTags !== true) {
                    this.pushTag(tag);
                } else {
                    this.sandbox.emit('husky.auto-complete.' + this.options.instanceName + '.is-matched', function(isMatched) {
                        if(isMatched === true) {
                            this.pushTag(tag);
                        }
                    }.bind(this));
                }
            },

            /**
             * Binds the tags to the element
             * @param newTag {String} newly added tag
             */
            setElementDataTags: function() {
                var tags = this.getTags();
                this.sandbox.dom.data(this.$el, this.options.elementTagDataName, tags);
            },

            /**
             * items for the list get loaded and plugins get started
             */
            initItems: function() {
                if (this.options.itemsUrl !== '') {
                    this.requestItems();
                } else {
                    //if no items need to be loaded start the plugins right ahead
                    this.startPlugins();
                }
            },

            /**
             * request the items for the list, merge them with the options and start the pluins
             */
            requestItems: function() {
                this.sandbox.util.ajax({
                    url: this.options.itemsUrl,

                    success: function(data) {
                        this.options.items = this.options.items.concat(data._embedded[this.options.resultKey]);
                        this.startPlugins();
                        DATA_LOADED.call(this);
                    }.bind(this),

                    error: function(error) {
                        this.sandbox.logger.log(error);
                    }.bind(this)
                });
                this.sandbox.emit(ITEM_REQUEST.call(this));
            },

            /**
             * Load the suggestions, render them, initialize toggler
             */
            initSuggestions: function() {
                if (this.options.suggestionsUrl !== '') {
                    this.requestSuggestions();
                } else {
                    //if no suggestions need to be loaded, render them right ahead
                    this.loadSuggestions();
                    this.renderSuggestions();
                    this.initToggler();
                }
            },

            /**
             * load suggestions from url, render them, initialize toggler
             */
            requestSuggestions: function() {
                this.sandbox.util.ajax({
                    url: this.options.suggestionsUrl,

                    success: function(data) {
                        this.options.suggestions = this.options.suggestions.concat(data[this.options.suggestionsKey]);
                        this.loadSuggestions();
                        this.renderSuggestions();
                        this.initToggler();
                    }.bind(this),

                    error: function(error) {
                        this.sandbox.logger.log(error);
                    }.bind(this)
                });
                this.sandbox.emit(SUGGESTION_REQUEST.call(this));
            },

            /**
             * Converts the suggestion-strings to objects with name and DOM-object
             */
            loadSuggestions: function() {
                if (!!this.options.suggestions.length) {
                    for (var i = -1, length = this.options.suggestions.length; ++i < length;) {
                        this.suggestions[i] = {
                            name: this.options.suggestions[i],
                            $el: this.sandbox.dom.createElement('<li/>')
                        };
                        this.sandbox.dom.html(this.suggestions[i].$el, this.suggestions[i][this.options.valueKey]);
                    }
                }
            },

            /**
             * Renders the suggestion container and appends the suggestion elements
             */
            renderSuggestions: function() {
                if (!!this.options.suggestions.length) {
                    var box, list, i = -1, length = this.suggestions.length;
                    box = this.sandbox.dom.parseHTML(
                        _.template(templates.suggestion)({
                            headline: this.options.suggestionsHeadline
                        })
                    );
                    list = this.sandbox.dom.children(box, 'ul');
                    for (; ++i < length;) {
                        this.sandbox.dom.append(list, this.suggestions[i].$el);
                        this.bindSuggestionEvents(this.suggestions[i]);
                    }
                    this.$suggestions = box;
                    this.sandbox.dom.append(this.$el, this.$suggestions);
                }
            },

            /**
             * Initializes the toggler object with position and DOM-element
             */
            initToggler: function() {
                if (!!this.options.suggestions.length) {
                    this.toggler = {
                        $el: this.sandbox.dom.find(this.options.togglerSelector, this.$el),
                        pos: togglerPosUp
                    };
                    this.sandbox.dom.prependClass(this.toggler.$el, this.options.arrowUpClass);
                }
            },

            /**
             * toggles the Toggler-icon
             */
            changeToggler: function() {
                if (this.toggler.pos === togglerPosDown) {
                    this.togglerUp();
                } else {
                    this.togglerDown();
                }
            },

            /**
             * toggles the suggestions-container
             */
            toggleSuggestions: function() {
                if (this.toggler.pos === togglerPosDown) {
                    this.hideSuggestions();
                } else {
                    this.showSuggestions();
                }
            },

            /**
             * hides the suggestions-container
             */
            hideSuggestions: function() {
                this.sandbox.dom.slideUp(this.$suggestions, this.options.slideDuration);
                this.sandbox.emit(SUGGESTIONS_CLOSED.call(this));
            },

            /**
             * shows the suggestions-container
             */
            showSuggestions: function() {
                this.sandbox.dom.slideDown(this.$suggestions, this.options.slideDuration);
                this.sandbox.emit(SUGGESTIONS_OPENED.call(this));
            },

            /**
             * Ensures the toggler is pointing down (sets CSS-classes)
             */
            togglerDown: function() {
                this.sandbox.dom.removeClass(this.toggler.$el, this.options.arrowUpClass);
                this.sandbox.dom.prependClass(this.toggler.$el, this.options.arrowDownClass);
                this.toggler.pos = togglerPosDown;
            },

            /**
             * Ensures the toggler is pointing up (sets CSS-classes)
             */
            togglerUp: function() {
                this.sandbox.dom.removeClass(this.toggler.$el, this.options.arrowDownClass);
                this.sandbox.dom.prependClass(this.toggler.$el, this.options.arrowUpClass);
                this.toggler.pos = togglerPosUp;
            },

            /**
             * Adds click event -> suggestion gets added to the list
             * @param suggestion {object} with DOM-element as property
             */
            bindSuggestionEvents: function(suggestion) {
                this.sandbox.dom.on(suggestion.$el, 'click', function() {
                    if (!this.sandbox.dom.hasClass(suggestion, this.options.suggestionDeactivatedClass)) {
                        this.pushTag(suggestion[this.options.valueKey]);
                        this.deactivateSuggestion(suggestion);
                        this.sandbox.emit(SUGGESTION_ADDED.call(this), suggestion);
                    }
                }.bind(this));
            },

            /**
             * gets called if an item is deleted from the list
             */
            itemDeleteHandler: function() {
                this.refreshSuggestions();
                this.sandbox.emit(ITEM_DELETED.call(this));
            },

            /**
             * Deactivated-CSS-class gets added to suggestion
             * @param suggestion {object} with DOM-element as property
             */
            deactivateSuggestion: function(suggestion) {
                this.sandbox.dom.addClass(suggestion.$el, this.options.suggestionDeactivatedClass);
            },

            /**
             * Activated-CSS-class gets added to suggestion
             * @param suggestion {object} with DOM-element as property
             */
            activateSuggestion: function(suggestion) {
                this.sandbox.dom.removeClass(suggestion.$el, this.options.suggestionDeactivatedClass);
            },

            /**
             * Checks if suggestions are used in list and activates them if not
             */
            refreshSuggestions: function() {
                for (var i = -1, length = this.suggestions.length; ++i < length;) {
                    if (this.sandbox.dom.hasClass(this.suggestions[i].$el, this.options.suggestionDeactivatedClass) === true) {
                        if (this.suggestionContainedInTags(this.suggestions[i]) === false) {
                            this.activateSuggestion(this.suggestions[i]);
                        }
                    }
                }
            },

            /**
             * Checks if suggestion is used in list
             * @param suggestion {object} with name as property
             * @returns {boolean}
             */
            suggestionContainedInTags: function(suggestion) {
                var tags = this.getTags(), i, length;
                for (i = -1, length = tags.length; ++i < length;) {
                    if (tags[i] === suggestion[this.options.valueKey]) {
                        return true;
                    }
                }
                return false;
            },

            /**
             * Pushes an Item to the list
             * @param value {String} string to push into the list
             * @returns {boolean}
             */
            pushTag: function(value) {
                if (this.tagApi !== null) {
                    this.tagApi.tagsManager('pushTag', value);
                    this.sandbox.emit(ITEM_ADDED.call(this), value);
                } else {
                    return false;
                }
            },

            /**
             * Pushes an array of items to the list
             * @param value {Array} array with items to push to the list
             */
            pushTags: function(tags) {
                for (var i = -1, length = tags.length; ++i < length;) {
                    this.pushTag(tags[i]);
                }
                this.sandbox.emit(ITEMS_ADDED.call(this));
            },

            /**
             * Returns an array with all list items
             * @returns {Array}
             */
            getTags: function() {
                if (this.tagApi !== null) {
                    return this.tagApi.tagsManager('tags');
                } else {
                    return [];
                }
            }
        };
    }
);

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/toolbar
 */

/**
 * @class Dependant Select
 * @constructor
 *
 * @param {Object} [options] Configuration object
 * @param {String} [options.url] url to fetch data from
 * @param {String} [options.preselect] the item that's selected on initialization
 * @param {String} [options.instanceName] instance name of this component
 * @param {Array} [options.data] Array of dropdown items
 * @param {Array} options.container array containers to put selects into
 * @param {String|Array} [options.defaultLabels] which text is displayed after select is initialized; can be customized for each and every select by defining an array
 * @param {Array} [options.selectOptions] array of Options to pass to the select at certain depth (first item's options will be assigned to first select)
 */
define('__component__$dependent-select@husky',[],function() {

    'use strict';

    var defaults = {
            url: null,
            data: [],
            instanceName: '',
            preselect: null,
            defaultLabels: 'Please choose',
            selectOptions: [],
            container: null,
            isNative: false,
            deselectField: false
        },
        constants = {
            childContainerClass: 'dependent-select-container',
            namespace: 'husky.dependent-select.'
        },


        /**
         * triggered when component is completely initialized
         * @event husky.dependent-select[.INSTANCE_NAME].initialized
         */
            INITIALIZED = function() {
            return getEventName.call(this, 'initialized');
        },

        /**
         * triggered when a select item is selected
         * @event husky.dependent-select[.INSTANCE_NAME].item.selected
         * @param id of the select that was selected
         * @param depth of the select that was selected
         */
            ITEM_SELECTED = function() {
            return getEventName.call(this, 'item.selected');
        },

        /**
         * triggered when a select item is deselected
         * @event husky.dependent-select[.INSTANCE_NAME].item.deselected
         * @param id of the select that was deselected
         * @param depth of the select that was deselected
         */
            ITEM_DESELECTED = function() {
            return getEventName.call(this, 'item.deselected');
        },

        /**
         * triggered when a not all items are selected anymore
         * @event husky.dependent-select[.INSTANCE_NAME].all.items.deselected
         * @param id of the select that was changed
         */
            ALL_ITEMS_DESELECTED = function() {
            return getEventName.call(this, 'all.items.deselected');
        },

        /**
         * triggered when all selects have been set
         * @event husky.dependent-select[.INSTANCE_NAME].all.items.selected
         * @param id of the last item that was selected
         */
            ALL_ITEMS_SELECTED = function() {
            return getEventName.call(this, 'all.items.selected');
        },

    // creates event strings based on
        getEventName = function(postFix) {
            return constants.namespace + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
        },

    // empties all selects beginning from a certain depth
        renderEmptySelect = function(depth) {
            var i, len, $child;
            for (i = depth, len = this.options.container.length; ++i < len;) {
                $child = findStopAndCreateNewChild.call(this, this.options.container[i]);
                this.sandbox.start([
                    {
                        name: 'select@husky',
                        options: {
                            el: $child,
                            disabled: true,
                            singleSelect: true,
                            instanceName: i,
                            data: [],
                            isNative: this.options.isNative,
                            deselectField: this.options.deselectField,
                            defaultLabel: this.sandbox.dom.isArray(this.options.defaultLabels) ? this.options.defaultLabels[depth] : this.options.defaultLabels
                        }
                    }
                ]);
            }
        },

    // searches array for a certain id
        getDataById = function(data, id) {
            for (var i = -1, len = data.length; ++i < len;) {
                if (data[i].id.toString() === id.toString()) {
                    return data[i];
                }
            }
            return null;
        },

        /**
         * creates a child element in a container
         * @param containerId
         * @returns {domObject}
         */
            findStopAndCreateNewChild = function(containerId) {
            var $container = this.$find(containerId),
                $child = this.sandbox.dom.find('.' + constants.childContainerClass, $container);
            if (!$container) {
                throw 'dependent-select: no container with id ' + containerId + ' could be found';
            }
            // stop child, if running
            if (!!$child) {
                this.sandbox.stop($child);
            }
            // create new child
            $child = this.sandbox.dom.createElement('<div class="' + constants.childContainerClass + '" />');
            this.sandbox.dom.append($container, $child);
            return $child;
        },

        // checks if all selects are selected
        checkAllSelected = function() {
            var $lastContainer = this.$find(this.options.container[this.options.container.length-1]),
                lastSelectElement = this.sandbox.dom.children($lastContainer)[0],
                selection = this.sandbox.dom.data(lastSelectElement,'selection');

            // if last element is selected
            if (!!lastSelectElement && typeof selection !== 'undefined') {
                if (!this.allSelected) {
                    this.allSelected = true;
                    this.sandbox.emit(ALL_ITEMS_SELECTED.call(this));
                }
            } else if (this.allSelected) {
                this.allSelected = false;
                this.sandbox.emit(ALL_ITEMS_DESELECTED.call(this));
            }
        },

    // renders selects at a certain depth
        renderSelect = function(data, depth, preselect) {

            depth = typeof depth !== 'undefined' ? depth : 0;

            if (!this.options.container || !this.options.container[depth]) {
                throw 'dependent-select: no container at depth ' + depth + ' specified';
            }

            var selectionCallback = null,
                deselectionCallback = null,
                options,
            // get child
                $child = findStopAndCreateNewChild.call(this, this.options.container[depth]);

            // create callback

            selectionCallback = function(id) {
                // if there are more selects left
                if (this.options.container.length > depth && !!data[0].items) {
                    var items = getDataById.call(this, data, id).items;
                    renderSelect.call(this, items, depth + 1);
                }
                // trigger events
                this.sandbox.emit(ITEM_SELECTED.call(this), id, depth);
                checkAllSelected.call(this);
            }.bind(this);
            deselectionCallback = function(id) {
                if (this.options.container.length > depth && !!data[0].items) {
                    if (id === null) {
                        renderEmptySelect.call(this, depth);
                    }
                }
                this.sandbox.emit(ITEM_DESELECTED.call(this), id, depth);
                checkAllSelected.call(this);
            }.bind(this);

            // make it possible to set some data for select
            if (!!this.options.selectOptions[depth]) {
                options = this.options.selectOptions[depth];
            }

            options = this.sandbox.util.extend(true, {}, {
                el: $child,
                singleSelect: true,
                instanceName: depth,
                data: data,
                selectCallback: selectionCallback,
                deselectCallback: deselectionCallback,
                isNative: this.options.isNative,
                preSelectedElements: !!preselect ? [preselect] : [],
                deselectField: this.options.deselectField,
                defaultLabel: this.sandbox.dom.isArray(this.options.defaultLabels) ? this.options.defaultLabels[depth] : this.options.defaultLabels
            }, options);

            // start select component
            this.sandbox.start([
                {
                    name: 'select@husky',
                    options: options
                }
            ]);
        };

    return {

        initialize: function() {

            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

            // load data and call render
            if (!!this.options.url) {
                this.sandbox.util.load(this.options.url)
                    .then(this.render.bind(this))
                    .fail(function(data) {
                        this.sandbox.logger.log('data could not be loaded:', data);
                    }.bind(this));
            } else if (!!this.options.data) {
                this.render(this.options.data);
            } else {
                this.sandbox.logger.log('no data provided for dependent select!');
            }
        },

        render: function(data) {
            // create items array
            renderSelect.call(this, data, 0, this.options.preselect);
            renderEmptySelect.call(this, !!this.options.preselect ? this.options.preselect.length : 0);

            // initialization finished
            this.sandbox.emit(INITIALIZED.call(this));
        }
    };
})
;

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/toolbar
 */

/**
 * @class Select
 * @constructor
 *
 * @param {Object} [options] Configuration object
 * @param {String} [options.checkedAllLabel] Label to display if all items of select are checked
 * @param {Array}  [options.data] array of data [string, object]
 * @param {Array}  [options.defaultLabel] default label which gets displayed
 * @param {Function} [options.deselectCallback] callbackfunction, when element is deselected
 * @param {String|Boolean} [options.deselectField=false]  allows deselection in case of single select; will be set if value is a string
 * @param {Boolean} [options.disabled=false]  disables the select field if set to true
 * @param {String} [options.instanceName] instance name of this component
 * @param {Boolean} [options.multipleSelect=false] allows multiple elements to be selected
 * @param {Array} [options.preSelectedElements] allows preselection of fields by defining the id attributes or strings
 * @param {Function} [options.selectCallback] callbackfunction, when element is selected
 * @param {String} [options.valueName] name of property which should be used
 * @param {String} [options.style] "normal", "small", "big", "action" for different appearance. Multiple assignments
 *                  are done by separating styles with a space (e.g. 'action big')
 * @param {String} [options.skin] style class to set to the elements component (e.g 'white')
 * @param {Boolean} [options.emitValues] If true the value is emited with events instead of the id
 * @param {Boolean} [options.fixedLabel] If true the label never gets changed
 * @param {String} [options.icon] icon to set into the label
 * @param {Function} [options.noItemsCallback] callback function which gets executed at click on label if no dropdown-items exist
 * @param {Boolean} [options.repeatSelect] If true dropdown item can be selected n-times in succession
 * @param {Boolean} [options.editable] If true the menu items are editable
 * @param {Object} [options.translations] translation keys for 'addItem' and 'editEntries'
 * @param {String} [options.direction] 'bottom', 'top', or 'auto' pop up direction of the drop down.
 * @param {String} [options.resultKey] key in result set - default is empty and the _embedded property of the result set will be taken
 * @param {String} [options.url] url to load data from
 * @param {Boolean} [options.isNative] should use native select
 * @param {Boolean} [options.showToolTip] Show tool-tip on hover - only works for single-selects
 * @param {Array} [options.defaultValue] Array of default items that should be selected if no item has been preselected
 * @param {Number} [options.dropdownPadding] The amount of padding on top or bottom of the dropdown list
 */

define('__component__$select@husky',[], function() {

    'use strict';

    var translations = {
            addItem: 'select.add-item',
            editEntries: 'select.edit-entries'
        },

        defaults = {
            data: [],                         // data array
            valueName: 'name',                // name of text property
            instanceName: 'undefined',        // instance name
            defaultLabel: 'Please choose',    // default label which gets displayed
            checkedAllLabel: 'All Languages', // Label if all checked
            preSelectedElements: [],          // Elements selected by default
            multipleSelect: false,            // Allows multiple elements to be selected
            deselectField: false,             // field for deselection is added to dropdown if value is a string
            disabled: false,                  // if true button is disabled
            selectCallback: null,
            preselectCallback: null,
            deselectCallback: null,
            style: 'normal',
            skin: '',
            emitValues: false,
            fixedLabel: false,
            icon: null,
            noItemsCallback: null,
            repeatSelect: false,
            editable: false,
            direction: 'auto',
            resultKey: '',
            showToolTip: false,
            translations: translations,
            isNative: false,
            defaultValue: null,
            dropdownPadding: 0,
        },

        constants = {
            selectClass: 'husky-select',
            labelClass: 'husky-select-label',
            listClass: 'husky-select-list',
            dropdownContainerClass: 'husky-select-dropdown-container',
            deselectFieldKey: 'deselectindex',
            deselectFieldDefaultValue: '',
            disabledClass: 'disabled',
            dropdownTopClass: 'top',
            templateRemoveSelector: '.remove-row',
            templateAddSelector: '#addRow',
            editableFieldKey: 'editableindex',
            typeRowSelector: '.type-row',
            contentInnerSelector: '.content-inner',
            toggleClass: '.toggle-icon',
            formElementSelector: '.form-element',
            selectContainerSelector: '.husky-select-container'
        },

        templates = {
            row: function() {
                return[
                    '<div class="grid-row type-row" data-id="">',
                    '   <div class="grid-col-10 pull-left"><input class="form-element" type="text" value=""/></div>',
                    '   <div class="grid-col-2 pull-right"><div class="remove-row btn gray-dark fit only-icon pull-right"><div class="fa-minus-circle"></div></div></div>',
                    '</div>'].join('');
            },
            addOverlayRow: function(valueField, item) {
                return [
                    '<div class="grid-row type-row" data-id="', item.id ,'">',
                    '    <div class="grid-col-10 pull-left"><input class="form-element" type="text" value="', item[valueField],'"/></div>',
                    '    <div class="grid-col-2 pull-right"><div class="remove-row btn gray-dark fit only-icon pull-right"><div class="fa-minus-circle"></div></div></div>',
                    '</div>'
                ].join('');
            },
            addOverlaySkeleton: function() {
                return [
                    '<div class="content-inner">',
                    '   <% _.each(data, function(item) { %>',
                    '        <%= rowTemplate(valueName, item) %>',
                    '   <% }); %>',
                    '</div>',
                    '<div class="grid-row">',
                    '   <div id="addRow" class="addButton">',
                    '   <span>', this.sandbox.translate(defaults.translations.addItem), '</span>',
                    '   </div>',
                    '</div>'
                ].join('');
            }
        },

        /**
         * triggered when component is completely initialized
         * @event husky.select[.INSTANCE_NAME].initialized
         */
        EVENT_INITIALIZED = function() {
            return getEventName.call(this, 'initialize');
        },

        /**
         * triggered when item has been deselected
         * @event husky.select[.INSTANCE_NAME].deselected.item
         * @param {String} key of deselected item
         */
        EVENT_DESELECTED_ITEM = function() {
            return getEventName.call(this, 'deselected.item');
        },
        /**
         * triggered when item has been selected
         * @event husky.select[.INSTANCE_NAME].selected.item
         * @param {String} key of selected item
         */
        EVENT_SELECTED_ITEM = function() {
            return getEventName.call(this, 'selected.item');
        },

        /**
         * triggered when item has been preselected
         * @event husky.select[.INSTANCE_NAME].preselected.item
         * @param {String} key of selected item
         */
        EVENT_PRESELECTED_ITEM = function() {
            return getEventName.call(this, 'preselected.item');
        },

        /**
         * used for enabling select
         * @event husky.select[.INSTANCE_NAME].enable
         */
        EVENT_ENABLE = function() {
            return getEventName.call(this, 'enable');
        },
        /**
         * used for disabling select
         * @event husky.select[.INSTANCE_NAME].disable
         */
        EVENT_DISABLE = function() {
            return getEventName.call(this, 'disable');
        },

        /**
         * used for toggling enabled/disabled dropdown menu
         * @event husky.select[.INSTANCE_NAME].toggle
         */
        EVENT_TOGGLE = function() {
            return getEventName.call(this, 'toggle');
        },
        /**
         * used to show dropdown selection
         * @event husky.select[.INSTANCE_NAME].show
         */
        EVENT_SHOW = function() {
            return getEventName.call(this, 'show');
        },
        /**
         * used for hiding dropdown selection
         * @event husky.select[.INSTANCE_NAME].hide
         */
        EVENT_HIDE = function() {
            return getEventName.call(this, 'hide');
        },
        /**
         * used for receiving all checked elements
         * @event husky.select[.INSTANCE_NAME].get-checked
         */
        EVENT_GET_CHECKED = function() {
            return getEventName.call(this, 'get-checked');
        },
        /**
         * update the elements of the dropdown list
         * @event husky.select[.INSTANCE_NAME].update
         */
        EVENT_UPDATE = function() {
            return getEventName.call(this, 'update');
        },
        /**
         * update the elements of the dropdown list
         * @event data-changed
         */
        EVENT_DATA_CHANGED = function() {
            return 'data-changed';
        },
        /**
         * Save event for elements to save
         * @event husky.select.[instanceName].save
         */
        EVENT_SAVE = function() {
            return getEventName.call(this, 'save');
        },

        /**
         * Deleted event for elements to delete
         * @event husky.select.[instanceName].delete
         */
        EVENT_DELETE = function() {
            return getEventName.call(this, 'delete');
        },

        /**
         * Revert event for elements to be removed from the select
         * @event husky.select.[instanceName].revert
         */
        EVENT_REVERT = function() {
            return getEventName.call(this, 'revert');
        },

        getEventName = function(suffix) {
            return 'husky.select.' + this.options.instanceName + '.' + suffix;
        },

        showCaret = function() {
            this.sandbox.dom.show(getCaret.call(this));
        },
        hideCaret = function() {
            this.sandbox.dom.hide(getCaret.call(this));
        },
        getCaret = function() {
            return this.$find(constants.toggleClass);
        },

        /**
         * Process styles of options and apply to select.
         *
         * @param {Object} $button
         */
        processStyles = function($button) {
            var allowedStyles = ['small', 'big', 'action'];
            if (!!this.options.style) {
                var styles = this.options.style.split(' ');
                this.sandbox.util.foreach(styles, function(style) {
                    if (allowedStyles.indexOf(style) > -1) {
                        this.sandbox.dom.addClass($button, style);
                    }
                }.bind(this));
            }
        };

    return {
        initialize: function() {
            var selectedIds;

            this.sandbox.logger.log('initialize', this);
            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

            // if deselectfield is set to true, set it to default value
            if (!!this.options.deselectField && this.options.deselectField.toString() === 'true') {
                this.options.deselectField = constants.deselectFieldDefaultValue;
            }

            this.selection = [];
            this.selectedElements = [];
            this.selectedElementsValues = [];
            this.dropdownVisible = false;

            // Used as a fallback to revert to the last committed data
            this.mergedData = null;

            // if no data was preselected search for data array
            if (!this.options.preSelectedElements || this.options.preSelectedElements.length === 0) {
                selectedIds = this.sandbox.dom.data(this.$el, 'selection');
                if (typeof selectedIds === 'string') {
                    this.options.preSelectedElements = selectedIds.split(',');
                } else if (Array.isArray(selectedIds)) {
                    this.options.preSelectedElements = selectedIds.map(String);
                } else if (typeof selectedIds === 'number') {
                    this.options.preSelectedElements.push(selectedIds.toString());
                } else if (!!this.options.defaultValue) {
                    this.options.preSelectedElements = this.options.defaultValue;
                }
            }

            if (this.options.url) {
                this.loadEntries(this.options.url);
            } else if (!!this.options.data) {
                this.mergedData = this.options.data.slice(0);
                this.render();
                this.sandbox.emit(EVENT_INITIALIZED.call(this));
            } else {
                this.sandbox.logger.warn('Neither data nor url defined!');
            }
        },

        /**
         * Loads entries from url
         * @param url
         */
        loadEntries: function(url) {
            this.sandbox.util.load(url)
                .then(function(response) {

                    if (!!this.options.resultKey) {
                        this.options.data = response._embedded[this.options.resultKey];
                    } else {
                        this.options.data = response._embedded;
                    }

                    this.mergedData = this.options.data.slice(0);
                    this.render();
                    this.sandbox.emit(EVENT_INITIALIZED.call(this));
                }.bind(this))
                .fail(function(request, message, error) {
                    this.sandbox.logger.warn(request, message, error);
                }.bind(this));
        },

        render: function() {
            // if preselected items are not present in the data.
            this.options.preSelectedElements = _.filter(this.options.preSelectedElements, function(item) {
                return _.contains(this.options.data, item)
                    || _.filter(
                        this.options.data,
                        function(data){
                            return data.id == item;
                        }.bind(this)
                    ).length > 0;
            }.bind(this));

            // add husky-select class to component
            this.sandbox.dom.addClass(this.$el, constants.selectClass);

            var $originalElement = this.sandbox.dom.$(this.options.el),
                button = this.sandbox.dom.createElement(
                    this.sandbox.util.template(
                        this.template.basicStructure.call(this, this.options.defaultLabel, this.options.icon),
                        {
                            isNative: this.options.isNative
                        }
                    )
                );
            this.sandbox.dom.append($originalElement, button);

            processStyles.call(this, button);

            // add skin style
            this.sandbox.dom.addClass(button, this.options.skin);

            this.$list = this.$find('.' + constants.listClass);
            this.$dropdownContainer = this.$find('.' + constants.dropdownContainerClass);
            this.$label = this.$find('.' + constants.labelClass);
            this.prepareData();

            // bind dom elements
            this.bindDOMEvents();
            this.bindCustomEvents();

            if (this.options.disabled === true) {
                this.disable();
            }

            this.addEditEntry();

        },

        //sets the button in enabled state
        enable: function() {
            this.sandbox.dom.removeClass(this.sandbox.dom.children(this.$el, '.husky-select-container'), constants.disabledClass);
            this.bindDOMEvents();
        },

        //sets the button i
        disable: function() {
            this.sandbox.dom.addClass(this.sandbox.dom.children(this.$el, '.husky-select-container'), constants.disabledClass);
            this.hideDropDown();
            this.sandbox.dom.off(this.$el);
        },

        // If 'editable' flag is set in the options an 'Edit' menu entry is
        // added to the list.
        addEditEntry: function() {
            if (this.options.editable === true) {
                this.addDivider();
                this.addDropdownElement(constants.editableFieldKey,
                    this.sandbox.translate(translations.editEntries),
                    false,
                    null,
                    null,
                    false
                );
            }
        },

        // prepares data for dropDown, if options.data not set load with ajax
        prepareData: function() {
            if (this.options.data.length > 0) {
                if (this.options.preSelectedElements.length > 0) {
                    this.options.preSelectedElements.map(function(el, index, arr) {
                        if (!!arr[index] || arr[index] === 0) {
                            arr[index] = el.toString();
                        }
                    });
                }
                this.generateDropDown(this.options.data);
            } else {
                this.sandbox.logger.log('error: data not set');
            }
        },

        /**
         * Adds a dropdown element to the dropdown list
         * @param id
         * @param value
         * @param disabled
         * @param callback
         * @param updateLabel
         * @param checkboxVisible
         */
        addDropdownElement: function(id, value, disabled, callback, updateLabel, checkboxVisible) {
            checkboxVisible = checkboxVisible !== false;
            var $item,
                template = (this.options.isNative) ? this.template.optionElement : this.template.menuElement,
                idString = (id !== null && typeof id !== 'undefined') ? id.toString() : this.sandbox.util.uniqueId();

            if (this.options.preSelectedElements.indexOf(idString) >= 0 ||
                this.options.preSelectedElements.indexOf(value) >= 0
            ) {
                this.setToolTip(value);
                $item = this.sandbox.dom.createElement(this.sandbox.util.template(
                    template.call(
                        this,
                        idString,
                        value,
                        'checked',
                        updateLabel,
                        true
                    ),
                    {
                        checked: true
                    }
                ));

                this.selectedElements.push(idString);
                this.selectedElementsValues.push(value);

                if (this.options.emitValues === true) {
                    this.triggerPreSelect(idString);
                } else {
                    this.triggerPreSelect(value);
                }
            } else {
                $item = this.sandbox.dom.createElement(
                    this.sandbox.util.template(
                        template.call(
                            this,
                            idString,
                            value,
                            '',
                            updateLabel,
                            checkboxVisible
                        ),
                        {
                            checked: false
                        }
                    )
                );
            }

            // store callback if callback is set
            if (typeof callback === 'function') {
                this.sandbox.dom.data($item, 'selectCallback', callback);
            }

            if (!!disabled && disabled === true) {
                this.sandbox.dom.addClass($item, 'disabled');

                if (this.options.isNative) {
                    this.sandbox.dom.attr($item, 'disabled', 'disabled');
                }
            }

            this.sandbox.dom.append(this.$list, $item);
        },

        /**
         * Adds a divider element to the dropdown list
         */
        addDivider: function() {
            var $item = this.sandbox.dom.$('<hr class="divider"/>');
            this.sandbox.dom.append(this.$list, $item);
        },

        /**
         * Generates the dropdown list with the given items
         * @param items
         */
        generateDropDown: function(items) {
            if (typeof this.options.deselectField === 'string' && this.options.deselectField !== 'false') {
                this.addDropdownElement(constants.deselectFieldKey, this.sandbox.translate(this.options.deselectField));
            }
            if (items.length > 0) {
                if (typeof(items[0]) === 'string') {
                    this.sandbox.util.each(items, function(index, value) {
                        this.addDropdownElement(index, this.sandbox.translate(value));
                    }.bind(this));
                } else if (typeof(items[0]) === 'object') {
                    this.sandbox.util.each(items, function(index, value) {
                        if (value.divider === true) {
                            this.addDivider();
                        } else {
                            this.addDropdownElement(value.id, this.sandbox.translate(value[this.options.valueName]), !!value.disabled, value.callback, this.sandbox.translate(value.updateLabel));
                        }

                    }.bind(this));
                }
                this.changeLabel();
                this.updateSelectionAttribute();
            }
        },

        // bind dom elements
        bindDOMEvents: function() {

            // toggle drop-down
            this.sandbox.dom.on(this.$el, 'click', this.labelClickHandler.bind(this), '.dropdown-label');

            // click on single item
            this.sandbox.dom.on(this.$el, 'click', function(event) {
                this.sandbox.dom.stopPropagation(event);
                if (this.sandbox.dom.hasClass(event.currentTarget, 'disabled') === false) {
                    this.clickItem(event);
                    return true;
                } else {
                    this.sandbox.dom.preventDefault(event);
                    return false;
                }
            }.bind(this), '.husky-select-dropdown-container li');

            this.sandbox.dom.on(this.$el, EVENT_DATA_CHANGED.call(this), this.dataChanged.bind(this));

            if (this.options.isNative) {
                this.sandbox.dom.on(this.$list, 'change', this.onSelectChange.bind(this));
            }
        },

        bindCustomEvents: function() {
            this.sandbox.on(EVENT_TOGGLE.call(this), this.toggleDropDown.bind(this));
            this.sandbox.on(EVENT_SHOW.call(this), this.showDropDown.bind(this));
            this.sandbox.on(EVENT_HIDE.call(this), this.hideDropDown.bind(this));
            this.sandbox.on(EVENT_DISABLE.call(this), this.disable.bind(this));
            this.sandbox.on(EVENT_ENABLE.call(this), this.enable.bind(this));
            this.sandbox.on(EVENT_GET_CHECKED.call(this), this.getChecked.bind(this));
            this.sandbox.on(EVENT_UPDATE.call(this), this.updateDropdown.bind(this));
            this.sandbox.on(EVENT_REVERT.call(this), this.revert.bind(this));
        },

        onSelectChange: function(event) {
            var selectedId = this.sandbox.dom.val(event.currentTarget),
                $selectedOption = this.sandbox.dom.find('option[value="' + selectedId + '"]', this.$list),
                callback = this.sandbox.dom.data($selectedOption, 'selectCallback'),
                selectedValue = this.sandbox.dom.text($selectedOption);

            this.selectedElements = [selectedId];
            this.selectedElementsValues = [selectedValue];

            if (selectedId === constants.deselectFieldKey) {
                this.selectedElements = [];
                this.selectedElementsValues = [];
            }

            // update data attribute
            this.updateSelectionAttribute();

            // change label
            this.changeLabel();

            if (!this.selectedElements.length) {
                this.triggerDeselect(selectedId);
            } else {
                this.triggerSelect(selectedId, selectedValue);
            }
        },

        /**
         * Should be triggered when data attributes have changed
         */
        dataChanged: function() {
            var selectedIds = this.sandbox.dom.data(this.$el, 'selection'),
                selectedValues = this.sandbox.dom.data(this.$el, 'selectionValues'),
                id, $checkbox;

            this.selectedElements = [];
            this.selectedElementsValues = [];

            if (typeof selectedIds === 'string') {
                this.selectedElements = selectedIds.split(',');
            } else if (Array.isArray(selectedIds)) {
                this.selectedElements = selectedIds.map(String);
            } else if (typeof selectedIds === 'number') {
                this.selectedElements.push(selectedIds.toString());
            }

            if (typeof selectedValues === 'string') {
                this.selectedElementsValues = selectedValues.split(',');
            } else if (!!Array.isArray(selectedValues)) {
                this.selectedElementsValues = selectedValues;
            }

            this.sandbox.util.foreach(this.sandbox.dom.children(this.$list), function($el) {
                id = this.sandbox.dom.data($el, 'id') || '';
                $checkbox = this.sandbox.dom.find('input[type=checkbox]', $el);
                if (this.selectedElements.indexOf(id) > -1) {
                    this.sandbox.dom.addClass($checkbox, 'is-selected');
                    this.sandbox.dom.prop($checkbox, 'checked', true);
                } else {
                    this.sandbox.dom.removeClass($checkbox, 'is-selected');
                    this.sandbox.dom.prop($checkbox, 'checked', false);
                }

            }.bind(this));

            this.changeLabel();
            this.updateSelectionAttribute();
        },

        /**
         * Reverts the dropdown list
         */
        revert: function() {
            this.updateDropdown(
                this.mergedData,
                this.selectedElements,
                false
            );
        },

        /**
         * Updates the dropdown list
         * @param data
         * @param preselected array of ids
         * @param merge - merge given data with dom
         */
        updateDropdown: function(data, preselected, merge) {
            if (!!merge) {
                data = this.mergeDomAndRequestData(
                    data, this.parseDataFromDom(this.domData, true));
                this.mergedData = data.slice(0);
            } else {
                // empty selection
                this.selectedElements = [];
                this.selectedElementsValues = [];
                this.updateSelectionAttribute();
                this.dataChanged();
            }
            if (preselected !== null) {
                this.options.preSelectedElements = preselected.map(String);
            }

            this.selectedElements = [];
            this.selectedElementsValues = [];

            this.sandbox.dom.empty(this.$list);

            this.options.data = data;

            if (!!data && data.length > 0) {
                showCaret.call(this);
                this.generateDropDown(data);
            } else {
                hideCaret.call(this);
                this.sandbox.logger.warn('error invalid data for update!');
            }
            this.addEditEntry();
        },

        updateSelectionAttribute: function() {
            this.sandbox.dom.data(this.$el, 'selection', this.selectedElements);
            this.sandbox.dom.data(this.$el, 'selectionValues', this.selectedElementsValues);
        },

        //unchecks all checked elements
        uncheckAll: function(clickedElementKey) {
            var elements = this.sandbox.dom.children(this.$list),
                i = -1,
                length = elements.length,
                key, index, $checkbox;

            for (; ++i < length;) {
                key = this.sandbox.dom.data(elements[i], 'id');

                if (key === undefined) {
                    key = '';
                }

                $checkbox = this.sandbox.dom.find('input[type=checkbox]', elements[i])[0];
                index = this.selectedElements.indexOf(key.toString());

                if (index >= 0) {
                    if (clickedElementKey !== key) {
                        this.sandbox.dom.removeClass($checkbox, 'is-selected');
                        this.sandbox.dom.prop($checkbox, 'checked', false);
                        this.selectedElements.splice(index, 1);
                        this.selectedElementsValues.splice(index, 1);
                        this.sandbox.emit(EVENT_DESELECTED_ITEM.call(this), key);
                    }
                }
            }
        },

        /**
         * Opens the overlay for edditing.
         */
        openEditDialog: function() {
            this.elementsToRemove = [];
            this.$elementsToRemove = [];

            var $element = this.sandbox.dom.createElement('<div/>'),
                $content = this.renderOverlayContent();
            this.sandbox.dom.append('body', $element);

            this.bindOverlayContentEvents($content);
            this.sandbox.start([
                { name: 'overlay@husky',
                    options: {
                        el: $element,
                        openOnStart: true,
                        removeOnClose: true,
                        instanceName: 'husky-select',
                        title: this.sandbox.translate('public.edit-entries'),
                        cancelCallback: function() {
                            this.onCloseWithCancel();
                        }.bind(this),
                        okCallback: function(data) {
                            this.onCloseWithOk(data);
                        }.bind(this),
                        data: $content
                    }
                }
            ]);
        },

        /**
         * Callback for close of overlay with ok button
         */
        onCloseWithOk: function(domData) {
            this.domData = domData;
            this.removeDeletedItems();
            if (!!domData) {
                this.saveNewEditedItemsAndClose(domData, 'PATCH');
            }
        },

        saveNewEditedItemsAndClose: function(domData) {
            var data = this.parseDataFromDom(domData),
                mergeData,
                changedData = this.getChangedData(data);

            if (changedData.length > 0) {
                mergeData = this.mergeDomAndRequestData(changedData,
                    this.parseDataFromDom(domData, true));
                this.options.data = mergeData.slice(0);
                this.updateDropdown(mergeData, this.selectedElements);
                this.sandbox.emit(EVENT_SAVE.call(this), changedData);
            }
        },

        /**
         * Merges data
         * @param updatedData
         * @param parsedDomData
         */
        mergeDomAndRequestData: function(updatedData, parsedDomData) {
            this.sandbox.util.foreach(parsedDomData, function(parsedEl) {
                this.sandbox.util.foreach(updatedData, function(updatedEl) {
                    if (parsedEl.id === updatedEl.id && parsedEl.id !== '') {
                        parsedEl[this.options.valueName] = updatedEl[this.options.valueName];
                    } else if (parsedEl[this.options.valueName] === updatedEl[this.options.valueName]) {
                        parsedEl.id = updatedEl.id;
                    }
                }.bind(this));
            }.bind(this));

            return parsedDomData;
        },

        /**
         * Bind overlay specific events
         * @param $element
         */
        bindOverlayContentEvents: function($element) {
            this.sandbox.dom.on($element,
                'click',
                this.markElementForRemoval.bind(this),
                constants.templateRemoveSelector);
            this.sandbox.dom.on($element,
                'click',
                this.addElement.bind(this),
                constants.templateAddSelector);
        },

        /**
         * Adds new item to the list or removes existing
         * @param id of element to remove
         * @param $row dom row of elment to remove
         */
        updateRemoveList: function(id, $row) {
            if (this.elementsToRemove.indexOf(id) === -1) {
                if (id !== null) {
                    this.elementsToRemove.push(id);
                }
                this.$elementsToRemove.push($row);
            } else {
                if (id !== null) {
                    this.elementsToRemove.splice(this.elementsToRemove.indexOf(id), 1);
                }
                this.$elementsToRemove.splice(this.elementsToRemove.indexOf($row), 1);
            }
        },

        /**
         * delete elements
         * @param id
         */
        deleteItem: function(id) {
            // Remove items
            var origData = this.options.data.slice(0);
            this.sandbox.util.each(origData, function(index, $el) {
                if (id === $el.id) {
                    var i = this.options.data.indexOf($el);
                    this.options.data.splice(i, 1);
                }
            }.bind(this));
            this.updateDropdown(
                this.options.data,
                this.selectedElements
            );
        },

        /**
         * Deletes removed items
         */
        removeDeletedItems: function() {
            // remove dom elements - important for the getChangedData method
            if (!!this.$elementsToRemove && this.$elementsToRemove.length > 0) {
                this.sandbox.util.each(this.$elementsToRemove, function(index, $el) {
                    this.sandbox.dom.remove($el);
                }.bind(this));
            }

            // remove items from db
            if (!!this.elementsToRemove && this.elementsToRemove.length > 0) {
                this.sandbox.util.each(this.elementsToRemove, function(index, el) {
                    this.deleteItem(el);
                }.bind(this));
                this.sandbox.emit(EVENT_DELETE.call(this), this.elementsToRemove);
                this.elementsToRemove = [];
                this.$elementsToRemove = [];
            }
        },

        /**
         * Extracts data from dom structure
         * @param domData
         * @param excludeDeleted
         */
        parseDataFromDom: function(domData, excludeDeleted) {
            var $rows = this.sandbox.dom.find(constants.typeRowSelector, domData),
                data = [],
                id, value, deleted, obj;

            this.sandbox.dom.each($rows, function(index, $el) {

                deleted = this.sandbox.dom.hasClass($el, 'faded');

                if (!!excludeDeleted) {
                    if (!deleted) {
                        id = this.sandbox.dom.data($el, 'id');
                        value = this.sandbox.dom.val(this.sandbox.dom.find('input', $el));
                        if (value !== '') {
                            obj = {id: id};
                            obj[this.options.valueName] = value;
                            data.push(obj);
                        }
                    }
                } else {
                    id = this.sandbox.dom.data($el, 'id');
                    value = this.sandbox.dom.val(this.sandbox.dom.find('input', $el));
                    if (value !== '') {
                        obj = {id: id};
                        obj[this.options.valueName] = value;
                        data.push(obj);
                    }
                }

            }.bind(this));

            return data;
        },

        /**
         * Compares original and new data
         */
        getChangedData: function(newData) {
            var changedData = [];
            this.sandbox.util.each(newData, function(idx, el) {
                if (el.id === '') {
                    changedData.push(el);
                } else {
                    this.sandbox.util.each(this.options.data, function(idx, origEl) {
                        if (el.id === origEl.id &&
                            el[this.options.valueName] !== origEl[this.options.valueName] &&
                            el[this.options.valueName] !== '') {
                            changedData.push(el);
                        }
                    }.bind(this));
                }
            }.bind(this));

            return changedData;
        },

        /**
         * Sets tooltip (the title attribute of container)
         * if options.showToolTip is true and multipleSelect is false
         *
         * @param string value
         */
        setToolTip: function(value) {
            if (this.options.showToolTip === true &&
                this.options.multipleSelect === false
            ) {
                var $container = this.$find(constants.selectContainerSelector);
                this.sandbox.dom.attr($container, 'title', value);
            }
        },

        /**
         * Callback for close of overlay with cancel button
         */
        onCloseWithCancel: function() {
            this.elementsToRemove = [];
            this.$elementsToRemove = [];
        },

        /**
         * Render content for the overlay
         */
        renderOverlayContent: function() {
            return this.sandbox.dom.createElement(this.sandbox.util.template(
                templates.addOverlaySkeleton.call(this),
                {
                    data: this.options.data,
                    rowTemplate: templates.addOverlayRow,
                    valueName: this.options.valueName
                }));
        },
        /**
         * Adds an element
         */
        addElement: function() {
            var $row = this.sandbox.dom.createElement(
                templates.addOverlayRow.call(
                    this,
                    this.options.valueName, {}
                ));
            this.sandbox.dom.append(
                this.sandbox.dom.find(
                    constants.contentInnerSelector),
                $row);

            this.sandbox.dom.focus(this.sandbox.dom.find(constants.formElementSelector, $row));
        },

        /**
         * Marks an element for removal
         */
        markElementForRemoval: function(event) {
            var $row = this.sandbox.dom.parent(this.sandbox.dom.parent(event.currentTarget)),
                id = this.sandbox.dom.data($row, 'id');

            if (id !== null) {
                this.updateRemoveList(id, $row);
            }

            this.toggleStateOfRow($row);
        },

        /**
         * Marks a row as removed
         * @param $row
         */
        toggleStateOfRow: function($row) {
            this.sandbox.dom.toggleClass($row, 'faded');
        },

        // trigger event with clicked item
        clickItem: function(event) {
            var key, value, $checkbox, index, callback, updateLabel;

            key = this.sandbox.dom.data(event.currentTarget, 'id').toString();
            callback = this.sandbox.dom.data(event.currentTarget, 'selectCallback');
            index = this.selectedElements.indexOf(key);
            updateLabel = this.sandbox.dom.attr(event.currentTarget, 'data-update-label');

            // Edit button was pressed
            if (key === constants.editableFieldKey) {
                this.hideDropDown();
                this.openEditDialog();
                return;
            }

            if (updateLabel !== 'false') {
                // if single select then uncheck all results
                if (this.options.multipleSelect === false) {
                    // if deselect was selected
                    if (key === constants.deselectFieldKey) {
                        index = 0;
                        this.uncheckAll(key);
                        // if new element was selected
                    } else if (index === -1) {
                        this.uncheckAll(key);
                        // same element was selected
                    } else if (this.options.repeatSelect !== true) {
                        this.hideDropDown();
                        return;
                    }
                }

            }

            value = this.sandbox.dom.text(this.sandbox.dom.find('.item-value', event.currentTarget));
            $checkbox = this.sandbox.dom.find('input[type=checkbox]', event.currentTarget)[0];

            if (!!updateLabel && updateLabel !== 'false') {
                // deselect
                if (index >= 0) {
                    this.sandbox.dom.removeClass($checkbox, 'is-selected');
                    this.sandbox.dom.prop($checkbox, 'checked', false);
                    this.selectedElements.splice(index, 1);
                    this.selectedElementsValues.splice(index, 1);

                    // select
                } else {
                    this.sandbox.dom.addClass($checkbox, 'is-selected');
                    this.sandbox.dom.prop($checkbox, 'checked', true);
                    this.selectedElements.push(key.toString());
                    this.selectedElementsValues.push(value);
                }

                // update data attribute
                this.updateSelectionAttribute();

                // change label
                this.changeLabel();
            }

            // hide if single select
            if (this.options.multipleSelect === false) {
                this.hideDropDown();
            }

            if (this.options.emitValues === true) {
                key = value;
            }

            // execute callback if configured
            if (typeof callback === 'function') {
                callback();
            }
            if (index >= 0) {
                this.triggerDeselect(key);
            } else {
                this.triggerSelect(key, value);
            }
            this.sandbox.dom.trigger(this.$el, 'change');
        },

        // triggers select callback or emits event
        triggerSelect: function(key, selectedValue) {
            this.setToolTip(selectedValue);

            // callback, if defined
            if (!!this.options.selectCallback) {
                this.options.selectCallback.call(this, key, selectedValue);
            } else {
                this.sandbox.emit(EVENT_SELECTED_ITEM.call(this), key, selectedValue);
            }
        },

        // triggers select callback or emits event
        triggerPreSelect: function(key) {
            // callback, if defined
            if (!!this.options.preselectCallback) {
                this.options.preselectCallback.call(this, key);
            } else if (!!this.options.selectCallback) {
                this.options.selectCallback.call(this, key);
            } else {
                this.sandbox.emit(EVENT_PRESELECTED_ITEM.call(this), key);
            }
        },

        // triggers deselect callback or emits event
        triggerDeselect: function(key) {
            // callback if defined
            if (!!this.options.deselectCallback) {
                this.options.deselectCallback.call(this, key !== constants.deselectFieldKey ? key : null);
            } else {
                this.sandbox.emit(EVENT_DESELECTED_ITEM.call(this), key);
            }
        },

        changeLabel: function() {
            if (this.options.fixedLabel !== true) {
                if (this.selectedElements.length === this.options.data.length && this.options.multipleSelect === true) {
                    this.sandbox.dom.text(this.$label, this.options.checkedAllLabel);
                } else if (this.selectedElements.length === 0) {
                    this.sandbox.dom.text(this.$label, this.options.defaultLabel);
                } else {

                    var text = "", labelWidth;
                    this.sandbox.util.each(this.selectedElementsValues, function(index, value) {
                        text += ' ' + value + ',';
                    });
                    this.sandbox.dom.text(this.$label, text.substring(1, text.length - 1));
                }
            }
        },

        /**
         * Handles the click on the dropdown label
         */
        labelClickHandler: function() {
            // if dropdown is not empty
            if (!this.sandbox.dom.is(this.$find('.' + constants.listClass), ':empty')) {
                this.toggleDropDown();
            } else {
                // else execute a no-items-callback if it exists
                if (typeof this.options.noItemsCallback === 'function') {
                    this.options.noItemsCallback();
                }
            }
        },

        // toggle dropDown visible
        toggleDropDown: function() {
            this.sandbox.logger.log('toggle dropdown ' + this.options.instanceName);

            if (this.dropdownVisible === true) {
                this.hideDropDown();
            } else {
                this.showDropDown();
            }
        },

        // make dropDown visible
        showDropDown: function() {
            if (this.options.isNative) {
                return;
            }

            this.sandbox.dom.removeClass(this.$dropdownContainer, 'hidden');
            this.sandbox.dom.on(this.sandbox.dom.window, 'click.dropdown.' + this.options.instanceName, this.hideDropDown.bind(this));
            this.dropdownVisible = true;

            this.sandbox.dom.removeClass(this.$dropdownContainer, constants.dropdownTopClass);
            var dropdownHeight = this.sandbox.dom.height(this.$dropdownContainer),
                dropdownTop = this.sandbox.dom.offset(this.$dropdownContainer).top,
                windowHeight = this.sandbox.dom.height(this.sandbox.dom.window),
                scrollTop = this.sandbox.dom.scrollTop(this.sandbox.dom.window),
                dropdownMaxHeight = windowHeight - dropdownTop;

            if (this.options.direction === 'top') {
                this.sandbox.dom.addClass(this.$dropdownContainer, constants.dropdownTopClass);
            } else if (
                this.options.direction !== 'bottom'
                && dropdownHeight + dropdownTop > windowHeight + scrollTop
                // only change direction if there is more space on the other side
                && windowHeight / 2 < dropdownTop - scrollTop
            ) {
                this.sandbox.dom.addClass(this.$dropdownContainer, constants.dropdownTopClass);
                dropdownMaxHeight = dropdownHeight + this.$dropdownContainer.offset().top;
            }

            this.$dropdownContainer.css({maxHeight: dropdownMaxHeight - this.options.dropdownPadding + 'px'});
        },

        // hide dropDown
        hideDropDown: function(event) {
            if (!!event) {
                if (!!this.sandbox.dom.find(event.target, this.$el).length) {
                    return false;
                } else {
                    this.sandbox.dom.off(this.sandbox.dom.window, 'click.dropdown.' + this.options.instanceName);
                }
            }
            this.sandbox.dom.addClass(this.$dropdownContainer, 'hidden');
            this.dropdownVisible = false;
            return true;
        },

        // return checked values
        getChecked: function(callback) {
            if (typeof callback === 'function') {
                callback(this.selectedElements);
            } else {
                throw 'error: callback is not a function';
            }
        },

        template: {
            basicStructure: function(defaultLabel, icon) {
                var iconSpan = '', dropdownStyle = 'display:none';
                if (!!icon) {
                    iconSpan = '<span class="fa-' + icon + ' icon"></span>';
                }

                if (!!this.options.data && !!this.options.data.length || this.options.editable) {
                    dropdownStyle = '';
                }
                return [
                    '<div class="husky-select-container">',
                    '   <div class="dropdown-label pointer">',
                    '       <% if (isNative) { %>',
                    '           <select class="' + constants.listClass + '">',
                    (
                        this.options.preSelectedElements.length === 0 ?
                        '           <option selected="true" disabled="true">' + defaultLabel + '</option>' : ''
                    ),
                    '           </select>',
                    '       <% } %>',
                    '       <div class="checkbox">',
                                iconSpan,
                    '           <span class="' + constants.labelClass + '">', defaultLabel, '</span>',
                    '       </div>',
                    '       <span class="fa-caret-down toggle-icon" style="' + dropdownStyle + '"></span>',
                    '   </div>',
                    '   <% if (!isNative) { %>',
                    '   <div class="grid-row dropdown-list dropdown-shadow hidden ' + constants.dropdownContainerClass + '">',
                    '       <ul class="' + constants.listClass + '"></ul>',
                    '   </div>',
                    '   <% } %>',
                    '</div>'
                ].join('');
            },
            menuElement: function(index, value, checked, updateLabel, checkboxVisible) {
                var hiddenClass = '',
                    update = 'true';

                if (this.options.multipleSelect === false || !checkboxVisible) {
                    hiddenClass = ' hidden';
                }

                if (updateLabel !== undefined && updateLabel === false) {
                    update = 'false';
                }

                return [
                    '<li data-id="', index, '" data-update-label="', update, '">',
                    '    <div>',
                        '        <div class="check' + hiddenClass + '">',
                    '           <div class="custom-checkbox no-spacing">',
                    '               <input type="checkbox" class="form-element"', checked, '/>',
                    '               <span class="icon"></span>',
                    '           </div>',
                    '        </div>',
                    '        <div class="item-value">', value, '</div>',
                    '    </div>',
                    '</li>'
                ].join('');
            },
            optionElement: function(index, value, checked, updateLabel, checkboxVisible) {
                return [
                    '<option <% if (checked) { print("selected "); } %>value="' + index + '">',
                        value,
                    '</option>'
                ].join('');
            }
        }
    };
});

/*
 * This file is part of the Sulu CMS.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * Name: button
 * Options:
 *  labels.inputPassword1: text for the label for the first password field
 *  labels.inputPassword2: text for the label for the second password field
 *  labels.generateLabel: text for the generate label
 *  instanceName: name of the instance of this component
 *
 * Provided Events:
 *  husky.passwords.fields.<<instanceName>>.generated.passwords: password generated
 *
 * Used Events
 *  husky.passwords.fields.<<instanceName>>.get.passwords: callback for passwords
 *
 */

define('__component__$password-fields@husky',[], function() {

    'use strict';

    var defaults = {
        instanceName: 'undefined',
        validation: false,
        labels: {
            inputPassword1: 'Password',
            inputPassword2: 'Repeat Password',
            generateLabel: 'Generate password'
        },
        values: {
            inputPassword1: '',
            inputPassword2: ''
        }

    };

    return {

        initialize: function() {
            this.sandbox.logger.log('initialize');
            this.sandbox.logger.log(arguments);

            // extend default options
            this.options = this.sandbox.util.extend({}, defaults, this.options);
            this.options.ids =
            {
                inputPassword1: 'husky-password-fields-' + this.options.instanceName + '-password1',
                inputPassword2: 'husky-password-fields-' + this.options.instanceName + '-password2',
                generateLabel: 'husky-password-fields-' + this.options.instanceName + '-generate'
            };
            this.render();
        },

        render: function() {

            this.$originalElement = this.sandbox.dom.$(this.options.el);
            this.$element = this.sandbox.dom.$(this.template());
            this.$originalElement.append(this.$element);

            if (!!this.options.validation) {
                this.sandbox.form.addField(this.options.validation, this.$el.find('#' + this.options.ids.inputPassword1));
                this.sandbox.form.addField(this.options.validation, this.$el.find('#' + this.options.ids.inputPassword2));
            }

            this.bindDomEvents();
            this.bindCustomEvents();
        },

        bindCustomEvents: function(){

            this.sandbox.on('husky.password.fields.' + this.options.instanceName + '.get.passwords', function(callback){

                if(typeof callback === 'function'){
                    var password1 = this.sandbox.dom.val('#' + this.options.ids.inputPassword1),
                        password2 = this.sandbox.dom.val('#' + this.options.ids.inputPassword2);
                    callback(password1, password2);
                }

            }.bind(this));

        },

        bindDomEvents: function() {

            this.$element.on('click', '#' + this.options.ids.generateLabel, function() {
                this.generatePasswords();
                this.sandbox.emit('husky.passwords.fields.' + this.options.instanceName + '.generated.passwords');
            }.bind(this));
        },

        generatePasswords: function() {
            var generatedPassword = 'test';

            // TODO generate password

            this.sandbox.dom.val('#' + this.options.ids.inputPassword1, generatedPassword);
            this.sandbox.dom.val('#' + this.options.ids.inputPassword2, generatedPassword);
        },

        template: function() {

            return [
                '<div class="husky-password-fields grid">',
                '    <div class="grid-row">',
                '        <div class="grid-col-6 form-group">',
                '                <label for="',this.options.ids.inputPassword1,'">', this.options.labels.inputPassword1, '</label>',
                '                <div class="align-right hidden" id="', this.options.ids.generateLabel, '">',
                '                    <span class="fa-key m-right-10"></span><span class="pointer">', this.options.labels.generateLabel, '</span>',
                '                </div>',
                '                <input class="form-element" value="', this.options.values.inputPassword1, '" type="text" id="', this.options.ids.inputPassword1, '"', (!!this.options.validation ? 'data-validation-equal="' + this.options.instanceName + '"' : ''), '/>',
                '        </div>',
                '        <div class="grid-col-6 form-group">',
                '                <label for="',this.options.ids.inputPassword2,'">', this.options.labels.inputPassword2, '</label>',
                '                <input class="form-element" value="', this.options.values.inputPassword2, '" type="text" id="', this.options.ids.inputPassword2, '"', (!!this.options.validation ? 'data-validation-equal="' + this.options.instanceName + '"' : ''), '/>',
                '        </div>',
                '    </div>',
                '</div>'
            ].join('');
        }
    };
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/column-navigation
 */

/**
 * @class ColumnNavigation
 * @constructor
 *
 * @params {Object} [options] Configuration object
 * @params {String} [options.url] url to load data.
 * @params {String} [options.fallbackUrl] url to load data from, when loading from the main url fails.
 * @params {String} [options.selected] id of selected element - needed to restore state
 * @params {String|Function} [options.actionIcon] icon class of action button
 * @params {String|Function} [options.unmarkIcon] icon class of action button to unmark
 * @params {String} [options.actionButtonOnGhostText] Text for the action button
 * @params {Boolean} [options.showActionButtonOnGhost] if set true create button for ghost page is enabled
 * @params {Array}  [options.data] array of data displayed in the settings dropdown
 * @params {String}  [options.data[].mode] if 'order' - column gets set in order mode if clicked
 * @params {Function}  [options.data[].enabler] Gets called each time the options change columns.
 *          Gets an object with a numberItems and a hasSelected property.
 *          If returns false the dropdown item will get disabled, enabled otherwise
 * @params {String} [options.instanceName] name of current instance
 * @params {String} [options.hasSubName] name of hasSub-key
 * @params {String} [options.idName] name of id-key
 * @params {String} [options.pathName] name of path-key
 * @params {String} [options.linkedName] name of linked-key
 * @params {String} [options.publishedStateName] name of draft-key
 * @params {String} [options.publishedName] name of published-key
 * @params {String} [options.titleName] name of title-key
 * @params {String} [options.noPageDescription] translation key for the "No-Page"-description
 * @params {String} [options.resultKey] The name of the array in the responded _embedded
 * @params {Boolean} [options.responsive] If true the resize listener gets initialized. Otherwise the column navigation just takes up 100 % of the height and width
 * @params {Boolean} [options.showOptions] hide or shows the options
 * @params {Boolean} [options.showActionIcon] if true the action icon is shown, not shown otherwise
 * @params {Boolean} [options.showStatus] hide or display status of elements
 * @params {String} [options.skin] css class which gets added to the components element. Available: '', 'fixed-height-small'
 * @params {Boolean} [options.markable] If true a node gets marked with a css class on click on the blue button
<<<<<<< 76cfc9f9d9eb8f8abf34f388ef807f5629319370
 * @params {Boolean} [options.singleMarkable] If true just one item is markable
=======
 * @params {Boolean} [options.actionOnGhostPage] If true action Button on ghost page will be shown
>>>>>>> Added actionOnGhostPage option in column navigation
 * @params {Array} [options.premarkedIds] an array of uuids of nodes which should be marked from the beginning on
 * @params {Array} [options.disableIds] an array of uuids which will be disabled
 * @params {Array} [options.disabledChildren] an array of uuids which will be disabled
 * @params {Boolean} [options.orderable] if true component-items can be ordered
 * @params {Boolean} [options.tooltipTranslations] translation-keys for the tooltips
 * @params {Boolean} [options.tooltipTranslations.ghost] translation-keys for ghost
 * @params {Boolean} [options.tooltipTranslations.shadow] translation-keys for shadow
 * @params {Boolean} [options.tooltipTranslations.unpublished] translation-keys for unpublished
 * @params {Boolean} [options.tooltipTranslations.internalLink] translation-keys for internal-link
 * @params {Boolean} [options.tooltipTranslations.externalLink] translation-keys for external-link
 * @params {Callback} [options.actionCallback] callback which will be called on action
 */
define('__component__$column-navigation@husky',[],function() {

    'use strict';

    var defaults = {
            url: null,
            fallbackUrl: null,
            prefilledData: null,
            selected: null,
            data: null,
            instanceName: '',
            hasSubName: 'hasSub',
            actionIcon: 'fa-pencil',
            unmarkIcon: 'fa-minus-circle',
            actionButtonOnGhostText: 'create',
            showActionButtonOnGhost: false,
            addButton: true,
            idName: 'id',
            pathName: 'path',
            linkedName: 'linked',
            publishedStateName: 'publishedState',
            publishedName: 'published',
            titleName: 'title',
            typeName: 'type',
            noPageDescription: 'public.no-pages',
            skin: '',
            resultKey: 'nodes',
            responsive: true,
            showOptions: true,
            showStatus: true,
            actionOnGhost: false,
            premarkedIds: [],
            disableIds: [],
            disabledChildren: false,
            markable: false,
            singleMarkable: false,
            orderable: true,
            showActionIcon: true,
            orderConfirmTitle: 'column-navigation.order-title',
            orderConfirmMessage: 'column-navigation.order-message',
            tooltipTranslations: {
                ghost: 'column-navigation.ghost',
                shadow: 'column-navigation.shadow',
                unpublished: 'public.unpublished',
                publishedWithDraft: 'public.published-with-draft',
                internalLink: 'public.internal-link',
                externalLink: 'public.external-link'
            },
            actionCallback: null
        },

        constants = {
            responsiveHeightRation: 7 / 10,
            columnWidth: 250,
            minVisibleRatio: 1 / 2,
            markedClass: 'marked',
            displayedcolumns: 2, // number of displayed columns with content
            wrapperClass: 'column-navigation-wrapper',
            disabledClass: 'disabled',
            columnClass: 'column',
            optionsClass: 'options',
            componentClass: 'husky-column-navigation',
            nodeLoaderClass: 'husky-column-navigation-loader',
            bigLoaderClass: 'column-navigation-loader',
            columnItemClass: 'column-item',
            iconsLeftClass: 'icons-left',
            iconsRightClass: 'icons-right',
            itemTextClass: 'item-text',
            orderInputClass: 'orderer',
            orderErrorClass: 'husky-validate-error',
            highlightClass: 'highlight-animation',
            orderModeClass: 'order-mode'
        },

        templates = {
            wrapper: ['<div class="', constants.wrapperClass, '"></div>'].join(''),
            container: ['<div class="column-navigation"></div>'].join(''),

            column: [
                '<div data-column="<%= columnNumber %>" class="', constants.columnClass, '" id="<%= id %>">',
                '    <ul></ul>',
                '</div>'
            ].join(''),

            noPage: [
                '<div class="no-page">',
                '    <span class="fa-coffee icon"></span>',
                '    <div class="text"><%= description %></div>',
                '</div>'
            ].join(''),

            optionsContainer: ['<div class="', constants.optionsClass, ' grid-row"></div>'].join(''),

            optionsAdd: [
                '<div class="align-center add pointer">',
                '    <span class="fa-plus-circle"></span>',
                '</div>'
            ].join(''),

            optionsSettings: [
                '<div class="align-center settings pointer drop-down-trigger">',
                '    <span class="fa-gear inline-block"></span><span class="dropdown-toggle inline-block"></span>',
                '</div>'
            ].join(''),

            optionsOk: [
                '<div class="align-center ok pointer">',
                '    <span class="fa-check"></span>',
                '</div>'
            ].join(''),

            item: [
                '<li data-id="<%= id %>" class="' + constants.columnItemClass + '">',
                '    <span class="' + constants.iconsLeftClass + '"></span>',
                '    <span title="<%= title %>" class="' + constants.itemTextClass + '"><%= title %></span>',
                '    <span class="' + constants.iconsRightClass + '"></span>',
                '</li>'
            ].join(''),

            orderInput: [
                '<span class="' + constants.orderInputClass + '">',
                '    <input type="text" class="form-element husky-validate" value="<%= value %>" />',
                '</span>'
            ].join('')
        },

        /**
         * namespace for events
         * @type {string}
         */
        eventNamespace = 'husky.column-navigation.',

        /**
         * @event husky.column-navigation.initialized
         * @description thrown after initialization has finished
         */
        INITIALIZED = function() {
            return createEventName.call(this, 'initialized');
        },

        /**
         * @event husky.column-navigation.loaded
         * @description the component has loaded everything successfully and will be rendered
         */
        LOADED = function() {
            return createEventName.call(this, 'loaded');
        },

        /**
         * @event husky.column-navigation.selected
         * @description an navigation element has been selected
         * @param {Object} selected object
         */
        SELECTED = function() {
            return createEventName.call(this, 'selected');
        },

        /**
         * @event husky.column-navigation.settings
         * @description an navigation element has been selected and a item from selected dropdown clicked
         * @param {Object} selected column navigation object
         * @param {Object} clicked dropdown item
         */
        SETTINGS = function() {
            return createEventName.call(this, 'settings');
        },

        /**
         * @event husky.column-navigation.add
         * @description the add button has been clicked
         * @param {Object} parent object from active column level
         */
        ADD = function() {
            return createEventName.call(this, 'add');
        },

        /**
         * @event husky.column-navigation.action
         * @description emitted if the action-icon of an item gets clicked
         * @param {Object} clicked item
         */
        ACTION = function() {
            return createEventName.call(this, 'action');
        },

        /**
         * @event husky.column-navigation.ordered
         * @description emited when the position of an item gets changed
         * @param {String} uuid of the reposition item
         * @param {Number} the new position of the item
         */
        ORDERED = function() {
            return createEventName.call(this, 'ordered');
        },

        /**
         * @event husky.column-navigation.order
         * @description listens on and sets the column containing an item with a given id in order-mode
         * @param {String} uuid of the item
         */
        ORDER = function() {
            return createEventName.call(this, 'order');
        },

        /**
         * @event husky.column-navigation.order-start
         * @description emited if a column is set in order-mode
         */
        ORDER_START = function() {
            return createEventName.call(this, 'order-start');
        },

        /**
         * @event husky.column-navigation.order-start
         * @description emited if a column is set in order-mode
         */
        ORDER_END = function() {
            return createEventName.call(this, 'order-end');
        },

        /**
         * @event husky.column-navigation.unmark
         * @description listens on and unmarks a node with a given id
         * @param {Number|String} the id of the node to unmark
         */
        UNMARK = function() {
            return createEventName.call(this, 'unmark');
        },

        /**
         * @event husky.column-navigation.get-marked
         * @description listens on and passes all the marked objects to a callback
         * @param {Function} callback to which the marked items get passed
         */
        GET_MARKED = function() {
            return createEventName.call(this, 'get-marked');
        },

        /**
         * @event husky.column-navigation.highlight
         * @description listens on and highlights an item with a given uuid
         * @param {Number|String} the id of the item to highlight
         */
        HIGHLIGHT = function() {
            return createEventName.call(this, 'highlight');
        },

        /**
         * @event husky.column-navigation.get-breadcrumb
         * @description the breadcrumb will be returned
         * @param {Function} callback function which will process the breadcrumb objects
         */
        BREADCRUMB = function() {
            return createEventName.call(this, 'get-breadcrumb');
        },

        /**
         * @event husky.column-navigation.resize
         * @description the element will be resized
         * @param {Function} callback function which will process the breadcrumb objects
         */
        RESIZE = function() {
            return createEventName.call(this, 'resize');
        },

        /**
         * @event husky.column-navigation.set-options
         * @description the element will be rerendered with given options
         * @param {{url, selected}} options
         */
        SET_OPTIONS = function() {
            return createEventName.call(this, 'set-options');
        },

        createContext = function(column) {
            if (column <= 0) {
                return this.columns[column];
            }

            var context = {
                numberItems: 0,
                hasSelected: false,
                selectedItem: null,
                parent: null,
                children: []
            };

            if (!!this.columns[column]) {
                context.numberItems = Object.keys(this.columns[column]).length;
                context.hasSelected = !!this.selected[column];
                context.children = this.columns[column];

                if (!!this.selected[column]) {
                    context.selectedItem = this.selected[column];
                }
            }

            if (column > 0) {
                context.parent = createContext.call(this, column - 1);
            }

            return context;
        },

        /**
         * Returns the action items for the given item data
         * @param {Object} data
         * @returns {string}
         */
        getActionIcon = function(data) {
            // if is markable change action Icon
            if (this.options.markable && this.options.actionIcon === defaults.actionIcon) {
                if (this.options.singleMarkable) {
                    this.options.actionIcon  = 'fa-check';
                } else {
                    this.options.actionIcon  = 'fa-plus-circle';
                }
            }

            var actionItem = this.options.actionIcon;

            if (typeof(this.options.actionIcon) === 'function') {
                actionItem = this.options.actionIcon(data);
            }

            return actionItem;
        },

        /**
         * Returns whether the add button in the column should be shown or not (can also differ between columns)
         * @returns {boolean}
         */
        getAddButton = function() {
            if (typeof(this.options.addButton) === 'function') {
                return this.options.addButton(createContext.call(this, this.lastHoveredColumn));
            }

            return this.options.addButton;
        },

        /**
         * Sets the width for all visible option buttons
         */
        setOptionButtonWidths = function() {
            var $optionButtons = this.$find('.' + constants.optionsClass + ' > :visible'),
                width = 100 / $optionButtons.length;

            $optionButtons.each(function(count, optionButton) {
                $(optionButton).width(width + '%');
            });

            this.$find('.' + constants.optionsClass + ' > :visible').removeClass('first');
            this.$find('.' + constants.optionsClass + ' > :visible').removeClass('last');
            this.$find('.' + constants.optionsClass + ' > :visible:first').addClass('first');
            this.$find('.' + constants.optionsClass + ' > :visible:last').addClass('last');
        },

        /** returns normalized event names */
        createEventName = function(postFix) {
            return eventNamespace + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
        };

    return {

        initialize: function() {
            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

            this.setProperties();
            this.render();
            this.startBigLoader();
            this.initData();
            this.bindDOMEvents();
            this.bindCustomEvents();

            this.sandbox.emit(INITIALIZED.call(this));
        },

        /**
         * initialize data
         */
        initData: function() {
            if (!!this.options.prefilledData) {
                this.handleResponse(this.options.prefilledData, 0);
            } else {
                this.load(this.options.url, 0, function() {
                    // when an error occured try again with the fallback url
                    if (!!this.options.fallbackUrl) {
                        this.initiallySelected = null;
                        this.load(this.options.fallbackUrl, 0);
                    }
                }.bind(this));
            }
        },

        /**
         * Initializes the class properties with their default-values
         */
        setProperties: function() {
            this.initiallySelected = this.options.selected;
            this.$selectedElement = null;
            this.filledColumns = 0;
            this.columnLoadStarted = false;
            this.isOrdering = false; // flag (only one item at a time can be ordered)
            this.inOrderMode = false; // flag (only one column at a time can be in order-mode)
            this.optionsLocked = false;
            this.dom = {
                $wrapper: null,
                $container: null,
                $add: null,
                $settings: null,
                $bigLoader: null,
                $loader: null,
                $ok: null,
                $lastClicked: null
            };

            this.columns = [];
            this.selected = [];

            this.marked = {};
            if (this.options.premarkedIds) {
                this.options.premarkedIds.forEach(function (premarkedId) {
                    // write empty object as value because element not available yet
                    this.marked[premarkedId] = {};
                }.bind(this));
            }
        },

        /**
         * Renders basic structure (wrapper) of column navigation
         */
        render: function() {
            this.renderWrapper();
            if (!!this.options.showOptions) { // Options-toolbar with "add" and "settings"
                this.renderOptions();
            }
            if (!!this.options.data) {
                this.initSettingsDropdown();
            }
            if (this.options.responsive === true) {
                this.setContainerHeight();
            }
        },

        /**
         * Returns the index of a column which contains an item with a given id
         * @param item - the id of the item
         * @returns {number} the index of the column if found, a value below 0 otherwise
         */
        getColumnForItem: function(item) {
            for (var i = -1, length = this.columns.length; ++i < length;) {
                if (!!this.columns[i] && !!this.columns[i][item]) {
                    return i;
                }
            }
            return -1;
        },

        /**
         * Renders the options-buttons. "Add" and "Settings"
         */
        renderOptions: function() {
            this.$optionsContainer = this.sandbox.dom.createElement(templates.optionsContainer);
            this.dom.$add = this.sandbox.dom.createElement(templates.optionsAdd);
            this.dom.$settings = this.sandbox.dom.createElement(templates.optionsSettings);
            this.dom.$ok = this.sandbox.dom.createElement(templates.optionsOk);
            this.sandbox.dom.hide(this.dom.$ok);
            this.sandbox.dom.append(this.$optionsContainer, this.dom.$add);
            this.sandbox.dom.append(this.$optionsContainer, this.dom.$settings);
            this.sandbox.dom.append(this.$optionsContainer, this.dom.$ok);
            this.hideOptions();
            this.sandbox.dom.append(this.dom.$wrapper, this.$optionsContainer);
        },

        /**
         * Renders the component wrapper and an additional container
         */
        renderWrapper: function() {
            this.sandbox.dom.addClass(this.$el, constants.componentClass);
            if (!!this.options.skin) {
                this.sandbox.dom.addClass(this.$el, this.options.skin);
            }

            this.dom.$wrapper = this.sandbox.dom.createElement(templates.wrapper);
            this.sandbox.dom.append(this.$el, this.dom.$wrapper);

            // navigation container
            this.dom.$container = this.sandbox.dom.createElement(templates.container);
            this.sandbox.dom.append(this.dom.$wrapper, this.dom.$container);
        },

        /**
         * Starts the big loader, before loading content during the initialization
         */
        startBigLoader: function() {
            if (this.dom.$bigLoader === null) {
                this.dom.$bigLoader = this.sandbox.dom.createElement('<div class="' + constants.bigLoaderClass + '"/>');
                this.sandbox.dom.hide(this.dom.$bigLoader);
                this.sandbox.dom.html(this.dom.$container, this.dom.$bigLoader);

                this.sandbox.start([
                    {
                        name: 'loader@husky',
                        options: {
                            el: this.dom.$bigLoader,
                            size: '100px',
                            color: '#e4e4e4'
                        }
                    }
                ]);
            }
            this.sandbox.dom.show(this.dom.$bigLoader);
        },

        /**
         * Detatches the big loader from the column-navigation
         */
        removeBigLoader: function() {
            this.sandbox.dom.hide(this.dom.$bigLoader);
        },

        /**
         * Sets the height of the container
         */
        setContainerHeight: function() {
            var height = this.sandbox.dom.height(this.sandbox.dom.$window),
                top = this.sandbox.dom.offset(this.$el).top;
            this.sandbox.dom.height(this.dom.$container, (height - top) * constants.responsiveHeightRation);
        },

        /**
         * Instantiats the dropdown component
         */
        initSettingsDropdown: function() {
            this.sandbox.start([
                {
                    name: 'dropdown@husky',
                    options: {
                        el: this.dom.$settings,
                        setParentDropDown: true,
                        instanceName: this.options.instanceName + '.settings.dropdown',
                        alignment: 'left',
                        data: this.options.data
                    }
                }
            ]);
        },

        /**
         * Loads data from a specific url and triggers the parsing
         * @param {String} url
         * @param {Number} columnNumber
         * @param {Function} errorCallback
         */
        load: function(url, columnNumber, errorCallback) {
            if (!!url) {
                this.columnLoadStarted = true;
                $.ajax({
                    url: url,
                    global: false,

                    success: function(response) {
                        this.handleResponse(response, columnNumber);
                    }.bind(this),

                    error: function(jqXHR, error) {
                        this.columnLoadStarted = false;
                        this.sandbox.logger.error("An error occured while fetching data from: ", error);
                        if (typeof errorCallback === 'function') {
                            errorCallback();
                        }
                    }.bind(this)
                });
            } else {
                this.sandbox.logger.log("husky.column.navigation -  url not set, aborted loading of data");
            }
        },

        /**
         * Handle data response
         * @param response
         * @param columnNumber
         */
        handleResponse: function(response, columnNumber) {
            this.removeBigLoader();
            this.columnLoadStarted = false;
            this.parseData(response, columnNumber);
            this.scrollIfNeeded(this.filledColumns + 1);
            this.setOverflowClass();
            this.showOptionsAtLast();
            this.sandbox.emit(LOADED.call(this));
        },

        /**
         * Removes removes data and removes dom elements
         * @param {Number} newColumn
         */
        removeColumns: function(newColumn) {
            var length = this.filledColumns,
                i;

            for (i = length; i >= newColumn; i--) {
                delete this.columns[i];
                this.sandbox.dom.remove('#column' + this.options.instanceName + '-' + i);
                this.filledColumns--;
            }
        },

        /**
         * Parses the received data and renders columns
         * @param {String} data
         * @param {Number} columnNumber
         */
        parseData: function(data, columnNumber) {
            // if there is nothing loaded yet, create a "root"-column
            if (columnNumber === 0) {
                this.columns[0] = data;
            }
            var newColNumber = columnNumber + 1;

            this.renderColumn(newColNumber, data);
            this.removeLoadingIconForSelected();
        },

        /**
         * Renderes a column
         * @param number - the number of the new column
         * @param data - the data for the column
         */
        renderColumn: function(number, data) {
            var $column = this.sandbox.dom.createElement(this.sandbox.util.template(templates.column)({
                columnNumber: number,
                id: 'column' + this.options.instanceName + '-' + number
            }));
            this.sandbox.dom.append(this.dom.$container, $column);
            this.filledColumns = number;
            this.renderItems($column, number, data);
        },

        /**
         * Renderes the items for a column
         * @param $column - the dom-object of the column
         * @param number - the number of the column to add the items to
         * @param data - the columns data object
         */
        renderItems: function($column, number, data) {
            var $list = this.sandbox.dom.find('ul', $column), nodeWithSubNodes, lastSelected;
            this.sandbox.util.each(data._embedded[this.options.resultKey], function(index, itemData) {
                itemData.order = (index + 1);
                if (itemData[this.options.idName] in this.marked) {
                    // write missing values in this marked
                    this.marked[this.options.idName] = itemData;
                }
                var $item = this.renderItem(itemData);
                itemData.$el = $item;
                this.storeDataItem(number, itemData);

                this.sandbox.dom.append($list, $item);
                this.setItemsTextWidth($item);

                // if there are sub-nodes for this node loaded (need to be rendered later, so we save a reference)
                if (!!itemData[this.options.hasSubName] && !!itemData._embedded[this.options.resultKey] && itemData._embedded[this.options.resultKey].length > 0) {
                    nodeWithSubNodes = itemData;
                    this.setElementSelected($item);
                    this.selected[number] = itemData;
                }

                // if this node is the specified as selected in the options, we select it
                if (!!this.initiallySelected && (this.initiallySelected === itemData[this.options.idName] ||
                    this.initiallySelected === itemData[this.options.pathName])
                ) {
                    this.setElementSelected($item);
                    this.selected[number] = itemData;
                    lastSelected = itemData;
                    this.initiallySelected = null;
                }
            }.bind(this));

            // here we come back the the node with the sub-nodes and recursively parse the sub-nodes-column
            if (!!nodeWithSubNodes) {
                this.parseData(nodeWithSubNodes, number);
                // otherwise, if there are no sub-nodes and we have a selected node - we display the "No-Pages"-column
            } else if (!!lastSelected && !lastSelected[this.options.hasSubName]) {
                this.insertAddColumn(lastSelected, number);
            }
        },

        /**
         * Takes the data object and returns a column-item
         * @param data - the item's data
         * @returns the items dom-object
         */
        renderItem: function(data) {
            var $item = this.sandbox.dom.createElement(this.sandbox.util.template(templates.item)({
                    title: this.sandbox.translate(this.sandbox.util.escapeHtml(data[this.options.titleName])),
                    id: data[this.options.idName]
                })),
                disabled = (this.options.disableIds.indexOf(data[this.options.idName]) !== -1);

            if (data[this.options.idName] in this.marked) { // if is marked
                this.sandbox.dom.addClass($item, constants.markedClass);
            }
            if (disabled) { // if is marked
                this.sandbox.dom.addClass($item, constants.disabledClass);
            }
            this.renderLeftInfo($item, data);
            this.renderItemText($item, data);
            this.renderRightInfo($item, data, disabled);

            return $item;
        },

        /**
         * Renders the left buttons and icons for an item
         * @param $item - dom object of the item
         * @param data - the item's data
         */
        renderLeftInfo: function($item, data) {
            var $container = this.sandbox.dom.find('.' + constants.iconsLeftClass, $item), tooltip;
            this.sandbox.dom.append($container, '<span class="fa-check pull-left marked-icon"></span>');
            if (!!this.options.showStatus) {
                // link
                if (!!data[this.options.linkedName]) {
                    if (data[this.options.linkedName] === 'internal') {
                        this.sandbox.dom.append($container,
                            '<span class="fa-internal-link col-icon" title="' + this.sandbox.translate(this.options.tooltipTranslations.internalLink) + '"></span>');
                    } else if (data[this.options.linkedName] === 'external') {
                        this.sandbox.dom.append($container,
                            '<span class="fa-external-link col-icon"  title="' + this.sandbox.translate(this.options.tooltipTranslations.externalLink) + '"></span>');
                    }
                }
                // type (ghost, shadow)
                if (!!data[this.options.typeName]) {
                    if (data[this.options.typeName].name === 'ghost') {
                        this.sandbox.dom.append($container,
                            '<span class="ghost col-icon"  title="' + this.sandbox.translate(this.options.tooltipTranslations.ghost) + '">' + data[this.options.typeName].value + '</span>');
                    } else if (data[this.options.typeName].name === 'shadow') {
                        this.sandbox.dom.append($container,
                            '<span class="fa-shadow-node col-icon"  title="' + this.sandbox.translate(this.options.tooltipTranslations.shadow) + '"></span>');
                    }
                }
                // unpublished
                if (!data[this.options.publishedStateName]) {
                    tooltip = this.sandbox.translate(this.options.tooltipTranslations.unpublished);
                    if (!!data[this.options.publishedName]) {
                        tooltip = this.sandbox.translate(this.options.tooltipTranslations.publishedWithDraft);
                        this.sandbox.dom.append(
                            $container,
                            '<span class="published-icon col-icon" title="' + tooltip + '"></span>'
                        );
                    }

                    this.sandbox.dom.append(
                        $container,
                        '<span class="draft-icon col-icon" title="' + tooltip + '"></span>'
                    );
                }
            }
            if (!!this.options.orderable) {
                this.sandbox.dom.append($container, this.sandbox.util.template(templates.orderInput)({
                    value: data.order
                }));
            }
        },

        /**
         * Renderes the text element of a column-item
         * @param $item - dom object of the item
         * @param data - the data for the item
         */
        renderItemText: function($item, data) {
            var $container = this.sandbox.dom.find('.' + constants.itemTextClass, $item);
            if (!!data[this.options.typeName] && data[this.options.typeName].name === 'ghost') {
                this.sandbox.dom.addClass($container, 'inactive');
            }
        },

        /**
         * Renders the right buttons and icons for an item
         * @param $item - dom object of the item
         * @param data - the data for the item
         * @param disabled - indicates if item is disabled
         */
        renderRightInfo: function($item, data, disabled) {
            var $container = this.sandbox.dom.find('.' + constants.iconsRightClass, $item),
                actionIcon = getActionIcon.call(this, data);

            // if is marked change action Icon
            if (data[this.options.idName] in this.marked) {
                actionIcon = this.options.unmarkIcon;
            }

            // show action icon only for non-ghost pages if actionOnGhost is disabled
            if ((!data[this.options.typeName] || (data[this.options.typeName].name !== 'ghost' || this.options.actionOnGhost)) &&
                this.options.showActionIcon === true && actionIcon && !disabled
            ) {
                this.sandbox.dom.append($container, '<span class="' + actionIcon + ' action col-icon"></span>');
            }

            // show create Button for ghost pages
            if ((!!data[this.options.typeName] && data[this.options.typeName].name === 'ghost') && this.options.showActionButtonOnGhost &&
                this.options.showActionIcon === true && actionIcon && !disabled
            ) {
                this.sandbox.dom.append($container, '<span class="action col-icon">' + this.options.actionButtonOnGhostText + '</span>');
            }

            if (!!data[this.options.hasSubName] && (!disabled || !this.options.disabledChildren)) {
                this.sandbox.dom.append($container, '<span class="fa-chevron-right arrow inactive col-icon"></span>');
            }
        },

        /**
         * Sets the width of the text-container of an item
         * @param {Object} $item the dom-object of an item
         */
        setItemsTextWidth: function($item) {
            var width, $itemText;

            $itemText = this.sandbox.dom.find('.item-text', $item);
            width = constants.columnWidth - this.sandbox.dom.outerWidth(this.sandbox.dom.find('.icons-left', $item));
            width -= parseInt(this.sandbox.dom.css($item, 'padding-right').replace('px', ''));
            width -= parseInt(this.sandbox.dom.css($item, 'padding-left').replace('px', ''));
            width -= this.sandbox.dom.outerWidth(this.sandbox.dom.find('.icons-right', $item));
            width -= 17; // FIXME: Better solution? (For potential scrollbar)
            this.sandbox.dom.width($itemText, width);
            this.cropItemsText($itemText);
        },

        /**
         * Sets the text width of all items in a column
         * @param column
         */
        setColumnTextWidth: function(column) {
            this.sandbox.util.each(this.columns[column], function(id, item) {
                this.setItemsTextWidth(item.$el);
            }.bind(this));
        },

        /**
         * Crops the item text of an item depending on its width
         * @param $itemText {Object}
         */
        cropItemsText: function($itemText) {
            var title = this.sandbox.dom.attr($itemText, 'title'),
                croppedTitle,
                maxLength = title.length,
                overflow;

            //set the item text to the original title
            this.sandbox.dom.html($itemText, title);
            this.sandbox.dom.css($itemText, 'font-weight', 'bold');

            overflow = (this.sandbox.dom.get($itemText, 0).scrollWidth > this.sandbox.dom.width($itemText));

            while (overflow === true) {
                maxLength--;
                croppedTitle = this.sandbox.util.cropMiddle(title, maxLength);
                this.sandbox.dom.html($itemText, croppedTitle);
                overflow = (this.sandbox.dom.get($itemText, 0).scrollWidth > this.sandbox.dom.width($itemText));
            }
            this.sandbox.dom.css($itemText, 'font-weight', '');
        },

        /**
         * Sets/removes all needed classes to display a node as selected
         * @param $element
         */
        setElementSelected: function($element) {
            this.sandbox.dom.addClass($element, 'selected');
            var $arrowElement = this.sandbox.dom.find('.arrow', $element);
            this.sandbox.dom.removeClass($arrowElement, 'inactive');
        },

        /**
         * Adds the loading icon to a contianer
         * @param $container
         */
        addLoadingIcon: function($container) {
            this.sandbox.dom.removeClass($container, 'fa-chevron-right inactive');

            if (this.dom.$loader === null) {
                this.dom.$loader = this.sandbox.dom.createElement('<div class="' + constants.nodeLoaderClass + '"/>');
                this.sandbox.dom.hide(this.dom.$loader);
            }
            if (this.sandbox.dom.is(this.dom.$loader, ':empty')) {
                this.sandbox.start([
                    {
                        name: 'loader@husky',
                        options: {
                            el: this.dom.$loader,
                            size: '16px',
                            color: '#666666'
                        }
                    }
                ]);
            }
            this.sandbox.dom.detach(this.dom.$loader);
            this.sandbox.dom.html($container, this.dom.$loader);
            this.sandbox.dom.show(this.dom.$loader);
        },

        /**
         * Removes loading icon from selected element
         */
        removeLoadingIconForSelected: function() {
            if (!!this.$selectedElement) {
                var $arrow = this.sandbox.dom.find('.arrow', this.$selectedElement);
                this.sandbox.dom.hide(this.dom.$loader);
                this.sandbox.dom.prependClass($arrow, 'fa-chevron-right');
            }
        },

        /**
         * Stores data in internal structure - seperated by column number
         * @param {Object} item
         * @param {Number} columnNumber
         */
        storeDataItem: function(columnNumber, item) {

            if (!this.columns[columnNumber]) {
                this.columns[columnNumber] = {};
            }
            this.columns[columnNumber][item[this.options.idName]] = item;
        },

        bindDOMEvents: function() {
            var selector = 'li:not(.selected' + (this.options.disabledChildren ? ', .' + constants.disabledClass : '') + ')';

            this.sandbox.dom.on(this.dom.$container, 'click', this.itemSelected.bind(this), selector);

            this.sandbox.dom.on(this.$el, 'mouseenter', this.itemMouseEnter.bind(this), '.column-navigation li');
            this.sandbox.dom.on(this.$el, 'mouseleave', this.itemMouseLeave.bind(this), '.column-navigation li');

            this.sandbox.dom.on(this.$el, 'mouseenter', this.showOptions.bind(this), '.column');
            this.sandbox.dom.on(this.dom.$add, 'click', this.addNode.bind(this));
            this.sandbox.dom.on(this.$el, 'click', this.actionClickHandler.bind(this), '.action');
            this.sandbox.dom.on(this.dom.$container, 'dblclick', this.actionClickHandler.bind(this), 'li');

            this.sandbox.dom.on(this.$el, 'click', function(event) {
                this.sandbox.dom.stopPropagation(event);
            }.bind(this), 'input[type="checkbox"]');

            if (this.options.responsive === true) {
                this.sandbox.dom.on(this.sandbox.dom.$window, 'resize', function() {
                    this.setContainerHeight();
                    this.setOverflowClass();
                }.bind(this));
            }

            if (this.options.orderable) {
                this.sandbox.dom.on(this.$el, 'click', function(event) {
                    this.sandbox.dom.stopPropagation(event);
                }.bind(this), '.' + constants.orderInputClass);

                this.sandbox.dom.on(this.$el, 'focus', function(event) {
                    if (this.isOrdering === true) {
                        this.sandbox.dom.blur(event.currentTarget);
                        return false;
                    }
                    this.sandbox.dom.select(event.currentTarget);
                }.bind(this), '.' + constants.orderInputClass + ' input');

                // save last clicked element -> used to trigger the default action after the ordering has finished
                this.sandbox.dom.on(this.dom.$wrapper, 'mousedown', function(event) {
                    this.lastClicked = this.sandbox.dom.$(event.target);
                }.bind(this));

                this.sandbox.dom.on(this.dom.$ok, 'click', this.closeOrderMode.bind(this));
                this.sandbox.dom.on(this.$el, 'keydown', this.orderKeyHandler.bind(this), '.' + constants.orderInputClass + ' input');
                this.sandbox.dom.on(this.$el, 'focusout', this.orderBlurHandler.bind(this), '.' + constants.orderInputClass);
            }
        },

        unbindDOMEvents: function() {
            this.$el.off();
            if (!!this.dom.$add) {
                this.dom.$add.off();
            }
            if (!!this.dom.$container) {
                this.dom.$container.off();
            }
            if (!!this.dom.$wrapper) {
                this.dom.$wrapper.off();
            }
            if (!!this.dom.$ok) {
                this.dom.$ok.off();
            }
        },

        /**
         * Handles the key-down event of a order-input
         * @param event
         */
        orderKeyHandler: function(event) {
            if (event.keyCode === 13) { // blur on enter
                this.sandbox.dom.blur(event.currentTarget);
            }
            if (event.keyCode === 27) { // cancel on esc
                var column = this.sandbox.dom.attr(this.sandbox.dom.parents(
                    event.currentTarget, '.' + constants.columnClass), 'data-column'),
                    item = this.sandbox.dom.attr(this.sandbox.dom.parents(
                        event.currentTarget, '.' + constants.columnItemClass), 'data-id');
                this.resetOrderInput(column, item);
                this.sandbox.dom.blur(event.currentTarget);
            }
        },

        /**
         * Handles the blur event of a order-input
         * @param event
         */
        orderBlurHandler: function(event) {
            if (this.isOrdering === false) {
                var column = this.sandbox.dom.attr(this.sandbox.dom.parents(event.currentTarget, '.' + constants.columnClass), 'data-column'),
                    item = this.sandbox.dom.attr(this.sandbox.dom.parents(event.currentTarget, '.' + constants.columnItemClass), 'data-id'),
                    $input = this.sandbox.dom.find('input', this.columns[column][item].$el);
                // if is an integer
                if (this.sandbox.dom.isNumeric(this.sandbox.dom.val($input)) &&
                    Math.floor(this.sandbox.dom.val($input)) == this.sandbox.dom.val($input)) {
                    this.orderPrepare(column, item);
                } else {
                    this.orderError(column, item);
                }
            }
        },

        /**
         * Looks if the position changes and executes the order or resets the inputs
         * @param column - column in which the item is
         * @param item - the id of the item
         */
        orderPrepare: function(column, item) {
            var $input = this.sandbox.dom.find('input', this.columns[column][item].$el),
                position = this.normalizeOrderPosition(column, parseInt(this.sandbox.dom.val($input), 10));
            this.sandbox.dom.val($input, position);
            this.sandbox.dom.removeClass(this.columns[column][item].$el, constants.orderErrorClass);
            if (position !== this.columns[column][item].order) {
                this.isOrdering = true;
                this.confirmOrder(
                    function() { // ok callback
                        this.order(column, item, position);
                        this.isOrdering = false;
                        this.sandbox.dom.click(this.lastClicked);
                    }.bind(this),
                    function() { // cancel callback
                        this.resetOrderInput(column, item);
                        this.isOrdering = false;
                        this.sandbox.dom.click(this.lastClicked);
                    }.bind(this)
                );
            }
        },

        /**
         * Changes the position of an item and updates the positions of the other items in the column
         * @param column - the index of the column in which the item is
         * @param item - the id of the item
         * @param newPosition - the new position of the item
         */
        order: function(column, item, newPosition) {
            var oldPosition = this.columns[column][item].order;
            this.sandbox.util.each(this.columns[column], function(itemId) {
                this.repositionItem(column, itemId, oldPosition, newPosition);
                this.resetOrderInput(column, itemId);
            }.bind(this));
            this.updateItemDomPosition(column, item);
            this.sandbox.util.delay(this.highlight.bind(this, item), 550);
            this.sandbox.emit(ORDERED.call(this), item, newPosition);
            this.isOrdering = false;
        },

        /**
         * Determines and resets the position of a single item, given a position-change
         * @param column - the column in which the item is
         * @param item - the id of the item
         * @param oldPosition - the old position of the position-change
         * @param newPosition - the new position of the position-change
         */
        repositionItem: function(column, item, oldPosition, newPosition) {
            if (this.columns[column][item].order === oldPosition) { // update property to new position
                this.columns[column][item].order = newPosition;
            } else { // update the order-position of the other items
                if (this.columns[column][item].order > oldPosition &&
                    this.columns[column][item].order <= newPosition) {
                    this.columns[column][item].order--;
                } else if (this.columns[column][item].order < oldPosition &&
                    this.columns[column][item].order >= newPosition) {
                    this.columns[column][item].order++;
                }
            }
        },

        /**
         * Brings an item to the position set in the order-position-property
         * @param column - the column of the item
         * @param item - the id of the item
         */
        updateItemDomPosition: function(column, item) {
            var $list = this.sandbox.dom.parent(this.columns[column][item].$el);
            this.sandbox.dom.detach(this.columns[column][item].$el);
            this.sandbox.dom.insertAt(
                this.columns[column][item].order - 1, '.' + constants.columnItemClass,
                $list, this.columns[column][item].$el
            );
            this.sandbox.dom.scrollAnimate(
                this.sandbox.dom.position(this.columns[column][item].$el).top,
                this.sandbox.dom.parents(this.columns[column][item].$el, '.' + constants.columnClass)
            );
        },

        /**
         * Resets the position input of an item
         * @param column
         * @param item
         */
        resetOrderInput: function(column, item) {
            var $input = this.sandbox.dom.find('input', this.columns[column][item].$el);
            this.sandbox.dom.val($input, this.columns[column][item].order);
        },

        /**
         * Normalizes a position used for ordering
         * @param column - the column for which the position gets normalized
         * @param position (int) - the position to normalize
         * @returns int - the normalized position
         */
        normalizeOrderPosition: function(column, position) {
            position = (position < 1) ? 1 : position;
            return (position > (Object.keys(this.columns[column])).length) ?
                Object.keys(this.columns[column]).length : position;
        },

        /**
         * Sets css-classes on an item, through which an input error get signalized
         * @param column - the column in which the item is
         * @param item - the id of the item
         */
        orderError: function(column, item) {
            this.sandbox.dom.addClass(this.columns[column][item].$el, constants.orderErrorClass);
        },

        /**
         * Shows a confirmation-overlay
         * @param okCallback - callback to execute if clicked on ok
         * @param closeCallback - callback to execute if clicked on ok
         */
        confirmOrder: function(okCallback, closeCallback) {
            this.sandbox.confirm.alert(this,
                this.options.orderConfirmTitle, this.options.orderConfirmMessage,
                okCallback, closeCallback);
        },

        /**
         * Sets an overflow-class to the container if the navigation is scrollable
         */
        setOverflowClass: function() {
            var $navigation = this.sandbox.dom.find('.column-navigation', this.$el);
            if (this.sandbox.dom.width($navigation) < this.sandbox.dom.get($navigation, 0).scrollWidth) {
                this.sandbox.dom.addClass($navigation, 'overflow');
            } else {
                this.sandbox.dom.removeClass($navigation, 'overflow');
            }
        },

        bindCustomEvents: function() {
            if (!!this.customEvents) {
                return;
            }

            this.sandbox.on(BREADCRUMB.call(this), this.getBreadCrumb.bind(this));
            this.sandbox.on(UNMARK.call(this), this.unmark.bind(this));
            this.sandbox.on(HIGHLIGHT.call(this), this.highlight.bind(this));
            this.sandbox.on(ORDER.call(this), this.startOrderModeItem.bind(this));
            this.sandbox.on(SET_OPTIONS.call(this), function(options) {
                this.unbindDOMEvents();
                this.$el.children().remove();
                this.options = this.sandbox.util.extend(true, {}, this.options, options);

                this.initialize();
            }.bind(this));
            this.sandbox.on(GET_MARKED.call(this), function (callback) {
                if (typeof callback === "function") {
                    callback(this.marked);
                }
            }.bind(this));

            this.sandbox.on('husky.dropdown.' + this.options.instanceName + '.settings.dropdown.item.click', this.dropdownItemClicked.bind(this));

            if (this.options.responsive === true) {
                this.sandbox.on(RESIZE.call(this), function() {
                    this.setContainerHeight();
                    this.setOverflowClass();
                }.bind(this));
            }

            this.customEvents = true;
        },

        /**
         * Sets a column containing an item with a given id in order-mode
         * @param itemId - the id of the item
         */
        startOrderModeItem: function(itemId) {
            var column = this.getColumnForItem(itemId);
            if (column > 0) {
                this.startOrderModeColumn(column);
            }
        },


        /**
         * Sets a column in order-mode
         * @param column - the index of the column
         */
        startOrderModeColumn: function(column) {
            if (this.inOrderMode === false && !!this.columns[column] && Object.keys(this.columns[column]).length > 1) {
                this.inOrderMode = true;
                var $column = this.$find('.' + constants.columnClass + '[data-column="' + column + '"]');
                this.lockOptions(column);
                this.sandbox.dom.addClass($column, constants.orderModeClass);
                this.sandbox.dom.hide(this.dom.$settings);
                this.sandbox.dom.hide(this.dom.$add);
                this.sandbox.dom.show(this.dom.$ok);
                setOptionButtonWidths.call(this);
                this.setColumnTextWidth(column);
                this.sandbox.emit(ORDER_START.call(this));
            }
        },

        /**
         * Closes the order mode
         */
        closeOrderMode: function() {
            if (this.inOrderMode === true) {
                var $column = this.$find('.' + constants.columnClass + '.' + constants.orderModeClass),
                    column = this.sandbox.dom.data($column, 'column');
                this.unlockOptions();
                this.sandbox.dom.removeClass($column, constants.orderModeClass);
                this.sandbox.dom.hide(this.dom.$ok);
                this.sandbox.dom.show(this.dom.$settings);
                this.sandbox.dom.show(this.dom.$add);
                this.setColumnTextWidth(column);
                this.inOrderMode = false;
                this.sandbox.emit(ORDER_END.call(this));
            }
        },

        /**
         * Highlights an item
         * @param item - the id of the item
         */
        highlight: function(item) {
            var column = this.getColumnForItem(item);
            if (column > 0) {
                this.sandbox.dom.addClass(this.columns[column][item].$el, constants.highlightClass);

                // remove class after effect has finished
                this.sandbox.dom.on(this.columns[column][item].$el, 'animationend webkitAnimationEnd oanimationend MSAnimationEnd', function() {
                    this.sandbox.dom.removeClass(this.columns[column][item].$el, constants.highlightClass);
                }.bind(this));
            }
        },

        /**
         * Gets execute after an item in the settings-dropdown has been clicked
         * @param dropdownItem - the dropdown-item
         */
        dropdownItemClicked: function(dropdownItem) {
            if (!!this.lastHoveredColumn && !!dropdownItem.mode && dropdownItem.mode === 'order') {
                this.startOrderModeColumn(this.lastHoveredColumn);
            }
            if (!!this.selected[this.lastHoveredColumn]) {
                if (!!dropdownItem.callback) {
                    dropdownItem.callback(dropdownItem, this.selected[this.lastHoveredColumn], this.columns[this.lastHoveredColumn]);
                } else {
                    this.sandbox.emit(SETTINGS.call(this), dropdownItem, this.selected[this.lastHoveredColumn], this.columns[this.lastHoveredColumn]);
                }
            }
        },

        /**
         * Returns all the marked objects.
         */
        mark: function(id, item) {
            var $element = this.$find('li[data-id="' + id + '"]');
            if (!!$element.length) {
                // if single markable unmark all the others
                if (this.options.singleMarkable === true) {
                    $.each(this.marked, function (key, value) {
                        this.unmark(key);
                    }.bind(this));
                }
                this.sandbox.dom.addClass($element, constants.markedClass);
                if(!(id in this.marked)) {
                    this.marked[id] = item;
                }
                this.setItemsTextWidth($element);
                // change unmark to mark icon
                var $markButton = $element.find('span:last-child > .' + this.options.actionIcon);
                $markButton.removeClass(this.options.actionIcon);
                this.sandbox.dom.prependClass($markButton, this.options.unmarkIcon);
            }
        },

        /**
         * Unmarks a node for a given id
         * @param id {Number|String} the id of the node to unmark
         */
        unmark: function(id) {
            var $element = this.$find('li[data-id="' + id + '"]');
            if (!!$element.length) {
                $element.removeClass(constants.markedClass);
                if(id in this.marked) {
                    delete this.marked[id];
                }
                // change unmark to mark icon
                var $unmarkButton = $element.find('span:last-child > .' + this.options.unmarkIcon);
                $unmarkButton.removeClass(this.options.unmarkIcon);
                this.sandbox.dom.prependClass($unmarkButton, this.options.actionIcon);
            }
        },

        /**
         * Sets the text width
         * @param {Object} event
         */
        itemMouseEnter: function(event) {
            this.setItemsTextWidth(event.currentTarget);
        },

        /**
         * Sets the text width
         * @param {Object} event
         */
        itemMouseLeave: function(event) {
            this.setItemsTextWidth(event.currentTarget);
        },

        /**
         * Returns the breadcrumb
         * @param {Function} callback
         */
        getBreadCrumb: function(callback) {
            if (typeof callback === 'function') {
                callback(this.selected);
            } else {
                this.sandbox.logger.log("callback is not a function");
            }
        },

        /**
         * Shows the options at the last available column
         */
        showOptionsAtLast: function() {
            var $lastColumn = this.sandbox.dom.last(this.sandbox.dom.find('.column', this.dom.$container));
            this.showOptions({
                currentTarget: $lastColumn
            });
        },

        /**
         * Shows the options below the last hovered column
         * @param {Object} event
         */
        showOptions: function(event) {
            if (this.optionsLocked === false) {
                var $currentTarget = this.sandbox.dom.$(event.currentTarget);

                this.displayOptions($currentTarget);

                this.sandbox.dom.off(this.dom.$container, 'scroll.column-navigation.' + this.options.instanceName);
                this.sandbox.dom.on(this.dom.$container, 'scroll.column-navigation.' + this.options.instanceName, this.displayOptions.bind(this, $currentTarget));
            }
        },

        /**
         * Shows the options below the passed column and locks them
         * @param column - the index of the column
         */
        lockOptions: function(column) {
            var $column = this.$find('.' + constants.columnClass + '[data-column="' + column + '"]');
            if (!!$column.length) {
                this.showOptions({currentTarget: $column});
                this.optionsLocked = true;
            }
        },

        /**
         * Unlocks the options
         */
        unlockOptions: function() {
            this.optionsLocked = false;
        },

        /**
         * Displays the options-navigation under a given column
         * @param $activeColumn {object} column for which the options will be inserted
         */
        displayOptions: function($activeColumn) {
            if (!!this.inOrderMode) {
                return;
            }

            var visibleRatio;

            this.lastHoveredColumn = this.sandbox.dom.data($activeColumn, 'column');

            // calculate the ratio of how much of the hovered column is visible
            if (this.sandbox.dom.position($activeColumn).left + this.sandbox.dom.width($activeColumn) > this.sandbox.dom.width(this.dom.$container)) {
                visibleRatio = (this.sandbox.dom.width(this.dom.$container) - this.sandbox.dom.position($activeColumn).left ) / this.sandbox.dom.width($activeColumn);
            } else {
                visibleRatio = (this.sandbox.dom.width($activeColumn) + this.sandbox.dom.position($activeColumn).left) / this.sandbox.dom.width($activeColumn);
            }

            // display the option only if the column is visible enough
            if (visibleRatio >= constants.minVisibleRatio) {
                this.sandbox.dom.css(this.$optionsContainer, {'visibility': 'visible'});
                this.moveOptions($activeColumn);
                this.toggleSettingDropdownItems(this.lastHoveredColumn);
            } else {
                this.hideOptions();
            }
        },

        /**
         * Calls the enabler for all dropdown-items (if exists) and disables
         * or enalbes the corresponding item
         * @param column - the index of the column on which the enalber should check the states of the items
         */
        toggleSettingDropdownItems: function(column) {
            if (!!this.options.data && this.options.data.length > 0) {
                var context = createContext.call(this, column);
                this.sandbox.util.each(this.options.data, function(index, item) {
                    if (!!item.enabler && typeof item.enabler === 'function') {
                        this.sandbox.emit(
                            'husky.dropdown.' + this.options.instanceName + '.settings.dropdown.item.toggle',
                            item.id,
                            item.enabler(context)
                        );
                    }
                }.bind(this));
            }
        },

        /**
         * Updates the position of the options
         * @param $activeColumn {object} dom-object of active column
         */
        moveOptions: function($activeColumn) {
            var marginLeft = this.sandbox.dom.position($activeColumn).left - 1;
            $(this.$optionsContainer).css('margin-left', marginLeft + 'px');

            if (getAddButton.call(this)) {
                $(this.dom.$add).show();
            } else {
                $(this.dom.$add).hide();
            }

            setOptionButtonWidths.call(this);
        },

        /**
         * Hides options
         */
        hideOptions: function() {
            this.sandbox.dom.css(this.$optionsContainer, {'visibility': 'hidden'});
        },

        /**
         * Item was selected and data will be loaded if has sub
         * @param {Object} event
         */
        itemSelected: function(event) {
            //only do something if no column is loading
            if (this.columnLoadStarted === false) {
                this.$selectedElement = this.sandbox.dom.$(event.currentTarget);
                var id = this.sandbox.dom.data(this.$selectedElement, 'id'),
                    $column = this.sandbox.dom.parent(this.sandbox.dom.parent(this.$selectedElement)),
                    column = $column.data('column'),
                    selectedItem = this.columns[column][id],
                    length = this.selected.length - 1,
                    i, $arrowElement;

                // if column is in order mode the items cannot be selected
                if ($column.hasClass(constants.orderModeClass)) {
                    this.$selectedElement.find('input').focus();
                    return false;
                }

                this.closeOrderMode();

                if (this.sandbox.dom.hasClass(this.$selectedElement, 'selected')) { // is element already selected

                    this.sandbox.emit(SELECTED.call(this), selectedItem);

                } else { // element not selected

                    this.removeCurrentSelected(column);

                    this.sandbox.dom.addClass(this.$selectedElement, 'selected');
                    $arrowElement = this.sandbox.dom.find('.arrow', this.$selectedElement);
                    this.addLoadingIcon($arrowElement);

                    if (!!selectedItem) {

                        // remove old elements from breadcrumb
                        for (i = length; i >= column; i--) {
                            delete this.selected[i];
                        }

                        // add element to breadcrumb
                        this.selected[column] = selectedItem;
                        this.sandbox.emit(SELECTED.call(this), selectedItem);

                        if (!!selectedItem[this.options.hasSubName]) {
                            this.load(selectedItem._links.children.href, column);
                        }

                        this.removeColumns(column + 1);
                        this.setOverflowClass();
                    }
                }

                // scroll for add column
                if (!selectedItem[this.options.hasSubName]) {
                    this.insertAddColumn(selectedItem, column);
                    this.scrollIfNeeded(column);
                    this.setOverflowClass();
                    this.toggleSettingDropdownItems(column);
                }
            }
        },

        insertAddColumn: function(selectedItem, column) {
            var $addColumn = this.sandbox.dom.createElement(this.sandbox.util.template(templates.column)({
                columnNumber: column + 1,
                id: 'column' + this.options.instanceName + '-' + (column + 1)
            }));
            // fill it with the "No Page"-content
            this.sandbox.dom.append($addColumn, this.sandbox.dom.createElement(
                this.sandbox.util.template(templates.noPage)({
                    description: this.sandbox.translate(this.options.noPageDescription)
                })
            ));
            this.sandbox.dom.append(this.dom.$container, $addColumn);
            this.filledColumns++;
        },

        /**
         * Scrolls if needed
         * @param column
         */
        scrollIfNeeded: function(column) {
            if (column > constants.displayedcolumns) {
                this.sandbox.dom.scrollLeft(this.dom.$container, (column - constants.displayedcolumns) * constants.columnWidth);
            }
        },

        /**
         * Removes the selected class from old elements
         * @param {Number} column
         */
        removeCurrentSelected: function(column) {
            var $items = this.sandbox.dom.find('li', '#column' + this.options.instanceName + '-' + column);

            this.sandbox.util.each($items, function(index, $el) {
                this.sandbox.dom.removeClass($el, 'selected');
                var $arrowElement = this.sandbox.dom.find('.arrow', $el);
                this.sandbox.dom.addClass($arrowElement, 'inactive');
            }.bind(this));
        },

        /**
         * Emits an add event
         */
        addNode: function() {
            var parent = this.selected[this.lastHoveredColumn - 1] || null;
            this.sandbox.emit(ADD.call(this), parent);
        },

        /**
         * Handles the click on action-icons as well as the double click on items
         * @param {Object} event
         */
        actionClickHandler: function(event) {
            var $listItem, id, item, column;

            if (this.inOrderMode === true) {
                return false;
            }

            if (this.sandbox.dom.hasClass(event.currentTarget, 'action') === true) {
                $listItem = this.sandbox.dom.parent(this.sandbox.dom.parent(event.currentTarget));
            } else {
                $listItem = this.sandbox.dom.$(event.currentTarget);
            }

            if ($listItem.hasClass(constants.disabledClass)) {
                return;
            }

            column = this.sandbox.dom.index(this.sandbox.dom.parents(event.currentTarget, '.column'));
            id = this.sandbox.dom.attr($listItem, 'data-id');
            item = this.columns[column][id];

            if (this.options.markable === true) {
                if (!(id in this.marked)) {
                    this.mark(id, item)
                } else {
                    this.unmark(id);
                }
            }

            this.sandbox.dom.stopPropagation(event);
            if (!!this.options.actionCallback) {
                this.options.actionCallback(item)
            } else {
                this.sandbox.emit(ACTION.call(this), item);
            }
        }
    };
});


/*****************************************************************************
 *
 *  Loader
 *  makes use of bouncing balls of http://tobiasahlin.com/spinkit/
 *
 *  Options (defaults)
 *      - size: size of element
 *      - color: color of bouncers
 *      - hidden: hide component on load
 *  Provides Events
 *      - husky.loader.show - shows loader
 *      - husky.loader.hide - hides loader
 *
 *  TODO: write tests
 *
 *****************************************************************************/



/**
 * @class Loader
 * @constructor
 *
 * @param {Object} [options] Configuration object
 * @param {String} [options.size] size of the spinner
 * @param {String} [options.color] color of the spinner
 */
define('__component__$loader@husky',[],function() {

    'use strict';

    var defaults = {
        size: '40px',
        color: '#666',
        hidden: false
        },

        namespace = 'husky.loader.',

        /**
         * makes loader visible
         *
         * @event husky.loader.show
         */
        SHOW = namespace + 'show',

        /**
         * hides the loader
         *
         * @event husky.loader.hide
         */
        HIDE = namespace + 'hide',

        /**
         * called when loader is initialized
         *
         * @event husky.loader.initialized
         */
        INITIALIZED = namespace + 'initialized',

        templates = {
            doubleBounce: [
                '<div class="spinner">',
                    '<div class="double-bounce1"></div>',
                    '<div class="double-bounce2"></div>',
                '</div>'].join('')
        },

        bindCustomEvents = function() {
            this.sandbox.on(SHOW, showLoader.bind(this));
            this.sandbox.on(HIDE, hideLoader.bind(this));
        },

        hideLoader = function() {
            this.sandbox.dom.hide(this.$spinner);
        },
        showLoader = function() {
            this.sandbox.dom.show(this.$spinner);
        };

    return {

        initialize: function() {
            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

            this.render();

            bindCustomEvents.call(this);

            this.sandbox.emit(INITIALIZED);
        },

        render: function() {
            this.$spinner = this.sandbox.dom.createElement(templates.doubleBounce);
            var $elements, styles;

            this.sandbox.dom.html(this.$el, this.$spinner);

            // style adjustments
            // size
            styles = {
                width: this.options.size,
                height: this.options.size
            };
            this.sandbox.dom.css(this.$spinner, styles);
            // color
            $elements = this.sandbox.dom.find('div', this.$spinner);
            this.sandbox.dom.css($elements, 'background-color', this.options.color);
            // visibility
            if (this.options.hidden) {
                this.sandbox.dom.hide(this.$spinner);
            }
        }
    };

});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/ckeditor
 */


/**
 * @class CKEditor
 * @constructor
 *
 * @params {Object} [options] Configuration object
 * @params {Function} [options.initialzedCallback] Callback when initialization is finished
 *
 */
define('__component__$ckeditor@husky',[], function() {

    'use strict';

    var defaults = {
            initializedCallback: null,
            instanceName: null,
            table: true,
            link: true,
            pasteFromWord: true,
            autoStart: true,
            placeholder: ''
        },

        /**
         * namespace for events
         * @type {string}
         */
        eventNamespace = 'husky.ckeditor.',

        /**
         * @event husky.ckeditor.initialized
         * @description emitted when the component is fully initialized
         */
        INITIALIZED = function() {
            return eventNamespace + (this.options.instanceName !== null ? this.options.instanceName + '.' : '') + 'initialized';
        },

        /**
         * @event husky.ckeditor.changed
         * @description the component has loaded everything successfully and will be rendered
         */
        CHANGED = function() {
            return eventNamespace + (this.options.instanceName !== null ? this.options.instanceName + '.' : '') + 'changed';
        },

        /**
         * @event husky.ckeditor.focusout
         * @description triggered when focus of editor is lost
         */
        FOCUSOUT = function() {
            return eventNamespace + (this.options.instanceName !== null ? this.options.instanceName + '.' : '') + 'focusout';
        },

        /**
         * @event husky.ckeditor.start
         * @description starts the used editor plugin
         */
        START = function() {
            return eventNamespace + (this.options.instanceName !== null ? this.options.instanceName + '.' : '') + 'start';
        },

        /**
         * @event husky.ckeditor.destroy
         * @description destroys the used editor plugin
         */
        DESTROY = function() {
            return eventNamespace + (this.options.instanceName !== null ? this.options.instanceName + '.' : '') + 'destroy';
        },

        /**
         * Removes the not needed elements from the config object for the ckeditor
         * @returns {Object} configuration object for ckeditor
         */
        getConfig = function() {
            var config = this.sandbox.util.extend(false, {}, this.options),
                toolbarBuilder = this.sandbox.ckeditor.getToolbarBuilder();

            if (!!config.toolbar) {
                toolbarBuilder = this.sandbox.ckeditor.createToolbarBuilder(config.toolbar);
            }

            // deactivate paste from Word
            if (!this.options.pasteFromWord === true) {
                toolbarBuilder.remove('paste', ['PasteFromWord']);
            }

            // deactivate embed links
            config.linkShowTargetTab = false;
            if (!this.options.link) {
                toolbarBuilder.remove('links', ['Link', 'Unlink']);
                config.linkShowTargetTab = true;
            }

            // activate tables
            if (!this.options.table) {
                toolbarBuilder.remove('insert', ['Table']);
            }

            // set height
            if (!!this.options.height) {
                config.autoGrow_minHeight = this.options.height;
                config.height = this.options.height;
            }

            // set maxHeight
            if (!!this.options.maxHeight) {
                config.autoGrow_maxHeight = this.options.maxHeight;
                // if height bigger maxHeight height = maxHeight
                if (config.height > config.autoGrow_maxHeight) {
                    config.height = config.autoGrow_maxHeight;
                }
            }

            // ENTER MODE
            if (!!this.options.enterMode) {
                config.enterMode = CKEDITOR['ENTER_' + this.options.enterMode.toUpperCase()];
            }

            // deactivate styles
            if (!config.stylesSet || 0 === config.stylesSet.length) {
                toolbarBuilder.remove('styles', ['Styles']);
            }

            config.toolbar = toolbarBuilder.get();

            delete config.initializedCallback;
            delete config.baseUrl;
            delete config.el;
            delete config.property;
            delete config.name;
            delete config.ref;
            delete config._ref;
            delete config.require;
            delete config.element;
            delete config.link;
            delete config.table;
            delete config.maxHeight;
            delete config.autoStart;
            delete config.placeholder;

            return config;
        };

    return {

        initialize: function() {
            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);
            this.editorContent = null;

            if (!!this.options.autoStart) {
                this.startEditor();
            } else {
                this.renderStartTemplate();
            }

            this.sandbox.on(START.call(this), this.startEditor.bind(this));
            this.sandbox.on(DESTROY.call(this), this.destroyEditor.bind(this));
        },

        renderStartTemplate: function() {
            var $content = $(this.$el.val()),
                text = $content.text(),
                $trigger = $(
                    [
                        '<textarea class="form-element ckeditor-preview" placeholder="', this.options.placeholder, '">',
                        text,
                        '</textarea>'
                    ].join('')
                );

            this.$el.hide();
            this.$el.parent().append($trigger);

            $trigger.one('focus', function(e) {
                $(e.currentTarget).remove();

                this.startEditor();

                this.editor.once('instanceReady', function() {
                    this.editor.focus();
                }.bind(this));
            }.bind(this));
        },

        /**
         * Binds Events to emit a custom changed event
         */
        bindEditorEvents: function() {
            this.editor.on('dialogShow', function() {
                this.sandbox.dom.addClass(this.sandbox.dom.parent('.cke_dialog_ui_button_ok'), 'sulu_ok_button');
                this.sandbox.dom.addClass(this.sandbox.dom.parent('.cke_dialog_ui_button_cancel'), 'sulu_cancel_button');
            }.bind(this));

            this.editor.on('change', function() {
                this.emitChangedEvent();
            }.bind(this));

            // check if the content of the editor has changed if the mode is switched (html/wisiwig)
            this.editor.on('mode', function() {
                if (this.data !== this.editor.getData()) {
                    this.emitChangedEvent();
                }
            }.bind(this));

            this.editor.on('instanceReady', function() {
                // bind class to editor
                this.sandbox.dom.addClass(this.sandbox.dom.find('.cke', this.sandbox.dom.parent(this.$el)), 'form-element');
                this.sandbox.emit(INITIALIZED.call(this));
            }.bind(this));

            this.editor.on('blur', function() {
                this.sandbox.emit(FOCUSOUT.call(this), this.editor.getData(), this.$el);
            }.bind(this));
        },

        /**
         * Emits the custom changed event
         */
        emitChangedEvent: function() {
            this.data = this.editor.getData();
            this.sandbox.emit(CHANGED.call(this), this.data, this.$el);
        },

        startEditor: function() {
            var config = getConfig.call(this);

            if (!this.options.autoStart) {
                this.$el.show();
            }

            this.editor = this.sandbox.ckeditor.init(this.$el, this.options.initializedCallback, config);
            if (!!this.editorContent) {
                this.editor.setData(this.editorContent);
            }

            this.data = this.editor.getData();
            this.bindEditorEvents();
        },

        destroyEditor: function() {
            if (!!this.editor) {
                this.editorContent = this.editor.getData();
                if (!!this.editor.window && !!this.editor.window.getFrame()) {
                    this.editor.destroy();
                } else {
                    delete CKEDITOR.instances[this.editor.name];
                }
            }
        },

        destroy: function() {
            this.destroyEditor();
        }
    };
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/overlay
 */

/**
 * @class Overlay
 * @constructor
 *
 * @params {Object} [options] Configuration object
 * @params {String} [options.trigger] List of events on which the overlay should be opened
 * @params {String} [options.triggerEl] Element that triggers the overlay
 * @params {String} [options.instanceName] instance name of the component
 * @params {Boolean} [options.openOnStart] if true overlay is opened after initialization
 * @params {Boolean} [options.removeOnClose] if overlay component gets removed on close
 * @params {String} [options.skin] set an overlay skin to manipulate overlay's appearance. Possible skins: '', 'small', 'medium', 'large', 'responsive-width'
 * @params {Boolean} [options.backdropClose] if true overlay closes with click on backdrop
 * @params {Boolean} [options.contentSpacing] Defines if there should be a spacing between overlay borders and content
 * @params {String} [options.type] The type of the overlay ('normal' or 'warning')
 * @params {Array} [options.buttonsDefaultAlign] the align of the buttons in the footer ('center', 'left' or 'right'). Can be overriden by each button individually
 * @params {Array} [options.supportKeyInput] if true pressing enter will submit the overlay and esc will close it
 * @params {Null|Number} [options.left] to fix the left position of the overlay. (px)
 * @params {Null|Number} [options.top] to fix the top position of the overlay. (px)
 * @params {Array} [options.propagateEvents] If false click-events will be stoped at the components-element
 *
 * @params {Array} [options.slides] array of slide objects, will be rendered in a row and can slided with events
 * @params {String} [options.slides[].title] the title of the overlay
 * @params {String} [options.slides[].subTitle] the sub-title of the overlay
 * @params {Function} [options.slides[].closeCallback] @deprecated Use 'cancelCallback' instead
 * @params {Function} [options.slides[].cancelCallback] callback which gets executed after the overlay gets canceled
 * @params {Function} [options.slides[].okCallback] callback which gets executed after the overlay gets submitted
 * @params {Boolean} [options.slides[].displayHeader] Boolean variable which determines wether or not the header gets rendered
 * @params {String|Object} [options.slides[].data] HTML or DOM-object which acts as the overlay-content
 * @params {String} [options.slides[].message] String to render as content. Used by warnings and errors
 * @params {String} [options.slides[].panelContent] The content to render in the panel container in the sub-header
 * @params {Boolean} [options.slides[].okInactive] If true all ok-buttons start deactivated
 * @params {String} [options.slides[].okDefaultText] The default text for ok buttons
 * @params {String} [options.slides[].cancelDefaultText] The default text for cancel buttons
 * @params {String} [options.slides[].type] The type of the overlay ('normal', 'error' or 'warning')
 * @params {Number} [options.startingSlide] The index of the slide to start with
 *
 * @params {Object} [options.slides[].languageChanger] If set language-changer will be displayed in the header
 * @params {Array} [options.slides[].languageChanger.locales] array of locale strings for the dropdown
 * @params {String} [options.slides[].languageChanger.preSelected] locale which is selected at the beginning
 *
 * @params {Array} [options.slides[].tabs] array of tabs-data to use instead of options.data and options.message
 * @params {String} [options.slides[].tabs.title] the title of the tab
 * @params {String|Object} [options.slides[].tabs.data] HTML or DOM-Object to display when tab is active
 *
 * @params {Array} [options.slides[].buttons] an array of buttons to add to the footer
 * @params {String} [options.slides[].buttons.type] type of the button ('ok', 'cancel')
 * @params {String} [options.slides[].buttons.icon] icon of the button
 * @params {String} [options.slides[].buttons.text] text of the button. If text and icon are not set the defaultText-options come into place
 * @params {Boolean} [options.slides[].buttons.inactive] If true button starts inactive
 */
define('__component__$overlay@husky',[], function() {

    'use strict';

    var defaults = {
            trigger: 'click',
            triggerEl: null,
            instanceName: 'undefined',
            openOnStart: false,
            removeOnClose: true,
            backdropClose: true,
            skin: '',
            supportKeyInput: true,
            contentSpacing: true,
            propagateEvents: true,
            type: 'normal',
            cssClass: '',
            slides: [],
            startingSlide: null,
            top: null,
            left: null
        },

        slideDefaults = {
            index: -1,
            title: '',
            subTitle: null,
            displayHeader: true,
            message: '',
            closeCallback: null,
            cancelCallback: null,
            okCallback: null,
            type: 'normal',
            data: '',
            panelContent: '',
            tabs: null,
            okInactive: false,
            buttonsDefaultAlign: 'center',
            cancelDefaultText: 'public.cancel',
            okDefaultText: 'public.ok',
            languageChanger: null,
            contentSpacing: true,
            cssClass: ''
        },

        internalSlideDefaults = {
            $el: null,
            $footer: null,
            $header: null,
            $content: null,
            $languageChanger: null,
            $subheader: null,
            tabs: null //contains tabs related data
        },

        constants = {
            closeSelector: '.close-button',
            footerSelector: '.overlay-footer',
            contentSelector: '.overlay-content',
            headerSelector: '.overlay-header',
            slidesSelector: '.slides',
            overlayOkSelector: '.overlay-ok',
            overlayCancelSelector: '.overlay-cancel',
            overlayOtherButtonsSelector: '.overlay-button',
            panelClass: 'panel'
        },

        types = {
            normal: {
                buttons: [
                    {
                        type: 'ok',
                        inactive: false,
                        align: 'right'
                    },
                    {
                        type: 'cancel',
                        inactive: false,
                        align: 'left'
                    }
                ]
            },
            alert: {
                cssClass: 'alert',
                removeOnClose: true,
                openOnStart: true,
                contentSpacing: false,
                instanceName: 'alert',
                buttons: [
                    {
                        type: 'ok',
                        inactive: false,
                        align: 'right'
                    },
                    {
                        type: 'cancel',
                        inactive: false,
                        align: 'left'
                    }
                ]
            }
        },

        buttonTypes = {
            OK: 'ok',
            CANCEL: 'cancel'
        },

        /** templates for component */
        templates = {
            overlaySkeleton: [
                '<div class="husky-overlay-container <%= overflowClass %> <%= skin %> <%= cssClass %>">',
                '   <div class="slides"></div>',
                '</div>'
            ].join(''),
            slideSkeleton: [
                '<div class="slide slide-<%= index %> <%= cssClass %>">',
                '   <% if(displayHeader) { %>',
                '      <div class="overlay-header<% if(subTitle) { %> with-sub-title<% } %>">',
                '           <span class="title"><%= title %></span>',
                '           <% if(subTitle) { %><div class="sub-title"><%= subTitle %></div><% } %>',
                '       </div>',
                '   <% } %>',
                '   <div class="overlay-content <%= spacingClass %>"></div>',
                '   <div class="overlay-footer">',
                '   </div>',
                '</div>'
            ].join(''),
            okButton: [
                '<div class="btn action overlay-ok<%= classes %>">',
                '   <% if (!!icon) { %>',
                '   <span class="fa-<%= icon %>"></span>',
                '   <% } %>',
                '   <span class="text"><%= text %></span>',
                '</div>',
                '<div class="loader" style="display: none;"></div>'
            ].join(''),
            cancelButton: [
                '<div class="btn gray black-text overlay-cancel<%= classes %>">',
                '   <% if (!!icon) { %>',
                '   <span class="fa-<%= icon %>"></span>',
                '   <% } %>',
                '   <span class="text"><%= text %></span>',
                '</div>'
            ].join(''),
            button: [
                '<div class="btn overlay-button <%= classes %>" data-button-number="<%=buttonNumber%>">',
                '   <% if (!!icon) { %>',
                '   <span class="fa-<%= icon %>"></span>',
                '   <% } %>',
                '   <span class="text"><%= text %></span>',
                '</div>'
            ].join(''),
            wrapper: [
                '<div class="husky-overlay-wrapper">',
                '   <div class="husky-overlay-scroll-wrapper"></div>',
                '   <div class="husky-overlay-backdrop"></div>',
                '</div>'
            ].join(''),
            subheader: [
                '<div class="overlay-subheader">',
                '   <div class="overlay-tabs"></div>',
                '</div>'
            ].join(''),
            message: [
                '<div class="message"><%= message %></div>'
            ].join('')
        },

        /**
         * namespace for events
         * @type {string}
         */
        eventNamespace = 'husky.overlay.',

        /**
         * raised after initialization process
         * @event husky.overlay.<instance-name>.initialize
         */
        INITIALIZED = function() {
            return createEventName.call(this, 'initialized');
        },

        /**
         * raised after overlay is opened
         * @event husky.overlay.<instance-name>.opened
         */
        OPENED = function() {
            return createEventName.call(this, 'opened');
        },

        /**
         * raised after overlay is closed
         * @event husky.overlay.<instance-name>.closed
         */
        CLOSED = function() {
            return createEventName.call(this, 'closed');
        },

        /**
         * raised after overlay is closing
         * @event husky.overlay.<instance-name>.closing
         */
        CLOSING = function() {
            return createEventName.call(this, 'closing');
        },

        /**
         * used to activate all ok buttons
         * @event husky.overlay.<instance-name>.okbutton.activate
         */
        OKBUTTON_ACTIVATE = function() {
            return createEventName.call(this, 'okbutton.activate');
        },

        /**
         * used to deactivate all ok buttons
         * @event husky.overlay.<instance-name>.okbutton.deactivate
         */
        OKBUTTON_DEACTIVATE = function() {
            return createEventName.call(this, 'okbutton.deactivate');
        },

        /**
         * removes the component
         * @event husky.overlay.<instance-name>.remove
         */
        REMOVE = function() {
            return createEventName.call(this, 'remove');
        },

        /**
         * opens the overlay
         * @event husky.overlay.<instance-name>.open
         */
        OPEN = function() {
            return createEventName.call(this, 'open');
        },

        /**
         * closes the overlay
         * @event husky.overlay.<instance-name>.close
         */
        CLOSE = function() {
            return createEventName.call(this, 'close');
        },

        /**
         * show loader instead of save button
         * @event husky.overlay.<instance-name>.show-loader
         */
        SHOW_LOADER = function() {
            return createEventName.call(this, 'show-loader');
        },

        /**
         * hide loader and show of save button
         * @event husky.overlay.<instance-name>.hide-loader
         */
        HIDE_LOADER = function() {
            return createEventName.call(this, 'hide-loader');
        },

        /**
         * emited after the language changer is changed
         * @event husky.overlay.<instance-name>.language-changed
         * @param {String} selected language
         * @param {Object} currently active tab
         */
        LANGUAGE_CHANGED = function() {
            return createEventName.call(this, 'language-changed');
        },

        /**
         * slide left
         * @event husky.overlay.<instance-name>.slide-left
         */
        SLIDE_LEFT = function() {
            return createEventName.call(this, 'slide-left');
        },

        /**
         * slide right
         * @event husky.overlay.<instance-name>.slide-right
         */
        SLIDE_RIGHT = function() {
            return createEventName.call(this, 'slide-right');
        },

        /**
         * slide to
         * @param {Number} slide number
         * @event husky.overlay.<instance-name>.slide-to
         */
        SLIDE_TO = function() {
            return createEventName.call(this, 'slide-to');
        },

        /** returns normalized event names */
        createEventName = function(postFix) {
            return eventNamespace + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
        };

    return {

        /**
         * Initialize component
         */
        initialize: function() {
            this.sandbox.logger.log('initialize', this);

            var type = this.options.type || defaults.type;
            // merge defaults, type defaults and options
            this.options = this.sandbox.util.extend(true, {}, defaults, types[type], this.options);

            this.setVariables();
            this.initSlideOptions();
            this.bindEvents();

            if (this.options.openOnStart) {
                this.openOverlay();
            }
        },

        /**
         * Initiates slide options
         *  - default slide - options interface
         *  - for each slide default values
         */
        initSlideOptions: function() {
            var key, type;
            if (this.options.slides.length === 0) {
                // no slide given: extract it from options
                this.slides[0] = this.sandbox.util.extend({}, slideDefaults);
                this.slides[0].index = 0;
                for (key in slideDefaults) {
                    // check options for slide property
                    if (this.options.hasOwnProperty(key)) {
                        this.slides[0][key] = this.options[key];
                    }
                }

                type = types[this.slides[0].type];
                this.slides[0] = this.sandbox.util.extend({}, type, this.slides[0]);
            } else {
                // extend each slide with type and defaults
                this.sandbox.util.foreach(this.options.slides, function(value, i) {
                    this.slides[i] = this.sandbox.util.extend({}, slideDefaults, value);

                    type = types[this.slides[i].type];
                    this.slides[i] = this.sandbox.util.extend({}, type, this.slides[i]);
                    if (this.slides[i].index === -1) {
                        this.slides[i].index = i;
                    }
                }.bind(this));
            }
        },

        /**
         * Binds general events
         */
        bindEvents: function() {
            if (!!this.$trigger) {
                this.sandbox.dom.on(this.$trigger, this.options.trigger + '.overlay.' + this.options.instanceName, function(event) {
                    this.sandbox.dom.preventDefault(event);
                    this.triggerHandler();
                }.bind(this));
            }

            this.sandbox.on(REMOVE.call(this), this.removeComponent.bind(this));
            this.sandbox.on(OKBUTTON_ACTIVATE.call(this), this.activateOkButtons.bind(this));
            this.sandbox.on(OKBUTTON_DEACTIVATE.call(this), this.deactivateOkButtons.bind(this));
            this.sandbox.on(OPEN.call(this), this.triggerHandler.bind(this));
            this.sandbox.on(CLOSE.call(this), this.closeHandler.bind(this));

            // emit language-changed-event when language dropdown gets changed
            this.sandbox.on('husky.select.' + this.options.instanceName + '.selected.item', function(localeIndex) {
                this.sandbox.emit(LANGUAGE_CHANGED.call(this),
                    this.slides[this.activeSlide].languageChanger.locales[localeIndex], //selected locale
                    this.activeTab
                );
            }.bind(this));
        },

        /**
         * Activates all ok buttons
         */
        activateOkButtons: function() {
            var $okButtons = this.sandbox.dom.find(constants.overlayOkSelector, this.overlay.$footer),
                i, length;
            for (i = -1, length = $okButtons.length; ++i < length;) {
                this.sandbox.dom.removeClass($okButtons[i], 'inactive gray');
            }
        },

        /**
         * Deactivates all ok buttons
         */
        deactivateOkButtons: function() {
            var $okButtons = this.sandbox.dom.find(constants.overlayOkSelector, this.overlay.$footer),
                i, length;
            for (i = -1, length = $okButtons.length; ++i < length;) {
                this.sandbox.dom.addClass($okButtons[i], 'inactive gray');
            }
        },

        /**
         * Removes the component
         */
        removeComponent: function() {
            this.sandbox.stop();
        },

        /**
         * Sets the default properties
         */
        setVariables: function() {
            this.$trigger = null;
            if (!!this.options.triggerEl) {
                this.$trigger = this.sandbox.dom.$(this.options.triggerEl);
            }

            this.overlay = {
                opened: false,
                collapsed: false,
                normalHeight: null,
                $el: null,
                $slides: null,
                slides: []
            };
            this.$wrapper = null;
            this.$backdrop = null;
            this.activeTab = null;
            this.slides = [];
            this.activeSlide = 0;
        },

        /**
         * Event handler for the overlay-trigger event
         */
        triggerHandler: function() {
            this.openOverlay();
        },

        /**
         * Opens the overlay. If Overlay doesn't exist it first initializes it
         */
        openOverlay: function() {
            //only open if closed
            if (this.overlay.opened === false) {
                this.overlay.opened = true;
                if (this.$wrapper == null) {
                    this.initWrapper();
                }
                if (this.overlay.$el === null) {
                    this.initSkeleton();
                    this.bindOverlayEvents();
                    this.bindCustomEvents();

                    if (this.options.okInactive === true) {
                        this.deactivateOkButtons();
                    }

                    this.sandbox.emit(INITIALIZED.call(this));

                    this.overlay.$content = this.sandbox.dom.find(constants.contentSelector, this.overlay.$el);

                    this.insertOverlay(true);

                    if (!!this.options.startingSlide) {
                        this.slideTo(this.options.startingSlide, false);
                    }

                    this.sandbox.start([{name: 'loader@husky', options: {el: this.$el.find('.loader'), size: '30px'}}]);
                } else {
                    this.insertOverlay(true);
                }
            }
        },

        /**
         * bind custom events
         */
        bindCustomEvents: function() {
            this.sandbox.on(SLIDE_LEFT.call(this), this.slideLeft.bind(this));
            this.sandbox.on(SLIDE_RIGHT.call(this), this.slideRight.bind(this));
            this.sandbox.on(SLIDE_TO.call(this), this.slideEvent.bind(this));
            this.sandbox.on(SHOW_LOADER.call(this), this.showLoader.bind(this));
            this.sandbox.on(HIDE_LOADER.call(this), this.hideLoader.bind(this));
        },

        showLoader: function() {
            this.$el.find('.btn').hide();
            this.$el.find('.loader').show();
        },

        hideLoader: function() {
            this.$el.find('.btn').show();
            this.$el.find('.loader').hide();
        },

        /**
         * slide left
         */
        slideLeft: function() {
            var slide = this.activeSlide - 1;
            if (slide < 0) {
                slide = this.slides.length - 1;
            }

            this.slideTo(slide, true);
        },

        /**
         * slide right
         */
        slideRight: function() {
            var slide = this.activeSlide + 1;
            if (slide >= this.slides.length) {
                slide = 0;
            }

            this.slideTo(slide, true);
        },

        /**
         * slide to given slide number
         *
         * @param {Number} slide
         */
        slideEvent: function(slide) {
            if (slide < 0 || slide >= this.slides.length) {
                this.sandbox.logger.error('Slide index out bounds');

                return;
            }

            this.slideTo(slide, true);
        },

        /**
         * slide to given number
         */
        slideTo: function(slide, animated) {
            this.activeSlide = slide;

            var width = this.sandbox.dom.outerWidth(this.sandbox.dom.find('.slide', this.overlay.$slides));
            animated = (typeof animated === 'undefined') ? true : animated;

            if (!animated) {
                this.overlay.$slides.addClass('no-animation');
            }
            this.overlay.$slides.css('left', '-' + slide * width + 'px');
            if (!animated) {
                // The following line makes sure the browsers apply the change in `left` before the class gets removed
                // http://stackoverflow.com/questions/11131875/what-is-the-cleanest-way-to-disable-css-transition-effects-temporarily
                this.overlay.$slides[0].offsetHeight;
                this.overlay.$slides.removeClass('no-animation');
            }
        },

        /**
         * Initializes the wrapper
         */
        initWrapper: function() {
            this.$wrapper = this.sandbox.dom.createElement(templates.wrapper);
            this.$backdrop = this.$wrapper.find('.husky-overlay-backdrop');
            this.$wrapper.hide();
        },

        /**
         * Removes the overlay-element from the DOM
         */
        closeOverlay: function() {
            this.sandbox.emit(CLOSING.call(this));

            this.overlay.opened = false;

            this.sandbox.emit(CLOSED.call(this));

            this.sandbox.dom.off('body', 'keydown.' + this.options.instanceName);
            this.$wrapper.hide();
            if (!this.options.removeOnClose) {
                this.sandbox.dom.detach(this.overlay.$el);
            } else {
                this.removeComponent();
            }
        },

        /**
         * Inserts the overlay-element into the DOM
         */
        insertOverlay: function(emitEvent) {
            this.$wrapper.find('.husky-overlay-scroll-wrapper').append(this.overlay.$el);
            this.sandbox.dom.append(this.$el, this.$wrapper);
            this.$wrapper.show();

            if (!!emitEvent) {
                this.sandbox.emit(OPENED.call(this));
            }

            // listen on key-inputs
            if (this.options.supportKeyInput === true) {
                this.sandbox.dom.off('body', 'keydown.' + this.options.instanceName);
                this.sandbox.dom.on('body', 'keydown.' + this.options.instanceName, this.keyHandler.bind(this));
            }
        },

        /**
         * Creates the overlay-element with a skeleton-template
         */
        initSkeleton: function() {
            this.overlay.$el = this.sandbox.dom.createElement(
                this.sandbox.util.template(templates.overlaySkeleton,
                    {
                        skin: (!!this.options.skin) ? this.options.skin : '',
                        cssClass: (!!this.options.cssClass) ? this.options.cssClass : '',
                        overflowClass: (this.slides.length > 1) ? 'overflow-hidden' : ''
                    }
                )
            );

            this.overlay.$slides = this.sandbox.dom.find(constants.slidesSelector, this.overlay.$el);

            var slide, $el;
            for (slide in this.slides) {
                if (this.slides.hasOwnProperty(slide)) {
                    $el = this.initSlideSkeleton(slide);
                    this.initButtons(slide);
                    this.setContent(slide);
                    // render a language changer into the header if configured
                    if (!!this.slides[slide].languageChanger) {
                        this.renderLanguageChanger(slide);
                    }
                    this.sandbox.dom.append(this.overlay.$slides, $el);
                }
            }
        },

        initSlideSkeleton: function(slide) {
            this.overlay.slides[slide] = this.sandbox.util.extend({}, internalSlideDefaults);

            this.overlay.slides[slide].$el = this.sandbox.dom.createElement(
                this.sandbox.util.template(templates.slideSkeleton, {
                    title: this.sandbox.util.cropMiddle(this.slides[slide].title, 58),
                    subTitle: !!this.slides[slide].subTitle ? this.slides[slide].subTitle : null,
                    index: this.slides[slide].index,
                    cssClass: this.slides[slide].cssClass,
                    displayHeader: this.slides[slide].displayHeader,
                    spacingClass: (!!this.slides[slide].contentSpacing) ? 'content-spacing' : ''
                })
            );
            this.overlay.slides[slide].$footer = this.sandbox.dom.find(constants.footerSelector, this.overlay.slides[slide].$el);
            this.overlay.slides[slide].$content = this.sandbox.dom.find(constants.contentSelector, this.overlay.slides[slide].$el);
            this.overlay.slides[slide].$header = this.sandbox.dom.find(constants.headerSelector, this.overlay.slides[slide].$el);

            if (!!this.slides[slide].languageChanger || !!this.slides[slide].tabs || !!this.slides[slide].panelContent) {
                this.overlay.slides[slide].$subheader = $(templates.subheader);
                this.overlay.slides[slide].$header.after(this.overlay.slides[slide].$subheader)
            }

            if (!!this.slides[slide].languageChanger || !!this.slides[slide].panelContent) {
                this.overlay.slides[slide].$panel =  $('<div class="' + constants.panelClass + '"/>');
                this.overlay.slides[slide].$subheader.append(this.overlay.slides[slide].$panel);
            }

            if (!!this.slides[slide].panelContent) {
                this.overlay.slides[slide].$panel.append('<div class="panel-content"/>');
                this.overlay.slides[slide].$panel.find('.panel-content').append(this.slides[slide].panelContent);
            }

            // add classes for various styling
            this.sandbox.dom.addClass(this.overlay.slides[slide].$footer, this.options.buttonsDefaultAlign);
            this.sandbox.dom.addClass(this.overlay.slides[slide].$el, this.options.cssClass);

            return this.overlay.slides[slide].$el;
        },

        /**
         * Renders a language changer and places it within the header
         */
        renderLanguageChanger: function(slide) {
            var $element = this.sandbox.dom.createElement('<div class="language-changer"/>');
            this.overlay.slides[slide].$panel.append($element);

            this.sandbox.start([
                {
                    name: 'select@husky',
                    options: {
                        el: $element,
                        data: this.slides[slide].languageChanger.locales,
                        preSelectedElements: [this.slides[slide].languageChanger.preSelected],
                        instanceName: this.options.instanceName
                    }
                }
            ]);
        },

        /**
         * Renders all buttons and appends them to the footer
         */
        initButtons: function(slide) {
            var i, length, $button, button, template, classes, text, inactive;
            for (i = -1, length = this.slides[slide].buttons.length; ++i < length;) {
                button = this.slides[slide].buttons[i];
                if (button.type === buttonTypes.OK) {
                    template = templates.okButton;
                    text = this.slides[slide].okDefaultText;
                    inactive = this.slides[slide].okInactive;
                } else if (button.type === buttonTypes.CANCEL) {
                    template = templates.cancelButton;
                    text = this.slides[slide].cancelDefaultText;
                } else {
                    template = templates.button;
                }

                classes = (!!button.classes) ? ' ' + button.classes : '';

                if (!!button.text) {
                    text = button.text;
                } else if (!!button.icon) {
                    text = '';
                }

                if (inactive !== true) {
                    inactive = button.inactive;
                }

                $button = this.sandbox.dom.createElement(this.sandbox.util.template(template, {
                    icon: button.icon,
                    text: this.sandbox.translate(text),
                    buttonNumber: i,
                    classes: (inactive === true) ? classes + ' inactive gray' : classes
                }));

                // add individual button align (if configured)
                if (!!button.align) {
                    this.sandbox.dom.addClass($button, button.align);
                }

                this.sandbox.dom.append(this.overlay.slides[slide].$footer, $button);
            }
        },

        /**
         * Sets the content of the overlay. If the data option is set set it as raw html.
         * If the message option is set render a template with the message
         */
        setContent: function(slide) {
            if (!!this.slides[slide].data) {
                this.sandbox.dom.append(this.overlay.slides[slide].$content, this.slides[slide].data);
            } else if (!!this.slides[slide].message) {
                this.sandbox.dom.append(this.overlay.slides[slide].$content, this.sandbox.util.template(templates.message, {
                    message: this.slides[slide].message
                }));

            } else if (this.slides[slide].tabs !== null) {
                this.renderTabs(slide);
            } else {
                this.sandbox.logger.log('Error: either options.data, options.message or options.tabs has to be set', this);
            }
        },

        /**
         * Renders the tab-contents of of the overlay
         * and initializes the tab component
         */
        renderTabs: function(slide) {
            this.overlay.slides[slide].tabs = [];

            for (var i = -1, length = this.slides[slide].tabs.length; ++i < length;) {
                this.overlay.slides[slide].tabs.push({
                    id: i,
                    name: this.slides[slide].tabs[i].title,
                    $el: this.sandbox.dom.createElement(this.slides[slide].tabs[i].data)
                });
                this.sandbox.dom.hide(this.overlay.slides[slide].tabs[i].$el);
                this.sandbox.dom.append(this.overlay.slides[slide].$content, this.overlay.slides[slide].tabs[i].$el);
            }
            // show first tab element at the beginning and start tab-bar
            this.showTab(this.overlay.slides[slide].tabs[0], slide);
            this.startTabsComponent(slide);
        },

        /**
         * Starts the tabs-component
         */
        startTabsComponent: function(slide) {
            this.sandbox.start([
                {
                    name: 'tabs@husky',
                    options: {
                        el: this.overlay.slides[slide].$subheader.find('.overlay-tabs'),
                        data: this.overlay.slides[slide].tabs,
                        instanceName: 'overlay' + this.options.instanceName,
                        skin: 'overlay'
                    }
                }
            ]);
        },

        /**
         * Support for key down events
         * @param event
         */
        keyHandler: function(event) {
            // esc
            if (event.keyCode === 27) {
                this.closeHandler();
            } else if (event.keyCode === 13) {
                // when enter is pressed in textarea overlay should not be closed
                if (event.target.tagName === 'TEXTAREA') {
                    return;
                }
                this.okHandler();
            }
        },

        /**
         * Binds overlay events
         */
        bindOverlayEvents: function() {
            if (this.options.propagateEvents === false) {
                this.sandbox.dom.on(this.overlay.$el, 'click', function(event) {
                    this.sandbox.dom.stopPropagation(event);
                }.bind(this));
                this.sandbox.dom.on(this.$wrapper, 'click', function(event) {
                    this.sandbox.dom.stopPropagation(event);
                }.bind(this));
            }

            // close handler for cancel buttons
            this.sandbox.dom.on(this.overlay.$el, 'click',
                this.closeHandler.bind(this), constants.overlayCancelSelector);

            // binds the events for ok-buttons
            this.sandbox.dom.on(this.overlay.$el, 'click',
                this.okHandler.bind(this), constants.overlayOkSelector);

            // binds the events for other buttons
            this.sandbox.dom.on(this.overlay.$el, 'click',
                this.buttonHandler.bind(this), constants.overlayOtherButtonsSelector);

            if (this.options.backdropClose === true) {
                this.$backdrop.on('click', this.closeHandler.bind(this));
            }

            this.bindOverlayCustomEvents();
        },

        /**
         * Binds custom events used by the overlay
         */
        bindOverlayCustomEvents: function() {
            if (this.overlay.tabs !== null) {
                this.sandbox.on('husky.tabs.overlay' + this.options.instanceName + '.item.select', this.showTab.bind(this));
            }
        },

        /**
         * Handles the click on an overlay tab
         */
        showTab: function(tab, slide) {
            slide = slide || this.activeSlide;
            this.activeTab = tab;
            this.hideAllTabsElements(slide);
            this.sandbox.dom.show(tab.$el);
        },

        /**
         * Hides all tab elements
         */
        hideAllTabsElements: function(slide) {
            for (var i = -1, length = this.overlay.slides[slide].tabs.length; ++i < length;) {
                this.sandbox.dom.hide(this.overlay.slides[slide].tabs[i].$el);
            }
        },

        /**
         * Handles the click on ok-buttons
         * @param event
         */
        okHandler: function(event) {
            // do nothing, if button is inactive
            if (!!event && this.sandbox.dom.hasClass(event.currentTarget, 'inactive')) {
                return;
            }
            if (!!event) {
                this.sandbox.dom.preventDefault(event);
                this.sandbox.dom.stopPropagation(event);
            }

            if (this.executeCallback(
                    this.slides[this.activeSlide].okCallback,
                    this.sandbox.dom.find(constants.contentSelector, this.overlay.$el)
                ) !== false) {
                this.closeOverlay();
            }
        },

        /**
         * Handles the click on cancel-buttons and close-icon
         * @param event
         */
        closeHandler: function(event) {
            var cancelCallback = this.slides[this.activeSlide].closeCallback ||
                this.slides[this.activeSlide].cancelCallback;

            if (!!event) {
                this.sandbox.dom.preventDefault(event);
                this.sandbox.dom.stopPropagation(event);
            }
            if (this.executeCallback(
                    cancelCallback,
                    this.sandbox.dom.find(constants.contentSelector, this.overlay.$el)
                ) !== false) {
                this.closeOverlay();
            }
        },

        /**
         * Handles the click on a button
         * @param event
         */
        buttonHandler: function(event) {
            if (!event || this.sandbox.dom.hasClass(event.currentTarget, 'inactive')) {
                return;
            }

            var buttonNumber = this.sandbox.dom.data(event.currentTarget, 'buttonNumber'),
                button = this.slides[this.activeSlide].buttons[buttonNumber];

            this.sandbox.dom.preventDefault(event);
            this.sandbox.dom.stopPropagation(event);

            this.executeCallback(
                button.callback,
                this.sandbox.dom.find(constants.contentSelector, this.overlay.$el)
            );
        },

        /**
         * Executes a passed callback
         * @param callback {Function} callback to execute
         * @param data {Object} dom content element
         */
        executeCallback: function(callback, data) {
            if (typeof callback === 'function') {
                return callback(data);
            }
        }
    };
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/label
 */

/**
 * @class Label
 * @constructor
 *
 * @param {Object} [options] Configuration object
 * @param {String} [options.instanceName] name of the instance
 * @param {String} [options.type] type of the lable (WARNING, ERROR or SUCCESS)
 * @param {String|Object} [options.html] html-string or DOM-object to insert into the label
 * @param {String} [options.title] Title of the label (if html is null)
 * @param {Number} [options.counter] Counter to display in the label
 * @param {String} [options.description] Description of the lable (if html is null)
 * @param {Boolean} [options.hasClose] if true close button gets appended to the label
 * @param {String} [options.effectType] either 'slide' or 'fade'
 * @param {Boolean} [options.autoVanish] if true label vanishes automatically
 * @param {Number} [options.vanishDelay] time in ms after which the vanish effect starts
 * @param {Number} [options.vanishDuration] duration of the vanish effect in ms
 * @param {Number} [options.showDuration] duration of the show effect in ms
 * @param {Function} [options.closeCallback] callback to execute if the close-button is clicked
 * @param {String} [options.insertMethod] insert method to use for inserting the label (append or prepend)
 * @param {String} [options.additionalLabelClasses] Additional classes which will be appended to the label
 * @param {Array} [buttons] An array of buttons to add to the label
 * @param {String|Number} [buttons[].id] The id of the button
 * @param {String} [buttons[].title] The title to show in the button
 * @param {String} [buttons[].skin] The skin of the button e.g 'critical'
 * @param {Function} [buttons[].onClick] The callback to execute on click
 */
define('__component__$label@husky',[],function() {

    'use strict';

    var defaults = {
        instanceName: 'undefined',
        type: 'WARNING',
        html: null,
        title: null,
        counter: 1,
        description: null,
        hasClose: true,
        effectType: 'slide',
        autoVanish: true,
        vanishDelay: 0,
        vanishDuration: 250,
        showDuration: 250,
        closeCallback: null,
        insertMethod: 'append',
        additionalLabelClasses: 'big',
        buttons: []
    },

    insertMethods = {
        APPEND: 'append',
        PREPEND: 'prepend'
    },

    constants = {
        textClass: 'text',
        closeClass: 'close',
        counterClass: 'counter',
        closeIconClass: 'fa-times-circle',
        buttonsClass: 'buttons',
        buttonClass: 'button',
        loaderColor: '#666',
        isLoading: 'is-loading',
        loaderClass: 'loader'
    },

    /**
     * default values for types
     */
    typesDefaults = {
        ERROR: {
            title: 'Error',
            labelClass: 'husky-label-error',
            vanishDelay: 10000
        },
        WARNING: {
            vanishDelay: 5000,
            title: 'Warning',
            labelClass: 'husky-label-warning'
        },
        SUCCESS: {
            vanishDelay: 5000,
            title: 'Success',
            labelClass: 'husky-label-success'
        },
        SUCCESS_ICON: {
            labelClass: 'husky-label-success-icon',
            effectType: 'fade',
            hasClose: false,
            vanishDelay: 2000,
            showDuration: 100
        }
    },

    /**
     * generates template template
     */
    templates = {
        basic: [
                '<div class="' + constants.closeClass + '">',
                '   <% if (!!hasClose) { %><span class="' + constants.closeIconClass + '"></span><% } %>',
                '</div>',
                '<div class="' + constants.textClass + '">',
                '   <% if (!!title) { %><strong><%= title %></strong><% } %>',
                '   <span><%= description %></span>',
                '   <div class="' + constants.counterClass + '"><span><%= counter %></span></div>',
                '</div>'].join(''),
        buttons: '<div class="' + constants.buttonsClass + '"></div>',
        button: '<div class="' + constants.buttonClass + '"><%= title %></div>'
    },

    eventNamespace = 'husky.label.',

    /**
     * raised after initialization process
     * @event husky.label.[INSTANCE_NAME].initialized
     */
    INITIALIZED = function() {
        return createEventName.call(this, 'initialized');
    },

    /**
     * raised before destroy process
     * @event husky.label.[INSTANCE_NAME].destroyed
     */
    DESTROYED = function() {
        return createEventName.call(this, 'destroyed');
    },

    /**
     * listens on and refreshes the vanish-out delay
     * @event husky.label.[INSTANCE_NAME].refresh
     * @param {String|Number} counter The counter number to display
     */
    REFRESH = function() {
        return createEventName.call(this, 'refresh');
    },

    /**
     * listens on and vanishes the label
     * @event husky.label.[INSTANCE_NAME].vanish
     */
    VANISH = function() {
        return createEventName.call(this, 'vanish');
    },

    /**
     * listens on and sets the label into loading state
     * @event husky.label.[INSTANCE_NAME].loading
     */
    LOADING = function() {
        return createEventName.call(this, 'loading');
    },

    /**
     * listens on and resets the state of the label (not loading)
     * @event husky.label.[INSTANCE_NAME].reset
     */
    RESET = function() {
        return createEventName.call(this, 'reset');
    },

    /** returns normalized event names */
    createEventName = function(postFix) {
        return eventNamespace + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
    };

    return {

        /**
         * Initialize the component
         */
        initialize: function() {

            //merge defaults with defaults of type and options
            this.options = this.sandbox.util.extend(true, {}, defaults, typesDefaults[this.options.type], this.options);
            this.vanishTimer = null;
            this.label = {
                $el: null,
                $content: null,
                $close: null,
                $buttons: null
            };
            this.buttons = {};

            this.bindCustomEvents();
            this.render();
            this.bindDomEvents();
            this.startEffects();

            this.sandbox.emit(INITIALIZED.call(this));
        },

        /**
         * Destroy the component (aura hook)
         */
        destroy: function() {
            this.sandbox.emit(DESTROYED.call(this));
        },

        /**
         * Binds the events for the component
         */
        bindDomEvents: function() {
            this.sandbox.dom.on(this.label.$close, 'click', function() {
                this.vanish();
                if (typeof this.options.closeCallback === 'function') {
                    this.options.closeCallback();
                }
            }.bind(this));
        },

        /**
         * Bind custom aura events
         */
        bindCustomEvents: function() {
            this.sandbox.on(REFRESH.call(this), this.refresh.bind(this));
            this.sandbox.on(LOADING.call(this), this.setLoadingState.bind(this));
            this.sandbox.on(RESET.call(this), this.resetState.bind(this));
            this.sandbox.on(VANISH.call(this), this.vanish.bind(this));
        },

        /**
         * Refreshes the vanish and rerenders the counter
         */
        refresh: function() {
            this.options.counter += 1;
            this.abortEffects();
            this.label.$el.find('.' + constants.counterClass + ' span').html(this.options.counter);
            this.updateCounterVisibility();
            this.startEffects();
        },

        /**
         * Sets the label into loading state.
         */
        setLoadingState: function() {
            var $loader = $('<div/>');
            $loader.addClass(constants.loaderClass);
            this.label.$el.append($loader);
            this.label.$el.addClass(constants.isLoading);

            this.sandbox.start([{
                name: 'loader@husky',
                options: {
                    el: $loader,
                    size: '20px',
                    color: constants.loaderColor
                }
            }]);
        },

        /**
         * Resets the state of the label (not loading)
         */
        resetState: function() {
            this.sandbox.stop(this.label.$el.find('*'));
            this.label.$el.removeClass(constants.isLoading);
        },

        /**
         * Starts the vanish effect
         */
        startEffects: function() {
            if (this.options.autoVanish === true) {
                this.vanishTimer = _.delay(function() {
                    this.vanish();
                }.bind(this), this.options.vanishDelay);
            }
        },

        /**
         * Cancels the vanish effect
         */
        abortEffects: function() {
            this.label.$el.stop();
            this.label.$el.removeAttr('style');
            clearTimeout(this.vanishTimer);
        },

        /**
         * Makes the label disapear
         */
        vanish: function() {
            if (this.options.effectType === 'slide') {
                this.label.$el.slideUp({
                    duration: this.options.vanishDuration,
                    done: this.close.bind(this)
                });
            } else if (this.options.effectType === 'fade') {
                this.label.$el.fadeOut({
                    duration: this.options.vanishDuration,
                    done: this.close.bind(this)
                });
            }
        },

        /**
         * Makes the label appear
         */
        show: function() {
            if (this.options.effectType === 'slide') {
                this.label.$el.slideDown({
                    duration: this.options.showDuration
                });
            } else if (this.options.effectType === 'fade') {
                this.label.$el.fadeIn({
                    duration: this.options.showDuration
                });
            }
        },

        /**
         * Renders the component
         */
        render: function() {
            this.renderElement();
            this.renderContent();
            this.renderButtons();

            this.updateCounterVisibility();
            this.insertLabel();
            this.show();
        },

        /**
         * Renders the main element
         */
        renderElement: function() {
            this.label.$el = this.sandbox.dom.createElement(
                '<div class="' + this.options.labelClass
                    + (!!this.options.additionalLabelClasses ? ' ' + this.options.additionalLabelClasses : '')
                    + '"/>'
            );
            this.label.$el.hide();
        },

        /**
         * Renders the content of the component
         */
        renderContent: function() {
            if (this.options.html !== null) {
                this.label.$content = this.sandbox.dom.createElement(this.options.html);
            } else {
                this.label.$content = this.sandbox.dom.createElement(this.sandbox.util.template(templates.basic, {
                    title: this.sandbox.translate(this.options.title),
                    description: this.sandbox.translate(this.options.description),
                    counter: this.options.counter,
                    hasClose: this.options.hasClose
                }));
            }

            this.label.$close = this.label.$content.find('.' + constants.closeIconClass);

            //append content to main element
            this.sandbox.dom.append(this.label.$el, this.label.$content);
        },

        /**
         * Renders the buttons of the label
         */
        renderButtons: function() {
            if (this.options.buttons.length > 0) {
                this.label.$buttons = $(templates.buttons);
                this.label.$el.append(this.label.$buttons);
                this.sandbox.util.foreach(this.options.buttons, this.renderButton.bind(this));
            }
        },

        /**
         * Renders a single button.
         * Precondition: label.$buttons has to be rendered.
         *
         * @param {Object} button The button data from which the element should be rendered
         */
        renderButton: function(button) {
            this.buttons[button.id] = button;

            this.buttons[button.id].$el = $(_.template(templates.button, {
                title: button.title
            }));
            if (!!button.skin) {
                this.buttons[button.id].$el.addClass(button.skin);
            }
            this.label.$buttons.append(this.buttons[button.id].$el);
            if (!!button.onClick) {
                this.buttons[button.id].$el.on('click', function() {
                    this.buttons[button.id].onClick();
                }.bind(this));
            }
        },

        /**
         * Hides or shows the counter-object
         */
        updateCounterVisibility: function() {
            if (this.options.counter > 1) {
                this.sandbox.dom.show(this.label.$el.find('.' + constants.counterClass));
            } else {
                this.sandbox.dom.hide(this.label.$el.find('.' + constants.counterClass));
            }
        },

        /**
         * Inserts the label into the DOM
         */
        insertLabel: function() {
            if (this.options.insertMethod === insertMethods.APPEND) {
                this.sandbox.dom.append(this.$el, this.label.$el);
            } else if (this.options.insertMethod === insertMethods.PREPEND) {
                this.sandbox.dom.prepend(this.$el, this.label.$el);
            } else {
                this.sandbox.logger.log('No insert method found for', this.options.insertMethod);
            }
        },

        /**
         * Handles closing the component
         */
        close: function() {
            this.sandbox.stop();
        }
    };

});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/process
 */

/**
 * @class Toggler
 * @constructor
 *
 * @params {Object} [options] Configuration object
 * @params {Boolean} [options.instanceName] name of the component instance, gets used for events and the checkbox name. Can also be set through the DOM-id
 * @params {Boolean} [options.checked] beginning state of the button
 * @params {Boolean} [options.outline] if true component gets a bright border
 */
define('__component__$toggler@husky',[], function() {

    'use strict';

    var defaults = {
            instanceName: 'undefined',
            hiddenCheckbox: true,
            outline: false
        },

        constants = {
            componentClass: 'husky-toggler',
            switchClass: 'switch',
            checkedClass: 'checked',
            wrapperClass: 'toggler-wrapper',
            outlineClass: 'outline'
        },

        /**
         * namespace for events
         * @type {string}
         */
        eventNamespace = 'husky.toggler.',

        /**
         * raised after initialization process
         * @event husky.toggler.<instance-name>.initialize
         */
        INITIALIZED = function() {
            return createEventName.call(this, 'initialized');
        },

        /**
         * raised after initialization process
         * @event husky.toggler.<instance-name>.changed
         * @param {boolean} on True if button is on
         */
        CHANGED = function() {
            return createEventName.call(this, 'changed');
        },

        /**
         * event to turn the button on or off
         * @event husky.toggler.<instance-name>.change
         * @param {boolean} on To turn the button on or off
         */
        CHANGE = function() {
            return createEventName.call(this, 'change');
        },

        /** returns normalized event names */
        createEventName = function(postFix) {
            return eventNamespace + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
        };

    return {

        /**
         * Initialize component
         */
        initialize: function() {
            //merge options with defaults
            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

            this.hasId = false;
            this.$checkbox = null;
            this.$wrapper = null;

            this.render();
            this.bindDomEvents();
            this.bindCustomEvents();

            if (this.$el.data('checked') === undefined) {
                this.setData(false);
            }

            this.sandbox.emit(INITIALIZED.call(this));
        },

        /**
         * Renders the component
         */
        render: function() {
            this.sandbox.dom.addClass(this.$el, constants.componentClass);

            if (this.sandbox.dom.attr(this.$el, 'id')) {
                this.options.instanceName = this.sandbox.dom.attr(this.$el, 'id');
                this.hasId = true;
            }

            if (this.options.outline === true) {
                this.sandbox.dom.addClass(this.$el, constants.outlineClass);
            }

            //wrapper to insert all content into
            this.$wrapper = this.sandbox.dom.createElement('<div class="' + constants.wrapperClass + '"/>');
            this.sandbox.dom.html(this.$el, this.$wrapper);

            this.generateHiddenCheckbox();

            if (this.getData() === true) {
                this.setChecked(false);
            }

            this.sandbox.dom.append(this.$wrapper, this.sandbox.dom.createElement('<div class="' + constants.switchClass + '"/>'));
        },

        /**
         * Binds the DOM related Events
         */
        bindDomEvents: function() {
            this.sandbox.dom.on(this.$el, 'click', function() {
                this.toggleButton();
            }.bind(this));

            //if toggler has id enable clicks on labels
            if (this.hasId === true) {
                this.sandbox.dom.on('label[for="' + this.options.instanceName + '"]', 'click', function() {
                    this.toggleButton();
                }.bind(this));
            }
        },

        /**
         * Binds custom events
         */
        bindCustomEvents: function() {
            this.sandbox.on(CHANGE.call(this), function(checked) {
                if (this.getData() !== checked) {
                    if (checked === true) {
                        this.setChecked(true);
                    } else {
                        this.unsetChecked(true);
                    }
                }
            }.bind(this));
        },

        /**
         * Toggles the button state
         */
        toggleButton: function() {
            if (this.getData() === true) {
                this.unsetChecked(true);
            } else {
                this.setChecked(true);
            }
        },

        /**
         * Switches the toggler on
         */
        setChecked: function(emit) {
            this.sandbox.dom.addClass(this.$el, constants.checkedClass);
            this.setData(true);
            this.updateCheckbox(this.getData());

            if (emit !== false) {
                this.sandbox.emit(CHANGED.call(this), true);
            }
        },

        /**
         * Switches the toggler off
         */
        unsetChecked: function(emit) {
            this.sandbox.dom.removeClass(this.$el, constants.checkedClass);
            this.setData(false);
            this.updateCheckbox(this.getData());

            if (emit !== false) {
                this.sandbox.emit(CHANGED.call(this), false);
            }
        },

        /**
         * Generates a hidden checkbox useful using toggler in forms
         */
        generateHiddenCheckbox: function() {
            if (this.options.hiddenCheckbox === true) {
                this.$checkbox = this.sandbox.dom.createElement('<input type="checkbox" name="' + this.options.instanceName + '"/>');
                this.sandbox.dom.append(this.$wrapper, this.$checkbox);
            }
        },

        /**
         * Updates the state of the checkbox
         * @param {boolean} checked True to set the sandbox checked, false to uncheck
         */
        updateCheckbox: function(checked) {
            if (this.$checkbox !== null) {
                this.sandbox.dom.prop(this.$checkbox, 'checked', checked);
            }
        },

        /**
         * Sets the data-checked attribute for the toggler
         */
        setData: function(isChecked) {
            this.$el.data('checked', !!isChecked);
        },

        /**
         * Returns the data-checked attribute for the toggler
         * @returns {boolean}
         */
        getData: function() {
            return this.$el.data('checked');
        }
    };
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/dropzone
 */

/**
 * @class Dropzone
 * @constructor
 *
 * @params {Object} [options] Configuration object
 * @params {String} [options.titleKey] The title to display if no items are dropped (can be a translation key)
 * @params {String} [options.descriptionKey] The description to display if no items are dropped (can be a translation key)
 * @params {String} [options.descriptionIcon] The icon to display if no items are dropped
 * @params {String} [options.cancelLoadingIcon] the icon which gets displayed while a file item gets uploaded
 * @params {String} [options.method] method to use for uploading 'PUT' or 'POST'
 * @params {String} [options.url] url to upload the files to
 * @params {String} [options.paramName] the name of the parameter which will contain the file(s). Note that if uploadMultiple is set to true the parameter Name gets extended with '[]'
 * @params {Object} [options.headers] additional headers to pass with each request
 * @params {Boolean} [options.uploadMultiple] if true a request can upload multiple files
 * @params {Number} [options.maxFiles] defines the maximum of files in the dropzone
 * @params {Function} [options.successCallback] callback which gets called if a file got successfully uploaded. First parameter is the file, the second the response
 * @params {Function} [options.beforeSendingCallback] callback which gets called before a file gets uploaded. First parameter is the file.
 * @params {Function} [options.removeFileCallback] callback which gets called after a file got removed. First parameter is the file.
 * @params {Function} [options.afterDropCallback] callback which gets called after a file got dropped. Has to return a promise. If the promise gets resolved the file gets uploaded
 * @params {Object} [options.pluginOptions] Options to pass to the dropzone-plugin to completely override all options set by husky. Use with care.
 * @params {Boolean} [options.showOverlay] if true the dropzone will be displayed in an overlay. The overlay gets opened when a file gets dragged onto the document
 * @params {String|Object} [options.overlayContainer] The selector or the dom element which will be the direct parent of the overlay.
 * @params {String} [options.skin] skin class for the dropzone. currently available: 'overlay', 'small' or '' (default)
 * @params {Boolean} [options.keepFilesAfterSuccess] True to not slide the files away after uploading them successfully
 * @params {Boolean} [options.dropzoneEnabled] Should the dropzone be enabled initially
 * @params {Boolean} [options.cancelUploadOnOverlayClose] Cancel the upload process when the user clicks on the overlay background
 * @params {Number} [options.maxFilesize] Maximum file size in mb
 * @params {String} [options.fileTooBigKey] Translation key for a file which is to big
 */
define('__component__$dropzone@husky',[], function() {

    'use strict';

    var defaults = {
            instanceName: 'undefined',
            titleKey: 'sulu.upload.dropzone-title',
            descriptionKey: 'sulu.upload.dropzone-desc',
            descriptionIcon: 'cloud-upload',
            cancelLoadingIcon: 'coffee',
            method: 'POST',
            url: '/',
            paramName: 'file',
            headers: {
                // to prevent a 406 error-response
                'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8'
            },
            uploadMultiple: false,
            successCallback: null,
            beforeSendingCallback: null,
            removeFileCallback: null,
            afterDropCallback: null,
            pluginOptions: {},
            maxFiles: null,
            fadeOutDuration: 200, //ms
            fadeOutDelay: 1500, //ms
            showOverlay: true,
            overlayContainer: 'body',
            keepFilesAfterSuccess: false,
            skin: '',
            dropzoneEnabled: true,
            cancelUploadOnOverlayClose: false,
            maxFilesize: 256, // mb
            fileTooBigKey: 'husky.upload.error.file-to-big' // can handle {{filesize}}, {{maxFilesize}}, {{filename}}
        },

        constants = {
            overlayClass: 'dropzone-overlay',
            contianerClass: 'husky-dropzone',
            descriptionClass: 'description',
            loaderClass: 'dropzone-loader',
            uploadedItemContainerClass: 'upload-items',
            uploadItemClass: 'item',
            droppedClass: 'dropped'
        },

        /** templates for component */
        templates = {
            basic: [
                '<div class="dropzone-container">',
                '   <div class="' + constants.descriptionClass + '">',
                        '<div class="fa-<%= icon %> icon"></div>',
                        '<span class="title"><%= title %></span>',
                        '<span class="addition"><%= description %></span>',
                '   </div>',
                '   <div class="' + constants.uploadedItemContainerClass + '"></div>',
                '</div>'
            ].join(''),
            uploadItem: [
                '<div class="' + constants.uploadItemClass + '">' +
                    '<div class="loading-content">',
                        '<div class="fa-<%= cancelIcon %> icon" data-dz-remove></div>',
                        '<div class="file-size" data-dz-size></div>',
                        '<div class="progress">',
                            '<div class="bar" data-dz-uploadprogress></div>',
                        '</div>',
                    '</div>',
                    '<div class="success-content">',
                        '<div class="image">',
                            '<img data-dz-thumbnail />',
                        '</div>',
                        '<div class="fa-check tick"></div>',
                    '</div>',
                '</div>'
            ].join('')
        },

        /**
         * namespace for events
         * @type {string}
         */
        eventNamespace = 'husky.dropzone.',

        /**
         * raised after initialization process
         * @event husky.dropzone.<instance-name>.initialize
         */
        INITIALIZED = function() {
            return createEventName.call(this, 'initialized');
        },

        /**
         * raised before sending a file
         * @event husky.dropzone.<instance-name>.uploading
         * @param {Object} the file
         */
        UPLOADING = function() {
            return createEventName.call(this, 'uploading');
        },

        /**
         * raised after file got successfully uploaded
         * @event husky.dropzone.<instance-name>.success
         * @param {Object} the file
         * @param {Object} the response
         */
        SUCCESS = function() {
            return createEventName.call(this, 'success');
        },

        /**
         * raised when an error occurs because of a to big file
         * @event husky.dropzone.<instance-name>.error
         */
        ERROR_FILE_TO_BIG = function() {
            return createEventName.call(this, 'error.file-to-big', true);
        },

        /**
         * raised when an error occurs
         * @event husky.dropzone.<instance-name>.error
         */
        ERROR = function() {
            return createEventName.call(this, 'error', true);
        },

        /**
         * listens on and opens the data-source folder-overlay
         * @event husky.dropzone.<instance-name>.open-data-source
         */
        OPEN_DATA_SOURCE = function() {
            return createEventName.call(this, 'open-data-source');
        },

        /**
         * listens on and shows dropzone popup
         * @event husky.dropzone.<instance-name>.show-popup
         */
        SHOW_POPUP = function() {
            return createEventName.call(this, 'show-popup');
        },

        /**
         * raised after files got uploaded and faded out from the dropzone
         * @event husky.dropzone.<instance-name>.files-added
         * @param {Array} all newly added files
         */
        FILES_ADDED = function() {
            return createEventName.call(this, 'files-added');
        },

        /**
         * listens on and changes the url of the dropzone
         * @event husky.dropzone.<instance-name>.change-url
         * @param {String} the new url
         */
        CHANGE_URL = function() {
            return createEventName.call(this, 'change-url');
        },

        /**
         * listens on and enable the dropzone
         * @event husky.dropzone.<instance-name>.enable
         */
        UPLOAD_ENABLE = function() {
            return createEventName.call(this, 'enable');
        },

        /**
         * listens on and disable the dropzone
         * @event husky.dropzone.<instance-name>.disable
         */
        UPLOAD_DISABLE = function() {
            return createEventName.call(this, 'disable');
        },

        /**
         * listens on and adds an image to the dropzone by a given url
         * @event husky.dropzone.<instance-name>.add-file
         * @param {String} url The url to the file to add
         */
        ADD_IMAGE = function() {
            return createEventName.call(this, 'add-image');
        },

        /** returns normalized event names */
        createEventName = function(postFix, global) {
            return [
                eventNamespace,
                ((!global && this.options.instanceName) ? this.options.instanceName + '.' : ''),
                postFix
            ].join('');
        },

        createJQueryEventName = function(event) {
            return event + '.dropzone.' + (this.options.instanceName ? this.options.instanceName : '');
        };

    return {

        /**
         * Initialize component
         */
        initialize: function() {
            this.sandbox.logger.log('initialize', this);

            // merge defaults, type defaults and options
            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);
            this.dropzone = null;
            this.dropzoneEnabled = this.options.dropzoneEnabled;
            this.$dropzone = null;
            this.$loader = null;
            this.lastUploadedFile = null;
            this.overlayOpened = false;
            this.url = this.options.url;
            this.filesDropped = 0;

            this.bindCustomEvents();
            this.render();
            this.bindDomEvents();

            this.sandbox.emit(INITIALIZED.call(this));
        },

        /**
         * Handler which gets called when the component gets destroyed.
         */
        destroy: function() {
            this.sandbox.stop(this.$loader);
            $('body').off('.dropzone' + this.options.instanceName);
            this.dropzone.destroy();
            this.$dropzone.remove();
        },

        /**
         * Binds dom related events
         */
        bindDomEvents: function() {
            this.$dropzone.find('.' + constants.descriptionClass).on('click', function(event) {
                event.stopPropagation();
            }.bind(this));

            if (this.options.showOverlay) {
                this.sandbox.dom.on(this.sandbox.dom.$document, createJQueryEventName.call(this, 'dragenter'), function() {
                    this.openOverlay();
                }.bind(this));
                this.sandbox.dom.on(this.sandbox.dom.$document, createJQueryEventName.call(this, 'drop'), function(event) {
                    if (this.dropzoneEnabled) {
                        this.addFiles(event.originalEvent.dataTransfer.files);
                    }
                }.bind(this));
                this.$dropzone.on('click', function() {
                    if (!!this.options.cancelUploadOnOverlayClose) {
                        this.sandbox.dom.removeClass(this.$dropzone, constants.droppedClass);
                        this.dropzone.removeAllFiles();
                    }
                    this.closeOverlay();
                }.bind(this));
            }
            this.sandbox.dom.on(this.sandbox.dom.$document,
                createJQueryEventName.call(this, 'dragover') + ' ' + createJQueryEventName.call(this, 'drop'),
                function(event) {
                    this.sandbox.dom.preventDefault(event);
                }.bind(this));

            $('body').on('keydown.dropzone' + this.options.instanceName, function(event) {
                // close overlay when esc pressed
                if (event.keyCode === 27) {
                    this.closeOverlay();
                }
            }.bind(this));
        },

        /**
         * Binds custom-related events
         */
        bindCustomEvents: function() {
            // opens the data-source folder-overlay
            this.sandbox.on(OPEN_DATA_SOURCE.call(this), function() {
                this.$dropzone.find('.' + constants.descriptionClass).trigger('click');
            }.bind(this));

            // change the url
            this.sandbox.on(CHANGE_URL.call(this), function(url) {
                this.url = url;
            }.bind(this));

            this.sandbox.on(UPLOAD_DISABLE.call(this), function() {
                this.dropzoneEnabled = false;
                this.dropzone.disable();
            }.bind(this));

            this.sandbox.on(UPLOAD_ENABLE.call(this), function() {
                this.dropzoneEnabled = true;
                this.dropzone.enable();
            }.bind(this));

            if (this.options.showOverlay) {
                this.sandbox.on(SHOW_POPUP.call(this), function() {
                    this.openOverlay();
                }.bind(this));
            }

            this.sandbox.on(ADD_IMAGE.call(this), this.addImage.bind(this));
        },

        /**
         * Adds an image to the the dropzone by a given url.
         *
         * @param {String} url The url to first load the image from and than upload it via the dropzone
         * @param {string} mimeType The mime type of the loaded image
         * @param {string} fileName The name under which the file should be saved
         */
        addImage: function(url, mimeType, fileName) {
            this.sandbox.util.loadImageAsBlob(url, mimeType).then(function(imageBlob) {
                if (!!fileName) {
                    imageBlob.name = fileName;
                }
                this.dropzone.addFile(imageBlob, mimeType);
            }.bind(this));
        },

        /**
         * Opens the dropzone in an overlay
         */
        openOverlay: function() {
            // open the overlay only if it's not already opened
            if (this.overlayOpened === false && !!this.dropzoneEnabled) {
                this.$dropzone.show();
                this.overlayOpened = true;
            }
        },

        /**
         * Closes the dropzone overlay
         */
        closeOverlay: function() {
            if (this.overlayOpened === true) {
                this.$dropzone.hide();
                this.overlayOpened = false;
            }
        },

        /**
         * Renders the component
         */
        render: function() {
            this.$dropzone = this.sandbox.dom.createElement('<div class="' + constants.contianerClass + '"/>');
            this.sandbox.dom.html(this.$dropzone, this.sandbox.util.template(templates.basic)({
                description: this.sandbox.translate(this.options.descriptionKey),
                title: this.sandbox.translate(this.options.titleKey),
                icon: this.options.descriptionIcon,
                instanceName: this.options.instanceName
            }));
            this.$dropzone.addClass(this.options.skin);

            if (!!this.options.showOverlay) {
                this.$dropzone.addClass(constants.overlayClass);
                this.$dropzone.hide();
            }

            if (!this.options.showOverlay || !this.options.overlayContainer) {
                this.$el.append(this.$dropzone);
            } else {
                $(this.options.overlayContainer).append(this.$dropzone);
            }
            this.renderLoader();
            this.startDropzone();
        },

        /**
         * Renderes the loader which gets dipslayed when uploading images
         */
        renderLoader: function() {
            this.$loader = $('<div class="' + constants.loaderClass + '"/>');
            this.$dropzone.prepend(this.$loader);

            this.sandbox.start([
                {
                    name: 'loader@husky',
                    options: {
                        el: this.$loader,
                        size: '40px',
                        color: '#ffffff'
                    }
                }
            ]);
        },

        /**
         * Starts the dropzone component
         */
        startDropzone: function() {
            var that = this,
                options = {
                    url: this.options.url,
                    method: this.options.method,
                    paramName: this.options.paramName,
                    uploadMultiple: this.options.uploadMultiple,
                    maxFiles: this.options.maxFiles,
                    maxFilesize: this.options.maxFilesize,
                    dictFileTooBig: this.sandbox.translate(this.options.fileTooBigKey),
                    headers: this.options.headers,
                    autoProcessQueue: !this.options.afterDropCallback,
                    previewTemplate: this.sandbox.util.template(templates.uploadItem)({
                        cancelIcon: this.options.cancelLoadingIcon
                    }),
                    clickable: this.$dropzone.find('.' + constants.descriptionClass).get(0),
                    previewsContainer: this.sandbox.dom.find('.' + constants.uploadedItemContainerClass, this.$dropzone)[0],
                    init: function() {
                        // store dropzone context
                        that.dropzone = this;

                        this.on('drop', function(event) {
                            this.sandbox.dom.stopPropagation(event);
                            this.filesDropped = event.dataTransfer.files.length;
                        }.bind(that));

                        // gets called for each added file (drop or via the upload window)
                        this.on('addedfile', function(file) {
                            that.sandbox.dom.addClass(that.$dropzone, constants.droppedClass);

                            // call the after-drop callback on the last file
                            if (typeof that.options.afterDropCallback === 'function') {
                                if (this.files.length === that.filesDropped || that.filesDropped === 0) { // if filesDropped is 0 the file(s) werent dropped but added via the popup window
                                    that.options.afterDropCallback(file).then(function() {
                                        that.sandbox.util.foreach(this.files, function(file) {
                                            that.sandbox.util.delay(this.processFile.bind(this, file), 0);
                                        }.bind(this));
                                    }.bind(this));
                                }
                            }

                            // prevent the the upload window to open on click on the preview item
                            that.sandbox.dom.on(file.previewElement, 'click', function(event) {
                                this.sandbox.dom.stopPropagation(event);
                            }.bind(that));
                        });

                        // gets called before each file is sent
                        this.on('sending', function(file) {
                            if (typeof this.options.beforeSendingCallback === 'function') {
                                this.options.beforeSendingCallback(file);
                            } else {
                                this.sandbox.emit(UPLOADING.call(this), file);
                            }
                        }.bind(that));

                        // gets called if the file was uploaded successfully
                        this.on('success', function(file, response) {
                            if (this.lastUploadedFile !== file) {
                                file.response = response;
                                this.lastUploadedFile = file;
                                if (typeof this.options.successCallback === 'function') {
                                    this.options.successCallback(file, response);
                                } else {
                                    this.sandbox.emit(SUCCESS.call(this), file, response);
                                }
                                this.removeAllFiles(this.options.keepFilesAfterSuccess);
                            }
                        }.bind(that));

                        // gets called if a file gets removed from the zone
                        this.on('removedfile', function(file) {
                            if (typeof this.options.removeFileCallback === 'function') {
                                this.options.removeFileCallback(file);
                            }
                        }.bind(that));

                        // gets called if all files are removed from the zone
                        this.on('reset', function() {
                            if (this.options.keepFilesAfterSuccess === false) {
                                this.sandbox.dom.removeClass(this.$dropzone, constants.droppedClass);
                            }
                        }.bind(that));

                        // enables the to change the url dynamically
                        this.on('processing', function() {
                            this.options.url = that.url;
                        });

                        // enables external error handling
                        this.on('error', function(file, message) {
                            this.removeFile(file);

                            if (file.size / 1024 > that.options.maxFilesize || file.xhr.status === 413) {
                                that.sandbox.emit(
                                    ERROR_FILE_TO_BIG.call(this),
                                    that.getMessage.call(that, that.options.fileTooBigKey, file),
                                    file
                                );
                            } else {
                                that.sandbox.emit(ERROR.call(this), message, file);
                            }
                        });
                    }
                };

            // merge the default plugin options with with passed ones
            options = this.sandbox.util.extend(true, {}, options, this.options.pluginOptions);
            this.sandbox.dropzone.initialize(this.$dropzone, options);
        },

        /**
         * Prepares message with placeholders.
         * @param {String} message
         * @param {{size, name}} file
         * @returns {String}
         */
        getMessage: function(message, file) {
            return this.sandbox.translate(message).replace('{{filesize}}', Math.round((file.size / 1048576) * 100) / 100)
                .replace('{{maxFilesize}}', this.options.maxFilesize)
                .replace('{{filename}}', this.sandbox.util.cropMiddle(file.name, 20));
        },

        /**
         * Adds an array of files to the dropzone to upload them
         * @param files {array} an array of files
         */
        addFiles: function(files) {
            this.sandbox.util.each(files, function(index, file) {
                this.dropzone.addFile(file);
            }.bind(this));
        },

        /**
         * Removes all files from the dropzone
         * but only if all dropped files got uploaded
         * @param keepDom {Boolean} true to keep the dom like it is
         */
        removeAllFiles: function(keepDom) {
            // if all files got uploaded
            if (this.dropzone.getUploadingFiles.call(this.dropzone).length === 0) {
                if (keepDom === true) {
                    this.afterFadeOut(true);
                } else {
                    this.sandbox.util.delay(
                        function() {
                            this.sandbox.dom.fadeOut(
                                this.sandbox.dom.find('.' + constants.uploadItemClass, this.$dropzone),
                                this.options.fadeOutDuration,
                                function() {
                                    if (this.sandbox.dom.find('.' + constants.uploadItemClass + ':animated', this.$dropzone).length === 0) {
                                        this.afterFadeOut();
                                    }
                                }.bind(this)
                            );
                        }.bind(this),
                        this.options.fadeOutDelay
                    );
                }
            }
        },

        /**
         * Function gets called after all dropped files
         * have faded out
         * @param keepDom {Boolean} true to keep the dom like it is
         */
        afterFadeOut: function(keepDom) {
            this.closeOverlay();
            this.sandbox.emit(FILES_ADDED.call(this), this.getResponseArray(this.dropzone.files));
            this.filesDropped = 0;
            if (keepDom === true) {
                this.dropzone.files = [];
            } else {
                this.sandbox.dom.removeClass(this.$dropzone, constants.droppedClass);
                this.dropzone.removeAllFiles();
            }
        },

        /**
         * Returns an array with responses for a given array of files
         * @param files {Array} array of files with response properties
         * @returns {Array} array of responses
         */
        getResponseArray: function(files) {
            var arrReturn = [], i, length;
            for (i = -1, length = files.length; ++i < length;) {
                arrReturn.push(files[i].response);
            }
            return arrReturn;
        },

        remove: function() {
            this.dropzone.disable();
            this.dropzone.destroy();

            this.sandbox.dom.off(this.sandbox.dom.$document, createJQueryEventName.call(this, 'dragenter'));
            this.sandbox.dom.off(this.sandbox.dom.$document, createJQueryEventName.call(this, 'drop'));
            this.sandbox.dom.off(this.sandbox.dom.$document, createJQueryEventName.call(this, 'dragover'));
        }
    };

});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/input
 */

/**
 * @class Input
 * @constructor
 *
 * @params {Object} [options] Configuration object
 * @params {String} [options.instanceName] name if the the input instance
 * @params {String} [options.inputId] DOM-id to give the actual input-tag
 * @params {String} [options.inputName] DOM-name to give the actual input-tag. Can be usefull in forms
 * @params {String} [options.value] value to set at the beginning
 * @params {String} [options.placeholder] html5-placeholder to use
 * @params {Boolean} [options.disabled] defines if input can be edited
 * @params {String} [options.skin] name of the skin to use. Currently 'phone', 'password', 'url', 'email', 'date', 'time', 'color'. Each skin brings it's own default values. For example the password skin has automatically inputType: 'password'
 * @params {Object} [options.datepickerOptions] config-object to pass to the datepicker component - you can find possible values here http://bootstrap-datepicker.readthedocs.org/en/release/options.html
 * @params {Object} [options.colorPickerOptions] config-object to pass to the colorpicker component
 * @params {Object} [options.lockOptions] config-object to pass to the lock component
 * @params {String} [options.frontIcon] name of icon to display in front
 * @params {String} [options.frontText] text to display in front
 * @params {String} [options.frontHtml] html to display in front
 * @params {String} [options.backIcon] name of icon to display in front
 * @params {String} [options.backText] text to display in back
 * @params {String} [options.backHtml] html to display in back
 * @params {String} [options.renderMethod] name of a special render method to execute. Currently 'colorpicker', 'datepicker', 'time'. For example 'colorpicker' initializes a colorpicker and sets a css-class
 * @params {String} [options.inputType] the actual type of the input. e.g. 'text' or 'password'
 */
define('__component__$input@husky',[], function() {

    'use strict';

    var defaults = {
            instanceName: 'undefined',
            inputId: '',
            inputName: '',
            value: '',
            placeholder: '',
            disabled: false,
            skin: null,
            datepickerOptions: {
                orientation: 'auto',
                startDate: -Infinity,
                endDate: Infinity,
                todayHighlight: true
            },
            colorPickerOptions: {},
            lockOptions: {
                locked: false
            },
            frontIcon: null,
            frontText: null,
            frontHtml: null,
            backIcon: null,
            backText: null,
            backHtml: null,
            renderMethod: null,
            inputType: 'text'
        },

        constants = {
            componentClass: 'husky-input',
            frontClass: 'front',
            backClass: 'back',
            textClass: 'text',
            inputClass: 'input',
            actionClass: 'action',
            colorFieldClass: 'color-field',
            colorpickerClass: 'colorpicker',
            datepickerClass: 'pickdate',
            linkClickableClass: 'clickable',
            lockIconClass: 'fa-lock',
            unlockIconClass: 'fa-unlock'
        },

        templates = {
            input: '<input type="<%= type %>" value="<%= value %>" placeholder="<%= placeholder %>" id="<%= id %>" name="<%= name %>" data-from="false" <%= disabled %>/>',
            colorPickerFront: '<div class="' + constants.colorFieldClass + '"/>'
        },

        renderMethods = {
            colorpicker: function() {
                this.renderColorPicker();
            },
            datepicker: function() {
                this.renderDatePicker();
            },
            time: function() {
                this.renderTime();
            },
            url: function() {
                this.renderLink('http://');
            },
            email: function() {
                this.renderLink('mailto:');
            },
            lock: function() {
                this.renderLock();
            }
        },

        frontClickedCallbacks = {
            lock: function(event) {
                this.lockClickedCallback(event);
            }
        },

        skins = {
            color: {
                frontHtml: templates.colorPickerFront,
                renderMethod: 'colorpicker'
            },
            phone: {
                frontIcon: 'phone'
            },
            password: {
                frontIcon: 'key',
                inputType: 'password'
            },
            url: {
                frontText: 'http://',
                renderMethod: 'url'
            },
            email: {
                frontIcon: 'envelope',
                renderMethod: 'email'
            },
            date: {
                frontIcon: 'calendar',
                renderMethod: 'datepicker'
            },
            time: {
                frontIcon: 'clock-o',
                placeholder: 'HH - MM',
                renderMethod: 'time'
            },
            lock: {
                frontIcon: 'lock',
                renderMethod: 'lock'
            }
        },

        /**
         * namespace for events
         * @type {string}
         */
        eventNamespace = 'husky.input.',

        /**
         * raised after initialization process
         * @event husky.input.<instance-name>.initialize
         */
        INITIALIZED = function() {
            return createEventName.call(this, 'initialized');
        },

        LOCKED = function() {
            return createEventName.call(this, 'locked');
        },

        UNLOCKED = function() {
            return createEventName.call(this, 'unlocked');
        },

        /** returns normalized event names */
        createEventName = function(postFix) {
            return eventNamespace + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
        };

    return {

        /**
         * Initialize component
         */
        initialize: function() {
            this.sandbox.logger.log('initialize', this);
            var instanceDefaults = this.sandbox.util.extend(true, {}, defaults);

            // merge skin defaults with defaults
            if (!!this.options.skin && !!skins[this.options.skin]) {
                instanceDefaults = this.sandbox.util.extend(true, {}, defaults, skins[this.options.skin]);
            }
            // merge defaults, skin defaults and options
            this.options = this.sandbox.util.extend(true, {}, instanceDefaults, this.options);

            this.input = {
                $front: null,
                $input: null,
                $back: null
            };
            this.linkProtocol = null;

            this.render();
            this.bindDomEvents();

            this.sandbox.emit(INITIALIZED.call(this));
        },

        /**
         * Renders the input component
         */
        render: function() {
            this.sandbox.dom.addClass(this.$el, constants.componentClass);
            // add action class if an action is set
            if (!!this.options.action) {
                this.sandbox.dom.addClass(this.$el, constants.actionClass);
            }
            this.renderFront();
            this.renderInput();
            this.renderBack();
            // call a render method specific for the skin
            // e.g. renderColorPicker
            if (!!this.options.renderMethod && !!renderMethods[this.options.renderMethod]) {
                renderMethods[this.options.renderMethod].call(this);
            }
        },

        /**
         * Binds Dom-related events
         */
        bindDomEvents: function() {
            this.sandbox.dom.on(this.$el, 'change', function(event) {
                event.stopPropagation();
                event.preventDefault();

                this.$el.trigger('change');
            }.bind(this), 'input');

            this.sandbox.dom.on(this.$el.find('.' + constants.frontClass), 'click', function(event) {
                // check if input has custom functionality, when front is clicked
                if (frontClickedCallbacks.hasOwnProperty(this.options.skin)) {
                    frontClickedCallbacks[this.options.skin].call(this, event);
                }
            }.bind(this));

            this.sandbox.dom.on(this.$el, 'click', function() {
                this.sandbox.dom.focus(this.input.$input);
            }.bind(this));

            this.sandbox.dom.on(this.input.$input, 'click', function() {
                this.sandbox.dom.focus(this.input.$input);
            }.bind(this));

            // delegate labels on input
            if (!!this.sandbox.dom.attr(this.$el, 'id')) {
                this.sandbox.dom.on('label[for="' + this.sandbox.dom.attr(this.$el, 'id') + '"]', 'click', function() {
                    this.sandbox.dom.focus(this.input.$input);
                }.bind(this));
            }

            // change the input value if the data attribute got changed
            this.sandbox.dom.on(this.$el, 'data-changed', function() {
                this.updateValue();
            }.bind(this));
        },

        /**
         * Renders the front of the input
         */
        renderFront: function() {
            if (!!this.options.frontIcon || !!this.options.frontText || !!this.options.frontHtml) {
                this.input.$front = this.sandbox.dom.createElement('<div class="' + constants.frontClass + '"/>');
                if (!!this.options.frontIcon) {
                    this.sandbox.dom.html(this.input.$front, '<a class="fa-' + this.options.frontIcon + '"></a>');
                } else if (!!this.options.frontText) {
                    this.sandbox.dom.html(this.input.$front,
                        '<a class="' + constants.textClass + '">' + this.options.frontText + '</a>');
                } else {
                    this.sandbox.dom.html(this.input.$front, this.options.frontHtml);
                }
                this.sandbox.dom.append(this.$el, this.input.$front);
            }
        },

        /**
         * Renders the actual html-input field
         * with validation constraints and so on
         */
        renderInput: function() {
            var $container = this.sandbox.dom.createElement('<div class="' + constants.inputClass + '"/>');
            this.input.$input = this.sandbox.dom.createElement(this.sandbox.util.template(templates.input)({
                type: this.options.inputType,
                name: (!!this.options.inputName) ? this.options.inputName : 'husky-input-' + this.options.instanceName,
                id: (!!this.options.inputId) ? this.options.inputId : 'husky-input-' + this.options.instanceName,
                value: this.options.value,
                placeholder: this.options.placeholder,
                disabled: (this.options.disabled === true) ? 'disabled' : ''
            }));
            this.sandbox.dom.append($container, this.input.$input);
            this.sandbox.dom.append(this.$el, $container);
        },

        /**
         * renders the back of the input
         */
        renderBack: function() {
            if (!!this.options.backIcon || !!this.options.backText || !!this.options.backHtml) {
                this.input.$back = this.sandbox.dom.createElement('<div class="' + constants.backClass + '"/>');
                if (!!this.options.backIcon) {
                    this.sandbox.dom.html(this.input.$back, '<span class="fa-' + this.options.backIcon + '"></span>');
                } else if (!!this.options.backText) {
                    this.sandbox.dom.html(this.input.$back,
                        '<span class="' + constants.textClass + '">' + this.options.backText + '</span>');
                } else {
                    this.sandbox.dom.html(this.input.$back, this.options.backHtml);
                }
                this.sandbox.dom.append(this.$el, this.input.$back);
            }
        },

        /**
         * Starts the colorpicker
         */
        renderColorPicker: function() {
            this.sandbox.dom.addClass(this.$el, constants.colorpickerClass);
            this.changeColorPreview(this.sandbox.dom.val(this.input.$input));
            this.sandbox.colorpicker.init(this.input.$input, this.sandbox.util.extend(true, {}, {
                change: this.changeColorPreview.bind(this)
            }, this.options.colorPickerOptions));
        },

        /**
         * Starts the datepicker
         */
        renderDatePicker: function() {
            this.sandbox.dom.addClass(this.$el, constants.datepickerClass);
            if (!this.options.placeholder) {
                this.sandbox.dom.attr(this.input.$input, 'placeholder', this.sandbox.globalize.getDatePattern());
            }

            // parse stard and end date
            if (!!this.options.datepickerOptions.startDate && typeof(this.options.datepickerOptions.startDate) === 'string') {
                this.options.datepickerOptions.startDate = new Date(this.options.datepickerOptions.startDate);
            }
            if (!!this.options.datepickerOptions.endDate && typeof(this.options.datepickerOptions.endDate) === 'string') {
                this.options.datepickerOptions.endDate = new Date(this.options.datepickerOptions.endDate);
            }

            this.sandbox.datepicker.init(this.input.$input, this.options.datepickerOptions).on('changeDate',
                function(event) {
                    this.setDatepickerValueAttr(event.date);
                }.bind(this));
            this.updateValue();

            this.bindDatepickerDomEvents();
        },

        /**
         * Attaches an event-listner to the input which makes an a-tag with clickable
         * @param protocol {String} a protocol to prepend to the input-value (e.g. 'http://' or 'mailto:')
         */
        renderLink: function(protocol) {
            this.linkProtocol = protocol;
            this.sandbox.dom.on(this.input.$input, 'keyup', this.changeFrontLink.bind(this));
            this.changeFrontLink();
        },

        /**
         * Lock-input specific actions. Makes sure the correct icon is rendered and the dom data is set.
         */
        renderLock: function() {
            var locked = this.options.lockOptions.locked;
            // always enable front icon
            this.sandbox.dom.addClass(this.sandbox.dom.find('a', this.input.$front), constants.linkClickableClass);
            this.renderLockIcon();
            this.$el.data('locked', locked);
        },

        /**
         * Gets called, when front icon of lock-input is clicked.
         *
         * @param {Object} event
         */
        lockClickedCallback: function(event) {

            if (this.options.disabled === true) {
                return;
            }

            var locked = !this.options.lockOptions.locked;

            // set options value
            this.options.lockOptions.locked = locked;
            // en/disable input
            this.sandbox.dom.prop(this.input.$input, 'disabled', locked);
            // render appropiate icon
            this.renderLockIcon();
            // set data
            this.$el.data('locked', locked);

            // trigger event
            if (locked) {
                this.sandbox.emit(LOCKED.call(this));
                event.stopPropagation();
            } else {
                this.sandbox.emit(UNLOCKED.call(this));
            }
        },

        /**
         * Renders the appropiate locked icon.
         */
        renderLockIcon: function() {
            var removeIcon = constants.lockIconClass,
                addIcon = constants.unlockIconClass;

            if (this.options.lockOptions.locked) {
                removeIcon = constants.unlockIconClass;
                addIcon = constants.lockIconClass;
            }

            this.exchangeFrontIconClass(addIcon, removeIcon);
        },

        /**
         * Exchanges a icon class with another.
         *
         * @param {String} addClass
         * @param {String} removeClass
         */
        exchangeFrontIconClass: function(addClass, removeClass) {
            var selector = this.sandbox.dom.find('a', this.input.$front);
            this.sandbox.dom.removeClass(selector, removeClass);
            this.sandbox.dom.addClass(selector, addClass);

            // if clickable-class is set, set it behind icon class
            if (this.sandbox.dom.hasClass(selector, constants.linkClickableClass)) {
                this.sandbox.dom.removeClass(selector, constants.linkClickableClass);
                this.sandbox.dom.addClass(selector, constants.linkClickableClass);
            }
        },

        /**
         * Changes the anchor-tag in the front-section
         */
        changeFrontLink: function() {
            var value = this.sandbox.dom.val(this.input.$input);
            if (!!value) {
                this.sandbox.dom.addClass(this.sandbox.dom.find('a', this.input.$front), constants.linkClickableClass);
                this.sandbox.dom.attr(this.sandbox.dom.find('a', this.input.$front), 'href', this.linkProtocol + value);
                if (this.linkProtocol.indexOf('mailto') < 0) {
                    this.sandbox.dom.attr(this.sandbox.dom.find('a', this.input.$front), 'target', '_blank');
                }
            } else {
                this.sandbox.dom.removeClass(this.sandbox.dom.find('a', this.input.$front),
                    constants.linkClickableClass);
                this.sandbox.dom.removeAttr(this.sandbox.dom.find('a', this.input.$front), 'href');
            }
        },

        /**
         * Binds Dom-events for the datepicker
         */
        bindDatepickerDomEvents: function() {
            this.sandbox.dom.on(this.input.$input, 'focusout', function() {
                this.setDatepickerValueAttr(this.sandbox.datepicker.getDate(this.input.$input));
            }.bind(this));
        },

        /**
         * Renders time specific things
         */
        renderTime: function() {
            this.sandbox.dom.attr(this.input.$input, 'placeholder', this.sandbox.globalize.getTimePatter());
        },

        /**
         * Updates the square which shows the current color for the colorpicker
         * @param color {String} new hex color
         */
        changeColorPreview: function(color) {
            this.sandbox.dom.css(this.$find('.' + constants.colorFieldClass), {
                'background-color': color
            });
        },

        /**
         * Sets the input value
         * @param value {String} new value
         */
        setValue: function(value) {
            if (this.options.renderMethod === 'colorpicker') {
                if (!!value) {
                    this.sandbox.colorpicker.value(this.input.$input, value);
                } else {
                    this.sandbox.dom.val(this.input.$input, value);
                }
            } else if (this.options.renderMethod === 'datepicker') {
                // if a date-time was passed, extract the date
                if (!!value) {
                    value = this.isoToDate(value);
                    value = new Date(value);
                }
                this.sandbox.datepicker.setDate(this.input.$input, value);
                this.setDatepickerValueAttr(this.sandbox.datepicker.getDate(this.input.$input));
            } else {
                this.sandbox.dom.val(this.input.$input, value);
                if (this.options.renderMethod === 'email' || this.options.renderMethod === 'url') {
                    this.changeFrontLink();
                }
            }
        },

        /**
         * Takes a iso date-time string and returns only the date part
         * @param datetime {String} iso-datetime-string
         */
        isoToDate: function(datetime) {
            if (datetime.indexOf('T') > 0) {
                return datetime.substr(0, datetime.indexOf('T'));
            }
            return datetime;
        },

        /**
         * Sets the value attribute for the datepicker
         * @param date {Object} a UTC date pbject
         */
        setDatepickerValueAttr: function(date) {
            if (!!date) {
                if (this.sandbox.dom.isNumeric(date.valueOf())) {
                    date = date.getFullYear() + '-' +
                        ('0' + (date.getMonth() + 1)).slice(-2) + '-' +
                        ('0' + date.getDate()).slice(-2);
                } else {
                    date = '';
                    this.sandbox.dom.val(this.input.$input, '');
                }
            }
            this.sandbox.dom.data(this.$el, 'value', date);
        },

        /**
         * Updates the value with what is in the
         * data attribute
         */
        updateValue: function() {
            this.setValue(this.sandbox.dom.data(this.$el, 'value'));
        }
    };

});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/data-navigation
 */

define('husky_components/data-navigation/list-view',[], function() {

    var templates = {
        list: function() {
            return [
                '<ul class="data-navigation-items">',
                '   <% if (!!data.children && !!data.children.length) { %>',
                templates.children(),
                '   <% } else if (!!data.children && data.children.length === 0) { %>',
                '       <li class="not-selectable">',
                '           <div class="data-navigation-info data-navigation-info-empty">',
                '               <div class="fa-coffee icon"></div>',
                '               <span><%=options.translates.noData%></span>',
                '           </div>',
                '       </li>',
                '   <% } else { %>',
                '       <li class="not-selectable">',
                '           <div class="data-navigation-loader-container">',
                '               <div class="spinner">',
                '                   <div class="double-bounce1" style="background-color: rgb(204, 204, 204);"></div>',
                '                   <div class="double-bounce2" style="background-color: rgb(204, 204, 204);"></div>',
                '               </div>',
                '           </div>',
                '       </li>',
                '   <% } %>',
                '</ul>'
            ].join('');
        },

        children: function() {
            return [
                '       <% _.each(data.children, function(child) { %>',
                '           <li data-id="<%= child.id %>" class="data-navigation-item">',
                '               <% if (!!child.preview && !child.locked) { %>',
                '                   <div class="data-navigation-item-thumb" style="background-image: url(\'<%=child.preview.url%>\')"></div>',
                '               <% } else { %>',
                '                   <div class="<% if(!child.locked) { %>fa-coffee<% } else { %>fa-lock<% } %> data-navigation-item-thumb" style="margin-top: 10px;padding-left: 10px;"></div>',
                '               <% } %>',
                '               <div class="data-navigation-item-name">',
                '                   <%= child[options.nameKey] %>',
                '                   <% if (!!child.hasSub) { %>',
                '                       <span class="fa-chevron-right data-navigation-item-next"></span>',
                '                   <% } %>',
                '               </div>',
                '           </li>',
                '       <% }) %>'
            ].join('');
        }
    };

    /**
     * @class ListView
     * @constructor
     */
    return function ListView() {
        return {
            /**
             * @method init
             */
            init: function() {
                this.template = _.template(templates.list());
                this.children = _.template(templates.children());
                this.$el = $('<div/>', {
                    'class': 'data-navigation-list'
                });

                return this;
            },

            /**
             * Removes the view
             * @method destroy
             * @chainable
             */
            destroy: function() {
                this.$el.off();
                this.$el.remove();
                this.$el = null;

                return this;
            },

            /**
             * @method render
             * @param {Object} data
             * @param {Object} options
             * @chainable
             */
            render: function(data, options) {
                data = data || {};
                var tpl = this.template({data: data, options: options});

                this.$el.html(tpl);

                return this;
            },

            /**
             * @method render
             * @param {Object} data
             * @param {Object} options
             * @chainable
             */
            append: function(data, options) {
                var tpl = this.children({data: {children: data}, options: options});

                this.$el.find('ul').append(tpl);
            },

            /**
             * Insert the html into the dom
             * @method placeAt
             * @chainable
             */
            placeAt: function(container) {
                $(container).append(this.$el);

                return this;
            }
        };
    }
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/data-navigation
 */

/**
 * @class DataNavigation
 * @constructor
 * @param {Object} [options] Configuration object
 * @param {String} [options.instanceName] - name of instance
 * @param {String} [options.rootUrl] - optional root url to fetch root data
 * @param {String} [options.url] url to fetch data from
 * @param {String} [options.resultKey] - key of array result
 * @param {String} [options.nameKey] - key of object name
 * @param {String} [options.childrenLinkKey] - key of children link in object
 * @param {String} [options.showAddButton] - Indicates if add button should be rendered
 * @param {Object} [options.translates] Holds the translates
 * @param {String} [options.locale] The locale option to send with each request
 * @param {Boolean} [options.globalEvents] Should global events be sent
 */
define('__component__$data-navigation@husky',[
    'husky_components/data-navigation/list-view'
], function(View) {

    'use strict';

    var defaultOptions = {
            url: null,
            rootUrl: null,
            resultKey: 'items',
            parentResultKey: 'parent',
            nameKey: 'name',
            childrenLinkKey: 'children',
            pageKey: 'page',
            limitKey: 'limit',
            searchKey: 'search',
            limit: 20,
            showAddButton: true,
            globalEvents: true,
            locale: null,
            translates: {
                noData: 'No Data',
                title: 'Data',
                addButton: 'Add Data',
                search: 'Search ...'
            }
        },

        templates = {
            header: function() {
                return [
                    '<div class="header-item data-navigation-back" style="display: none;">',
                    '    <div class="icon-container"><span class="fa-chevron-left"/></div>',
                    '</div>',
                    '<div class="header-item data-navigation-search" style="display: none;">',
                    '    <div class="icon-container"><span class="fa-search fa-flip-horizontal"/></div>',
                    '    <div class="content-container">',
                    '        <div class="search-container"/>',
                    '    </div>',
                    '</div>',
                    '<% if (!!options.showAddButton) { %>',
                    '   <div class="header-item ', constants.addButtonClass, '">',
                    '       <div class="icon-container"><span class="fa-plus-circle"/></div>',
                    '       <div class="content-container">',
                    '           <span class="add-container"><%= options.translates.addButton %></span>',
                    '       </div>',
                    '   </div>',
                    '<% } %>',
                    '<div class="header-item header-filler"/>'
                ].join('');
            },

            main: function() {
                return [
                    '<div class="data-navigation">',
                    '   <div class="data-navigation-header"></div>',
                    '   <div class="data-navigation-list-container iscroll">',
                    '       <div class="data-navigation-list-scroll iscroll-inner"></div>',
                    '       <div class="loader"></div>',
                    '   </div>',
                    '</div>'
                ].join('');
            }
        },

        constants = {
            ROOT_ID: 'root',
            addButtonClass: 'data-navigation-add'
        },

        eventNamespace = 'husky.data-navigation.',

        /**
         * Creates the eventnames
         * @param postFix {String} event name to append
         */
        createEventName = function(postFix) {
            return eventNamespace + (this.options.instanceName ? this.options.instanceName + '.' : '') + postFix;
        },

        /**
         * raised after initialization has finished
         * @event husky.data-navigation.initialized
         */
        INITIALIZED = function() {
            return createEventName.call(this, 'initialized');
        },

        SELECT = function() {
            return createEventName.call(this, 'select');
        },

        /**
         * raised after the item was selected
         * @param item {Object} selected item
         * @event husky.data-navigation.select
         */
        SELECTED = function() {
            return createEventName.call(this, 'selected')
        },

        /**
         * raised after the item was selected without instancename
         * @param item {Object} selected item
         * @event husky.data-navigation.select
         */
        SELECTED_GLOBAL = function() {
            return eventNamespace + 'selected';
        },

        /**
         * raised when clicked on the add button
         * @param item {Object} current item
         * @event husky.data-navigation.add
         */
        ADD = function() {
            return createEventName.call(this, 'add')
        },

        /**
         * hides the add button of the data navigation
         * @event husky.data-navigation.add-button.hide
         */
        HIDE_ADD_BUTTON = function() {
           return createEventName.call(this, 'add-button.hide');
        },

        /**
         * shows the add button of the data navigation
         * @event husky.data-navigation.add-button.show
         */
        SHOW_ADD_BUTTON = function() {
            return createEventName.call(this, 'add-button.show');
        },

        /**
         * raised when clicked on the item icon
         * @param item {Object} item to navigate into
         * @event husky.data-navigation.content.navigate
         */
        NAVIGATE = function() {
            return createEventName.call(this, 'navigate')
        },

        /**
         * Setter for current url navigation will be reloaded with data
         * @param url {string} url to load data
         * @event husky.data-navigation.content.set-url
         */
        SET_URL = function() {
            return createEventName.call(this, 'set-url')
        },

        /**
         * Setter for current locale
         * @param {string} locale locale to set
         * @event husky.data-navigation.content.set-locale
         */
        SET_LOCALE = function() {
            return createEventName.call(this, 'set-locale');
        },

        /**
         * Navigation will be reloaded with current url
         * @event husky.data-navigation.content.reload
         */
        RELOAD = function() {
            return createEventName.call(this, 'reload')
        },

        /**
         * Clear cache completely
         * @event husky.data-navigation.content.clear-cache
         */
        CLEAR_CACHE = function() {
            return createEventName.call(this, 'clear-cache')
        };

    return {

        page: 1,
        loading: false,

        /**
         * @method initialize
         */
        initialize: function() {
            this.currentView = null;
            this.locale = null;
            this.cache = this.sandbox.cacheFactory.create();
            this.options = this.sandbox.util.extend(true, {}, defaultOptions, this.options);
            this.mainTpl = this.sandbox.util.template(templates.main());
            this.headerTpl = this.sandbox.util.template(templates.header());
            this.whenLocaleReceived = $.Deferred();

            if (!!this.options.locale) {
                this.locale = this.options.locale;
                this.whenLocaleReceived.resolve();
            }

            this.render();
            this.bindCustomEvents();
            this.bindDOMEvents();

            this.sandbox.once('husky.loader.initialized', function() {
                this.showLoader();

                this.whenLocaleReceived.then(function() {
                    this.load()
                        .then(function(data) {
                            this.hideLoader();
                            this.sandbox.emit(INITIALIZED.call(this));

                            this.currentView = this.createView(data);
                            this.updateHeader(data);
                            this.storeData(data);
                            this.appendView();
                        }.bind(this));
                }.bind(this));
            }.bind(this));
        },

        /**
         * Sets the locale for the component and resolves the
         * corresponding promise.
         *
         * @param {String} locale The locale to set
         */
        setLocale: function(locale) {
            this.locale = locale;
            this.whenLocaleReceived.resolve();
        },

        /**
         * Component destructor function
         * @method remove
         */
        remove: function() {
            this.cache.destroy();
            this.sandbox.infiniteScroll.destroy('.iscroll');
        },

        /**
         * Initial render method. Inserts the main template into the dom
         * @method render
         */
        render: function() {
            var mainTpl = this.mainTpl({options: this.options});
            this.$el.html(mainTpl);
            var headerTpl = this.headerTpl({options: this.options});
            this.$el.find('.data-navigation-header').html(headerTpl);

            this.sandbox.start([
                {
                    name: 'loader@husky',
                    options: {
                        el: this.sandbox.dom.find('.loader', this.$el),
                        color: '#ccc'
                    }
                }
            ]);

            this.startInfiniteScroll();
            this.startSearch();
        },

        startSearch: function() {
            this.sandbox.start([
                {
                    name: 'search@husky',
                    options: {
                        el: this.sandbox.dom.find('.search-container', this.$el),
                        appearance: 'white',
                        instanceName: 'data-navigation',
                        placeholderText: this.options.translates.search
                    }
                }
            ]);
        },

        updateHeader: function(data) {
            if (data.current.id !== 'root') {
                $('.data-navigation-back').show();
                var parent = (!!data.parent) ? data.parent.id : 'root';
                this.sandbox.dom.data($('.data-navigation-back'), 'parent-id', parent);
            } else {
                $('.data-navigation-back').hide();
            }

            if (data.children.length !== 0 || !!this.searchTerm) {
                $('.data-navigation-search').show();
            } else {
                $('.data-navigation-search').hide();
            }
        },

        /**
         * start infinite scrolling
         * @method startInfiniteScroll
         */
        startInfiniteScroll: function() {
            this.sandbox.infiniteScroll.initialize('.iscroll', this.loadNextPage.bind(this));
        },

        /**
         * load children from next page
         * @method startInfiniteScroll
         */
        loadNextPage: function() {
            var def = this.sandbox.data.deferred();

            if (!!this.data.hasNextPage && !this.loading) {
                this.showLoader();

                this.page += 1;
                this.sandbox.util.load(this.getUrl(this.getCurrentUrl()), {
                    locale: this.locale
                }).then(function(data) {
                    var children = data._embedded[this.options.resultKey] || [];
                    this.data.children = this.data.children.concat(children);
                    this.data.hasNextPage = this.page < data.pages;
                    this.currentView.append(children, this.options);

                    this.hideLoader();
                    def.resolve();
                }.bind(this));
            } else {
                def.resolve();
            }

            return def;
        },

        /**
         * @method bindDOMEvents
         */
        bindDOMEvents: function() {
            this.$el.on('click', '.data-navigation-item', this.selectChildrenDataHandler.bind(this));
            this.$el.on('click', '.data-navigation-item-thumb', this.selectChildrenDataHandler.bind(this));
            this.$el.on('click', '.data-navigation-back', this.selectParentDataHandler.bind(this));
            this.$el.on('click', '.data-navigation-add', this.addHandler.bind(this));

            this.$el.on('click', '.data-navigation-search > .icon-container', function() {
                if (!$('.data-navigation-header').hasClass('header-search')) {
                    $('.data-navigation-header').addClass('header-search');
                    $('.data-navigation-search .icon-container span').attr('class', 'fa-times');
                    $('.data-navigation-search input').select();
                } else {
                    this.clearSearch();
                }
            }.bind(this));
        },

        /**
         * @method bindCustomEvents
         */
        bindCustomEvents: function() {
            this.sandbox.on(SET_URL.call(this), function(url) {
                if (url !== this.getCurrentUrl) {
                    this.setUrl(url, true);
                } else {
                    this.cache.deleteAll();
                }
            }.bind(this));

            this.sandbox.on(SET_LOCALE.call(this), this.setLocale.bind(this));

            this.sandbox.on(SELECT.call(this), function(id) {
                this.showChildren(id);
            }.bind(this));

            this.sandbox.on(SHOW_ADD_BUTTON.call(this), this.showAddButton.bind(this));

            this.sandbox.on(HIDE_ADD_BUTTON.call(this), this.hideAddButton.bind(this));

            this.sandbox.on(RELOAD.call(this), function(callback) {
                this.setUrl(this.getCurrentUrl(), true, callback);
            }.bind(this));

            this.sandbox.on(CLEAR_CACHE.call(this), function() {
                this.cache.deleteAll();
            }.bind(this));

            this.sandbox.on('husky.search.data-navigation', function(searchTerm) {
                this.searchTerm = searchTerm;

                this.setUrl(this.getCurrentUrl(), true);
            }.bind(this));

            this.sandbox.on('husky.search.data-navigation.reset', function() {
                this.searchTerm = null;

                this.setUrl(this.getCurrentUrl(), true);
            }.bind(this));
        },

        getCurrentUrl: function() {
            var url = this.options.rootUrl;

            if (!!this.data.current.item) {
                url = this.data.current.item._links[this.options.childrenLinkKey].href;
            }

            return url;
        },

        setUrl: function(url, clearCache, callback) {
            if (!!clearCache) {
                this.cache.deleteAll();
            }

            this.load(url)
                .then(this.storeData.bind(this))
                .then(function(data) {
                    this.updateHeader(data);
                    this.currentView.render(data, this.options);
                    if (typeof callback === 'function') {
                        callback.call(this);
                    }
                }.bind(this));
        },

        /**
         * Fetch the data from the server
         * @method load
         * @param {String} url
         */
        load: function(url) {
            this.page = 1;
            this.loading = true;

            return this.sandbox.util.load(this.getUrl(url), {
                locale: this.locale
            }).then(this.parse.bind(this))
                .then(function(data) {
                    this.loading = false;
                    return data;
                }.bind(this));
        },

        hideLoader: function() {
            this.sandbox.dom.hide(this.sandbox.dom.find('.loader', this.$el));
        },

        showLoader: function() {
            this.sandbox.dom.show(this.sandbox.dom.find('.loader', this.$el));
        },

        /**
         * @method clearSearch
         */
        clearSearch: function() {
            $('.data-navigation-header').removeClass('header-search');
            $('.data-navigation-search .icon-container span').attr('class', 'fa-search fa-flip-horizontal');
            this.sandbox.emit('husky.search.data-navigation.clear');
            if (!!this.searchTerm) {
                this.sandbox.emit('husky.search.data-navigation.reset');
                this.searchTerm = null;
            }
        },

        /**
         * Returns full url with search, page and limit parameter
         * @param url
         * @returns {string}
         */
        getUrl: function(url) {
            url = url || this.options.url || this.options.rootUrl;

            var delimiter = (url.indexOf('?') === -1) ? '?' : '&';

            url = [
                url, delimiter, this.options.pageKey, '=', this.page, '&',
                this.options.limitKey, '=', this.options.limit
            ].join('');

            if (!!this.searchTerm) {
                url = [
                    url, '&', this.options.searchKey, '=', this.searchTerm
                ].join('');
            }

            return url;
        },

        /**
         * @method parse
         * @param {Object} response Response
         */
        parse: function(response) {
            var current = {item: response},
                parent = response._embedded.parent;

            current.id = response.id || constants.ROOT_ID;
            current.name = response[this.options.nameKey];

            if (!!parent) {
                parent.id = parent.id || constants.ROOT_ID;
            } else if (current.id === constants.ROOT_ID) {
                current.item = null;
            }

            this.data = {
                children: response._embedded[this.options.resultKey] || null,
                parent: parent,
                current: current,
                hasNextPage: true
            };

            return this.data;
        },

        /**
         * caches the data inside a cache object
         * @method storeData
         * @param  {Object} data
         */
        storeData: function(data) {
            var current = data.current;

            if (!this.searchTerm) {
                this.cache.put(current.id, data);
            }

            return data;
        },

        /**
         * check if we already have the data.
         * If this is not the case, we fetch the data from the server
         * @method getItems
         */
        getItems: function(id) {
            var dfd = $.Deferred(),
                data = this.cache.get(id),
                item, url;

            if (!data) {
                if (id === constants.ROOT_ID || id == null) {
                    if (!!this.data.parent) {
                        url = this.data.parent._links[this.options.childrenLinkKey].href;
                    } else {
                        url = this.options.rootUrl;
                    }
                } else {
                    // underscores where returns a list but we only want the first item
                    item = _.where(this.data.children, {id: id})[0];

                    if (!item && !!this.data.parent && this.data.parent.id === id) {
                        item = this.data.parent;
                    }

                    url = item._links[this.options.childrenLinkKey].href;
                }

                return this.load(url).then(this.storeData.bind(this));
            } else {
                this.data = data;
                dfd.resolve(data);
            }

            return dfd.promise();
        },

        /**
         * Return item from local cache
         * @param id
         */
        getItem: function(id) {
            return _.where(this.data.children, {id: id})[0];
        },

        /**
         * Show the children for the item with the given id
         *
         * @param id
         * @returns {*}
         */
        showChildren: function(id) {
            var oldView = this.currentView;

            this.clearSearch();
            this.currentView = this.createView();
            this.appendView(oldView);

            return this.getItems(id)
                .then(function(data) {
                    this.updateHeader(data);
                    this.currentView.render(data, this.options);
                }.bind(this));
        },

        /**
         * @method openChildrenHandler
         * @param {Object} event
         */
        openChildrenHandler: function(event) {
            return this.showChildren($(event.currentTarget).closest('li').data('id'));
        },

        /**
         * @method openParentHandler
         */
        openParentHandler: function(event) {
            var $item = $(event.currentTarget).closest('.data-navigation-back'),
                id = this.sandbox.dom.data($item, 'parent-id'),
                newView = this.createView();

            this.clearSearch();
            this.prependView(newView);

            return this.getItems(id)
                .then(function(data) {
                    this.updateHeader(data);
                    newView.render(data, this.options);
                }.bind(this));
        },

        /**
         * Navigate to content and throw selected event
         * @method selectDataHandler
         */
        selectParentDataHandler: function(event) {
            event.stopPropagation();

            this.sandbox.emit(SELECTED.call(this), this.data.parent);
            if (this.options.globalEvents) {
                this.sandbox.emit(SELECTED_GLOBAL.call(this), this.data.parent);
            }

            this.openParentHandler(event);
        },

        /**
         * Navigate to content
         * @method selectDataHandler
         */
        navigateParentDataHandler: function(event) {
            event.stopPropagation();

            this.openParentHandler(event);
        },

        /**
         * Navigate to content and throw selected event
         * @method selectDataHandler
         */
        selectChildrenDataHandler: function(event) {
            event.stopPropagation();

            var $item = $(event.currentTarget).closest('li'),
                id = $item.data('id'),
                item = this.getItem(id);

            this.sandbox.emit(SELECTED.call(this), item);
            if (this.options.globalEvents) {
                this.sandbox.emit(SELECTED_GLOBAL.call(this), item);
            }

            this.openChildrenHandler(event);
        },

        /**
         * Throw an event to tell that the content should be loaded
         * @method showContentHandler
         */
        navigateChildrenHandler: function(event) {
            event.stopPropagation();

            var $item = $(event.currentTarget).closest('li'),
                id = $item.data('id'),
                item = this.getItem(id);

            this.sandbox.emit(NAVIGATE.call(this), item);

            this.openChildrenHandler(event);
        },

        /**
         * Throw an event to tell that the add button was clicked
         * @method addHandler
         */
        addHandler: function() {
            this.sandbox.emit(ADD.call(this), this.data.current.item);
        },

        /**
         * Shows the add button
         */
        showAddButton: function() {
            $('.' + constants.addButtonClass).show();
        },

        /**
         * Hides the add button
         */
        hideAddButton: function() {
            $('.' + constants.addButtonClass).hide();
        },

        /**
         * Initialize a new view
         * @method createView
         * @param  {Object} data
         */
        createView: function(data) {
            var view = (new View(this.options)).init();
            return view.render(data, this.options);
        },

        /**
         * Append the view into the dom
         * @method appendView
         * @param {Object} view
         */
        appendView: function(view) {
            if (!!view) {
                this.currentView.$el.addClass('is-animated');
                this.playAppendAnimation(view);
            }

            this.currentView.placeAt('.data-navigation-list-container .iscroll-inner');
        },

        /**
         * @method playAppendAnimation
         * @param {Object} oldView
         */
        playAppendAnimation: function(oldView) {
            this.currentView.$el
                .css({
                    left: '100%'
                })
                .animate({
                    left: '0%'
                }, {
                    duration: 250,
                    done: function() {
                        oldView.destroy();
                        this.currentView.$el.removeClass('is-animated');
                    }.bind(this)
                });
        },

        /**
         * Handle the back button action
         * @method prependView
         * @param {Object} view
         */
        prependView: function(view) {
            view.placeAt('.data-navigation-list-container .iscroll-inner');
            this.currentView.$el.addClass('is-animated');
            this.playPrependAnimation(view);
        },

        /**
         * @method playPrependAnimation
         * @param {Object} view
         */
        playPrependAnimation: function(view) {
            var oldView = this.currentView;
            this.currentView = view;

            oldView.$el
                .stop()
                .css({
                    left: '0%'
                })
                .animate({
                    left: '100%'
                }, {
                    duration: 250,
                    done: function() {
                        oldView.destroy();
                    }.bind(this)
                });
        }
    };
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/input
 */

/**
 * @class Url-Input
 * @constructor
 */
define('__component__$url-input@husky',['services/husky/url-validator'], function(urlValidator) {
    var defaults = {
            instanceName: 'url-input',
            inputClass: '',
            scheme: 'http://',
            specificPart: ''
        },

        constants = {
            triggerClass: 'drop-down-trigger',
            toggleClass: 'dropdown-toggle',
            schemeClass: 'scheme',
            specificPartClass: 'specific-part-input',
            linkClass: 'link',
            dataKey: 'url-data',
            dataChangedEvent: 'data-changed'
        },

        templates = {
            skeleton: [
                '<div class="front">',
                '    <a class="<%= constants.schemeClass %> <%= constants.linkClass %> text text-link"',
                '       href="<%= url %>" target="_blank"><%= data.scheme %></a>',
                '<% if (items.length > 1) { %>',
                '    <span class="<%= constants.triggerClass %>">',
                '        <span class="<%= constants.toggleClass %>" style="display: inline-block;"/>',
                '    </span>',
                '<% } %>',
                '</div>',
                '<div class="input">',
                '    <input type="text"',
                '           class="<%= constants.specificPartClass %> <%= options.inputClass %>" ',
                '           value="<%= data.specificPart %>"/>',
                '</div>'
            ].join('')
        };

    return {
        /**
         * Initialize component
         */
        initialize: function() {
            // the array will be extend with all default.schemes if this.options.schemes are set
            if (!this.options.schemes || this.options.schemes.length === 0) {
                this.options.schemes = urlValidator.getDefaultSchemes();
            }

            // merge defaults
            this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

            if (this.getData() === undefined) {
                this.setData({scheme: this.options.scheme, specificPart: this.options.specificPart});
            }

            this.prepareItems(this.options.schemes);
            this.render();
            this.bindDomEvents();
        },

        /**
         * Prepare schemes to use it in dropdown.
         */
        prepareItems: function(schemes) {
            this.items = this.sandbox.util.arrayMap(schemes, function(scheme) {
                return {
                    id: scheme,
                    name: scheme
                }
            }.bind(this));
        },

        /**
         * Render skeleton of component
         */
        render: function() {
            this.$el.addClass('husky-input');
            var data = this.getData();

            this.html(this.sandbox.util.template(templates.skeleton, {
                constants: constants,
                options: this.options,
                url: this.getUrl(data),
                data: data,
                items: this.items
            }));

            this.sandbox.start([{
                name: 'dropdown@husky',
                options: {
                    el: this.$find('.front'),
                    trigger: this.$find('.' + constants.triggerClass),
                    instanceName: constants.instanceName,
                    data: this.items,
                    clickCallback: this.selectSchemeHandler.bind(this)
                }
            }]);
        },

        /**
         * Bind necessary dom events.
         */
        bindDomEvents: function() {
            this.sandbox.dom.on(
                this.$find('.' + constants.linkClass),
                'click',
                this.linkHandler.bind(this)
            );

            this.sandbox.dom.on(
                this.$find('.' + constants.specificPartClass),
                'focusout',
                this.specificPartChangedHandler.bind(this)
            );

            this.sandbox.dom.on(
                this.$el,
                constants.dataChangedEvent,
                this.dataChangedHandler.bind(this)
            );
        },

        /**
         * Handles click on scheme.
         */
        linkHandler: function() {
            var data = this.getData(),
                url = this.getUrl(data);

            if (!url) {
                return false;
            }
        },

        /**
         * Handles input changed.
         */
        specificPartChangedHandler: function(e) {
            this.sandbox.dom.stopPropagation(e);

            var $element = this.$find('.' + constants.specificPartClass),
                specificPart = $element.val(),
                match = urlValidator.match(specificPart, this.options.schemes),
                data = {specificPart: specificPart};

            if (!!match) {
                data.scheme = match.scheme;
                data.specificPart = match.specificPart;
            }

            this.setData(data);
            this.dataChangedHandler();
        },

        /**
         * Handles data changed.
         */
        dataChangedHandler: function() {
            var data = this.getData();
            this.$find('.' + constants.specificPartClass).val(data.specificPart);
            this.$find('.' + constants.schemeClass).html(data.scheme);
            this.$find('.' + constants.schemeClass).attr('href', this.getUrl(data));
        },

        /**
         * Handles scheme dropdown value click.
         *
         * @param {{id, name}} item
         */
        selectSchemeHandler: function(item) {
            this.setData({scheme: item.id});

            this.dataChangedHandler();
        },

        /**
         * Set data to dom-data.
         *
         * @param {{scheme, specificPart}} data
         */
        setData: function(data) {
            var newData = this.sandbox.util.extend(true, {}, this.$el.data(constants.dataKey), data);

            this.$el.data(constants.dataKey, newData);
            this.$el.trigger('change');

            return newData;
        },

        /**
         * Returns URL generated from given data.
         *
         * @param {{scheme, specificPart}} data
         *
         * @returns {String}
         */
        getUrl: function(data) {
            if (!!data && !!data.scheme && !!data.specificPart) {
                return data.scheme + data.specificPart;
            }

            return null;
        },

        /**
         * Returns dom-data.
         *
         * @returns {{scheme, specificPart}}
         */
        getData: function() {
            return this.$el.data(constants.dataKey);
        }
    };
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    if (window.Backbone) {
        define('backbone', [], function() {
            return window.Backbone;
        });
    } else {
        require.config({
            paths: { backbone: 'bower_components/backbone/backbone' },
            shim: { backbone: { exports: 'Backbone', deps: ['underscore', 'jquery'] } }
        });
    }

    define('husky_extensions/backbone',['backbone'], {
        name: 'Backbone',

        initialize: function(app) {
            var core = app.core,
                sandbox = app.sandbox,
                _ = app.sandbox.util._,
                views = {};

            core.mvc = require('backbone');

            sandbox.mvc = {};

            sandbox.mvc.routes = [];

            sandbox.mvc.Router = function(options) {
                return core.mvc.Router.extend(options);
            };

            sandbox.mvc.View = function(options) {
                return core.mvc.View.extend(options);
            };

            sandbox.mvc.Model = function(options) {
                return core.mvc.Model.extend(options);
            };

            sandbox.mvc.Collection = function(options) {
                return core.mvc.Collection.extend(options);
            };

            sandbox.mvc.history = core.mvc.history;

            define('mvc/view', function() {
                return sandbox.mvc.View;
            });

            define('mvc/model', function() {
                return sandbox.mvc.Model;
            });

            define('mvc/collection', function() {
                return sandbox.mvc.Collection;
            });

            // Injecting a Backbone view in the Component just before initialization.
            // This View's class will be built and cached this first time the component is included.
            app.components.before('initialize', function(options) {

                if (!this) {
                    throw new Error('Missing context!');
                }

                // check component needs a view
                if (!!this.view) {

                    var View = views[options.ref],
                        ext;

                    if (!View) {
                        ext = _.pick(this, 'model', 'collection', 'id', 'attributes', 'className', 'tagName', 'events');
                        views[options.ref] = View = sandbox.mvc.View(ext);
                    }
                    this.view = new View({ el: this.$el });
                    this.view.sandbox = this.sandbox;
                    this.view.parent = this;
                }
            });

            app.components.before('remove', function() {

                if (!this) {
                    throw new Error('Missing context!');
                }

                this.view && this.view.stopListening();
            });
        },

        afterAppStart: function(app) {
            app.sandbox.util._.delay(function() {
                if (!app.core.mvc.History.started) {
                    app.core.mvc.history.start();
                }
            }, 250);
        }
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 * @module husky/components/collection-navigation
 */


(function() {

    'use strict';

    define('husky_extensions/cache-factory',[], {
        name: 'cacheFactory',

        initialize: function(app) {
            var caches = {};

            /**
             * @class Cache
             */
            function Cache(cacheId) {
                var store = {};

                return {
                    /**
                     * @method get
                     */
                    get: function(key) {
                        return store[key] || null;
                    },

                    /**
                     * change an entry
                     * @method put
                     */
                    put: function(key, value) {
                        store[key] = value;
                        return value;
                    },

                    /**
                     * Returns the item of the given index
                     * @method delete
                     * @param {Mixed} key
                     * @return {Object}
                     */
                    delete: function(key) {
                        delete store[key];
                    },

                    /**
                     * Clears the cache object of any entries
                     * @method deleteAll
                     */
                    deleteAll: function() {
                        store = {};
                    },

                    /**
                     * Destroys the cache instance
                     * @method destroy
                     * @return {Object}
                     */
                    destroy: function() {
                        delete caches[cacheId];
                    }
                };
            };

            app.sandbox.cacheFactory = {
                /**
                 * Initialize a new cache and store the refernce
                 * @method create
                 * @param  {Number} cacheId
                 */
                create: function(cacheId) {
                    cacheId = cacheId || app.core.util.uniqueId('cache');

                    if (cacheId in caches) {
                        throw new Error('cacheId already exists.');
                    }

                    return caches[cacheId] = new Cache(cacheId);
                },

                /**
                 * Get a cache reference
                 * @method get 
                 * @param  {[Number} cacheId
                 */
                get: function(cacheId) {
                    return caches[cacheId];
                }
            };
        }
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    require.config({
        paths: {
            ckeditor: 'bower_components/ckeditor/ckeditor',
            jqueryAdapter: 'bower_components/ckeditor/adapters/jquery'
        },
        shim: {
            jqueryAdapter: {
                deps: ['jquery', 'ckeditor']
            }
        }
    });

    define('husky_extensions/ckeditor-extension',['underscore', 'services/husky/util', 'ckeditor', 'jqueryAdapter'], function(_, Util) {

        var getConfig = function() {
            return {
                format_tags: 'p;h1;h2;h3;h4;h5;h6',
                width: '100%',
                defaultLanguage: 'en',
                removeButtons: '',
                removePlugins: 'elementspath,magicline',
                removeDialogTabs: 'image:advanced;link:advanced',
                allowedContent: true,
                resize_enabled: false,
                enterMode: 'P',
                uiColor: '#ffffff',
                skin: 'husky'
            };
        };

        /**
         * Builder to extend basic toolbar.
         *
         * @param toolbar
         *
         * @constructor
         */
        function ToolbarBuilder(toolbar) {
            this.toolbar = toolbar;
        }

        /**
         * Add toolbar items.
         *
         * @param {String} name
         * @param {String[]} items
         */
        ToolbarBuilder.prototype.add = function(name, items) {
            if (!this.toolbar[name]) {
                this.toolbar[name] = [];
            }

            this.toolbar[name] = this.toolbar[name].concat(items);
        };

        /**
         * Remove toolbar items.
         *
         * @param {String} name
         * @param {String[]} items
         */
        ToolbarBuilder.prototype.remove = function(name, items) {
            if (!this.toolbar[name]) {
                this.toolbar[name] = [];
            }

            this.toolbar[name] = _.difference(this.toolbar[name], items);
        };

        /**
         * Returns toolbar.
         *
         * @returns {{name, items}[]}
         */
        ToolbarBuilder.prototype.get = function() {
            return _.map(this.toolbar, function(items, name) {
                return {name: name, items: items};
            });
        };

        /**
         * Returns raw toolbar data.
         *
         * @returns {Object}
         */
        ToolbarBuilder.getRaw = function() {
            return this.toolbar;
        };

        return {

            name: 'ckeditor',

            initialize: function(app) {

                var availableToolbarSections = {
                        semantics: ['Format'],
                        basicstyles: ['Superscript', 'Subscript', 'Italic', 'Bold', 'Underline', 'Strike'],
                        blockstyles: ['JustifyLeft', 'JustifyCenter', 'JustifyRight', 'JustifyBlock'],
                        list: ['NumberedList', 'BulletedList'],
                        paste: ['PasteFromWord'],
                        links: ['Link'],
                        insert: ['Table'],
                        styles: ['Styles'],
                        code: ['Source']
                    },
                    toolbar,
                    plugins = ['justify', 'format', 'sourcearea', 'link', 'table', 'pastefromword', 'autogrow'],
                    icons = {
                        Format: 'font',
                        Strike: 'strikethrough',
                        JustifyLeft: 'align-left',
                        JustifyCenter: 'align-center',
                        JustifyRight: 'align-right',
                        JustifyBlock: 'align-justify',
                        NumberedList: 'list-ol',
                        BulletedList: 'list',
                        PasteFromWord: 'clipboard',
                        Styles: 'header',
                        Source: 'code'
                    },
                    relations = {
                        Link: ['Unlink']
                    },

                    /**
                     * Expands a given toolbar. Iterates over all sections and
                     * all section items in the toolbar. If there is a relation
                     * defined for a section item it adds all items of the relations
                     * to that section.
                     *
                     * @param {Object} toolbar The toolbar to expand
                     * @returns {Object} the expanded toolbar
                     */
                    expandToolbar = function(toolbar) {
                        var expandedToolbar = {}, expandedSection;

                        Util.each(toolbar, function(sectionName, section) {
                            expandedSection = [];

                            Util.foreach(section, function(sectionItem) {
                                expandedSection.push(sectionItem);
                                if (!!relations[sectionItem]) {
                                    expandedSection = expandedSection.concat(relations[sectionItem]);
                                }
                            }.bind(this));

                            expandedToolbar[sectionName] = expandedSection;
                        }.bind(this));

                        return expandedToolbar;
                    };

                app.sandbox.ckeditor = {

                    addPlugin: function(name, plugin) {
                        plugins.push(name);

                        CKEDITOR.plugins.add(name, plugin);
                    },

                    addToolbarButton: function(sectionName, button, icon, buttonRelations) {
                        if (!availableToolbarSections[sectionName]) {
                            availableToolbarSections[sectionName] = [];
                        }

                        availableToolbarSections[sectionName].push(button);

                        if (!!icon) {
                            icons[button] = icon;
                        }

                        relations[button] = buttonRelations;
                    },

                    setToolbar: function(newToolbar) {
                        toolbar = Util.deepCopy(newToolbar);
                    },

                    getAvailableToolbarSections: function() {
                        return Util.deepCopy(availableToolbarSections);
                    },

                    getToolbar: function() {
                        if (!toolbar) {
                            return expandToolbar(this.getAvailableToolbarSections());
                        }

                        return expandToolbar(toolbar);
                    },

                    getIcon: function(button) {
                        if (icons.hasOwnProperty(button)) {
                            return icons[button];
                        }

                        return button.toLowerCase();
                    },

                    getToolbarBuilder: function() {
                        return new ToolbarBuilder(this.getToolbar());
                    },

                    createToolbarBuilder: function(sections) {
                        var toolbarSections = {};
                        for (var i = 0, length = sections.length; i < length; ++i) {
                            toolbarSections[sections[i].name] = sections[i].items;
                        }

                        return new ToolbarBuilder(toolbarSections);
                    },

                    // callback when editor is ready
                    init: function(selector, callback, config) {
                        var configuration = app.sandbox.util.extend(true, {}, getConfig.call(), config), $editor;
                        configuration.extraPlugins = plugins.join(',');

                        if (!!callback && typeof callback === 'function') {
                            $editor = $(selector).ckeditor(callback, configuration);
                        } else {
                            $editor = $(selector).ckeditor(configuration);
                        }

                        // customize ckeditor dialog appearance on 'dialogDefinition' (=> open)
                        // and filter link/target options
                        CKEDITOR.on('dialogDefinition', function(ev) {
                            // take the dialog name and its definition from the event
                            // data.
                            var dialogName = ev.data.name,
                                dialogDefinition = ev.data.definition;

                            // check if the definition is from the dialog we're
                            // interested in (the "Link" dialog).
                            if (dialogName == 'link') {
                                // get a reference to the "Link Info" and "Target" tab.
                                var infoTab = dialogDefinition.getContents('info'),
                                    targetTab = dialogDefinition.getContents('target'),

                                // get a reference to the link type
                                    linkOptions = infoTab.get('linkType'),
                                    targetOptions = targetTab.get('linkTargetType'),

                                // list of included link options
                                    includedLinkOptions = [
                                        'url',
                                        'email'
                                    ],
                                    selectedLinkOption = [],

                                // list of included link target options
                                    includedTargetOptions = [
                                        'notSet',
                                        '_blank',
                                        '_self'
                                    ],
                                    selectedTargetOptions = [];

                                // just show included link options
                                for (var i = 0; i < linkOptions.items.length; i++) {
                                    if (includedLinkOptions.indexOf(linkOptions.items[i][1]) !== -1) {
                                        selectedLinkOption.push(linkOptions.items[i]);
                                    }
                                }

                                // just show included target options
                                for (var i = 0; i < targetOptions.items.length; i++) {
                                    if (includedTargetOptions.indexOf(targetOptions.items[i][1]) !== -1) {
                                        selectedTargetOptions.push(targetOptions.items[i]);
                                    }
                                }

                                linkOptions.items = selectedLinkOption;
                                targetOptions.items = selectedTargetOptions;
                            }
                        });

                        return $editor.editor;
                    },

                    getInstance: function(id) {
                        var instance = CKEDITOR.instances[id];

                        if (!!instance) {
                            return instance;
                        }
                    }
                };
            }

        };
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
define('husky_extensions/collection',[],function() {

    'use strict';

    // simplified backbone collection
    var Collection = {
        byId: {},
        objs: [],

        add: function(obj) {
            this.byId[obj.id] = obj;
            this.objs.push(obj);
        },

        get: function(obj) {
            return this.byId[obj.id || obj];
        }
    };

    return {

        name: 'Collection',

        initialize: function(app) {
            app.sandbox.data.Collection = Collection;
        }

    };
});

/*
 * jQuery MiniColors: A tiny color picker built on jQuery
 *
 * Copyright: Cory LaViska for A Beautiful Site, LLC: http://www.abeautifulsite.net/
 *
 * Contribute: https://github.com/claviska/jquery-minicolors
 *
 * @license: http://opensource.org/licenses/MIT
 *
 */
(function (factory) {
    /* jshint ignore:start */
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as an anonymous module.
        define('jquery-minicolors',['jquery'], factory);
    } else if (typeof exports === 'object') {
        // Node/CommonJS
        module.exports = factory(require('jquery'));
    } else {
        // Browser globals
        factory(jQuery);
    }
    /* jshint ignore:end */
}(function ($) {

    // Defaults
    $.minicolors = {
        defaults: {
            animationSpeed: 50,
            animationEasing: 'swing',
            change: null,
            changeDelay: 0,
            control: 'hue',
            dataUris: true,
            defaultValue: '',
            hide: null,
            hideSpeed: 100,
            inline: false,
            letterCase: 'lowercase',
            opacity: false,
            position: 'bottom left',
            show: null,
            showSpeed: 100,
            theme: 'default'
        }
    };

    // Public methods
    $.extend($.fn, {
        minicolors: function(method, data) {

            switch(method) {

                // Destroy the control
                case 'destroy':
                    $(this).each( function() {
                        destroy($(this));
                    });
                    return $(this);

                // Hide the color picker
                case 'hide':
                    hide();
                    return $(this);

                // Get/set opacity
                case 'opacity':
                    // Getter
                    if( data === undefined ) {
                        // Getter
                        return $(this).attr('data-opacity');
                    } else {
                        // Setter
                        $(this).each( function() {
                            updateFromInput($(this).attr('data-opacity', data));
                        });
                    }
                    return $(this);

                // Get an RGB(A) object based on the current color/opacity
                case 'rgbObject':
                    return rgbObject($(this), method === 'rgbaObject');

                // Get an RGB(A) string based on the current color/opacity
                case 'rgbString':
                case 'rgbaString':
                    return rgbString($(this), method === 'rgbaString');

                // Get/set settings on the fly
                case 'settings':
                    if( data === undefined ) {
                        return $(this).data('minicolors-settings');
                    } else {
                        // Setter
                        $(this).each( function() {
                            var settings = $(this).data('minicolors-settings') || {};
                            destroy($(this));
                            $(this).minicolors($.extend(true, settings, data));
                        });
                    }
                    return $(this);

                // Show the color picker
                case 'show':
                    show( $(this).eq(0) );
                    return $(this);

                // Get/set the hex color value
                case 'value':
                    if( data === undefined ) {
                        // Getter
                        return $(this).val();
                    } else {
                        // Setter
                        $(this).each( function() {
                            updateFromInput($(this).val(data));
                        });
                    }
                    return $(this);

                // Initializes the control
                default:
                    if( method !== 'create' ) data = method;
                    $(this).each( function() {
                        init($(this), data);
                    });
                    return $(this);

            }

        }
    });

    // Initialize input elements
    function init(input, settings) {

        var minicolors = $('<div class="minicolors" />'),
            defaults = $.minicolors.defaults;

        // Do nothing if already initialized
        if( input.data('minicolors-initialized') ) return;

        // Handle settings
        settings = $.extend(true, {}, defaults, settings);

        // The wrapper
        minicolors
            .addClass('minicolors-theme-' + settings.theme)
            .toggleClass('minicolors-with-opacity', settings.opacity)
            .toggleClass('minicolors-no-data-uris', settings.dataUris !== true);

        // Custom positioning
        if( settings.position !== undefined ) {
            $.each(settings.position.split(' '), function() {
                minicolors.addClass('minicolors-position-' + this);
            });
        }

        // The input
        input
            .addClass('minicolors-input')
            .data('minicolors-initialized', false)
            .data('minicolors-settings', settings)
            .prop('size', 7)
            .wrap(minicolors)
            .after(
                '<div class="minicolors-panel minicolors-slider-' + settings.control + '">' +
                    '<div class="minicolors-slider minicolors-sprite">' +
                        '<div class="minicolors-picker"></div>' +
                    '</div>' +
                    '<div class="minicolors-opacity-slider minicolors-sprite">' +
                        '<div class="minicolors-picker"></div>' +
                    '</div>' +
                    '<div class="minicolors-grid minicolors-sprite">' +
                        '<div class="minicolors-grid-inner"></div>' +
                        '<div class="minicolors-picker"><div></div></div>' +
                    '</div>' +
                '</div>'
            );

        // The swatch
        if( !settings.inline ) {
            input.after('<span class="minicolors-swatch minicolors-sprite"><span class="minicolors-swatch-color"></span></span>');
            input.next('.minicolors-swatch').on('click', function(event) {
                event.preventDefault();
                input.focus();
            });
        }

        // Prevent text selection in IE
        input.parent().find('.minicolors-panel').on('selectstart', function() { return false; }).end();

        // Inline controls
        if( settings.inline ) input.parent().addClass('minicolors-inline');

        updateFromInput(input, false);

        input.data('minicolors-initialized', true);

    }

    // Returns the input back to its original state
    function destroy(input) {

        var minicolors = input.parent();

        // Revert the input element
        input
            .removeData('minicolors-initialized')
            .removeData('minicolors-settings')
            .removeProp('size')
            .removeClass('minicolors-input');

        // Remove the wrap and destroy whatever remains
        minicolors.before(input).remove();

    }

    // Shows the specified dropdown panel
    function show(input) {

        var minicolors = input.parent(),
            panel = minicolors.find('.minicolors-panel'),
            settings = input.data('minicolors-settings');

        // Do nothing if uninitialized, disabled, inline, or already open
        if( !input.data('minicolors-initialized') ||
            input.prop('disabled') ||
            minicolors.hasClass('minicolors-inline') ||
            minicolors.hasClass('minicolors-focus')
        ) return;

        hide();

        minicolors.addClass('minicolors-focus');
        panel
            .stop(true, true)
            .fadeIn(settings.showSpeed, function() {
                if( settings.show ) settings.show.call(input.get(0));
            });

    }

    // Hides all dropdown panels
    function hide() {

        $('.minicolors-focus').each( function() {

            var minicolors = $(this),
                input = minicolors.find('.minicolors-input'),
                panel = minicolors.find('.minicolors-panel'),
                settings = input.data('minicolors-settings');

            panel.fadeOut(settings.hideSpeed, function() {
                if( settings.hide ) settings.hide.call(input.get(0));
                minicolors.removeClass('minicolors-focus');
            });

        });
    }

    // Moves the selected picker
    function move(target, event, animate) {

        var input = target.parents('.minicolors').find('.minicolors-input'),
            settings = input.data('minicolors-settings'),
            picker = target.find('[class$=-picker]'),
            offsetX = target.offset().left,
            offsetY = target.offset().top,
            x = Math.round(event.pageX - offsetX),
            y = Math.round(event.pageY - offsetY),
            duration = animate ? settings.animationSpeed : 0,
            wx, wy, r, phi;

        // Touch support
        if( event.originalEvent.changedTouches ) {
            x = event.originalEvent.changedTouches[0].pageX - offsetX;
            y = event.originalEvent.changedTouches[0].pageY - offsetY;
        }

        // Constrain picker to its container
        if( x < 0 ) x = 0;
        if( y < 0 ) y = 0;
        if( x > target.width() ) x = target.width();
        if( y > target.height() ) y = target.height();

        // Constrain color wheel values to the wheel
        if( target.parent().is('.minicolors-slider-wheel') && picker.parent().is('.minicolors-grid') ) {
            wx = 75 - x;
            wy = 75 - y;
            r = Math.sqrt(wx * wx + wy * wy);
            phi = Math.atan2(wy, wx);
            if( phi < 0 ) phi += Math.PI * 2;
            if( r > 75 ) {
                r = 75;
                x = 75 - (75 * Math.cos(phi));
                y = 75 - (75 * Math.sin(phi));
            }
            x = Math.round(x);
            y = Math.round(y);
        }

        // Move the picker
        if( target.is('.minicolors-grid') ) {
            picker
                .stop(true)
                .animate({
                    top: y + 'px',
                    left: x + 'px'
                }, duration, settings.animationEasing, function() {
                    updateFromControl(input, target);
                });
        } else {
            picker
                .stop(true)
                .animate({
                    top: y + 'px'
                }, duration, settings.animationEasing, function() {
                    updateFromControl(input, target);
                });
        }

    }

    // Sets the input based on the color picker values
    function updateFromControl(input, target) {

        function getCoords(picker, container) {

            var left, top;
            if( !picker.length || !container ) return null;
            left = picker.offset().left;
            top = picker.offset().top;

            return {
                x: left - container.offset().left + (picker.outerWidth() / 2),
                y: top - container.offset().top + (picker.outerHeight() / 2)
            };

        }

        var hue, saturation, brightness, x, y, r, phi,

            hex = input.val(),
            opacity = input.attr('data-opacity'),

            // Helpful references
            minicolors = input.parent(),
            settings = input.data('minicolors-settings'),
            swatch = minicolors.find('.minicolors-swatch'),

            // Panel objects
            grid = minicolors.find('.minicolors-grid'),
            slider = minicolors.find('.minicolors-slider'),
            opacitySlider = minicolors.find('.minicolors-opacity-slider'),

            // Picker objects
            gridPicker = grid.find('[class$=-picker]'),
            sliderPicker = slider.find('[class$=-picker]'),
            opacityPicker = opacitySlider.find('[class$=-picker]'),

            // Picker positions
            gridPos = getCoords(gridPicker, grid),
            sliderPos = getCoords(sliderPicker, slider),
            opacityPos = getCoords(opacityPicker, opacitySlider);

        // Handle colors
        if( target.is('.minicolors-grid, .minicolors-slider') ) {

            // Determine HSB values
            switch(settings.control) {

                case 'wheel':
                    // Calculate hue, saturation, and brightness
                    x = (grid.width() / 2) - gridPos.x;
                    y = (grid.height() / 2) - gridPos.y;
                    r = Math.sqrt(x * x + y * y);
                    phi = Math.atan2(y, x);
                    if( phi < 0 ) phi += Math.PI * 2;
                    if( r > 75 ) {
                        r = 75;
                        gridPos.x = 69 - (75 * Math.cos(phi));
                        gridPos.y = 69 - (75 * Math.sin(phi));
                    }
                    saturation = keepWithin(r / 0.75, 0, 100);
                    hue = keepWithin(phi * 180 / Math.PI, 0, 360);
                    brightness = keepWithin(100 - Math.floor(sliderPos.y * (100 / slider.height())), 0, 100);
                    hex = hsb2hex({
                        h: hue,
                        s: saturation,
                        b: brightness
                    });

                    // Update UI
                    slider.css('backgroundColor', hsb2hex({ h: hue, s: saturation, b: 100 }));
                    break;

                case 'saturation':
                    // Calculate hue, saturation, and brightness
                    hue = keepWithin(parseInt(gridPos.x * (360 / grid.width()), 10), 0, 360);
                    saturation = keepWithin(100 - Math.floor(sliderPos.y * (100 / slider.height())), 0, 100);
                    brightness = keepWithin(100 - Math.floor(gridPos.y * (100 / grid.height())), 0, 100);
                    hex = hsb2hex({
                        h: hue,
                        s: saturation,
                        b: brightness
                    });

                    // Update UI
                    slider.css('backgroundColor', hsb2hex({ h: hue, s: 100, b: brightness }));
                    minicolors.find('.minicolors-grid-inner').css('opacity', saturation / 100);
                    break;

                case 'brightness':
                    // Calculate hue, saturation, and brightness
                    hue = keepWithin(parseInt(gridPos.x * (360 / grid.width()), 10), 0, 360);
                    saturation = keepWithin(100 - Math.floor(gridPos.y * (100 / grid.height())), 0, 100);
                    brightness = keepWithin(100 - Math.floor(sliderPos.y * (100 / slider.height())), 0, 100);
                    hex = hsb2hex({
                        h: hue,
                        s: saturation,
                        b: brightness
                    });

                    // Update UI
                    slider.css('backgroundColor', hsb2hex({ h: hue, s: saturation, b: 100 }));
                    minicolors.find('.minicolors-grid-inner').css('opacity', 1 - (brightness / 100));
                    break;

                default:
                    // Calculate hue, saturation, and brightness
                    hue = keepWithin(360 - parseInt(sliderPos.y * (360 / slider.height()), 10), 0, 360);
                    saturation = keepWithin(Math.floor(gridPos.x * (100 / grid.width())), 0, 100);
                    brightness = keepWithin(100 - Math.floor(gridPos.y * (100 / grid.height())), 0, 100);
                    hex = hsb2hex({
                        h: hue,
                        s: saturation,
                        b: brightness
                    });

                    // Update UI
                    grid.css('backgroundColor', hsb2hex({ h: hue, s: 100, b: 100 }));
                    break;

            }

            // Adjust case
            input.val( convertCase(hex, settings.letterCase) );

        }

        // Handle opacity
        if( target.is('.minicolors-opacity-slider') ) {
            if( settings.opacity ) {
                opacity = parseFloat(1 - (opacityPos.y / opacitySlider.height())).toFixed(2);
            } else {
                opacity = 1;
            }
            if( settings.opacity ) input.attr('data-opacity', opacity);
        }

        // Set swatch color
        swatch.find('SPAN').css({
            backgroundColor: hex,
            opacity: opacity
        });

        // Handle change event
        doChange(input, hex, opacity);

    }

    // Sets the color picker values from the input
    function updateFromInput(input, preserveInputValue) {

        var hex,
            hsb,
            opacity,
            x, y, r, phi,

            // Helpful references
            minicolors = input.parent(),
            settings = input.data('minicolors-settings'),
            swatch = minicolors.find('.minicolors-swatch'),

            // Panel objects
            grid = minicolors.find('.minicolors-grid'),
            slider = minicolors.find('.minicolors-slider'),
            opacitySlider = minicolors.find('.minicolors-opacity-slider'),

            // Picker objects
            gridPicker = grid.find('[class$=-picker]'),
            sliderPicker = slider.find('[class$=-picker]'),
            opacityPicker = opacitySlider.find('[class$=-picker]');

        // Determine hex/HSB values
        hex = convertCase(parseHex(input.val(), true), settings.letterCase);
        if( !hex ){
            hex = convertCase(parseHex(settings.defaultValue, true), settings.letterCase);
        }
        hsb = hex2hsb(hex);

        // Update input value
        if( !preserveInputValue ) input.val(hex);

        // Determine opacity value
        if( settings.opacity ) {
            // Get from data-opacity attribute and keep within 0-1 range
            opacity = input.attr('data-opacity') === '' ? 1 : keepWithin(parseFloat(input.attr('data-opacity')).toFixed(2), 0, 1);
            if( isNaN(opacity) ) opacity = 1;
            input.attr('data-opacity', opacity);
            swatch.find('SPAN').css('opacity', opacity);

            // Set opacity picker position
            y = keepWithin(opacitySlider.height() - (opacitySlider.height() * opacity), 0, opacitySlider.height());
            opacityPicker.css('top', y + 'px');
        }

        // Update swatch
        swatch.find('SPAN').css('backgroundColor', hex);

        // Determine picker locations
        switch(settings.control) {

            case 'wheel':
                // Set grid position
                r = keepWithin(Math.ceil(hsb.s * 0.75), 0, grid.height() / 2);
                phi = hsb.h * Math.PI / 180;
                x = keepWithin(75 - Math.cos(phi) * r, 0, grid.width());
                y = keepWithin(75 - Math.sin(phi) * r, 0, grid.height());
                gridPicker.css({
                    top: y + 'px',
                    left: x + 'px'
                });

                // Set slider position
                y = 150 - (hsb.b / (100 / grid.height()));
                if( hex === '' ) y = 0;
                sliderPicker.css('top', y + 'px');

                // Update panel color
                slider.css('backgroundColor', hsb2hex({ h: hsb.h, s: hsb.s, b: 100 }));
                break;

            case 'saturation':
                // Set grid position
                x = keepWithin((5 * hsb.h) / 12, 0, 150);
                y = keepWithin(grid.height() - Math.ceil(hsb.b / (100 / grid.height())), 0, grid.height());
                gridPicker.css({
                    top: y + 'px',
                    left: x + 'px'
                });

                // Set slider position
                y = keepWithin(slider.height() - (hsb.s * (slider.height() / 100)), 0, slider.height());
                sliderPicker.css('top', y + 'px');

                // Update UI
                slider.css('backgroundColor', hsb2hex({ h: hsb.h, s: 100, b: hsb.b }));
                minicolors.find('.minicolors-grid-inner').css('opacity', hsb.s / 100);
                break;

            case 'brightness':
                // Set grid position
                x = keepWithin((5 * hsb.h) / 12, 0, 150);
                y = keepWithin(grid.height() - Math.ceil(hsb.s / (100 / grid.height())), 0, grid.height());
                gridPicker.css({
                    top: y + 'px',
                    left: x + 'px'
                });

                // Set slider position
                y = keepWithin(slider.height() - (hsb.b * (slider.height() / 100)), 0, slider.height());
                sliderPicker.css('top', y + 'px');

                // Update UI
                slider.css('backgroundColor', hsb2hex({ h: hsb.h, s: hsb.s, b: 100 }));
                minicolors.find('.minicolors-grid-inner').css('opacity', 1 - (hsb.b / 100));
                break;

            default:
                // Set grid position
                x = keepWithin(Math.ceil(hsb.s / (100 / grid.width())), 0, grid.width());
                y = keepWithin(grid.height() - Math.ceil(hsb.b / (100 / grid.height())), 0, grid.height());
                gridPicker.css({
                    top: y + 'px',
                    left: x + 'px'
                });

                // Set slider position
                y = keepWithin(slider.height() - (hsb.h / (360 / slider.height())), 0, slider.height());
                sliderPicker.css('top', y + 'px');

                // Update panel color
                grid.css('backgroundColor', hsb2hex({ h: hsb.h, s: 100, b: 100 }));
                break;

        }

        // Fire change event, but only if minicolors is fully initialized
        if( input.data('minicolors-initialized') ) {
            doChange(input, hex, opacity);
        }

    }

    // Runs the change and changeDelay callbacks
    function doChange(input, hex, opacity) {

        var settings = input.data('minicolors-settings'),
            lastChange = input.data('minicolors-lastChange');

        // Only run if it actually changed
        if( !lastChange || lastChange.hex !== hex || lastChange.opacity !== opacity ) {

            // Remember last-changed value
            input.data('minicolors-lastChange', {
                hex: hex,
                opacity: opacity
            });

            // Fire change event
            if( settings.change ) {
                if( settings.changeDelay ) {
                    // Call after a delay
                    clearTimeout(input.data('minicolors-changeTimeout'));
                    input.data('minicolors-changeTimeout', setTimeout( function() {
                        settings.change.call(input.get(0), hex, opacity);
                    }, settings.changeDelay));
                } else {
                    // Call immediately
                    settings.change.call(input.get(0), hex, opacity);
                }
            }
            input.trigger('change').trigger('input');
        }

    }

    // Generates an RGB(A) object based on the input's value
    function rgbObject(input) {
        var hex = parseHex($(input).val(), true),
            rgb = hex2rgb(hex),
            opacity = $(input).attr('data-opacity');
        if( !rgb ) return null;
        if( opacity !== undefined ) $.extend(rgb, { a: parseFloat(opacity) });
        return rgb;
    }

    // Genearates an RGB(A) string based on the input's value
    function rgbString(input, alpha) {
        var hex = parseHex($(input).val(), true),
            rgb = hex2rgb(hex),
            opacity = $(input).attr('data-opacity');
        if( !rgb ) return null;
        if( opacity === undefined ) opacity = 1;
        if( alpha ) {
            return 'rgba(' + rgb.r + ', ' + rgb.g + ', ' + rgb.b + ', ' + parseFloat(opacity) + ')';
        } else {
            return 'rgb(' + rgb.r + ', ' + rgb.g + ', ' + rgb.b + ')';
        }
    }

    // Converts to the letter case specified in settings
    function convertCase(string, letterCase) {
        return letterCase === 'uppercase' ? string.toUpperCase() : string.toLowerCase();
    }

    // Parses a string and returns a valid hex string when possible
    function parseHex(string, expand) {
        string = string.replace(/[^A-F0-9]/ig, '');
        if( string.length !== 3 && string.length !== 6 ) return '';
        if( string.length === 3 && expand ) {
            string = string[0] + string[0] + string[1] + string[1] + string[2] + string[2];
        }
        return '#' + string;
    }

    // Keeps value within min and max
    function keepWithin(value, min, max) {
        if( value < min ) value = min;
        if( value > max ) value = max;
        return value;
    }

    // Converts an HSB object to an RGB object
    function hsb2rgb(hsb) {
        var rgb = {};
        var h = Math.round(hsb.h);
        var s = Math.round(hsb.s * 255 / 100);
        var v = Math.round(hsb.b * 255 / 100);
        if(s === 0) {
            rgb.r = rgb.g = rgb.b = v;
        } else {
            var t1 = v;
            var t2 = (255 - s) * v / 255;
            var t3 = (t1 - t2) * (h % 60) / 60;
            if( h === 360 ) h = 0;
            if( h < 60 ) { rgb.r = t1; rgb.b = t2; rgb.g = t2 + t3; }
            else if( h < 120 ) {rgb.g = t1; rgb.b = t2; rgb.r = t1 - t3; }
            else if( h < 180 ) {rgb.g = t1; rgb.r = t2; rgb.b = t2 + t3; }
            else if( h < 240 ) {rgb.b = t1; rgb.r = t2; rgb.g = t1 - t3; }
            else if( h < 300 ) {rgb.b = t1; rgb.g = t2; rgb.r = t2 + t3; }
            else if( h < 360 ) {rgb.r = t1; rgb.g = t2; rgb.b = t1 - t3; }
            else { rgb.r = 0; rgb.g = 0; rgb.b = 0; }
        }
        return {
            r: Math.round(rgb.r),
            g: Math.round(rgb.g),
            b: Math.round(rgb.b)
        };
    }

    // Converts an RGB object to a hex string
    function rgb2hex(rgb) {
        var hex = [
            rgb.r.toString(16),
            rgb.g.toString(16),
            rgb.b.toString(16)
        ];
        $.each(hex, function(nr, val) {
            if (val.length === 1) hex[nr] = '0' + val;
        });
        return '#' + hex.join('');
    }

    // Converts an HSB object to a hex string
    function hsb2hex(hsb) {
        return rgb2hex(hsb2rgb(hsb));
    }

    // Converts a hex string to an HSB object
    function hex2hsb(hex) {
        var hsb = rgb2hsb(hex2rgb(hex));
        if( hsb.s === 0 ) hsb.h = 360;
        return hsb;
    }

    // Converts an RGB object to an HSB object
    function rgb2hsb(rgb) {
        var hsb = { h: 0, s: 0, b: 0 };
        var min = Math.min(rgb.r, rgb.g, rgb.b);
        var max = Math.max(rgb.r, rgb.g, rgb.b);
        var delta = max - min;
        hsb.b = max;
        hsb.s = max !== 0 ? 255 * delta / max : 0;
        if( hsb.s !== 0 ) {
            if( rgb.r === max ) {
                hsb.h = (rgb.g - rgb.b) / delta;
            } else if( rgb.g === max ) {
                hsb.h = 2 + (rgb.b - rgb.r) / delta;
            } else {
                hsb.h = 4 + (rgb.r - rgb.g) / delta;
            }
        } else {
            hsb.h = -1;
        }
        hsb.h *= 60;
        if( hsb.h < 0 ) {
            hsb.h += 360;
        }
        hsb.s *= 100/255;
        hsb.b *= 100/255;
        return hsb;
    }

    // Converts a hex string to an RGB object
    function hex2rgb(hex) {
        hex = parseInt(((hex.indexOf('#') > -1) ? hex.substring(1) : hex), 16);
        return {
            /* jshint ignore:start */
            r: hex >> 16,
            g: (hex & 0x00FF00) >> 8,
            b: (hex & 0x0000FF)
            /* jshint ignore:end */
        };
    }

    // Handle events
    $(document)
        // Hide on clicks outside of the control
        .on('mousedown.minicolors touchstart.minicolors', function(event) {
            if( !$(event.target).parents().add(event.target).hasClass('minicolors') ) {
                hide();
            }
        })
        // Start moving
        .on('mousedown.minicolors touchstart.minicolors', '.minicolors-grid, .minicolors-slider, .minicolors-opacity-slider', function(event) {
            var target = $(this);
            event.preventDefault();
            $(document).data('minicolors-target', target);
            move(target, event, true);
        })
        // Move pickers
        .on('mousemove.minicolors touchmove.minicolors', function(event) {
            var target = $(document).data('minicolors-target');
            if( target ) move(target, event);
        })
        // Stop moving
        .on('mouseup.minicolors touchend.minicolors', function() {
            $(this).removeData('minicolors-target');
        })
        // Show panel when swatch is clicked
        .on('mousedown.minicolors touchstart.minicolors', '.minicolors-swatch', function(event) {
            var input = $(this).parent().find('.minicolors-input');
            event.preventDefault();
            show(input);
        })
        // Show on focus
        .on('focus.minicolors', '.minicolors-input', function() {
            var input = $(this);
            if( !input.data('minicolors-initialized') ) return;
            show(input);
        })
        // Fix hex on blur
        .on('blur.minicolors', '.minicolors-input', function() {
            var input = $(this),
                settings = input.data('minicolors-settings');
            if( !input.data('minicolors-initialized') ) return;

            // Parse Hex
            input.val(parseHex(input.val(), true));

            // Is it blank?
            if( input.val() === '' ) input.val(parseHex(settings.defaultValue, true));

            // Adjust case
            input.val( convertCase(input.val(), settings.letterCase) );

        })
        // Handle keypresses
        .on('keydown.minicolors', '.minicolors-input', function(event) {
            var input = $(this);
            if( !input.data('minicolors-initialized') ) return;
            switch(event.keyCode) {
                case 9: // tab
                    hide();
                    break;
                case 13: // enter
                case 27: // esc
                    hide();
                    input.blur();
                    break;
            }
        })
        // Update on keyup
        .on('keyup.minicolors', '.minicolors-input', function() {
            var input = $(this);
            if( !input.data('minicolors-initialized') ) return;
            updateFromInput(input, true);
        })
        // Update on paste
        .on('paste.minicolors', '.minicolors-input', function() {
            var input = $(this);
            if( !input.data('minicolors-initialized') ) return;
            setTimeout( function() {
                updateFromInput(input, true);
            }, 1);
        });

}));
/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    require.config({
        paths: { "jquery-minicolors": 'bower_components/jquery-minicolors/jquery.minicolors' },
        shim: { backbone: { deps: ['jquery'] } }
    });

    define('husky_extensions/colorpicker',['jquery-minicolors'], {
        name: 'minicolor',

        initialize: function(app) {
            app.sandbox.colorpicker = {

                init: function(selector, configs) {
                    return app.core.dom.$(selector).minicolors(configs);
                },

                value: function(selector, value) {
                    return app.core.dom.$(selector).minicolors('value', value);
                }
            };
        }
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    define('husky_extensions/confirm',{

        name: 'Confirm',

        initialize: function(app) {

            app.sandbox.confirm = {

                /**
                 * Shows a confirmation box
                 * @param that - the context in which the box gets displayed. Must be an aura-component with a sandbox and an element
                 * @param title - the title (or translation key) for the box
                 * @param message - the message (or translation key) for the box
                 * @param okCallback - callback to execute after box was confirmed
                 * @param closeCallback - callback to execute on cancel
                 * @param [type] default 'alert'
                 */
                alert: function (that, title, message, okCallback, closeCallback, type) {
                    var $element = app.core.dom.createElement('<div/>'),
                        type = type || 'alert';
                    app.core.dom.append(that.$el, $element);

                    that.sandbox.start([
                        {
                            name: 'overlay@husky',
                            options: {
                                el: $element,
                                title: app.sandbox.translate(title),
                                message: app.sandbox.translate(message),
                                closeCallback: closeCallback,
                                okCallback: okCallback,
                                type: type
                            }
                        }
                    ]);
                }
            };
        }
    });
})();

/* =========================================================
 * bootstrap-datepicker.js
 * Repo: https://github.com/eternicode/bootstrap-datepicker/
 * Demo: http://eternicode.github.io/bootstrap-datepicker/
 * Docs: http://bootstrap-datepicker.readthedocs.org/
 * Forked from http://www.eyecon.ro/bootstrap-datepicker
 * =========================================================
 * Started by Stefan Petre; improvements by Andrew Rowls + contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================= */

(function($, undefined){

	var $window = $(window);

	function UTCDate(){
		return new Date(Date.UTC.apply(Date, arguments));
	}
	function UTCToday(){
		var today = new Date();
		return UTCDate(today.getFullYear(), today.getMonth(), today.getDate());
	}
	function alias(method){
		return function(){
			return this[method].apply(this, arguments);
		};
	}

	var DateArray = (function(){
		var extras = {
			get: function(i){
				return this.slice(i)[0];
			},
			contains: function(d){
				// Array.indexOf is not cross-browser;
				// $.inArray doesn't work with Dates
				var val = d && d.valueOf();
				for (var i=0, l=this.length; i < l; i++)
					if (this[i].valueOf() === val)
						return i;
				return -1;
			},
			remove: function(i){
				this.splice(i,1);
			},
			replace: function(new_array){
				if (!new_array)
					return;
				if (!$.isArray(new_array))
					new_array = [new_array];
				this.clear();
				this.push.apply(this, new_array);
			},
			clear: function(){
				this.length = 0;
			},
			copy: function(){
				var a = new DateArray();
				a.replace(this);
				return a;
			}
		};

		return function(){
			var a = [];
			a.push.apply(a, arguments);
			$.extend(a, extras);
			return a;
		};
	})();


	// Picker object

	var Datepicker = function(element, options){
		this.dates = new DateArray();
		this.viewDate = UTCToday();
		this.focusDate = null;

		this._process_options(options);

		this.element = $(element);
		this.isInline = false;
		this.isInput = this.element.is('input');
		this.component = this.element.is('.date') ? this.element.find('.add-on, .input-group-addon, .btn') : false;
		this.hasInput = this.component && this.element.find('input').length;
		if (this.component && this.component.length === 0)
			this.component = false;

		this.picker = $(DPGlobal.template);
		this._buildEvents();
		this._attachEvents();

		if (this.isInline){
			this.picker.addClass('datepicker-inline').appendTo(this.element);
		}
		else {
			this.picker.addClass('datepicker-dropdown dropdown-menu');
		}

		if (this.o.rtl){
			this.picker.addClass('datepicker-rtl');
		}

		this.viewMode = this.o.startView;

		if (this.o.calendarWeeks)
			this.picker.find('tfoot th.today, tfoot th.clear')
						.attr('colspan', function(i, val){
							return parseInt(val) + 1;
						});

		this._allow_update = false;

		this.setStartDate(this._o.startDate);
		this.setEndDate(this._o.endDate);
		this.setDaysOfWeekDisabled(this.o.daysOfWeekDisabled);

		this.fillDow();
		this.fillMonths();

		this._allow_update = true;

		this.update();
		this.showMode();

		if (this.isInline){
			this.show();
		}
	};

	Datepicker.prototype = {
		constructor: Datepicker,

		_process_options: function(opts){
			// Store raw options for reference
			this._o = $.extend({}, this._o, opts);
			// Processed options
			var o = this.o = $.extend({}, this._o);

			// Check if "de-DE" style date is available, if not language should
			// fallback to 2 letter code eg "de"
			var lang = o.language;
			if (!dates[lang]){
				lang = lang.split('-')[0];
				if (!dates[lang])
					lang = defaults.language;
			}
			o.language = lang;

			switch (o.startView){
				case 2:
				case 'decade':
					o.startView = 2;
					break;
				case 1:
				case 'year':
					o.startView = 1;
					break;
				default:
					o.startView = 0;
			}

			switch (o.minViewMode){
				case 1:
				case 'months':
					o.minViewMode = 1;
					break;
				case 2:
				case 'years':
					o.minViewMode = 2;
					break;
				default:
					o.minViewMode = 0;
			}

			o.startView = Math.max(o.startView, o.minViewMode);

			// true, false, or Number > 0
			if (o.multidate !== true){
				o.multidate = Number(o.multidate) || false;
				if (o.multidate !== false)
					o.multidate = Math.max(0, o.multidate);
				else
					o.multidate = 1;
			}
			o.multidateSeparator = String(o.multidateSeparator);

			o.weekStart %= 7;
			o.weekEnd = ((o.weekStart + 6) % 7);

			var format = DPGlobal.parseFormat(o.format);
			if (o.startDate !== -Infinity){
				if (!!o.startDate){
					if (o.startDate instanceof Date)
						o.startDate = this._local_to_utc(this._zero_time(o.startDate));
					else
						o.startDate = DPGlobal.parseDate(o.startDate, format, o.language);
				}
				else {
					o.startDate = -Infinity;
				}
			}
			if (o.endDate !== Infinity){
				if (!!o.endDate){
					if (o.endDate instanceof Date)
						o.endDate = this._local_to_utc(this._zero_time(o.endDate));
					else
						o.endDate = DPGlobal.parseDate(o.endDate, format, o.language);
				}
				else {
					o.endDate = Infinity;
				}
			}

			o.daysOfWeekDisabled = o.daysOfWeekDisabled||[];
			if (!$.isArray(o.daysOfWeekDisabled))
				o.daysOfWeekDisabled = o.daysOfWeekDisabled.split(/[,\s]*/);
			o.daysOfWeekDisabled = $.map(o.daysOfWeekDisabled, function(d){
				return parseInt(d, 10);
			});

			var plc = String(o.orientation).toLowerCase().split(/\s+/g),
				_plc = o.orientation.toLowerCase();
			plc = $.grep(plc, function(word){
				return (/^auto|left|right|top|bottom$/).test(word);
			});
			o.orientation = {x: 'auto', y: 'auto'};
			if (!_plc || _plc === 'auto')
				; // no action
			else if (plc.length === 1){
				switch (plc[0]){
					case 'top':
					case 'bottom':
						o.orientation.y = plc[0];
						break;
					case 'left':
					case 'right':
						o.orientation.x = plc[0];
						break;
				}
			}
			else {
				_plc = $.grep(plc, function(word){
					return (/^left|right$/).test(word);
				});
				o.orientation.x = _plc[0] || 'auto';

				_plc = $.grep(plc, function(word){
					return (/^top|bottom$/).test(word);
				});
				o.orientation.y = _plc[0] || 'auto';
			}
		},
		_events: [],
		_secondaryEvents: [],
		_applyEvents: function(evs){
			for (var i=0, el, ch, ev; i < evs.length; i++){
				el = evs[i][0];
				if (evs[i].length === 2){
					ch = undefined;
					ev = evs[i][1];
				}
				else if (evs[i].length === 3){
					ch = evs[i][1];
					ev = evs[i][2];
				}
				el.on(ev, ch);
			}
		},
		_unapplyEvents: function(evs){
			for (var i=0, el, ev, ch; i < evs.length; i++){
				el = evs[i][0];
				if (evs[i].length === 2){
					ch = undefined;
					ev = evs[i][1];
				}
				else if (evs[i].length === 3){
					ch = evs[i][1];
					ev = evs[i][2];
				}
				el.off(ev, ch);
			}
		},
		_buildEvents: function(){
			if (this.isInput){ // single input
				this._events = [
					[this.element, {
						focus: $.proxy(this.show, this),
						keyup: $.proxy(function(e){
							if ($.inArray(e.keyCode, [27,37,39,38,40,32,13,9]) === -1)
								this.update();
						}, this),
						keydown: $.proxy(this.keydown, this)
					}]
				];
			}
			else if (this.component && this.hasInput){ // component: input + button
				this._events = [
					// For components that are not readonly, allow keyboard nav
					[this.element.find('input'), {
						focus: $.proxy(this.show, this),
						keyup: $.proxy(function(e){
							if ($.inArray(e.keyCode, [27,37,39,38,40,32,13,9]) === -1)
								this.update();
						}, this),
						keydown: $.proxy(this.keydown, this)
					}],
					[this.component, {
						click: $.proxy(this.show, this)
					}]
				];
			}
			else if (this.element.is('div')){  // inline datepicker
				this.isInline = true;
			}
			else {
				this._events = [
					[this.element, {
						click: $.proxy(this.show, this)
					}]
				];
			}
			this._events.push(
				// Component: listen for blur on element descendants
				[this.element, '*', {
					blur: $.proxy(function(e){
						this._focused_from = e.target;
					}, this)
				}],
				// Input: listen for blur on element
				[this.element, {
					blur: $.proxy(function(e){
						this._focused_from = e.target;
					}, this)
				}]
			);

			this._secondaryEvents = [
				[this.picker, {
					click: $.proxy(this.click, this)
				}],
				[$(window), {
					resize: $.proxy(this.place, this)
				}],
				[$(document), {
					'mousedown touchstart': $.proxy(function(e){
						// Clicked outside the datepicker, hide it
						if (!(
							this.element.is(e.target) ||
							this.element.find(e.target).length ||
							this.picker.is(e.target) ||
							this.picker.find(e.target).length
						)){
							this.hide();
						}
					}, this)
				}]
			];
		},
		_attachEvents: function(){
			this._detachEvents();
			this._applyEvents(this._events);
		},
		_detachEvents: function(){
			this._unapplyEvents(this._events);
		},
		_attachSecondaryEvents: function(){
			this._detachSecondaryEvents();
			this._applyEvents(this._secondaryEvents);
		},
		_detachSecondaryEvents: function(){
			this._unapplyEvents(this._secondaryEvents);
		},
		_trigger: function(event, altdate){
			var date = altdate || this.dates.get(-1),
				local_date = this._utc_to_local(date);

			this.element.trigger({
				type: event,
				date: local_date,
				dates: $.map(this.dates, this._utc_to_local),
				format: $.proxy(function(ix, format){
					if (arguments.length === 0){
						ix = this.dates.length - 1;
						format = this.o.format;
					}
					else if (typeof ix === 'string'){
						format = ix;
						ix = this.dates.length - 1;
					}
					format = format || this.o.format;
					var date = this.dates.get(ix);
					return DPGlobal.formatDate(date, format, this.o.language);
				}, this)
			});
		},

		show: function(){
			if (!this.isInline)
				this.picker.appendTo('body');
			this.picker.show();
			this.place();
			this._attachSecondaryEvents();
			this._trigger('show');
		},

		hide: function(){
			if (this.isInline)
				return;
			if (!this.picker.is(':visible'))
				return;
			this.focusDate = null;
			this.picker.hide().detach();
			this._detachSecondaryEvents();
			this.viewMode = this.o.startView;
			this.showMode();

			if (
				this.o.forceParse &&
				(
					this.isInput && this.element.val() ||
					this.hasInput && this.element.find('input').val()
				)
			)
				this.setValue();
			this._trigger('hide');
		},

		remove: function(){
			this.hide();
			this._detachEvents();
			this._detachSecondaryEvents();
			this.picker.remove();
			delete this.element.data().datepicker;
			if (!this.isInput){
				delete this.element.data().date;
			}
		},

		_utc_to_local: function(utc){
			return utc && new Date(utc.getTime() + (utc.getTimezoneOffset()*60000));
		},
		_local_to_utc: function(local){
			return local && new Date(local.getTime() - (local.getTimezoneOffset()*60000));
		},
		_zero_time: function(local){
			return local && new Date(local.getFullYear(), local.getMonth(), local.getDate());
		},
		_zero_utc_time: function(utc){
			return utc && new Date(Date.UTC(utc.getUTCFullYear(), utc.getUTCMonth(), utc.getUTCDate()));
		},

		getDates: function(){
			return $.map(this.dates, this._utc_to_local);
		},

		getUTCDates: function(){
			return $.map(this.dates, function(d){
				return new Date(d);
			});
		},

		getDate: function(){
			return this._utc_to_local(this.getUTCDate());
		},

		getUTCDate: function(){
			return new Date(this.dates.get(-1));
		},

		setDates: function(){
			var args = $.isArray(arguments[0]) ? arguments[0] : arguments;
			this.update.apply(this, args);
			this._trigger('changeDate');
			this.setValue();
		},

		setUTCDates: function(){
			var args = $.isArray(arguments[0]) ? arguments[0] : arguments;
			this.update.apply(this, $.map(args, this._utc_to_local));
			this._trigger('changeDate');
			this.setValue();
		},

		setDate: alias('setDates'),
		setUTCDate: alias('setUTCDates'),

		setValue: function(){
			var formatted = this.getFormattedDate();
			if (!this.isInput){
				if (this.component){
					this.element.find('input').val(formatted).change();
				}
			}
			else {
				this.element.val(formatted).change();
			}
		},

		getFormattedDate: function(format){
			if (format === undefined)
				format = this.o.format;

			var lang = this.o.language;
			return $.map(this.dates, function(d){
				return DPGlobal.formatDate(d, format, lang);
			}).join(this.o.multidateSeparator);
		},

		setStartDate: function(startDate){
			this._process_options({startDate: startDate});
			this.update();
			this.updateNavArrows();
		},

		setEndDate: function(endDate){
			this._process_options({endDate: endDate});
			this.update();
			this.updateNavArrows();
		},

		setDaysOfWeekDisabled: function(daysOfWeekDisabled){
			this._process_options({daysOfWeekDisabled: daysOfWeekDisabled});
			this.update();
			this.updateNavArrows();
		},

		place: function(){
			if (this.isInline)
				return;
			var calendarWidth = this.picker.outerWidth(),
				calendarHeight = this.picker.outerHeight(),
				visualPadding = 10,
				windowWidth = $window.width(),
				windowHeight = $window.height(),
				scrollTop = $window.scrollTop();

			var parentsZindex = [];
			this.element.parents().each(function() {
				var itemZIndex = $(this).css('z-index');
				if ( itemZIndex !== 'auto' && itemZIndex !== 0 ) parentsZindex.push( parseInt( itemZIndex ) );
			});
			var zIndex = Math.max.apply( Math, parentsZindex ) + 10;
			var offset = this.component ? this.component.parent().offset() : this.element.offset();
			var height = this.component ? this.component.outerHeight(true) : this.element.outerHeight(false);
			var width = this.component ? this.component.outerWidth(true) : this.element.outerWidth(false);
			var left = offset.left,
				top = offset.top;

			this.picker.removeClass(
				'datepicker-orient-top datepicker-orient-bottom '+
				'datepicker-orient-right datepicker-orient-left'
			);

			if (this.o.orientation.x !== 'auto'){
				this.picker.addClass('datepicker-orient-' + this.o.orientation.x);
				if (this.o.orientation.x === 'right')
					left -= calendarWidth - width;
			}
			// auto x orientation is best-placement: if it crosses a window
			// edge, fudge it sideways
			else {
				// Default to left
				this.picker.addClass('datepicker-orient-left');
				if (offset.left < 0)
					left -= offset.left - visualPadding;
				else if (offset.left + calendarWidth > windowWidth)
					left = windowWidth - calendarWidth - visualPadding;
			}

			// auto y orientation is best-situation: top or bottom, no fudging,
			// decision based on which shows more of the calendar
			var yorient = this.o.orientation.y,
				top_overflow, bottom_overflow;
			if (yorient === 'auto'){
				top_overflow = -scrollTop + offset.top - calendarHeight;
				bottom_overflow = scrollTop + windowHeight - (offset.top + height + calendarHeight);
				if (Math.max(top_overflow, bottom_overflow) === bottom_overflow)
					yorient = 'top';
				else
					yorient = 'bottom';
			}
			this.picker.addClass('datepicker-orient-' + yorient);
			if (yorient === 'top')
				top += height;
			else
				top -= calendarHeight + parseInt(this.picker.css('padding-top'));

			this.picker.css({
				top: top,
				left: left,
				zIndex: zIndex
			});
		},

		_allow_update: true,
		update: function(){
			if (!this._allow_update)
				return;

			var oldDates = this.dates.copy(),
				dates = [],
				fromArgs = false;
			if (arguments.length){
				$.each(arguments, $.proxy(function(i, date){
					if (date instanceof Date)
						date = this._local_to_utc(date);
					dates.push(date);
				}, this));
				fromArgs = true;
			}
			else {
				dates = this.isInput
						? this.element.val()
						: this.element.data('date') || this.element.find('input').val();
				if (dates && this.o.multidate)
					dates = dates.split(this.o.multidateSeparator);
				else
					dates = [dates];
				delete this.element.data().date;
			}

			dates = $.map(dates, $.proxy(function(date){
				return DPGlobal.parseDate(date, this.o.format, this.o.language);
			}, this));
			dates = $.grep(dates, $.proxy(function(date){
				return (
					date < this.o.startDate ||
					date > this.o.endDate ||
					!date
				);
			}, this), true);
			this.dates.replace(dates);

			if (this.dates.length)
				this.viewDate = new Date(this.dates.get(-1));
			else if (this.viewDate < this.o.startDate)
				this.viewDate = new Date(this.o.startDate);
			else if (this.viewDate > this.o.endDate)
				this.viewDate = new Date(this.o.endDate);

			if (fromArgs){
				// setting date by clicking
				this.setValue();
			}
			else if (dates.length){
				// setting date by typing
				if (String(oldDates) !== String(this.dates))
					this._trigger('changeDate');
			}
			if (!this.dates.length && oldDates.length)
				this._trigger('clearDate');

			this.fill();
		},

		fillDow: function(){
			var dowCnt = this.o.weekStart,
				html = '<tr>';
			if (this.o.calendarWeeks){
				var cell = '<th class="cw">&nbsp;</th>';
				html += cell;
				this.picker.find('.datepicker-days thead tr:first-child').prepend(cell);
			}
			while (dowCnt < this.o.weekStart + 7){
				html += '<th class="dow">'+dates[this.o.language].daysMin[(dowCnt++)%7]+'</th>';
			}
			html += '</tr>';
			this.picker.find('.datepicker-days thead').append(html);
		},

		fillMonths: function(){
			var html = '',
			i = 0;
			while (i < 12){
				html += '<span class="month">'+dates[this.o.language].monthsShort[i++]+'</span>';
			}
			this.picker.find('.datepicker-months td').html(html);
		},

		setRange: function(range){
			if (!range || !range.length)
				delete this.range;
			else
				this.range = $.map(range, function(d){
					return d.valueOf();
				});
			this.fill();
		},

		getClassNames: function(date){
			var cls = [],
				year = this.viewDate.getUTCFullYear(),
				month = this.viewDate.getUTCMonth(),
				today = new Date();
			if (date.getUTCFullYear() < year || (date.getUTCFullYear() === year && date.getUTCMonth() < month)){
				cls.push('old');
			}
			else if (date.getUTCFullYear() > year || (date.getUTCFullYear() === year && date.getUTCMonth() > month)){
				cls.push('new');
			}
			if (this.focusDate && date.valueOf() === this.focusDate.valueOf())
				cls.push('focused');
			// Compare internal UTC date with local today, not UTC today
			if (this.o.todayHighlight &&
				date.getUTCFullYear() === today.getFullYear() &&
				date.getUTCMonth() === today.getMonth() &&
				date.getUTCDate() === today.getDate()){
				cls.push('today');
			}
			if (this.dates.contains(date) !== -1)
				cls.push('active');
			if (date.valueOf() < this.o.startDate || date.valueOf() > this.o.endDate ||
				$.inArray(date.getUTCDay(), this.o.daysOfWeekDisabled) !== -1){
				cls.push('disabled');
			}
			if (this.range){
				if (date > this.range[0] && date < this.range[this.range.length-1]){
					cls.push('range');
				}
				if ($.inArray(date.valueOf(), this.range) !== -1){
					cls.push('selected');
				}
			}
			return cls;
		},

		fill: function(){
			var d = new Date(this.viewDate),
				year = d.getUTCFullYear(),
				month = d.getUTCMonth(),
				startYear = this.o.startDate !== -Infinity ? this.o.startDate.getUTCFullYear() : -Infinity,
				startMonth = this.o.startDate !== -Infinity ? this.o.startDate.getUTCMonth() : -Infinity,
				endYear = this.o.endDate !== Infinity ? this.o.endDate.getUTCFullYear() : Infinity,
				endMonth = this.o.endDate !== Infinity ? this.o.endDate.getUTCMonth() : Infinity,
				todaytxt = dates[this.o.language].today || dates['en'].today || '',
				cleartxt = dates[this.o.language].clear || dates['en'].clear || '',
				tooltip;
			if (isNaN(year) || isNaN(month)) return;
			this.picker.find('.datepicker-days thead th.datepicker-switch')
						.text(dates[this.o.language].months[month]+' '+year);
			this.picker.find('tfoot th.today')
						.text(todaytxt)
						.toggle(this.o.todayBtn !== false);
			this.picker.find('tfoot th.clear')
						.text(cleartxt)
						.toggle(this.o.clearBtn !== false);
			this.updateNavArrows();
			this.fillMonths();
			var prevMonth = UTCDate(year, month-1, 28),
				day = DPGlobal.getDaysInMonth(prevMonth.getUTCFullYear(), prevMonth.getUTCMonth());
			prevMonth.setUTCDate(day);
			prevMonth.setUTCDate(day - (prevMonth.getUTCDay() - this.o.weekStart + 7)%7);
			var nextMonth = new Date(prevMonth);
			nextMonth.setUTCDate(nextMonth.getUTCDate() + 42);
			nextMonth = nextMonth.valueOf();
			var html = [];
			var clsName;
			while (prevMonth.valueOf() < nextMonth){
				if (prevMonth.getUTCDay() === this.o.weekStart){
					html.push('<tr>');
					if (this.o.calendarWeeks){
						// ISO 8601: First week contains first thursday.
						// ISO also states week starts on Monday, but we can be more abstract here.
						var
							// Start of current week: based on weekstart/current date
							ws = new Date(+prevMonth + (this.o.weekStart - prevMonth.getUTCDay() - 7) % 7 * 864e5),
							// Thursday of this week
							th = new Date(Number(ws) + (7 + 4 - ws.getUTCDay()) % 7 * 864e5),
							// First Thursday of year, year from thursday
							yth = new Date(Number(yth = UTCDate(th.getUTCFullYear(), 0, 1)) + (7 + 4 - yth.getUTCDay())%7*864e5),
							// Calendar week: ms between thursdays, div ms per day, div 7 days
							calWeek =  (th - yth) / 864e5 / 7 + 1;
						html.push('<td class="cw">'+ calWeek +'</td>');

					}
				}
				clsName = this.getClassNames(prevMonth);
				clsName.push('day');

				if (this.o.beforeShowDay !== $.noop){
					var before = this.o.beforeShowDay(this._utc_to_local(prevMonth));
					if (before === undefined)
						before = {};
					else if (typeof(before) === 'boolean')
						before = {enabled: before};
					else if (typeof(before) === 'string')
						before = {classes: before};
					if (before.enabled === false)
						clsName.push('disabled');
					if (before.classes)
						clsName = clsName.concat(before.classes.split(/\s+/));
					if (before.tooltip)
						tooltip = before.tooltip;
				}

				clsName = $.unique(clsName);
				html.push('<td class="'+clsName.join(' ')+'"' + (tooltip ? ' title="'+tooltip+'"' : '') + '>'+prevMonth.getUTCDate() + '</td>');
				tooltip = null;
				if (prevMonth.getUTCDay() === this.o.weekEnd){
					html.push('</tr>');
				}
				prevMonth.setUTCDate(prevMonth.getUTCDate()+1);
			}
			this.picker.find('.datepicker-days tbody').empty().append(html.join(''));

			var months = this.picker.find('.datepicker-months')
						.find('th:eq(1)')
							.text(year)
							.end()
						.find('span').removeClass('active');

			$.each(this.dates, function(i, d){
				if (d.getUTCFullYear() === year)
					months.eq(d.getUTCMonth()).addClass('active');
			});

			if (year < startYear || year > endYear){
				months.addClass('disabled');
			}
			if (year === startYear){
				months.slice(0, startMonth).addClass('disabled');
			}
			if (year === endYear){
				months.slice(endMonth+1).addClass('disabled');
			}

			html = '';
			year = parseInt(year/10, 10) * 10;
			var yearCont = this.picker.find('.datepicker-years')
								.find('th:eq(1)')
									.text(year + '-' + (year + 9))
									.end()
								.find('td');
			year -= 1;
			var years = $.map(this.dates, function(d){
					return d.getUTCFullYear();
				}),
				classes;
			for (var i = -1; i < 11; i++){
				classes = ['year'];
				if (i === -1)
					classes.push('old');
				else if (i === 10)
					classes.push('new');
				if ($.inArray(year, years) !== -1)
					classes.push('active');
				if (year < startYear || year > endYear)
					classes.push('disabled');
				html += '<span class="' + classes.join(' ') + '">'+year+'</span>';
				year += 1;
			}
			yearCont.html(html);
		},

		updateNavArrows: function(){
			if (!this._allow_update)
				return;

			var d = new Date(this.viewDate),
				year = d.getUTCFullYear(),
				month = d.getUTCMonth();
			switch (this.viewMode){
				case 0:
					if (this.o.startDate !== -Infinity && year <= this.o.startDate.getUTCFullYear() && month <= this.o.startDate.getUTCMonth()){
						this.picker.find('.prev').css({visibility: 'hidden'});
					}
					else {
						this.picker.find('.prev').css({visibility: 'visible'});
					}
					if (this.o.endDate !== Infinity && year >= this.o.endDate.getUTCFullYear() && month >= this.o.endDate.getUTCMonth()){
						this.picker.find('.next').css({visibility: 'hidden'});
					}
					else {
						this.picker.find('.next').css({visibility: 'visible'});
					}
					break;
				case 1:
				case 2:
					if (this.o.startDate !== -Infinity && year <= this.o.startDate.getUTCFullYear()){
						this.picker.find('.prev').css({visibility: 'hidden'});
					}
					else {
						this.picker.find('.prev').css({visibility: 'visible'});
					}
					if (this.o.endDate !== Infinity && year >= this.o.endDate.getUTCFullYear()){
						this.picker.find('.next').css({visibility: 'hidden'});
					}
					else {
						this.picker.find('.next').css({visibility: 'visible'});
					}
					break;
			}
		},

		click: function(e){
			e.preventDefault();
			var target = $(e.target).closest('span, td, th'),
				year, month, day;
			if (target.length === 1){
				switch (target[0].nodeName.toLowerCase()){
					case 'th':
						switch (target[0].className){
							case 'datepicker-switch':
								this.showMode(1);
								break;
							case 'prev':
							case 'next':
								var dir = DPGlobal.modes[this.viewMode].navStep * (target[0].className === 'prev' ? -1 : 1);
								switch (this.viewMode){
									case 0:
										this.viewDate = this.moveMonth(this.viewDate, dir);
										this._trigger('changeMonth', this.viewDate);
										break;
									case 1:
									case 2:
										this.viewDate = this.moveYear(this.viewDate, dir);
										if (this.viewMode === 1)
											this._trigger('changeYear', this.viewDate);
										break;
								}
								this.fill();
								break;
							case 'today':
								var date = new Date();
								date = UTCDate(date.getFullYear(), date.getMonth(), date.getDate(), 0, 0, 0);

								this.showMode(-2);
								var which = this.o.todayBtn === 'linked' ? null : 'view';
								this._setDate(date, which);
								break;
							case 'clear':
								var element;
								if (this.isInput)
									element = this.element;
								else if (this.component)
									element = this.element.find('input');
								if (element)
									element.val("").change();
								this.update();
								this._trigger('changeDate');
								if (this.o.autoclose)
									this.hide();
								break;
						}
						break;
					case 'span':
						if (!target.is('.disabled')){
							this.viewDate.setUTCDate(1);
							if (target.is('.month')){
								day = 1;
								month = target.parent().find('span').index(target);
								year = this.viewDate.getUTCFullYear();
								this.viewDate.setUTCMonth(month);
								this._trigger('changeMonth', this.viewDate);
								if (this.o.minViewMode === 1){
									this._setDate(UTCDate(year, month, day));
								}
							}
							else {
								day = 1;
								month = 0;
								year = parseInt(target.text(), 10)||0;
								this.viewDate.setUTCFullYear(year);
								this._trigger('changeYear', this.viewDate);
								if (this.o.minViewMode === 2){
									this._setDate(UTCDate(year, month, day));
								}
							}
							this.showMode(-1);
							this.fill();
						}
						break;
					case 'td':
						if (target.is('.day') && !target.is('.disabled')){
							day = parseInt(target.text(), 10)||1;
							year = this.viewDate.getUTCFullYear();
							month = this.viewDate.getUTCMonth();
							if (target.is('.old')){
								if (month === 0){
									month = 11;
									year -= 1;
								}
								else {
									month -= 1;
								}
							}
							else if (target.is('.new')){
								if (month === 11){
									month = 0;
									year += 1;
								}
								else {
									month += 1;
								}
							}
							this._setDate(UTCDate(year, month, day));
						}
						break;
				}
			}
			if (this.picker.is(':visible') && this._focused_from){
				$(this._focused_from).focus();
			}
			delete this._focused_from;
		},

		_toggle_multidate: function(date){
			var ix = this.dates.contains(date);
			if (!date){
				this.dates.clear();
			}
			if (this.o.multidate === 1 && ix === 0){
                // single datepicker, don't remove selected date
            }
			else if (ix !== -1){
				this.dates.remove(ix);
			}
			else {
				this.dates.push(date);
			}
			if (typeof this.o.multidate === 'number')
				while (this.dates.length > this.o.multidate)
					this.dates.remove(0);
		},

		_setDate: function(date, which){
			if (!which || which === 'date')
				this._toggle_multidate(date && new Date(date));
			if (!which || which  === 'view')
				this.viewDate = date && new Date(date);

			this.fill();
			this.setValue();
			this._trigger('changeDate');
			var element;
			if (this.isInput){
				element = this.element;
			}
			else if (this.component){
				element = this.element.find('input');
			}
			if (element){
				element.change();
			}
			if (this.o.autoclose && (!which || which === 'date')){
				this.hide();
			}
		},

		moveMonth: function(date, dir){
			if (!date)
				return undefined;
			if (!dir)
				return date;
			var new_date = new Date(date.valueOf()),
				day = new_date.getUTCDate(),
				month = new_date.getUTCMonth(),
				mag = Math.abs(dir),
				new_month, test;
			dir = dir > 0 ? 1 : -1;
			if (mag === 1){
				test = dir === -1
					// If going back one month, make sure month is not current month
					// (eg, Mar 31 -> Feb 31 == Feb 28, not Mar 02)
					? function(){
						return new_date.getUTCMonth() === month;
					}
					// If going forward one month, make sure month is as expected
					// (eg, Jan 31 -> Feb 31 == Feb 28, not Mar 02)
					: function(){
						return new_date.getUTCMonth() !== new_month;
					};
				new_month = month + dir;
				new_date.setUTCMonth(new_month);
				// Dec -> Jan (12) or Jan -> Dec (-1) -- limit expected date to 0-11
				if (new_month < 0 || new_month > 11)
					new_month = (new_month + 12) % 12;
			}
			else {
				// For magnitudes >1, move one month at a time...
				for (var i=0; i < mag; i++)
					// ...which might decrease the day (eg, Jan 31 to Feb 28, etc)...
					new_date = this.moveMonth(new_date, dir);
				// ...then reset the day, keeping it in the new month
				new_month = new_date.getUTCMonth();
				new_date.setUTCDate(day);
				test = function(){
					return new_month !== new_date.getUTCMonth();
				};
			}
			// Common date-resetting loop -- if date is beyond end of month, make it
			// end of month
			while (test()){
				new_date.setUTCDate(--day);
				new_date.setUTCMonth(new_month);
			}
			return new_date;
		},

		moveYear: function(date, dir){
			return this.moveMonth(date, dir*12);
		},

		dateWithinRange: function(date){
			return date >= this.o.startDate && date <= this.o.endDate;
		},

		keydown: function(e){
			if (this.picker.is(':not(:visible)')){
				if (e.keyCode === 27) // allow escape to hide and re-show picker
					this.show();
				return;
			}
			var dateChanged = false,
				dir, newDate, newViewDate,
				focusDate = this.focusDate || this.viewDate;
			switch (e.keyCode){
				case 27: // escape
					if (this.focusDate){
						this.focusDate = null;
						this.viewDate = this.dates.get(-1) || this.viewDate;
						this.fill();
					}
					else
						this.hide();
					e.preventDefault();
					break;
				case 37: // left
				case 39: // right
					if (!this.o.keyboardNavigation)
						break;
					dir = e.keyCode === 37 ? -1 : 1;
					if (e.ctrlKey){
						newDate = this.moveYear(this.dates.get(-1) || UTCToday(), dir);
						newViewDate = this.moveYear(focusDate, dir);
						this._trigger('changeYear', this.viewDate);
					}
					else if (e.shiftKey){
						newDate = this.moveMonth(this.dates.get(-1) || UTCToday(), dir);
						newViewDate = this.moveMonth(focusDate, dir);
						this._trigger('changeMonth', this.viewDate);
					}
					else {
						newDate = new Date(this.dates.get(-1) || UTCToday());
						newDate.setUTCDate(newDate.getUTCDate() + dir);
						newViewDate = new Date(focusDate);
						newViewDate.setUTCDate(focusDate.getUTCDate() + dir);
					}
					if (this.dateWithinRange(newDate)){
						this.focusDate = this.viewDate = newViewDate;
						this.setValue();
						this.fill();
						e.preventDefault();
					}
					break;
				case 38: // up
				case 40: // down
					if (!this.o.keyboardNavigation)
						break;
					dir = e.keyCode === 38 ? -1 : 1;
					if (e.ctrlKey){
						newDate = this.moveYear(this.dates.get(-1) || UTCToday(), dir);
						newViewDate = this.moveYear(focusDate, dir);
						this._trigger('changeYear', this.viewDate);
					}
					else if (e.shiftKey){
						newDate = this.moveMonth(this.dates.get(-1) || UTCToday(), dir);
						newViewDate = this.moveMonth(focusDate, dir);
						this._trigger('changeMonth', this.viewDate);
					}
					else {
						newDate = new Date(this.dates.get(-1) || UTCToday());
						newDate.setUTCDate(newDate.getUTCDate() + dir * 7);
						newViewDate = new Date(focusDate);
						newViewDate.setUTCDate(focusDate.getUTCDate() + dir * 7);
					}
					if (this.dateWithinRange(newDate)){
						this.focusDate = this.viewDate = newViewDate;
						this.setValue();
						this.fill();
						e.preventDefault();
					}
					break;
				case 32: // spacebar
					// Spacebar is used in manually typing dates in some formats.
					// As such, its behavior should not be hijacked.
					break;
				case 13: // enter
					focusDate = this.focusDate || this.dates.get(-1) || this.viewDate;
					if (this.o.keyboardNavigation) {
						this._toggle_multidate(focusDate);
						dateChanged = true;
					}
					this.focusDate = null;
					this.viewDate = this.dates.get(-1) || this.viewDate;
					this.setValue();
					this.fill();
					if (this.picker.is(':visible')){
						e.preventDefault();
						if (this.o.autoclose)
							this.hide();
					}
					break;
				case 9: // tab
					this.focusDate = null;
					this.viewDate = this.dates.get(-1) || this.viewDate;
					this.fill();
					this.hide();
					break;
			}
			if (dateChanged){
				if (this.dates.length)
					this._trigger('changeDate');
				else
					this._trigger('clearDate');
				var element;
				if (this.isInput){
					element = this.element;
				}
				else if (this.component){
					element = this.element.find('input');
				}
				if (element){
					element.change();
				}
			}
		},

		showMode: function(dir){
			if (dir){
				this.viewMode = Math.max(this.o.minViewMode, Math.min(2, this.viewMode + dir));
			}
			this.picker
				.find('>div')
				.hide()
				.filter('.datepicker-'+DPGlobal.modes[this.viewMode].clsName)
					.css('display', 'block');
			this.updateNavArrows();
		}
	};

	var DateRangePicker = function(element, options){
		this.element = $(element);
		this.inputs = $.map(options.inputs, function(i){
			return i.jquery ? i[0] : i;
		});
		delete options.inputs;

		$(this.inputs)
			.datepicker(options)
			.bind('changeDate', $.proxy(this.dateUpdated, this));

		this.pickers = $.map(this.inputs, function(i){
			return $(i).data('datepicker');
		});
		this.updateDates();
	};
	DateRangePicker.prototype = {
		updateDates: function(){
			this.dates = $.map(this.pickers, function(i){
				return i.getUTCDate();
			});
			this.updateRanges();
		},
		updateRanges: function(){
			var range = $.map(this.dates, function(d){
				return d.valueOf();
			});
			$.each(this.pickers, function(i, p){
				p.setRange(range);
			});
		},
		dateUpdated: function(e){
			// `this.updating` is a workaround for preventing infinite recursion
			// between `changeDate` triggering and `setUTCDate` calling.  Until
			// there is a better mechanism.
			if (this.updating)
				return;
			this.updating = true;

			var dp = $(e.target).data('datepicker'),
				new_date = dp.getUTCDate(),
				i = $.inArray(e.target, this.inputs),
				l = this.inputs.length;
			if (i === -1)
				return;

			$.each(this.pickers, function(i, p){
				if (!p.getUTCDate())
					p.setUTCDate(new_date);
			});

			if (new_date < this.dates[i]){
				// Date being moved earlier/left
				while (i >= 0 && new_date < this.dates[i]){
					this.pickers[i--].setUTCDate(new_date);
				}
			}
			else if (new_date > this.dates[i]){
				// Date being moved later/right
				while (i < l && new_date > this.dates[i]){
					this.pickers[i++].setUTCDate(new_date);
				}
			}
			this.updateDates();

			delete this.updating;
		},
		remove: function(){
			$.map(this.pickers, function(p){ p.remove(); });
			delete this.element.data().datepicker;
		}
	};

	function opts_from_el(el, prefix){
		// Derive options from element data-attrs
		var data = $(el).data(),
			out = {}, inkey,
			replace = new RegExp('^' + prefix.toLowerCase() + '([A-Z])');
		prefix = new RegExp('^' + prefix.toLowerCase());
		function re_lower(_,a){
			return a.toLowerCase();
		}
		for (var key in data)
			if (prefix.test(key)){
				inkey = key.replace(replace, re_lower);
				out[inkey] = data[key];
			}
		return out;
	}

	function opts_from_locale(lang){
		// Derive options from locale plugins
		var out = {};
		// Check if "de-DE" style date is available, if not language should
		// fallback to 2 letter code eg "de"
		if (!dates[lang]){
			lang = lang.split('-')[0];
			if (!dates[lang])
				return;
		}
		var d = dates[lang];
		$.each(locale_opts, function(i,k){
			if (k in d)
				out[k] = d[k];
		});
		return out;
	}

	var old = $.fn.datepicker;
	$.fn.datepicker = function(option){
		var args = Array.apply(null, arguments);
		args.shift();
		var internal_return;
		this.each(function(){
			var $this = $(this),
				data = $this.data('datepicker'),
				options = typeof option === 'object' && option;
			if (!data){
				var elopts = opts_from_el(this, 'date'),
					// Preliminary otions
					xopts = $.extend({}, defaults, elopts, options),
					locopts = opts_from_locale(xopts.language),
					// Options priority: js args, data-attrs, locales, defaults
					opts = $.extend({}, defaults, locopts, elopts, options);
				if ($this.is('.input-daterange') || opts.inputs){
					var ropts = {
						inputs: opts.inputs || $this.find('input').toArray()
					};
					$this.data('datepicker', (data = new DateRangePicker(this, $.extend(opts, ropts))));
				}
				else {
					$this.data('datepicker', (data = new Datepicker(this, opts)));
				}
			}
			if (typeof option === 'string' && typeof data[option] === 'function'){
				internal_return = data[option].apply(data, args);
				if (internal_return !== undefined)
					return false;
			}
		});
		if (internal_return !== undefined)
			return internal_return;
		else
			return this;
	};

	var defaults = $.fn.datepicker.defaults = {
		autoclose: false,
		beforeShowDay: $.noop,
		calendarWeeks: false,
		clearBtn: false,
		daysOfWeekDisabled: [],
		endDate: Infinity,
		forceParse: true,
		format: 'mm/dd/yyyy',
		keyboardNavigation: true,
		language: 'en',
		minViewMode: 0,
		multidate: false,
		multidateSeparator: ',',
		orientation: "auto",
		rtl: false,
		startDate: -Infinity,
		startView: 0,
		todayBtn: false,
		todayHighlight: false,
		weekStart: 0
	};
	var locale_opts = $.fn.datepicker.locale_opts = [
		'format',
		'rtl',
		'weekStart'
	];
	$.fn.datepicker.Constructor = Datepicker;
	var dates = $.fn.datepicker.dates = {
		en: {
			days: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"],
			daysShort: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"],
			daysMin: ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa", "Su"],
			months: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
			monthsShort: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
			today: "Today",
			clear: "Clear"
		}
	};

	var DPGlobal = {
		modes: [
			{
				clsName: 'days',
				navFnc: 'Month',
				navStep: 1
			},
			{
				clsName: 'months',
				navFnc: 'FullYear',
				navStep: 1
			},
			{
				clsName: 'years',
				navFnc: 'FullYear',
				navStep: 10
		}],
		isLeapYear: function(year){
			return (((year % 4 === 0) && (year % 100 !== 0)) || (year % 400 === 0));
		},
		getDaysInMonth: function(year, month){
			return [31, (DPGlobal.isLeapYear(year) ? 29 : 28), 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][month];
		},
		validParts: /dd?|DD?|mm?|MM?|yy(?:yy)?/g,
		nonpunctuation: /[^ -\/:-@\[\u3400-\u9fff-`{-~\t\n\r]+/g,
		parseFormat: function(format){
			// IE treats \0 as a string end in inputs (truncating the value),
			// so it's a bad format delimiter, anyway
			var separators = format.replace(this.validParts, '\0').split('\0'),
				parts = format.match(this.validParts);
			if (!separators || !separators.length || !parts || parts.length === 0){
				throw new Error("Invalid date format.");
			}
			return {separators: separators, parts: parts};
		},
		parseDate: function(date, format, language){
			if (!date)
				return undefined;
			if (date instanceof Date)
				return date;
			if (typeof format === 'string')
				format = DPGlobal.parseFormat(format);
			var part_re = /([\-+]\d+)([dmwy])/,
				parts = date.match(/([\-+]\d+)([dmwy])/g),
				part, dir, i;
			if (/^[\-+]\d+[dmwy]([\s,]+[\-+]\d+[dmwy])*$/.test(date)){
				date = new Date();
				for (i=0; i < parts.length; i++){
					part = part_re.exec(parts[i]);
					dir = parseInt(part[1]);
					switch (part[2]){
						case 'd':
							date.setUTCDate(date.getUTCDate() + dir);
							break;
						case 'm':
							date = Datepicker.prototype.moveMonth.call(Datepicker.prototype, date, dir);
							break;
						case 'w':
							date.setUTCDate(date.getUTCDate() + dir * 7);
							break;
						case 'y':
							date = Datepicker.prototype.moveYear.call(Datepicker.prototype, date, dir);
							break;
					}
				}
				return UTCDate(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(), 0, 0, 0);
			}
			parts = date && date.match(this.nonpunctuation) || [];
			date = new Date();
			var parsed = {},
				setters_order = ['yyyy', 'yy', 'M', 'MM', 'm', 'mm', 'd', 'dd'],
				setters_map = {
					yyyy: function(d,v){
						return d.setUTCFullYear(v);
					},
					yy: function(d,v){
						return d.setUTCFullYear(2000+v);
					},
					m: function(d,v){
						if (isNaN(d))
							return d;
						v -= 1;
						while (v < 0) v += 12;
						v %= 12;
						d.setUTCMonth(v);
						while (d.getUTCMonth() !== v)
							d.setUTCDate(d.getUTCDate()-1);
						return d;
					},
					d: function(d,v){
						return d.setUTCDate(v);
					}
				},
				val, filtered;
			setters_map['M'] = setters_map['MM'] = setters_map['mm'] = setters_map['m'];
			setters_map['dd'] = setters_map['d'];
			date = UTCDate(date.getFullYear(), date.getMonth(), date.getDate(), 0, 0, 0);
			var fparts = format.parts.slice();
			// Remove noop parts
			if (parts.length !== fparts.length){
				fparts = $(fparts).filter(function(i,p){
					return $.inArray(p, setters_order) !== -1;
				}).toArray();
			}
			// Process remainder
			function match_part(){
				var m = this.slice(0, parts[i].length),
					p = parts[i].slice(0, m.length);
				return m === p;
			}
			if (parts.length === fparts.length){
				var cnt;
				for (i=0, cnt = fparts.length; i < cnt; i++){
					val = parseInt(parts[i], 10);
					part = fparts[i];
					if (isNaN(val)){
						switch (part){
							case 'MM':
								filtered = $(dates[language].months).filter(match_part);
								val = $.inArray(filtered[0], dates[language].months) + 1;
								break;
							case 'M':
								filtered = $(dates[language].monthsShort).filter(match_part);
								val = $.inArray(filtered[0], dates[language].monthsShort) + 1;
								break;
						}
					}
					parsed[part] = val;
				}
				var _date, s;
				for (i=0; i < setters_order.length; i++){
					s = setters_order[i];
					if (s in parsed && !isNaN(parsed[s])){
						_date = new Date(date);
						setters_map[s](_date, parsed[s]);
						if (!isNaN(_date))
							date = _date;
					}
				}
			}
			return date;
		},
		formatDate: function(date, format, language){
			if (!date)
				return '';
			if (typeof format === 'string')
				format = DPGlobal.parseFormat(format);
			var val = {
				d: date.getUTCDate(),
				D: dates[language].daysShort[date.getUTCDay()],
				DD: dates[language].days[date.getUTCDay()],
				m: date.getUTCMonth() + 1,
				M: dates[language].monthsShort[date.getUTCMonth()],
				MM: dates[language].months[date.getUTCMonth()],
				yy: date.getUTCFullYear().toString().substring(2),
				yyyy: date.getUTCFullYear()
			};
			val.dd = (val.d < 10 ? '0' : '') + val.d;
			val.mm = (val.m < 10 ? '0' : '') + val.m;
			date = [];
			var seps = $.extend([], format.separators);
			for (var i=0, cnt = format.parts.length; i <= cnt; i++){
				if (seps.length)
					date.push(seps.shift());
				date.push(val[format.parts[i]]);
			}
			return date.join('');
		},
		headTemplate: '<thead>'+
							'<tr>'+
								'<th class="prev">&laquo;</th>'+
								'<th colspan="5" class="datepicker-switch"></th>'+
								'<th class="next">&raquo;</th>'+
							'</tr>'+
						'</thead>',
		contTemplate: '<tbody><tr><td colspan="7"></td></tr></tbody>',
		footTemplate: '<tfoot>'+
							'<tr>'+
								'<th colspan="7" class="today"></th>'+
							'</tr>'+
							'<tr>'+
								'<th colspan="7" class="clear"></th>'+
							'</tr>'+
						'</tfoot>'
	};
	DPGlobal.template = '<div class="datepicker">'+
							'<div class="datepicker-days">'+
								'<table class=" table-condensed">'+
									DPGlobal.headTemplate+
									'<tbody></tbody>'+
									DPGlobal.footTemplate+
								'</table>'+
							'</div>'+
							'<div class="datepicker-months">'+
								'<table class="table-condensed">'+
									DPGlobal.headTemplate+
									DPGlobal.contTemplate+
									DPGlobal.footTemplate+
								'</table>'+
							'</div>'+
							'<div class="datepicker-years">'+
								'<table class="table-condensed">'+
									DPGlobal.headTemplate+
									DPGlobal.contTemplate+
									DPGlobal.footTemplate+
								'</table>'+
							'</div>'+
						'</div>';

	$.fn.datepicker.DPGlobal = DPGlobal;


	/* DATEPICKER NO CONFLICT
	* =================== */

	$.fn.datepicker.noConflict = function(){
		$.fn.datepicker = old;
		return this;
	};


	/* DATEPICKER DATA-API
	* ================== */

	$(document).on(
		'focus.datepicker.data-api click.datepicker.data-api',
		'[data-provide="datepicker"]',
		function(e){
			var $this = $(this);
			if ($this.data('datepicker'))
				return;
			e.preventDefault();
			// component click requires us to explicitly show it
			$this.datepicker('show');
		}
	);
	$(function(){
		$('[data-provide="datepicker-inline"]').datepicker();
	});

}(window.jQuery));

define("bootstrap-datepicker", function(){});

// Azerbaijani
;(function($){
    $.fn.datepicker.dates['az'] = {
        days: ["Bazar", "Bazar ertəsi", "Çərşənbə axşamı", "Çərşənbə", "Cümə axşamı", "Cümə", "Şənbə", "Bazar"],
        daysShort: ["B.", "B.e", "Ç.a", "Ç.", "C.a", "C.", "Ş.", "B."],
        daysMin: ["B.", "B.e", "Ç.a", "Ç.", "C.a", "C.", "Ş.", "B."],
        months: ["Yanvar", "Fevral", "Mart", "Aprel", "May", "İyun", "İyul", "Avqust", "Sentyabr", "Oktyabr", "Noyabr", "Dekabr"],
        monthsShort: ["Yan", "Fev", "Mar", "Apr", "May", "İyun", "İyul", "Avq", "Sen", "Okt", "Noy", "Dek"],
        today: "Bu gün",
        weekStart: 1
    };
}(jQuery));

define("datepicker-ar", function(){});

/**
 * Bulgarian translation for bootstrap-datepicker
 * Apostol Apostolov <apostol.s.apostolov@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['bg'] = {
		days: ["Неделя", "Понеделник", "Вторник", "Сряда", "Четвъртък", "Петък", "Събота", "Неделя"],
		daysShort: ["Нед", "Пон", "Вто", "Сря", "Чет", "Пет", "Съб", "Нед"],
		daysMin: ["Н", "П", "В", "С", "Ч", "П", "С", "Н"],
		months: ["Януари", "Февруари", "Март", "Април", "Май", "Юни", "Юли", "Август", "Септември", "Октомври", "Ноември", "Декември"],
		monthsShort: ["Ян", "Фев", "Мар", "Апр", "Май", "Юни", "Юли", "Авг", "Сеп", "Окт", "Ное", "Дек"],
		today: "днес"
	};
}(jQuery));

define("datepicker-az", function(){});

/**
 * Arabic translation for bootstrap-datepicker
 * Mohammed Alshehri <alshehri866@gmail.com>
 */
;(function($){
    $.fn.datepicker.dates['ar'] = {
        days: ["الأحد", "الاثنين", "الثلاثاء", "الأربعاء", "الخميس", "الجمعة", "السبت", "الأحد"],
        daysShort: ["أحد", "اثنين", "ثلاثاء", "أربعاء", "خميس", "جمعة", "سبت", "أحد"],
        daysMin: ["ح", "ن", "ث", "ع", "خ", "ج", "س", "ح"],
        months: ["يناير", "فبراير", "مارس", "أبريل", "مايو", "يونيو", "يوليو", "أغسطس", "سبتمبر", "أكتوبر", "نوفمبر", "ديسمبر"],
        monthsShort: ["يناير", "فبراير", "مارس", "أبريل", "مايو", "يونيو", "يوليو", "أغسطس", "سبتمبر", "أكتوبر", "نوفمبر", "ديسمبر"],
        today: "هذا اليوم",
        rtl: true
    };
}(jQuery));

define("datepicker-bg", function(){});

/**
 * Catalan translation for bootstrap-datepicker
 * J. Garcia <jogaco.en@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['ca'] = {
		days: ["Diumenge", "Dilluns", "Dimarts", "Dimecres", "Dijous", "Divendres", "Dissabte", "Diumenge"],
		daysShort: ["Diu",  "Dil", "Dmt", "Dmc", "Dij", "Div", "Dis", "Diu"],
		daysMin: ["dg", "dl", "dt", "dc", "dj", "dv", "ds", "dg"],
		months: ["Gener", "Febrer", "Març", "Abril", "Maig", "Juny", "Juliol", "Agost", "Setembre", "Octubre", "Novembre", "Desembre"],
		monthsShort: ["Gen", "Feb", "Mar", "Abr", "Mai", "Jun", "Jul", "Ago", "Set", "Oct", "Nov", "Des"],
		today: "Avui",
		clear: "Esborrar",
		weekStart: 1,
		format: "dd/mm/yyyy"
	};
}(jQuery));

define("datepicker-ca", function(){});

/**
 * Czech translation for bootstrap-datepicker
 * Matěj Koubík <matej@koubik.name>
 * Fixes by Michal Remiš <michal.remis@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['cs'] = {
		days: ["Neděle", "Pondělí", "Úterý", "Středa", "Čtvrtek", "Pátek", "Sobota", "Neděle"],
		daysShort: ["Ned", "Pon", "Úte", "Stř", "Čtv", "Pát", "Sob", "Ned"],
		daysMin: ["Ne", "Po", "Út", "St", "Čt", "Pá", "So", "Ne"],
		months: ["Leden", "Únor", "Březen", "Duben", "Květen", "Červen", "Červenec", "Srpen", "Září", "Říjen", "Listopad", "Prosinec"],
		monthsShort: ["Led", "Úno", "Bře", "Dub", "Kvě", "Čer", "Čnc", "Srp", "Zář", "Říj", "Lis", "Pro"],
		today: "Dnes",
		clear: "Vymazat",
		weekStart: 1,
		format: "d.m.yyyy"
	};
}(jQuery));

define("datepicker-cs", function(){});

/**
 * Welsh translation for bootstrap-datepicker
 * S. Morris <s.morris@bangor.ac.uk>
 */
;(function($){
	$.fn.datepicker.dates['cy'] = {
		days: ["Sul", "Llun", "Mawrth", "Mercher", "Iau", "Gwener", "Sadwrn", "Sul"],
		daysShort: ["Sul", "Llu", "Maw", "Mer", "Iau", "Gwe", "Sad", "Sul"],
		daysMin: ["Su", "Ll", "Ma", "Me", "Ia", "Gwe", "Sa", "Su"],
		months: ["Ionawr", "Chewfror", "Mawrth", "Ebrill", "Mai", "Mehefin", "Gorfennaf", "Awst", "Medi", "Hydref", "Tachwedd", "Rhagfyr"],
		monthsShort: ["Ion", "Chw", "Maw", "Ebr", "Mai", "Meh", "Gor", "Aws", "Med", "Hyd", "Tach", "Rha"],
		today: "Heddiw"
	};
}(jQuery));

define("datepicker-cy", function(){});

/**
 * Danish translation for bootstrap-datepicker
 * Christian Pedersen <http://github.com/chripede>
 */
;(function($){
	$.fn.datepicker.dates['da'] = {
		days: ["Søndag", "Mandag", "Tirsdag", "Onsdag", "Torsdag", "Fredag", "Lørdag", "Søndag"],
		daysShort: ["Søn", "Man", "Tir", "Ons", "Tor", "Fre", "Lør", "Søn"],
		daysMin: ["Sø", "Ma", "Ti", "On", "To", "Fr", "Lø", "Sø"],
		months: ["Januar", "Februar", "Marts", "April", "Maj", "Juni", "Juli", "August", "September", "Oktober", "November", "December"],
		monthsShort: ["Jan", "Feb", "Mar", "Apr", "Maj", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dec"],
		today: "I Dag",
		clear: "Nulstil"
	};
}(jQuery));

define("datepicker-da", function(){});

/**
 * German translation for bootstrap-datepicker
 * Sam Zurcher <sam@orelias.ch>
 */
;(function($){
	$.fn.datepicker.dates['de'] = {
		days: ["Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag", "Sonntag"],
		daysShort: ["Son", "Mon", "Die", "Mit", "Don", "Fre", "Sam", "Son"],
		daysMin: ["So", "Mo", "Di", "Mi", "Do", "Fr", "Sa", "So"],
		months: ["Januar", "Februar", "März", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember"],
		monthsShort: ["Jan", "Feb", "Mär", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dez"],
		today: "Heute",
		clear: "Löschen",
		weekStart: 1,
		format: "dd.mm.yyyy"
	};
}(jQuery));

define("datepicker-de", function(){});

/**
 * Greek translation for bootstrap-datepicker
 */
;(function($){
  $.fn.datepicker.dates['el'] = {
    days: ["Κυριακή", "Δευτέρα", "Τρίτη", "Τετάρτη", "Πέμπτη", "Παρασκευή", "Σάββατο", "Κυριακή"],
    daysShort: ["Κυρ", "Δευ", "Τρι", "Τετ", "Πεμ", "Παρ", "Σαβ", "Κυρ"],
    daysMin: ["Κυ", "Δε", "Τρ", "Τε", "Πε", "Πα", "Σα", "Κυ"],
    months: ["Ιανουάριος", "Φεβρουάριος", "Μάρτιος", "Απρίλιος", "Μάιος", "Ιούνιος", "Ιούλιος", "Αύγουστος", "Σεπτέμβριος", "Οκτώβριος", "Νοέμβριος", "Δεκέμβριος"],
    monthsShort: ["Ιαν", "Φεβ", "Μαρ", "Απρ", "Μάι", "Ιουν", "Ιουλ", "Αυγ", "Σεπ", "Οκτ", "Νοε", "Δεκ"],
    today: "Σήμερα",
    clear: "Καθαρισμός",
    weekStart: 1,
    format: "d/m/yyyy"
  };
}(jQuery));

define("datepicker-el", function(){});

/**
 * Spanish translation for bootstrap-datepicker
 * Bruno Bonamin <bruno.bonamin@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['es'] = {
		days: ["Domingo", "Lunes", "Martes", "Miércoles", "Jueves", "Viernes", "Sábado", "Domingo"],
		daysShort: ["Dom", "Lun", "Mar", "Mié", "Jue", "Vie", "Sáb", "Dom"],
		daysMin: ["Do", "Lu", "Ma", "Mi", "Ju", "Vi", "Sa", "Do"],
		months: ["Enero", "Febrero", "Marzo", "Abril", "Mayo", "Junio", "Julio", "Agosto", "Septiembre", "Octubre", "Noviembre", "Diciembre"],
		monthsShort: ["Ene", "Feb", "Mar", "Abr", "May", "Jun", "Jul", "Ago", "Sep", "Oct", "Nov", "Dic"],
		today: "Hoy",
		clear: "Borrar",
		weekStart: 1,
		format: "dd/mm/yyyy"
	};
}(jQuery));

define("datepicker-es", function(){});

/**
 * Estonian translation for bootstrap-datepicker
 * Ando Roots <https://github.com/anroots>
 * Fixes by Illimar Tambek <<https://github.com/ragulka>
 */
;(function($){
	$.fn.datepicker.dates['et'] = {
		days: ["Pühapäev", "Esmaspäev", "Teisipäev", "Kolmapäev", "Neljapäev", "Reede", "Laupäev", "Pühapäev"],
		daysShort: ["Pühap", "Esmasp", "Teisip", "Kolmap", "Neljap", "Reede", "Laup", "Pühap"],
		daysMin: ["P", "E", "T", "K", "N", "R", "L", "P"],
		months: ["Jaanuar", "Veebruar", "Märts", "Aprill", "Mai", "Juuni", "Juuli", "August", "September", "Oktoober", "November", "Detsember"],
		monthsShort: ["Jaan", "Veebr", "Märts", "Apr", "Mai", "Juuni", "Juuli", "Aug", "Sept", "Okt", "Nov", "Dets"],
		today: "Täna",
		clear: "Tühjenda",
		weekStart: 1,
		format: "dd.mm.yyyy"
	};
}(jQuery));

define("datepicker-et", function(){});

/**
 * Persian translation for bootstrap-datepicker
 * Mostafa Rokooie <mostafa.rokooie@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['fa'] = {
		days: ["یک‌شنبه", "دوشنبه", "سه‌شنبه", "چهارشنبه", "پنج‌شنبه", "جمعه", "شنبه", "یک‌شنبه"],
		daysShort: ["یک", "دو", "سه", "چهار", "پنج", "جمعه", "شنبه", "یک"],
		daysMin: ["ی", "د", "س", "چ", "پ", "ج", "ش", "ی"],
		months: ["ژانویه", "فوریه", "مارس", "آوریل", "مه", "ژوئن", "ژوئیه", "اوت", "سپتامبر", "اکتبر", "نوامبر", "دسامبر"],
		monthsShort: ["ژان", "فور", "مار", "آور", "مه", "ژون", "ژوی", "اوت", "سپت", "اکت", "نوا", "دسا"],
		today: "امروز",
		clear: "پاک کن",
		weekStart: 1,
		format: "yyyy/mm/dd"
	};
}(jQuery));

define("datepicker-fa", function(){});

/**
 * Finnish translation for bootstrap-datepicker
 * Jaakko Salonen <https://github.com/jsalonen>
 */
;(function($){
	$.fn.datepicker.dates['fi'] = {
		days: ["sunnuntai", "maanantai", "tiistai", "keskiviikko", "torstai", "perjantai", "lauantai", "sunnuntai"],
		daysShort: ["sun", "maa", "tii", "kes", "tor", "per", "lau", "sun"],
		daysMin: ["su", "ma", "ti", "ke", "to", "pe", "la", "su"],
		months: ["tammikuu", "helmikuu", "maaliskuu", "huhtikuu", "toukokuu", "kesäkuu", "heinäkuu", "elokuu", "syyskuu", "lokakuu", "marraskuu", "joulukuu"],
		monthsShort: ["tam", "hel", "maa", "huh", "tou", "kes", "hei", "elo", "syy", "lok", "mar", "jou"],
		today: "tänään",
		weekStart: 1,
		format: "d.m.yyyy"
	};
}(jQuery));

define("datepicker-fi", function(){});

/**
 * French translation for bootstrap-datepicker
 * Nico Mollet <nico.mollet@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['fr'] = {
		days: ["Dimanche", "Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi", "Dimanche"],
		daysShort: ["Dim", "Lun", "Mar", "Mer", "Jeu", "Ven", "Sam", "Dim"],
		daysMin: ["D", "L", "Ma", "Me", "J", "V", "S", "D"],
		months: ["Janvier", "Février", "Mars", "Avril", "Mai", "Juin", "Juillet", "Août", "Septembre", "Octobre", "Novembre", "Décembre"],
		monthsShort: ["Jan", "Fév", "Mar", "Avr", "Mai", "Jui", "Jul", "Aou", "Sep", "Oct", "Nov", "Déc"],
		today: "Aujourd'hui",
		clear: "Effacer",
		weekStart: 1,
		format: "dd/mm/yyyy"
	};
}(jQuery));

define("datepicker-fr", function(){});

;(function($){
	$.fn.datepicker.dates['gl'] = {
		days: ["Domingo", "Luns", "Martes", "Mércores", "Xoves", "Venres", "Sábado", "Domingo"],
		daysShort: ["Dom", "Lun", "Mar", "Mér", "Xov", "Ven", "Sáb", "Dom"],
		daysMin: ["Do", "Lu", "Ma", "Me", "Xo", "Ve", "Sa", "Do"],
		months: ["Xaneiro", "Febreiro", "Marzo", "Abril", "Maio", "Xuño", "Xullo", "Agosto", "Setembro", "Outubro", "Novembro", "Decembro"],
		monthsShort: ["Xan", "Feb", "Mar", "Abr", "Mai", "Xun", "Xul", "Ago", "Sep", "Out", "Nov", "Dec"],
		today: "Hoxe",
		clear: "Limpar",
		weekStart: 1,
		format: "dd/mm/yyyy"
	};
}(jQuery));

define("datepicker-gl", function(){});

/**
 * Hebrew translation for bootstrap-datepicker
 * Sagie Maoz <sagie@maoz.info>
 */
;(function($){
  $.fn.datepicker.dates['he'] = {
      days: ["ראשון", "שני", "שלישי", "רביעי", "חמישי", "שישי", "שבת", "ראשון"],
      daysShort: ["א", "ב", "ג", "ד", "ה", "ו", "ש", "א"],
      daysMin: ["א", "ב", "ג", "ד", "ה", "ו", "ש", "א"],
      months: ["ינואר", "פברואר", "מרץ", "אפריל", "מאי", "יוני", "יולי", "אוגוסט", "ספטמבר", "אוקטובר", "נובמבר", "דצמבר"],
      monthsShort: ["ינו", "פבר", "מרץ", "אפר", "מאי", "יונ", "יול", "אוג", "ספט", "אוק", "נוב", "דצמ"],
      today: "היום",
      rtl: true
  };
}(jQuery));

define("datepicker-he", function(){});

/**
 * Croatian localisation
 */
;(function($){
	$.fn.datepicker.dates['hr'] = {
		days: ["Nedjelja", "Ponedjeljak", "Utorak", "Srijeda", "Četvrtak", "Petak", "Subota", "Nedjelja"],
		daysShort: ["Ned", "Pon", "Uto", "Sri", "Čet", "Pet", "Sub", "Ned"],
		daysMin: ["Ne", "Po", "Ut", "Sr", "Če", "Pe", "Su", "Ne"],
		months: ["Siječanj", "Veljača", "Ožujak", "Travanj", "Svibanj", "Lipanj", "Srpanj", "Kolovoz", "Rujan", "Listopad", "Studeni", "Prosinac"],
		monthsShort: ["Sij", "Velj", "Ožu", "Tra", "Svi", "Lip", "Srp", "Kol", "Ruj", "Lis", "Stu", "Pro"],
		today: "Danas"
	};
}(jQuery));

define("datepicker-hr", function(){});

/**
 * Hungarian translation for bootstrap-datepicker
 * Sotus László <lacisan@gmail.com>
 */
;(function($){
  $.fn.datepicker.dates['hu'] = {
		days: ["Vasárnap", "Hétfő", "Kedd", "Szerda", "Csütörtök", "Péntek", "Szombat", "Vasárnap"],
		daysShort: ["Vas", "Hét", "Ked", "Sze", "Csü", "Pén", "Szo", "Vas"],
		daysMin: ["Va", "Hé", "Ke", "Sz", "Cs", "Pé", "Sz", "Va"],
		months: ["Január", "Február", "Március", "Április", "Május", "Június", "Július", "Augusztus", "Szeptember", "Október", "November", "December"],
		monthsShort: ["Jan", "Feb", "Már", "Ápr", "Máj", "Jún", "Júl", "Aug", "Sze", "Okt", "Nov", "Dec"],
		today: "Ma",
		weekStart: 1,
		format: "yyyy.mm.dd"
	};
}(jQuery));

define("datepicker-hu", function(){});

/**
 * Bahasa translation for bootstrap-datepicker
 * Azwar Akbar <azwar.akbar@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['id'] = {
		days: ["Minggu", "Senin", "Selasa", "Rabu", "Kamis", "Jumat", "Sabtu", "Minggu"],
		daysShort: ["Mgu", "Sen", "Sel", "Rab", "Kam", "Jum", "Sab", "Mgu"],
		daysMin: ["Mg", "Sn", "Sl", "Ra", "Ka", "Ju", "Sa", "Mg"],
		months: ["Januari", "Februari", "Maret", "April", "Mei", "Juni", "Juli", "Agustus", "September", "Oktober", "November", "Desember"],
		monthsShort: ["Jan", "Feb", "Mar", "Apr", "Mei", "Jun", "Jul", "Ags", "Sep", "Okt", "Nov", "Des"],
		today: "Hari Ini",
		clear: "Kosongkan"
	};
}(jQuery));

define("datepicker-id", function(){});

/**
 * Icelandic translation for bootstrap-datepicker
 * Hinrik Örn Sigurðsson <hinrik.sig@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['is'] = {
		days: ["Sunnudagur", "Mánudagur", "Þriðjudagur", "Miðvikudagur", "Fimmtudagur", "Föstudagur", "Laugardagur", "Sunnudagur"],
		daysShort: ["Sun", "Mán", "Þri", "Mið", "Fim", "Fös", "Lau", "Sun"],
		daysMin: ["Su", "Má", "Þr", "Mi", "Fi", "Fö", "La", "Su"],
		months: ["Janúar", "Febrúar", "Mars", "Apríl", "Maí", "Júní", "Júlí", "Ágúst", "September", "Október", "Nóvember", "Desember"],
		monthsShort: ["Jan", "Feb", "Mar", "Apr", "Maí", "Jún", "Júl", "Ágú", "Sep", "Okt", "Nóv", "Des"],
		today: "Í Dag"
	};
}(jQuery));

define("datepicker-is", function(){});

/**
 * Italian translation for bootstrap-datepicker
 * Enrico Rubboli <rubboli@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['it'] = {
		days: ["Domenica", "Lunedì", "Martedì", "Mercoledì", "Giovedì", "Venerdì", "Sabato", "Domenica"],
		daysShort: ["Dom", "Lun", "Mar", "Mer", "Gio", "Ven", "Sab", "Dom"],
		daysMin: ["Do", "Lu", "Ma", "Me", "Gi", "Ve", "Sa", "Do"],
		months: ["Gennaio", "Febbraio", "Marzo", "Aprile", "Maggio", "Giugno", "Luglio", "Agosto", "Settembre", "Ottobre", "Novembre", "Dicembre"],
		monthsShort: ["Gen", "Feb", "Mar", "Apr", "Mag", "Giu", "Lug", "Ago", "Set", "Ott", "Nov", "Dic"],
		today: "Oggi",
		clear: "Cancella",
		weekStart: 1,
		format: "dd/mm/yyyy"
	};
}(jQuery));

define("datepicker-it", function(){});

/**
 * Japanese translation for bootstrap-datepicker
 * Norio Suzuki <https://github.com/suzuki/>
 */
;(function($){
	$.fn.datepicker.dates['ja'] = {
		days: ["日曜", "月曜", "火曜", "水曜", "木曜", "金曜", "土曜", "日曜"],
		daysShort: ["日", "月", "火", "水", "木", "金", "土", "日"],
		daysMin: ["日", "月", "火", "水", "木", "金", "土", "日"],
		months: ["1月", "2月", "3月", "4月", "5月", "6月", "7月", "8月", "9月", "10月", "11月", "12月"],
		monthsShort: ["1月", "2月", "3月", "4月", "5月", "6月", "7月", "8月", "9月", "10月", "11月", "12月"],
		today: "今日",
		format: "yyyy/mm/dd",
		clear: "クリア"
	};
}(jQuery));

define("datepicker-ja", function(){});

/**
 * Georgian translation for bootstrap-datepicker
 * Levan Melikishvili <levani0101@yahoo.com>
 */
;(function($){
    $.fn.datepicker.dates['ka'] = {
        days: ["კვირა", "ორშაბათი", "სამშაბათი", "ოთხშაბათი", "ხუთშაბათი", "პარასკევი", "შაბათი", "კვირა"],
        daysShort: ["კვი", "ორშ", "სამ", "ოთხ", "ხუთ", "პარ", "შაბ", "კვი"],
        daysMin: ["კვ", "ორ", "სა", "ოთ", "ხუ", "პა", "შა", "კვ"],
        months: ["იანვარი", "თებერვალი", "მარტი", "აპრილი", "მაისი", "ივნისი", "ივლისი", "აგვისტო", "სექტემბერი", "ოქტომები", "ნოემბერი", "დეკემბერი"],
        monthsShort: ["იან", "თებ", "მარ", "აპრ", "მაი", "ივნ", "ივლ", "აგვ", "სექ", "ოქტ", "ნოე", "დეკ"],
        today: "დღეს",
        clear: "გასუფთავება",
        weekStart: 1,
        format: "dd.mm.yyyy"
    };
}(jQuery));

define("datepicker-ka", function(){});

/**
 * Kazakh translation for bootstrap-datepicker
 * Yerzhan Tolekov <era.tolekov@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['kk'] = {
		days: ["Жексенбі", "Дүйсенбі", "Сейсенбі", "Сәрсенбі", "Бейсенбі", "Жұма", "Сенбі", "Жексенбі"],
		daysShort: ["Жек", "Дүй", "Сей", "Сәр", "Бей", "Жұм", "Сен", "Жек"],
		daysMin: ["Жк", "Дс", "Сс", "Ср", "Бс", "Жм", "Сн", "Жк"],
		months: ["Қаңтар", "Ақпан", "Наурыз", "Сәуір", "Мамыр", "Маусым", "Шілде", "Тамыз", "Қыркүйек", "Қазан", "Қараша", "Желтоқсан"],
		monthsShort: ["Қаң", "Ақп", "Нау", "Сәу", "Мамыр", "Мау", "Шлд", "Тмз", "Қыр", "Қзн", "Қар", "Жел"],
		today: "Бүгін",
		weekStart: 1
	};
}(jQuery));

define("datepicker-kk", function(){});

/**
 * Korean translation for bootstrap-datepicker
 * Gu Youn <http://github.com/guyoun>
 */
;(function($){
	$.fn.datepicker.dates['kr'] = {
		days: ["일요일", "월요일", "화요일", "수요일", "목요일", "금요일", "토요일", "일요일"],
		daysShort: ["일", "월", "화", "수", "목", "금", "토", "일"],
		daysMin: ["일", "월", "화", "수", "목", "금", "토", "일"],
		months: ["1월", "2월", "3월", "4월", "5월", "6월", "7월", "8월", "9월", "10월", "11월", "12월"],
		monthsShort: ["1월", "2월", "3월", "4월", "5월", "6월", "7월", "8월", "9월", "10월", "11월", "12월"]
	};
}(jQuery));

define("datepicker-kr", function(){});

/**
 * Lithuanian translation for bootstrap-datepicker
 * Šarūnas Gliebus <ssharunas@yahoo.co.uk>
 */

;(function($){
    $.fn.datepicker.dates['lt'] = {
        days: ["Sekmadienis", "Pirmadienis", "Antradienis", "Trečiadienis", "Ketvirtadienis", "Penktadienis", "Šeštadienis", "Sekmadienis"],
        daysShort: ["S", "Pr", "A", "T", "K", "Pn", "Š", "S"],
        daysMin: ["Sk", "Pr", "An", "Tr", "Ke", "Pn", "Št", "Sk"],
        months: ["Sausis", "Vasaris", "Kovas", "Balandis", "Gegužė", "Birželis", "Liepa", "Rugpjūtis", "Rugsėjis", "Spalis", "Lapkritis", "Gruodis"],
        monthsShort: ["Sau", "Vas", "Kov", "Bal", "Geg", "Bir", "Lie", "Rugp", "Rugs", "Spa", "Lap", "Gru"],
        today: "Šiandien",
        weekStart: 1
    };
}(jQuery));

define("datepicker-lt", function(){});

/**
 * Latvian translation for bootstrap-datepicker
 * Artis Avotins <artis@apit.lv>
 */

;(function($){
    $.fn.datepicker.dates['lv'] = {
        days: ["Svētdiena", "Pirmdiena", "Otrdiena", "Trešdiena", "Ceturtdiena", "Piektdiena", "Sestdiena", "Svētdiena"],
        daysShort: ["Sv", "P", "O", "T", "C", "Pk", "S", "Sv"],
        daysMin: ["Sv", "Pr", "Ot", "Tr", "Ce", "Pk", "Se", "Sv"],
        months: ["Janvāris", "Februāris", "Marts", "Aprīlis", "Maijs", "Jūnijs", "Jūlijs", "Augusts", "Septembris", "Oktobris", "Novembris", "Decembris"],
        monthsShort: ["Jan", "Feb", "Mar", "Apr", "Mai", "Jūn", "Jūl", "Aug", "Sep", "Okt", "Nov", "Dec"],
        today: "Šodien",
        weekStart: 1
    };
}(jQuery));

define("datepicker-lv", function(){});

/**
 * Macedonian translation for bootstrap-datepicker
 * Marko Aleksic <psybaron@gmail.com>
 */
;(function($){
    $.fn.datepicker.dates['mk'] = {
        days: ["Недела", "Понеделник", "Вторник", "Среда", "Четврток", "Петок", "Сабота", "Недела"],
        daysShort: ["Нед", "Пон", "Вто", "Сре", "Чет", "Пет", "Саб", "Нед"],
        daysMin: ["Не", "По", "Вт", "Ср", "Че", "Пе", "Са", "Не"],
        months: ["Јануари", "Февруари", "Март", "Април", "Мај", "Јуни", "Јули", "Август", "Септември", "Октомври", "Ноември", "Декември"],
        monthsShort: ["Јан", "Фев", "Мар", "Апр", "Мај", "Јун", "Јул", "Авг", "Сеп", "Окт", "Ное", "Дек"],
        today: "Денес",
        format: "dd.mm.yyyy"
    };
}(jQuery));

define("datepicker-mk", function(){});

/**
 * Malay translation for bootstrap-datepicker
 * Ateman Faiz <noorulfaiz@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['ms'] = {
		days: ["Ahad", "Isnin", "Selasa", "Rabu", "Khamis", "Jumaat", "Sabtu", "Ahad"],
		daysShort: ["Aha", "Isn", "Sel", "Rab", "Kha", "Jum", "Sab", "Aha"],
		daysMin: ["Ah", "Is", "Se", "Ra", "Kh", "Ju", "Sa", "Ah"],
		months: ["Januari", "Februari", "Mac", "April", "Mei", "Jun", "Julai", "Ogos", "September", "Oktober", "November", "Disember"],
		monthsShort: ["Jan", "Feb", "Mar", "Apr", "Mei", "Jun", "Jul", "Ogo", "Sep", "Okt", "Nov", "Dis"],
		today: "Hari Ini"
	};
}(jQuery));

define("datepicker-ms", function(){});

/**
 * Norwegian (bokmål) translation for bootstrap-datepicker
 * Fredrik Sundmyhr <http://github.com/fsundmyhr>
 */
;(function($){
	$.fn.datepicker.dates['nb'] = {
		days: ["Søndag", "Mandag", "Tirsdag", "Onsdag", "Torsdag", "Fredag", "Lørdag", "Søndag"],
		daysShort: ["Søn", "Man", "Tir", "Ons", "Tor", "Fre", "Lør", "Søn"],
		daysMin: ["Sø", "Ma", "Ti", "On", "To", "Fr", "Lø", "Sø"],
		months: ["Januar", "Februar", "Mars", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Desember"],
		monthsShort: ["Jan", "Feb", "Mar", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Des"],
		today: "I Dag"
	};
}(jQuery));

define("datepicker-nb", function(){});

/**
 * Dutch translation for bootstrap-datepicker
 * Reinier Goltstein <mrgoltstein@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['nl'] = {
		days: ["Zondag", "Maandag", "Dinsdag", "Woensdag", "Donderdag", "Vrijdag", "Zaterdag", "Zondag"],
		daysShort: ["Zo", "Ma", "Di", "Wo", "Do", "Vr", "Za", "Zo"],
		daysMin: ["Zo", "Ma", "Di", "Wo", "Do", "Vr", "Za", "Zo"],
		months: ["Januari", "Februari", "Maart", "April", "Mei", "Juni", "Juli", "Augustus", "September", "Oktober", "November", "December"],
		monthsShort: ["Jan", "Feb", "Mrt", "Apr", "Mei", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dec"],
		today: "Vandaag",
		clear: "Wissen",
		weekStart: 1,
		format: "dd-mm-yyyy"
	};
}(jQuery));

define("datepicker-nl", function(){});

/**
 * Belgium-Dutch translation for bootstrap-datepicker
 * Julien Poulin <poulin_julien@hotmail.com>
 */
;(function($){
  $.fn.datepicker.dates['nl-BE'] = {
    days: ["Zondag", "Maandag", "Dinsdag", "Woensdag", "Donderdag", "Vrijdag", "Zaterdag", "Zondag"],
    daysShort: ["Zo", "Ma", "Di", "Wo", "Do", "Vr", "Za", "Zo"],
    daysMin: ["Zo", "Ma", "Di", "Wo", "Do", "Vr", "Za", "Zo"],
    months: ["Januari", "Februari", "Maart", "April", "Mei", "Juni", "Juli", "Augustus", "September", "Oktober", "November", "December"],
    monthsShort: ["Jan", "Feb", "Mrt", "Apr", "Mei", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dec"],
    today: "Vandaag",
    clear: "Leegmaken",
    weekStart: 1,
    format: "dd/mm/yyyy"
  };
}(jQuery));

define("datepicker-nl-BE", function(){});

/**
 *  Norwegian translation for bootstrap-datepicker
 **/
;(function($){
  $.fn.datepicker.dates['no'] = {
    days: ['Søndag','Mandag','Tirsdag','Onsdag','Torsdag','Fredag','Lørdag'],
    daysShort: ['Søn','Man','Tir','Ons','Tor','Fre','Lør'],
    daysMin: ['Sø','Ma','Ti','On','To','Fr','Lø'],
    months: ['Januar','Februar','Mars','April','Mai','Juni','Juli','August','September','Oktober','November','Desember'],
    monthsShort: ['Jan','Feb','Mar','Apr','Mai','Jun','Jul','Aug','Sep','Okt','Nov','Des'],
    today: 'I dag',
    clear: 'Nullstill',
    weekStart: 1,
    format: 'dd.mm.yyyy'
  };
}(jQuery));

define("datepicker-no", function(){});

/**
 * Polish translation for bootstrap-datepicker
 * Robert <rtpm@gazeta.pl>
 */
;(function($){
        $.fn.datepicker.dates['pl'] = {
                days: ["Niedziela", "Poniedziałek", "Wtorek", "Środa", "Czwartek", "Piątek", "Sobota", "Niedziela"],
                daysShort: ["Nie", "Pn", "Wt", "Śr", "Czw", "Pt", "So", "Nie"],
                daysMin: ["N", "Pn", "Wt", "Śr", "Cz", "Pt", "So", "N"],
                months: ["Styczeń", "Luty", "Marzec", "Kwiecień", "Maj", "Czerwiec", "Lipiec", "Sierpień", "Wrzesień", "Październik", "Listopad", "Grudzień"],
                monthsShort: ["Sty", "Lu", "Mar", "Kw", "Maj", "Cze", "Lip", "Sie", "Wrz", "Pa", "Lis", "Gru"],
                today: "Dzisiaj",
                weekStart: 1,
                clear: "Wyczyść"
        };
}(jQuery));

define("datepicker-pl", function(){});

/**
 * Portuguese translation for bootstrap-datepicker
 * Original code: Cauan Cabral <cauan@radig.com.br>
 * Tiago Melo <tiago.blackcode@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['pt'] = {
		days: ["Domingo", "Segunda", "Terça", "Quarta", "Quinta", "Sexta", "Sábado", "Domingo"],
		daysShort: ["Dom", "Seg", "Ter", "Qua", "Qui", "Sex", "Sáb", "Dom"],
		daysMin: ["Do", "Se", "Te", "Qu", "Qu", "Se", "Sa", "Do"],
		months: ["Janeiro", "Fevereiro", "Março", "Abril", "Maio", "Junho", "Julho", "Agosto", "Setembro", "Outubro", "Novembro", "Dezembro"],
		monthsShort: ["Jan", "Fev", "Mar", "Abr", "Mai", "Jun", "Jul", "Ago", "Set", "Out", "Nov", "Dez"],
		today: "Hoje",
		clear: "Limpar"
	};
}(jQuery));

define("datepicker-pt", function(){});

/**
 * Brazilian translation for bootstrap-datepicker
 * Cauan Cabral <cauan@radig.com.br>
 */
;(function($){
	$.fn.datepicker.dates['pt-BR'] = {
		days: ["Domingo", "Segunda", "Terça", "Quarta", "Quinta", "Sexta", "Sábado", "Domingo"],
		daysShort: ["Dom", "Seg", "Ter", "Qua", "Qui", "Sex", "Sáb", "Dom"],
		daysMin: ["Do", "Se", "Te", "Qu", "Qu", "Se", "Sa", "Do"],
		months: ["Janeiro", "Fevereiro", "Março", "Abril", "Maio", "Junho", "Julho", "Agosto", "Setembro", "Outubro", "Novembro", "Dezembro"],
		monthsShort: ["Jan", "Fev", "Mar", "Abr", "Mai", "Jun", "Jul", "Ago", "Set", "Out", "Nov", "Dez"],
		today: "Hoje",
		clear: "Limpar"
	};
}(jQuery));

define("datepicker-pt-BR", function(){});

/**
 * Romanian translation for bootstrap-datepicker
 * Cristian Vasile <cristi.mie@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['ro'] = {
		days: ["Duminică", "Luni", "Marţi", "Miercuri", "Joi", "Vineri", "Sâmbătă", "Duminică"],
		daysShort: ["Dum", "Lun", "Mar", "Mie", "Joi", "Vin", "Sâm", "Dum"],
		daysMin: ["Du", "Lu", "Ma", "Mi", "Jo", "Vi", "Sâ", "Du"],
		months: ["Ianuarie", "Februarie", "Martie", "Aprilie", "Mai", "Iunie", "Iulie", "August", "Septembrie", "Octombrie", "Noiembrie", "Decembrie"],
		monthsShort: ["Ian", "Feb", "Mar", "Apr", "Mai", "Iun", "Iul", "Aug", "Sep", "Oct", "Nov", "Dec"],
		today: "Astăzi",
		clear: "Șterge",
		weekStart: 1
	};
}(jQuery));

define("datepicker-ro", function(){});

/**
 * Serbian cyrillic translation for bootstrap-datepicker
 * Bojan Milosavlević <milboj@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['rs'] = {
		days: ["Недеља","Понедељак", "Уторак", "Среда", "Четвртак", "Петак", "Субота", "Недеља"],
		daysShort: ["Нед", "Пон", "Уто", "Сре", "Чет", "Пет", "Суб", "Нед"],
		daysMin: ["Н", "По", "У", "Ср", "Ч", "Пе", "Су", "Н"],
		months: ["Јануар", "Фебруар", "Март", "Април", "Мај", "Јун", "Јул", "Август", "Септембар", "Октобар", "Новембар", "Децембар"],
		monthsShort: ["Јан", "Феб", "Мар", "Апр", "Мај", "Јун", "Јул", "Авг", "Сеп", "Окт", "Нов", "Дец"],
		today: "Данас",
		weekStart: 1,
		format: "dd.mm.yyyy"
	};
}(jQuery));

define("datepicker-rs", function(){});

/**
 * Serbian latin translation for bootstrap-datepicker
 * Bojan Milosavlević <milboj@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['rs-latin'] = {
		days: ["Nedelja","Ponedeljak", "Utorak", "Sreda", "Četvrtak", "Petak", "Subota", "Nedelja"],
		daysShort: ["Ned", "Pon", "Uto", "Sre", "Čet", "Pet", "Sub", "Ned"],
		daysMin: ["N", "Po", "U", "Sr", "Č", "Pe", "Su", "N"],
		months: ["Januar", "Februar", "Mart", "April", "Maj", "Jun", "Jul", "Avgust", "Septembar", "Oktobar", "Novembar", "Decembar"],
		monthsShort: ["Jan", "Feb", "Mar", "Apr", "Maj", "Jun", "Jul", "Avg", "Sep", "Okt", "Nov", "Dec"],
		today: "Danas",
		weekStart: 1,
		format: "dd.mm.yyyy"
	};
}(jQuery));

define("datepicker-rs-latin", function(){});

/**
 * Russian translation for bootstrap-datepicker
 * Victor Taranenko <darwin@snowdale.com>
 */
;(function($){
	$.fn.datepicker.dates['ru'] = {
		days: ["Воскресенье", "Понедельник", "Вторник", "Среда", "Четверг", "Пятница", "Суббота", "Воскресенье"],
		daysShort: ["Вск", "Пнд", "Втр", "Срд", "Чтв", "Птн", "Суб", "Вск"],
		daysMin: ["Вс", "Пн", "Вт", "Ср", "Чт", "Пт", "Сб", "Вс"],
		months: ["Январь", "Февраль", "Март", "Апрель", "Май", "Июнь", "Июль", "Август", "Сентябрь", "Октябрь", "Ноябрь", "Декабрь"],
		monthsShort: ["Янв", "Фев", "Мар", "Апр", "Май", "Июн", "Июл", "Авг", "Сен", "Окт", "Ноя", "Дек"],
		today: "Сегодня",
		clear: "Очистить",
		format: "dd.mm.yyyy",
		weekStart: 1
	};
}(jQuery));

define("datepicker-ru", function(){});

/**
 * Slovak translation for bootstrap-datepicker
 * Marek Lichtner <marek@licht.sk>
 * Fixes by Michal Remiš <michal.remis@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates["sk"] = {
		days: ["Nedeľa", "Pondelok", "Utorok", "Streda", "Štvrtok", "Piatok", "Sobota", "Nedeľa"],
		daysShort: ["Ned", "Pon", "Uto", "Str", "Štv", "Pia", "Sob", "Ned"],
		daysMin: ["Ne", "Po", "Ut", "St", "Št", "Pia", "So", "Ne"],
		months: ["Január", "Február", "Marec", "Apríl", "Máj", "Jún", "Júl", "August", "September", "Október", "November", "December"],
		monthsShort: ["Jan", "Feb", "Mar", "Apr", "Máj", "Jún", "Júl", "Aug", "Sep", "Okt", "Nov", "Dec"],
		today: "Dnes"
	};
}(jQuery));

define("datepicker-sk", function(){});

/**
 * Slovene translation for bootstrap-datepicker
 * Gregor Rudolf <gregor.rudolf@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['sl'] = {
		days: ["Nedelja", "Ponedeljek", "Torek", "Sreda", "Četrtek", "Petek", "Sobota", "Nedelja"],
		daysShort: ["Ned", "Pon", "Tor", "Sre", "Čet", "Pet", "Sob", "Ned"],
		daysMin: ["Ne", "Po", "To", "Sr", "Če", "Pe", "So", "Ne"],
		months: ["Januar", "Februar", "Marec", "April", "Maj", "Junij", "Julij", "Avgust", "September", "Oktober", "November", "December"],
		monthsShort: ["Jan", "Feb", "Mar", "Apr", "Maj", "Jun", "Jul", "Avg", "Sep", "Okt", "Nov", "Dec"],
		today: "Danes"
	};
}(jQuery));

define("datepicker-sl", function(){});

/**
 * Albanian translation for bootstrap-datepicker
 * Tomor Pupovci <http://www.github.com/ttomor>
 */
;(function($){
	$.fn.datepicker.dates['sq'] = {
		days: ["E Diel", "E Hënë", "E Martē", "E Mërkurë", "E Enjte", "E Premte", "E Shtunë", "E Diel"],
		daysShort: ["Die", "Hën", "Mar", "Mër", "Enj", "Pre", "Shtu", "Die"],
		daysMin: ["Di", "Hë", "Ma", "Më", "En", "Pr", "Sht", "Di"],
		months: ["Janar", "Shkurt", "Mars", "Prill", "Maj", "Qershor", "Korrik", "Gusht", "Shtator", "Tetor", "Nëntor", "Dhjetor"],
		monthsShort: ["Jan", "Shk", "Mar", "Pri", "Maj", "Qer", "Korr", "Gu", "Sht", "Tet", "Nën", "Dhjet"],
		today: "Sot"
	};
}(jQuery));


define("datepicker-sq", function(){});

/**
 * Swedish translation for bootstrap-datepicker
 * Patrik Ragnarsson <patrik@starkast.net>
 */
;(function($){
	$.fn.datepicker.dates['sv'] = {
		days: ["Söndag", "Måndag", "Tisdag", "Onsdag", "Torsdag", "Fredag", "Lördag", "Söndag"],
		daysShort: ["Sön", "Mån", "Tis", "Ons", "Tor", "Fre", "Lör", "Sön"],
		daysMin: ["Sö", "Må", "Ti", "On", "To", "Fr", "Lö", "Sö"],
		months: ["Januari", "Februari", "Mars", "April", "Maj", "Juni", "Juli", "Augusti", "September", "Oktober", "November", "December"],
		monthsShort: ["Jan", "Feb", "Mar", "Apr", "Maj", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dec"],
		today: "Idag",
		format: "yyyy-mm-dd",
		weekStart: 1,
		clear: "Rensa"
	};
}(jQuery));

define("datepicker-sv", function(){});

/**
 * Swahili translation for bootstrap-datepicker
 * Edwin Mugendi <https://github.com/edwinmugendi>
 * Source: http://scriptsource.org/cms/scripts/page.php?item_id=entry_detail&uid=xnfaqyzcku
 */
;(function($){
    $.fn.datepicker.dates['sw'] = {
        days: ["Jumapili", "Jumatatu", "Jumanne", "Jumatano", "Alhamisi", "Ijumaa", "Jumamosi", "Jumapili"],
        daysShort: ["J2", "J3", "J4", "J5", "Alh", "Ij", "J1", "J2"],
        daysMin: ["2", "3", "4", "5", "A", "I", "1", "2"],
        months: ["Januari", "Februari", "Machi", "Aprili", "Mei", "Juni", "Julai", "Agosti", "Septemba", "Oktoba", "Novemba", "Desemba"],
        monthsShort: ["Jan", "Feb", "Mac", "Apr", "Mei", "Jun", "Jul", "Ago", "Sep", "Okt", "Nov", "Des"],
        today: "Leo"
    };
}(jQuery));

define("datepicker-sw", function(){});

/**
 * Thai translation for bootstrap-datepicker
 * Suchau Jiraprapot <seroz24@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['th'] = {
		days: ["อาทิตย์", "จันทร์", "อังคาร", "พุธ", "พฤหัส", "ศุกร์", "เสาร์", "อาทิตย์"],
		daysShort: ["อา", "จ", "อ", "พ", "พฤ", "ศ", "ส", "อา"],
		daysMin: ["อา", "จ", "อ", "พ", "พฤ", "ศ", "ส", "อา"],
		months: ["มกราคม", "กุมภาพันธ์", "มีนาคม", "เมษายน", "พฤษภาคม", "มิถุนายน", "กรกฎาคม", "สิงหาคม", "กันยายน", "ตุลาคม", "พฤศจิกายน", "ธันวาคม"],
		monthsShort: ["ม.ค.", "ก.พ.", "มี.ค.", "เม.ย.", "พ.ค.", "มิ.ย.", "ก.ค.", "ส.ค.", "ก.ย.", "ต.ค.", "พ.ย.", "ธ.ค."],
		today: "วันนี้"
	};
}(jQuery));

define("datepicker-th", function(){});

/**
 * Turkish translation for bootstrap-datepicker
 * Serkan Algur <kaisercrazy_2@hotmail.com>
 */
;(function($){
	$.fn.datepicker.dates['tr'] = {
		days: ["Pazar", "Pazartesi", "Salı", "Çarşamba", "Perşembe", "Cuma", "Cumartesi", "Pazar"],
		daysShort: ["Pz", "Pzt", "Sal", "Çrş", "Prş", "Cu", "Cts", "Pz"],
		daysMin: ["Pz", "Pzt", "Sa", "Çr", "Pr", "Cu", "Ct", "Pz"],
		months: ["Ocak", "Şubat", "Mart", "Nisan", "Mayıs", "Haziran", "Temmuz", "Ağustos", "Eylül", "Ekim", "Kasım", "Aralık"],
		monthsShort: ["Oca", "Şub", "Mar", "Nis", "May", "Haz", "Tem", "Ağu", "Eyl", "Eki", "Kas", "Ara"],
		today: "Bugün",
		format: "dd.mm.yyyy"
	};
}(jQuery));


define("datepicker-tr", function(){});

/**
 * Vietnamese translation for bootstrap-datepicker
 * An Vo <https://github.com/anvoz/>
 */
;(function($){
	$.fn.datepicker.dates['vi'] = {
		days: ["Chủ nhật", "Thứ hai", "Thứ ba", "Thứ tư", "Thứ năm", "Thứ sáu", "Thứ bảy", "Chủ nhật"],
		daysShort: ["CN", "Thứ 2", "Thứ 3", "Thứ 4", "Thứ 5", "Thứ 6", "Thứ 7", "CN"],
		daysMin: ["CN", "T2", "T3", "T4", "T5", "T6", "T7", "CN"],
		months: ["Tháng 1", "Tháng 2", "Tháng 3", "Tháng 4", "Tháng 5", "Tháng 6", "Tháng 7", "Tháng 8", "Tháng 9", "Tháng 10", "Tháng 11", "Tháng 12"],
		monthsShort: ["Th1", "Th2", "Th3", "Th4", "Th5", "Th6", "Th7", "Th8", "Th9", "Th10", "Th11", "Th12"],
		today: "Hôm nay",
		clear: "Xóa",
		format: "dd/mm/yyyy"
	};
}(jQuery));

define("datepicker-vi", function(){});

/**
 * Simplified Chinese translation for bootstrap-datepicker
 * Yuan Cheung <advanimal@gmail.com>
 */
;(function($){
	$.fn.datepicker.dates['zh-CN'] = {
		days: ["星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六", "星期日"],
		daysShort: ["周日", "周一", "周二", "周三", "周四", "周五", "周六", "周日"],
		daysMin:  ["日", "一", "二", "三", "四", "五", "六", "日"],
		months: ["一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"],
		monthsShort: ["一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"],
		today: "今日",
		format: "yyyy年mm月dd日",
		weekStart: 1,
		clear: "清空"
	};
}(jQuery));

define("datepicker-zh-CN", function(){});

/**
 * Traditional Chinese translation for bootstrap-datepicker
 * Rung-Sheng Jang <daniel@i-trend.co.cc>
 * FrankWu  <frankwu100@gmail.com> Fix more appropriate use of Traditional Chinese habit
 */
;(function($){
	$.fn.datepicker.dates['zh-TW'] = {
		days: ["星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六", "星期日"],
		daysShort: ["週日", "週一", "週二", "週三", "週四", "週五", "週六", "週日"],
		daysMin:  ["日", "一", "二", "三", "四", "五", "六", "日"],
		months: ["一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"],
		monthsShort: ["一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"],
		today: "今天",
		format: "yyyy年mm月dd日",
		weekStart: 1
	};
}(jQuery));

define("datepicker-zh-TW", function(){});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    require.config({
        paths: {
            "bootstrap-datepicker": 'bower_components/bootstrap-datepicker/bootstrap-datepicker',
            "datepicker-az": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.az',
            "datepicker-bg": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.bg',
            "datepicker-ar": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.ar',
            "datepicker-ca": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.ca',
            "datepicker-cs": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.cs',
            "datepicker-cy": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.cy',
            "datepicker-da": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.da',
            "datepicker-de": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.de',
            "datepicker-el": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.el',
            "datepicker-es": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.es',
            "datepicker-et": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.et',
            "datepicker-sq": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.sq',
            "datepicker-fa": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.fa',
            "datepicker-fi": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.fi',
            "datepicker-fr": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.fr',
            "datepicker-gl": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.gl',
            "datepicker-he": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.he',
            "datepicker-hr": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.hr',
            "datepicker-hu": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.hu',
            "datepicker-id": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.id',
            "datepicker-is": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.is',
            "datepicker-it": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.it',
            "datepicker-ja": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.ja',
            "datepicker-ka": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.ka',
            "datepicker-kk": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.kk',
            "datepicker-kr": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.kr',
            "datepicker-lt": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.lt',
            "datepicker-lv": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.lv',
            "datepicker-mk": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.mk',
            "datepicker-ms": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.ms',
            "datepicker-nb": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.nb',
            "datepicker-nl": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.nl',
            "datepicker-nl-BE": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.nl-BE',
            "datepicker-no": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.no',
            "datepicker-pl": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.pl',
            "datepicker-pt": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.pt',
            "datepicker-pt-BR": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.pt-BR',
            "datepicker-ro": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.ro',
            "datepicker-rs": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.rs',
            "datepicker-rs-latin": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.rs-latin',
            "datepicker-ru": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.ru',
            "datepicker-sk": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.sk',
            "datepicker-sl": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.sl',
            "datepicker-sv": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.sv',
            "datepicker-sw": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.sw',
            "datepicker-th": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.th',
            "datepicker-tr": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.tr',
            "datepicker-vi": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.vi',
            "datepicker-zh-CN": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.zh-CN',
            "datepicker-zh-TW": 'bower_components/bootstrap-datepicker/locales/bootstrap-datepicker.zh-TW'
        },
        shim: {
            "datepicker-az": {deps: ['bootstrap-datepicker']},
            "datepicker-bg": {deps: ['bootstrap-datepicker']},
            "datepicker-ar": {deps: ['bootstrap-datepicker']},
            "datepicker-ca": {deps: ['bootstrap-datepicker']},
            "datepicker-cs": {deps: ['bootstrap-datepicker']},
            "datepicker-cy": {deps: ['bootstrap-datepicker']},
            "datepicker-da": {deps: ['bootstrap-datepicker']},
            "datepicker-de": {deps: ['bootstrap-datepicker']},
            "datepicker-el": {deps: ['bootstrap-datepicker']},
            "datepicker-es": {deps: ['bootstrap-datepicker']},
            "datepicker-et": {deps: ['bootstrap-datepicker']},
            "datepicker-sq": {deps: ['bootstrap-datepicker']},
            "datepicker-fa": {deps: ['bootstrap-datepicker']},
            "datepicker-fi": {deps: ['bootstrap-datepicker']},
            "datepicker-fr": {deps: ['bootstrap-datepicker']},
            "datepicker-gl": {deps: ['bootstrap-datepicker']},
            "datepicker-he": {deps: ['bootstrap-datepicker']},
            "datepicker-hr": {deps: ['bootstrap-datepicker']},
            "datepicker-hu": {deps: ['bootstrap-datepicker']},
            "datepicker-id": {deps: ['bootstrap-datepicker']},
            "datepicker-is": {deps: ['bootstrap-datepicker']},
            "datepicker-it": {deps: ['bootstrap-datepicker']},
            "datepicker-ja": {deps: ['bootstrap-datepicker']},
            "datepicker-ka": {deps: ['bootstrap-datepicker']},
            "datepicker-kk": {deps: ['bootstrap-datepicker']},
            "datepicker-kr": {deps: ['bootstrap-datepicker']},
            "datepicker-lt": {deps: ['bootstrap-datepicker']},
            "datepicker-lv": {deps: ['bootstrap-datepicker']},
            "datepicker-mk": {deps: ['bootstrap-datepicker']},
            "datepicker-ms": {deps: ['bootstrap-datepicker']},
            "datepicker-nb": {deps: ['bootstrap-datepicker']},
            "datepicker-nl": {deps: ['bootstrap-datepicker']},
            "datepicker-nl-BE": {deps: ['bootstrap-datepicker']},
            "datepicker-no": {deps: ['bootstrap-datepicker']},
            "datepicker-pl": {deps: ['bootstrap-datepicker']},
            "datepicker-pt": {deps: ['bootstrap-datepicker']},
            "datepicker-pt-BR": {deps: ['bootstrap-datepicker']},
            "datepicker-ro": {deps: ['bootstrap-datepicker']},
            "datepicker-rs": {deps: ['bootstrap-datepicker']},
            "datepicker-rs-latin": {deps: ['bootstrap-datepicker']},
            "datepicker-ru": {deps: ['bootstrap-datepicker']},
            "datepicker-sk": {deps: ['bootstrap-datepicker']},
            "datepicker-sl": {deps: ['bootstrap-datepicker']},
            "datepicker-sv": {deps: ['bootstrap-datepicker']},
            "datepicker-sw": {deps: ['bootstrap-datepicker']},
            "datepicker-th": {deps: ['bootstrap-datepicker']},
            "datepicker-tr": {deps: ['bootstrap-datepicker']},
            "datepicker-vi": {deps: ['bootstrap-datepicker']},
            "datepicker-zh-CN": {deps: ['bootstrap-datepicker']},
            "datepicker-zh-TW": {deps: ['bootstrap-datepicker']}
        }
    });

    define('husky_extensions/datepicker',['bootstrap-datepicker',
        'datepicker-ar', 'datepicker-az', 'datepicker-bg', 'datepicker-ca', 'datepicker-cs', 'datepicker-cy',
        'datepicker-da', 'datepicker-de', 'datepicker-el', 'datepicker-es', 'datepicker-et', 'datepicker-fa',
        'datepicker-fi', 'datepicker-fr', 'datepicker-gl', 'datepicker-he', 'datepicker-hr', 'datepicker-hu',
        'datepicker-id', 'datepicker-is', 'datepicker-it', 'datepicker-ja', 'datepicker-ka', 'datepicker-kk',
        'datepicker-kr', 'datepicker-lt', 'datepicker-lv', 'datepicker-mk', 'datepicker-ms', 'datepicker-nb',
        'datepicker-nl', 'datepicker-nl-BE', 'datepicker-no', 'datepicker-pl', 'datepicker-pt', 'datepicker-pt-BR',
        'datepicker-ro', 'datepicker-rs', 'datepicker-rs-latin', 'datepicker-ru', 'datepicker-sk', 'datepicker-sl',
        'datepicker-sq', 'datepicker-sv', 'datepicker-sw', 'datepicker-th', 'datepicker-tr','datepicker-vi',
        'datepicker-zh-CN', 'datepicker-zh-TW'], {

        name: 'bootstrap-datepicker',

        initialize: function(app) {

            app.sandbox.datepicker = {

                init: function(selector, configs) {
                    var settings = {
                        format: app.sandbox.globalize.getDatePattern().toLowerCase(),
                        language: app.sandbox.globalize.getLocale(),
                        autoclose: true
                    };
                    settings = app.sandbox.util.extend(true, {}, settings, configs);
                    return app.core.dom.$(selector).datepicker(settings);
                },

                setValue: function(selector, value) {
                    return app.core.dom.$(selector).datepicker('update', value);
                },

                getUTCDate: function(selector) {
                    return app.core.dom.$(selector).datepicker('getUTCDate');
                },

                getDate: function(selector) {
                    return app.core.dom.$(selector).datepicker('getDate');
                },

                setUTCDate: function(selector, date) {
                    return app.core.dom.$(selector).datepicker('setUTCDate', date);
                },

                setDate: function(selector, date) {
                    return app.core.dom.$(selector).datepicker('setDate', date);
                }
            };
        }
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    require.config({
        paths: { 'dropzone': 'bower_components/dropzone/dropzone' },
        shim: {
            dropzone: {exports: 'Dropzone', deps: ['jquery']}
        }
    });

    define('husky_extensions/dropzone',['dropzone'], function(Dropzone) {
        return {
            name: 'dropzone',

            initialize: function(app) {
                // Disable confirmation
                Dropzone.confirm = function(question, accepted) {
                    accepted();
                };

                app.sandbox.dropzone = {
                    initialize: function(selector, configs) {
                        return app.core.dom.$(selector).dropzone(configs);
                    }
                };
            }
        };
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    require.config({
        paths: {
            'globalize_lib': 'bower_components/globalize/lib/globalize',
            'cultures': 'bower_components/globalize/lib/cultures'
        }
    });

    define('husky_extensions/globalize',['globalize_lib'], function() {
        var normalizeCultureName = function(cultureName) {
            cultureName = cultureName.replace('_', '-');

            if (cultureName.indexOf('-') > -1) {
                cultureName =
                    cultureName.substring(0, cultureName.indexOf('-')) +
                    cultureName.substring(cultureName.indexOf('-'), cultureName.length).toUpperCase();
            }

            return cultureName;
        },

        initializeLanguages = function(app) {
            if (!!app.config.culture) {
                if (!app.config.culture.messages) {
                    app.config.culture.messages = {};
                }

                app.loadLanguage(app.config.culture.name).then(function() {
                    app.setLanguage(
                        app.config.culture.name,
                        app.config.culture.messages,
                        app.config.culture.defaultMessages
                    );
                });
            }

            app.sandbox.globalize.setCurrency('');
        };

        return {
            name: 'husky-validation',

            initialize: function(app) {

                app.sandbox.globalize = {
                    addCultureInfo: function(cultureName, messages) {
                        cultureName = normalizeCultureName(cultureName);
                        Globalize.addCultureInfo(cultureName, {
                            messages: messages
                        });
                    },

                    getLocale: function() {
                        return Globalize.culture().name;
                    },
                    getDatePattern: function() {
                        return Globalize.culture().calendar.patterns.d;
                    },
                    getTimePatter: function() {
                        return Globalize.culture().calendar.patterns.t;
                    },
                    setCurrency: function(symbol) {
                        Globalize.culture().numberFormat.currency.symbol = symbol;
                    }
                };

                app.sandbox.translate = function(key) {
                    var translation;
                    if (!app.config.culture || !Globalize.culture().name) {
                        return key;
                    }

                    try {
                        translation = Globalize.localize(key, Globalize.culture().name);
                    } catch (e) {
                        app.logger.warn('Globalize threw an error when translating key "' + key + '", failling back to key. Error: ' + e);
                        return key;
                    }

                    return (!!translation || translation === '') ? translation : key;
                };

                app.sandbox.date = {
                    /**
                     * returns formatted date string
                     * @param {string|Date} date
                     * @param {Boolean} returnDateAndTime
                     * @returns {string}
                     */
                    format: function(date, returnDateAndTime) {
                        var returnDate, returnTime;
                        if(typeof date === 'string'){
                            date = this.parse(date);
                        }

                        returnDate = Globalize.format(date, Globalize.culture().calendar.patterns.d);
                        if (returnDateAndTime === true) {
                            returnTime = Globalize.format(date, Globalize.culture().calendar.patterns.t);
                        }

                        return ( (!!returnDate) ? returnDate : '' ) +
                               ( (!!returnDate && !!returnTime) ? ' ': '' ) +
                               ( (!!returnTime) ? returnTime : '' );
                    },

                    /**
                     * parse ISO8601 string : string -> Date
                     * Parse an ISO-8601 date, including possible timezone,
                     * into a Javascript Date object.
                     * (inspired by: http://stackoverflow.com/questions/439630/how-do-you-create-a-javascript-date-object-with-a-set-timezone-without-using-a-s)
                     *
                     * Test strings: parseISO8601String(x).toISOString()
                     * "2013-01-31T12:34"              -> "2013-01-31T12:34:00.000Z"
                     * "2013-01-31T12:34:56"           -> "2013-01-31T12:34:56.000Z"
                     * "2013-01-31T12:34:56.78"        -> "2013-01-31T12:34:56.780Z"
                     * "2013-01-31T12:34:56.78+0100"   -> "2013-01-31T11:34:56.780Z"
                     * "2013-01-31T12:34:56.78+0530"   -> "2013-01-31T07:04:56.780Z"
                     * "2013-01-31T12:34:56.78-0330"   -> "2013-01-31T16:04:56.780Z"
                     * "2013-01-31T12:34:56-0330"      -> "2013-01-31T16:04:56.000Z"
                     * "2013-01-31T12:34:56Z"          -> "2013-01-31T12:34:56.000Z"
                     *
                     * @param {string} dateString
                     * @returns {Date}
                     */
                    parse: function(dateString) {
                        var timebitsRegex = /^([0-9]{4})-([0-9]{2})-([0-9]{2})T([0-9]{2}):([0-9]{2})(?::([0-9]*)(\.[0-9]*)?)?(?:([+-])([0-9]{2})([0-9]{2}))?/,
                            timebits = timebitsRegex.exec(dateString),
                            resultDate, utcdate, offsetMinutes;

                        if (timebits) {
                            utcdate = Date.UTC(parseInt(timebits[1]),
                                parseInt(timebits[2]) - 1, // months are zero-offset (!)
                                parseInt(timebits[3]),
                                parseInt(timebits[4]), parseInt(timebits[5]), // hh:mm
                                (timebits[6] && parseInt(timebits[6]) || 0),  // optional seconds
                                (timebits[7] && parseFloat(timebits[7]) * 1000) || 0); // optional fraction
                            // utcdate is milliseconds since the epoch
                            if (timebits[9] && timebits[10]) {
                                offsetMinutes = parseInt(timebits[9]) * 60 + parseInt(timebits[10]);
                                utcdate += (timebits[8] === '+' ? -1 : +1) * offsetMinutes * 60000;
                            }
                            resultDate = new Date(utcdate);
                        } else {
                            resultDate = null;
                        }
                        return resultDate;
                    }
                };

                /**
                 * function calls this.sandbox.translate for an array of keys and returns an array of translations
                 * @param array
                 * @returns {Array}
                 */
                app.sandbox.translateArray = function(array) {
                    var translations = [];
                    app.sandbox.util.foreach(array, function(key) {
                        translations.push(app.sandbox.translate(key));
                    }.bind(this));
                    return translations;
                };

                /**
                 * formats a number; calls Globalize.format (see globalize documentation for any details)
                 *  https://github.com/jquery/globalize/tree/v0.1.1#numbers
                 * @param number
                 * @param types Possible types: n (number), d (decimal-digits), p (percentage), c (currency)
                 */
                app.sandbox.numberFormat = function(number, types) {
                    return Globalize.format(number, types);
                };

                /**
                 * Parses a float value according to the given culture
                 * @param value
                 * @param radix default 10
                 * @param culture current culture if no culture given
                 * @returns {*}
                 */
                app.sandbox.parseFloat = function(value, radix, culture) {
                    return Globalize.parseFloat(value, radix, culture);
                };

                /**
                 * Set culture with given name and messages
                 *
                 * @param cultureName
                 * @param messages
                 * @param defaultMessages will be used as fallback messages
                 */
                app.setLanguage = function(cultureName, messages, defaultMessages) {
                    cultureName = normalizeCultureName(cultureName);

                    Globalize.culture(cultureName);

                    app.sandbox.globalize.addCultureInfo(cultureName, messages);
                    app.sandbox.globalize.addCultureInfo('default', defaultMessages);
                };

                app.loadLanguage = function(cultureName) {
                    var deferred = $.Deferred();

                    cultureName = normalizeCultureName(cultureName);

                    if (cultureName !== 'en') {
                        require(['cultures/globalize.culture.' + cultureName], function() {
                            deferred.resolve();
                        });
                    } else {
                        deferred.resolve();
                    }

                    return deferred.promise();
                };

                initializeLanguages(app);
            }
        };
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

/**
 * see https://github.com/farhadi/html5sortable for documentation
 */

(function() {

    'use strict';

    require.config({
        paths: {
            'jquery': 'bower_components/jquery/jquery',
            'html.sortable': 'bower_components/html.sortable/html.sortable'
        },
        shim: {
            jquery: { exports: '$' },
            html5sortable: {deps: ['jquery'], exports: 'jQuery.fn.sortable'}
        }
    });


    define('husky_extensions/html.sortable',['html.sortable'], function() {

        return {
            name: 'html.sortable',

            initialize: function(app) {
                app.core.dom.sortable = function(selector, options) {
                    return $(selector).sortable(options);
                };
            }
        };
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    require.config({
        paths: { "form": 'bower_components/husky-validation/dist/validation' }
    });

    define('husky_extensions/husky-validation',['form'], function(Form) {
        return  {
            name: 'husky-validation',

            initialize: function(app) {
                app.sandbox.form = {

                    create: function(selector, options) {
                        return new Form($(selector), options);
                    },

                    getObject: function(selector) {
                        return $(selector).data('form-object');
                    },

                    validate: function(selector, force) {
                        if ($(selector).length === 0) {
                            return false;
                        }

                        if (!force) {
                            force = false;
                        }

                        return  app.sandbox.form.getObject(selector).validation.validate(force);
                    },

                    isValid: function(selector) {
                        return app.sandbox.form.getObject(selector).validation.isValid();
                    },

                    addField: function(selector, elementSelector) {
                        return app.sandbox.form.getObject(selector).addField(elementSelector);
                    },

                    removeField: function(selector, elementSelector) {
                        return app.sandbox.form.getObject(selector).removeField(elementSelector);
                    },

                    addConstraint: function(selector, elementSelector, constraintName, options) {
                        app.sandbox.form.getObject(selector).validation.addConstraint(elementSelector, constraintName, options);
                    },

                    updateConstraint: function(selector, elementSelector, constraintName, options) {
                        app.sandbox.form.getObject(selector).validation.updateConstraint(elementSelector, constraintName, options);
                    },

                    deleteConstraint: function(selector, elementSelector, constraintName) {
                        app.sandbox.form.getObject(selector).validation.deleteConstraint(elementSelector, constraintName);
                    },

                    setData: function(selector, data) {
                        return app.sandbox.form.getObject(selector).mapper.setData(data);
                    },

                    getData: function(selector, returnMapperId, $el) {
                        return  app.sandbox.form.getObject(selector).mapper.getData($el, returnMapperId);
                    },

                    addToCollection: function(selector, propertyName, data, append) {
                        return  app.sandbox.form.getObject(selector).mapper.addToCollection(propertyName, data, append);
                    },

                    editInCollection: function(selector, mapperId, data) {
                        return  app.sandbox.form.getObject(selector).mapper.editInCollection(mapperId, data);
                    },

                    removeFromCollection: function(selector, mapperId) {
                        return  app.sandbox.form.getObject(selector).mapper.removeFromCollection(mapperId);
                    },

                    addCollectionFilter: function(selector, arrayName, callback) {
                        app.sandbox.form.getObject(selector).mapper.addCollectionFilter(arrayName, callback);
                    },

                    removeCollectionFilter: function(selector, arrayName) {
                        app.sandbox.form.getObject(selector).mapper.removeCollectionFilter(arrayName);
                    },

                    element: {
                        getObject: function(elementSelector) {
                            return $(elementSelector).data('element');
                        },

                        validate: function(elementSelector, force) {
                            return app.sandbox.form.element.getObject(elementSelector).validate(force);
                        },

                        addConstraint: function(elementSelector, constraintName, options) {
                            app.sandbox.form.element.getObject(elementSelector).addConstraint(constraintName, options);
                        },

                        updateConstraint: function(elementSelector, constraintName, options) {
                            app.sandbox.form.element.getObject(elementSelector).updateConstraint(constraintName, options);
                        },

                        deleteConstraint: function(elementSelector, constraintName) {
                            app.sandbox.form.element.getObject(elementSelector).deleteConstraint(constraintName);
                        },

                        hasConstraint: function(elementSelector, constraintName) {
                            return app.sandbox.form.element.getObject(elementSelector).hasConstraint(constraintName);
                        },

                        getConstraint: function(elementSelector, constraintName) {
                            return  app.sandbox.form.element.getObject(elementSelector).getConstraint(constraintName);
                        },

                        setValue: function(elementSelector, value) {
                            app.sandbox.form.element.getObject(elementSelector).setValue(value);
                        },

                        getValue: function(elementSelector) {
                            return app.sandbox.form.element.getObject(elementSelector).getValue();
                        }
                    }

                };
            }
        };
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    define('husky_extensions/infinite-scroll',[],function() {

        var scrollTimer = null,

            scrollListener = function(selector, offset, callback) {
                if (!$(selector).data('blocked')) {
                    if (!!scrollTimer) {
                        clearTimeout(scrollTimer);   // clear any previous pending timer
                    }
                    scrollTimer = setTimeout(scrollHandler.bind(this, selector, offset, callback), 50);
                }
            },

            scrollHandler = function(selector, offset, callback) {
                var scrollContainer = $(selector),
                    containerHeight = scrollContainer[0].clientHeight,
                    contentHeight = scrollContainer[0].scrollHeight,
                    scrollTop = scrollContainer.scrollTop(),
                    padding = (isNaN(offset)) ? 0 : offset;

                // check if user scrolled to bottom
                if ((containerHeight + scrollTop + padding - contentHeight) >= 0) {

                    scrollContainer.data('blocked', true);
                    var result = callback();

                    if (!!result && !!result.then) {
                        result.then(function() {
                            $(selector).removeData('blocked');
                        });
                    } else {
                        $(selector).removeData('blocked');
                    }
                }
            };

        return {

            name: 'infinite-scroll',

            initialize: function(app) {
                app.sandbox.infiniteScroll = {

                    /**
                     * Listen for scroll-to-bottom on the element of the given selector
                     * @param selector element to listen on
                     * @param callback function to execute on scrolled-to-bottom
                     * @param offset execute callback before reaching bottom. value in px
                     */
                    initialize: function(selector, callback, offset) {
                        app.sandbox.dom.on(selector, 'scroll.infinite', function() {
                            scrollListener(selector, offset, callback);
                        });
                    },

                    /**
                     * Unbind the scroll event listener
                     * @param selector
                     */
                    destroy: function(selector) {
                        app.sandbox.dom.off(selector, 'scroll.infinite');
                    }
                }
            }
        };
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

/**
 * Introduces functionality used by multiple components, which are displaying some items in a list
 */
define('husky_extensions/itembox',[],function() {

    'use strict';

    var defaults = {
            instanceName: null,
            url: null,
            eventNamespace: 'husky.itembox',
            idsParameter: 'ids',
            resultKey: null,
            idKey: 'id',
            dataAttribute: '',
            dataDefault: {},
            sortable: true,
            removable: true,
            hideAddButton: false,
            hidePositionElement: false,
            hideConfigButton: false,
            hideSelectedCounter: false,
            defaultDisplayOption: 'top',
            actionIcon: 'fa-plus-circle',
            displayOptions: {
                leftTop: true,
                top: true,
                rightTop: true,
                left: true,
                middle: true,
                right: true,
                leftBottom: true,
                bottom: true,
                rightBottom: true
            },
            translations: {
                noContentSelected: 'listbox.nocontent-selected',
                viewAll: 'public.view-all',
                viewLess: 'public.view-less',
                of: 'public.of',
                visible: 'public.visible',
                elementsSelected: 'public.elements-selected'
            }
        },

        constants = {
            displayOptionSelectedClass: 'selected',
            itemInvisibleClass: 'invisible-item',
            noContentClass: 'no-content',
            isLoadingClass: 'is-loading'
        },

        /**
         * returns the normalized event names
         * @param eventName {string} The name of the concrete event without prefix
         * @returns {string} Returns the prefixed event name
         */
        createEventName = function(eventName) {
            // TODO extract to extension?
            return [
                this.options.eventNamespace,
                '.',
                (this.options.instanceName ? this.options.instanceName + '.' : ''),
                eventName
            ].join('');
        },

        templates = {
            skeleton: function() {
                return [
                    '<div class="white-box form-element" id="', this.ids.container, '">',
                    '    <div class="header">',
                    '        <span class="', this.options.actionIcon, ' icon left action', !!this.options.hideAddButton ? ' hidden' : '', '" id="', this.ids.addButton, '"></span>',
                    '        <span class="selected-counter', !!this.options.hideSelectedCounter ? ' hidden' : '', '">',
                    '            <span class="num">0</span><span> ', this.sandbox.translate(this.options.translations.elementsSelected) ,'</span>',
                    '        </span>',
                    '        <span class="fa-cog icon right border', !!this.options.hideConfigButton ? ' hidden' : '', '" id="', this.ids.configButton, '"></span>',
                    '        <span class="no-content-message">', this.sandbox.translate(this.options.translations.noContentSelected), '</span>',
                    '        <div class="position', !!this.options.hidePositionElement ? ' hidden' : '', '">',
                    '            <div class="husky-position" id="', this.ids.displayOption, '">',
                    '                <div class="top left ', (!this.options.displayOptions.leftTop ? 'inactive' : ''), '" data-position="leftTop"></div>',
                    '                <div class="top middle ', (!this.options.displayOptions.top ? 'inactive' : ''), '" data-position="top"></div>',
                    '                <div class="top right ', (!this.options.displayOptions.rightTop ? 'inactive' : ''), '" data-position="rightTop"></div>',
                    '                <div class="middle left ', (!this.options.displayOptions.left ? 'inactive' : ''), '" data-position="left"></div>',
                    '                <div class="middle middle ', (!this.options.displayOptions.middle ? 'inactive' : ''), '" data-position="middle"></div>',
                    '                <div class="middle right ', (!this.options.displayOptions.right ? 'inactive' : ''), '" data-position="right"></div>',
                    '                <div class="bottom left ', (!this.options.displayOptions.leftBottom ? 'inactive' : ''), '" data-position="leftBottom"></div>',
                    '                <div class="bottom middle ', (!this.options.displayOptions.bottom ? 'inactive' : ''), '" data-position="bottom"></div>',
                    '                <div class="bottom right ', (!this.options.displayOptions.rightBottom ? 'inactive' : ''), '" data-position="rightBottom"></div>',
                    '            </div>',
                    '        </div>',
                    '    </div>',
                    '    <div class="content" id="', this.ids.content, '"></div>',
                    '</div>'
                ].join('');
            },

            item: function(id, content) {
                return [
                    '<li data-id="', id, '">',
                    !!this.options.sortable ? '    <span class="fa-ellipsis-v icon move"></span>' : '',
                    '    <span class="num"></span>',
                    '    <div class="item-content">' + content + '</div>',
                    !!this.options.removable ? '    <span class="fa-times remove"></span>' : '',
                    '</li>'
                ].join('');
            }
        },

        bindCustomEvents = function() {
            this.sandbox.on(this.DATA_CHANGED(), this.changeData.bind(this));
            this.sandbox.on(this.DATA_RETRIEVED(), this.renderContent.bind(this));
        },

        bindDomEvents = function() {
            // change display options on click on a positon square
            this.sandbox.dom.on(
                this.getId('displayOption') + ' > div:not(.inactive)',
                'click',
                this.changeDisplayOption.bind(this)
            );

            // click on the add button
            this.sandbox.dom.on(this.$addButton, 'click', function() {
                this.sandbox.emit(this.ADD_BUTTON_CLICKED());
            }.bind(this));

            // click on the config button
            this.sandbox.dom.on(this.$configButton, 'click', function() {
                this.sandbox.emit(this.CONFIG_BUTTON_CLICKED());
            }.bind(this));

            // remove a row from the itembox
            this.sandbox.dom.on(this.getId('content'), 'click', this.removeItem.bind(this), 'li .remove');
        },

        initSortable = function() {
            var $sortable = this.sandbox.dom.find('.sortable', this.$el),
                sortable;

            this.sandbox.dom.sortable($sortable, 'destroy');

            sortable = this.sandbox.dom.sortable('.sortable', {
                handle: '.move',
                forcePlaceholderSize: true
            });

            this.sandbox.dom.unbind(sortable, 'unbind');

            sortable.bind('sortupdate', function() {
                var ids = this.updateOrder();

                this.sortHandler(ids);
            }.bind(this));
        },

        createItemList = function() {
            return this.sandbox.dom.createElement('<ul class="items-list sortable"/>');
        },

        itembox = {
            /**
             * raised when the data changed and the list should be reloaded
             * @event husky.itembox.data-changed
             * @return {string}
             */
            DATA_CHANGED: function() {
                return createEventName.call(this, 'data-changed');
            },

            /**
             * raised when data has returned from the ajax request
             * @event husky.itembox.data-retrieved
             * @return {string}
             */
            DATA_RETRIEVED: function() {
                return createEventName.call(this, 'data-retrieved');
            },

            /**
             * raised when the display option has changed
             * @event husky.itembox.display-position-changed
             * @return {string}
             */
            DISPLAY_OPTION_CHANGED: function() {
                return createEventName.call(this, 'display-position-changed');
            },

            /**
             * raised when the add button was clicked
             * @event husky.itembox.add-button-clicked
             * @return {string}
             */
            ADD_BUTTON_CLICKED: function() {
                return createEventName.call(this, 'add-button-clicked');
            },

            /**
             * raised when the config button was clicked
             * @event husky.itembox.config-button-clicked
             * @return {string}
             */
            CONFIG_BUTTON_CLICKED: function() {
                return createEventName.call(this, 'config-button-clicked');
            },

            /**
             * render the itembox
             */
            render: function() {
                this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

                var data = this.getData();

                this.viewAll = true;

                this.ids = {
                    container: 'listbox-' + this.options.instanceName + '-container',
                    addButton: 'listbox-' + this.options.instanceName + '-add',
                    configButton: 'listbox-' + this.options.instanceName + '-config',
                    displayOption: 'listbox-' + this.options.instanceName + '-display-option',
                    content: 'listbox-' + this.options.instanceName + '-content'
                };

                this.sandbox.dom.html(this.$el, templates.skeleton.call(this));

                this.$container = this.sandbox.dom.find(this.getId('container'), this.$el);
                this.$addButton = this.sandbox.dom.find(this.getId('addButton'), this.$el);
                this.$configButton = this.sandbox.dom.find(this.getId('configButton'), this.$el);
                this.$content = this.sandbox.dom.find(this.getId('content'), this.$el);
                this.$list = null;

                this.addNoContentClass();

                if (!this.isDataEmpty(data)) {
                    this.loadContent(data);
                } else {
                    this.sandbox.dom.data(this.$el, this.options.dataAttribute, this.options.dataDefault);
                }

                this.setDisplayOption(this.options.defaultDisplayOption);
                this.startLoader();
                bindCustomEvents.call(this);
                bindDomEvents.call(this);
            },

            /**
             * adds a class to the components element which makes a no-content text visible
             */
            addNoContentClass: function() {
                this.$list = null;
                this.sandbox.dom.addClass(this.$container, constants.noContentClass);
            },

            /**
             * removes the no-content class from the components element
             */
            removeNoContentClass: function() {
                this.sandbox.dom.removeClass(this.$container, constants.noContentClass);
            },

            /**
             * Returns the data currently stored in this component
             * @param deepCopy {boolean} True if deep cop should be returned, otherwise false
             * @returns {object}
             */
            getData: function() {
                return this.sandbox.util.deepCopy(this.sandbox.dom.data(this.$el, this.options.dataAttribute));
            },

            /**
             * Throws a data-changed event if the data actually has changed
             * @param data {object} The data to set
             * @param reload {boolean} True if the itembox list should be reloaded afterwards
             */
            setData: function(data, reload) {
                var oldData = this.sandbox.dom.data(this.$el, this.options.dataAttribute);
                reload = typeof(reload) === 'undefined' ? true : reload;

                if (!this.sandbox.util.isEqual(oldData, data)) {
                    this.sandbox.emit(this.DATA_CHANGED(), data, this.$el, reload);
                }
            },

            /**
             * Event handler for the changed data event, sets data to element and reloads the list if specified
             * @param data {object} The data to set
             * @param $el {object} The element to which the data should be bound
             * @param reload {boolean} True if the list should be reloaded, otherwise false
             */
            changeData: function (data, $el, reload) {
                this.sandbox.dom.data(this.$el, this.options.dataAttribute, data);

                if (!!reload) {
                    this.loadContent(data);
                }
            },

            /**
             * Updates the visibility of all items
             */
            updateVisibility: function() {
                if (!this.$list || !this.$list.find('li').length) {
                    this.addNoContentClass();
                } else {
                    this.removeNoContentClass();
                }
            },

            /**
             * Loads the content based on the given data
             * @param data {object}
             */
            loadContent: function(data) {
                this.$container.addClass(constants.isLoadingClass);

                // reset items visible when new content is loaded
                this.viewAll = false;

                if (!!data && (!data.length || data.length > 0)) {
                    this.sandbox.util.load(this.getUrl(data))
                        .then(function(data) {
                            this.$container.removeClass(constants.isLoadingClass);
                            this.sandbox.emit(this.DATA_RETRIEVED(), data._embedded[this.options.resultKey]);
                        }.bind(this))
                        .fail(function(error) {
                            this.sandbox.logger.error(error);
                        }.bind(this));
                } else {
                    this.$container.removeClass(constants.isLoadingClass);
                    this.sandbox.emit(this.DATA_RETRIEVED(), []);
                }
            },

            /**
             * Renders the data into the list
             * @param data {object} The data to render
             */
            renderContent: function(data) {
                if (data.length > 0) {
                    var length = data.length;

                    this.$list = createItemList.call(this);

                    for (var i = -1; ++i < length;) {
                        this.addItem(data[i], false, false);
                    }

                    this.sandbox.dom.html(this.$content, this.$list);

                    initSortable.call(this);
                    this.updateOrder();
                    this.updateSelectedCounter();
                    this.updateVisibility();
                } else {
                    // remove content
                    this.$list.children().remove();

                    this.addNoContentClass();
                }
            },

            /**
             * Starts the loader for the content
             */
            startLoader: function() {
                var $loader = this.sandbox.dom.createElement('<div class="loader"/>');
                this.sandbox.dom.append(this.$container.find('.header'), $loader);

                this.sandbox.start([
                    {
                        name: 'loader@husky',
                        options: {
                            el: $loader,
                            size: '20px',
                            color: '#999999'
                        }
                    }
                ]);
            },

            /**
             * Set the display option
             * @param displayOption {string} The string representation of the display option
             */
            setDisplayOption: function(displayOption) {
                var $element = this.sandbox.dom.find('[data-position="' + displayOption + '"]', this.$container);

                // deselect the current positon element
                this.sandbox.dom.removeClass(
                    this.sandbox.dom.find('.' + constants.displayOptionSelectedClass, this.getId('displayOption')),
                    constants.displayOptionSelectedClass
                );

                // select clicked on
                this.sandbox.dom.addClass($element, constants.displayOptionSelectedClass);
            },

            /**
             * DOM event handler for clicking on the display option
             * @param event
             */
            changeDisplayOption: function(event) {
                // TODO move display options to own component?
                var position = this.sandbox.dom.data(event.currentTarget, 'position');

                this.setDisplayOption(position);

                this.sandbox.emit(
                    this.DISPLAY_OPTION_CHANGED(),
                    position
                );
            },

            /**
             * Updates the order of the number in the list
             * @returns {Array}
             */
            updateOrder: function() {
                var $elements = this.sandbox.dom.find('li', this.$content),
                    ids = [];

                this.sandbox.util.foreach($elements, function($element, index) {
                    var $number = this.sandbox.dom.find('.num', $element);
                    $number.html(index + 1);
                    ids.push(this.sandbox.dom.data($element, 'id'));
                }.bind(this));

                return ids;
            },

            /**
             * Updates the selected-counter-element with the number of list-items
             */
            updateSelectedCounter: function() {
                var number = this.$content.find('li').length;
                this.$find('.selected-counter .num').html(number);
            },

            /**
             * Adds an item to the list
             * @param item {object} The item to display in the list
             * @param reinitialize {boolean} Defines if the sorting, order and visibility list should be reinitialized
             * @param scroll {boolean} Defines if true scrolls to the added item
             */
            addItem: function(item, reinitialize, scroll) {
                if (typeof(reinitialize) === 'undefined') {
                    reinitialize = true;
                }
                if (typeof(scroll) === 'undefined') {
                    scroll = true;
                }

                if (!this.$list) {
                    this.$list = createItemList.call(this);
                    this.sandbox.dom.html(this.$content, this.$list);
                }

                var $item = this.sandbox.dom.createElement(templates.item.call(
                    this,
                    item[this.options.idKey],
                    this.getItemContent(item)
                ));

                this.sandbox.dom.append(this.$list, $item);

                if (!!reinitialize) {
                    if (this.options.sortable) {
                        initSortable.call(this);
                    }

                    this.updateOrder();
                    this.updateVisibility();
                    this.updateSelectedCounter();
                }

                if (!!scroll) {
                    this.sandbox.dom.scrollAnimate(this.$content.get(0).scrollHeight, this.$content);
                }
            },

            /**
             * Remove item by id
             * @param {int} itemId
             */
            removeItemById: function(itemId) {
                var $removeItem = this.sandbox.dom.find('li[data-id=' + itemId + ']', this.$el);

                this.sandbox.dom.remove($removeItem);
                this.removeHandler(itemId);

                this.updateOrder();
                this.updateVisibility();
                this.updateSelectedCounter();
            },

            /**
             * DOM event handler for removing an item from the list
             * @param event
             */
            removeItem: function(event) {
                var $removeItem = this.sandbox.dom.parents(event.currentTarget, 'li'),
                    itemId = this.sandbox.dom.data($removeItem, 'id');

                this.sandbox.dom.remove($removeItem);
                this.removeHandler(itemId);

                this.updateOrder();
                this.updateVisibility();
                this.updateSelectedCounter();
            },

            /**
             * Checks if the given data is empty, can be overriden by the concrete implementation.
             * Especially useful if data is not an array.
             *
             * @param data {object} The data to check
             */
            isDataEmpty: function(data) {
                return this.sandbox.util.isEmpty(data);
            },

            /**
             * Returns the selector for the given id.
             *
             * @param type {string} The type of the element, for which the id should be returned
             *
             * @returns {string} The id of the element
             */
            getId: function(type) {
                return ['#', this.ids[type]].join('');
            },

            /**
             * Returns the URL for the list based on the data.
             *
             * @param data {object} The data for which the URL should be generated
             */
            getUrl: function(data) {
                throw new Error('"getUrl" not implemented');
            },

            /**
             * Returns the HTML for an item in the list.
             *
             * @param item
             */
            getItemContent: function(item) {
                throw new Error('"getItemContent" not implemented');
            },

            /**
             * This function is called when the sorting has been updated.
             *
             * @param ids {array} The new order of the ids
             */
            sortHandler: function(ids) {
                throw new Error('"sortHandler" not implemented');
            },

            /**
             * Handler, which is called when a row is removed.
             *
             * @param id {number} The id of the item to remove
             */
            removeHandler: function(id) {
                throw new Error('"removeHandler" not implemented');
            }
        };

    return {
        name: 'itembox',

        initialize: function(app) {
            app.components.addType('itembox', itembox);
        }
    }
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

/**
 * Introduces functionality used by multiple components, which are displaying some items in a list
 */
define('husky_extensions/itemgrid',[],function() {

    'use strict';

    var defaults = {
            instanceName: null,
            url: null,
            eventNamespace: 'husky.itemgrid',
            idsParameter: 'ids',
            resultKey: null,
            idKey: 'id',
            dataAttribute: '',
            dataDefault: {},
            sortable: false,
            removable: true,
            translations: {
                noContentSelected: 'listbox.nocontent-selected',
                viewAll: 'public.view-all',
                viewLess: 'public.view-less',
                of: 'public.of',
                visible: 'public.visible',
                elementsSelected: 'public.elements-selected'
            }
        },

        constants = {
            noContentClass: 'no-content',
            addIcon: 'fa-plus-circle',
            deleteIcon: 'fa-trash-o'
        },

        /**
         * returns the normalized event names
         * @param eventName {string} The name of the concrete event without prefix
         * @returns {string} Returns the prefixed event name
         */
        createEventName = function(eventName) {
            return [
                this.options.eventNamespace,
                '.',
                (this.options.instanceName ? this.options.instanceName + '.' : ''),
                eventName
            ].join('');
        },

        templates = {
            skeleton: function() {
                return [
                    '<div class="listgrid" id="', this.ids.container, '">',
                    '    <div class="header">',
                    '        <span class="', constants.addIcon ,' icon left action add-button" id="', this.ids.addButton, '"></span>',
                    '        <span class="', constants.deleteIcon ,' icon left action delete-button" id="', this.ids.deleteButton, '"></span>',
                    '    </div>',
                    '    <div class="content" id="', this.ids.content, '"></div>',
                    '</div>'
                ].join('');
            }
        },

        bindCustomEvents = function() {
            this.sandbox.on(this.DATA_CHANGED(), this.changeData.bind(this));

            this.sandbox.on(this.DELETE_BUTTON_CLICKED(), function() {
                this.sandbox.emit('husky.datagrid.' + this.options.instanceName + '.items.get-selected', function(ids) {
                    deleteItems.call(this, ids);
                }.bind(this));
            }.bind(this));

            this.sandbox.on(this.ITEM_ADD(), function(item) {
                addItem.call(this, item);
            }.bind(this));

            this.sandbox.on(this.ITEM_SAVE(), function(item) {
                editItem.call(this, item);
            }.bind(this));
        },

        bindDomEvents = function() {
            // click on the add button
            this.sandbox.dom.on(this.$addButton, 'click', function() {
                this.sandbox.emit(this.ADD_BUTTON_CLICKED());
            }.bind(this));

            // click on the config button
            this.sandbox.dom.on(this.$deleteButton, 'click', function() {
                this.sandbox.emit(this.DELETE_BUTTON_CLICKED());
            }.bind(this));
        },

        addItem = function(newItem) {
            var data = this.getData();

            if (this.sandbox.util.typeOf(data) !== 'array' || this.sandbox.util.typeOf(data[0]) !== 'object') {
                data = [];
            }

            data.push(newItem);
            this.setData(data);
        },

        editItem = function(editedItem) {
            var newData = [], data = this.getData();
            data.forEach(function(item) {
                if (item.id === editedItem.id) {
                    newData.push(editedItem);
                } else {
                    newData.push(item);
                }
            }.bind(this));

            this.setData(newData);
        },

        deleteItems = function(ids) {
            this.sandbox.sulu.showDeleteDialog(function(wasConfirmed) {
                if (wasConfirmed) {
                    var newData = [], data = this.getData();
                    data.forEach(function(item) {
                        if (ids.indexOf(item.id) > -1) {
                            this.sandbox.emit('husky.datagrid.' + this.options.instanceName + '.record.remove', item.id);
                            this.sandbox.emit('sulu.header.toolbar.item.enable', 'save', false);
                        } else {
                            newData.push(item);
                        }
                    }.bind(this));

                    this.setData(newData);
                }
            }.bind(this));
        },

        itemgrid = {
            /**
             * raised when the data changed and the list should be reloaded
             * @event husky.itembox.data-changed
             * @return {string}
             */
            DATA_CHANGED: function() {
                return createEventName.call(this, 'data-changed');
            },

            /**
             * raised when the add button was clicked
             * @event husky.itembox.add-button-clicked
             * @return {string}
             */
            ADD_BUTTON_CLICKED: function() {
                return createEventName.call(this, 'add-button-clicked');
            },

            /**
             * raised when the config button was clicked
             * @event husky.itembox.config-button-clicked
             * @return {string}
             */
            DELETE_BUTTON_CLICKED: function() {
                return createEventName.call(this, 'delete-button-clicked');
            },

            /**
             * raised when a new item should be persisted
             * @event husky.itembox.config-button-clicked
             * @return {string}
             */
            ITEM_ADD: function() {
                return createEventName.call(this, 'add-item');
            },

            /**
             * raised when an existing item should be persisted
             * @event husky.itembox.config-button-clicked
             * @return {string}
             */
            ITEM_SAVE: function() {
                return createEventName.call(this, 'save-item');
            },
            /**
             * render the itembox
             */
            render: function() {
                this.options = this.sandbox.util.extend(true, {}, defaults, this.options);

                var data = this.getData();

                this.viewAll = true;

                this.ids = {
                    container: 'listbox-' + this.options.instanceName + '-container',
                    addButton: 'listbox-' + this.options.instanceName + '-add',
                    deleteButton: 'listbox-' + this.options.instanceName + '-delete',
                    content: 'listbox-' + this.options.instanceName + '-content'
                };

                this.sandbox.dom.html(this.$el, templates.skeleton.call(this));

                this.$container = this.sandbox.dom.find(this.getId('container'), this.$el);
                this.$addButton = this.sandbox.dom.find(this.getId('addButton'), this.$el);
                this.$deleteButton = this.sandbox.dom.find(this.getId('deleteButton'), this.$el);
                this.$content = this.sandbox.dom.find(this.getId('content'), this.$el);

                bindCustomEvents.call(this);
                bindDomEvents.call(this);

                this.renderContent(data);
            },

            /**
             * Returns the data currently stored in this component
             * @param deepCopy {boolean} True if deep cop should be returned, otherwise false
             * @returns {object}
             */
            getData: function() {
                return this.sandbox.util.deepCopy(this.sandbox.dom.data(this.$el, this.options.dataAttribute));
            },

            /**
             * Throws a data-changed event if the data actually has changed
             * @param data {object} The data to set
             * @param reload {boolean} True if the itembox list should be reloaded afterwards
             */
            setData: function(data, reload) {
                this.sandbox.emit(this.DATA_CHANGED(), data, this.$el, reload);
            },

            /**
             * Event handler for the changed data event, sets data to element and reloads the list if specified
             * @param data {object} The data to set
             * @param $el {object} The element to which the data should be bound
             * @param reload {boolean} True if the list should be reloaded, otherwise false
             */
            changeData: function (data) {
                this.updateOrder(data);

                this.sandbox.emit('husky.datagrid.' + this.options.instanceName + '.data.set', data);

                this.sandbox.dom.data(this.$el, this.options.dataAttribute, data);

                this.sandbox.emit('sulu.header.toolbar.item.enable', 'save', false);
            },

            /**
             * Gets data and returns them in sorted order
             * @param data {object} The data, which should be set into the grid
             */
            updateOrder: function(data) {
                throw new Error('"updateOrder" not implemented');
            },

            /**
             * Callback action for item click
             * @param id
             * @param item
             */
            editItem: function(id, item) {
                throw new Error('"editItem" not implemented');
            },

            /**
             * Returns the selector for the given id
             * @param type {string} The type of the element, for which the id should be returned
             * @returns {string} The id of the element
             */
            getId: function(type) {
                return ['#', this.ids[type]].join('');
            },

            /**
             * Renders the data into the list
             * @param data {object} The data to render
             */
            renderContent: function(data) {
                var template = [
                        '<div id="', this.options.instanceName, '-list-toolbar-container"></div>',
                        '<div id="itemgrid-list-', this.options.instanceName, '"></div>'
                    ].join('');

                this.sandbox.dom.html(this.$content, template);

                //start list-toolbar and datagrid
                this.sandbox.start([
                    {
                        name: 'datagrid@husky',
                        options: {
                            el: '#itemgrid-list-' + this.options.instanceName,
                            instanceName: this.options.instanceName,
                            view: 'table',
                            data: data,
                            matchings: this.options.fieldList,
                            viewOptions: {
                                table: {
                                    noItemsText: 'public.empty-list'
                                }
                            },
                            actionCallback: function (id, item) {
                                this.editItem.call(this, id, item);
                            }.bind(this)
                        }
                    }
                ]);
            }
        };

    return {
        name: 'itemgrid',

        initialize: function(app) {
            app.components.addType('itemgrid', itemgrid);
        }
    }
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    if (window.jQuery) {
        define('jquery', [], function() {
            return window.jQuery;
        });
    } else {
        require.config({
            paths: { backbone: 'bower_components/jquery/jquery' },
            shim: { backbone: { exports: '$' } }
        });
    }

    // add remove event for jquery
    var initRemoveEvent = function() {
        var ev = new $.Event('remove'),
            orig = $.fn.remove;
        $.fn.remove = function() {
            $(this).trigger(ev);
            return orig.apply(this, arguments);
        };
    };

    define('husky_extensions/jquery',['jquery'], {

        name: 'jQuery',

        initialize: function(app) {

            initRemoveEvent();

            app.core.dom.window = window;

            app.core.dom.$window = $(window);

            app.core.dom.document = document;

            app.core.dom.$document = $(document);

            app.core.dom.$ = $;

            app.core.dom.createElement = function(selector, props) {
                props = props || {};
                return $(selector, props);
            };

            app.core.dom.html = function(selector, content) {
                if (typeof content !== 'undefined') {
                    return $(selector).html(content);
                }
                return $(selector).html();
            };

            app.core.dom.outerHTML = function(selector) {
                return $(selector).wrapAll('<div></div>').parent().html();
            };

            app.core.dom.parseHTML = function(data) {
                return $.parseHTML(data);
            };

            app.core.dom.each = function(selector, callback) {
                $(selector).each(callback);
            };

            app.core.dom.append = function(selector, element) {
                return $(selector).append(element);
            };

            app.core.dom.prepend = function(selector, element) {
                return $(selector).prepend(element);
            };

            app.core.dom.before = function(selector, element) {
                return $(selector).before(element);
            };

            app.core.dom.after = function(selector, element) {
                return $(selector).after(element);
            };

            app.core.dom.css = function(selector, style, value) {
                if (typeof value !== 'undefined') {
                    return $(selector).css(style, value);
                } else {
                    return $(selector).css(style);
                }
            };

            app.core.dom.addClass = function(selector, classes) {
                return $(selector).addClass(classes);
            };

            app.core.dom.removeClass = function(selector, classes) {
                if (!classes) {
                    return $(selector).removeClass();
                }
                return $(selector).removeClass(classes);
            };

            app.core.dom.toggleClass = function(selector, classes) {
                return $(selector).toggleClass(classes);
            };

            app.core.dom.hasClass = function(selector, classes) {
                return $(selector).hasClass(classes);
            };

            app.core.dom.prependClass = function(selector, classes) {
                var $el = $(selector),
                    oldClasses = $el.attr('class');

                /* prepend class */
                classes = classes + ' ' + oldClasses;
                $el.attr('class', classes);
            };

            app.core.dom.parent = function(selector, filter) {
                return $(selector).parent(filter);
            };

            app.core.dom.width = function(selector, value) {
                if (typeof value !== 'undefined') {
                    return $(selector).width(value);
                } else {
                    return $(selector).width();
                }
            };

            app.core.dom.outerWidth = function(selector) {
                return $(selector).outerWidth();
            };

            app.core.dom.height = function(selector, value) {
                if (typeof value === 'undefined') {
                    return $(selector).height();
                }
                return $(selector).height(value);
            };

            app.core.dom.outerHeight = function(selector, includeMargin) {
                if (typeof includeMargin === 'undefined') {
                    return $(selector).outerHeight();
                } else {
                    return $(selector).outerHeight(includeMargin);
                }
            };

            app.core.dom.outerWidth = function(selector, includeMargin) {
                if (typeof includeMargin === 'undefined') {
                    return $(selector).outerWidth();
                } else {
                    return $(selector).outerWidth(includeMargin);
                }
            };

            app.core.dom.offset = function(selector, attributes) {
                if (attributes) {
                    return $(selector).offset(attributes);
                } else {
                    return $(selector).offset();
                }

            };

            app.core.dom.position = function(selector) {
                return $(selector).position();
            };

            app.core.dom.remove = function(context, selector) {
                return $(context).remove(selector);
            };

            app.core.dom.detach = function(context, selector) {
                return $(context).detach(selector);
            };

            app.core.dom.attr = function(selector, attributeName, value) {
                if (typeof value === 'undefined') {
                    attributeName = attributeName || {};
                    return $(selector).attr(attributeName);
                } else {
                    return $(selector).attr(attributeName, value);
                }
            };

            app.core.dom.removeAttr = function(selector, attributeName) {
                return $(selector).removeAttr(attributeName);
            };

            app.core.dom.is = function(selector, type) {
                return $(selector).is(type);
            };

            app.core.dom.filter = function(selector, filter) {
                return $(selector).filter(filter);
            };

            app.core.dom.isArray = function(selector) {
                return $.isArray(selector);
            };

            app.core.dom.isNumeric = function(number) {
                return $.isNumeric(number);
            };

            app.core.dom.data = function(selector, key, value) {
                if (!!value || value === '') {
                    return $(selector).data(key, value);
                } else {
                    return $(selector).data(key);
                }
            };

            app.core.dom.onReady = function(callback) {
                $(window).on('load', callback);
            };

            app.core.dom.val = function(selector, value) {
                if (!!value || value === '') {
                    return $(selector).val(value);
                } else {
                    return $(selector).val();
                }
            };

            app.core.dom.clearVal = function(selector) {
                return $(selector).val('');
            };

            app.core.dom.blur = function(selector) {
                return $(selector).blur();
            };

            app.core.dom.on = function(selector, event, callback, filter) {
                if (!!filter) {
                    $(selector).on(event, filter, callback);
                } else {
                    $(selector).on(event, callback);
                }
            };

            app.core.dom.one = function(selector, event, callback, filter) {
                if (!!filter) {
                    $(selector).one(event, filter, callback);
                } else {
                    $(selector).one(event, callback);
                }
            };

            app.core.dom.off = function(selector, event, handler, filter) {
                if (!!filter) {
                    $(selector).off(event, filter, handler);
                } else {
                    $(selector).off(event, handler);
                }
            };

            app.core.dom.trigger = function(selector, eventType, params) {
                $(selector).trigger(eventType, params);
            };

            app.core.dom.select = function(selector, handler) {
                if (!!handler) {
                    return $(selector).select(handler);
                } else {
                    return $(selector).select();
                }
            };

            app.core.dom.toggleClass = function(selector, className) {
                $(selector).toggleClass(className);
            };

            app.core.dom.parent = function(selector, filter) {
                return $(selector).parent(filter);
            };

            app.core.dom.parents = function(selector, filter) {
                return $(selector).parents(filter);
            };

            app.core.dom.children = function(selector, filter) {
                return $(selector).children(filter);
            };

            app.core.dom.next = function(selector, filter) {
                return $(selector).next(filter);
            };

            app.core.dom.prev = function(selector, filter) {
                return $(selector).prev(filter);
            };

            app.core.dom.closest = function(selector, filter) {
                return $(selector).closest(filter);
            };

            app.core.dom.clone = function(selector) {
                return $(selector).clone();
            };

            app.core.dom.text = function(selector, value) {
                if (!!value || value === '') {
                    return $(selector).text(value);
                } else {
                    return $(selector).text();
                }
            };

            app.core.dom.prop = function(selector, propertyName, value) {
                if (value !== undefined) {
                    return $(selector).prop(propertyName, value);
                } else {
                    return $(selector).prop(propertyName);
                }
            };

            app.core.dom.mouseleave = function(selector, handler) {
                $(selector).mouseleave(handler);
            };

            app.core.dom.stopPropagation = function(event) {
                event.stopPropagation();
            };

            app.core.dom.stop = function(selector) {
                $(selector).stop();
            };

            app.core.dom.preventDefault = function(event) {
                event.preventDefault();
            };

            app.core.dom.hide = function(selector) {
                return $(selector).hide();
            };

            app.core.dom.show = function(selector) {
                return $(selector).show();
            };

            app.core.dom.trim = function(string) {
                return $.trim(string);
            };

            app.core.dom.map = function(selector, callback) {
                return $(selector).map(callback);
            };

            app.core.dom.get = function(selector, index) {
                return $(selector).get(index);
            };

            app.core.dom.toggle = function(selector) {
                return $(selector).toggle();
            };

            app.core.dom.index = function(selector, filter) {
                if (!!filter) {
                    return $(selector).index(filter);
                } else {
                    return $(selector).index();
                }
            };

            app.core.dom.keypress = function(selector, callback) {
                $(selector).keypress(callback);
            };

            app.core.dom.insertAt = function(index, selector, $container, $item) {
                if (index === 0) {
                    app.core.dom.prepend($container, $item);
                } else {
                    var $before = app.core.dom.find(selector + ':nth-child(' + index + ')', $container);
                    app.core.dom.after($before, $item);
                }
            };

            app.core.dom.scrollToTop = function(itemSelector) {
                $(window).scrollTop($(itemSelector).offset().top);
            };

            app.core.dom.scrollTop = function(selector, position) {
                if (typeof position !== 'undefined') {
                    return $(selector).scrollTop(position);
                } else {
                    return $(selector).scrollTop();
                }
            };

            app.core.dom.scrollLeft = function(selector, value) {
                if (typeof value !== 'undefined') {
                    return $(selector).scrollLeft(value);
                } else {
                    return $(selector).scrollLeft();
                }
            };

            app.core.dom.scrollAnimate = function(position, selector) {
                if (!!selector) {
                    $(selector).animate({
                        scrollTop: position
                    }, 500);
                } else {
                    $('html, body').animate({
                        scrollTop: position
                    }, 500);
                }
            };

            app.core.dom.slideUp = function(selector, duration, complete) {
                $(selector).slideUp(duration, complete);
            };

            app.core.dom.slideDown = function(selector, duration, complete) {
                $(selector).slideDown(duration, complete);
            };

            app.core.dom.last = function(selector) {
                return $(selector).last();
            };

            app.core.dom.fadeIn = function(selector, duration, complete) {
                $(selector).fadeIn(duration, complete);
            };

            app.core.dom.fadeTo = function(selector, duration, opacity, complete) {
                $(selector).fadeTo(duration, opacity, complete);
            };

            app.core.dom.fadeOut = function(selector, duration, complete) {
                $(selector).fadeOut(duration, complete);
            };

            app.core.dom.when = function(deffereds) {
                return $.when(deffereds);
            };

            app.core.dom.unbind = function(selector, eventType) {
                $(selector).unbind(eventType);
            };

            app.core.dom.focus = function(selector) {
                $(selector).focus();
            };

            app.core.dom.click = function(selector) {
                $(selector).click();
            };

            app.core.dom.submit = function(selector) {
                $(selector).submit();
            };

            app.core.dom.animate = function(selector, properties, options) {
                $(selector).animate(properties, options);
            };

            app.core.dom.empty = function(selector){
              $(selector).empty();
            };

            app.core.dom.replaceWith = function(selector, newContent) {
              $(selector).replaceWith(newContent);
            };

            app.core.dom.inArray = function(value, array) {
                return $.inArray(value, array);
            };

            app.core.dom.wrapAll = function(selector, html){
                return $(selector).wrapAll(html);
            };

            app.core.dom.first = function(selector) {
                return $(selector).first();
            };

            /**
             * Awesome visible method. Returns false if any part of a given element is not visible
             * Method is copied and slightly adapted from https://github.com/teamdf/jquery-visible/
             * @param selector {Stirng|Object} element selector or jquery dom object
             * @param partial {Boolean} if true only returns false if every part of the element is not visible
             * @param hidden {Boolean} if true checks whether the element is visible, as well as wheter it's within the viewport too. Defaults to true
             * @param direction {String} to specify the direction to check for visibility. This can either be 'horizontal', 'vertical' or 'both'. Default is to 'both'
             * @returns {Boolean} true or false whether the element is visible or not
             */
            app.core.dom.visible = function (selector, partial, hidden, direction) {
                if (this.length < 1) {
                    return;
                }
                direction = (direction) ? direction : 'both';
                hidden = (typeof hidden === 'undefined') ? true : hidden;

                var $element = $(selector), $t = $element.length > 1 ? $element.eq(0) : $element,
                    t = $t.get(0),
                    vpWidth = app.sandbox.dom.$window.width(),
                    vpHeight = app.sandbox.dom.$window.height(),
                    clientSize = hidden === true ? t.offsetWidth * t.offsetHeight : true,
                    rec, tViz, bViz, lViz, rViz, vVisible, hVisible, viewTop, viewBottom, viewLeft, viewRight,
                    offset, _top, _bottom, _left, _right, compareTop, compareBottom, compareLeft, compareRight;

                if (typeof t.getBoundingClientRect === 'function') {

                    // Use this native browser method, if available.
                    rec = t.getBoundingClientRect();
                    tViz = rec.top >= 0 && rec.top < vpHeight;
                    bViz = rec.bottom > 0 && rec.bottom <= vpHeight;
                    lViz = rec.left >= 0 && rec.left < vpWidth;
                    rViz = rec.right > 0 && rec.right <= vpWidth;
                    vVisible = partial ? tViz || bViz : tViz && bViz;
                    hVisible = partial ? lViz || lViz : lViz && rViz;

                    if (direction === 'both') {
                        return clientSize && vVisible && hVisible;
                    }
                    else if (direction === 'vertical') {
                        return clientSize && vVisible;
                    }
                    else if (direction === 'horizontal') {
                        return clientSize && hVisible;
                    }
                } else {

                    viewTop = app.sandbox.dom.$window.scrollTop();
                    viewBottom = viewTop + vpHeight;
                    viewLeft = app.sandbox.dom.$window.scrollLeft();
                    viewRight = viewLeft + vpWidth;
                    offset = $t.offset();
                    _top = offset.top;
                    _bottom = _top + $t.height();
                    _left = offset.left;
                    _right = _left + $t.width();
                    compareTop = partial === true ? _bottom : _top;
                    compareBottom = partial === true ? _top : _bottom;
                    compareLeft = partial === true ? _right : _left;
                    compareRight = partial === true ? _left : _right;

                    if (direction === 'both') {
                        return !!clientSize && ((compareBottom <= viewBottom) && (compareTop >= viewTop)) && ((compareRight <= viewRight) && (compareLeft >= viewLeft));
                    }
                    else if (direction === 'vertical') {
                        return !!clientSize && ((compareBottom <= viewBottom) && (compareTop >= viewTop));
                    }
                    else if (direction === 'horizontal') {
                        return !!clientSize && ((compareRight <= viewRight) && (compareLeft >= viewLeft));
                    }
                }
            };

            app.core.util.ajax = $.ajax;

            app.core.util.ajaxError = function(callback) {
                $(document).ajaxError(callback);
            };

            app.core.util.when = function(deferreds){
                return $.when(deferreds);
            };
        }
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
define('husky_extensions/model',[],function() {

    'use strict';

    // simplified backbone model
    var Model = {
        get: function(attr) {
            return this[attr];
        },

        set: function(attr, value) {
            !!value && (this[attr] = value);
            return this;
        }
    };

    return {

        name: 'Model',

        initialize: function(app) {
            app.sandbox.data.Model = Model;
        }

    };
});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    if (window.TagsManager) {
        define('tagsManager', [], function() {
            return window.TagsManager;
        });
    } else {
        require.config({
            paths: { "tagsManager": 'bower_components/tagmanager/tagmanager' },
            shim: { backbone: { deps: ['jquery'] } }
        });
    }

    define('husky_extensions/tagmanager',['tagsManager'], {
        name: 'tagsManager',

        initialize: function(app) {
            app.sandbox.autocompleteList = {

                init: function(selector, configs) {
                    return app.core.dom.$(selector).tagsManager(configs);
                }

            };
        }
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
define('husky_extensions/template',['underscore', 'jquery'], function(_, $) {

    'use strict';

    function TemplateManager(options) {
        this.options = _.defaults(options || {}, {
            type: 'html',
            compiler: _.template,
            lookup: [],
            components: 'aura_components'
        });

        this.cache = {};
        this.options.lookup.push(this.cache);
    }

    TemplateManager.prototype = {
        load: function(names, widget) {
            names = _.isString(names) ? [names] : names;

            var resolved = {},
                unresolved = [],
                deferred, files;

            _.each(names, function(name) {
                var template = this.find(widget + '/' + name);
                if (template) {
                    resolved[name] = template;
                } else {
                    unresolved.push(name);
                }
            }, this);

            deferred = new $.Deferred();

            if (!unresolved.length) {
                deferred.resolve(resolved);
            } else {
                files = _.map(unresolved, function(u) {
                    var path = u + '.' + this.options.type;
                    if (u.indexOf('/') !== 0) {
                        // TODO default path
                    }
                    return 'text!' + path;
                }, this);

                require(files, _.bind(function() {
                    _.each(Array.prototype.slice.call(arguments), function(template, i) {
                        resolved[unresolved[i]] = this.options.compiler(template);
                        this.cache[widget + '/' + name] = this.options.compiler(template);
                    }, this);

                    deferred.resolve(resolved);
                }, this), function(error) {
                    deferred.reject(error);
                });
            }

            return deferred.promise();
        },

        find: function(name, namespace) {
            namespace = namespace || 'aura';
            var template = $('script[data-' + namespace + '-template="' + name + '"]').html(),
                hash;

            if (template) {
                return this.options.compiler(template);
            } else {
                hash = _.find(this.options.lookup, function(h) {
                    return !!h[name];
                }) || {};

                return hash[name];
            }
        }
    };

    return function(app) {
        var manager = new TemplateManager(app.config.template);

        app.core.Components.Base.prototype.renderTemplate = function(tplName, context) {

            if (typeof this.loadedTemplates[tplName] === 'function') {
                var tpl = this.loadedTemplates[tplName],
                    defaults = {
                        translate: this.sandbox.translate
                    };

                context = this.sandbox.util.extend({}, defaults, context);

                return tpl(context);
            } else {
                return 'Template ' + tplName + ' not found.';
            }
        };

        app.components.before('initialize', function() {

            this.loadedTemplates = {};

            if (!this.templates || !this.templates.length) {
                return;
            }

            var loading = manager.load(this.templates, this.options.name);
            loading.done(_.bind(function(templates) {
                this.loadedTemplates = templates;
            }, this));

            return loading.promise();
        });
    };

});

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    if (window.Typeahead) {
        define('typeahead', [], function() {
            return window.Typeahead;
        });
    } else {
        require.config({
            paths: { "typeahead": 'bower_components/typeahead.js/typeahead.bundle' },
            shim: { backbone: { deps: ['jquery'] } }
        });
    }

    define('husky_extensions/typeahead',['typeahead'], {
        name: 'typeahead',

        initialize: function(app) {
            app.sandbox.autocomplete = {

                init: function(selector, configs) {
                    var engine = this.createEngine(configs);

                    engine.initialize();

                    return app.core.dom.$(selector).typeahead({
                        highlight: true
                    }, {
                        templates: configs.templates,
                        displayKey: configs.displayKey,
                        source: engine.ttAdapter()
                    });
                },

                createEngine: function(configs) {
                    configs = app.sandbox.util.extend(true, {
                        datumTokenizer: function(d) {
                            return Bloodhound.tokenizers.whitespace(d[configs.displayKey]);
                        },
                        queryTokenizer: Bloodhound.tokenizers.whitespace
                    }, configs);

                    return new Bloodhound(configs);
                },

                setValue: function(selector, value) {
                    return app.core.dom.$(selector).typeahead('val', value);
                }
            };
        }
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */
(function() {

    'use strict';

    require.config({
        paths: { "uri-template": 'bower_components/massiveart-uritemplate/uritemplate' }
    });

    define('husky_extensions/uri-template',['uri-template'], function(UriTemplate) {

        return  {

            name: 'uri-template',

            initialize: function(app) {

                app.sandbox.uritemplate = {

                    parse: function(param) {
                        return UriTemplate.parse(param);
                    },

                    expand: function(template, params) {
                        return template.expand(params);
                    }

                };

            }
        };
    });
})();

/**
 * This file is part of Husky frontend development framework.
 *
 * (c) MASSIVE ART WebServices GmbH
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 *
 */

/**
 * Provides the util function into a sandbox.
 */
define('husky_extensions/util',['services/husky/util'], function(util) {

    'use strict';

    return {
        name: 'Util',

        initialize: function(app) {
            for (var key in util) {
                app.core.util[key] = util[key];
            }
        }
    };
});

